@@ -32,6 +32,10 @@ shift 2
 
 # Ensure uniform sorting for pathname expansion
 export LC_ALL=C
+# Prevent OpenMP from creating additional threads
+export OMP_NUM_THREADS=2
+# Allow oversubscription for MPI (needed for Openmpi@3.0)
+export OMPI_MCA_rmaps_base_oversubscribe=1
 
 case $STAGE in
   run)
@@ -103,7 +107,15 @@ case $STAGE in
       #
       case "${NUMDIFF_EXECUTABLE}" in
         *numdiff*)
-          "${NUMDIFF_EXECUTABLE}" -a 1e-6 -r 1e-8 -s ' \t\n:<>=,;' \
+          #
+          # Configure numdiff with
+          #   - absolute precision set to 1e-6, everything below is
+          #     regarded as being equal to 0
+          #   - relative differences of 1e-8
+          #   - [space][tab][newline]=,:;<>[](){}^ as separators between
+          #     numbers
+          #
+          "${NUMDIFF_EXECUTABLE}" -a 1e-6 -r 1e-8 -s ' \t\n=,:;<>[](){}^' \
                                 "${file}" output > diff${variant}
           ;;
         *)
