@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 2016 by the deal.II authors
+// Copyright (C) 2016 - 2017 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -13,13 +13,11 @@
 //
 // ---------------------------------------------------------------------
 
-#ifndef dealii__cell_accessor_wrapper_h
-#define dealii__cell_accessor_wrapper_h
+#ifndef dealii_cell_accessor_wrapper_h
+#define dealii_cell_accessor_wrapper_h
 
 #include <deal.II/base/config.h>
 
-#ifdef DEAL_II_WITH_CXX11
-
 #include <deal.II/grid/tria_accessor.h>
 
 DEAL_II_NAMESPACE_OPEN
@@ -115,7 +113,7 @@ namespace python
     DeclException2(ExcVertexDoesNotExist,
                    int, int,
                    << "Requested vertex number " << arg1
-                   << "does not exist. The largest vertex number "
+                   << " does not exist. The largest vertex number "
                    << "acceptable is "<< arg2-1);
 
   private:
@@ -140,5 +138,3 @@ namespace python
 DEAL_II_NAMESPACE_CLOSE
 
 #endif
-
-#endif
