@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 2013, 2015 by the deal.II authors
+// Copyright (C) 2013 - 2017 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -246,7 +246,7 @@ of accessor classes:
 
 - The TriaAccessor class provides you with data that identifies the geometric
   properties of cells, faces, lines, quads, and hexes that make up a
-  triangulation, as well as mother-child relationships.
+  triangulation, as well as parent-child relationships.
 
 - The CellAccessor class is derived from the TriaAccessor class for cases
   where an object has full dimension, i.e. is a cell rather than for example a
@@ -282,51 +282,27 @@ as the function that generate such objects:
   <tr>
     <th>Triangulation</th>
     <td>typename Triangulation::cell_iterator</td>
-    <td>triangulation.begin()</td>
+    <td>Triangulation::begin()</td>
   </tr>
 
   <tr>
     <th>DoFHandler</th>
     <td>typename DoFHandler::cell_iterator</td>
-    <td>dof_handler.begin()</td>
+    <td>DoFHandler::begin()</td>
   </tr>
 
   <tr>
     <th>hp::DoFHandler</th>
     <td>typename hp::DoFHandler::cell_iterator</td>
-    <td>hp_dof_handler.begin()</td>
+    <td>hp::DoFHandler::begin()</td>
   </tr>
 </table>
 
+The Triangulation class supports iterating across cell faces with <tt>typename
+Triangulation::face_iterator</tt>, which is the type returned by
+Triangulation::begin_face().
 
-<table border=1>
-  <tr>
-    <th>Class</th>
-    <th>face_iterator type</th>
-    <th>function call</th>
-  </tr>
-
-  <tr>
-    <th>Triangulation</th>
-    <td>typename Triangulation::face_iterator</td>
-    <td>triangulation.begin_face()</td>
-  </tr>
-
-  <tr>
-    <th>DoFHandler</th>
-    <td>typename DoFHandler::face_iterator</td>
-    <td>dof_handler.begin_face()</td>
-  </tr>
-
-  <tr>
-    <th>hp::DoFHandler</th>
-    <td>typename hp::DoFHandler::face_iterator</td>
-    <td>hp_dof_handler.begin_face()</td>
-  </tr>
-</table>
-
-
-Likewise, active iterators have the following properties:
+Active iterators have the following properties:
 
 <table border=1>
   <tr>
@@ -338,49 +314,25 @@ Likewise, active iterators have the following properties:
   <tr>
     <th>Triangulation</th>
     <td>typename Triangulation::active_cell_iterator</td>
-    <td>triangulation.begin_active()</td>
+    <td>Triangulation::begin_active()</td>
   </tr>
 
   <tr>
     <th>DoFHandler</th>
     <td>typename DoFHandler::active_cell_iterator</td>
-    <td>dof_handler.begin_active()</td>
+    <td>DoFHandler::begin_active()</td>
   </tr>
 
   <tr>
     <th>hp::DoFHandler</th>
     <td>typename hp::DoFHandler::active_cell_iterator</td>
-    <td>hp_dof_handler.begin_active()</td>
-  </tr>
-</table>
-
-
-<table border=1>
-  <tr>
-    <th>Class</th>
-    <th>face_iterator type</th>
-    <th>function call</th>
-  </tr>
-
-  <tr>
-    <th>Triangulation</th>
-    <td>typename Triangulation::active_face_iterator</td>
-    <td>triangulation.begin_active_face()</td>
-  </tr>
-
-  <tr>
-    <th>DoFHandler</th>
-    <td>typename DoFHandler::active_face_iterator</td>
-    <td>dof_handler.begin_active_face()</td>
-  </tr>
-
-  <tr>
-    <th>hp::DoFHandler</th>
-    <td>typename hp::DoFHandler::active_face_iterator</td>
-    <td>hp_dof_handler.begin_active_face()</td>
+    <td>hp::DoFHandler::begin_active()</td>
   </tr>
 </table>
 
+The Triangulation class also supports iterating across active cell faces with
+<tt>typename Triangulation::active_face_iterator</tt>, which is the type returned by
+Triangulation::begin_active_face().
 
 In addition to these types and calls that act on cells and faces (logical
 concepts that depend on the dimension: a cell is a quadrilateral in 2d, but
@@ -397,11 +349,13 @@ stated in the
   internal::Triangulation::Iterators<3,spacedim> classes for Triangulation
   iterators,
 
-- internal::DoFHandler::Iterators<DoFHandlerType<1,spacedim> >,
-  internal::DoFHandler::Iterators<DoFHandlerType<2,spacedim> >, and
-  internal::DoFHandler::Iterators<DoFHandlerType<3,spacedim> > classes for DoFHandler
-  and hp::DoFHandler iterators,
-
+- <a class="el"
+href="structinternal_1_1DoFHandler_1_1Iterators_3_01DoFHandlerType_3_011_00_01spacedim_01_4_00_01lda_01_4.html">internal::DoFHandler::Iterators&lt;DoFHandlerType&lt;1,spacedim&gt;,
+lda&gt;</a>, <a class="el"
+href="structinternal_1_1Triangulation_1_1Iterators_3_012_00_01spacedim_01_4.html">internal::DoFHandler::Iterators&lt;DoFHandlerType&lt;1,spacedim&gt;,
+lda&gt;</a>, and <a class="el"
+href="structinternal_1_1Triangulation_1_1Iterators_3_013_00_01spacedim_01_4.html">internal::DoFHandler::Iterators&lt;DoFHandlerType&lt;1,spacedim&gt;,
+lda&gt;</a> classes for DoFHandler and hp::DoFHandler iterators.
 
 @section IteratorAccessorInternals Iterator and accessor internals
 
