@@ -1,6 +1,6 @@
 /* ---------------------------------------------------------------------
  *
- * Copyright (C) 2001 - 2016 by the deal.II authors
+ * Copyright (C) 2001 - 2018 by the deal.II authors
  *
  * This file is part of the deal.II library.
  *
@@ -30,7 +30,7 @@
 #include <deal.II/grid/grid_out.h>
 #include <deal.II/dofs/dof_handler.h>
 #include <deal.II/dofs/dof_accessor.h>
-#include <deal.II/fe/fe_q.h>
+#include <deal.II/fe/fe_nothing.h>
 #include <deal.II/fe/fe_values.h>
 
 // This is the only new one: in it, we declare the MappingQ class
@@ -48,13 +48,13 @@ namespace Step10
   using namespace dealii;
 
   // Now, as we want to compute the value of $\pi$, we have to compare to
-  // somewhat. These are the first few digits of $\pi$, which we define
+  // something. These are the first few digits of $\pi$, which we define
   // beforehand for later use. Since we would like to compute the difference
   // between two numbers which are quite accurate, with the accuracy of the
   // computed approximation to $\pi$ being in the range of the number of
   // digits which a double variable can hold, we rather declare the reference
   // value as a <code>long double</code> and give it a number of extra digits:
-  const long double pi = 3.141592653589793238462643;
+  const long double pi = 3.141592653589793238462643L;
 
 
 
@@ -75,15 +75,13 @@ namespace Step10
     std::cout << "Output of grids into gnuplot files:" << std::endl
               << "===================================" << std::endl;
 
-    // So first generate a coarse triangulation of the circle and
-    // associate a suitable boundary description to it. Note that the
-    // default value of the argument to the SphericalManifold
-    // constructor is a center at the origin.
+    // So first generate a coarse triangulation of the circle and associate a
+    // suitable boundary description to it. By default,
+    // GridGenerator::hyper_ball attaches a SphericalManifold to the boundary
+    // (and uses FlatManifold for the interior) so we simply call that
+    // function and move on:
     Triangulation<dim> triangulation;
     GridGenerator::hyper_ball (triangulation);
-    static const SphericalManifold<dim> boundary;
-    triangulation.set_all_manifold_ids_on_boundary(0);
-    triangulation.set_manifold (0, boundary);
 
     // Next generate output for this grid and for a once refined grid. Note
     // that we have hidden the mesh refinement in the loop header, which might
@@ -95,17 +93,9 @@ namespace Step10
         std::cout << "Refinement level: " << refinement << std::endl;
 
         // Then have a string which denotes the base part of the names of the
-        // files into which we write the output. Note that in the parentheses
-        // in the initializer we do arithmetic on characters, which assumes
-        // that first the characters denoting numbers are placed consecutively
-        // (which is probably true for all reasonable character sets
-        // nowadays), but also assumes that the increment
-        // <code>refinement</code> is less than ten. This is therefore more a
-        // quick hack if we know exactly the values which the increment can
-        // assume. A better implementation would use the
-        // <code>std::istringstream</code> class to generate a name.
-        std::string filename_base = "ball";
-        filename_base += '0'+refinement;
+        // files into which we write the output (ending with the refinement
+        // level).
+        std::string filename_base = "ball_" + Utilities::to_string(refinement);
 
         // Then output the present grid for $Q_1$, $Q_2$, and $Q_3$ mappings:
         for (unsigned int degree=1; degree<4; ++degree)
@@ -142,14 +132,14 @@ namespace Step10
             // sufficient to give us the impression of seeing a curved line,
             // rather than a set of straight lines.
             GridOut grid_out;
-            GridOutFlags::Gnuplot gnuplot_flags(false, 30);
+            GridOutFlags::Gnuplot gnuplot_flags(false, 60);
             grid_out.set_flags(gnuplot_flags);
 
-            // Finally, generate a filename and a file for output using the
-            // same evil hack as above:
-            std::string filename = filename_base+"_mapping_q";
-            filename += ('0'+degree);
-            filename += ".dat";
+            // Finally, generate a filename and a file for output:
+            std::string filename = filename_base
+                                   + "_mapping_q_"
+                                   + Utilities::to_string(degree)
+                                   + ".dat";
             std::ofstream gnuplot_file (filename.c_str());
 
             // Then write out the triangulation to this file. The last
@@ -215,21 +205,16 @@ namespace Step10
         Triangulation<dim> triangulation;
         GridGenerator::hyper_ball (triangulation);
 
-        static const SphericalManifold<dim> boundary;
-        triangulation.set_all_manifold_ids_on_boundary (0);
-        triangulation.set_manifold(0, boundary);
-
         const MappingQ<dim> mapping (degree);
 
-        // We now create a dummy finite element. Here we could choose any
-        // finite element, as we are only interested in the `JxW' values
-        // provided by the FEValues object below. Nevertheless, we have to
-        // provide a finite element since in this example we abuse the
-        // FEValues class a little in that we only ask it to provide us with
-        // the weights of certain quadrature points, in contrast to the usual
-        // purpose (and name) of the FEValues class which is to provide the
-        // values of finite elements at these points.
-        const FE_Q<dim>     dummy_fe (1);
+        // We now create a finite element. Unlike the rest of the example
+        // programs, we do not actually need to do any computations with shape
+        // functions; we only need the `JxW' values from an FEValues
+        // object. Hence we use the special finite element class FE_Nothing
+        // which has exactly zero degrees of freedom per cell (as the name
+        // implies, the local basis on each cell is the empty set). A more
+        // typical usage of FE_Nothing is shown in step-46.
+        const FE_Nothing<dim> fe;
 
         // Likewise, we need to create a DoFHandler object. We do not actually
         // use it, but it will provide us with `active_cell_iterators' that
@@ -250,8 +235,7 @@ namespace Step10
         // computation of the mapping from unit to real cell. In previous
         // examples, this argument was omitted, resulting in the implicit use
         // of an object of type MappingQ1.
-        FEValues<dim> fe_values (mapping, dummy_fe, quadrature,
-                                 update_JxW_values);
+        FEValues<dim> fe_values (mapping, fe, quadrature, update_JxW_values);
 
         // We employ an object of the ConvergenceTable class to store all
         // important data like the approximated values for $\pi$ and the error
@@ -275,7 +259,7 @@ namespace Step10
             // our special case but we call it to make the DoFHandler happy --
             // otherwise it would throw an assertion in the FEValues::reinit
             // function below.
-            dof_handler.distribute_dofs (dummy_fe);
+            dof_handler.distribute_dofs (fe);
 
             // We define the variable area as `long double' like we did for
             // the pi variable before.
@@ -291,7 +275,7 @@ namespace Step10
               {
                 fe_values.reinit (cell);
                 for (unsigned int i=0; i<fe_values.n_quadrature_points; ++i)
-                  area += fe_values.JxW (i);
+                  area += static_cast<long double>(fe_values.JxW (i));
               }
 
             // ...and store the resulting area values and the errors in the
@@ -352,10 +336,6 @@ namespace Step10
         Triangulation<dim> triangulation;
         GridGenerator::hyper_ball (triangulation);
 
-        static const SphericalManifold<dim> boundary;
-        triangulation.set_all_manifold_ids_on_boundary (0);
-        triangulation.set_manifold (0, boundary);
-
         const MappingQ<dim> mapping (degree);
         const FE_Q<dim>     fe (1);
 
@@ -390,11 +370,11 @@ namespace Step10
                     // iterator and the number of the face.
                     fe_face_values.reinit (cell, face_no);
                     for (unsigned int i=0; i<fe_face_values.n_quadrature_points; ++i)
-                      perimeter += fe_face_values.JxW (i);
+                      perimeter += static_cast<long double>(fe_face_values.JxW (i));
                   }
             // Then store the evaluated values in the table...
-            table.add_value("eval.pi", static_cast<double> (perimeter/2.));
-            table.add_value("error",   static_cast<double> (std::fabs(perimeter/2.-pi)));
+            table.add_value("eval.pi", static_cast<double> (perimeter/2.0L));
+            table.add_value("error",   static_cast<double> (std::fabs(perimeter/2.0L-pi)));
           }
 
         // ...and end this function as we did in the previous one:
