@@ -1,6 +1,6 @@
 /* ---------------------------------------------------------------------
  *
- * Copyright (C) 2001 - 2016 by the deal.II authors
+ * Copyright (C) 2001 - 2018 by the deal.II authors
  *
  * This file is part of the deal.II library.
  *
@@ -31,7 +31,6 @@
 #include <deal.II/lac/constraint_matrix.h>
 #include <deal.II/grid/tria.h>
 #include <deal.II/grid/grid_generator.h>
-#include <deal.II/grid/manifold_lib.h>
 #include <deal.II/grid/tria_accessor.h>
 #include <deal.II/grid/tria_iterator.h>
 #include <deal.II/dofs/dof_handler.h>
@@ -279,7 +278,7 @@ namespace Step11
                                         system_matrix);
     VectorTools::create_right_hand_side (mapping, dof_handler,
                                          QGauss<dim>(gauss_degree),
-                                         ConstantFunction<dim>(-2),
+                                         Functions::ConstantFunction<dim>(-2),
                                          system_rhs);
     // That's quite simple, right?
     //
@@ -314,7 +313,7 @@ namespace Step11
     Vector<double> tmp (system_rhs.size());
     VectorTools::create_boundary_right_hand_side (mapping, dof_handler,
                                                   QGauss<dim-1>(gauss_degree),
-                                                  ConstantFunction<dim>(1),
+                                                  Functions::ConstantFunction<dim>(1),
                                                   tmp);
     // Then add the contributions from the boundary to those from the interior
     // of the domain:
@@ -361,7 +360,7 @@ namespace Step11
     Vector<float> norm_per_cell (triangulation.n_active_cells());
     VectorTools::integrate_difference (mapping, dof_handler,
                                        solution,
-                                       ZeroFunction<dim>(),
+                                       Functions::ZeroFunction<dim>(),
                                        norm_per_cell,
                                        QGauss<dim>(gauss_degree+1),
                                        VectorTools::H1_seminorm);
@@ -414,9 +413,6 @@ namespace Step11
   void LaplaceProblem<dim>::run ()
   {
     GridGenerator::hyper_ball (triangulation);
-    static const SphericalManifold<dim> boundary;
-    triangulation.set_all_manifold_ids_on_boundary(0);
-    triangulation.set_manifold (0, boundary);
 
     for (unsigned int cycle=0; cycle<6; ++cycle, triangulation.refine_global(1))
       {
