@@ -1,6 +1,6 @@
 /* ---------------------------------------------------------------------
  *
- * Copyright (C) 2001 - 2015 by the deal.II authors
+ * Copyright (C) 2001 - 2018 by the deal.II authors
  *
  * This file is part of the deal.II library.
  *
@@ -55,7 +55,6 @@
 #include <iostream>
 #include <fstream>
 #include <list>
-#include <sstream>
 
 // The last step is as in all previous programs:
 namespace Step13
@@ -392,13 +391,13 @@ namespace Step13
     {}
 
 
-    // After the description above, the function generating the actual output
+    // Following the description above, the function generating the actual output
     // is now relatively straightforward. The only particularly interesting
     // feature over previous example programs is the use of the
     // DataOutBase::default_suffix function, returning the usual
     // suffix for files of a given format (e.g. ".eps" for encapsulated
     // postscript files, ".gnuplot" for Gnuplot files), and of the generic
-    // <code>DataOut::write</code> function with a second argument, which
+    // DataOut::write() function with a second argument, which internally
     // branches to the actual output functions for the different graphics
     // formats, based on the value of the format descriptor passed as second
     // argument.
@@ -418,12 +417,10 @@ namespace Step13
       data_out.add_data_vector (solution, "solution");
       data_out.build_patches ();
 
-      std::ostringstream filename;
-      filename << output_name_base << "-"
-               << this->refinement_cycle
-               << data_out.default_suffix (output_format)
-               << std::ends;
-      std::ofstream out (filename.str().c_str());
+      std::ofstream out (output_name_base
+                         + "-"
+                         + std::to_string(this->refinement_cycle)
+                         + data_out.default_suffix (output_format));
 
       data_out.write (out, output_format);
     }
@@ -497,7 +494,7 @@ namespace Step13
     // which uses a counter in the triangulation class to denote the fact that
     // there is still an object out there using this triangulation, thus
     // leading to an abort in case the triangulation is attempted to be
-    // destructed while this object still uses it.
+    // destroyed while this object still uses it.
     //
     // Note that while the pointer itself is declared constant
     // (i.e. throughout the lifetime of this object, the pointer points to the
@@ -777,20 +774,20 @@ namespace Step13
 
       WorkStream::run(dof_handler.begin_active(),
                       dof_handler.end(),
-                      std_cxx11::bind(&Solver<dim>::local_assemble_matrix,
-                                      this,
-                                      std_cxx11::_1,
-                                      std_cxx11::_2,
-                                      std_cxx11::_3),
-                      std_cxx11::bind(&Solver<dim>::copy_local_to_global,
-                                      this,
-                                      std_cxx11::_1,
-                                      std_cxx11::ref(linear_system)),
+                      std::bind(&Solver<dim>::local_assemble_matrix,
+                                this,
+                                std::placeholders::_1,
+                                std::placeholders::_2,
+                                std::placeholders::_3),
+                      std::bind(&Solver<dim>::copy_local_to_global,
+                                this,
+                                std::placeholders::_1,
+                                std::ref(linear_system)),
                       AssemblyScratchData(*fe, *quadrature),
                       AssemblyCopyData());
       linear_system.hanging_node_constraints.condense (linear_system.matrix);
 
-      // The syntax above using <code>std_cxx11::bind</code> requires
+      // The syntax above using <code>std::bind</code> requires
       // some explanation. There are multiple version of
       // WorkStream::run that expect different arguments. In step-9,
       // we used one version that took a pair of iterators, a pair of
@@ -824,7 +821,7 @@ namespace Step13
       // typical way to generate such function objects is using
       // <code>std::bind</code> (or, if the compiler is too old, a
       // replacement for it, which we generically call
-      // <code>std_cxx11::bind</code>) which takes a pointer to a
+      // <code>std::bind</code>) which takes a pointer to a
       // (member) function and then <i>binds</i> individual arguments
       // to fixed values. For example, you can create a function that
       // takes an iterator, a scratch object and a copy object by
@@ -840,9 +837,9 @@ namespace Step13
       // that can then be filled by WorkStream::run().
       //
       // There remains the question of what the
-      // <code>std_cxx11::_1</code>, <code>std_cxx11::_2</code>, etc.,
+      // <code>std::placeholders::_1</code>, <code>std::placeholders::_2</code>, etc.,
       // mean. (These arguments are called <i>placeholders</i>.) The
-      // idea of using <code>std_cxx11::bind</code> in the first of
+      // idea of using <code>std::bind</code> in the first of
       // the two cases above is that it produces an object that can be
       // called with three arguments. But how are the three arguments
       // the function object is being called with going to be
@@ -855,27 +852,24 @@ namespace Step13
       // games in other circumstances. Consider, for example, having a
       // function <code>void f(double x, double y)</code>. Then,
       // creating a variable <code>p</code> of type
-      // <code>std_cxx11::function@<void f(double,double)@></code> and
-      // initializing <code>p=std_cxx11::bind(&f, std_cxx11::_2,
-      // std_cxx11::_1)</code> then calling <code>p(1,2)</code> will
+      // <code>std::function@<void f(double,double)@></code> and
+      // initializing <code>p=std::bind(&f, std::placeholders::_2,
+      // std::placeholders::_1)</code> then calling <code>p(1,2)</code> will
       // result in calling <code>f(2,1)</code>.
       //
-      // @note Once deal.II can rely on every compiler being able to
-      // fully understand the syntax of the C++11 standard, one can
-      // use C++'s version of <a
+      // @note An alternative to using <code>std::bind</code> is to use C++'s
+      // version of <a
       // href="http://en.wikipedia.org/wiki/Anonymous_function">lambda
-      // functions</a> to achieve the same goal. In essence, a lambda
-      // function is a function without a name that is defined right
-      // at the one place where it is going to be used -- i.e., where
-      // we pass the third and fourth argument to WorkStream::run. The
-      // functions one would define in these locations would take 3
-      // and 1 arguments, respectively, and all they do is call
+      // functions</a>. In essence, a lambda function is a function without a
+      // name that is defined right at the one place where it is going to be
+      // used -- i.e., where we pass the third and fourth argument to
+      // WorkStream::run. The functions one would define in these locations
+      // would take 3 and 1 arguments, respectively, and all they do is call
       // <code>Solver::local_assemble_matrix</code> and
-      // <code>Solver::copy_local_to_global</code> with the required
-      // number of arguments, utilizing what the lambda function has
-      // gotten as arguments itself. We won't show the syntax this
-      // would require since it is no less confusing than the one used
-      // above.
+      // <code>Solver::copy_local_to_global</code> with the required number of
+      // arguments, utilizing what the lambda function has gotten as arguments
+      // itself. We won't show the syntax this would require since it is no
+      // less confusing than the one used above.
 
       // At this point, we have assembled the matrix and condensed
       // it. The right hand side may or may not have been completely
@@ -1466,7 +1460,7 @@ namespace Step13
 
     // Create a solver object of the kind indicated by the argument to this
     // function. If the name is not recognized, throw an exception!
-    LaplaceSolver::Base<dim> *solver = 0;
+    LaplaceSolver::Base<dim> *solver = nullptr;
     if (solver_name == "global")
       solver = new LaplaceSolver::RefinementGlobal<dim> (triangulation, fe,
                                                          quadrature,
