@@ -1,6 +1,6 @@
 /* ---------------------------------------------------------------------
  *
- * Copyright (C) 2003 - 2016 by the deal.II authors
+ * Copyright (C) 2003 - 2018 by the deal.II authors
  *
  * This file is part of the deal.II library.
  *
@@ -40,7 +40,6 @@
 #include <deal.II/grid/tria_iterator.h>
 #include <deal.II/grid/grid_generator.h>
 #include <deal.II/grid/grid_refinement.h>
-#include <deal.II/grid/manifold_lib.h>
 
 #include <deal.II/dofs/dof_accessor.h>
 #include <deal.II/dofs/dof_tools.h>
@@ -88,7 +87,6 @@
 // This is C++:
 #include <iostream>
 #include <fstream>
-#include <sstream>
 
 using namespace dealii;
 
@@ -261,7 +259,7 @@ namespace Step16
   void LaplaceProblem<dim>::setup_system ()
   {
     dof_handler.distribute_dofs (fe);
-    dof_handler.distribute_mg_dofs (fe);
+    dof_handler.distribute_mg_dofs ();
 
     deallog << "   Number of degrees of freedom: "
             << dof_handler.n_dofs()
@@ -281,11 +279,10 @@ namespace Step16
     constraints.clear ();
     DoFTools::make_hanging_node_constraints (dof_handler, constraints);
 
-    std::set<types::boundary_id>         dirichlet_boundary_ids;
-    typename FunctionMap<dim>::type      dirichlet_boundary_functions;
-    ZeroFunction<dim>                    homogeneous_dirichlet_bc (1);
-    dirichlet_boundary_ids.insert(0);
-    dirichlet_boundary_functions[0] = &homogeneous_dirichlet_bc;
+    std::set<types::boundary_id>          dirichlet_boundary_ids;
+    Functions::ZeroFunction<dim>          homogeneous_dirichlet_bc;
+    const typename FunctionMap<dim>::type dirichlet_boundary_functions
+    = { { types::boundary_id(0), &homogeneous_dirichlet_bc } };
     VectorTools::interpolate_boundary_values (static_cast<const DoFHandler<dim>&>(dof_handler),
                                               dirichlet_boundary_functions,
                                               constraints);
@@ -592,19 +589,16 @@ namespace Step16
     data_out.add_data_vector (solution, "solution");
     data_out.build_patches ();
 
-    std::ostringstream filename;
-    filename << "solution-"
-             << cycle
-             << ".vtk";
-
-    std::ofstream output (filename.str().c_str());
+    std::ofstream output ("solution-"
+                          + std::to_string(cycle)
+                          + ".vtk");
     data_out.write_vtk (output);
   }
 
 
   // @sect4{LaplaceProblem::run}
 
-  // Like several of the functions above, this is almost exactly a copy of of
+  // Like several of the functions above, this is almost exactly a copy of
   // the corresponding function in step-6. The only difference is the call to
   // <code>assemble_multigrid</code> that takes care of forming the matrices
   // on every level that we need in the multigrid method.
@@ -618,11 +612,6 @@ namespace Step16
         if (cycle == 0)
           {
             GridGenerator::hyper_ball (triangulation);
-
-            static const SphericalManifold<dim> boundary;
-            triangulation.set_all_manifold_ids_on_boundary(0);
-            triangulation.set_manifold (0, boundary);
-
             triangulation.refine_global (1);
           }
         else
