@@ -1,6 +1,6 @@
 /* ---------------------------------------------------------------------
  *
- * Copyright (C) 2000 - 2016 by the deal.II authors
+ * Copyright (C) 2000 - 2018 by the deal.II authors
  *
  * This file is part of the deal.II library.
  *
@@ -36,7 +36,6 @@
 #include <deal.II/grid/grid_refinement.h>
 #include <deal.II/grid/tria_accessor.h>
 #include <deal.II/grid/tria_iterator.h>
-#include <deal.II/grid/tria_boundary_lib.h>
 #include <deal.II/dofs/dof_handler.h>
 #include <deal.II/dofs/dof_accessor.h>
 #include <deal.II/dofs/dof_tools.h>
@@ -88,7 +87,6 @@
 // And this is simply C++ again:
 #include <fstream>
 #include <iostream>
-#include <sstream>
 
 // The last step is as in all previous programs:
 namespace Step17
@@ -480,7 +478,7 @@ namespace Step17
     std::vector<double>     lambda_values (n_q_points);
     std::vector<double>     mu_values (n_q_points);
 
-    ConstantFunction<dim> lambda(1.), mu(1.);
+    Functions::ConstantFunction<dim> lambda(1.), mu(1.);
 
     RightHandSide<dim>      right_hand_side;
     std::vector<Vector<double> > rhs_values (n_q_points,
@@ -616,7 +614,7 @@ namespace Step17
     std::map<types::global_dof_index,double> boundary_values;
     VectorTools::interpolate_boundary_values (dof_handler,
                                               0,
-                                              ZeroFunction<dim>(dim),
+                                              Functions::ZeroFunction<dim>(dim),
                                               boundary_values);
     MatrixTools::apply_boundary_values (boundary_values,
                                         system_matrix,
@@ -782,7 +780,7 @@ namespace Step17
                                         localized_solution,
                                         local_error_per_cell,
                                         ComponentMask(),
-                                        0,
+                                        nullptr,
                                         MultithreadInfo::n_threads(),
                                         this_mpi_process);
 
@@ -941,17 +939,12 @@ namespace Step17
 
     // This being done, process zero goes ahead with setting up the
     // output file as in step-8, and attaching the (localized)
-    // solution vector to the output object. (The code to generate the
-    // output file name is stolen and slightly modified from step-5,
-    // since we expect that we can do a number of cycles greater than
-    // 10, which is the maximum of what the code in step-8 could
-    // handle.)
+    // solution vector to the output object.
     if (this_mpi_process == 0)
       {
-        std::ostringstream filename;
-        filename << "solution-" << cycle << ".vtk";
-
-        std::ofstream output (filename.str().c_str());
+        std::ofstream output ("solution-"
+                              + std::to_string(cycle)
+                              + ".vtk");
 
         DataOut<dim> data_out;
         data_out.attach_dof_handler (dof_handler);
@@ -960,16 +953,16 @@ namespace Step17
         switch (dim)
           {
           case 1:
-            solution_names.push_back ("displacement");
+            solution_names.emplace_back ("displacement");
             break;
           case 2:
-            solution_names.push_back ("x_displacement");
-            solution_names.push_back ("y_displacement");
+            solution_names.emplace_back ("x_displacement");
+            solution_names.emplace_back ("y_displacement");
             break;
           case 3:
-            solution_names.push_back ("x_displacement");
-            solution_names.push_back ("y_displacement");
-            solution_names.push_back ("z_displacement");
+            solution_names.emplace_back ("x_displacement");
+            solution_names.emplace_back ("y_displacement");
+            solution_names.emplace_back ("z_displacement");
             break;
           default:
             Assert (false, ExcInternalError());
