@@ -1,6 +1,6 @@
 /* ---------------------------------------------------------------------
  *
- * Copyright (C) 2000 - 2017 by the deal.II authors
+ * Copyright (C) 2000 - 2018 by the deal.II authors
  *
  * This file is part of the deal.II library.
  *
@@ -74,7 +74,6 @@
 // This is then simply C++ again:
 #include <fstream>
 #include <iostream>
-#include <sstream>
 #include <iomanip>
 
 // The last step is as in all previous programs:
@@ -349,7 +348,7 @@ namespace Step18
   // good number of more member functions now, which we will explain below.
   //
   // The external interface of the class, however, is unchanged: it has a
-  // public constructor and desctructor, and it has a <code>run</code>
+  // public constructor and destructor, and it has a <code>run</code>
   // function that initiated all the work.
   template <int dim>
   class TopLevel
@@ -639,7 +638,7 @@ namespace Step18
   // For the purposes of this program, we choose a simple form of boundary
   // displacement: we displace the top boundary with constant velocity
   // downwards. The rest of the boundary is either going to be fixed (and is
-  // then described using an object of type <code>ZeroFunction</code>) or free
+  // then described using an object of type <code>Functions::ZeroFunction</code>) or free
   // (Neumann-type, in which case nothing special has to be done).  The
   // implementation of the class describing the constant downward motion
   // should then be obvious using the knowledge we gained through all the
@@ -820,31 +819,9 @@ namespace Step18
               cell->face(f)->set_boundary_id (3);
           }
 
-    // In order to make sure that new vertices are placed correctly on
-    // mesh refinement, we have to associate objects describing those
-    // parts of the boundary that do not consist of straight
-    // parts. Corresponding to the cylinder shell generator function
-    // used above, there are classes that can be used to describe the
-    // geometry of cylinders. The library implements both boundary
-    // classes as well as manifold classes, where also the interior
-    // part of mesh is refined according to the geometrical
-    // description. For this example, we use a single cylindrical
-    // manifold both for the interior part and for the boundary
-    // parts. Note that the manifold object need to live as long as
-    // the triangulation does; we can achieve this by making the
-    // objects static, which means that they live as long as the
-    // program runs:
-    static const CylindricalManifold<dim> cylindrical_manifold (2);
-
-    // We tell the triangulation to reset all its manifold indicators
-    // to 0, and then attach the cylindrical manifold to it:
-    triangulation.set_all_manifold_ids(0);
-    triangulation.set_manifold (0, cylindrical_manifold);
-
     // Once all this is done, we can refine the mesh once globally:
     triangulation.refine_global (1);
 
-
     // As the final step, we need to set up a clean state of the data that we
     // store in the quadrature points on all cells that are treated on the
     // present processor.
@@ -1139,7 +1116,7 @@ namespace Step18
     VectorTools::
     interpolate_boundary_values (dof_handler,
                                  0,
-                                 ZeroFunction<dim> (dim),
+                                 Functions::ZeroFunction<dim> (dim),
                                  boundary_values);
     VectorTools::
     interpolate_boundary_values (dof_handler,
@@ -1249,16 +1226,16 @@ namespace Step18
     switch (dim)
       {
       case 1:
-        solution_names.push_back ("delta_x");
+        solution_names.emplace_back ("delta_x");
         break;
       case 2:
-        solution_names.push_back ("delta_x");
-        solution_names.push_back ("delta_y");
+        solution_names.emplace_back ("delta_x");
+        solution_names.emplace_back ("delta_y");
         break;
       case 3:
-        solution_names.push_back ("delta_x");
-        solution_names.push_back ("delta_y");
-        solution_names.push_back ("delta_z");
+        solution_names.emplace_back ("delta_x");
+        solution_names.emplace_back ("delta_y");
+        solution_names.emplace_back ("delta_z");
         break;
       default:
         Assert (false, ExcNotImplemented());
@@ -1389,7 +1366,7 @@ namespace Step18
 
         // Finally, we write the paraview record, that references all .pvtu files and
         // their respective time. Note that the variable times_and_names is declared
-        // static, so it will retain the entries from the pervious timesteps.
+        // static, so it will retain the entries from the previous timesteps.
         static std::vector<std::pair<double,std::string> > times_and_names;
         times_and_names.push_back (std::pair<double,std::string> (present_time, pvtu_master_filename));
         std::ofstream pvd_output ("solution.pvd");
@@ -1510,7 +1487,7 @@ namespace Step18
                                         incremental_displacement,
                                         error_per_cell,
                                         ComponentMask(),
-                                        0,
+                                        nullptr,
                                         MultithreadInfo::n_threads(),
                                         this_mpi_process);
 
