@@ -1,6 +1,6 @@
 /* ---------------------------------------------------------------------
  *
- * Copyright (C) 2005 - 2016 by the deal.II authors
+ * Copyright (C) 2005 - 2018 by the deal.II authors
  *
  * This file is part of the deal.II library.
  *
@@ -245,7 +245,7 @@ namespace Step19
     // the name of the executable at the zeroth index:
     std::list<std::string> args;
     for (int i=1; i<argc; ++i)
-      args.push_back (argv[i]);
+      args.emplace_back(argv[i]);
 
     // Then process all these parameters. If the parameter is <code>-p</code>,
     // then there must be a parameter file following (which we should then
@@ -456,6 +456,7 @@ namespace Step19
             return;
           }
         AssertThrow (false, ExcNotImplemented());
+        break;
 
       case 2:
         switch (dimensions.second)
@@ -469,6 +470,7 @@ namespace Step19
             return;
           }
         AssertThrow (false, ExcNotImplemented());
+        break;
 
       case 3:
         switch (dimensions.second)
