@@ -1,6 +1,6 @@
 /* ---------------------------------------------------------------------
  *
- * Copyright (C) 2005 - 2016 by the deal.II authors
+ * Copyright (C) 2005 - 2018 by the deal.II authors
  *
  * This file is part of the deal.II library.
  *
@@ -598,7 +598,7 @@ namespace Step20
     // efficient choice because SolverCG instances allocate memory whenever
     // they are created; this is just a tutorial so such inefficiencies are
     // acceptable for the sake of exposition.
-    SolverControl solver_control (std::max(src.size(), static_cast<std::size_t> (200)),
+    SolverControl solver_control (std::max<unsigned int>(src.size(), 200),
                                   1e-8*src.l2_norm());
     SolverCG<>    cg (solver_control);
 
@@ -867,7 +867,7 @@ namespace Step20
   void MixedLaplaceProblem<dim>::output_results () const
   {
     std::vector<std::string> solution_names(dim, "u");
-    solution_names.push_back ("p");
+    solution_names.emplace_back("p");
     std::vector<DataComponentInterpretation::DataComponentInterpretation>
     interpretation (dim,
                     DataComponentInterpretation::component_is_part_of_vector);
