@@ -1,6 +1,6 @@
 /* ---------------------------------------------------------------------
  *
- * Copyright (C) 2006 - 2017 by the deal.II authors
+ * Copyright (C) 2006 - 2018 by the deal.II authors
  *
  * This file is part of the deal.II library.
  *
@@ -59,7 +59,6 @@
 
 #include <iostream>
 #include <fstream>
-#include <sstream>
 
 // In this program, we use a tensor-valued coefficient. Since it may have a
 // spatial dependence, we consider it a tensor-valued function. The following
@@ -223,7 +222,7 @@ namespace Step21
   //
   // We therefore simply create a function that returns zero in all
   // components. We do that by simply forward every function to the
-  // ZeroFunction class. Why not use that right away in the places of this
+  // Functions::ZeroFunction class. Why not use that right away in the places of this
   // program where we presently use the <code>InitialValues</code> class?
   // Because this way it is simpler to later go back and choose a different
   // function for initial values.
@@ -247,7 +246,7 @@ namespace Step21
   InitialValues<dim>::value (const Point<dim>  &p,
                              const unsigned int component) const
   {
-    return ZeroFunction<dim>(dim+2).value (p, component);
+    return Functions::ZeroFunction<dim>(dim+2).value (p, component);
   }
 
 
@@ -256,7 +255,7 @@ namespace Step21
   InitialValues<dim>::vector_value (const Point<dim> &p,
                                     Vector<double>   &values) const
   {
-    ZeroFunction<dim>(dim+2).vector_value (p, values);
+    Functions::ZeroFunction<dim>(dim+2).vector_value (p, values);
   }
 
 
@@ -487,7 +486,7 @@ namespace Step21
   void InverseMatrix<MatrixType>::vmult (Vector<double>       &dst,
                                          const Vector<double> &src) const
   {
-    SolverControl solver_control (std::max(src.size(), static_cast<std::size_t> (200)),
+    SolverControl solver_control (std::max<unsigned int>(src.size(), 200),
                                   1e-8*src.l2_norm());
     SolverCG<>    cg (solver_control);
 
@@ -1086,7 +1085,14 @@ namespace Step21
   // @sect4{TwoPhaseFlowProblem::output_results}
 
   // There is nothing surprising here. Since the program will do a lot of time
-  // steps, we create an output file only every fifth time step.
+  // steps, we create an output file only every fifth time step and skip all
+  // other time steps at the top of the file already.
+  //
+  // When creating file names for output close to the bottom of the function,
+  // we convert the number of the time step to a string representation that
+  // is padded by leading zeros to four digits. We do this because this way
+  // all output file names have the same length, and consequently sort well
+  // when creating a directory listing.
   template <int dim>
   void TwoPhaseFlowProblem<dim>::output_results ()  const
   {
@@ -1097,18 +1103,18 @@ namespace Step21
     switch (dim)
       {
       case 2:
-        solution_names.push_back ("u");
-        solution_names.push_back ("v");
-        solution_names.push_back ("p");
-        solution_names.push_back ("S");
+        solution_names.emplace_back("u");
+        solution_names.emplace_back("v");
+        solution_names.emplace_back("p");
+        solution_names.emplace_back("S");
         break;
 
       case 3:
-        solution_names.push_back ("u");
-        solution_names.push_back ("v");
-        solution_names.push_back ("w");
-        solution_names.push_back ("p");
-        solution_names.push_back ("S");
+        solution_names.emplace_back("u");
+        solution_names.emplace_back("v");
+        solution_names.emplace_back("w");
+        solution_names.emplace_back("p");
+        solution_names.emplace_back("S");
         break;
 
       default:
@@ -1122,12 +1128,9 @@ namespace Step21
 
     data_out.build_patches (degree+1);
 
-    std::ostringstream filename;
-    filename << "solution-"
-             << Utilities::int_to_string(timestep_number,4)
-             << ".vtk";
-
-    std::ofstream output (filename.str().c_str());
+    std::ofstream output ("solution-"
+                          + Utilities::int_to_string(timestep_number,4)
+                          + ".vtk");
     data_out.write_vtk (output);
   }
 
