@@ -1,6 +1,6 @@
 /* ---------------------------------------------------------------------
  *
- * Copyright (C) 2008 - 2016 by the deal.II authors
+ * Copyright (C) 2008 - 2018 by the deal.II authors
  *
  * This file is part of the deal.II library.
  *
@@ -37,7 +37,6 @@
 #include <deal.II/grid/grid_generator.h>
 #include <deal.II/grid/tria_accessor.h>
 #include <deal.II/grid/tria_iterator.h>
-#include <deal.II/grid/tria_boundary_lib.h>
 #include <deal.II/grid/grid_tools.h>
 #include <deal.II/grid/grid_refinement.h>
 
@@ -66,7 +65,7 @@
 // This is C++:
 #include <iostream>
 #include <fstream>
-#include <sstream>
+#include <memory>
 
 // As in all programs, the namespace dealii is included:
 namespace Step22
@@ -103,12 +102,15 @@ namespace Step22
   // @sect3{The <code>StokesProblem</code> class template}
 
   // This is an adaptation of step-20, so the main class and the data types
-  // are the same as used there. In this example we also use adaptive grid
-  // refinement, which is handled in analogy to step-6. According to the
-  // discussion in the introduction, we are also going to use the
-  // ConstraintMatrix for implementing Dirichlet boundary conditions. Hence,
-  // we change the name <code>hanging_node_constraints</code> into
-  // <code>constraints</code>.
+  // are nearly the same as used there. The only difference is that we have an
+  // additional member <code>preconditioner_matrix</code>, that is used for
+  // preconditioning the Schur complement, and a corresponding sparsity pattern
+  // <code>preconditioner_sparsity_pattern</code>.
+  // In this example we also use adaptive grid refinement, which is handled
+  // in analogy to step-6. According to the discussion in the introduction,
+  // we are also going to use the ConstraintMatrix for implementing Dirichlet
+  // boundary conditions. Hence, we change the name
+  // <code>hanging_node_constraints</code> into <code>constraints</code>.
   template <int dim>
   class StokesProblem
   {
@@ -134,6 +136,9 @@ namespace Step22
     BlockSparsityPattern      sparsity_pattern;
     BlockSparseMatrix<double> system_matrix;
 
+    BlockSparsityPattern      preconditioner_sparsity_pattern;
+    BlockSparseMatrix<double> preconditioner_matrix;
+
     BlockVector<double> solution;
     BlockVector<double> system_rhs;
 
@@ -149,7 +154,7 @@ namespace Step22
     // in how many places a preconditioner object is still referenced, it can
     // never create a memory leak, and can never produce a dangling pointer to
     // an already destroyed object:
-    std_cxx11::shared_ptr<typename InnerPreconditioner<dim>::type> A_preconditioner;
+    std::shared_ptr<typename InnerPreconditioner<dim>::type> A_preconditioner;
   };
 
   // @sect3{Boundary values and right hand side}
@@ -401,8 +406,8 @@ namespace Step22
   // releases the pointer to the preconditioner object (if the shared pointer
   // pointed at anything at all at this point) since it will definitely not be
   // needed any more after this point and will have to be re-computed after
-  // assembling the matrix, and unties the sparse matrix from its sparsity
-  // pattern object.
+  // assembling the matrix, and unties the sparse matrices from their sparsity
+  // pattern objects.
   //
   // We then proceed with distributing degrees of freedom and renumbering
   // them: In order to make the ILU preconditioner (in 3D) work efficiently,
@@ -410,7 +415,7 @@ namespace Step22
   // reduces the bandwidth of the matrix, or maybe more importantly: in such a
   // way that the ILU is as close as possible to a real LU decomposition. On
   // the other hand, we need to preserve the block structure of velocity and
-  // pressure already seen in in step-20 and step-21. This is done in two
+  // pressure already seen in step-20 and step-21. This is done in two
   // steps: First, all dofs are renumbered to improve the ILU and then we
   // renumber once again by components. Since
   // <code>DoFRenumbering::component_wise</code> does not touch the
@@ -442,6 +447,7 @@ namespace Step22
   {
     A_preconditioner.reset ();
     system_matrix.clear ();
+    preconditioner_matrix.clear ();
 
     dof_handler.distribute_dofs (fe);
     DoFRenumbering::Cuthill_McKee (dof_handler);
@@ -505,27 +511,30 @@ namespace Step22
               << " (" << n_u << '+' << n_p << ')'
               << std::endl;
 
-    // The next task is to allocate a sparsity pattern for the system matrix
-    // we will create. We could do this in the same way as in step-20,
-    // i.e. directly build an object of type SparsityPattern through
-    // DoFTools::make_sparsity_pattern. However, there is a major reason not
-    // to do so: In 3D, the function DoFTools::max_couplings_between_dofs
-    // yields a conservative but rather large number for the coupling between
-    // the individual dofs, so that the memory initially provided for the
-    // creation of the sparsity pattern of the matrix is far too much -- so
-    // much actually that the initial sparsity pattern won't even fit into the
-    // physical memory of most systems already for moderately-sized 3D
-    // problems, see also the discussion in step-18.  Instead, we first build
-    // a temporary object that uses a different data structure that doesn't
-    // require allocating more memory than necessary but isn't suitable for
-    // use as a basis of SparseMatrix or BlockSparseMatrix objects; in a
-    // second step we then copy this object into an object of
-    // BlockSparsityPattern. This is entirely analogous to what we already did
-    // in step-11 and step-18.
+    // The next task is to allocate a sparsity pattern for the system matrix we
+    // will create and one for the preconditioner matrix. We could do this in
+    // the same way as in step-20, i.e. directly build an object of type
+    // SparsityPattern through DoFTools::make_sparsity_pattern. However, there
+    // is a major reason not to do so:
+    // In 3D, the function DoFTools::max_couplings_between_dofs yields a
+    // conservative but rather large number for the coupling between the
+    // individual dofs, so that the memory initially provided for the creation
+    // of the sparsity pattern of the matrix is far too much -- so much actually
+    // that the initial sparsity pattern won't even fit into the physical memory
+    // of most systems already for moderately-sized 3D problems, see also the
+    // discussion in step-18. Instead, we first build temporary objects that use
+    // a different data structure that doesn't require allocating more memory
+    // than necessary but isn't suitable for use as a basis of SparseMatrix or
+    // BlockSparseMatrix objects; in a second step we then copy these objects
+    // into objects of type BlockSparsityPattern. This is entirely analogous to
+    // what we already did in step-11 and step-18. In particular, we make use of
+    // the fact that we will never write into the $(1,1)$ block of the system
+    // matrix and that this is the only block to be filled for the
+    // preconditioner matrix.
     //
-    // All this is done inside a new scope, which
-    // means that the memory of <code>dsp</code> will be released once the
-    // information has been copied to <code>sparsity_pattern</code>.
+    // All this is done inside new scopes, which means that the memory of
+    // <code>dsp</code> will be released once the information has been copied to
+    // <code>sparsity_pattern</code>.
     {
       BlockDynamicSparsityPattern dsp (2,2);
 
@@ -536,13 +545,50 @@ namespace Step22
 
       dsp.collect_sizes();
 
-      DoFTools::make_sparsity_pattern (dof_handler, dsp, constraints, false);
+      Table<2,DoFTools::Coupling> coupling (dim+1, dim+1);
+
+      for (unsigned int c=0; c<dim+1; ++c)
+        for (unsigned int d=0; d<dim+1; ++d)
+          if (! ((c==dim) && (d==dim)))
+            coupling[c][d] = DoFTools::always;
+          else
+            coupling[c][d] = DoFTools::none;
+
+      DoFTools::make_sparsity_pattern (dof_handler, coupling, dsp, constraints, false);
+
       sparsity_pattern.copy_from (dsp);
     }
 
-    // Finally, the system matrix, solution and right hand side are created
-    // from the block structure as in step-20:
+    {
+      BlockDynamicSparsityPattern preconditioner_dsp (2,2);
+
+      preconditioner_dsp.block(0,0).reinit (n_u, n_u);
+      preconditioner_dsp.block(1,0).reinit (n_p, n_u);
+      preconditioner_dsp.block(0,1).reinit (n_u, n_p);
+      preconditioner_dsp.block(1,1).reinit (n_p, n_p);
+
+      preconditioner_dsp.collect_sizes();
+
+      Table<2,DoFTools::Coupling> preconditioner_coupling (dim+1, dim+1);
+
+      for (unsigned int c=0; c<dim+1; ++c)
+        for (unsigned int d=0; d<dim+1; ++d)
+          if (((c==dim) && (d==dim)))
+            preconditioner_coupling[c][d] = DoFTools::always;
+          else
+            preconditioner_coupling[c][d] = DoFTools::none;
+
+      DoFTools::make_sparsity_pattern (dof_handler, preconditioner_coupling,
+                                       preconditioner_dsp, constraints, false);
+
+      preconditioner_sparsity_pattern.copy_from (preconditioner_dsp);
+    }
+
+    // Finally, the system matrix, the preconsitioner matrix, the solution and
+    // the right hand side vector are created from the block structure similar
+    // to the approach in step-20:
     system_matrix.reinit (sparsity_pattern);
+    preconditioner_matrix.reinit (preconditioner_sparsity_pattern);
 
     solution.reinit (2);
     solution.block(0).reinit (n_u);
@@ -560,13 +606,14 @@ namespace Step22
 
   // The assembly process follows the discussion in step-20 and in the
   // introduction. We use the well-known abbreviations for the data structures
-  // that hold the local matrix, right hand side, and global numbering of the
+  // that hold the local matrices, right hand side, and global numbering of the
   // degrees of freedom for the present cell.
   template <int dim>
   void StokesProblem<dim>::assemble_system ()
   {
     system_matrix=0;
     system_rhs=0;
+    preconditioner_matrix = 0;
 
     QGauss<dim>   quadrature_formula(degree+2);
 
@@ -581,6 +628,7 @@ namespace Step22
     const unsigned int   n_q_points      = quadrature_formula.size();
 
     FullMatrix<double>   local_matrix (dofs_per_cell, dofs_per_cell);
+    FullMatrix<double>   local_preconditioner_matrix (dofs_per_cell, dofs_per_cell);
     Vector<double>       local_rhs (dofs_per_cell);
 
     std::vector<types::global_dof_index> local_dof_indices (dofs_per_cell);
@@ -628,6 +676,7 @@ namespace Step22
       {
         fe_values.reinit (cell);
         local_matrix = 0;
+        local_preconditioner_matrix = 0;
         local_rhs = 0;
 
         right_hand_side.vector_value_list(fe_values.get_quadrature_points(),
@@ -648,10 +697,11 @@ namespace Step22
                   {
                     local_matrix(i,j) += (2 * (symgrad_phi_u[i] * symgrad_phi_u[j])
                                           - div_phi_u[i] * phi_p[j]
-                                          - phi_p[i] * div_phi_u[j]
-                                          + phi_p[i] * phi_p[j])
+                                          - phi_p[i] * div_phi_u[j])
                                          * fe_values.JxW(q);
 
+                    local_preconditioner_matrix(i,j) += (phi_p[i] * phi_p[j])
+                                                        * fe_values.JxW(q);
                   }
 
                 // For the right-hand side we use the fact that the shape
@@ -674,15 +724,7 @@ namespace Step22
               }
           }
 
-        // Note that in the above computation of the local matrix contribution
-        // we added the term <code> phi_p[i] * phi_p[j] </code>, yielding a
-        // pressure mass matrix in the $(1,1)$ block of the matrix as
-        // discussed in the introduction. That this term only ends up in the
-        // $(1,1)$ block stems from the fact that both of the factors in
-        // <code>phi_p[i] * phi_p[j]</code> are only non-zero when all the
-        // other terms vanish (and the other way around).
-        //
-        // Note also that operator* is overloaded for symmetric tensors,
+        // Note that operator* is overloaded for symmetric tensors,
         // yielding the scalar product between the two tensors in the first
         // line of the local matrix contribution.
 
@@ -690,19 +732,25 @@ namespace Step22
         // simultaneously use the ConstraintMatrix object to apply Dirichlet
         // boundary conditions and eliminate hanging node constraints, as we
         // discussed in the introduction), we have to be careful about one
-        // thing, though. We have only built half of the local matrix
-        // because of symmetry, but we're going to save the full system matrix
-        // in order to use the standard functions for solution. This is done
+        // thing, though. We have only built half of the local matrices
+        // because of symmetry, but we're going to save the full matrices
+        // in order to use the standard functions for solving. This is done
         // by flipping the indices in case we are pointing into the empty part
-        // of the local matrix.
+        // of the local matrices.
         for (unsigned int i=0; i<dofs_per_cell; ++i)
           for (unsigned int j=i+1; j<dofs_per_cell; ++j)
-            local_matrix(i,j) = local_matrix(j,i);
+            {
+              local_matrix(i,j) = local_matrix(j,i);
+              local_preconditioner_matrix(i,j) = local_preconditioner_matrix(j,i);
+            }
 
         cell->get_dof_indices (local_dof_indices);
         constraints.distribute_local_to_global (local_matrix, local_rhs,
                                                 local_dof_indices,
                                                 system_matrix, system_rhs);
+        constraints.distribute_local_to_global (local_preconditioner_matrix,
+                                                local_dof_indices,
+                                                preconditioner_matrix);
       }
 
     // Before we're going to solve this linear system, we generate a
@@ -715,7 +763,7 @@ namespace Step22
     std::cout << "   Computing preconditioner..." << std::endl << std::flush;
 
     A_preconditioner
-      = std_cxx11::shared_ptr<typename InnerPreconditioner<dim>::type>(new typename InnerPreconditioner<dim>::type());
+      = std::make_shared<typename InnerPreconditioner<dim>::type>();
     A_preconditioner->initialize (system_matrix.block(0,0),
                                   typename InnerPreconditioner<dim>::type::AdditionalData());
 
@@ -762,8 +810,7 @@ namespace Step22
 
       // Now to the preconditioner to the Schur complement. As explained in
       // the introduction, the preconditioning is done by a mass matrix in the
-      // pressure variable.  It is stored in the $(1,1)$ block of the system
-      // matrix (that is not used anywhere else but in preconditioning).
+      // pressure variable.
       //
       // Actually, the solver needs to have the preconditioner in the form
       // $P^{-1}$, so we need to create an inverse operation. Once again, we
@@ -784,11 +831,11 @@ namespace Step22
       // 1.2. It needs about twice the number of iterations, but the costs for
       // its generation are almost negligible.
       SparseILU<double> preconditioner;
-      preconditioner.initialize (system_matrix.block(1,1),
+      preconditioner.initialize (preconditioner_matrix.block(1,1),
                                  SparseILU<double>::AdditionalData());
 
       InverseMatrix<SparseMatrix<double>,SparseILU<double> >
-      m_inverse (system_matrix.block(1,1), preconditioner);
+      m_inverse (preconditioner_matrix.block(1,1), preconditioner);
 
       // With the Schur complement and an efficient preconditioner at hand, we
       // can solve the respective equation for the pressure (i.e. block 0 in
@@ -844,9 +891,9 @@ namespace Step22
   // <code>DataComponentInterpretation</code> namespace: as with the filename,
   // we create a vector in which the first <code>dim</code> components refer
   // to the velocities and are given the tag
-  // <code>DataComponentInterpretation::component_is_part_of_vector</code>; we
+  // DataComponentInterpretation::component_is_part_of_vector; we
   // finally push one tag
-  // <code>DataComponentInterpretation::component_is_scalar</code> to describe
+  // DataComponentInterpretation::component_is_scalar to describe
   // the grouping of the pressure variable.
 
   // The rest of the function is then the same as in step-20.
@@ -855,7 +902,7 @@ namespace Step22
   StokesProblem<dim>::output_results (const unsigned int refinement_cycle)  const
   {
     std::vector<std::string> solution_names (dim, "velocity");
-    solution_names.push_back ("pressure");
+    solution_names.emplace_back("pressure");
 
     std::vector<DataComponentInterpretation::DataComponentInterpretation>
     data_component_interpretation
@@ -870,12 +917,9 @@ namespace Step22
                               data_component_interpretation);
     data_out.build_patches ();
 
-    std::ostringstream filename;
-    filename << "solution-"
-             << Utilities::int_to_string (refinement_cycle, 2)
-             << ".vtk";
-
-    std::ofstream output (filename.str().c_str());
+    std::ofstream output ("solution-"
+                          + Utilities::int_to_string(refinement_cycle, 2)
+                          + ".vtk");
     data_out.write_vtk (output);
   }
 
