@@ -1,6 +1,6 @@
 /* ---------------------------------------------------------------------
  *
- * Copyright (C) 2006 - 2017 by the deal.II authors
+ * Copyright (C) 2006 - 2018 by the deal.II authors
  *
  * This file is part of the deal.II library.
  *
@@ -38,7 +38,6 @@
 #include <deal.II/grid/grid_generator.h>
 #include <deal.II/grid/tria_accessor.h>
 #include <deal.II/grid/tria_iterator.h>
-#include <deal.II/grid/manifold_lib.h>
 
 #include <deal.II/dofs/dof_handler.h>
 #include <deal.II/dofs/dof_accessor.h>
@@ -274,9 +273,6 @@ namespace Step24
   {
     const Point<dim> center;
     GridGenerator::hyper_ball (triangulation, center, 1.);
-    static const SphericalManifold<dim> boundary_description (center);
-    triangulation.set_all_manifold_ids_on_boundary(0);
-    triangulation.set_manifold (0,boundary_description);
     triangulation.refine_global (7);
 
     time_step = GridTools::minimal_cell_diameter(triangulation) /
@@ -518,7 +514,7 @@ namespace Step24
         G2.add (wave_speed, tmp);
 
         system_rhs_p = G1;
-        system_rhs_p.add(time_step * theta , G2);
+        system_rhs_p.add(time_step * theta, G2);
 
         solve_p ();
 
