@@ -1,6 +1,6 @@
 /* ---------------------------------------------------------------------
  *
- * Copyright (C) 2009 - 2017 by the deal.II authors
+ * Copyright (C) 2009 - 2018 by the deal.II authors
  *
  * This file is part of the deal.II library.
  *
@@ -21,7 +21,12 @@
 // @sect3{Include files}
 
 // We start with a bunch of include files that have already been explained in
-// previous tutorial programs:
+// previous tutorial programs. One new one is <code>timer.h</code>: This is
+// the first example program that uses the Timer class. The Timer keeps track
+// of both the elapsed wall clock time (that is, the amount of time that a
+// clock mounted on the wall would measure) and CPU clock time (the amount of
+// time that the current process uses on the CPUs). We will use a Timer below
+// to measure how much CPU time each grid refinement cycle takes.
 #include <deal.II/base/timer.h>
 #include <deal.II/base/quadrature_lib.h>
 #include <deal.II/base/function.h>
@@ -42,7 +47,6 @@
 #include <deal.II/grid/grid_generator.h>
 #include <deal.II/grid/tria_accessor.h>
 #include <deal.II/grid/tria_iterator.h>
-#include <deal.II/grid/tria_boundary_lib.h>
 #include <deal.II/dofs/dof_handler.h>
 #include <deal.II/dofs/dof_accessor.h>
 #include <deal.II/dofs/dof_tools.h>
@@ -644,7 +648,7 @@ namespace Step28
     for (unsigned int i=0; i<dim; ++i)
       VectorTools::interpolate_boundary_values (dof_handler,
                                                 2*i+1,
-                                                ZeroFunction<dim>(),
+                                                Functions::ZeroFunction<dim>(),
                                                 boundary_values);
   }
 
@@ -1640,11 +1644,15 @@ namespace Step28
 
     double k_eff_old = k_eff;
 
-    Timer timer;
-    timer.start ();
-
     for (unsigned int cycle=0; cycle<parameters.n_refinement_cycles; ++cycle)
       {
+        // We will measure the CPU time that each cycle takes below. The
+        // constructor for Timer calls Timer::start(), so once we create a
+        // timer we can query it for information. Since we use a thread pool
+        // to assemble the system matrices, the CPU time we measure (if we run
+        // with more than one thread) will be larger than the wall time.
+        Timer timer;
+
         std::cout << "Cycle " << cycle << ':' << std::endl;
 
         if (cycle == 0)
@@ -1687,7 +1695,7 @@ namespace Step28
           {
             for (unsigned int group=0; group<parameters.n_groups; ++group)
               {
-                energy_groups[group]->assemble_ingroup_rhs (ZeroFunction<dim>());
+                energy_groups[group]->assemble_ingroup_rhs (Functions::ZeroFunction<dim>());
 
                 for (unsigned int bgroup=0; bgroup<parameters.n_groups; ++bgroup)
                   energy_groups[group]->assemble_cross_group_rhs (*energy_groups[bgroup]);
@@ -1717,11 +1725,15 @@ namespace Step28
         for (unsigned int group=0; group<parameters.n_groups; ++group)
           energy_groups[group]->output_results (cycle);
 
+        // Print out information about the simulation as well as the elapsed
+        // CPU time. We can call Timer::cpu_time() without first calling
+        // Timer::stop() to get the elapsed CPU time at the point of calling
+        // the function.
         std::cout << std::endl;
         std::cout << "   Cycle=" << cycle
                   << ", n_dofs=" << energy_groups[0]->n_dofs() + energy_groups[1]->n_dofs()
                   << ",  k_eff=" << k_eff
-                  << ", time=" << timer()
+                  << ", time=" << timer.cpu_time()
                   << std::endl;
 
 
