@@ -1,6 +1,6 @@
 /* ---------------------------------------------------------------------
  *
- * Copyright (C) 2007 - 2016 by the deal.II authors
+ * Copyright (C) 2007 - 2018 by the deal.II authors
  *
  * This file is part of the deal.II library.
  *
@@ -143,7 +143,7 @@ namespace Step29
   {
   public:
     ParameterReader(ParameterHandler &);
-    void read_parameters(const std::string);
+    void read_parameters(const std::string &);
 
   private:
     void declare_parameters();
@@ -249,7 +249,7 @@ namespace Step29
   // the input file whose filename is provided by the caller. After the call
   // to this function is complete, the <code>prm</code> object can be used to
   // retrieve the values of the parameters read in from the file:
-  void ParameterReader::read_parameters (const std::string parameter_file)
+  void ParameterReader::read_parameters (const std::string &parameter_file)
   {
     declare_parameters();
 
@@ -433,7 +433,6 @@ namespace Step29
     // compute execution time when this function is done:
     deallog << "Generating grid... ";
     Timer timer;
-    timer.start ();
 
     // Then we query the values for the focal distance of the transducer lens
     // and the number of mesh refinement steps from our ParameterHandler
@@ -487,12 +486,8 @@ namespace Step29
           }
     // For the circle part of the transducer lens, a SphericalManifold object
     // is used (which, of course, in 2D just represents a circle), with center
-    // computed as above. By marking this object as <code>static</code>, we
-    // ensure that it lives until the end of the program and thereby longer
-    // than the triangulation object we will associate with it. We then assign
-    // this boundary-object to the part of the boundary with boundary indicator 1:
-    static const SphericalManifold<dim> boundary(focal_point);
-    triangulation.set_manifold(1, boundary);
+    // computed as above.
+    triangulation.set_manifold(1, SphericalManifold<dim>(focal_point));
 
     // Now global refinement is executed. Cells near the transducer location
     // will be automatically refined according to the circle shaped boundary
@@ -504,7 +499,7 @@ namespace Step29
     // function:
     timer.stop ();
     deallog << "done ("
-            << timer()
+            << timer.cpu_time()
             << "s)"
             << std::endl;
 
@@ -525,7 +520,6 @@ namespace Step29
   {
     deallog << "Setting up system... ";
     Timer timer;
-    timer.start();
 
     dof_handler.distribute_dofs (fe);
 
@@ -539,7 +533,7 @@ namespace Step29
 
     timer.stop ();
     deallog << "done ("
-            << timer()
+            << timer.cpu_time()
             << "s)"
             << std::endl;
 
@@ -558,7 +552,6 @@ namespace Step29
   {
     deallog << "Assembling system matrix... ";
     Timer timer;
-    timer.start ();
 
     // First we query wavespeed and frequency from the ParameterHandler object
     // and store them in local variables, as they will be used frequently
@@ -779,7 +772,7 @@ namespace Step29
 
     timer.stop ();
     deallog << "done ("
-            << timer()
+            << timer.cpu_time()
             << "s)"
             << std::endl;
   }
@@ -805,7 +798,6 @@ namespace Step29
   {
     deallog << "Solving linear system... ";
     Timer timer;
-    timer.start ();
 
     // The code to solve the linear system is short: First, we allocate an
     // object of the right type. The following <code>initialize</code> call
@@ -823,7 +815,7 @@ namespace Step29
 
     timer.stop ();
     deallog << "done ("
-            << timer ()
+            << timer.cpu_time()
             << "s)"
             << std::endl;
   }
@@ -843,7 +835,6 @@ namespace Step29
   {
     deallog << "Generating output... ";
     Timer timer;
-    timer.start ();
 
     // Define objects of our <code>ComputeIntensity</code> class and a DataOut
     // object:
@@ -877,8 +868,8 @@ namespace Step29
     // The solution vectors $v$ and $w$ are added to the DataOut object in the
     // usual way:
     std::vector<std::string> solution_names;
-    solution_names.push_back ("Re_u");
-    solution_names.push_back ("Im_u");
+    solution_names.emplace_back("Re_u");
+    solution_names.emplace_back("Im_u");
 
     data_out.add_data_vector (solution, solution_names);
 
@@ -895,7 +886,7 @@ namespace Step29
 
     timer.stop ();
     deallog << "done ("
-            << timer()
+            << timer.cpu_time()
             << "s)"
             << std::endl;
   }
