@@ -1,6 +1,6 @@
 /* ---------------------------------------------------------------------
  *
- * Copyright (C) 1999 - 2016 by the deal.II authors
+ * Copyright (C) 1999 - 2018 by the deal.II authors
  *
  * This file is part of the deal.II library.
  *
@@ -36,7 +36,7 @@
 #include <deal.II/grid/tria_iterator.h>
 #include <deal.II/dofs/dof_accessor.h>
 
-// In this file contains the description of the Lagrange interpolation finite
+// This file contains the description of the Lagrange interpolation finite
 // element:
 #include <deal.II/fe/fe_q.h>
 
@@ -44,7 +44,7 @@
 // matrices, as shown in previous examples:
 #include <deal.II/dofs/dof_tools.h>
 
-// The next two file are needed for assembling the matrix using quadrature on
+// The next two files are needed for assembling the matrix using quadrature on
 // each cell. The classes declared in them will be explained below:
 #include <deal.II/fe/fe_values.h>
 #include <deal.II/base/quadrature_lib.h>
@@ -359,7 +359,7 @@ void Step3::assemble_system ()
   //
   // The shortcuts, finally, are only defined to make the following loops a
   // bit more readable. You will see them in many places in larger programs,
-  // and `dofs_per_cell' and `n_q_points' are more or less by convention the
+  // and `dofs_per_cell` and `n_q_points` are more or less by convention the
   // standard names for these purposes:
   const unsigned int   dofs_per_cell = fe.dofs_per_cell;
   const unsigned int   n_q_points    = quadrature_formula.size();
@@ -387,18 +387,19 @@ void Step3::assemble_system ()
   // the type, types::global_dof_index, used here):
   std::vector<types::global_dof_index> local_dof_indices (dofs_per_cell);
 
-  // Now for the loop over all cells. We have seen before how this works, so
-  // the following code should be familiar including the conventional names
-  // for these variables:
-  DoFHandler<2>::active_cell_iterator cell = dof_handler.begin_active();
-  DoFHandler<2>::active_cell_iterator endc = dof_handler.end();
-  for (; cell!=endc; ++cell)
+  // Now for the loop over all cells. We have seen before how this works for a
+  // triangulation. A DoFHandler has cell iterators that are exactly analogous
+  // to those of a Triangulation, but with extra information about the degrees
+  // of freedom for the finite element you're using. Looping over the active
+  // cells of a degree-of-freedom handler works the same as for a triangulation.
+  //
+  // Note that we declare the type of the cell as `const auto &` instead of
+  // `auto` this time around. In step 1, we were modifying the cells of the
+  // triangulation by flagging them with refinement indicators. Here we're only
+  // examining the cells without modifying them, so it's good practice to
+  // declare `cell` as `const` in order to enforce this invariant.
+  for (const auto &cell: dof_handler.active_cell_iterators())
     {
-      // @note As already mentioned in step-1, there is a more convenient way
-      // of writing such loops if your compiler supports the C++11
-      // standard. See @ref CPP11 "the deal.II C++11 page" to see
-      // how this works.
-      //
       // We are now sitting on one cell, and we would like the values and
       // gradients of the shape functions be computed, as well as the
       // determinants of the Jacobian matrices of the mapping between
@@ -496,15 +497,15 @@ void Step3::assemble_system ()
   // boundary by different numbers and tell the interpolate_boundary_values
   // function to only compute the boundary values on a certain part of the
   // boundary (e.g. the clamped part, or the inflow boundary). By default, all
-  // boundaries have the number `0', and since we have not changed that, this
-  // is still so; therefore, if we give `0' as the desired portion of the
+  // boundaries have the number `0`, and since we have not changed that, this
+  // is still so; therefore, if we give `0` as the desired portion of the
   // boundary, this means we get the whole boundary. If you have boundaries
   // with kinds of boundaries, you have to number them differently. The
   // function call below will then only determine boundary values for parts of
   // the boundary.
   //
   // The function describing the boundary values is an object of type Function
-  // or of a derived class. One of the derived classes is ZeroFunction, which
+  // or of a derived class. One of the derived classes is Functions::ZeroFunction, which
   // describes (not unexpectedly) a function which is zero everywhere. We
   // create such an object in-place and pass it to the
   // VectorTools::interpolate_boundary_values() function.
@@ -517,7 +518,7 @@ void Step3::assemble_system ()
   std::map<types::global_dof_index,double> boundary_values;
   VectorTools::interpolate_boundary_values (dof_handler,
                                             0,
-                                            ZeroFunction<2>(),
+                                            Functions::ZeroFunction<2>(),
                                             boundary_values);
   // Now that we got the list of boundary DoFs and their respective boundary
   // values, let's use them to modify the system of equations
@@ -622,32 +623,56 @@ void Step3::run ()
 
 // @sect3{The <code>main</code> function}
 
-// This is the main function of the program. Since the concept of a main
-// function is mostly a remnant from the pre-object era in C/C++ programming,
-// it often does not much more than creating an object of the top-level class
-// and calling its principle function.
+// This is the main function of the program. Since the concept of a
+// main function is mostly a remnant from the pre-object oriented era
+// before C++ programming, it often does not do much more than
+// creating an object of the top-level class and calling its principle
+// function.
 //
-// Finally, the first line of the function is used to enable output of the
-// deal.II logstream to the screen.  The deallog (which stands for deal-log,
-// not de-allog) variable represents a stream to which some parts of the
-// library write output. For example, iterative solvers will generate
-// diagnostics (starting residual, number of solver steps, final residual) as
-// can be seen when running this tutorial program.
+// Finally, the first line of the function is used to enable output of
+// some diagnostics that deal.II can generate.  The @p deallog
+// variable (which stands for deal-log, not de-allog) represents a
+// stream to which some parts of the library write output. For
+// example, iterative solvers will generate diagnostics (starting
+// residual, number of solver steps, final residual) as can be seen
+// when running this tutorial program.
 //
-// The output of deallog can be redirected to the console, to a file, or
-// both. But both are disabled by default. The output is nested in a way so
-// that each function can use a prefix string (separated by colons) for each
-// line of output; if it calls another function, that may also use its prefix
-// which is then printed after the one of the calling function. By running
-// this example (or looking at the "Results" section), you will see the solver
-// statistics prefixed with "DEAL:CG", which is two prefixes.  Since output
-// from functions which are nested deep below is usually not as important as
-// top-level output, you can give the deallog variable a maximal depth of
-// nested output for output to console and file. A depth of 0 (the default)
-// will disable output to the screen, while a value of 2 or higher will cause
-// the solver info in this example to be printed. Imagine that different
-// solvers can be nested, which we will see in step-22 for example, and you
-// might not want to see all this information.
+// The output of @p deallog can be written to the console, to a file,
+// or both. Both are disabled by default since over the years we have
+// learned that a program should only generate output when a user
+// explicitly asks for it. But this can be changed, and to explain how
+// this can be done, we need to explain how @p deallog works: When
+// individual parts of the library want to log output, they open a
+// "context" or "section" into which this output will be placed. At
+// the end of the part that wants to write output, one exits this
+// section again. Since a function may call another one from within
+// the scope where this output section is open, output may in fact be
+// nested hierarchically into these sections. The LogStream class of
+// which @p deallog is a variable calls each of these sections a
+// "prefix" because all output is printed with this prefix at the left
+// end of the line, with prefixes separated by colons. There is always
+// a default prefix called "DEAL" (a hint at deal.II's history as the
+// successor of a previous library called "DEAL" and from which the
+// LogStream class is one of the few pieces of code that were taken
+// into deal.II).
+//
+// By default, @p logstream only outputs lines with zero prefixes --
+// i.e., all output is disabled because the default "DEAL" prefix is
+// always there. But one can set a different maximal number of
+// prefixes for lines that should be output to something larger, and
+// indeed here we set it to two by calling
+// LogStream::depth_console(). This means that for all screen output,
+// a context that has pushed one additional prefix beyond the default
+// "DEAL" is allowed to print its output to the screen ("console"),
+// whereas all further nested sections that would have three or more
+// prefixes active would write to @p deallog, but @p deallog does not
+// forward this output to the screen. Thus, running this example (or
+// looking at the "Results" section), you will see the solver
+// statistics prefixed with "DEAL:CG", which is two prefixes. This is
+// sufficient for the context of the current program, but you will see
+// examples later on (e.g., in step-22) where solvers are nested more
+// deeply and where you may get useful information by setting the
+// depth even higher.
 int main ()
 {
   deallog.depth_console (2);
