@@ -1,6 +1,6 @@
 /* ---------------------------------------------------------------------
  *
- * Copyright (C) 2009 - 2017 by the deal.II authors
+ * Copyright (C) 2009 - 2018 by the deal.II authors
  *
  * This file is part of the deal.II library.
  *
@@ -49,7 +49,6 @@
 #include <deal.II/grid/grid_refinement.h>
 #include <deal.II/grid/tria_accessor.h>
 #include <deal.II/grid/tria_iterator.h>
-#include <deal.II/grid/tria_boundary_lib.h>
 #include <deal.II/grid/grid_in.h>
 
 #include <deal.II/dofs/dof_handler.h>
@@ -517,11 +516,13 @@ namespace Step35
     //
     // To do this in practice, we declare a "synchronous" iterator -- an
     // object that internally consists of several (in our case two) iterators,
-    // and each time the synchronous iteration is moved up one step, each of
-    // the iterators stored internally is moved up one step as well, thereby
+    // and each time the synchronous iteration is moved forward one step, each of
+    // the iterators stored internally is moved forward one step as well, thereby
     // always staying in sync. As it so happens, there is a deal.II class that
-    // facilitates this sort of thing.
-    typedef std_cxx11::tuple< typename DoFHandler<dim>::active_cell_iterator,
+    // facilitates this sort of thing. (What is important here is to know that
+    // two DoFHandler objects built on the same triangulation will walk over
+    // the cells of the triangulation in the same order.)
+    typedef std::tuple< typename DoFHandler<dim>::active_cell_iterator,
             typename DoFHandler<dim>::active_cell_iterator
             > IteratorTuple;
 
@@ -776,9 +777,9 @@ namespace Step35
       {
         vel_exact.set_time (t_0);
         vel_exact.set_component(d);
-        VectorTools::interpolate (dof_handler_velocity, ZeroFunction<dim>(), u_n_minus_1[d]);
+        VectorTools::interpolate (dof_handler_velocity, Functions::ZeroFunction<dim>(), u_n_minus_1[d]);
         vel_exact.advance_time (dt);
-        VectorTools::interpolate (dof_handler_velocity, ZeroFunction<dim>(), u_n[d]);
+        VectorTools::interpolate (dof_handler_velocity, Functions::ZeroFunction<dim>(), u_n[d]);
       }
   }
 
@@ -898,11 +899,11 @@ namespace Step35
                                  InitGradScratchData &scratch,
                                  InitGradPerTaskData &data)
   {
-    scratch.fe_val_vel.reinit (std_cxx11::get<0> (*SI));
-    scratch.fe_val_pres.reinit (std_cxx11::get<1> (*SI));
+    scratch.fe_val_vel.reinit (std::get<0> (*SI));
+    scratch.fe_val_pres.reinit (std::get<1> (*SI));
 
-    std_cxx11::get<0> (*SI)->get_dof_indices (data.vel_local_dof_indices);
-    std_cxx11::get<1> (*SI)->get_dof_indices (data.pres_local_dof_indices);
+    std::get<0> (*SI)->get_dof_indices (data.vel_local_dof_indices);
+    std::get<1> (*SI)->get_dof_indices (data.pres_local_dof_indices);
 
     data.local_grad = 0.;
     for (unsigned int q=0; q<scratch.nqp; ++q)
@@ -1047,7 +1048,7 @@ namespace Step35
                 VectorTools::
                 interpolate_boundary_values (dof_handler_velocity,
                                              *boundaries,
-                                             ZeroFunction<dim>(),
+                                             Functions::ZeroFunction<dim>(),
                                              boundary_values);
                 break;
               case 2:
@@ -1062,14 +1063,14 @@ namespace Step35
                   VectorTools::
                   interpolate_boundary_values (dof_handler_velocity,
                                                *boundaries,
-                                               ZeroFunction<dim>(),
+                                               Functions::ZeroFunction<dim>(),
                                                boundary_values);
                 break;
               case 4:
                 VectorTools::
                 interpolate_boundary_values (dof_handler_velocity,
                                              *boundaries,
-                                             ZeroFunction<dim>(),
+                                             Functions::ZeroFunction<dim>(),
                                              boundary_values);
                 break;
               default:
@@ -1215,7 +1216,7 @@ namespace Step35
     static std::map<types::global_dof_index, double> bval;
     if (reinit_prec)
       VectorTools::interpolate_boundary_values (dof_handler_pressure, 3,
-                                                ZeroFunction<dim>(), bval);
+                                                Functions::ZeroFunction<dim>(), bval);
 
     MatrixTools::apply_boundary_values (bval, pres_iterative, phi_n, pres_tmp);
 
@@ -1296,7 +1297,7 @@ namespace Step35
             ((dim + 1)*dof_handler_velocity.n_dofs() +
              dof_handler_pressure.n_dofs()),
             ExcInternalError());
-    static Vector<double> joint_solution (joint_dof_handler.n_dofs());
+    Vector<double> joint_solution (joint_dof_handler.n_dofs());
     std::vector<types::global_dof_index> loc_joint_dof_indices (joint_fe.dofs_per_cell),
         loc_vel_dof_indices (fe_velocity.dofs_per_cell),
         loc_pres_dof_indices (fe_pressure.dofs_per_cell);
@@ -1308,8 +1309,8 @@ namespace Step35
     for (; joint_cell != joint_endc; ++joint_cell, ++vel_cell, ++pres_cell)
       {
         joint_cell->get_dof_indices (loc_joint_dof_indices);
-        vel_cell->get_dof_indices (loc_vel_dof_indices),
-                 pres_cell->get_dof_indices (loc_pres_dof_indices);
+        vel_cell->get_dof_indices (loc_vel_dof_indices);
+        pres_cell->get_dof_indices (loc_pres_dof_indices);
         for (unsigned int i=0; i<joint_fe.dofs_per_cell; ++i)
           switch (joint_fe.system_to_base_index(i).first.first)
             {
@@ -1337,8 +1338,8 @@ namespace Step35
             }
       }
     std::vector<std::string> joint_solution_names (dim, "v");
-    joint_solution_names.push_back ("p");
-    joint_solution_names.push_back ("rot_u");
+    joint_solution_names.emplace_back("p");
+    joint_solution_names.emplace_back("rot_u");
     DataOut<dim> data_out;
     data_out.attach_dof_handler (joint_dof_handler);
     std::vector< DataComponentInterpretation::DataComponentInterpretation >
