@@ -1,6 +1,6 @@
 /* ---------------------------------------------------------------------
  *
- * Copyright (C) 2009 - 2017 by the deal.II authors
+ * Copyright (C) 2009 - 2018 by the deal.II authors
  *
  * This file is part of the deal.II library.
  *
@@ -36,7 +36,6 @@
 #include <deal.II/grid/tria.h>
 #include <deal.II/grid/tria_accessor.h>
 #include <deal.II/grid/tria_iterator.h>
-#include <deal.II/grid/tria_boundary_lib.h>
 #include <deal.II/grid/grid_generator.h>
 
 #include <deal.II/multigrid/multigrid.h>
@@ -58,7 +57,6 @@
 
 #include <iostream>
 #include <fstream>
-#include <sstream>
 
 
 namespace Step37
@@ -804,14 +802,13 @@ namespace Step37
   void LaplaceProblem<dim>::setup_system ()
   {
     Timer time;
-    time.start ();
     setup_time = 0;
 
     system_matrix.clear();
     mg_matrices.clear_elements();
 
     dof_handler.distribute_dofs (fe);
-    dof_handler.distribute_mg_dofs (fe);
+    dof_handler.distribute_mg_dofs ();
 
     pcout << "Number of degrees of freedom: "
           << dof_handler.n_dofs()
@@ -827,12 +824,12 @@ namespace Step37
                                             constraints);
     VectorTools::interpolate_boundary_values (dof_handler,
                                               0,
-                                              ZeroFunction<dim>(),
+                                              Functions::ZeroFunction<dim>(),
                                               constraints);
     constraints.close();
     setup_time += time.wall_time();
     time_details << "Distribute DoFs & B.C.     (CPU/wall) "
-                 << time() << "s/" << time.wall_time() << "s" << std::endl;
+                 << time.cpu_time() << "s/" << time.wall_time() << "s" << std::endl;
     time.restart();
 
     {
@@ -841,7 +838,7 @@ namespace Step37
         MatrixFree<dim,double>::AdditionalData::none;
       additional_data.mapping_update_flags = (update_gradients | update_JxW_values |
                                               update_quadrature_points);
-      std_cxx11::shared_ptr<MatrixFree<dim,double> >
+      std::shared_ptr<MatrixFree<dim,double> >
       system_mf_storage(new MatrixFree<dim,double>());
       system_mf_storage->reinit (dof_handler, constraints, QGauss<1>(fe.degree+1),
                                  additional_data);
@@ -855,7 +852,7 @@ namespace Step37
 
     setup_time += time.wall_time();
     time_details << "Setup matrix-free system   (CPU/wall) "
-                 << time() << "s/" << time.wall_time() << "s" << std::endl;
+                 << time.cpu_time() << "s/" << time.wall_time() << "s" << std::endl;
     time.restart();
 
     // Next, initialize the matrices for the multigrid method on all the
@@ -891,7 +888,7 @@ namespace Step37
         additional_data.mapping_update_flags = (update_gradients | update_JxW_values |
                                                 update_quadrature_points);
         additional_data.level_mg_handler = level;
-        std_cxx11::shared_ptr<MatrixFree<dim,float> >
+        std::shared_ptr<MatrixFree<dim,float> >
         mg_mf_storage_level(new MatrixFree<dim,float>());
         mg_mf_storage_level->reinit(dof_handler, level_constraints,
                                     QGauss<1>(fe.degree+1), additional_data);
@@ -902,7 +899,7 @@ namespace Step37
       }
     setup_time += time.wall_time();
     time_details << "Setup matrix-free levels   (CPU/wall) "
-                 << time() << "s/" << time.wall_time() << "s" << std::endl;
+                 << time.cpu_time() << "s/" << time.wall_time() << "s" << std::endl;
   }
 
 
@@ -936,7 +933,7 @@ namespace Step37
 
     setup_time += time.wall_time();
     time_details << "Assemble right hand side   (CPU/wall) "
-                 << time() << "s/" << time.wall_time() << "s" << std::endl;
+                 << time.cpu_time() << "s/" << time.wall_time() << "s" << std::endl;
   }
 
 
@@ -955,7 +952,7 @@ namespace Step37
     MGTransferMatrixFree<dim,float> mg_transfer(mg_constrained_dofs);
     mg_transfer.build(dof_handler);
     setup_time += time.wall_time();
-    time_details << "MG build transfer time     (CPU/wall) " << time()
+    time_details << "MG build transfer time     (CPU/wall) " << time.cpu_time()
                  << "s/" << time.wall_time() << "s\n";
     time.restart();
 
@@ -1089,13 +1086,14 @@ namespace Step37
     SolverControl solver_control (100, 1e-12*system_rhs.l2_norm());
     SolverCG<LinearAlgebra::distributed::Vector<double> > cg (solver_control);
     setup_time += time.wall_time();
-    time_details << "MG build smoother time     (CPU/wall) " << time()
+    time_details << "MG build smoother time     (CPU/wall) " << time.cpu_time()
                  << "s/" << time.wall_time() << "s\n";
     pcout << "Total setup time               (wall) " << setup_time
           << "s\n";
 
     time.reset();
     time.start();
+    constraints.set_zero(solution);
     cg.solve (system_matrix, solution, system_rhs,
               preconditioner);
 
@@ -1103,7 +1101,7 @@ namespace Step37
 
     pcout << "Time solve ("
           << solver_control.last_step()
-          << " iterations)  (CPU/wall) " << time() << "s/"
+          << " iterations)  (CPU/wall) " << time.cpu_time() << "s/"
           << time.wall_time() << "s\n";
   }
 
@@ -1130,29 +1128,23 @@ namespace Step37
     data_out.add_data_vector (solution, "solution");
     data_out.build_patches ();
 
-    std::ostringstream filename;
-    filename << "solution-"
-             << cycle
-             << "." << Utilities::MPI::this_mpi_process(MPI_COMM_WORLD)
-             << ".vtu";
-
-    std::ofstream output (filename.str().c_str());
+    std::ofstream output ("solution-"
+                          + std::to_string(cycle)
+                          + "."
+                          + std::to_string(Utilities::MPI::this_mpi_process(MPI_COMM_WORLD))
+                          + ".vtu");
     data_out.write_vtu (output);
 
     if (Utilities::MPI::this_mpi_process(MPI_COMM_WORLD) == 0)
       {
         std::vector<std::string> filenames;
         for (unsigned int i=0; i<Utilities::MPI::n_mpi_processes(MPI_COMM_WORLD); ++i)
-          {
-            std::ostringstream filename;
-            filename << "solution-"
-                     << cycle
-                     << "."
-                     << i
-                     << ".vtu";
+          filenames.emplace_back("solution-"
+                                 + std::to_string(cycle)
+                                 + "."
+                                 + std::to_string(i)
+                                 + ".vtu");
 
-            filenames.push_back(filename.str().c_str());
-          }
         std::string master_name = "solution-" + Utilities::to_string(cycle) + ".pvtu";
         std::ofstream master_output (master_name.c_str());
         data_out.write_pvtu_record (master_output, filenames);
@@ -1166,9 +1158,22 @@ namespace Step37
   // The function that runs the program is very similar to the one in
   // step-16. We do few refinement steps in 3D compared to 2D, but that's
   // it.
+  //
+  // Before we run the program, we output some information about the detected
+  // vectorization level as discussed in the introduction.
   template <int dim>
   void LaplaceProblem<dim>::run ()
   {
+    {
+      const unsigned int n_vect_doubles = VectorizedArray<double>::n_array_elements;
+      const unsigned int n_vect_bits = 8*sizeof(double)*n_vect_doubles;
+
+      pcout << "Vectorization over " << n_vect_doubles
+            << " doubles = " << n_vect_bits << " bits ("
+            << Utilities::System::get_current_vectorization_level()
+            << "), VECTORIZATION_LEVEL=" << DEAL_II_COMPILER_VECTORIZATION_LEVEL << std::endl;
+    }
+
     for (unsigned int cycle=0; cycle<9-dim; ++cycle)
       {
         pcout << "Cycle " << cycle << std::endl;
