@@ -1,6 +1,6 @@
 /* ---------------------------------------------------------------------
  *
- * Copyright (C) 2012 - 2017 by the deal.II authors
+ * Copyright (C) 2012 - 2018 by the deal.II authors
  *
  * This file is part of the deal.II library.
  *
@@ -41,7 +41,7 @@
 #include <deal.II/lac/trilinos_sparse_matrix.h>
 #include <deal.II/lac/trilinos_block_sparse_matrix.h>
 #include <deal.II/lac/trilinos_vector.h>
-#include <deal.II/lac/trilinos_block_vector.h>
+#include <deal.II/lac/trilinos_parallel_block_vector.h>
 #include <deal.II/lac/trilinos_precondition.h>
 #include <deal.II/lac/trilinos_solver.h>
 
@@ -74,9 +74,11 @@
 #include <fstream>
 #include <iostream>
 
-// This final include file provides the <code>mkdir</code> function
-// that we will use to create a directory for output files, if necessary:
+// Finally, we include two system headers that let us create a directory for
+// output files. The first header provides the <code>mkdir</code> function and
+// the second lets us determine what happened if <code>mkdir</code> fails.
 #include <sys/stat.h>
+#include <errno.h>
 
 namespace Step42
 {
@@ -751,7 +753,7 @@ namespace Step42
     // one shot at setting it; the same is true for the mesh refinement
     // criterion):
     const std::string                                  base_mesh;
-    const std_cxx11::shared_ptr<const Function<dim> >  obstacle;
+    const std::shared_ptr<const Function<dim> >  obstacle;
 
     struct RefinementStrategy
     {
@@ -876,8 +878,13 @@ namespace Step42
     output_dir = prm.get("output directory");
     if (output_dir != "" && *(output_dir.rbegin()) != '/')
       output_dir += "/";
-    const int ierr = mkdir(output_dir.c_str(), 0777);
-    AssertThrow(ierr == 0, ExcIO());
+
+    // If necessary, create a new directory for the output.
+    if (Utilities::MPI::this_mpi_process(mpi_communicator) == 0)
+      {
+        const int ierr = mkdir(output_dir.c_str(), 0777);
+        AssertThrow(ierr == 0 || errno == EEXIST, ExcIO());
+      }
 
     pcout << "    Using output directory '" << output_dir << "'" << std::endl;
     pcout << "    FE degree " << fe_degree << std::endl;
@@ -916,11 +923,14 @@ namespace Step42
         const Point<dim> center(0, 0, 0);
         const double radius = 0.8;
         GridGenerator::half_hyper_ball(triangulation, center, radius);
+        // Since we will attach a different manifold below, we immediately
+        // clear the default manifold description:
+        triangulation.reset_all_manifolds();
 
         GridTools::transform(&rotate_half_sphere, triangulation);
         GridTools::shift(Point<dim>(0.5, 0.5, 0.5), triangulation);
 
-        static SphericalManifold<dim> manifold_description(Point<dim>(0.5, 0.5, 0.5));
+        SphericalManifold<dim> manifold_description(Point<dim>(0.5, 0.5, 0.5));
         GridTools::copy_boundary_to_manifold_id(triangulation);
         triangulation.set_manifold(0, manifold_description);
       }
@@ -1132,7 +1142,7 @@ namespace Step42
   // element with a set of interpolation points derived from the same quadrature
   // formula. The remainder of the function is relatively straightforward: we
   // put the resulting matrix into the given argument; because we know the
-  // matrix is diagonal, it is sufficient to have a loop over only $i$ not
+  // matrix is diagonal, it is sufficient to have a loop over only $i$ and
   // not over $j$. Strictly speaking, we could even avoid multiplying the
   // shape function's values at quadrature point <code>q_point</code> by itself
   // because we know the shape value to be a vector with exactly one one which
@@ -1619,10 +1629,9 @@ namespace Step42
   //   will have an inhomogeneity and will look like $x_0 = x_1/2 + \text{gap}/2$.
   //   So the corresponding entries in the
   //   ride-hang-side are non-zero with a
-  //   meaningless value. These values we have to
-  //   to set to zero.
+  //   meaningless value. These values we have to set to zero.
   // - Like in step-40, we need to shuffle between vectors that do and do
-  //   do not have ghost elements when solving or using the solution.
+  //   not have ghost elements when solving or using the solution.
   //
   // The rest of the function is similar to step-40 and
   // step-41 except that we use a BiCGStab solver
@@ -1846,7 +1855,6 @@ namespace Step42
             old_solution = solution;
           }
 
-        old_active_set = active_set;
         previous_residual_norm = residual_norm;
 
 
@@ -1861,6 +1869,8 @@ namespace Step42
             if (residual_norm < 1e-10)
               break;
           }
+
+        old_active_set = active_set;
       }
   }
 
