@@ -1,6 +1,6 @@
 /* ---------------------------------------------------------------------
  *
- * Copyright (C) 2010 - 2017 by the deal.II authors
+ * Copyright (C) 2010 - 2018 by the deal.II authors
  *
  * This file is part of the deal.II library.
  *
@@ -32,7 +32,6 @@
 #include <deal.II/base/utilities.h>
 #include <deal.II/base/function.h>
 #include <deal.II/base/tensor_function.h>
-#include <deal.II/base/std_cxx11/shared_ptr.h>
 #include <deal.II/base/index_set.h>
 
 #include <deal.II/lac/full_matrix.h>
@@ -63,12 +62,12 @@
 #include <deal.II/lac/trilinos_sparse_matrix.h>
 #include <deal.II/lac/trilinos_block_sparse_matrix.h>
 #include <deal.II/lac/trilinos_vector.h>
-#include <deal.II/lac/trilinos_block_vector.h>
+#include <deal.II/lac/trilinos_parallel_block_vector.h>
 #include <deal.II/lac/trilinos_precondition.h>
 
 #include <iostream>
 #include <fstream>
-#include <sstream>
+#include <memory>
 
 
 // At the end of this top-matter, we open a namespace for the current project
@@ -591,8 +590,8 @@ namespace Step43
     const double                         porosity;
     const double                         AOS_threshold;
 
-    std_cxx11::shared_ptr<TrilinosWrappers::PreconditionIC> Amg_preconditioner;
-    std_cxx11::shared_ptr<TrilinosWrappers::PreconditionIC> Mp_preconditioner;
+    std::shared_ptr<TrilinosWrappers::PreconditionIC> Amg_preconditioner;
+    std::shared_ptr<TrilinosWrappers::PreconditionIC> Mp_preconditioner;
 
     bool                                rebuild_saturation_matrix;
 
@@ -981,12 +980,10 @@ namespace Step43
   {
     assemble_darcy_preconditioner ();
 
-    Amg_preconditioner = std_cxx11::shared_ptr<TrilinosWrappers::PreconditionIC>
-                         (new TrilinosWrappers::PreconditionIC());
+    Amg_preconditioner = std::make_shared<TrilinosWrappers::PreconditionIC> ();
     Amg_preconditioner->initialize(darcy_preconditioner_matrix.block(0,0));
 
-    Mp_preconditioner = std_cxx11::shared_ptr<TrilinosWrappers::PreconditionIC>
-                        (new TrilinosWrappers::PreconditionIC());
+    Mp_preconditioner = std::make_shared<TrilinosWrappers::PreconditionIC> ();
     Mp_preconditioner->initialize(darcy_preconditioner_matrix.block(1,1));
 
   }
@@ -1824,8 +1821,8 @@ namespace Step43
         }
     }
     std::vector<std::string> joint_solution_names (dim, "velocity");
-    joint_solution_names.push_back ("pressure");
-    joint_solution_names.push_back ("saturation");
+    joint_solution_names.emplace_back ("pressure");
+    joint_solution_names.emplace_back ("saturation");
 
     std::vector<DataComponentInterpretation::DataComponentInterpretation>
     data_component_interpretation
