@@ -1,6 +1,6 @@
 /* ---------------------------------------------------------------------
  *
- * Copyright (C) 2008 - 2017 by the deal.II authors
+ * Copyright (C) 2008 - 2018 by the deal.II authors
  *
  * This file is part of the deal.II library.
  *
@@ -48,7 +48,6 @@
 #include <deal.II/lac/block_sparsity_pattern.h>
 
 #include <deal.II/grid/grid_generator.h>
-#include <deal.II/grid/tria_boundary_lib.h>
 #include <deal.II/grid/grid_tools.h>
 
 #include <deal.II/dofs/dof_renumbering.h>
@@ -85,7 +84,6 @@ namespace Step45
 
     MPI_Comm                                    mpi_communicator;
 
-    HyperShellBoundary<dim>                     boundary;
     parallel::distributed::Triangulation<dim>   triangulation;
     FESystem<dim>                               fe;
     DoFHandler<dim>                             dof_handler;
@@ -294,7 +292,7 @@ namespace Step45
     mpi_communicator (MPI_COMM_WORLD),
     triangulation (mpi_communicator),
     fe (FE_Q<dim>(degree+1), dim,
-        FE_Q<dim>(degree)  , 1),
+        FE_Q<dim>(degree), 1),
     dof_handler (triangulation),
     pcout (std::cout,
            Utilities::MPI::this_mpi_process(mpi_communicator) == 0),
@@ -350,9 +348,6 @@ namespace Step45
 // parallel::distributed::Triangulation::add_periodicity.
     triangulation.add_periodicity(periodicity_vector);
 
-    triangulation.set_boundary(0, boundary);
-    triangulation.set_boundary(1, boundary);
-
     triangulation.refine_global (4-dim);
   }
 
@@ -657,7 +652,7 @@ namespace Step45
   StokesProblem<dim>::output_results (const unsigned int refinement_cycle)  const
   {
     std::vector<std::string> solution_names (dim, "velocity");
-    solution_names.push_back ("pressure");
+    solution_names.emplace_back ("pressure");
 
     std::vector<DataComponentInterpretation::DataComponentInterpretation>
     data_component_interpretation
@@ -676,14 +671,11 @@ namespace Step45
     data_out.add_data_vector (subdomain, "subdomain");
     data_out.build_patches (mapping, degree+1);
 
-    std::ostringstream filename;
-    filename << "solution-"
-             << Utilities::int_to_string (refinement_cycle, 2)
-             << "."
-             << Utilities::int_to_string (triangulation.locally_owned_subdomain(),2)
-             << ".vtu";
-
-    std::ofstream output (filename.str().c_str());
+    std::ofstream output ("solution-"
+                          + Utilities::int_to_string(refinement_cycle, 2)
+                          + "."
+                          + Utilities::int_to_string (triangulation.locally_owned_subdomain(),2)
+                          + ".vtu");
     data_out.write_vtu (output);
 
     if (Utilities::MPI::this_mpi_process(MPI_COMM_WORLD) == 0)
