@@ -1,6 +1,6 @@
 /* ---------------------------------------------------------------------
  *
- * Copyright (C) 2011 - 2015 by the deal.II authors
+ * Copyright (C) 2011 - 2018 by the deal.II authors
  *
  * This file is part of the deal.II library.
  *
@@ -60,7 +60,6 @@
 
 #include <iostream>
 #include <fstream>
-#include <sstream>
 
 
 namespace Step46
@@ -399,7 +398,7 @@ namespace Step46
       const FEValuesExtractors::Vector displacements(dim+1);
       VectorTools::interpolate_boundary_values (dof_handler,
                                                 0,
-                                                ZeroFunction<dim>(dim+1+dim),
+                                                Functions::ZeroFunction<dim>(dim+1+dim),
                                                 constraints,
                                                 fe_collection.component_mask(displacements));
     }
@@ -889,9 +888,9 @@ namespace Step46
   output_results (const unsigned int refinement_cycle)  const
   {
     std::vector<std::string> solution_names (dim, "velocity");
-    solution_names.push_back ("pressure");
+    solution_names.emplace_back("pressure");
     for (unsigned int d=0; d<dim; ++d)
-      solution_names.push_back ("displacement");
+      solution_names.emplace_back("displacement");
 
     std::vector<DataComponentInterpretation::DataComponentInterpretation>
     data_component_interpretation
@@ -910,12 +909,9 @@ namespace Step46
                               data_component_interpretation);
     data_out.build_patches ();
 
-    std::ostringstream filename;
-    filename << "solution-"
-             << Utilities::int_to_string (refinement_cycle, 2)
-             << ".vtk";
-
-    std::ofstream output (filename.str().c_str());
+    std::ofstream output ("solution-"
+                          + Utilities::int_to_string(refinement_cycle, 2)
+                          + ".vtk");
     data_out.write_vtk (output);
   }
 
