@@ -1,6 +1,6 @@
 /* ---------------------------------------------------------------------
  *
- * Copyright (C) 1999 - 2016 by the deal.II authors
+ * Copyright (C) 1999 - 2018 by the deal.II authors
  *
  * This file is part of the deal.II library.
  *
@@ -54,9 +54,7 @@
 // This is C++ ...
 #include <fstream>
 #include <iostream>
-// ... and this is too: We will convert integers to strings using the C++
-// stringstream class <code>ostringstream</code>:
-#include <sstream>
+
 
 // Finally, this has been discussed in previous tutorial programs before:
 using namespace dealii;
@@ -190,10 +188,7 @@ void Step5<dim>::assemble_system ()
   // change in this part, compared to step-4, is that we will use the
   // <code>coefficient</code> function defined above to compute the
   // coefficient value at each quadrature point.
-  typename DoFHandler<dim>::active_cell_iterator
-  cell = dof_handler.begin_active(),
-  endc = dof_handler.end();
-  for (; cell!=endc; ++cell)
+  for (const auto &cell: dof_handler.active_cell_iterators())
     {
       cell_matrix = 0;
       cell_rhs = 0;
@@ -235,7 +230,7 @@ void Step5<dim>::assemble_system ()
   std::map<types::global_dof_index,double> boundary_values;
   VectorTools::interpolate_boundary_values (dof_handler,
                                             0,
-                                            ZeroFunction<dim>(),
+                                            Functions::ZeroFunction<dim>(),
                                             boundary_values);
   MatrixTools::apply_boundary_values (boundary_values,
                                       system_matrix,
@@ -347,28 +342,10 @@ void Step5<dim>::output_results (const unsigned int cycle) const
   // Finally, we need the filename to which the results are to be written. We
   // would like to have it of the form <code>solution-N.eps</code>, where N is
   // the number of the refinement cycle. Thus, we have to convert an integer
-  // to a part of a string; this can be done using the <code>sprintf</code>
-  // function, but in C++ there is a more elegant way: write everything into a
-  // special stream (just like writing into a file or to the screen) and
-  // retrieve what you wrote as a string. This applies the usual conversions
-  // from integer to strings, and one could as well use stream modifiers such
-  // as <code>setw</code>, <code>setprecision</code>, and so on. In C++, you
-  // can do this by using the so-called stringstream classes:
-  std::ostringstream filename;
-
-  // In order to now actually generate a filename, we fill the stringstream
-  // variable with the base of the filename, then the number part, and finally
-  // the suffix indicating the file type:
-  filename << "solution-"
-           << cycle
-           << ".eps";
-
-  // We can get whatever we wrote to the stream using the <code>str()</code>
-  // function. The result is a string which we have to convert to a char*
-  // using the <code>c_str()</code> function. Use that as filename for the
-  // output stream and then write the data to the file:
-  std::ofstream output (filename.str().c_str());
-
+  // to a part of a string; this is most easily done using the C++ function
+  // <code>std::to_string</code>. With the so-constructed filename, we can
+  // then open an output stream and write the data to that file:
+  std::ofstream output ("solution-" + std::to_string(cycle) + ".eps");
   data_out.write_eps (output);
 }
 
@@ -416,11 +393,13 @@ void Step5<dim>::run ()
   // the condition which is given as first argument is valid, and if not
   // throws an exception (its second argument) which will usually terminate
   // the program giving information where the error occurred and what the
-  // reason was. This generally reduces the time to find programming errors
+  // reason was. (A longer discussion of what exactly the @p Assert macro
+  // does can be found in the @ref Exceptions "exception documentation module".)
+  // This generally reduces the time to find programming errors
   // dramatically and we have found assertions an invaluable means to program
   // fast.
   //
-  // On the other hand, all these checks (there are over 9000 of them in the
+  // On the other hand, all these checks (there are over 10,000 of them in the
   // library at present) should not slow down the program too much if you want
   // to do large computations. To this end, the <code>Assert</code> macro is
   // only used in debug mode and expands to nothing if in optimized
@@ -462,12 +441,14 @@ void Step5<dim>::run ()
   // <code>GridIn</code> class to find out what input formats are presently
   // supported.)
 
-  // The grid in the file describes a circle. Therefore we have to use
-  // a manifold object which tells the triangulation where to put new
-  // points on the boundary when the grid is refined. This works in
-  // the same way as in the first example, but in this case we only
-  // set the manifold ids of the boundary.
-  static const SphericalManifold<dim> boundary;
+  // The grid in the file describes a circle. Therefore we have to use a
+  // manifold object which tells the triangulation where to put new points on
+  // the boundary when the grid is refined. Unlike step-1, since GridIn does
+  // not know that the domain has a circular boundary (unlike
+  // GridGenerator::hyper_shell) we have to explicitly attach a manifold to
+  // the boundary after creating the triangulation to get the correct result
+  // when we refine the mesh.
+  const SphericalManifold<dim> boundary;
   triangulation.set_all_manifold_ids_on_boundary(0);
   triangulation.set_manifold (0, boundary);
 
