@@ -1,6 +1,6 @@
 /* ---------------------------------------------------------------------
  *
- * Copyright (C) 2013 - 2016 by the deal.II authors
+ * Copyright (C) 2013 - 2018 by the deal.II authors
  *
  * This file is part of the deal.II library.
  *
@@ -39,7 +39,6 @@
 #include <deal.II/grid/grid_generator.h>
 #include <deal.II/grid/grid_refinement.h>
 #include <deal.II/grid/tria_iterator.h>
-#include <deal.II/grid/tria_boundary_lib.h>
 #include <deal.II/dofs/dof_handler.h>
 #include <deal.II/dofs/dof_accessor.h>
 #include <deal.II/dofs/dof_renumbering.h>
@@ -1034,10 +1033,10 @@ namespace Step51
 
       WorkStream::run(dof_handler_u_post.begin_active(),
                       dof_handler_u_post.end(),
-                      std_cxx11::bind (&HDG<dim>::postprocess_one_cell,
-                                       std_cxx11::ref(*this),
-                                       std_cxx11::_1, std_cxx11::_2, std_cxx11::_3),
-                      std_cxx11::function<void(const unsigned int &)>(),
+                      std::bind (&HDG<dim>::postprocess_one_cell,
+                                 std::ref(*this),
+                                 std::placeholders::_1, std::placeholders::_2, std::placeholders::_3),
+                      std::function<void(const unsigned int &)>(),
                       scratch,
                       0U);
     }
@@ -1212,7 +1211,7 @@ namespace Step51
     // We first define the names and types of the local solution,
     // and add the data to @p data_out.
     std::vector<std::string> names (dim, "gradient");
-    names.push_back ("solution");
+    names.emplace_back("solution");
     std::vector<DataComponentInterpretation::DataComponentInterpretation>
     component_interpretation
     (dim+1, DataComponentInterpretation::component_is_part_of_vector);
