@@ -1,6 +1,6 @@
 /* ---------------------------------------------------------------------
  *
- * Copyright (C) 2014 - 2016 by the deal.II authors
+ * Copyright (C) 2014 - 2018 by the deal.II authors
  *
  * This file is part of the deal.II library.
  *
@@ -39,16 +39,18 @@
 // the Functions::InterpolatedUniformGridData class that is declared in the
 // first of the following header files. Because the data is large, the file we
 // read from is stored as gzip compressed data and we make use of some
-// BOOST-provided functionality to read directly from gzipped data. We wrap
-// the BOOST includes with a preprocessor macro that disables certain annoying
-// compiler warnings.
+// BOOST-provided functionality to read directly from gzipped data.
 #include <deal.II/base/function_lib.h>
 
-DEAL_II_DISABLE_EXTRA_DIAGNOSTICS
 #include <boost/iostreams/filtering_stream.hpp>
 #include <boost/iostreams/filter/gzip.hpp>
 #include <boost/iostreams/device/file.hpp>
-DEAL_II_ENABLE_EXTRA_DIAGNOSTICS
+
+// The last include file is required because we will be using a feature that is
+// not part of the C++11 standard. As some of the C++14 features are very useful,
+// we provide their implementation in an internal namespace, if the compiler
+// does not support them:
+#include <deal.II/base/std_cxx14/memory.h>
 
 #include <iostream>
 #include <fstream>
@@ -87,8 +89,8 @@ namespace Step53
   private:
     const Functions::InterpolatedUniformGridData<2> topography_data;
 
-    static std_cxx11::array<std::pair<double,double>,2> get_endpoints ();
-    static std_cxx11::array<unsigned int,2>             n_intervals ();
+    static std::array<std::pair<double,double>,2> get_endpoints ();
+    static std::array<unsigned int,2>             n_intervals ();
     static std::vector<double>                           get_data ();
   };
 
@@ -128,20 +130,20 @@ namespace Step53
   }
 
 
-  std_cxx11::array<std::pair<double,double>,2>
+  std::array<std::pair<double,double>,2>
   AfricaTopography::get_endpoints ()
   {
-    std_cxx11::array<std::pair<double,double>,2> endpoints;
+    std::array<std::pair<double,double>,2> endpoints;
     endpoints[0] = std::make_pair (-6.983333, 11.966667);
     endpoints[1] = std::make_pair (25, 35.95);
     return endpoints;
   }
 
 
-  std_cxx11::array<unsigned int,2>
+  std::array<unsigned int,2>
   AfricaTopography::n_intervals ()
   {
-    std_cxx11::array<unsigned int,2> endpoints;
+    std::array<unsigned int,2> endpoints;
     endpoints[0] = 379;
     endpoints[1] = 219;
     return endpoints;
@@ -214,11 +216,13 @@ namespace Step53
   public:
     virtual
     Point<3>
-    pull_back(const Point<3> &space_point) const;
+    pull_back(const Point<3> &space_point) const override;
 
     virtual
     Point<3>
-    push_forward(const Point<3> &chart_point) const;
+    push_forward(const Point<3> &chart_point) const override;
+
+    virtual std::unique_ptr<Manifold<3,3> > clone() const override;
 
   private:
     static const double    R;
@@ -255,6 +259,22 @@ namespace Step53
   }
 
 
+  // This function is required by the interface of the Manifold base
+  // class, and allows you to clone the AfricaGeometry class. This is
+  // where we use a feature that is only available in C++14, namely the
+  // make_unique function, that simplifies the creation of
+  // std::unique_ptr objects. Notice that, while the function returns an
+  // std::unique_ptr<Manifold<3,3> >, we internally create a
+  // unique_ptr<AfricaGeometry>. C++11 knows how to handle these cases,
+  // and is able to transform a unique pointer to a derived class to a
+  // unique pointer to its base class automatically:
+  std::unique_ptr<Manifold<3,3> >
+  AfricaGeometry::clone() const
+  {
+    return std_cxx14::make_unique<AfricaGeometry>();
+  }
+
+
   // The following two functions then define the forward and inverse
   // transformations that correspond to the WGS 84 reference shape of
   // Earth. The forward transform follows the formula shown in the
@@ -382,9 +402,9 @@ namespace Step53
                                                  corner_points[0], corner_points[1],
                                                  true);
 
-      GridTools::transform (std_cxx11::bind(&AfricaGeometry::push_forward,
-                                            std_cxx11::cref(geometry),
-                                            std_cxx11::_1),
+      GridTools::transform (std::bind(&AfricaGeometry::push_forward,
+                                      std::cref(geometry),
+                                      std::placeholders::_1),
                             triangulation);
     }
 
