@@ -1,6 +1,6 @@
 /* ---------------------------------------------------------------------
  *
- * Copyright (C) 2009 - 2017 by the deal.II authors
+ * Copyright (C) 2009 - 2018 by the deal.II authors
  *
  * This file is part of the deal.II library.
  *
@@ -27,7 +27,6 @@
 #include <deal.II/grid/grid_generator.h>
 #include <deal.II/grid/grid_in.h>
 #include <deal.II/grid/grid_out.h>
-#include <deal.II/grid/tria_boundary_lib.h>
 #include <deal.II/numerics/data_out.h>
 #include <deal.II/numerics/vector_tools.h>
 
@@ -138,8 +137,8 @@ namespace Step54
   // this function we import the CAD shape upon which we want to
   // generate and refine our triangulation. We assume that the CAD
   // surface is contained in the @p cad_file_name file (we provide an
-  // example IGES file in this directory called
-  // "DTMB-5415_bulbous_bow.iges" that represents the bulbous bow of a
+  // example IGES file in the input directory called
+  // "input/DTMB-5415_bulbous_bow.iges" that represents the bulbous bow of a
   // ship). The presence of several convex and concave high curvature
   // regions makes the geometry we provided a particularly meaningful
   // example.
@@ -259,17 +258,14 @@ namespace Step54
     // point that lies on the wire and splits it into two equal arcs
     // lying between the edge vertices. We first check
     // that the wires vector contains at least one element and then
-    // create a boundary object for it. (The object is marked as
-    // @p static to ensure that it survives past the end of the
-    // current function, as we want to attach it to the triangulation
-    // object for later use).
+    // create a Manifold object for it.
     //
     // Once the projector is created, we then assign it to all the parts of
     // the triangulation with manifold_id = 2:
     Assert(wires.size() > 0,
            ExcMessage("I could not find any wire in the CAD file you gave me. Bailing out."));
 
-    static OpenCASCADE::ArclengthProjectionLineManifold<2,3>
+    OpenCASCADE::ArclengthProjectionLineManifold<2,3>
     line_projector (wires[0], tolerance);
 
     tria.set_manifold(2, line_projector);
@@ -286,12 +282,13 @@ namespace Step54
     switch (surface_projection_kind)
       {
       case NormalProjection:
-        static OpenCASCADE::NormalProjectionBoundary<2,3>
+      {
+        OpenCASCADE::NormalProjectionBoundary<2,3>
         normal_projector(bow_surface, tolerance);
-
         tria.set_manifold(1,normal_projector);
 
         break;
+      }
 
       // @p If surface_projection_kind value is @p DirectionalProjection, we select the
       // OpenCASCADE::DirectionalProjectionBoundary class. The new mesh points will
@@ -301,12 +298,13 @@ namespace Step54
       // OpenCASCADE::DirectionalProjectionBoundary constructor. In this case,
       // the projection is done along the y-axis.
       case DirectionalProjection:
-        static OpenCASCADE::DirectionalProjectionBoundary<2,3>
+      {
+        OpenCASCADE::DirectionalProjectionBoundary<2,3>
         directional_projector(bow_surface, Point<3>(0.0,1.0,0.0), tolerance);
-
         tria.set_manifold(1,directional_projector);
 
         break;
+      }
 
       // As a third option, if @p surface_projection_kind value
       // is @p NormalToMeshProjection, we select the
@@ -318,12 +316,13 @@ namespace Step54
       // requires a shape (containing at least a face) and a
       // tolerance.
       case NormalToMeshProjection:
-        static OpenCASCADE::NormalToMeshProjectionBoundary<2,3>
+      {
+        OpenCASCADE::NormalToMeshProjectionBoundary<2,3>
         normal_to_mesh_projector(bow_surface, tolerance);
-
         tria.set_manifold(1,normal_to_mesh_projector);
 
         break;
+      }
 
       // Finally, we use good software cleanliness by ensuring that this
       // really covers all possible options of the @p case statement. If we
@@ -401,7 +400,7 @@ int main ()
       using namespace Step54;
 
       const std::string in_mesh_filename = "input/initial_mesh_3d.vtk";
-      const std::string cad_file_name = "DTMB-5415_bulbous_bow.iges";
+      const std::string cad_file_name = "input/DTMB-5415_bulbous_bow.iges";
 
       cout << "----------------------------------------------------------" << endl;
       cout << "Testing projection in direction normal to CAD surface" << endl;
