@@ -1,6 +1,6 @@
 /* ---------------------------------------------------------------------
  *
- * Copyright (C) 2016 by the deal.II authors
+ * Copyright (C) 2016 - 2018 by the deal.II authors
  *
  * This file is part of the deal.II library.
  *
@@ -38,7 +38,6 @@
 #include <deal.II/grid/grid_generator.h>
 #include <deal.II/grid/tria_accessor.h>
 #include <deal.II/grid/tria_iterator.h>
-#include <deal.II/grid/tria_boundary_lib.h>
 #include <deal.II/grid/grid_tools.h>
 #include <deal.II/grid/grid_refinement.h>
 
@@ -72,8 +71,8 @@
 #include <deal.II/multigrid/mg_smoother.h>
 #include <deal.II/multigrid/mg_matrix.h>
 
+#include <iostream>
 #include <fstream>
-#include <sstream>
 
 namespace Step56
 {
@@ -347,9 +346,9 @@ namespace Step56
 
 
 
-  template <class PreconditionerA, class PreconditionerMp>
+  template <class PreconditionerAType, class PreconditionerSType>
   void
-  BlockSchurPreconditioner<PreconditionerA, PreconditionerMp>::
+  BlockSchurPreconditioner<PreconditionerAType, PreconditionerSType>::
   vmult (BlockVector<double>       &dst,
          const BlockVector<double> &src) const
   {
@@ -515,7 +514,7 @@ namespace Step56
         // velocity space in a separate DoFHandler as described in the
         // introduction.
         velocity_dof_handler.distribute_dofs(velocity_fe);
-        velocity_dof_handler.distribute_mg_dofs(velocity_fe);
+        velocity_dof_handler.distribute_mg_dofs();
 
         // The following block of code initializes the MGConstrainedDofs
         // (using the boundary conditions for the velocity), and the
@@ -1032,7 +1031,7 @@ namespace Step56
   StokesProblem<dim>::output_results (const unsigned int refinement_cycle)  const
   {
     std::vector<std::string> solution_names (dim, "velocity");
-    solution_names.push_back ("pressure");
+    solution_names.emplace_back("pressure");
 
     std::vector<DataComponentInterpretation::DataComponentInterpretation>
     data_component_interpretation
@@ -1047,12 +1046,9 @@ namespace Step56
                               data_component_interpretation);
     data_out.build_patches ();
 
-    std::ostringstream filename;
-    filename << "solution-"
-             << Utilities::int_to_string (refinement_cycle, 2)
-             << ".vtk";
-
-    std::ofstream output (filename.str().c_str());
+    std::ofstream output ("solution-"
+                          + Utilities::int_to_string(refinement_cycle, 2)
+                          + ".vtk");
     data_out.write_vtk (output);
   }
 
@@ -1157,4 +1153,3 @@ int main ()
 
   return 0;
 }
-
