@@ -1,6 +1,6 @@
 /* ---------------------------------------------------------------------
  *
- * Copyright (C) 2008 - 2017 by the deal.II authors
+ * Copyright (C) 2008 - 2018 by the deal.II authors
  *
  * This file is part of the deal.II library.
  *
@@ -41,7 +41,6 @@
 #include <deal.II/grid/tria_iterator.h>
 #include <deal.II/grid/manifold_lib.h>
 #include <deal.II/grid/grid_refinement.h>
-#include <deal.II/grid/tria_boundary_lib.h>
 #include <deal.II/grid/grid_tools.h>
 
 #include <deal.II/dofs/dof_handler.h>
@@ -70,7 +69,6 @@
 
 #include <fstream>
 #include <iostream>
-#include <sstream>
 
 namespace Step57
 {
@@ -337,7 +335,7 @@ namespace Step57
       DoFTools::make_hanging_node_constraints(dof_handler, zero_constraints);
       VectorTools::interpolate_boundary_values(dof_handler,
                                                0,
-                                               ZeroFunction<dim>(dim+1),
+                                               Functions::ZeroFunction<dim>(dim+1),
                                                zero_constraints,
                                                fe.component_mask(velocities));
     }
@@ -610,7 +608,7 @@ namespace Step57
     BlockVector<double> tmp (dofs_per_block);
 
     // Transfer solution from coarse to fine mesh and apply boundary value
-    // constraints to the new transfered solution. Note that present_solution
+    // constraints to the new transferred solution. Note that present_solution
     // is still a vector corresponding to the old mesh.
     solution_transfer.interpolate(present_solution, tmp);
     nonzero_constraints.distribute(tmp);
@@ -754,12 +752,14 @@ namespace Step57
   }
 
   // @sect4{StationaryNavierStokes::output_results}
-  // This function is the same as in step-22.
+  // This function is the same as in step-22 except that we choose a name
+  // for the output file that also contains the Reynolds number (i.e., the
+  // inverse of the viscosity in the current context).
   template <int dim>
   void StationaryNavierStokes<dim>::output_results (const unsigned int output_index)  const
   {
     std::vector<std::string> solution_names (dim, "velocity");
-    solution_names.push_back ("pressure");
+    solution_names.emplace_back("pressure");
 
     std::vector<DataComponentInterpretation::DataComponentInterpretation>
     data_component_interpretation
@@ -773,13 +773,10 @@ namespace Step57
                               data_component_interpretation);
     data_out.build_patches ();
 
-    std::ostringstream filename;
-    filename << 1.0/viscosity
-             << "-solution-"
-             << Utilities::int_to_string (output_index, 4)
-             << ".vtk";
-
-    std::ofstream output (filename.str().c_str());
+    std::ofstream output (std::to_string(1.0/viscosity)
+                          + "-solution-"
+                          + Utilities::int_to_string (output_index, 4)
+                          + ".vtk");
     data_out.write_vtk (output);
   }
 
@@ -790,10 +787,10 @@ namespace Step57
   template <int dim>
   void StationaryNavierStokes<dim>::process_solution(unsigned int refinement)
   {
-    std::ostringstream filename;
-    filename << (1.0/viscosity) << "-line-" << refinement << ".txt";
-
-    std::ofstream f (filename.str().c_str());
+    std::ofstream f (std::to_string(1.0/viscosity)
+                     + "-line-"
+                     + std::to_string(refinement)
+                     + ".txt");
     f << "# y u_x u_y" << std::endl;
 
     Point<dim> p;
