@@ -1,6 +1,6 @@
 /* ---------------------------------------------------------------------
  *
- * Copyright (C) 2000 - 2016 by the deal.II authors
+ * Copyright (C) 2000 - 2018 by the deal.II authors
  *
  * This file is part of the deal.II library.
  *
@@ -23,29 +23,27 @@
 // The first few files have already been covered in previous examples and will
 // thus not be further commented on.
 #include <deal.II/base/quadrature_lib.h>
-#include <deal.II/base/function.h>
-#include <deal.II/base/logstream.h>
-#include <deal.II/lac/vector.h>
-#include <deal.II/lac/full_matrix.h>
-#include <deal.II/lac/sparse_matrix.h>
-#include <deal.II/lac/dynamic_sparsity_pattern.h>
-#include <deal.II/lac/solver_cg.h>
-#include <deal.II/lac/precondition.h>
-#include <deal.II/grid/tria.h>
+#include <deal.II/base/function_lib.h>
+
 #include <deal.II/dofs/dof_handler.h>
-#include <deal.II/grid/grid_generator.h>
-#include <deal.II/grid/tria_accessor.h>
-#include <deal.II/grid/tria_iterator.h>
-#include <deal.II/grid/manifold_lib.h>
-#include <deal.II/dofs/dof_accessor.h>
 #include <deal.II/dofs/dof_tools.h>
+
 #include <deal.II/fe/fe_values.h>
-#include <deal.II/numerics/vector_tools.h>
-#include <deal.II/numerics/matrix_tools.h>
+
+#include <deal.II/grid/tria.h>
+#include <deal.II/grid/grid_generator.h>
+
+#include <deal.II/lac/dynamic_sparsity_pattern.h>
+#include <deal.II/lac/full_matrix.h>
+#include <deal.II/lac/precondition.h>
+#include <deal.II/lac/solver_cg.h>
+#include <deal.II/lac/sparse_matrix.h>
+#include <deal.II/lac/vector.h>
+
 #include <deal.II/numerics/data_out.h>
+#include <deal.II/numerics/vector_tools.h>
 
 #include <fstream>
-#include <iostream>
 
 // From the following include file we will import the declaration of
 // H1-conforming finite element shape functions. This family of finite
@@ -109,18 +107,22 @@ private:
   void refine_grid ();
   void output_results (const unsigned int cycle) const;
 
-  Triangulation<dim>   triangulation;
+  Triangulation<dim> triangulation;
+
+  FE_Q<dim>          fe;
+  DoFHandler<dim>    dof_handler;
 
-  DoFHandler<dim>      dof_handler;
-  FE_Q<dim>            fe;
 
   // This is the new variable in the main class. We need an object which holds
   // a list of constraints to hold the hanging nodes and the boundary
   // conditions.
   ConstraintMatrix     constraints;
 
-  SparsityPattern      sparsity_pattern;
+  // The sparsity pattern and sparse matrix are deliberately declared in the
+  // opposite of the order used in step-2 through step-5 to demonstrate the
+  // primary use of the Subscriptor and SmartPointer classes.
   SparseMatrix<double> system_matrix;
+  SparsityPattern      sparsity_pattern;
 
   Vector<double>       solution;
   Vector<double>       system_rhs;
@@ -153,97 +155,93 @@ double coefficient (const Point<dim> &p)
 template <int dim>
 Step6<dim>::Step6 ()
   :
-  dof_handler (triangulation),
-  fe (2)
+  fe (2),
+  dof_handler (triangulation)
 {}
 
 
 // @sect4{Step6::~Step6}
 
-// Here comes the added destructor of the class. The reason why we want to add
-// it is a subtle change in the order of data elements in the class as
-// compared to all previous examples: the <code>dof_handler</code> object was
-// defined before and not after the <code>fe</code> object. Of course we could
-// have left this order unchanged, but we would like to show what happens if
-// the order is reversed since this produces a rather nasty side-effect and
-// results in an error which is difficult to track down if one does not know
-// what happens.
+// Here comes the added destructor of the class. Some objects in deal.II store
+// pointers to other objects: in particular a SparseMatrix stores a SmartPointer
+// pointing to the SparsityPattern with which it was initialized. This example
+// deliberately declares the SparseMatrix before the SparsityPattern to make this
+// dependency clearer. Of course we could have left this order unchanged, but
+// we would like to show what happens if the order is reversed since this
+// produces a rather nasty side-effect and results in an error which is
+// difficult to track down if one does not know what happens.
 //
-// Basically what happens is the following: when we distribute the degrees of
-// freedom using the function call <code>dof_handler.distribute_dofs()</code>,
-// the <code>dof_handler</code> also stores a pointer to the finite element in
-// use. Since this pointer is used every now and then until either the degrees
-// of freedom are re-distributed using another finite element object or until
-// the <code>dof_handler</code> object is destroyed, it would be unwise if we
-// would allow the finite element object to be deleted before the
-// <code>dof_handler</code> object. To disallow this, the DoF handler
-// increases a counter inside the finite element object which counts how many
-// objects use that finite element (this is what the
+// What happens is the following: when we initialize a SparseMatrix,
+// the matrix stores a pointer to the provided SparsityPattern instead of
+// copying it.  Since this pointer is used until either another
+// SparsityPattern is attached or the SparseMatrix is destructed, it would be
+// unwise to allow the SparsityPattern to be destructed before the
+// SparseMatrix. To disallow this, the SparseMatrix increases a counter inside
+// the SparsityPattern which counts how many objects use it (this is what the
 // <code>Subscriptor</code>/<code>SmartPointer</code> class pair is used for,
 // in case you want something like this for your own programs; see step-7 for
-// a more complete discussion of this topic). The finite element object will
-// refuse its destruction if that counter is larger than zero, since then some
-// other objects might rely on the persistence of the finite element
-// object. An exception will then be thrown and the program will usually abort
-// upon the attempt to destroy the finite element.
+// a more complete discussion of this topic). If we try to destroy the object
+// while the counter is larger than zero then the program will either abort
+// (the default) or print an error message and continue: see the documentation
+// of AssertNothrow for more details. In either case the program contains a
+// bug and this facility will, hopefully, point out where.
 //
-// To be fair, such exceptions about still used objects are not particularly
+// To be fair, such errors due to object dependencies are not particularly
 // popular among programmers using deal.II, since they only tell us that
-// something is wrong, namely that some other object is still using the object
-// that is presently being destructed, but most of the time not who this user
-// is. It is therefore often rather time-consuming to find out where the
-// problem exactly is, although it is then usually straightforward to remedy
-// the situation. However, we believe that the effort to find invalid
-// references to objects that do no longer exist is less if the problem is
-// detected once the reference becomes invalid, rather than when non-existent
-// objects are actually accessed again, since then usually only invalid data
-// is accessed, but no error is immediately raised.
+// something is wrong, namely that <i>some</i> other object is still using the
+// object that is presently being destroyed, but most of the time not
+// <em>which</em> object is still using it. It is therefore often rather
+// time-consuming to find out where the problem exactly is, although it is
+// then usually straightforward to remedy the situation. However, we believe
+// that the effort to find invalid pointers to objects that no longer exist is
+// less if the problem is detected once the pointer becomes invalid, rather
+// than when non-existent objects are actually accessed again, since then
+// usually only invalid data is accessed, but no error is immediately raised.
 //
 // Coming back to the present situation, if we did not write this destructor,
-// the compiler will generate code that triggers exactly the behavior sketched
-// above. The reason is that member variables of the <code>Step6</code> class
-// are destructed bottom-up (i.e. in reverse order of their declaration in the
-// class), as always in C++. Thus, the finite element object will be
-// destructed before the DoF handler object, since its declaration is below
-// the one of the DoF handler. This triggers the situation above, and an
-// exception will be raised when the <code>fe</code> object is
-// destructed. What needs to be done is to tell the <code>dof_handler</code>
-// object to release its lock to the finite element. Of course, the
-// <code>dof_handler</code> will only release its lock if it really does not
-// need the finite element any more, i.e. when all finite element related data
-// is deleted from it. For this purpose, the <code>DoFHandler</code> class has
-// a function <code>clear</code> which deletes all degrees of freedom, and
-// releases its lock to the finite element. After this, you can safely
-// destruct the finite element object since its internal counter is then zero.
+// then the compiler would generate code that triggers exactly the behavior
+// described above. The reason is that member variables of the
+// <code>Step6</code> class are destroyed bottom-up (i.e., in reverse order of
+// their declaration in the class), as always in C++. Thus, the
+// SparsityPattern will be destroyed before the SparseMatrix, since its
+// declaration is below the declaration of the sparsity pattern. This triggers
+// the situation above, and without manual intervention the program will abort
+// when the SparsityPattern is destroyed. What needs to be done is to
+// tell the SparseMatrix to release its pointer to the SparsityPattern. Of
+// course, the SparseMatrix will only release its pointer if it really does
+// not need the SparsityPattern any more. For this purpose, the SparseMatrix
+// class has a function SparseMatrix::clear() which resets the object to its
+// default-constructed state by deleting all data and resetting its pointer to
+// the SparsityPattern to 0. After this, you can safely destruct the
+// SparsityPattern since its internal counter will be zero.
 //
-// For completeness, we add the output of the exception that would have been
-// triggered without this destructor, to the end of the results section of
-// this example.
+// We show the output of the other case (where we do not call
+// SparseMatrix::clear()) in the results section below.
 template <int dim>
 Step6<dim>::~Step6 ()
 {
-  dof_handler.clear ();
+  system_matrix.clear();
 }
 
 
 // @sect4{Step6::setup_system}
 
-// The next function is setting up all the variables that describe the linear
-// finite element problem, such as the DoF handler, the matrices, and
+// The next function sets up all the variables that describe the linear
+// finite element problem, such as the DoFHandler, matrices, and
 // vectors. The difference to what we did in step-5 is only that we now also
 // have to take care of hanging node constraints. These constraints are
-// handled almost transparently by the library, i.e. you only need to know
+// handled almost exclusively by the library, i.e. you only need to know
 // that they exist and how to get them, but you do not have to know how they
 // are formed or what exactly is done with them.
 //
 // At the beginning of the function, you find all the things that are the same
 // as in step-5: setting up the degrees of freedom (this time we have
 // quadratic elements, but there is no difference from a user code perspective
-// to the linear -- or cubic, for that matter -- case), generating the
-// sparsity pattern, and initializing the solution and right hand side
+// to the linear -- or any other degree, for that matter -- case), generating
+// the sparsity pattern, and initializing the solution and right hand side
 // vectors. Note that the sparsity pattern will have significantly more
-// entries per row now, since there are now 9 degrees of freedom per cell, not
-// only four, that can couple with each other.
+// entries per row now, since there are now 9 degrees of freedom per cell
+// (rather than only four), that can couple with each other.
 template <int dim>
 void Step6<dim>::setup_system ()
 {
@@ -252,43 +250,31 @@ void Step6<dim>::setup_system ()
   solution.reinit (dof_handler.n_dofs());
   system_rhs.reinit (dof_handler.n_dofs());
 
-
-  // After setting up all the degrees of freedoms, here are now the
-  // differences compared to step-5, all of which are related to constraints
-  // associated with the hanging nodes. In the class declaration, we have
-  // already allocated space for an object <code>constraints</code> that will
-  // hold a list of these constraints (they form a matrix, which is reflected
-  // in the name of the class, but that is immaterial for the moment). Now we
-  // have to fill this object. This is done using the following function calls
-  // (the first clears the contents of the object that may still be left over
-  // from computations on the previous mesh before the last adaptive
-  // refinement):
+  // We may now populate the ConstraintMatrix with the hanging node
+  // constraints. Since we will call this function in a loop we first clear
+  // the current set of constraints from the last system and then compute new ones:
   constraints.clear ();
   DoFTools::make_hanging_node_constraints (dof_handler,
                                            constraints);
 
 
-  // Now we are ready to interpolate the ZeroFunction to our boundary with
-  // indicator 0 (the whole boundary) and store the resulting constraints in
-  // our <code>constraints</code> object. Note that we do not to apply the
-  // boundary conditions after assembly, like we did in earlier steps.  As
-  // almost all the stuff, the interpolation of boundary values works also for
-  // higher order elements without the need to change your code for that. We
-  // note that for proper results, it is important that the elimination of
-  // boundary nodes from the system of equations happens *after* the
-  // elimination of hanging nodes. For that reason we are filling the boundary
-  // values into the ContraintMatrix after the hanging node constraints.
+  // Now we are ready to interpolate the boundary values with indicator 0 (the
+  // whole boundary) and store the resulting constraints in our
+  // <code>constraints</code> object. Note that we do not to apply the
+  // boundary conditions after assembly, like we did in earlier steps: instead
+  // we put all constraints on our function space in the ConstraintMatrix. We
+  // can add constraints to the ConstraintMatrix in either order: if two
+  // constraints conflict then the constraint matrix either abort or throw an
+  // exception via the Assert macro.
   VectorTools::interpolate_boundary_values (dof_handler,
                                             0,
-                                            ZeroFunction<dim>(),
+                                            Functions::ZeroFunction<dim>(),
                                             constraints);
 
-
-  // The next step is <code>closing</code> this object. After all constraints
-  // have been added, they need to be sorted and rearranged to perform some
-  // actions more efficiently. This postprocessing is done using the
-  // <code>close()</code> function, after which no further constraints may be
-  // added any more:
+  // After all constraints have been added, they need to be sorted and
+  // rearranged to perform some actions more efficiently. This postprocessing
+  // is done using the <code>close()</code> function, after which no further
+  // constraints may be added any more:
   constraints.close ();
 
   // Now we first build our compressed sparsity pattern like we did in the
@@ -311,12 +297,11 @@ void Step6<dim>::setup_system ()
 
   // Now all non-zero entries of the matrix are known (i.e. those from
   // regularly assembling the matrix and those that were introduced by
-  // eliminating constraints). We can thus copy our intermediate object to the
+  // eliminating constraints). We may copy our intermediate object to the
   // sparsity pattern:
   sparsity_pattern.copy_from(dsp);
 
-  // Finally, the so-constructed sparsity pattern serves as the basis on top
-  // of which we will create the sparse matrix:
+  // We may now, finally, initialize the sparse matrix:
   system_matrix.reinit (sparsity_pattern);
 }
 
@@ -335,8 +320,9 @@ void Step6<dim>::setup_system ()
 //
 // Second, to copy the local matrix and vector on each cell into the global
 // system, we are no longer using a hand-written loop. Instead, we use
-// <code>ConstraintMatrix::distribute_local_to_global</code> that internally
-// executes this loop and eliminates all the constraints at the same time.
+// ConstraintMatrix::distribute_local_to_global() that internally executes
+// this loop while performing Gaussian elimination on rows and columns
+// corresponding to constrained degrees on freedom.
 //
 // The rest of the code that forms the local contributions remains
 // unchanged. It is worth noting, however, that under the hood several things
@@ -347,8 +333,7 @@ void Step6<dim>::setup_system ()
 // code. Secondly, the <code>fe_values</code> object of course needs to do
 // other things as well, since the shape functions are now quadratic, rather
 // than linear, in each coordinate variable. Again, however, this is something
-// that is completely transparent to user code and nothing that you have to
-// worry about.
+// that is completely handled by the library.
 template <int dim>
 void Step6<dim>::assemble_system ()
 {
@@ -406,8 +391,10 @@ void Step6<dim>::assemble_system ()
   // Now we are done assembling the linear system. The constraint matrix took
   // care of applying the boundary conditions and also eliminated hanging node
   // constraints. The constrained nodes are still in the linear system (there
-  // is a one on the diagonal of the matrix and all other entries for this
-  // line are set to zero) but the computed values are invalid. We compute the
+  // is a nonzero entry, chosen in a way that the matrix is well conditioned,
+  // on the diagonal of the matrix and all other entries for this line are set
+  // to zero) but the computed values are invalid (i.e., the correspond entry
+  // in <code>system_rhs</code> is currently meaningless). We compute the
   // correct values for these nodes at the end of the <code>solve</code>
   // function.
 }
@@ -447,8 +434,8 @@ void Step6<dim>::solve ()
 
 // @sect4{Step6::refine_grid}
 
-// Instead of global refinement, we now use a slightly more elaborate
-// scheme. We will use the <code>KellyErrorEstimator</code> class which
+// We use a sophisticated error estimation scheme to refine the mesh instead
+// of global refinement. We will use the KellyErrorEstimator class which
 // implements an error estimator for the Laplace equation; it can in principle
 // handle variable coefficients, but we will not use these advanced features,
 // but rather use its most simple form since we are not interested in
@@ -458,8 +445,8 @@ void Step6<dim>::solve ()
 // Although the error estimator derived by Kelly et al. was originally
 // developed for the Laplace equation, we have found that it is also well
 // suited to quickly generate locally refined grids for a wide class of
-// problems. Basically, it looks at the jumps of the gradients of the solution
-// over the faces of cells (which is a measure for the second derivatives) and
+// problems. This error estimator uses the solution gradient's jump at
+// cell faces (which is a measure for the second derivatives) and
 // scales it by the size of the cell. It is therefore a measure for the local
 // smoothness of the solution at the place of each cell and it is thus
 // understandable that it yields reasonable grids also for hyperbolic
@@ -471,12 +458,11 @@ void Step6<dim>::solve ()
 // The way the estimator works is to take a <code>DoFHandler</code> object
 // describing the degrees of freedom and a vector of values for each degree of
 // freedom as input and compute a single indicator value for each active cell
-// of the triangulation (i.e. one value for each of the
-// <code>triangulation.n_active_cells()</code> cells). To do so, it needs two
-// additional pieces of information: a quadrature formula on the faces
-// (i.e. quadrature formula on <code>dim-1</code> dimensional objects. We use
-// a 3-point Gauss rule again, a pick that is consistent and appropriate with
-// the choice bi-quadratic finite element shape functions in this program.
+// of the triangulation (i.e. one value for each of the active cells). To do
+// so, it needs two additional pieces of information: a face quadrature formula,
+// i.e., a quadrature formula on <code>dim-1</code> dimensional objects. We use
+// a 3-point Gauss rule again, a choice that is consistent and appropriate with
+// the bi-quadratic finite element shape functions in this program.
 // (What constitutes a suitable quadrature rule here of course depends on
 // knowledge of the way the error estimator evaluates the solution field. As
 // said above, the jump of the gradient is integrated over each face, which
@@ -497,7 +483,7 @@ void Step6<dim>::solve ()
 // using the default constructor of the map in the place where the function
 // call expects the respective function argument.
 //
-// The output, as mentioned is a vector of values for all cells. While it may
+// The output is a vector of values for all active cells. While it may
 // make sense to compute the <b>value</b> of a solution degree of freedom
 // very accurately, it is usually not necessary to compute the <b>error indicator</b>
 // corresponding to the solution on a cell particularly accurately. We therefore
@@ -561,40 +547,33 @@ void Step6<dim>::refine_grid ()
 // next cycle with mesh refinement, we want to output the results from this
 // cycle.
 //
-// In the present program, we will not write the solution (except for in the
-// last step, see the next function), but only the meshes that we generated,
-// as a two-dimensional Encapsulated Postscript (EPS) file.
+// We have already seen in step-1 how this can be achieved for the
+// mesh itself. The only thing we have to change is the generation of
+// the file name, since it should contain the number of the present
+// refinement cycle provided to this function as an argument. To this
+// end, we simply append the number of the refinement cycle as a
+// string to the file name.
 //
-// We have already seen in step-1 how this can be achieved. The only thing we
-// have to change is the generation of the file name, since it should contain
-// the number of the present refinement cycle provided to this function as an
-// argument. The most general way is to use the std::stringstream class as
-// shown in step-5, but here's a little hack that makes it simpler if we know
-// that we have less than 10 iterations: assume that the %numbers `0' through
-// `9' are represented consecutively in the character set used on your machine
-// (this is in fact the case in all known character sets), then '0'+cycle
-// gives the character corresponding to the present cycle number. Of course,
-// this will only work if the number of cycles is actually less than 10, and
-// rather than waiting for the disaster to happen, we safeguard our little
-// hack with an explicit assertion at the beginning of the function. If this
-// assertion is triggered, i.e. when <code>cycle</code> is larger than or
-// equal to 10, an exception of type <code>ExcNotImplemented</code> is raised,
-// indicating that some functionality is not implemented for this case (the
-// functionality that is missing, of course, is the generation of file names
-// for that case):
+// We also output the solution in the same way as we did before, with
+// a similarly constructed file name.
 template <int dim>
 void Step6<dim>::output_results (const unsigned int cycle) const
 {
-  Assert (cycle < 10, ExcNotImplemented());
+  {
+    GridOut grid_out;
+    std::ofstream output ("grid-" + std::to_string(cycle) + ".eps");
+    grid_out.write_eps (triangulation, output);
+  }
 
-  std::string filename = "grid-";
-  filename += ('0' + cycle);
-  filename += ".eps";
+  {
+    DataOut<dim> data_out;
+    data_out.attach_dof_handler (dof_handler);
+    data_out.add_data_vector (solution, "solution");
+    data_out.build_patches ();
 
-  std::ofstream output (filename.c_str());
-
-  GridOut grid_out;
-  grid_out.write_eps (triangulation, output);
+    std::ofstream output ("solution-" + std::to_string(cycle) + ".vtk");
+    data_out.write_vtk (output);
+  }
 }
 
 
@@ -634,11 +613,6 @@ void Step6<dim>::run ()
       if (cycle == 0)
         {
           GridGenerator::hyper_ball (triangulation);
-
-          static const SphericalManifold<dim> boundary;
-          triangulation.set_all_manifold_ids_on_boundary(0);
-          triangulation.set_manifold (0, boundary);
-
           triangulation.refine_global (1);
         }
       else
@@ -659,24 +633,6 @@ void Step6<dim>::run ()
       solve ();
       output_results (cycle);
     }
-
-  // After we have finished computing the solution on the finest mesh, and
-  // writing all the grids to disk, we want to also write the actual solution
-  // on this final mesh to a file. As already done in one of the previous
-  // examples, we use the EPS format for output, and to obtain a reasonable
-  // view on the solution, we rescale the z-axis by a factor of four.
-  DataOutBase::EpsFlags eps_flags;
-  eps_flags.z_scaling = 4;
-
-  DataOut<dim> data_out;
-  data_out.set_flags (eps_flags);
-
-  data_out.attach_dof_handler (dof_handler);
-  data_out.add_data_vector (solution, "solution");
-  data_out.build_patches ();
-
-  std::ofstream output ("final-solution.eps");
-  data_out.write_eps (output);
 }
 
 
