@@ -1,6 +1,6 @@
 /* ---------------------------------------------------------------------
  *
- * Copyright (C) 2000 - 2016 by the deal.II authors
+ * Copyright (C) 2000 - 2018 by the deal.II authors
  *
  * This file is part of the deal.II library.
  *
@@ -297,7 +297,7 @@ namespace Step8
     // value 1.0. Although we could omit the respective factors in the
     // assemblage of the matrix, we use them here for purpose of
     // demonstration.
-    ConstantFunction<dim> lambda(1.), mu(1.);
+    Functions::ConstantFunction<dim> lambda(1.), mu(1.);
 
     // Like the two constant functions above, we will call the function
     // right_hand_side just once per cell to make things simpler.
@@ -434,17 +434,17 @@ namespace Step8
 
     // The interpolation of the boundary values needs a small modification:
     // since the solution function is vector-valued, so need to be the
-    // boundary values. The <code>ZeroFunction</code> constructor accepts a
+    // boundary values. The <code>Functions::ZeroFunction</code> constructor accepts a
     // parameter that tells it that it shall represent a vector valued,
     // constant zero function with that many components. By default, this
-    // parameter is equal to one, in which case the <code>ZeroFunction</code>
+    // parameter is equal to one, in which case the <code>Functions::ZeroFunction</code>
     // object would represent a scalar function. Since the solution vector has
     // <code>dim</code> components, we need to pass <code>dim</code> as number
     // of components to the zero function as well.
     std::map<types::global_dof_index,double> boundary_values;
     VectorTools::interpolate_boundary_values (dof_handler,
                                               0,
-                                              ZeroFunction<dim>(dim),
+                                              Functions::ZeroFunction<dim>(dim),
                                               boundary_values);
     MatrixTools::apply_boundary_values (boundary_values,
                                         system_matrix,
@@ -514,70 +514,63 @@ namespace Step8
   // valued. The <code>DataOut</code> class takes care of this automatically,
   // but we have to give each component of the solution vector a different
   // name.
+  //
+  // To do this, the DataOut::add_vector() function wants a vector of
+  // strings. Since the number of components is the same as the number
+  // of dimensions we are working in, we use the <code>switch</code>
+  // statement below.
+  //
+  // We note that some graphics programs have restriction on what
+  // characters are allowed in the names of variables. deal.II therefore
+  // supports only the minimal subset of these characters that is supported
+  // by all programs. Basically, these are letters, numbers, underscores,
+  // and some other characters, but in particular no whitespace and
+  // minus/hyphen. The library will throw an exception otherwise, at least
+  // if in debug mode.
+  //
+  // After listing the 1d, 2d, and 3d case, it is good style to let the
+  // program die if we run upon a case which we did not consider. Remember
+  // that the <code>Assert</code> macro generates an exception if the
+  // condition in the first parameter is not satisfied. Of course, the
+  // condition <code>false</code> can never be satisfied, so the program
+  // will always abort whenever it gets to the default statement:
   template <int dim>
   void ElasticProblem<dim>::output_results (const unsigned int cycle) const
   {
-    std::string filename = "solution-";
-    filename += ('0' + cycle);
-    Assert (cycle < 10, ExcInternalError());
-
-    filename += ".vtk";
-    std::ofstream output (filename.c_str());
-
     DataOut<dim> data_out;
     data_out.attach_dof_handler (dof_handler);
 
-
-
-    // As said above, we need a different name for each component of the
-    // solution function. To pass one name for each component, a vector of
-    // strings is used. Since the number of components is the same as the
-    // number of dimensions we are working in, the following
-    // <code>switch</code> statement is used.
-    //
-    // We note that some graphics programs have restriction as to what
-    // characters are allowed in the names of variables. The library therefore
-    // supports only the minimal subset of these characters that is supported
-    // by all programs. Basically, these are letters, numbers, underscores,
-    // and some other characters, but in particular no whitespace and
-    // minus/hyphen. The library will throw an exception otherwise, at least
-    // if in debug mode.
-    //
-    // After listing the 1d, 2d, and 3d case, it is good style to let the
-    // program die if we run upon a case which we did not consider. Remember
-    // that the <code>Assert</code> macro generates an exception if the
-    // condition in the first parameter is not satisfied. Of course, the
-    // condition <code>false</code> can never be satisfied, so the program
-    // will always abort whenever it gets to the default statement:
     std::vector<std::string> solution_names;
     switch (dim)
       {
       case 1:
-        solution_names.push_back ("displacement");
+        solution_names.emplace_back("displacement");
         break;
       case 2:
-        solution_names.push_back ("x_displacement");
-        solution_names.push_back ("y_displacement");
+        solution_names.emplace_back("x_displacement");
+        solution_names.emplace_back("y_displacement");
         break;
       case 3:
-        solution_names.push_back ("x_displacement");
-        solution_names.push_back ("y_displacement");
-        solution_names.push_back ("z_displacement");
+        solution_names.emplace_back("x_displacement");
+        solution_names.emplace_back("y_displacement");
+        solution_names.emplace_back("z_displacement");
         break;
       default:
         Assert (false, ExcNotImplemented());
       }
 
-    // After setting up the names for the different components of the solution
-    // vector, we can add the solution vector to the list of data vectors
-    // scheduled for output. Note that the following function takes a vector
-    // of strings as second argument, whereas the one which we have used in
-    // all previous examples accepted a string there. In fact, the latter
-    // function is only a shortcut for the function which we call here: it
-    // puts the single string that is passed to it into a vector of strings
-    // with only one element and forwards that to the other function.
+    // After setting up the names for the different components of the
+    // solution vector, we can add the solution vector to the list of
+    // data vectors scheduled for output. Note that the following
+    // function takes a vector of strings as second argument, whereas
+    // the one which we have used in all previous examples accepted a
+    // string there. (In fact, the function we had used before would
+    // convert the single string into a vector with only one element
+    // and forwards that to the other function.)
     data_out.add_data_vector (solution, solution_names);
     data_out.build_patches ();
+
+    std::ofstream output ("solution-" + std::to_string(cycle) + ".vtk");
     data_out.write_vtk (output);
   }
 
@@ -596,7 +589,7 @@ namespace Step8
   // will be only four quadrature points in each direction on the
   // domain. However, the right hand side function was chosen to be rather
   // localized and in that case, by pure chance, it happens that all quadrature
-  // points lie at points where the the right hand side function is zero (in
+  // points lie at points where the right hand side function is zero (in
   // mathematical terms, the quadrature points happen to be at points outside
   // the <i>support</i> of the right hand side function). The right hand side
   // vector computed with quadrature will then contain only zeroes (even though
