@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 2000 - 2016 by the deal.II authors
+// Copyright (C) 2000 - 2017 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -13,8 +13,8 @@
 //
 // ---------------------------------------------------------------------
 
-#ifndef dealii__any_data_h
-#define dealii__any_data_h
+#ifndef dealii_any_data_h
+#define dealii_any_data_h
 
 #include <deal.II/base/config.h>
 #include <deal.II/base/exceptions.h>
@@ -37,7 +37,7 @@ class AnyData :
 {
 public:
   /// Default constructor for empty object
-  AnyData();
+  AnyData() = default;
 
   /// Number of stored data objects.
   unsigned int size() const;
@@ -194,11 +194,6 @@ private:
 };
 
 
-inline
-AnyData::AnyData()
-{}
-
-
 unsigned int
 inline
 AnyData::size () const
@@ -215,7 +210,7 @@ AnyData::entry (const unsigned int i)
 {
   AssertIndexRange(i, size());
   type *p = boost::any_cast<type>(&data[i]);
-  Assert(p != 0,
+  Assert(p != nullptr,
          ExcTypeMismatch(typeid(type).name(),data[i].type().name()));
   return *p;
 }
@@ -228,9 +223,9 @@ AnyData::entry (const unsigned int i) const
 {
   AssertIndexRange(i, size());
   const type *p = boost::any_cast<type>(&data[i]);
-  if (p==0 )
+  if (p==nullptr )
     p = boost::any_cast<const type>(&data[i]);
-  Assert(p != 0,
+  Assert(p != nullptr,
          ExcTypeMismatch(typeid(type).name(),data[i].type().name()));
   return *p;
 }
@@ -243,9 +238,9 @@ AnyData::read(const unsigned int i) const
 {
   AssertIndexRange(i, size());
   const type *p = boost::any_cast<type>(&data[i]);
-  if (p==0)
+  if (p==nullptr)
     p = boost::any_cast<const type>(&data[i]);
-  Assert(p != 0,
+  Assert(p != nullptr,
          ExcTypeMismatch(typeid(type).name(),data[i].type().name()));
   return *p;
 }
@@ -258,9 +253,9 @@ AnyData::read_ptr(const unsigned int i) const
 {
   AssertIndexRange(i, size());
   const type *const *p = boost::any_cast<type *>(&data[i]);
-  if (p==0)
+  if (p==nullptr)
     p = boost::any_cast<const type *>(&data[i]);
-  Assert(p != 0,
+  Assert(p != nullptr,
          ExcTypeMismatch(typeid(type *).name(),data[i].type().name()));
   return *p;
 }
@@ -273,10 +268,10 @@ AnyData::try_read_ptr(const unsigned int i) const
 {
   AssertIndexRange(i, size());
   const type *const *p = boost::any_cast<type *>(&data[i]);
-  if (p==0)
+  if (p==nullptr)
     p = boost::any_cast<const type *>(&data[i]);
-  if (p==0)
-    return 0;
+  if (p==nullptr)
+    return nullptr;
   return *p;
 }
 
@@ -357,7 +352,7 @@ AnyData::entry (const std::string &n) const
 {
   const unsigned int i = find(n);
   const type *p = boost::any_cast<type>(&data[i]);
-  Assert(p != 0,
+  Assert(p != nullptr,
          ExcTypeMismatch(typeid(type).name(),data[i].type().name()));
   return *p;
 }
@@ -383,9 +378,9 @@ AnyData::read_ptr(const std::string &n) const
 {
   const unsigned int i = find(n);
   const type *const *p = boost::any_cast<type *>(&data[i]);
-  if (p==0)
+  if (p==nullptr)
     p = boost::any_cast<const type *>(&data[i]);
-  Assert(p != 0,
+  Assert(p != nullptr,
          ExcTypeMismatch(typeid(type).name(),data[i].type().name()));
   return *p;
 }
@@ -417,7 +412,7 @@ AnyData::try_read(const std::string &n) const
     std::find(names.begin(), names.end(), n);
   // Return null pointer if not found
   if (it == names.end())
-    return 0;
+    return nullptr;
 
   // Compute index and return casted pointer
   unsigned int i=it-names.begin();
@@ -470,13 +465,3 @@ void AnyData::list(StreamType &os) const
 DEAL_II_NAMESPACE_CLOSE
 
 #endif
-
-
-
-
-
-
-
-
-
-
