@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 2010 - 2015 by the deal.II authors
+// Copyright (C) 2010 - 2018 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -14,8 +14,8 @@
 // ---------------------------------------------------------------------
 
 
-#ifndef dealii__operator_h
-#define dealii__operator_h
+#ifndef dealii_operator_h
+#define dealii_operator_h
 
 #include <deal.II/base/config.h>
 #include <deal.II/algorithms/any_data.h>
@@ -73,7 +73,7 @@ namespace Algorithms
     /**
      * The virtual destructor.
      */
-    ~OperatorBase();
+    virtual ~OperatorBase() = default;
 
     /**
      * The actual operation, which is implemented in a derived class.
@@ -106,13 +106,22 @@ namespace Algorithms
   template <typename VectorType>
   class OutputOperator : public Subscriptor
   {
-    OutputOperator(const OutputOperator<VectorType> &);
   public:
+    /**
+     * Constructor initializing member variables with invalid data.
+     */
     OutputOperator ();
+
+    /**
+     * The copy constructor is deleted since objects of this class
+     * should not be copied.
+     */
+    OutputOperator(const OutputOperator<VectorType> &) = delete;
+
     /**
      * Empty virtual destructor.
      */
-    virtual ~OutputOperator();
+    virtual ~OutputOperator() = default;
 
     /**
      * Set the stream @p os to which data is written. If no stream is selected
@@ -146,7 +155,7 @@ namespace Algorithms
   /**
    * Set the step number in OutputOperator by shifting an integer value.
    *
-   * @relates OutputOperator
+   * @relatesalso OutputOperator
    */
   template <typename VectorType>
   inline
