@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 2010 - 2016 by the deal.II authors
+// Copyright (C) 2010 - 2017 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -14,8 +14,8 @@
 // ---------------------------------------------------------------------
 
 
-#ifndef dealii__theta_timestepping_h
-#define dealii__theta_timestepping_h
+#ifndef dealii_theta_timestepping_h
+#define dealii_theta_timestepping_h
 
 #include <deal.II/base/smartpointer.h>
 #include <deal.II/algorithms/operator.h>
@@ -236,10 +236,7 @@ namespace Algorithms
      * The current time in the timestepping scheme.
      */
     double current_time() const;
-    /**
-     * The current step size.
-     */
-    double step_size() const;
+
     /**
      * The weight between implicit and explicit part.
      */
