@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 2011 - 2017 by the deal.II authors
+// Copyright (C) 2011 - 2018 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -14,11 +14,10 @@
 // ---------------------------------------------------------------------
 
 
-#ifndef dealii__aligned_vector_h
-#define dealii__aligned_vector_h
+#ifndef dealii_aligned_vector_h
+#define dealii_aligned_vector_h
 
 #include <deal.II/base/config.h>
-#include <deal.II/base/std_cxx11/type_traits.h>
 #include <deal.II/base/exceptions.h>
 #include <deal.II/base/memory_consumption.h>
 #include <deal.II/base/utilities.h>
@@ -35,6 +34,7 @@
 #include <boost/serialization/split_member.hpp>
 
 #include <cstring>
+#include <type_traits>
 
 
 
@@ -99,15 +99,11 @@ public:
    */
   AlignedVector (const AlignedVector<T> &vec);
 
-#ifdef DEAL_II_WITH_CXX11
   /**
    * Move constructor. Create a new aligned vector by stealing the contents of
    * @p vec.
-   *
-   * @note This constructor is only available if deal.II is built with C++11.
    */
-  AlignedVector (AlignedVector<T> &&vec);
-#endif
+  AlignedVector (AlignedVector<T> &&vec) noexcept;
 
   /**
    * Assignment to the input vector @p vec.
@@ -117,15 +113,11 @@ public:
   AlignedVector &
   operator = (const AlignedVector<T> &vec);
 
-#ifdef DEAL_II_WITH_CXX11
   /**
    * Move assignment operator.
-   *
-   * @note This operator is only available if deal.II is built with C++11.
    */
   AlignedVector &
-  operator = (AlignedVector<T> &&vec);
-#endif
+  operator = (AlignedVector<T> &&vec) noexcept;
 
   /**
    * Change the size of the vector. It keeps old elements previously available
@@ -139,14 +131,35 @@ public:
 
   /**
    * Change the size of the vector. It keeps old elements previously
-   * available, and initializes each element with the specified data. If the
-   * new vector size is shorter than the old one, the memory is not released
-   * unless the new size is zero.
+   * available, and initializes each newly added element to a
+   * default-constructed object of type @p T.
+   *
+   * If the new vector size is shorter than the old one, the memory is
+   * not immediately released unless the new size is zero; however,
+   * the size of the current object is of course set to the requested
+   * value.
+   *
+   * @dealiiOperationIsMultithreaded
+   */
+  void resize (const size_type size_in);
+
+  /**
+   * Change the size of the vector. It keeps old elements previously
+   * available, and initializes each newly added element with the
+   * provided initializer.
+   *
+   * If the new vector size is shorter than the old one, the memory is
+   * not immediately released unless the new size is zero; however,
+   * the size of the current object is of course set to the requested
+   * value.
+   *
+   * @note This method can only be invoked for classes that define the copy
+   * assignment operator. Otherwise, compilation will fail.
    *
    * @dealiiOperationIsMultithreaded
    */
   void resize (const size_type size_in,
-               const T        &init = T());
+               const T        &init);
 
   /**
    * Reserve memory space for @p size elements. If the argument @p size is set
@@ -189,6 +202,17 @@ public:
   void insert_back (ForwardIterator begin,
                     ForwardIterator end);
 
+  /**
+   * Fills the vector with size() copies of a default constructed object.
+   *
+   * @note Unlike the other fill() function, this method can also be
+   * invoked for classes that do not define a copy assignment
+   * operator.
+   *
+   * @dealiiOperationIsMultithreaded
+   */
+  void fill ();
+
   /**
    * Fills the vector with size() copies of the given input.
    *
@@ -303,7 +327,7 @@ private:
 namespace internal
 {
   /**
-   * Move and class that actually issues the copy commands in AlignedVector.
+   * A class that actually issues the copy commands in AlignedVector.
    * This class is based on the specialized for loop base class
    * ParallelForLoop in parallel.h whose purpose is the following: When
    * calling a parallel for loop on AlignedVector with apply_to_subranges, it
@@ -315,10 +339,10 @@ namespace internal
    * for all possible types, which makes the copy operation much cleaner
    * (thanks to a virtual function, whose cost is negligible in this context).
    *
-   * @relates AlignedVector
+   * @relatesalso AlignedVector
    */
   template <typename T>
-  class AlignedVectorMove : private parallel::ParallelForInteger
+  class AlignedVectorCopy : private parallel::ParallelForInteger
   {
     static const std::size_t minimum_parallel_grain_size = 160000/sizeof(T)+1;
   public:
@@ -326,22 +350,20 @@ namespace internal
      * Constructor. Issues a parallel call if there are sufficiently many
      * elements, otherwise works in serial. Copies the data from the half-open
      * interval between @p source_begin and @p source_end to array starting at
-     * @p destination (by calling the copy constructor with placement new). If
-     * the flag copy_source is set to @p true, the elements from the source
-     * array are simply copied. If it is set to @p false, the data is moved
-     * between the two arrays by invoking the destructor on the source range
-     * (preparing for a subsequent call to free).
+     * @p destination (by calling the copy constructor with placement new).
+     *
+     * The elements from the source array are simply copied via the placement
+     * new copy constructor.
      */
-    AlignedVectorMove (T *source_begin,
-                       T *source_end,
-                       T *destination,
-                       const bool copy_source)
+    AlignedVectorCopy (const T *const source_begin,
+                       const T *const source_end,
+                       T *const destination)
       :
       source_ (source_begin),
-      destination_ (destination),
-      copy_source_ (copy_source)
+      destination_ (destination)
     {
       Assert (source_end >= source_begin, ExcInternalError());
+      Assert (source_end == source_begin || destination != nullptr, ExcInternalError());
       const std::size_t size = source_end - source_begin;
       if (size < minimum_parallel_grain_size)
         apply_to_subrange (0, size);
@@ -363,38 +385,98 @@ namespace internal
       // (void*) to silence compiler warning for virtual classes (they will
       // never arrive here because they are non-trivial).
 
-      if (std_cxx11::is_trivial<T>::value == true)
+      if (std::is_trivial<T>::value == true)
         std::memcpy ((void *)(destination_+begin), (void *)(source_+begin),
                      (end-begin)*sizeof(T));
-      else if (copy_source_ == false)
-        for (std::size_t i=begin; i<end; ++i)
-          {
-            // initialize memory (copy construct by placement new), and
-            // destruct the source
-            new (&destination_[i]) T(source_[i]);
-            source_[i].~T();
-          }
       else
         for (std::size_t i=begin; i<end; ++i)
           new (&destination_[i]) T(source_[i]);
     }
 
   private:
-    T *source_;
-    T *destination_;
-    const bool copy_source_;
+    const T *const source_;
+    T *const destination_;
   };
 
+
+  /**
+   * Like AlignedVectorCopy, but use move operations instead of copy operations.
+   *
+   * @relatesalso AlignedVector
+   */
+  template <typename T>
+  class AlignedVectorMove : private parallel::ParallelForInteger
+  {
+    static const std::size_t minimum_parallel_grain_size = 160000/sizeof(T)+1;
+  public:
+    /**
+     * Constructor. Issues a parallel call if there are sufficiently many
+     * elements, otherwise works in serial. Moves the data from the half-open
+     * interval between @p source_begin and @p source_end to array starting at
+     * @p destination (by calling the move constructor with placement new).
+     *
+     * The data is moved between the two arrays by invoking the destructor on
+     * the source range (preparing for a subsequent call to free).
+     */
+    AlignedVectorMove (T *const source_begin,
+                       T *const source_end,
+                       T *const destination)
+      :
+      source_ (source_begin),
+      destination_ (destination)
+    {
+      Assert (source_end >= source_begin, ExcInternalError());
+      Assert (source_end == source_begin || destination != nullptr, ExcInternalError());
+      const std::size_t size = source_end - source_begin;
+      if (size < minimum_parallel_grain_size)
+        apply_to_subrange (0, size);
+      else
+        apply_parallel (0, size, minimum_parallel_grain_size);
+    }
+
+    /**
+     * This method moves elements from the source to the destination given in
+     * the constructor on a subrange given by two integers.
+     */
+    virtual void apply_to_subrange (const std::size_t begin,
+                                    const std::size_t end) const
+    {
+      if (end == begin)
+        return;
+
+      // for classes trivial assignment can use memcpy. cast element to
+      // (void*) to silence compiler warning for virtual classes (they will
+      // never arrive here because they are non-trivial).
+
+      if (std::is_trivial<T>::value == true)
+        std::memcpy ((void *)(destination_+begin), (void *)(source_+begin),
+                     (end-begin)*sizeof(T));
+      else
+        for (std::size_t i=begin; i<end; ++i)
+          {
+            // initialize memory (copy construct by placement new), and
+            // destruct the source
+            new (&destination_[i]) T(std::move(source_[i]));
+            source_[i].~T();
+          }
+    }
+
+  private:
+    T *const source_;
+    T *const destination_;
+  };
+
+
   /**
    * Class that issues the set commands for AlignedVector.
    *
-   * @tparam initialize_memory Sets whether the the set command should
+   * @tparam initialize_memory Sets whether the set command should
    * initialize memory (with a call to the copy constructor) or rather use the
    * copy assignment operator. A template is necessary to select the
    * appropriate operation since some classes might define only one of those
    * two operations.
    *
-   * @relates AlignedVector
+   * @relatesalso AlignedVector
    */
   template <typename T, bool initialize_memory>
   class AlignedVectorSet : private parallel::ParallelForInteger
@@ -407,7 +489,7 @@ namespace internal
      */
     AlignedVectorSet (const std::size_t size,
                       const T &element,
-                      T *destination)
+                      T *const destination)
       :
       element_ (element),
       destination_ (destination),
@@ -415,12 +497,13 @@ namespace internal
     {
       if (size == 0)
         return;
+      Assert (destination != nullptr, ExcInternalError());
 
       // do not use memcmp for long double because on some systems it does not
       // completely fill its memory and may lead to false positives in
       // e.g. valgrind
-      if (std_cxx11::is_trivial<T>::value == true &&
-          types_are_equal<T,long double>::value == false)
+      if (std::is_trivial<T>::value == true &&
+          std::is_same<T,long double>::value == false)
         {
           const unsigned char zero [sizeof(T)] = {};
           // cast element to (void*) to silence compiler warning for virtual
@@ -445,11 +528,11 @@ namespace internal
       // element to (void*) to silence compiler warning for virtual
       // classes (they will never arrive here because they are
       // non-trivial).
-      if (std_cxx11::is_trivial<T>::value == true && trivial_element)
+      if (std::is_trivial<T>::value == true && trivial_element)
         std::memset ((void *)(destination_+begin), 0, (end-begin)*sizeof(T));
       else
         copy_construct_or_assign(begin, end,
-                                 ::dealii::internal::bool2type<initialize_memory>());
+                                 std::integral_constant<bool, initialize_memory>());
     }
 
   private:
@@ -460,7 +543,7 @@ namespace internal
     // copy assignment operation
     void copy_construct_or_assign(const std::size_t begin,
                                   const std::size_t end,
-                                  ::dealii::internal::bool2type<false>) const
+                                  std::integral_constant<bool, false>) const
     {
       for (std::size_t i=begin; i<end; ++i)
         destination_[i] = element_;
@@ -469,13 +552,89 @@ namespace internal
     // copy constructor (memory initialization)
     void copy_construct_or_assign(const std::size_t begin,
                                   const std::size_t end,
-                                  ::dealii::internal::bool2type<true>) const
+                                  std::integral_constant<bool, true>) const
     {
       for (std::size_t i=begin; i<end; ++i)
         new (&destination_[i]) T(element_);
     }
   };
 
+
+
+  /**
+   * Class that issues the set commands for AlignedVector.
+   *
+   * @tparam initialize_memory Sets whether the set command should
+   * initialize memory (with a call to the copy constructor) or rather use the
+   * copy assignment operator. A template is necessary to select the
+   * appropriate operation since some classes might define only one of those
+   * two operations.
+   *
+   * @relatesalso AlignedVector
+   */
+  template <typename T, bool initialize_memory>
+  class AlignedVectorDefaultInitialize : private parallel::ParallelForInteger
+  {
+    static const std::size_t minimum_parallel_grain_size = 160000/sizeof(T)+1;
+  public:
+    /**
+     * Constructor. Issues a parallel call if there are sufficiently many
+     * elements, otherwise work in serial.
+     */
+    AlignedVectorDefaultInitialize (const std::size_t size,
+                                    T *const destination)
+      :
+      destination_ (destination)
+    {
+      if (size == 0)
+        return;
+      Assert (destination != nullptr, ExcInternalError());
+
+      if (size < minimum_parallel_grain_size)
+        apply_to_subrange (0, size);
+      else
+        apply_parallel (0, size, minimum_parallel_grain_size);
+    }
+
+    /**
+     * This initializes elements on a subrange given by two integers.
+     */
+    virtual void apply_to_subrange (const std::size_t begin,
+                                    const std::size_t end) const
+    {
+      // for classes with trivial assignment of zero can use memset. cast
+      // element to (void*) to silence compiler warning for virtual
+      // classes (they will never arrive here because they are
+      // non-trivial).
+      if (std::is_trivial<T>::value == true)
+        std::memset ((void *)(destination_+begin), 0, (end-begin)*sizeof(T));
+      else
+        default_construct_or_assign(begin, end,
+                                    std::integral_constant<bool, initialize_memory>());
+    }
+
+  private:
+    mutable T *destination_;
+
+    // copy assignment operation
+    void default_construct_or_assign(const std::size_t begin,
+                                     const std::size_t end,
+                                     std::integral_constant<bool, false>) const
+    {
+      for (std::size_t i=begin; i<end; ++i)
+        destination_[i] = std::move(T());
+    }
+
+    // copy constructor (memory initialization)
+    void default_construct_or_assign(const std::size_t begin,
+                                     const std::size_t end,
+                                     std::integral_constant<bool, true>) const
+    {
+      for (std::size_t i=begin; i<end; ++i)
+        new (&destination_[i]) T;
+    }
+  };
+
 } // end of namespace internal
 
 
@@ -486,9 +645,9 @@ template < class T >
 inline
 AlignedVector<T>::AlignedVector ()
   :
-  _data (0),
-  _end_data (0),
-  _end_allocated (0)
+  _data (nullptr),
+  _end_data (nullptr),
+  _end_allocated (nullptr)
 {}
 
 
@@ -498,9 +657,9 @@ inline
 AlignedVector<T>::AlignedVector (const size_type size,
                                  const T &init)
   :
-  _data (0),
-  _end_data (0),
-  _end_allocated (0)
+  _data (nullptr),
+  _end_data (nullptr),
+  _end_allocated (nullptr)
 {
   if (size > 0)
     resize (size, init);
@@ -521,32 +680,30 @@ template < class T >
 inline
 AlignedVector<T>::AlignedVector (const AlignedVector<T> &vec)
   :
-  _data (0),
-  _end_data (0),
-  _end_allocated (0)
+  _data (nullptr),
+  _end_data (nullptr),
+  _end_allocated (nullptr)
 {
   // copy the data from vec
   reserve (vec._end_data - vec._data);
   _end_data = _end_allocated;
-  internal::AlignedVectorMove<T> (vec._data, vec._end_data, _data, true);
+  internal::AlignedVectorCopy<T> (vec._data, vec._end_data, _data);
 }
 
 
 
-#ifdef DEAL_II_WITH_CXX11
 template < class T >
 inline
-AlignedVector<T>::AlignedVector (AlignedVector<T> &&vec)
-  :
-  _data (vec._data),
-  _end_data (vec._end_data),
-  _end_allocated (vec._end_allocated)
+AlignedVector<T>::AlignedVector (AlignedVector<T> &&vec) noexcept
+:
+_data (vec._data),
+      _end_data (vec._end_data),
+      _end_allocated (vec._end_allocated)
 {
   vec._data = nullptr;
   vec._end_data = nullptr;
   vec._end_allocated = nullptr;
 }
-#endif
 
 
 
@@ -557,17 +714,16 @@ AlignedVector<T>::operator = (const AlignedVector<T> &vec)
 {
   resize(0);
   resize_fast (vec._end_data - vec._data);
-  internal::AlignedVectorMove<T> (vec._data, vec._end_data, _data, true);
+  internal::AlignedVectorCopy<T> (vec._data, vec._end_data, _data);
   return *this;
 }
 
 
 
-#ifdef DEAL_II_WITH_CXX11
 template < class T >
 inline
 AlignedVector<T> &
-AlignedVector<T>::operator = (AlignedVector<T> &&vec)
+AlignedVector<T>::operator = (AlignedVector<T> &&vec) noexcept
 {
   clear();
 
@@ -581,7 +737,6 @@ AlignedVector<T>::operator = (AlignedVector<T> &&vec)
 
   return *this;
 }
-#endif
 
 
 
@@ -591,7 +746,7 @@ void
 AlignedVector<T>::resize_fast (const size_type size_in)
 {
   const size_type old_size = size();
-  if (std_cxx11::is_trivial<T>::value == false && size_in < old_size)
+  if (std::is_trivial<T>::value == false && size_in < old_size)
     {
       // call destructor on fields that are released. doing it backward
       // releases the elements in reverse order as compared to how they were
@@ -604,8 +759,32 @@ AlignedVector<T>::resize_fast (const size_type size_in)
 
   // need to still set the values in case the class is non-trivial because
   // virtual classes etc. need to run their (default) constructor
-  if (std_cxx11::is_trivial<T>::value == false && size_in > old_size)
-    dealii::internal::AlignedVectorSet<T,true> (size_in-old_size, T(), _data+old_size);
+  if (std::is_trivial<T>::value == false && size_in > old_size)
+    dealii::internal::AlignedVectorDefaultInitialize<T,true> (size_in-old_size, _data+old_size);
+}
+
+
+
+template < class T >
+inline
+void
+AlignedVector<T>::resize (const size_type size_in)
+{
+  const size_type old_size = size();
+  if (std::is_trivial<T>::value == false && size_in < old_size)
+    {
+      // call destructor on fields that are released. doing it backward
+      // releases the elements in reverse order as compared to how they were
+      // created
+      while (_end_data != _data+size_in)
+        (--_end_data)->~T();
+    }
+  reserve (size_in);
+  _end_data = _data + size_in;
+
+  // finally set the desired init values
+  if (size_in > old_size)
+    dealii::internal::AlignedVectorDefaultInitialize<T,true> (size_in-old_size, _data+old_size);
 }
 
 
@@ -617,7 +796,7 @@ AlignedVector<T>::resize (const size_type size_in,
                           const T        &init)
 {
   const size_type old_size = size();
-  if (std_cxx11::is_trivial<T>::value == false && size_in < old_size)
+  if (std::is_trivial<T>::value == false && size_in < old_size)
     {
       // call destructor on fields that are released. doing it backward
       // releases the elements in reverse order as compared to how they were
@@ -666,11 +845,11 @@ AlignedVector<T>::reserve (const size_type size_alloc)
       if (_end_data != _data)
         {
           dealii::internal::AlignedVectorMove<T>(new_data, new_data + old_size,
-                                                 _data, false);
+                                                 _data);
           free(new_data);
         }
       else
-        Assert(new_data == 0, ExcInternalError());
+        Assert(new_data == nullptr, ExcInternalError());
     }
   else if (size_alloc == 0)
     clear();
@@ -683,17 +862,17 @@ inline
 void
 AlignedVector<T>::clear ()
 {
-  if (_data != 0)
+  if (_data != nullptr)
     {
-      if (std_cxx11::is_trivial<T>::value == false)
+      if (std::is_trivial<T>::value == false)
         while (_end_data != _data)
           (--_end_data)->~T();
 
       free(_data);
     }
-  _data = 0;
-  _end_data = 0;
-  _end_allocated = 0;
+  _data = nullptr;
+  _end_data = nullptr;
+  _end_allocated = nullptr;
 }
 
 
@@ -706,7 +885,7 @@ AlignedVector<T>::push_back (const T in_data)
   Assert (_end_data <= _end_allocated, ExcInternalError());
   if (_end_data == _end_allocated)
     reserve (std::max(2*capacity(),static_cast<size_type>(16)));
-  if (std_cxx11::is_trivial<T>::value == false)
+  if (std::is_trivial<T>::value == false)
     new (_end_data++) T(in_data);
   else
     *_end_data++ = in_data;
@@ -749,7 +928,7 @@ AlignedVector<T>::insert_back (ForwardIterator begin,
   reserve (old_size + (end-begin));
   for ( ; begin != end; ++begin, ++_end_data)
     {
-      if (std_cxx11::is_trivial<T>::value == false)
+      if (std::is_trivial<T>::value == false)
         new (_end_data) T;
       *_end_data = *begin;
     }
@@ -757,6 +936,16 @@ AlignedVector<T>::insert_back (ForwardIterator begin,
 
 
 
+template < class T >
+inline
+void
+AlignedVector<T>::fill ()
+{
+  dealii::internal::AlignedVectorDefaultInitialize<T,false> (size(), _data);
+}
+
+
+
 template < class T >
 inline
 void
@@ -923,7 +1112,7 @@ AlignedVector<T>::memory_consumption () const
 /**
  * Relational operator == for AlignedVector
  *
- * @relates AlignedVector
+ * @relatesalso AlignedVector
  */
 template < class T >
 bool operator == (const AlignedVector<T> &lhs,
@@ -944,7 +1133,7 @@ bool operator == (const AlignedVector<T> &lhs,
 /**
  * Relational operator != for AlignedVector
  *
- * @relates AlignedVector
+ * @relatesalso AlignedVector
  */
 template < class T >
 bool operator != (const AlignedVector<T> &lhs,
