@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 2013 - 2017 by the deal.II authors
+// Copyright (C) 2013 - 2018 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -13,8 +13,8 @@
 //
 // ---------------------------------------------------------------------
 
-#ifndef dealii__derivative_form_h
-#define dealii__derivative_form_h
+#ifndef dealii_derivative_form_h
+#define dealii_derivative_form_h
 
 #include <deal.II/base/tensor.h>
 
@@ -116,7 +116,7 @@ public:
    * transformation F. That is to say if $DF$ is square, it computes
    * $\det(DF)$, in case DF is not square returns $\sqrt{\det(DF^T * DF)}$.
    */
-  double determinant () const;
+  Number determinant () const;
 
   /**
    * Assuming that the current object stores the Jacobian of a mapping
@@ -325,8 +325,7 @@ DerivativeForm<order,dim,spacedim,Number>::norm () const
 
 template <int order, int dim, int spacedim, typename Number>
 inline
-double
-DerivativeForm<order,dim,spacedim,Number>::determinant () const
+Number DerivativeForm<order, dim, spacedim, Number>::determinant() const
 {
   Assert( order==1, ExcMessage("Only for order == 1."));
   if (dim == spacedim)
@@ -337,7 +336,7 @@ DerivativeForm<order,dim,spacedim,Number>::determinant () const
   else
     {
       Assert( spacedim>dim, ExcMessage("Only for spacedim>dim."));
-      const DerivativeForm<1,spacedim,dim> DF_t = this->transpose();
+      const DerivativeForm<1,spacedim,dim,Number> DF_t = this->transpose();
       Tensor<2,dim,Number> G; //First fundamental form
       for (unsigned int i=0; i<dim; ++i)
         for (unsigned int j=0; j<dim; ++j)
@@ -392,7 +391,7 @@ DerivativeForm<order, dim, spacedim, Number>::memory_consumption ()
  * is what this function does.  If @p T is DerivativeForm<1,dim,1> it computes
  * $DF * T$, if @p T is DerivativeForm<1,dim,rank> it computes $T*DF^{T}$.
  *
- * @relates DerivativeForm
+ * @relatesalso DerivativeForm
  * @author Sebastian Pauletti, 2011
  */
 template <int spacedim, int dim, typename Number>
@@ -412,7 +411,7 @@ apply_transformation (const DerivativeForm<1,dim,spacedim,Number> &DF,
 /**
  * Similar to previous apply_transformation. It computes $T*DF^{T}$.
  *
- * @relates DerivativeForm
+ * @relatesalso DerivativeForm
  * @author Sebastian Pauletti, 2011
  */
 //rank=2
@@ -433,7 +432,7 @@ apply_transformation (const DerivativeForm<1,dim,spacedim,Number> &DF,
 /**
  * Similar to previous apply_transformation. It computes $DF2*DF1^{T}$
  *
- * @relates DerivativeForm
+ * @relatesalso DerivativeForm
  * @author Sebastian Pauletti, 2011
  */
 template <int spacedim, int dim, typename Number>
@@ -455,7 +454,7 @@ apply_transformation (const DerivativeForm<1,dim,spacedim,Number> &DF1,
  * Transpose of a rectangular DerivativeForm DF, mostly for compatibility
  * reasons.
  *
- * @relates DerivativeForm
+ * @relatesalso DerivativeForm
  * @author Sebastian Pauletti, 2011
  */
 template <int dim, int spacedim, typename Number>
