@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 2010 - 2015 by the deal.II authors
+// Copyright (C) 2010 - 2018 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -14,8 +14,8 @@
 // ---------------------------------------------------------------------
 
 
-#ifndef dealii__event_h
-#define dealii__event_h
+#ifndef dealii_event_h
+#define dealii_event_h
 
 #include <deal.II/base/config.h>
 
@@ -285,7 +285,7 @@ namespace Algorithms
   /**
    * Output shift operator for events. Calls Event::print().
    *
-   * @relates Event
+   * @relatesalso Event
    */
   template <class OS>
   OS &operator << (OS &o, const Event &e)
