@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 1998 - 2017 by the deal.II authors
+// Copyright (C) 1998 - 2018 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -13,14 +13,21 @@
 //
 // ---------------------------------------------------------------------
 
-#ifndef dealii__exceptions_h
-#define dealii__exceptions_h
+#ifndef dealii_exceptions_h
+#define dealii_exceptions_h
 
 #include <deal.II/base/config.h>
 
 #include <exception>
-#include <string>
 #include <ostream>
+#include <string>
+#include <type_traits>
+
+#ifdef DEAL_II_WITH_CUDA
+#include <cusolverSp.h>
+#include <cusparse.h>
+#endif
+
 
 DEAL_II_NAMESPACE_OPEN
 
@@ -55,7 +62,13 @@ public:
   /**
    * Destructor.
    */
-  virtual ~ExceptionBase () DEAL_II_NOEXCEPT;
+  virtual ~ExceptionBase () noexcept;
+
+  /**
+   * Copy operator. This operator is deleted since exception objects
+   * are not copyable.
+   */
+  ExceptionBase operator= (const ExceptionBase &) = delete;
 
   /**
    * Set the file name and line of where the exception appeared as well as the
@@ -72,7 +85,7 @@ public:
   /**
    * Override the standard function that returns the description of the error.
    */
-  virtual const char *what() const DEAL_II_NOEXCEPT;
+  virtual const char *what() const noexcept;
 
   /**
    * Get exception name.
@@ -142,11 +155,6 @@ protected:
 #endif
 
 private:
-  /**
-   * Copy operator. Made private since it is not used and not implemented.
-   */
-  ExceptionBase operator= (const ExceptionBase &exc);
-
   /**
    * Internal function that generates the c_string. Called by what().
    */
@@ -159,232 +167,6 @@ private:
   mutable std::string what_str;
 };
 
-
-
-/**
- * In this namespace, functions in connection with the Assert and AssertThrow
- * mechanism are declared.
- *
- * @ingroup Exceptions
- */
-namespace deal_II_exceptions
-{
-
-  /**
-   * Set a string that is printed upon output of the message indicating a
-   * triggered <tt>Assert</tt> statement. This string, which is printed in
-   * addition to the usual output may indicate information that is otherwise
-   * not readily available unless we are using a debugger. For example, with
-   * distributed programs on cluster computers, the output of all processes is
-   * redirected to the same console window. In this case, it is convenient to
-   * set as additional name the name of the host on which the program runs, so
-   * that one can see in which instance of the program the exception occurred.
-   *
-   * The string pointed to by the argument is copied, so doesn't need to be
-   * stored after the call to this function.
-   *
-   * Previously set additional output is replaced by the argument given to
-   * this function.
-   */
-  void set_additional_assert_output (const char *const p);
-
-  /**
-   * Calling this function disables printing a stacktrace along with the other
-   * output printed when an exception occurs. Most of the time, you will want
-   * to see such a stacktrace; suppressing it, however, is useful if one wants
-   * to compare the output of a program across different machines and systems,
-   * since the stacktrace shows memory addresses and library names/paths that
-   * depend on the exact setup of a machine.
-   */
-  void suppress_stacktrace_in_exceptions ();
-
-  /**
-   * Calling this function switches off the use of <tt>std::abort()</tt> when
-   * an exception is created using the Assert() macro. Instead, the Exception
-   * will be thrown using 'throw', so it can be caught if desired. Generally,
-   * you want to abort the execution of a program when Assert() is called, but
-   * it needs to be switched off if you want to log all exceptions created, or
-   * if you want to test if an assertion is working correctly. This is done
-   * for example in regression tests. Please note that some fatal errors will
-   * still call abort(), e.g. when an exception is caught during exception
-   * handling.
-   */
-  void disable_abort_on_exception ();
-
-  /**
-   * The functions in this namespace are in connection with the Assert and
-   * AssertThrow mechanism but are solely for internal purposes and are not
-   * for use outside the exception handling and throwing mechanism.
-   *
-   * @ingroup Exceptions
-   */
-  namespace internals
-  {
-
-    /**
-     * Conditionally abort the program.
-     *
-     * Depending on whether deal_II_exceptions::disable_abort_on_exception was
-     * called, this function either aborts the program flow by printing the
-     * error message provided by @p exc and calling <tt>std::abort()</tt>, or
-     * throws @p exc instead (if @p nothrow is set to <tt>false</tt>).
-     *
-     * If the boolean @p nothrow is set to true and
-     * deal_II_exceptions::disable_abort_on_exception was called, the
-     * exception type is just printed to deallog and program flow
-     * continues. This is useful if throwing an exception is prohibited
-     * (e.g. in a destructor with <tt>noexcept(true)</tt> or
-     * <tt>throw()</tt>).
-     */
-    void abort (const ExceptionBase &exc, bool nothrow = false);
-
-    /**
-     * An enum describing how to treat an exception in issue_error.
-     */
-    enum ExceptionHandling
-    {
-      /**
-       * Abort the program by calling <code>std::abort</code> unless
-       * deal_II_exceptions::disable_abort_on_exception has been called: in
-       * that case the program will throw an exception.
-       */
-      abort_on_exception,
-      /**
-       * Throw the exception normally.
-       */
-      throw_on_exception,
-      /**
-       * Call <code>std::abort</code> as long as
-       * deal_II_exceptions::disable_abort_on_exception has not been called:
-       * if it has, then just print a description of the exception to deallog.
-       */
-      abort_nothrow_on_exception
-    };
-
-    /**
-     * This routine does the main work for the exception generation mechanism
-     * used in the <tt>Assert</tt> macro.
-     *
-     * @ref ExceptionBase
-     */
-    template <class exc>
-    void issue_error (ExceptionHandling handling,
-                      const char *file,
-                      int         line,
-                      const char *function,
-                      const char *cond,
-                      const char *exc_name,
-                      exc         e)
-    {
-      // Fill the fields of the exception object
-      e.set_fields (file, line, function, cond, exc_name);
-
-      switch (handling)
-        {
-        case abort_on_exception:
-          dealii::deal_II_exceptions::internals::abort(e);
-          break;
-        case abort_nothrow_on_exception:
-          dealii::deal_II_exceptions::internals::abort(e, /*nothrow =*/ true);
-          break;
-        case throw_on_exception:
-          throw e;
-        }
-    }
-
-  } /*namespace internals*/
-
-} /*namespace deal_II_exceptions*/
-
-
-
-/**
- * This is the main routine in the exception mechanism for debug mode error
- * checking. It asserts that a certain condition is fulfilled, otherwise
- * issues an error and aborts the program.
- *
- * See the <tt>ExceptionBase</tt> class for more information.
- *
- * @note Active in DEBUG mode only
- * @ingroup Exceptions
- * @author Wolfgang Bangerth, 1997, 1998, Matthias Maier, 2013
- */
-#ifdef DEBUG
-#define Assert(cond, exc)                                                   \
-  {                                                                           \
-    if (!(cond))                                                              \
-      ::dealii::deal_II_exceptions::internals::                               \
-      issue_error(::dealii::deal_II_exceptions::internals::abort_on_exception,\
-                  __FILE__, __LINE__, __PRETTY_FUNCTION__, #cond, #exc, exc); \
-  }
-#else
-#define Assert(cond, exc)                                                   \
-  {}
-#endif
-
-
-
-/**
- * A variant of the <tt>Assert</tt> macro above that exhibits the same runtime
- * behaviour as long as disable_abort_on_exception was not called.
- *
- * However, if disable_abort_on_exception was called, this macro merely prints
- * the exception that would be thrown to deallog and continues normally
- * without throwing an exception.
- *
- * See the <tt>ExceptionBase</tt> class for more information.
- *
- * @note Active in DEBUG mode only
- * @ingroup Exceptions
- * @author Wolfgang Bangerth, 1997, 1998, Matthias Maier, 2013
- */
-#ifdef DEBUG
-#define AssertNothrow(cond, exc)                                            \
-  {                                                                           \
-    if (!(cond))                                                              \
-      ::dealii::deal_II_exceptions::internals::                               \
-      issue_error(                                                            \
-          ::dealii::deal_II_exceptions::internals::abort_nothrow_on_exception,  \
-          __FILE__, __LINE__, __PRETTY_FUNCTION__, #cond, #exc, exc);           \
-  }
-#else
-#define AssertNothrow(cond, exc)                                            \
-  {}
-#endif
-
-
-
-/**
- * This is the main routine in the exception mechanism for run-time mode error
- * checking. It assert that a certain condition is fulfilled, otherwise issues
- * an error and aborts the program.
- *
- * See the <tt>ExceptionBase</tt> class for more information.
- *
- * @note Active in both DEBUG and RELEASE modes
- * @ingroup Exceptions
- * @author Wolfgang Bangerth, 1997, 1998, Matthias Maier, 2013
- */
-#ifdef DEAL_II_HAVE_BUILTIN_EXPECT
-#define AssertThrow(cond, exc)                                              \
-  {                                                                           \
-    if (__builtin_expect(!(cond), false))                                     \
-      ::dealii::deal_II_exceptions::internals::                               \
-      issue_error(::dealii::deal_II_exceptions::internals::throw_on_exception,\
-                  __FILE__, __LINE__, __PRETTY_FUNCTION__, #cond, #exc, exc); \
-  }
-#else /*ifdef DEAL_II_HAVE_BUILTIN_EXPECT*/
-#define AssertThrow(cond, exc)                                              \
-  {                                                                           \
-    if (!(cond))                                                              \
-      ::dealii::deal_II_exceptions::internals::                               \
-      issue_error(::dealii::deal_II_exceptions::internals::throw_on_exception,\
-                  __FILE__, __LINE__, __PRETTY_FUNCTION__, #cond, #exc, exc); \
-  }
-#endif /*ifdef DEAL_II_HAVE_BUILTIN_EXPECT*/
-
-
-
 #ifndef DOXYGEN
 
 /**
@@ -410,7 +192,7 @@ namespace deal_II_exceptions
   {                                                                       \
   public:                                                                 \
     Exception (const std::string &msg = defaulttext) : arg (msg) {}       \
-    virtual ~Exception () DEAL_II_NOEXCEPT {}                             \
+    virtual ~Exception () noexcept {}                                     \
     virtual void print_info (std::ostream &out) const {                   \
       out << "    " << arg << std::endl;                                  \
     }                                                                     \
@@ -427,8 +209,8 @@ namespace deal_II_exceptions
 #define DeclException1(Exception1, type1, outsequence)                    \
   class Exception1 : public dealii::ExceptionBase {                       \
   public:                                                                 \
-    Exception1 (const type1 a1) : arg1 (a1) {}                            \
-    virtual ~Exception1 () DEAL_II_NOEXCEPT {}                            \
+    Exception1 (const type1 a1) : arg1 (a1) {}    /*NOLINT*/              \
+    virtual ~Exception1 () noexcept {}                                    \
     virtual void print_info (std::ostream &out) const {                   \
       out << "    " outsequence << std::endl;                             \
     }                                                                     \
@@ -446,9 +228,9 @@ namespace deal_II_exceptions
 #define DeclException2(Exception2, type1, type2, outsequence)             \
   class Exception2 : public dealii::ExceptionBase {                       \
   public:                                                                 \
-    Exception2 (const type1 a1, const type2 a2) :                         \
+    Exception2 (const type1 a1, const type2 a2) : /*NOLINT*/              \
       arg1 (a1), arg2(a2) {}                                              \
-    virtual ~Exception2 () DEAL_II_NOEXCEPT {}                            \
+    virtual ~Exception2 () noexcept {}                                    \
     virtual void print_info (std::ostream &out) const {                   \
       out << "    " outsequence << std::endl;                             \
     }                                                                     \
@@ -467,9 +249,9 @@ namespace deal_II_exceptions
 #define DeclException3(Exception3, type1, type2, type3, outsequence)      \
   class Exception3 : public dealii::ExceptionBase {                       \
   public:                                                                 \
-    Exception3 (const type1 a1, const type2 a2, const type3 a3) :         \
+    Exception3 (const type1 a1, const type2 a2, const type3 a3) : /*NOLINT*/ \
       arg1 (a1), arg2(a2), arg3(a3) {}                                    \
-    virtual ~Exception3 () DEAL_II_NOEXCEPT {}                            \
+    virtual ~Exception3 () noexcept {}                                    \
     virtual void print_info (std::ostream &out) const {                   \
       out << "    " outsequence << std::endl;                             \
     }                                                                     \
@@ -489,10 +271,10 @@ namespace deal_II_exceptions
 #define DeclException4(Exception4, type1, type2, type3, type4, outsequence) \
   class Exception4 : public dealii::ExceptionBase {                       \
   public:                                                                 \
-    Exception4 (const type1 a1, const type2 a2,                           \
-                const type3 a3, const type4 a4) :                         \
+    Exception4 (const type1 a1, const type2 a2,   /*NOLINT*/              \
+                const type3 a3, const type4 a4) : /*NOLINT*/              \
       arg1 (a1), arg2(a2), arg3(a3), arg4(a4) {}                          \
-    virtual ~Exception4 () DEAL_II_NOEXCEPT {}                            \
+    virtual ~Exception4 () noexcept {}                                    \
     virtual void print_info (std::ostream &out) const {                   \
       out << "    " outsequence << std::endl;                             \
     }                                                                     \
@@ -513,10 +295,10 @@ namespace deal_II_exceptions
 #define DeclException5(Exception5, type1, type2, type3, type4, type5, outsequence) \
   class Exception5 : public dealii::ExceptionBase {                       \
   public:                                                                 \
-    Exception5 (const type1 a1, const type2 a2, const type3 a3,           \
-                const type4 a4, const type5 a5) :                         \
+    Exception5 (const type1 a1, const type2 a2, const type3 a3,/*NOLINT*/ \
+                const type4 a4, const type5 a5) :              /*NOLINT*/ \
       arg1 (a1), arg2(a2), arg3(a3), arg4(a4), arg5(a5) {}                \
-    virtual ~Exception5 () DEAL_II_NOEXCEPT {}                            \
+    virtual ~Exception5 () noexcept {}                                    \
     virtual void print_info (std::ostream &out) const {                   \
       out << "    " outsequence << std::endl;                             \
     }                                                                     \
@@ -1084,6 +866,14 @@ namespace StandardExceptions
                     "find a valid muparser library on your system and also did "
                     "not choose the one that comes bundled with deal.II.");
 
+  /**
+   * This function requires support for the Assimp library.
+   */
+  DeclExceptionMsg (ExcNeedsAssimp,
+                    "You are attempting to use functionality that is only available "
+                    "if deal.II was configured to use Assimp, but cmake did not "
+                    "find a valid Assimp library.");
+
 #ifdef DEAL_II_WITH_CUDA
   /**
    * This exception is raised if an error happened in a CUDA kernel.
@@ -1094,6 +884,13 @@ namespace StandardExceptions
   DeclException1 (ExcCudaError,
                   char *,
                   << arg1);
+  /**
+   * This exception is raised if an error happened in a cuSPARSE function.
+   */
+  DeclException1 (ExcCusparseError,
+                  std::string,
+                  << "There was an error in a cuSPARSE function: "
+                  << arg1);
 #endif
 //@}
 
@@ -1133,6 +930,311 @@ namespace StandardExceptions
 } /*namespace StandardExceptions*/
 
 
+
+/**
+ * In this namespace, functions in connection with the Assert and AssertThrow
+ * mechanism are declared.
+ *
+ * @ingroup Exceptions
+ */
+namespace deal_II_exceptions
+{
+
+  /**
+   * Set a string that is printed upon output of the message indicating a
+   * triggered <tt>Assert</tt> statement. This string, which is printed in
+   * addition to the usual output may indicate information that is otherwise
+   * not readily available unless we are using a debugger. For example, with
+   * distributed programs on cluster computers, the output of all processes is
+   * redirected to the same console window. In this case, it is convenient to
+   * set as additional name the name of the host on which the program runs, so
+   * that one can see in which instance of the program the exception occurred.
+   *
+   * The string pointed to by the argument is copied, so doesn't need to be
+   * stored after the call to this function.
+   *
+   * Previously set additional output is replaced by the argument given to
+   * this function.
+   *
+   * @see Exceptions
+   */
+  void set_additional_assert_output (const char *const p);
+
+  /**
+   * Calling this function disables printing a stacktrace along with the other
+   * output printed when an exception occurs. Most of the time, you will want
+   * to see such a stacktrace; suppressing it, however, is useful if one wants
+   * to compare the output of a program across different machines and systems,
+   * since the stacktrace shows memory addresses and library names/paths that
+   * depend on the exact setup of a machine.
+   *
+   * @see Exceptions
+   */
+  void suppress_stacktrace_in_exceptions ();
+
+  /**
+   * Calling this function switches off the use of <tt>std::abort()</tt> when
+   * an exception is created using the Assert() macro. Instead, the Exception
+   * will be thrown using 'throw', so it can be caught if desired. Generally,
+   * you want to abort the execution of a program when Assert() is called, but
+   * it needs to be switched off if you want to log all exceptions created, or
+   * if you want to test if an assertion is working correctly. This is done
+   * for example in regression tests. Please note that some fatal errors will
+   * still call abort(), e.g. when an exception is caught during exception
+   * handling.
+   *
+   * @see Exceptions
+   */
+  void disable_abort_on_exception ();
+
+  /**
+   * The functions in this namespace are in connection with the Assert and
+   * AssertThrow mechanism but are solely for internal purposes and are not
+   * for use outside the exception handling and throwing mechanism.
+   *
+   * @ingroup Exceptions
+   */
+  namespace internals
+  {
+
+    /**
+     * Conditionally abort the program.
+     *
+     * Depending on whether deal_II_exceptions::disable_abort_on_exception was
+     * called, this function either aborts the program flow by printing the
+     * error message provided by @p exc and calling <tt>std::abort()</tt>, or
+     * throws @p exc instead.
+     */
+    [[noreturn]]
+    void abort (const ExceptionBase &exc);
+
+    /**
+     * An enum describing how to treat an exception in issue_error.
+     */
+    enum ExceptionHandling
+    {
+      /**
+       * Abort the program by calling <code>std::abort</code> unless
+       * deal_II_exceptions::disable_abort_on_exception has been called: in
+       * that case the program will throw an exception.
+       */
+      abort_on_exception,
+      /**
+       * Throw the exception normally.
+       */
+      throw_on_exception,
+      /**
+       * Call <code>std::abort</code> as long as
+       * deal_II_exceptions::disable_abort_on_exception has not been called:
+       * if it has, then just print a description of the exception to deallog.
+       */
+      abort_nothrow_on_exception
+    };
+
+    /**
+     * This routine does the main work for the exception generation mechanism
+     * used in the <tt>Assert</tt> and <tt>AssertThrow</tt> macros: as the
+     * name implies, this function either ends with a call to <tt>abort</tt>
+     * or throwing an exception.
+     *
+     * The actual exception object (the last argument) is typically an unnamed
+     * object created in place; because we modify it, we can't take it by
+     * const reference, and temporaries don't bind to non-const references.
+     * So take it by value (=copy it) with a templated type to avoid slicing
+     * -- the performance implications are pretty minimal anyway.
+     *
+     * @ref ExceptionBase
+     */
+    template <class ExceptionType>
+    [[noreturn]]
+    void issue_error_noreturn (ExceptionHandling  handling,
+                               const char       *file,
+                               int               line,
+                               const char       *function,
+                               const char       *cond,
+                               const char       *exc_name,
+                               ExceptionType     e)
+    {
+      // Fill the fields of the exception object
+      e.set_fields (file, line, function, cond, exc_name);
+
+      switch (handling)
+        {
+        case abort_on_exception:
+          dealii::deal_II_exceptions::internals::abort(e);
+        case throw_on_exception:
+          throw e;
+        // this function should never return (and AssertNothrow can);
+        // something must have gone wrong in the error handling code for us
+        // to get this far, so throw an exception.
+        case abort_nothrow_on_exception:
+        default:
+          throw ::dealii::StandardExceptions::ExcInternalError();
+        }
+    }
+
+    /**
+     * Internal function that does the work of issue_error_nothrow.
+     */
+    void
+    do_issue_error_nothrow(const ExceptionBase &e) noexcept;
+
+    /**
+     * Exception generation mechanism in case we must not throw.
+     *
+     * @ref ExceptionBase
+     *
+     * @note This function is defined with a template for the same reasons as
+     * issue_error_noreturn().
+     */
+    template <class ExceptionType>
+    void issue_error_nothrow (ExceptionHandling,
+                              const char    *file,
+                              int            line,
+                              const char    *function,
+                              const char    *cond,
+                              const char    *exc_name,
+                              ExceptionType  e) noexcept
+    {
+      static_assert(std::is_base_of<ExceptionBase, ExceptionType>::value,
+      "The provided exception must inherit from ExceptionBase.");
+      // Fill the fields of the exception object
+      e.set_fields (file, line, function, cond, exc_name);
+      // avoid moving a bunch of code into the header by dispatching to
+      // another function:
+      do_issue_error_nothrow(e);
+    }
+#ifdef DEAL_II_WITH_CUDA
+    /**
+     * Return a string given an error code. This is similar to the cudaGetErrorString
+     * function but there is no equivalent function for cuSPARSE.
+     */
+    std::string get_cusparse_error_string(const cusparseStatus_t error_code);
+
+    /**
+     * Return a string given an error code. This is similar to the cudaGetErrorString
+     * function but there is no equivalent function for cuSOLVER.
+     */
+    std::string get_cusolver_error_string(const cusolverStatus_t error_code);
+#endif
+  } /*namespace internals*/
+
+} /*namespace deal_II_exceptions*/
+
+
+
+/**
+ * A macro that serves as the main routine in the exception mechanism for debug mode
+ * error checking. It asserts that a certain condition is fulfilled, otherwise
+ * issues an error and aborts the program.
+ *
+ * A more detailed description can be found in the
+ * @ref Exceptions
+ * module. It is first used in step-5 and step-6.
+ * See also the <tt>ExceptionBase</tt> class for more information.
+ *
+ * @note Active in DEBUG mode only
+ * @ingroup Exceptions
+ * @author Wolfgang Bangerth, 1997, 1998, Matthias Maier, 2013
+ */
+#ifdef DEBUG
+#  ifdef DEAL_II_HAVE_BUILTIN_EXPECT
+#    define Assert(cond, exc)                                                \
+{                                                                            \
+  if (__builtin_expect(!(cond), false))                                      \
+    ::dealii::deal_II_exceptions::internals:: issue_error_noreturn(          \
+        ::dealii::deal_II_exceptions::internals::abort_on_exception,         \
+        __FILE__, __LINE__, __PRETTY_FUNCTION__, #cond, #exc, exc);          \
+}
+#  else /*ifdef DEAL_II_HAVE_BUILTIN_EXPECT*/
+#    define Assert(cond, exc)                                                \
+{                                                                            \
+  if (!(cond))                                                               \
+    ::dealii::deal_II_exceptions::internals:: issue_error_noreturn(          \
+        ::dealii::deal_II_exceptions::internals::abort_on_exception,         \
+        __FILE__, __LINE__, __PRETTY_FUNCTION__, #cond, #exc, exc);          \
+}
+#  endif /*ifdef DEAL_II_HAVE_BUILTIN_EXPECT*/
+#else
+#define Assert(cond, exc)                                                    \
+  {}
+#endif
+
+
+
+/**
+ * A variant of the <tt>Assert</tt> macro above that exhibits the same runtime
+ * behavior as long as disable_abort_on_exception was not called.
+ *
+ * However, if disable_abort_on_exception was called, this macro merely prints
+ * the exception that would be thrown to deallog and continues normally
+ * without throwing an exception.
+ *
+ * A more detailed description can be found in the
+ * @ref Exceptions
+ * module, in the discussion about the corner case at the bottom of the page.
+ *
+ * @note Active in DEBUG mode only
+ * @ingroup Exceptions
+ * @author Wolfgang Bangerth, 1997, 1998, Matthias Maier, 2013
+ */
+#ifdef DEBUG
+#  ifdef DEAL_II_HAVE_BUILTIN_EXPECT
+#    define AssertNothrow(cond, exc)                                             \
+{                                                                                \
+  if (__builtin_expect(!(cond), false))                                          \
+    ::dealii::deal_II_exceptions::internals::issue_error_nothrow(                \
+        ::dealii::deal_II_exceptions::internals::abort_nothrow_on_exception,     \
+        __FILE__, __LINE__, __PRETTY_FUNCTION__, #cond, #exc, exc);              \
+}
+#  else /*ifdef DEAL_II_HAVE_BUILTIN_EXPECT*/
+#    define AssertNothrow(cond, exc)                                             \
+{                                                                                \
+  if (!(cond))                                                                   \
+    ::dealii::deal_II_exceptions::internals::issue_error_nothrow(                \
+        ::dealii::deal_II_exceptions::internals::abort_nothrow_on_exception,     \
+        __FILE__, __LINE__, __PRETTY_FUNCTION__, #cond, #exc, exc);              \
+}
+#  endif /*ifdef DEAL_II_HAVE_BUILTIN_EXPECT*/
+#else
+#define AssertNothrow(cond, exc)                                                 \
+  {}
+#endif
+
+/**
+ * A macro that serves as the main routine in the exception mechanism for dynamic
+ * error checking. It asserts that a certain condition is fulfilled, otherwise
+ * throws an exception via the C++ @p throw mechanism. This exception can
+ * be caught via a @p catch clause, as is shown in step-6 and all following
+ * tutorial programs.
+ *
+ * A more detailed description can be found in the
+ * @ref Exceptions
+ * module. It is first used in step-9 and step-13.
+ * See also the <tt>ExceptionBase</tt> class for more information.
+ *
+ * @note Active in both DEBUG and RELEASE modes
+ * @ingroup Exceptions
+ * @author Wolfgang Bangerth, 1997, 1998, Matthias Maier, 2013
+ */
+#ifdef DEAL_II_HAVE_BUILTIN_EXPECT
+#define AssertThrow(cond, exc)                                                 \
+{                                                                              \
+  if (__builtin_expect(!(cond), false))                                        \
+    ::dealii::deal_II_exceptions::internals:: issue_error_noreturn(            \
+        ::dealii::deal_II_exceptions::internals::throw_on_exception,           \
+        __FILE__, __LINE__, __PRETTY_FUNCTION__, #cond, #exc, exc);            \
+}
+#else /*ifdef DEAL_II_HAVE_BUILTIN_EXPECT*/
+#define AssertThrow(cond, exc)                                                 \
+{                                                                              \
+  if (!(cond))                                                                 \
+    ::dealii::deal_II_exceptions::internals::issue_error_noreturn(             \
+        ::dealii::deal_II_exceptions::internals::throw_on_exception,           \
+        __FILE__, __LINE__, __PRETTY_FUNCTION__, #cond, #exc, exc);            \
+}
+#endif /*ifdef DEAL_II_HAVE_BUILTIN_EXPECT*/
+
 /**
  * Special assertion for dimension mismatch.
  *
@@ -1143,8 +1245,8 @@ namespace StandardExceptions
  * @ingroup Exceptions
  * @author Guido Kanschat 2007
  */
-#define AssertDimension(dim1,dim2) Assert((dim1) == (dim2),       \
-                                          dealii::ExcDimensionMismatch((dim1),(dim2)))
+#define AssertDimension(dim1,dim2) \
+  Assert((dim1) == (dim2), dealii::ExcDimensionMismatch((dim1),(dim2)))
 
 
 /**
@@ -1154,9 +1256,21 @@ namespace StandardExceptions
  * @ingroup Exceptions
  * @author Guido Kanschat 2010
  */
-#define AssertVectorVectorDimension(vec,dim1,dim2) AssertDimension((vec).size(), (dim1)) \
-  for (unsigned int i=0;i<dim1;++i) { AssertDimension((vec)[i].size(), (dim2)); }
+#define AssertVectorVectorDimension(vec,dim1,dim2)                              \
+  AssertDimension((vec).size(), (dim1));                                        \
+  for (unsigned int i=0;i<dim1;++i) {AssertDimension((vec)[i].size(), (dim2));}
 
+namespace internal
+{
+  // Workaround to allow for commas in template parameter lists
+  // in preprocessor macros as found in
+  // https://stackoverflow.com/questions/13842468/comma-in-c-c-macro
+  template<typename T> struct argument_type;
+  template<typename T, typename U> struct argument_type<T(U)>
+  {
+    typedef U type;
+  };
+}
 
 /**
  * An assertion that tests that a given index is within the half-open
@@ -1167,23 +1281,11 @@ namespace StandardExceptions
  * @ingroup Exceptions
  * @author Guido Kanschat 2007
  */
-#define AssertIndexRange(index,range) Assert((index) < (range), \
-                                             dealii::ExcIndexRange((index),0,(range)))
-
-/**
- * An assertion that tests that a given index is within the half-open
- * range <code>[0,range)</code>. It throws an exception object
- * <code>ExcIndexRange(index,0,range)</code> if the assertion
- * fails.
- *
- * This variation of the AssertIndexRange assertion is used for indices of type
- * types::global_dof_index for which we need to make special accommodations because
- * they may not fit into the regular 32-bit integer indices used in AssertIndexRange.
- *
- * @ingroup Exceptions
- */
-#define AssertGlobalIndexRange(index,range) Assert((index) < (range), \
-                                                   dealii::ExcIndexRange<types::global_dof_index>((index),0,(range)))
+#define AssertIndexRange(index,range)                                          \
+Assert((index) < (range),                                                      \
+       dealii::ExcIndexRangeType<typename ::dealii::internal::argument_type<   \
+       void(typename std::common_type<decltype(index),                         \
+            decltype(range)>::type)>::type>((index),0,(range)))
 
 /**
  * An assertion that checks whether a number is finite or not. We explicitly
@@ -1194,8 +1296,9 @@ namespace StandardExceptions
  * @ingroup Exceptions
  * @author Wolfgang Bangerth, 2015
  */
-#define AssertIsFinite(number) Assert(dealii::numbers::is_finite(number), \
-                                      dealii::ExcNumberNotFinite(std::complex<double>(number)))
+#define AssertIsFinite(number)                                                 \
+Assert(dealii::numbers::is_finite(number),                                     \
+       dealii::ExcNumberNotFinite(std::complex<double>(number)))
 
 #ifdef DEAL_II_WITH_MPI
 /**
@@ -1208,8 +1311,8 @@ namespace StandardExceptions
  * @ingroup Exceptions
  * @author David Wells, 2016
  */
-#define AssertThrowMPI(error_code) AssertThrow(error_code == MPI_SUCCESS, \
-                                               dealii::ExcMPI(error_code))
+#define AssertThrowMPI(error_code)                                             \
+AssertThrow(error_code == MPI_SUCCESS, dealii::ExcMPI(error_code))
 #else
 #define AssertThrowMPI(error_code) {}
 #endif // DEAL_II_WITH_MPI
@@ -1222,10 +1325,38 @@ namespace StandardExceptions
  * @ingroup Exceptions
  * @author Bruno Turcksin, 2016
  */
-// For now use AssertThrow instead of Assert because macros are not passed
-// correctly to nvcc.
-#define AssertCuda(error_code) AssertThrow(error_code == cudaSuccess, \
-                                           dealii::ExcCudaError(cudaGetErrorString(error_code)))
+#ifdef DEBUG
+#define AssertCuda(error_code) Assert(error_code == cudaSuccess, \
+                                      dealii::ExcCudaError(cudaGetErrorString(error_code)))
+#else
+#define AssertCuda(error_code) { (void) (error_code); }
+#endif
+
+/**
+ * An assertion that checks that the error code produced by calling a cuSPARSE
+ * routine is equal to CUSPARSE_STATUS_SUCCESS.
+ *
+ * @ingroup Exceptions
+ * @author Bruno Turcksin, 2018
+ */
+#ifdef DEBUG
+#define AssertCusparse(error_code) Assert(error_code == CUSPARSE_STATUS_SUCCESS, \
+                                          dealii::ExcCusparseError( \
+                                              dealii::deal_II_exceptions::internals:: \
+                                              get_cusparse_error_string(error_code)))
+#else
+#define AssertCusparse(error_code) { (void) (error_code); }
+#endif
+
+#ifdef DEBUG
+#define AssertCusolver(error_code) Assert(error_code == CUSOLVER_STATUS_SUCCESS, \
+                                          dealii::ExcCusparseError( \
+                                              dealii::deal_II_exceptions::internals:: \
+                                              get_cusolver_error_string(error_code)))
+#else
+#define AssertCusolver(error_code) { (void) (error_code); }
+#endif
+
 #endif
 
 using namespace StandardExceptions;
