@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 2007 - 2015 by the deal.II authors
+// Copyright (C) 2007 - 2017 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -13,8 +13,8 @@
 //
 // ---------------------------------------------------------------------
 
-#ifndef dealii__flow_function_h
-#define dealii__flow_function_h
+#ifndef dealii_flow_function_h
+#define dealii_flow_function_h
 
 
 #include <deal.II/base/config.h>
@@ -56,7 +56,7 @@ namespace Functions
     /**
      * Virtual destructor.
      */
-    virtual ~FlowFunction();
+    virtual ~FlowFunction() = default;
 
     /**
      * Store an adjustment for the pressure function, such that its mean value
@@ -145,7 +145,8 @@ namespace Functions
      */
     PoisseuilleFlow<dim> (const double r,
                           const double Re);
-    virtual ~PoisseuilleFlow();
+
+    virtual ~PoisseuilleFlow() = default;
 
     virtual void vector_values (const std::vector<Point<dim> > &points,
                                 std::vector<std::vector<double> > &values) const;
@@ -187,7 +188,8 @@ namespace Functions
      * Change the viscosity and the reaction parameter.
      */
     void set_parameters (const double viscosity, const double reaction);
-    virtual ~StokesCosine();
+
+    virtual ~StokesCosine() = default;
 
     virtual void vector_values (const std::vector<Point<dim> > &points,
                                 std::vector<std::vector<double> > &values) const;
@@ -205,7 +207,16 @@ namespace Functions
 
 
   /**
-   * The solution to Stokes' equations on an L-shaped domain.
+   * A singular solution to Stokes' equations on a 2d L-shaped domain.
+   *
+   * This function satisfies $-\triangle \mathbf{u} + \nabla p = 0$ and
+   * represents a typical singular solution around a reentrant corner of an
+   * L-shaped domain that can be created using GridGenerator::hyper_L(). The
+   * velocity vanishes on the two faces of the re-entrant corner and
+   * $\nabla\mathbf{u}$ and $p$ are singular at the origin while they are
+   * smooth in the rest of the domain because they can be written as a product
+   * of a smooth function and the term $r^{\lambda-1}$ where $r$ is the radius
+   * and $\lambda \approx 0.54448$ is a fixed parameter.
    *
    * Taken from Houston, Sch&ouml;tzau, Wihler, proceeding ENUMATH 2003.
    *
@@ -235,9 +246,10 @@ namespace Functions
     double Psi_3(double phi) const;
     /// The 4th derivative of Psi()
     double Psi_4(double phi) const;
-    /// The angle of the reentrant corner
+    /// The angle of the reentrant corner, set to 3*pi/2
     const double omega;
-    /// The exponent of the radius
+    /// The exponent of the radius, computed as the solution to
+    /// $\sin(\lambda\omega)+\lambda \sin(\omega)=0$
     static const double lambda;
     /// Cosine of lambda times omega
     const double coslo;
@@ -266,7 +278,8 @@ namespace Functions
      * Stokes problem.
      */
     Kovasznay (const double Re, bool Stokes = false);
-    virtual ~Kovasznay();
+
+    virtual ~Kovasznay() = default;
 
     virtual void vector_values (const std::vector<Point<2> > &points,
                                 std::vector<std::vector<double> > &values) const;
