@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 2016 - 2017 by the deal.II authors
+// Copyright (C) 2016 - 2018 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -13,12 +13,14 @@
 //
 // ---------------------------------------------------------------------
 
-#ifndef dealii__geometric_utilities_h
-#define dealii__geometric_utilities_h
+#ifndef dealii_geometric_utilities_h
+#define dealii_geometric_utilities_h
 
 #include <deal.II/base/config.h>
 #include <deal.II/base/point.h>
-#include <deal.II/base/std_cxx11/array.h>
+
+
+#include <array>
 
 
 DEAL_II_NAMESPACE_OPEN
@@ -44,7 +46,7 @@ namespace GeometricUtilities
     /**
      * Return spherical coordinates of a Cartesian point @p point.
      * The returned array is filled with radius, azimuth angle $\in [0,2 \pi)$
-     * and polar/inclination angle $ \in [0,\pi]$ (ommited in 2D).
+     * and polar/inclination angle $ \in [0,\pi]$ (omitted in 2D).
      *
      * In 3D the transformation is given by
      * @f{align*}{
@@ -54,14 +56,14 @@ namespace GeometricUtilities
      * @f}
      */
     template <int dim>
-    std_cxx11::array<double,dim>
+    std::array<double,dim>
     to_spherical(const Point<dim> &point);
 
     /**
      * Return the Cartesian coordinates of a spherical point defined by @p scoord
      * which is filled with radius $r \in [0,\infty)$, azimuth angle
      * $\theta \in [0,2 \pi)$ and polar/inclination angle $\phi \in [0,\pi]$
-     * (ommited in 2D).
+     * (omitted in 2D).
      *
      * In 3D the transformation is given by
      * @f{align*}{
@@ -72,7 +74,7 @@ namespace GeometricUtilities
      */
     template <std::size_t dim>
     Point<dim>
-    from_spherical(const std_cxx11::array<double,dim> &scoord);
+    from_spherical(const std::array<double,dim> &scoord);
 
   }
 }
