@@ -1,7 +1,7 @@
 
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 2013 - 2016 by the deal.II authors
+// Copyright (C) 2013 - 2017 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -14,16 +14,17 @@
 //
 // ---------------------------------------------------------------------
 
-#ifndef dealii__graph_coloring_h
-#define dealii__graph_coloring_h
+#ifndef dealii_graph_coloring_h
+#define dealii_graph_coloring_h
 
 
 #include <deal.II/base/config.h>
 #include <deal.II/base/thread_management.h>
-#include <deal.II/base/std_cxx11/function.h>
 #include <boost/unordered_map.hpp>
 #include <boost/unordered_set.hpp>
+#include <deal.II/lac/sparsity_tools.h>
 
+#include <functional>
 #include <set>
 #include <vector>
 
@@ -108,7 +109,7 @@ namespace GraphColoring
     std::vector<std::vector<Iterator> >
     create_partitioning(const Iterator &begin,
                         const typename identity<Iterator>::type &end,
-                        const std_cxx11::function<std::vector<types::global_dof_index> (const Iterator &)> &get_conflict_indices)
+                        const std::function<std::vector<types::global_dof_index> (const Iterator &)> &get_conflict_indices)
     {
       // Number of iterators.
       unsigned int n_iterators = 0;
@@ -210,7 +211,7 @@ namespace GraphColoring
     template <typename Iterator>
     void
     make_dsatur_coloring(std::vector<Iterator> &partition,
-                         const std_cxx11::function<std::vector<types::global_dof_index> (const Iterator &)> &get_conflict_indices,
+                         const std::function<std::vector<types::global_dof_index> (const Iterator &)> &get_conflict_indices,
                          std::vector<std::vector<Iterator> > &partition_coloring)
     {
       partition_coloring.clear ();
@@ -514,7 +515,7 @@ namespace GraphColoring
   std::vector<std::vector<Iterator> >
   make_graph_coloring(const Iterator &begin,
                       const typename identity<Iterator>::type &end,
-                      const std_cxx11::function<std::vector<types::global_dof_index> (const typename identity<Iterator>::type &)> &get_conflict_indices)
+                      const std::function<std::vector<types::global_dof_index> (const typename identity<Iterator>::type &)> &get_conflict_indices)
   {
     Assert (begin != end, ExcMessage ("GraphColoring is not prepared to deal with empty ranges!"));
 
@@ -542,12 +543,21 @@ namespace GraphColoring
     return internal::gather_colors(partition_coloring);
   }
 
+  /**
+   * GraphColoring::color_sparsity_pattern, a wrapper function for
+   * SparsityTools::color_sparsity_pattern, is an alternate method for
+   * coloring using graph connections represented by SparsityPattern.
+   * For further details, refer to SparsityTools::color_sparsity_pattern.
+   */
+  unsigned int color_sparsity_pattern (const SparsityPattern     &sparsity_pattern,
+                                       std::vector<unsigned int> &color_indices);
+
 } // End graph_coloring namespace
 
 DEAL_II_NAMESPACE_CLOSE
 
 
 //----------------------------   graph_coloring.h     ---------------------------
-// end of #ifndef dealii__graph_coloring_h
+// end of #ifndef dealii_graph_coloring_h
 #endif
 //----------------------------   graph_coloring.h     ---------------------------
