@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 2009 - 2017 by the deal.II authors
+// Copyright (C) 2009 - 2018 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -13,8 +13,8 @@
 //
 // ---------------------------------------------------------------------
 
-#ifndef dealii__index_set_h
-#define dealii__index_set_h
+#ifndef dealii_index_set_h
+#define dealii_index_set_h
 
 #include <deal.II/base/config.h>
 #include <deal.II/base/utilities.h>
@@ -108,7 +108,6 @@ public:
    */
   explicit IndexSet (const size_type size);
 
-#ifdef DEAL_II_WITH_CXX11
   /**
    * Copy constructor.
    */
@@ -123,14 +122,13 @@ public:
    * Move constructor. Create a new IndexSet by transferring the internal data
    * of the input set.
    */
-  IndexSet (IndexSet &&is);
+  IndexSet (IndexSet &&is) noexcept;
 
   /**
    * Move assignment operator. Transfer the internal data of the input set into
    * the current one.
    */
-  IndexSet &operator= (IndexSet &&is);
-#endif
+  IndexSet &operator= (IndexSet &&is) noexcept;
 
 #ifdef DEAL_II_WITH_TRILINOS
   /**
@@ -316,26 +314,26 @@ public:
                      const size_type end) const;
 
   /**
-   * Removes all elements contained in @p other from this set. In other words,
+   * Remove all elements contained in @p other from this set. In other words,
    * if $x$ is the current object and $o$ the argument, then we compute $x
    * \leftarrow x \backslash o$.
    */
   void subtract_set (const IndexSet &other);
 
   /**
-   * Removes and returns the last element of the last range.
-   * Throws an exception if the IndexSet is empty.
+   * Remove and return the last element of the last range.
+   * This function throws an exception if the IndexSet is empty.
    */
   size_type pop_back ();
 
   /**
-   * Removes and returns the first element of the first range.
-   * Throws an exception if the IndexSet is empty.
+   * Remove and return the first element of the first range.
+   * This function throws an exception if the IndexSet is empty.
    */
   size_type pop_front ();
 
   /**
-   * Fills the given vector with all indices contained in this IndexSet.
+   * Fill the given vector with all indices contained in this IndexSet.
    */
   void fill_index_vector(std::vector<size_type> &indices) const;
 
@@ -355,7 +353,7 @@ public:
   void fill_binary_vector (VectorType &vector) const;
 
   /**
-   * Outputs a text representation of this IndexSet to the given stream. Used
+   * Output a text representation of this IndexSet to the given stream. Used
    * for testing.
    */
   template <class StreamType>
@@ -528,7 +526,7 @@ public:
    * Class that represents an iterator pointing to a contiguous interval
    * $[a,b[$ as returned by IndexSet::begin_interval().
    */
-  class IntervalIterator : public std::iterator<std::forward_iterator_tag,IntervalAccessor>
+  class IntervalIterator
   {
   public:
     /**
@@ -600,6 +598,17 @@ public:
      */
     int operator - (const IntervalIterator &p) const;
 
+    /**
+     * Mark the class as forward iterator and declare some typedefs which are
+     * standard for iterators and are used by algorithms to enquire about the
+     * specifics of the iterators they work on.
+     */
+    typedef std::forward_iterator_tag iterator_category;
+    typedef IntervalAccessor          value_type;
+    typedef std::ptrdiff_t            difference_type;
+    typedef IntervalAccessor         *pointer;
+    typedef IntervalAccessor         &reference;
+
   private:
     /**
      * Accessor that contains what IndexSet and interval we are pointing at.
@@ -611,7 +620,7 @@ public:
    * Class that represents an iterator pointing to a single element in the
    * IndexSet as returned by IndexSet::begin().
    */
-  class ElementIterator : public std::iterator<std::forward_iterator_tag,size_type>
+  class ElementIterator
   {
   public:
     /**
@@ -673,6 +682,17 @@ public:
      */
     std::ptrdiff_t operator - (const ElementIterator &p) const;
 
+    /**
+     * Mark the class as forward iterator and declare some typedefs which are
+     * standard for iterators and are used by algorithms to enquire about the
+     * specifics of the iterators they work on.
+     */
+    typedef std::forward_iterator_tag iterator_category;
+    typedef size_type                 value_type;
+    typedef std::ptrdiff_t            difference_type;
+    typedef size_type                *pointer;
+    typedef size_type                &reference;
+
   private:
     /**
      * Advance iterator by one.
@@ -882,13 +902,14 @@ private:
  *     ...
  * @endcode
  *
- * @relates IndexSet
+ * @relatesalso IndexSet
  */
 inline
 IndexSet complete_index_set (const unsigned int N)
 {
   IndexSet is (N);
   is.add_range(0, N);
+  is.compress();
   return is;
 }
 
@@ -898,39 +919,74 @@ IndexSet complete_index_set (const unsigned int N)
 /* IntervalAccessor */
 
 inline
-IndexSet::IntervalAccessor::IntervalAccessor(const IndexSet *idxset, const IndexSet::size_type range_idx)
-  : index_set(idxset), range_idx(range_idx)
+IndexSet::IntervalAccessor::
+IntervalAccessor(const IndexSet *idxset,
+                 const IndexSet::size_type range_idx)
+  :
+  index_set(idxset),
+  range_idx(range_idx)
 {
   Assert(range_idx < idxset->n_intervals(), ExcInternalError("Invalid range index"));
 }
 
-inline
-IndexSet::IntervalAccessor::IntervalAccessor(const IndexSet *idxset)
-  : index_set(idxset), range_idx(numbers::invalid_dof_index)
-{}
+
 
 inline
-IndexSet::size_type IndexSet::IntervalAccessor::n_elements() const
+IndexSet::IntervalAccessor::
+IntervalAccessor(const IndexSet *idxset)
+  :
+  index_set(idxset),
+  range_idx(numbers::invalid_dof_index)
+{}
+
+
+
+inline
+IndexSet::IntervalAccessor::
+IntervalAccessor(const IndexSet::IntervalAccessor &other)
+  :
+  index_set (other.index_set),
+  range_idx(other.range_idx)
+{
+  Assert( range_idx == numbers::invalid_dof_index || is_valid(),
+          ExcMessage("invalid iterator"));
+}
+
+
+
+inline
+IndexSet::size_type
+IndexSet::IntervalAccessor::n_elements() const
 {
   Assert(is_valid(), ExcMessage("invalid iterator"));
   return index_set->ranges[range_idx].end - index_set->ranges[range_idx].begin;
 }
 
-inline
-bool IndexSet::IntervalAccessor::is_valid() const
-{
-  return index_set != NULL && range_idx < index_set->n_intervals();
-}
+
 
 inline
-IndexSet::ElementIterator IndexSet::IntervalAccessor::begin() const
+bool
+IndexSet::IntervalAccessor::is_valid() const
+{
+  return index_set != nullptr && range_idx < index_set->n_intervals();
+}
+
+
+
+inline
+IndexSet::ElementIterator
+IndexSet::IntervalAccessor::begin() const
 {
   Assert(is_valid(), ExcMessage("invalid iterator"));
-  return IndexSet::ElementIterator(index_set, range_idx, index_set->ranges[range_idx].begin);
+  return IndexSet::ElementIterator(index_set, range_idx,
+                                   index_set->ranges[range_idx].begin);
 }
 
+
+
 inline
-IndexSet::ElementIterator IndexSet::IntervalAccessor::end() const
+IndexSet::ElementIterator
+IndexSet::IntervalAccessor::end() const
 {
   Assert(is_valid(), ExcMessage("invalid iterator"));
 
@@ -941,6 +997,8 @@ IndexSet::ElementIterator IndexSet::IntervalAccessor::end() const
     return index_set->end();
 }
 
+
+
 inline
 IndexSet::size_type
 IndexSet::IntervalAccessor::last() const
@@ -950,12 +1008,7 @@ IndexSet::IntervalAccessor::last() const
   return index_set->ranges[range_idx].end-1;
 }
 
-inline
-IndexSet::IntervalAccessor::IntervalAccessor(const IndexSet::IntervalAccessor &other)
-  : index_set (other.index_set), range_idx(other.range_idx)
-{
-  Assert( range_idx == numbers::invalid_dof_index || is_valid(),  ExcMessage("invalid iterator"));
-}
+
 
 inline
 IndexSet::IntervalAccessor &
@@ -963,28 +1016,40 @@ IndexSet::IntervalAccessor::operator = (const IndexSet::IntervalAccessor &other)
 {
   index_set = other.index_set;
   range_idx = other.range_idx;
-  Assert( range_idx == numbers::invalid_dof_index || is_valid(),  ExcMessage("invalid iterator"));
+  Assert( range_idx == numbers::invalid_dof_index || is_valid(),
+          ExcMessage("invalid iterator"));
   return *this;
 }
 
+
+
 inline
-bool IndexSet::IntervalAccessor::operator == (const IndexSet::IntervalAccessor &other) const
+bool
+IndexSet::IntervalAccessor::operator == (const IndexSet::IntervalAccessor &other) const
 {
-  Assert (index_set == other.index_set, ExcMessage("Can not compare accessors pointing to different IndexSets"));
+  Assert (index_set == other.index_set,
+          ExcMessage("Can not compare accessors pointing to different IndexSets"));
   return range_idx == other.range_idx;
 }
 
+
+
 inline
-bool IndexSet::IntervalAccessor::operator < (const IndexSet::IntervalAccessor &other) const
+bool
+IndexSet::IntervalAccessor::operator < (const IndexSet::IntervalAccessor &other) const
 {
-  Assert (index_set == other.index_set, ExcMessage("Can not compare accessors pointing to different IndexSets"));
+  Assert (index_set == other.index_set,
+          ExcMessage("Can not compare accessors pointing to different IndexSets"));
   return range_idx < other.range_idx;
 }
 
+
+
 inline
 void IndexSet::IntervalAccessor::advance ()
 {
-  Assert(is_valid(), ExcMessage("Impossible to advance an IndexSet::IntervalIterator that is invalid"));
+  Assert (is_valid(),
+          ExcMessage("Impossible to advance an IndexSet::IntervalIterator that is invalid"));
   ++range_idx;
 
   // set ourselves to invalid if we walk off the end
@@ -992,28 +1057,42 @@ void IndexSet::IntervalAccessor::advance ()
     range_idx = numbers::invalid_dof_index;
 }
 
+
 /* IntervalIterator */
 
 inline
-IndexSet::IntervalIterator::IntervalIterator(const IndexSet *idxset, const IndexSet::size_type range_idx)
-  : accessor(idxset, range_idx)
+IndexSet::IntervalIterator::IntervalIterator(const IndexSet *idxset,
+                                             const IndexSet::size_type range_idx)
+  :
+  accessor(idxset, range_idx)
 {}
 
+
+
 inline
 IndexSet::IntervalIterator::IntervalIterator()
-  : accessor(NULL)
+  :
+  accessor(nullptr)
 {}
 
+
+
 inline
 IndexSet::IntervalIterator::IntervalIterator(const IndexSet *idxset)
-  : accessor(idxset)
+  :
+  accessor(idxset)
 {}
 
+
+
 inline
 IndexSet::IntervalIterator::IntervalIterator(const IndexSet::IntervalIterator &other)
-  : accessor(other.accessor)
+  :
+  accessor(other.accessor)
 {}
 
+
+
 inline
 IndexSet::IntervalIterator &
 IndexSet::IntervalIterator::operator = (const IntervalIterator &other)
@@ -1023,6 +1102,7 @@ IndexSet::IntervalIterator::operator = (const IntervalIterator &other)
 }
 
 
+
 inline
 IndexSet::IntervalIterator &
 IndexSet::IntervalIterator::operator++ ()
@@ -1031,6 +1111,8 @@ IndexSet::IntervalIterator::operator++ ()
   return *this;
 }
 
+
+
 inline
 IndexSet::IntervalIterator
 IndexSet::IntervalIterator::operator++ (int)
@@ -1040,6 +1122,8 @@ IndexSet::IntervalIterator::operator++ (int)
   return iter;
 }
 
+
+
 inline
 const IndexSet::IntervalAccessor &
 IndexSet::IntervalIterator::operator* () const
@@ -1047,6 +1131,8 @@ IndexSet::IntervalIterator::operator* () const
   return accessor;
 }
 
+
+
 inline
 const IndexSet::IntervalAccessor *
 IndexSet::IntervalIterator::operator-> () const
@@ -1054,31 +1140,52 @@ IndexSet::IntervalIterator::operator-> () const
   return &accessor;
 }
 
+
+
 inline
-bool IndexSet::IntervalIterator::operator == (const IndexSet::IntervalIterator &other) const
+bool
+IndexSet::IntervalIterator::operator == (const IndexSet::IntervalIterator &other) const
 {
   return accessor == other.accessor;
 }
 
+
+
 inline
-bool IndexSet::IntervalIterator::operator != (const IndexSet::IntervalIterator &other) const
+bool
+IndexSet::IntervalIterator::operator != (const IndexSet::IntervalIterator &other) const
 {
   return !(*this == other);
 }
 
+
+
 inline
-bool IndexSet::IntervalIterator::operator < (const IndexSet::IntervalIterator &other) const
+bool
+IndexSet::IntervalIterator::operator < (const IndexSet::IntervalIterator &other) const
 {
   return accessor < other.accessor;
 }
 
-inline
-int IndexSet::IntervalIterator::operator - (const IndexSet::IntervalIterator &other) const
-{
-  Assert (accessor.index_set == other.accessor.index_set, ExcMessage("Can not compare iterators belonging to different IndexSets"));
 
-  const size_type lhs = (accessor.range_idx == numbers::invalid_dof_index) ? accessor.index_set->ranges.size() : accessor.range_idx;
-  const size_type rhs = (other.accessor.range_idx == numbers::invalid_dof_index) ? accessor.index_set->ranges.size() : other.accessor.range_idx;
+
+inline
+int
+IndexSet::IntervalIterator::operator - (const IndexSet::IntervalIterator &other) const
+{
+  Assert (accessor.index_set == other.accessor.index_set,
+          ExcMessage("Can not compare iterators belonging to different IndexSets"));
+
+  const size_type lhs = (accessor.range_idx == numbers::invalid_dof_index)
+                        ?
+                        accessor.index_set->ranges.size()
+                        :
+                        accessor.range_idx;
+  const size_type rhs = (other.accessor.range_idx == numbers::invalid_dof_index)
+                        ?
+                        accessor.index_set->ranges.size()
+                        :
+                        other.accessor.range_idx;
 
   if (lhs > rhs)
     return static_cast<int>(lhs - rhs);
@@ -1087,23 +1194,18 @@ int IndexSet::IntervalIterator::operator - (const IndexSet::IntervalIterator &ot
 }
 
 
+
 /* ElementIterator */
 
 inline
-bool IndexSet::ElementIterator::is_valid() const
-{
-  Assert(
-    (range_idx == numbers::invalid_dof_index && idx == numbers::invalid_dof_index)
-    ||
-    (range_idx < index_set->ranges.size() && idx<index_set->ranges[range_idx].end)
-    , ExcInternalError("Invalid ElementIterator state."));
-
-  return range_idx < index_set->ranges.size() && idx<index_set->ranges[range_idx].end;
-}
-
-inline
-IndexSet::ElementIterator::ElementIterator(const IndexSet *idxset, const IndexSet::size_type range_idx, const IndexSet::size_type index)
-  : index_set(idxset), range_idx(range_idx), idx(index)
+IndexSet::ElementIterator::
+ElementIterator(const IndexSet *idxset,
+                const IndexSet::size_type range_idx,
+                const IndexSet::size_type index)
+  :
+  index_set(idxset),
+  range_idx(range_idx),
+  idx(index)
 {
   Assert(range_idx < index_set->ranges.size(),
          ExcMessage("Invalid range index for IndexSet::ElementIterator constructor."));
@@ -1113,30 +1215,62 @@ IndexSet::ElementIterator::ElementIterator(const IndexSet *idxset, const IndexSe
          ExcInternalError("Invalid index argument for IndexSet::ElementIterator constructor."));
 }
 
+
+
 inline
 IndexSet::ElementIterator::ElementIterator(const IndexSet *idxset)
-  : index_set(idxset), range_idx(numbers::invalid_dof_index), idx(numbers::invalid_dof_index)
+  :
+  index_set(idxset),
+  range_idx(numbers::invalid_dof_index),
+  idx(numbers::invalid_dof_index)
 {}
 
+
+
+inline
+bool
+IndexSet::ElementIterator::is_valid() const
+{
+  Assert((range_idx == numbers::invalid_dof_index && idx == numbers::invalid_dof_index)
+         ||
+         (range_idx < index_set->ranges.size() && idx<index_set->ranges[range_idx].end),
+         ExcInternalError("Invalid ElementIterator state."));
+
+  return (range_idx < index_set->ranges.size() &&
+          idx<index_set->ranges[range_idx].end);
+}
+
+
+
+
 inline
 IndexSet::size_type
 IndexSet::ElementIterator::operator* () const
 {
-  Assert(is_valid(), ExcMessage("Impossible to dereference an IndexSet::ElementIterator that is invalid"));
+  Assert (is_valid(),
+          ExcMessage("Impossible to dereference an IndexSet::ElementIterator that is invalid"));
   return idx;
 }
 
+
+
 inline
-bool IndexSet::ElementIterator::operator == (const IndexSet::ElementIterator &other) const
+bool
+IndexSet::ElementIterator::operator == (const IndexSet::ElementIterator &other) const
 {
-  Assert (index_set == other.index_set, ExcMessage("Can not compare iterators belonging to different IndexSets"));
+  Assert (index_set == other.index_set,
+          ExcMessage("Can not compare iterators belonging to different IndexSets"));
   return range_idx == other.range_idx && idx==other.idx;
 }
 
+
+
 inline
-void IndexSet::ElementIterator::advance ()
+void
+IndexSet::ElementIterator::advance ()
 {
-  Assert(is_valid(), ExcMessage("Impossible to advance an IndexSet::ElementIterator that is invalid"));
+  Assert (is_valid(),
+          ExcMessage("Impossible to advance an IndexSet::ElementIterator that is invalid"));
   if (idx < index_set->ranges[range_idx].end)
     ++idx;
   // end of this range?
@@ -1157,6 +1291,8 @@ void IndexSet::ElementIterator::advance ()
     }
 }
 
+
+
 inline
 IndexSet::ElementIterator &
 IndexSet::ElementIterator::operator++ ()
@@ -1165,6 +1301,8 @@ IndexSet::ElementIterator::operator++ ()
   return *this;
 }
 
+
+
 inline
 IndexSet::ElementIterator
 IndexSet::ElementIterator::operator++ (int)
@@ -1174,23 +1312,32 @@ IndexSet::ElementIterator::operator++ (int)
   return it;
 }
 
+
+
 inline
 bool IndexSet::ElementIterator::operator != (const IndexSet::ElementIterator &other) const
 {
   return !(*this == other);
 }
 
+
+
 inline
 bool IndexSet::ElementIterator::operator < (const IndexSet::ElementIterator &other) const
 {
-  Assert (index_set == other.index_set, ExcMessage("Can not compare iterators belonging to different IndexSets"));
+  Assert (index_set == other.index_set,
+          ExcMessage("Can not compare iterators belonging to different IndexSets"));
   return range_idx < other.range_idx || (range_idx == other.range_idx && idx<other.idx);
 }
 
+
+
 inline
-std::ptrdiff_t IndexSet::ElementIterator::operator - (const IndexSet::ElementIterator &other) const
+std::ptrdiff_t
+IndexSet::ElementIterator::operator - (const IndexSet::ElementIterator &other) const
 {
-  Assert (index_set == other.index_set, ExcMessage("Can not compare iterators belonging to different IndexSets"));
+  Assert (index_set == other.index_set,
+          ExcMessage("Can not compare iterators belonging to different IndexSets"));
   if (*this == other)
     return 0;
   if (!(*this < other))
@@ -1230,6 +1377,7 @@ IndexSet::Range::Range ()
 {}
 
 
+
 inline
 IndexSet::Range::Range (const size_type i1,
                         const size_type i2)
@@ -1240,8 +1388,67 @@ IndexSet::Range::Range (const size_type i1,
 {}
 
 
+
 /* IndexSet itself */
 
+inline
+IndexSet::IndexSet ()
+  :
+  is_compressed (true),
+  index_space_size (0),
+  largest_range (numbers::invalid_unsigned_int)
+{}
+
+
+
+inline
+IndexSet::IndexSet (const size_type size)
+  :
+  is_compressed (true),
+  index_space_size (size),
+  largest_range (numbers::invalid_unsigned_int)
+{}
+
+
+
+inline
+IndexSet::IndexSet (IndexSet &&is) noexcept
+:
+ranges (std::move(is.ranges)),
+       is_compressed (is.is_compressed),
+       index_space_size (is.index_space_size),
+       largest_range (is.largest_range)
+{
+  is.ranges.clear ();
+  is.is_compressed = true;
+  is.index_space_size = 0;
+  is.largest_range = numbers::invalid_unsigned_int;
+
+  compress ();
+}
+
+
+
+inline
+IndexSet &IndexSet::operator= (IndexSet &&is) noexcept
+{
+  ranges = std::move (is.ranges);
+  is_compressed = is.is_compressed;
+  index_space_size = is.index_space_size;
+  largest_range = is.largest_range;
+
+  is.ranges.clear ();
+  is.is_compressed = true;
+  is.index_space_size = 0;
+  is.largest_range = numbers::invalid_unsigned_int;
+
+  compress ();
+
+  return *this;
+}
+
+
+
 inline
 IndexSet::ElementIterator IndexSet::begin() const
 {
@@ -1252,6 +1459,8 @@ IndexSet::ElementIterator IndexSet::begin() const
     return end();
 }
 
+
+
 inline
 IndexSet::ElementIterator IndexSet::at(const size_type global_index) const
 {
@@ -1301,16 +1510,20 @@ IndexSet::ElementIterator IndexSet::at(const size_type global_index) const
 }
 
 
+
 inline
-IndexSet::ElementIterator IndexSet::end() const
+IndexSet::ElementIterator
+IndexSet::end() const
 {
   compress();
   return IndexSet::ElementIterator(this);
 }
 
 
+
 inline
-IndexSet::IntervalIterator IndexSet::begin_intervals() const
+IndexSet::IntervalIterator
+IndexSet::begin_intervals() const
 {
   compress();
   if (ranges.size()>0)
@@ -1319,8 +1532,11 @@ IndexSet::IntervalIterator IndexSet::begin_intervals() const
     return end_intervals();
 }
 
+
+
 inline
-IndexSet::IntervalIterator IndexSet::end_intervals() const
+IndexSet::IntervalIterator
+IndexSet::end_intervals() const
 {
   compress();
   return IndexSet::IntervalIterator(this);
@@ -1328,66 +1544,6 @@ IndexSet::IntervalIterator IndexSet::end_intervals() const
 
 
 
-inline
-IndexSet::IndexSet ()
-  :
-  is_compressed (true),
-  index_space_size (0),
-  largest_range (numbers::invalid_unsigned_int)
-{}
-
-
-
-inline
-IndexSet::IndexSet (const size_type size)
-  :
-  is_compressed (true),
-  index_space_size (size),
-  largest_range (numbers::invalid_unsigned_int)
-{}
-
-
-
-#ifdef DEAL_II_WITH_CXX11
-
-inline
-IndexSet::IndexSet (IndexSet &&is)
-  :
-  ranges (std::move(is.ranges)),
-  is_compressed (is.is_compressed),
-  index_space_size (is.index_space_size),
-  largest_range (is.largest_range)
-{
-  is.ranges.clear ();
-  is.is_compressed = true;
-  is.index_space_size = 0;
-  is.largest_range = numbers::invalid_unsigned_int;
-
-  compress ();
-}
-
-
-inline
-IndexSet &IndexSet::operator= (IndexSet &&is)
-{
-  ranges = std::move (is.ranges);
-  is_compressed = is.is_compressed;
-  index_space_size = is.index_space_size;
-  largest_range = is.largest_range;
-
-  is.ranges.clear ();
-  is.is_compressed = true;
-  is.index_space_size = 0;
-  is.largest_range = numbers::invalid_unsigned_int;
-
-  compress ();
-
-  return *this;
-}
-
-#endif
-
-
 inline
 void
 IndexSet::clear ()
@@ -1546,6 +1702,7 @@ IndexSet::is_contiguous () const
 }
 
 
+
 inline
 bool
 IndexSet::is_empty () const
@@ -1648,6 +1805,7 @@ IndexSet::nth_index_in_set (const unsigned int n) const
 }
 
 
+
 inline
 IndexSet::size_type
 IndexSet::index_within_set (const size_type n) const
