@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 1998 - 2015 by the deal.II authors
+// Copyright (C) 1998 - 2017 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -13,8 +13,8 @@
 //
 // ---------------------------------------------------------------------
 
-#ifndef dealii__job_identifier_h
-#define dealii__job_identifier_h
+#ifndef dealii_job_identifier_h
+#define dealii_job_identifier_h
 
 
 #include <deal.II/base/config.h>
@@ -49,10 +49,11 @@ public:
   static const char *program_id();
 
   /**
-   * Obtain the base name of the file currently being compiled. That is, if
-   * the file is <tt>mypath/file.cc</tt> return just <tt>file</tt>. Typically,
-   * this can be called from a program with the argument <tt>__FILE__</tt> and
-   * is used in the deal.II test suite.
+   * Obtain the base name of the filename passed as argument. That is,
+   * if the file is <tt>mypath/file.cc</tt> return just
+   * <tt>file</tt>. For example, this function can be called from a
+   * user program with argument <tt>__FILE__</tt> to create an
+   * identifier for the program being run.
    */
   static std::string base_name(const char *filename);
 
