@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 2008 - 2016 by the deal.II authors
+// Copyright (C) 2008 - 2018 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -13,8 +13,8 @@
 //
 // ---------------------------------------------------------------------
 
-#ifndef dealii__parallel_h
-#define dealii__parallel_h
+#ifndef dealii_parallel_h
+#define dealii_parallel_h
 
 
 #include <deal.II/base/config.h>
@@ -23,12 +23,10 @@
 #include <deal.II/base/synchronous_iterator.h>
 #include <deal.II/base/thread_management.h>
 
-#include <deal.II/base/std_cxx11/tuple.h>
-#include <deal.II/base/std_cxx11/bind.h>
-#include <deal.II/base/std_cxx11/function.h>
-#include <deal.II/base/std_cxx11/shared_ptr.h>
-
 #include <cstddef>
+#include <tuple>
+#include <memory>
+#include <functional>
 
 #ifdef DEAL_II_WITH_THREADS
 #  include <tbb/parallel_for.h>
@@ -91,7 +89,7 @@ namespace parallel
       {
         for (typename Range::const_iterator p=range.begin();
              p != range.end(); ++p)
-          apply (f, p.iterators);
+          apply (f, *p);
       }
 
     private:
@@ -107,9 +105,9 @@ namespace parallel
       static
       void
       apply (const F &f,
-             const std_cxx11::tuple<I1,I2> &p)
+             const std::tuple<I1,I2> &p)
       {
-        *std_cxx11::get<1>(p) = f (*std_cxx11::get<0>(p));
+        *std::get<1>(p) = f (*std::get<0>(p));
       }
 
       /**
@@ -119,10 +117,10 @@ namespace parallel
       static
       void
       apply (const F &f,
-             const std_cxx11::tuple<I1,I2,I3> &p)
+             const std::tuple<I1,I2,I3> &p)
       {
-        *std_cxx11::get<2>(p) = f (*std_cxx11::get<0>(p),
-                                   *std_cxx11::get<1>(p));
+        *std::get<2>(p) = f (*std::get<0>(p),
+                             *std::get<1>(p));
       }
 
       /**
@@ -133,11 +131,11 @@ namespace parallel
       static
       void
       apply (const F &f,
-             const std_cxx11::tuple<I1,I2,I3,I4> &p)
+             const std::tuple<I1,I2,I3,I4> &p)
       {
-        *std_cxx11::get<3>(p) = f (*std_cxx11::get<0>(p),
-                                   *std_cxx11::get<1>(p),
-                                   *std_cxx11::get<2>(p));
+        *std::get<3>(p) = f (*std::get<0>(p),
+                             *std::get<1>(p),
+                             *std::get<2>(p));
       }
     };
 
@@ -195,7 +193,7 @@ namespace parallel
     for (OutputIterator in = begin_in; in != end_in;)
       *out++ = predicate (*in++);
 #else
-    typedef std_cxx11::tuple<InputIterator,OutputIterator> Iterators;
+    typedef std::tuple<InputIterator,OutputIterator> Iterators;
     typedef SynchronousIterators<Iterators> SyncIterators;
     Iterators x_begin (begin_in, out);
     Iterators x_end (end_in, OutputIterator());
@@ -252,7 +250,7 @@ namespace parallel
       *out++ = predicate (*in1++, *in2++);
 #else
     typedef
-    std_cxx11::tuple<InputIterator1,InputIterator2,OutputIterator>
+    std::tuple<InputIterator1,InputIterator2,OutputIterator>
     Iterators;
     typedef SynchronousIterators<Iterators> SyncIterators;
     Iterators x_begin (begin_in1, in2, out);
@@ -312,7 +310,7 @@ namespace parallel
       *out++ = predicate (*in1++, *in2++, *in3++);
 #else
     typedef
-    std_cxx11::tuple<InputIterator1,InputIterator2,InputIterator3,OutputIterator>
+    std::tuple<InputIterator1,InputIterator2,InputIterator3,OutputIterator>
     Iterators;
     typedef SynchronousIterators<Iterators> SyncIterators;
     Iterators x_begin (begin_in1, in2, in3, out);
@@ -371,11 +369,11 @@ namespace parallel
    *   {
    *     parallel::apply_to_subranges
    *        (0, A.n_rows(),
-   *         std_cxx11::bind (&mat_vec_on_subranges,
-   *                          std_cxx11::_1, std_cxx11::_2,
-   *                          std_cxx11::cref(A),
-   *                          std_cxx11::cref(x),
-   *                          std_cxx11::ref(y)),
+   *         std::bind (&mat_vec_on_subranges,
+   *                    std::placeholders::_1, std::placeholders::_2,
+   *                    std::cref(A),
+   *                    std::cref(x),
+   *                    std::ref(y)),
    *         50);
    *   }
    *
@@ -391,10 +389,10 @@ namespace parallel
    *   }
    * @endcode
    *
-   * Note how we use the <code>std_cxx11::bind</code> function to convert
+   * Note how we use the <code>std::bind</code> function to convert
    * <code>mat_vec_on_subranges</code> from a function that takes 5 arguments
    * to one taking 2 by binding the remaining arguments (the modifiers
-   * <code>std_cxx11::ref</code> and <code>std_cxx11::cref</code> make sure
+   * <code>std::ref</code> and <code>std::cref</code> make sure
    * that the enclosed variables are actually passed by reference and constant
    * reference, rather than by value). The resulting function object requires
    * only two arguments, begin_row and end_row, with all other arguments
@@ -437,9 +435,9 @@ namespace parallel
 #else
     tbb::parallel_for (tbb::blocked_range<RangeType>
                        (begin, end, grainsize),
-                       std_cxx11::bind (&internal::apply_to_subranges<RangeType,Function>,
-                                        std_cxx11::_1,
-                                        std_cxx11::cref(f)),
+                       std::bind (&internal::apply_to_subranges<RangeType,Function>,
+                                  std::placeholders::_1,
+                                  std::cref(f)),
                        tbb::auto_partitioner());
 #endif
   }
@@ -479,7 +477,7 @@ namespace parallel
      * Destructor. Made virtual to ensure that derived classes also have
      * virtual destructors.
      */
-    virtual ~ParallelForInteger ();
+    virtual ~ParallelForInteger () = default;
 
     /**
      * This function runs the for loop over the given range
@@ -591,7 +589,7 @@ namespace parallel
        * The function object to be used to reduce the result of two calls into
        * one number.
        */
-      const std_cxx11::function<ResultType (ResultType, ResultType)> reductor;
+      const std::function<ResultType (ResultType, ResultType)> reductor;
     };
 #endif
   }
@@ -614,10 +612,10 @@ namespace parallel
    *      std::sqrt
    *       (parallel::accumulate_from_subranges<double>
    *        (0, A.n_rows(),
-   *         std_cxx11::bind (&mat_norm_sqr_on_subranges,
-   *                          std_cxx11::_1, std_cxx11::_2,
-   *                          std_cxx11::cref(A),
-   *                          std_cxx11::cref(x)),
+   *         std::bind (&mat_norm_sqr_on_subranges,
+   *                     std::placeholders::_1, std::placeholders::_2,
+   *                     std::cref(A),
+   *                     std::cref(x)),
    *         50);
    *   }
    *
@@ -705,25 +703,14 @@ namespace parallel
       /**
        * Constructor.
        */
-      TBBPartitioner()
-#ifdef DEAL_II_WITH_THREADS
-        :
-        my_partitioner(new tbb::affinity_partitioner()),
-        in_use(false)
-#endif
-      {}
+      TBBPartitioner();
 
 #ifdef DEAL_II_WITH_THREADS
       /**
        * Destructor. Check that the object is not in use any more, i.e., all
        * loops have been completed.
        */
-      ~TBBPartitioner()
-      {
-        Assert(in_use == false,
-               ExcInternalError("A vector partitioner goes out of scope, but "
-                                "it appears to be still in use."));
-      }
+      ~TBBPartitioner();
 
       /**
        * Return an affinity partitioner. In case the partitioner owned by the
@@ -731,37 +718,22 @@ namespace parallel
        * released it yet, a new object is created. To free the partitioner
        * again, return it by the release_one_partitioner() call.
        */
-      std_cxx11::shared_ptr<tbb::affinity_partitioner>
-      acquire_one_partitioner()
-      {
-        dealii::Threads::Mutex::ScopedLock lock(mutex);
-        if (in_use)
-          return std_cxx11::shared_ptr<tbb::affinity_partitioner>(new tbb::affinity_partitioner());
-
-        in_use = true;
-        return my_partitioner;
-      }
+      std::shared_ptr<tbb::affinity_partitioner>
+      acquire_one_partitioner();
 
       /**
        * After using the partitioner in a tbb loop through
        * acquire_one_partitioner(), this call makes the partitioner available
        * again.
        */
-      void release_one_partitioner(std_cxx11::shared_ptr<tbb::affinity_partitioner> &p)
-      {
-        if (p.get() == my_partitioner.get())
-          {
-            dealii::Threads::Mutex::ScopedLock lock(mutex);
-            in_use = false;
-          }
-      }
+      void release_one_partitioner(std::shared_ptr<tbb::affinity_partitioner> &p);
 
     private:
       /**
        * The stored partitioner that can accumulate knowledge over several
        * runs of tbb::parallel_for
        */
-      std_cxx11::shared_ptr<tbb::affinity_partitioner> my_partitioner;
+      std::shared_ptr<tbb::affinity_partitioner> my_partitioner;
 
       /**
        * A flag to indicate whether the partitioner has been acquired but not
@@ -781,7 +753,7 @@ namespace parallel
 
 namespace internal
 {
-  namespace Vector
+  namespace VectorImplementation
   {
     /**
      * If we do computations on vectors in parallel (say, we add two vectors
@@ -801,10 +773,10 @@ namespace internal
   }
 
 
-  namespace SparseMatrix
+  namespace SparseMatrixImplementation
   {
     /**
-     * Like internal::Vector::minimum_parallel_grain_size, but now denoting
+     * Like internal::VectorImplementation::minimum_parallel_grain_size, but now denoting
      * the number of rows of a matrix that should be worked on as a minimum.
      */
     extern unsigned int minimum_parallel_grain_size;
@@ -845,11 +817,6 @@ namespace parallel
 #endif
 
 
-  inline
-  ParallelForInteger::~ParallelForInteger ()
-  {}
-
-
   inline
   void
   ParallelForInteger::apply_parallel (const std::size_t begin,
