@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 2005 - 2016 by the deal.II authors
+// Copyright (C) 2005 - 2017 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -13,8 +13,8 @@
 //
 // ---------------------------------------------------------------------
 
-#ifndef dealii__path_search_h
-#define dealii__path_search_h
+#ifndef dealii_path_search_h
+#define dealii_path_search_h
 
 
 #include <deal.II/base/config.h>
