@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 1998 - 2016 by the deal.II authors
+// Copyright (C) 1998 - 2018 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -13,8 +13,8 @@
 //
 // ---------------------------------------------------------------------
 
-#ifndef dealii__point_h
-#define dealii__point_h
+#ifndef dealii_point_h
+#define dealii_point_h
 
 
 #include <deal.II/base/config.h>
@@ -25,16 +25,31 @@
 DEAL_II_NAMESPACE_OPEN
 
 /**
- * A class that represents a point in a space with arbitrary dimension
- * <tt>dim</tt>.
+ * A class that represents a point in a Cartesian space of dimension
+ * @p dim .
  *
- * Objects of this class are used to represent points, i.e., vectors anchored
- * at the origin of a Cartesian vector space. They are, among other uses,
- * passed to functions that operate on points in spaces of a priori fixed
- * dimension: rather than using functions like <tt>double f(double x)</tt> and
- * <tt>double f(double x, double y)</tt>, you should use <tt>double
- * f(Point<dim> &p)</tt> instead as it allows writing dimension independent
- * code.
+ * Objects of this class are used to represent points (i.e., vectors
+ * anchored at the origin) of a vector space equipped with a <a
+ * href="https://en.wikipedia.org/wiki/Cartesian_coordinate_system">Cartesian
+ * coordinate system</a>. They are, among other uses, passed to
+ * functions that operate on points in spaces of a priori fixed
+ * dimension: rather than using functions like <code>double f(const
+ * double x)</code> and <code>double f(const double x, const double
+ * y)</code>, you can use <code>double f(const Point<dim> &p)</code>
+ * instead as it allows writing dimension independent code.
+ *
+ * deal.II specifically uses Point objects as indicating points that
+ * are represented by Cartesian coordinates, i.e., where a point in @p
+ * dim space dimensions is characterized by signed distances along the
+ * axes of a coordinate system spanned by @p dim mutually orthogonal
+ * unit vectors (called the "coordinate axes"). This choice of
+ * representing a vector makes addition and scaling of vectors
+ * particularly simple: one only has to add or multiply each
+ * coordinate value. On the other hand, adding or scaling vectors is
+ * not nearly as simple when a vector is represented in other kinds of
+ * coordinate systems (e.g., <a
+ * href="https://en.wikipedia.org/wiki/Spherical_coordinate_system">spherical
+ * coordinate systems</a>).
  *
  *
  * <h3>What's a <code>Point@<dim@></code> and what is a
@@ -63,7 +78,7 @@ DEAL_II_NAMESPACE_OPEN
  * a tensor (because it lacks the transformation properties under rotation of
  * the coordinate system) and should consequently not be represented by either
  * of these classes. Use an array of size 3 in this case, or the
- * <code>std_cxx11::array</code> class. Alternatively, as in the case of
+ * <code>std::array</code> class. Alternatively, as in the case of
  * vector-valued functions, you can use objects of type Vector or
  * <code>std::vector</code>.
  *
@@ -190,7 +205,7 @@ public:
   /**
    * Multiply the current point by a factor.
    *
-   * @relates EnableIfScalar
+   * @relatesalso EnableIfScalar
    */
   template <typename OtherNumber>
   Point<dim,typename ProductType<Number, typename EnableIfScalar<OtherNumber>::type>::type>
@@ -227,6 +242,12 @@ public:
    */
   typename numbers::NumberTraits<Number>::real_type distance (const Point<dim,Number> &p) const;
 
+  /**
+   * Return the squared Euclidean distance of <tt>this</tt> point to the point
+   * <tt>p</tt>.
+   */
+  typename numbers::NumberTraits<Number>::real_type distance_square (const Point<dim,Number> &p) const;
+
   /**
    * @}
    */
@@ -243,6 +264,8 @@ public:
 
 #ifndef DOXYGEN
 
+// At least clang-3.7 requires us to have a user-defined constructor
+// and we can't use 'Point<dim,Number>::Point () = default' here.
 template <int dim, typename Number>
 inline
 Point<dim,Number>::Point ()
@@ -295,19 +318,12 @@ Point<dim,Number>::Point (const Number x,
                       "that takes two arguments. Point<dim> objects with dim!=2 "
                       "require initialization with the constructor that takes 'dim' "
                       "arguments."));
-  // we can only get here if we pass the assertion. use the switch anyway so
-  // as to avoid compiler warnings about uninitialized elements or writing
+  // we can only get here if we pass the assertion. use the indirection anyway
+  // so as to avoid compiler warnings about uninitialized elements or writing
   // beyond the end of the 'values' array
-  switch (dim)
-    {
-    case 2:
-      this->values[0] = x;
-      this->values[1] = y;
-      break;
-
-    default:
-      ;
-    }
+  constexpr unsigned int y_index = (dim<2)?0:1;
+  this->values[0] = x;
+  this->values[y_index] = y;
 }
 
 
@@ -324,20 +340,14 @@ Point<dim,Number>::Point (const Number x,
                       "require initialization with the constructor that takes 'dim' "
                       "arguments."));
 
-  // we can only get here if we pass the assertion. use the switch anyway so
-  // as to avoid compiler warnings about uninitialized elements or writing
+  // we can only get here if we pass the assertion. use the indirection anyway
+  // so as to avoid compiler warnings about uninitialized elements or writing
   // beyond the end of the 'values' array
-  switch (dim)
-    {
-    case 3:
-      this->values[0] = x;
-      this->values[1] = y;
-      this->values[2] = z;
-      break;
-
-    default:
-      ;
-    }
+  constexpr unsigned int y_index = (dim<2)?0:1;
+  constexpr unsigned int z_index = (dim<3)?0:2;
+  this->values[0] = x;
+  this->values[y_index] = y;
+  this->values[z_index] = z;
 }
 
 
@@ -422,7 +432,7 @@ Point<dim,Number>::operator - () const
 
 
 template <int dim, typename Number>
-template<typename OtherNumber>
+template <typename OtherNumber>
 inline
 Point<dim,typename ProductType<Number, typename EnableIfScalar<OtherNumber>::type>::type>
 Point<dim,Number>::operator * (const OtherNumber factor) const
@@ -436,7 +446,7 @@ Point<dim,Number>::operator * (const OtherNumber factor) const
 
 
 template <int dim, typename Number>
-template<typename OtherNumber>
+template <typename OtherNumber>
 inline
 Point<dim,typename ProductType<Number, typename EnableIfScalar<OtherNumber>::type>::type>
 Point<dim,Number>::operator / (const OtherNumber factor) const
@@ -476,14 +486,24 @@ inline
 typename numbers::NumberTraits<Number>::real_type
 Point<dim,Number>::distance (const Point<dim,Number> &p) const
 {
-  Number sum = Number();
+  return std::sqrt(distance_square(p));
+}
+
+
+
+template <int dim, typename Number>
+inline
+typename numbers::NumberTraits<Number>::real_type
+Point<dim,Number>::distance_square (const Point<dim,Number> &p) const
+{
+  Number sum = internal::NumberType<Number>::value(0.0);
   for (unsigned int i=0; i<dim; ++i)
     {
-      const Number diff=this->values[i]-p(i);
+      const Number diff=static_cast<Number>(this->values[i])-p(i);
       sum += numbers::NumberTraits<Number>::abs_square (diff);
     }
 
-  return std::sqrt(sum);
+  return sum;
 }
 
 
@@ -508,8 +528,8 @@ Point<dim,Number>::serialize(Archive &ar, const unsigned int)
 /**
  * Global operator scaling a point vector by a scalar.
  *
- * @relates Point
- * @relates EnableIfScalar
+ * @relatesalso Point
+ * @relatesalso EnableIfScalar
  */
 template <int dim, typename Number, typename OtherNumber>
 inline
@@ -525,7 +545,7 @@ operator * (const OtherNumber        factor,
 /**
  * Output operator for points. Print the elements consecutively, with a space
  * in between.
- * @relates Point
+ * @relatesalso Point
  */
 template <int dim, typename Number>
 inline
@@ -542,9 +562,8 @@ std::ostream &operator << (std::ostream            &out,
 
 
 /**
- * Output operator for points. Print the elements consecutively, with a space
- * in between.
- * @relates Point
+ * Input operator for points. Inputs the elements consecutively.
+ * @relatesalso Point
  */
 template <int dim, typename Number>
 inline
