@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 2000 - 2017 by the deal.II authors
+// Copyright (C) 2000 - 2018 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -13,8 +13,8 @@
 //
 // ---------------------------------------------------------------------
 
-#ifndef dealii__polynomial_h
-#define dealii__polynomial_h
+#ifndef dealii_polynomial_h
+#define dealii_polynomial_h
 
 
 
@@ -22,8 +22,8 @@
 #include <deal.II/base/exceptions.h>
 #include <deal.II/base/subscriptor.h>
 #include <deal.II/base/point.h>
-#include <deal.II/base/std_cxx11/shared_ptr.h>
 
+#include <memory>
 #include <vector>
 
 DEAL_II_NAMESPACE_OPEN
@@ -233,7 +233,7 @@ namespace Polynomials
                           const number factor);
 
     /**
-     * Transforms polynomial form of product of linear factors into standard
+     * Transform polynomial form of product of linear factors into standard
      * form, $\sum_i a_i x^i$. Deletes all data structures related to the
      * product form.
      */
@@ -531,13 +531,14 @@ namespace Polynomials
      * polynomial, we keep one pointer to the list of coefficients; we do so
      * rather than keeping a vector of vectors in order to simplify
      * programming multithread-safe. In order to avoid memory leak, we use a
-     * shared_ptr in order to correctly free the memory of the vectors when
+     * unique_ptr in order to correctly free the memory of the vectors when
      * the global destructor is called.
      */
-    static std::vector<std_cxx11::shared_ptr<const std::vector<double> > > recursive_coefficients;
+    static std::vector<std::unique_ptr<const std::vector<double> > > recursive_coefficients;
   };
 
 
+
   /**
    * Polynomials for Hermite interpolation condition.
    *
@@ -585,6 +586,164 @@ namespace Polynomials
     static std::vector<Polynomial<double> >
     generate_complete_basis (const unsigned int p);
   };
+
+
+
+  /**
+   * Polynomials for a variant of Hermite polynomials with better condition
+   * number in the interpolation than the basis from HermiteInterpolation.
+   *
+   * In analogy to the actual Hermite polynomials this basis evaluates the
+   * first polynomial $p_0$ to 1 at $x=0$ and has both a zero value and zero
+   * derivative at $x=1$. Likewise, the last polynomial $p_n$ evaluates to 1
+   * at $x=1$ but has zero value and zero derivative at $x=0$. The second
+   * polynomial $p_1$ and the second to last polynomial $p_{n-1}$ represent
+   * the derivative degree of freedom at $x=0$ and $x=1$, respectively. As
+   * such, they are zero at both the end points $x=0, x=1$ and have zero
+   * derivative at the opposite end, $p_1'(1)=0$ and $p_{n-1}'(0)=0$. As
+   * opposed to the original Hermite polynomials, $p_0$ does not have zero
+   * derivative at $x=0$. The additional degree of freedom is used to make
+   * $p_0$ and $p_1$ orthogonal, which for $n=3$ results in a root at
+   * $x=\frac{2}{7}$ for $p_0$ and at $x=\frac{5}{7}$ for $p_n$,
+   * respectively. Furthermore, the extension of these polynomials to higher
+   * degrees $n>3$ is constructed by adding additional nodes inside the unit
+   * interval, again ensuring better conditioning. The nodes are computed as
+   * the roots of the Jacobi polynomials for $\alpha=\beta=2$ which are
+   * orthogonal against the generating function $x^2(1-x)^2$ with the Hermite
+   * property. Then, these polynomials are constructed in the usual way as
+   * Lagrange polynomials with double roots at $x=0$ and $x=1$. For example at
+   * $n=4$, all of $p_0, p_1, p_3, p_4$ get an additional root at $x=0.5$
+   * through the factor $(x-0.5)$. In summary, this basis is dominated by
+   * nodal contributions, but it is not a nodal one because the second and
+   * second to last polynomials that are non-nodal, and due to the presence of
+   * double nodes in $x=0$ and $x=1$.
+   *
+   * The basis only contains Hermite information at <code>degree>=3</code>,
+   * but it is also implemented for degrees between 0 and two. For the linear
+   * case, the usual hat functions are implemented, whereas the polynomials
+   * for <code>degree=2</code> are $p_0(x)=(1-x)^2$, $p_1(x)=4x(x-1)$, and
+   * $p_2(x)=x^2$, in accordance with the construction principle for degree 3
+   * that allows a non-zero of $p_0$ and $p_2$.
+   *
+   * These two relaxations improve the condition number of the mass matrix
+   * (i.e., interpolation) significantly, as can be seen from the following
+   * table:
+   *
+   * <table align="center" border="1">
+   *   <tr>
+   *    <th>&nbsp;</th>
+   *    <th colspan="2">Condition number mass matrix</th>
+   *   </tr>
+   *   <tr>
+   *    <th>degree</th>
+   *    <th>HermiteInterpolation</th>
+   *    <th>HermiteLikeInterpolation</th>
+   *   </tr>
+   *   <tr>
+   *    <th>n=3</th>
+   *    <th>1057</th>
+   *    <th>17.18</th>
+   *   </tr>
+   *   <tr>
+   *    <th>n=4</th>
+   *    <th>6580</th>
+   *    <th>16.83</th>
+   *   </tr>
+   *   <tr>
+   *    <th>n=5</th>
+   *    <th>1.875e+04</th>
+   *    <th>19.37</th>
+   *   </tr>
+   *   <tr>
+   *    <th>n=6</th>
+   *    <th>6.033e+04</th>
+   *    <th>18.99</th>
+   *   </tr>
+   *   <tr>
+   *    <th>n=10</th>
+   *    <th>9.756e+05</th>
+   *    <th>25.65</th>
+   *   </tr>
+   *   <tr>
+   *    <th>n=15</th>
+   *    <th>9.431e+06</th>
+   *    <th>36.47</th>
+   *   </tr>
+   *   <tr>
+   *    <th>n=25</th>
+   *    <th>2.220e+08</th>
+   *    <th>62.28</th>
+   *   </tr>
+   *   <tr>
+   *    <th>n=35</th>
+   *    <th>2.109e+09</th>
+   *    <th>91.50</th>
+   *   </tr>
+   * </table>
+   *
+   * This polynomial inherits the advantageous property of Hermite polynomials
+   * where only two functions have value and/or derivative nonzero on a face
+   * but gives better condition numbers of interpolation, which improves the
+   * performance of some iterative schemes like conjugate gradients with
+   * point-Jacobi.
+   *
+   * @author Martin Kronbichler
+   * @date 2018
+   */
+  class HermiteLikeInterpolation : public Polynomial<double>
+  {
+  public:
+    /**
+     * Constructor for the polynomial with index <tt>index</tt> within the set
+     * up polynomials of degree @p degree.
+     */
+    HermiteLikeInterpolation (const unsigned int degree,
+                              const unsigned int index);
+
+    /**
+     * Return the polynomials with index <tt>0</tt> up to <tt>degree+1</tt> in
+     * a space of degree up to <tt>degree</tt>.
+     */
+    static std::vector<Polynomial<double> >
+    generate_complete_basis (const unsigned int degree);
+  };
+
+
+
+  /*
+   * Evaluate a Jacobi polynomial $ P_n^{\alpha, \beta}(x) $ specified by the
+   * parameters @p alpha, @p beta, @p n, where @p n is the degree of the
+   * Jacobi polynomial.
+   *
+   * @note The Jacobi polynomials are not orthonormal and are defined on the
+   * unit interval $[0, 1]$ as usual for deal.II, rather than $[-1, +1]$ often
+   * used in literature. @p x is the point of evaluation.
+   */
+  template <typename Number>
+  Number
+  jacobi_polynomial_value(const unsigned int degree,
+                          const int alpha,
+                          const int beta,
+                          const Number x);
+
+
+  /**
+   * Compute the roots of the Jacobi polynomials on the unit interval $[0, 1]$
+   * of the given degree. These roots are used in several places inside the
+   * deal.II library, such as the Gauss-Lobatto quadrature formula or for the
+   * Hermite-like interpolation.
+   *
+   * The algorithm uses a Newton algorithm, using the zeros of the Chebyshev
+   * polynomials as an initial guess. This code has been tested for alpha and
+   * beta equal to zero (Legendre case), one (Gauss-Lobatto case) as well as
+   * two, so be careful when using it for other values as the Newton iteration
+   * might or might not converge.
+   */
+  template <typename Number>
+  std::vector<Number>
+  jacobi_polynomial_roots(const unsigned int degree,
+                          const int alpha,
+                          const int beta);
 }
 
 
@@ -666,6 +825,126 @@ namespace Polynomials
     ar &lagrange_weight;
   }
 
+
+
+  template <typename Number>
+  Number
+  jacobi_polynomial_value(const unsigned int degree,
+                          const int alpha,
+                          const int beta,
+                          const Number x)
+  {
+    Assert(alpha >=0 && beta >= 0,
+           ExcNotImplemented("Negative alpha/beta coefficients not supported"));
+    // the Jacobi polynomial is evaluated using a recursion formula.
+    Number p0, p1;
+
+    // The recursion formula is defined for the interval [-1, 1], so rescale
+    // to that interval here
+    const Number xeval = Number(-1) + 2. * x;
+
+    // initial values P_0(x), P_1(x):
+    p0 = 1.0;
+    if (degree==0)
+      return p0;
+    p1 = ((alpha+beta+2)*xeval + (alpha-beta))/2;
+    if (degree==1)
+      return p1;
+
+    for (unsigned int i=1; i<degree; ++i)
+      {
+        const Number v  = 2*i + (alpha + beta);
+        const Number a1 = 2*(i+1)*(i + (alpha + beta + 1))*v;
+        const Number a2 = (v + 1)*(alpha*alpha - beta*beta);
+        const Number a3 = v*(v + 1)*(v + 2);
+        const Number a4 = 2*(i+alpha)*(i+beta)*(v + 2);
+
+        const Number pn = ((a2 + a3*xeval)*p1 - a4*p0)/a1;
+        p0 = p1;
+        p1 = pn;
+      }
+    return p1;
+  }
+
+
+
+  template <typename Number>
+  std::vector<Number>
+  jacobi_polynomial_roots(const unsigned int degree,
+                          const int alpha,
+                          const int beta)
+  {
+    std::vector<Number> x(degree, 0.5);
+
+    // compute zeros with a Newton algorithm.
+
+    // Set tolerance. For long double we might not always get the additional
+    // precision in a run time environment (e.g. with valgrind), so we must
+    // limit the tolerance to double. Since we do a Newton iteration, doing
+    // one more iteration after the residual has indicated convergence will be
+    // enough for all number types due to the quadratic convergence of
+    // Newton's method
+
+    const Number tolerance
+      = 4 * std::max(static_cast<Number>(std::numeric_limits<double>::epsilon()),
+                     std::numeric_limits<Number>::epsilon());
+
+    // The following implementation follows closely the one given in the
+    // appendix of the book by Karniadakis and Sherwin: Spectral/hp element
+    // methods for computational fluid dynamics (Oxford University Press,
+    // 2005)
+
+    // If symmetric, we only need to compute the half of points
+    const unsigned int n_points = (alpha == beta ? degree/2 : degree);
+    for (unsigned int k=0; k<n_points; ++k)
+      {
+        // we take the zeros of the Chebyshev polynomial (alpha=beta=-0.5) as
+        // initial values, corrected by the initial value
+        Number r = 0.5-0.5*std::cos(static_cast<Number> (2*k+1)/(2*degree) *
+                                    numbers::PI );
+        if (k>0)
+          r = (r + x[k-1])/2;
+
+        unsigned int converged = numbers::invalid_unsigned_int;
+        for (unsigned int it=1; it<1000; ++it)
+          {
+            Number s = 0.;
+            for (unsigned int i=0; i<k; ++i)
+              s += 1./(r - x[i]);
+
+            // derivative of P_n^{alpha,beta}, rescaled to [0, 1]
+            const Number J_x = (alpha+beta+degree+1)*
+                               jacobi_polynomial_value(degree-1, alpha+1,
+                                                       beta+1, r);
+
+            // value of P_n^{alpha,beta}
+            const Number f = jacobi_polynomial_value(degree, alpha, beta, r);
+            const Number delta = f/(f*s - J_x);
+            r += delta;
+            if (converged == numbers::invalid_unsigned_int &&
+                std::abs(delta) < tolerance)
+              converged = it;
+
+            // do one more iteration to ensure accuracy also for tighter
+            // types than double (e.g. long double)
+            if (it == converged + 1)
+              break;
+          }
+
+        Assert(converged != numbers::invalid_unsigned_int,
+               ExcMessage("Newton iteration for zero of Jacobi polynomial "
+                          "did not converge."));
+
+        x[k] = r;
+      }
+
+    // in case we assumed symmetry, fill up the missing values
+    for (unsigned int k=n_points; k<degree; ++k)
+      x[k] = 1.0-x[degree-k-1];
+
+    return x;
+  }
+
 }
 DEAL_II_NAMESPACE_CLOSE
 
