@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 2002 - 2016 by the deal.II authors
+// Copyright (C) 2002 - 2017 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -13,8 +13,8 @@
 //
 // ---------------------------------------------------------------------
 
-#ifndef dealii__polynomial_space_h
-#define dealii__polynomial_space_h
+#ifndef dealii_polynomial_space_h
+#define dealii_polynomial_space_h
 
 
 #include <deal.II/base/config.h>
@@ -280,7 +280,7 @@ PolynomialSpace<dim>::PolynomialSpace (const std::vector<Pol> &pols)
 }
 
 
-template<int dim>
+template <int dim>
 inline
 unsigned int
 PolynomialSpace<dim>::n() const
@@ -290,7 +290,7 @@ PolynomialSpace<dim>::n() const
 
 
 
-template<int dim>
+template <int dim>
 inline
 unsigned int
 PolynomialSpace<dim>::degree() const
