@@ -13,8 +13,8 @@
 //
 // ---------------------------------------------------------------------
 
-#ifndef dealii__polynomials_abf_h
-#define dealii__polynomials_abf_h
+#ifndef dealii_polynomials_abf_h
+#define dealii_polynomials_abf_h
 
 
 #include <deal.II/base/config.h>
@@ -27,7 +27,7 @@
 #include <deal.II/base/table.h>
 #include <deal.II/base/thread_management.h>
 
-#include <deal.II/base/std_cxx11/shared_ptr.h>
+#include <memory>
 #include <vector>
 
 DEAL_II_NAMESPACE_OPEN
