@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 2015 - 2016 by the deal.II authors
+// Copyright (C) 2015 - 2018 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -14,8 +14,8 @@
 // ---------------------------------------------------------------------
 
 
-#ifndef dealii__polynomials_rannacher_turek_h
-#define dealii__polynomials_rannacher_turek_h
+#ifndef dealii_polynomials_rannacher_turek_h
+#define dealii_polynomials_rannacher_turek_h
 
 #include <deal.II/base/point.h>
 #include <deal.II/base/tensor.h>
@@ -94,7 +94,7 @@ public:
 
 namespace internal
 {
-  namespace PolynomialsRannacherTurek
+  namespace PolynomialsRannacherTurekImplementation
   {
     template <int order, int dim>
     inline
@@ -199,7 +199,7 @@ Tensor<order,dim>
 PolynomialsRannacherTurek<dim>::compute_derivative (const unsigned int i,
                                                     const Point<dim> &p) const
 {
-  return internal::PolynomialsRannacherTurek::compute_derivative<order> (i, p);
+  return internal::PolynomialsRannacherTurekImplementation::compute_derivative<order> (i, p);
 }
 
 
