@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 2005 - 2015 by the deal.II authors
+// Copyright (C) 2005 - 2017 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -13,8 +13,8 @@
 //
 // ---------------------------------------------------------------------
 
-#ifndef dealii__qprojector_h
-#define dealii__qprojector_h
+#ifndef dealii_qprojector_h
+#define dealii_qprojector_h
 
 
 #include <deal.II/base/quadrature.h>
@@ -412,11 +412,6 @@ template <>
 Quadrature<1>
 QProjector<1>::project_to_all_subfaces (const Quadrature<0> &quadrature);
 
-
-template <>
-bool
-QIterated<1>::uses_both_endpoints (const Quadrature<1> &base_quadrature);
-
 template <>
 QIterated<1>::QIterated (const Quadrature<1> &base_quadrature,
                          const unsigned int   n_copies);
