@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 1998 - 2017 by the deal.II authors
+// Copyright (C) 1998 - 2018 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -13,8 +13,8 @@
 //
 // ---------------------------------------------------------------------
 
-#ifndef dealii__quadrature_lib_h
-#define dealii__quadrature_lib_h
+#ifndef dealii_quadrature_lib_h
+#define dealii_quadrature_lib_h
 
 
 #include <deal.II/base/config.h>
@@ -80,48 +80,6 @@ public:
    * direction).
    */
   QGaussLobatto(const unsigned int n);
-
-protected:
-  /**
-   * Compute Legendre-Gauss-Lobatto quadrature points in the interval $[-1,
-   * +1]$. They are equal to the roots of the corresponding Jacobi polynomial
-   * (specified by @p alpha, @p beta).  @p q is the number of points.
-   *
-   * @return Vector containing nodes.
-   */
-  std::vector<long double>
-  compute_quadrature_points (const unsigned int q,
-                             const int alpha,
-                             const int beta) const;
-
-  /**
-   * Compute Legendre-Gauss-Lobatto quadrature weights. The quadrature points
-   * and weights are related to Jacobi polynomial specified by @p alpha, @p
-   * beta. @p x denotes the quadrature points.
-   *
-   * @return Vector containing weights.
-   */
-  std::vector<long double>
-  compute_quadrature_weights (const std::vector<long double> &x,
-                              const int alpha,
-                              const int beta) const;
-
-  /**
-   * Evaluate a Jacobi polynomial $ P^{\alpha, \beta}_n(x) $ specified by the
-   * parameters @p alpha, @p beta, @p n. Note: The Jacobi polynomials are not
-   * orthonormal and defined on the interval $[-1, +1]$. @p x is the point of
-   * evaluation.
-   */
-  long double JacobiP(const long double x,
-                      const int alpha,
-                      const int beta,
-                      const unsigned int n) const;
-
-  /**
-   * Evaluate the Gamma function $ \Gamma(n) = (n-1)! $.
-   * @param n  point of evaluation (integer).
-   */
-  static long double gamma(const unsigned int n);
 };
 
 
@@ -295,13 +253,11 @@ public:
              const double alpha = 1,
              const bool factor_out_singular_weight=false);
 
-#ifdef DEAL_II_WITH_CXX11
   /**
    * Move constructor. We cannot rely on the move constructor for `Quadrature`,
    * since it does not know about the additional member `fraction` of this class.
    */
-  QGaussLogR(QGaussLogR<dim> &&) = default;
-#endif
+  QGaussLogR(QGaussLogR<dim> &&) noexcept = default;
 
 protected:
   /**
@@ -528,8 +484,6 @@ public:
 
 };
 
-/*@}*/
-
 /**
  * Gauss-Chebyshev quadrature rules integrate the weighted product
  * $\int_{-1}^1 f(x) w(x) dx$ with weight given by: $w(x) = 1/\sqrt{1-x^2}$.
@@ -538,7 +492,7 @@ public:
  * we rescale the quadrature formula so that it is defined on the interval
  * $[0,1]$ instead of $[-1,1]$. So the quadrature formulas integrate exactly
  * the integral $\int_0^1 f(x) w(x) dx$ with the weight: $w(x) =
- * 1/sqrt{x(1-x)}$. For details see: M. Abramowitz & I.A. Stegun: Handbook of
+ * 1/\sqrt{x(1-x)}$. For details see: M. Abramowitz & I.A. Stegun: Handbook of
  * Mathematical Functions, par. 25.4.38
  *
  * @author Giuseppe Pitton, Luca Heltai 2015
@@ -549,16 +503,6 @@ class QGaussChebyshev : public Quadrature<dim>
 public:
   /// Generate a formula with <tt>n</tt> quadrature points
   QGaussChebyshev(const unsigned int n);
-
-private:
-  /// Computes the points of the quadrature formula.
-  static std::vector<double>
-  get_quadrature_points(const unsigned int n);
-
-  /// Computes the weights of the quadrature formula.
-  static std::vector<double>
-  get_quadrature_weights(const unsigned int n);
-
 };
 
 
@@ -571,7 +515,7 @@ private:
  * of quadrature points. Here we rescale the quadrature formula so that it is
  * defined on the interval $[0,1]$ instead of $[-1,1]$. So the quadrature
  * formulas integrate exactly the integral $\int_0^1 f(x) w(x) dx$ with the
- * weight: $w(x) = 1/sqrt{x(1-x)}$. By default the quadrature is constructed
+ * weight: $w(x) = 1/\sqrt{x(1-x)}$. By default the quadrature is constructed
  * with the left endpoint as quadrature node, but the quadrature node can be
  * imposed at the right endpoint through the variable ep that can assume the
  * values left or right.
@@ -600,24 +544,14 @@ public:
   QGaussRadauChebyshev(const unsigned int n,
                        EndPoint ep=QGaussRadauChebyshev::left);
 
-#ifdef DEAL_II_WITH_CXX11
   /**
    * Move constructor. We cannot rely on the move constructor for `Quadrature`,
    * since it does not know about the additional member `ep` of this class.
    */
-  QGaussRadauChebyshev(QGaussRadauChebyshev<dim> &&) = default;
-#endif
+  QGaussRadauChebyshev(QGaussRadauChebyshev<dim> &&) noexcept = default;
 
 private:
   const EndPoint ep;
-  /// Computes the points of the quadrature formula.
-  static std::vector<double>
-  get_quadrature_points(const unsigned int n, EndPoint ep);
-
-  /// Computes the weights of the quadrature formula.
-  static std::vector<double>
-  get_quadrature_weights(const unsigned int n, EndPoint ep);
-
 };
 
 /**
@@ -629,7 +563,7 @@ private:
  * where $n$ is the number of quadrature points. Here we rescale the
  * quadrature formula so that it is defined on the interval $[0,1]$ instead of
  * $[-1,1]$. So the quadrature formulas integrate exactly the integral
- * $\int_0^1 f(x) w(x) dx$ with the weight: $w(x) = 1/sqrt{x(1-x)}$. For
+ * $\int_0^1 f(x) w(x) dx$ with the weight: $w(x) = 1/\sqrt{x(1-x)}$. For
  * details see: M. Abramowitz & I.A. Stegun: Handbook of Mathematical
  * Functions, par. 25.4.40
  *
@@ -641,34 +575,239 @@ class QGaussLobattoChebyshev : public Quadrature<dim>
 public:
   /// Generate a formula with <tt>n</tt> quadrature points
   QGaussLobattoChebyshev(const unsigned int n);
+};
 
-private:
-  /// Computes the points of the quadrature formula.
-  static std::vector<double>
-  get_quadrature_points(const unsigned int n);
+/**
+ * Given an arbitrary quadrature formula, return one that chops the quadrature
+ * points above the hyper-plane defined by $\sum_i x_i = 1$. In other words,
+ * it extracts those quadrature points from the base formula that satisfy
+ * $\sum_i (\mathbf x_q)_i \le 1+10^{-12}$."
+ *
+ * In general the resulting quadrature is not very useful, unless the
+ * quadrature you started from has been constructed specifically to integrate
+ * over triangles or tetrahedra. This class only ensures that the resulting
+ * quadrature formula only has quadrature points in the reference simplex or on
+ * its boundary.
+ *
+ * No transformation is applied to the weights, and the weights referring to points
+ * that live outside the reference simplex are simply discarded.
+ *
+ * The main use of this quadrature formula is not to chop tensor product
+ * quadratures. Ideally you should pass to this class a quadrature formula
+ * constructed directly using points and weights in the reference simplex,
+ * capable of integrating on triangles or tetrahedra.
+ *
+ * For finite elements based on quadrilaterals and hexahedra, a QSimplex
+ * quadrature formula is not very useful on its own. This class is typically
+ * used in conjunction with other classes, like QSplit, to patch the reference
+ * element using several QSimplex quadrature formulas.
+ *
+ * Such quadrature formulas are useful to integrate functions with
+ * singularities at certain points, or functions that present jumps along a
+ * co-dimension one surface inside the reference element, like in the extended
+ * finite element method (XFEM).
+ *
+ * @author Luca Heltai, 2017.
+ */
+template <int dim>
+class QSimplex : public Quadrature<dim>
+{
+public:
+  /**
+   * Construct a quadrature that only contains the points that are in the lower
+   * left reference simplex.
+   *
+   * @param[in] quad The input quadrature.
+   */
+  QSimplex(const Quadrature<dim> &quad);
 
-  /// Computes the weights of the quadrature formula.
-  static std::vector<double>
-  get_quadrature_weights(const unsigned int n);
+  /**
+   * Return an affine transformation of this quadrature, that can be used to integrate
+   * on the simplex identified by `vertices`.
+   *
+   * Both the quadrature point locations and the weights are transformed, so that you
+   * can effectively use the resulting quadrature to integrate on the simplex.
+   *
+   * The transformation is defined as
+   * \f[
+   * x = v_0 + B \hat x
+   * \f]
+   * where the matrix $B$ is given by $B_{ij} = v[j][i]-v[0][i]$.
+   *
+   * The weights are scaled with the absolute value of the determinant of $B$,
+   * that is $J := |\text{det}(B)|$. If $J$ is zero, an empty quadrature is
+   * returned. This may happen, in two dimensions, if the three vertices are
+   * aligned, or in three dimensions if the four vertices are on the same
+   * plane.
+   *
+   * @param[in] vertices The vertices of the simplex you wish to integrate on
+   * @return A quadrature object that can be used to integrate on the simplex
+   */
+  Quadrature<dim>
+  compute_affine_transformation(const std::array<Point<dim>, dim+1> &vertices) const;
 
 };
 
+/**
+ * A quadrature that implements a polar transformation from a square to a
+ * triangle to integrate singularities in the origin of the reference simplex.
+ * The quadrature is obtained through the following polar transformation:
+ *
+ * \f[
+ *  \begin{pmatrix}
+ *  x \\
+ *  y
+ *  \end{pmatrix}
+ *  =
+ * \begin{pmatrix}
+ *  \frac{\hat x}{\sin(\theta)+\cos(\theta)} cos(\theta) \\
+ *  \frac{\hat x}{\sin(\theta)+\cos(\theta)} sin(\theta)
+ *  \end{pmatrix}
+ *  \qquad \theta := \frac\pi 2 \hat y
+ * \f]
+ *
+ * @author Luca Heltai, 2017
+ */
+class  QTrianglePolar: public QSimplex<2>
+{
+public:
+  /**
+   * Construct a QTrianglePolar quadrature, with different formulas in the
+   * radial and angular directions.
+   *
+   * @param radial_quadrature Radial quadrature
+   * @param angular_quadrature Angular quadrature
+   */
+  QTrianglePolar(const Quadrature<1> &radial_quadrature,
+                 const Quadrature<1> &angular_quadrature);
+
+  /**
+   * Call the other constructor, with QGauss<1>(n) for both radial and
+   * angular quadrature.
+   *
+   * @param n Order of QGauss quadrature
+   */
+  QTrianglePolar(const unsigned int &n);
+};
+
+/**
+ * A quadrature that implements the Duffy transformation from a square to a
+ * triangle to integrate singularities in the origin of the reference
+ * simplex.
+ *
+ * The Duffy transformation is defined as
+ * \f[
+ * \begin{pmatrix}
+ * x\\
+ * y
+ * \end{pmatrix}
+ * =
+ * \begin{pmatrix}
+ * \hat x^\beta (1-\hat y)\\
+ * \hat x^\beta \hat y
+ * \end{pmatrix}
+ * \f]
+ *
+ * with determinant of the Jacobian equal to $J= \beta \hat \x^{2\beta-1}$.
+ * Such transformation maps the reference square $[0,1]\times[0,1]$ to the
+ * reference simplex, by collapsing the left side of the square and squeezing
+ * quadrature points towards the origin, and then shearing the resulting
+ * triangle to the reference one. This transformation shows good convergence
+ * properties when $\beta = 1$ with singularities of order $1/R$ in the origin,
+ * but different $\beta$ values can be selected to increase convergence and/or
+ * accuracy when higher order Gauss rules are used (see "Generalized Duffy
+ * transformation for integrating vertex singularities", S. E. Mousavi, N.
+ * Sukumar, Computational Mechanics 2009).
+ *
+ * When $\beta = 1$, this transformation is also known as the Lachat-Watson
+ * transformation.
+ *
+ * @author Luca Heltai, Nicola Giuliani, 2017.
+ */
+class  QDuffy: public QSimplex<2>
+{
+public:
+  /**
+   * Constructor that allows the specification of different quadrature rules
+   * along the "radial" and "angular" directions.
+   *
+   * Since this quadrature is not based on a Polar change of coordinates, it
+   * is not fully proper to talk about radial and angular directions. However,
+   * the effect of the Duffy transformation is similar to a polar change
+   * of coordinates, since the resulting quadrature points are aligned radially
+   * with respect to the singularity.
+   *
+   * @param radial_quadrature Base quadrature to use in the radial direction
+   * @param angular_quadrature Base quadrature to use in the angular direction
+   */
+  QDuffy(const Quadrature<1> &radial_quadrature,
+         const Quadrature<1> &angular_quadrature,
+         const double beta = 1.0);
+
+  /**
+   * Call the above constructor with QGauss<1>(n) quadrature formulas for
+   * both the radial and angular quadratures.
+   *
+   * @param n Order of QGauss quadrature
+   */
+  QDuffy(const unsigned int n,
+         const double beta);
+};
+
+/**
+ * A quadrature to use when the cell should be split into subregions to
+ * integrate using one or more base quadratures.
+ *
+ * @author Luca Heltai, 2017.
+ */
+template<int dim>
+class QSplit : public Quadrature<dim>
+{
+public:
+  /**
+   * Construct a quadrature formula by splitting the reference hyper cube into
+   * the minimum number of simplices that have vertex zero coinciding with
+   * @p split_point, and patch together affine transformations of the @p base
+   * quadrature. The point @p split_point should be in the reference element,
+   * and an exception is thrown if this is not the case.
+   *
+   * In two dimensions, the resulting quadrature formula will be composed of
+   * two, three, or four triangular quadrature formulas if @p split_point
+   * coincides with one of the vertices, if it lies on one of the edges, or if
+   * it is internal to the reference element respectively.
+   *
+   * The same is true for the three dimensional case, with six, eight, ten, or
+   * twelve tetrahedral quadrature formulas if @p split_point coincides with one
+   * of the vertices, if it lies on one of the edges, on one of the faces, or
+   * if it is internal to the reference element respectively.
+   *
+   * The resulting quadrature can be used, for example, to integrate functions
+   * with integrable singularities at the split point, provided that you select
+   * as base quadrature one that can integrate singular points on vertex zero
+   * of the reference simplex.
+   *
+   * An example usage in dimension two is given by:
+   * @code
+   * const unsigned int order = 5;
+   * QSplit<2> quad(QTrianglePolar(order), Point<2>(.3,.4));
+   * @endcode
+   *
+   * The resulting quadrature will look like the following:
+   * @image html split_quadrature.png ""
+   *
+   * @param base Base QSimplex quadrature to use
+   * @param split_point Where to split the hyper cube
+   */
+  QSplit(const QSimplex<dim> &base,
+         const Point<dim> &split_point);
+};
+
+/*@}*/
+
 /* -------------- declaration of explicit specializations ------------- */
 
 template <> QGauss<1>::QGauss (const unsigned int n);
 template <> QGaussLobatto<1>::QGaussLobatto (const unsigned int n);
-template <>
-std::vector<long double> QGaussLobatto<1>::
-compute_quadrature_points(const unsigned int, const int, const int) const;
-template <>
-std::vector<long double> QGaussLobatto<1>::
-compute_quadrature_weights(const std::vector<long double> &, const int, const int) const;
-template <>
-long double QGaussLobatto<1>::
-JacobiP(const long double, const int, const int, const unsigned int) const;
-template <>
-long double
-QGaussLobatto<1>::gamma(const unsigned int n);
 
 template <> std::vector<double> QGaussLog<1>::get_quadrature_points(const unsigned int);
 template <> std::vector<double> QGaussLog<1>::get_quadrature_weights(const unsigned int);
