@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 2016 - 2017 by the deal.II authors
+// Copyright (C) 2016 - 2018 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -13,14 +13,11 @@
 //
 // ---------------------------------------------------------------------
 
-#ifndef dealii__quadrature_point_data_h
-#define dealii__quadrature_point_data_h
+#ifndef dealii_quadrature_point_data_h
+#define dealii_quadrature_point_data_h
 
-// must include config before checking for DEAL_II_WITH_CXX11
 #include <deal.II/base/config.h>
 
-#ifdef DEAL_II_WITH_CXX11
-
 #include <deal.II/base/quadrature.h>
 #include <deal.II/base/subscriptor.h>
 #include <deal.II/grid/tria.h>
@@ -52,8 +49,6 @@ DEAL_II_NAMESPACE_OPEN
  * For this reason, this class may not be sufficiently flexible when, for example,
  * adopting a level-set approach to describe material behavior.
  *
- * @note This class is only available if deal.II is configured with C++11 support.
- *
  * @author Denis Davydov, Jean-Paul Pelteret, 2016
  */
 template <typename CellIteratorType, typename DataType>
@@ -63,12 +58,12 @@ public:
   /**
    * Default constructor.
    */
-  CellDataStorage();
+  CellDataStorage() = default;
 
   /**
    * Default destructor.
    */
-  ~CellDataStorage();
+  ~CellDataStorage() = default;
 
   /**
    * Initialize data on the @p cell to store @p number_of_data_points_per_cell of objects of type @p T .
@@ -91,7 +86,7 @@ public:
    * @pre The type @p T needs to either equal @p DataType, or be a class derived
    * from @p DataType. @p T needs to be default constructible.
    */
-  template<typename T=DataType>
+  template <typename T=DataType>
   void initialize(const CellIteratorType &cell,
                   const unsigned int number_of_data_points_per_cell);
 
@@ -100,7 +95,7 @@ public:
    * until, but not including, @p cell_end for all locally owned cells, i.e.
    * for which `cell->is_locally_owned()==true` .
    */
-  template<typename T=DataType>
+  template <typename T=DataType>
   void initialize(const CellIteratorType &cell_start,
                   const CellIteratorType &cell_end,
                   const unsigned int number_of_data_points_per_cell);
@@ -131,7 +126,7 @@ public:
    *
    * @pre The type @p T needs to match the class provided to initialize() .
    */
-  template<typename T=DataType>
+  template <typename T=DataType>
   std::vector<std::shared_ptr<T> > get_data(const CellIteratorType &cell);
 
   /**
@@ -144,7 +139,7 @@ public:
    *
    * @pre The type @p T needs to match the class provided to initialize() .
    */
-  template<typename T=DataType>
+  template <typename T=DataType>
   std::vector<std::shared_ptr<const T> > get_data(const CellIteratorType &cell) const;
 
 private:
@@ -179,12 +174,12 @@ public:
   /**
    * Default constructor.
    */
-  TransferableQuadraturePointData();
+  TransferableQuadraturePointData() = default;
 
   /**
    * Default virtual destructor.
    */
-  virtual ~TransferableQuadraturePointData();
+  virtual ~TransferableQuadraturePointData() = default;
 
   /**
    * Return the total number of values which will be
@@ -327,7 +322,7 @@ namespace parallel
     {
     public:
       static_assert(std::is_base_of<TransferableQuadraturePointData, DataType>::value,
-                    "User's DataType class should be derived from QPData");
+                    "User's DataType class should be derived from TransferableQuadraturePointData");
 
       /**
        * A typedef for a cell.
@@ -449,7 +444,7 @@ namespace parallel
       unsigned int offset;
 
       /**
-       * A pointer to the CellDataStorage class whose data will be transfered.
+       * A pointer to the CellDataStorage class whose data will be transferred.
        */
       CellDataStorage<CellIteratorType,DataType> *data_storage;
 
@@ -471,37 +466,12 @@ namespace parallel
 
 // -------------------  inline and template functions ----------------
 
-//--------------------------------------------------------------------
-//                         TransferableQuadraturePointData
-//--------------------------------------------------------------------
-inline TransferableQuadraturePointData::TransferableQuadraturePointData()
-{}
-
-inline TransferableQuadraturePointData::~TransferableQuadraturePointData()
-{}
-
 //--------------------------------------------------------------------
 //                         CellDataStorage
 //--------------------------------------------------------------------
 
 template <typename CellIteratorType, typename DataType>
-CellDataStorage<CellIteratorType,DataType>::
-CellDataStorage()
-  :
-  Subscriptor()
-{}
-
-
-
-template <typename CellIteratorType, typename DataType>
-CellDataStorage<CellIteratorType,DataType>::
-~CellDataStorage()
-{}
-
-
-
-template <typename CellIteratorType, typename DataType>
-template<typename T>
+template <typename T>
 void CellDataStorage<CellIteratorType,DataType>::initialize(const CellIteratorType &cell,
                                                             const unsigned int n_q_points)
 {
@@ -511,7 +481,7 @@ void CellDataStorage<CellIteratorType,DataType>::initialize(const CellIteratorTy
   if (map.find(cell) == map.end())
     {
       map[cell] = std::vector<std::shared_ptr<DataType> >(n_q_points);
-      // we need to initialise one-by-one as the std::vector<>(q, T())
+      // we need to initialize one-by-one as the std::vector<>(q, T())
       // will end with a single same T object stored in each element of the vector:
       auto it = map.find(cell);
       for (unsigned int q=0; q < n_q_points; q++)
@@ -522,7 +492,7 @@ void CellDataStorage<CellIteratorType,DataType>::initialize(const CellIteratorTy
 
 
 template <typename CellIteratorType, typename DataType>
-template<typename T>
+template <typename T>
 void CellDataStorage<CellIteratorType,DataType>::initialize(const CellIteratorType &cell_start,
                                                             const CellIteratorType &cell_end,
                                                             const unsigned int number)
@@ -559,7 +529,7 @@ void CellDataStorage<CellIteratorType,DataType>::clear()
   auto it = map.begin();
   while (it != map.end())
     {
-      // loop over all objects and see if noone is using them
+      // loop over all objects and see if no one is using them
       for (unsigned int i = 0; i < it->second.size(); i++)
         {
           Assert(it->second[i].unique(),
@@ -608,7 +578,7 @@ CellDataStorage<CellIteratorType,DataType>::get_data(const CellIteratorType &cel
   Assert(it != map.end(), ExcMessage("Could not find QP data for the cell"));
 
   // Cast base class to the desired class. This has to be done irrespectively of
-  // T==DataType as we need to return shapred_ptr<const T> to make sure the user
+  // T==DataType as we need to return shared_ptr<const T> to make sure the user
   // does not modify the content of QP objects
   std::vector<std::shared_ptr<const T>> res (it->second.size());
   for (unsigned int q = 0; q < res.size(); q++)
@@ -628,7 +598,7 @@ CellDataStorage<CellIteratorType,DataType>::get_data(const CellIteratorType &cel
  * whereas the second index represents different values stored at each
  * quadrature point in the DataType class.
  */
-template<typename CellIteratorType, typename DataType>
+template <typename CellIteratorType, typename DataType>
 void pack_cell_data
 (const CellIteratorType &cell,
  const CellDataStorage<CellIteratorType,DataType> *data_storage,
@@ -661,7 +631,7 @@ void pack_cell_data
 /*
  * the opposite of the pack function above.
  */
-template<typename CellIteratorType, typename DataType>
+template <typename CellIteratorType, typename DataType>
 void unpack_to_cell_data
 (const CellIteratorType &cell,
  const FullMatrix<double> &values_at_qp,
@@ -693,7 +663,7 @@ namespace parallel
 {
   namespace distributed
   {
-    template<int dim, typename DataType>
+    template <int dim, typename DataType>
     ContinuousQuadratureDataTransfer<dim,DataType>::ContinuousQuadratureDataTransfer
     (const FiniteElement<dim> &projection_fe_,
      const Quadrature<dim> &lhs_quadrature,
@@ -725,13 +695,13 @@ namespace parallel
 
 
 
-    template<int dim, typename DataType>
+    template <int dim, typename DataType>
     ContinuousQuadratureDataTransfer<dim,DataType>::~ContinuousQuadratureDataTransfer()
     {}
 
 
 
-    template<int dim, typename DataType>
+    template <int dim, typename DataType>
     void ContinuousQuadratureDataTransfer<dim,DataType>::prepare_for_coarsening_and_refinement
     (parallel::distributed::Triangulation<dim> &tr_,
      CellDataStorage<CellIteratorType,DataType> &data_storage_)
@@ -763,24 +733,24 @@ namespace parallel
       data_size_in_bytes = sizeof(double) * dofs_per_cell * number_of_values;
 
       offset = triangulation->register_data_attach(data_size_in_bytes,
-                                                   std_cxx11::bind(&ContinuousQuadratureDataTransfer<dim,DataType>::pack_function,
-                                                       this,
-                                                       std_cxx11::_1,
-                                                       std_cxx11::_2,
-                                                       std_cxx11::_3));
+                                                   std::bind(&ContinuousQuadratureDataTransfer<dim,DataType>::pack_function,
+                                                             this,
+                                                             std::placeholders::_1,
+                                                             std::placeholders::_2,
+                                                             std::placeholders::_3));
     }
 
 
 
-    template<int dim, typename DataType>
+    template <int dim, typename DataType>
     void ContinuousQuadratureDataTransfer<dim,DataType>::interpolate ()
     {
       triangulation->notify_ready_to_unpack(offset,
-                                            std_cxx11::bind(&ContinuousQuadratureDataTransfer<dim,DataType>::unpack_function,
-                                                            this,
-                                                            std_cxx11::_1,
-                                                            std_cxx11::_2,
-                                                            std_cxx11::_3));
+                                            std::bind(&ContinuousQuadratureDataTransfer<dim,DataType>::unpack_function,
+                                                      this,
+                                                      std::placeholders::_1,
+                                                      std::placeholders::_2,
+                                                      std::placeholders::_3));
 
       // invalidate the pointers
       data_storage = nullptr;
@@ -789,7 +759,7 @@ namespace parallel
 
 
 
-    template<int dim, typename DataType>
+    template <int dim, typename DataType>
     void ContinuousQuadratureDataTransfer<dim,DataType>::pack_function
     (const typename parallel::distributed::Triangulation<dim,dim>::cell_iterator &cell,
      const typename parallel::distributed::Triangulation<dim,dim>::CellStatus /*status*/,
@@ -807,7 +777,7 @@ namespace parallel
 
 
 
-    template<int dim, typename DataType>
+    template <int dim, typename DataType>
     void ContinuousQuadratureDataTransfer<dim,DataType>::
     unpack_function (const typename parallel::distributed::Triangulation<dim,dim>::cell_iterator &cell,
                      const typename parallel::distributed::Triangulation<dim,dim>::CellStatus status,
@@ -858,6 +828,4 @@ namespace parallel
 #endif // DOXYGEN
 DEAL_II_NAMESPACE_CLOSE
 
-#endif // CXX11
-
 #endif
