@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 2003 - 2015 by the deal.II authors
+// Copyright (C) 2003 - 2017 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -14,8 +14,8 @@
 // ---------------------------------------------------------------------
 
 
-#ifndef dealii__quadrature_selector_h
-#define dealii__quadrature_selector_h
+#ifndef dealii_quadrature_selector_h
+#define dealii_quadrature_selector_h
 
 
 #include <deal.II/base/quadrature.h>
@@ -36,7 +36,7 @@ DEAL_II_NAMESPACE_OPEN
  * @ingroup Quadrature
  * @author Ralf Schulz, 2003
  */
-template<int dim>
+template <int dim>
 class QuadratureSelector : public Quadrature<dim>
 {
 public:
