@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 1998 - 2015 by the deal.II authors
+// Copyright (C) 1998 - 2017 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -13,8 +13,8 @@
 //
 // ---------------------------------------------------------------------
 
-#ifndef dealii__smartpointer_h
-#define dealii__smartpointer_h
+#ifndef dealii_smartpointer_h
+#define dealii_smartpointer_h
 
 
 #include <deal.II/base/config.h>
@@ -59,7 +59,7 @@ DEAL_II_NAMESPACE_OPEN
  * @ingroup memory
  * @author Guido Kanschat, Wolfgang Bangerth, 1998 - 2009
  */
-template<typename T, typename P = void>
+template <typename T, typename P = void>
 class SmartPointer
 {
 public:
@@ -200,7 +200,7 @@ template <typename T, typename P>
 inline
 SmartPointer<T,P>::SmartPointer ()
   :
-  t (0), id(typeid(P).name())
+  t (nullptr), id(typeid(P).name())
 {}
 
 
@@ -211,7 +211,7 @@ SmartPointer<T,P>::SmartPointer (T *t)
   :
   t (t), id(typeid(P).name())
 {
-  if (t != 0)
+  if (t != nullptr)
     t->subscribe(id);
 }
 
@@ -223,7 +223,7 @@ SmartPointer<T,P>::SmartPointer (T *t, const char *id)
   :
   t (t), id(id)
 {
-  if (t != 0)
+  if (t != nullptr)
     t->subscribe(id);
 }
 
@@ -236,7 +236,7 @@ SmartPointer<T,P>::SmartPointer (const SmartPointer<T,Q> &tt)
   :
   t (tt.t), id(tt.id)
 {
-  if (t != 0)
+  if (t != nullptr)
     t->subscribe(id);
 }
 
@@ -248,7 +248,7 @@ SmartPointer<T,P>::SmartPointer (const SmartPointer<T,P> &tt)
   :
   t (tt.t), id(tt.id)
 {
-  if (t != 0)
+  if (t != nullptr)
     t->subscribe(id);
 }
 
@@ -258,7 +258,7 @@ template <typename T, typename P>
 inline
 SmartPointer<T,P>::~SmartPointer ()
 {
-  if (t != 0)
+  if (t != nullptr)
     t->unsubscribe(id);
 }
 
@@ -269,11 +269,11 @@ inline
 void
 SmartPointer<T,P>::clear ()
 {
-  if (t != 0)
+  if (t != nullptr)
     {
       t->unsubscribe(id);
       delete t;
-      t = 0;
+      t = nullptr;
     }
 }
 
@@ -288,10 +288,10 @@ SmartPointer<T,P> &SmartPointer<T,P>::operator = (T *tt)
   if (t == tt)
     return *this;
 
-  if (t != 0)
+  if (t != nullptr)
     t->unsubscribe(id);
   t = tt;
-  if (tt != 0)
+  if (tt != nullptr)
     tt->subscribe(id);
   return *this;
 }
@@ -310,10 +310,10 @@ SmartPointer<T,P>::operator = (const SmartPointer<T,Q> &tt)
   if (&tt == this)
     return *this;
 
-  if (t != 0)
+  if (t != nullptr)
     t->unsubscribe(id);
   t = static_cast<T *>(tt);
-  if (tt != 0)
+  if (tt != nullptr)
     tt->subscribe(id);
   return *this;
 }
@@ -331,10 +331,10 @@ SmartPointer<T,P>::operator = (const SmartPointer<T,P> &tt)
   if (&tt == this)
     return *this;
 
-  if (t != 0)
+  if (t != nullptr)
     t->unsubscribe(id);
   t = static_cast<T *>(tt);
-  if (tt != 0)
+  if (tt != nullptr)
     tt->subscribe(id);
   return *this;
 }
@@ -354,7 +354,7 @@ template <typename T, typename P>
 inline
 T &SmartPointer<T,P>::operator * () const
 {
-  Assert(t != 0, ExcNotInitialized());
+  Assert(t != nullptr, ExcNotInitialized());
   return *t;
 }
 
@@ -364,7 +364,7 @@ template <typename T, typename P>
 inline
 T *SmartPointer<T,P>::operator -> () const
 {
-  Assert(t != 0, ExcNotInitialized());
+  Assert(t != nullptr, ExcNotInitialized());
   return t;
 }
 
@@ -390,12 +390,12 @@ template <typename T, typename P>
 inline
 void SmartPointer<T,P>::swap (T *&tt)
 {
-  if (t != 0)
+  if (t != nullptr)
     t->unsubscribe (id);
 
   std::swap (t, tt);
 
-  if (t != 0)
+  if (t != nullptr)
     t->subscribe (id);
 }
 
