@@ -13,13 +13,12 @@
 //
 // ---------------------------------------------------------------------
 
-#ifndef dealii__std_cxx11_function_h
-#define dealii__std_cxx11_function_h
+#ifndef dealii_std_cxx11_function_h
+#define dealii_std_cxx11_function_h
 
 
 #include <deal.II/base/config.h>
 
-#ifdef DEAL_II_WITH_CXX11
 
 #  include <functional>
 DEAL_II_NAMESPACE_OPEN
@@ -29,17 +28,6 @@ namespace std_cxx11
 }
 DEAL_II_NAMESPACE_CLOSE
 
-#else
-
-#include <boost/function.hpp>
-DEAL_II_NAMESPACE_OPEN
-namespace std_cxx11
-{
-  using boost::function;
-}
-DEAL_II_NAMESPACE_CLOSE
-
-#endif
 
 // then allow using the old namespace name instead of the new one
 DEAL_II_NAMESPACE_OPEN
