@@ -13,13 +13,11 @@
 //
 // ---------------------------------------------------------------------
 
-#ifndef dealii__std_cxx11_iterator_h
-#define dealii__std_cxx11_iterator_h
+#ifndef dealii_std_cxx11_iterator_h
+#define dealii_std_cxx11_iterator_h
 
 #include <deal.II/base/config.h>
 
-#ifdef DEAL_II_WITH_CXX11
-
 #  include <iterator>
 DEAL_II_NAMESPACE_OPEN
 namespace std_cxx11
@@ -29,16 +27,4 @@ namespace std_cxx11
 }
 DEAL_II_NAMESPACE_CLOSE
 
-#else
-
-#include <boost/range.hpp>
-DEAL_II_NAMESPACE_OPEN
-namespace std_cxx11
-{
-  using boost::begin;
-  using boost::end;
-}
-DEAL_II_NAMESPACE_CLOSE
-#endif
-
 #endif
