@@ -13,14 +13,12 @@
 //
 // ---------------------------------------------------------------------
 
-#ifndef dealii__std_cxx11_mutex_h
-#define dealii__std_cxx11_mutex_h
+#ifndef dealii_std_cxx11_mutex_h
+#define dealii_std_cxx11_mutex_h
 
 
 #include <deal.II/base/config.h>
 
-#ifdef DEAL_II_WITH_CXX11
-
 #  include <mutex>
 DEAL_II_NAMESPACE_OPEN
 namespace std_cxx11
@@ -29,18 +27,6 @@ namespace std_cxx11
 }
 DEAL_II_NAMESPACE_CLOSE
 
-#else
-
-#  include <boost/thread/mutex.hpp>
-DEAL_II_NAMESPACE_OPEN
-namespace std_cxx11
-{
-  using boost::mutex;
-}
-DEAL_II_NAMESPACE_CLOSE
-
-#endif
-
 // then allow using the old namespace name instead of the new one
 DEAL_II_NAMESPACE_OPEN
 namespace std_cxx1x = std_cxx11;
