@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 1998 - 2016 by the deal.II authors
+// Copyright (C) 1998 - 2018 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -13,16 +13,17 @@
 //
 // ---------------------------------------------------------------------
 
-#ifndef dealii__subscriptor_h
-#define dealii__subscriptor_h
+#ifndef dealii_subscriptor_h
+#define dealii_subscriptor_h
 
 
 #include <deal.II/base/config.h>
 #include <deal.II/base/exceptions.h>
 
-#include <typeinfo>
+#include <atomic>
 #include <map>
 #include <string>
+#include <typeinfo>
 
 DEAL_II_NAMESPACE_OPEN
 
@@ -41,14 +42,17 @@ DEAL_II_NAMESPACE_OPEN
  * The utility of this class is even enhanced by providing identifying strings
  * to the functions subscribe() and unsubscribe(). In case of a hanging
  * subscription during destruction, this string will be listed in the
- * exception's message. For reasons of efficiency, these strings are handled
- * as <tt>const char*</tt>. Therefore, the pointers provided to subscribe()
- * and to unsubscribe() must be the same. Strings with equal contents will not
- * be recognized to be the same. The handling in SmartPointer will take care
- * of this.
+ * exception's message. These strings are represented as <code>const char
+ * *</code> pointers since the underlying buffer comes from (and is managed
+ * by) the run-time type information system: more exactly, these pointers are
+ * the result the function call <code>typeid(x).name()</code> where
+ * <code>x</code> is some object. Therefore, the pointers provided to
+ * subscribe() and to unsubscribe() must be the same. Strings with equal
+ * contents will not be recognized to be the same. The handling in
+ * SmartPointer will take care of this.
  *
- * @note Due to a problem with <tt>volatile</tt> declarations, this additional
- * feature is switched off if multithreading is used.
+ * @note This feature is switched off if multithreading is used (i.e., if
+ * <code>DEAL_II_WITH_THREADS</code> is on).
  *
  * @ingroup memory
  * @author Guido Kanschat, 1998 - 2005
@@ -69,15 +73,13 @@ public:
    */
   Subscriptor(const Subscriptor &);
 
-#ifdef DEAL_II_WITH_CXX11
   /**
    * Move constructor.
    *
    * An object inheriting from Subscriptor can only be moved if no other
    * objects are subscribing to it.
    */
-  Subscriptor(Subscriptor &&);
-#endif
+  Subscriptor(Subscriptor &&) noexcept;
 
   /**
    * Destructor, asserting that the counter is zero.
@@ -92,20 +94,18 @@ public:
    */
   Subscriptor &operator = (const Subscriptor &);
 
-#ifdef DEAL_II_WITH_CXX11
   /**
    * Move assignment operator.
    *
    * Asserts that the counter for the moved object is zero.
    */
-  Subscriptor &operator = (Subscriptor &&);
-#endif
+  Subscriptor &operator = (Subscriptor &&) noexcept;
 
   /**
    * Subscribes a user of the object. The subscriber may be identified by text
    * supplied as <tt>identifier</tt>.
    */
-  void subscribe (const char *identifier = 0) const;
+  void subscribe (const char *identifier = nullptr) const;
 
   /**
    * Unsubscribes a user from the object.
@@ -113,7 +113,7 @@ public:
    * @note The <tt>identifier</tt> must be the <b>same pointer</b> as the one
    * supplied to subscribe(), not just the same text.
    */
-  void unsubscribe (const char *identifier = 0) const;
+  void unsubscribe (const char *identifier = nullptr) const;
 
   /**
    * Return the present number of subscriptions to this object. This allows to
@@ -136,7 +136,7 @@ public:
    * Exception: Object may not be deleted, since it is used.
    */
   DeclException3(ExcInUse,
-                 int, char *, std::string &,
+                 int, std::string, std::string,
                  << "Object of class " << arg2
                  << " is still used by " << arg1 << " other objects."
                  << "\n\n"
@@ -150,7 +150,7 @@ public:
    * A subscriber with the identification string given to
    * Subscriptor::unsubscribe() did not subscribe to the object.
    */
-  DeclException2(ExcNoSubscriber, char *, char *,
+  DeclException2(ExcNoSubscriber, std::string, std::string,
                  << "No subscriber with identifier <" << arg2
                  << "> subscribes to this object of class " << arg1
                  << ". Consequently, it cannot be unsubscribed.");
@@ -195,12 +195,12 @@ private:
    * We use the <tt>mutable</tt> keyword in order to allow subscription to
    * constant objects also.
    *
-   * In multithreaded mode, this counter may be modified by different threads.
-   * We thus have to mark it <tt>volatile</tt>. However, this is counter-
-   * productive in non-MT mode since it may pessimize code. So use the macro
-   * defined in <tt>deal.II/base/config.h</tt> to selectively add volatility.
+   * This counter may be read from and written to concurrently in
+   * multithreaded code: hence we use the <code>std::atomic</code> class
+   * template.
+   *
    */
-  mutable DEAL_VOLATILE unsigned int counter;
+  mutable std::atomic<unsigned int> counter;
 
   /**
    * In this map, we count subscriptions for each different identification
@@ -217,11 +217,19 @@ private:
   mutable const std::type_info *object_info;
 
   /**
-   * Check that there are no objects subscribing to this object and throw an
-   * error if there are, in order to guarantee that it is safe to either move
-   * or destroy this object.
+   * Check that there are no objects subscribing to this object. If this check
+   * passes then it is safe to destroy the current object. It this check fails
+   * then this function will either abort or print an error message to deallog
+   * (by using the AssertNothrow mechanism), but will not throw an exception.
+   *
+   * @note Since this function is just a consistency check it does nothing in
+   * release mode.
+   *
+   * @note If this function is called when there is an uncaught exception
+   * then, rather than aborting, this function prints an error message to the
+   * standard error stream and returns.
    */
-  void check_no_subscribers () const;
+  void check_no_subscribers () const noexcept;
 };
 
 //---------------------------------------------------------------------------
