@@ -13,8 +13,8 @@
 //
 // ---------------------------------------------------------------------
 
-#ifndef dealii__table_handler_h
-#define dealii__table_handler_h
+#ifndef dealii_table_handler_h
+#define dealii_table_handler_h
 
 
 #include <deal.II/base/config.h>
@@ -54,7 +54,7 @@ namespace internal
     /**
      * Default constructor.
      */
-    TableEntry ();
+    TableEntry () = default;
 
     /**
      * Constructor. Initialize this table element with the value
@@ -183,7 +183,7 @@ namespace internal
  *   {
  *     table.add_value("numbers", i);
  *     table.add_value("squares", i*i);
- *     table.add_value("square roots", sqrt(i));
+ *     table.add_value("square roots", std::sqrt(1.*i));
  *   }
  *                                  // merge the second and third column
  * table.add_column_to_supercolumn("squares", "squares and roots");
@@ -346,6 +346,30 @@ public:
    */
   TableHandler ();
 
+
+  /**
+   * Declare the existence of a column in the table by giving it a name.
+   * As discussed in the documentation of the class, this is not usually
+   * necessary -- just adding a value for a given column key via the
+   * add_value() function also declares the column. This function is
+   * therefore only necessary in cases where you want a column to
+   * also show up even if you never add an entry to any row in this column;
+   * or, more likely, if you want to prescribe the order in which columns
+   * are later printed by declaring columns in a particular order before
+   * entries are ever put into them.
+   *
+   * (The latter objective can also be achieved by adding entries to
+   * the table in whatever order they are produced by a program,
+   * and later calling set_column_order(). However, this approach
+   * requires knowing -- in one central place of your software --
+   * all of the columns keys that other parts of the software have
+   * written into, and how they should be sorted. This is easily
+   * possible for small programs, but may not be feasible for
+   * large code bases in which parts of the code base are only
+   * executed based on run-time parameters.)
+   */
+  void declare_column (const std::string &key);
+
   /**
    * Adds a column (if not yet existent) with the key <tt>key</tt> and adds
    * the value of type <tt>T</tt> to the column. Values of type <tt>T</tt>
@@ -356,6 +380,21 @@ public:
   void add_value (const std::string &key,
                   const T            value);
 
+  /**
+   * If a row is only partially filled, then set all elements of that
+   * row for which no elements exist in a particular column to the
+   * empty string. This is akin to the 'auto_fill_mode' described in
+   * the introduction, but more general because it allows you to start
+   * writing into a column for a new row without having to know that
+   * that column had been written to in the previous row.
+   *
+   * If all columns have been written into in the current row, then
+   * this function doesn't do anything at all. In other words,
+   * conceptually the function "completes" the current row, though its
+   * use case is to "start" a new row.
+   */
+  void start_new_row ();
+
   /**
    * Switch auto-fill mode on or off. See the general documentation of this
    * class for a description of what auto-fill mode does.
@@ -429,7 +468,7 @@ public:
   void set_tex_table_label (const std::string &table_label);
 
   /**
-   * Set the caption the the supercolumn <tt>superkey</tt> for tex output.
+   * Set the caption the supercolumn <tt>superkey</tt> for tex output.
    * You may want to chose this different from <tt>superkey</tt>, if it
    * contains formulas or similar constructs.
    */
@@ -561,7 +600,7 @@ protected:
      */
     template <class Archive>
     void save(Archive &ar, const unsigned int version) const;
-    template<class Archive>
+    template <class Archive>
     void load(Archive &ar, const unsigned int version);
     BOOST_SERIALIZATION_SPLIT_MEMBER()
 
@@ -640,14 +679,14 @@ protected:
   unsigned int n_rows() const;
 
   /**
-   * Stores the column and supercolumn keys in the order desired by the user.
+   * A variable storing the column and supercolumn keys in the order desired by the user.
    * By default this is the order of adding the columns. This order may be
-   * changed by <tt>set_column_order(...)</tt>.
+   * changed by set_column_order().
    */
   std::vector<std::string> column_order;
 
   /**
-   * Maps the column keys to the columns (not supercolumns).
+   * A map from the column keys to the columns (not supercolumns).
    *
    * The field is declared mutable so that the write_text() and write_tex()
    * functions can be const, even though they may pad columns below if
@@ -656,7 +695,7 @@ protected:
   mutable std::map<std::string,Column> columns;
 
   /**
-   * Maps each supercolumn key to the the keys of its subcolumns in the right
+   * A map from each supercolumn key to the keys of its subcolumns in the right
    * order.  It is allowed that a supercolumn has got the same key as a
    * column.
    *
@@ -666,7 +705,7 @@ protected:
   std::map<std::string, std::vector<std::string> > supercolumns;
 
   /**
-   * Maps the supercolumn keys to the captions of the supercolumns that are
+   * A map from the supercolumn keys to the captions of the supercolumns that are
    * used in tex output.
    *
    * By default these are just the supercolumn keys but they may be changed by
@@ -818,27 +857,26 @@ namespace internal
   }
 }
 
+
+
 template <typename T>
 void TableHandler::add_value (const std::string &key,
                               const T            value)
 {
   // see if the column already exists
   if (columns.find(key) == columns.end())
-    {
-      std::pair<std::string, Column> new_column(key, Column(key));
-      columns.insert(new_column);
-      column_order.push_back(key);
-    }
+    declare_column (key);
 
   if (auto_fill_mode == true)
     {
       // follow the algorithm given in the introduction to this class
       // of padding columns as necessary
-      unsigned int n = 0;
+      unsigned int max_col_length = 0;
       for (std::map< std::string, Column >::iterator p = columns.begin(); p != columns.end(); ++p)
-        n = (n >= p->second.entries.size() ? n : p->second.entries.size());
+        max_col_length = std::max(max_col_length,
+                                  static_cast<unsigned int>(p->second.entries.size()));
 
-      while (columns[key].entries.size()+1 < n)
+      while (columns[key].entries.size()+1 < max_col_length)
         {
           columns[key].entries.push_back (internal::TableEntry(T()));
           internal::TableEntry &entry = columns[key].entries.back();
@@ -851,10 +889,12 @@ void TableHandler::add_value (const std::string &key,
   columns[key].entries.push_back (internal::TableEntry(value));
   internal::TableEntry &entry = columns[key].entries.back();
   entry.cache_string(columns[key].scientific, columns[key].precision);
-  columns[key].max_length = std::max(columns[key].max_length, static_cast<unsigned int>(entry.get_cached_string().length()));
+  columns[key].max_length = std::max(columns[key].max_length,
+                                     static_cast<unsigned int>(entry.get_cached_string().length()));
 }
 
 
+
 template <class Archive>
 void
 TableHandler::Column::save(Archive &ar, const unsigned int /*version*/) const
@@ -866,7 +906,9 @@ TableHandler::Column::save(Archive &ar, const unsigned int /*version*/) const
   & max_length;
 }
 
-template<class Archive>
+
+
+template <class Archive>
 void
 TableHandler::Column::load(Archive &ar, const unsigned int /*version*/)
 {
