@@ -13,8 +13,8 @@
 //
 // ---------------------------------------------------------------------
 
-#ifndef dealii__tensor_accessors_h
-#define dealii__tensor_accessors_h
+#ifndef dealii_tensor_accessors_h
+#define dealii_tensor_accessors_h
 
 #include <deal.II/base/config.h>
 #include <deal.II/base/template_constraints.h>
@@ -52,7 +52,7 @@ DEAL_II_NAMESPACE_OPEN
  * <code>operator[](unsigned int)</code> that returns a (const or non-const)
  * reference of <code>value_type</code>:
  * @code
- *   template<...>
+ *   template <...>
  *   class T
  *   {
  *     typedef ... value_type;
@@ -185,10 +185,8 @@ namespace TensorAccessors
   internal::ReorderedIndexView<index, rank, T>
   reordered_index_view(T &t)
   {
-#ifdef DEAL_II_WITH_CXX11
     static_assert(0 <= index && index < rank,
                   "The specified index must lie within the range [0,rank)");
-#endif
 
     return internal::ReorderedIndexView<index, rank, T>(t);
   }
@@ -216,7 +214,7 @@ namespace TensorAccessors
    *
    * @author Matthias Maier, 2015
    */
-  template<int rank, typename T, typename ArrayType> typename
+  template <int rank, typename T, typename ArrayType> typename
   ReturnType<rank, T>::value_type &
   extract(T &t, const ArrayType &indices)
   {
@@ -266,14 +264,12 @@ namespace TensorAccessors
   inline DEAL_II_ALWAYS_INLINE
   void contract(T1 &result, const T2 &left, const T3 &right)
   {
-#ifdef DEAL_II_WITH_CXX11
     static_assert(rank_1 >= no_contr, "The rank of the left tensor must be "
                   "equal or greater than the number of "
                   "contractions");
     static_assert(rank_2 >= no_contr, "The rank of the right tensor must be "
                   "equal or greater than the number of "
                   "contractions");
-#endif
 
     internal::Contract<no_contr, rank_1, rank_2, dim>::template contract<T1, T2, T3>
     (result, left, right);
@@ -335,7 +331,7 @@ namespace TensorAccessors
      * StoreIndex and ReorderedIndexView that return rvalues, we have to
      * return by value.
      */
-    template<typename T>
+    template <typename T>
     struct ReferenceType
     {
       typedef T &type;
@@ -542,10 +538,10 @@ namespace TensorAccessors
     // Straightforward recursion implemented by specializing ExtractHelper
     // for position == rank. We use the type trait ReturnType<rank, T> to
     // have an idea what the final type will be.
-    template<int position, int rank>
+    template <int position, int rank>
     struct ExtractHelper
     {
-      template<typename T, typename ArrayType>
+      template <typename T, typename ArrayType>
       inline
       static
       typename ReturnType<rank - position, T>::value_type &
@@ -560,10 +556,10 @@ namespace TensorAccessors
 
     // For position == rank there is nothing to extract, just return the
     // object.
-    template<int rank>
+    template <int rank>
     struct ExtractHelper<rank, rank>
     {
-      template<typename T, typename ArrayType>
+      template <typename T, typename ArrayType>
       inline
       static
       T &extract(T &t,
@@ -593,7 +589,7 @@ namespace TensorAccessors
     class Contract
     {
     public:
-      template<typename T1, typename T2, typename T3>
+      template <typename T1, typename T2, typename T3>
       inline DEAL_II_ALWAYS_INLINE static
       void contract(T1 &result, const T2 &left, const T3 &right)
       {
@@ -621,7 +617,7 @@ namespace TensorAccessors
     class Contract<no_contr, no_contr, rank_2, dim>
     {
     public:
-      template<typename T1, typename T2, typename T3>
+      template <typename T1, typename T2, typename T3>
       inline DEAL_II_ALWAYS_INLINE static
       void contract(T1 &result, const T2 &left, const T3 &right)
       {
@@ -649,7 +645,7 @@ namespace TensorAccessors
     class Contract<no_contr, no_contr, no_contr, dim>
     {
     public:
-      template<typename T1, typename T2, typename T3>
+      template <typename T1, typename T2, typename T3>
       inline DEAL_II_ALWAYS_INLINE static
       void contract(T1 &result, const T2 &left, const T3 &right)
       {
@@ -665,11 +661,13 @@ namespace TensorAccessors
     class Contract2
     {
     public:
-      template<typename T1, typename T2, typename T3>
+      template <typename T1, typename T2, typename T3>
       inline DEAL_II_ALWAYS_INLINE static
       T1 contract2(const T2 &left, const T3 &right)
       {
-        T1 result = T1();
+        // Some auto-differentiable numbers need explicit
+        // zero initialization.
+        T1 result = dealii::internal::NumberType<T1>::value(0.0);
         for (unsigned int i = 0; i < dim; ++i)
           result += Contract2<no_contr - 1, dim>::template contract2<T1>(left[i], right[i]);
         return result;
@@ -683,7 +681,7 @@ namespace TensorAccessors
     class Contract2<0, dim>
     {
     public:
-      template<typename T1, typename T2, typename T3>
+      template <typename T1, typename T2, typename T3>
       inline DEAL_II_ALWAYS_INLINE static
       T1 contract2(const T2 &left, const T3 &right)
       {
@@ -710,11 +708,13 @@ namespace TensorAccessors
     class Contract3
     {
     public:
-      template<typename T1, typename T2, typename T3, typename T4>
+      template <typename T1, typename T2, typename T3, typename T4>
       static inline
       T1 contract3(const T2 &left, const T3 &middle, const T4 &right)
       {
-        T1 result = T1();
+        // Some auto-differentiable numbers need explicit
+        // zero initialization.
+        T1 result = dealii::internal::NumberType<T1>::value(0.0);
         for (unsigned int i = 0; i < dim; ++i)
           result += Contract3<rank_1 - 1, rank_2, dim>::template contract3<T1>(left[i], middle[i], right);
         return result;
@@ -736,11 +736,13 @@ namespace TensorAccessors
     class Contract3<0, rank_2, dim>
     {
     public:
-      template<typename T1, typename T2, typename T3, typename T4>
+      template <typename T1, typename T2, typename T3, typename T4>
       static inline
       T1 contract3(const T2 &left, const T3 &middle, const T4 &right)
       {
-        T1 result = T1();
+        // Some auto-differentiable numbers need explicit
+        // zero initialization.
+        T1 result = dealii::internal::NumberType<T1>::value(0.0);
         for (unsigned int i = 0; i < dim; ++i)
           result += Contract3<0, rank_2 - 1, dim>::template contract3<T1>(left, middle[i], right[i]);
         return result;
@@ -754,7 +756,7 @@ namespace TensorAccessors
     class Contract3<0, 0, dim>
     {
     public:
-      template<typename T1, typename T2, typename T3, typename T4>
+      template <typename T1, typename T2, typename T3, typename T4>
       static inline
       T1 contract3(const T2 &left, const T3 &middle, const T4 &right)
       {
@@ -769,4 +771,4 @@ namespace TensorAccessors
 
 DEAL_II_NAMESPACE_CLOSE
 
-#endif /* dealii__tensor_accessors_h */
+#endif /* dealii_tensor_accessors_h */
