@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 1999 - 2015 by the deal.II authors
+// Copyright (C) 1999 - 2017 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -13,8 +13,8 @@
 //
 // ---------------------------------------------------------------------
 
-#ifndef dealii__tensor_function_h
-#define dealii__tensor_function_h
+#ifndef dealii_tensor_function_h
+#define dealii_tensor_function_h
 
 
 #include <deal.II/base/config.h>
@@ -75,7 +75,7 @@ public:
    * usually not used by their true type, but rather through pointers to this
    * base class.
    */
-  virtual ~TensorFunction ();
+  virtual ~TensorFunction () = default;
 
   /**
    * Return the value of the function at the given point.
@@ -127,7 +127,7 @@ public:
   ConstantTensorFunction (const dealii::Tensor<rank, dim, Number> &value,
                           const Number initial_time = 0.0);
 
-  virtual ~ConstantTensorFunction ();
+  virtual ~ConstantTensorFunction () = default;
 
   virtual typename dealii::TensorFunction<rank, dim, Number>::value_type value (const Point<dim> &p) const;
 
