@@ -13,14 +13,14 @@
 //
 // ---------------------------------------------------------------------
 
-#ifndef dealii__time_stepping_h
-#define dealii__time_stepping_h
+#ifndef dealii_time_stepping_h
+#define dealii_time_stepping_h
 
 
 #include <deal.II/base/config.h>
 #include <deal.II/base/signaling_nan.h>
-#include <deal.II/base/std_cxx11/function.h>
 
+#include <functional>
 #include <vector>
 
 DEAL_II_NAMESPACE_OPEN
@@ -83,7 +83,7 @@ namespace TimeStepping
     /**
      * Virtual destructor.
      */
-    virtual ~TimeStepping() {}
+    virtual ~TimeStepping() = default;
 
     /**
      * Purely virtual function. This function is used to advance from time @p
@@ -96,11 +96,11 @@ namespace TimeStepping
      * function returns the time at the end of the time step.
      */
     virtual double evolve_one_time_step
-    (std::vector<std_cxx11::function<VectorType (const double, const VectorType &)> >               &F,
-     std::vector<std_cxx11::function<VectorType (const double, const double, const VectorType &)> > &J_inverse,
-     double                                                                                         t,
-     double                                                                                         delta_t,
-     VectorType                                                                                     &y) = 0;
+    (std::vector<std::function<VectorType (const double, const VectorType &)> >               &F,
+     std::vector<std::function<VectorType (const double, const double, const VectorType &)> > &J_inverse,
+     double                                                                                   t,
+     double                                                                                   delta_t,
+     VectorType                                                                               &y) = 0;
 
     /**
      * Empty structure used to store information.
@@ -128,7 +128,7 @@ namespace TimeStepping
     /**
      * Virtual destructor.
      */
-    virtual ~RungeKutta() {}
+    virtual ~RungeKutta() = default;
 
     /**
      * Purely virtual method used to initialize the Runge-Kutta method.
@@ -147,11 +147,11 @@ namespace TimeStepping
      * methods, @p F and @ J_inverse can only contain one element.
      */
     double evolve_one_time_step
-    (std::vector<std_cxx11::function<VectorType (const double, const VectorType &)> >               &F,
-     std::vector<std_cxx11::function<VectorType (const double, const double, const VectorType &)> > &J_inverse,
-     double                                                                                         t,
-     double                                                                                         delta_t,
-     VectorType &y);
+    (std::vector<std::function<VectorType (const double, const VectorType &)> >               &F,
+     std::vector<std::function<VectorType (const double, const double, const VectorType &)> > &J_inverse,
+     double                                                                                   t,
+     double                                                                                   delta_t,
+     VectorType                                                                               &y);
 
     /**
      * Purely virtual function. This function is used to advance from time @p
@@ -165,11 +165,11 @@ namespace TimeStepping
      * evolve_one_time_step returns the time at the end of the time step.
      */
     virtual double evolve_one_time_step
-    (std_cxx11::function<VectorType (const double, const VectorType &)>               f,
-     std_cxx11::function<VectorType (const double, const double, const VectorType &)> id_minus_tau_J_inverse,
-     double                                                                           t,
-     double                                                                           delta_t,
-     VectorType                                                                       &y) = 0;
+    (const std::function<VectorType (const double, const VectorType &)>               &f,
+     const std::function<VectorType (const double, const double, const VectorType &)> &id_minus_tau_J_inverse,
+     double                                                                     t,
+     double                                                                     delta_t,
+     VectorType                                                                 &y) = 0;
 
   protected:
     /**
@@ -210,7 +210,7 @@ namespace TimeStepping
      * you will want to call <code>initialize(runge_kutta_method)</code>
      * before it can be used.
      */
-    ExplicitRungeKutta() {}
+    ExplicitRungeKutta() = default;
 
     /**
      * Constructor. This function calls initialize(runge_kutta_method).
@@ -234,11 +234,11 @@ namespace TimeStepping
      * end of the time step.
      */
     double evolve_one_time_step
-    (std_cxx11::function<VectorType (const double, const VectorType &)>               f,
-     std_cxx11::function<VectorType (const double, const double, const VectorType &)> id_minus_tau_J_inverse,
-     double                                                                           t,
-     double                                                                           delta_t,
-     VectorType &y);
+    (const std::function<VectorType (const double, const VectorType &)>                &f,
+     const std::function<VectorType (const double, const double, const VectorType &)>  &id_minus_tau_J_inverse,
+     double                                                                      t,
+     double                                                                      delta_t,
+     VectorType                                                                  &y);
 
     /**
      * This function is used to advance from time @p t to t+ @p delta_t. This
@@ -248,10 +248,10 @@ namespace TimeStepping
      * step.
      */
     double evolve_one_time_step
-    (std_cxx11::function<VectorType (const double, const VectorType &)> f,
-     double                                                             t,
-     double                                                             delta_t,
-     VectorType                                                         &y);
+    (const std::function<VectorType (const double, const VectorType &)> &f,
+     double                                                       t,
+     double                                                       delta_t,
+     VectorType                                                   &y);
 
     /**
      * This structure stores the name of the method used.
@@ -276,11 +276,11 @@ namespace TimeStepping
      * Compute the different stages needed.
      */
     void compute_stages
-    (std_cxx11::function<VectorType (const double, const VectorType &)> f,
-     const double                                                       t,
-     const double                                                       delta_t,
-     const VectorType                                                   &y,
-     std::vector<VectorType>                                            &f_stages) const;
+    (const std::function<VectorType (const double, const VectorType &)> &f,
+     const double                                                 t,
+     const double                                                 delta_t,
+     const VectorType                                             &y,
+     std::vector<VectorType>                                      &f_stages) const;
 
     /**
      * Status structure of the object.
@@ -305,7 +305,7 @@ namespace TimeStepping
      * set_newton_solver_parameters(unsigned int,double) need to be called
      * before the object can be used.
      */
-    ImplicitRungeKutta() {}
+    ImplicitRungeKutta() = default;
 
     /**
      * Constructor. This function calls initialize(runge_kutta_method) and
@@ -333,8 +333,8 @@ namespace TimeStepping
      * returns the time at the end of the time step.
      */
     double evolve_one_time_step
-    (std_cxx11::function<VectorType (const double, const VectorType &)>               f,
-     std_cxx11::function<VectorType (const double, const double, const VectorType &)> id_minus_tau_J_inverse,
+    (const std::function<VectorType (const double, const VectorType &)>               &f,
+     const std::function<VectorType (const double, const double, const VectorType &)> &id_minus_tau_J_inverse,
      double                                                                           t,
      double                                                                           delta_t,
      VectorType                                                                       &y);
@@ -374,8 +374,8 @@ namespace TimeStepping
      * Compute the different stages needed.
      */
     void compute_stages
-    (std_cxx11::function<VectorType (const double, const VectorType &)>               f,
-     std_cxx11::function<VectorType (const double, const double, const VectorType &)> id_minus_tau_J_inverse,
+    (const std::function<VectorType (const double, const VectorType &)>               &f,
+     const std::function<VectorType (const double, const double, const VectorType &)> &id_minus_tau_J_inverse,
      double                                                                           t,
      double                                                                           delta_t,
      VectorType                                                                       &y,
@@ -384,17 +384,17 @@ namespace TimeStepping
     /**
      * Newton solver used for the implicit stages.
      */
-    void newton_solve(std_cxx11::function<void (const VectorType &,VectorType &)> get_residual,
-                      std_cxx11::function<VectorType (const VectorType &)>        id_minus_tau_J_inverse,
+    void newton_solve(const std::function<void (const VectorType &,VectorType &)> &get_residual,
+                      const std::function<VectorType (const VectorType &)>        &id_minus_tau_J_inverse,
                       VectorType                                                  &y);
 
     /**
      * Compute the residual needed by the Newton solver.
      */
-    void compute_residual(std_cxx11::function<VectorType (const double, const VectorType &)> f,
+    void compute_residual(const std::function<VectorType (const double, const VectorType &)> &f,
                           double                                                             t,
                           double                                                             delta_t,
-                          const VectorType                                                   &old_y,
+                          const VectorType                                                   &new_y,
                           const VectorType                                                   &y,
                           VectorType                                                         &tendency,
                           VectorType                                                         &residual) const;
@@ -439,7 +439,7 @@ namespace TimeStepping
      * set_time_adaptation_parameters(double, double, double, double, double,
      * double) need to be called before the object can be used.
      */
-    EmbeddedExplicitRungeKutta() {}
+    EmbeddedExplicitRungeKutta() = default;
 
     /**
      * Constructor. This function calls initialize(runge_kutta_method) and
@@ -483,10 +483,10 @@ namespace TimeStepping
      * at the end of the time step.
      */
     double evolve_one_time_step
-    (std_cxx11::function<VectorType (const double, const VectorType &)>               f,
-     std_cxx11::function<VectorType (const double, const double, const VectorType &)> id_minus_tau_J_inverse,
-     double                                                                           t,
-     double                                                                           delta_t,
+    (const std::function<VectorType (const double, const VectorType &)>               &f,
+     const std::function<VectorType (const double, const double, const VectorType &)> &id_minus_tau_J_inverse,
+     double                                                                     t,
+     double                                                                     delta_t,
      VectorType &y);
 
     /**
@@ -497,9 +497,9 @@ namespace TimeStepping
      * step.
      */
     double evolve_one_time_step
-    (std_cxx11::function<VectorType (const double, const VectorType &)> f,
-     double                                                             t,
-     double                                                             delta_t,
+    (const std::function<VectorType (const double, const VectorType &)> &f,
+     double                                                       t,
+     double                                                       delta_t,
      VectorType &y);
 
     /**
@@ -536,11 +536,11 @@ namespace TimeStepping
     /**
      * Compute the different stages needed.
      */
-    void compute_stages(std_cxx11::function<VectorType (const double, const VectorType &)> f,
-                        const double                                                       t,
-                        const double                                                       delta_t,
-                        const VectorType                                                   &y,
-                        std::vector<VectorType>                                            &f_stages);
+    void compute_stages(const std::function<VectorType (const double, const VectorType &)> &f,
+                        const double                                                 t,
+                        const double                                                 delta_t,
+                        const VectorType                                             &y,
+                        std::vector<VectorType>                                      &f_stages);
 
     /**
      * This parameter is the factor (>1) by which the time step is multiplied
