@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 2014 - 2017 by the deal.II authors
+// Copyright (C) 2014 - 2018 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -13,13 +13,14 @@
 //
 // ---------------------------------------------------------------------
 
-#ifndef dealii__time_stepping_templates_h
-#define dealii__time_stepping_templates_h
+#ifndef dealii_time_stepping_templates_h
+#define dealii_time_stepping_templates_h
 
-#include <deal.II/base/std_cxx11/bind.h>
 #include <deal.II/base/exceptions.h>
 #include <deal.II/base/time_stepping.h>
 
+#include <functional>
+
 DEAL_II_NAMESPACE_OPEN
 
 namespace TimeStepping
@@ -30,8 +31,8 @@ namespace TimeStepping
 
   template <typename VectorType>
   double RungeKutta<VectorType>::evolve_one_time_step(
-    std::vector<std_cxx11::function<VectorType (const double, const VectorType &)> > &F,
-    std::vector<std_cxx11::function<VectorType (const double, const double, const VectorType &)> > &J_inverse,
+    std::vector<std::function<VectorType (const double, const VectorType &)> > &F,
+    std::vector<std::function<VectorType (const double, const double, const VectorType &)> > &J_inverse,
 
     double t,
     double delta_t,
@@ -54,7 +55,10 @@ namespace TimeStepping
   template <typename VectorType>
   ExplicitRungeKutta<VectorType>::ExplicitRungeKutta(const runge_kutta_method method)
   {
-    initialize(method);
+    // virtual functions called in constructors and destructors never use the
+    // override in a derived class
+    // for clarity be explicit on which function is called
+    ExplicitRungeKutta<VectorType>::initialize(method);
   }
 
 
@@ -137,8 +141,8 @@ namespace TimeStepping
 
   template <typename VectorType>
   double ExplicitRungeKutta<VectorType>::evolve_one_time_step
-  (std_cxx11::function<VectorType (const double, const VectorType &)> f,
-   std_cxx11::function<VectorType (const double, const double, const VectorType &)> /*id_minus_tau_J_inverse*/,
+  (const std::function<VectorType (const double, const VectorType &)> &f,
+   const std::function<VectorType (const double, const double, const VectorType &)> &/*id_minus_tau_J_inverse*/,
    double                                                             t,
    double                                                             delta_t,
    VectorType                                                         &y)
@@ -150,7 +154,7 @@ namespace TimeStepping
 
   template <typename VectorType>
   double ExplicitRungeKutta<VectorType>::evolve_one_time_step
-  (std_cxx11::function<VectorType (const double, const VectorType &)> f,
+  (const std::function<VectorType (const double, const VectorType &)> &f,
    double                                                             t,
    double                                                             delta_t,
    VectorType                                                         &y)
@@ -178,7 +182,7 @@ namespace TimeStepping
 
   template <typename VectorType>
   void ExplicitRungeKutta<VectorType>::compute_stages
-  (std_cxx11::function<VectorType (const double, const VectorType &)> f,
+  (const std::function<VectorType (const double, const VectorType &)> &f,
    const double                                                       t,
    const double                                                       delta_t,
    const VectorType                                                   &y,
@@ -210,7 +214,10 @@ namespace TimeStepping
     max_it(max_it),
     tolerance(tolerance)
   {
-    initialize(method);
+    // virtual functions called in constructors and destructors never use the
+    // override in a derived class
+    // for clarity be explicit on which function is called
+    ImplicitRungeKutta<VectorType>::initialize(method);
   }
 
 
@@ -280,8 +287,8 @@ namespace TimeStepping
 
   template <typename VectorType>
   double ImplicitRungeKutta<VectorType>::evolve_one_time_step
-  (std_cxx11::function<VectorType (const double, const VectorType &)> f,
-   std_cxx11::function<VectorType (const double, const double, const VectorType &)> id_minus_tau_J_inverse,
+  (const std::function<VectorType (const double, const VectorType &)> &f,
+   const std::function<VectorType (const double, const double, const VectorType &)> &id_minus_tau_J_inverse,
    double                                                             t,
    double                                                             delta_t,
    VectorType                                                         &y)
@@ -323,8 +330,8 @@ namespace TimeStepping
 
   template <typename VectorType>
   void ImplicitRungeKutta<VectorType>::compute_stages(
-    std_cxx11::function<VectorType (const double, const VectorType &)> f,
-    std_cxx11::function<VectorType (const double, const double, const VectorType &)> id_minus_tau_J_inverse,
+    const std::function<VectorType (const double, const VectorType &)> &f,
+    const std::function<VectorType (const double, const double, const VectorType &)> &id_minus_tau_J_inverse,
     double t,
     double delta_t,
     VectorType &y,
@@ -340,9 +347,9 @@ namespace TimeStepping
         // Solve the nonlinear system using Newton's method
         const double new_t = t+this->c[i]*delta_t;
         const double new_delta_t = this->a[i][i]*delta_t;
-        newton_solve(std_cxx11::bind(&ImplicitRungeKutta<VectorType>::compute_residual,this,f,new_t,new_delta_t,
-                                     std_cxx11::cref(old_y),std_cxx11::_1,std_cxx11::ref(f_stages[i]),std_cxx11::_2),
-                     std_cxx11::bind(id_minus_tau_J_inverse,new_t,new_delta_t,std_cxx11::_1),y);
+        newton_solve(std::bind(&ImplicitRungeKutta<VectorType>::compute_residual,this,f,new_t,new_delta_t,
+                               std::cref(old_y),std::placeholders::_1,std::ref(f_stages[i]),std::placeholders::_2),
+                     std::bind(id_minus_tau_J_inverse,new_t,new_delta_t,std::placeholders::_1),y);
       }
   }
 
@@ -350,8 +357,8 @@ namespace TimeStepping
 
   template <typename VectorType>
   void ImplicitRungeKutta<VectorType>::newton_solve(
-    std_cxx11::function<void (const VectorType &,VectorType &)> get_residual,
-    std_cxx11::function<VectorType (const VectorType &)> id_minus_tau_J_inverse,
+    const std::function<void (const VectorType &,VectorType &)> &get_residual,
+    const std::function<VectorType (const VectorType &)> &id_minus_tau_J_inverse,
     VectorType &y)
   {
     VectorType residual(y);
@@ -376,7 +383,7 @@ namespace TimeStepping
 
   template <typename VectorType>
   void ImplicitRungeKutta<VectorType>::compute_residual
-  (std_cxx11::function<VectorType (const double, const VectorType &)> f,
+  (const std::function<VectorType (const double, const VectorType &)> &f,
    double                                                             t,
    double                                                             delta_t,
    const VectorType                                                   &old_y,
@@ -414,9 +421,13 @@ namespace TimeStepping
     refine_tol(refine_tol),
     coarsen_tol(coarsen_tol),
     last_same_as_first(false),
-    last_stage(NULL)
+    last_stage(nullptr),
+    status {}
   {
-    initialize(method);
+    // virtual functions called in constructors and destructors never use the
+    // override in a derived class
+    // for clarity be explicit on which function is called
+    EmbeddedExplicitRungeKutta<VectorType>::initialize(method);
   }
 
 
@@ -664,18 +675,18 @@ namespace TimeStepping
   template <typename VectorType>
   void EmbeddedExplicitRungeKutta<VectorType>::free_memory()
   {
-    if (last_stage!=NULL)
+    if (last_stage!=nullptr)
       delete last_stage;
 
-    last_stage = NULL;
+    last_stage = nullptr;
   }
 
 
 
   template <typename VectorType>
   double EmbeddedExplicitRungeKutta<VectorType>::evolve_one_time_step(
-    std_cxx11::function<VectorType (const double, const VectorType &)> f,
-    std_cxx11::function<VectorType (const double, const double, const VectorType &)> /*id_minus_tau_J_inverse*/,
+    const std::function<VectorType (const double, const VectorType &)> &f,
+    const std::function<VectorType (const double, const double, const VectorType &)> &/*id_minus_tau_J_inverse*/,
     double t,
     double delta_t,
     VectorType &y)
@@ -687,7 +698,7 @@ namespace TimeStepping
 
   template <typename VectorType>
   double EmbeddedExplicitRungeKutta<VectorType>::evolve_one_time_step(
-    std_cxx11::function<VectorType (const double, const VectorType &)> f,
+    const std::function<VectorType (const double, const VectorType &)> &f,
     double t, double delta_t, VectorType &y)
   {
     bool done = false;
@@ -761,7 +772,7 @@ namespace TimeStepping
     // Save the last stage if necessary
     if (last_same_as_first==true)
       {
-        if (last_stage==NULL)
+        if (last_stage==nullptr)
           last_stage = new VectorType(f_stages.back());
         else
           *last_stage = f_stages.back();
@@ -803,7 +814,7 @@ namespace TimeStepping
 
   template <typename VectorType>
   void EmbeddedExplicitRungeKutta<VectorType>::compute_stages(
-    std_cxx11::function<VectorType (const double, const VectorType &)> f,
+    const std::function<VectorType (const double, const VectorType &)> &f,
     const double t,
     const double delta_t,
     const VectorType &y,
@@ -816,7 +827,7 @@ namespace TimeStepping
     // of the first stage.
     if (last_same_as_first==true)
       {
-        if (last_stage!=NULL)
+        if (last_stage!=nullptr)
           {
             f_stages[0] = *last_stage;
             i = 1;
