@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 2005 - 2016 by the deal.II authors
+// Copyright (C) 2005 - 2018 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -13,8 +13,8 @@
 //
 // ---------------------------------------------------------------------
 
-#ifndef dealii__utilities_h
-#define dealii__utilities_h
+#ifndef dealii_utilities_h
+#define dealii_utilities_h
 
 #include <deal.II/base/config.h>
 #include <deal.II/base/exceptions.h>
@@ -23,6 +23,8 @@
 #include <utility>
 #include <functional>
 #include <string>
+#include <tuple>
+#include <type_traits>
 
 #ifdef DEAL_II_WITH_TRILINOS
 #  include <Epetra_Comm.h>
@@ -34,6 +36,18 @@
 #  endif
 #endif
 
+#include <boost/archive/binary_oarchive.hpp>
+#include <boost/archive/binary_iarchive.hpp>
+#include <boost/serialization/vector.hpp>
+#include <boost/serialization/array.hpp>
+
+#ifdef DEAL_II_WITH_ZLIB
+#  include <boost/iostreams/stream.hpp>
+#  include <boost/iostreams/filtering_stream.hpp>
+#  include <boost/iostreams/device/back_inserter.hpp>
+#  include <boost/iostreams/filter/gzip.hpp>
+#endif
+
 DEAL_II_NAMESPACE_OPEN
 
 
@@ -47,6 +61,13 @@ DEAL_II_NAMESPACE_OPEN
  */
 namespace Utilities
 {
+  /**
+   * Return a string of the form "deal.II version x.y.z" where "x.y.z"
+   * identifies the version of deal.II you are using. This information
+   * is also provided by the DEAL_II_PACKAGE_NAME and
+   * DEAL_II_PACKAGE_VERSION preprocessor variables.
+   */
+  std::string dealii_version_string ();
 
   /**
    * Convert a number @p value to a string, with as many digits as given to
@@ -132,6 +153,7 @@ namespace Utilities
   std::vector<double>
   string_to_double (const std::vector<std::string> &s);
 
+
   /**
    * Given a string that contains text separated by a @p delimiter, split it
    * into its components; for each component, remove leading and trailing
@@ -176,7 +198,17 @@ namespace Utilities
    */
   std::vector<std::string>
   split_string_list (const std::string &s,
-                     const char         delimiter = ',');
+                     const std::string &delimiter = ",");
+
+
+  /**
+   * Specialization of split_string_list() for the case where the delimiter
+   * is a single char.
+   */
+  std::vector<std::string>
+  split_string_list (const std::string &s,
+                     const char delimiter);
+
 
   /**
    * Take a text, usually a documentation or something, and try to break it
@@ -231,6 +263,7 @@ namespace Utilities
   /**
    * Generate a random number from a normalized Gaussian probability
    * distribution centered around @p a and with standard deviation @p sigma.
+   * The returned number will be different every time the function is called.
    *
    * This function is reentrant, i.e., it can safely be called from multiple
    * threads at the same time. In addition, each thread will get the same
@@ -246,8 +279,8 @@ namespace Utilities
    * @note Like the system function rand(), this function produces the same
    * sequence of random numbers every time a program is started. This is an
    * important property for debugging codes, but it makes it impossible to
-   * really verify statistics properties of a code. For rand(), you can call
-   * srand() to "seed" the random number generator to get different sequences
+   * really verify statistical properties of a code. For `rand()`, you can call
+   * `srand()` to "seed" the random number generator to get different sequences
    * of random numbers every time a program is called. However, this function
    * does not allow seeding the random number generator. If you need this, as
    * above, use one of the C++ or BOOST facilities.
@@ -277,9 +310,24 @@ namespace Utilities
    *
    * Use this class as in <code>fixed_int_power@<5,2@>::%value</code> to
    * compute 5<sup>2</sup>.
+   *
+   * @deprecated This template has been deprecated in favor of C++11's support
+   * for <code>constexpr</code> calculations, e.g., use
+   *
+   * @code
+   * constexpr int value = Utilities::pow(2, dim);
+   * @endcode
+   *
+   * instead of
+   *
+   * @code
+   * const int value = Utilities::fixed_int_power<2, dim>::value;
+   * @endcode
+   *
+   * to obtain a constant expression for <code>value</code>.
    */
   template <int a, int N>
-  struct fixed_int_power
+  struct DEAL_II_DEPRECATED fixed_int_power
   {
     static const int value = a *fixed_int_power<a,N-1>::value;
   };
@@ -287,18 +335,32 @@ namespace Utilities
   /**
    * Base case for the power operation with <code>N=0</code>, which gives the
    * result 1.
+   *
+   * @deprecated This template is deprecated: see the note in the general
+   * version of this template for more information.
    */
   template <int a>
-  struct fixed_int_power<a,0>
+  struct DEAL_II_DEPRECATED fixed_int_power<a,0>
   {
     static const int value = 1;
   };
 
+  /**
+   * A replacement for <code>std::pow</code> that allows compile-time
+   * calculations for constant expression arguments.
+   */
+  constexpr
+  unsigned int
+  pow(const unsigned int base, const unsigned int iexp)
+  {
+    return iexp == 0 ? 1 : base*dealii::Utilities::pow(base, iexp - 1);
+  }
+
   /**
    * Optimized replacement for <tt>std::lower_bound</tt> for searching within
    * the range of column indices. Slashes execution time by approximately one
-   * half for the present application, partly because because the binary
-   * search is replaced by a linear search for small loop lengths.
+   * half for the present application, partly because the binary search is
+   * replaced by a linear search for small loop lengths.
    *
    * Another reason for this function is rather obscure: when using the GCC
    * libstdc++ function std::lower_bound, complexity is O(log(N)) as required.
@@ -315,7 +377,7 @@ namespace Utilities
    * This function simply makes the assumption that the sequence is sorted,
    * and we simply don't do the additional check.
    */
-  template<typename Iterator, typename T>
+  template <typename Iterator, typename T>
   Iterator
   lower_bound (Iterator  first,
                Iterator  last,
@@ -327,7 +389,7 @@ namespace Utilities
    * compare individual elements of the sequence of objects pointed to by the
    * iterators.
    */
-  template<typename Iterator, typename T, typename Comp>
+  template <typename Iterator, typename T, typename Comp>
   Iterator
   lower_bound (Iterator   first,
                Iterator   last,
@@ -366,6 +428,182 @@ namespace Utilities
   std::vector<unsigned long long int>
   invert_permutation (const std::vector<unsigned long long int> &permutation);
 
+  /**
+   * Given an arbitrary object of type T, use boost::serialization utilities
+   * to pack the object into a vector of characters and append it to the
+   * given buffer. The number of elements that have been added to the buffer
+   * will be returned. The object can be unpacked using the Utilities::unpack
+   * function below.
+   *
+   * If the library has been compiled with ZLIB enabled, then the output buffer
+   * is compressed.
+   *
+   * If many consecutive calls with the same buffer are considered, it is
+   * recommended for reasons of performance to ensure that its capacity is
+   * sufficient.
+   *
+   * @author Timo Heister, Wolfgang Bangerth, 2017.
+   */
+  template <typename T>
+  size_t pack (const T &object, std::vector<char> &dest_buffer);
+
+  /**
+   * Creates and returns a buffer solely for the given object, using the
+   * above mentioned pack function.
+   *
+   * @author Timo Heister, Wolfgang Bangerth, 2017.
+   */
+  template <typename T>
+  std::vector<char> pack (const T &object);
+
+  /**
+   * Given a vector of characters, obtained through a call to the function
+   * Utilities::pack, restore its content in an object of type T.
+   *
+   * This function uses boost::serialization utilities to unpack the object
+   * from a vector of characters, and it is the inverse of the function
+   * Utilities::pack().
+   *
+   * @note Since no arguments to this function depend on the template type
+   *  @p T, you must manually specify the template argument when calling
+   *  this function.
+   *
+   * @note If you want to pack() or unpack() arrays of objects, then the
+   *  following works:
+   *  @code
+   *    double array[3] = {1,2,3};
+   *    std::vector<char> buffer = Utilities::pack(array);
+   *  @endcode
+   *  However, the converse does not:
+   *  @code
+   *    array = Utilities::unpack<double[3]>(buffer);
+   *  @endcode
+   *  This is because C++ does not allow functions to return arrays.
+   *  Consequently, there is a separate unpack() function for arrays, see
+   *  below.
+   *
+   * @author Timo Heister, Wolfgang Bangerth, 2017.
+   */
+  template <typename T>
+  T unpack (const std::vector<char> &buffer);
+
+  /**
+   * Same unpack function as above, but takes constant iterators on
+   * (a fraction of) a given packed buffer of type std::vector<char> instead.
+   *
+   * @author Timo Heister, Wolfgang Bangerth, 2017.
+   */
+  template <typename T>
+  T unpack (const std::vector<char>::iterator &begin,
+            const std::vector<char>::iterator &end);
+
+  /**
+   * Given a vector of characters, obtained through a call to the function
+   * Utilities::pack, restore its content in an array of type T.
+   *
+   * This function uses boost::serialization utilities to unpack the object
+   * from a vector of characters, and it is the inverse of the function
+   * Utilities::pack().
+   *
+   * @note This function exists due to a quirk of C++. Specifically,
+   *  if you want to pack() or unpack() arrays of objects, then the
+   *  following works:
+   *  @code
+   *    double array[3] = {1,2,3};
+   *    std::vector<char> buffer = Utilities::pack(array);
+   *  @endcode
+   *  However, the converse does not:
+   *  @code
+   *    array = Utilities::unpack<double[3]>(buffer);
+   *  @endcode
+   *  This is because C++ does not allow functions to return arrays.
+   *  The current function therefore allows to write
+   *  @code
+   *    Utilities::unpack(buffer, array);
+   *  @endcode
+   *  Note that unlike the other unpack() function, it is not necessary
+   *  to explicitly specify the template arguments since they can be
+   *  deduced from the second argument.
+   *
+   * @author Timo Heister, Wolfgang Bangerth, 2017.
+   */
+  template <typename T, int N>
+  void unpack (const std::vector<char> &buffer,
+               T (&unpacked_object)[N]);
+
+  /**
+   * Same unpack function as above, but takes constant iterators on
+   * (a fraction of) a given packed buffer of type std::vector<char> instead.
+   *
+   * @author Timo Heister, Wolfgang Bangerth, 2017.
+   */
+  template <typename T, int N>
+  void unpack (const std::vector<char>::iterator &begin,
+               const std::vector<char>::iterator &end,
+               T (&unpacked_object)[N]);
+
+  /**
+   * Convert an object of type `std::unique_ptr<From>` to an object of
+   * type `std::unique_ptr<To>`, where it is assumed that we can cast
+   * the pointer to `From` to a pointer to `To` using a `dynamic_cast`
+   * -- in other words, we assume that `From` and `To` are connected
+   * through a class hierarchy, and that the object pointed to is in
+   * fact of a type that contains both a `From` and a `To`. An example
+   * is if either `To` is derived from `From` or the other way around.
+   *
+   * The function throws an exception of type `std::bad_cast` if the
+   * `dynamic_cast` does not succeed. This is the same exception you
+   * would get if a regular `dynamic_cast` between object types (but not
+   * pointer types) does not succeed.
+   *
+   * An example of how this function works is as follows:
+   * @code
+   *   class B { ... };            // A base class. Assume that it has virtual
+   *                               // functions so that dynamic_cast can work.
+   *   class D : public B { ... }; // A derived class
+   *
+   *
+   *   std::unique_ptr<B> create_object (...) {...}  // A factory function
+   *
+   *   void foo (...)
+   *   {
+   *     std::unique_ptr<B> b = create_object (...);
+   *
+   *     // Assume that we know for some reason that the object above must
+   *     // have created a D object but returned it as a std::unique_ptr<B>.
+   *     // In order to access the D functionality, we need to cast the
+   *     // pointer. Use the equivalent to dynamic_cast:
+   *     std::unique_ptr<D> d = dynamic_unique_cast<D>(std::move(b));
+   *
+   *     // If the object really was a D, then 'd' now points to it. Note
+   *     // also that in accordance with the semantics of std::unique_ptr,
+   *     // it was necessary to std::move the 'b' object, and indeed 'b'
+   *     // now no longer points to anything -- ownership has been
+   *     // transferred to 'd'!
+   * @endcode
+   *
+   * @note This function does not try to convert the `Deleter` objects stored
+   *   by `std::unique_ptr` objects. The function therefore only works if the
+   *   deleter objects are at their defaults, i.e., if they are of type
+   *   `std::default_delete<To>` and `std::default_delete<From>`.
+   */
+  template <typename To, typename From>
+  std::unique_ptr<To>
+  dynamic_unique_cast (std::unique_ptr<From> &&p)
+  {
+    // Let's see if we can cast from 'From' to 'To'. If so, do the cast,
+    // and then release the pointer from the old
+    // owner
+    if (To *cast = dynamic_cast<To *>(p.get()))
+      {
+        std::unique_ptr<To> result(cast);
+        p.release();
+        return result;
+      }
+    else
+      throw std::bad_cast();
+  }
+
   /**
    * A namespace for utility functions that probe system properties.
    *
@@ -383,6 +621,40 @@ namespace Utilities
      */
     double get_cpu_load ();
 
+    /**
+     * Return the current level of vectorization as described by DEAL_II_COMPILER_VECTORIZATION_LEVEL
+     * in vectorization.h as a string. The list of possible return values is:
+     *
+     * <table>
+     * <tr>
+     *   <td><tt>VECTORIZATION_LEVEL</tt></td>
+     *   <td>Return Value</td>
+     *   <td>Width in bits</td>
+     * </tr>
+     * <tr>
+     *   <td>0</td>
+     *   <td>disabled</td>
+     *   <td>64</td>
+     * </tr>
+     * <tr>
+     *   <td>1</td>
+     *   <td>SSE2</td>
+     *   <td>128</td>
+     * </tr>
+     * <tr>
+     *   <td>2</td>
+     *   <td>AVX</td>
+     *   <td>256</td>
+     * </tr>
+     * <tr>
+     *   <td>3</td>
+     *   <td>AVX512</td>
+     *   <td>512</td>
+     * </tr>
+     * </table>
+     */
+    const std::string get_current_vectorization_level();
+
     /**
      * Structure that hold information about memory usage in kB. Used by
      * get_memory_stats(). See man 5 proc entry /status for details.
@@ -412,7 +684,7 @@ namespace Utilities
 
 
     /**
-     * Fills the @p stats structure with information about the memory
+     * Fill the @p stats structure with information about the memory
      * consumption of this process. This is only implemented on Linux.
      */
     void get_memory_stats (MemoryStats &stats);
@@ -450,11 +722,6 @@ namespace Utilities
      * leaving this task to the calling site.
      */
     void posix_memalign (void **memptr, size_t alignment, size_t size);
-
-    /**
-     * @deprecated Use Utilities::MPI::job_supports_mpi() instead.
-     */
-    bool job_supports_mpi () DEAL_II_DEPRECATED;
   }
 
 
@@ -550,7 +817,11 @@ namespace Utilities
 
     /**
      * Return the number of MPI processes there exist in the given
-     * communicator object. If this is a sequential job, it returns 1.
+     * @ref GlossMPICommunicator "communicator"
+     * object. If this is a sequential job (i.e., the program
+     * is not using MPI at all, or is using MPI but has been started with
+     * only one MPI process), then the communicator necessarily involves
+     * only one process and the function returns 1.
      */
     unsigned int get_n_mpi_processes (const Epetra_Comm &mpi_communicator);
 
@@ -591,9 +862,11 @@ namespace Utilities
   inline
   T fixed_power (const T n)
   {
-    Assert (N>0, ExcNotImplemented());
+    Assert (N>=0, ExcNotImplemented());
     switch (N)
       {
+      case 0:
+        return dealii::internal::NumberType<T>::value(1);
       case 1:
         return n;
       case 2:
@@ -612,7 +885,7 @@ namespace Utilities
 
 
 
-  template<typename Iterator, typename T>
+  template <typename Iterator, typename T>
   inline
   Iterator
   lower_bound (Iterator  first,
@@ -625,7 +898,7 @@ namespace Utilities
 
 
 
-  template<typename Iterator, typename T, typename Comp>
+  template <typename Iterator, typename T, typename Comp>
   inline
   Iterator
   lower_bound (Iterator    first,
@@ -659,26 +932,32 @@ namespace Utilities
                 if (!comp(*first, val))
                   return first;
                 ++first;
+                DEAL_II_FALLTHROUGH;
               case 6:
                 if (!comp(*first, val))
                   return first;
                 ++first;
+                DEAL_II_FALLTHROUGH;
               case 5:
                 if (!comp(*first, val))
                   return first;
                 ++first;
+                DEAL_II_FALLTHROUGH;
               case 4:
                 if (!comp(*first, val))
                   return first;
                 ++first;
+                DEAL_II_FALLTHROUGH;
               case 3:
                 if (!comp(*first, val))
                   return first;
                 ++first;
+                DEAL_II_FALLTHROUGH;
               case 2:
                 if (!comp(*first, val))
                   return first;
                 ++first;
+                DEAL_II_FALLTHROUGH;
               case 1:
                 if (!comp(*first, val))
                   return first;
@@ -716,9 +995,235 @@ namespace Utilities
           len = half;
       }
   }
+
+
+// --------------------- non-inline functions
+
+  template <typename T>
+  size_t pack (const T &object, std::vector<char> &dest_buffer)
+  {
+    // see if the object is small and copyable via memcpy. if so, use
+    // this fast path. otherwise, we have to go through the BOOST
+    // serialization machinery
+    //
+    // we have to work around the fact that GCC 4.8.x claims to be C++
+    // conforming, but is not actually as it does not implement
+    // std::is_trivially_copyable.
+#if __GNUG__ && __GNUC__ < 5
+    if (  __has_trivial_copy(T) && sizeof(T)<256)
+#else
+#  ifdef DEAL_II_WITH_CXX17
+    if constexpr (std::is_trivially_copyable<T>() && sizeof(T)<256)
+#  else
+    if (std::is_trivially_copyable<T>() && sizeof(T)<256)
+#  endif
+#endif
+      {
+        const size_t previous_size = dest_buffer.size();
+        dest_buffer.resize (previous_size + sizeof(T));
+
+        std::memcpy (dest_buffer.data() + previous_size, &object, sizeof(T));
+
+        return sizeof(T);
+      }
+    else
+      {
+        // use buffer as the target of a compressing
+        // stream into which we serialize the current object
+        const size_t previous_size = dest_buffer.size();
+        {
+#ifdef DEAL_II_WITH_ZLIB
+          boost::iostreams::filtering_ostream out;
+          out.push(boost::iostreams::gzip_compressor
+                   (boost::iostreams::gzip_params
+                    (boost::iostreams::gzip::best_compression)));
+          out.push(boost::iostreams::back_inserter(dest_buffer));
+
+          boost::archive::binary_oarchive archive(out);
+          archive << object;
+          out.flush();
+#else
+          std::ostringstream out;
+          boost::archive::binary_oarchive archive(out);
+          archive << object;
+
+          const std::string &s = out.str();
+          dest_buffer.reserve (dest_buffer.size() + s.size());
+          std::move (s.begin(), s.end(), std::back_inserter(dest_buffer));
+#endif
+        }
+        return (dest_buffer.size() - previous_size);
+      }
+  }
+
+
+  template <typename T>
+  std::vector<char> pack (const T &object)
+  {
+    std::vector<char> buffer;
+    pack<T> (object, buffer);
+    return buffer;
+  }
+
+
+  template <typename T>
+  T unpack (const std::vector<char>::const_iterator &cbegin,
+            const std::vector<char>::const_iterator &cend)
+  {
+    // see if the object is small and copyable via memcpy. if so, use
+    // this fast path. otherwise, we have to go through the BOOST
+    // serialization machinery
+    //
+    // we have to work around the fact that GCC 4.8.x claims to be C++
+    // conforming, but is not actually as it does not implement
+    // std::is_trivially_copyable.
+#if __GNUG__ && __GNUC__ < 5
+    if (  __has_trivial_copy(T) && sizeof(T)<256)
+#else
+#  ifdef DEAL_II_WITH_CXX17
+    if constexpr (std::is_trivially_copyable<T>() && sizeof(T)<256)
+#  else
+    if (std::is_trivially_copyable<T>() && sizeof(T)<256)
+#  endif
+#endif
+      {
+        Assert (std::distance(cbegin, cend) == sizeof(T), ExcInternalError());
+        T object;
+        std::memcpy (&object, &*cbegin, sizeof(T));
+        return object;
+      }
+    else
+      {
+        std::string decompressed_buffer;
+        T object;
+
+        // first decompress the buffer
+        {
+#ifdef DEAL_II_WITH_ZLIB
+          boost::iostreams::filtering_ostream decompressing_stream;
+          decompressing_stream.push(boost::iostreams::gzip_decompressor());
+          decompressing_stream.push(boost::iostreams::back_inserter(decompressed_buffer));
+          decompressing_stream.write (&*cbegin, std::distance(cbegin, cend));
+#else
+          decompressed_buffer.assign (cbegin, cend);
+#endif
+        }
+
+        // then restore the object from the buffer
+        std::istringstream in(decompressed_buffer);
+        boost::archive::binary_iarchive archive(in);
+
+        archive >> object;
+        return object;
+      }
+  }
+
+
+  template <typename T>
+  T unpack(const std::vector<char> &buffer)
+  {
+    return unpack<T> (buffer.cbegin(), buffer.cend());
+  }
+
+
+  template <typename T, int N>
+  void unpack (const std::vector<char>::const_iterator &cbegin,
+               const std::vector<char>::const_iterator &cend,
+               T (&unpacked_object)[N])
+  {
+    // see if the object is small and copyable via memcpy. if so, use
+    // this fast path. otherwise, we have to go through the BOOST
+    // serialization machinery
+    //
+    // we have to work around the fact that GCC 4.8.x claims to be C++
+    // conforming, but is not actually as it does not implement
+    // std::is_trivially_copyable.
+    if (
+#if __GNUG__ && __GNUC__ < 5
+      __has_trivial_copy(T)
+#else
+      std::is_trivially_copyable<T>()
+#endif
+      &&
+      sizeof(T)*N<256)
+      {
+        Assert (std::distance(cbegin, cend) == sizeof(T)*N, ExcInternalError());
+        std::memcpy (unpacked_object, &*cbegin, sizeof(T)*N);
+      }
+    else
+      {
+        std::string decompressed_buffer;
+
+        // first decompress the buffer
+        {
+#ifdef DEAL_II_WITH_ZLIB
+          boost::iostreams::filtering_ostream decompressing_stream;
+          decompressing_stream.push(boost::iostreams::gzip_decompressor());
+          decompressing_stream.push(boost::iostreams::back_inserter(decompressed_buffer));
+          decompressing_stream.write (&*cbegin, std::distance(cbegin, cend));
+#else
+          decompressed_buffer.assign (cbegin, cend);
+#endif
+        }
+
+        // then restore the object from the buffer
+        std::istringstream in(decompressed_buffer);
+        boost::archive::binary_iarchive archive(in);
+
+        archive >> unpacked_object;
+      }
+  }
+
+
+  template <typename T, int N>
+  void unpack (const std::vector<char> &buffer,
+               T (&unpacked_object)[N])
+  {
+    unpack<T,N> (buffer.cbegin(), buffer.cend(), unpacked_object);
+  }
+
 }
 
 
 DEAL_II_NAMESPACE_CLOSE
 
+#ifndef DOXYGEN
+namespace boost
+{
+  namespace serialization
+  {
+
+    // Provides boost and c++11 with a way to serialize tuples and pairs automatically
+    template<int N>
+    struct Serialize
+    {
+      template<class Archive, typename... Args>
+      static void serialize(Archive &ar, std::tuple<Args...> &t, const unsigned int version)
+      {
+        ar &std::get<N-1>(t);
+        Serialize<N-1>::serialize(ar, t, version);
+      }
+    };
+
+    template<>
+    struct Serialize<0>
+    {
+      template<class Archive, typename... Args>
+      static void serialize(Archive &ar, std::tuple<Args...> &t, const unsigned int version)
+      {
+        (void) ar;
+        (void) t;
+        (void) version;
+      }
+    };
+
+    template<class Archive, typename... Args>
+    void serialize(Archive &ar, std::tuple<Args...> &t, const unsigned int version)
+    {
+      Serialize<sizeof...(Args)>::serialize(ar, t, version);
+    }
+  }
+}
+#endif
+
 #endif
