@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 2004 - 2016 by the deal.II authors
+// Copyright (C) 2004 - 2018 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -13,8 +13,8 @@
 //
 // ---------------------------------------------------------------------
 
-#ifndef dealii__vector_slice_h
-#define dealii__vector_slice_h
+#ifndef dealii_vector_slice_h
+#define dealii_vector_slice_h
 
 #include <deal.II/base/config.h>
 #include <deal.II/base/exceptions.h>
@@ -134,7 +134,7 @@ private:
  * Helper function for creating temporary objects without typing template
  * arguments.
  *
- * @relates VectorSlice
+ * @relatesalso VectorSlice
  * @author Guido Kanschat, 2004
  */
 template <typename VectorType>
@@ -152,7 +152,7 @@ make_slice (VectorType &v)
  * Helper function for creating temporary objects without typing template
  * arguments.
  *
- * @relates VectorSlice
+ * @relatesalso VectorSlice
  * @author Guido Kanschat, 2004
  */
 template <typename VectorType>
