@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 2016 by the deal.II authors
+// Copyright (C) 2016 - 2017 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -13,8 +13,8 @@
 //
 // ---------------------------------------------------------------------
 
-#ifndef dealii__p4est_wrappers_h
-#define dealii__p4est_wrappers_h
+#ifndef dealii_p4est_wrappers_h
+#define dealii_p4est_wrappers_h
 
 #include <deal.II/base/geometry_info.h>
 
@@ -472,16 +472,14 @@ namespace internal
       static
       size_t (&connectivity_memory_used) (types<3>::connectivity *p4est);
 
-      template <int spacedim>
-      static void iterate(dealii::internal::p4est::types<3>::forest *parallel_forest,
-                          dealii::internal::p4est::types<3>::ghost *parallel_ghost,
-                          void *user_data);
+
 
       static const unsigned int max_level = P8EST_MAXLEVEL;
     };
 
 
 
+
     /**
      * This struct templatizes the p4est iterate structs and function
      * prototypes, which are used to execute callback functions for faces,
@@ -537,7 +535,7 @@ namespace internal
 
 
     /**
-     * Returns whether q1 and q2 are equal
+     * Return whether q1 and q2 are equal
      */
     template <int dim>
     bool
@@ -547,7 +545,7 @@ namespace internal
 
 
     /**
-     * Returns whether q1 is an ancestor of q2
+     * Return whether q1 is an ancestor of q2
      */
     template <int dim>
     bool
@@ -557,7 +555,7 @@ namespace internal
 
 
     /**
-     * Returns whether the children of a coarse cell are stored locally
+     * Return whether the children of a coarse cell are stored locally
      */
     template <int dim>
     bool
@@ -565,18 +563,15 @@ namespace internal
                          const typename types<dim>::topidx coarse_grid_cell);
 
 
+
     /**
-     * This is the callback data structure used to fill
-     * vertices_with_ghost_neighbors via the p4est_iterate tool
+     * Compute the ghost neighbors surrounding each vertex by querying p4est
      */
     template <int dim, int spacedim>
-    struct FindGhosts
-    {
-      typename dealii::parallel::distributed::Triangulation<dim,spacedim> *triangulation;
-      sc_array_t                                                          *subids;
-      std::map<unsigned int, std::set<dealii::types::subdomain_id> >      *vertices_with_ghost_neighbors;
-    };
-
+    std::map<unsigned int, std::set<dealii::types::subdomain_id> >
+    compute_vertices_with_ghost_neighbors(const dealii::parallel::distributed::Triangulation<dim,spacedim> &tria,
+                                          typename dealii::internal::p4est::types<dim>::forest *parallel_forest,
+                                          typename dealii::internal::p4est::types<dim>::ghost *parallel_ghost);
 
   }
 }
@@ -585,4 +580,4 @@ DEAL_II_NAMESPACE_CLOSE
 
 #endif //DEAL_II_WITH_P4EST
 
-#endif //dealii__p4est_wrappers_h
+#endif //dealii_p4est_wrappers_h
