@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 2009 - 2016 by the deal.II authors
+// Copyright (C) 2009 - 2018 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -13,8 +13,8 @@
 //
 // ---------------------------------------------------------------------
 
-#ifndef dealii__distributed_solution_transfer_h
-#define dealii__distributed_solution_transfer_h
+#ifndef dealii_distributed_solution_transfer_h
+#define dealii_distributed_solution_transfer_h
 
 #include <deal.II/base/config.h>
 #include <deal.II/distributed/tria.h>
@@ -31,7 +31,7 @@ namespace parallel
   namespace distributed
   {
     /**
-     * Transfers a discrete FE function (like a solution vector) by
+     * Transfer a discrete FE function (like a solution vector) by
      * interpolation while refining and/or coarsening a distributed grid and
      * handles the necessary communication.
      *
@@ -46,7 +46,8 @@ namespace parallel
      * so that all locally_active elements can be read. On the other hand,
      * ghosted vectors are generally not writable, so for calls to
      * interpolate() or deserialize() you need to supply distributed vectors
-     * without ghost elements.
+     * without ghost elements. More precisely, during interpolation the
+     * current algorithm writes into all locally active degrees of freedom.
      *
      * <h3>Transferring a solution</h3> Here VectorType is your favorite
      * vector type, e.g. PETScWrappers::MPI::Vector,
@@ -74,6 +75,15 @@ namespace parallel
      * soltrans.interpolate(interpolated_solution);
      * @endcode
      *
+     * Different from PETSc and Trilinos vectors, LinearAlgebra::distributed::Vector
+     * allows writing into ghost elements. For a ghosted vector the
+     * interpolation step can be accomplished via
+     * @code
+     * interpolated_solution.zero_out_ghosts();
+     * soltrans.interpolate(interpolated_solution);
+     * interpolated_solution.update_ghost_values();
+     * @endcode
+     *
      * <h3>Use for Serialization</h3>
      *
      * This class can be used to serialize and later deserialize a distributed
@@ -112,21 +122,35 @@ namespace parallel
      * @ingroup distributed
      * @author Timo Heister, 2009-2011
      */
-    template<int dim, typename VectorType, typename DoFHandlerType=DoFHandler<dim> >
+    template <int dim,
+              typename VectorType,
+              typename DoFHandlerType = DoFHandler<dim> >
     class SolutionTransfer
     {
+#ifndef DEAL_II_MSVC
+      static_assert (dim == DoFHandlerType::dimension,
+                     "The dimension explicitly provided as a template "
+                     "argument, and the dimension of the DoFHandlerType "
+                     "template argument must match.");
+#endif
     public:
       /**
-       * Constructor, takes the current DoFHandler as argument.
+       * Constructor.
+       *
+       * @param[in] dof The DoFHandler or hp::DoFHandler on which all
+       *   operations will happen. At the time when this constructor
+       *   is called, the DoFHandler still points to the triangulation
+       *   before the refinement in question happens.
        */
       SolutionTransfer(const DoFHandlerType &dof);
+
       /**
        * Destructor.
        */
-      ~SolutionTransfer();
+      ~SolutionTransfer() = default;
 
       /**
-       * Prepares the @p SolutionTransfer for coarsening and refinement. It
+       * Prepare the current object for coarsening and refinement. It
        * stores the dof indices of each cell and stores the dof values of the
        * vectors in @p all_in in each cell that'll be coarsened. @p all_in
        * includes all vectors that are to be interpolated onto the new
@@ -135,7 +159,7 @@ namespace parallel
       void prepare_for_coarsening_and_refinement (const std::vector<const VectorType *> &all_in);
 
       /**
-       * Same as previous function but for only one discrete function to be
+       * Same as the previous function but for only one discrete function to be
        * interpolated.
        */
       void prepare_for_coarsening_and_refinement (const VectorType &in);
