@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 2008 - 2017 by the deal.II authors
+// Copyright (C) 2008 - 2018 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -13,8 +13,8 @@
 //
 // ---------------------------------------------------------------------
 
-#ifndef dealii__distributed_tria_h
-#define dealii__distributed_tria_h
+#ifndef dealii_distributed_tria_h
+#define dealii_distributed_tria_h
 
 
 #include <deal.II/base/config.h>
@@ -23,9 +23,6 @@
 #include <deal.II/base/template_constraints.h>
 #include <deal.II/grid/tria.h>
 
-#include <deal.II/base/std_cxx11/function.h>
-#include <deal.II/base/std_cxx11/tuple.h>
-
 #include <deal.II/distributed/tria_base.h>
 #include <deal.II/distributed/p4est_wrappers.h>
 
@@ -33,6 +30,9 @@
 #include <vector>
 #include <list>
 #include <utility>
+#include <functional>
+#include <tuple>
+#include <type_traits>
 
 #ifdef DEAL_II_WITH_MPI
 #  include <mpi.h>
@@ -55,11 +55,11 @@ DEAL_II_NAMESPACE_OPEN
 
 namespace internal
 {
-  namespace DoFHandler
+  namespace DoFHandlerImplementation
   {
     namespace Policy
     {
-      template <int, int> class ParallelDistributed;
+      template <typename> class ParallelDistributed;
     }
   }
 }
@@ -157,7 +157,7 @@ namespace parallel
      * information into the function. C++ provides a nice mechanism for this
      * that is best explained using an example:
      * @code
-     *     #include <deal.II/base/std_cxx11/bind.h>
+     *     #include <functional>
      *
      *     template <int dim>
      *     void set_boundary_ids (parallel::distributed::Triangulation<dim> &triangulation)
@@ -173,13 +173,13 @@ namespace parallel
      *       ... create the coarse mesh ...
      *
      *       coarse_grid.signals.post_refinement.connect
-     *         (std_cxx11::bind (&set_boundary_ids<dim>,
-     *                           std_cxx11::ref(coarse_grid)));
+     *         (std::bind (&set_boundary_ids<dim>,
+     *                     std::ref(coarse_grid)));
      *
      *     }
      * @endcode
      *
-     * What the call to <code>std_cxx11::bind</code> does is to produce an
+     * What the call to <code>std::bind</code> does is to produce an
      * object that can be called like a function with no arguments. It does so
      * by taking the address of a function that does, in fact, take an
      * argument but permanently fix this one argument to a reference to the
@@ -197,7 +197,7 @@ namespace parallel
      * static, but possibly private) member function of the
      * <code>MyClass</code> class, then the following will work:
      * @code
-     *     #include <deal.II/base/std_cxx11/bind.h>
+     *     #include <functional>
      *
      *     template <int dim>
      *     void
@@ -215,9 +215,9 @@ namespace parallel
      *       ... create the coarse mesh ...
      *
      *       coarse_grid.signals.post_refinement.connect
-     *         (std_cxx11::bind (&MyGeometry<dim>::set_boundary_ids,
-     *                           std_cxx11::cref(*this),
-     *                           std_cxx11::ref(coarse_grid)));
+     *         (std::bind (&MyGeometry<dim>::set_boundary_ids,
+     *                     std::cref(*this),
+     *                     std::ref(coarse_grid)));
      *     }
      * @endcode
      * Here, like any other member function, <code>set_boundary_ids</code>
@@ -608,65 +608,159 @@ namespace parallel
                 const bool autopartition = true);
 
       /**
-       * Register a function with the current Triangulation object that will
-       * be used to attach data to active cells before
-       * execute_coarsening_and_refinement(). In
-       * execute_coarsening_and_refinement() the Triangulation will call the
-       * given function pointer and provide @p size bytes to store data. If
-       * necessary, this data will be transferred to the new owner of that
-       * cell during repartitioning the tree. See notify_ready_to_unpack() on
-       * how to retrieve the data.
+       * Register a function that can be used to attach data of fixed size
+       * to cells. This is useful for two purposes: (i) Upon refinement and
+       * coarsening of a triangulation (in
+       * parallel::distributed::Triangulation::execute_coarsening_and_refinement()),
+       * one needs to be able to store one or more data vectors per cell that
+       * characterizes the solution values on the cell so that this data can
+       * then be transferred to the new owning processor of the cell (or
+       * its parent/children) when the mesh is re-partitioned; (ii) when
+       * serializing a computation to a file, it is necessary to attach
+       * data to cells so that it can be saved (in
+       * parallel::distributed::Triangulation::save()) along with the cell's
+       * other information and, if necessary, later be reloaded from disk
+       * with a different subdivision of cells among the processors.
        *
-       * Callers need to store the return value.  It specifies an offset of
-       * the position at which data can later be retrieved during a call to
+       * The way this function works is that it allows any number of interest
+       * parties to register their intent to attach data to cells. One example
+       * of classes that do this is parallel::distributed::SolutionTransfer
+       * where each parallel::distributed::SolutionTransfer object that works
+       * on the current Triangulation object then needs to register its intent.
+       * Each of these parties registers a call-back function (the second
+       * argument here, @p pack_callback) that will be called whenever the
+       * triangulation's execute_coarsening_and_refinement() or save()
+       * functions are called. The first argument here specifies the number
+       * of bytes the interest party will want to store per cell. (This
+       * needs to be a constant per cell.)
+       *
+       * The current function then returns an integer handle that corresponds
+       * to the number of data set that the callback provided here will attach.
+       * While this number could be given a precise meaning, this is
+       * not important: You will never actually have to do anything with
+       * this number except return it to the notify_ready_to_unpack() function.
+       * In other words, each interested party (i.e., the caller of the current
+       * function) needs to store their respective returned handle for later use
+       * when unpacking data in the callback provided to
        * notify_ready_to_unpack().
        *
-       * The CellStatus argument in the callback function will tell you if the
-       * given cell will be coarsened, refined, or will persist as is (this
-       * can be different than the coarsen and refine flags set by you). If it
-       * is
+       * Whenever @p pack_callback is then called by
+       * execute_coarsening_and_refinement() or load() on a given cell, it
+       * receives a number of arguments. In particular, the first
+       * argument passed to the callback indicates the cell for which
+       * it is supposed to attach data. This is always an active cell.
        *
-       * - CELL_PERSIST: the cell won't be refined/coarsened, but might be
-       * moved to a different processor
-       * - CELL_REFINE: this cell will be refined into 4/8 cells, you can not
-       * access the children (because they don't exist yet)
-       * - CELL_COARSEN: the children of this cell will be coarsened into the
-       * given cell (you can access the active children!)
+       * The second, CellStatus, argument provided to the callback function
+       * will tell you if the given cell will be coarsened, refined, or will
+       * persist as is. (This status may be different than the refinement
+       * or coarsening flags set on that cell, to accommodate things such as
+       * the "one hanging node per edge" rule.). Specifically, the values for
+       * this argument mean the following:
        *
-       * When unpacking the data with notify_ready_to_unpack() you can access
-       * the children of the cell if the status is CELL_REFINE but not for
-       * CELL_COARSEN. As a consequence you need to handle coarsening while
-       * packing and refinement during unpacking.
+       * - `CELL_PERSIST`: The cell won't be refined/coarsened, but might be
+       * moved to a different processor. If this is the case, the callback
+       * will want to pack up the data on this cell into an array and store
+       * it at the provided address for later unpacking wherever this cell
+       * may land.
+       * - `CELL_REFINE`: this cell will be refined into 4 or 8 cells (in 2d
+       * and 3d, respectively). However, because these children don't exist
+       * yet, you cannot access them at the time when the callback is
+       * called. If the callback is called with this value, the callback
+       * will want to pack up the data on this cell into an array and store
+       * it at the provided address for later unpacking in a way so that
+       * it can then be transferred to the children of the cell that will
+       * then be available. In other words, if the data the callback
+       * will want to pack up corresponds to a finite element field, then
+       * the prolongation from parent to (new) children will have to happen
+       * during unpacking.
+       * - `CELL_COARSEN`: the children of this cell will be coarsened into the
+       * given cell. These children still exist, so if this is the value
+       * given to the callback as second argument, the callback will want
+       * to transfer data from the children to the current parent cell and
+       * pack it up so that it can later be unpacked again on a cell that
+       * then no longer has any children (and may also be located on a
+       * different processor). In other words, if the data the callback
+       * will want to pack up corresponds to a finite element field, then
+       * it will need to do the restriction from children to parent at
+       * this point.
        *
-       * @note The two functions can also be used for serialization of data
-       * using save() and load() in the same way. Then the status will always
-       * be CELL_PERSIST.
+       * @note If this function is used for serialization of data
+       *   using save() and load(), then the cell status argument with which
+       *   the callback is called will always `CELL_PERSIST`.
+       *
+       * The third argument given to the callback is a pointer to a memory
+       * location at which the callback can store its data. It is allowed
+       * to store exactly as many bytes as were passed to the current
+       * function via the @p size argument.
+       *
+       * @note The purpose of this function is to register intent to
+       *   attach data for a single, subsequent call to
+       *   execute_coarsening_and_refinement() and notify_ready_to_unpack(),
+       *   save(), load(). Consequently, notify_ready_to_unpack(), save(),
+       *   and load() all forget the registered callbacks once these
+       *   callbacks have been called, and you will have to re-register
+       *   them with a triangulation if you want them to be active for
+       *   another call to these functions.
        */
       unsigned int
       register_data_attach (const std::size_t size,
-                            const std_cxx11::function<void (const cell_iterator &,
-                                                            const CellStatus,
-                                                            void *)> &pack_callback);
+                            const std::function<void (const cell_iterator &,
+                                                      const CellStatus,
+                                                      void *)> &pack_callback);
 
       /**
-       * The supplied callback function is called for each newly locally owned
-       * cell and corresponding data saved with register_data_attach().  This
-       * function needs to be called after execute_coarsening_and_refinement()
-       * with the offset returned by register_data_attach().
+       * This function is the opposite of register_data_attach(). It is called
+       * <i>after</i> the execute_coarsening_and_refinement() or save()/load()
+       * functions are done when classes and functions that have previously
+       * attached data to a triangulation for either transfer to other
+       * processors, across mesh refinement, or serialization of data to
+       * a file are ready to receive that data back. The important part about
+       * this process is that the triangulation cannot do this right away from
+       * the end of execute_coarsening_and_refinement() or load() via a
+       * previously attached callback function (as the register_data_attach()
+       * function does) because the classes that eventually want the data
+       * back may need to do some setup between the point in time where the
+       * mesh has been recreated and when the data can actually be received.
+       * An example is the parallel::distributed::SolutionTransfer class
+       * that can really only receive the data once not only the mesh is
+       * completely available again on the current processor, but only
+       * after a DoFHandler has been reinitialized and distributed
+       * degrees of freedom. In other words, there is typically a significant
+       * amount of set up that needs to happen in user space before the classes
+       * that can receive data attached to cell are ready to actually do so.
+       * When they are, they use the current function to tell the triangulation
+       * object that now is the time when they are ready by calling the
+       * current function.
+       *
+       * The supplied callback function is then called for each newly locally owned
+       * cell. The first argument to the callback is an iterator that designated
+       * the cell; the second argument indicates the status of the cell in
+       * question; and the third argument points to a memory area that contains the
+       * data that was previously saved from the callback provided to
+       * register_data_attach().
        *
        * The CellStatus will indicate if the cell was refined, coarsened, or
-       * persisted unchanged. The cell_iterator will either by an active,
+       * persisted unchanged. The @p cell_iterator argument to the callback
+       * will then either be an active,
        * locally owned cell (if the cell was not refined), or the immediate
        * parent if it was refined during execute_coarsening_and_refinement().
        * Therefore, contrary to during register_data_attach(), you can now
-       * access the children if the status is CELL_REFINE but no longer for
-       * callbacks with status CELL_COARSEN.
+       * access the children if the status is `CELL_REFINE` but no longer for
+       * callbacks with status `CELL_COARSEN`.
+       *
+       * The first argument to this function, `handle`, corresponds to
+       * the return value of register_data_attach(). (The precise
+       * meaning of what the numeric value of this handle is supposed
+       * to represent is neither important, nor should you try to use
+       * it for anything other than transmit information between a
+       * call to register_data_attach() to the corresponding call to
+       * notify_ready_to_unpack().)
        */
       void
-      notify_ready_to_unpack (const unsigned int offset,
-                              const std_cxx11::function<void (const cell_iterator &,
-                                                              const CellStatus,
-                                                              const void *)> &unpack_callback);
+      notify_ready_to_unpack (const unsigned int handle,
+                              const std::function<void (const cell_iterator &,
+                                                        const CellStatus,
+                                                        const void *)> &unpack_callback);
 
       /**
        * Return a permutation vector for the order the coarse cells are handed
@@ -705,7 +799,7 @@ namespace parallel
        */
       virtual void
       add_periodicity
-      (const std::vector<GridTools::PeriodicFacePair<cell_iterator> > &);
+      (const std::vector<dealii::GridTools::PeriodicFacePair<cell_iterator> > &);
 
 
     private:
@@ -713,7 +807,6 @@ namespace parallel
       /**
        * Override the function to update the number cache so we can fill data
        * like @p level_ghost_owners.
-       *
        */
       virtual void update_number_cache ();
 
@@ -739,6 +832,7 @@ namespace parallel
        * triangulation.
        */
       typename dealii::internal::p4est::types<dim>::forest *parallel_forest;
+
       /**
        * A data structure that holds some information about the ghost cells of
        * the triangulation.
@@ -746,50 +840,47 @@ namespace parallel
       typename dealii::internal::p4est::types<dim>::ghost  *parallel_ghost;
 
       /**
-       * A flag that indicates whether refinement of a triangulation is
-       * currently in progress. This flag is used to disambiguate whether a
-       * call to execute_coarsening_and_triangulation came from the outside or
-       * through a recursive call. While the first time we want to take over
-       * work to copy things from a refined p4est, the other times we don't
-       * want to get in the way as these latter calls to
-       * Triangulation::execute_coarsening_and_refinement() are simply there
-       * in order to re-create a triangulation that matches the p4est.
+       * A structure that stores information about the data that has been, or
+       * will be, attached to cells via the register_data_attach() function
+       * and later retrieved via notify_ready_to_unpack().
        */
-      bool refinement_in_progress;
+      struct CellAttachedData
+      {
+        /**
+         * Cumulative size in bytes of the buffers that those functions that
+         * have called register_data_attach() want to attach to each cell.
+         * This number only pertains to fixed-sized buffers where the data
+         * attached to each cell has exactly the same size.
+         */
+        unsigned int cumulative_fixed_data_size;
 
+        /**
+         * number of functions that get attached to the Triangulation through
+         * register_data_attach() for example SolutionTransfer.
+         */
+        unsigned int n_attached_datas;
 
-      /**
-       * number of bytes that get attached to the Triangulation through
-       * register_data_attach() for example SolutionTransfer.
-       */
-      unsigned int attached_data_size;
+        /**
+         * number of functions that need to unpack their data after a call from
+         * load()
+         */
+        unsigned int n_attached_deserialize;
 
-      /**
-       * number of functions that get attached to the Triangulation through
-       * register_data_attach() for example SolutionTransfer.
-       */
-      unsigned int n_attached_datas;
+        using pack_callback_t = std::function<void (typename Triangulation<dim,spacedim>::cell_iterator,
+                                                    CellStatus,
+                                                    void *)>;
 
-      /**
-       * number of functions that need to unpack their data after a call from
-       * load()
-       */
-      unsigned int n_attached_deserialize;
-
-      typedef  std_cxx11::function<
-      void(typename Triangulation<dim,spacedim>::cell_iterator, CellStatus, void *)
-      > pack_callback_t;
-
-      typedef std::pair<unsigned int, pack_callback_t> callback_pair_t;
-
-      typedef std::list<callback_pair_t> callback_list_t;
-
-      /**
-       * List of callback functions registered by register_data_attach() that
-       * are going to be called for packing data.
-       */
-      callback_list_t attached_data_pack_callbacks;
+        /**
+         * List of callback functions registered by register_data_attach() that
+         * are going to be called for packing data. The key of this map
+         * variable is the offset at which each callback is allowed to
+         * write into the per-cell buffer (counted in bytes) whereas the
+         * value of the map is the callback function object.
+         */
+        std::map<unsigned int, pack_callback_t> pack_callbacks;
+      };
 
+      CellAttachedData cell_attached_data;
 
       /**
        * Two arrays that store which p4est tree corresponds to which coarse
@@ -826,8 +917,8 @@ namespace parallel
        *
        * This function exists in 2d and 3d variants.
        */
-      void copy_new_triangulation_to_p4est (dealii::internal::int2type<2>);
-      void copy_new_triangulation_to_p4est (dealii::internal::int2type<3>);
+      void copy_new_triangulation_to_p4est (std::integral_constant<int, 2>);
+      void copy_new_triangulation_to_p4est (std::integral_constant<int, 3>);
 
       /**
        * Copy the local part of the refined forest from p4est into the
@@ -838,7 +929,20 @@ namespace parallel
       /**
        * Internal function notifying all registered classes to attach their
        * data before repartitioning occurs. Called from
-       * execute_coarsening_and_refinement().
+       * execute_coarsening_and_refinement() and save(). The function
+       * recursively visits all deal.II cells and corresponding p4est
+       * quadrants and calls the callbacks registered via
+       * register_data_attach() on the ones where data needs to be
+       * stored.
+       *
+       * This function is odd in that it is called on a p4est triangulation
+       * and a deal.II triangulation that may not be in sync when it is called
+       * from execute_coarsening_and_refinement(). Specifically, the p4est
+       * trees have already been refined and coarsened, but the deal.II
+       * triangulation has not. Consequently, when walking the two recursively,
+       * it can reason about which cells will remain after the deal.II
+       * triangulation has been brought up to date with regard to the p4est
+       * trees.
        */
       void attach_mesh_data();
 
@@ -859,25 +963,14 @@ namespace parallel
       get_cell_weights();
 
       /**
-       * Fills a map that, for each vertex, lists all the processors whose
-       * subdomains are adjacent to that vertex. Used by
-       * DoFHandler::Policy::ParallelDistributed.
+       * Override the implementation in parallel::Triangulation because
+       * we can ask p4est about ghost neighbors across periodic boundaries.
+       *
+       * Specifically, this function determines the neighboring subdomains that
+       * are adjacent to each vertex.
        */
-      void
-      fill_vertices_with_ghost_neighbors
-      (std::map<unsigned int, std::set<dealii::types::subdomain_id> >
-       &vertices_with_ghost_neighbors);
-
-      /**
-       * Fills a map that, for each vertex, lists all the processors whose
-       * subdomains are adjacent to that vertex on the given level for the
-       * multigrid hierarchy. Used by DoFHandler::Policy::ParallelDistributed.
-       */
-      void
-      fill_level_vertices_with_ghost_neighbors
-      (const int level,
-       std::map<unsigned int, std::set<dealii::types::subdomain_id> >
-       &vertices_with_ghost_neighbors);
+      virtual std::map<unsigned int, std::set<dealii::types::subdomain_id> >
+      compute_vertices_with_ghost_neighbors () const;
 
       /**
        * This method returns a bit vector of length tria.n_vertices()
@@ -891,7 +984,7 @@ namespace parallel
       std::vector<bool>
       mark_locally_active_vertices_on_level(const int level) const;
 
-      template <int, int> friend class dealii::internal::DoFHandler::Policy::ParallelDistributed;
+      template <typename> friend class dealii::internal::DoFHandlerImplementation::Policy::ParallelDistributed;
 
       template <int,int,class> friend class dealii::FETools::internal::ExtrapolateImplementation;
     };
@@ -906,11 +999,25 @@ namespace parallel
     class Triangulation<1,spacedim> : public dealii::parallel::Triangulation<1,spacedim>
     {
     public:
+
+      /**
+       * dummy settings
+       */
+      enum Settings
+      {
+        default_setting = 0x0,
+        mesh_reconstruction_after_repartitioning = 0x1,
+        construct_multigrid_hierarchy = 0x2
+      };
+
       /**
        * Constructor. The argument denotes the MPI communicator to be used for
        * the triangulation.
        */
-      Triangulation (MPI_Comm mpi_communicator);
+      Triangulation (MPI_Comm mpi_communicator,
+                     const typename dealii::Triangulation<1,spacedim>::MeshSmoothing
+                     smooth_grid = (dealii::Triangulation<1,spacedim>::none),
+                     const Settings settings = default_setting);
 
       /**
        * Destructor.
@@ -962,6 +1069,35 @@ namespace parallel
       void
       communicate_locally_moved_vertices (const std::vector<bool> &vertex_locally_moved);
 
+      /**
+       * This function is not implemented, but needs to be present for the compiler.
+       */
+      void load(const char *filename,
+                const bool autopartition = true);
+
+      /**
+       * This function is not implemented, but needs to be present for the compiler.
+       */
+      void save(const char *filename) const;
+
+      /**
+       * This function is not implemented, but needs to be present for the compiler.
+       */
+      unsigned int
+      register_data_attach (const std::size_t size,
+                            const std::function<void (const typename dealii::Triangulation<1,spacedim>::cell_iterator &,
+                                                      const typename dealii::Triangulation<1,spacedim>::CellStatus,
+                                                      void *)> &pack_callback);
+
+      /**
+       * This function is not implemented, but needs to be present for the compiler.
+       */
+      void
+      notify_ready_to_unpack (const unsigned int offset,
+                              const std::function<void (const typename dealii::Triangulation<1,spacedim>::cell_iterator &,
+                                                        const typename dealii::Triangulation<1,spacedim>::CellStatus,
+                                                        const void *)> &unpack_callback);
+
       /**
        * Dummy arrays. This class isn't usable but the compiler wants to see
        * these variables at a couple places anyway.
@@ -969,16 +1105,6 @@ namespace parallel
       std::vector<types::global_dof_index> coarse_cell_to_p4est_tree_permutation;
       std::vector<types::global_dof_index> p4est_tree_to_coarse_cell_permutation;
 
-      /**
-       * dummy settings
-       */
-      enum Settings
-      {
-        default_setting = 0x0,
-        mesh_reconstruction_after_repartitioning = 0x1,
-        construct_multigrid_hierarchy = 0x2
-      };
-
 
 //TODO: The following variable should really be private, but it is used in dof_handler_policy.cc ...
       /**
@@ -990,20 +1116,15 @@ namespace parallel
        * Like above, this method, which is only implemented for dim = 2 or 3,
        * needs a stub because it is used in dof_handler_policy.cc
        */
-      void
-      fill_vertices_with_ghost_neighbors
-      (std::map<unsigned int, std::set<dealii::types::subdomain_id> >
-       &vertices_with_ghost_neighbors);
+      std::map<unsigned int, std::set<dealii::types::subdomain_id> >
+      compute_vertices_with_ghost_neighbors () const;
 
       /**
        * Like above, this method, which is only implemented for dim = 2 or 3,
        * needs a stub because it is used in dof_handler_policy.cc
        */
-      void
-      fill_level_vertices_with_ghost_neighbors
-      (const unsigned int level,
-       std::map<unsigned int, std::set<dealii::types::subdomain_id> >
-       &vertices_with_ghost_neighbors);
+      std::map<unsigned int, std::set<dealii::types::subdomain_id> >
+      compute_level_vertices_with_ghost_neighbors (const unsigned int level) const;
 
       /**
        * Like above, this method, which is only implemented for dim = 2 or 3,
@@ -1030,18 +1151,19 @@ namespace parallel
      * parallel::distributed::Triangulation objects throughout the library
      * even if it is disabled.
      *
-     * Since the constructor of this class is private, no such objects can
-     * actually be created if we don't have p4est available.
+     * Since the constructor of this class is deleted, no such objects
+     * can actually be created as this would be pointless given that
+     * p4est is not available.
      */
     template <int dim, int spacedim = dim>
     class Triangulation : public dealii::parallel::Triangulation<dim,spacedim>
     {
-    private:
+    public:
       /**
-       * Constructor.
+       * Constructor. Deleted to make sure that objects of this type cannot be
+       * constructed (see also the class documentation).
        */
-      Triangulation ();
-
+      Triangulation () = delete;
     };
   }
 }
