@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 2008 - 2016 by the deal.II authors
+// Copyright (C) 2008 - 2017 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -13,8 +13,8 @@
 //
 // ---------------------------------------------------------------------
 
-#ifndef dealii__distributed__tria_base_h
-#define dealii__distributed__tria_base_h
+#ifndef dealii_distributed_tria_base_h
+#define dealii_distributed_tria_base_h
 
 
 #include <deal.II/base/config.h>
@@ -24,9 +24,8 @@
 #include <deal.II/base/mpi.h>
 #include <deal.II/grid/tria.h>
 
-#include <deal.II/base/std_cxx1x/function.h>
-#include <deal.II/base/std_cxx1x/tuple.h>
-
+#include <functional>
+#include <tuple>
 #include <set>
 #include <vector>
 #include <list>
@@ -152,6 +151,13 @@ namespace parallel
     const std::set<types::subdomain_id> &
     level_ghost_owners () const;
 
+    /**
+     * Return a map that, for each vertex, lists all the processors whose
+     * subdomains are adjacent to that vertex.
+     */
+    virtual std::map<unsigned int, std::set<dealii::types::subdomain_id> >
+    compute_vertices_with_ghost_neighbors () const;
+
   protected:
     /**
      * MPI communicator to be used for the triangulation. We create a unique
@@ -213,6 +219,11 @@ namespace parallel
      * Update the number_cache variable after mesh creation or refinement.
      */
     virtual void update_number_cache ();
+
+    /**
+     * Store MPI ranks of level ghost owners of this processor on all levels.
+     */
+    void fill_level_ghost_owners ();
   };
 
 } // namespace parallel
