@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 2009 - 2016 by the deal.II authors
+// Copyright (C) 2009 - 2017 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -13,8 +13,8 @@
 //
 // ---------------------------------------------------------------------
 
-#ifndef dealii__block_info_h
-#define dealii__block_info_h
+#ifndef dealii_block_info_h
+#define dealii_block_info_h
 
 #include <deal.II/base/subscriptor.h>
 #include <deal.II/base/memory_consumption.h>
