@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 1998 - 2017 by the deal.II authors
+// Copyright (C) 1998 - 2018 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -13,8 +13,8 @@
 //
 // ---------------------------------------------------------------------
 
-#ifndef dealii__dof_accessor_h
-#define dealii__dof_accessor_h
+#ifndef dealii_dof_accessor_h
+#define dealii_dof_accessor_h
 
 
 #include <deal.II/base/config.h>
@@ -37,12 +37,12 @@ template <int, int> class FiniteElement;
 
 namespace internal
 {
-  namespace DoFCellAccessor
+  namespace DoFCellAccessorImplementation
   {
     struct Implementation;
   }
 
-  namespace DoFHandler
+  namespace DoFHandlerImplementation
   {
     struct Implementation;
     namespace Policy
@@ -53,7 +53,7 @@ namespace internal
 
   namespace hp
   {
-    namespace DoFHandler
+    namespace DoFHandlerImplementation
     {
       struct Implementation;
     }
@@ -68,7 +68,7 @@ namespace internal
 
 namespace internal
 {
-  namespace DoFAccessor
+  namespace DoFAccessorImplementation
   {
     /**
      * This is a switch class which only declares a @p typedef. It is meant to
@@ -198,7 +198,7 @@ namespace internal
  * 2012, 2013
  */
 template <int structdim, typename DoFHandlerType, bool level_dof_access>
-class DoFAccessor : public dealii::internal::DoFAccessor::Inheritance<structdim, DoFHandlerType::dimension, DoFHandlerType::space_dimension>::BaseClass
+class DoFAccessor : public dealii::internal::DoFAccessorImplementation::Inheritance<structdim, DoFHandlerType::dimension, DoFHandlerType::space_dimension>::BaseClass
 {
 public:
 
@@ -219,7 +219,7 @@ public:
    * exception classes simpler.
    */
   typedef
-  typename dealii::internal::DoFAccessor::Inheritance<structdim, dimension, space_dimension>::BaseClass
+  typename dealii::internal::DoFAccessorImplementation::Inheritance<structdim, dimension, space_dimension>::BaseClass
   BaseClass;
 
   /**
@@ -287,6 +287,18 @@ public:
    */
   template <bool level_dof_access2>
   DoFAccessor(const DoFAccessor<structdim, DoFHandlerType, level_dof_access2> &);
+
+  /**
+   * Copy operator. These operators are usually used in a context like <tt>iterator a,b;
+   * *a=*b;</tt>. Presumably, the intent here is to copy the object pointed to
+   * by @p b to the object pointed to by @p a. However, the result of
+   * dereferencing an iterator is not an object but an accessor; consequently,
+   * this operation is not useful for iterators on DoF handler objects.
+   * Consequently, this operator is declared as deleted and can not be used.
+   */
+  DoFAccessor<structdim,DoFHandlerType, level_dof_access> &
+  operator = (const DoFAccessor<structdim,DoFHandlerType, level_dof_access> &da) = delete;
+
   /**
    * @}
    */
@@ -333,7 +345,7 @@ public:
    * a line itself, then the only valid index is @p i equals to zero, and the
    * function returns an iterator to itself.
    */
-  typename dealii::internal::DoFHandler::Iterators<DoFHandlerType, level_dof_access>::line_iterator
+  typename dealii::internal::DoFHandlerImplementation::Iterators<DoFHandlerType, level_dof_access>::line_iterator
   line (const unsigned int i) const;
 
   /**
@@ -341,7 +353,7 @@ public:
    * a quad itself, then the only valid index is @p i equals to zero, and the
    * function returns an iterator to itself.
    */
-  typename dealii::internal::DoFHandler::Iterators<DoFHandlerType, level_dof_access>::quad_iterator
+  typename dealii::internal::DoFHandlerImplementation::Iterators<DoFHandlerType, level_dof_access>::quad_iterator
   quad (const unsigned int i) const;
 
   /**
@@ -540,7 +552,8 @@ public:
    *
    * @ingroup Exceptions
    */
-  DeclException0 (ExcInvalidObject);
+  DeclExceptionMsg (ExcInvalidObject, "This accessor object has not been "
+                    "associated with any DoFHandler object.");
   /**
    * Exception
    *
@@ -625,10 +638,9 @@ protected:
    * a cell object, there can only be a single set of degrees of freedom, and
    * fe_index has to match the result of active_fe_index().
    */
-  void set_dof_index
-  (const unsigned int i,
-   const types::global_dof_index index,
-   const unsigned int fe_index = DoFHandlerType::default_fe_index) const;
+  void set_dof_index (const unsigned int i,
+                      const types::global_dof_index index,
+                      const unsigned int fe_index = DoFHandlerType::default_fe_index) const;
 
   void set_mg_dof_index (const int level, const unsigned int i, const types::global_dof_index index) const;
 
@@ -670,18 +682,6 @@ protected:
   template <int, class, bool> friend class DoFAccessor;
 
 private:
-  /**
-   * Copy operator. This is normally used in a context like <tt>iterator a,b;
-   * *a=*b;</tt>. Presumably, the intent here is to copy the object pointed to
-   * by @p b to the object pointed to by @p a. However, the result of
-   * dereferencing an iterator is not an object but an accessor; consequently,
-   * this operation is not useful for iterators on triangulations. We declare
-   * this function here private, thus it may not be used from outside.
-   * Furthermore it is not implemented and will give a linker error if used
-   * anyway.
-   */
-  DoFAccessor<structdim,DoFHandlerType, level_dof_access> &
-  operator = (const DoFAccessor<structdim,DoFHandlerType, level_dof_access> &da);
 
   /**
    * Make the DoFHandler class a friend so that it can call the set_xxx()
@@ -690,11 +690,11 @@ private:
   template <int dim, int spacedim> friend class DoFHandler;
   template <int dim, int spacedim> friend class hp::DoFHandler;
 
-  friend struct dealii::internal::DoFHandler::Policy::Implementation;
-  friend struct dealii::internal::DoFHandler::Implementation;
-  friend struct dealii::internal::hp::DoFHandler::Implementation;
-  friend struct dealii::internal::DoFCellAccessor::Implementation;
-  friend struct dealii::internal::DoFAccessor::Implementation;
+  friend struct dealii::internal::DoFHandlerImplementation::Policy::Implementation;
+  friend struct dealii::internal::DoFHandlerImplementation::Implementation;
+  friend struct dealii::internal::hp::DoFHandlerImplementation::Implementation;
+  friend struct dealii::internal::DoFCellAccessorImplementation::Implementation;
+  friend struct dealii::internal::DoFAccessorImplementation::Implementation;
 };
 
 
@@ -801,6 +801,17 @@ public:
   template <int dim2, class DoFHandlerType2, bool level_dof_access2>
   DoFAccessor (const DoFAccessor<dim2, DoFHandlerType2, level_dof_access2> &);
 
+  /**
+   * Copy operator. These operators are usually used in a context like <tt>iterator a,b;
+   * *a=*b;</tt>. Presumably, the intent here is to copy the object pointed to
+   * by @p b to the object pointed to by @p a. However, the result of
+   * dereferencing an iterator is not an object but an accessor; consequently,
+   * this operation is not useful for iterators on DoF handler objects.
+   * Consequently, this operator is declared as deleted and can not be used.
+   */
+  DoFAccessor<0,DoFHandlerType<1,spacedim>, level_dof_access> &
+  operator = (const DoFAccessor<0,DoFHandlerType<1,spacedim>, level_dof_access> &da) = delete;
+
   /**
    * @}
    */
@@ -811,12 +822,6 @@ public:
   const DoFHandlerType<1,spacedim> &
   get_dof_handler () const;
 
-  /**
-   * Copy operator.
-   */
-  DoFAccessor<0,DoFHandlerType<1,spacedim>, level_dof_access> &
-  operator = (const DoFAccessor<0,DoFHandlerType<1,spacedim>, level_dof_access> &da);
-
   /**
    * Implement the copy operator needed for the iterator classes.
    */
@@ -850,7 +855,7 @@ public:
    * Since meshes with dimension 1 do not have quads this method just throws
    * an exception.
    */
-  typename dealii::internal::DoFHandler::Iterators<DoFHandlerType<1,spacedim>, level_dof_access>::line_iterator
+  typename dealii::internal::DoFHandlerImplementation::Iterators<DoFHandlerType<1,spacedim>, level_dof_access>::line_iterator
   line (const unsigned int i) const;
 
   /**
@@ -859,7 +864,7 @@ public:
    * Since meshes with dimension 1 do not have quads this method just throws
    * an exception.
    */
-  typename dealii::internal::DoFHandler::Iterators<DoFHandlerType<1,spacedim>, level_dof_access>::quad_iterator
+  typename dealii::internal::DoFHandlerImplementation::Iterators<DoFHandlerType<1,spacedim>, level_dof_access>::quad_iterator
   quad (const unsigned int i) const;
 
   /**
@@ -1014,7 +1019,8 @@ public:
    *
    * @ingroup Exceptions
    */
-  DeclException0 (ExcInvalidObject);
+  DeclExceptionMsg (ExcInvalidObject, "This accessor object has not been "
+                    "associated with any DoFHandler object.");
   /**
    * Exception
    *
@@ -1129,13 +1135,88 @@ protected:
   template <int, int> friend class DoFHandler;
   template <int, int> friend class hp::DoFHandler;
 
-  friend struct dealii::internal::DoFHandler::Policy::Implementation;
-  friend struct dealii::internal::DoFHandler::Implementation;
-  friend struct dealii::internal::hp::DoFHandler::Implementation;
-  friend struct dealii::internal::DoFCellAccessor::Implementation;
+  friend struct dealii::internal::DoFHandlerImplementation::Policy::Implementation;
+  friend struct dealii::internal::DoFHandlerImplementation::Implementation;
+  friend struct dealii::internal::hp::DoFHandlerImplementation::Implementation;
+  friend struct dealii::internal::DoFCellAccessorImplementation::Implementation;
 };
 
 
+
+/* -------------------------------------------------------------------------- */
+
+
+/**
+ * A class that represents DoF accessor objects to iterators that don't make sense
+ * such as quad iterators in on 1d meshes.  This class can not be used to
+ * create objects (it will in fact throw an exception if this should ever be
+ * attempted but it sometimes allows code to be written in a simpler way in a
+ * dimension independent way. For example, it allows to write code that works
+ * on quad iterators that is dimension independent -- i.e., also compiles
+ * in 1d -- because quad iterators
+ * (via the current class) exist and are syntactically correct. You can not
+ * expect, however, to ever create an actual object of one of these iterators
+ * in 1d, meaning you need to expect to wrap the code block in which you use
+ * quad iterators into something like <code>if (dim@>1)</code> -- which makes
+ * eminent sense anyway.
+ *
+ * This class provides the minimal interface necessary for Accessor classes to
+ * interact with Iterator classes. However, this is only for syntactic
+ * correctness, none of the functions do anything but generate errors.
+ *
+ * @ingroup Accessors
+ * @author Wolfgang Bangerth, 2017
+ */
+template <int structdim, int dim, int spacedim=dim>
+class DoFInvalidAccessor : public InvalidAccessor<structdim,dim,spacedim>
+{
+public:
+  /**
+   * Propagate typedef from base class to this class.
+   */
+  typedef typename InvalidAccessor<structdim,dim,spacedim>::AccessorData AccessorData;
+
+  /**
+   * Constructor.  This class is used for iterators that do not make
+   * sense in a given dimension, for example quads for 1d meshes. Consequently,
+   * while the creation of such objects is syntactically valid, they make no
+   * semantic sense, and we generate an exception when such an object is
+   * actually generated.
+   */
+  DoFInvalidAccessor (const Triangulation<dim,spacedim> *parent     =  0,
+                      const int                 level      = -1,
+                      const int                 index      = -1,
+                      const AccessorData       *local_data =  0);
+
+  /**
+   * Copy constructor.  This class is used for iterators that do not make
+   * sense in a given dimension, for example quads for 1d meshes. Consequently,
+   * while the creation of such objects is syntactically valid, they make no
+   * semantic sense, and we generate an exception when such an object is
+   * actually generated.
+   */
+  DoFInvalidAccessor (const DoFInvalidAccessor<structdim,dim,spacedim> &);
+
+  /**
+   * Conversion from other accessors to the current invalid one. This of
+   * course also leads to a run-time error.
+   */
+  template <typename OtherAccessor>
+  DoFInvalidAccessor (const OtherAccessor &);
+
+  /**
+   * Set the index of the <i>i</i>th degree of freedom of this object to @p
+   * index. Since the current object doesn't point to anything useful, like
+   * all other functions in this class this function only throws an exception.
+   */
+  void set_dof_index (const unsigned int i,
+                      const types::global_dof_index index,
+                      const unsigned int fe_index = DoFHandler<dim,spacedim>::default_fe_index) const;
+};
+
+
+
+
 /* -------------------------------------------------------------------------- */
 
 
@@ -1228,6 +1309,17 @@ public:
   explicit
   DoFCellAccessor (const DoFAccessor<dim2, DoFHandlerType2, level_dof_access2> &);
 
+  /**
+   * Copy operator. These operators are usually used in a context like <tt>iterator a,b;
+   * *a=*b;</tt>. Presumably, the intent here is to copy the object pointed to
+   * by @p b to the object pointed to by @p a. However, the result of
+   * dereferencing an iterator is not an object but an accessor; consequently,
+   * this operation is not useful for iterators on DoF handler objects.
+   * Consequently, this operator is declared as deleted and can not be used.
+   */
+  DoFCellAccessor<DoFHandlerType, level_dof_access> &
+  operator = (const DoFCellAccessor<DoFHandlerType, level_dof_access> &da) = delete;
+
   /**
    * @}
    */
@@ -1678,6 +1770,15 @@ public:
    * indices on non-active cells since they do not have finite element spaces
    * associated with them without having any degrees of freedom. Consequently,
    * this function will produce an exception when called on non-active cells.
+   *
+   * @note When using parallel meshes, either through the
+   * parallel::shared::Triangulation or parallel::distributed::Triangulation
+   * classes, it is only allowed to call this function on locally
+   * owned or ghost cells. No information is available on artificial cells.
+   * Furthermore, @p active_fe_index information is exchanged from locally
+   * owned cells on one processor to other processors where they may be
+   * ghost cells, during the call to hp::DoFHandler::distribute_dofs().
+   * See the documentation of hp::DoFHandler for more information.
    */
   unsigned int active_fe_index () const;
 
@@ -1693,8 +1794,22 @@ public:
    * indices to non-active cells since they do not have finite element spaces
    * associated with them without having any degrees of freedom. Consequently,
    * this function will produce an exception when called on non-active cells.
+   *
+   * @note When using parallel meshes, either through the
+   * parallel::shared::Triangulation or parallel::distributed::Triangulation
+   * classes, it is only allowed to call this function on locally
+   * owned cells (see
+   * @ref GlossLocallyOwnedCell "this glossary entry").
+   * This
+   * is because otherwise a common source of errors would be if one
+   * processor sets a different @p active_fe_index on a ghost cell than
+   * the processor that actually owns the cell does. To avoid this mistake,
+   * one can only set @p active_fe_index information on locally owned
+   * cells, and this information is then mirrored to all processors that
+   * have this cell as a ghost cell -- see the documentation of the
+   * hp::DoFHandler class.
    */
-  void set_active_fe_index (const unsigned int i);
+  void set_active_fe_index (const unsigned int i) const;
   /**
    * @}
    */
@@ -1716,25 +1831,13 @@ public:
   void update_cell_dof_indices_cache () const;
 
 private:
-  /**
-   * Copy operator. This is normally used in a context like <tt>iterator a,b;
-   * *a=*b;</tt>. Presumably, the intent here is to copy the object pointed to
-   * by @p b to the object pointed to by @p a. However, the result of
-   * dereferencing an iterator is not an object but an accessor; consequently,
-   * this operation is not useful for iterators on triangulations. We declare
-   * this function here private, thus it may not be used from outside.
-   * Furthermore it is not implemented and will give a linker error if used
-   * anyway.
-   */
-  DoFCellAccessor<DoFHandlerType, level_dof_access> &
-  operator = (const DoFCellAccessor<DoFHandlerType, level_dof_access> &da);
 
   /**
    * Make the DoFHandler class a friend so that it can call the
    * update_cell_dof_indices_cache() function
    */
   template <int dim, int spacedim> friend class DoFHandler;
-  friend struct dealii::internal::DoFCellAccessor::Implementation;
+  friend struct dealii::internal::DoFCellAccessorImplementation::Implementation;
 };
 
 
@@ -1748,6 +1851,21 @@ DoFAccessor<sd, DoFHandlerType, level_dof_access>::is_level_cell()
 
 
 
+template <int structdim, int dim, int spacedim>
+template <typename OtherAccessor>
+DoFInvalidAccessor<structdim, dim, spacedim>::
+DoFInvalidAccessor (const OtherAccessor &)
+{
+  Assert (false,
+          ExcMessage ("You are attempting an illegal conversion between "
+                      "iterator/accessor types. The constructor you call "
+                      "only exists to make certain template constructs "
+                      "easier to write as dimension independent code but "
+                      "the conversion is not valid in the current context."));
+}
+
+
+
 DEAL_II_NAMESPACE_CLOSE
 
 // include more templates
