@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 2006 - 2015 by the deal.II authors
+// Copyright (C) 2006 - 2018 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -13,8 +13,8 @@
 //
 // ---------------------------------------------------------------------
 
-#ifndef dealii__dof_faces_h
-#define dealii__dof_faces_h
+#ifndef dealii_dof_faces_h
+#define dealii_dof_faces_h
 
 
 #include <deal.II/base/config.h>
@@ -32,7 +32,7 @@ namespace internal
    *
    * @ingroup dofs
    */
-  namespace DoFHandler
+  namespace DoFHandlerImplementation
   {
 
     /**
@@ -64,15 +64,15 @@ namespace internal
      *
      * @author Tobias Leicht, 2006
      */
-    template<int dim>
+    template <int dim>
     class DoFFaces
     {
+    public:
       /**
-       * Make the constructor private to prevent the use of this template,
-       * only the specializations should be used
+       * Constructor. This constructor is deleted to prevent the use of this template,
+       * as only the specializations should be used
        */
-    private:
-      DoFFaces();
+      DoFFaces() = delete;
     };
 
     /**
@@ -81,7 +81,7 @@ namespace internal
      *
      * @author Tobias Leicht, 2006
      */
-    template<>
+    template <>
     class DoFFaces<1>
     {
     public:
@@ -106,14 +106,14 @@ namespace internal
      *
      * @author Tobias Leicht, 2006
      */
-    template<>
+    template <>
     class DoFFaces<2>
     {
     public:
       /**
        * The object containing the data of DoFs on lines.
        */
-      internal::DoFHandler::DoFObjects<1> lines;
+      internal::DoFHandlerImplementation::DoFObjects<1> lines;
 
       /**
        * Determine an estimate for the memory consumption (in bytes) of this
@@ -136,19 +136,19 @@ namespace internal
      *
      * @author Tobias Leicht, 2006
      */
-    template<>
+    template <>
     class DoFFaces<3>
     {
     public:
       /**
        * The object containing the data of DoFs on lines.
        */
-      internal::DoFHandler::DoFObjects<1> lines;
+      internal::DoFHandlerImplementation::DoFObjects<1> lines;
 
       /**
        * The object containing the data of DoFs on quads.
        */
-      internal::DoFHandler::DoFObjects<2> quads;
+      internal::DoFHandlerImplementation::DoFObjects<2> quads;
 
       /**
        * Determine an estimate for the memory consumption (in bytes) of this
