@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 1998 - 2017 by the deal.II authors
+// Copyright (C) 1998 - 2018 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -13,8 +13,8 @@
 //
 // ---------------------------------------------------------------------
 
-#ifndef dealii__dof_handler_h
-#define dealii__dof_handler_h
+#ifndef dealii_dof_handler_h
+#define dealii_dof_handler_h
 
 
 
@@ -23,19 +23,20 @@
 #include <deal.II/base/smartpointer.h>
 #include <deal.II/base/index_set.h>
 #include <deal.II/base/iterator_range.h>
-#include <deal.II/base/std_cxx11/shared_ptr.h>
 #include <deal.II/dofs/block_info.h>
 #include <deal.II/dofs/dof_iterator_selector.h>
 #include <deal.II/dofs/number_cache.h>
 #include <deal.II/dofs/dof_faces.h>
 #include <deal.II/dofs/dof_levels.h>
 #include <deal.II/dofs/function_map.h>
+#include <deal.II/hp/fe_collection.h>
 
 #include <boost/serialization/split_member.hpp>
 
 #include <vector>
 #include <map>
 #include <set>
+#include <memory>
 
 DEAL_II_NAMESPACE_OPEN
 
@@ -44,7 +45,7 @@ template <int dim, int spacedim> class Triangulation;
 
 namespace internal
 {
-  namespace DoFHandler
+  namespace DoFHandlerImplementation
   {
     struct Implementation;
 
@@ -55,12 +56,12 @@ namespace internal
     }
   }
 
-  namespace DoFAccessor
+  namespace DoFAccessorImplementation
   {
     struct Implementation;
   }
 
-  namespace DoFCellAccessor
+  namespace DoFCellAccessorImplementation
   {
     struct Implementation;
   }
@@ -87,7 +88,7 @@ namespace internal
  * matrices also refer to all degrees of freedom and some kind of condensation
  * is needed to restrict the systems of equations to the unconstrained degrees
  * of freedom only. The actual layout of storage of the indices is described
- * in the dealii::internal::DoFHandler::DoFLevel class documentation.
+ * in the dealii::internal::DoFHandlerImplementation::DoFLevel class documentation.
  *
  * The class offers iterators to traverse all cells, in much the same way as
  * the Triangulation class does. Using the begin() and end() functions (and
@@ -190,8 +191,8 @@ namespace internal
 template <int dim, int spacedim=dim>
 class DoFHandler  :  public Subscriptor
 {
-  typedef dealii::internal::DoFHandler::Iterators<DoFHandler<dim,spacedim>, false> ActiveSelector;
-  typedef dealii::internal::DoFHandler::Iterators<DoFHandler<dim,spacedim>, true> LevelSelector;
+  typedef dealii::internal::DoFHandlerImplementation::Iterators<DoFHandler<dim,spacedim>, false> ActiveSelector;
+  typedef dealii::internal::DoFHandlerImplementation::Iterators<DoFHandler<dim,spacedim>, true> LevelSelector;
 public:
   typedef typename ActiveSelector::CellAccessor         cell_accessor;
   typedef typename ActiveSelector::FaceAccessor         face_accessor;
@@ -255,7 +256,35 @@ public:
    */
   typedef typename ActiveSelector::cell_iterator        cell_iterator;
 
+  /**
+   * A typedef that is used to identify iterators that point to faces.
+   * The concept of iterators is discussed at length in the
+   * @ref Iterators "iterators documentation module".
+   *
+   * While the actual data type of the typedef is hidden behind a few layers
+   * of (unfortunately necessary) indirections, it is in essence
+   * TriaIterator<DoFAccessor>. The
+   * TriaIterator class works like a pointer to objects that when
+   * you dereference it yields an object of type DoFAccessor. DoFAccessor,
+   * in turn, is a class that can be used to query DoF indices on faces,
+   * but it is also derived from TriaAccessor and consequently can be used
+   * to query geometric properties such as vertices of faces, their area, etc.
+   *
+   * @ingroup Iterators
+   */
   typedef typename ActiveSelector::face_iterator        face_iterator;
+
+  /**
+   * A typedef that is used to identify iterators that point to active faces,
+   * i.e., to faces that have no children. Active faces must be faces of at
+   * least one active cell.
+   *
+   * Other than the "active" qualification, this typedef is identical to the
+   * @p face_iterator typedef. In particular, dereferencing either yields
+   * the same kind of object.
+   *
+   * @ingroup Iterators
+   */
   typedef typename ActiveSelector::active_face_iterator active_face_iterator;
 
   typedef typename LevelSelector::CellAccessor          level_cell_accessor;
@@ -283,7 +312,10 @@ public:
    *
    * Please note that you should not rely on it having a certain value, but
    * rather take its symbolic name.
+   *
+   * @deprecated Use numbers::invalid_dof_index instead.
    */
+  DEAL_II_DEPRECATED
   static const types::global_dof_index invalid_dof_index = numbers::invalid_dof_index;
 
   /**
@@ -307,13 +339,29 @@ public:
   /**
    * Constructor. Take @p tria as the triangulation to work on.
    */
-  DoFHandler ( const Triangulation<dim,spacedim> &tria);
+  DoFHandler (const Triangulation<dim,spacedim> &tria);
+
+  /**
+   * Copy constructor. DoFHandler objects are large and expensive.
+   * They should not be copied, in particular not by accident, but
+   * rather deliberately constructed. As a consequence, this constructor
+   * is explicitly removed from the interface of this class.
+   */
+  DoFHandler (const DoFHandler &) = delete;
 
   /**
    * Destructor.
    */
   virtual ~DoFHandler ();
 
+  /**
+   * Copy operator. DoFHandler objects are large and expensive.
+   * They should not be copied, in particular not by accident, but
+   * rather deliberately constructed. As a consequence, this operator
+   * is explicitly removed from the interface of this class.
+   */
+  DoFHandler &operator = (const DoFHandler &) = delete;
+
   /**
    * Assign a Triangulation and a FiniteElement to the DoFHandler and compute
    * the distribution of degrees of freedom over the mesh.
@@ -322,24 +370,42 @@ public:
                   const FiniteElement<dim,spacedim> &fe);
 
   /**
-   * Go through the triangulation and "distribute" the degrees of freedoms
-   * needed for the given finite element. "Distributing" degrees of freedom
-   * involved allocating memory to store the information that describes it
-   * (e.g., whether it is located on a vertex, edge, face, etc) and to
-   * sequentially enumerate all degrees of freedom. In other words, while the
-   * mesh and the finite element object by themselves simply define a finite
-   * element space $V_h$, the process of distributing degrees of freedom makes
-   * sure that there is a basis for this space and that the shape functions of
-   * this basis are enumerated in an indexable, predictable way.
+   * Go through the triangulation and "distribute" the degrees of
+   * freedoms needed for the given finite element. "Distributing"
+   * degrees of freedom involves allocating memory to store the
+   * indices on all entities on which degrees of freedom can be
+   * located (e.g., vertices, edges, faces, etc.) and to then enumerate
+   * all degrees of freedom. In other words, while the mesh and the
+   * finite element object by themselves simply define a finite
+   * element space $V_h$, the process of distributing degrees of
+   * freedom makes sure that there is a basis for this space and that
+   * the shape functions of this basis are enumerated in an indexable,
+   * predictable way.
    *
-   * The purpose of this function is first discussed in the introduction to
-   * the step-2 tutorial program.
+   * The exact order in which degrees of freedom on a mesh are
+   * ordered, i.e., the order in which basis functions of the finite
+   * element space are enumerated, is something that deal.II treats as
+   * an implementation detail. By and large, degrees of freedom are
+   * enumerated in the same order in which we traverse cells, but you
+   * should not rely on any specific numbering. In contrast, if you
+   * want a particular ordering, use the functions in namespace
+   * DoFRenumbering.
    *
-   * @note A pointer of the finite element given as argument is stored.
-   * Therefore, the lifetime of the finite element object shall be longer than
-   * that of this object. If you don't want this behavior, you may want to
-   * call the @p clear member function which also releases the lock of this
-   * object to the finite element.
+   * This function is first discussed in the introduction to the
+   * step-2 tutorial program.
+   *
+   * @note This function makes a copy of the finite element given as
+   * argument, and stores it as a member variable. Consequently, it is
+   * possible to write code such as
+   * @code
+   *   dof_handler.distribute_dofs (FE_Q<dim>(2));
+   * @endcode
+   * You can then access the finite element later on by calling
+   * DoFHandler::get_fe().  However, it is often more convenient to
+   * keep a named finite element object as a member variable in your
+   * main class and refer to it directly whenever you need to access
+   * properties of the finite element (such as FiniteElement::dofs_per_cell).
+   * This is what all tutorial programs do.
    */
   virtual void distribute_dofs (const FiniteElement<dim,spacedim> &fe);
 
@@ -348,9 +414,19 @@ public:
    * multigrid. The active DoFs need to be distributed using distribute_dofs()
    * before calling this function and the @p fe needs to be identical to the
    * finite element passed to distribute_dofs().
+   *
+   * @deprecated Use the version without parameter instead.
    */
+  DEAL_II_DEPRECATED
   virtual void distribute_mg_dofs (const FiniteElement<dim, spacedim> &fe);
 
+  /**
+   * Distribute level degrees of freedom on each level for geometric
+   * multigrid. The active DoFs need to be distributed using distribute_dofs()
+   * before calling this function.
+   */
+  virtual void distribute_mg_dofs ();
+
   /**
    * This function returns whether this DoFHandler has DoFs distributed on
    * each multigrid level or in other words if distribute_mg_dofs() has been
@@ -383,20 +459,17 @@ public:
   void initialize_local_block_info();
 
   /**
-   * Clear all data of this object and especially delete the lock this object
-   * has to the finite element used the last time when @p distribute_dofs was
-   * called.
+   * Clear all data of this object.
    */
   virtual void clear ();
 
   /**
-   * Renumber degrees of freedom based on a list of new dof numbers for all
-   * the dofs.
+   * Renumber degrees of freedom based on a list of new DoF indices for each
+   * of the degrees of freedom.
    *
    * This function is called by the functions in DoFRenumbering function after
-   * computing the ordering of the degrees of freedom. This function is
-   * called, for example, by the functions in the DoFRenumbering namespace,
-   * but it can of course also be called from user code.
+   * computing a new ordering of the degree of freedom indices. However, it
+   * can of course also be called from user code.
    *
    * @arg new_number This array must have a size equal to the number of
    * degrees of freedom owned by the current processor, i.e. the size must be
@@ -407,6 +480,7 @@ public:
    * IndexSet returned by locally_owned_dofs(). In the case of a sequential
    * mesh this means that the array is a list of new indices for each of the
    * degrees of freedom on the current mesh. In the case that we have a
+   * parallel::shared::Triangulation or
    * parallel::distributed::Triangulation underlying this DoFHandler object,
    * the array is a list of new indices for all the locally owned degrees of
    * freedom, enumerated in the same order as the currently locally owned
@@ -417,6 +491,29 @@ public:
    * locally_owned_dofs() is complete in the sequential case, the latter
    * convention for the content of the array reduces to the former in the case
    * that only one processor participates in the mesh.
+   *
+   * @note While it follows from the above, it may be surprising to know that
+   *   the <i>number</i> of locally owned degrees of freedom in a parallel
+   *   computation is an invariant
+   *   under renumbering, even if the <i>indices</i> associated with these
+   *   locally owned degrees of freedom are not. At a fundamental level,
+   *   this invariant exists because the <i>decision</i> whether a degree of
+   *   freedom is locally owned or not has nothing to do with that
+   *   degree of freedom's (old or new) index. Indeed, degrees of freedom
+   *   are locally owned if they are on a locally owned cell and not on
+   *   an interface between cells where the neighboring cell has a lower
+   *   @ref GlossSubdomainId "subdomain id". Since both of these conditions
+   *   are independent of the index associated with the DoF, a locally
+   *   owned degree of freedom will also be locally owned after renumbering.
+   *   On the other hand, properties such as whether the set of indices
+   *   of locally owned DoFs forms a contiguous range or not
+   *   (i.e., whether the locally_owned_dofs() returns an IndexSet object
+   *   for which IndexSet::is_contiguous() returns @p true) are of
+   *   course affected by the exact renumbering performed here. For example,
+   *   while the initial numbering of DoF indices done in distribute_dofs()
+   *   yields a contiguous numbering, the renumbering performed by
+   *   DoFRenumbering::component_wise() will, in general, not yield
+   *   contiguous locally owned DoF indices.
    */
   void renumber_dofs (const std::vector<types::global_dof_index> &new_numbers);
 
@@ -588,7 +685,6 @@ public:
    *       ...do the local integration on 'cell'...;
    *     }
    * @endcode
-   * To use this feature, you need a compiler that supports C++11.
    *
    * @return The half open range <code>[this->begin_active(),
    * this->end())</code>
@@ -685,6 +781,13 @@ public:
    * In either case, included in the returned number are those DoFs which are
    * constrained by hanging nodes, see
    * @ref constraints.
+   *
+   * Mathematically speaking, the number returned by this function equals the
+   * dimension of the finite element space (without taking into account
+   * constraints) that corresponds to (i) the mesh on which it is defined,
+   * and (ii) the finite element that is used by the current object. It
+   * also, of course, equals the number of shape functions that span this
+   * space.
    */
   types::global_dof_index n_dofs () const;
 
@@ -706,18 +809,21 @@ public:
    * Return the number of degrees of freedom located on those parts of the
    * boundary which have a boundary indicator listed in the given set. The
    * reason that a @p map rather than a @p set is used is the same as
-   * described in the section on the @p make_boundary_sparsity_pattern
-   * function.
+   * described in the documentation of that variant of
+   * DoFTools::make_boundary_sparsity_pattern() that takes a map.
+   * To this end, the type of the @p boundary_ids argument is the same
+   * as typename FunctionMap<spacedim,number>::type.
    *
-   * The type of boundary_ids equals typename FunctionMap<spacedim,number>::type .
+   * There is, however, another overload of this function that takes
+   * a @p set argument (see below).
    */
-  template<typename number>
+  template <typename number>
   types::global_dof_index
   n_boundary_dofs (const std::map<types::boundary_id, const Function<spacedim,number>*> &boundary_ids) const;
 
   /**
-   * Same function, but with different data type of the argument, which is
-   * here simply a list of the boundary indicators under consideration.
+   * Return the number of degrees of freedom located on those parts of the
+   * boundary which have a boundary indicator listed in the given set. The
    */
   types::global_dof_index
   n_boundary_dofs (const std::set<types::boundary_id> &boundary_ids) const;
@@ -743,9 +849,15 @@ public:
   /**
    * Return the number of degrees of freedom that belong to this process.
    *
-   * If this is a sequential job, then the result equals that produced by
-   * n_dofs(). On the other hand, if we are operating on a
-   * parallel::distributed::Triangulation, then it includes only the degrees
+   * If this is a sequential DoFHandler, then the result equals that produced by
+   * n_dofs(). (Here, "sequential" means that either
+   * the whole program does not use MPI, or that it uses MPI
+   * but only uses a single MPI process, or that there are multiple MPI
+   * processes but the Triangulation on which this DoFHandler builds
+   * works only on one MPI process.)
+   * On the other hand, if we are operating on a
+   * parallel::distributed::Triangulation or parallel::shared::Triangulation,
+   * then it includes only the degrees
    * of freedom that the current processor owns. Note that in this case this
    * does not include all degrees of freedom that have been distributed on the
    * current processor's image of the mesh: in particular, some of the degrees
@@ -768,21 +880,22 @@ public:
    */
   const IndexSet &locally_owned_mg_dofs(const unsigned int level) const;
 
-
   /**
    * Return a vector that stores the locally owned DoFs of each processor. If
    * you are only interested in the number of elements each processor owns
    * then n_locally_owned_dofs_per_processor() is a better choice.
    *
-   * If this is a sequential job, then the vector has a single element that
+   * If this is a sequential DoFHandler, then the vector has a single element that
    * equals the IndexSet representing the entire range [0,n_dofs()].
+   * (Here, "sequential" means that either
+   * the whole program does not use MPI, or that it uses MPI
+   * but only uses a single MPI process, or that there are multiple MPI
+   * processes but the Triangulation on which this DoFHandler builds
+   * works only on one MPI process.)
    */
   const std::vector<IndexSet> &
   locally_owned_dofs_per_processor () const;
 
-  const std::vector<IndexSet> &
-  locally_owned_mg_dofs_per_processor (const unsigned int level) const;
-
   /**
    * Return a vector that stores the number of degrees of freedom each
    * processor that participates in this triangulation owns locally. The sum
@@ -792,23 +905,46 @@ public:
    * Each element of the vector returned by this function equals the number of
    * elements of the corresponding sets returned by global_dof_indices().
    *
-   * If this is a sequential job, then the vector has a single element equal
-   * to n_dofs().
+   * If this is a sequential DoFHandler, then the vector has a single element equal
+   * to n_dofs(). (Here, "sequential" means that either
+   * the whole program does not use MPI, or that it uses MPI
+   * but only uses a single MPI process, or that there are multiple MPI
+   * processes but the Triangulation on which this DoFHandler builds
+   * works only on one MPI process.)
    */
   const std::vector<types::global_dof_index> &
   n_locally_owned_dofs_per_processor () const;
 
   /**
-   * Return a constant reference to the selected finite element object.
+   * Return a vector that stores the locally owned DoFs of each processor on
+   * the given level @p level.
+   *
+   * If this is a sequential DoFHandler, then the vector has a single element that
+   * equals the IndexSet representing the entire range [0,n_dofs()].
+   * (Here, "sequential" means that either
+   * the whole program does not use MPI, or that it uses MPI
+   * but only uses a single MPI process, or that there are multiple MPI
+   * processes but the Triangulation on which this DoFHandler builds
+   * works only on one MPI process.)
    */
-  const FiniteElement<dim,spacedim> &get_fe () const;
+  const std::vector<IndexSet> &
+  locally_owned_mg_dofs_per_processor (const unsigned int level) const;
 
   /**
-   * Return a constant reference to the triangulation underlying this object.
-   *
-   * @deprecated Use get_triangulation() instead.
+   * Return a constant reference to the selected finite element object.
+   * Since there is only one FiniteElement @p index must be equal to zero
+   * which is also the default value.
    */
-  const Triangulation<dim,spacedim> &get_tria () const DEAL_II_DEPRECATED;
+  const FiniteElement<dim,spacedim> &
+  get_fe (const unsigned int index=0) const;
+
+  /**
+    * Return a constant reference to the set of finite element objects that
+    * are used by this @p DoFHandler. Since this object only contains one
+    * FiniteElement, only this one object is returned wrapped in a
+    * hp::FECollection.
+    */
+  const hp::FECollection<dim,spacedim> &get_fe_collection () const;
 
   /**
    * Return a constant reference to the triangulation underlying this object.
@@ -841,13 +977,6 @@ public:
 
   BOOST_SERIALIZATION_SPLIT_MEMBER()
 
-  /**
-   * We are trying to renumber the degrees of freedom, but somehow did not
-   * count correctly.
-   *
-   * @ingroup Exceptions
-   */
-  DeclException0 (ExcRenumberingIncomplete);
   /**
    * Exception
    * @ingroup Exceptions
@@ -890,22 +1019,6 @@ public:
 
 
 private:
-  /**
-   * Copy constructor. I can see no reason why someone might want to use it,
-   * so I don't provide it. Since this class has pointer members, making it
-   * private prevents the compiler to provide it's own, incorrect one if
-   * anyone chose to copy such an object.
-   */
-  DoFHandler (const DoFHandler &);
-
-  /**
-   * Copy operator. I can see no reason why someone might want to use it, so I
-   * don't provide it. Since this class has pointer members, making it private
-   * prevents the compiler to provide it's own, incorrect one if anyone chose
-   * to copy such an object.
-   */
-  DoFHandler &operator = (const DoFHandler &);
-
 
   /**
    * An object containing information on the block structure.
@@ -918,22 +1031,18 @@ private:
   SmartPointer<const Triangulation<dim,spacedim>,DoFHandler<dim,spacedim> >
   tria;
 
+
   /**
-   * Store a pointer to the finite element given latest for the distribution
-   * of dofs. In order to avoid destruction of the object before the lifetime
-   * of the DoF handler, we subscribe to the finite element object. To unlock
-   * the FE before the end of the lifetime of this DoF handler, use the
-   * <tt>clear()</tt> function (this clears all data of this object as well,
-   * though).
+   * Store a hp::FECollection object containing the (one)
+   * FiniteElement this object is initialized with.
    */
-  SmartPointer<const FiniteElement<dim,spacedim>,DoFHandler<dim,spacedim> >
-  selected_fe;
+  hp::FECollection<dim,spacedim> fe_collection;
 
   /**
    * An object that describes how degrees of freedom should be distributed and
    * renumbered.
    */
-  std_cxx11::shared_ptr<dealii::internal::DoFHandler::Policy::PolicyBase<dim,spacedim> > policy;
+  std::unique_ptr<dealii::internal::DoFHandlerImplementation::Policy::PolicyBase<dim,spacedim> > policy;
 
   /**
    * A structure that contains all sorts of numbers that characterize the
@@ -942,12 +1051,12 @@ private:
    * For most members of this structure, there is an accessor function in this
    * class that returns its value.
    */
-  dealii::internal::DoFHandler::NumberCache number_cache;
+  dealii::internal::DoFHandlerImplementation::NumberCache number_cache;
 
   /**
    * Data structure like number_cache, but for each multigrid level.
    */
-  std::vector<dealii::internal::DoFHandler::NumberCache> mg_number_cache;
+  std::vector<dealii::internal::DoFHandlerImplementation::NumberCache> mg_number_cache;
 
   /**
    * A data structure that is used to store the DoF indices associated with a
@@ -963,11 +1072,6 @@ private:
      */
     MGVertexDoFs ();
 
-    /**
-     * Destructor.
-     */
-    ~MGVertexDoFs ();
-
     /**
      * A function that is called to allocate the necessary amount of memory to
      * store the indices of the DoFs that live on this vertex for the given
@@ -993,21 +1097,18 @@ private:
      */
     types::global_dof_index
     get_index (const unsigned int level,
-               const unsigned int dof_number) const;
+               const unsigned int dof_number,
+               const unsigned int dofs_per_vertex) const;
 
     /**
      * Set the index of the <code>dof_number</code>th degree of freedom for
      * the given level stored for the current vertex to <code>index</code>.
      */
-    void set_index (const unsigned int level,
-                    const unsigned int dof_number,
+    void set_index (const unsigned int            level,
+                    const unsigned int            dof_number,
+                    const unsigned int            dofs_per_vertex,
                     const types::global_dof_index index);
 
-    /**
-     * Exception.
-     */
-    DeclException0 (ExcNoMemory);
-
   private:
     /**
      * Coarsest level for which this object stores DoF indices.
@@ -1022,25 +1123,24 @@ private:
     /**
      * A pointer to an array where we store the indices of the DoFs that live
      * on the various levels this vertex exists on.
+     *
+     * The starting offset of the DoFs that belong to a @p level are given by
+     * <code>dofs_per_vertex * (level-coarsest_level)</code>. @p dofs_per_vertex
+     * must therefore be passed as an argument to the functions that set or
+     * read an index.
      */
-    types::global_dof_index *indices;
-
-    /**
-     * This array stores, for each level starting with coarsest_level, the
-     * offset in the <code>indices</code> array where the DoF indices for each
-     * level are stored.
-     */
-    types::global_dof_index *indices_offset;
+    std::unique_ptr<types::global_dof_index[]> indices;
   };
 
-  void clear_mg_space ();
-
   /**
-   * Free all used memory.
+   * Free all memory used for non-multigrid data structures.
    */
   void clear_space ();
 
-  void reserve_space ();
+  /**
+   * Free all memory used for multigrid data structures.
+   */
+  void clear_mg_space ();
 
   template <int structdim>
   types::global_dof_index get_dof_index (const unsigned int obj_level,
@@ -1048,7 +1148,7 @@ private:
                                          const unsigned int fe_index,
                                          const unsigned int local_index) const;
 
-  template<int structdim>
+  template <int structdim>
   void set_dof_index (const unsigned int obj_level,
                       const unsigned int obj_index,
                       const unsigned int fe_index,
@@ -1070,56 +1170,44 @@ private:
    * Space to store the DoF numbers for the different levels. Analogous to the
    * <tt>levels[]</tt> tree of the Triangulation objects.
    */
-  std::vector<dealii::internal::DoFHandler::DoFLevel<dim>*> levels;
+  std::vector<std::unique_ptr<dealii::internal::DoFHandlerImplementation::DoFLevel<dim> > > levels;
 
-  std::vector<dealii::internal::DoFHandler::DoFLevel<dim>*> mg_levels;
+  std::vector<std::unique_ptr<dealii::internal::DoFHandlerImplementation::DoFLevel<dim> > > mg_levels;
 
   /**
    * Space to store DoF numbers of faces. They are not stored in
    * <tt>levels</tt> since faces are not organized hierarchically, but in a
    * flat array.
    */
-  dealii::internal::DoFHandler::DoFFaces<dim> *faces;
+  std::unique_ptr<dealii::internal::DoFHandlerImplementation::DoFFaces<dim> > faces;
 
-  dealii::internal::DoFHandler::DoFFaces<dim> *mg_faces;
+  std::unique_ptr<dealii::internal::DoFHandlerImplementation::DoFFaces<dim> > mg_faces;
 
   /**
    * Make accessor objects friends.
    */
   template <int, class, bool> friend class DoFAccessor;
   template <class, bool> friend class DoFCellAccessor;
-  friend struct dealii::internal::DoFAccessor::Implementation;
-  friend struct dealii::internal::DoFCellAccessor::Implementation;
+  friend struct dealii::internal::DoFAccessorImplementation::Implementation;
+  friend struct dealii::internal::DoFCellAccessorImplementation::Implementation;
 
-  friend struct dealii::internal::DoFHandler::Implementation;
-  friend struct dealii::internal::DoFHandler::Policy::Implementation;
+  friend struct dealii::internal::DoFHandlerImplementation::Implementation;
+  friend struct dealii::internal::DoFHandlerImplementation::Policy::Implementation;
 
   // explicitly check for sensible template arguments, but not on windows
   // because MSVC creates bogus warnings during normal compilation
-#ifdef DEAL_II_WITH_CXX11
 #ifndef DEAL_II_MSVC
   static_assert (dim<=spacedim,
                  "The dimension <dim> of a DoFHandler must be less than or "
                  "equal to the space dimension <spacedim> in which it lives.");
 #endif
-#endif
 };
 
 
 
 
-/* -------------- declaration of explicit specializations ------------- */
-
 #ifndef DOXYGEN
 
-template <> types::global_dof_index DoFHandler<1>::n_boundary_dofs () const;
-template <> types::global_dof_index DoFHandler<1>::n_boundary_dofs (const std::set<types::boundary_id> &) const;
-
-template <> void DoFHandler<1>::renumber_dofs(unsigned int,const std::vector<types::global_dof_index>  &new_numbers);
-template <> void DoFHandler<2>::renumber_dofs(unsigned int,const std::vector<types::global_dof_index>  &new_numbers);
-template <> void DoFHandler<3>::renumber_dofs(unsigned int,const std::vector<types::global_dof_index>  &new_numbers);
-
-
 /* ----------------------- Inline functions ---------------------------------- */
 
 
@@ -1131,6 +1219,8 @@ DoFHandler<dim,spacedim>::has_level_dofs() const
   return mg_number_cache.size()>0;
 }
 
+
+
 template <int dim, int spacedim>
 inline
 bool
@@ -1139,6 +1229,8 @@ DoFHandler<dim,spacedim>::has_active_dofs() const
   return number_cache.n_global_dofs>0;
 }
 
+
+
 template <int dim, int spacedim>
 inline
 types::global_dof_index
@@ -1147,7 +1239,9 @@ DoFHandler<dim,spacedim>::n_dofs () const
   return number_cache.n_global_dofs;
 }
 
-template<int dim, int spacedim>
+
+
+template <int dim, int spacedim>
 inline
 types::global_dof_index DoFHandler<dim, spacedim>::n_dofs (const unsigned int level) const
 {
@@ -1157,6 +1251,7 @@ types::global_dof_index DoFHandler<dim, spacedim>::n_dofs (const unsigned int le
 }
 
 
+
 template <int dim, int spacedim>
 unsigned int
 DoFHandler<dim, spacedim>::n_locally_owned_dofs() const
@@ -1165,6 +1260,7 @@ DoFHandler<dim, spacedim>::n_locally_owned_dofs() const
 }
 
 
+
 template <int dim, int spacedim>
 const IndexSet &
 DoFHandler<dim, spacedim>::locally_owned_dofs() const
@@ -1172,14 +1268,19 @@ DoFHandler<dim, spacedim>::locally_owned_dofs() const
   return number_cache.locally_owned_dofs;
 }
 
+
+
 template <int dim, int spacedim>
 const IndexSet &
 DoFHandler<dim, spacedim>::locally_owned_mg_dofs(const unsigned int level) const
 {
-  Assert(level < this->get_triangulation().n_global_levels(), ExcMessage("invalid level in locally_owned_mg_dofs"));
+  Assert(level < this->get_triangulation().n_global_levels(),
+         ExcMessage("invalid level in locally_owned_mg_dofs"));
   return mg_number_cache[level].locally_owned_dofs;
 }
 
+
+
 template <int dim, int spacedim>
 const std::vector<types::global_dof_index> &
 DoFHandler<dim, spacedim>::n_locally_owned_dofs_per_processor() const
@@ -1188,6 +1289,7 @@ DoFHandler<dim, spacedim>::n_locally_owned_dofs_per_processor() const
 }
 
 
+
 template <int dim, int spacedim>
 const std::vector<IndexSet> &
 DoFHandler<dim, spacedim>::locally_owned_dofs_per_processor () const
@@ -1195,6 +1297,8 @@ DoFHandler<dim, spacedim>::locally_owned_dofs_per_processor () const
   return number_cache.locally_owned_dofs_per_processor;
 }
 
+
+
 template <int dim, int spacedim>
 const std::vector<IndexSet> &
 DoFHandler<dim, spacedim>::locally_owned_mg_dofs_per_processor (const unsigned int level) const
@@ -1204,24 +1308,28 @@ DoFHandler<dim, spacedim>::locally_owned_mg_dofs_per_processor (const unsigned i
 }
 
 
+
 template <int dim, int spacedim>
 inline
 const FiniteElement<dim,spacedim> &
-DoFHandler<dim,spacedim>::get_fe () const
+DoFHandler<dim,spacedim>::get_fe
+(const unsigned int index) const
 {
-  Assert(selected_fe!=0, ExcMessage("You are trying to access the DoFHandler's FiniteElement object before it has been initialized."));
-  return *selected_fe;
+  (void) index;
+  Assert(index == 0, ExcMessage("There is only one FiniteElement stored. The index must be zero!"));
+  return get_fe_collection()[0];
 }
 
 
 
 template <int dim, int spacedim>
 inline
-const Triangulation<dim,spacedim> &
-DoFHandler<dim,spacedim>::get_tria () const
+const hp::FECollection<dim,spacedim> &
+DoFHandler<dim,spacedim>::get_fe_collection () const
 {
-  Assert(tria != 0, ExcNotInitialized());
-  return *tria;
+  Assert(fe_collection.size() > 0,
+         ExcMessage("You are trying to access the DoFHandler's FECollection object before it has been initialized."));
+  return fe_collection;
 }
 
 
@@ -1231,7 +1339,9 @@ inline
 const Triangulation<dim,spacedim> &
 DoFHandler<dim,spacedim>::get_triangulation () const
 {
-  Assert(tria != 0, ExcNotInitialized());
+  Assert(tria != nullptr,
+         ExcMessage("This DoFHandler object has not been associated "
+                    "with a triangulation."));
   return *tria;
 }
 
@@ -1246,6 +1356,7 @@ DoFHandler<dim,spacedim>::block_info () const
 }
 
 
+
 template <int dim, int spacedim>
 template <typename number>
 types::global_dof_index
@@ -1267,17 +1378,18 @@ DoFHandler<dim,spacedim>::n_boundary_dofs (const std::map<types::boundary_id, co
 namespace internal
 {
   /**
-   * returns a string representing the dynamic type of the given argument.
+   * Return a string representing the dynamic type of the given argument.
    * This is basically the same what typeid(...).name() does, but it turns out
    * this is broken on Intel 13+.
    *
    * Defined in dof_handler.cc.
    */
-  template<int dim, int spacedim>
-  std::string policy_to_string(const dealii::internal::DoFHandler::Policy::PolicyBase<dim,spacedim> &policy);
+  template <int dim, int spacedim>
+  std::string policy_to_string(const dealii::internal::DoFHandlerImplementation::Policy::PolicyBase<dim,spacedim> &policy);
 }
 
 
+
 template <int dim, int spacedim>
 template <class Archive>
 void DoFHandler<dim,spacedim>::save (Archive &ar,
@@ -1286,20 +1398,35 @@ void DoFHandler<dim,spacedim>::save (Archive &ar,
   ar &block_info_object;
   ar &vertex_dofs;
   ar &number_cache;
-  ar &levels;
-  ar &faces;
+
+  // some versions of gcc have trouble with loading vectors of
+  // std::unique_ptr objects because std::unique_ptr does not
+  // have a copy constructor. do it one level at a time
+  unsigned int n_levels = levels.size();
+  ar &n_levels;
+  for (unsigned int i = 0; i < levels.size(); ++i)
+    ar &levels[i];
+
+  // boost dereferences a nullptr when serializing a nullptr
+  // at least up to 1.65.1. This causes problems with clang-5.
+  // Therefore, work around it.
+  bool faces_is_nullptr = (faces.get()==nullptr);
+  ar &faces_is_nullptr;
+  if (!faces_is_nullptr)
+    ar &faces;
 
   // write out the number of triangulation cells and later check during
   // loading that this number is indeed correct; same with something that
   // identifies the FE and the policy
   unsigned int n_cells = tria->n_cells();
-  std::string  fe_name = selected_fe->get_name();
+  std::string  fe_name = this->get_fe(0).get_name();
   std::string  policy_name = internal::policy_to_string(*policy);
 
   ar &n_cells &fe_name &policy_name;
 }
 
 
+
 template <int dim, int spacedim>
 template <class Archive>
 void DoFHandler<dim,spacedim>::load (Archive &ar,
@@ -1313,14 +1440,27 @@ void DoFHandler<dim,spacedim>::load (Archive &ar,
   // pointer object still points to something useful, that object is not
   // destroyed and we end up with a memory leak. consequently, first delete
   // previous content before re-loading stuff
-  for (unsigned int i=0; i<levels.size(); ++i)
-    delete levels[i];
-  levels.resize (0);
-  delete faces;
-  faces = 0;
+  levels.clear();
+  faces.reset();
 
-  ar &levels;
-  ar &faces;
+  // some versions of gcc have trouble with loading vectors of
+  // std::unique_ptr objects because std::unique_ptr does not
+  // have a copy constructor. do it one level at a time
+  unsigned int size;
+  ar &size;
+  levels.resize(size);
+  for (unsigned int i = 0; i < levels.size(); ++i)
+    {
+      std::unique_ptr<internal::DoFHandlerImplementation::DoFLevel<dim>> level;
+      ar &level;
+      levels[i] = std::move(level);
+    }
+
+  //Workaround for nullptr, see in save().
+  bool faces_is_nullptr = true;
+  ar &faces_is_nullptr;
+  if (!faces_is_nullptr)
+    ar &faces;
 
   // these are the checks that correspond to the last block in the save()
   // function
@@ -1333,41 +1473,47 @@ void DoFHandler<dim,spacedim>::load (Archive &ar,
   AssertThrow (n_cells == tria->n_cells(),
                ExcMessage ("The object being loaded into does not match the triangulation "
                            "that has been stored previously."));
-  AssertThrow (fe_name == selected_fe->get_name(),
+  AssertThrow (fe_name == this->get_fe(0).get_name(),
                ExcMessage ("The finite element associated with this DoFHandler does not match "
                            "the one that was associated with the DoFHandler previously stored."));
   AssertThrow (policy_name == internal::policy_to_string(*policy),
-               ExcMessage (std::string ("The policy currently associated with this DoFHandler (")
+               ExcMessage ("The policy currently associated with this DoFHandler ("
                            + internal::policy_to_string(*policy)
-                           +std::string(") does not match the one that was associated with the "
-                                        "DoFHandler previously stored (")
+                           + ") does not match the one that was associated with the "
+                           "DoFHandler previously stored ("
                            + policy_name
                            + ")."));
 }
 
 
-template<int dim, int spacedim>
+
+
+template <int dim, int spacedim>
 inline
-types::global_dof_index DoFHandler<dim, spacedim>::MGVertexDoFs::get_index (
-  const unsigned int level,
-  const unsigned int dof_number) const
+types::global_dof_index
+DoFHandler<dim, spacedim>::MGVertexDoFs::get_index (const unsigned int level,
+                                                    const unsigned int dof_number,
+                                                    const unsigned int dofs_per_vertex) const
 {
   Assert ((level >= coarsest_level) && (level <= finest_level), ExcInvalidLevel (level));
-  return indices[indices_offset[level - coarsest_level] + dof_number];
+  return indices[dofs_per_vertex * (level - coarsest_level) + dof_number];
 }
 
 
-template<int dim, int spacedim>
+
+template <int dim, int spacedim>
 inline
-void DoFHandler<dim, spacedim>::MGVertexDoFs::set_index (
-  const unsigned int level,
-  const unsigned int dof_number,
-  const types::global_dof_index index)
+void
+DoFHandler<dim, spacedim>::MGVertexDoFs::set_index (const unsigned int            level,
+                                                    const unsigned int            dof_number,
+                                                    const unsigned int            dofs_per_vertex,
+                                                    const types::global_dof_index index)
 {
   Assert ((level >= coarsest_level) && (level <= finest_level), ExcInvalidLevel (level));
-  indices[indices_offset[level - coarsest_level] + dof_number] = index;
+  indices[dofs_per_vertex * (level - coarsest_level) + dof_number] = index;
 }
 
+
 #endif // DOXYGEN
 
 DEAL_II_NAMESPACE_CLOSE
