@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 1998 - 2015 by the deal.II authors
+// Copyright (C) 1998 - 2018 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -13,8 +13,8 @@
 //
 // ---------------------------------------------------------------------
 
-#ifndef dealii__dof_levels_h
-#define dealii__dof_levels_h
+#ifndef dealii_dof_levels_h
+#define dealii_dof_levels_h
 
 
 #include <deal.II/base/config.h>
@@ -29,7 +29,7 @@ DEAL_II_NAMESPACE_OPEN
 
 namespace internal
 {
-  namespace DoFHandler
+  namespace DoFHandlerImplementation
   {
 
 
@@ -51,7 +51,7 @@ namespace internal
      * The indices of degrees of freedom located on lower dimensional objects,
      * i.e. on lines for 2D and on quads and lines for 3D are treated
      * similarly than that on cells. However, these geometrical objects, which
-     * are called faces as a generalisation, are not organised in a
+     * are called faces as a generalization, are not organised in a
      * hierarchical structure of levels. Therefore, the degrees of freedom
      * located on these objects are stored in separate classes, namely the
      * <tt>DoFFaces</tt> classes.
@@ -122,7 +122,7 @@ namespace internal
               cell_dof_indices_cache.size(),
               ExcInternalError());
 
-      return &cell_dof_indices_cache[obj_index*dofs_per_cell];
+      return cell_dof_indices_cache.data()+(obj_index*dofs_per_cell);
     }
 
 
