@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 2006 - 2015 by the deal.II authors
+// Copyright (C) 2006 - 2018 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -13,8 +13,8 @@
 //
 // ---------------------------------------------------------------------
 
-#ifndef dealii__dof_objects_h
-#define dealii__dof_objects_h
+#ifndef dealii_dof_objects_h
+#define dealii_dof_objects_h
 
 
 #include <deal.II/base/config.h>
@@ -27,7 +27,7 @@ template <int, int> class DoFHandler;
 
 namespace internal
 {
-  namespace DoFHandler
+  namespace DoFHandlerImplementation
   {
     template <int> class DoFLevel;
     template <int> class DoFFaces;
