@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 2003 - 2016 by the deal.II authors
+// Copyright (C) 2003 - 2018 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -13,8 +13,8 @@
 //
 // ---------------------------------------------------------------------
 
-#ifndef dealii__dof_renumbering_h
-#define dealii__dof_renumbering_h
+#ifndef dealii_dof_renumbering_h
+#define dealii_dof_renumbering_h
 
 
 #include <deal.II/base/config.h>
@@ -29,7 +29,11 @@ DEAL_II_NAMESPACE_OPEN
 
 /**
  * Implementation of a number of renumbering algorithms for the degrees of
- * freedom on a triangulation.
+ * freedom on a triangulation. The functions in this namespace compute
+ * new indices for each degree of freedom of a DoFHandler or
+ * hp::DoFHandler object, and then call DoFHandler::renumber_dofs() or
+ * hp::DoFHandler::renumber_dofs().
+ *
  *
  * <h3>Cuthill-McKee like algorithms</h3>
  *
@@ -518,27 +522,58 @@ namespace DoFRenumbering
    * comparison of various algorithms in the documentation of the
    * DoFRenumbering namespace.
    *
+   * @param dof_handler The DoFHandler or hp::DoFHandler object to work on.
+   * @param reversed_numbering Whether to use the original Cuthill-McKee
+   *   algorithm, or to reverse the ordering.
+   * @param use_constraints Whether or not to use hanging node constraints in
+   *   determining the reordering of degrees of freedom.
+   * @param starting_indices A set of degrees of freedom that form the first
+   *   level of renumbered degrees of freedom. If the set is empty, then a
+   *   single starting entry is chosen automatically among those that have the
+   *   smallest number of others that couple with it.
+   *
+   * <h4> Operation in parallel </h4>
+   *
    * If the given DoFHandler uses a distributed triangulation (i.e., if
    * dof_handler.locally_owned() is not the complete index set), the
    * renumbering is performed on each processor's degrees of freedom
-   * individually, without any communication between processors.
+   * individually, without any communication between processors. In other
+   * words, the resulting renumbering is an attempt at minimizing the bandwidth
+   * of <i>each diagonal block of the matrix corresponding to one processor</i>
+   * separately, without making an attempt at minimizing the bandwidth of
+   * the global matrix. Furthermore, the renumbering reuses exactly the
+   * same set of DoF indices that each processor used before. In other words,
+   * if the previous numbering of DoFs on one processor used a contiguous
+   * range of DoF indices, then so will the DoFs on that processor after
+   * the renumbering, and they will occupy the same range. The same is true
+   * if the previous numbering of DoFs on a processor consisted of a number
+   * of index ranges or single indices: after renumbering, the locally owned
+   * DoFs on that processor will use the exact same indices, just in a
+   * different order.
    *
-   * @param dof_handler The DoFHandler or hp::DoFHandler object to work on.
-   * @param reversed_numbering Whether to use the original Cuthill-McKee
-   * algorithm, or to reverse the ordering.
-   * @param use_constraints Whether or not to use hanging node constraints in
-   * determining the reordering of degrees of freedom.
-   * @param starting_indices A set of degrees of freedom that form the first
-   * level of renumbered degrees of freedom. If the set is empty, then a
-   * single starting entry is chosen automatically among those that have the
-   * smallest number of others that couple with it. If the DoFHandler is built
-   * on a parallel triangulation, then on every processor, these starting
-   * indices need to be a (possibly empty) subset of the
-   * @ref GlossLocallyOwnedDof "locally owned degrees of freedom".
-   * These will then be used as starting indices for the local renumbering on
-   * the current processor. (In other words, this argument will in fact be
-   * different on every processor unless you pass an empty list as is the
-   * default.)
+   * In addition, if the DoFHandler is built on a parallel triangulation, then
+   * on every processor, the starting indices for renumbering need to be a
+   * (possibly empty) subset of the
+   * @ref GlossLocallyActiveDof "locally active degrees of freedom". In
+   * general, these starting indices will be different on each processor
+   * (unless of course you pass an empty list as is the default),
+   * and each processor will use them as starting indices for the local
+   * renumbering on that processor.
+   *
+   * The starting indices must be locally active degrees of freedom, but
+   * the function will only renumber the locally owned subset of the
+   * locally owned DoFs. The function accepts starting indices from the
+   * largest set of locally active degrees of freedom because a typical
+   * renumbering operation with this function starts with indices that
+   * are located on the boundary -- in the case of the current function,
+   * that would be the boundary between processor subdomains. Since the
+   * degrees of freedom that are located on subdomain interfaces may
+   * be owned by either one of the two processors that own the adjacent
+   * subdomains, it is not always easy to identify starting indices that
+   * are locally owned. On the other hand, all degrees of freedom on subdomain
+   * interfaces are locally active, and so the function accepts them as
+   * starting indices even though it can only renumber them on a given
+   * processor if they are also locally owned.
    */
   template <typename DoFHandlerType>
   void
@@ -550,8 +585,10 @@ namespace DoFRenumbering
 
   /**
    * Compute the renumbering vector needed by the Cuthill_McKee() function.
-   * Does not perform the renumbering on the DoFHandler dofs but returns the
-   * renumbering vector.
+   * This function does not perform the renumbering on the DoFHandler DoFs but
+   * only returns the renumbering vector.
+   *
+   * See the Cuthill_McKee() function for an explanation of the arguments.
    */
   template <typename DoFHandlerType>
   void
@@ -614,26 +651,17 @@ namespace DoFRenumbering
    * For finite elements with only one component, or a single non-primitive
    * base element, this function is the identity operation.
    */
-  template <int dim, int spacedim>
+  template <typename DoFHandlerType>
   void
-  component_wise (DoFHandler<dim,spacedim>        &dof_handler,
+  component_wise (DoFHandlerType                  &dof_handler,
                   const std::vector<unsigned int> &target_component
                   = std::vector<unsigned int>());
 
 
-  /**
-   * Sort the degrees of freedom by component. It does the same thing as the
-   * above function.
-   */
-  template <int dim>
-  void
-  component_wise (hp::DoFHandler<dim>             &dof_handler,
-                  const std::vector<unsigned int> &target_component = std::vector<unsigned int> ());
-
   /**
    * Sort the degrees of freedom by component. It does the same thing as the
    * above function, only that it does this for one single level of a
-   * multilevel discretization. The non-multigrid part of the the DoFHandler
+   * multilevel discretization. The non-multigrid part of the DoFHandler
    * is not touched.
    */
   template <typename DoFHandlerType>
@@ -648,13 +676,13 @@ namespace DoFRenumbering
    * Does not perform the renumbering on the DoFHandler dofs but returns the
    * renumbering vector.
    */
-  template <int dim, int spacedim, class ITERATOR, class ENDITERATOR>
+  template <int dim, int spacedim, typename CellIterator>
   types::global_dof_index
-  compute_component_wise (std::vector<types::global_dof_index> &new_dof_indices,
-                          const ITERATOR &start,
-                          const ENDITERATOR &end,
-                          const std::vector<unsigned int> &target_component,
-                          bool is_level_operation);
+  compute_component_wise (std::vector<types::global_dof_index>        &new_dof_indices,
+                          const CellIterator                          &start,
+                          const typename identity<CellIterator>::type &end,
+                          const std::vector<unsigned int>             &target_component,
+                          const bool                                   is_level_operation);
 
   /**
    * @}
@@ -679,7 +707,7 @@ namespace DoFRenumbering
   /**
    * Sort the degrees of freedom by vector block. It does the same thing as
    * the above function, only that it does this for one single level of a
-   * multilevel discretization. The non-multigrid part of the the DoFHandler
+   * multilevel discretization. The non-multigrid part of the DoFHandler
    * is not touched.
    */
   template <int dim, int spacedim>
@@ -728,10 +756,76 @@ namespace DoFRenumbering
    */
 
   /**
-   * Renumber the degrees cell by cell in hierarchical order (also known as
-   * z-order). The main usage is that this guarantees the same ordering
-   * independent of the number of processors involved in a parallel
-   * distributed computation.
+   * Renumber the degrees cell by cell by traversing the cells in
+   * @ref GlossZOrder "Z order".
+   *
+   * There are two reasons to use this function:
+   * - It produces a predictable ordering of degrees of freedom
+   *   that is independent of how exactly you arrived at a mesh.
+   *   In particular, in general the order of cells of a mesh
+   *   depends on the order in which cells were marked for
+   *   refinement and coarsening during the refinement cycles
+   *   the mesh has undergone. On the other hand, the z-order
+   *   of cells is independent of the mesh's history, and so yields a
+   *   predictable DoF numbering.
+   * - For meshes based on parallel::distributed::Triangulation,
+   *   the
+   *   @ref GlossLocallyOwnedCell "locally owned cells"
+   *   of
+   *   each MPI process are contiguous in Z order. That means that
+   *   numbering degrees of freedom by visiting cells in Z order yields
+   *   @ref GlossLocallyOwnedDof "locally owned DoF indices"
+   *   that consist
+   *   of contiguous ranges for each process. This is also true for the
+   *   default ordering of DoFs on such triangulations, but the default
+   *   ordering creates an enumeration that also depends on how many
+   *   processors participate in the mesh, whereas the one generated
+   *   by this function enumerates the degrees of freedom on a particular
+   *   cell with indices that will be the same regardless of how many
+   *   processes the mesh is split up between.
+   *
+   * For meshes based on parallel::shared::Triangulation, the situation is
+   * more complex. Here, the set of locally owned cells is determined by
+   * a partitioning algorithm (selected by passing an object of type
+   * parallel::shared::Triangulation::Settings to the constructor of the
+   * triangulation), and in general these partitioning algorithms may
+   * assign cells to
+   * @ref GlossSubdomainId "subdomains"
+   * based on
+   * decisions that may have nothing to do with the Z order. (Though it
+   * is possible to select these flags in a way so that partitioning
+   * uses the Z order.) As a consequence, the cells of one subdomain
+   * are not contiguous in Z order, and if one renumbered degrees of freedom
+   * based on the Z order of cells, one would generally end up with DoF
+   * indices that on each processor do not form a contiguous range.
+   * This is often inconvenient (for example, because PETSc cannot store
+   * vectors and matrices for which the locally owned set of indices
+   * is not contiguous), and consequently this function uses the following
+   * algorithm for parallel::shared::Triangulation objects:
+   * - It determines how many degrees of freedom each processor owns.
+   *   This is an invariant under renumbering, and consequently we can
+   *   use how many DoFs each processor owns at the beginning of the current
+   *   function. Let us call this number $n_P$ for processor $P$.
+   * - It determines for each processor a contiguous range of new
+   *   DoF indices $[b_P,e_P)$ so that $e_P-b_P=n_P$, $b_0=0$, and
+   *   $b_P=e_{P-1}$.
+   * - It traverses the <i>locally owned cells</i> in Z order and
+   *   renumbers the locally owned degrees of freedom on these cells
+   *   so that the new numbers fit within the interval $[b_P,e_P)$.
+   * In other words, the <i>locally owned degrees of freedom</i> on each
+   * processor are sorted according to the Z order of the locally
+   * owned cells they are on, but this property may not hold globally,
+   * across cells. This is because the partitioning algorithm may have
+   * decided that, for example, processor 0 owns a cell that comes
+   * <i>later</i> in Z order than one of the cells assigned to processor 1.
+   * On the other hand, the algorithm described above assigns the
+   * degrees of freedom on this cell <i>earlier</i> indices than
+   * all of the indices owned by processor 1.
+   *
+   * @note This function generates an ordering that is independent of the previous
+   * numbering of degrees of freedom. In other words, any information that may
+   * have been produced by a previous call to a renumbering function is
+   * ignored.
    */
   template <int dim>
   void
@@ -840,9 +934,9 @@ namespace DoFRenumbering
    * numbering is performed cell-wise, otherwise it is performed based on the
    * location of the support points.
    *
-   * The cells are sorted such that the centers of higher numbers are further
+   * The cells are sorted such that the centers of cells numbered higher are further
    * downstream with respect to the constant vector @p direction than the
-   * centers of lower numbers. Even if this yields a downstream numbering with
+   * centers of cells numbered lower. Even if this yields a downstream numbering with
    * respect to the flux on the edges for fairly general grids, this might not
    * be guaranteed for all meshes.
    *
@@ -861,48 +955,48 @@ namespace DoFRenumbering
    */
   template <typename DoFHandlerType>
   void
-  downstream (DoFHandlerType                               &dof_handler,
-              const Point<DoFHandlerType::space_dimension> &direction,
-              const bool                                    dof_wise_renumbering = false);
+  downstream (DoFHandlerType                                  &dof_handler,
+              const Tensor<1,DoFHandlerType::space_dimension> &direction,
+              const bool                                       dof_wise_renumbering = false);
 
 
   /**
    * Cell-wise downstream numbering with respect to a constant flow direction
-   * on one level. See the other function with the same name.
+   * on one level of a multigrid hierarchy. See the other function with the same name.
    */
   template <typename DoFHandlerType>
   void
-  downstream (DoFHandlerType                               &dof_handler,
-              const unsigned int                            level,
-              const Point<DoFHandlerType::space_dimension> &direction,
-              const bool                                    dof_wise_renumbering = false);
+  downstream (DoFHandlerType                                  &dof_handler,
+              const unsigned int                               level,
+              const Tensor<1,DoFHandlerType::space_dimension> &direction,
+              const bool                                       dof_wise_renumbering = false);
 
   /**
-   * Compute the renumbering vector needed by the downstream() function. Does
+   * Compute the set of renumbering indices needed by the downstream() function. Does
    * not perform the renumbering on the DoFHandler dofs but returns the
    * renumbering vector.
    */
   template <typename DoFHandlerType>
   void
-  compute_downstream (std::vector<types::global_dof_index>         &new_dof_indices,
-                      std::vector<types::global_dof_index>         &reverse,
-                      const DoFHandlerType                         &dof_handler,
-                      const Point<DoFHandlerType::space_dimension> &direction,
-                      const bool                                    dof_wise_renumbering);
+  compute_downstream (std::vector<types::global_dof_index>            &new_dof_indices,
+                      std::vector<types::global_dof_index>            &reverse,
+                      const DoFHandlerType                            &dof_handler,
+                      const Tensor<1,DoFHandlerType::space_dimension> &direction,
+                      const bool                                       dof_wise_renumbering);
 
   /**
-   * Compute the renumbering vector needed by the downstream() function. Does
+   * Compute the set of renumbering indices needed by the downstream() function. Does
    * not perform the renumbering on the DoFHandler dofs but returns the
    * renumbering vector.
    */
   template <typename DoFHandlerType>
   void
-  compute_downstream (std::vector<types::global_dof_index>         &new_dof_indices,
-                      std::vector<types::global_dof_index>         &reverse,
-                      const DoFHandlerType                         &dof_handler,
-                      const unsigned int                            level,
-                      const Point<DoFHandlerType::space_dimension> &direction,
-                      const bool                                    dof_wise_renumbering);
+  compute_downstream (std::vector<types::global_dof_index>            &new_dof_indices,
+                      std::vector<types::global_dof_index>            &reverse,
+                      const DoFHandlerType                            &dof_handler,
+                      const unsigned int                               level,
+                      const Tensor<1,DoFHandlerType::space_dimension> &direction,
+                      const bool                                       dof_wise_renumbering);
 
   /**
    * Cell-wise clockwise numbering.
@@ -919,8 +1013,8 @@ namespace DoFRenumbering
                 const bool                                    counter = false);
 
   /**
-   * Cell-wise clockwise numbering on one level. See the other function with
-   * the same name.
+   * Cell-wise clockwise numbering on one level of a multigrid
+   * hierarchy. See the other function with the same name.
    */
   template <typename DoFHandlerType>
   void
@@ -1049,9 +1143,13 @@ namespace DoFRenumbering
    * Renumber the degrees of freedom such that they are associated with the
    * subdomain id of the cells they are living on, i.e. first all degrees of
    * freedom that belong to cells with subdomain zero, then all with subdomain
-   * one, etc. This is useful when doing parallel computations after assigning
-   * subdomain ids using a partitioner (see the
-   * GridTools::partition_triangulation function for this).
+   * one, etc. This is useful when doing parallel computations with a standard
+   * Triangulation after assigning subdomain ids using a partitioner (see the
+   * GridTools::partition_triangulation function for this). Calling this
+   * function is unnecessary when using a parallel::shared::Triangulation or
+   * parallel::distributed::Triangulation, as the degrees of freedom are
+   * already enumerated according to the MPI process id. Therefore, if the
+   * underlying triangulation is of this type then an error will be thrown.
    *
    * Note that degrees of freedom associated with faces, edges, and vertices
    * may be associated with multiple subdomains if they are sitting on
@@ -1081,18 +1179,25 @@ namespace DoFRenumbering
    * @}
    */
 
+
+
   /**
    * Exception
    *
    * @ingroup Exceptions
    */
-  DeclException0 (ExcRenumberingIncomplete);
+  DeclExceptionMsg (ExcDoFHandlerNotInitialized,
+                    "The DoFHandler on which this function should work has not "
+                    "been initialized, i.e., it doesn't appear that DoF indices "
+                    "have been distributed on it.");
+
   /**
    * Exception
    *
    * @ingroup Exceptions
    */
   DeclException0 (ExcInvalidComponentOrder);
+
   /**
    * The function is only implemented for Discontinuous Galerkin Finite
    * elements.
@@ -1102,28 +1207,6 @@ namespace DoFRenumbering
   DeclException0 (ExcNotDGFEM);
 }
 
-/* ------------------------- inline functions -------------- */
-
-#ifndef DOXYGEN
-namespace DoFRenumbering
-{
-  template <typename DoFHandlerType>
-  void
-  inline
-  downstream (DoFHandlerType                               &dof,
-              const Point<DoFHandlerType::space_dimension> &direction,
-              const bool                                    dof_wise_renumbering)
-  {
-    std::vector<types::global_dof_index> renumbering(dof.n_dofs());
-    std::vector<types::global_dof_index> reverse(dof.n_dofs());
-    compute_downstream(renumbering, reverse, dof, direction,
-                       dof_wise_renumbering);
-
-    dof.renumber_dofs(renumbering);
-  }
-}
-#endif
-
 
 DEAL_II_NAMESPACE_CLOSE
 
