@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 2001 - 2016 by the deal.II authors
+// Copyright (C) 2001 - 2017 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -13,15 +13,15 @@
 //
 // ---------------------------------------------------------------------
 
-#ifndef dealii__function_map_h
-#define dealii__function_map_h
+#ifndef dealii_function_map_h
+#define dealii_function_map_h
 
 #include <deal.II/base/config.h>
 #include <map>
 
 DEAL_II_NAMESPACE_OPEN
 
-template <int spacedim, typename Number> class Function;
+template <int spacedim, typename RangeNumberType> class Function;
 
 
 
@@ -70,7 +70,7 @@ template <int spacedim, typename Number> class Function;
  * @ingroup functions
  * @author Wolfgang Bangerth, Ralf Hartmann, 2001
  */
-template<int dim,typename Number=double>
+template <int dim,typename Number=double>
 struct FunctionMap
 {
   /**
