@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 2009 - 2015 by the deal.II authors
+// Copyright (C) 2009 - 2017 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -13,8 +13,8 @@
 //
 // ---------------------------------------------------------------------
 
-#ifndef dealii__fe_block_mask_h
-#define dealii__fe_block_mask_h
+#ifndef dealii_fe_block_mask_h
+#define dealii_fe_block_mask_h
 
 #include <deal.II/base/config.h>
 #include <deal.II/base/exceptions.h>
@@ -77,7 +77,7 @@ public:
    * constructor results in a block mask that always returns <code>true</code>
    * whenever asked whether a block is selected.
    */
-  BlockMask ();
+  BlockMask () = default;
 
   /**
    * Initialize an object of this type with a set of selected blocks specified
@@ -230,11 +230,6 @@ std::ostream &operator << (std::ostream &out,
 
 // -------------------- inline functions ---------------------
 
-inline
-BlockMask::BlockMask()
-{}
-
-
 inline
 BlockMask::BlockMask(const std::vector<bool> &block_mask)
   :
