@@ -13,8 +13,8 @@
 //
 // ---------------------------------------------------------------------
 
-#ifndef dealii__fe_abf_h
-#define dealii__fe_abf_h
+#ifndef dealii_fe_abf_h
+#define dealii_fe_abf_h
 
 #include <deal.II/base/config.h>
 #include <deal.II/base/table.h>
@@ -126,21 +126,14 @@ public:
   // documentation inherited from the base class
   virtual
   void
-  convert_generalized_support_point_values_to_nodal_values (const std::vector<Vector<double> > &support_point_values,
-                                                            std::vector<double>                &nodal_values) const;
-
-  virtual void interpolate(std::vector<double>                &local_dofs,
-                           const std::vector<double>          &values) const DEAL_II_DEPRECATED;
-
-  virtual void interpolate(std::vector<double>                &local_dofs,
-                           const std::vector<Vector<double> > &values,
-                           const unsigned int                  offset = 0) const DEAL_II_DEPRECATED;
-
-  virtual void interpolate(std::vector<double> &local_dofs,
-                           const VectorSlice<const std::vector<std::vector<double> > > &values) const DEAL_II_DEPRECATED;
+  convert_generalized_support_point_values_to_dof_values (const std::vector<Vector<double> > &support_point_values,
+                                                          std::vector<double>                &nodal_values) const;
 
   virtual std::size_t memory_consumption () const;
-  virtual FiniteElement<dim> *clone() const;
+
+  virtual
+  std::unique_ptr<FiniteElement<dim,dim> >
+  clone() const;
 
 private:
   /**
