@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 2000 - 2016 by the deal.II authors
+// Copyright (C) 2000 - 2017 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -13,8 +13,8 @@
 //
 // ---------------------------------------------------------------------
 
-#ifndef dealii__fe_base_h
-#define dealii__fe_base_h
+#ifndef dealii_fe_base_h
+#define dealii_fe_base_h
 
 #include <deal.II/base/config.h>
 #include <deal.II/base/exceptions.h>
