@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 2000 - 2016 by the deal.II authors
+// Copyright (C) 2000 - 2017 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -13,8 +13,8 @@
 //
 // ---------------------------------------------------------------------
 
-#ifndef dealii__fe_bernstein_h
-#define dealii__fe_bernstein_h
+#ifndef dealii_fe_bernstein_h
+#define dealii_fe_bernstein_h
 
 #include <deal.II/base/config.h>
 #include <deal.II/base/tensor_product_polynomials.h>
@@ -75,7 +75,7 @@ public:
    * this element from defining an interpolation matrix. An exception will be
    * thrown.
    *
-   * Overrides the implementation from FE_Q_Base.
+   * This function overrides the implementation from FE_Q_Base.
    */
   virtual void
   get_interpolation_matrix (const FiniteElement<dim,spacedim> &source,
@@ -86,7 +86,7 @@ public:
    * this element from defining a restriction matrix. An exception will be
    * thrown.
    *
-   * Overrides the implementation from FE_Q_Base.
+   * This function overrides the implementation from FE_Q_Base.
    */
   virtual const FullMatrix<double> &
   get_restriction_matrix (const unsigned int child,
@@ -97,7 +97,7 @@ public:
    * this element from defining a prolongation matrix. An exception will be
    * thrown.
    *
-   * Overrides the implementation from FE_Q_Base.
+   * This function overrides the implementation from FE_Q_Base.
    */
   virtual const FullMatrix<double> &
   get_prolongation_matrix (const unsigned int child,
@@ -192,14 +192,11 @@ public:
    */
   virtual std::string get_name () const;
 
-protected:
+  virtual
+  std::unique_ptr<FiniteElement<dim,spacedim> >
+  clone() const;
 
-  /**
-   * @p clone function instead of a copy constructor.
-   *
-   * This function is needed by the constructors of @p FESystem.
-   */
-  virtual FiniteElement<dim,spacedim> *clone() const;
+protected:
 
   /**
    * Only for internal use. Its full name is @p get_dofs_per_object_vector
