@@ -13,8 +13,8 @@
 //
 // ---------------------------------------------------------------------
 
-#ifndef dealii__fe_dg_vector_h
-#define dealii__fe_dg_vector_h
+#ifndef dealii_fe_dg_vector_h
+#define dealii_fe_dg_vector_h
 
 #include <deal.II/base/config.h>
 #include <deal.II/base/table.h>
@@ -59,9 +59,6 @@ public:
    * Constructor for the vector element of degree @p p.
    */
   FE_DGVector (const unsigned int p, MappingType m);
-public:
-
-  FiniteElement<dim, spacedim> *clone() const;
 
   /**
    * Return a string that uniquely identifies a finite element. This class
@@ -70,6 +67,9 @@ public:
    */
   virtual std::string get_name () const;
 
+  virtual
+  std::unique_ptr<FiniteElement<dim,spacedim> >
+  clone() const;
 
   /**
    * This function returns @p true, if the shape function @p shape_index has
@@ -80,16 +80,6 @@ public:
   virtual bool has_support_on_face (const unsigned int shape_index,
                                     const unsigned int face_index) const;
 
-  virtual void interpolate(std::vector<double>                &local_dofs,
-                           const std::vector<double>          &values) const DEAL_II_DEPRECATED;
-
-  virtual void interpolate(std::vector<double>                &local_dofs,
-                           const std::vector<Vector<double> > &values,
-                           const unsigned int                  offset = 0) const DEAL_II_DEPRECATED;
-
-  virtual void interpolate(std::vector<double> &local_dofs,
-                           const VectorSlice<const std::vector<std::vector<double> > > &values) const DEAL_II_DEPRECATED;
-
   virtual std::size_t memory_consumption () const;
 
 private:
@@ -102,25 +92,6 @@ private:
   static std::vector<unsigned int>
   get_dpo_vector (const unsigned int degree);
 
-  /**
-   * Initialize the @p generalized_support_points field of the FiniteElement
-   * class and fill the tables with @p interior_weights. Called from the
-   * constructor.
-   *
-   * See the
-   * @ref GlossGeneralizedSupport "glossary entry on generalized support points"
-   * for more information.
-   */
-  void initialize_support_points (const unsigned int degree);
-
-  /**
-   * Initialize the interpolation from functions on refined mesh cells onto
-   * the father cell. According to the philosophy of the Raviart-Thomas
-   * element, this restriction operator preserves the divergence of a function
-   * weakly.
-   */
-  void initialize_restriction ();
-
   /**
    * Fields of cell-independent data.
    *
