@@ -13,14 +13,17 @@
 //
 // ---------------------------------------------------------------------
 
-#ifndef dealii__fe_dg_vector_templates_h
-#define dealii__fe_dg_vector_templates_h
+#ifndef dealii_fe_dg_vector_templates_h
+#define dealii_fe_dg_vector_templates_h
 
 
 #include <deal.II/fe/fe_dg_vector.h>
 #include <deal.II/fe/fe_tools.h>
 #include <deal.II/base/quadrature_lib.h>
 
+#include <deal.II/base/std_cxx14/memory.h>
+
+
 DEAL_II_NAMESPACE_OPEN
 
 
@@ -51,10 +54,10 @@ FE_DGVector<PolynomialType,dim,spacedim>::FE_DGVector (
 
 
 template <class PolynomialType, int dim, int spacedim>
-FiniteElement<dim, spacedim> *
+std::unique_ptr<FiniteElement<dim,spacedim> >
 FE_DGVector<PolynomialType,dim,spacedim>::clone() const
 {
-  return new FE_DGVector<PolynomialType, dim, spacedim>(*this);
+  return std_cxx14::make_unique<FE_DGVector<PolynomialType, dim, spacedim>>(*this);
 }
 
 
@@ -90,35 +93,6 @@ FE_DGVector<PolynomialType,dim,spacedim>::has_support_on_face
 }
 
 
-template <class PolynomialType, int dim, int spacedim>
-void
-FE_DGVector<PolynomialType,dim,spacedim>::interpolate
-(std::vector<double> &,
- const std::vector<double> &) const
-{
-  Assert(false, ExcNotImplemented());
-}
-
-
-template <class PolynomialType, int dim, int spacedim>
-void
-FE_DGVector<PolynomialType,dim,spacedim>::interpolate
-(std::vector<double> & /*local_dofs*/,
- const std::vector<Vector<double> > & /*values*/,
- const unsigned int /*offset*/) const
-{
-  Assert(false, ExcNotImplemented());
-}
-
-template <class PolynomialType, int dim, int spacedim>
-void
-FE_DGVector<PolynomialType,dim,spacedim>::interpolate
-(std::vector<double> & /*local_dofs*/,
- const VectorSlice<const std::vector<std::vector<double> > > & /*values*/) const
-{
-  Assert(false, ExcNotImplemented());
-}
-
 
 template <class PolynomialType, int dim, int spacedim>
 std::size_t
