@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 2004 - 2016 by the deal.II authors
+// Copyright (C) 2004 - 2018 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -13,8 +13,8 @@
 //
 // ---------------------------------------------------------------------
 
-#ifndef dealii__fe_dgp_monomial_h
-#define dealii__fe_dgp_monomial_h
+#ifndef dealii_fe_dgp_monomial_h
+#define dealii_fe_dgp_monomial_h
 
 #include <deal.II/base/config.h>
 #include <deal.II/base/polynomials_p.h>
@@ -386,7 +386,7 @@ public:
                             FullMatrix<double>           &matrix) const;
 
   /**
-   * Return the matrix interpolating from a face of of one element to the face
+   * Return the matrix interpolating from a face of one element to the face
    * of the neighboring element. The size of the matrix is then @p
    * dofs_per_face times <tt>source.dofs_per_face</tt>.
    *
@@ -401,7 +401,7 @@ public:
                                  FullMatrix<double>       &matrix) const;
 
   /**
-   * Return the matrix interpolating from a face of of one element to the face
+   * Return the matrix interpolating from a face of one element to the face
    * of the neighboring element. The size of the matrix is then @p
    * dofs_per_face times <tt>source.dofs_per_face</tt>.
    *
@@ -433,14 +433,9 @@ public:
    */
   virtual std::size_t memory_consumption () const;
 
-protected:
-
-  /**
-   * @p clone function instead of a copy constructor.
-   *
-   * This function is needed by the constructors of @p FESystem.
-   */
-  virtual FiniteElement<dim> *clone() const;
+  virtual
+  std::unique_ptr<FiniteElement<dim,dim> >
+  clone() const;
 
 private:
 
@@ -452,11 +447,6 @@ private:
    */
   static std::vector<unsigned int> get_dpo_vector (const unsigned int degree);
 
-  /**
-   * Initialize the embedding matrices. Called from the constructor.
-   */
-  void initialize_embedding ();
-
   /**
    * Initialize the restriction matrices. Called from the constructor.
    */
