@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 2001 - 2017 by the deal.II authors
+// Copyright (C) 2001 - 2018 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -13,8 +13,8 @@
 //
 // ---------------------------------------------------------------------
 
-#ifndef dealii__fe_dgq_h
-#define dealii__fe_dgq_h
+#ifndef dealii_fe_dgq_h
+#define dealii_fe_dgq_h
 
 #include <deal.II/base/config.h>
 #include <deal.II/base/tensor_product_polynomials.h>
@@ -134,7 +134,7 @@ public:
                             FullMatrix<double>           &matrix) const;
 
   /**
-   * Return the matrix interpolating from a face of of one element to the face
+   * Return the matrix interpolating from a face of one element to the face
    * of the neighboring element. The size of the matrix is then
    * <tt>source.dofs_per_face</tt> times <tt>this->dofs_per_face</tt>.
    *
@@ -149,7 +149,7 @@ public:
                                  FullMatrix<double>       &matrix) const;
 
   /**
-   * Return the matrix interpolating from a face of of one element to the face
+   * Return the matrix interpolating from a face of one element to the face
    * of the neighboring element. The size of the matrix is then
    * <tt>source.dofs_per_face</tt> times <tt>this->dofs_per_face</tt>.
    *
@@ -194,7 +194,7 @@ public:
    * single child cell is returned here.
    *
    * The matrix @p P is the concatenation, not the sum of the cell matrices @p
-   * P_i. That is, if the same non-zero entry <tt>j,k</tt> exists in in two
+   * P_i. That is, if the same non-zero entry <tt>j,k</tt> exists in two
    * different child matrices @p P_i, the value should be the same in both
    * matrices and it is copied into the matrix @p P only once.
    *
@@ -300,6 +300,18 @@ public:
   virtual std::pair<Table<2,bool>, std::vector<unsigned int> >
   get_constant_modes () const;
 
+  /**
+   * Implementation of the corresponding function in the FiniteElement
+   * class.  Since the current element is interpolatory, the nodal
+   * values are exactly the support point values. Furthermore, since
+   * the current element is scalar, the support point values need to
+   * be vectors of length 1.
+   */
+  virtual
+  void
+  convert_generalized_support_point_values_to_dof_values (const std::vector<Vector<double> > &support_point_values,
+                                                          std::vector<double>                &nodal_values) const;
+
   /**
    * Determine an estimate for the memory consumption (in bytes) of this
    * object.
@@ -310,6 +322,9 @@ public:
    */
   virtual std::size_t memory_consumption () const;
 
+  virtual
+  std::unique_ptr<FiniteElement<dim,spacedim> >
+  clone() const;
 
 protected:
   /**
@@ -322,13 +337,6 @@ protected:
    */
   FE_DGQ (const std::vector<Polynomials::Polynomial<double> > &polynomials);
 
-  /**
-   * @p clone function instead of a copy constructor.
-   *
-   * This function is needed by the constructors of @p FESystem.
-   */
-  virtual FiniteElement<dim, spacedim> *clone() const;
-
 private:
   /**
    * Only for internal use. Its full name is @p get_dofs_per_object_vector
@@ -341,9 +349,10 @@ private:
   /**
    * Compute renumbering for rotation of degrees of freedom.
    *
-   * Rotates a tensor product numbering of degrees of freedom by 90 degrees.
-   * It is used to compute the transfer matrices of the children by using only
-   * the matrix for the first child.
+   * This function rotates a tensor product numbering of degrees of
+   * freedom by 90 degrees.  It is used to compute the transfer
+   * matrices of the children by using only the matrix for the first
+   * child.
    *
    * The direction parameter determines the type of rotation. It is one
    * character of @p xXyYzZ. The character determines the axis of rotation,
@@ -367,7 +376,7 @@ private:
   template <int dim1, int spacedim1> friend class FE_DGQ;
 
   /**
-   * Allows @p MappingQ class to access to build_renumbering function.
+   * Allow @p MappingQ class to access to build_renumbering function.
    */
   template <int dim1, int spacedim1> friend class MappingQ;
 };
@@ -409,13 +418,20 @@ public:
    */
   virtual std::string get_name () const;
 
-protected:
   /**
-   * @p clone function instead of a copy constructor.
-   *
-   * This function is needed by the constructors of @p FESystem.
+   * Implementation of the corresponding function in the FiniteElement
+   * class.  Since the current element is interpolatory, the nodal
+   * values are exactly the support point values. Furthermore, since
+   * the current element is scalar, the support point values need to
+   * be vectors of length 1.
    */
-  virtual FiniteElement<dim,spacedim> *clone() const;
+  virtual
+  void
+  convert_generalized_support_point_values_to_dof_values (const std::vector<Vector<double> > &support_point_values,
+                                                          std::vector<double>                &nodal_values) const;
+  virtual
+  std::unique_ptr<FiniteElement<dim,spacedim> >
+  clone() const;
 };
 
 
@@ -456,29 +472,29 @@ public:
    */
   virtual std::string get_name () const;
 
-protected:
-  /**
-   * @p clone function instead of a copy constructor.
-   *
-   * This function is needed by the constructors of @p FESystem.
-   */
-  virtual FiniteElement<dim,spacedim> *clone() const;
+  virtual
+  std::unique_ptr<FiniteElement<dim,spacedim> >
+  clone() const;
 };
 
 
 
 /**
  * Implementation of scalar, discontinuous tensor product elements based on
- * Hermite polynomials, described by the polynomial space
- * Polynomials::HermiteInterpolation. As opposed to the basic FE_DGQ element,
+ * Hermite-like polynomials, described by the polynomial space
+ * Polynomials::HermiteLikeInterpolation. As opposed to the basic FE_DGQ element,
  * these elements are not interpolatory and no support points are defined.
  *
- * This element is only a Hermite polynomials for degrees larger or equal to
- * three. For degrees zero to two, a usual Lagrange basis is selected.
+ * Note that Hermite polynomials are only available for degrees larger or
+ * equal to three, and thus the beneficial properties of
+ * Polynomials::HermiteLikeInterpolation with only two basis functions having
+ * a non-trivial value or derivative on a face per dimension is only present
+ * for higher degrees. To facilitate usage also for degrees zero to two, a
+ * usual Lagrange basis is constructed by this class.
  *
  * See the base class documentation in FE_DGQ for details.
  *
- * @author Martin Kronbichler, 2017
+ * @author Martin Kronbichler, 2017, 2018
  */
 template <int dim,int spacedim=dim>
 class FE_DGQHermite : public FE_DGQ<dim,spacedim>
@@ -486,19 +502,10 @@ class FE_DGQHermite : public FE_DGQ<dim,spacedim>
 public:
   /**
    * Constructor for tensor product polynomials based on
-   * Polynomials::HermiteInterpolation.
+   * Polynomials::HermiteLikeInterpolation.
    */
   FE_DGQHermite (const unsigned int degree);
 
-  /**
-   * Return a list of constant modes of the element. For the Hermite basis of
-   * degree three and larger, it returns one row where the first two elements
-   * (corresponding to the value left and the value at the right) are set to
-   * true and all other elements are set to false.
-   */
-  virtual std::pair<Table<2,bool>, std::vector<unsigned int> >
-  get_constant_modes () const;
-
   /**
    * Return a string that uniquely identifies a finite element. This class
    * returns <tt>FE_DGQHermite<dim>(degree)</tt>, with <tt>dim</tt> and
@@ -507,13 +514,9 @@ public:
    */
   virtual std::string get_name () const;
 
-protected:
-  /**
-   * @p clone function instead of a copy constructor.
-   *
-   * This function is needed by the constructors of @p FESystem.
-   */
-  virtual FiniteElement<dim,spacedim> *clone() const;
+  virtual
+  std::unique_ptr<FiniteElement<dim,spacedim> >
+  clone() const;
 };
 
 
