@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 2016 - 2017 by the deal.II authors
+// Copyright (C) 2016 - 2018 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -13,16 +13,11 @@
 //
 // ---------------------------------------------------------------------
 
-#ifndef dealii__fe_enriched_h
-#define dealii__fe_enriched_h
+#ifndef dealii_fe_enriched_h
+#define dealii_fe_enriched_h
 
 #include <deal.II/base/config.h>
 
-// We require c++14 here even though we only need features that are part of
-// c++11, but it turns out that gcc 4.6.x and 4.7.x don't support c++11
-// features like delegating constructors.
-
-#ifdef DEAL_II_WITH_CXX14
 #include <deal.II/fe/fe.h>
 #include <deal.II/fe/fe_system.h>
 #include <deal.II/fe/fe_nothing.h>
@@ -136,7 +131,7 @@ DEAL_II_NAMESPACE_OPEN
  * @code
  *   FE_Enriched<dim> fe_non_enriched(FE_Q<dim>(1),
  *                                    FE_Nothing<dim>(1,true),
- *                                    NULL);
+ *                                    nullptr);
  * @endcode
  * and will result in the correct constraints for enriched DoFs attributed to
  * support points on the interface between the two regions.
@@ -145,13 +140,19 @@ DEAL_II_NAMESPACE_OPEN
  *
  * When using this class, please cite
  * @code{.bib}
- * @Article{Davydov2016,
- *   Title                    = {On the h-adaptive PUM and hp-adaptive FEM approaches applied to PDEs in quantum mechanics.},
- *   Author                   = {Davydov, D and Gerasimov, T and Pelteret, J.-P. and Steinmann, P.},
- *   eprinttype               = {arXiv},
- *   eprint                   = {1612.02305},
- *   eprintclass              = {physics.comp-ph},
- *   Year                     = {2016},
+ * @Article{Davydov2017,
+ *  author    = {Denis Davydov and Tymofiy Gerasimov and Jean-Paul Pelteret and Paul Steinmann},
+ *  title     = {Convergence study of the h-adaptive PUM and the hp-adaptive FEM applied to eigenvalue problems in quantum mechanics},
+ *  journal   = {Advanced Modeling and Simulation in Engineering Sciences},
+ *  year      = {2017},
+ *  volume    = {4},
+ *  number    = {1},
+ *  pages     = {7},
+ *  month     = {Dec},
+ *  issn      = {2213-7467},
+ *  day       = {12},
+ *  doi       = {10.1186/s40323-017-0093-0},
+ *  url       = {https://doi.org/10.1186/s40323-017-0093-0},
  * }
  * @endcode
  * The PUM was introduced in
@@ -197,8 +198,6 @@ DEAL_II_NAMESPACE_OPEN
  * The ordering of the shape function, @p interface_constrains, the @p prolongation (embedding)
  * and the @p restriction matrices are taken from the FESystem class.
  *
- * @note This class is only available when deal.II is compiled with C++11.
- *
  * @ingroup fe
  *
  * @author Denis Davydov, 2016.
@@ -280,12 +279,9 @@ private:
                const std::vector<std::vector<std::function<const Function<spacedim> *(const typename Triangulation<dim, spacedim>::cell_iterator &) > > > &functions);
 public:
 
-  /**
-   * @p clone function instead of a copy constructor.
-   *
-   * This function is needed by the constructors of @p FESystem.
-   */
-  virtual FiniteElement<dim,spacedim> *clone() const;
+  virtual
+  std::unique_ptr<FiniteElement<dim,spacedim> >
+  clone() const;
 
   virtual
   UpdateFlags
@@ -357,7 +353,7 @@ public:
   virtual bool hp_constraints_are_implemented () const;
 
   /**
-   * Return the matrix interpolating from a face of of one element to the face
+   * Return the matrix interpolating from a face of one element to the face
    * of the neighboring element.  The size of the matrix is then
    * <tt>source.dofs_per_face</tt> times <tt>this->dofs_per_face</tt>.
    *
@@ -373,7 +369,7 @@ public:
                                  FullMatrix<double>       &matrix) const;
 
   /**
-   * Return the matrix interpolating from a face of of one element to the
+   * Return the matrix interpolating from a face of one element to the
    * subface of the neighboring element.  The size of the matrix is then
    * <tt>source.dofs_per_face</tt> times <tt>this->dofs_per_face</tt>.
    *
@@ -480,21 +476,21 @@ protected:
      * ownership), we store the cast result in a std::unique_ptr to indicate
      * that InternalData owns the object.
      */
-    InternalData ( std::unique_ptr<typename FESystem<dim,spacedim>::InternalData> fesystem_data);
+    InternalData (std::unique_ptr<typename FESystem<dim,spacedim>::InternalData> fesystem_data);
 
     /**
-     * Gives read-access to the pointer to a @p InternalData of the @p
+     * Give read-access to the pointer to a @p InternalData of the @p
      * <code>base_no</code>th base element of FESystem's data.
      */
     typename FiniteElement<dim,spacedim>::InternalDataBase &
     get_fe_data (const unsigned int base_no) const;
 
     /**
-     * Gives read-access to the pointer to an object into which the
+     * Give read-access to the pointer to an object into which the
      * <code>base_no</code>th base element will write its output when calling
      * FiniteElement::fill_fe_values() and similar functions.
      */
-    internal::FEValues::FiniteElementRelatedData<dim,spacedim> &
+    internal::FEValuesImplementation::FiniteElementRelatedData<dim,spacedim> &
     get_fe_output_object (const unsigned int base_no) const;
 
     /**
@@ -556,8 +552,8 @@ protected:
    * enrichment functions at each quadrature point depending on @p flags.
    */
   template <int dim_1>
-  typename FiniteElement<dim,spacedim>::InternalDataBase *
-  setup_data (std::unique_ptr<typename FiniteElement<dim,spacedim>::InternalDataBase> fes_data,
+  std::unique_ptr<typename FiniteElement<dim,spacedim>::InternalDataBase>
+  setup_data (std::unique_ptr<typename FESystem<dim,spacedim>::InternalData> fes_data,
               const UpdateFlags      flags,
               const Quadrature<dim_1> &quadrature) const;
 
@@ -567,23 +563,23 @@ protected:
    * (FEValues) then has to assume ownership (which includes destruction when it is no
    * more needed).
    */
-  virtual typename FiniteElement<dim,spacedim>::InternalDataBase *
+  virtual std::unique_ptr<typename FiniteElement<dim,spacedim>::InternalDataBase>
   get_data (const UpdateFlags      flags,
             const Mapping<dim,spacedim>    &mapping,
             const Quadrature<dim> &quadrature,
-            dealii::internal::FEValues::FiniteElementRelatedData< dim, spacedim > &output_data) const;
+            dealii::internal::FEValuesImplementation::FiniteElementRelatedData< dim, spacedim > &output_data) const;
 
-  virtual typename FiniteElement<dim,spacedim>::InternalDataBase *
+  virtual std::unique_ptr<typename FiniteElement<dim,spacedim>::InternalDataBase>
   get_face_data (const UpdateFlags      update_flags,
                  const Mapping<dim,spacedim>    &mapping,
                  const Quadrature<dim-1> &quadrature,
-                 dealii::internal::FEValues::FiniteElementRelatedData< dim, spacedim >        &output_data) const;
+                 dealii::internal::FEValuesImplementation::FiniteElementRelatedData< dim, spacedim >        &output_data) const;
 
-  virtual typename FiniteElement<dim,spacedim>::InternalDataBase *
+  virtual std::unique_ptr<typename FiniteElement<dim,spacedim>::InternalDataBase>
   get_subface_data (const UpdateFlags      update_flags,
                     const Mapping<dim,spacedim>    &mapping,
                     const Quadrature<dim-1> &quadrature,
-                    dealii::internal::FEValues::FiniteElementRelatedData<dim, spacedim> &output_data) const;
+                    dealii::internal::FEValuesImplementation::FiniteElementRelatedData<dim, spacedim> &output_data) const;
 
   virtual
   void fill_fe_values (const typename Triangulation<dim, spacedim>::cell_iterator &cell,
@@ -591,9 +587,9 @@ protected:
                        const Quadrature<dim> &quadrature,
                        const Mapping<dim, spacedim> &mapping,
                        const typename Mapping<dim, spacedim>::InternalDataBase &mapping_internal,
-                       const dealii::internal::FEValues::MappingRelatedData<dim, spacedim> &mapping_data,
+                       const dealii::internal::FEValuesImplementation::MappingRelatedData<dim, spacedim> &mapping_data,
                        const typename FiniteElement<dim,spacedim>::InternalDataBase &fe_internal,
-                       dealii::internal::FEValues::FiniteElementRelatedData<dim, spacedim> &output_data
+                       dealii::internal::FEValuesImplementation::FiniteElementRelatedData<dim, spacedim> &output_data
                       ) const;
 
   virtual
@@ -603,9 +599,9 @@ protected:
                         const Quadrature<dim-1> &quadrature,
                         const Mapping<dim, spacedim> &mapping,
                         const typename Mapping<dim, spacedim>::InternalDataBase &mapping_internal,
-                        const dealii::internal::FEValues::MappingRelatedData<dim, spacedim> &mapping_data,
+                        const dealii::internal::FEValuesImplementation::MappingRelatedData<dim, spacedim> &mapping_data,
                         const typename FiniteElement<dim,spacedim>::InternalDataBase &fe_internal,
-                        dealii::internal::FEValues::FiniteElementRelatedData<dim, spacedim> &output_data
+                        dealii::internal::FEValuesImplementation::FiniteElementRelatedData<dim, spacedim> &output_data
                       ) const;
 
   virtual
@@ -616,9 +612,9 @@ protected:
                           const Quadrature<dim-1> &quadrature,
                           const Mapping<dim, spacedim> &mapping,
                           const typename Mapping< dim, spacedim >::InternalDataBase &mapping_internal,
-                          const dealii::internal::FEValues::MappingRelatedData<dim, spacedim> &mapping_data,
+                          const dealii::internal::FEValuesImplementation::MappingRelatedData<dim, spacedim> &mapping_data,
                           const typename FiniteElement<dim,spacedim>::InternalDataBase &fe_internal,
-                          dealii::internal::FEValues::FiniteElementRelatedData<dim, spacedim> &output_data
+                          dealii::internal::FEValuesImplementation::FiniteElementRelatedData<dim, spacedim> &output_data
                          ) const;
 
 private:
@@ -632,7 +628,7 @@ private:
   /**
    * The underlying FESystem object.
    */
-  FESystem<dim,spacedim> fe_system;
+  const std::unique_ptr<const FESystem<dim,spacedim> > fe_system;
 
   /**
    * After calling fill_fe_(face/subface_)values this function
@@ -643,14 +639,12 @@ private:
   void
   multiply_by_enrichment (const Quadrature<dim_1>                                       &quadrature,
                           const InternalData                                            &fe_data,
-                          const internal::FEValues::MappingRelatedData<dim,spacedim>    &mapping_data,
+                          const internal::FEValuesImplementation::MappingRelatedData<dim,spacedim>    &mapping_data,
                           const typename Triangulation< dim, spacedim >::cell_iterator  &cell,
-                          internal::FEValues::FiniteElementRelatedData<dim,spacedim>    &output_data) const;
+                          internal::FEValuesImplementation::FiniteElementRelatedData<dim,spacedim>    &output_data) const;
 };
 
 //}
 DEAL_II_NAMESPACE_CLOSE
 
-#endif // CXX14
-
-#endif // dealii__fe_enriched_h
+#endif // dealii_fe_enriched_h
