@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 2009 - 2016 by the deal.II authors
+// Copyright (C) 2009 - 2018 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -13,8 +13,8 @@
 //
 // ---------------------------------------------------------------------
 
-#ifndef dealii__fe_face_h
-#define dealii__fe_face_h
+#ifndef dealii_fe_face_h
+#define dealii_fe_face_h
 
 #include <deal.II/base/config.h>
 #include <deal.II/base/tensor_product_polynomials.h>
@@ -25,7 +25,7 @@ DEAL_II_NAMESPACE_OPEN
 
 
 /**
- * A finite element, which is a tensor product polynomial on each face and
+ * A finite element that is a tensor product polynomial on each face and
  * undefined in the interior of the cells. The basis functions on the faces
  * are Lagrange polynomials based on the support points of the
  * (dim-1)-dimensional Gauss--Lobatto quadrature rule. For element degree one
@@ -39,10 +39,13 @@ DEAL_II_NAMESPACE_OPEN
  * demonstrated in the step-51 tutorial program.
  *
  * @note Since this element is defined only on faces, only FEFaceValues and
- * FESubfaceValues will be able to extract reasonable values from any face
- * polynomial. In order to make the use of FESystem simpler, using a (cell)
- * FEValues object will not fail using this finite element space, but all
- * shape function values extracted will be equal to zero.
+ * FESubfaceValues will provide useful information. On the other hand, if you
+ * use this element with FEValues for cell integration, then the values
+ * and derivatives of shape functions will have invalid values and will not
+ * likely produce anything useful. In order to make the use of this element
+ * as part of an FESystem simpler, using a (cell) FEValues object will not fail
+ * outright, but those components of shape functions of the combined element
+ * that correspond to FE_FaceQ will have the invalid values mentioned above.
  *
  * @ingroup fe
  * @author Guido Kanschat, Martin Kronbichler
@@ -59,7 +62,9 @@ public:
    */
   FE_FaceQ (const unsigned int p);
 
-  virtual FiniteElement<dim,spacedim> *clone() const;
+  virtual
+  std::unique_ptr<FiniteElement<dim,spacedim> >
+  clone() const;
 
   /**
    * Return a string that uniquely identifies a finite element. This class
@@ -69,7 +74,19 @@ public:
   virtual std::string get_name () const;
 
   /**
-   * Return the matrix interpolating from a face of of one element to the face
+   * Implementation of the corresponding function in the FiniteElement
+   * class.  Since the current element is interpolatory, the nodal
+   * values are exactly the support point values. Furthermore, since
+   * the current element is scalar, the support point values need to
+   * be vectors of length 1.
+   */
+  virtual
+  void
+  convert_generalized_support_point_values_to_dof_values (const std::vector<Vector<double> > &support_point_values,
+                                                          std::vector<double>                &nodal_values) const;
+
+  /**
+   * Return the matrix interpolating from a face of one element to the face
    * of the neighboring element.  The size of the matrix is then
    * <tt>source.dofs_per_face</tt> times <tt>this->dofs_per_face</tt>. This
    * element only provides interpolation matrices for elements of the same
@@ -81,7 +98,7 @@ public:
                                  FullMatrix<double>       &matrix) const;
 
   /**
-   * Return the matrix interpolating from a face of of one element to the face
+   * Return the matrix interpolating from a face of one element to the face
    * of the neighboring element.  The size of the matrix is then
    * <tt>source.dofs_per_face</tt> times <tt>this->dofs_per_face</tt>. This
    * element only provides interpolation matrices for elements of the same
@@ -100,6 +117,52 @@ public:
   virtual bool has_support_on_face (const unsigned int shape_index,
                                     const unsigned int face_index) const;
 
+  /**
+   * @name Functions to support hp
+   * @{
+   */
+
+  /**
+   * If, on a vertex, several finite elements are active, the hp code first
+   * assigns the degrees of freedom of each of these FEs different global
+   * indices. It then calls this function to find out which of them should get
+   * identical values, and consequently can receive the same global DoF index.
+   * This function therefore returns a list of identities between DoFs of the
+   * present finite element object with the DoFs of @p fe_other, which is a
+   * reference to a finite element object representing one of the other finite
+   * elements active on this particular vertex. The function computes which of
+   * the degrees of freedom of the two finite element objects are equivalent,
+   * both numbered between zero and the corresponding value of dofs_per_vertex
+   * of the two finite elements. The first index of each pair denotes one of
+   * the vertex dofs of the present element, whereas the second is the
+   * corresponding index of the other finite element.
+   *
+   * The set of such constraints is non-empty only for dim==1.
+   */
+  virtual
+  std::vector<std::pair<unsigned int, unsigned int> >
+  hp_vertex_dof_identities (const FiniteElement<dim, spacedim> &fe_other) const;
+
+  /**
+   * Same as hp_vertex_dof_indices(), except that the function treats degrees
+   * of freedom on lines.
+   *
+   * The set of such constraints is non-empty only for dim==2.
+   */
+  virtual
+  std::vector<std::pair<unsigned int, unsigned int> >
+  hp_line_dof_identities (const FiniteElement<dim, spacedim> &fe_other) const;
+
+  /**
+   * Same as hp_vertex_dof_indices(), except that the function treats degrees
+   * of freedom on quads.
+   *
+   * The set of such constraints is non-empty only for dim==3.
+   */
+  virtual
+  std::vector<std::pair<unsigned int, unsigned int> >
+  hp_quad_dof_identities (const FiniteElement<dim, spacedim> &fe_other) const;
+
   /**
    * Return whether this element implements its hanging node constraints in
    * the new way, which has to be used to make elements "hp compatible".
@@ -118,6 +181,9 @@ public:
   virtual
   FiniteElementDomination::Domination
   compare_for_face_domination (const FiniteElement<dim,spacedim> &fe_other) const;
+  /**
+   * @}
+   */
 
   /**
    * Return a list of constant modes of the element. For this element, it
@@ -156,10 +222,9 @@ public:
    */
   FE_FaceQ (const unsigned int p);
 
-  /**
-   * Clone method.
-   */
-  virtual FiniteElement<1,spacedim> *clone() const;
+  virtual
+  std::unique_ptr<FiniteElement<1,spacedim>>
+                                          clone() const;
 
   /**
    * Return a string that uniquely identifies a finite element. This class
@@ -174,7 +239,7 @@ public:
   requires_update_flags (const UpdateFlags update_flags) const;
 
   /**
-   * Return the matrix interpolating from a face of of one element to the face
+   * Return the matrix interpolating from a face of one element to the face
    * of the neighboring element.  The size of the matrix is then
    * <tt>source.dofs_per_face</tt> times <tt>this->dofs_per_face</tt>. This
    * element only provides interpolation matrices for elements of the same
@@ -186,7 +251,7 @@ public:
                                  FullMatrix<double>       &matrix) const;
 
   /**
-   * Return the matrix interpolating from a face of of one element to the face
+   * Return the matrix interpolating from a face of one element to the face
    * of the neighboring element.  The size of the matrix is then
    * <tt>source.dofs_per_face</tt> times <tt>this->dofs_per_face</tt>. This
    * element only provides interpolation matrices for elements of the same
@@ -211,6 +276,47 @@ public:
    */
   virtual bool hp_constraints_are_implemented () const;
 
+  /**
+   * If, on a vertex, several finite elements are active, the hp code first
+   * assigns the degrees of freedom of each of these FEs different global
+   * indices. It then calls this function to find out which of them should get
+   * identical values, and consequently can receive the same global DoF index.
+   * This function therefore returns a list of identities between DoFs of the
+   * present finite element object with the DoFs of @p fe_other, which is a
+   * reference to a finite element object representing one of the other finite
+   * elements active on this particular vertex. The function computes which of
+   * the degrees of freedom of the two finite element objects are equivalent,
+   * both numbered between zero and the corresponding value of dofs_per_vertex
+   * of the two finite elements. The first index of each pair denotes one of
+   * the vertex dofs of the present element, whereas the second is the
+   * corresponding index of the other finite element.
+   *
+   * The set of such constraints is non-empty only for dim==1.
+   */
+  virtual
+  std::vector<std::pair<unsigned int, unsigned int> >
+  hp_vertex_dof_identities (const FiniteElement<1, spacedim> &fe_other) const;
+
+  /**
+   * Same as hp_vertex_dof_indices(), except that the function treats degrees
+   * of freedom on lines.
+   *
+   * The set of such constraints is non-empty only for dim==2.
+   */
+  virtual
+  std::vector<std::pair<unsigned int, unsigned int> >
+  hp_line_dof_identities (const FiniteElement<1, spacedim> &fe_other) const;
+
+  /**
+   * Same as hp_vertex_dof_indices(), except that the function treats degrees
+   * of freedom on quads.
+   *
+   * The set of such constraints is non-empty only for dim==3.
+   */
+  virtual
+  std::vector<std::pair<unsigned int, unsigned int> >
+  hp_quad_dof_identities (const FiniteElement<1, spacedim> &fe_other) const;
+
   /**
    * Return whether this element dominates the one given as argument when they
    * meet at a common face, whether it is the other way around, whether
@@ -239,24 +345,23 @@ protected:
 
 
   virtual
-  typename FiniteElement<1,spacedim>::InternalDataBase *
+  std::unique_ptr<typename FiniteElement<1,spacedim>::InternalDataBase>
   get_data (const UpdateFlags                                                  /*update_flags*/,
             const Mapping<1,spacedim>                                         &/*mapping*/,
             const Quadrature<1>                                               &/*quadrature*/,
-            dealii::internal::FEValues::FiniteElementRelatedData<1, spacedim> &/*output_data*/) const
+            dealii::internal::FEValuesImplementation::FiniteElementRelatedData<1, spacedim> &/*output_data*/) const
   {
-    return new typename FiniteElement<1, spacedim>::InternalDataBase;
+    return std_cxx14::make_unique<typename FiniteElement<1, spacedim>::InternalDataBase>();
   }
 
-  typename FiniteElement<1,spacedim>::InternalDataBase *
+  std::unique_ptr<typename FiniteElement<1,spacedim>::InternalDataBase>
   get_face_data(const UpdateFlags update_flags,
                 const Mapping<1,spacedim> &/*mapping*/,
                 const Quadrature<0> &quadrature,
-                dealii::internal::FEValues::FiniteElementRelatedData<1, spacedim> &/*output_data*/) const
+                dealii::internal::FEValuesImplementation::FiniteElementRelatedData<1, spacedim> &/*output_data*/) const
   {
     // generate a new data object and initialize some fields
-    typename FiniteElement<1,spacedim>::InternalDataBase *data =
-      new typename FiniteElement<1,spacedim>::InternalDataBase;
+    auto data = std_cxx14::make_unique<typename FiniteElement<1,spacedim>::InternalDataBase>();
     data->update_each = requires_update_flags(update_flags);
 
     const unsigned int n_q_points = quadrature.size();
@@ -269,14 +374,14 @@ protected:
         Assert(false, ExcNotImplemented());
       }
 
-    return data;
+    return std::move(data);
   }
 
-  typename FiniteElement<1,spacedim>::InternalDataBase *
+  std::unique_ptr<typename FiniteElement<1,spacedim>::InternalDataBase>
   get_subface_data(const UpdateFlags                                                  update_flags,
                    const Mapping<1,spacedim>                                         &mapping,
                    const Quadrature<0>                                               &quadrature,
-                   dealii::internal::FEValues::FiniteElementRelatedData<1, spacedim> &output_data) const
+                   dealii::internal::FEValuesImplementation::FiniteElementRelatedData<1, spacedim> &output_data) const
   {
     return get_face_data(update_flags, mapping, quadrature, output_data);
   }
@@ -288,9 +393,9 @@ protected:
                   const Quadrature<1>                                               &quadrature,
                   const Mapping<1,spacedim>                                         &mapping,
                   const typename Mapping<1,spacedim>::InternalDataBase              &mapping_internal,
-                  const dealii::internal::FEValues::MappingRelatedData<1, spacedim> &mapping_data,
+                  const dealii::internal::FEValuesImplementation::MappingRelatedData<1, spacedim> &mapping_data,
                   const typename FiniteElement<1,spacedim>::InternalDataBase        &fe_internal,
-                  dealii::internal::FEValues::FiniteElementRelatedData<1, spacedim> &output_data) const;
+                  dealii::internal::FEValuesImplementation::FiniteElementRelatedData<1, spacedim> &output_data) const;
 
   virtual
   void
@@ -299,9 +404,9 @@ protected:
                        const Quadrature<0>                                               &quadrature,
                        const Mapping<1,spacedim>                                         &mapping,
                        const typename Mapping<1,spacedim>::InternalDataBase              &mapping_internal,
-                       const dealii::internal::FEValues::MappingRelatedData<1, spacedim> &mapping_data,
+                       const dealii::internal::FEValuesImplementation::MappingRelatedData<1, spacedim> &mapping_data,
                        const typename FiniteElement<1,spacedim>::InternalDataBase        &fe_internal,
-                       dealii::internal::FEValues::FiniteElementRelatedData<1, spacedim> &output_data) const;
+                       dealii::internal::FEValuesImplementation::FiniteElementRelatedData<1, spacedim> &output_data) const;
 
   virtual
   void
@@ -311,9 +416,9 @@ protected:
                           const Quadrature<0>                                               &quadrature,
                           const Mapping<1,spacedim>                                         &mapping,
                           const typename Mapping<1,spacedim>::InternalDataBase              &mapping_internal,
-                          const dealii::internal::FEValues::MappingRelatedData<1, spacedim> &mapping_data,
+                          const dealii::internal::FEValuesImplementation::MappingRelatedData<1, spacedim> &mapping_data,
                           const typename FiniteElement<1,spacedim>::InternalDataBase        &fe_internal,
-                          dealii::internal::FEValues::FiniteElementRelatedData<1, spacedim> &output_data) const;
+                          dealii::internal::FEValuesImplementation::FiniteElementRelatedData<1, spacedim> &output_data) const;
 
 private:
   /**
@@ -327,7 +432,7 @@ private:
 
 
 /**
- * A finite element, which is a Legendre element of complete polynomials on
+ * A finite element that is a Legendre element of complete polynomials on
  * each face (i.e., it is the face equivalent of what FE_DGP is on cells) and
  * undefined in the interior of the cells. The basis functions on the faces
  * are from Polynomials::Legendre.
@@ -338,10 +443,13 @@ private:
  * hybridizes methods can be found in the step-51 tutorial program.
  *
  * @note Since this element is defined only on faces, only FEFaceValues and
- * FESubfaceValues will be able to extract reasonable values from any face
- * polynomial. In order to make the use of FESystem simpler, using a (cell)
- * FEValues object will not fail using this finite element space, but all
- * shape function values extracted will be equal to zero.
+ * FESubfaceValues will provide useful information. On the other hand, if you
+ * use this element with FEValues for cell integration, then the values
+ * and derivatives of shape functions will have invalid values and will not
+ * likely produce anything useful. In order to make the use of this element
+ * as part of an FESystem simpler, using a (cell) FEValues object will not fail
+ * outright, but those components of shape functions of the combined element
+ * that correspond to FE_FaceP will have the invalid values mentioned above.
  *
  * @ingroup fe
  * @author Martin Kronbichler
@@ -358,10 +466,9 @@ public:
    */
   FE_FaceP(unsigned int p);
 
-  /**
-   * Clone method.
-   */
-  virtual FiniteElement<dim,spacedim> *clone() const;
+  virtual
+  std::unique_ptr<FiniteElement<dim,spacedim> >
+  clone() const;
 
   /**
    * Return a string that uniquely identifies a finite element. This class
@@ -371,7 +478,7 @@ public:
   virtual std::string get_name () const;
 
   /**
-   * Return the matrix interpolating from a face of of one element to the face
+   * Return the matrix interpolating from a face of one element to the face
    * of the neighboring element.  The size of the matrix is then
    * <tt>source.dofs_per_face</tt> times <tt>this->dofs_per_face</tt>. This
    * element only provides interpolation matrices for elements of the same
@@ -383,7 +490,7 @@ public:
                                  FullMatrix<double>       &matrix) const;
 
   /**
-   * Return the matrix interpolating from a face of of one element to the face
+   * Return the matrix interpolating from a face of one element to the face
    * of the neighboring element.  The size of the matrix is then
    * <tt>source.dofs_per_face</tt> times <tt>this->dofs_per_face</tt>. This
    * element only provides interpolation matrices for elements of the same
@@ -441,6 +548,7 @@ private:
 
 /**
  * FE_FaceP in 1D, i.e., with degrees of freedom on the element vertices.
+ * See the documentation of the general template for more information.
  */
 template <int spacedim>
 class FE_FaceP<1,spacedim> : public FE_FaceQ<1,spacedim>
