@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 2002 - 2017 by the deal.II authors
+// Copyright (C) 2002 - 2018 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -13,8 +13,8 @@
 //
 // ---------------------------------------------------------------------
 
-#ifndef dealii__fe_nedelec_h
-#define dealii__fe_nedelec_h
+#ifndef dealii_fe_nedelec_h
+#define dealii_fe_nedelec_h
 
 #include <deal.II/base/config.h>
 #include <deal.II/base/table.h>
@@ -239,7 +239,7 @@ public:
    * single child cell is returned here.
    *
    * The matrix @p P is the concatenation, not the sum of the cell matrices @p
-   * P_i. That is, if the same non-zero entry <tt>j,k</tt> exists in in two
+   * P_i. That is, if the same non-zero entry <tt>j,k</tt> exists in two
    * different child matrices @p P_i, the value should be the same in both
    * matrices and it is copied into the matrix @p P only once.
    *
@@ -258,18 +258,8 @@ public:
   // documentation inherited from the base class
   virtual
   void
-  convert_generalized_support_point_values_to_nodal_values (const std::vector<Vector<double> > &support_point_values,
-                                                            std::vector<double>                &nodal_values) const;
-
-  virtual void interpolate (std::vector<double> &local_dofs,
-                            const std::vector<double> &values) const DEAL_II_DEPRECATED;
-
-  virtual void interpolate (std::vector<double> &local_dofs,
-                            const std::vector<Vector<double> > &values,
-                            const unsigned int offset = 0) const DEAL_II_DEPRECATED;
-
-  virtual void interpolate (std::vector<double> &local_dofs,
-                            const VectorSlice<const std::vector<std::vector<double> > > &values) const DEAL_II_DEPRECATED;
+  convert_generalized_support_point_values_to_dof_values (const std::vector<Vector<double> > &support_point_values,
+                                                          std::vector<double>                &nodal_values) const;
 
   /**
    * Return a list of constant modes of the element.
@@ -278,7 +268,10 @@ public:
   get_constant_modes () const;
 
   virtual std::size_t memory_consumption () const;
-  virtual FiniteElement<dim> *clone() const;
+
+  virtual
+  std::unique_ptr<FiniteElement<dim,dim> >
+  clone() const;
 
 private:
   /**
