@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 2004 - 2016 by the deal.II authors
+// Copyright (C) 2004 - 2018 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -13,12 +13,13 @@
 //
 // ---------------------------------------------------------------------
 
-#ifndef dealii__fe_poly_h
-#define dealii__fe_poly_h
+#ifndef dealii_fe_poly_h
+#define dealii_fe_poly_h
 
 
 #include <deal.II/fe/fe.h>
 #include <deal.II/base/quadrature.h>
+#include <deal.II/base/std_cxx14/memory.h>
 
 DEAL_II_NAMESPACE_OPEN
 
@@ -30,11 +31,11 @@ DEAL_II_NAMESPACE_OPEN
  * FiniteElement classes based on polynomial spaces like the
  * TensorProductPolynomials or PolynomialSpace classes.
  *
- * Every class conforming to the following interface can be used as template
- * parameter PolynomialType.
+ * Every class that has the following public member variables and
+ * functions can be used as template parameter @p PolynomialType.
  *
  * @code
- * static const unsigned int dimension;
+ *  static const unsigned int dimension;
  *
  *  void compute (const Point<dim>            &unit_point,
  *                std::vector<double>         &values,
@@ -43,8 +44,8 @@ DEAL_II_NAMESPACE_OPEN
  *                std::vector<Tensor<3,dim> > &third_derivatives,
  *                std::vector<Tensor<4,dim> > &fourth_derivatives) const;
  *
- * double compute_value (const unsigned int i,
- *                       const Point<dim> &p) const;
+ *  double compute_value (const unsigned int i,
+ *                        const Point<dim> &p) const;
  *
  *  template <int order>
  *  Tensor<order,dim> compute_derivative (const unsigned int i,
@@ -218,15 +219,15 @@ protected:
 
 
   virtual
-  typename FiniteElement<dim,spacedim>::InternalDataBase *
+  std::unique_ptr<typename FiniteElement<dim,spacedim>::InternalDataBase>
   get_data(const UpdateFlags                                                    update_flags,
            const Mapping<dim,spacedim>                                         &/*mapping*/,
            const Quadrature<dim>                                               &quadrature,
-           dealii::internal::FEValues::FiniteElementRelatedData<dim, spacedim> &output_data) const
+           dealii::internal::FEValuesImplementation::FiniteElementRelatedData<dim, spacedim> &output_data) const
   {
     // generate a new data object and
     // initialize some fields
-    InternalData *data = new InternalData;
+    auto data = std_cxx14::make_unique<InternalData>();
     data->update_each = requires_update_flags(update_flags);
 
     const unsigned int n_q_points = quadrature.size();
@@ -321,7 +322,7 @@ protected:
             for (unsigned int k=0; k<this->dofs_per_cell; ++k)
               data->shape_3rd_derivatives[k][i] = third_derivatives[k];
         }
-    return data;
+    return std::move(data);
   }
 
   virtual
@@ -331,9 +332,9 @@ protected:
                   const Quadrature<dim>                                               &quadrature,
                   const Mapping<dim,spacedim>                                         &mapping,
                   const typename Mapping<dim,spacedim>::InternalDataBase              &mapping_internal,
-                  const dealii::internal::FEValues::MappingRelatedData<dim, spacedim> &mapping_data,
+                  const dealii::internal::FEValuesImplementation::MappingRelatedData<dim, spacedim> &mapping_data,
                   const typename FiniteElement<dim,spacedim>::InternalDataBase        &fe_internal,
-                  dealii::internal::FEValues::FiniteElementRelatedData<dim, spacedim> &output_data) const;
+                  dealii::internal::FEValuesImplementation::FiniteElementRelatedData<dim, spacedim> &output_data) const;
 
   virtual
   void
@@ -342,9 +343,9 @@ protected:
                        const Quadrature<dim-1>                                             &quadrature,
                        const Mapping<dim,spacedim>                                         &mapping,
                        const typename Mapping<dim,spacedim>::InternalDataBase              &mapping_internal,
-                       const dealii::internal::FEValues::MappingRelatedData<dim, spacedim> &mapping_data,
+                       const dealii::internal::FEValuesImplementation::MappingRelatedData<dim, spacedim> &mapping_data,
                        const typename FiniteElement<dim,spacedim>::InternalDataBase        &fe_internal,
-                       dealii::internal::FEValues::FiniteElementRelatedData<dim, spacedim> &output_data) const;
+                       dealii::internal::FEValuesImplementation::FiniteElementRelatedData<dim, spacedim> &output_data) const;
 
   virtual
   void
@@ -354,9 +355,9 @@ protected:
                           const Quadrature<dim-1>                                             &quadrature,
                           const Mapping<dim,spacedim>                                         &mapping,
                           const typename Mapping<dim,spacedim>::InternalDataBase              &mapping_internal,
-                          const dealii::internal::FEValues::MappingRelatedData<dim, spacedim> &mapping_data,
+                          const dealii::internal::FEValuesImplementation::MappingRelatedData<dim, spacedim> &mapping_data,
                           const typename FiniteElement<dim,spacedim>::InternalDataBase        &fe_internal,
-                          dealii::internal::FEValues::FiniteElementRelatedData<dim, spacedim> &output_data) const;
+                          dealii::internal::FEValuesImplementation::FiniteElementRelatedData<dim, spacedim> &output_data) const;
 
   /**
    * Fields of cell-independent data.
@@ -432,8 +433,8 @@ protected:
    * is the Jacobian pushed-forward second derivative.
    */
   void
-  correct_third_derivatives (internal::FEValues::FiniteElementRelatedData<dim,spacedim>       &output_data,
-                             const internal::FEValues::MappingRelatedData<dim,spacedim>       &mapping_data,
+  correct_third_derivatives (internal::FEValuesImplementation::FiniteElementRelatedData<dim,spacedim>       &output_data,
+                             const internal::FEValuesImplementation::MappingRelatedData<dim,spacedim>       &mapping_data,
                              const unsigned int                                                n_q_points,
                              const unsigned int                                                dof) const;
 
