@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 2009 - 2016 by the deal.II authors
+// Copyright (C) 2009 - 2018 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -13,8 +13,8 @@
 //
 // ---------------------------------------------------------------------
 
-#ifndef dealii__fe_poly_face_templates_h
-#define dealii__fe_poly_face_templates_h
+#ifndef dealii_fe_poly_face_templates_h
+#define dealii_fe_poly_face_templates_h
 
 
 #include <deal.II/base/qprojector.h>
@@ -75,16 +75,25 @@ template <class PolynomialType, int dim, int spacedim>
 void
 FE_PolyFace<PolynomialType,dim,spacedim>::
 fill_fe_values (const typename Triangulation<dim,spacedim>::cell_iterator &,
-                const CellSimilarity::Similarity                                     ,
+                const CellSimilarity::Similarity,
                 const Quadrature<dim> &,
                 const Mapping<dim,spacedim> &,
                 const typename Mapping<dim,spacedim>::InternalDataBase &,
-                const dealii::internal::FEValues::MappingRelatedData<dim, spacedim> &,
+                const dealii::internal::FEValuesImplementation::MappingRelatedData<dim, spacedim> &,
                 const typename FiniteElement<dim,spacedim>::InternalDataBase &,
-                dealii::internal::FEValues::FiniteElementRelatedData<dim, spacedim> &) const
+                dealii::internal::FEValuesImplementation::FiniteElementRelatedData<dim, spacedim> &) const
 {
-  // Do nothing, since we do not have
-  // values in the interior
+  // Do nothing, since we do not have values in the interior. Since
+  // FEValues initializes the output variables for this function
+  // with invalid values, this means that we simply leave them at
+  // the invalid values -- typically, signaling NaNs. This means
+  // that when you later look at those components of shape
+  // functions or solution vectors that correspond to the
+  // face element, you will see signaling_NaNs. This simply means
+  // that you should not use them -- the shape functions and the
+  // solution do not actually live inside the cell, and so any
+  // attempt at evaluating it there *should* yield an invalid
+  // result.
 }
 
 
@@ -97,15 +106,15 @@ fill_fe_face_values (const typename Triangulation<dim,spacedim>::cell_iterator &
                      const Quadrature<dim-1>                                             &quadrature,
                      const Mapping<dim,spacedim> &,
                      const typename Mapping<dim,spacedim>::InternalDataBase &,
-                     const dealii::internal::FEValues::MappingRelatedData<dim, spacedim> &,
+                     const dealii::internal::FEValuesImplementation::MappingRelatedData<dim, spacedim> &,
                      const typename FiniteElement<dim,spacedim>::InternalDataBase        &fe_internal,
-                     dealii::internal::FEValues::FiniteElementRelatedData<dim, spacedim> &output_data) const
+                     dealii::internal::FEValuesImplementation::FiniteElementRelatedData<dim, spacedim> &output_data) const
 {
   // convert data object to internal
   // data for this class. fails with
   // an exception if that is not
   // possible
-  Assert (dynamic_cast<const InternalData *> (&fe_internal) != 0, ExcInternalError());
+  Assert (dynamic_cast<const InternalData *> (&fe_internal) != nullptr, ExcInternalError());
   const InternalData &fe_data = static_cast<const InternalData &> (fe_internal);
 
   if (fe_data.update_each & update_values)
@@ -124,9 +133,8 @@ fill_fe_face_values (const typename Triangulation<dim,spacedim>::cell_iterator &
                 for (unsigned int k=0; k<this->dofs_per_quad; ++k)
                   output_data.shape_values(foffset+k,i) = fe_data.shape_values[k+this->first_face_quad_index][i];
               }
-
-            // fall through...
           }
+          DEAL_II_FALLTHROUGH;
 
           case 2:
           {
@@ -141,9 +149,8 @@ fill_fe_face_values (const typename Triangulation<dim,spacedim>::cell_iterator &
                         = fe_data.shape_values[k+(line*this->dofs_per_line)+this->first_face_line_index][i];
                   }
               }
-
-            // fall through...
           }
+          DEAL_II_FALLTHROUGH;
 
           case 1:
           {
@@ -168,15 +175,15 @@ fill_fe_subface_values (const typename Triangulation<dim,spacedim>::cell_iterato
                         const Quadrature<dim-1>                                             &quadrature,
                         const Mapping<dim,spacedim> &,
                         const typename Mapping<dim,spacedim>::InternalDataBase &,
-                        const dealii::internal::FEValues::MappingRelatedData<dim, spacedim> &,
+                        const dealii::internal::FEValuesImplementation::MappingRelatedData<dim, spacedim> &,
                         const typename FiniteElement<dim,spacedim>::InternalDataBase        &fe_internal,
-                        dealii::internal::FEValues::FiniteElementRelatedData<dim, spacedim> &output_data) const
+                        dealii::internal::FEValuesImplementation::FiniteElementRelatedData<dim, spacedim> &output_data) const
 {
   // convert data object to internal
   // data for this class. fails with
   // an exception if that is not
   // possible
-  Assert (dynamic_cast<const InternalData *> (&fe_internal) != 0, ExcInternalError());
+  Assert (dynamic_cast<const InternalData *> (&fe_internal) != nullptr, ExcInternalError());
   const InternalData &fe_data = static_cast<const InternalData &> (fe_internal);
 
   const unsigned int foffset = fe_data.shape_values.size() * face_no;
