@@ -13,8 +13,8 @@
 //
 // ---------------------------------------------------------------------
 
-#ifndef dealii__fe_q_hierarchical_h
-#define dealii__fe_q_hierarchical_h
+#ifndef dealii_fe_q_hierarchical_h
+#define dealii_fe_q_hierarchical_h
 
 #include <deal.II/base/config.h>
 #include <deal.II/base/tensor_product_polynomials.h>
@@ -553,6 +553,10 @@ public:
    */
   virtual std::string get_name () const;
 
+  virtual
+  std::unique_ptr<FiniteElement<dim,dim> >
+  clone() const;
+
   /**
    * This function returns @p true, if the shape function @p shape_index has
    * non-zero function values somewhere on the face @p face_index.
@@ -688,39 +692,6 @@ public:
   virtual std::pair<Table<2,bool>, std::vector<unsigned int> >
   get_constant_modes () const;
 
-  /**
-   * This function is not implemented and throws an exception if called.
-   */
-  virtual
-  void interpolate(std::vector<double>       &local_dofs,
-                   const std::vector<double> &values) const DEAL_II_DEPRECATED;
-
-  /**
-   * This function is not implemented and throws an exception if called.
-   */
-  virtual
-  void
-  interpolate(std::vector<double>                &local_dofs,
-              const std::vector<Vector<double> > &values,
-              const unsigned int offset = 0) const DEAL_II_DEPRECATED;
-
-  /**
-   * This function is not implemented and throws an exception if called.
-   */
-  virtual
-  void
-  interpolate(std::vector<double> &local_dofs,
-              const VectorSlice<const std::vector<std::vector<double> > > &values) const DEAL_II_DEPRECATED;
-
-
-protected:
-  /**
-   * @p clone function instead of a copy constructor.
-   *
-   * This function is needed by the constructors of @p FESystem.
-   */
-  virtual FiniteElement<dim> *clone() const;
-
 private:
 
   /**
