@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 2003 - 2017 by the deal.II authors
+// Copyright (C) 2003 - 2018 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -13,8 +13,8 @@
 //
 // ---------------------------------------------------------------------
 
-#ifndef dealii__fe_raviart_thomas_h
-#define dealii__fe_raviart_thomas_h
+#ifndef dealii_fe_raviart_thomas_h
+#define dealii_fe_raviart_thomas_h
 
 #include <deal.II/base/config.h>
 #include <deal.II/base/table.h>
@@ -93,11 +93,11 @@ DEAL_II_NAMESPACE_OPEN
  * quadrature rules themselves. The generalized support points are a set of
  * points such that this quadrature can be performed with sufficient accuracy.
  * The points needed are those of QGauss<sub>k+1</sub> on each face as well as
- * QGauss<sub>k</sub> in the interior of the cell (or none for
+ * QGauss<sub>k+1</sub> in the interior of the cell (or none for
  * RT<sub>0</sub>).
  *
  *
- * @author Guido Kanschat, 2005, based on previous Work by Wolfgang Bangerth
+ * @author Guido Kanschat, 2005, based on previous work by Wolfgang Bangerth.
  */
 template <int dim>
 class FE_RaviartThomas
@@ -117,6 +117,10 @@ public:
    */
   virtual std::string get_name () const;
 
+  // documentation inherited from the base class
+  virtual
+  std::unique_ptr<FiniteElement<dim,dim> >
+  clone() const;
 
   /**
    * This function returns @p true, if the shape function @p shape_index has
@@ -131,18 +135,8 @@ public:
   // documentation inherited from the base class
   virtual
   void
-  convert_generalized_support_point_values_to_nodal_values (const std::vector<Vector<double> > &support_point_values,
-                                                            std::vector<double>                &nodal_values) const;
-
-  virtual void interpolate(std::vector<double>                &local_dofs,
-                           const std::vector<double>          &values) const DEAL_II_DEPRECATED;
-
-  virtual void interpolate(std::vector<double>                &local_dofs,
-                           const std::vector<Vector<double> > &values,
-                           const unsigned int                  offset = 0) const DEAL_II_DEPRECATED;
-
-  virtual void interpolate(std::vector<double> &local_dofs,
-                           const VectorSlice<const std::vector<std::vector<double> > > &values) const DEAL_II_DEPRECATED;
+  convert_generalized_support_point_values_to_dof_values (const std::vector<Vector<double> > &support_point_values,
+                                                          std::vector<double>                &nodal_values) const;
 
   /**
    * Return a list of constant modes of the element. This method is currently
@@ -152,7 +146,6 @@ public:
   get_constant_modes () const;
 
   virtual std::size_t memory_consumption () const;
-  virtual FiniteElement<dim> *clone() const;
 
 private:
   /**
@@ -264,23 +257,15 @@ public:
    */
   virtual std::string get_name () const;
 
-  virtual FiniteElement<dim> *clone () const;
-
   // documentation inherited from the base class
+  virtual
+  std::unique_ptr<FiniteElement<dim,dim> >
+  clone() const;
+
   virtual
   void
-  convert_generalized_support_point_values_to_nodal_values (const std::vector<Vector<double> > &support_point_values,
-                                                            std::vector<double>                &nodal_values) const;
-  virtual void interpolate(std::vector<double>                &local_dofs,
-                           const std::vector<double> &values) const DEAL_II_DEPRECATED;
-
-  virtual void interpolate(std::vector<double>                &local_dofs,
-                           const std::vector<Vector<double> > &values,
-                           const unsigned int                  offset = 0) const DEAL_II_DEPRECATED;
-
-  virtual void interpolate(std::vector<double> &local_dofs,
-                           const VectorSlice<const std::vector<std::vector<double> > > &values) const DEAL_II_DEPRECATED;
-
+  convert_generalized_support_point_values_to_dof_values (const std::vector<Vector<double> > &support_point_values,
+                                                          std::vector<double>                &nodal_values) const;
 
   virtual void get_face_interpolation_matrix (const FiniteElement<dim> &source,
                                               FullMatrix<double>       &matrix) const;
