@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 2000 - 2017 by the deal.II authors
+// Copyright (C) 2000 - 2018 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -13,8 +13,8 @@
 //
 // ---------------------------------------------------------------------
 
-#ifndef dealii__fe_tools_H
-#define dealii__fe_tools_H
+#ifndef dealii_fe_tools_H
+#define dealii_fe_tools_H
 
 
 
@@ -24,6 +24,7 @@
 #include <deal.II/base/geometry_info.h>
 #include <deal.II/base/tensor.h>
 #include <deal.II/base/symmetric_tensor.h>
+#include <deal.II/base/std_cxx14/memory.h>
 #include <deal.II/distributed/tria.h>
 #include <deal.II/fe/component_mask.h>
 #include <deal.II/lac/parallel_vector.h>
@@ -83,7 +84,7 @@ namespace FETools
     /**
      * Create a FiniteElement and return a pointer to it.
      */
-    virtual FiniteElement<dim,spacedim> *
+    virtual std::unique_ptr<FiniteElement<dim,spacedim> >
     get (const unsigned int degree) const = 0;
 
     /**
@@ -91,12 +92,13 @@ namespace FETools
      * implemented for FE_Q) and return a pointer to it.
      */
 
-    virtual FiniteElement<dim,spacedim> *
+    virtual std::unique_ptr<FiniteElement<dim,spacedim> >
     get (const Quadrature<1> &quad) const = 0;
+
     /**
      * Virtual destructor doing nothing but making the compiler happy.
      */
-    virtual ~FEFactoryBase();
+    virtual ~FEFactoryBase() = default;
   };
 
   /**
@@ -117,14 +119,14 @@ namespace FETools
     /**
      * Create a FiniteElement and return a pointer to it.
      */
-    virtual FiniteElement<FE::dimension,FE::space_dimension> *
+    virtual std::unique_ptr<FiniteElement<FE::dimension,FE::space_dimension> >
     get (const unsigned int degree) const;
 
     /**
      * Create a FiniteElement from a quadrature formula (currently only
      * implemented for FE_Q) and return a pointer to it.
      */
-    virtual FiniteElement<FE::dimension,FE::space_dimension> *
+    virtual std::unique_ptr<FiniteElement<FE::dimension,FE::space_dimension> >
     get (const Quadrature<1> &quad) const;
   };
 
@@ -143,7 +145,7 @@ namespace FETools
    * While the first vector is checked to have the correct size, the second
    * one is reinitialized for convenience.
    */
-  template<int dim, int spacedim>
+  template <int dim, int spacedim>
   void compute_component_wise(
     const FiniteElement<dim,spacedim>                &fe,
     std::vector<unsigned int>               &renumbering,
@@ -164,7 +166,7 @@ namespace FETools
    * indicates the index which this degree of freedom receives in a numbering
    * scheme, where the first block is numbered completely before the second.
    */
-  template<int dim, int spacedim>
+  template <int dim, int spacedim>
   void compute_block_renumbering (
     const FiniteElement<dim,spacedim>  &fe,
     std::vector<types::global_dof_index> &renumbering,
@@ -176,7 +178,7 @@ namespace FETools
    * @{
    */
   /**
-   * Gives the interpolation matrix that interpolates a @p fe1- function to a
+   * Compute the interpolation matrix that interpolates a @p fe1-function to a
    * @p fe2-function on each cell. The interpolation_matrix needs to be of
    * size <tt>(fe2.dofs_per_cell, fe1.dofs_per_cell)</tt>.
    *
@@ -191,7 +193,7 @@ namespace FETools
                            FullMatrix<number> &interpolation_matrix);
 
   /**
-   * Gives the interpolation matrix that interpolates a @p fe1- function to a
+   * Compute the interpolation matrix that interpolates a @p fe1-function to a
    * @p fe2-function, and interpolates this to a second @p fe1-function on
    * each cell. The interpolation_matrix needs to be of size
    * <tt>(fe1.dofs_per_cell, fe1.dofs_per_cell)</tt>.
@@ -208,14 +210,15 @@ namespace FETools
                                 FullMatrix<number> &interpolation_matrix);
 
   /**
-   * Gives the unit matrix minus the back interpolation matrix.  The @p
-   * difference_matrix needs to be of size <tt>(fe1.dofs_per_cell,
-   * fe1.dofs_per_cell)</tt>.
+   * Compute the identity matrix minus the back interpolation matrix.
+   * The @p difference_matrix will be of size <tt>(fe1.dofs_per_cell,
+   * fe1.dofs_per_cell)</tt> after this function. Previous content
+   * of the argument will be overwritten.
    *
-   * This function gives the matrix that transforms a @p fe1 function $z$ to
+   * This function computes the matrix that transforms a @p fe1 function $z$ to
    * $z-I_hz$ where $I_h$ denotes the interpolation operator from the @p fe1
-   * space to the @p fe2 space. This matrix hence is useful to evaluate error-
-   * representations where $z$ denotes the dual solution.
+   * space to the @p fe2 space. This matrix hence is useful to evaluate
+   * error-representations where $z$ denotes the dual solution.
    */
   template <int dim, typename number, int spacedim>
   void
@@ -291,7 +294,7 @@ namespace FETools
    *   that returns a scalar.
    * - That the finite element has exactly @p dim vector components.
    * - That the function $f_j$ is given by whatever the element implements
-   *   through the FiniteElement::convert_generalized_support_point_values_to_nodal_values()
+   *   through the FiniteElement::convert_generalized_support_point_values_to_dof_values()
    *   function.
    *
    * @param fe The finite element for which the operations above are to be
@@ -302,17 +305,6 @@ namespace FETools
   FullMatrix<double>
   compute_node_matrix(const FiniteElement<dim,spacedim> &fe);
 
-  /**
-   * Same as the function above, but return the matrix by reference through
-   * the first argument, rather than as the function's return value.
-   *
-   * @deprecated
-   */
-  template <int dim, int spacedim>
-  void
-  compute_node_matrix(FullMatrix<double> &M,
-                      const FiniteElement<dim,spacedim> &fe) DEAL_II_DEPRECATED;
-
   /**
    * For all possible (isotropic and anisotropic) refinement cases compute the
    * embedding matrices from a coarse cell to the child cells. Each column of
@@ -402,16 +394,20 @@ namespace FETools
    * FiniteElement classes in order to fill the respective
    * FiniteElement::restriction matrices.
    *
-   * @arg fe The finite element class for which we compute the projection
-   * matrices.  @arg matrices A reference to
-   * <tt>RefinementCase<dim>::isotropic_refinement</tt> vectors of FullMatrix
-   * objects. Each vector corresponds to one RefinementCase @p refinement_case
-   * and is of the vector size
-   * <tt>GeometryInfo<dim>::n_children(refinement_case)</tt>. This is the
-   * format used in FiniteElement, where we want to use this function mostly.
+   * @arg[in] fe The finite element class for which we compute the projection
+   *   matrices.
    *
-   * @arg isotropic_only Set to <code>true</code> if you only want to compute
-   * matrices for isotropic refinement.
+   * @arg[out] matrices A reference to a set of
+   *   <tt>RefinementCase<dim>::isotropic_refinement</tt> vectors of FullMatrix
+   *   objects. Each vector corresponds to one RefinementCase @p refinement_case
+   *   and is of the vector size
+   *   <tt>GeometryInfo<dim>::n_children(refinement_case)</tt>. This is the
+   *   format used in FiniteElement, where we want to use this function mostly.
+   *
+   * @arg[in] isotropic_only If set to <code>true</code>, then this
+   *   function only computes data for the isotropic refinement case. The
+   *   other elements of the output vector are left untouched (but still
+   *   exist).
    */
   template <int dim, typename number, int spacedim>
   void compute_projection_matrices(
@@ -420,7 +416,7 @@ namespace FETools
     const bool isotropic_only = false);
 
   /**
-   * Projects scalar data defined in quadrature points to a finite element
+   * Project scalar data defined in quadrature points to a finite element
    * space on a single cell.
    *
    * What this function does is the following: assume that there is scalar
@@ -558,7 +554,6 @@ namespace FETools
 
 
 
-
   /**
    * This method implements the
    * FETools::compute_projection_from_quadrature_points_matrix method for
@@ -569,13 +564,36 @@ namespace FETools
    * information.
    */
   template <int dim, int spacedim>
-  void
-  compute_projection_from_face_quadrature_points_matrix (const FiniteElement<dim, spacedim> &fe,
-                                                         const Quadrature<dim-1>    &lhs_quadrature,
-                                                         const Quadrature<dim-1>    &rhs_quadrature,
-                                                         const typename DoFHandler<dim, spacedim>::active_cell_iterator &cell,
-                                                         const unsigned int          face,
-                                                         FullMatrix<double>         &X);
+  void compute_projection_from_face_quadrature_points_matrix(
+    const FiniteElement<dim, spacedim> &fe,
+    const Quadrature<dim - 1> &lhs_quadrature,
+    const Quadrature<dim - 1> &rhs_quadrature,
+    const typename DoFHandler<dim, spacedim>::active_cell_iterator &cell,
+    const unsigned int face,
+    FullMatrix<double> &X);
+
+
+
+  /**
+   * Wrapper around
+   * FiniteElement::convert_generalized_support_point_values_to_dof_values()
+   * that works with arbitrary number types.
+   *
+   * @param[in] finite_element The FiniteElement to compute dof values for.
+   * @param[in] support_point_values An array of size @p dofs_per_cell
+   *   (which equals the number of points the get_generalized_support_points()
+   *   function will return) where each element is a vector with as many entries
+   *   as the element has vector components. This array should contain
+   *   the values of a function at the generalized support points of the
+   *   finite element.
+   * @param[out] dof_values An array of size @p dofs_per_cell that contains
+   *   the node functionals of the element applied to the given function.
+   */
+  template <int dim, int spacedim, typename number>
+  void convert_generalized_support_point_values_to_dof_values(
+    const FiniteElement<dim, spacedim> &finite_element,
+    const std::vector<Vector<number> > &support_point_values,
+    std::vector<number>                &dof_values);
 
 
 
@@ -585,7 +603,7 @@ namespace FETools
    */
   //@{
   /**
-   * Gives the interpolation of a the @p dof1-function @p u1 to a @p
+   * Compute the interpolation of a the @p dof1-function @p u1 to a @p
    * dof2-function @p u2. @p dof1 and @p dof2 need to be DoFHandlers based on
    * the same triangulation.
    *
@@ -623,7 +641,7 @@ namespace FETools
                OutVector                           &u2);
 
   /**
-   * Gives the interpolation of a the @p dof1-function @p u1 to a @p
+   * Compute the interpolation of a the @p dof1-function @p u1 to a @p
    * dof2-function @p u2. @p dof1 and @p dof2 need to be DoFHandlers (or
    * hp::DoFHandlers) based on the same triangulation.  @p constraints is a
    * hanging node constraints object corresponding to @p dof2. This object is
@@ -649,7 +667,7 @@ namespace FETools
                     OutVector                           &u2);
 
   /**
-   * Gives the interpolation of the @p fe1-function @p u1 to a @p
+   * Compute the interpolation of the @p fe1-function @p u1 to a @p
    * fe2-function, and interpolates this to a second @p fe1-function named @p
    * u1_interpolated.
    *
@@ -672,7 +690,7 @@ namespace FETools
                          OutVector                          &u1_interpolated);
 
   /**
-   * Gives the interpolation of the @p dof1-function @p u1 to a @p
+   * Compute the interpolation of the @p dof1-function @p u1 to a @p
    * dof2-function, and interpolates this to a second @p dof1-function named
    * @p u1_interpolated.  @p constraints1 and @p constraints2 are the hanging
    * node constraints corresponding to @p dof1 and @p dof2, respectively.
@@ -692,7 +710,7 @@ namespace FETools
                          OutVector                      &u1_interpolated);
 
   /**
-   * Gives $(Id-I_h)z_1$ for a given @p dof1-function $z_1$, where $I_h$ is
+   * Compute $(Id-I_h)z_1$ for a given @p dof1-function $z_1$, where $I_h$ is
    * the interpolation from @p fe1 to @p fe2. The result $(Id-I_h)z_1$ is
    * written into @p z1_difference.
    *
@@ -707,7 +725,7 @@ namespace FETools
                                 OutVector                         &z1_difference);
 
   /**
-   * Gives $(Id-I_h)z_1$ for a given @p dof1-function $z_1$, where $I_h$ is
+   * Compute $(Id-I_h)z_1$ for a given @p dof1-function $z_1$, where $I_h$ is
    * the interpolation from @p fe1 to @p fe2. The result $(Id-I_h)z_1$ is
    * written into @p z1_difference.  @p constraints1 and @p constraints2 are
    * the hanging node constraints corresponding to @p dof1 and @p dof2,
@@ -715,8 +733,8 @@ namespace FETools
    * elements on grids with hanging nodes (locally refined grids) are
    * involved.
    *
-   * For parallel computations with PETSc, supply @p z1 with ghost elements
-   * and @p z1_difference without ghost elements.
+   * For parallel computations, supply @p z1 with ghost elements and @p
+   * z1_difference without ghost elements.
    */
   template <int dim, class InVector, class OutVector, int spacedim>
   void interpolation_difference(const DoFHandler<dim,spacedim> &dof1,
@@ -743,7 +761,7 @@ namespace FETools
                    OutVector                      &u2);
 
   /**
-   * Gives the patchwise extrapolation of a @p dof1 function @p z1 to a @p
+   * Compute the patchwise extrapolation of a @p dof1 function @p z1 to a @p
    * dof2 function @p z2.  @p dof1 and @p dof2 need to be DoFHandler objects
    * based on the same triangulation. This function is used, for example, for
    * extrapolating patchwise a piecewise linear solution to a piecewise
@@ -808,7 +826,7 @@ namespace FETools
                     OutVector                      &z2);
 
   /**
-   * Gives the patchwise extrapolation of a @p dof1 function @p z1 to a @p
+   * Compute the patchwise extrapolation of a @p dof1 function @p z1 to a @p
    * dof2 function @p z2.  @p dof1 and @p dof2 need to be DoFHandler objects
    * based on the same triangulation.  @p constraints is a hanging node
    * constraints object corresponding to @p dof2. This object is necessary
@@ -987,6 +1005,16 @@ namespace FETools
                           const std::vector<unsigned int>                       &multiplicities,
                           const bool do_tensor_product = true);
 
+    /**
+     * Same as above for an arbitrary number of parameters of type
+     * <code>std::pair<std::unique_ptr<FiniteElement<dim, spacedim>>, unsigned int></code>
+     * and <code>do_tensor_product = true</code>.
+     */
+    template <int dim, int spacedim>
+    FiniteElementData<dim>
+    multiply_dof_numbers
+    (const std::initializer_list<std::pair<std::unique_ptr<FiniteElement<dim, spacedim>>, unsigned int>> &fe_systems);
+
     /**
      * Same as above but for a specific number of sub-elements.
      */
@@ -994,13 +1022,13 @@ namespace FETools
     FiniteElementData<dim>
     multiply_dof_numbers (const FiniteElement<dim,spacedim> *fe1,
                           const unsigned int            N1,
-                          const FiniteElement<dim,spacedim> *fe2=NULL,
+                          const FiniteElement<dim,spacedim> *fe2=nullptr,
                           const unsigned int            N2=0,
-                          const FiniteElement<dim,spacedim> *fe3=NULL,
+                          const FiniteElement<dim,spacedim> *fe3=nullptr,
                           const unsigned int            N3=0,
-                          const FiniteElement<dim,spacedim> *fe4=NULL,
+                          const FiniteElement<dim,spacedim> *fe4=nullptr,
                           const unsigned int            N4=0,
-                          const FiniteElement<dim,spacedim> *fe5=NULL,
+                          const FiniteElement<dim,spacedim> *fe5=nullptr,
                           const unsigned int            N5=0);
 
     /**
@@ -1020,6 +1048,15 @@ namespace FETools
     compute_restriction_is_additive_flags (const std::vector<const FiniteElement<dim,spacedim>*> &fes,
                                            const std::vector<unsigned int>              &multiplicities);
 
+    /**
+     * Same as above for an arbitrary number of parameters of type
+     * <code>std::pair<std::unique_ptr<FiniteElement<dim, spacedim>>, unsigned int></code>.
+     */
+    template <int dim, int spacedim>
+    std::vector<bool>
+    compute_restriction_is_additive_flags
+    (const std::initializer_list<std::pair<std::unique_ptr<FiniteElement<dim, spacedim>>, unsigned int>> &fe_systems);
+
     /**
      * Take a @p FiniteElement object and return a boolean vector
      * describing the @p restriction_is_additive_flags (see the
@@ -1038,15 +1075,16 @@ namespace FETools
     std::vector<bool>
     compute_restriction_is_additive_flags (const FiniteElement<dim,spacedim> *fe1,
                                            const unsigned int        N1,
-                                           const FiniteElement<dim,spacedim> *fe2=NULL,
+                                           const FiniteElement<dim,spacedim> *fe2=nullptr,
                                            const unsigned int        N2=0,
-                                           const FiniteElement<dim,spacedim> *fe3=NULL,
+                                           const FiniteElement<dim,spacedim> *fe3=nullptr,
                                            const unsigned int        N3=0,
-                                           const FiniteElement<dim,spacedim> *fe4=NULL,
+                                           const FiniteElement<dim,spacedim> *fe4=nullptr,
                                            const unsigned int        N4=0,
-                                           const FiniteElement<dim,spacedim> *fe5=NULL,
+                                           const FiniteElement<dim,spacedim> *fe5=nullptr,
                                            const unsigned int        N5=0);
 
+
     /**
      * Compute the nonzero components for each shape function of a
      * composed finite element described by a list of finite elements
@@ -1069,6 +1107,16 @@ namespace FETools
                                 const std::vector<unsigned int>              &multiplicities,
                                 const bool do_tensor_product = true);
 
+    /**
+     * Same as above for an arbitrary number of parameters of type
+     * <code>std::pair<std::unique_ptr<FiniteElement<dim, spacedim>>, unsigned int></code>
+     * and <code>do_tensor_product = true</code>.
+     */
+    template <int dim, int spacedim>
+    std::vector<ComponentMask>
+    compute_nonzero_components
+    (const std::initializer_list<std::pair<std::unique_ptr<FiniteElement<dim, spacedim>>, unsigned int>> &fe_systems);
+
     /**
      * Compute the non-zero vector components of a composed finite
      * element. This function is similar to the previous one, except
@@ -1091,13 +1139,13 @@ namespace FETools
     std::vector<ComponentMask>
     compute_nonzero_components (const FiniteElement<dim,spacedim> *fe1,
                                 const unsigned int        N1,
-                                const FiniteElement<dim,spacedim> *fe2=NULL,
+                                const FiniteElement<dim,spacedim> *fe2=nullptr,
                                 const unsigned int        N2=0,
-                                const FiniteElement<dim,spacedim> *fe3=NULL,
+                                const FiniteElement<dim,spacedim> *fe3=nullptr,
                                 const unsigned int        N3=0,
-                                const FiniteElement<dim,spacedim> *fe4=NULL,
+                                const FiniteElement<dim,spacedim> *fe4=nullptr,
                                 const unsigned int        N4=0,
-                                const FiniteElement<dim,spacedim> *fe5=NULL,
+                                const FiniteElement<dim,spacedim> *fe5=nullptr,
                                 const unsigned int        N5=0,
                                 const bool do_tensor_product = true);
 
@@ -1170,9 +1218,8 @@ namespace FETools
    * If no finite element can be reconstructed from this string, an exception
    * of type @p FETools::ExcInvalidFEName is thrown.
    *
-   * The function returns a pointer to a newly create finite element. It is in
-   * the caller's responsibility to destroy the object pointed to at an
-   * appropriate later time.
+   * The function returns a std::unique_ptr to a newly created finite element
+   * meaning the caller obtains ownership over the returned object.
    *
    * Since the value of the template argument can't be deduced from the
    * (string) argument given to this function, you have to explicitly specify
@@ -1184,12 +1231,8 @@ namespace FETools
    * function, use the add_fe_name() function.  This function does not work if
    * one wants to get a codimension 1 finite element.
    */
-  template <int dim, int spacedim
-#ifdef DEAL_II_WITH_CXX11
-            =dim
-#endif
-            >
-  FiniteElement<dim, spacedim> *
+  template <int dim, int spacedim = dim>
+  std::unique_ptr<FiniteElement<dim, spacedim> >
   get_fe_by_name (const std::string &name);
 
 
@@ -1197,8 +1240,9 @@ namespace FETools
    * @deprecated Use get_fe_by_name() with two template parameters instead
    */
   template <int dim>
+  DEAL_II_DEPRECATED
   FiniteElement<dim,dim> *
-  get_fe_from_name (const std::string &name) DEAL_II_DEPRECATED;
+  get_fe_from_name (const std::string &name);
 
 
   /**
@@ -1353,12 +1397,85 @@ namespace FETools
 namespace FETools
 {
   template <class FE>
-  FiniteElement<FE::dimension, FE::space_dimension> *
+  std::unique_ptr<FiniteElement<FE::dimension, FE::space_dimension> >
   FEFactory<FE>::get (const unsigned int degree) const
   {
-    return new FE(degree);
+    return std_cxx14::make_unique<FE>(degree);
   }
-}
+
+  namespace Compositing
+  {
+    template <int dim, int spacedim>
+    std::vector<bool>
+    compute_restriction_is_additive_flags
+    (const std::initializer_list<std::pair<std::unique_ptr<FiniteElement<dim, spacedim>>, unsigned int>> &fe_systems)
+    {
+      std::vector<const FiniteElement<dim,spacedim>*> fes;
+      std::vector<unsigned int> multiplicities;
+
+      const auto extract =
+        [&fes, &multiplicities]
+        (const std::pair<std::unique_ptr<FiniteElement<dim, spacedim>>, unsigned int> &fe_system)
+      {
+        fes.push_back(fe_system.first.get());
+        multiplicities.push_back(fe_system.second);
+      };
+
+      for (const auto &p : fe_systems)
+        extract (p);
+
+      return compute_restriction_is_additive_flags (fes, multiplicities);
+    }
+
+
+
+    template <int dim, int spacedim>
+    FiniteElementData<dim>
+    multiply_dof_numbers
+    (const std::initializer_list<std::pair<std::unique_ptr<FiniteElement<dim, spacedim>>, unsigned int>> &fe_systems)
+    {
+      std::vector<const FiniteElement<dim,spacedim>*> fes;
+      std::vector<unsigned int> multiplicities;
+
+      const auto extract =
+        [&fes, &multiplicities]
+        (const std::pair<std::unique_ptr<FiniteElement<dim, spacedim>>, unsigned int> &fe_system)
+      {
+        fes.push_back(fe_system.first.get());
+        multiplicities.push_back(fe_system.second);
+      };
+
+      for (const auto &p : fe_systems)
+        extract (p);
+
+      return multiply_dof_numbers (fes, multiplicities, true);
+    }
+
+
+
+    template <int dim, int spacedim>
+    std::vector<ComponentMask>
+    compute_nonzero_components
+    (const std::initializer_list<std::pair<std::unique_ptr<FiniteElement<dim, spacedim>>, unsigned int>> &fe_systems)
+    {
+      std::vector<const FiniteElement<dim,spacedim>*> fes;
+      std::vector<unsigned int> multiplicities;
+
+      const auto extract =
+        [&fes, &multiplicities]
+        (const std::pair<std::unique_ptr<FiniteElement<dim, spacedim>>, unsigned int> &fe_system)
+      {
+        fes.push_back(fe_system.first.get());
+        multiplicities.push_back(fe_system.second);
+      };
+
+      for (const auto &p : fe_systems)
+        extract (p);
+
+      return compute_nonzero_components (fes, multiplicities, true);
+    }
+  } //Compositing
+} //FETools
 
 #endif
 
@@ -1366,7 +1483,4 @@ namespace FETools
 
 DEAL_II_NAMESPACE_CLOSE
 
-/*----------------------------   fe_tools.h     ---------------------------*/
-/* end of #ifndef dealii__fe_tools_H */
-#endif
-/*----------------------------   fe_tools.h     ---------------------------*/
+#endif /* dealii_fe_tools_H */
