@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 1998 - 2016 by the deal.II authors
+// Copyright (C) 1998 - 2018 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -13,8 +13,8 @@
 //
 // ---------------------------------------------------------------------
 
-#ifndef dealii__fe_update_flags_h
-#define dealii__fe_update_flags_h
+#ifndef dealii_fe_update_flags_h
+#define dealii_fe_update_flags_h
 
 
 #include <deal.II/base/config.h>
@@ -99,7 +99,18 @@ enum UpdateFlags
   update_boundary_forms = 0x0010,
   //! Transformed quadrature points
   /**
-   * Compute the quadrature points transformed into real cell coordinates.
+   * Compute the quadrature points location in real cell coordinates.
+   *
+   * FEValues objects take the quadrature point locations on the
+   * reference cell as an argument of the constructor (via the
+   * Quadrature object). For most finite elements, knowing the
+   * location of quadrature points on the reference cell is all that
+   * is necessary to evaluate shape functions, evaluate the mapping,
+   * and other things. On the other hand, if you want to evaluate a
+   * right hand side function $f(\mathbf x_q)$ at quadrature point
+   * locations $\mathbf x_q$ on the real cell, you need to pass this
+   * flag to the FEValues constructor to make sure you can later
+   * access them.
    */
   update_quadrature_points = 0x0020,
   //! Transformed quadrature weights
@@ -242,7 +253,8 @@ enum UpdateFlags
  */
 template <class StreamType>
 inline
-StreamType &operator << (StreamType &s, UpdateFlags u)
+StreamType &operator << (StreamType &s,
+                         const UpdateFlags u)
 {
   s << " UpdateFlags|";
   if (u & update_values)                                  s << "values|";
@@ -281,7 +293,8 @@ StreamType &operator << (StreamType &s, UpdateFlags u)
  */
 inline
 UpdateFlags
-operator | (UpdateFlags f1, UpdateFlags f2)
+operator | (const UpdateFlags f1,
+            const UpdateFlags f2)
 {
   return static_cast<UpdateFlags> (
            static_cast<unsigned int> (f1) |
@@ -299,7 +312,8 @@ operator | (UpdateFlags f1, UpdateFlags f2)
  */
 inline
 UpdateFlags &
-operator |= (UpdateFlags &f1, UpdateFlags f2)
+operator |= (UpdateFlags &f1,
+             const UpdateFlags f2)
 {
   f1 = f1 | f2;
   return f1;
@@ -317,7 +331,8 @@ operator |= (UpdateFlags &f1, UpdateFlags f2)
  */
 inline
 UpdateFlags
-operator & (UpdateFlags f1, UpdateFlags f2)
+operator & (const UpdateFlags f1,
+            const UpdateFlags f2)
 {
   return static_cast<UpdateFlags> (
            static_cast<unsigned int> (f1) &
@@ -333,7 +348,8 @@ operator & (UpdateFlags f1, UpdateFlags f2)
  */
 inline
 UpdateFlags &
-operator &= (UpdateFlags &f1, UpdateFlags f2)
+operator &= (UpdateFlags &f1,
+             const UpdateFlags f2)
 {
   f1 = f1 & f2;
   return f1;
@@ -378,7 +394,7 @@ namespace CellSimilarity
 
 namespace internal
 {
-  namespace FEValues
+  namespace FEValuesImplementation
   {
     /**
      * A class that stores all of the mapping related data used in
