@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 1998 - 2015 by the deal.II authors
+// Copyright (C) 1998 - 2018 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -13,8 +13,8 @@
 //
 // ---------------------------------------------------------------------
 
-#ifndef dealii__fe_values_extractors_h
-#define dealii__fe_values_extractors_h
+#ifndef dealii_fe_values_extractors_h
+#define dealii_fe_values_extractors_h
 
 
 #include <deal.II/base/config.h>
@@ -168,7 +168,7 @@ namespace FEValuesExtractors
 
 
   /**
-   * Extractor for a (possible non-)symmetric tensor of a rank specified by
+   * Extractor for a general tensor of a given rank specified by
    * the template argument. For a second order tensor, this represents a
    * collection of <code>(dim*dim)</code> components of a vector-valued
    * element. The value of <code>dim</code> is defined by the FEValues object
