@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 2000 - 2016 by the deal.II authors
+// Copyright (C) 2000 - 2018 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -13,21 +13,22 @@
 //
 // ---------------------------------------------------------------------
 
-#ifndef dealii__mapping_h
-#define dealii__mapping_h
+#ifndef dealii_mapping_h
+#define dealii_mapping_h
 
 
 #include <deal.II/base/config.h>
 #include <deal.II/base/derivative_form.h>
-#include <deal.II/base/std_cxx11/array.h>
-#include <deal.II/base/array_view.h>
 #include <deal.II/grid/tria.h>
 #include <deal.II/fe/fe_update_flags.h>
 
+#include <array>
 #include <cmath>
+#include <memory>
 
 DEAL_II_NAMESPACE_OPEN
 
+template <typename ElementType> class ArrayView;
 template <int dim> class Quadrature;
 template <int dim, int spacedim> class FEValues;
 template <int dim, int spacedim> class FEValuesBase;
@@ -291,7 +292,8 @@ public:
   /**
    * Virtual destructor.
    */
-  virtual ~Mapping ();
+  virtual
+  ~Mapping () override = default;
 
   /**
    * Return a pointer to a copy of the present object. The caller of this copy
@@ -303,7 +305,7 @@ public:
    * This function is mainly used by the hp::MappingCollection class.
    */
   virtual
-  Mapping<dim,spacedim> *clone () const = 0;
+  std::unique_ptr<Mapping<dim,spacedim>> clone () const = 0;
 
   /**
    * Return the mapped vertices of a cell.
@@ -320,7 +322,7 @@ public:
    * <code>cell-@>vertex(v)</code>.
    */
   virtual
-  std_cxx11::array<Point<spacedim>, GeometryInfo<dim>::vertices_per_cell>
+  std::array<Point<spacedim>, GeometryInfo<dim>::vertices_per_cell>
   get_vertices (const typename Triangulation<dim,spacedim>::cell_iterator &cell) const;
 
   /**
@@ -343,7 +345,7 @@ public:
    */
 
   /**
-   * Maps the point @p p on the unit cell to the corresponding point on the
+   * Map the point @p p on the unit cell to the corresponding point on the
    * real cell @p cell.
    *
    * @param cell Iterator to the cell that will be used to define the mapping.
@@ -359,7 +361,7 @@ public:
                                const Point<dim>                                          &p) const = 0;
 
   /**
-   * Maps the point @p p on the real @p cell to the corresponding point on the
+   * Map the point @p p on the real @p cell to the corresponding point on the
    * unit cell, and return its coordinates. This function provides the inverse
    * of the mapping provided by transform_unit_to_real_cell().
    *
@@ -392,7 +394,7 @@ public:
                                const Point<spacedim>                                     &p) const = 0;
 
   /**
-   * Transforms the point @p p on the real @p cell to the corresponding point
+   * Transform the point @p p on the real @p cell to the corresponding point
    * on the unit cell, and then projects it to a dim-1  point on the face with
    * the given face number @p face_no. Ideally the point @p p is near the face
    * @p face_no, but any point in the cell can technically be projected.
@@ -525,12 +527,6 @@ public:
    */
   class InternalDataBase
   {
-  private:
-    /**
-     * Copy construction is forbidden.
-     */
-    InternalDataBase (const InternalDataBase &);
-
   public:
     /**
      * Constructor. Sets update_flags to @p update_default and @p first_cell
@@ -538,10 +534,16 @@ public:
      */
     InternalDataBase ();
 
+    /**
+     * Copy construction is forbidden.
+     */
+    InternalDataBase (const InternalDataBase &) = delete;
+
     /**
      * Virtual destructor for derived classes
      */
-    virtual ~InternalDataBase ();
+    virtual
+    ~InternalDataBase () = default;
 
     /**
      * A set of update flags specifying the kind of information that an
@@ -562,7 +564,8 @@ public:
     /**
      * Return an estimate (in bytes) or the memory consumption of this object.
      */
-    virtual std::size_t memory_consumption () const;
+    virtual
+    std::size_t memory_consumption () const;
   };
 
 
@@ -643,7 +646,7 @@ protected:
    * the returned object, knowing its real (derived) type.
    */
   virtual
-  InternalDataBase *
+  std::unique_ptr<InternalDataBase>
   get_data (const UpdateFlags      update_flags,
             const Quadrature<dim> &quadrature) const = 0;
 
@@ -675,7 +678,7 @@ protected:
    * the returned object, knowing its real (derived) type.
    */
   virtual
-  InternalDataBase *
+  std::unique_ptr<InternalDataBase>
   get_face_data (const UpdateFlags        update_flags,
                  const Quadrature<dim-1> &quadrature) const = 0;
 
@@ -708,7 +711,7 @@ protected:
    * the returned object, knowing its real (derived) type.
    */
   virtual
-  InternalDataBase *
+  std::unique_ptr<InternalDataBase>
   get_subface_data (const UpdateFlags        update_flags,
                     const Quadrature<dim-1> &quadrature) const = 0;
 
@@ -801,7 +804,7 @@ protected:
                   const CellSimilarity::Similarity                              cell_similarity,
                   const Quadrature<dim>                                        &quadrature,
                   const typename Mapping<dim,spacedim>::InternalDataBase       &internal_data,
-                  dealii::internal::FEValues::MappingRelatedData<dim,spacedim> &output_data) const = 0;
+                  dealii::internal::FEValuesImplementation::MappingRelatedData<dim,spacedim> &output_data) const = 0;
 
   /**
    * This function is the equivalent to Mapping::fill_fe_values(), but for
@@ -827,12 +830,13 @@ protected:
    * filled; which ones need to be filled is determined by the update flags
    * stored inside the @p internal_data object.
    */
-  virtual void
+  virtual
+  void
   fill_fe_face_values (const typename Triangulation<dim,spacedim>::cell_iterator    &cell,
                        const unsigned int                                            face_no,
                        const Quadrature<dim-1>                                      &quadrature,
                        const typename Mapping<dim,spacedim>::InternalDataBase       &internal_data,
-                       dealii::internal::FEValues::MappingRelatedData<dim,spacedim> &output_data) const = 0;
+                       dealii::internal::FEValuesImplementation::MappingRelatedData<dim,spacedim> &output_data) const = 0;
 
   /**
    * This function is the equivalent to Mapping::fill_fe_values(), but for
@@ -860,13 +864,14 @@ protected:
    * filled; which ones need to be filled is determined by the update flags
    * stored inside the @p internal_data object.
    */
-  virtual void
+  virtual
+  void
   fill_fe_subface_values (const typename Triangulation<dim,spacedim>::cell_iterator     &cell,
                           const unsigned int                                             face_no,
                           const unsigned int                                             subface_no,
                           const Quadrature<dim-1>                                       &quadrature,
                           const typename Mapping<dim,spacedim>::InternalDataBase        &internal_data,
-                          dealii::internal::FEValues::MappingRelatedData<dim, spacedim> &output_data) const = 0;
+                          dealii::internal::FEValuesImplementation::MappingRelatedData<dim, spacedim> &output_data) const = 0;
 
   /**
    * @}
@@ -921,8 +926,8 @@ public:
    * cell is also represented by a vector field, but again transforms
    * differently, namely by the Piola transform
    * @f[
-   *  \mathbf u(\mathbf x) = \frac{1}{\text{det}\;J(\mathbf x)}
-   * J(\mathbf x) \hat{\mathbf  u}(\mathbf x).
+   *  \mathbf u(\mathbf x) = \frac{1}{\text{det}\;J(\hat{\mathbf x})}
+   * J(\hat{\mathbf x}) \hat{\mathbf  u}(\hat{\mathbf x}).
    * @f]
    * </ul>
    *
@@ -1022,11 +1027,10 @@ public:
    * J(\hat{\mathbf  x})^{-1}.
    * @f]
    * <li> @p mapping_piola_gradient: it assumes $\mathbf u(\mathbf x) =
-   * \frac{1}{\text{det}\;J(\mathbf x)} J(\mathbf x) \hat{\mathbf  u}(\mathbf
-   * x)$ so that
+   * \frac{1}{\text{det}\;J(\hat{\mathbf x})} J(\hat{\mathbf x}) \hat{\mathbf  u}(\hat{\mathbf x})$ so that
    * @f[
    * \mathbf T(\mathbf x) =
-   * \frac{1}{\text{det}\;J(\mathbf x)}
+   * \frac{1}{\text{det}\;J(\hat{\mathbf x})}
    * J(\hat{\mathbf  x}) \hat{\mathbf  T}(\hat{\mathbf  x})
    * J(\hat{\mathbf  x})^{-1}.
    * @f]
@@ -1129,11 +1133,11 @@ public:
    * J_{jJ}(\hat{\mathbf  x})^{-1} J_{kK}(\hat{\mathbf  x})^{-1}.
    * @f]
    * <li> @p mapping_piola_hessian: it assumes $\mathbf u_i(\mathbf x) =
-   * \frac{1}{\text{det}\;J(\mathbf x)} J_{iI}(\mathbf x) \hat{\mathbf
-   * u}(\mathbf x)$ so that
+   * \frac{1}{\text{det}\;J(\hat{\mathbf x})} J_{iI}(\hat{\mathbf x}) \hat{\mathbf
+   * u}(\hat{\mathbf x})$ so that
    * @f[
    * \mathbf T_{ijk}(\mathbf x) =
-   * \frac{1}{\text{det}\;J(\mathbf x)}
+   * \frac{1}{\text{det}\;J(\hat{\mathbf x})}
    * J_{iI}(\hat{\mathbf  x}) \hat{\mathbf  T}_{IJK}(\hat{\mathbf  x})
    * J_{jJ}(\hat{\mathbf  x})^{-1} J_{kK}(\hat{\mathbf  x})^{-1}.
    * @f]
