@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 2001 - 2016 by the deal.II authors
+// Copyright (C) 2001 - 2018 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -13,8 +13,8 @@
 //
 // ---------------------------------------------------------------------
 
-#ifndef dealii__mapping_c1_h
-#define dealii__mapping_c1_h
+#ifndef dealii_mapping_c1_h
+#define dealii_mapping_c1_h
 
 
 #include <deal.II/base/config.h>
@@ -33,12 +33,9 @@ DEAL_II_NAMESPACE_OPEN
  * class chooses them such that the discretized boundary is globally
  * continuously differentiable.
  *
- * To use this class, make sure that the Boundary::get_normals_at_vertices()
- * function is implemented for the user's boundary object.
- *
  * @author Wolfgang Bangerth, 2001, 2015
  */
-template<int dim, int spacedim=dim>
+template <int dim, int spacedim=dim>
 class MappingC1 : public MappingQ<dim,spacedim>
 {
 public:
@@ -53,7 +50,7 @@ public:
    * then assumes ownership of it.
    */
   virtual
-  Mapping<dim,spacedim> *clone () const;
+  std::unique_ptr<Mapping<dim,spacedim>> clone () const override;
 
 protected:
 
@@ -76,32 +73,28 @@ protected:
      * located on bounding lines to the vector @p a. Points located on the
      * line but on vertices are not included.
      *
-     * Needed by the <tt>compute_support_points_simple(laplace)</tt>
-     * functions. For <tt>dim=1</tt> this function is empty.
-     *
      * This function chooses the respective points not such that they are
      * interpolating the boundary (as does the base class), but rather such
      * that the resulting cubic mapping is a continuous one.
      */
-    virtual void
+    virtual
+    void
     add_line_support_points (const typename Triangulation<dim>::cell_iterator &cell,
-                             std::vector<Point<dim> > &a) const;
+                             std::vector<Point<dim> > &a) const override;
 
     /**
      * For <tt>dim=3</tt>. Append the support points of all shape functions
      * located on bounding faces (quads in 3d) to the vector @p a. Points
      * located on the line but on vertices are not included.
      *
-     * Needed by the @p compute_support_points_laplace function. For
-     * <tt>dim=1</tt> and 2 this function is empty.
-     *
      * This function chooses the respective points not such that they are
      * interpolating the boundary (as does the base class), but rather such
      * that the resulting cubic mapping is a continuous one.
      */
-    virtual void
+    virtual
+    void
     add_quad_support_points(const typename Triangulation<dim>::cell_iterator &cell,
-                            std::vector<Point<dim> > &a) const;
+                            std::vector<Point<dim> > &a) const override;
   };
 };
 
