@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 2001 - 2016 by the deal.II authors
+// Copyright (C) 2001 - 2018 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -13,8 +13,8 @@
 //
 // ---------------------------------------------------------------------
 
-#ifndef dealii__mapping_fe_h
-#define dealii__mapping_fe_h
+#ifndef dealii_mapping_fe_h
+#define dealii_mapping_fe_h
 
 
 #include <deal.II/base/config.h>
@@ -24,8 +24,9 @@
 #include <deal.II/dofs/dof_handler.h>
 #include <deal.II/fe/mapping.h>
 #include <deal.II/fe/fe.h>
+#include <deal.II/fe/fe_values.h>
 
-#include <deal.II/base/std_cxx11/array.h>
+#include <array>
 
 
 DEAL_II_NAMESPACE_OPEN
@@ -111,7 +112,7 @@ public:
    */
   MappingFEField (const DoFHandlerType &euler_dof_handler,
                   const VectorType     &euler_vector,
-                  const ComponentMask   mask = ComponentMask());
+                  const ComponentMask   &mask = ComponentMask());
 
   /**
    * Copy constructor.
@@ -123,16 +124,26 @@ public:
    * then assumes ownership of it.
    */
   virtual
-  Mapping<dim,spacedim> *clone () const;
-
-
+  std::unique_ptr<Mapping<dim,spacedim>> clone () const override;
 
   /**
-   * Always returns @p false.
+   * See the documentation of Mapping::preserves_vertex_locations()
+   * for the purpose of this function. The implementation in this
+   * class always returns @p false.
    */
   virtual
-  bool preserves_vertex_locations () const;
+  bool preserves_vertex_locations () const override;
 
+  /**
+   * Return the mapped vertices of a cell.
+   *
+   * This mapping ignores the vertices of the Triangulation it is associated to,
+   * and constructs the position of the vertices according to the @p euler_vector
+   * that was passed at construction time.
+   */
+  virtual
+  std::array<Point<spacedim>, GeometryInfo<dim>::vertices_per_cell>
+  get_vertices (const typename Triangulation<dim,spacedim>::cell_iterator &cell) const override;
 
   /**
    * @name Mapping points between reference and real cells
@@ -143,13 +154,13 @@ public:
   virtual
   Point<spacedim>
   transform_unit_to_real_cell (const typename Triangulation<dim,spacedim>::cell_iterator &cell,
-                               const Point<dim>                                 &p) const;
+                               const Point<dim>                                 &p) const override;
 
   // for documentation, see the Mapping base class
   virtual
   Point<dim>
   transform_real_to_unit_cell (const typename Triangulation<dim,spacedim>::cell_iterator &cell,
-                               const Point<spacedim>                            &p) const;
+                               const Point<spacedim>                            &p) const override;
 
   /**
    * @}
@@ -166,7 +177,7 @@ public:
   transform (const ArrayView<const Tensor<1,dim> >                  &input,
              const MappingType                                       type,
              const typename Mapping<dim,spacedim>::InternalDataBase &internal,
-             const ArrayView<Tensor<1,spacedim> >                   &output) const;
+             const ArrayView<Tensor<1,spacedim> >                   &output) const override;
 
   // for documentation, see the Mapping base class
   virtual
@@ -174,7 +185,7 @@ public:
   transform (const ArrayView<const DerivativeForm<1, dim, spacedim> > &input,
              const MappingType                                         type,
              const typename Mapping<dim,spacedim>::InternalDataBase   &internal,
-             const ArrayView<Tensor<2,spacedim> >                     &output) const;
+             const ArrayView<Tensor<2,spacedim> >                     &output) const override;
 
   // for documentation, see the Mapping base class
   virtual
@@ -182,7 +193,7 @@ public:
   transform (const ArrayView<const Tensor<2, dim> >                 &input,
              const MappingType                                       type,
              const typename Mapping<dim,spacedim>::InternalDataBase &internal,
-             const ArrayView<Tensor<2,spacedim> >                   &output) const;
+             const ArrayView<Tensor<2,spacedim> >                   &output) const override;
 
   // for documentation, see the Mapping base class
   virtual
@@ -190,7 +201,7 @@ public:
   transform (const ArrayView<const DerivativeForm<2, dim, spacedim> > &input,
              const MappingType                                         type,
              const typename Mapping<dim,spacedim>::InternalDataBase   &internal,
-             const ArrayView<Tensor<3,spacedim> >                     &output) const;
+             const ArrayView<Tensor<3,spacedim> >                     &output) const override;
 
   // for documentation, see the Mapping base class
   virtual
@@ -198,13 +209,12 @@ public:
   transform (const ArrayView<const Tensor<3, dim> >                 &input,
              const MappingType                                       type,
              const typename Mapping<dim,spacedim>::InternalDataBase &internal,
-             const ArrayView<Tensor<3,spacedim> >                   &output) const;
+             const ArrayView<Tensor<3,spacedim> >                   &output) const override;
 
   /**
    * @}
    */
 
-
   /**
    * Return the degree of the mapping, i.e. the value which was passed to the
    * constructor.
@@ -232,7 +242,7 @@ private:
   // documentation can be found in Mapping::requires_update_flags()
   virtual
   UpdateFlags
-  requires_update_flags (const UpdateFlags update_flags) const;
+  requires_update_flags (const UpdateFlags update_flags) const override;
 
 public:
   /**
@@ -253,7 +263,7 @@ public:
      * Constructor.
      */
     InternalData(const FiniteElement<dim,spacedim> &fe,
-                 const ComponentMask mask);
+                 const ComponentMask &mask);
 
     /**
      * Shape function at quadrature point. Shape functions are in tensor
@@ -319,7 +329,8 @@ public:
     /**
      * Return an estimate (in bytes) or the memory consumption of this object.
      */
-    virtual std::size_t memory_consumption () const;
+    virtual
+    std::size_t memory_consumption () const override;
 
     /**
      * Values of shape functions. Access by function @p shape.
@@ -372,7 +383,7 @@ public:
      *
      * Filled once.
      */
-    std_cxx11::array<std::vector<Tensor<1,dim> >, GeometryInfo<dim>::faces_per_cell *(dim-1)> unit_tangentials;
+    std::array<std::vector<Tensor<1,dim> >, GeometryInfo<dim>::faces_per_cell *(dim-1)> unit_tangentials;
 
     /**
      * Number of shape functions. If this is a Q1 mapping, then it is simply
@@ -442,21 +453,21 @@ private:
 
   // documentation can be found in Mapping::get_data()
   virtual
-  InternalData *
+  std::unique_ptr<typename Mapping<dim,spacedim>::InternalDataBase>
   get_data (const UpdateFlags,
-            const Quadrature<dim> &quadrature) const;
+            const Quadrature<dim> &quadrature) const override;
 
   // documentation can be found in Mapping::get_face_data()
   virtual
-  typename Mapping<dim,spacedim>::InternalDataBase *
+  std::unique_ptr<typename Mapping<dim,spacedim>::InternalDataBase>
   get_face_data (const UpdateFlags flags,
-                 const Quadrature<dim-1>& quadrature) const;
+                 const Quadrature<dim-1>& quadrature) const override;
 
   // documentation can be found in Mapping::get_subface_data()
   virtual
-  typename Mapping<dim,spacedim>::InternalDataBase *
+  std::unique_ptr<typename Mapping<dim,spacedim>::InternalDataBase>
   get_subface_data (const UpdateFlags flags,
-                    const Quadrature<dim-1>& quadrature) const;
+                    const Quadrature<dim-1>& quadrature) const override;
 
   // documentation can be found in Mapping::fill_fe_values()
   virtual
@@ -465,24 +476,26 @@ private:
                   const CellSimilarity::Similarity                           cell_similarity,
                   const Quadrature<dim>                                     &quadrature,
                   const typename Mapping<dim,spacedim>::InternalDataBase    &internal_data,
-                  internal::FEValues::MappingRelatedData<dim,spacedim>      &output_data) const;
+                  internal::FEValuesImplementation::MappingRelatedData<dim,spacedim>      &output_data) const override;
 
   // documentation can be found in Mapping::fill_fe_face_values()
-  virtual void
+  virtual
+  void
   fill_fe_face_values (const typename Triangulation<dim,spacedim>::cell_iterator &cell,
                        const unsigned int                                         face_no,
                        const Quadrature<dim-1>                                   &quadrature,
                        const typename Mapping<dim,spacedim>::InternalDataBase    &internal_data,
-                       internal::FEValues::MappingRelatedData<dim,spacedim>      &output_data) const;
+                       internal::FEValuesImplementation::MappingRelatedData<dim,spacedim>      &output_data) const override;
 
   // documentation can be found in Mapping::fill_fe_subface_values()
-  virtual void
+  virtual
+  void
   fill_fe_subface_values (const typename Triangulation<dim,spacedim>::cell_iterator &cell,
                           const unsigned int                                         face_no,
                           const unsigned int                                         subface_no,
                           const Quadrature<dim-1>                                   &quadrature,
                           const typename Mapping<dim,spacedim>::InternalDataBase    &internal_data,
-                          internal::FEValues::MappingRelatedData<dim,spacedim>      &output_data) const;
+                          internal::FEValuesImplementation::MappingRelatedData<dim,spacedim>      &output_data) const override;
 
   /**
    * @}
@@ -494,16 +507,6 @@ private:
    */
   SmartPointer<const VectorType, MappingFEField<dim,spacedim,VectorType,DoFHandlerType> > euler_vector;
 
-  /**
-   * A FiniteElement object which is only needed in 3D, since it knows how to
-   * reorder shape functions/DoFs on non-standard faces. This is used to
-   * reorder support points in the same way. We could make this a pointer to
-   * prevent construction in 1D and 2D, but since memory and time requirements
-   * are not particularly high this seems unnecessary at the moment.
-   */
-  SmartPointer<const FiniteElement<dim,spacedim>, MappingFEField<dim,spacedim,VectorType,DoFHandlerType> > fe;
-
-
   /**
    * Pointer to the DoFHandler to which the mapping vector is associated.
    */
@@ -511,7 +514,7 @@ private:
 
 private:
   /**
-   * Transforms a point @p p on the unit cell to the point @p p_real on the
+   * Transform a point @p p on the unit cell to the point @p p_real on the
    * real cell @p cell and returns @p p_real.
    *
    * This function is called by @p transform_unit_to_real_cell and multiple
@@ -530,7 +533,7 @@ private:
 
 
   /**
-   * Transforms the point @p p on the real cell to the corresponding point on
+   * Transform the point @p p on the real cell to the corresponding point on
    * the unit cell @p cell by a Newton iteration.
    *
    * Takes a reference to an @p InternalData that is assumed to be previously
@@ -558,7 +561,8 @@ private:
   /**
    * See the documentation of the base class for detailed information.
    */
-  virtual void
+  virtual
+  void
   compute_shapes_virtual (const std::vector<Point<dim> > &unit_points,
                           typename MappingFEField<dim, spacedim, VectorType, DoFHandlerType>::InternalData &data) const;
 
@@ -578,6 +582,17 @@ private:
    */
   std::vector<unsigned int> fe_to_real;
 
+  /**
+   * FEValues object used to query the given finite element field at the
+   * support points in the reference configuration.
+   */
+  mutable FEValues<dim,spacedim> fe_values;
+
+  /**
+   * A variable to guard access to the fe_values variable.
+   */
+  mutable Threads::Mutex fe_values_mutex;
+
   void
   compute_data (const UpdateFlags      update_flags,
                 const Quadrature<dim>  &q,
@@ -603,9 +618,6 @@ private:
 
 #ifndef DOXYGEN
 
-
-
-
 #endif // DOXYGEN
 
 DEAL_II_NAMESPACE_CLOSE
