@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 2016 by the deal.II authors
+// Copyright (C) 2016 - 2018 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -13,8 +13,8 @@
 //
 // ---------------------------------------------------------------------
 
-#ifndef dealii__mapping_manifold_h
-#define dealii__mapping_manifold_h
+#ifndef dealii_mapping_manifold_h
+#define dealii_mapping_manifold_h
 
 
 #include <deal.II/base/config.h>
@@ -63,7 +63,7 @@ public:
   /**
    * Constructor.
    */
-  MappingManifold ();
+  MappingManifold () = default;
 
   /**
    * Copy constructor.
@@ -72,14 +72,14 @@ public:
 
   // for documentation, see the Mapping base class
   virtual
-  Mapping<dim,spacedim> *clone () const;
+  std::unique_ptr<Mapping<dim,spacedim>> clone () const override;
 
   /**
    * Always returns @p true because this class assumes that the
    * vertices always lies on the underlying Manifold.
    */
   virtual
-  bool preserves_vertex_locations () const;
+  bool preserves_vertex_locations () const override;
 
   /**
    * @name Mapping points between reference and real cells
@@ -90,13 +90,13 @@ public:
   virtual
   Point<spacedim>
   transform_unit_to_real_cell (const typename Triangulation<dim,spacedim>::cell_iterator &cell,
-                               const Point<dim>                                 &p) const;
+                               const Point<dim>                                 &p) const override;
 
   // for documentation, see the Mapping base class
   virtual
   Point<dim>
   transform_real_to_unit_cell (const typename Triangulation<dim,spacedim>::cell_iterator &cell,
-                               const Point<spacedim>                            &p) const;
+                               const Point<spacedim>                            &p) const override;
 
   /**
    * @}
@@ -113,7 +113,7 @@ public:
   transform (const ArrayView<const Tensor<1,dim> >                  &input,
              const MappingType                                       type,
              const typename Mapping<dim,spacedim>::InternalDataBase &internal,
-             const ArrayView<Tensor<1,spacedim> >                   &output) const;
+             const ArrayView<Tensor<1,spacedim> >                   &output) const override;
 
   // for documentation, see the Mapping base class
   virtual
@@ -121,7 +121,7 @@ public:
   transform (const ArrayView<const DerivativeForm<1, dim, spacedim> > &input,
              const MappingType                                         type,
              const typename Mapping<dim,spacedim>::InternalDataBase   &internal,
-             const ArrayView<Tensor<2,spacedim> >                     &output) const;
+             const ArrayView<Tensor<2,spacedim> >                     &output) const override;
 
   // for documentation, see the Mapping base class
   virtual
@@ -129,7 +129,7 @@ public:
   transform (const ArrayView<const Tensor<2, dim> >                 &input,
              const MappingType                                       type,
              const typename Mapping<dim,spacedim>::InternalDataBase &internal,
-             const ArrayView<Tensor<2,spacedim> >                   &output) const;
+             const ArrayView<Tensor<2,spacedim> >                   &output) const override;
 
   // for documentation, see the Mapping base class
   virtual
@@ -137,7 +137,7 @@ public:
   transform (const ArrayView<const DerivativeForm<2, dim, spacedim> > &input,
              const MappingType                                         type,
              const typename Mapping<dim,spacedim>::InternalDataBase   &internal,
-             const ArrayView<Tensor<3,spacedim> >                     &output) const;
+             const ArrayView<Tensor<3,spacedim> >                     &output) const override;
 
   // for documentation, see the Mapping base class
   virtual
@@ -145,7 +145,7 @@ public:
   transform (const ArrayView<const Tensor<3, dim> >                 &input,
              const MappingType                                       type,
              const typename Mapping<dim,spacedim>::InternalDataBase &internal,
-             const ArrayView<Tensor<3,spacedim> >                   &output) const;
+             const ArrayView<Tensor<3,spacedim> >                   &output) const override;
 
   /**
    * @}
@@ -174,7 +174,7 @@ public:
     /**
      * Constructor.
      */
-    InternalData();
+    InternalData() = default;
 
     /**
      * Initialize the object's member variables related to cell data based on
@@ -217,7 +217,8 @@ public:
     /**
      * Return an estimate (in bytes) or the memory consumption of this object.
      */
-    virtual std::size_t memory_consumption () const;
+    virtual
+    std::size_t memory_consumption () const override;
 
     /**
      * The current cell vertices.
@@ -251,7 +252,7 @@ public:
      * we call this function with a Quadrature formula constructed
      * using the vertices of the current cell, and the values of the
      * basis functions of an FE_Q(1) finite element evaluated at the
-     * quadrature point itslef. While the vertices of the cell change
+     * quadrature point itself. While the vertices of the cell change
      * for every cell, the weights can be computed once for each
      * quadrature point. We store this information in the following
      * variable, where the first index runs through the quadrature
@@ -291,7 +292,7 @@ public:
      *
      * Filled once.
      */
-    std_cxx11::array<std::vector<Tensor<1,dim> >, GeometryInfo<dim>::faces_per_cell *(dim-1)> unit_tangentials;
+    std::array<std::vector<Tensor<1,dim> >, GeometryInfo<dim>::faces_per_cell *(dim-1)> unit_tangentials;
 
     /**
      * Tensors of covariant transformation at each of the quadrature points.
@@ -336,25 +337,25 @@ public:
   // documentation can be found in Mapping::requires_update_flags()
   virtual
   UpdateFlags
-  requires_update_flags (const UpdateFlags update_flags) const;
+  requires_update_flags (const UpdateFlags update_flags) const override;
 
   // documentation can be found in Mapping::get_data()
   virtual
-  InternalData *
+  std::unique_ptr<typename Mapping<dim,spacedim>::InternalDataBase>
   get_data (const UpdateFlags,
-            const Quadrature<dim> &quadrature) const;
+            const Quadrature<dim> &quadrature) const override;
 
   // documentation can be found in Mapping::get_face_data()
   virtual
-  InternalData *
+  std::unique_ptr<typename Mapping<dim,spacedim>::InternalDataBase>
   get_face_data (const UpdateFlags flags,
-                 const Quadrature<dim-1>& quadrature) const;
+                 const Quadrature<dim-1>& quadrature) const override;
 
   // documentation can be found in Mapping::get_subface_data()
   virtual
-  InternalData *
+  std::unique_ptr<typename Mapping<dim,spacedim>::InternalDataBase>
   get_subface_data (const UpdateFlags flags,
-                    const Quadrature<dim-1>& quadrature) const;
+                    const Quadrature<dim-1>& quadrature) const override;
 
   // documentation can be found in Mapping::fill_fe_values()
   virtual
@@ -363,24 +364,26 @@ public:
                   const CellSimilarity::Similarity                               cell_similarity,
                   const Quadrature<dim>                                         &quadrature,
                   const typename Mapping<dim,spacedim>::InternalDataBase        &internal_data,
-                  dealii::internal::FEValues::MappingRelatedData<dim, spacedim> &output_data) const;
+                  dealii::internal::FEValuesImplementation::MappingRelatedData<dim, spacedim> &output_data) const override;
 
   // documentation can be found in Mapping::fill_fe_face_values()
-  virtual void
+  virtual
+  void
   fill_fe_face_values (const typename Triangulation<dim,spacedim>::cell_iterator     &cell,
                        const unsigned int                                             face_no,
                        const Quadrature<dim-1>                                       &quadrature,
                        const typename Mapping<dim,spacedim>::InternalDataBase        &internal_data,
-                       dealii::internal::FEValues::MappingRelatedData<dim, spacedim> &output_data) const;
+                       dealii::internal::FEValuesImplementation::MappingRelatedData<dim, spacedim> &output_data) const override;
 
   // documentation can be found in Mapping::fill_fe_subface_values()
-  virtual void
+  virtual
+  void
   fill_fe_subface_values (const typename Triangulation<dim,spacedim>::cell_iterator     &cell,
                           const unsigned int                                             face_no,
                           const unsigned int                                             subface_no,
                           const Quadrature<dim-1>                                       &quadrature,
                           const typename Mapping<dim,spacedim>::InternalDataBase        &internal_data,
-                          dealii::internal::FEValues::MappingRelatedData<dim, spacedim> &output_data) const;
+                          dealii::internal::FEValuesImplementation::MappingRelatedData<dim, spacedim> &output_data) const override;
 
   /**
    * @}
@@ -395,7 +398,7 @@ public:
 
 #ifndef DOXYGEN
 
-template<int dim, int spacedim>
+template <int dim, int spacedim>
 inline
 void
 MappingManifold<dim,spacedim>::InternalData::store_vertices (const typename Triangulation<dim,spacedim>::cell_iterator &cell) const
