@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 2000 - 2016 by the deal.II authors
+// Copyright (C) 2000 - 2018 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -13,8 +13,8 @@
 //
 // ---------------------------------------------------------------------
 
-#ifndef dealii__mapping_q1_h
-#define dealii__mapping_q1_h
+#ifndef dealii_mapping_q1_h
+#define dealii_mapping_q1_h
 
 
 #include <deal.II/base/config.h>
@@ -63,7 +63,7 @@ public:
 
   // for documentation, see the Mapping base class
   virtual
-  MappingQ1<dim,spacedim> *clone () const;
+  std::unique_ptr<Mapping<dim,spacedim>> clone () const override;
 };
 
 
