@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 2000 - 2017 by the deal.II authors
+// Copyright (C) 2000 - 2018 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -13,20 +13,21 @@
 //
 // ---------------------------------------------------------------------
 
-#ifndef dealii__mapping_q_generic_h
-#define dealii__mapping_q_generic_h
+#ifndef dealii_mapping_q_generic_h
+#define dealii_mapping_q_generic_h
 
 
 #include <deal.II/base/config.h>
 #include <deal.II/base/derivative_form.h>
 #include <deal.II/base/table.h>
 #include <deal.II/base/quadrature_lib.h>
+#include <deal.II/base/vectorization.h>
 #include <deal.II/grid/tria_iterator.h>
 #include <deal.II/fe/mapping.h>
 #include <deal.II/fe/fe_q.h>
+#include <deal.II/matrix_free/shape_info.h>
 
-#include <deal.II/base/std_cxx11/array.h>
-
+#include <array>
 #include <cmath>
 
 DEAL_II_NAMESPACE_OPEN
@@ -85,7 +86,7 @@ template <int,int> class MappingQ;
  *
  * As described above, one often only knows a manifold description of a
  * surface but not the interior of the computational domain. In such a case, a
- * StraightBoundary object will be assigned to the interior entities that
+ * FlatManifold object will be assigned to the interior entities that
  * describes a usual planar coordinate system where the additional points for
  * the higher order mapping are placed exactly according to a bi-/trilinear
  * mapping. When combined with a non-flat manifold on the boundary, for
@@ -98,9 +99,10 @@ template <int,int> class MappingQ;
  * transformation from the reference cell to such a cell would in general
  * contain inverted regions close to the boundary.
  *
- * In order to avoid this situation, this class applies a smoothing on cells
- * adjacent to the boundary by using so-called Laplace smoothing by
- * default. In the algorithm that computes additional points, the
+ * In order to avoid this situation, this class applies an algorithm for
+ * making this transition smooth using a so-called transfinite interpolation
+ * that is essentially a linear blend between the descriptions along the
+ * surrounding entities. In the algorithm that computes additional points, the
  * compute_mapping_support_points() method, all the entities of the cells are
  * passed through hierarchically, starting from the lines to the quads and
  * finally hexes. Points on objects higher up in the hierarchy are obtained
@@ -112,24 +114,13 @@ template <int,int> class MappingQ;
  * account when interpolating the position of the additional points inside the
  * quad and thus always result in a well-defined transformation.
  *
- * While the smoothing approach works well for filling holes or avoiding
- * inversions with low and medium convergence orders up to approximately three
- * to four, there is nonetheless an inherent shortcoming when switching from a
- * curved manifold to a flat manifold over a face (and the associated
- * smoothing). The finite element theory (see e.g. Strang and Fix, 1973,
- * Sections 2.2 and 3.3 and in particular Theorem 3.6) requires the
- * transformation to be globally C^0 continuous also over several elements and
- * to be uniform as the mesh is refined. Even though the Laplace smoothing
- * fixes the discontinuity within one layer of cells, it cannot provide
- * uniformity as the change is always within one layer of elements only. For
- * example, the convergence rates for solving the Laplacian on a circle where
- * only the boundary is deformed and the above mesh smoothing algorithm is
- * applied will typically not exceed 3.5 (or 3 in the elements adjacent to the
- * boundary), even for fourth or fifth degree polynomials. In such a case, the
- * curved manifold needs to be switched to a flat manifold in a smooth way
- * that does not depend on the mesh size and eventually covers a region of
- * cells instead of only those that are immediately adjacent to the circular
- * boundary.
+ * The interpolation scheme used in this class makes sure that curved
+ * descriptions can go over to flat descriptions within a single layer of
+ * elements, maintaining the overall optimal convergence rates of the finite
+ * element interpolation. However, one does often get better solution
+ * qualities if the transition between curved boundaries and flat interior
+ * domains is spread over a larger range as the mesh is refined. This is
+ * provided by the special manifold TransfiniteInterpolationManifold.
  *
  * @author Wolfgang Bangerth, 2015, Martin Kronbichler, 2017
  */
@@ -151,7 +142,7 @@ public:
 
   // for documentation, see the Mapping base class
   virtual
-  Mapping<dim,spacedim> *clone () const;
+  std::unique_ptr<Mapping<dim,spacedim>> clone () const override;
 
   /**
    * Return the degree of the mapping, i.e. the value which was passed to the
@@ -164,7 +155,7 @@ public:
    * this class preserves vertex locations.
    */
   virtual
-  bool preserves_vertex_locations () const;
+  bool preserves_vertex_locations () const override;
 
   /**
    * @name Mapping points between reference and real cells
@@ -175,13 +166,13 @@ public:
   virtual
   Point<spacedim>
   transform_unit_to_real_cell (const typename Triangulation<dim,spacedim>::cell_iterator &cell,
-                               const Point<dim>                                 &p) const;
+                               const Point<dim>                                 &p) const override;
 
   // for documentation, see the Mapping base class
   virtual
   Point<dim>
   transform_real_to_unit_cell (const typename Triangulation<dim,spacedim>::cell_iterator &cell,
-                               const Point<spacedim>                            &p) const;
+                               const Point<spacedim>                            &p) const override;
 
   /**
    * @}
@@ -198,7 +189,7 @@ public:
   transform (const ArrayView<const Tensor<1,dim> >                  &input,
              const MappingType                                       type,
              const typename Mapping<dim,spacedim>::InternalDataBase &internal,
-             const ArrayView<Tensor<1,spacedim> >                   &output) const;
+             const ArrayView<Tensor<1,spacedim> >                   &output) const override;
 
   // for documentation, see the Mapping base class
   virtual
@@ -206,7 +197,7 @@ public:
   transform (const ArrayView<const DerivativeForm<1, dim, spacedim> > &input,
              const MappingType                                         type,
              const typename Mapping<dim,spacedim>::InternalDataBase   &internal,
-             const ArrayView<Tensor<2,spacedim> >                     &output) const;
+             const ArrayView<Tensor<2,spacedim> >                     &output) const override;
 
   // for documentation, see the Mapping base class
   virtual
@@ -214,7 +205,7 @@ public:
   transform (const ArrayView<const Tensor<2, dim> >                 &input,
              const MappingType                                       type,
              const typename Mapping<dim,spacedim>::InternalDataBase &internal,
-             const ArrayView<Tensor<2,spacedim> >                   &output) const;
+             const ArrayView<Tensor<2,spacedim> >                   &output) const override;
 
   // for documentation, see the Mapping base class
   virtual
@@ -222,7 +213,7 @@ public:
   transform (const ArrayView<const DerivativeForm<2, dim, spacedim> > &input,
              const MappingType                                         type,
              const typename Mapping<dim,spacedim>::InternalDataBase   &internal,
-             const ArrayView<Tensor<3,spacedim> >                     &output) const;
+             const ArrayView<Tensor<3,spacedim> >                     &output) const override;
 
   // for documentation, see the Mapping base class
   virtual
@@ -230,7 +221,7 @@ public:
   transform (const ArrayView<const Tensor<3, dim> >                 &input,
              const MappingType                                       type,
              const typename Mapping<dim,spacedim>::InternalDataBase &internal,
-             const ArrayView<Tensor<3,spacedim> >                   &output) const;
+             const ArrayView<Tensor<3,spacedim> >                   &output) const override;
 
   /**
    * @}
@@ -367,7 +358,8 @@ public:
     /**
      * Return an estimate (in bytes) or the memory consumption of this object.
      */
-    virtual std::size_t memory_consumption () const;
+    virtual
+    std::size_t memory_consumption () const override;
 
     /**
      * Values of shape functions. Access by function @p shape.
@@ -420,7 +412,7 @@ public:
      *
      * Filled once.
      */
-    std_cxx11::array<std::vector<Tensor<1,dim> >, GeometryInfo<dim>::faces_per_cell *(dim-1)> unit_tangentials;
+    std::array<std::vector<Tensor<1,dim> >, GeometryInfo<dim>::faces_per_cell *(dim-1)> unit_tangentials;
 
     /**
      * The polynomial degree of the mapping. Since the objects here are also
@@ -449,6 +441,48 @@ public:
      */
     QGaussLobatto<1> line_support_points;
 
+    /**
+      * In case the quadrature rule given represents a tensor product
+      * we need to store the evaluations of the 1d polynomials at the
+      * the 1d quadrature points. That is what this variable is for.
+      */
+    internal::MatrixFreeFunctions::ShapeInfo<VectorizedArray<double>> shape_info;
+
+    /**
+     * In case the quadrature rule given represents a tensor product
+     * we need to store temporary data in this object.
+     */
+    mutable AlignedVector<VectorizedArray<double> > scratch;
+
+    /**
+     * In case the quadrature rule given represents a tensor product
+     * the values at the mapped support points are stored in this object.
+     */
+    mutable AlignedVector<VectorizedArray<double> > values_dofs;
+
+    /**
+     * In case the quadrature rule given represents a tensor product
+     * the values at the quadrature points are stored in this object.
+     */
+    mutable AlignedVector<VectorizedArray<double> > values_quad;
+
+    /**
+     * In case the quadrature rule given represents a tensor product
+     * the gradients at the quadrature points are stored in this object.
+     */
+    mutable AlignedVector<VectorizedArray<double> > gradients_quad;
+
+    /**
+     * In case the quadrature rule given represents a tensor product
+     * the hessians at the quadrature points are stored in this object.
+     */
+    mutable AlignedVector<VectorizedArray<double> > hessians_quad;
+
+    /**
+     * Indicates whether the given Quadrature object is a tensor product.
+     */
+    bool tensor_product_quadrature;
+
     /**
      * Tensors of covariant transformation at each of the quadrature points.
      * The matrix stored is the Jacobian * G^{-1}, where G = Jacobian^{t} *
@@ -496,25 +530,25 @@ public:
   // documentation can be found in Mapping::requires_update_flags()
   virtual
   UpdateFlags
-  requires_update_flags (const UpdateFlags update_flags) const;
+  requires_update_flags (const UpdateFlags update_flags) const override;
 
   // documentation can be found in Mapping::get_data()
   virtual
-  InternalData *
+  std::unique_ptr<typename Mapping<dim,spacedim>::InternalDataBase>
   get_data (const UpdateFlags,
-            const Quadrature<dim> &quadrature) const;
+            const Quadrature<dim> &quadrature) const override;
 
   // documentation can be found in Mapping::get_face_data()
   virtual
-  InternalData *
+  std::unique_ptr<typename Mapping<dim,spacedim>::InternalDataBase>
   get_face_data (const UpdateFlags flags,
-                 const Quadrature<dim-1>& quadrature) const;
+                 const Quadrature<dim-1>& quadrature) const override;
 
   // documentation can be found in Mapping::get_subface_data()
   virtual
-  InternalData *
+  std::unique_ptr<typename Mapping<dim,spacedim>::InternalDataBase>
   get_subface_data (const UpdateFlags flags,
-                    const Quadrature<dim-1>& quadrature) const;
+                    const Quadrature<dim-1>& quadrature) const override;
 
   // documentation can be found in Mapping::fill_fe_values()
   virtual
@@ -523,24 +557,26 @@ public:
                   const CellSimilarity::Similarity                               cell_similarity,
                   const Quadrature<dim>                                         &quadrature,
                   const typename Mapping<dim,spacedim>::InternalDataBase        &internal_data,
-                  dealii::internal::FEValues::MappingRelatedData<dim, spacedim> &output_data) const;
+                  dealii::internal::FEValuesImplementation::MappingRelatedData<dim, spacedim> &output_data) const override;
 
   // documentation can be found in Mapping::fill_fe_face_values()
-  virtual void
+  virtual
+  void
   fill_fe_face_values (const typename Triangulation<dim,spacedim>::cell_iterator     &cell,
                        const unsigned int                                             face_no,
                        const Quadrature<dim-1>                                       &quadrature,
                        const typename Mapping<dim,spacedim>::InternalDataBase        &internal_data,
-                       dealii::internal::FEValues::MappingRelatedData<dim, spacedim> &output_data) const;
+                       dealii::internal::FEValuesImplementation::MappingRelatedData<dim, spacedim> &output_data) const override;
 
   // documentation can be found in Mapping::fill_fe_subface_values()
-  virtual void
+  virtual
+  void
   fill_fe_subface_values (const typename Triangulation<dim,spacedim>::cell_iterator     &cell,
                           const unsigned int                                             face_no,
                           const unsigned int                                             subface_no,
                           const Quadrature<dim-1>                                       &quadrature,
                           const typename Mapping<dim,spacedim>::InternalDataBase        &internal_data,
-                          dealii::internal::FEValues::MappingRelatedData<dim, spacedim> &output_data) const;
+                          dealii::internal::FEValuesImplementation::MappingRelatedData<dim, spacedim> &output_data) const override;
 
   /**
    * @}
@@ -570,7 +606,7 @@ protected:
    * shape functions/DoFs on non-standard faces. This is used to reorder
    * support points in the same way.
    */
-  const std_cxx11::unique_ptr<FE_Q<dim> > fe_q;
+  const std::unique_ptr<FE_Q<dim> > fe_q;
 
   /**
    * A vector of tables of weights by which we multiply the locations of the
@@ -621,26 +657,27 @@ protected:
    * The default implementation of this function is appropriate for most
    * cases. It takes the locations of support points on the boundary of the
    * cell from the underlying manifold. Interior support points (ie. support
-   * points in quads for 2d, in hexes for 3d) are then computed using the
-   * solution of a Laplace equation with the position of the outer support
-   * points as boundary values, in order to make the transformation as smooth
-   * as possible.
+   * points in quads for 2d, in hexes for 3d) are then computed using an
+   * interpolation from the lower-dimensional entities (lines, quads) in order
+   * to make the transformation as smooth as possible without introducing
+   * additional boundary layers within the cells due to the placement of
+   * support points.
    *
    * The function works its way from the vertices (which it takes from the
    * given cell) via the support points on the line (for which it calls the
    * add_line_support_points() function) and the support points on the quad
    * faces (in 3d, for which it calls the add_quad_support_points() function).
    * It then adds interior support points that are either computed by
-   * interpolation from the surrounding points using weights computed by
-   * solving a Laplace equation, or if dim<spacedim, it asks the underlying
-   * manifold for the locations of interior points.
+   * interpolation from the surrounding points using weights for transfinite
+   * interpolation, or if dim<spacedim, it asks the underlying manifold for
+   * the locations of interior points.
    */
   virtual
   std::vector<Point<spacedim> >
   compute_mapping_support_points (const typename Triangulation<dim,spacedim>::cell_iterator &cell) const;
 
   /**
-   * Transforms the point @p p on the real cell to the corresponding point on
+   * Transform the point @p p on the real cell to the corresponding point on
    * the unit cell @p cell by a Newton iteration.
    */
   Point<dim>
@@ -649,14 +686,13 @@ protected:
                                         const Point<dim> &initial_p_unit) const;
 
   /**
-   * For <tt>dim=2,3</tt>. Append the support points of all shape functions
-   * located on bounding lines of the given cell to the vector @p a. Points
-   * located on the vertices of a line are not included.
+   * Append the support points of all shape functions located on bounding
+   * lines of the given cell to the vector @p a. Points located on the
+   * vertices of a line are not included.
    *
-   * Needed by the @p compute_support_points() function. For <tt>dim=1</tt>
-   * this function is empty. The function uses the underlying manifold object
-   * of the line (or, if none is set, of the cell) for the location of the
-   * requested points.
+   * This function uses the underlying manifold object of the line (or, if
+   * none is set, of the cell) for the location of the requested points. This
+   * function is usually called by compute_mapping_support_points() function.
    *
    * This function is made virtual in order to allow derived classes to choose
    * shape function support points differently than the present class, which
@@ -668,14 +704,14 @@ protected:
                            std::vector<Point<spacedim> > &a) const;
 
   /**
-   * For <tt>dim=3</tt>. Append the support points of all shape functions
-   * located on bounding faces (quads in 3d) of the given cell to the vector
-   * @p a. Points located on the vertices or lines of a quad are not included.
+   * Append the support points of all shape functions located on bounding
+   * faces (quads in 3d) of the given cell to the vector @p a. This function
+   * is only defined for <tt>dim=3</tt>. Points located on the vertices or
+   * lines of a quad are not included.
    *
-   * Needed by the @p compute_support_points() function. For <tt>dim=1</tt>
-   * and <tt>dim=2</tt> this function is empty. The function uses the
-   * underlying manifold object of the quad (or, if none is set, of the cell)
-   * for the location of the requested points.
+   * This function uses the underlying manifold object of the quad (or, if
+   * none is set, of the cell) for the location of the requested points. This
+   * function is usually called by compute_mapping_support_points().
    *
    * This function is made virtual in order to allow derived classes to choose
    * shape function support points differently than the present class, which
@@ -701,7 +737,7 @@ protected:
 
 #ifndef DOXYGEN
 
-template<int dim, int spacedim>
+template <int dim, int spacedim>
 inline
 const double &
 MappingQGeneric<dim,spacedim>::InternalData::shape (const unsigned int qpoint,
@@ -715,7 +751,7 @@ MappingQGeneric<dim,spacedim>::InternalData::shape (const unsigned int qpoint,
 
 
 
-template<int dim, int spacedim>
+template <int dim, int spacedim>
 inline
 double &
 MappingQGeneric<dim,spacedim>::InternalData::shape (const unsigned int qpoint,
@@ -728,7 +764,7 @@ MappingQGeneric<dim,spacedim>::InternalData::shape (const unsigned int qpoint,
 }
 
 
-template<int dim, int spacedim>
+template <int dim, int spacedim>
 inline
 const Tensor<1,dim> &
 MappingQGeneric<dim,spacedim>::InternalData::derivative (const unsigned int qpoint,
@@ -742,7 +778,7 @@ MappingQGeneric<dim,spacedim>::InternalData::derivative (const unsigned int qpoi
 
 
 
-template<int dim, int spacedim>
+template <int dim, int spacedim>
 inline
 Tensor<1,dim> &
 MappingQGeneric<dim,spacedim>::InternalData::derivative (const unsigned int qpoint,
