@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 1998 - 2016 by the deal.II authors
+// Copyright (C) 1998 - 2018 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -13,15 +13,16 @@
 //
 // ---------------------------------------------------------------------
 
-#ifndef dealii__cell_id_h
-#define dealii__cell_id_h
+#ifndef dealii_cell_id_h
+#define dealii_cell_id_h
 
 #include <deal.II/base/config.h>
 #include <deal.II/base/exceptions.h>
-#include <deal.II/base/std_cxx11/array.h>
 
+#include <array>
 #include <vector>
 #include <iostream>
+#include <cstdint>
 
 #ifdef DEAL_II_WITH_P4EST
 #include <deal.II/distributed/p4est_wrappers.h>
@@ -32,11 +33,15 @@ DEAL_II_NAMESPACE_OPEN
 template <int, int> class Triangulation;
 
 /**
- * A class to represent a unique ID for a cell in a Triangulation.  This class
+ * A class to represent a unique ID for a cell in a Triangulation. It is
+ * returned by <tt>cell->id()</tt> if <tt>cell</tt> is a cell iterator.
+ *
+ * This class
  * stores the index of the coarse cell from which a cell is descendant,
  * together with information on how to reach the cell from that coarse cell
- * (i.e., which child index to take on each level when moving from one cell to
- * its children). The important point about this class is that an object of
+ * (i.e., which child index to take on each level of the triangulation when
+ * moving from one cell to its children). The important point about this
+ * class is that an object of
  * the current class uniquely identifies a cell in triangulation, and it even
  * does so in the context of objects of type
  * parallel::distributed::Triangulation where the local portion of a mesh may
@@ -46,9 +51,9 @@ template <int, int> class Triangulation;
  * and index of the iterators pointing to that cell <i>within the
  * triangulation stored on each of the processors</i> may (and in general
  * will) be different. In other words, CellId provides the tool with which it
- * is possible to uniquely identify cells in a parallel triangulation, and
- * consequently makes it possible to exchange data between processors tied to
- * individual cells.
+ * is possible to globally and uniquely identify cells in a parallel
+ * triangulation, and consequently makes it possible to exchange, between
+ * processors, data tied to individual cells.
  *
  * @note How this data is internally represented is not of importance (and not
  * exposed on purpose).
@@ -63,7 +68,7 @@ public:
    * (using 2 times 32 bit for storage), a limitation that is identical to
    * the one used by p4est.
    */
-  typedef std_cxx11::array<unsigned int, 4> binary_type;
+  typedef std::array<unsigned int, 4> binary_type;
 
   /**
    * Construct a CellId object with a given @p coarse_cell_id and vector of
@@ -72,10 +77,11 @@ public:
    * each entry denotes which child to pick from one refinement level to the
    * next, starting with the coarse cell, until we get to the cell represented
    * by the current object. Therefore, each entry should be a number between 0
-   * and the number of children of a cell in the current space dimension.
+   * and the number of children of a cell in the current space dimension (i.e.,
+   * GeometryInfo<dim>::max_children_per_cell).
    */
   CellId(const unsigned int coarse_cell_id,
-         const std::vector<unsigned char> &child_indices);
+         const std::vector<std::uint8_t> &child_indices);
 
   /**
    * Construct a CellId object with a given @p coarse_cell_id and array of
@@ -84,12 +90,13 @@ public:
    * each entry denotes which child to pick from one refinement level to the
    * next, starting with the coarse cell, until we get to the cell represented
    * by the current object. Therefore, each entry should be a number between 0
-   * and the number of children of a cell in the current space dimension.
-   * @p child_indices shall have at least @p n_child_indices valid entries.
+   * and the number of children of a cell in the current space dimension (i.e.,
+   * GeometryInfo<dim>::max_children_per_cell). The array
+   * @p child_indices must have at least @p n_child_indices valid entries.
    */
-  CellId(const unsigned int coarse_cell_id,
-         const unsigned int n_child_indices,
-         const unsigned char *child_indices);
+  CellId(const unsigned int  coarse_cell_id,
+         const unsigned int  n_child_indices,
+         const std::uint8_t *child_indices);
 
   /**
    * Construct a CellId object with a given binary representation that was
@@ -110,13 +117,13 @@ public:
   /**
    * Return a compact and fast binary representation of this CellId.
    */
-  template<int dim>
+  template <int dim>
   binary_type to_binary() const;
 
   /**
    * Return a cell_iterator to the cell represented by this CellId.
    */
-  template<int dim, int spacedim>
+  template <int dim, int spacedim>
   typename Triangulation<dim,spacedim>::cell_iterator
   to_cell(const Triangulation<dim,spacedim> &tria) const;
 
@@ -137,6 +144,12 @@ public:
    */
   bool operator<(const CellId &other) const;
 
+  /**
+   * Boost serialization function
+   */
+  template<class Archive>
+  void serialize(Archive &ar, const unsigned int version );
+
 private:
   /**
    * The number of the coarse cell within whose tree the cell
@@ -160,9 +173,9 @@ private:
    * the array can be extended.
    */
 #ifdef DEAL_II_WITH_P4EST
-  std_cxx11::array<char,internal::p4est::functions<2>::max_level> child_indices;
+  std::array<std::uint8_t,internal::p4est::functions<2>::max_level> child_indices;
 #else
-  std_cxx11::array<char,30> child_indices;
+  std::array<std::uint8_t,30> child_indices;
 #endif
 
   friend std::istream &operator>> (std::istream &is, CellId &cid);
@@ -182,15 +195,26 @@ std::ostream &operator<< (std::ostream &os,
   os << cid.coarse_cell_id << '_' << cid.n_child_indices << ':';
   for (unsigned int i=0; i<cid.n_child_indices; ++i)
     // write the child indices. because they are between 0 and 2^dim-1, they all
-    // just have one digit, so we could write them as integers. it's
-    // probably clearer to write them as one-digit characters starting
-    // at '0'
-    os << static_cast<unsigned char>(cid.child_indices[i] + '0');
+    // just have one digit, so we could write them as one character
+    // objects. it's probably clearer to write them as one-digit characters
+    // starting at '0'
+    os << static_cast<unsigned char>('0' + cid.child_indices[i]);
   return os;
 }
 
 
 
+/**
+ * Serialization function
+ */
+template<class Archive>
+void CellId::serialize(Archive &ar, const unsigned int /*version*/)
+{
+  ar &coarse_cell_id;
+  ar &n_child_indices;
+  ar &child_indices;
+}
+
 /**
  * Read a CellId object from a stream.
  */
@@ -211,11 +235,11 @@ std::istream &operator>> (std::istream &is,
   is >> dummy;
   Assert(dummy==':', ExcMessage("invalid CellId"));
 
-  char value;
+  unsigned char value;
   for (unsigned int i=0; i<cid.n_child_indices; ++i)
     {
       // read the one-digit child index (as an integer number) and
-      // convert it back into unsigned char
+      // convert it back into unsigned integer type
       is >> value;
       cid.child_indices[i] = value-'0';
     }
