@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 2016 by the deal.II authors
+// Copyright (C) 2016 - 2018 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -13,8 +13,8 @@
 //
 // ---------------------------------------------------------------------
 
-#ifndef dealii__composition_manifold_h
-#define dealii__composition_manifold_h
+#ifndef dealii_composition_manifold_h
+#define dealii_composition_manifold_h
 
 
 /*----------------------------   composition_manifold.h     ------------*/
@@ -25,6 +25,7 @@
 #include <deal.II/base/thread_management.h>
 #include <deal.II/base/point.h>
 #include <deal.II/base/derivative_form.h>
+#include <deal.II/base/std_cxx14/memory.h>
 #include <deal.II/grid/tria.h>
 #include <deal.II/grid/tria_iterator.h>
 #include <deal.II/grid/tria_accessor.h>
@@ -39,7 +40,7 @@ DEAL_II_NAMESPACE_OPEN
  * the embedding space of the second ChartManifold. If the first
  * ChartManifold is periodic, so is the resulting ChartManifold, with
  * the same periodicity. Periodicity on the second ChartManifold is not
- * allowed, and the constructor will throw an axception if the second
+ * allowed, and the constructor will throw an exception if the second
  * Manifold is periodic.
  *
  * This class only works for dim <= chartdim <= intermediate_spacedim
@@ -75,6 +76,11 @@ public:
   CompositionManifold(const ChartManifold<dim1, intermediate_dim, chartdim> &F,
                       const ChartManifold<dim2, spacedim, intermediate_dim> &G);
 
+  /**
+   * Make a clone of this Manifold.
+   */
+  virtual std::unique_ptr<Manifold<dim, spacedim> > clone() const override;
+
   /**
    * Pull back the given point in spacedim to the Euclidean chartdim
    * dimensional space. This function calls the pull_back() function
@@ -88,7 +94,7 @@ public:
   /**
    * Push forward the chartdim dimensional point to a spacedim
    * Euclidean point. The function calls first the push_forward() of
-   * F, and then the push_foward() of G.
+   * F, and then the push_forward() of G.
    */
   virtual
   Point<spacedim>
@@ -135,6 +141,14 @@ CompositionManifold<dim,spacedim,chartdim,intermediate_dim,dim1,dim2>::Compositi
 }
 
 
+template <int dim, int spacedim, int chartdim, int intermediate_dim,
+          int dim1, int dim2>
+std::unique_ptr<Manifold<dim,spacedim> >
+CompositionManifold<dim,spacedim,chartdim,intermediate_dim,dim1,dim2>::clone() const
+{
+  return std_cxx14::make_unique<CompositionManifold<dim,spacedim,chartdim,intermediate_dim,dim1,dim2> >(*F, *G);
+}
+
 
 template <int dim, int spacedim, int chartdim, int intermediate_dim,
           int dim1, int dim2>
