@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 2002 - 2017 by the deal.II authors
+// Copyright (C) 2002 - 2018 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -13,19 +13,18 @@
 //
 // ---------------------------------------------------------------------
 
-#ifndef dealii__filtered_iterator_h
-#define dealii__filtered_iterator_h
+#ifndef dealii_filtered_iterator_h
+#define dealii_filtered_iterator_h
 
 
+#include <deal.II/base/std_cxx14/memory.h>
 #include <deal.II/base/config.h>
 #include <deal.II/base/exceptions.h>
 #include <deal.II/base/iterator_range.h>
 #include <deal.II/grid/tria_iterator_base.h>
 
 #include <set>
-#ifdef DEAL_II_WITH_CXX11
 #include <tuple>
-#endif
 
 DEAL_II_NAMESPACE_OPEN
 
@@ -236,7 +235,7 @@ namespace IteratorFilters
      * have to be evaluated to true and state if the iterator must be locally
      * owned.
      */
-    MaterialIdEqualTo (const std::set<types::material_id> material_ids,
+    MaterialIdEqualTo (const std::set<types::material_id> &material_ids,
                        const bool only_locally_owned = false);
 
     /**
@@ -282,7 +281,7 @@ namespace IteratorFilters
      * shall have to be evaluated to true and state if the iterator must be
      * locally owned.
      */
-    ActiveFEIndexEqualTo (const std::set<unsigned int> active_fe_indices,
+    ActiveFEIndexEqualTo (const std::set<unsigned int> &active_fe_indices,
                           const bool only_locally_owned = false);
 
     /**
@@ -376,7 +375,7 @@ namespace IteratorFilters
  * @endcode
  * then
  * @code
- *   std_cxx11::bind (&level_equal_to<active_cell_iterator>, std_cxx11::_1, 3)
+ *   std::bind (&level_equal_to<active_cell_iterator>, std::placeholders::_1, 3)
  * @endcode
  * is another valid predicate (here: a function that returns true if either
  * the iterator is past the end or the level is equal to the second argument;
@@ -448,7 +447,7 @@ namespace IteratorFilters
  * Since one often only has some iterator and wants to set a filtered iterator
  * to the first one that satisfies a predicate (for example, the first one for
  * which the user flag is set, or the first one with a given subdomain id),
- * there are assignement functions #set_to_next_positive and
+ * there are assignment functions #set_to_next_positive and
  * #set_to_previous_positive that assign the next or last previous iterator
  * that satisfies the predicate, i.e. they follow the list of iterators in
  * either direction until they find a matching one (or the past-the-end
@@ -520,7 +519,7 @@ public:
 
   /**
    * Constructor. Set the iterator to the default state and use the given
-   * predicate for filtering subsequent assignement and iteration.
+   * predicate for filtering subsequent assignment and iteration.
    */
   template <typename Predicate>
   FilteredIterator (Predicate p);
@@ -530,7 +529,7 @@ public:
    * iterator with the given value.
    *
    * If the initial value @p bi does not satisfy the predicate @p p then it is
-   * advanced until we either hit the the past-the-end iterator, or the
+   * advanced until we either hit the past-the-end iterator, or the
    * predicate is satisfied. This allows, for example, to write code like
    * @code
    *   FilteredIterator<typename Triangulation<dim>::active_cell_iterator>
@@ -693,7 +692,7 @@ private:
      * Mark the destructor virtual to allow destruction through pointers to
      * the base class.
      */
-    virtual ~PredicateBase () {}
+    virtual ~PredicateBase () = default;
 
     /**
      * Abstract function which in derived classes denotes the evaluation of
@@ -705,7 +704,7 @@ private:
      * Generate a copy of this object, i.e. of the actual type of this
      * pointer.
      */
-    virtual PredicateBase *clone () const = 0;
+    virtual std::unique_ptr<PredicateBase> clone () const = 0;
   };
 
 
@@ -729,13 +728,13 @@ private:
     /**
      * Evaluate the iterator with the stored copy of the predicate.
      */
-    virtual bool operator () (const BaseIterator &bi) const;
+    virtual bool operator () (const BaseIterator &bi) const override;
 
     /**
      * Generate a copy of this object, i.e. of the actual type of this
      * pointer.
      */
-    virtual PredicateBase *clone () const;
+    virtual std::unique_ptr<PredicateBase> clone () const override;
 
   private:
     /**
@@ -748,7 +747,7 @@ private:
    * Pointer to an object that encapsulated the actual data type of the
    * predicate given to the constructor.
    */
-  const PredicateBase *predicate;
+  std::unique_ptr<const PredicateBase> predicate;
 
 };
 
@@ -761,7 +760,7 @@ private:
  * explicitly specify the type of the base iterator by hand -- it is deduced
  * automatically here.
  *
- * @author Wolfgang Bangerth @relates FilteredIterator
+ * @author Wolfgang Bangerth @relatesalso FilteredIterator
  */
 template <typename BaseIterator, typename Predicate>
 FilteredIterator<BaseIterator>
@@ -775,12 +774,11 @@ make_filtered_iterator (const BaseIterator &i,
 
 
 
-#ifdef DEAL_II_WITH_CXX11
 namespace internal
 {
-  namespace FilteredIterator
+  namespace FilteredIteratorImplementation
   {
-    // The following classes create a nested sequencee of
+    // The following classes create a nested sequence of
     // FilteredIterator<FilteredIterator<...<BaseIterator>...>> with as many
     // levels of FilteredIterator classes as there are elements in the TypeList
     // if the latter is given as a std::tuple<Args...>.
@@ -832,7 +830,7 @@ namespace internal
  * @endcode
  *
  * @author Bruno Turcksin, 2016
- * @relates FilteredIterator
+ * @relatesalso FilteredIterator
  * @ingroup CPP11
  */
 template <typename BaseIterator, typename Predicate>
@@ -880,11 +878,11 @@ filter_iterators (IteratorRange<BaseIterator> i,
  * @endcode
  *
  * @author Bruno Turcksin, 2016
- * @relates FilteredIterator
+ * @relatesalso FilteredIterator
  * @ingroup CPP11
  */
 template <typename BaseIterator, typename Predicate, typename... Targs>
-IteratorRange<typename internal::FilteredIterator::
+IteratorRange<typename internal::FilteredIteratorImplementation::
 NestFilteredIterators<BaseIterator,std::tuple<Predicate, Targs...> >::type>
 filter_iterators (IteratorRange<BaseIterator> i,
                   const Predicate &p,
@@ -894,7 +892,6 @@ filter_iterators (IteratorRange<BaseIterator> i,
   auto fi = filter_iterators(i,p);
   return filter_iterators(fi, args...);
 }
-#endif
 
 
 /* ------------------ Inline functions and templates ------------ */
@@ -948,8 +945,7 @@ inline
 FilteredIterator<BaseIterator>::
 ~FilteredIterator ()
 {
-  delete predicate;
-  predicate = 0;
+  predicate.reset();
 }
 
 
@@ -1182,11 +1178,11 @@ operator () (const BaseIterator &bi) const
 
 template <typename BaseIterator>
 template <typename Predicate>
-typename FilteredIterator<BaseIterator>::PredicateBase *
+std::unique_ptr<typename FilteredIterator<BaseIterator>::PredicateBase>
 FilteredIterator<BaseIterator>::PredicateTemplate<Predicate>::
 clone () const
 {
-  return new PredicateTemplate (predicate);
+  return std_cxx14::make_unique<PredicateTemplate> (predicate);
 }
 
 
@@ -1293,19 +1289,14 @@ namespace IteratorFilters
   MaterialIdEqualTo::MaterialIdEqualTo (const types::material_id material_id,
                                         const bool only_locally_owned)
     :
-    // Note: matrial_ids is a const member and has to be populated with a
-    // constructor. Unfortunately, C++98/03 does not allow the use of an
-    // initializer list. Therefore, treat material_id as an array of one
-    // element.
-    // This is well defined according to [expr.add].4 (ISO 14882).
-    material_ids (&material_id, &material_id+1),
-    only_locally_owned (only_locally_owned)
+    material_ids {material_id},
+               only_locally_owned (only_locally_owned)
   {}
 
 
 
   inline
-  MaterialIdEqualTo::MaterialIdEqualTo (const std::set<types::material_id> material_ids,
+  MaterialIdEqualTo::MaterialIdEqualTo (const std::set<types::material_id> &material_ids,
                                         const bool only_locally_owned)
     :
     material_ids (material_ids),
@@ -1331,19 +1322,14 @@ namespace IteratorFilters
   ActiveFEIndexEqualTo::ActiveFEIndexEqualTo (const unsigned int active_fe_index,
                                               const bool only_locally_owned)
     :
-    // Note: active_fe_indices is a const member and has to be populated
-    // with a constructor. Unfortunately, C++98/03 does not allow the use
-    // of an initializer list. Therefore, treat active_fe_index as an array
-    // of one element.
-    // This is well defined according to [expr.add].4 (ISO 14882).
-    active_fe_indices (&active_fe_index, &active_fe_index+1),
-    only_locally_owned (only_locally_owned)
+    active_fe_indices {active_fe_index},
+                    only_locally_owned (only_locally_owned)
   {}
 
 
 
   inline
-  ActiveFEIndexEqualTo::ActiveFEIndexEqualTo (const std::set<unsigned int> active_fe_indices,
+  ActiveFEIndexEqualTo::ActiveFEIndexEqualTo (const std::set<unsigned int> &active_fe_indices,
                                               const bool only_locally_owned)
     :
     active_fe_indices (active_fe_indices),
@@ -1381,5 +1367,3 @@ DEAL_II_NAMESPACE_CLOSE
 /*------------------------- filtered_iterator.h ------------------------*/
 #endif
 /*------------------------- filtered_iterator.h ------------------------*/
-
-
