@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 1999 - 2017 by the deal.II authors
+// Copyright (C) 1999 - 2018 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -13,17 +13,18 @@
 //
 // ---------------------------------------------------------------------
 
-#ifndef dealii__grid_generator_h
-#define dealii__grid_generator_h
+#ifndef dealii_grid_generator_h
+#define dealii_grid_generator_h
 
 
 #include <deal.II/base/config.h>
-#include <deal.II/base/std_cxx11/array.h>
 #include <deal.II/base/exceptions.h>
 #include <deal.II/base/point.h>
 #include <deal.II/base/table.h>
 #include <deal.II/base/function.h>
 #include <deal.II/grid/tria.h>
+
+#include <array>
 #include <map>
 
 DEAL_II_NAMESPACE_OPEN
@@ -32,11 +33,20 @@ DEAL_II_NAMESPACE_OPEN
  * This namespace provides a collection of functions for generating
  * triangulations for some basic geometries.
  *
- * Some of these functions receive a flag @p colorize. If this is set, parts
- * of the boundary receive different
- * @ref GlossBoundaryIndicator "boundary indicators"),
- * allowing them to be distinguished for the purpose of attaching geometry
- * objects and evaluating different boundary conditions.
+ * Some of these functions receive a flag @p colorize. If this is set, parts of
+ * the boundary receive different
+ * @ref GlossBoundaryIndicator "boundary indicators"
+ * allowing them to be distinguished for the purpose of evaluating
+ * different boundary conditions.
+ *
+ * If the domain is curved, each of the domain parts that should be
+ * refined by following an appropriate Manifold description will
+ * receive a different
+ * @ref GlossManifoldIndicator "manifold indicator",
+ * and the correct Manifold descriptor will be attached to
+ * the Triangulation. Notice that if you later transform the
+ * triangulation, you have to make sure you attach the correct new Manifold
+ * to the triangulation.
  *
  * @ingroup grid
  */
@@ -70,7 +80,7 @@ namespace GridGenerator
    * cells. See hyper_rectangle(), if different lengths in different ordinate
    * directions are required.
    *
-   * @note The triangulation needs to be void upon calling this function.
+   * @note The triangulation passed as argument needs to be empty when calling this function.
    */
   template <int dim, int spacedim>
   void hyper_cube (Triangulation<dim,spacedim>  &tria,
@@ -117,7 +127,7 @@ namespace GridGenerator
    * dimensional space with the remaining entries set to zero. For example, a
    * <tt>Triangulation@<2,3@></tt> will be a square in the xy plane with z=0.
    *
-   * @note The triangulation needs to be void upon calling this function.
+   * @note The triangulation passed as argument needs to be empty when calling this function.
    */
   template <int dim, int spacedim>
   void subdivided_hyper_cube (Triangulation<dim,spacedim>  &tria,
@@ -137,18 +147,14 @@ namespace GridGenerator
    * GeometryInfo class. Importantly, however, in 3d colorization does not set
    * @p boundary_ids of <i>edges</i>, but only of <i>faces</i>, because each
    * boundary edge is shared between two faces and it is not clear how the
-   * boundary id of an edge should be set in that case. This may later on lead
-   * to problems if one wants to assign boundary or manifold objects to parts
-   * of the boundary with certain boundary indicators since then the boundary
-   * object may not apply to the edges bounding the face it is meant to
-   * describe.
+   * boundary id of an edge should be set in that case.
    *
    * Additionally, if @p colorize is @p true, material ids are assigned to the
    * cells according to the octant their center is in: being in the right half
    * space for any coordinate direction <i>x<sub>i</sub></i> adds
    * 2<sup>i</sup>. For instance, a cell with center point (1,-1,1) yields a
    * material id 5, assuming that the center of the hyper rectangle lies at
-   * the origin.
+   * the origin. No manifold id is set for the cells.
    *
    * If @p dim < @p spacedim, this will create a @p dim dimensional object in
    * the first @p dim coordinate directions embedded into the @p spacedim
@@ -156,7 +162,7 @@ namespace GridGenerator
    * <tt>Triangulation@<2,3@></tt> will be a rectangle in the xy plane with
    * z=0, defined by the two opposing corners @p p1 and @p p2.
    *
-   * @note The triangulation needs to be void upon calling this function.
+   * @note The triangulation passed as argument needs to be empty when calling this function.
    */
   template <int dim, int spacedim>
   void hyper_rectangle (Triangulation<dim,spacedim> &tria,
@@ -175,9 +181,9 @@ namespace GridGenerator
    * direction is 1.
    *
    * If the @p colorize flag is set, the @p boundary_ids of the surfaces are
-   * assigned, such that the lower one in @p x-direction is 0, the upper one
-   * is 1 (the left and the right vertical face). The indicators for the
-   * surfaces in @p y-direction are 2 and 3, the ones for @p z are 4 and 5.
+   * assigned, such that the lower one in @p x-direction is 0, the upper one is
+   * 1 (the left and the right vertical face). The indicators for the surfaces
+   * in @p y-direction are 2 and 3, the ones for @p z are 4 and 5.
    * Additionally, material ids are assigned to the cells according to the
    * octant their center is in: being in the right half plane for any
    * coordinate direction <i>x<sub>i</sub></i> adds 2<sup>i</sup>. For
@@ -259,7 +265,7 @@ namespace GridGenerator
                               const std::vector< std::vector<double> > &spacing,
                               const Point<dim>                         &p,
                               const Table<dim,types::material_id>      &material_id,
-                              const bool                                colorize=false);
+                              const bool                               colorize=false);
 
   /**
    * \brief Rectangular domain with rectangular pattern of holes
@@ -320,7 +326,7 @@ namespace GridGenerator
    *
    * @note This function is implemented in 2d only.
    *
-   * @note The triangulation needs to be void upon calling this function.
+   * @note The triangulation passed as argument needs to be empty when calling this function.
    */
   template <int dim>
   void
@@ -340,7 +346,7 @@ namespace GridGenerator
    * vertices in the array of <code>corners</code> will no longer refer to the
    * same triangulation.
    *
-   * @note The triangulation needs to be void upon calling this function.
+   * @note The triangulation passed as argument needs to be empty when calling this function.
    */
   template <int dim>
   void
@@ -356,7 +362,7 @@ namespace GridGenerator
    * of subdivisions in each of the @p dim directions. Colorizing is done
    * according to hyper_rectangle().
    *
-   * @note The triangulation needs to be void upon calling this function.
+   * @note The triangulation passed as argument needs to be empty when calling this function.
    */
   template <int dim>
   void
@@ -370,7 +376,7 @@ namespace GridGenerator
    * number of subdivisions in each of the @p dim directions may vary.
    * Colorizing is done according to hyper_rectangle().
    *
-   * @note The triangulation needs to be void upon calling this function.
+   * @note The triangulation passed as argument needs to be empty when calling this function.
    */
   template <int dim>
   void
@@ -409,7 +415,7 @@ namespace GridGenerator
   void
   subdivided_parallelepiped (Triangulation<dim, spacedim>  &tria,
                              const Point<spacedim> &origin,
-                             const std_cxx11::array<Tensor<1,spacedim>,dim> &edges,
+                             const std::array<Tensor<1,spacedim>,dim> &edges,
                              const std::vector<unsigned int> &subdivisions = std::vector<unsigned int>(),
                              const bool colorize = false);
 
@@ -425,7 +431,7 @@ namespace GridGenerator
    *
    * Presently only available in 2d and 3d.
    *
-   * @note The triangulation needs to be void upon calling this function.
+   * @note The triangulation passed as argument needs to be empty when calling this function.
    */
   template <int dim>
   void enclosed_hyper_cube (Triangulation<dim> &tria,
@@ -435,47 +441,57 @@ namespace GridGenerator
                             const bool          colorize = false);
 
   /**
-   * Initialize the given triangulation with a hyperball, i.e. a circle or a
+   * Initialize the given triangulation with several coarse mesh cells
+   * that cover a hyperball, i.e. a circle or a
    * ball around @p center with given @p radius.
    *
    * In order to avoid degenerate cells at the boundaries, the circle is
-   * triangulated by five cells, the ball by seven cells. The diameter of the
+   * triangulated by five cells, the ball by seven cells. Specifically, these
+   * cells are one cell in the center plus one "cap" cell on each of the faces
+   * of this center cell. This ensures that under repeated refinement, none
+   * of the cells at the outer boundary will degenerate to have an interior
+   * angle approaching 180 degrees. The diameter of the
    * center cell is chosen so that the aspect ratio of the boundary cells
    * after one refinement is optimized.
    *
    * This function is declared to exist for triangulations of all space
    * dimensions, but throws an error if called in 1d.
    *
-   * You should attach a SphericalManifold to the cells and faces for correct
-   * placement of vertices upon refinement and to be able to use higher order
-   * mappings.
+   * By default, the manifold_id is set to 0 on the boundary faces, 1 on the
+   * the boundary cells, and types::flat_manifold_id on the central cell and on
+   * internal faces.
    *
-   * @note The triangulation needs to be void upon calling this function.
+   * A SphericalManifold is attached by default to the boundary faces for
+   * correct placement of boundary vertices upon refinement and to be able to
+   * use higher order mappings. However, it turns out that this strategy may
+   * not be the optimal one to create a good a mesh for a hyperball. The
+   * "Possibilities for extensions" section of step-6 has an extensive
+   * discussion of how one would construct better meshes and what one needs to
+   * do for it. Selecting the argument @p
+   * attach_spherical_manifold_on_boundary_cells to true attaches a
+   * SphericalManifold manifold also to the boundary cells, and not only to the
+   * boundary faces.
+   *
+   * @note The triangulation passed as argument needs to be empty when calling this function.
    */
   template <int dim>
   void hyper_ball (Triangulation<dim> &tria,
                    const Point<dim>   &center = Point<dim>(),
-                   const double        radius = 1.);
+                   const double        radius = 1.,
+                   const bool attach_spherical_manifold_on_boundary_cells=false);
 
   /**
    * Creates a hyper sphere, i.e., a surface of a ball in @p spacedim
    * dimensions. This function only exists for dim+1=spacedim in 2 and 3 space
-   * dimensions.
+   * dimensions. (To create a mesh of a ball, use GridGenerator::hyper_ball().)
    *
-   * You should attach a SphericalManifold to the cells and faces for correct
-   * placement of vertices upon refinement and to be able to use higher order
-   * mappings.
+   * By default, all manifold ids of the triangulation are set to zero, and a
+   * SphericalManifold is attached to the grid.
    *
    * The following pictures are generated with:
    * @code
    * Triangulation<2,3>   triangulation;
-   *
-   * static SphericalManifold<2,3> surface_description;
-   *
    * GridGenerator::hyper_sphere(triangulation);
-   *
-   * triangulation.set_all_manifold_ids(0);
-   * triangulation.set_manifold (0, surface_description);
    * triangulation.refine_global(3);
    * @endcode
    *
@@ -486,24 +502,24 @@ namespace GridGenerator
    * @image html sphere.png
    * @image html sphere_section.png
    *
-   * @note The triangulation needs to be void upon calling this function.
+   * @note The triangulation passed as argument needs to be empty when calling this function.
    */
 
-  template <int dim, int spacedim>
-  void hyper_sphere (Triangulation<dim,spacedim> &tria,
-                     const Point<spacedim>   &center = Point<spacedim>(),
-                     const double        radius = 1.);
+  template <int spacedim>
+  void hyper_sphere (Triangulation<spacedim-1,spacedim> &tria,
+                     const Point<spacedim>              &center = Point<spacedim>(),
+                     const double                        radius = 1.);
 
   /**
    * This class produces a hyper-ball intersected with the positive orthant
    * relative to @p center, which contains three elements in 2d and four in 3d.
    *
    * The boundary indicators for the final triangulation are 0 for the curved
-   * boundary and 1 for the cut plane.
+   * boundary and 1 for the cut plane. The manifold id for the curved boundary is
+   * set to zero, and a SphericalManifold is attached to it.
    *
-   * The appropriate boundary class is HyperBallBoundary.
-   *
-   * @note The triangulation needs to be void upon calling this function.
+   * @note The triangulation passed as argument needs to be empty when calling
+   * this function.
    */
   template <int dim>
   void quarter_hyper_ball (Triangulation<dim> &tria,
@@ -516,12 +532,10 @@ namespace GridGenerator
    * <i>x</i>-axis.
    *
    * The boundary indicators for the final triangulation are 0 for the curved
-   * boundary and 1 for the cut plane.
+   * boundary and 1 for the cut plane. The manifold id for the curved boundary is
+   * set to zero, and a SphericalManifold is attached to it.
    *
-   * The appropriate boundary class is HalfHyperBallBoundary, or
-   * HyperBallBoundary.
-   *
-   * @note The triangulation needs to be void upon calling this function.
+   * @note The triangulation passed as argument needs to be empty when calling this function.
    */
   template <int dim>
   void half_hyper_ball (Triangulation<dim> &tria,
@@ -546,7 +560,10 @@ namespace GridGenerator
    * the GridTools::transform() function using a rotation operator as
    * argument.
    *
-   * @note The triangulation needs to be void upon calling this function.
+   * The manifold id for the hull of the cylinder is set to zero, and a
+   * CylindricalManifold is attached to it.
+   *
+   * @note The triangulation passed as argument needs to be empty when calling this function.
    */
   template <int dim>
   void cylinder (Triangulation<dim> &tria,
@@ -558,7 +575,7 @@ namespace GridGenerator
    * <tt>x=-half_length</tt> to <tt>x=half_length</tt> and its projection into
    * the @p yz-plane is a circle of radius @p radius_0 at
    * <tt>x=-half_length</tt> and a circle of radius @p radius_1 at
-   * <tt>x=+half_length</tt>.  In between the radius is linearly decreasing.
+   * <tt>x=+half_length</tt>. In between the radius is linearly decreasing.
    *
    * In two dimensions, the cone is a trapezoid from <tt>x=-half_length</tt>
    * to <tt>x=+half_length</tt> and from <tt>y=-radius_0</tt> to
@@ -568,12 +585,12 @@ namespace GridGenerator
    *
    * The boundaries are colored according to the following scheme: 0 for the
    * hull of the cone, 1 for the left hand face and 2 for the right hand face.
+   * Both the boundary indicators and the manifold indicators are set.
    *
-   * An example of use can be found in the documentation of the ConeBoundary
-   * class, with which you probably want to associate boundary indicator 0
-   * (the hull of the cone).
+   * In three dimensions, the manifold id of the hull is set to zero, and a
+   * CylindricalManifold is attached to it.
    *
-   * @note The triangulation needs to be void upon calling this function.
+   * @note The triangulation passed as argument needs to be empty when calling this function.
    *
    * @author Markus B&uuml;rg, 2009
    */
@@ -631,10 +648,11 @@ namespace GridGenerator
    * test convergence of schemes when the solution lacks regularity.
    *
    * If the @p colorize flag is set, the @p boundary_ids of the
-   * surfaces are assigned, such that the left boundary is 0, and the
-   * others are set with growing number accordingly to the
-   * counterclockwise. Colorize option works only with 2-dimensional
-   * problem. This function will create the classical L-shape in 2d
+   * surfaces are assigned such that the left boundary is 0 and the
+   * others are assigned counterclockwise in ascending order.
+   * The @p colorize option only works in two dimensions.
+   *
+   * This function will create the classical L-shape in 2d
    * and it will look like the following in 3d:
    *
    * @image html hyper_l.png
@@ -647,7 +665,7 @@ namespace GridGenerator
    * This function exists for triangulations of all space
    * dimensions, but throws an error if called in 1d.
    *
-   * @note The triangulation needs to be void upon calling this function.
+   * @note The triangulation passed as argument needs to be empty when calling this function.
    */
   template <int dim>
   void hyper_L (Triangulation<dim> &tria,
@@ -670,7 +688,7 @@ namespace GridGenerator
    * If @p colorize is set to @p true, the faces forming the slit are marked
    * with boundary id 1 and 2, respectively.
    *
-   * @note The triangulation needs to be void upon calling this function.
+   * @note The triangulation passed as argument needs to be empty when calling this function.
    */
   template <int dim>
   void hyper_cube_slit (Triangulation<dim> &tria,
@@ -690,11 +708,8 @@ namespace GridGenerator
    * both the faces and the edges of these boundaries. If the flag is @p
    * false, both have indicator zero.
    *
-   * You should attach a SphericalManifold to the cells and faces for correct
-   * placement of vertices upon refinement and to be able to use higher order
-   * mappings. Alternatively, it is also possible to attach a
-   * HyperShellBoundary to the inner and outer boundary. This will create
-   * inferior meshes as described below.
+   * All manifold ids are set to zero, and a SphericalManifold is attached to
+   * every cell and face of the triangulation.
    *
    * In 2d, the number <tt>n_cells</tt> of elements for this initial
    * triangulation can be chosen arbitrarily. If the number of initial cells
@@ -704,43 +719,19 @@ namespace GridGenerator
    * In 3d, only certain numbers are allowed, 6 (or the default 0) for a
    * surface based on a hexahedron (i.e. 6 panels on the inner sphere extruded
    * in radial direction to form 6 cells), 12 for the rhombic dodecahedron,
-   * and 96 (see below).
+   * and 96. This choice dates from an older version of deal.II before the
+   * Manifold classes were implemented: today all three choices are roughly
+   * equivalent (after performing global refinement, of course).
    *
-   * While the SphericalManifold, that is demonstrated in the documentation of
-   * the
-   * @ref manifold "documentation module on manifolds",
-   * creates reasonable meshes for any number of @p n_cells if attached to all
-   * cells and boundaries, the situation is less than ideal when only
-   * attaching a HyperShellBoundary. Then, only vertices on the boundaries are
-   * placed at the correct distance from the center. As an example, the 3d
-   * meshes give rise to the following meshes upon one refinement:
+   * The grids with 12 and 96 cells are plotted below:
    *
-   * @image html hypershell3d-6.png
    * @image html hypershell3d-12.png
-   *
-   * Neither of these meshes is particularly good since one ends up with
-   * poorly shaped cells at the inner edge upon refinement. For example, this
-   * is the middle plane of the mesh for the <code>n_cells=6</code>:
-   *
-   * @image html hyper_shell_6_cross_plane.png
-   *
-   * The mesh generated with <code>n_cells=12</code> is better but still not
-   * good. As a consequence, you may also specify <code>n_cells=96</code> as a
-   * third option. The mesh generated in this way is based on a once refined
-   * version of the one with <code>n_cells=12</code>, where all internal nodes
-   * are re-placed along a shell somewhere between the inner and outer
-   * boundary of the domain. The following two images compare half of the
-   * hyper shell for <code>n_cells=12</code> and <code>n_cells=96</code> (note
-   * that the doubled radial lines on the cross section are artifacts of the
-   * visualization):
-   *
-   * @image html hyper_shell_12_cut.png
-   * @image html hyper_shell_96_cut.png
+   * @image html hypershell3d-96.png
    *
    * @note This function is declared to exist for triangulations of all space
    * dimensions, but throws an error if called in 1d.
    *
-   * @note The triangulation needs to be void upon calling this function.
+   * @note The triangulation passed as argument needs to be empty when calling this function.
    */
   template <int dim>
   void hyper_shell (Triangulation<dim> &tria,
@@ -769,7 +760,10 @@ namespace GridGenerator
    * boundary where $x=0$, get indicator 0, 1, and 2, respectively. Otherwise
    * all indicators are set to 0.
    *
-   * @note The triangulation needs to be void upon calling this function.
+   * All manifold ids are set to zero, and a SphericalManifold is attached
+   * to the triangulation.
+   *
+   * @note The triangulation passed as argument needs to be empty when calling this function.
    */
   template <int dim>
   void half_hyper_shell (Triangulation<dim> &tria,
@@ -798,7 +792,10 @@ namespace GridGenerator
    * indicators are set to 0. In 3d indicator 2 is at the face x=0, 3 at y=0,
    * 4 at z=0.
    *
-   * @note The triangulation needs to be void upon calling this function.
+   * All manifold ids are set to zero, and a SphericalManifold is attached
+   * to the triangulation.
+   *
+   * @note The triangulation passed as argument needs to be empty when calling this function.
    */
   template <int dim>
   void quarter_hyper_shell (Triangulation<dim> &tria,
@@ -822,7 +819,10 @@ namespace GridGenerator
    * sense in 1D and 2D. Also keep in mind that this object is rotated
    * and positioned differently than the one created by cylinder().
    *
-   * @note The triangulation needs to be void upon calling this function.
+   * All manifold ids are set to zero, and a CylindricalManifold is attached
+   * to the triangulation.
+   *
+   * @note The triangulation passed as argument needs to be empty when calling this function.
    */
   template <int dim>
   void cylinder_shell (Triangulation<dim> &tria,
@@ -838,14 +838,13 @@ namespace GridGenerator
    * Produce the volume or surface mesh of a torus. The axis of the torus is
    * the $y$-axis while the plane of the torus is the $x$-$z$ plane.
    *
-   * If @p dim is 3, the mesh will be the volume of the torus. By default,
-   * the boundary faces will have manifold id 0 and you should attach a
-   * TorusManifold to it. The cells will have manifold id 1 and you should
-   * attach a SphericalManifold to it.
+   * If @p dim is 3, the mesh will be the volume of the torus and this
+   * function attaches a TorusManifold to all boundary cells and faces (which
+   * are marked with a manifold id of 0).
    *
-   * If @p dim is 2, the mesh will describe the surface of the torus. All
-   * cells and faces will have manifold id 0 and you should attach a
-   * TorusManifold to it.
+   * If @p dim is 2, the mesh will describe the surface of the torus and this
+   * function attaches a TorusManifold to all cells and faces (which are
+   * marked with a manifold id of 0).
    *
    * @param tria The triangulation to be filled.
    *
@@ -861,12 +860,17 @@ namespace GridGenerator
               const double R,
               const double r);
 
-
-
   /**
-   * This class produces a square in the <i>xy</i>-plane with a circular hole
-   * in the middle. Square and circle are centered at the origin. In 3d, this
-   * geometry is extruded in $z$ direction to the interval $[0,L]$.
+   * This class produces a square in the <i>xy</i>-plane with a cylindrical
+   * hole in the middle. The square and the circle are centered at the
+   * origin. In 3d, this geometry is extruded in $z$ direction to the interval
+   * $[0,L]$.
+   *
+   * The inner boundary has a manifold id of $0$ and a boundary id of
+   * $6$. This function attaches a PolarManifold or CylindricalManifold to the
+   * interior boundary in 2d and 3d respectively. The other faces have
+   * boundary ids of $0, 1, 2, 3, 4$, or $5$ given in the standard order of
+   * faces in 2d or 3d.
    *
    * @image html cubes_hole.png
    *
@@ -883,7 +887,7 @@ namespace GridGenerator
    * and 6 in 3d). If @p colorize is set to false, then flat faces get the
    * number 0 and the hole gets number 1.
    */
-  template<int dim>
+  template <int dim>
   void hyper_cube_with_cylindrical_hole (
     Triangulation<dim> &triangulation,
     const double        inner_radius = .25,
@@ -944,6 +948,9 @@ namespace GridGenerator
    * will currently have to set boundary indicators again by hand in the
    * output triangulation.
    *
+   * @note Unlike most GridGenerator functions, this function does not attach
+   * any manifolds to @p result, nor does it set any manifold ids.
+   *
    * @note For a related operation on refined meshes when both meshes are
    * derived from the same coarse mesh, see
    * GridGenerator::create_union_triangulation().
@@ -979,6 +986,10 @@ namespace GridGenerator
    * from meshes for simpler geometries, then this is not the function for
    * you. Instead, consider GridGenerator::merge_triangulations().
    *
+   * @note This function assumes that both @p triangulation_1 and @p
+   * triangulation_2 have the same manifold descriptions. The output
+   * Triangulation @p has the same manifold ids as these two triangulations.
+   *
    * @pre Both of the source conditions need to be available entirely locally.
    * In other words, they can not be objects of type
    * parallel::distributed::Triangulation.
@@ -1011,6 +1022,9 @@ namespace GridGenerator
    * cells as are in @p input_triangulation, with the exception of the cells
    * listed in @p cells_to_remove.
    *
+   * @note Unlike most GridGenerator functions, this function does not attach
+   * any manifolds to @p result, nor does it set any manifold ids.
+   *
    * @pre Because we cannot create triangulations de novo that contain
    * adaptively refined cells, the input triangulation needs to have all of
    * its cells on the same level. Oftentimes, this will in fact be the
@@ -1027,7 +1041,6 @@ namespace GridGenerator
                                            const std::set<typename Triangulation<dim, spacedim>::active_cell_iterator> &cells_to_remove,
                                            Triangulation<dim, spacedim>       &result);
 
-
   /**
    * Take a 2d Triangulation that is being extruded in z direction by the
    * total height of @p height using @p n_slices slices (minimum is 2). The
@@ -1037,6 +1050,10 @@ namespace GridGenerator
    *
    * @note The 2d input triangulation @p input must be a coarse mesh that has
    * no refined cells.
+   *
+   * @note Since @p input and @p output have different spatial dimensions no
+   * manifold objects are copied (nor are any manifold ids set) by this
+   * function.
    */
   void
   extrude_triangulation (const Triangulation<2, 2> &input,
@@ -1044,6 +1061,30 @@ namespace GridGenerator
                          const double               height,
                          Triangulation<3,3>        &result);
 
+  /**
+   * Overload of the previous function. Take a 2d Triangulation that is being
+   * extruded. Differing from the previous function taking height and number of
+   * slices for uniform extrusion, this function takes z-axis values
+   * @p slice_coordinates where the slicing will happen. The boundary indicators
+   * of the faces of @p input are going to be assigned to the corresponding side
+   * walls in z direction. The bottom and top get the next two free boundary
+   * indicators.
+   *
+   * @note The 2d input triangulation @p input must be a coarse mesh that has
+   * no refined cells.
+   *
+   * @note Since @p input and @p output have different spatial dimensions no
+   * manifold objects are copied (nor are any manifold ids set) by this
+   * function.
+   *
+   * @author Weixiong Zheng, 2018
+   */
+  void
+  extrude_triangulation (const Triangulation<2, 2>             &input,
+                         const std::vector<double> &slice_coordinates,
+                         Triangulation<3,3>                   &result);
+
+
   /**
    * Given an input triangulation @p in_tria, this function makes a new flat
    * triangulation @p out_tria which contains a single level with all active
@@ -1068,6 +1109,10 @@ namespace GridGenerator
    * parallel::distributed::Triangulation, as well as when the input
    * Triangulation contains hanging nodes.
    *
+   * @note Since @p input and @p output have different spatial dimensions no
+   * manifold objects are copied by this function: you must attach new
+   * manifold objects to @p out_tria.
+   *
    * @author Luca Heltai, 2014
    */
   template <int dim, int spacedim1, int spacedim2>
@@ -1116,12 +1161,12 @@ namespace GridGenerator
    * mesh. The surface mesh is then refined in the same way as the faces of
    * the volume mesh are. In order to ensure that the surface mesh has the
    * same vertices as the volume mesh, it is therefore important that you
-   * assign appropriate boundary objects through Triangulation::set_boundary()
-   * to the surface mesh object before calling this function. If you don't,
-   * the refinement will happen under the assumption that all faces are
-   * straight (i.e using the StraightBoundary class) rather than any curved
-   * boundary object you may want to use to determine the location of new
-   * vertices.
+   * assign appropriate boundary descriptions through
+   * Triangulation::set_manifold() to the surface mesh object before calling
+   * this function. If you don't, the refinement will happen under the
+   * assumption that all faces are straight (i.e using the FlatManifold class)
+   * rather than utilizing the Manifold object you may want to use to determine
+   * the location of new vertices.
    *
    * @tparam MeshType A type that satisfies the requirements of the
    * @ref ConceptMeshType "MeshType concept".
@@ -1168,6 +1213,10 @@ namespace GridGenerator
    * faces to their corresponding surface mesh cells, for example to
    * accommodate different geometry descriptions in the case of curved
    * boundaries (but this is not currently implemented).
+   *
+   * @note Since @p volume_mesh and @p surface_mesh have different spatial
+   * dimensions no manifold objects are copied by this function: you must
+   * attach new manifold objects to @p surface_mesh.
    */
   template <template <int,int> class MeshType, int dim, int spacedim>
 #ifndef _MSC_VER
