@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 1999 - 2017 by the deal.II authors
+// Copyright (C) 1999 - 2018 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -13,8 +13,8 @@
 //
 // ---------------------------------------------------------------------
 
-#ifndef dealii__grid_in_h
-#define dealii__grid_in_h
+#ifndef dealii_grid_in_h
+#define dealii_grid_in_h
 
 
 #include <deal.II/base/config.h>
@@ -33,8 +33,8 @@ template <int dim> struct CellData;
 /**
  * This class implements an input mechanism for grid data. It allows to read a
  * grid structure into a triangulation object. At present, UCD (unstructured
- * cell data), DB Mesh, XDA, Gmsh, Tecplot, NetCDF, UNV, VTK, and Cubit are
- * supported as input format for grid data. Any numerical data other than
+ * cell data), DB Mesh, XDA, Gmsh, Tecplot, NetCDF, UNV, VTK, ASSIMP, and Cubit
+ * are supported as input format for grid data. Any numerical data other than
  * geometric (vertex locations) and topological (how vertices form cells,
  * faces, and edges) information is ignored, but the readers for the various
  * formats generally do read information that associates material ids or
@@ -138,8 +138,8 @@ template <int dim> struct CellData;
  * several example files. If the reader does not grok your files, it should be
  * fairly simple to extend it.
  *
- * <li> <tt>Gmsh 1.0 mesh</tt> format: this format is used by the @p GMSH mesh
- * generator (see http://www.geuz.org/gmsh/). The documentation in the @p GMSH
+ * <li> <tt>Gmsh 1.0 mesh</tt> format: this format is used by the @p Gmsh mesh
+ * generator (see http://www.geuz.org/gmsh/). The documentation in the @p Gmsh
  * manual explains how to generate meshes compatible with the deal.II library
  * (i.e. quads rather than triangles). In order to use this format, Gmsh has
  * to output the file in the old format 1.0. This is done adding the line
@@ -325,7 +325,9 @@ public:
     /// Use read_tecplot()
     tecplot,
     /// Use read_vtk()
-    vtk
+    vtk,
+    /// Use read_assimp()
+    assimp,
   };
 
   /**
@@ -432,7 +434,7 @@ public:
 
   /**
    * Read grid data from an msh file, either version 1 or version 2 of that
-   * file format. The GMSH formats are documented at
+   * file format. The Gmsh formats are documented at
    * http://www.geuz.org/gmsh/.
    *
    * @note The input function of deal.II does not distinguish between newline
@@ -455,6 +457,46 @@ public:
    */
   void read_tecplot (std::istream &in);
 
+  /**
+   * Read in a file supported by Assimp, and generate a Triangulation
+   * out of it.  If you specify a @p mesh_index, only the mesh with
+   * the given index will be extracted, otherwise all meshes which are
+   * present in the file will be used to generate the Triangulation.
+   *
+   * This function can only be used to read two-dimensional meshes (possibly
+   * embedded in three dimensions). This is the standard for graphical software
+   * such as blender, or 3D studio max, and that is what the original Assimp
+   * library was built for. We "bend" it to deal.II to support complex
+   * co-dimension one meshes and complex two-dimensional meshes.
+   *
+   * If @p remove_duplicates is set to true (the default), then
+   * duplicated vertices will be removed if their distance is lower
+   * than @p tol.
+   *
+   * Only the elements compatible with the given dimension and space dimension
+   * will be extracted from the mesh, and only those elements that are
+   * compatible with deal.II are supported. If you set
+   * `ignore_unsupported_element_types`, all the other element types are simply
+   * ignored by this algorithm. If your mesh contains a mixture of triangles
+   * and quadrilaterals, for example, only the quadrilaterals will be
+   * extracted. The resulting mesh (as represented in the Triangulation object)
+   * may not make any sense if you are mixing compatible and incompatible
+   * element types. If `ignore_unsupported_element_types` is set to `false`,
+   * then an exception is thrown when an unsupported type is encountered.
+   *
+   * @param filename The file to read from
+   * @param mesh_index Index of the mesh within the file
+   * @param remove_duplicates Remove duplicated vertices
+   * @param tol Tolerance to use when removing vertices
+   * @param ignore_unsupported_element_types Don't throw exceptions if we
+   *        encounter unsupported types during parsing
+   */
+  void read_assimp(const std::string &filename,
+                   const unsigned int mesh_index=numbers::invalid_unsigned_int,
+                   const bool remove_duplicates=true,
+                   const double tol = 1e-12,
+                   const bool ignore_unsupported_element_types = true);
+
   /**
    * Return the standard suffix for a file in this format.
    */
@@ -649,7 +691,6 @@ void
 GridIn<3>::debug_output_grid (const std::vector<CellData<3> > &cells,
                               const std::vector<Point<3> >    &vertices,
                               std::ostream                    &out);
-
 #endif // DOXYGEN
 
 DEAL_II_NAMESPACE_CLOSE
