@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 1999 - 2016 by the deal.II authors
+// Copyright (C) 1999 - 2018 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -13,8 +13,8 @@
 //
 // ---------------------------------------------------------------------
 
-#ifndef dealii__grid_out_h
-#define dealii__grid_out_h
+#ifndef dealii_grid_out_h
+#define dealii_grid_out_h
 
 
 
@@ -750,7 +750,7 @@ namespace GridOutFlags
 
   /**
    * Flags for grid output in Vtu format. These flags are the same as those
-   * declared in DataOutBase::VtuFlags.
+   * declared in DataOutBase::VtkFlags.
    *
    * @ingroup output
    */
@@ -919,12 +919,12 @@ public:
   template <int dim, int spacedim>
   void write_gnuplot (const Triangulation<dim,spacedim> &tria,
                       std::ostream           &out,
-                      const Mapping<dim,spacedim> *mapping=0) const;
+                      const Mapping<dim,spacedim> *mapping=nullptr) const;
 
   /**
    * Write the triangulation in the msh format.
    *
-   * Msh is the format used by Gmsh and it is described in the gmsh user's
+   * Msh is the format used by Gmsh and it is described in the Gmsh user's
    * guide. Besides the usual output of the grid only, you can decide through
    * additional flags (see below, and the documentation of the
    * GridOutFlags::Msh() class) whether boundary faces with non-zero boundary
@@ -998,7 +998,7 @@ public:
   template <int dim, int spacedim>
   void write_eps (const Triangulation<dim, spacedim> &tria,
                   std::ostream             &out,
-                  const Mapping<dim, spacedim>       *mapping=0) const;
+                  const Mapping<dim, spacedim>       *mapping=nullptr) const;
 
   /**
    * Write two-dimensional XFig-file.
@@ -1021,7 +1021,7 @@ public:
   template <int dim, int spacedim>
   void write_xfig (const Triangulation<dim, spacedim> &tria,
                    std::ostream              &out,
-                   const Mapping<dim, spacedim>        *mapping=0) const;
+                   const Mapping<dim, spacedim>        *mapping=nullptr) const;
 
   /**
    * Write the triangulation in the SVG format.
@@ -1074,6 +1074,20 @@ public:
 
   /**
    * Write triangulation in VTK format.
+   *
+   * Due to the way this function writes data to the output stream,
+   * the resulting output files correspond to a faithful representation
+   * of the mesh in that all cells are visible for visualization. However,
+   * the data is not in a format that allows reading this file in again
+   * through the GridIn class. This is because every vertex of the mesh is
+   * duplicated as many times as there are adjacent cells. In other words,
+   * every cell has its own, separate set of vertices that are at the
+   * same location as the vertices of other cells, but are separately
+   * numbered. If such a file is read in through the GridIn class, then
+   * that will result in a mesh that has the correct cells and vertex
+   * locations, but because the vertices are logically separate (though at
+   * the same locations) all cells are unconnected and have no neighbors
+   * across faces.
    */
   template <int dim, int spacedim>
   void write_vtk (const Triangulation<dim,spacedim> &tria,
@@ -1081,6 +1095,20 @@ public:
 
   /**
    * Write triangulation in VTU format.
+   *
+   * Due to the way this function writes data to the output stream,
+   * the resulting output files correspond to a faithful representation
+   * of the mesh in that all cells are visible for visualization. However,
+   * the data is not in a format that allows reading this file in again
+   * through the GridIn class. This is because every vertex of the mesh is
+   * duplicated as many times as there are adjacent cells. In other words,
+   * every cell has its own, separate set of vertices that are at the
+   * same location as the vertices of other cells, but are separately
+   * numbered. If such a file is read in through the GridIn class, then
+   * that will result in a mesh that has the correct cells and vertex
+   * locations, but because the vertices are logically separate (though at
+   * the same locations) all cells are unconnected and have no neighbors
+   * across faces.
    */
   template <int dim, int spacedim>
   void write_vtu (const Triangulation<dim,spacedim> &tria,
@@ -1095,9 +1123,9 @@ public:
    * set as true to view each level of a multilevel method. The input
    * <tt>include_artificial</tt> can be set as true to view the artificial cells for
    * each processor. Each .vtu and .pvtu file will have the attributes subdomain,
-   * level_subdomain, level, and proc_writing. The level value can be used to seperate the
+   * level_subdomain, level, and proc_writing. The level value can be used to separate the
    * image into the view of the grid on each level of a multilevel method and the
-   * proc_writing value can be used to seperate the image into each processor's owned and
+   * proc_writing value can be used to separate the image into each processor's owned and
    * ghost cells.
    * This is accomplished by applying the "warp by scalar" filter in paraview
    * to each of the values. After opening the .pvtu file of a mesh where the input
@@ -1108,7 +1136,7 @@ public:
    * then click Apply. This will give you the following image.
    * @image html write_mesh_vtu_levels.png
    * If the <tt>view_levels</tt> remains at false, thereby only giving the mesh for the active
-   * level, it is enough to seperate the image into the views written by different processors.
+   * level, it is enough to separate the image into the views written by different processors.
    * This is shown in the following image where the <tt>include_artificial</tt> input is set as true.
    * @image html write_mesh_vtu_active.png
    * Note: Depending on the size of the mesh you may need to increase the "Scale Factor" input
@@ -1128,7 +1156,7 @@ public:
   void write (const Triangulation<dim,spacedim> &tria,
               std::ostream                      &out,
               const OutputFormat                 output_format,
-              const Mapping<dim,spacedim>       *mapping=0) const;
+              const Mapping<dim,spacedim>       *mapping=nullptr) const;
 
   /**
    * Write mesh in default format set by ParameterHandler.
@@ -1136,7 +1164,7 @@ public:
   template <int dim, int spacedim>
   void write (const Triangulation<dim,spacedim> &tria,
               std::ostream                      &out,
-              const Mapping<dim,spacedim>       *mapping=0) const;
+              const Mapping<dim,spacedim>       *mapping=nullptr) const;
 
   /**
    * Set flags for DX output
@@ -1144,7 +1172,7 @@ public:
   void set_flags (const GridOutFlags::DX &flags);
 
   /**
-   * Set flags for GMSH output
+   * Set flags for Gmsh output
    */
   void set_flags (const GridOutFlags::Msh &flags);
 
@@ -1275,7 +1303,7 @@ private:
   GridOutFlags::DX dx_flags;
 
   /**
-   * Flags for GMSH output. Can be changed by using the set_flags(const
+   * Flags for Gmsh output. Can be changed by using the set_flags(const
    * GridOutFlags::Msh&) function.
    */
   GridOutFlags::Msh     msh_flags;
@@ -1340,7 +1368,7 @@ private:
    * printed which are on the boundary and which have a boundary indicator not
    * equal to zero, since the latter is the default for boundary faces.
    *
-   * Since, in GMSH, geometric elements are continuously numbered, this
+   * Since, in Gmsh, geometric elements are continuously numbered, this
    * function requires a parameter @p next_element_index providing the next
    * geometric element number. This index should have a numerical value equal
    * to one more than the index previously used to write a geometric element
@@ -1350,7 +1378,7 @@ private:
    *
    * @warning @p next_element_index should be (at least) one larger than the
    * current number of triangulation elements (lines, cells, faces) that have
-   * been written to @p out. GMSH will not load the saved file correctly if
+   * been written to @p out. Gmsh will not load the saved file correctly if
    * there are repeated indices.
    *
    * This function unfortunately can not be included in the regular @p
@@ -1393,7 +1421,7 @@ private:
    * printed which are on the boundary and which have a boundary indicator not
    * equal to zero, since the latter is the default for boundary faces.
    *
-   * Since, in GMSH, geometric elements are continuously numbered, this
+   * Since, in Gmsh, geometric elements are continuously numbered, this
    * function requires a parameter @p next_element_index providing the next
    * geometric element number. This index should have a numerical value equal
    * to one more than the index previously used to write a geometric element
@@ -1403,7 +1431,7 @@ private:
    *
    * @warning @p next_element_index should be (at least) one larger than the
    * current number of triangulation elements (lines, cells, faces) that have
-   * been written to @p out. GMSH will not load the saved file correctly if
+   * been written to @p out. Gmsh will not load the saved file correctly if
    * there are repeated indices.
    *
    * This function unfortunately can not be included in the regular @p
