@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 2000 - 2016 by the deal.II authors
+// Copyright (C) 2000 - 2018 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -13,8 +13,8 @@
 //
 // ---------------------------------------------------------------------
 
-#ifndef dealii__grid_refinement_h
-#define dealii__grid_refinement_h
+#ifndef dealii_grid_refinement_h
+#define dealii_grid_refinement_h
 
 
 #include <deal.II/base/config.h>
@@ -26,18 +26,22 @@ DEAL_II_NAMESPACE_OPEN
 
 // forward declarations
 template <int dim, int spacedim> class Triangulation;
-
+template <typename Number> class Vector;
 
 /**
- * Collection of functions controlling refinement and coarsening of
- * Triangulation objects.
+ * This namespace provides a collection of functions that aid in refinement
+ * and coarsening of triangulations. Despite the name of the namespace, the
+ * functions do not actually <i>refine</i> the triangulation, but only
+ * <i>mark cells for refinement or coarsening</i>. In other words, they
+ * perform the "mark" part of the typical "solve-estimate-mark-refine"
+ * cycle of the adaptive finite element loop.
  *
  * The functions in this namespace form two categories. There are the
  * auxiliary functions refine() and coarsen(). More important for users are
  * the other functions, which implement refinement strategies, as being found
  * in the literature on adaptive finite element methods. For mathematical
  * discussion of these methods, consider works by D&ouml;rfler, Morin,
- * Nochetto, Rannacher, Stevenson and many more.
+ * Nochetto, Rannacher, Stevenson, and others.
  *
  * @ingroup grid
  * @author Wolfgang Bangerth, Thomas Richter, Guido Kanschat 1998, 2000, 2009
@@ -83,7 +87,8 @@ namespace GridRefinement
                                              const double        bottom_fraction_of_cells);
 
   /**
-   * This function provides a refinement strategy with predictable growth in
+   * This function provides a strategy to mark cells for refinement and
+   * coarsening with the goal of providing predictable growth in
    * the size of the mesh by refining a given fraction of all cells.
    *
    * The function takes a vector of refinement @p criteria and two values
@@ -145,17 +150,18 @@ namespace GridRefinement
    * this number is only an indicator. The default value of this argument is
    * to impose no limit on the number of cells.
    */
-  template <int dim, class VectorType, int spacedim>
+  template <int dim, typename Number, int spacedim>
   void
   refine_and_coarsen_fixed_number
   (Triangulation<dim,spacedim> &triangulation,
-   const VectorType            &criteria,
+   const Vector<Number>        &criteria,
    const double                top_fraction_of_cells,
    const double                bottom_fraction_of_cells,
    const unsigned int          max_n_cells = std::numeric_limits<unsigned int>::max());
 
   /**
-   * This function provides a refinement strategy controlling the reduction of
+   * This function provides a strategy to mark cells for refinement and
+   * coarsening with the goal of controlling the reduction of
    * the error estimate.
    *
    * Also known as the <b>bulk criterion</b> or D&ouml;rfler marking,
@@ -209,11 +215,11 @@ namespace GridRefinement
    * this number is only an indicator. The default value of this argument is
    * to impose no limit on the number of cells.
    */
-  template <int dim, class VectorType, int spacedim>
+  template <int dim, typename Number, int spacedim>
   void
   refine_and_coarsen_fixed_fraction
   (Triangulation<dim,spacedim> &tria,
-   const VectorType            &criteria,
+   const Vector<Number>        &criteria,
    const double                top_fraction,
    const double                bottom_fraction,
    const unsigned int          max_n_cells = std::numeric_limits<unsigned int>::max());
@@ -221,7 +227,8 @@ namespace GridRefinement
 
 
   /**
-   * Refine the triangulation by flagging certain cells to reach a grid that
+   * This function flags cells of a triangulation for refinement with the
+   * aim to reach a grid that
    * is optimal with respect to an objective function that tries to balance
    * reducing the error and increasing the numerical cost when the mesh is
    * refined. Specifically, this function makes the assumption that if you
@@ -257,8 +264,8 @@ namespace GridRefinement
    * $m$ cells that will be refined. Note that $N(m)$ is an increasing
    * function of $m$ whereas $\eta^\text{exp}(m)$ is a decreasing function.
    *
-   * This function then tries to find that number $m$ of cells to refine for
-   * which the objective function
+   * This function then tries to find that number $m$ of cells to mark for
+   * refinement for which the objective function
    * @f[
    *   J(m) = N(m)^{\text{order}/d} \eta^\text{exp}(m)
    * @f]
@@ -292,18 +299,18 @@ namespace GridRefinement
    * thesis, University of Heidelberg, 2005. See in particular Section 4.3,
    * pp. 42-43.
    */
-  template <int dim, class VectorType, int spacedim>
+  template <int dim, typename Number, int spacedim>
   void
   refine_and_coarsen_optimize (Triangulation<dim,spacedim> &tria,
-                               const VectorType            &criteria,
+                               const Vector<Number>        &criteria,
                                const unsigned int          order=2);
 
   /**
-   * Flag all mesh cells for which the value in @p criteria exceeds @p
+   * Mark all mesh cells for which the value in @p criteria exceeds @p
    * threshold for refinement, but only flag up to @p max_to_mark cells.
    *
    * The vector @p criteria contains a nonnegative value for each active cell,
-   * ordered in the canonical order of of Triangulation::active_cell_iterator.
+   * ordered in the canonical order of Triangulation::active_cell_iterator.
    *
    * The cells are only flagged for refinement, they are not actually refined.
    * To do so, you have to call
@@ -312,18 +319,18 @@ namespace GridRefinement
    * This function does not implement a refinement strategy, it is more a
    * helper function for the actual strategies.
    */
-  template <int dim, class VectorType, int spacedim>
+  template <int dim, typename Number, int spacedim>
   void refine (Triangulation<dim,spacedim> &tria,
-               const VectorType            &criteria,
+               const Vector<Number>        &criteria,
                const double                threshold,
                const unsigned int          max_to_mark = numbers::invalid_unsigned_int);
 
   /**
-   * Flag all mesh cells for which the value in @p criteria is less than @p
+   * Mark all mesh cells for which the value in @p criteria is less than @p
    * threshold for coarsening.
    *
    * The vector @p criteria contains a nonnegative value for each active cell,
-   * ordered in the canonical order of of Triangulation::active_cell_iterator.
+   * ordered in the canonical order of Triangulation::active_cell_iterator.
    *
    * The cells are only flagged for coarsening, they are not actually
    * coarsened. To do so, you have to call
@@ -332,9 +339,9 @@ namespace GridRefinement
    * This function does not implement a refinement strategy, it is more a
    * helper function for the actual strategies.
    */
-  template <int dim, class VectorType, int spacedim>
+  template <int dim, typename Number, int spacedim>
   void coarsen (Triangulation<dim,spacedim> &tria,
-                const VectorType            &criteria,
+                const Vector<Number>        &criteria,
                 const double                threshold);
 
   /**
@@ -354,4 +361,4 @@ namespace GridRefinement
 
 DEAL_II_NAMESPACE_CLOSE
 
-#endif //dealii__grid_refinement_h
+#endif //dealii_grid_refinement_h
