@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 2000 - 2017 by the deal.II authors
+// Copyright (C) 2000 - 2018 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -13,8 +13,8 @@
 //
 // ---------------------------------------------------------------------
 
-#ifndef dealii__grid_reordering_h
-#define dealii__grid_reordering_h
+#ifndef dealii_grid_reordering_h
+#define dealii_grid_reordering_h
 
 
 #include <deal.II/base/config.h>
@@ -254,7 +254,7 @@ DeclExceptionMsg (ExcMeshNotOrientable,
  * which we could order as <tt>(9 6 7 10)</tt> above). However, since opposite
  * lines have to have the same direction, this in turn would force us to
  * rotate the cell left of it, and then the one left to that, and so on until
- * we reach the left end of the grid. This is therefore an example we we have
+ * we reach the left end of the grid. This is therefore an example we have
  * to track back right until the first column of three cells to find a
  * consistent ordering, if we had initially ordered them uniformly.
  *
@@ -288,7 +288,7 @@ DeclExceptionMsg (ExcMeshNotOrientable,
  * These two examples demonstrate that if we have added a certain number of
  * cells in some orientation of faces and can't add the next one without
  * introducing faces that had already been added in another direction, then it
- * might not be sufficient to only rotate cells in the neighborhood of the the
+ * might not be sufficient to only rotate cells in the neighborhood of the
  * cell that we failed to add. It might be necessary to go back a long way and
  * rotate cells that have been entered long ago.
  *
@@ -470,18 +470,18 @@ DeclExceptionMsg (ExcMeshNotOrientable,
  * It is obvious that this algorithm has linear run-time, since it only ever
  * touches each face exactly once.
  *
- * The algorithm just described is implemented in a specialization of this
- * class for the 2d case. A similar, but slightly more complex algorithm is
- * implemented in a specialization for 3d. It using sheets instead of strings
- * of cells to work on. If a grid is orientable, then the algorithm is able to
- * do its work in linear time; if it is not orientable, then it aborts in
+ * The algorithm just described in the two-dimensional case is
+ * implemented for both 2d and (in generalized form) for 3d in this
+ * class. The 3d case uses sheets instead of strings of cells to work
+ * on. If a grid is orientable, then the algorithm is able to do its
+ * work in linear time; if it is not orientable, then it aborts in
  * linear time as well.
  *
  * Both algorithms are described in the paper "On orienting edges of
- * unstructured two- and three-dimensional meshes", R. Agelek, M. Anderson, W.
- * Bangerth, W. L. Barth, accepted for publication in the ACM Transactions
- * on Mathematical Software. A preprint is available as <a
- * href="http://arxiv.org/abs/1512.02137">arxiv
+ * unstructured two- and three-dimensional meshes", R. Agelek,
+ * M. Anderson, W.  Bangerth, W. L. Barth, ACM Transactions on
+ * Mathematical Software, vol. 44, article 5, 2017. A preprint is
+ * available as <a href="http://arxiv.org/abs/1512.02137">arxiv
  * 1512.02137</a>.
  *
  *
@@ -672,17 +672,17 @@ public:
 
 
 // declaration of explicit specializations
-template<>
+template <>
 void
 GridReordering<2>::invert_all_cells_of_negative_grid(const std::vector<Point<2> > &all_vertices,
                                                      std::vector<CellData<2> >    &cells);
 
-template<>
+template <>
 void
 GridReordering<2,3>::invert_all_cells_of_negative_grid(const std::vector<Point<3> > &all_vertices,
                                                        std::vector<CellData<2> >    &cells);
 
-template<>
+template <>
 void
 GridReordering<3>::invert_all_cells_of_negative_grid(const std::vector<Point<3> > &all_vertices,
                                                      std::vector<CellData<3> >    &cells);
