@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 1999 - 2016 by the deal.II authors
+// Copyright (C) 1999 - 2017 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -13,8 +13,8 @@
 //
 // ---------------------------------------------------------------------
 
-#ifndef dealii__intergrid_map_h
-#define dealii__intergrid_map_h
+#ifndef dealii_intergrid_map_h
+#define dealii_intergrid_map_h
 
 #include <deal.II/base/config.h>
 #include <deal.II/base/smartpointer.h>
@@ -143,12 +143,12 @@ public:
   void clear ();
 
   /**
-   * Return a pointer to the source grid.
+   * Return a reference to the source grid.
    */
   const MeshType &get_source_grid () const;
 
   /**
-   * Return a pointer to the destination grid.
+   * Return a reference to the destination grid.
    */
   const MeshType &get_destination_grid () const;
 
