@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 1998 - 2017 by the deal.II authors
+// Copyright (C) 1998 - 2018 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -13,13 +13,14 @@
 //
 // ---------------------------------------------------------------------
 
-#ifndef dealii__tria_manifold_h
-#define dealii__tria_manifold_h
+#ifndef dealii_tria_manifold_h
+#define dealii_tria_manifold_h
 
 
 /*----------------------------   manifold.h     ---------------------------*/
 
 #include <deal.II/base/config.h>
+#include <deal.II/base/array_view.h>
 #include <deal.II/base/subscriptor.h>
 #include <deal.II/base/quadrature_lib.h>
 #include <deal.II/base/thread_management.h>
@@ -38,6 +39,25 @@ template <int, typename> class Table;
  */
 namespace Manifolds
 {
+  /**
+   * A <code>constexpr</code> helper function that returns the number of
+   * default points for the structure type pointed to by the given
+   * <code>MeshIteratorType</code>. See the documentation of
+   * Manifolds::get_default_points_and_weights() for more information.
+   */
+  template <typename MeshIteratorType>
+  inline
+  constexpr std::size_t n_default_points_per_cell()
+  {
+    // Note that in C++11 a constexpr function can only have a return
+    // statement, so we cannot alias the structure dimension
+    return GeometryInfo<MeshIteratorType::AccessorType::structure_dimension>::vertices_per_cell
+           + GeometryInfo<MeshIteratorType::AccessorType::structure_dimension>::lines_per_cell
+           + GeometryInfo<MeshIteratorType::AccessorType::structure_dimension>::quads_per_cell
+           + GeometryInfo<MeshIteratorType::AccessorType::structure_dimension>::hexes_per_cell
+           - 1; // don't count the cell itself, just the bounding objects
+  }
+
   /**
    * Given a general mesh iterator, construct a quadrature object that
    * contains the following points:
@@ -51,12 +71,13 @@ namespace Manifolds
    *   are the vertices, the line mid-points, and the face mid-points.
    *   This results in a quadrature object with 26 (8+12+6) points.
    *
-   * The quadrature weights for these points are either chosen identically
-   * and equal to one over the number of quadrature points (if @p with_laplace
-   * is @p false), or in a way that gives points closer to the cell center
-   * (measured on the reference cell) a higher weight. These weights correspond
-   * to solving a Laplace equation and evaluating the solution at the quadrature
-   * points (if @p with_laplace is @p true).
+   * The quadrature weights for these points are either chosen identically and
+   * equal to one over the number of quadrature points (if @p
+   * with_interpolation is @p false), or in a way that gives points closer to
+   * the cell center (measured on the reference cell) a higher weight. These
+   * weights correspond to the weights applied to lines and vertices in
+   * transfinite interpolation (see TransfiniteInterpolationManifold for a
+   * more thorough description) if @p with_interpolation is @p true.
    *
    * The function is primarily used to construct the input argument
    * for the Manifold::get_new_point() function, which computes a new
@@ -71,8 +92,8 @@ namespace Manifolds
    *
    * @param[in] iterator A mesh iterator that points to either a line, quad,
    *   or hex.
-   * @param[in] with_laplace Whether or not to compute the quadrature weights
-   *   by solving a Laplace equation, as discussed above.
+   * @param[in] with_interpolation Whether or not to compute the quadrature weights
+   *   from transfinite interpolation, as discussed above.
    * @tparam MeshIteratorType An iterator type that corresponds to either
    *   Triangulation::cell_iterator (or variants such as
    *   Triangulation::active_cell_iterator or DoFHandler::cell_iterator) or
@@ -80,12 +101,13 @@ namespace Manifolds
    *   <code>cell-@>face(f)</code> or <code>cell-@>line(l)</code>.
    */
   template <typename MeshIteratorType>
+  DEAL_II_DEPRECATED
   Quadrature<MeshIteratorType::AccessorType::space_dimension>
   get_default_quadrature(const MeshIteratorType &iterator,
-                         const bool              with_laplace = false) DEAL_II_DEPRECATED;
+                         const bool              with_interpolation = false);
 
   /**
-   * Given a general mesh iterator, construct vectors of quadrature points and
+   * Given a general mesh iterator, construct arrays of quadrature points and
    * weights that contain the following points:
    * - If the iterator points to a line, then the quadrature points
    *   are the two vertices of the line. This results in a point vector
@@ -97,12 +119,13 @@ namespace Manifolds
    *   are the vertices, the line mid-points, and the face mid-points.
    *   This results in a points vector with 26 (8+12+6) points.
    *
-   * The quadrature weights for these points are either chosen identically
-   * and equal to one over the number of quadrature points (if @p with_laplace
-   * is @p false), or in a way that gives points closer to the cell center
-   * (measured on the reference cell) a higher weight. These weights correspond
-   * to solving a Laplace equation and evaluating the solution at the quadrature
-   * points (if @p with_laplace is @p true).
+   * The quadrature weights for these points are either chosen identically and
+   * equal to one over the number of quadrature points (if @p
+   * with_interpolation is @p false), or in a way that gives points closer to
+   * the cell center (measured on the reference cell) a higher weight. These
+   * weights correspond to the weights applied to lines and vertices in
+   * transfinite interpolation (see TransfiniteInterpolationManifold for a
+   * more thorough description) if @p with_interpolation is @p true.
    *
    * The function is primarily used to construct the input argument
    * for the Manifold::get_new_point() function, which computes a new
@@ -117,8 +140,8 @@ namespace Manifolds
    *
    * @param[in] iterator A mesh iterator that points to either a line, quad,
    *   or hex.
-   * @param[in] with_laplace Whether or not to compute the quadrature weights
-   *   by solving a Laplace equation, as discussed above.
+   * @param[in] with_interpolation Whether or not to compute the quadrature weights
+   *   from transfinite interpolation, as discussed above.
    * @tparam MeshIteratorType An iterator type that corresponds to either
    *   Triangulation::cell_iterator (or variants such as
    *   Triangulation::active_cell_iterator or DoFHandler::cell_iterator) or
@@ -126,13 +149,13 @@ namespace Manifolds
    *   <code>cell-@>face(f)</code> or <code>cell-@>line(l)</code>.
    */
   template <typename MeshIteratorType>
-  std::pair<std::vector<Point<MeshIteratorType::AccessorType::space_dimension> >,
-      std::vector<double> >
+  std::pair<std::array<Point<MeshIteratorType::AccessorType::space_dimension>,
+      n_default_points_per_cell<MeshIteratorType>()>,
+      std::array<double, n_default_points_per_cell<MeshIteratorType>()> >
       get_default_points_and_weights(const MeshIteratorType &iterator,
-                                     const bool              with_laplace = false);
+                                     const bool              with_interpolation = false);
 }
 
-
 /**
  * Manifolds are used to describe the geometry of boundaries of domains as
  * well as the geometry of the interior. Manifold objects are therefore
@@ -181,7 +204,7 @@ namespace Manifolds
  * intermediate points. Internally the
  * Manifold::get_intermediate_point() calls the
  * Manifold::project_to_manifold() function after computing the convex
- * conbination of the given points.  This allows derived classes to
+ * combination of the given points.  This allows derived classes to
  * only overload Manifold::project_to_manifold() for simple
  * situations. This is often useful when describing manifolds that are
  * embedded in higher dimensional space, e.g., the surface of a
@@ -263,7 +286,7 @@ namespace Manifolds
  * and $\mathbf x_2$, with weights $w_1$ and $w_2=1-w_1$, simply
  * requires computing the point $\mathbf s(w_1)$. Computing a new
  * point as a weighted average of more than two points can be done
- * by considering pairwise geodetics, finding suitable points on
+ * by considering pairwise geodesics, finding suitable points on
  * the geodetic between the first two points, then on the geodetic
  * between this new point and the third given point, etc.
  *
@@ -302,11 +325,9 @@ class Manifold : public Subscriptor
 public:
 
   // explicitly check for sensible template arguments
-#ifdef DEAL_II_WITH_CXX11
   static_assert (dim<=spacedim,
                  "The dimension <dim> of a Manifold must be less than or "
                  "equal to the space dimension <spacedim> in which it lives.");
-#endif
 
 
   /**
@@ -327,7 +348,14 @@ public:
    * Destructor. Does nothing here, but needs to be declared virtual to make
    * class hierarchies derived from this class possible.
    */
-  virtual ~Manifold ();
+  virtual ~Manifold () = default;
+
+  /**
+   * Return a copy of this manifold.
+   *
+   * Every derived class should implement this operation in a sensible manner.
+   */
+  virtual std::unique_ptr<Manifold<dim,spacedim> > clone() const = 0;
 
   /**
    * @name Computing the location of points.
@@ -357,27 +385,6 @@ public:
                           const Point<spacedim> &p2,
                           const double w) const;
 
-  /**
-   * Return the point which shall become the new vertex surrounded by the
-   * given points which make up the quadrature. We use a quadrature object,
-   * which should be filled with the surrounding points together with
-   * appropriate weights.
-   *
-   * In its default implementation it uses a pair-wise reduction of
-   * the points in the quadrature formula by calling the function
-   * get_intermediate_point() on the first two points, then on the
-   * resulting point and the next, until all points in the quadrature
-   * have been taken into account. User classes can get away by simply
-   * implementing the get_intermediate_point() function. Notice that
-   * by default the get_intermediate_point() function calls the
-   * project_to_manifold() function with the convex combination of its
-   * arguments. For simple situations you may get away by implementing
-   * only the project_to_manifold() function.
-   */
-  virtual
-  Point<spacedim>
-  get_new_point (const Quadrature<spacedim> &quad) const DEAL_II_DEPRECATED;
-
   /**
    * Return the point which shall become the new vertex surrounded by the
    * given points @p surrounding_points. @p weights contains appropriate
@@ -396,20 +403,18 @@ public:
    */
   virtual
   Point<spacedim>
-  get_new_point (const std::vector<Point<spacedim> > &surrounding_points,
-                 const std::vector<double>           &weights) const;
+  get_new_point (const ArrayView<const Point<spacedim>> &surrounding_points,
+                 const ArrayView<const double>          &weights) const;
+
 
   /**
-   * Compute a new set of points that interpolate between the given points
-   * @p surrounding_points. @p weights is a table with as many columns as
-   * @p surrounding_points.size(). The number of rows in @p weights determines
-   * how many new points will be computed and appended to the last input
-   * argument @p new_points. After exit of this function, the size of
-   * @p new_points equals the size at entry plus the number of rows in
-   * @p weights.
+   * Compute a new set of points that interpolate between the given points @p
+   * surrounding_points. @p weights is a table with as many columns as @p
+   * surrounding_points.size(). The number of rows in @p weights must match
+   * the length of @p new_points.
    *
    * In its default implementation, this function simply calls get_new_point()
-   * on each row of @p weights and appends those points to the output vector
+   * on each row of @p weights and writes those points into the output array
    * @p new_points. However, this function is more efficient if multiple new
    * points need to be generated like in MappingQGeneric and the manifold does
    * expensive transformations between a chart space and the physical space,
@@ -419,31 +424,29 @@ public:
    * by implementing only the get_new_point() function.
    *
    * The implementation does not allow for @p surrounding_points and
-   * @p new_points to point to the same vector, so make sure to pass different
+   * @p new_points to point to the same array, so make sure to pass different
    * objects into the function.
    */
   virtual
   void
-  add_new_points (const std::vector<Point<spacedim> > &surrounding_points,
-                  const Table<2,double>               &weights,
-                  std::vector<Point<spacedim> >       &new_points) const;
+  get_new_points (const ArrayView<const Point<spacedim>> &surrounding_points,
+                  const Table<2,double>                  &weights,
+                  ArrayView<Point<spacedim>>              new_points) const;
 
   /**
    * Given a point which lies close to the given manifold, it modifies it and
    * projects it to manifold itself.
    *
    * This class is used by the default implementation of the function
-   * get_new_point(). It should be made pure virtual, but for historical
-   * reason, derived classes like Boundary<dim, spacedim> do not implement it.
-   * The default behavior of this class, however, is to throw an exception
-   * when called.
+   * get_new_point() and should be implemented by derived classes. The default
+   * implementation simply throws an exception if called.
    *
    * If your manifold is simple, you could implement this function only, and
    * the default behavior should work out of the box.
    */
   virtual
-  Point<spacedim> project_to_manifold (const std::vector<Point<spacedim> > &surrounding_points,
-                                       const Point<spacedim> &candidate) const;
+  Point<spacedim> project_to_manifold (const ArrayView<const Point<spacedim>> &surrounding_points,
+                                       const Point<spacedim>                  &candidate) const;
 
   /**
    * Backward compatibility interface.  Return the point which shall become
@@ -559,7 +562,7 @@ public:
    * $\mathbf s'(0) \approx \frac{\mathbf s(\epsilon)-\mathbf x_1}{\epsilon}$
    * for a small value of $\epsilon$, and the evaluation of $\mathbf s(\epsilon)$
    * is done by calling get_new_point(). If possible, derived classes should
-   * override this function by an implemention of the exact derivative.
+   * override this function by an implementation of the exact derivative.
    *
    * @param x1 The first point that describes the geodesic, and the one
    *   at which the "direction" is to be evaluated.
@@ -609,7 +612,7 @@ public:
    * The face iterator gives an indication which face this function is
    * supposed to compute the normal vector for.  This is useful if the
    * boundary of the domain is composed of different nondifferential
-   * pieces (for example when using the StraightBoundary class to
+   * pieces (for example when using the FlatManifold class to
    * approximate a geometry that is completely described by the coarse
    * mesh, with piecewise (bi-)linear components between the vertices,
    * but where the boundary may have a kink at the vertices itself).
@@ -620,16 +623,8 @@ public:
    * rotates it outward (with respect to the coordinate system of the edge)
    * by 90 degrees. In 3d, the default implementation is more
    * complicated, aiming at avoiding problems with numerical round-off
-   * for points close to one of the vertices. If the point p is closer
-   * to the center of the face than to any of the vertices, the
-   * normal vector is computed by the cross product of the tangent
-   * vectors from p to either vertex zero and one of the face (if
-   * the closest vertex is either vertex two or three), or of the tangent
-   * vectors from p to vertices two and three (if the closest vertex is
-   * either vertex zero or one). On the other hand, if the point p
-   * is closer to one of the vertices than to the center of the face,
-   * then we take the cross product of the tangent vectors from p
-   * to the two vertices that are adjacent to the closest one.
+   * for points close to one of the vertices, and avoiding tangent directions
+   * that are linearly dependent.
    */
   virtual
   Tensor<1,spacedim>
@@ -705,29 +700,9 @@ public:
                 const double tolerance=1e-10);
 
   /**
-   * Let the new point be the average sum of surrounding vertices.
-   *
-   * This particular implementation constructs the weighted average of the
-   * surrounding points, and then calls internally the function
-   * project_to_manifold(). The reason why we do it this way, is to allow lazy
-   * programmers to implement only the project_to_manifold() function for their
-   * own Manifold classes which are small (or trivial) perturbations of a flat
-   * manifold. This is the case whenever the coarse mesh is a decent
-   * approximation of the manifold geometry. In this case, the middle point of
-   * a cell is close to true middle point of the manifold, and a projection
-   * may suffice.
-   *
-   * For most simple geometries, it is possible to get reasonable results by
-   * deriving your own Manifold class from FlatManifold, and write a new
-   * interface only for the project_to_manifold function. You will have good
-   * approximations also with large deformations, as long as in the coarsest
-   * mesh size you are trying to refine, the middle point is not too far from
-   * the manifold mid point, i.e., as long as the coarse mesh size is small
-   * enough.
+   * Return a copy of this manifold.
    */
-  virtual
-  Point<spacedim>
-  get_new_point(const Quadrature<spacedim> &quad) const DEAL_II_DEPRECATED;
+  virtual std::unique_ptr<Manifold<dim,spacedim> > clone() const override;
 
   /**
    * Let the new point be the average sum of surrounding vertices.
@@ -752,17 +727,15 @@ public:
    */
   virtual
   Point<spacedim>
-  get_new_point(const std::vector<Point<spacedim> > &surrounding_points,
-                const std::vector<double>           &weights) const;
+  get_new_point(const ArrayView<const Point<spacedim>> &surrounding_points,
+                const ArrayView<const double>          &weights) const override;
+
 
   /**
-   * Compute a new set of points that interpolate between the given points
-   * @p surrounding_points. @p weights is a table with as many columns as
-   * @p surrounding_points.size(). The number of rows in @p weights determines
-   * how many new points will be computed and appended to the last input
-   * argument @p new_points. After exit of this function, the size of
-   * @p new_points equals the size at entry plus the number of rows in
-   * @p weights.
+   * Compute a new set of points that interpolate between the given points @p
+   * surrounding_points. @p weights is a table with as many columns as @p
+   * surrounding_points.size(). The number of rows in @p weights must match
+   * the length of @p new_points.
    *
    * For this particular implementation, the interpolation of the
    * @p surrounding_points according to the @p weights is simply performed in
@@ -770,9 +743,9 @@ public:
    */
   virtual
   void
-  add_new_points (const std::vector<Point<spacedim> > &surrounding_points,
-                  const Table<2,double>               &weights,
-                  std::vector<Point<spacedim> >       &new_points) const;
+  get_new_points (const ArrayView<const Point<spacedim>> &surrounding_points,
+                  const Table<2,double>                  &weights,
+                  ArrayView<Point<spacedim>>              new_points) const override;
 
   /**
    * Project to FlatManifold. This is the identity function for flat,
@@ -783,8 +756,8 @@ public:
    */
   virtual
   Point<spacedim>
-  project_to_manifold (const std::vector<Point<spacedim> > &points,
-                       const Point<spacedim> &candidate) const;
+  project_to_manifold (const ArrayView<const Point<spacedim>> &points,
+                       const Point<spacedim>                  &candidate) const override;
 
   /**
    * Return a vector that, at $\mathbf x_1$, is tangential to
@@ -810,7 +783,32 @@ public:
   virtual
   Tensor<1,spacedim>
   get_tangent_vector (const Point<spacedim> &x1,
-                      const Point<spacedim> &x2) const;
+                      const Point<spacedim> &x2) const override;
+
+  /**
+   * Return the normal vector to the given face at point p taking into account
+   * that quadrilateral faces of hexahedral cells in 3d may not be planar.
+   * In those cases, the face is assumed to have a geometry described by a
+   * bilinear function, and the normal vector is computed by embedding this
+   * bilinear form into a Cartesian space with a flat metric.
+   */
+  virtual
+  Tensor<1,spacedim>
+  normal_vector (const typename Triangulation<dim,spacedim>::face_iterator &face,
+                 const Point<spacedim> &p) const override;
+
+  /**
+   * Compute the normal vectors to the boundary at each vertex of the
+   * given face taking into account that quadrilateral faces of hexahedral
+   * cells in 3d may not be planar. In those cases, the face is assumed to
+   * have a geometry described by a bilinear function, and the normal vector
+   * is computed by embedding this bilinear form into a Cartesian space with
+   * a flat metric.
+   */
+  virtual
+  void
+  get_normals_at_vertices (const typename Triangulation<dim,spacedim>::face_iterator &face,
+                           typename Manifold<dim, spacedim>::FaceVertexNormals &face_vertex_normals) const override;
 
   /**
    * Return the periodicity of this Manifold.
@@ -938,11 +936,9 @@ class ChartManifold : public Manifold<dim,spacedim>
 {
 public:
   // explicitly check for sensible template arguments
-#ifdef DEAL_II_WITH_CXX11
   static_assert (dim<=spacedim,
                  "The dimension <dim> of a ChartManifold must be less than or "
                  "equal to the space dimension <spacedim> in which it lives.");
-#endif
 
   /**
    * Constructor. The optional argument can be used to specify the periodicity
@@ -964,8 +960,7 @@ public:
    * Destructor. Does nothing here, but needs to be declared to make it
    * virtual.
    */
-  virtual ~ChartManifold ();
-
+  virtual ~ChartManifold () override = default;
 
   /**
    * Refer to the general documentation of this class and the documentation of
@@ -973,7 +968,9 @@ public:
    */
   virtual
   Point<spacedim>
-  get_new_point(const Quadrature<spacedim> &quad) const DEAL_II_DEPRECATED;
+  get_intermediate_point (const Point<spacedim> &p1,
+                          const Point<spacedim> &p2,
+                          const double w) const override;
 
   /**
    * Refer to the general documentation of this class and the documentation of
@@ -981,17 +978,14 @@ public:
    */
   virtual
   Point<spacedim>
-  get_new_point(const std::vector<Point<spacedim> > &surrounding_points,
-                const std::vector<double>           &weights) const;
+  get_new_point(const ArrayView<const Point<spacedim>> &surrounding_points,
+                const ArrayView<const double>          &weights) const override;
 
   /**
-   * Compute a new set of points that interpolate between the given points
-   * @p surrounding_points. @p weights is a table with as many columns as
-   * @p surrounding_points.size(). The number of rows in @p weights determines
-   * how many new points will be computed and appended to the last input
-   * argument @p new_points. After exit of this function, the size of
-   * @p new_points equals the size at entry plus the number of rows in
-   * @p weights.
+   * Compute a new set of points that interpolate between the given points @p
+   * surrounding_points. @p weights is a table with as many columns as @p
+   * surrounding_points.size(). The number of rows in @p weights must match
+   * the length of @p new_points.
    *
    * The implementation of this function first transforms the
    * @p surrounding_points to the chart space by calling pull_back(). Then, new
@@ -1010,10 +1004,9 @@ public:
    */
   virtual
   void
-  add_new_points (const std::vector<Point<spacedim> > &surrounding_points,
-                  const Table<2,double>               &weights,
-                  std::vector<Point<spacedim> >       &new_points) const;
-
+  get_new_points (const ArrayView<const Point<spacedim>> &surrounding_points,
+                  const Table<2,double>                  &weights,
+                  ArrayView<Point<spacedim>>              new_points) const override;
   /**
    * Pull back the given point in spacedim to the Euclidean chartdim
    * dimensional space.
@@ -1112,7 +1105,7 @@ public:
   virtual
   Tensor<1,spacedim>
   get_tangent_vector (const Point<spacedim> &x1,
-                      const Point<spacedim> &x2) const;
+                      const Point<spacedim> &x2) const override;
 
   /**
    * Return the periodicity associated with the submanifold.
@@ -1135,8 +1128,6 @@ private:
 };
 
 
-
-
 /* -------------- declaration of explicit specializations ------------- */
 
 #ifndef DOXYGEN
@@ -1187,45 +1178,51 @@ namespace Manifolds
   template <typename MeshIteratorType>
   Quadrature<MeshIteratorType::AccessorType::space_dimension>
   get_default_quadrature(const MeshIteratorType &iterator,
-                         const bool              with_laplace)
+                         const bool              with_interpolation)
   {
-    const std::pair<std::vector<Point<MeshIteratorType::AccessorType::space_dimension> >,
-          std::vector<double> > points_and_weights = get_default_points_and_weights(iterator,
-                                                     with_laplace);
-    return Quadrature<MeshIteratorType::AccessorType::space_dimension>(points_and_weights.first,
-           points_and_weights.second);
+    const auto points_and_weights = get_default_points_and_weights(iterator, with_interpolation);
+    static const int spacedim = MeshIteratorType::AccessorType::space_dimension;
+    return Quadrature<spacedim>
+           (std::vector<Point<spacedim>>(points_and_weights.first.begin(),
+                                         points_and_weights.first.end()),
+            std::vector<double>(points_and_weights.second.begin(),
+                                points_and_weights.second.end()));
   }
 
-  template <typename MeshIteratorType>
-  std::pair<std::vector<Point<MeshIteratorType::AccessorType::space_dimension> >,
-      std::vector<double> >
-      get_default_points_and_weights(const MeshIteratorType &iterator,
-                                     const bool              with_laplace)
-  {
-    const int spacedim = MeshIteratorType::AccessorType::space_dimension;
-    const int dim = MeshIteratorType::AccessorType::structure_dimension;
 
-    std::pair<std::vector<Point<spacedim> >,
-        std::vector<double> > points_weights;
+
+  template <typename MeshIteratorType>
+  std::pair<std::array<Point<MeshIteratorType::AccessorType::space_dimension>,
+      n_default_points_per_cell<MeshIteratorType>()>,
+      std::array<double, n_default_points_per_cell<MeshIteratorType>()> >
+      get_default_points_and_weights(const MeshIteratorType &iterator,
+                                     const bool              with_interpolation)
+  {
+    const int dim = MeshIteratorType::AccessorType::structure_dimension;
+    const int spacedim = MeshIteratorType::AccessorType::space_dimension;
+    constexpr std::size_t points_per_cell = n_default_points_per_cell<MeshIteratorType>();
+
+    std::pair<std::array<Point<spacedim>, points_per_cell>, std::array<double, points_per_cell> >
+    points_weights;
 
 
     // note that the exact weights are chosen such as to minimize the
     // distortion of the four new quads from the optimal shape; their
     // derivation and values is copied over from the
-    // @p{MappingQ::set_laplace_on_vector} function
+    // interpolation function in the mapping
     switch (dim)
       {
       case 1:
-        points_weights.first.resize(2);
-        points_weights.second.resize(2);
+        Assert(points_weights.first.size() == 2, ExcInternalError());
+        Assert(points_weights.second.size() == 2, ExcInternalError());
         points_weights.first[0] = iterator->vertex(0);
         points_weights.second[0] = .5;
         points_weights.first[1] = iterator->vertex(1);
         points_weights.second[1] = .5;
         break;
       case 2:
-        points_weights.first.resize(8);
-        points_weights.second.resize(8);
+        Assert(points_weights.first.size() == 8, ExcInternalError());
+        Assert(points_weights.second.size() == 8, ExcInternalError());
 
         for (unsigned int i=0; i<4; ++i)
           {
@@ -1235,10 +1232,10 @@ namespace Manifolds
                                           iterator->line(i)->get_manifold().get_new_point_on_line(iterator->line(i)) );
           }
 
-        if (with_laplace)
+        if (with_interpolation)
           {
-            std::fill(points_weights.second.begin(), points_weights.second.begin()+4, 1.0/16.0);
-            std::fill(points_weights.second.begin()+4, points_weights.second.end(), 3.0/16.0);
+            std::fill(points_weights.second.begin(), points_weights.second.begin()+4, -0.25);
+            std::fill(points_weights.second.begin()+4, points_weights.second.end(), 0.5);
           }
         else
           std::fill(points_weights.second.begin(), points_weights.second.end(), 1.0/8.0);
@@ -1251,38 +1248,43 @@ namespace Manifolds
           GeometryInfo<dim>::vertices_per_cell+
           GeometryInfo<dim>::lines_per_cell+
           GeometryInfo<dim>::faces_per_cell;
-        points_weights.first.resize(np);
-        points_weights.second.resize(np);
-        std::vector<Point<3> > *sp3 = reinterpret_cast<std::vector<Point<3> > *>(&points_weights.first);
+        Assert(points_weights.first.size() == np, ExcInternalError());
+        Assert(points_weights.second.size() == np, ExcInternalError());
+        auto *sp3 = reinterpret_cast<std::array<Point<3>, n_default_points_per_cell<decltype(hex)>()> *>
+                    (&points_weights.first);
 
         unsigned int j=0;
 
         // note that the exact weights are chosen such as to minimize the
-        // distortion of the eight new hexes from the optimal shape; their
-        // derivation and values is copied over from the
-        // @p{MappingQ::set_laplace_on_vector} function
-        for (unsigned int i=0; i<GeometryInfo<dim>::vertices_per_cell; ++i, ++j)
+        // distortion of the eight new hexes from the optimal shape through
+        // transfinite interpolation from the faces and vertices, see
+        // TransfiniteInterpolationManifold for a deeper explanation of the
+        // mechanisms
+        if (with_interpolation)
           {
-            (*sp3)[j] = hex->vertex(i);
-            points_weights.second[j] = 1.0/128.0;
+            for (unsigned int i=0; i<GeometryInfo<dim>::vertices_per_cell; ++i, ++j)
+              {
+                (*sp3)[j] = hex->vertex(i);
+                points_weights.second[j] = 1.0/8.0;
+              }
+            for (unsigned int i=0; i<GeometryInfo<dim>::lines_per_cell; ++i, ++j)
+              {
+                (*sp3)[j] = (hex->line(i)->has_children() ?
+                             hex->line(i)->child(0)->vertex(1) :
+                             hex->line(i)->get_manifold().get_new_point_on_line(hex->line(i)));
+                points_weights.second[j] = -1.0/4.0;
+              }
+            for (unsigned int i=0; i<GeometryInfo<dim>::faces_per_cell; ++i, ++j)
+              {
+                (*sp3)[j] = (hex->quad(i)->has_children() ?
+                             hex->quad(i)->isotropic_child(0)->vertex(3) :
+                             hex->quad(i)->get_manifold().get_new_point_on_quad(hex->quad(i)));
+                points_weights.second[j] = 1.0/2.0;
+              }
           }
-        for (unsigned int i=0; i<GeometryInfo<dim>::lines_per_cell; ++i, ++j)
-          {
-            (*sp3)[j] = (hex->line(i)->has_children() ?
-                         hex->line(i)->child(0)->vertex(1) :
-                         hex->line(i)->get_manifold().get_new_point_on_line(hex->line(i)));
-            points_weights.second[j] = 7.0/192.0;
-          }
-        for (unsigned int i=0; i<GeometryInfo<dim>::faces_per_cell; ++i, ++j)
-          {
-            (*sp3)[j] = (hex->quad(i)->has_children() ?
-                         hex->quad(i)->isotropic_child(0)->vertex(3) :
-                         hex->quad(i)->get_manifold().get_new_point_on_quad(hex->quad(i)));
-            points_weights.second[j] = 1.0/12.0;
-          }
-        // Overwrite the weights with 1/np if we don't want to use
-        // laplace vectors.
-        if (with_laplace == false)
+        else
+          // Overwrite the weights with 1/np if we don't want to use
+          // interpolation.
           std::fill(points_weights.second.begin(), points_weights.second.end(), 1.0/np);
       }
       break;
