@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 1999 - 2016 by the deal.II authors
+// Copyright (C) 1999 - 2017 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -13,8 +13,8 @@
 //
 // ---------------------------------------------------------------------
 
-#ifndef dealii__persistent_tria_h
-#define dealii__persistent_tria_h
+#ifndef dealii_persistent_tria_h
+#define dealii_persistent_tria_h
 
 
 #include <deal.II/base/config.h>
@@ -133,7 +133,7 @@ public:
   /**
    * Destructor.
    */
-  virtual ~PersistentTriangulation ();
+  virtual ~PersistentTriangulation () = default;
 
   /**
    * Overloaded version of the same function in the base class which stores
