@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 1998 - 2016 by the deal.II authors
+// Copyright (C) 1998 - 2018 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -13,8 +13,8 @@
 //
 // ---------------------------------------------------------------------
 
-#ifndef dealii__tria_boundary_h
-#define dealii__tria_boundary_h
+#ifndef dealii_tria_boundary_h
+#define dealii_tria_boundary_h
 
 
 /*----------------------------   boundary-function.h     ---------------------------*/
@@ -73,12 +73,15 @@ DEAL_II_NAMESPACE_OPEN
  * right into the middle of the given points, and HyperBallBoundary creating a
  * hyperball with given radius around a given center point.
  *
+ * @deprecated This class has been deprecated in favor of the more general
+ * Manifold class.
+ *
  * @ingroup boundary
  * @author Wolfgang Bangerth, 1999, 2001, 2009, Ralf Hartmann, 2001, 2008,
  * Luca Heltai, 2014
  */
 template <int dim, int spacedim=dim>
-class Boundary : public FlatManifold<dim, spacedim>
+class DEAL_II_DEPRECATED Boundary : public FlatManifold<dim, spacedim>
 {
 public:
 
@@ -86,8 +89,12 @@ public:
    * Destructor. Does nothing here, but needs to be declared to make it
    * virtual.
    */
-  virtual ~Boundary ();
+  virtual ~Boundary () override = default;
 
+  /**
+   * Clone this Boundary.
+   */
+  virtual std::unique_ptr<Manifold<dim,spacedim> > clone() const override;
 
   /**
    * Return intermediate points on a line spaced according to the interior
@@ -210,7 +217,7 @@ private:
   /**
    * Point generator for the intermediate points on a boundary.
    */
-  mutable std::vector<std_cxx11::shared_ptr<QGaussLobatto<1> > > points;
+  mutable std::vector<std::shared_ptr<QGaussLobatto<1> > > points;
 
   /**
    * Mutex for protecting the points array.
@@ -230,12 +237,15 @@ private:
  * boundary of the domain is given by the polygon/polyhedron defined by the
  * boundary of the initial coarse triangulation.
  *
+ * @deprecated This class has been deprecated in favor of the more general
+ * FlatManifold class.
+ *
  * @ingroup boundary
  *
  * @author Wolfgang Bangerth, 1998, 2001, Ralf Hartmann, 2001
  */
 template <int dim, int spacedim=dim>
-class StraightBoundary : public Boundary<dim,spacedim>
+class DEAL_II_DEPRECATED StraightBoundary : public Boundary<dim,spacedim>
 {
 public:
   /**
@@ -243,6 +253,11 @@ public:
    */
   StraightBoundary ();
 
+  /**
+   * Clone this Boundary.
+   */
+  virtual std::unique_ptr<Manifold<dim,spacedim> > clone() const override;
+
   /**
    * Let the new point be the arithmetic mean of the two vertices of the line.
    *
@@ -250,7 +265,7 @@ public:
    * the base class for more information.
    */
   virtual Point<spacedim>
-  get_new_point_on_line (const typename Triangulation<dim,spacedim>::line_iterator &line) const;
+  get_new_point_on_line (const typename Triangulation<dim,spacedim>::line_iterator &line) const override;
 
   /**
    * Let the new point be the arithmetic mean of the four vertices of this
@@ -262,10 +277,10 @@ public:
    */
   virtual
   Point<spacedim>
-  get_new_point_on_quad (const typename Triangulation<dim,spacedim>::quad_iterator &quad) const;
+  get_new_point_on_quad (const typename Triangulation<dim,spacedim>::quad_iterator &quad) const override;
 
   /**
-   * Gives <tt>n=points.size()</tt> points that splits the StraightBoundary
+   * Return <tt>n=points.size()</tt> points that split the StraightBoundary
    * line into $n+1$ partitions of equal lengths.
    *
    * Refer to the general documentation of this class and the documentation of
@@ -274,10 +289,10 @@ public:
   virtual
   void
   get_intermediate_points_on_line (const typename Triangulation<dim,spacedim>::line_iterator &line,
-                                   std::vector<Point<spacedim> > &points) const;
+                                   std::vector<Point<spacedim> > &points) const override;
 
   /**
-   * Gives <tt>n=points.size()=m*m</tt> points that splits the
+   * Return <tt>n=points.size()=m*m</tt> points that splits the
    * StraightBoundary quad into $(m+1)(m+1)$ subquads of equal size.
    *
    * Refer to the general documentation of this class and the documentation of
@@ -286,7 +301,7 @@ public:
   virtual
   void
   get_intermediate_points_on_quad (const typename Triangulation<dim,spacedim>::quad_iterator &quad,
-                                   std::vector<Point<spacedim> > &points) const;
+                                   std::vector<Point<spacedim> > &points) const override;
 
   /**
    * Implementation of the function declared in the base class.
@@ -297,7 +312,7 @@ public:
   virtual
   Tensor<1,spacedim>
   normal_vector (const typename Triangulation<dim,spacedim>::face_iterator &face,
-                 const Point<spacedim> &p) const;
+                 const Point<spacedim> &p) const override;
 
   /**
    * Compute the normals to the boundary at the vertices of the given face.
@@ -308,7 +323,7 @@ public:
   virtual
   void
   get_normals_at_vertices (const typename Triangulation<dim,spacedim>::face_iterator &face,
-                           typename Boundary<dim,spacedim>::FaceVertexNormals &face_vertex_normals) const;
+                           typename Boundary<dim,spacedim>::FaceVertexNormals &face_vertex_normals) const override;
 
   /**
    * Given a candidate point and a line segment characterized by the iterator,
@@ -327,7 +342,7 @@ public:
   virtual
   Point<spacedim>
   project_to_surface (const typename Triangulation<dim,spacedim>::line_iterator &line,
-                      const Point<spacedim> &candidate) const;
+                      const Point<spacedim> &candidate) const override;
 
   /**
    * Same function as above but for a point that is to be projected onto the
@@ -343,7 +358,7 @@ public:
   virtual
   Point<spacedim>
   project_to_surface (const typename Triangulation<dim,spacedim>::quad_iterator &quad,
-                      const Point<spacedim> &candidate) const;
+                      const Point<spacedim> &candidate) const override;
 
   /**
    * Same function as above but for a point that is to be projected onto the
@@ -360,7 +375,7 @@ public:
   virtual
   Point<spacedim>
   project_to_surface (const typename Triangulation<dim,spacedim>::hex_iterator &hex,
-                      const Point<spacedim> &candidate) const;
+                      const Point<spacedim> &candidate) const override;
 };
 
 
@@ -386,21 +401,6 @@ void
 Boundary<1,3>::
 get_intermediate_points_on_face (const Triangulation<1,3>::face_iterator &,
                                  std::vector<Point<3> > &) const;
-template <>
-void
-StraightBoundary<1,1>::
-get_normals_at_vertices (const Triangulation<1,1>::face_iterator &,
-                         Boundary<1,1>::FaceVertexNormals &) const;
-template <>
-void
-StraightBoundary<2,2>::
-get_normals_at_vertices (const Triangulation<2,2>::face_iterator &face,
-                         Boundary<2,2>::FaceVertexNormals &face_vertex_normals) const;
-template <>
-void
-StraightBoundary<3,3>::
-get_normals_at_vertices (const Triangulation<3,3>::face_iterator &face,
-                         Boundary<3,3>::FaceVertexNormals &face_vertex_normals) const;
 
 template <>
 Point<3>
