@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 1999 - 2016 by the deal.II authors
+// Copyright (C) 1999 - 2018 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -13,8 +13,8 @@
 //
 // ---------------------------------------------------------------------
 
-#ifndef dealii__tria_boundary_lib_h
-#define dealii__tria_boundary_lib_h
+#ifndef dealii_tria_boundary_lib_h
+#define dealii_tria_boundary_lib_h
 
 
 #include <deal.II/base/config.h>
@@ -41,12 +41,17 @@ DEAL_II_NAMESPACE_OPEN
  * which would seem natural, since this way we can use the
  * StraightBoundary::in_between() function.
  *
+ * @deprecated The boundary classes have been deprecated in favor of the more
+ * general Manifold classes. For this class CylindricalManifold is an appropriate
+ * replacement which is expected to produce better results under mesh refinement.
+ * For a two-dimensional cylinder use FlatManifold instead.
+ *
  * @ingroup boundary
  *
  * @author Guido Kanschat, 2001, Wolfgang Bangerth, 2007
  */
 template <int dim, int spacedim = dim>
-class CylinderBoundary : public StraightBoundary<dim,spacedim>
+class DEAL_II_DEPRECATED CylinderBoundary : public StraightBoundary<dim,spacedim>
 {
 public:
   /**
@@ -68,18 +73,23 @@ public:
                     const Point<spacedim> &point_on_axis);
 
   /**
-   * Refer to the general documentation of this class and the documentation of
-   * the base class.
+   * Clone this Boundary object.
    */
-  virtual Point<spacedim>
-  get_new_point_on_line (const typename Triangulation<dim,spacedim>::line_iterator &line) const;
+  virtual std::unique_ptr<Manifold<dim,spacedim> > clone() const override;
 
   /**
    * Refer to the general documentation of this class and the documentation of
    * the base class.
    */
   virtual Point<spacedim>
-  get_new_point_on_quad (const typename Triangulation<dim,spacedim>::quad_iterator &quad) const;
+  get_new_point_on_line (const typename Triangulation<dim,spacedim>::line_iterator &line) const override;
+
+  /**
+   * Refer to the general documentation of this class and the documentation of
+   * the base class.
+   */
+  virtual Point<spacedim>
+  get_new_point_on_quad (const typename Triangulation<dim,spacedim>::quad_iterator &quad) const override;
 
   /**
    * Refer to the general documentation of this class and the documentation of
@@ -89,7 +99,7 @@ public:
    */
   virtual void
   get_intermediate_points_on_line (const typename Triangulation<dim,spacedim>::line_iterator &line,
-                                   std::vector<Point<spacedim> > &points) const;
+                                   std::vector<Point<spacedim> > &points) const override;
 
   /**
    * Refer to the general documentation of this class and the documentation of
@@ -99,7 +109,7 @@ public:
    */
   virtual void
   get_intermediate_points_on_quad (const typename Triangulation<dim,spacedim>::quad_iterator &quad,
-                                   std::vector<Point<spacedim> > &points) const;
+                                   std::vector<Point<spacedim> > &points) const override;
 
   /**
    * Compute the normals to the boundary at the vertices of the given face.
@@ -109,7 +119,7 @@ public:
    */
   virtual void
   get_normals_at_vertices (const typename Triangulation<dim,spacedim>::face_iterator &face,
-                           typename Boundary<dim,spacedim>::FaceVertexNormals &face_vertex_normals) const;
+                           typename Boundary<dim,spacedim>::FaceVertexNormals &face_vertex_normals) const override;
 
   /**
    * Return the radius of the cylinder.
@@ -189,10 +199,14 @@ private:
  * @image html cone_2d.png
  * @image html cone_3d.png
  *
+ * @deprecated The boundary classes have been deprecated in favor of the more
+ * general Manifold classes. For this class CylindricalManifold is an appropriate
+ * replacement for spacedim==3, else StraightManifold can be used.
+ *
  * @author Markus B&uuml;rg, 2009
  */
 template <int dim>
-class ConeBoundary : public StraightBoundary<dim>
+class DEAL_II_DEPRECATED ConeBoundary : public StraightBoundary<dim>
 {
 public:
   /**
@@ -207,6 +221,11 @@ public:
                 const Point<dim> x_0,
                 const Point<dim> x_1);
 
+  /**
+   * Clone this Boundary object.
+   */
+  virtual std::unique_ptr<Manifold<dim,dim> > clone() const override;
+
   /**
    * Return the radius of the (truncated) cone at given point <tt>x</tt> on
    * the axis.
@@ -219,7 +238,7 @@ public:
    */
   virtual
   Point<dim>
-  get_new_point_on_line (const typename Triangulation<dim>::line_iterator &line) const;
+  get_new_point_on_line (const typename Triangulation<dim>::line_iterator &line) const override;
 
   /**
    * Refer to the general documentation of this class and the documentation of
@@ -227,7 +246,7 @@ public:
    */
   virtual
   Point<dim>
-  get_new_point_on_quad (const typename Triangulation<dim>::quad_iterator &quad) const;
+  get_new_point_on_quad (const typename Triangulation<dim>::quad_iterator &quad) const override;
 
   /**
    * Refer to the general documentation of this class and the documentation of
@@ -238,7 +257,7 @@ public:
   virtual
   void
   get_intermediate_points_on_line (const typename Triangulation<dim>::line_iterator &line,
-                                   std::vector<Point<dim> > &points) const;
+                                   std::vector<Point<dim> > &points) const override;
 
   /**
    * Refer to the general documentation of this class and the documentation of
@@ -249,7 +268,7 @@ public:
   virtual
   void
   get_intermediate_points_on_quad (const typename Triangulation<dim>::quad_iterator &quad,
-                                   std::vector<Point<dim> > &points) const;
+                                   std::vector<Point<dim> > &points) const override;
 
   /**
    * Compute the normals to the boundary at the vertices of the given face.
@@ -260,7 +279,19 @@ public:
   virtual
   void
   get_normals_at_vertices (const typename Triangulation<dim>::face_iterator &face,
-                           typename Boundary<dim>::FaceVertexNormals &face_vertex_normals) const;
+                           typename Boundary<dim>::FaceVertexNormals &face_vertex_normals) const override;
+
+  /**
+   * Implementation of the function declared in the base class.
+   *
+   * Refer to the general documentation of this class and the documentation of
+   * the base class.
+   */
+  virtual
+  Tensor<1,dim>
+  normal_vector (const typename Triangulation<dim>::face_iterator &face,
+                 const Point<dim> &p) const override;
+
 
 protected:
   /**
@@ -314,10 +345,14 @@ private:
  *
  * @ingroup boundary
  *
+ * @deprecated The boundary classes have been deprecated in favor of the more
+ * general Manifold classes. For this class SphericalManifold is an appropriate
+ * replacement which is expected to produce better results under mesh refinement.
+ *
  * @author Wolfgang Bangerth, 1998, Ralf Hartmann, 2001
  */
 template <int dim, int spacedim=dim>
-class HyperBallBoundary : public StraightBoundary<dim,spacedim>
+class DEAL_II_DEPRECATED HyperBallBoundary : public StraightBoundary<dim,spacedim>
 {
 public:
   /**
@@ -327,12 +362,9 @@ public:
                      const double     radius = 1.0);
 
   /**
-   * Refer to the general documentation of this class and the documentation of
-   * the base class.
+   * Clone this Boundary object.
    */
-  virtual
-  Point<spacedim>
-  get_new_point_on_line (const typename Triangulation<dim,spacedim>::line_iterator &line) const;
+  virtual std::unique_ptr<Manifold<dim,spacedim> > clone() const override;
 
   /**
    * Refer to the general documentation of this class and the documentation of
@@ -340,7 +372,15 @@ public:
    */
   virtual
   Point<spacedim>
-  get_new_point_on_quad (const typename Triangulation<dim,spacedim>::quad_iterator &quad) const;
+  get_new_point_on_line (const typename Triangulation<dim,spacedim>::line_iterator &line) const override;
+
+  /**
+   * Refer to the general documentation of this class and the documentation of
+   * the base class.
+   */
+  virtual
+  Point<spacedim>
+  get_new_point_on_quad (const typename Triangulation<dim,spacedim>::quad_iterator &quad) const override;
 
   /**
    * Refer to the general documentation of this class and the documentation of
@@ -351,7 +391,7 @@ public:
   virtual
   void
   get_intermediate_points_on_line (const typename Triangulation<dim,spacedim>::line_iterator &line,
-                                   std::vector<Point<spacedim> > &points) const;
+                                   std::vector<Point<spacedim> > &points) const override;
 
   /**
    * Refer to the general documentation of this class and the documentation of
@@ -362,7 +402,7 @@ public:
   virtual
   void
   get_intermediate_points_on_quad (const typename Triangulation<dim,spacedim>::quad_iterator &quad,
-                                   std::vector<Point<spacedim> > &points) const;
+                                   std::vector<Point<spacedim> > &points) const override;
 
   /**
    * Implementation of the function declared in the base class.
@@ -373,7 +413,7 @@ public:
   virtual
   Tensor<1,spacedim>
   normal_vector (const typename Triangulation<dim,spacedim>::face_iterator &face,
-                 const Point<spacedim> &p) const;
+                 const Point<spacedim> &p) const override;
 
   /**
    * Compute the normals to the boundary at the vertices of the given face.
@@ -384,7 +424,7 @@ public:
   virtual
   void
   get_normals_at_vertices (const typename Triangulation<dim,spacedim>::face_iterator &face,
-                           typename Boundary<dim,spacedim>::FaceVertexNormals &face_vertex_normals) const;
+                           typename Boundary<dim,spacedim>::FaceVertexNormals &face_vertex_normals) const override;
 
   /**
    * Return the center of the ball.
@@ -452,10 +492,14 @@ private:
  *
  * @ingroup boundary
  *
+ * @deprecated The boundary classes have been deprecated in favor of the more
+ * general Manifold classes. For this class SphericalManifold is an appropriate
+ * replacement which is expected to produce better results under mesh refinement.
+ *
  * @author Wolfgang Bangerth, 1999, 2001
  */
 template <int dim>
-class HalfHyperBallBoundary : public HyperBallBoundary<dim>
+class DEAL_II_DEPRECATED HalfHyperBallBoundary : public HyperBallBoundary<dim>
 {
 public:
   /**
@@ -465,16 +509,21 @@ public:
                          const double     radius = 1.0);
 
   /**
-   * Check if on the line <tt>x==0</tt>, otherwise pass to the base class.
+   * Clone this Boundary object.
    */
-  virtual Point<dim>
-  get_new_point_on_line (const typename Triangulation<dim>::line_iterator &line) const;
+  virtual std::unique_ptr<Manifold<dim,dim> > clone() const override;
 
   /**
    * Check if on the line <tt>x==0</tt>, otherwise pass to the base class.
    */
   virtual Point<dim>
-  get_new_point_on_quad (const typename Triangulation<dim>::quad_iterator &quad) const;
+  get_new_point_on_line (const typename Triangulation<dim>::line_iterator &line) const override;
+
+  /**
+   * Check if on the line <tt>x==0</tt>, otherwise pass to the base class.
+   */
+  virtual Point<dim>
+  get_new_point_on_quad (const typename Triangulation<dim>::quad_iterator &quad) const override;
 
   /**
    * Refer to the general documentation of this class and the documentation of
@@ -484,7 +533,7 @@ public:
    */
   virtual void
   get_intermediate_points_on_line (const typename Triangulation<dim>::line_iterator &line,
-                                   std::vector<Point<dim> > &points) const;
+                                   std::vector<Point<dim> > &points) const override;
 
   /**
    * Refer to the general documentation of this class and the documentation of
@@ -494,7 +543,7 @@ public:
    */
   virtual void
   get_intermediate_points_on_quad (const typename Triangulation<dim>::quad_iterator &quad,
-                                   std::vector<Point<dim> > &points) const;
+                                   std::vector<Point<dim> > &points) const override;
 
   /**
    * Compute the normals to the boundary at the vertices of the given face.
@@ -504,7 +553,7 @@ public:
    */
   virtual void
   get_normals_at_vertices (const typename Triangulation<dim>::face_iterator &face,
-                           typename Boundary<dim>::FaceVertexNormals &face_vertex_normals) const;
+                           typename Boundary<dim>::FaceVertexNormals &face_vertex_normals) const override;
 };
 
 
@@ -516,10 +565,14 @@ public:
  *
  * @ingroup boundary
  *
+ * @deprecated The boundary classes have been deprecated in favor of the more
+ * general Manifold classes. For this class SphericalManifold is an appropriate
+ * replacement which is expected to produce better results under mesh refinement.
+ *
  * @author Wolfgang Bangerth, 1999
  */
 template <int dim>
-class HyperShellBoundary : public HyperBallBoundary<dim>
+class DEAL_II_DEPRECATED HyperShellBoundary : public HyperBallBoundary<dim>
 {
 public:
   /**
@@ -529,6 +582,11 @@ public:
    * radius as argument. This radius will be ignored
    */
   HyperShellBoundary (const Point<dim> &center = Point<dim>());
+
+  /**
+   * Clone this Boundary object.
+   */
+  virtual std::unique_ptr<Manifold<dim,dim> > clone() const override;
 };
 
 
@@ -543,10 +601,14 @@ public:
  *
  * @ingroup boundary
  *
+ * @deprecated The boundary classes have been deprecated in favor of the more
+ * general Manifold classes. For this class SphericalManifold is an appropriate
+ * replacement which is expected to produce better results under mesh refinement.
+ *
  * @author Wolfgang Bangerth, 2000, 2009
  */
 template <int dim>
-class HalfHyperShellBoundary : public HyperShellBoundary<dim>
+class DEAL_II_DEPRECATED HalfHyperShellBoundary : public HyperShellBoundary<dim>
 {
 public:
   /**
@@ -560,17 +622,22 @@ public:
                           const double inner_radius = -1,
                           const double outer_radius = -1);
 
+  /**
+   * Clone this Boundary object.
+   */
+  virtual std::unique_ptr<Manifold<dim,dim> > clone() const override;
+
   /**
    * Construct a new point on a line.
    */
   virtual Point<dim>
-  get_new_point_on_line (const typename Triangulation<dim>::line_iterator &line) const;
+  get_new_point_on_line (const typename Triangulation<dim>::line_iterator &line) const override;
 
   /**
    * Construct a new point on a quad.
    */
   virtual Point<dim>
-  get_new_point_on_quad (const typename Triangulation<dim>::quad_iterator &quad) const;
+  get_new_point_on_quad (const typename Triangulation<dim>::quad_iterator &quad) const override;
 
   /**
    * Refer to the general documentation of this class and the documentation of
@@ -580,7 +647,7 @@ public:
    */
   virtual void
   get_intermediate_points_on_line (const typename Triangulation<dim>::line_iterator &line,
-                                   std::vector<Point<dim> > &points) const;
+                                   std::vector<Point<dim> > &points) const override;
 
   /**
    * Refer to the general documentation of this class and the documentation of
@@ -590,7 +657,7 @@ public:
    */
   virtual void
   get_intermediate_points_on_quad (const typename Triangulation<dim>::quad_iterator &quad,
-                                   std::vector<Point<dim> > &points) const;
+                                   std::vector<Point<dim> > &points) const override;
 
   /**
    * Compute the normals to the boundary at the vertices of the given face.
@@ -600,7 +667,7 @@ public:
    */
   virtual void
   get_normals_at_vertices (const typename Triangulation<dim>::face_iterator &face,
-                           typename Boundary<dim>::FaceVertexNormals &face_vertex_normals) const;
+                           typename Boundary<dim>::FaceVertexNormals &face_vertex_normals) const override;
 
 private:
   /**
@@ -618,9 +685,12 @@ private:
  *
  * This class is only implemented for <tt>dim=2</tt>,<tt>spacedim=3</tt>, that
  * is, just the surface.
+ *
+ * @deprecated This class has been deprecated in favor of the more general
+ * TorusManifold class.
  */
 template <int dim, int spacedim>
-class TorusBoundary : public Boundary<dim,spacedim>
+class DEAL_II_DEPRECATED TorusBoundary : public Boundary<dim,spacedim>
 {
 public:
   /**
@@ -628,32 +698,37 @@ public:
    */
   TorusBoundary (const double R, const double r);
 
+  /**
+   * Clone this Boundary object.
+   */
+  virtual std::unique_ptr<Manifold<dim,spacedim> > clone() const override;
+
 //Boundary Refinement Functions
   /**
    * Construct a new point on a line.
    */
   virtual Point<spacedim>
-  get_new_point_on_line (const typename Triangulation<dim,spacedim>::line_iterator &line) const;
+  get_new_point_on_line (const typename Triangulation<dim,spacedim>::line_iterator &line) const override;
 
   /**
    * Construct a new point on a quad.
    */
   virtual Point<spacedim>
-  get_new_point_on_quad (const typename Triangulation<dim,spacedim>::quad_iterator &quad) const;
+  get_new_point_on_quad (const typename Triangulation<dim,spacedim>::quad_iterator &quad) const override;
 
   /**
    * Construct a new points on a line.
    */
   virtual void   get_intermediate_points_on_line (
     const typename Triangulation< dim, spacedim >::line_iterator   &line,
-    std::vector< Point< spacedim > >         &points) const;
+    std::vector< Point< spacedim > >         &points) const override;
 
   /**
    * Construct a new points on a quad.
    */
   virtual void  get_intermediate_points_on_quad (
     const typename Triangulation< dim, spacedim >::quad_iterator &quad,
-    std::vector< Point< spacedim > >         &points ) const;
+    std::vector< Point< spacedim > >         &points ) const override;
 
   /**
    * Get the normal from cartesian coordinates. This normal does not have unit
@@ -661,7 +736,7 @@ public:
    */
   virtual void get_normals_at_vertices (
     const typename Triangulation< dim, spacedim >::face_iterator &face,
-    typename Boundary<dim,spacedim>::FaceVertexNormals &face_vertex_normals) const;
+    typename Boundary<dim,spacedim>::FaceVertexNormals &face_vertex_normals) const override;
 
 private:
   //Handy functions
