@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 2006 - 2015 by the deal.II authors
+// Copyright (C) 2006 - 2018 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -13,8 +13,8 @@
 //
 // ---------------------------------------------------------------------
 
-#ifndef dealii__tria_faces_h
-#define dealii__tria_faces_h
+#ifndef dealii_tria_faces_h
+#define dealii_tria_faces_h
 
 #include <deal.II/base/config.h>
 #include <deal.II/grid/tria_object.h>
@@ -25,7 +25,7 @@ DEAL_II_NAMESPACE_OPEN
 
 namespace internal
 {
-  namespace Triangulation
+  namespace TriangulationImplementation
   {
     /**
      * General template for information belonging to the faces of a
@@ -45,15 +45,15 @@ namespace internal
      * @author Tobias Leicht, 2006
      */
 
-    template<int dim>
+    template <int dim>
     class TriaFaces
     {
-    private:
+    public:
       /**
-       * Make the constructor private so no one can use this general template.
-       * Only the specializations should be used.
+       * Constructor. This constructor is deleted to prevent the use of this template,
+       * as only the specializations should be used
        */
-      TriaFaces();
+      TriaFaces() = delete;
     };
 
 
@@ -64,7 +64,7 @@ namespace internal
      * <tt>dim@>=2</tt> the use of TriaFaces is reasonable, for <tt>dim=1</tt>
      * the class is empty.
      */
-    template<>
+    template <>
     class TriaFaces<1>
     {
 
@@ -87,7 +87,7 @@ namespace internal
     /**
      * In <tt>dim=2</tt> the cells are quads, the faces accordingly are lines.
      */
-    template<>
+    template <>
     class TriaFaces<2>
     {
     public:
@@ -116,7 +116,7 @@ namespace internal
      * In <tt>dim=3</tt> the cells are hexes, the faces accordingly are quads.
      * In addition to that we also have to enable the storage of lines.
      */
-    template<>
+    template <>
     class TriaFaces<3>
     {
     public:
