@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 1998 - 2016 by the deal.II authors
+// Copyright (C) 1998 - 2018 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -13,8 +13,8 @@
 //
 // ---------------------------------------------------------------------
 
-#ifndef dealii__tria_iterator_h
-#define dealii__tria_iterator_h
+#ifndef dealii_tria_iterator_h
+#define dealii_tria_iterator_h
 
 
 /*----------------------------   tria-iterator.h     ---------------------------*/
@@ -222,7 +222,7 @@ template <typename> class TriaActiveIterator;
  * @author documentation update Guido Kanschat, 2004
  */
 template <typename Accessor>
-class TriaRawIterator : public std::iterator<std::bidirectional_iterator_tag,Accessor>
+class TriaRawIterator
 {
 public:
   /**
@@ -272,7 +272,7 @@ public:
   TriaRawIterator (const Triangulation<Accessor::dimension,Accessor::space_dimension> *parent,
                    const int level,
                    const int index,
-                   const typename AccessorType::AccessorData *local_data = 0);
+                   const typename AccessorType::AccessorData *local_data = nullptr);
 
   /**
    * This is a conversion operator (constructor) which takes another iterator
@@ -464,6 +464,16 @@ public:
    */
   std::size_t memory_consumption () const;
 
+  /**
+   * Mark the class as bidirectional iterator and declare some typedefs which
+   * are standard for iterators and are used by algorithms to enquire about the
+   * specifics of the iterators they work on.
+   */
+  typedef std::bidirectional_iterator_tag iterator_category;
+  typedef Accessor                        value_type;
+  typedef int                             difference_type;
+  typedef Accessor                       *pointer;
+  typedef Accessor                       &reference;
 
   /**@name Exceptions*/
   /*@{*/
@@ -482,8 +492,7 @@ public:
                        "past_the_end" : "invalid")));
 
   /**
-   * Exception for lower-dimensional TriaObjects without level, i.e. objects
-   * faces are constructed with.
+   * Exception.
    */
   DeclException1 (ExcDereferenceInvalidObject,
                   Accessor,
@@ -568,7 +577,7 @@ public:
   TriaIterator (const Triangulation<Accessor::dimension,Accessor::space_dimension> *parent,
                 const int                 level,
                 const int                 index,
-                const typename Accessor::AccessorData *local_data = 0);
+                const typename Accessor::AccessorData *local_data = nullptr);
 
   /**
    * Construct from an accessor of type OtherAccessor that is convertible to
@@ -674,6 +683,17 @@ public:
   TriaIterator<Accessor> operator -- (int);
   /*@}*/
 
+  /**
+   * Declare some typedefs which are standard for iterators and are used
+   * by algorithms to enquire about the specifics of the iterators they
+   * work on.
+   */
+  typedef typename TriaRawIterator<Accessor>::iterator_category iterator_category;
+  typedef typename TriaRawIterator<Accessor>::value_type        value_type;
+  typedef typename TriaRawIterator<Accessor>::pointer           pointer;
+  typedef typename TriaRawIterator<Accessor>::reference         reference;
+  typedef typename TriaRawIterator<Accessor>::difference_type   difference_type;
+
   /**
    * Exception
    */
@@ -843,6 +863,17 @@ public:
   TriaActiveIterator<Accessor> operator -- (int);
   /*@}*/
 
+  /**
+   * Declare some typedefs which are standard for iterators and are used
+   * by algorithms to enquire about the specifics of the iterators they
+   * work on.
+   */
+  typedef typename TriaIterator<Accessor>::iterator_category iterator_category;
+  typedef typename TriaIterator<Accessor>::value_type        value_type;
+  typedef typename TriaIterator<Accessor>::pointer           pointer;
+  typedef typename TriaIterator<Accessor>::reference         reference;
+  typedef typename TriaIterator<Accessor>::difference_type   difference_type;
+
   /**
    * Exception
    */
