@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 1998 - 2015 by the deal.II authors
+// Copyright (C) 1998 - 2017 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -13,8 +13,8 @@
 //
 // ---------------------------------------------------------------------
 
-#ifndef dealii__tria_iterator_templates_h
-#define dealii__tria_iterator_templates_h
+#ifndef dealii_tria_iterator_templates_h
+#define dealii_tria_iterator_templates_h
 
 
 #include <deal.II/base/config.h>
@@ -37,7 +37,7 @@ template <typename Accessor>
 inline
 TriaRawIterator<Accessor>::TriaRawIterator ()
   :
-  accessor (0, -2, -2, 0)
+  accessor (nullptr, -2, -2, nullptr)
 {}
 
 
@@ -68,7 +68,7 @@ TriaRawIterator<Accessor>::TriaRawIterator (
   const TriaAccessorBase<Accessor::structure_dimension,Accessor::dimension,Accessor::space_dimension> &tria_accessor,
   const typename Accessor::AccessorData *local_data)
   :
-  accessor(0, -2, -2, local_data)
+  accessor(nullptr, -2, -2, local_data)
 {
   accessor.copy_from(tria_accessor);
 }
