@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 1999 - 2015 by the deal.II authors
+// Copyright (C) 1999 - 2017 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -13,8 +13,8 @@
 //
 // ---------------------------------------------------------------------
 
-#ifndef dealii__tria_iterator_base_h
-#define dealii__tria_iterator_base_h
+#ifndef dealii_tria_iterator_base_h
+#define dealii_tria_iterator_base_h
 
 
 #include <deal.II/base/config.h>
