@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 1998 - 2015 by the deal.II authors
+// Copyright (C) 1998 - 2018 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -13,8 +13,8 @@
 //
 // ---------------------------------------------------------------------
 
-#ifndef dealii__tria_levels_h
-#define dealii__tria_levels_h
+#ifndef dealii_tria_levels_h
+#define dealii_tria_levels_h
 
 
 #include <deal.II/base/config.h>
@@ -25,11 +25,13 @@
 
 #include <boost/serialization/utility.hpp>
 
+#include <cstdint>
+
 DEAL_II_NAMESPACE_OPEN
 
 namespace internal
 {
-  namespace Triangulation
+  namespace TriangulationImplementation
   {
     /**
      * Store all information which belongs to one level of the multilevel
@@ -62,7 +64,7 @@ namespace internal
        * equals the length of the @p lines vector, in two dimensions that of
        * the @p quads vector, etc.
        */
-      std::vector<unsigned char> refine_flags;
+      std::vector<std::uint8_t> refine_flags;
 
       /**
        * Same meaning as the one above, but specifies whether a cell must be
@@ -188,14 +190,6 @@ namespace internal
       void serialize(Archive &ar,
                      const unsigned int version);
 
-      /**
-       * Exception
-       */
-      DeclException3 (ExcMemoryWasted,
-                      char *, int, int,
-                      << "The container " << arg1 << " contains "
-                      << arg2 << " elements, but it`s capacity is "
-                      << arg3 << ".");
       /**
        * Exception
        */
@@ -212,11 +206,11 @@ namespace internal
      * instead of TriaObjects. Refer to the documentation of the general class
      * template for details.
      */
-    template<>
+    template <>
     class TriaLevel<3>
     {
     public:
-      std::vector<unsigned char> refine_flags;
+      std::vector<std::uint8_t> refine_flags;
       std::vector<bool> coarsen_flags;
       std::vector<unsigned int> active_cell_indices;
       std::vector<std::pair<int,int> > neighbors;
@@ -249,14 +243,6 @@ namespace internal
       void serialize(Archive &ar,
                      const unsigned int version);
 
-      /**
-       * Exception
-       */
-      DeclException3 (ExcMemoryWasted,
-                      char *, int, int,
-                      << "The container " << arg1 << " contains "
-                      << arg2 << " elements, but it`s capacity is "
-                      << arg3 << ".");
       /**
        * Exception
        */
