@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 2006 - 2017 by the deal.II authors
+// Copyright (C) 2006 - 2018 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -13,8 +13,8 @@
 //
 // ---------------------------------------------------------------------
 
-#ifndef dealii__tria_objects_h
-#define dealii__tria_objects_h
+#ifndef dealii_tria_objects_h
+#define dealii_tria_objects_h
 
 #include <deal.II/base/config.h>
 #include <deal.II/base/exceptions.h>
@@ -43,7 +43,7 @@ template <int, int, int> class TriaAccessor;
 
 namespace internal
 {
-  namespace Triangulation
+  namespace TriangulationImplementation
   {
 
     /**
@@ -301,14 +301,6 @@ namespace internal
       void serialize(Archive &ar,
                      const unsigned int version);
 
-      /**
-       * Exception
-       */
-      DeclException3 (ExcMemoryWasted,
-                      char *, int, int,
-                      << "The container " << arg1 << " contains "
-                      << arg2 << " elements, but it`s capacity is "
-                      << arg3 << ".");
       /**
        * Exception
        * @ingroup Exceptions
@@ -318,14 +310,6 @@ namespace internal
                       << "The containers have sizes " << arg1 << " and "
                       << arg2 << ", which is not as expected.");
 
-      /**
-       * Exception
-       */
-      DeclException2 (ExcWrongIterator,
-                      char *, char *,
-                      << "You asked for the next free " << arg1 << "_iterator, "
-                      "but you can only ask for " << arg2 <<"_iterators.");
-
       /**
        * dealii::Triangulation objects can either access a user pointer or a
        * user index. What you tried to do is trying to access one of those
@@ -371,7 +355,7 @@ namespace internal
          */
         UserData()
         {
-          p = 0;
+          p = nullptr;
         }
 
         /**
@@ -430,13 +414,15 @@ namespace internal
 
 
       /**
-       * For edges, we enforce a standard convention that opposite edges
-       * should be parallel. Now, that's enforceable in most cases, and we
-       * have code that makes sure that if a mesh allows this to happen, that
-       * we have this convention. We also know that it is always possible to
-       * have opposite faces have parallel normal vectors. (For both things,
-       * see the Agelek, Anderson, Bangerth, Barth paper mentioned in the
-       * documentation of the GridReordering class.)
+       * For edges, we enforce a standard convention that opposite
+       * edges should be parallel. Now, that's enforceable in most
+       * cases, and we have code that makes sure that if a mesh allows
+       * this to happen, that we have this convention. We also know
+       * that it is always possible to have opposite faces have
+       * parallel normal vectors. (For both things, see the paper by
+       * Agelek, Anderson, Bangerth, Barth in the ACM Transactions on
+       * Mathematical Software mentioned in the documentation of the
+       * GridReordering class.)
        *
        * The problem is that we originally had another condition, namely that
        * faces 0, 2 and 6 have normals that point into the cell, while the
@@ -601,7 +587,7 @@ namespace internal
     }
 
 
-    template<typename G>
+    template <typename G>
     inline
     bool
     TriaObjects<G>::
@@ -611,7 +597,7 @@ namespace internal
     }
 
 
-    template<typename G>
+    template <typename G>
     inline
     void *&
     TriaObjects<G>::user_pointer (const unsigned int i)
@@ -625,7 +611,7 @@ namespace internal
     }
 
 
-    template<typename G>
+    template <typename G>
     inline
     const void *
     TriaObjects<G>::user_pointer (const unsigned int i) const
@@ -639,7 +625,7 @@ namespace internal
     }
 
 
-    template<typename G>
+    template <typename G>
     inline
     unsigned int &
     TriaObjects<G>::user_index (const unsigned int i)
@@ -653,7 +639,7 @@ namespace internal
     }
 
 
-    template<typename G>
+    template <typename G>
     inline
     void
     TriaObjects<G>::clear_user_data (const unsigned int i)
@@ -674,7 +660,7 @@ namespace internal
     {}
 
 
-    template<typename G>
+    template <typename G>
     inline
     unsigned int TriaObjects<G>::user_index (const unsigned int i) const
     {
@@ -687,17 +673,17 @@ namespace internal
     }
 
 
-    template<typename G>
+    template <typename G>
     inline
     void TriaObjects<G>::clear_user_data ()
     {
       user_data_type = data_unknown;
       for (unsigned int i=0; i<user_data.size(); ++i)
-        user_data[i].p = 0;
+        user_data[i].p = nullptr;
     }
 
 
-    template<typename G>
+    template <typename G>
     inline
     void TriaObjects<G>::clear_user_flags ()
     {
@@ -705,7 +691,7 @@ namespace internal
     }
 
 
-    template<typename G>
+    template <typename G>
     template <class Archive>
     void
     TriaObjects<G>::UserData::serialize (Archive &ar,
@@ -862,7 +848,7 @@ namespace internal
 
 // declaration of explicit specializations
 
-    template<>
+    template <>
     void
     TriaObjects<TriaObject<2> >::monitor_memory (const unsigned int) const;
 
