@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 2006 - 2015 by the deal.II authors
+// Copyright (C) 2006 - 2018 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -13,8 +13,8 @@
 //
 // ---------------------------------------------------------------------
 
-#ifndef dealii__hp_dof_faces_h
-#define dealii__hp_dof_faces_h
+#ifndef dealii_hp_dof_faces_h
+#define dealii_hp_dof_faces_h
 
 
 #include <deal.II/base/config.h>
@@ -24,13 +24,6 @@
 
 DEAL_II_NAMESPACE_OPEN
 
-namespace hp
-{
-  template <int dim, int spacedim>
-  class FECollection;
-}
-
-
 namespace internal
 {
   namespace hp
@@ -42,7 +35,7 @@ namespace internal
      * the DoF indices on cells.
      *
      * The things we store here is very similar to what is stored in the
-     * internal::DoFHandler::DoFObjects classes (see there for more
+     * internal::DoFHandlerImplementation::DoFObjects classes (see there for more
      * information, in particular on the layout of the class hierarchy, and
      * the use of file names).
      *
@@ -51,7 +44,7 @@ namespace internal
      * For hp methods, not all cells may use the same finite element, and it
      * is consequently more complicated to determine where the DoF indices for
      * a given line, quad, or hex are stored. As described in the
-     * documentation of the internal::DoFHandler::DoFLevel class, we can
+     * documentation of the internal::DoFHandlerImplementation::DoFLevel class, we can
      * compute the location of the first line DoF, for example, by calculating
      * the offset as <code>line_index *
      * dof_handler.get_fe().dofs_per_line</code>. This of course doesn't work
@@ -237,7 +230,7 @@ namespace internal
      * @ingroup dofs
      * @author Tobias Leicht, 2006
      */
-    template<int dim>
+    template <int dim>
     class DoFIndicesOnFaces;
 
 
@@ -248,7 +241,7 @@ namespace internal
      * @ingroup hp
      * @author Tobias Leicht, 2006
      */
-    template<>
+    template <>
     class DoFIndicesOnFaces<1>
     {
     public:
@@ -274,7 +267,7 @@ namespace internal
      * @ingroup hp
      * @author Tobias Leicht, 2006
      */
-    template<>
+    template <>
     class DoFIndicesOnFaces<2>
     {
     public:
@@ -305,7 +298,7 @@ namespace internal
      * @ingroup hp
      * @author Tobias Leicht, 2006
      */
-    template<>
+    template <>
     class DoFIndicesOnFaces<3>
     {
     public:
@@ -371,12 +364,12 @@ namespace internal
       Assert ((fe_index != dealii::hp::DoFHandler<dim,spacedim>::default_fe_index),
               ExcMessage ("You need to specify a FE index when working "
                           "with hp DoFHandlers"));
-      Assert (fe_index < dof_handler.get_fe().size(),
-              ExcIndexRange (fe_index, 0, dof_handler.get_fe().size()));
+      Assert (fe_index < dof_handler.get_fe_collection().size(),
+              ExcIndexRange (fe_index, 0, dof_handler.get_fe_collection().size()));
       Assert (local_index <
-              dof_handler.get_fe()[fe_index].template n_dofs_per_object<structdim>(),
+              dof_handler.get_fe(fe_index).template n_dofs_per_object<structdim>(),
               ExcIndexRange(local_index, 0,
-                            dof_handler.get_fe()[fe_index]
+                            dof_handler.get_fe(fe_index)
                             .template n_dofs_per_object<structdim>()));
       Assert (obj_index < dof_offsets.size(),
               ExcIndexRange (obj_index, 0, dof_offsets.size()));
@@ -406,7 +399,7 @@ namespace internal
             return *(pointer + 1 + local_index);
           else
             pointer += static_cast<types::global_dof_index>(
-                         dof_handler.get_fe()[*pointer]
+                         dof_handler.get_fe(*pointer)
                          .template n_dofs_per_object<structdim>() + 1);
         }
     }
@@ -428,12 +421,12 @@ namespace internal
       Assert ((fe_index != dealii::hp::DoFHandler<dim,spacedim>::default_fe_index),
               ExcMessage ("You need to specify a FE index when working "
                           "with hp DoFHandlers"));
-      Assert (fe_index < dof_handler.get_fe().size(),
-              ExcIndexRange (fe_index, 0, dof_handler.get_fe().size()));
+      Assert (fe_index < dof_handler.get_fe_collection().size(),
+              ExcIndexRange (fe_index, 0, dof_handler.get_fe_collection().size()));
       Assert (local_index <
-              dof_handler.get_fe()[fe_index].template n_dofs_per_object<structdim>(),
+              dof_handler.get_fe(fe_index).template n_dofs_per_object<structdim>(),
               ExcIndexRange(local_index, 0,
-                            dof_handler.get_fe()[fe_index]
+                            dof_handler.get_fe(fe_index)
                             .template n_dofs_per_object<structdim>()));
       Assert (obj_index < dof_offsets.size(),
               ExcIndexRange (obj_index, 0, dof_offsets.size()));
@@ -465,7 +458,7 @@ namespace internal
               return;
             }
           else
-            pointer += dof_handler.get_fe()[*pointer]
+            pointer += dof_handler.get_fe(*pointer)
                        .template n_dofs_per_object<structdim>() + 1;
         }
     }
@@ -507,7 +500,7 @@ namespace internal
           else
             {
               ++counter;
-              pointer += dof_handler.get_fe()[*pointer]
+              pointer += dof_handler.get_fe(*pointer)
                          .template n_dofs_per_object<structdim>() + 1;
             }
         }
@@ -557,14 +550,14 @@ namespace internal
 
           const unsigned int fe_index = *pointer;
 
-          Assert (fe_index < dof_handler.get_fe().size(),
+          Assert (fe_index < dof_handler.get_fe_collection().size(),
                   ExcInternalError());
 
           if (counter == n)
             return fe_index;
 
           ++counter;
-          pointer += dof_handler.get_fe()[fe_index]
+          pointer += dof_handler.get_fe(fe_index)
                      .template n_dofs_per_object<structdim>() + 1;
         }
     }
@@ -586,8 +579,8 @@ namespace internal
       Assert ((fe_index != dealii::hp::DoFHandler<dim,spacedim>::default_fe_index),
               ExcMessage ("You need to specify a FE index when working "
                           "with hp DoFHandlers"));
-      Assert (fe_index < dof_handler.get_fe().size(),
-              ExcIndexRange (fe_index, 0, dof_handler.get_fe().size()));
+      Assert (fe_index < dof_handler.get_fe_collection().size(),
+              ExcIndexRange (fe_index, 0, dof_handler.get_fe_collection().size()));
 
       // make sure we are on an
       // object for which DoFs have
@@ -615,7 +608,7 @@ namespace internal
             return true;
           else
             pointer += static_cast<types::global_dof_index>(
-                         dof_handler.get_fe()[*pointer]
+                         dof_handler.get_fe(*pointer)
                          .template n_dofs_per_object<structdim>()+1);
         }
     }
