@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 2003 - 2017 by the deal.II authors
+// Copyright (C) 2003 - 2018 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -13,8 +13,8 @@
 //
 // ---------------------------------------------------------------------
 
-#ifndef dealii__hp_dof_level_h
-#define dealii__hp_dof_level_h
+#ifndef dealii_hp_dof_level_h
+#define dealii_hp_dof_level_h
 
 
 #include <deal.II/base/config.h>
@@ -36,12 +36,12 @@ namespace internal
 {
   namespace hp
   {
-    namespace DoFHandler
+    namespace DoFHandlerImplementation
     {
       struct Implementation;
     }
   }
-  namespace DoFCellAccessor
+  namespace DoFCellAccessorImplementation
   {
     struct Implementation;
   }
@@ -318,8 +318,8 @@ namespace internal
        * class that needs to create these data structures.
        */
       template <int, int> friend class dealii::hp::DoFHandler;
-      friend struct dealii::internal::hp::DoFHandler::Implementation;
-      friend struct dealii::internal::DoFCellAccessor::Implementation;
+      friend struct dealii::internal::hp::DoFHandlerImplementation::Implementation;
+      friend struct dealii::internal::DoFCellAccessorImplementation::Implementation;
     };
 
 
