@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 2003 - 2016 by the deal.II authors
+// Copyright (C) 2003 - 2018 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -13,8 +13,8 @@
 //
 // ---------------------------------------------------------------------
 
-#ifndef dealii__hp_fe_values_h
-#define dealii__hp_fe_values_h
+#ifndef dealii_hp_fe_values_h
+#define dealii_hp_fe_values_h
 
 #include <deal.II/base/config.h>
 #include <deal.II/fe/fe.h>
@@ -24,7 +24,7 @@
 #include <deal.II/fe/fe_values.h>
 
 #include <map>
-#include <deal.II/base/std_cxx11/shared_ptr.h>
+#include <memory>
 
 DEAL_II_NAMESPACE_OPEN
 
@@ -153,7 +153,7 @@ namespace internal
        * Initially, all entries have zero pointers, and we will allocate them
        * lazily as needed in select_fe_values().
        */
-      dealii::Table<3,std_cxx11::shared_ptr<FEValuesType> > fe_values_table;
+      dealii::Table<3,std::shared_ptr<FEValuesType> > fe_values_table;
 
       /**
        * Set of indices pointing at the fe_values object selected last time
@@ -279,7 +279,7 @@ namespace hp
      * passed to the constructor of this class with index given by
      * <code>cell-@>active_fe_index()</code>. Consequently, the
      * hp::FECollection argument given to this object should really be the
-     * same as that used in the construction of the hp::DofHandler associated
+     * same as that used in the construction of the hp::DoFHandler associated
      * with the present cell. On the other hand, if a value is given for this
      * argument, it overrides the choice of
      * <code>cell-@>active_fe_index()</code>.
@@ -414,7 +414,7 @@ namespace hp
      * passed to the constructor of this class with index given by
      * <code>cell-@>active_fe_index()</code>. Consequently, the
      * hp::FECollection argument given to this object should really be the
-     * same as that used in the construction of the hp::DofHandler associated
+     * same as that used in the construction of the hp::DoFHandler associated
      * with the present cell. On the other hand, if a value is given for this
      * argument, it overrides the choice of
      * <code>cell-@>active_fe_index()</code>.
