@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 2010 - 2016 by the deal.II authors
+// Copyright (C) 2010 - 2017 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -13,8 +13,8 @@
 //
 // ---------------------------------------------------------------------
 
-#ifndef dealii__integrators_divergence_h
-#define dealii__integrators_divergence_h
+#ifndef dealii_integrators_divergence_h
+#define dealii_integrators_divergence_h
 
 
 #include <deal.II/base/config.h>
@@ -266,7 +266,7 @@ namespace LocalIntegrators
      * @author Guido Kanschat
      * @date 2011
      */
-    template<int dim>
+    template <int dim>
     void
     u_dot_n_matrix (
       FullMatrix<double> &M,
@@ -303,7 +303,7 @@ namespace LocalIntegrators
      * @author Guido Kanschat
      * @date 2011
      */
-    template<int dim, typename number>
+    template <int dim, typename number>
     void
     u_dot_n_residual (
       Vector<number> &result,
@@ -339,7 +339,7 @@ namespace LocalIntegrators
      * @author Guido Kanschat
      * @date 2013
      */
-    template<int dim, typename number>
+    template <int dim, typename number>
     void
     u_times_n_residual (
       Vector<number> &result,
@@ -374,7 +374,7 @@ namespace LocalIntegrators
      * @author Guido Kanschat
      * @date 2011
      */
-    template<int dim>
+    template <int dim>
     void
     u_dot_n_matrix (
       FullMatrix<double> &M11,
@@ -427,10 +427,11 @@ namespace LocalIntegrators
      * @deprecated Use LocalIntegrators::GradDiv::cell_matrix() instead.
      */
     template <int dim>
+    DEAL_II_DEPRECATED
     void grad_div_matrix (
       FullMatrix<double> &M,
       const FEValuesBase<dim> &fe,
-      const double factor = 1.) DEAL_II_DEPRECATED;
+      const double factor = 1.);
 
     template <int dim>
     void grad_div_matrix (
@@ -445,11 +446,12 @@ namespace LocalIntegrators
      * @deprecated Use LocalIntegrators::GradDiv::cell_residual() instead.
      */
     template <int dim, typename number>
+    DEAL_II_DEPRECATED
     void grad_div_residual (
       Vector<number> &result,
       const FEValuesBase<dim> &fetest,
       const VectorSlice<const std::vector<std::vector<Tensor<1,dim> > > > &input,
-      const double factor = 1.) DEAL_II_DEPRECATED;
+      const double factor = 1.);
 
     template <int dim, typename number>
     void grad_div_residual (
@@ -473,7 +475,7 @@ namespace LocalIntegrators
      * @author Guido Kanschat
      * @date 2011
      */
-    template<int dim>
+    template <int dim>
     void
     u_dot_n_jump_matrix (
       FullMatrix<double> &M11,
