@@ -13,8 +13,8 @@
 //
 // ---------------------------------------------------------------------
 
-#ifndef dealii__integrators_laplace_h
-#define dealii__integrators_laplace_h
+#ifndef dealii_integrators_laplace_h
+#define dealii_integrators_laplace_h
 
 
 #include <deal.II/base/config.h>
@@ -48,7 +48,7 @@ namespace LocalIntegrators
      * @author Guido Kanschat
      * @date 2008, 2009, 2010
      */
-    template<int dim>
+    template <int dim>
     void cell_matrix (
       FullMatrix<double> &M,
       const FEValuesBase<dim> &fe,
@@ -527,7 +527,7 @@ namespace LocalIntegrators
      * @author Guido Kanschat
      * @date 2012
      */
-    template<int dim>
+    template <int dim>
     void
     ip_residual(
       Vector<double> &result1,
@@ -594,7 +594,7 @@ namespace LocalIntegrators
      * @author Guido Kanschat
      * @date 2012
      */
-    template<int dim>
+    template <int dim>
     void
     ip_residual(
       Vector<double> &result1,
