@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 2010 - 2015 by the deal.II authors
+// Copyright (C) 2010 - 2017 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -13,8 +13,8 @@
 //
 // ---------------------------------------------------------------------
 
-#ifndef dealii__integrators_patches_h
-#define dealii__integrators_patches_h
+#ifndef dealii_integrators_patches_h
+#define dealii_integrators_patches_h
 
 
 #include <deal.II/base/config.h>
