@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 2010 - 2016 by the deal.II authors
+// Copyright (C) 2010 - 2017 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -13,8 +13,8 @@
 //
 // ---------------------------------------------------------------------
 
-#ifndef dealii__arpack_solver_h
-#define dealii__arpack_solver_h
+#ifndef dealii_arpack_solver_h
+#define dealii_arpack_solver_h
 
 #include <deal.II/base/config.h>
 #include <deal.II/base/smartpointer.h>
@@ -160,18 +160,36 @@ public:
   };
 
   /**
-   * Standardized data struct to pipe additional data to the solver, should it
-   * be needed.
+   * Standardized data struct to pipe additional data to the solver.
    */
   struct AdditionalData
   {
-    const unsigned int number_of_arnoldi_vectors;
-    const WhichEigenvalues eigenvalue_of_interest;
-    const bool symmetric;
-    AdditionalData(
+    /**
+     * Constructor. By default, set the number of Arnoldi vectors (Lanczos
+     * vectors if the problem is symmetric) to 15. Set the solver to find the
+     * eigenvalues of largest magnitude for a non-symmetric problem).
+     */
+    explicit AdditionalData(
       const unsigned int number_of_arnoldi_vectors = 15,
       const WhichEigenvalues eigenvalue_of_interest = largest_magnitude,
       const bool symmetric = false);
+
+    /**
+     * Number of Arnoldi/Lanczos vectors. This number should be less than the
+     * size of the problem but greater than 2 times the number of eigenvalues
+     * (or n_eigenvalues if it is set) plus one.
+     */
+    const unsigned int number_of_arnoldi_vectors;
+
+    /**
+     * Specify the eigenvalues of interest.
+     */
+    const WhichEigenvalues eigenvalue_of_interest;
+
+    /**
+     * Specify if the problem is symmetric or not.
+     */
+    const bool symmetric;
   };
 
   /**
@@ -455,26 +473,20 @@ void ArpackSolver::solve (const MatrixType1                  &/*system_matrix*/,
           ArpackExcSmallNumberofArnoldiVectors(
             additional_data.number_of_arnoldi_vectors, nev));
 
-  /* ARPACK mode for dsaupd/dnaupd, here only mode 3,
-   * i.e. shift-invert mode
-   */
+  // ARPACK mode for dsaupd/dnaupd, here only mode 3, i.e. shift-invert mode
   int mode = 3;
 
   // reverse communication parameter
   int ido = 0;
 
-  /**
-   * 'G' generalized eigenvalue problem 'I' standard eigenvalue problem
-   */
+  // 'G' generalized eigenvalue problem 'I' standard eigenvalue problem
   char bmat[2] = "G";
 
-  /**
-   * Specify the eigenvalues of interest, possible parameters "LA"
-   * algebraically largest "SA" algebraically smallest "LM" largest magnitude
-   * "SM" smallest magnitude "LR" largest real part "SR" smallest real part
-   * "LI" largest imaginary part "SI" smallest imaginary part "BE" both ends
-   * of spectrum simultaneous
-   */
+  // Specify the eigenvalues of interest, possible parameters "LA" algebraically
+  // largest "SA" algebraically smallest "LM" largest magnitude "SM" smallest
+  // magnitude "LR" largest real part "SR" smallest real part "LI" largest
+  // imaginary part "SI" smallest imaginary part "BE" both ends of spectrum
+  // simultaneous.
   char which[3];
   switch (additional_data.eigenvalue_of_interest)
     {
@@ -529,10 +541,8 @@ void ArpackSolver::solve (const MatrixType1                  &/*system_matrix*/,
   // maximum number of iterations
   iparam[2] = control().max_steps();
 
-  /**
-   * Set the mode of dsaupd. 1 is exact shifting, 2 is user-supplied shifts,
-   * 3 is shift-invert mode, 4 is buckling mode, 5 is Cayley mode.
-   */
+  // Set the mode of dsaupd. 1 is exact shifting, 2 is user-supplied shifts,
+  // 3 is shift-invert mode, 4 is buckling mode, 5 is Cayley mode.
 
   iparam[6] = mode;
   std::vector<int> ipntr (14, 0);
@@ -550,12 +560,12 @@ void ArpackSolver::solve (const MatrixType1                  &/*system_matrix*/,
       // call of ARPACK dsaupd/dnaupd routine
       if (additional_data.symmetric)
         dsaupd_(&ido, bmat, &n, which, &nev, &tol,
-                &resid[0], &ncv, &v[0], &ldv, &iparam[0], &ipntr[0],
-                &workd[0], &workl[0], &lworkl, &info);
+                resid.data(), &ncv, v.data(), &ldv, iparam.data(), ipntr.data(),
+                workd.data(), workl.data(), &lworkl, &info);
       else
         dnaupd_(&ido, bmat, &n, which, &nev, &tol,
-                &resid[0], &ncv, &v[0], &ldv, &iparam[0], &ipntr[0],
-                &workd[0], &workl[0], &lworkl, &info);
+                resid.data(), &ncv, v.data(), &ldv, iparam.data(), ipntr.data(),
+                workd.data(), workl.data(), &lworkl, &info);
 
       if (ido == 99)
         break;
@@ -652,9 +662,7 @@ void ArpackSolver::solve (const MatrixType1                  &/*system_matrix*/,
     }
   else
     {
-      /**
-       * 1 - compute eigenvectors, 0 - only eigenvalues
-       */
+      // 1 - compute eigenvectors, 0 - only eigenvalues
       int rvec = 1;
 
       // which eigenvectors
@@ -671,19 +679,19 @@ void ArpackSolver::solve (const MatrixType1                  &/*system_matrix*/,
       if (additional_data.symmetric)
         {
           std::vector<double> z (ldz*nev, 0.);
-          dseupd_(&rvec, &howmany, &select[0], &eigenvalues_real[0],
-                  &z[0], &ldz, &sigmar, bmat, &n, which, &nev, &tol,
-                  &resid[0], &ncv, &v[0], &ldv,
-                  &iparam[0], &ipntr[0], &workd[0], &workl[0], &lworkl, &info);
+          dseupd_(&rvec, &howmany, select.data(), eigenvalues_real.data(),
+                  z.data(), &ldz, &sigmar, bmat, &n, which, &nev, &tol,
+                  resid.data(), &ncv, v.data(), &ldv,
+                  iparam.data(), ipntr.data(), workd.data(), workl.data(), &lworkl, &info);
         }
       else
         {
           std::vector<double> workev (3*ncv, 0.);
-          dneupd_(&rvec, &howmany, &select[0], &eigenvalues_real[0],
-                  &eigenvalues_im[0], &v[0], &ldz, &sigmar, &sigmai,
-                  &workev[0], bmat, &n, which, &nev, &tol,
-                  &resid[0], &ncv, &v[0], &ldv,
-                  &iparam[0], &ipntr[0], &workd[0], &workl[0], &lworkl, &info);
+          dneupd_(&rvec, &howmany, select.data(), eigenvalues_real.data(),
+                  eigenvalues_im.data(), v.data(), &ldz, &sigmar, &sigmai,
+                  workev.data(), bmat, &n, which, &nev, &tol,
+                  resid.data(), &ncv, v.data(), &ldv,
+                  iparam.data(), ipntr.data(), workd.data(), workl.data(), &lworkl, &info);
         }
 
       if (info == 1)
