@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 2000 - 2016 by the deal.II authors
+// Copyright (C) 2000 - 2018 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -13,8 +13,8 @@
 //
 // ---------------------------------------------------------------------
 
-#ifndef dealii__block_indices_h
-#define dealii__block_indices_h
+#ifndef dealii_block_indices_h
+#define dealii_block_indices_h
 
 
 #include <deal.II/base/config.h>
@@ -51,7 +51,8 @@ DEAL_II_NAMESPACE_OPEN
  * ones.
  *
  * @ingroup data
- * @see @ref GlossBlockLA "Block (linear algebra)"
+ * @see
+ * @ref GlossBlockLA "Block (linear algebra)"
  * @author Wolfgang Bangerth, Guido Kanschat, 2000, 2007, 2011
  */
 class BlockIndices : public Subscriptor
@@ -74,21 +75,16 @@ public:
    */
   BlockIndices (const std::vector<size_type> &block_sizes);
 
-#ifdef DEAL_II_WITH_CXX11
   /**
    * Move constructor. Initialize a new object by stealing the internal data of
    * another BlockIndices object.
-   *
-   * @note This constructor is only available if deal.II is configured with
-   * C++11 support.
    */
-  BlockIndices (BlockIndices &&b);
+  BlockIndices (BlockIndices &&b) noexcept;
 
   /**
    * Copy constructor.
    */
   BlockIndices (const BlockIndices &) = default;
-#endif
 
   /**
    * Specialized constructor for a structure with blocks of equal size.
@@ -181,13 +177,11 @@ public:
    */
   BlockIndices &operator = (const BlockIndices &b);
 
-#ifdef DEAL_II_WITH_CXX11
   /**
    * Move assignment operator. Move another BlockIndices object onto the
    * current one by transferring its contents.
    */
-  BlockIndices &operator = (BlockIndices &&);
-#endif
+  BlockIndices &operator = (BlockIndices &&) noexcept;
 
   /**
    * Compare whether two objects are the same, i.e. whether the number of
@@ -310,20 +304,16 @@ BlockIndices::BlockIndices (const std::vector<size_type> &block_sizes)
 
 
 
-#ifdef DEAL_II_WITH_CXX11
-
 inline
-BlockIndices::BlockIndices (BlockIndices &&b)
-  :
-  n_blocks(b.n_blocks),
-  start_indices(std::move(b.start_indices))
+BlockIndices::BlockIndices (BlockIndices &&b) noexcept
+:
+n_blocks(b.n_blocks),
+         start_indices(std::move(b.start_indices))
 {
   b.n_blocks = 0;
   b.start_indices = std::vector<size_type>(1, 0);
 }
 
-#endif
-
 
 
 inline
@@ -432,10 +422,9 @@ BlockIndices::operator = (const BlockIndices &b)
 
 
 
-#ifdef DEAL_II_WITH_CXX11
 inline
 BlockIndices &
-BlockIndices::operator = (BlockIndices &&b)
+BlockIndices::operator = (BlockIndices &&b) noexcept
 {
   start_indices = std::move(b.start_indices);
   n_blocks = b.n_blocks;
@@ -445,7 +434,6 @@ BlockIndices::operator = (BlockIndices &&b)
 
   return *this;
 }
-#endif
 
 
 
@@ -493,7 +481,7 @@ BlockIndices::memory_consumption () const
  * C++ standard library which uses a temporary object. The function simply
  * exchanges the data of the two objects.
  *
- * @relates BlockIndices
+ * @relatesalso BlockIndices
  * @author Wolfgang Bangerth, 2000
  */
 inline
