@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 2010 - 2017 by the deal.II authors
+// Copyright (C) 2010 - 2018 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -13,15 +13,14 @@
 //
 // ---------------------------------------------------------------------
 
-#ifndef dealii__block_linear_operator_h
-#define dealii__block_linear_operator_h
+#ifndef dealii_block_linear_operator_h
+#define dealii_block_linear_operator_h
 
 #include <deal.II/base/config.h>
 #include <deal.II/base/exceptions.h>
 
 #include <deal.II/lac/linear_operator.h>
 
-#ifdef DEAL_II_WITH_CXX11
 
 DEAL_II_NAMESPACE_OPEN
 
@@ -29,9 +28,9 @@ DEAL_II_NAMESPACE_OPEN
 
 namespace internal
 {
-  namespace BlockLinearOperator
+  namespace BlockLinearOperatorImplementation
   {
-    template<typename PayloadBlockType = internal::LinearOperator::EmptyPayload>
+    template <typename PayloadBlockType = internal::LinearOperatorImplementation::EmptyPayload>
     class EmptyBlockPayload;
   }
 }
@@ -40,12 +39,12 @@ template <typename Number> class BlockVector;
 
 template <typename Range = BlockVector<double>,
           typename Domain = Range,
-          typename BlockPayload = internal::BlockLinearOperator::EmptyBlockPayload<> >
+          typename BlockPayload = internal::BlockLinearOperatorImplementation::EmptyBlockPayload<> >
 class BlockLinearOperator;
 
 template <typename Range = BlockVector<double>,
           typename Domain = Range,
-          typename BlockPayload = internal::BlockLinearOperator::EmptyBlockPayload<>,
+          typename BlockPayload = internal::BlockLinearOperatorImplementation::EmptyBlockPayload<>,
           typename BlockMatrixType>
 BlockLinearOperator<Range, Domain, BlockPayload>
 block_operator(const BlockMatrixType &matrix);
@@ -53,21 +52,21 @@ block_operator(const BlockMatrixType &matrix);
 template <size_t m, size_t n,
           typename Range = BlockVector<double>,
           typename Domain = Range,
-          typename BlockPayload = internal::BlockLinearOperator::EmptyBlockPayload<> >
+          typename BlockPayload = internal::BlockLinearOperatorImplementation::EmptyBlockPayload<> >
 BlockLinearOperator<Range, Domain, BlockPayload>
 block_operator(const std::array<std::array<LinearOperator<typename Range::BlockType, typename Domain::BlockType, typename BlockPayload::BlockType>, n>, m> &);
 
 template <size_t m,
           typename Range = BlockVector<double>,
           typename Domain = Range,
-          typename BlockPayload = internal::BlockLinearOperator::EmptyBlockPayload<> >
+          typename BlockPayload = internal::BlockLinearOperatorImplementation::EmptyBlockPayload<> >
 BlockLinearOperator<Range, Domain, BlockPayload>
 block_diagonal_operator(const std::array<LinearOperator<typename Range::BlockType, typename Domain::BlockType, typename BlockPayload::BlockType>, m> &);
 
 template <size_t m,
           typename Range = BlockVector<double>,
           typename Domain = Range,
-          typename BlockPayload = internal::BlockLinearOperator::EmptyBlockPayload<> >
+          typename BlockPayload = internal::BlockLinearOperatorImplementation::EmptyBlockPayload<> >
 BlockLinearOperator<Range, Domain, BlockPayload>
 block_diagonal_operator(const LinearOperator<typename Range::BlockType, typename Domain::BlockType, typename BlockPayload::BlockType> &op);
 
@@ -80,21 +79,21 @@ block_diagonal_operator(const LinearOperator<typename Range::BlockType, typename
 
 template <typename Range = BlockVector<double>,
           typename Domain = Range,
-          typename BlockPayload = internal::BlockLinearOperator::EmptyBlockPayload<>,
+          typename BlockPayload = internal::BlockLinearOperatorImplementation::EmptyBlockPayload<>,
           typename BlockMatrixType>
 BlockLinearOperator<Range, Domain, BlockPayload>
 block_diagonal_operator(const BlockMatrixType &block_matrix);
 
 template <typename Range = BlockVector<double>,
           typename Domain = Range,
-          typename BlockPayload = internal::BlockLinearOperator::EmptyBlockPayload<> >
+          typename BlockPayload = internal::BlockLinearOperatorImplementation::EmptyBlockPayload<> >
 LinearOperator<Domain, Range, typename BlockPayload::BlockType>
 block_forward_substitution(const BlockLinearOperator<Range, Domain, BlockPayload> &,
                            const BlockLinearOperator<Domain, Range, BlockPayload> &);
 
 template <typename Range = BlockVector<double>,
           typename Domain = Range,
-          typename BlockPayload = internal::BlockLinearOperator::EmptyBlockPayload<> >
+          typename BlockPayload = internal::BlockLinearOperatorImplementation::EmptyBlockPayload<> >
 LinearOperator<Domain, Range, typename BlockPayload::BlockType>
 block_back_substitution(const BlockLinearOperator<Range, Domain, BlockPayload> &,
                         const BlockLinearOperator<Domain, Range, BlockPayload> &);
@@ -123,21 +122,50 @@ block_back_substitution(const BlockLinearOperator<Range, Domain, BlockPayload> &
  * const auto block_op_a = block_operator(A);
  * @endcode
  *
+ * Alternatively, there are several helper functions available for creating
+ * instances from multiple independent matrices of possibly different types.
+ * Here is an example of a block diagonal matrix created from a FullMatrix and
+ * a SparseMatrixEZ:
+ *
+ * @code
+ * FullMatrix<double> top_left(2, 2);
+ * top_left(0, 0) = 2.0;
+ * top_left(0, 1) = -1.0;
+ * top_left(1, 0) = -1.0;
+ * top_left(1, 1) = 2.0;
+ *
+ * SparseMatrixEZ<double> bottom_right(4, 4, 4);
+ * for (std::size_t row_n = 0; row_n < 4; ++row_n)
+ *   {
+ *     bottom_right.add(row_n, row_n, 1.0);
+ *     if (row_n < 3)
+ *       bottom_right.add(row_n, row_n + 1, -1.0);
+ *   }
+ *
+ * auto top_left_op = linear_operator(top_left);
+ * auto bottom_right_op = linear_operator(bottom_right);
+ * std::array<decltype(top_left_op), 2> operators {{top_left_op, bottom_right_op}};
+ * auto block_op = block_diagonal_operator (operators);
+ *
+ * std::vector<BlockVector<double>::size_type> block_sizes {2, 4};
+ * BlockVector<double> src(block_sizes);
+ * src = 2.0;
+ * BlockVector<double> dst(block_sizes);
+ * block_op.vmult(dst, src); // now equal to 2, 2, 0, 0, 0, 2
+ * @endcode
+ *
+ *
  * A BlockLinearOperator can be sliced to a LinearOperator at any time. This
- * removes all information about the underlying block structure (beacuse above
+ * removes all information about the underlying block structure (because above
  * <code>std::function</code> objects are no longer available) - the linear
  * operator interface, however, remains intact.
  *
  * @note This class makes heavy use of <code>std::function</code> objects and
- * lambda functions. This flexibiliy comes with a run-time penalty. Only use
+ * lambda functions. This flexibility comes with a run-time penalty. Only use
  * this object to encapsulate object with medium to large individual block
  * sizes, and small block structure (as a rule of thumb, matrix blocks greater
  * than $1000\times1000$).
  *
- * @note This class is only available if deal.II was configured with C++11
- * support, i.e., if <code>DEAL_II_WITH_CXX11</code> is enabled during cmake
- * configure.
- *
  * @author Matthias Maier, 2015
  *
  * @ingroup LAOperators
@@ -189,7 +217,7 @@ public:
    * an object @p op for which the conversion function
    * <code>block_operator</code> is defined.
    */
-  template<typename Op>
+  template <typename Op>
   BlockLinearOperator(const Op &op)
   {
     *this = block_operator<Range, Domain, BlockPayload, Op>(op);
@@ -200,7 +228,7 @@ public:
    * LinearOperator. This constructor calls the corresponding block_operator()
    * specialization.
    */
-  template<size_t m, size_t n>
+  template <size_t m, size_t n>
   BlockLinearOperator(const std::array<std::array<BlockType, n>, m> &ops)
   {
     *this = block_operator<m, n, Range, Domain, BlockPayload>(ops);
@@ -211,7 +239,7 @@ public:
    * @p ops of LinearOperator. This constructor calls the corresponding
    * block_operator() specialization.
    */
-  template<size_t m>
+  template <size_t m>
   BlockLinearOperator(const std::array<BlockType, m> &ops)
   {
     *this = block_diagonal_operator<m, Range, Domain, BlockPayload>(ops);
@@ -284,7 +312,7 @@ public:
 
 namespace internal
 {
-  namespace BlockLinearOperator
+  namespace BlockLinearOperatorImplementation
   {
     // Populate the LinearOperator interfaces with the help of the
     // BlockLinearOperator functions
@@ -328,6 +356,16 @@ namespace internal
         Assert(v.n_blocks() == m, ExcDimensionMismatch(v.n_blocks(), m));
         Assert(u.n_blocks() == n, ExcDimensionMismatch(u.n_blocks(), n));
 
+        // Bug: We need a mechanism similar to
+        // "apply_with_intermediate_storage" for LinearOperator to do the
+        // matrix vector multiplication correctly. Currently, if u and v
+        // are equal, the first vmult will garble up the ith block and
+        // subsequent multiplications are wrong.
+        Assert(!PointerComparison::equal(&v, &u),
+               ExcMessage("BlockLinearOperator::vmult currently requires that "
+                          "source and destination vectors are different memory "
+                          "locations"));
+
         for (unsigned int i = 0; i < m; ++i)
           {
             op.block(i, 0).vmult(v.block(i), u.block(0));
@@ -343,6 +381,16 @@ namespace internal
         Assert(v.n_blocks() == m, ExcDimensionMismatch(v.n_blocks(), m));
         Assert(u.n_blocks() == n, ExcDimensionMismatch(u.n_blocks(), n));
 
+        // Bug: We need a mechanism similar to
+        // "apply_with_intermediate_storage" for LinearOperator to do the
+        // matrix vector multiplication correctly. Currently, if u and v
+        // are equal, the first vmult will garble up the ith block and
+        // subsequent multiplications are wrong.
+        Assert(!PointerComparison::equal(&v, &u),
+               ExcMessage("BlockLinearOperator::vmult_add currently requires that "
+                          "source and destination vectors are different memory "
+                          "locations"));
+
         for (unsigned int i = 0; i < m; ++i)
           for (unsigned int j = 0; j < n; ++j)
             op.block(i, j).vmult_add(v.block(i), u.block(j));
@@ -355,6 +403,16 @@ namespace internal
         Assert(v.n_blocks() == n, ExcDimensionMismatch(v.n_blocks(), n));
         Assert(u.n_blocks() == m, ExcDimensionMismatch(u.n_blocks(), m));
 
+        // Bug: We need a mechanism similar to
+        // "apply_with_intermediate_storage" for LinearOperator to do the
+        // matrix vector multiplication correctly. Currently, if u and v
+        // are equal, the first vmult will garble up the ith block and
+        // subsequent multiplications are wrong.
+        Assert(!PointerComparison::equal(&v, &u),
+               ExcMessage("BlockLinearOperator::Tvmult currently requires that "
+                          "source and destination vectors are different memory "
+                          "locations"));
+
         for (unsigned int i = 0; i < n; ++i)
           {
             op.block(0, i).Tvmult(v.block(i), u.block(0));
@@ -370,6 +428,16 @@ namespace internal
         Assert(v.n_blocks() == n, ExcDimensionMismatch(v.n_blocks(), n));
         Assert(u.n_blocks() == m, ExcDimensionMismatch(u.n_blocks(), m));
 
+        // Bug: We need a mechanism similar to
+        // "apply_with_intermediate_storage" for LinearOperator to do the
+        // matrix vector multiplication correctly. Currently, if u and v
+        // are equal, the first vmult will garble up the ith block and
+        // subsequent multiplications are wrong.
+        Assert(!PointerComparison::equal(&v, &u),
+               ExcMessage("BlockLinearOperator::Tvmult_add currently requires that "
+                          "source and destination vectors are different memory "
+                          "locations"));
+
         for (unsigned int i = 0; i < n; ++i)
           for (unsigned int j = 0; j < m; ++j)
             op.block(j, i).Tvmult_add(v.block(i), u.block(j));
@@ -383,15 +451,15 @@ namespace internal
      * subblocks.
      *
      * This is the Payload class typically associated with deal.II's native
-     * BlockSparseMatrix. To use Trilinos and PETSc BlockSparseMatrices it is
-     * necessary to initialize a BlockLinearOperator with their associated
-     * BlockPayload.
+     * BlockSparseMatrix. To use either TrilinosWrappers::BlockSparseMatrix or
+     * PETScWrappers::BlockSparseMatrix one must initialize a
+     * BlockLinearOperator with their associated BlockPayload.
      *
      * @author Jean-Paul Pelteret, Matthias Maier, 2016
      *
      * @ingroup LAOperators
      */
-    template<typename PayloadBlockType>
+    template <typename PayloadBlockType>
     class EmptyBlockPayload
     {
     public:
@@ -423,7 +491,7 @@ namespace internal
 //@{
 
 /**
- * @relates BlockLinearOperator
+ * @relatesalso BlockLinearOperator
  *
  * A function that encapsulates a @p block_matrix into a BlockLinearOperator.
  *
@@ -466,18 +534,18 @@ block_operator(const BlockMatrixType &block_matrix)
     return BlockType(block_matrix.block(i, j));
   };
 
-  internal::BlockLinearOperator::populate_linear_operator_functions(return_op);
+  internal::BlockLinearOperatorImplementation::populate_linear_operator_functions(return_op);
   return return_op;
 }
 
 
 
 /**
- * @relates BlockLinearOperator
+ * @relatesalso BlockLinearOperator
  *
  * A variant of above function that encapsulates a given collection @p ops of
  * LinearOperators into a block structure. Here, it is assumed that Range and
- * Domain are blockvectors, i.e., derived from
+ * Domain are block vectors, i.e., derived from
  * @ref BlockVectorBase.
  * The individual linear operators in @p ops must act on the underlying vector
  * type of the block vectors, i.e., on Domain::BlockType yielding a result in
@@ -508,7 +576,7 @@ block_operator(const std::array<std::array<LinearOperator<typename Range::BlockT
 
   typedef typename BlockLinearOperator<Range, Domain, BlockPayload>::BlockType BlockType;
 
-  BlockLinearOperator<Range, Domain, BlockPayload> return_op ((BlockPayload())); // TODO: Create block payload so that this can be initialised correctly
+  BlockLinearOperator<Range, Domain, BlockPayload> return_op ((BlockPayload())); // TODO: Create block payload so that this can be initialized correctly
 
   return_op.n_block_rows = []() -> unsigned int
   {
@@ -528,14 +596,14 @@ block_operator(const std::array<std::array<LinearOperator<typename Range::BlockT
     return ops[i][j];
   };
 
-  internal::BlockLinearOperator::populate_linear_operator_functions(return_op);
+  internal::BlockLinearOperatorImplementation::populate_linear_operator_functions(return_op);
   return return_op;
 }
 
 
 
 /**
- * @relates BlockLinearOperator
+ * @relatesalso BlockLinearOperator
  *
  * This function extracts the diagonal blocks of @p block_matrix (either a
  * block matrix type or a BlockLinearOperator) and creates a
@@ -585,14 +653,14 @@ block_diagonal_operator(const BlockMatrixType &block_matrix)
       return null_operator(BlockType(block_matrix.block(i, j)));
   };
 
-  internal::BlockLinearOperator::populate_linear_operator_functions(return_op);
+  internal::BlockLinearOperatorImplementation::populate_linear_operator_functions(return_op);
   return return_op;
 }
 
 
 
 /**
- * @relates BlockLinearOperator
+ * @relatesalso BlockLinearOperator
  *
  * A variant of above function that builds up a block diagonal linear operator
  * from an array @p ops of diagonal elements (off-diagonal blocks are assumed
@@ -644,7 +712,7 @@ block_diagonal_operator(const std::array<LinearOperator<typename Range::BlockTyp
 
 
 /**
- * @relates BlockLinearOperator
+ * @relatesalso BlockLinearOperator
  *
  * A variant of above function that only takes a single LinearOperator
  * argument @p op and creates a blockdiagonal linear operator with @p m copies
@@ -676,8 +744,8 @@ block_diagonal_operator(const LinearOperator<typename Range::BlockType, typename
 //@{
 
 /**
- * @relates LinearOperator
- * @relates BlockLinearOperator
+ * @relatesalso LinearOperator
+ * @relatesalso BlockLinearOperator
  *
  * This function implements forward substitution to invert a lower block
  * triangular matrix. As arguments, it takes a BlockLinearOperator @p
@@ -763,8 +831,8 @@ block_forward_substitution(const BlockLinearOperator<Range, Domain, BlockPayload
     if (m == 0)
       return;
 
-    static GrowingVectorMemory<typename  Range::BlockType> vector_memory;
-    typename Range::BlockType *tmp = vector_memory.alloc();
+    GrowingVectorMemory<typename Range::BlockType> vector_memory;
+    typename VectorMemory<typename Range::BlockType>::Pointer tmp (vector_memory);
 
     diagonal_inverse.block(0, 0).vmult_add(v.block(0), u.block(0));
 
@@ -778,8 +846,6 @@ block_forward_substitution(const BlockLinearOperator<Range, Domain, BlockPayload
         *tmp *= -1.;
         diagonal_inverse.block(i, i).vmult_add(v.block(i),*tmp);
       }
-
-    vector_memory.free(tmp);
   };
 
   return return_op;
@@ -788,8 +854,8 @@ block_forward_substitution(const BlockLinearOperator<Range, Domain, BlockPayload
 
 
 /**
- * @relates LinearOperator
- * @relates BlockLinearOperator
+ * @relatesalso LinearOperator
+ * @relatesalso BlockLinearOperator
  *
  * This function implements back substitution to invert an upper block
  * triangular matrix. As arguments, it takes a BlockLinearOperator @p
@@ -872,8 +938,8 @@ block_back_substitution(const BlockLinearOperator<Range, Domain, BlockPayload> &
            ExcDimensionMismatch(diagonal_inverse.n_block_cols(), m));
     Assert(v.n_blocks() == m, ExcDimensionMismatch(v.n_blocks(), m));
     Assert(u.n_blocks() == m, ExcDimensionMismatch(u.n_blocks(), m));
-    static GrowingVectorMemory<typename  Range::BlockType> vector_memory;
-    typename  Range::BlockType *tmp = vector_memory.alloc();
+    GrowingVectorMemory<typename Range::BlockType> vector_memory;
+    typename VectorMemory<typename Range::BlockType>::Pointer tmp (vector_memory);
 
     if (m == 0)
       return;
@@ -890,8 +956,6 @@ block_back_substitution(const BlockLinearOperator<Range, Domain, BlockPayload> &
         *tmp *= -1.;
         diagonal_inverse.block(i, i).vmult_add(v.block(i),*tmp);
       }
-
-    vector_memory.free(tmp);
   };
 
   return return_op;
@@ -901,5 +965,4 @@ block_back_substitution(const BlockLinearOperator<Range, Domain, BlockPayload> &
 
 DEAL_II_NAMESPACE_CLOSE
 
-#endif // DEAL_II_WITH_CXX11
 #endif
