@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 2001 - 2016 by the deal.II authors
+// Copyright (C) 2001 - 2018 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -13,8 +13,8 @@
 //
 // ---------------------------------------------------------------------
 
-#ifndef dealii__block_matrix_array_h
-#define dealii__block_matrix_array_h
+#ifndef dealii_block_matrix_array_h
+#define dealii_block_matrix_array_h
 
 #include <deal.II/base/config.h>
 #include <deal.II/base/subscriptor.h>
@@ -105,13 +105,15 @@ DEAL_II_NAMESPACE_OPEN
  * The remaining code of this sample program concerns preconditioning and is
  * described in the documentation of BlockTrianglePrecondition.
  *
+ * @deprecated This class has been superseded by BlockLinearOperator.
+ *
  * @see
  * @ref GlossBlockLA "Block (linear algebra)"
  * @author Guido Kanschat
  * @date 2000-2005, 2010
  */
 template <typename number = double, typename BlockVectorType=BlockVector<number> >
-class BlockMatrixArray : public Subscriptor
+class DEAL_II_DEPRECATED BlockMatrixArray : public Subscriptor
 {
 public:
   /**
@@ -272,7 +274,7 @@ protected:
      * Constructor initializing all data fields. A PointerMatrix object is
      * generated for <tt>matrix</tt>.
      */
-    template<typename MatrixType>
+    template <typename MatrixType>
     Entry (const MatrixType &matrix,
            size_type row,
            size_type col,
@@ -318,6 +320,15 @@ protected:
      * The matrix block itself.
      */
     PointerMatrixBase<typename BlockVectorType::BlockType > *matrix;
+
+    /**
+     * Assignment operator.
+     *
+     * @note Since the copy constructor is destructive (see its documentation)
+     * and only exists for convenience there is no reasonable way to implement
+     * this, so it is explicitly deleted.
+     */
+    Entry &operator= (const Entry &) = delete;
   };
 
   /**
@@ -334,13 +345,6 @@ private:
    * number of blocks per row.
    */
   unsigned int block_cols;
-
-  /**
-   * Assignment operator. Since the copy constructor is destructive (see its
-   * documentation) and only exists for convenience there is no reasonable way
-   * to implement this. Hence this operator is both private and unimplemented.
-   */
-  Entry &operator= (const Entry &);
 };
 
 /*@}*/
@@ -396,12 +400,14 @@ private:
  * for solving.
  * @until Error
  *
+ * @deprecated This class has been superseded by block_back_substitution and
+ * block_forward_substitution, which build on BlockLinearOperator.
  *
  * @ingroup Preconditioners
  * @author Guido Kanschat, 2001, 2005
  */
 template <typename number = double, typename BlockVectorType = BlockVector<number> >
-class BlockTrianglePrecondition
+class DEAL_II_DEPRECATED BlockTrianglePrecondition
   : private BlockMatrixArray<number,BlockVectorType>
 {
 public:
