@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 2000 - 2015 by the deal.II authors
+// Copyright (C) 2000 - 2018 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -13,8 +13,8 @@
 //
 // ---------------------------------------------------------------------
 
-#ifndef dealii__block_sparse_matrix_templates_h
-#define dealii__block_sparse_matrix_templates_h
+#ifndef dealii_block_sparse_matrix_templates_h
+#define dealii_block_sparse_matrix_templates_h
 
 
 #include <deal.II/base/config.h>
@@ -23,18 +23,14 @@
 
 DEAL_II_NAMESPACE_OPEN
 
-
-template <typename number>
-BlockSparseMatrix<number>::BlockSparseMatrix ()
-{}
-
-
-
 template <typename number>
 BlockSparseMatrix<number>::
 BlockSparseMatrix (const BlockSparsityPattern &sparsity)
 {
-  reinit (sparsity);
+  // virtual functions called in constructors and destructors never use the
+  // override in a derived class
+  // for clarity be explicit on which function is called
+  BlockSparseMatrix<number>::reinit (sparsity);
 }
 
 
@@ -44,8 +40,13 @@ BlockSparseMatrix<number>::~BlockSparseMatrix ()
 {
   // delete previous content of
   // the subobjects array
-  clear ();
-  sparsity_pattern = 0;
+  try
+    {
+      clear ();
+    }
+  catch (...)
+    {}
+  sparsity_pattern = nullptr;
 }
 
 
@@ -78,7 +79,7 @@ void
 BlockSparseMatrix<number>::clear ()
 {
   BlockMatrixBase<SparseMatrix<number> >::clear();
-  sparsity_pattern = 0;
+  sparsity_pattern = nullptr;
 }
 
 
