@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 2002 - 2015 by the deal.II authors
+// Copyright (C) 2002 - 2017 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -13,8 +13,8 @@
 //
 // ---------------------------------------------------------------------
 
-#ifndef dealii__block_sparse_matrix_ez_templates_h
-#define dealii__block_sparse_matrix_ez_templates_h
+#ifndef dealii_block_sparse_matrix_ez_templates_h
+#define dealii_block_sparse_matrix_ez_templates_h
 
 
 #include <deal.II/base/config.h>
@@ -23,13 +23,6 @@
 
 DEAL_II_NAMESPACE_OPEN
 
-
-template <typename number>
-BlockSparseMatrixEZ<number>::BlockSparseMatrixEZ ()
-{}
-
-
-
 template <typename number>
 BlockSparseMatrixEZ<number>::
 BlockSparseMatrixEZ (const unsigned int rows,
