@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 2000 - 2016 by the deal.II authors
+// Copyright (C) 2000 - 2017 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -13,8 +13,8 @@
 //
 // ---------------------------------------------------------------------
 
-#ifndef dealii__block_sparsity_pattern_h
-#define dealii__block_sparsity_pattern_h
+#ifndef dealii_block_sparsity_pattern_h
+#define dealii_block_sparsity_pattern_h
 
 
 #include <deal.II/base/config.h>
@@ -382,7 +382,7 @@ public:
    * useful if you want such objects as member variables in other classes. You
    * can make the structure usable by calling the reinit() function.
    */
-  BlockSparsityPattern ();
+  BlockSparsityPattern () = default;
 
   /**
    * Initialize the matrix with the given number of block rows and columns.
@@ -457,7 +457,7 @@ public:
  *
  * @dontinclude block_dynamic_sparsity_pattern.cc
  *
- * After the the DoFHandler <tt>dof</tt> and the ConstraintMatrix
+ * After the DoFHandler <tt>dof</tt> and the ConstraintMatrix
  * <tt>constraints</tt> have been set up with a system element, we must count
  * the degrees of freedom in each matrix block:
  *
@@ -488,7 +488,7 @@ public:
    * useful if you want such objects as member variables in other classes. You
    * can make the structure usable by calling the reinit() function.
    */
-  BlockDynamicSparsityPattern ();
+  BlockDynamicSparsityPattern () = default;
 
   /**
    * Initialize the matrix with the given number of block rows and columns.
@@ -604,7 +604,7 @@ namespace TrilinosWrappers
      * useful if you want such objects as member variables in other classes.
      * You can make the structure usable by calling the reinit() function.
      */
-    BlockSparsityPattern ();
+    BlockSparsityPattern () = default;
 
     /**
      * Initialize the matrix with the given number of block rows and columns.
@@ -633,7 +633,8 @@ namespace TrilinosWrappers
      *
      * @deprecated Use the respective method with IndexSet arguments instead.
      */
-    BlockSparsityPattern (const std::vector<Epetra_Map> &parallel_partitioning) DEAL_II_DEPRECATED;
+    DEAL_II_DEPRECATED
+    BlockSparsityPattern (const std::vector<Epetra_Map> &parallel_partitioning);
 
     /**
      * Initialize the pattern with an array of index sets that specifies both
@@ -680,7 +681,8 @@ namespace TrilinosWrappers
      *
      * @deprecated Use the respective method with IndexSet arguments instead.
      */
-    void reinit (const std::vector<Epetra_Map> &parallel_partitioning) DEAL_II_DEPRECATED;
+    DEAL_II_DEPRECATED
+    void reinit (const std::vector<Epetra_Map> &parallel_partitioning);
 
     /**
      * Resize the matrix to a square tensor product of matrices. See the
@@ -840,7 +842,7 @@ BlockSparsityPatternBase<SparsityPatternType>::add_entries (const size_type row,
     {
       const size_type col = *it;
 
-      const std::pair<size_type , size_type>
+      const std::pair<size_type, size_type>
       col_index = this->column_indices.global_to_local(col);
 
       const size_type local_index = counter_within_block[col_index.first]++;
@@ -862,7 +864,7 @@ BlockSparsityPatternBase<SparsityPatternType>::add_entries (const size_type row,
   // where we should start reading out
   // data. Now let's write the data into
   // the individual blocks!
-  const std::pair<size_type , size_type>
+  const std::pair<size_type, size_type>
   row_index = this->row_indices.global_to_local (row);
   for (size_type block_col=0; block_col<n_block_cols(); ++block_col)
     {
@@ -887,7 +889,7 @@ BlockSparsityPatternBase<SparsityPatternType>::exists (const size_type i,
   // if you get an error here, are
   // you sure you called
   // <tt>collect_sizes()</tt> before?
-  const std::pair<size_type , size_type>
+  const std::pair<size_type, size_type>
   row_index = row_indices.global_to_local (i),
   col_index = column_indices.global_to_local (j);
   return sub_objects[row_index.first][col_index.first]->exists (row_index.second,
@@ -902,7 +904,7 @@ unsigned int
 BlockSparsityPatternBase<SparsityPatternType>::
 row_length (const size_type row) const
 {
-  const std::pair<size_type , size_type>
+  const std::pair<size_type, size_type>
   row_index = row_indices.global_to_local (row);
 
   unsigned int c = 0;
@@ -940,7 +942,7 @@ BlockDynamicSparsityPattern::column_number (const size_type row,
                                             const unsigned int index) const
 {
   // .first= ith block, .second = jth row in that block
-  const std::pair<size_type ,size_type >
+  const std::pair<size_type,size_type >
   row_index = row_indices.global_to_local (row);
 
   Assert(index<row_length(row), ExcIndexRange(index, 0, row_length(row)));
