@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 1999 - 2017 by the deal.II authors
+// Copyright (C) 1999 - 2018 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -13,14 +13,15 @@
 //
 // ---------------------------------------------------------------------
 
-#ifndef dealii__block_vector_h
-#define dealii__block_vector_h
+#ifndef dealii_block_vector_h
+#define dealii_block_vector_h
 
 
 #include <deal.II/base/config.h>
 #include <deal.II/base/exceptions.h>
 #include <deal.II/lac/block_indices.h>
 #include <deal.II/lac/block_vector_base.h>
+#include <deal.II/lac/vector_operation.h>
 #include <deal.II/lac/vector_type_traits.h>
 
 #include <cstdio>
@@ -32,7 +33,10 @@ DEAL_II_NAMESPACE_OPEN
 #ifdef DEAL_II_WITH_TRILINOS
 namespace TrilinosWrappers
 {
-  class BlockVector;
+  namespace MPI
+  {
+    class BlockVector;
+  }
 }
 #endif
 
@@ -105,19 +109,12 @@ public:
   BlockVector (const BlockVector<Number> &V);
 
 
-#ifdef DEAL_II_WITH_CXX11
   /**
    * Move constructor. Creates a new vector by stealing the internal data of
    * the given argument vector.
-   *
-   * @note This constructor is only available if deal.II is configured with
-   * C++11 support.
    */
-  BlockVector (BlockVector<Number> &&/*v*/) = default;
-#endif
+  BlockVector (BlockVector<Number> &&/*v*/) noexcept = default;
 
-
-#ifndef DEAL_II_EXPLICIT_CONSTRUCTOR_BUG
   /**
    * Copy constructor taking a BlockVector of another data type. This will
    * fail if there is no conversion path from <tt>OtherNumber</tt> to
@@ -133,15 +130,13 @@ public:
   template <typename OtherNumber>
   explicit
   BlockVector (const BlockVector<OtherNumber> &v);
-#endif
-
 
 #ifdef DEAL_II_WITH_TRILINOS
   /**
    * A copy constructor taking a (parallel) Trilinos block vector and copying
    * it into the deal.II own format.
    */
-  BlockVector (const TrilinosWrappers::BlockVector &v);
+  BlockVector (const TrilinosWrappers::MPI::BlockVector &v);
 
 #endif
   /**
@@ -173,7 +168,7 @@ public:
   /**
    * Destructor. Clears memory
    */
-  ~BlockVector ();
+  ~BlockVector () = default;
 
   /**
    * Call the compress() function on all the subblocks.
@@ -201,16 +196,11 @@ public:
   BlockVector<Number> &
   operator= (const BlockVector<Number> &v);
 
-#ifdef DEAL_II_WITH_CXX11
   /**
    * Move the given vector. This operator replaces the present vector with
    * the contents of the given argument vector.
-   *
-   * @note This operator is only available if deal.II is configured with C++11
-   * support.
    */
-  BlockVector<Number> &operator= (BlockVector<Number> &&/*v*/) = default;
-#endif
+  BlockVector<Number> &operator= (BlockVector<Number> &&/*v*/) = default; // NOLINT
 
   /**
    * Copy operator for template arguments of different types. Resize the
@@ -232,7 +222,7 @@ public:
    * vector.
    */
   BlockVector<Number> &
-  operator= (const TrilinosWrappers::BlockVector &V);
+  operator= (const TrilinosWrappers::MPI::BlockVector &V);
 #endif
 
   /**
@@ -313,19 +303,12 @@ public:
    * only swaps the pointers to the data of the two vectors and therefore does
    * not need to allocate temporary storage and move data around.
    *
-   * This function is analogous to the the swap() function of all C++ standard
+   * This function is analogous to the swap() function of all C++ standard
    * containers. Also, there is a global function swap(u,v) that simply calls
    * <tt>u.swap(v)</tt>, again in analogy to standard functions.
    */
   void swap (BlockVector<Number> &v);
 
-  /**
-   * Output of vector in user-defined format.
-   *
-   * This function is deprecated.
-   */
-  void print (const char *format = 0) const DEAL_II_DEPRECATED;
-
   /**
    * Print to a stream.
    */
@@ -470,7 +453,7 @@ void BlockVector<Number>::scale (const BlockVector2 &v)
  * standard library which uses a temporary object. The function simply
  * exchanges the data of the two vectors.
  *
- * @relates BlockVector
+ * @relatesalso BlockVector
  * @author Wolfgang Bangerth, 2000
  */
 template <typename Number>
@@ -484,7 +467,7 @@ void swap (BlockVector<Number> &u,
 
 namespace internal
 {
-  namespace LinearOperator
+  namespace LinearOperatorImplementation
   {
     template <typename> class ReinitHelper;
 
@@ -492,7 +475,7 @@ namespace internal
      * A helper class used internally in linear_operator.h. Specialization for
      * BlockVector<number>.
      */
-    template<typename number>
+    template <typename number>
     class ReinitHelper<BlockVector<number> >
     {
     public:
@@ -525,7 +508,7 @@ namespace internal
  * @author Uwe Koecher, 2017
  */
 template <typename Number>
-struct is_serial_vector< BlockVector< Number > > : std_cxx11::true_type
+struct is_serial_vector< BlockVector< Number > > : std::true_type
 {
 };
 
