@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 2004 - 2017 by the deal.II authors
+// Copyright (C) 2004 - 2018 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -13,17 +13,20 @@
 //
 // ---------------------------------------------------------------------
 
-#ifndef dealii__block_vector_base_h
-#define dealii__block_vector_base_h
+#ifndef dealii_block_vector_base_h
+#define dealii_block_vector_base_h
 
 
 #include <deal.II/base/config.h>
+#include <deal.II/base/numbers.h>
 #include <deal.II/base/exceptions.h>
 #include <deal.II/base/subscriptor.h>
 #include <deal.II/base/memory_consumption.h>
 #include <deal.II/lac/exceptions.h>
 #include <deal.II/lac/block_indices.h>
 #include <deal.II/lac/vector.h>
+#include <deal.II/lac/vector_operation.h>
+
 #include <vector>
 #include <iterator>
 #include <cmath>
@@ -88,7 +91,7 @@ public:
    * derived from BlockVectorBase<T>).
    */
   static const bool value = (sizeof(check_for_block_vector
-                                    ((VectorType *)0))
+                                    ((VectorType *)nullptr))
                              ==
                              sizeof(yes_type));
 };
@@ -211,9 +214,7 @@ namespace internal
      * @author Wolfgang Bangerth, 2001
      */
     template <class BlockVectorType, bool Constness>
-    class Iterator :
-      public std::iterator<std::random_access_iterator_tag,
-      typename Types<BlockVectorType,Constness>::value_type>
+    class Iterator
     {
     public:
       /**
@@ -235,7 +236,7 @@ namespace internal
        * by algorithms to enquire about the specifics of the iterators they
        * work on.
        */
-      typedef std::random_access_iterator_tag               iterator_type;
+      typedef std::random_access_iterator_tag               iterator_category;
       typedef std::ptrdiff_t                                difference_type;
       typedef typename BlockVectorType::reference           reference;
       typedef value_type                                   *pointer;
@@ -267,11 +268,8 @@ namespace internal
        * Copy constructor from an iterator of different constness.
        *
        * @note Constructing a non-const iterator from a const iterator does
-       * not make sense. If deal.II was configured with C++11 support, then
-       * attempting this will result in a compile-time error (via
-       * <code>static_assert</code>). If deal.II was not configured with C++11
-       * support, then attempting this will result in a thrown exception in
-       * debug mode.
+       * not make sense. Attempting this will result in a compile-time error
+       * (via <code>static_assert</code>).
        */
       Iterator (const Iterator<BlockVectorType,!Constness> &c);
 
@@ -423,16 +421,6 @@ namespace internal
                         "different block vectors. There is no reasonable way "
                         "to do this.");
 
-      /**
-       * Exception thrown when one attempts to copy construct a non-const
-       * iterator from a const iterator.
-       *
-       * @note when deal.II is compiled with C++11 support this check is
-       * instead performed at compile time via <code>static_assert</code>.
-       */
-      DeclExceptionMsg (ExcCastingAwayConstness,
-                        "Constructing a non-const iterator from a const "
-                        "iterator does not make sense.");
       //@}
     private:
       /**
@@ -487,9 +475,9 @@ namespace internal
 /**
  * A vector composed of several blocks each representing a vector of its own.
  *
- * The BlockVector is a collection of Vectors (e.g. of either deal.II Vector
- * objects or PETScWrappers::Vector object). Each of the vectors inside can
- * have a different size.
+ * The BlockVector is a collection of vectors of a given type (e.g., deal.II
+ * Vector objects, PETScWrappers::MPI::Vector objects, etc.). Each of the
+ * vectors inside can have a different size.
  *
  * The functionality of BlockVector includes everything a Vector can do, plus
  * the access to a single Vector inside the BlockVector by <tt>block(i)</tt>.
@@ -561,41 +549,22 @@ public:
    */
   typedef typename BlockType::real_type real_type;
 
-  /**
-   * A variable that indicates whether this vector supports distributed data
-   * storage. If true, then this vector also needs an appropriate compress()
-   * function that allows communicating recent set or add operations to
-   * individual elements to be communicated to other processors.
-   *
-   * For the current class, the variable equals the value declared for the
-   * type of the individual blocks.
-   *
-   * @deprecated instead of using this variable, please use the type trait value
-   * <code>is_serial_vector< VectorType >::value</code>
-   */
-  static const bool supports_distributed_data DEAL_II_DEPRECATED = !is_serial_vector<BlockType>::value;
-
   /**
    * Default constructor.
    */
-  BlockVectorBase ();
+  BlockVectorBase () = default;
 
-#ifdef DEAL_II_WITH_CXX11
   /**
    * Copy constructor.
    */
-  BlockVectorBase (const BlockVectorBase &V) = default;
+  BlockVectorBase (const BlockVectorBase &/*V*/) = default;
 
   /**
    * Move constructor. Each block of the argument vector is moved into the current
    * object if the underlying <code>VectorType</code> is move-constructible,
    * otherwise they are copied.
-   *
-   * @note This constructor is only available if deal.II is configured with
-   * C++11 support.
    */
-  BlockVectorBase (BlockVectorBase &&/*V*/) = default;
-#endif
+  BlockVectorBase (BlockVectorBase &&/*V*/) noexcept = default;
 
   /**
    * Update internal structures after resizing vectors. Whenever you reinited
@@ -713,18 +682,50 @@ public:
   reference operator[] (const size_type i);
 
   /**
-   * A collective get operation: instead of getting individual elements of a
-   * vector, this function allows to get a whole set of elements at once. The
+   * Instead of getting individual elements of a vector via operator(),
+   * this function allows getting a whole set of elements at once. The
    * indices of the elements to be read are stated in the first argument, the
    * corresponding values are returned in the second.
+   *
+   * If the current vector is called @p v, then this function is the equivalent
+   * to the code
+   * @code
+   *   for (unsigned int i=0; i<indices.size(); ++i)
+   *     values[i] = v[indices[i]];
+   * @endcode
+   *
+   * @pre The sizes of the @p indices and @p values arrays must be identical.
    */
   template <typename OtherNumber>
   void extract_subvector_to (const std::vector<size_type> &indices,
                              std::vector<OtherNumber> &values) const;
 
   /**
-   * Just as the above, but with pointers. Useful in minimizing copying of
-   * data around.
+   * Instead of getting individual elements of a vector via operator(),
+   * this function allows getting a whole set of elements at once. In
+   * contrast to the previous function, this function obtains the
+   * indices of the elements by dereferencing all elements of the iterator
+   * range provided by the first two arguments, and puts the vector
+   * values into memory locations obtained by dereferencing a range
+   * of iterators starting at the location pointed to by the third
+   * argument.
+   *
+   * If the current vector is called @p v, then this function is the equivalent
+   * to the code
+   * @code
+   *   ForwardIterator indices_p = indices_begin;
+   *   OutputIterator  values_p  = values_begin;
+   *   while (indices_p != indices_end)
+   *   {
+   *     *values_p = v[*indices_p];
+   *     ++indices_p;
+   *     ++values_p;
+   *   }
+   * @endcode
+   *
+   * @pre It must be possible to write into as many memory locations
+   *   starting at @p values_begin as there are iterators between
+   *   @p indices_begin and @p indices_end.
    */
   template <typename ForwardIterator, typename OutputIterator>
   void extract_subvector_to (ForwardIterator          indices_begin,
@@ -743,14 +744,12 @@ public:
   BlockVectorBase &
   operator= (const BlockVectorBase &V);
 
-#ifdef DEAL_II_WITH_CXX11
   /**
    * Move assignment operator. Move each block of the given argument
    * vector into the current object if `VectorType` is
    * move-constructible, otherwise copy them.
    */
-  BlockVectorBase &operator= (BlockVectorBase &&/*V*/) = default;
-#endif
+  BlockVectorBase &operator= (BlockVectorBase &&/*V*/) = default; // NOLINT
 
   /**
    * Copy operator for template arguments of different types.
@@ -779,7 +778,7 @@ public:
   value_type operator* (const BlockVectorBase &V) const;
 
   /**
-   * Return square of the $l_2$-norm.
+   * Return the square of the $l_2$-norm.
    */
   real_type norm_sqr () const;
 
@@ -822,6 +821,9 @@ public:
    * means to load the calling vector @p this twice. Since most vector
    * operations are memory transfer limited, this reduces the time by 25\% (or
    * 50\% if @p W equals @p this).
+   *
+   * For complex-valued vectors, the scalar product in the second step is implemented as
+   * $\left<v,w\right>=\sum_i v_i \bar{w_i}$.
    */
   value_type add_and_dot (const value_type       a,
                           const BlockVectorBase &V,
@@ -892,13 +894,6 @@ public:
    */
   void add (const value_type s);
 
-  /**
-   * U+=V. Simple vector addition, equal to the <tt>operator +=</tt>.
-   *
-   * This function is deprecated use the <tt>operator +=</tt> instead.
-   */
-  void add (const BlockVectorBase &V) DEAL_II_DEPRECATED;
-
   /**
    * U+=a*V. Simple addition of a scaled vector.
    */
@@ -965,15 +960,8 @@ public:
             const value_type b, const BlockVectorBase &W);
 
   /**
-   * This function does nothing but is there for compatibility with the @p
-   * PETScWrappers::Vector class.
-   *
-   * For the PETSc vector wrapper class, this function updates the ghost
-   * values of the PETSc vector. This is necessary after any modification
-   * before reading ghost values.
-   *
-   * However, for the implementation of this class, it is immaterial and thus
-   * an empty function.
+   * Update the ghost values by calling <code>update_ghost_values</code> for
+   * each block.
    */
   void update_ghost_values () const;
 
@@ -1044,15 +1032,10 @@ namespace internal
     {
       // Only permit copy-constructing const iterators from non-const
       // iterators, and not vice versa (i.e., Constness must always be
-      // true). As mentioned above, try to check this at compile time if C++11
-      // support is available.
-#ifdef DEAL_II_WITH_CXX11
+      // true).
       static_assert(Constness == true,
                     "Constructing a non-const iterator from a const iterator "
                     "does not make sense.");
-#else
-      Assert(Constness == true, ExcCastingAwayConstness());
-#endif
     }
 
 
@@ -1498,13 +1481,6 @@ namespace internal
 
 
 
-template <class VectorType>
-inline
-BlockVectorBase<VectorType>::BlockVectorBase ()
-{}
-
-
-
 template <class VectorType>
 inline
 std::size_t
@@ -1718,9 +1694,9 @@ BlockVectorBase<VectorType>::mean_value () const
   value_type sum = 0.;
   // need to do static_cast as otherwise it won't work with value_type=complex<T>
   for (size_type i=0; i<n_blocks(); ++i)
-    sum += components[i].mean_value() * static_cast<double>(components[i].size());
+    sum += components[i].mean_value() * (typename numbers::NumberTraits<value_type>::real_type(components[i].size()));
 
-  return sum/static_cast<double>(size());
+  return sum/(typename numbers::NumberTraits<value_type>::real_type(size()));
 }
 
 
@@ -1824,7 +1800,7 @@ BlockVectorBase<VectorType>::add (const std::vector<size_type> &indices,
 {
   Assert (indices.size() == values.size(),
           ExcDimensionMismatch(indices.size(), values.size()));
-  add (indices.size(), &indices[0], &values[0]);
+  add (indices.size(), indices.data(), values.data());
 }
 
 
@@ -1872,14 +1848,6 @@ void BlockVectorBase<VectorType>::add (const value_type a)
 
 
 
-template <class VectorType>
-void BlockVectorBase<VectorType>::add (const BlockVectorBase<VectorType> &v)
-{
-  *this += v;
-}
-
-
-
 template <class VectorType>
 void BlockVectorBase<VectorType>::add (const value_type a,
                                        const BlockVectorBase<VectorType> &v)
