@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 2008 - 2016 by the deal.II authors
+// Copyright (C) 2008 - 2017 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -13,8 +13,8 @@
 //
 // ---------------------------------------------------------------------
 
-#ifndef dealii__chunk_sparse_matrix_h
-#define dealii__chunk_sparse_matrix_h
+#ifndef dealii_chunk_sparse_matrix_h
+#define dealii_chunk_sparse_matrix_h
 
 
 #include <deal.II/base/config.h>
@@ -24,10 +24,13 @@
 #include <deal.II/lac/identity_matrix.h>
 #include <deal.II/lac/exceptions.h>
 
+#include <memory>
+
+
 DEAL_II_NAMESPACE_OPEN
 
-template<typename number> class Vector;
-template<typename number> class FullMatrix;
+template <typename number> class Vector;
+template <typename number> class FullMatrix;
 
 /*! @addtogroup Matrix1
  *@{
@@ -798,12 +801,6 @@ public:
    */
   number diag_element (const size_type i) const;
 
-  /**
-   * Same as above, but return a writeable reference. You're sure you know
-   * what you do?
-   */
-  number &diag_element (const size_type i);
-
   /**
    * Extracts a copy of the values and indices in the given matrix row.
    *
@@ -1314,7 +1311,10 @@ public:
   /**
    * Exception
    */
-  DeclException0 (ExcSourceEqualsDestination);
+  DeclExceptionMsg (ExcSourceEqualsDestination,
+                    "You are attempting an operation on two matrices that "
+                    "are the same object, but the operation requires that the "
+                    "two objects are in fact different.");
   //@}
 private:
   /**
@@ -1325,12 +1325,13 @@ private:
   SmartPointer<const ChunkSparsityPattern,ChunkSparseMatrix<number> > cols;
 
   /**
-   * Array of values for all the nonzero entries. The position within the
-   * matrix, i.e.  the row and column number for a given entry can only be
-   * deduced using the sparsity pattern. The same holds for the more common
-   * operation of finding an entry by its coordinates.
+   * Array of values for all the nonzero entries. The position of an
+   * entry within the matrix, i.e., the row and column number for a
+   * given value in this array can only be deduced using the sparsity
+   * pattern. The same holds for the more common operation of finding
+   * an entry by its coordinates.
    */
-  number *val;
+  std::unique_ptr<number[]> val;
 
   /**
    * Allocated size of #val. This can be larger than the actually used part if
@@ -1368,7 +1369,7 @@ inline
 typename ChunkSparseMatrix<number>::size_type
 ChunkSparseMatrix<number>::m () const
 {
-  Assert (cols != 0, ExcNotInitialized());
+  Assert (cols != nullptr, ExcNotInitialized());
   return cols->rows;
 }
 
@@ -1378,7 +1379,7 @@ inline
 typename ChunkSparseMatrix<number>::size_type
 ChunkSparseMatrix<number>::n () const
 {
-  Assert (cols != 0, ExcNotInitialized());
+  Assert (cols != nullptr, ExcNotInitialized());
   return cols->cols;
 }
 
@@ -1389,7 +1390,7 @@ inline
 const ChunkSparsityPattern &
 ChunkSparseMatrix<number>::get_sparsity_pattern () const
 {
-  Assert (cols != 0, ExcNotInitialized());
+  Assert (cols != nullptr, ExcNotInitialized());
   return *cols;
 }
 
@@ -1427,7 +1428,7 @@ void ChunkSparseMatrix<number>::set (const size_type i,
 
   AssertIsFinite(value);
 
-  Assert (cols != 0, ExcNotInitialized());
+  Assert (cols != nullptr, ExcNotInitialized());
   // it is allowed to set elements of the matrix that are not part of the
   // sparsity pattern, if the value to which we set it is zero
   const size_type index = compute_location(i,j);
@@ -1450,7 +1451,7 @@ void ChunkSparseMatrix<number>::add (const size_type i,
 
   AssertIsFinite(value);
 
-  Assert (cols != 0, ExcNotInitialized());
+  Assert (cols != nullptr, ExcNotInitialized());
 
   if (value != 0.)
     {
@@ -1486,8 +1487,8 @@ inline
 ChunkSparseMatrix<number> &
 ChunkSparseMatrix<number>::operator *= (const number factor)
 {
-  Assert (cols != 0, ExcNotInitialized());
-  Assert (val != 0, ExcNotInitialized());
+  Assert (cols != nullptr, ExcNotInitialized());
+  Assert (val != nullptr, ExcNotInitialized());
 
   const size_type chunk_size = cols->get_chunk_size();
 
@@ -1495,8 +1496,8 @@ ChunkSparseMatrix<number>::operator *= (const number factor)
   // the padding elements in chunks that overlap the boundaries of the actual
   // matrix -- but since multiplication with a number does not violate the
   // invariant of keeping these elements at zero nothing can happen
-  number             *val_ptr    = val;
-  const number *const end_ptr    = val +
+  number             *val_ptr    = val.get();
+  const number *const end_ptr    = val.get() +
                                    cols->sparsity_pattern.n_nonzero_elements()
                                    *
                                    chunk_size * chunk_size;
@@ -1513,8 +1514,8 @@ inline
 ChunkSparseMatrix<number> &
 ChunkSparseMatrix<number>::operator /= (const number factor)
 {
-  Assert (cols != 0, ExcNotInitialized());
-  Assert (val != 0, ExcNotInitialized());
+  Assert (cols != nullptr, ExcNotInitialized());
+  Assert (val != nullptr, ExcNotInitialized());
   Assert (factor !=0, ExcDivideByZero());
 
   const number factor_inv = 1. / factor;
@@ -1525,8 +1526,8 @@ ChunkSparseMatrix<number>::operator /= (const number factor)
   // the padding elements in chunks that overlap the boundaries of the actual
   // matrix -- but since multiplication with a number does not violate the
   // invariant of keeping these elements at zero nothing can happen
-  number             *val_ptr    = val;
-  const number *const end_ptr    = val +
+  number             *val_ptr    = val.get();
+  const number *const end_ptr    = val.get() +
                                    cols->sparsity_pattern.n_nonzero_elements()
                                    *
                                    chunk_size * chunk_size;
@@ -1544,7 +1545,7 @@ inline
 number ChunkSparseMatrix<number>::operator () (const size_type i,
                                                const size_type j) const
 {
-  Assert (cols != 0, ExcNotInitialized());
+  Assert (cols != nullptr, ExcNotInitialized());
   AssertThrow (compute_location(i,j) != SparsityPattern::invalid_entry,
                ExcInvalidIndex(i,j));
   return val[compute_location(i,j)];
@@ -1557,7 +1558,7 @@ inline
 number ChunkSparseMatrix<number>::el (const size_type i,
                                       const size_type j) const
 {
-  Assert (cols != 0, ExcNotInitialized());
+  Assert (cols != nullptr, ExcNotInitialized());
   const size_type index = compute_location(i,j);
 
   if (index != ChunkSparsityPattern::invalid_entry)
@@ -1572,7 +1573,7 @@ template <typename number>
 inline
 number ChunkSparseMatrix<number>::diag_element (const size_type i) const
 {
-  Assert (cols != 0, ExcNotInitialized());
+  Assert (cols != nullptr, ExcNotInitialized());
   Assert (m() == n(),  ExcNotQuadratic());
   AssertIndexRange(i, m());
 
