@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 2008 - 2016 by the deal.II authors
+// Copyright (C) 2008 - 2018 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -13,8 +13,8 @@
 //
 // ---------------------------------------------------------------------
 
-#ifndef dealii__chunk_sparse_matrix_templates_h
-#define dealii__chunk_sparse_matrix_templates_h
+#ifndef dealii_chunk_sparse_matrix_templates_h
+#define dealii_chunk_sparse_matrix_templates_h
 
 
 #include <deal.II/base/template_constraints.h>
@@ -44,7 +44,7 @@ namespace internal
 // the vectorization features of modern processors. to make this happen,
 // we will have to vectorize the functions in the following namespace, either
 // by hand or by using, for example, optimized BLAS versions for them.
-  namespace ChunkSparseMatrix
+  namespace ChunkSparseMatrixImplementation
   {
     /**
      * Declare type for container size.
@@ -213,7 +213,7 @@ namespace internal
       const size_type irregular_col = n/chunk_size;
 
       typename OutVector::iterator dst_ptr = dst.begin()+chunk_size*begin_row;
-      const number *val_ptr= &values[rowstart[begin_row]*chunk_size*chunk_size];
+      const number    *val_ptr    = &values[rowstart[begin_row]*chunk_size*chunk_size];
       const size_type *colnum_ptr = &colnums[rowstart[begin_row]];
       for (unsigned int chunk_row=begin_row; chunk_row<last_regular_row;
            ++chunk_row)
@@ -271,9 +271,9 @@ namespace internal
               val_ptr += chunk_size * chunk_size;
             }
         }
-      Assert(std::size_t(colnum_ptr-&colnums[0]) == rowstart[end_row],
+      Assert(std::size_t(colnum_ptr-colnums) == rowstart[end_row],
              ExcInternalError());
-      Assert(std::size_t(val_ptr-&values[0]) ==
+      Assert(std::size_t(val_ptr - values) ==
              rowstart[end_row] * chunk_size * chunk_size,
              ExcInternalError());
     }
@@ -285,8 +285,8 @@ namespace internal
 template <typename number>
 ChunkSparseMatrix<number>::ChunkSparseMatrix ()
   :
-  cols(0, "ChunkSparseMatrix"),
-  val(0),
+  cols(nullptr, "ChunkSparseMatrix"),
+  val(nullptr),
   max_len(0)
 {}
 
@@ -296,11 +296,11 @@ template <typename number>
 ChunkSparseMatrix<number>::ChunkSparseMatrix (const ChunkSparseMatrix &m)
   :
   Subscriptor (m),
-  cols(0, "ChunkSparseMatrix"),
-  val(0),
+  cols(nullptr, "ChunkSparseMatrix"),
+  val(nullptr),
   max_len(0)
 {
-  Assert (m.cols==0 && m.val==0 && m.max_len==0,
+  Assert (m.cols==nullptr && m.val==nullptr && m.max_len==0,
           ExcMessage("This constructor can only be called if the provided argument "
                      "is an empty matrix. This constructor can not be used to "
                      "copy-construct a non-empty matrix. Use the "
@@ -314,7 +314,7 @@ ChunkSparseMatrix<number> &
 ChunkSparseMatrix<number>::operator = (const ChunkSparseMatrix<number> &m)
 {
   (void)m;
-  Assert (m.cols==0 && m.val==0 && m.max_len==0,
+  Assert (m.cols==nullptr && m.val==nullptr && m.max_len==0,
           ExcMessage("This operator can only be called if the provided right "
                      "hand side is an empty matrix. This operator can not be "
                      "used to copy a non-empty matrix. Use the "
@@ -328,11 +328,14 @@ ChunkSparseMatrix<number>::operator = (const ChunkSparseMatrix<number> &m)
 template <typename number>
 ChunkSparseMatrix<number>::ChunkSparseMatrix (const ChunkSparsityPattern &c)
   :
-  cols(0, "ChunkSparseMatrix"),
-  val(0),
+  cols(nullptr, "ChunkSparseMatrix"),
+  val(nullptr),
   max_len(0)
 {
-  reinit (c);
+  // virtual functions called in constructors and destructors never use the
+  // override in a derived class
+  // for clarity be explicit on which function is called
+  ChunkSparseMatrix<number>::reinit (c);
 }
 
 
@@ -341,15 +344,18 @@ template <typename number>
 ChunkSparseMatrix<number>::ChunkSparseMatrix (const ChunkSparsityPattern &c,
                                               const IdentityMatrix  &id)
   :
-  cols(0, "ChunkSparseMatrix"),
-  val(0),
+  cols(nullptr, "ChunkSparseMatrix"),
+  val(nullptr),
   max_len(0)
 {
   (void)id;
   Assert (c.n_rows() == id.m(), ExcDimensionMismatch (c.n_rows(), id.m()));
   Assert (c.n_cols() == id.n(), ExcDimensionMismatch (c.n_cols(), id.n()));
 
-  reinit (c);
+  // virtual functions called in constructors and destructors never use the
+  // override in a derived class
+  // for clarity be explicit on which function is called
+  ChunkSparseMatrix<number>::reinit (c);
   for (size_type i=0; i<n(); ++i)
     this->set(i,i,1.);
 }
@@ -359,19 +365,16 @@ ChunkSparseMatrix<number>::ChunkSparseMatrix (const ChunkSparsityPattern &c,
 template <typename number>
 ChunkSparseMatrix<number>::~ChunkSparseMatrix ()
 {
-  cols = 0;
-
-  if (val != 0)
-    delete[] val;
+  cols = nullptr;
 }
 
 
 
 namespace internal
 {
-  namespace ChunkSparseMatrix
+  namespace ChunkSparseMatrixImplementation
   {
-    template<typename T>
+    template <typename T>
     void zero_subrange (const unsigned int begin,
                         const unsigned int end,
                         T *dst)
@@ -390,7 +393,7 @@ ChunkSparseMatrix<number>::operator = (const double d)
   (void)d;
   Assert (d==0, ExcScalarAssignmentOnlyForZeroValue());
 
-  Assert (cols != 0, ExcNotInitialized());
+  Assert (cols != nullptr, ExcNotInitialized());
   Assert (cols->sparsity_pattern.compressed || cols->empty(),
           ChunkSparsityPattern::ExcNotCompressed());
 
@@ -404,17 +407,17 @@ ChunkSparseMatrix<number>::operator = (const double d)
   const unsigned int matrix_size = cols->sparsity_pattern.n_nonzero_elements()
                                    * cols->chunk_size * cols->chunk_size;
   const unsigned int grain_size =
-    internal::SparseMatrix::minimum_parallel_grain_size *
+    internal::SparseMatrixImplementation::minimum_parallel_grain_size *
     (matrix_size+m()) / m();
   if (matrix_size>grain_size)
     parallel::apply_to_subranges (0U, matrix_size,
-                                  std_cxx11::bind(&internal::ChunkSparseMatrix::template
-                                                  zero_subrange<number>,
-                                                  std_cxx11::_1, std_cxx11::_2,
-                                                  val),
+                                  std::bind(&internal::ChunkSparseMatrixImplementation::template
+                                            zero_subrange<number>,
+                                            std::placeholders::_1, std::placeholders::_2,
+                                            val.get()),
                                   grain_size);
   else if (matrix_size > 0)
-    std::memset (&val[0], 0, matrix_size*sizeof(number));
+    std::memset (val.get(), 0, matrix_size*sizeof(number));
 
   return *this;
 }
@@ -448,9 +451,7 @@ ChunkSparseMatrix<number>::reinit (const ChunkSparsityPattern &sparsity)
 
   if (cols->empty())
     {
-      if (val != 0)
-        delete[] val;
-      val = 0;
+      val.reset ();
       max_len = 0;
       return;
     }
@@ -462,9 +463,7 @@ ChunkSparseMatrix<number>::reinit (const ChunkSparsityPattern &sparsity)
                       chunk_size * chunk_size;
   if (N > max_len || max_len == 0)
     {
-      if (val != 0)
-        delete[] val;
-      val = new number[N];
+      val = std_cxx14::make_unique<number[]>(N);
       max_len = N;
     }
 
@@ -480,9 +479,8 @@ template <typename number>
 void
 ChunkSparseMatrix<number>::clear ()
 {
-  cols = 0;
-  if (val) delete[] val;
-  val = 0;
+  cols = nullptr;
+  val.reset ();
   max_len = 0;
 }
 
@@ -492,7 +490,7 @@ template <typename number>
 bool
 ChunkSparseMatrix<number>::empty () const
 {
-  if (cols == 0)
+  if (cols == nullptr)
     return true;
   else
     return cols->empty();
@@ -504,7 +502,7 @@ template <typename number>
 typename ChunkSparseMatrix<number>::size_type
 ChunkSparseMatrix<number>::n_nonzero_elements () const
 {
-  Assert (cols != 0, ExcNotInitialized());
+  Assert (cols != nullptr, ExcNotInitialized());
   return cols->n_nonzero_elements ();
 }
 
@@ -514,16 +512,16 @@ template <typename number>
 typename ChunkSparseMatrix<number>::size_type
 ChunkSparseMatrix<number>::n_actually_nonzero_elements () const
 {
-  Assert (cols != 0, ExcNotInitialized());
+  Assert (cols != nullptr, ExcNotInitialized());
 
   // count those elements that are nonzero, even if they lie in the padding
   // around the matrix. since we have the invariant that padding elements are
   // zero, nothing bad can happen here
   const size_type chunk_size = cols->get_chunk_size();
-  return std::count_if(&val[0],
-                       &val[cols->sparsity_pattern.n_nonzero_elements () *
-                            chunk_size * chunk_size],
-                       std_cxx11::bind(std::not_equal_to<double>(), std_cxx11::_1, 0));
+  return std::count_if(val.get(),
+                       val.get() + cols->sparsity_pattern.n_nonzero_elements ()
+                       * chunk_size * chunk_size,
+                       std::bind(std::not_equal_to<double>(), std::placeholders::_1, 0));
 }
 
 
@@ -532,7 +530,7 @@ template <typename number>
 void
 ChunkSparseMatrix<number>::symmetrize ()
 {
-  Assert (cols != 0, ExcNotInitialized());
+  Assert (cols != nullptr, ExcNotInitialized());
   Assert (cols->rows == cols->cols, ExcNotQuadratic());
 
   Assert (false, ExcNotImplemented());
@@ -545,16 +543,16 @@ template <typename somenumber>
 ChunkSparseMatrix<number> &
 ChunkSparseMatrix<number>::copy_from (const ChunkSparseMatrix<somenumber> &matrix)
 {
-  Assert (cols != 0, ExcNotInitialized());
-  Assert (val != 0, ExcNotInitialized());
+  Assert (cols != nullptr, ExcNotInitialized());
+  Assert (val != nullptr, ExcNotInitialized());
   Assert (cols == matrix.cols, ExcDifferentChunkSparsityPatterns());
 
   // copy everything, including padding elements
   const size_type chunk_size = cols->get_chunk_size();
-  std::copy (&matrix.val[0],
-             &matrix.val[cols->sparsity_pattern.n_nonzero_elements()
-                         * chunk_size * chunk_size],
-             &val[0]);
+  std::copy (matrix.val.get(),
+             matrix.val.get() + cols->sparsity_pattern.n_nonzero_elements()
+             * chunk_size * chunk_size,
+             val.get());
 
   return *this;
 }
@@ -584,16 +582,16 @@ void
 ChunkSparseMatrix<number>::add (const number factor,
                                 const ChunkSparseMatrix<somenumber> &matrix)
 {
-  Assert (cols != 0, ExcNotInitialized());
-  Assert (val != 0, ExcNotInitialized());
+  Assert (cols != nullptr, ExcNotInitialized());
+  Assert (val != nullptr, ExcNotInitialized());
   Assert (cols == matrix.cols, ExcDifferentChunkSparsityPatterns());
 
   // add everything, including padding elements
   const size_type     chunk_size = cols->get_chunk_size();
-  number             *val_ptr    = &val[0];
+  number             *val_ptr    = val.get();
   const somenumber   *matrix_ptr = &matrix.val[0];
-  const number *const end_ptr    = &val[cols->sparsity_pattern.n_nonzero_elements()
-                                        * chunk_size * chunk_size];
+  const number *const end_ptr    = val.get() + cols->sparsity_pattern.n_nonzero_elements()
+                                   * chunk_size * chunk_size;
 
   while (val_ptr != end_ptr)
     *val_ptr++ += factor **matrix_ptr++;
@@ -661,8 +659,8 @@ void
 ChunkSparseMatrix<number>::vmult (OutVector &dst,
                                   const InVector &src) const
 {
-  Assert (cols != 0, ExcNotInitialized());
-  Assert (val != 0, ExcNotInitialized());
+  Assert (cols != nullptr, ExcNotInitialized());
+  Assert (val != nullptr, ExcNotInitialized());
   Assert(m() == dst.size(), ExcDimensionMismatch(m(),dst.size()));
   Assert(n() == src.size(), ExcDimensionMismatch(n(),src.size()));
 
@@ -682,15 +680,15 @@ void
 ChunkSparseMatrix<number>::Tvmult (OutVector &dst,
                                    const InVector &src) const
 {
-  Assert (val != 0, ExcNotInitialized());
-  Assert (cols != 0, ExcNotInitialized());
+  Assert (val != nullptr, ExcNotInitialized());
+  Assert (cols != nullptr, ExcNotInitialized());
   Assert(n() == dst.size(), ExcDimensionMismatch(n(),dst.size()));
   Assert(m() == src.size(), ExcDimensionMismatch(m(),src.size()));
 
   Assert (!PointerComparison::equal(&src, &dst), ExcSourceEqualsDestination());
 
-  Assert (cols != 0, ExcNotInitialized());
-  Assert (val != 0, ExcNotInitialized());
+  Assert (cols != nullptr, ExcNotInitialized());
+  Assert (val != nullptr, ExcNotInitialized());
   Assert(m() == dst.size(), ExcDimensionMismatch(m(),dst.size()));
   Assert(n() == src.size(), ExcDimensionMismatch(n(),src.size()));
 
@@ -710,23 +708,23 @@ void
 ChunkSparseMatrix<number>::vmult_add (OutVector &dst,
                                       const InVector &src) const
 {
-  Assert (cols != 0, ExcNotInitialized());
-  Assert (val != 0, ExcNotInitialized());
+  Assert (cols != nullptr, ExcNotInitialized());
+  Assert (val != nullptr, ExcNotInitialized());
   Assert(m() == dst.size(), ExcDimensionMismatch(m(),dst.size()));
   Assert(n() == src.size(), ExcDimensionMismatch(n(),src.size()));
 
   Assert (!PointerComparison::equal(&src, &dst), ExcSourceEqualsDestination());
   parallel::apply_to_subranges (0U, cols->sparsity_pattern.n_rows(),
-                                std_cxx11::bind (&internal::ChunkSparseMatrix::vmult_add_on_subrange
-                                                 <number,InVector,OutVector>,
-                                                 std_cxx11::cref(*cols),
-                                                 std_cxx11::_1, std_cxx11::_2,
-                                                 val,
-                                                 cols->sparsity_pattern.rowstart,
-                                                 cols->sparsity_pattern.colnums,
-                                                 std_cxx11::cref(src),
-                                                 std_cxx11::ref(dst)),
-                                internal::SparseMatrix::minimum_parallel_grain_size/cols->chunk_size+1);
+                                std::bind (&internal::ChunkSparseMatrixImplementation::vmult_add_on_subrange
+                                           <number,InVector,OutVector>,
+                                           std::cref(*cols),
+                                           std::placeholders::_1, std::placeholders::_2,
+                                           val.get(),
+                                           cols->sparsity_pattern.rowstart.get(),
+                                           cols->sparsity_pattern.colnums.get(),
+                                           std::cref(src),
+                                           std::ref(dst)),
+                                internal::SparseMatrixImplementation::minimum_parallel_grain_size/cols->chunk_size+1);
 
 }
 
@@ -737,8 +735,8 @@ void
 ChunkSparseMatrix<number>::Tvmult_add (OutVector &dst,
                                        const InVector &src) const
 {
-  Assert (cols != 0, ExcNotInitialized());
-  Assert (val != 0, ExcNotInitialized());
+  Assert (cols != nullptr, ExcNotInitialized());
+  Assert (val != nullptr, ExcNotInitialized());
   Assert(m() == dst.size(), ExcDimensionMismatch(m(),dst.size()));
   Assert(n() == src.size(), ExcDimensionMismatch(n(),src.size()));
 
@@ -758,8 +756,8 @@ ChunkSparseMatrix<number>::Tvmult_add (OutVector &dst,
 
   // like in vmult_add, but don't keep an iterator into dst around since we're
   // not traversing it sequentially this time
-  const number    *val_ptr    = val;
-  const size_type *colnum_ptr = cols->sparsity_pattern.colnums;
+  const number    *val_ptr    = val.get();
+  const size_type *colnum_ptr = cols->sparsity_pattern.colnums.get();
 
   for (size_type chunk_row=0; chunk_row<n_regular_chunk_rows; ++chunk_row)
     {
@@ -771,7 +769,7 @@ ChunkSparseMatrix<number>::Tvmult_add (OutVector &dst,
           if ((cols_have_padding == false)
               ||
               (*colnum_ptr != cols->sparsity_pattern.n_cols()-1))
-            internal::ChunkSparseMatrix::chunk_Tvmult_add
+            internal::ChunkSparseMatrixImplementation::chunk_Tvmult_add
             (cols->chunk_size,
              val_ptr,
              src.begin() + chunk_row * cols->chunk_size,
@@ -830,8 +828,8 @@ template <typename somenumber>
 somenumber
 ChunkSparseMatrix<number>::matrix_norm_square (const Vector<somenumber> &v) const
 {
-  Assert (cols != 0, ExcNotInitialized());
-  Assert (val != 0, ExcNotInitialized());
+  Assert (cols != nullptr, ExcNotInitialized());
+  Assert (val != nullptr, ExcNotInitialized());
   Assert(m() == v.size(), ExcDimensionMismatch(m(),v.size()));
   Assert(n() == v.size(), ExcDimensionMismatch(n(),v.size()));
 
@@ -852,8 +850,8 @@ ChunkSparseMatrix<number>::matrix_norm_square (const Vector<somenumber> &v) cons
        n_chunk_rows-1 :
        n_chunk_rows);
 
-  const number    *val_ptr    = val;
-  const size_type *colnum_ptr = cols->sparsity_pattern.colnums;
+  const number    *val_ptr    = val.get();
+  const size_type *colnum_ptr = cols->sparsity_pattern.colnums.get();
   typename Vector<somenumber>::const_iterator v_ptr = v.begin();
 
   for (size_type chunk_row=0; chunk_row<n_regular_chunk_rows; ++chunk_row)
@@ -867,7 +865,7 @@ ChunkSparseMatrix<number>::matrix_norm_square (const Vector<somenumber> &v) cons
               ||
               (*colnum_ptr != cols->sparsity_pattern.n_cols()-1))
             result +=
-              internal::ChunkSparseMatrix::
+              internal::ChunkSparseMatrixImplementation::
               chunk_matrix_scalar_product<somenumber>
               (cols->chunk_size,
                val_ptr,
@@ -940,8 +938,8 @@ somenumber
 ChunkSparseMatrix<number>::matrix_scalar_product (const Vector<somenumber> &u,
                                                   const Vector<somenumber> &v) const
 {
-  Assert (cols != 0, ExcNotInitialized());
-  Assert (val != 0, ExcNotInitialized());
+  Assert (cols != nullptr, ExcNotInitialized());
+  Assert (val != nullptr, ExcNotInitialized());
   Assert(m() == u.size(), ExcDimensionMismatch(m(),u.size()));
   Assert(n() == v.size(), ExcDimensionMismatch(n(),v.size()));
 
@@ -960,8 +958,8 @@ ChunkSparseMatrix<number>::matrix_scalar_product (const Vector<somenumber> &u,
        n_chunk_rows-1 :
        n_chunk_rows);
 
-  const number    *val_ptr    = val;
-  const size_type *colnum_ptr = cols->sparsity_pattern.colnums;
+  const number    *val_ptr    = val.get();
+  const size_type *colnum_ptr = cols->sparsity_pattern.colnums.get();
   typename Vector<somenumber>::const_iterator u_ptr = u.begin();
 
   for (size_type chunk_row=0; chunk_row<n_regular_chunk_rows; ++chunk_row)
@@ -975,7 +973,7 @@ ChunkSparseMatrix<number>::matrix_scalar_product (const Vector<somenumber> &u,
               ||
               (*colnum_ptr != cols->sparsity_pattern.n_cols()-1))
             result +=
-              internal::ChunkSparseMatrix::
+              internal::ChunkSparseMatrixImplementation::
               chunk_matrix_scalar_product<somenumber>
               (cols->chunk_size,
                val_ptr,
@@ -1046,8 +1044,8 @@ template <typename number>
 typename ChunkSparseMatrix<number>::real_type
 ChunkSparseMatrix<number>::l1_norm () const
 {
-  Assert (cols != 0, ExcNotInitialized());
-  Assert (val != 0, ExcNotInitialized());
+  Assert (cols != nullptr, ExcNotInitialized());
+  Assert (val != nullptr, ExcNotInitialized());
 
   const size_type n_chunk_rows = cols->sparsity_pattern.n_rows();
 
@@ -1078,8 +1076,8 @@ template <typename number>
 typename ChunkSparseMatrix<number>::real_type
 ChunkSparseMatrix<number>::linfty_norm () const
 {
-  Assert (cols != 0, ExcNotInitialized());
-  Assert (val != 0, ExcNotInitialized());
+  Assert (cols != nullptr, ExcNotInitialized());
+  Assert (val != nullptr, ExcNotInitialized());
 
   // this function works like l1_norm(). it can be made more efficient
   // (without allocating a temporary vector) as is done in the SparseMatrix
@@ -1118,7 +1116,7 @@ ChunkSparseMatrix<number>::frobenius_norm () const
   //
   // padding elements are zero, so we can add them up as well
   real_type norm_sqr = 0;
-  for (const number *ptr = &val[0]; ptr != &val[max_len]; ++ptr)
+  for (const number *ptr = val.get(); ptr != val.get() + max_len; ++ptr)
     norm_sqr +=  numbers::NumberTraits<number>::abs_square(*ptr);
 
   return std::sqrt (norm_sqr);
@@ -1133,8 +1131,8 @@ ChunkSparseMatrix<number>::residual (Vector<somenumber>       &dst,
                                      const Vector<somenumber> &u,
                                      const Vector<somenumber> &b) const
 {
-  Assert (cols != 0, ExcNotInitialized());
-  Assert (val != 0, ExcNotInitialized());
+  Assert (cols != nullptr, ExcNotInitialized());
+  Assert (val != nullptr, ExcNotInitialized());
   Assert(m() == dst.size(), ExcDimensionMismatch(m(),dst.size()));
   Assert(m() == b.size(), ExcDimensionMismatch(m(),b.size()));
   Assert(n() == u.size(), ExcDimensionMismatch(n(),u.size()));
@@ -1162,8 +1160,8 @@ ChunkSparseMatrix<number>::residual (Vector<somenumber>       &dst,
        n_chunk_rows-1 :
        n_chunk_rows);
 
-  const number       *val_ptr    = val;
-  const size_type *colnum_ptr = cols->sparsity_pattern.colnums;
+  const number    *val_ptr    = val.get();
+  const size_type *colnum_ptr = cols->sparsity_pattern.colnums.get();
   typename Vector<somenumber>::iterator dst_ptr = dst.begin();
 
   for (size_type chunk_row=0; chunk_row<n_regular_chunk_rows; ++chunk_row)
@@ -1176,7 +1174,7 @@ ChunkSparseMatrix<number>::residual (Vector<somenumber>       &dst,
           if ((cols_have_padding == false)
               ||
               (*colnum_ptr != cols->sparsity_pattern.n_cols()-1))
-            internal::ChunkSparseMatrix::chunk_vmult_subtract
+            internal::ChunkSparseMatrixImplementation::chunk_vmult_subtract
             (cols->chunk_size,
              val_ptr,
              u.begin() + *colnum_ptr * cols->chunk_size,
@@ -1249,8 +1247,8 @@ ChunkSparseMatrix<number>::precondition_Jacobi (Vector<somenumber>       &dst,
 {
   (void)dst;
   (void)src;
-  Assert (cols != 0, ExcNotInitialized());
-  Assert (val != 0, ExcNotInitialized());
+  Assert (cols != nullptr, ExcNotInitialized());
+  Assert (val != nullptr, ExcNotInitialized());
   Assert (m() == n(), ExcMessage("This operation is only valid on square matrices."));
 
   Assert (dst.size() == n(), ExcDimensionMismatch (dst.size(), n()));
@@ -1272,8 +1270,8 @@ ChunkSparseMatrix<number>::precondition_SSOR (Vector<somenumber>       &dst,
   // CVS archives to see the original version which is much clearer...
   (void)dst;
   (void)src;
-  Assert (cols != 0, ExcNotInitialized());
-  Assert (val != 0, ExcNotInitialized());
+  Assert (cols != nullptr, ExcNotInitialized());
+  Assert (val != nullptr, ExcNotInitialized());
   Assert (m() == n(), ExcMessage("This operation is only valid on square matrices."));
 
   Assert (dst.size() == n(), ExcDimensionMismatch (dst.size(), n()));
@@ -1290,8 +1288,8 @@ ChunkSparseMatrix<number>::precondition_SOR (Vector<somenumber> &dst,
                                              const Vector<somenumber> &src,
                                              const number om) const
 {
-  Assert (cols != 0, ExcNotInitialized());
-  Assert (val != 0, ExcNotInitialized());
+  Assert (cols != nullptr, ExcNotInitialized());
+  Assert (val != nullptr, ExcNotInitialized());
   Assert (m() == n(), ExcMessage("This operation is only valid on square matrices."));
 
 
@@ -1307,8 +1305,8 @@ ChunkSparseMatrix<number>::precondition_TSOR (Vector<somenumber> &dst,
                                               const Vector<somenumber> &src,
                                               const number om) const
 {
-  Assert (cols != 0, ExcNotInitialized());
-  Assert (val != 0, ExcNotInitialized());
+  Assert (cols != nullptr, ExcNotInitialized());
+  Assert (val != nullptr, ExcNotInitialized());
   Assert (m() == n(), ExcMessage("This operation is only valid on square matrices."));
 
 
@@ -1324,8 +1322,8 @@ ChunkSparseMatrix<number>::SOR (Vector<somenumber> &dst,
                                 const number /*om*/) const
 {
   (void)dst;
-  Assert (cols != 0, ExcNotInitialized());
-  Assert (val != 0, ExcNotInitialized());
+  Assert (cols != nullptr, ExcNotInitialized());
+  Assert (val != nullptr, ExcNotInitialized());
   Assert (m() == n(), ExcMessage("This operation is only valid on square matrices."));
   Assert (m() == dst.size(), ExcDimensionMismatch(m(),dst.size()));
 
@@ -1340,8 +1338,8 @@ ChunkSparseMatrix<number>::TSOR (Vector<somenumber> &dst,
                                  const number /*om*/) const
 {
   (void)dst;
-  Assert (cols != 0, ExcNotInitialized());
-  Assert (val != 0, ExcNotInitialized());
+  Assert (cols != nullptr, ExcNotInitialized());
+  Assert (val != nullptr, ExcNotInitialized());
   Assert (m() == n(), ExcMessage("This operation is only valid on square matrices."));
   Assert (m() == dst.size(), ExcDimensionMismatch(m(),dst.size()));
 
@@ -1360,8 +1358,8 @@ ChunkSparseMatrix<number>::PSOR (Vector<somenumber> &dst,
   (void)dst;
   (void)permutation;
   (void)inverse_permutation;
-  Assert (cols != 0, ExcNotInitialized());
-  Assert (val != 0, ExcNotInitialized());
+  Assert (cols != nullptr, ExcNotInitialized());
+  Assert (val != nullptr, ExcNotInitialized());
   Assert (m() == n(), ExcMessage("This operation is only valid on square matrices."));
 
   Assert (m() == dst.size(), ExcDimensionMismatch(m(),dst.size()));
@@ -1385,8 +1383,8 @@ ChunkSparseMatrix<number>::TPSOR (Vector<somenumber> &dst,
   (void)dst;
   (void)permutation;
   (void)inverse_permutation;
-  Assert (cols != 0, ExcNotInitialized());
-  Assert (val != 0, ExcNotInitialized());
+  Assert (cols != nullptr, ExcNotInitialized());
+  Assert (val != nullptr, ExcNotInitialized());
   Assert (m() == n(), ExcMessage("This operation is only valid on square matrices."));
 
   Assert (m() == dst.size(), ExcDimensionMismatch(m(),dst.size()));
@@ -1409,8 +1407,8 @@ ChunkSparseMatrix<number>::SOR_step (Vector<somenumber> &v,
 {
   (void)v;
   (void)b;
-  Assert (cols != 0, ExcNotInitialized());
-  Assert (val != 0, ExcNotInitialized());
+  Assert (cols != nullptr, ExcNotInitialized());
+  Assert (val != nullptr, ExcNotInitialized());
   Assert (m() == n(), ExcMessage("This operation is only valid on square matrices."));
 
   Assert (m() == v.size(), ExcDimensionMismatch(m(),v.size()));
@@ -1430,8 +1428,8 @@ ChunkSparseMatrix<number>::TSOR_step (Vector<somenumber> &v,
 {
   (void)v;
   (void)b;
-  Assert (cols != 0, ExcNotInitialized());
-  Assert (val != 0, ExcNotInitialized());
+  Assert (cols != nullptr, ExcNotInitialized());
+  Assert (val != nullptr, ExcNotInitialized());
   Assert (m() == n(), ExcMessage("This operation is only valid on square matrices."));
 
   Assert (m() == v.size(), ExcDimensionMismatch(m(),v.size()));
@@ -1462,8 +1460,8 @@ ChunkSparseMatrix<number>::SSOR (Vector<somenumber> &dst,
                                  const number /*om*/) const
 {
   (void)dst;
-  Assert (cols != 0, ExcNotInitialized());
-  Assert (val != 0, ExcNotInitialized());
+  Assert (cols != nullptr, ExcNotInitialized());
+  Assert (val != nullptr, ExcNotInitialized());
   Assert (m() == n(), ExcMessage("This operation is only valid on square matrices."));
 
   Assert (m() == dst.size(), ExcDimensionMismatch(m(),dst.size()));
@@ -1478,8 +1476,8 @@ void ChunkSparseMatrix<number>::print (std::ostream &out) const
 {
   AssertThrow (out, ExcIO());
 
-  Assert (cols != 0, ExcNotInitialized());
-  Assert (val != 0, ExcNotInitialized());
+  Assert (cols != nullptr, ExcNotInitialized());
+  Assert (val != nullptr, ExcNotInitialized());
 
   Assert (false, ExcNotImplemented());
 
@@ -1497,8 +1495,8 @@ void ChunkSparseMatrix<number>::print_formatted (std::ostream &out,
 {
   AssertThrow (out, ExcIO());
 
-  Assert (cols != 0, ExcNotInitialized());
-  Assert (val != 0, ExcNotInitialized());
+  Assert (cols != nullptr, ExcNotInitialized());
+  Assert (val != nullptr, ExcNotInitialized());
 
   unsigned int width = width_;
 
@@ -1545,8 +1543,8 @@ void ChunkSparseMatrix<number>::print_pattern (std::ostream &out,
 {
   AssertThrow (out, ExcIO());
 
-  Assert (cols != 0, ExcNotInitialized());
-  Assert (val != 0, ExcNotInitialized());
+  Assert (cols != nullptr, ExcNotInitialized());
+  Assert (val != nullptr, ExcNotInitialized());
 
   const size_type chunk_size = cols->get_chunk_size();
 
@@ -1588,9 +1586,9 @@ ChunkSparseMatrix<number>::block_write (std::ostream &out) const
   // first the simple objects, bracketed in [...]
   out << '[' << max_len << "][";
   // then write out real data
-  out.write (reinterpret_cast<const char *>(&val[0]),
-             reinterpret_cast<const char *>(&val[max_len])
-             - reinterpret_cast<const char *>(&val[0]));
+  out.write (reinterpret_cast<const char *>(val.get()),
+             reinterpret_cast<const char *>(val.get() + max_len)
+             - reinterpret_cast<const char *>(val.get()));
   out << ']';
 
   AssertThrow (out, ExcIO());
@@ -1617,13 +1615,12 @@ ChunkSparseMatrix<number>::block_read (std::istream &in)
   AssertThrow (c == '[', ExcIO());
 
   // reallocate space
-  delete[] val;
-  val  = new number[max_len];
+  val = std_cxx14::make_unique<number[]>(max_len);
 
   // then read data
-  in.read (reinterpret_cast<char *>(&val[0]),
-           reinterpret_cast<char *>(&val[max_len])
-           - reinterpret_cast<char *>(&val[0]));
+  in.read (reinterpret_cast<char *>(val.get()),
+           reinterpret_cast<char *>(val.get() + max_len)
+           - reinterpret_cast<char *>(val.get()));
   in >> c;
   AssertThrow (c == ']', ExcIO());
 }
