@@ -13,11 +13,12 @@
 //
 // ---------------------------------------------------------------------
 
-#ifndef dealii__diagonal_matrix_h
-#define dealii__diagonal_matrix_h
+#ifndef dealii_diagonal_matrix_h
+#define dealii_diagonal_matrix_h
 
 
 #include <deal.II/lac/vector.h>
+#include <deal.II/lac/vector_operation.h>
 
 DEAL_II_NAMESPACE_OPEN
 
@@ -52,7 +53,7 @@ public:
   /**
    * Constructor.
    */
-  DiagonalMatrix();
+  DiagonalMatrix() = default;
 
   /**
    * Initialize with a given vector by copying the content of the vector
@@ -68,7 +69,7 @@ public:
   void compress (VectorOperation::values operation);
 
   /**
-   * Returns a reference to the underlying vector for manipulation of the
+   * Return a reference to the underlying vector for manipulation of the
    * entries on the matrix diagonal.
    */
   VectorType &get_vector();
@@ -79,7 +80,7 @@ public:
   void clear();
 
   /**
-   * Returns a read-only reference to the underlying vector.
+   * Return a read-only reference to the underlying vector.
    */
   const VectorType &get_vector() const;
 
@@ -178,6 +179,15 @@ public:
   void Tvmult_add (VectorType       &dst,
                    const VectorType &src) const;
 
+  /**
+   * Initialize vector @p dst to have the same size and partition as
+   * @p diagonal member of this class.
+   *
+   * This is a part of the interface required
+   * by linear_operator().
+   */
+  void initialize_dof_vector(VectorType &dst) const;
+
   /**
    * Return the memory consumption of this object.
    */
@@ -194,14 +204,6 @@ private:
 
 #ifndef DOXYGEN
 
-template <typename VectorType>
-DiagonalMatrix<VectorType>::DiagonalMatrix()
-  :
-  Subscriptor()
-{}
-
-
-
 template <typename VectorType>
 void
 DiagonalMatrix<VectorType>::clear()
@@ -229,6 +231,15 @@ DiagonalMatrix<VectorType>::reinit(const VectorType &vec)
 
 
 
+template <typename VectorType>
+void
+DiagonalMatrix<VectorType>::initialize_dof_vector(VectorType &dst) const
+{
+  dst.reinit(diagonal);
+}
+
+
+
 template <typename VectorType>
 void
 DiagonalMatrix<VectorType>::compress (VectorOperation::values operation)
@@ -305,7 +316,7 @@ DiagonalMatrix<VectorType>::add (const size_type  row,
                                  const size_type  n_cols,
                                  const size_type *col_indices,
                                  const number2   *values,
-                                 const bool       ,
+                                 const bool,
                                  const bool       )
 {
   for (size_type i=0; i<n_cols; ++i)
