@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 2011 - 2016 by the deal.II authors
+// Copyright (C) 2011 - 2017 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -13,8 +13,8 @@
 //
 // ---------------------------------------------------------------------
 
-#ifndef dealii__dynamic_sparsity_pattern_h
-#define dealii__dynamic_sparsity_pattern_h
+#ifndef dealii_dynamic_sparsity_pattern_h
+#define dealii_dynamic_sparsity_pattern_h
 
 
 #include <deal.II/base/config.h>
@@ -420,6 +420,14 @@ public:
    */
   void symmetrize ();
 
+  /**
+   * Construct and store in this object the sparsity pattern corresponding to
+   * the product of @p left and @p right sparsity pattern.
+   */
+  template <typename SparsityPatternTypeLeft, typename SparsityPatternTypeRight>
+  void compute_mmult_pattern(const SparsityPatternTypeLeft &left,
+                             const SparsityPatternTypeRight &right);
+
   /**
    * Print the sparsity pattern. The output consists of one line per row of
    * the format <tt>[i,j1,j2,j3,...]</tt>. <i>i</i> is the row number and
@@ -598,11 +606,6 @@ private:
      */
     std::vector<size_type> entries;
 
-    /**
-     * Constructor.
-     */
-    Line ();
-
     /**
      * Add the given column number to this line.
      */
@@ -986,12 +989,6 @@ DynamicSparsityPattern::add_entries (const size_type row,
 
 
 
-inline
-DynamicSparsityPattern::Line::Line ()
-{}
-
-
-
 inline
 types::global_dof_index
 DynamicSparsityPattern::row_length (const size_type row) const
