@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 2001 - 2016 by the deal.II authors
+// Copyright (C) 2001 - 2018 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -13,8 +13,8 @@
 //
 // ---------------------------------------------------------------------
 
-#ifndef dealii__filtered_matrix_h
-#define dealii__filtered_matrix_h
+#ifndef dealii_filtered_matrix_h
+#define dealii_filtered_matrix_h
 
 
 
@@ -183,13 +183,16 @@ template <class VectorType> class FilteredMatrixBlock;
  * <h3>Thread-safety</h3>
  *
  * The functions that operate as a matrix and do not change the internal state
- * of this object are synchronised and thus threadsafe. Consequently, you do
+ * of this object are synchronized and thus threadsafe. Consequently, you do
  * not need to serialize calls to @p vmult or @p residual .
  *
  * @author Wolfgang Bangerth 2001, Luca Heltai 2006, Guido Kanschat 2007, 2008
+ *
+ * @deprecated Use LinearOperator instead. See the documentation of
+ * constrained_linear_operator().
  */
 template <typename VectorType>
-class FilteredMatrix : public Subscriptor
+class DEAL_II_DEPRECATED FilteredMatrix : public Subscriptor
 {
 public:
   class const_iterator;
@@ -410,8 +413,9 @@ public:
    * the second parameter to @p true to use a faster algorithm. Note: This
    * method is deprecated as matrix_is_symmetric parameter is no longer used.
    */
+  DEAL_II_DEPRECATED
   void apply_constraints (VectorType &v,
-                          const bool matrix_is_symmetric) const DEAL_II_DEPRECATED;
+                          const bool matrix_is_symmetric) const;
   /**
    * Apply the constraints to a right hand side vector. This needs to be done
    * before starting to solve with the filtered matrix.
@@ -438,7 +442,7 @@ public:
    *
    * @note The result vector of this multiplication will have the constraint
    * entries set to zero, independent of the previous value of <tt>dst</tt>.
-   * We excpect that in most cases this is the required behavior.
+   * We expect that in most cases this is the required behavior.
    */
   void vmult_add (VectorType       &dst,
                   const VectorType &src) const;
@@ -448,7 +452,7 @@ public:
    *
    * @note The result vector of this multiplication will have the constraint
    * entries set to zero, independent of the previous value of <tt>dst</tt>.
-   * We excpect that in most cases this is the required behavior.
+   * We expect that in most cases this is the required behavior.
    */
   void Tvmult_add (VectorType       &dst,
                    const VectorType &src) const;
@@ -512,7 +516,7 @@ private:
   /**
    * Pointer to the sparsity pattern used for this matrix.
    */
-  std_cxx11::shared_ptr<PointerMatrixBase<VectorType> > matrix;
+  std::shared_ptr<PointerMatrixBase<VectorType> > matrix;
 
   /**
    * Sorted list of pairs denoting the index of the variable and the value to
@@ -547,7 +551,7 @@ private:
 
 //--------------------------------Iterators--------------------------------------//
 
-template<typename VectorType>
+template <typename VectorType>
 inline
 FilteredMatrix<VectorType>::Accessor::Accessor
 (const FilteredMatrix<VectorType> *matrix,
@@ -562,7 +566,7 @@ FilteredMatrix<VectorType>::Accessor::Accessor
 
 
 
-template<typename VectorType>
+template <typename VectorType>
 inline
 types::global_dof_index
 FilteredMatrix<VectorType>::Accessor::row() const
@@ -572,7 +576,7 @@ FilteredMatrix<VectorType>::Accessor::row() const
 
 
 
-template<typename VectorType>
+template <typename VectorType>
 inline
 types::global_dof_index
 FilteredMatrix<VectorType>::Accessor::column() const
@@ -582,7 +586,7 @@ FilteredMatrix<VectorType>::Accessor::column() const
 
 
 
-template<typename VectorType>
+template <typename VectorType>
 inline
 double
 FilteredMatrix<VectorType>::Accessor::value() const
@@ -592,7 +596,7 @@ FilteredMatrix<VectorType>::Accessor::value() const
 
 
 
-template<typename VectorType>
+template <typename VectorType>
 inline
 void
 FilteredMatrix<VectorType>::Accessor::advance()
@@ -604,7 +608,7 @@ FilteredMatrix<VectorType>::Accessor::advance()
 
 
 
-template<typename VectorType>
+template <typename VectorType>
 inline
 FilteredMatrix<VectorType>::const_iterator::const_iterator
 (const FilteredMatrix<VectorType> *matrix,
@@ -615,7 +619,7 @@ FilteredMatrix<VectorType>::const_iterator::const_iterator
 
 
 
-template<typename VectorType>
+template <typename VectorType>
 inline
 typename FilteredMatrix<VectorType>::const_iterator &
 FilteredMatrix<VectorType>::const_iterator::operator++ ()
@@ -897,7 +901,7 @@ FilteredMatrix<VectorType>::vmult (VectorType &dst, const VectorType &src) const
   if (!expect_constrained_source)
     {
       GrowingVectorMemory<VectorType> mem;
-      VectorType *tmp_vector = mem.alloc();
+      typename VectorMemory<VectorType>::Pointer tmp_vector (mem);
       // first copy over src vector and
       // pre-filter
       tmp_vector->reinit(src, true);
@@ -905,7 +909,6 @@ FilteredMatrix<VectorType>::vmult (VectorType &dst, const VectorType &src) const
       pre_filter (*tmp_vector);
       // then let matrix do its work
       matrix->vmult (dst, *tmp_vector);
-      mem.free(tmp_vector);
     }
   else
     {
@@ -926,7 +929,7 @@ FilteredMatrix<VectorType>::Tvmult (VectorType &dst, const VectorType &src) cons
   if (!expect_constrained_source)
     {
       GrowingVectorMemory<VectorType> mem;
-      VectorType *tmp_vector = mem.alloc();
+      typename VectorMemory<VectorType>::Pointer tmp_vector (mem);
       // first copy over src vector and
       // pre-filter
       tmp_vector->reinit(src, true);
@@ -934,7 +937,6 @@ FilteredMatrix<VectorType>::Tvmult (VectorType &dst, const VectorType &src) cons
       pre_filter (*tmp_vector);
       // then let matrix do its work
       matrix->Tvmult (dst, *tmp_vector);
-      mem.free(tmp_vector);
     }
   else
     {
@@ -955,7 +957,7 @@ FilteredMatrix<VectorType>::vmult_add (VectorType &dst, const VectorType &src) c
   if (!expect_constrained_source)
     {
       GrowingVectorMemory<VectorType> mem;
-      VectorType *tmp_vector = mem.alloc();
+      typename VectorMemory<VectorType>::Pointer tmp_vector (mem);
       // first copy over src vector and
       // pre-filter
       tmp_vector->reinit(src, true);
@@ -963,7 +965,6 @@ FilteredMatrix<VectorType>::vmult_add (VectorType &dst, const VectorType &src) c
       pre_filter (*tmp_vector);
       // then let matrix do its work
       matrix->vmult_add (dst, *tmp_vector);
-      mem.free(tmp_vector);
     }
   else
     {
@@ -984,7 +985,7 @@ FilteredMatrix<VectorType>::Tvmult_add (VectorType &dst, const VectorType &src)
   if (!expect_constrained_source)
     {
       GrowingVectorMemory<VectorType> mem;
-      VectorType *tmp_vector = mem.alloc();
+      typename VectorMemory<VectorType>::Pointer tmp_vector (mem);
       // first copy over src vector and
       // pre-filter
       tmp_vector->reinit(src, true);
@@ -992,7 +993,6 @@ FilteredMatrix<VectorType>::Tvmult_add (VectorType &dst, const VectorType &src)
       pre_filter (*tmp_vector);
       // then let matrix do its work
       matrix->Tvmult_add (dst, *tmp_vector);
-      mem.free(tmp_vector);
     }
   else
     {
