@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 1999 - 2017 by the deal.II authors
+// Copyright (C) 1999 - 2018 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -13,8 +13,8 @@
 //
 // ---------------------------------------------------------------------
 
-#ifndef dealii__full_matrix_templates_h
-#define dealii__full_matrix_templates_h
+#ifndef dealii_full_matrix_templates_h
+#define dealii_full_matrix_templates_h
 
 
 // TODO: this file has a lot of operations between matrices and matrices or
@@ -108,7 +108,7 @@ FullMatrix<number> &
 FullMatrix<number>::operator = (const LAPACKFullMatrix<number2> &M)
 {
   Assert (this->m() == M.n_rows(), ExcDimensionMismatch(this->m(), M.n_rows()));
-  Assert (this->n() == M.n_cols(), ExcDimensionMismatch(this->n(), M.n_rows()));
+  Assert (this->n() == M.n_cols(), ExcDimensionMismatch(this->n(), M.n_cols()));
   for (size_type i=0; i<this->m(); ++i)
     for (size_type j=0; j<this->n(); ++j)
       (*this)(i,j) = M(i,j);
@@ -255,10 +255,10 @@ number FullMatrix<number>::residual (Vector<number2> &dst,
   number res = 0.;
   const size_type size_m = m(),
                   size_n = n();
-  for (size_type i=0; i<size_n; ++i)
+  for (size_type i=0; i<size_m; ++i)
     {
       number s = number(right(i));
-      for (size_type j=0; j<size_m; ++j)
+      for (size_type j=0; j<size_n; ++j)
         s -= number(src(j)) * (*this)(i,j);
       dst(i) = s;
       res += s*s;
@@ -282,10 +282,10 @@ void FullMatrix<number>::forward (Vector<number2>       &dst,
   size_type nu = ( (m()<n()) ? m() : n());
   for (i=0; i<nu; ++i)
     {
-      number s = number(src(i));
+      typename ProductType<number,number2>::type s = src(i);
       for (j=0; j<i; ++j)
-        s -= number(dst(j)) * (*this)(i,j);
-      dst(i) = s/(*this)(i,i);
+        s -= typename ProductType<number,number2>::type(dst(j)) * (*this)(i,j);
+      dst(i) = number2(s)/number2((*this)(i,i));
       AssertIsFinite(dst(i));
     }
 }
@@ -301,12 +301,12 @@ void FullMatrix<number>::backward (Vector<number2>       &dst,
 
   size_type j;
   size_type nu = (m()<n() ? m() : n());
-  for (int i=nu-1; i>=0; --i)
+  for (std::make_signed<size_type>::type i=nu-1; i>=0; --i)
     {
-      number2 s = src(i);
+      typename ProductType<number,number2>::type s = src(i);
       for (j=i+1; j<nu; ++j)
-        s -= dst(j) * number2((*this)(i,j));
-      dst(i) = s/number2((*this)(i,i));
+        s -= typename ProductType<number,number2>::type(dst(j)) * (*this)(i,j);
+      dst(i) = number2(s)/number2((*this)(i,i));
       AssertIsFinite(dst(i));
     }
 }
@@ -321,14 +321,10 @@ void FullMatrix<number>::fill (const FullMatrix<number2> &src,
                                const size_type src_offset_i,
                                const size_type src_offset_j)
 {
-  Assert (dst_offset_i < m(),
-          ExcIndexRange (dst_offset_i, 0, m()));
-  Assert (dst_offset_j < n(),
-          ExcIndexRange (dst_offset_j, 0, n()));
-  Assert (src_offset_i < src.m(),
-          ExcIndexRange (src_offset_i, 0, src.m()));
-  Assert (src_offset_j < src.n(),
-          ExcIndexRange (src_offset_j, 0, src.n()));
+  AssertIndexRange (dst_offset_i, m());
+  AssertIndexRange (dst_offset_j, n());
+  AssertIndexRange (src_offset_i, src.m());
+  AssertIndexRange (src_offset_j, src.n());
 
   // Compute maximal size of copied block
   const size_type rows = std::min (m() - dst_offset_i,
@@ -368,7 +364,7 @@ void FullMatrix<number>::add_row (const size_type i,
 {
   Assert (!this->empty(), ExcEmptyMatrix());
 
-  for (size_type k=0; k<m(); ++k)
+  for (size_type k=0; k<n(); ++k)
     (*this)(i,k) += s*(*this)(j,k);
 }
 
@@ -382,7 +378,7 @@ void FullMatrix<number>::add_row (const size_type i,
 {
   Assert (!this->empty(), ExcEmptyMatrix());
 
-  const size_type size_m = m();
+  const size_type size_m = n();
   for (size_type l=0; l<size_m; ++l)
     (*this)(i,l) += s*(*this)(j,l) + t*(*this)(k,l);
 }
@@ -394,7 +390,7 @@ void FullMatrix<number>::add_col (const size_type i, const number s,
 {
   Assert (!this->empty(), ExcEmptyMatrix());
 
-  for (size_type k=0; k<n(); ++k)
+  for (size_type k=0; k<m(); ++k)
     (*this)(k,i) += s*(*this)(k,j);
 }
 
@@ -406,7 +402,7 @@ void FullMatrix<number>::add_col (const size_type i, const number s,
 {
   Assert (!this->empty(), ExcEmptyMatrix());
 
-  for (size_t l=0; l<n(); ++l)
+  for (size_t l=0; l<m(); ++l)
     (*this)(l,i) += s*(*this)(l,j) + t*(*this)(l,k);
 }
 
@@ -527,12 +523,15 @@ void FullMatrix<number>::mmult (FullMatrix<number2>       &dst,
   // works for us (it is usually not efficient to use BLAS for very small
   // matrices):
 #ifdef DEAL_II_WITH_LAPACK
-  if ((types_are_equal<number,double>::value
+  if ((std::is_same<number,double>::value
        ||
-       types_are_equal<number,float>::value)
+       std::is_same<number,float>::value)
       &&
-      types_are_equal<number,number2>::value)
-    if (this->n()*this->m()*src.n() > 300)
+      std::is_same<number,number2>::value)
+    if (this->n()*this->m()*src.n() > 300 &&
+        src.n() <= std::numeric_limits<types::blas_int>::max() &&
+        this->m() <= std::numeric_limits<types::blas_int>::max() &&
+        this->n() <= std::numeric_limits<types::blas_int>::max())
       {
         // In case we have the BLAS function gemm detected by CMake, we
         // use that algorithm for matrix-matrix multiplication since it
@@ -546,9 +545,9 @@ void FullMatrix<number>::mmult (FullMatrix<number2>       &dst,
         // transpose matrices, and read the result as if it were row-wise
         // again. In other words, we calculate (B^T A^T)^T, which is AB.
 
-        const int m = src.n();
-        const int n = this->m();
-        const int k = this->n();
+        const types::blas_int m = static_cast<types::blas_int>(src.n());
+        const types::blas_int n = static_cast<types::blas_int>(this->m());
+        const types::blas_int k = static_cast<types::blas_int>(this->n());
         const char *notrans = "n";
 
         const number alpha = 1.;
@@ -597,12 +596,15 @@ void FullMatrix<number>::Tmmult (FullMatrix<number2>       &dst,
   // works for us (it is usually not efficient to use BLAS for very small
   // matrices):
 #ifdef DEAL_II_WITH_LAPACK
-  if ((types_are_equal<number,double>::value
+  if ((std::is_same<number,double>::value
        ||
-       types_are_equal<number,float>::value)
+       std::is_same<number,float>::value)
       &&
-      types_are_equal<number,number2>::value)
-    if (this->n()*this->m()*src.n() > 300)
+      std::is_same<number,number2>::value)
+    if (this->n()*this->m()*src.n() > 300 &&
+        src.n() <= std::numeric_limits<types::blas_int>::max() &&
+        this->n() <= std::numeric_limits<types::blas_int>::max() &&
+        this->m() <= std::numeric_limits<types::blas_int>::max())
       {
         // In case we have the BLAS function gemm detected by CMake, we
         // use that algorithm for matrix-matrix multiplication since it
@@ -616,9 +618,9 @@ void FullMatrix<number>::Tmmult (FullMatrix<number2>       &dst,
         // transpose matrices, and read the result as if it were row-wise
         // again. In other words, we calculate (B^T A)^T, which is A^T B.
 
-        const int m = src.n();
-        const int n = this->n();
-        const int k = this->m();
+        const types::blas_int m = static_cast<types::blas_int>(src.n());
+        const types::blas_int n = static_cast<types::blas_int>(this->n());
+        const types::blas_int k = static_cast<types::blas_int>(this->m());
         const char *trans = "t";
         const char *notrans = "n";
 
@@ -687,12 +689,15 @@ void FullMatrix<number>::mTmult (FullMatrix<number2>       &dst,
   // works for us (it is usually not efficient to use BLAS for very small
   // matrices):
 #ifdef DEAL_II_WITH_LAPACK
-  if ((types_are_equal<number,double>::value
+  if ((std::is_same<number,double>::value
        ||
-       types_are_equal<number,float>::value)
+       std::is_same<number,float>::value)
       &&
-      types_are_equal<number,number2>::value)
-    if (this->n()*this->m()*src.m() > 300)
+      std::is_same<number,number2>::value)
+    if (this->n()*this->m()*src.m() > 300 &&
+        src.m() <= std::numeric_limits<types::blas_int>::max() &&
+        this->n() <= std::numeric_limits<types::blas_int>::max() &&
+        this->m() <= std::numeric_limits<types::blas_int>::max())
       {
         // In case we have the BLAS function gemm detected by CMake, we
         // use that algorithm for matrix-matrix multiplication since it
@@ -706,9 +711,9 @@ void FullMatrix<number>::mTmult (FullMatrix<number2>       &dst,
         // transpose matrices, and read the result as if it were row-wise
         // again. In other words, we calculate (B A^T)^T, which is AB^T.
 
-        const int m = src.m();
-        const int n = this->m();
-        const int k = this->n();
+        const types::blas_int m = static_cast<types::blas_int>(src.m());
+        const types::blas_int n = static_cast<types::blas_int>(this->m());
+        const types::blas_int k = static_cast<types::blas_int>(this->n());
         const char *notrans = "n";
         const char *trans = "t";
 
@@ -775,12 +780,15 @@ void FullMatrix<number>::TmTmult (FullMatrix<number2>       &dst,
   // works for us (it is usually not efficient to use BLAS for very small
   // matrices):
 #ifdef DEAL_II_WITH_LAPACK
-  if ((types_are_equal<number,double>::value
+  if ((std::is_same<number,double>::value
        ||
-       types_are_equal<number,float>::value)
+       std::is_same<number,float>::value)
       &&
-      types_are_equal<number,number2>::value)
-    if (this->n()*this->m()*src.m() > 300)
+      std::is_same<number,number2>::value)
+    if (this->n()*this->m()*src.m() > 300 &&
+        src.m() <= std::numeric_limits<types::blas_int>::max() &&
+        this->n() <= std::numeric_limits<types::blas_int>::max() &&
+        this->m() <= std::numeric_limits<types::blas_int>::max())
       {
         // In case we have the BLAS function gemm detected by CMake, we
         // use that algorithm for matrix-matrix multiplication since it
@@ -794,9 +802,9 @@ void FullMatrix<number>::TmTmult (FullMatrix<number2>       &dst,
         // transpose matrices, and read the result as if it were row-wise
         // again. In other words, we calculate (B A)^T, which is A^T B^T.
 
-        const int m = src.m();
-        const int n = this->n();
-        const int k = this->m();
+        const types::blas_int m = static_cast<types::blas_int>(src.m());
+        const types::blas_int n = static_cast<types::blas_int>(this->n());
+        const types::blas_int k = static_cast<types::blas_int>(this->m());
         const char *trans = "t";
 
         const number alpha = 1.;
@@ -903,13 +911,13 @@ FullMatrix<number>::matrix_norm_square (const Vector<number2> &v) const
 
   for (size_type row=0; row<n_rows; ++row)
     {
-      number s = 0.;
+      number2 s = 0.;
       const number *const val_end_of_row = val_ptr+n_rows;
       const number2 *v_ptr = v.begin();
       while (val_ptr != val_end_of_row)
-        s += number(*val_ptr++) * number(*v_ptr++);
+        s += number2(*val_ptr++) * number2(*v_ptr++);
 
-      sum += s * number(numbers::NumberTraits<number2>::conjugate(v(row)));
+      sum += s * numbers::NumberTraits<number2>::conjugate(v(row));
     }
 
   return sum;
@@ -924,7 +932,7 @@ FullMatrix<number>::matrix_scalar_product (const Vector<number2> &u,
 {
   Assert (!this->empty(), ExcEmptyMatrix());
 
-  Assert(m() == u.size(), ExcDimensionMismatch(m(),v.size()));
+  Assert(m() == u.size(), ExcDimensionMismatch(m(),u.size()));
   Assert(n() == v.size(), ExcDimensionMismatch(n(),v.size()));
 
   number2 sum = 0.;
@@ -934,13 +942,13 @@ FullMatrix<number>::matrix_scalar_product (const Vector<number2> &u,
 
   for (size_type row=0; row<n_rows; ++row)
     {
-      number s = 0.;
+      number2 s = number2(0.);
       const number *const val_end_of_row = val_ptr+n_cols;
       const number2 *v_ptr = v.begin();
       while (val_ptr != val_end_of_row)
-        s += number(*val_ptr++) * number(*v_ptr++);
+        s += number2(*val_ptr++) * number2(*v_ptr++);
 
-      sum += s * number(u(row));
+      sum += s * u(row);
     }
 
   return sum;
@@ -1085,14 +1093,10 @@ void FullMatrix<number>::add (const FullMatrix<number2> &src,
                               const size_type src_offset_i,
                               const size_type src_offset_j)
 {
-  Assert (dst_offset_i < m(),
-          ExcIndexRange (dst_offset_i, 0, m()));
-  Assert (dst_offset_j < n(),
-          ExcIndexRange (dst_offset_j, 0, n()));
-  Assert (src_offset_i < src.m(),
-          ExcIndexRange (src_offset_i, 0, src.m()));
-  Assert (src_offset_j < src.n(),
-          ExcIndexRange (src_offset_j, 0, src.n()));
+  AssertIndexRange (dst_offset_i, m());
+  AssertIndexRange (dst_offset_j, n());
+  AssertIndexRange (src_offset_i, src.m());
+  AssertIndexRange (src_offset_j, src.n());
 
   // Compute maximal size of copied block
   const size_type rows = std::min (m() - dst_offset_i, src.m() - src_offset_i);
@@ -1115,14 +1119,10 @@ void FullMatrix<number>::Tadd (const FullMatrix<number2> &src,
                                const size_type src_offset_i,
                                const size_type src_offset_j)
 {
-  Assert (dst_offset_i < m(),
-          ExcIndexRange (dst_offset_i, 0, m()));
-  Assert (dst_offset_j < n(),
-          ExcIndexRange (dst_offset_j, 0, n()));
-  Assert (src_offset_i < src.n(),
-          ExcIndexRange (src_offset_i, 0, src.n()));
-  Assert (src_offset_j < src.m(),
-          ExcIndexRange (src_offset_j, 0, src.m()));
+  AssertIndexRange (dst_offset_i, m());
+  AssertIndexRange (dst_offset_j, n());
+  AssertIndexRange (src_offset_i, src.n());
+  AssertIndexRange (src_offset_j, src.m());
 
   // Compute maximal size of copied block
   const size_type rows = std::min (m() - dst_offset_i, src.n() - src_offset_j);
@@ -1165,6 +1165,56 @@ FullMatrix<number>::operator == (const FullMatrix<number> &M) const
 }
 
 
+namespace internal
+{
+  namespace
+  {
+    // LAPACKFullMatrix is not implemented for
+    // complex numbers or long doubles
+    template <typename number, typename = void>
+    struct Determinant
+    {
+      static number value (const FullMatrix<number> &)
+      {
+        AssertThrow(false, ExcNotImplemented());
+        return 0.0;
+      }
+    };
+
+
+    // LAPACKFullMatrix is only implemented for
+    // floats and doubles
+    template <typename number>
+    struct Determinant<number, typename std::enable_if<
+      std::is_same<number,float>::value ||
+      std::is_same<number,double>::value
+      >::type>
+    {
+#ifdef DEAL_II_WITH_LAPACK
+      static number value (const FullMatrix<number> &A)
+      {
+        using s_type = typename LAPACKFullMatrix<number>::size_type;
+        AssertIndexRange (A.m()-1, std::numeric_limits<s_type>::max());
+        AssertIndexRange (A.n()-1, std::numeric_limits<s_type>::max());
+        LAPACKFullMatrix<number> lp_A (static_cast<s_type>(A.m()),
+                                       static_cast<s_type>(A.n()));
+        lp_A = A;
+        lp_A.compute_lu_factorization();
+        return lp_A.determinant();
+      }
+#else
+      static number value (const FullMatrix<number> &)
+      {
+        AssertThrow(false, ExcNeedsLAPACK());
+        return 0.0;
+      }
+#endif
+    };
+
+  }
+}
+
+
 template <typename number>
 number
 FullMatrix<number>::determinant () const
@@ -1188,8 +1238,7 @@ FullMatrix<number>::determinant () const
                +(*this)(2,0)*(*this)(0,1)*(*this)(1,2)
                -(*this)(2,0)*(*this)(0,2)*(*this)(1,1));
     default:
-      Assert (false, ExcNotImplemented());
-      return 0;
+      return internal::Determinant<number>::value(*this);
     };
 }
 
@@ -1270,7 +1319,7 @@ FullMatrix<number>::invert (const FullMatrix<number2> &M)
     {
       // avoid overwriting source
       // by destination matrix:
-      FullMatrix<number2> M2 = M;
+      const FullMatrix<number2> M2 = M; // NOLINT
       invert(M2);
     }
   else
@@ -1441,7 +1490,7 @@ FullMatrix<number>::cholesky (const FullMatrix<number2> &A)
     {
       // avoid overwriting source
       // by destination matrix:
-      FullMatrix<number2> A2 = A;
+      const FullMatrix<number2> A2 = A; // NOLINT
       cholesky(A2);
     }
   else
@@ -1496,6 +1545,17 @@ void
 FullMatrix<number>::left_invert (const FullMatrix<number2> &A)
 {
   Assert (!A.empty(), ExcEmptyMatrix());
+
+  // If the matrix is square, simply do a
+  // standard inversion
+  if (A.m() == A.n())
+    {
+      FullMatrix<number2> left_inv(A.n(),A.m());
+      left_inv.invert(A);
+      *this = std::move(left_inv);
+      return;
+    }
+
   Assert(A.m()>A.n(), ExcDimensionMismatch(A.m(), A.n()));
   Assert(this->m()==A.n(), ExcDimensionMismatch(this->m(), A.n()));
   Assert(this->n()==A.m(), ExcDimensionMismatch(this->n(), A.m()));
@@ -1524,6 +1584,17 @@ void
 FullMatrix<number>::right_invert (const FullMatrix<number2> &A)
 {
   Assert (!A.empty(), ExcEmptyMatrix());
+
+  // If the matrix is square, simply do a
+  // standard inversion
+  if (A.m() == A.n())
+    {
+      FullMatrix<number2> right_inv(A.n(),A.m());
+      right_inv.invert(A);
+      *this = std::move(right_inv);
+      return;
+    }
+
   Assert(A.n()>A.m(), ExcDimensionMismatch(A.n(), A.m()));
   Assert(this->m()==A.n(), ExcDimensionMismatch(this->m(), A.n()));
   Assert(this->n()==A.m(), ExcDimensionMismatch(this->n(), A.m()));
@@ -1551,27 +1622,28 @@ template <typename number>
 template <int dim>
 void
 FullMatrix<number>::copy_from (const Tensor<2,dim> &T,
-                               const size_type src_r_i,
-                               const size_type src_r_j,
-                               const size_type src_c_i,
-                               const size_type src_c_j,
+                               const unsigned int src_r_i,
+                               const unsigned int src_r_j,
+                               const unsigned int src_c_i,
+                               const unsigned int src_c_j,
                                const size_type dst_r,
                                const size_type dst_c)
 {
-
   Assert (!this->empty(), ExcEmptyMatrix());
-  Assert(this->m()-dst_r>src_r_j-src_r_i,
-         ExcIndexRange(this->m()-dst_r,0,src_r_j-src_r_i));
-  Assert(this->n()-dst_c>src_c_j-src_c_i,
-         ExcIndexRange(this->n()-dst_c,0,src_c_j-src_c_i));
-  Assert(dim>src_r_j, ExcIndexRange(dim,0,src_r_j));
-  Assert(dim>src_c_j, ExcIndexRange(dim,0,src_r_j));
-  Assert(src_r_j>=src_r_i, ExcIndexRange(src_r_j,0,src_r_i));
-  Assert(src_c_j>=src_c_i, ExcIndexRange(src_r_j,0,src_r_i));
+  AssertIndexRange(src_r_j-src_r_i, this->m()-dst_r);
+  AssertIndexRange(src_c_j-src_c_i, this->n()-dst_c);
+  AssertIndexRange(src_r_j,dim);
+  AssertIndexRange(src_c_j,dim);
+  AssertIndexRange(src_r_i,src_r_j+1);
+  AssertIndexRange(src_c_i,src_c_j+1);
 
   for (size_type i=0; i<src_r_j-src_r_i+1; i++)
     for (size_type j=0; j<src_c_j-src_c_i+1; j++)
-      (*this)(i+dst_r,j+dst_c) = number(T[i+src_r_i][j+src_c_i]);
+      {
+        const unsigned int src_r_index = static_cast<unsigned int>(i+src_r_i);
+        const unsigned int src_c_index = static_cast<unsigned int>(j+src_c_i);
+        (*this)(i+dst_r,j+dst_c) = number(T[src_r_index][src_c_index]);
+      }
 
 }
 
@@ -1584,23 +1656,24 @@ FullMatrix<number>::copy_to (Tensor<2,dim> &T,
                              const size_type src_r_j,
                              const size_type src_c_i,
                              const size_type src_c_j,
-                             const size_type dst_r,
-                             const size_type dst_c) const
+                             const unsigned int dst_r,
+                             const unsigned int dst_c) const
 {
   Assert (!this->empty(), ExcEmptyMatrix());
-  Assert(dim-dst_r>src_r_j-src_r_i,
-         ExcIndexRange(dim-dst_r,0,src_r_j-src_r_i));
-  Assert(dim-dst_c>src_c_j-src_c_i,
-         ExcIndexRange(dim-dst_c,0,src_c_j-src_c_i));
-  Assert(this->m()>src_r_j, ExcIndexRange(dim,0,src_r_j));
-  Assert(this->n()>src_c_j, ExcIndexRange(dim,0,src_r_j));
-  Assert(src_r_j>=src_r_i, ExcIndexRange(src_r_j,0,src_r_i));
-  Assert(src_c_j>=src_c_i, ExcIndexRange(src_r_j,0,src_r_i));
-
+  AssertIndexRange(src_r_j-src_r_i,dim-dst_r);
+  AssertIndexRange(src_c_j-src_c_i,dim-dst_c);
+  AssertIndexRange(src_r_j,this->m());
+  AssertIndexRange(src_r_j,this->n());
+  AssertIndexRange(src_r_i,src_r_j+1);
+  AssertIndexRange(src_c_j,src_c_j+1);
 
   for (size_type i=0; i<src_r_j-src_r_i+1; i++)
     for (size_type j=0; j<src_c_j-src_c_i+1; j++)
-      T[i+dst_r][j+dst_c] = double ((*this)(i+src_r_i,j+src_c_i));
+      {
+        const unsigned int dst_r_index = static_cast<unsigned int>(i+dst_r);
+        const unsigned int dst_c_index = static_cast<unsigned int>(j+dst_c);
+        T[dst_r_index][dst_c_index] = double ((*this)(i+src_r_i,j+src_c_i));
+      }
 }
 
 
@@ -1645,7 +1718,7 @@ FullMatrix<number>::print_formatted (
   // set output format, but store old
   // state
   std::ios::fmtflags old_flags = out.flags();
-  unsigned int old_precision = out.precision (precision);
+  std::streamsize old_precision = out.precision (precision);
 
   if (scientific)
     {
@@ -1663,7 +1736,11 @@ FullMatrix<number>::print_formatted (
   for (size_type i=0; i<m(); ++i)
     {
       for (size_type j=0; j<n(); ++j)
-        if (std::abs((*this)(i,j)) > threshold)
+        // we might have complex numbers, so use abs also to check for nan
+        // since there is no isnan on complex numbers
+        if (std::isnan(std::abs((*this)(i,j))))
+          out << std::setw(width) << (*this)(i,j) << ' ';
+        else if (std::abs((*this)(i,j)) > threshold)
           out << std::setw(width)
               << (*this)(i,j) * number(denominator) << ' ';
         else
@@ -1691,10 +1768,11 @@ FullMatrix<number>::gauss_jordan ()
   // efficient to use Lapack for very small
   // matrices):
 #ifdef DEAL_II_WITH_LAPACK
-  if (types_are_equal<number,double>::value
+  if (std::is_same<number,double>::value
       ||
-      types_are_equal<number,float>::value)
-    if (this->n_cols() > 15)
+      std::is_same<number,float>::value)
+    if (this->n_cols() > 15 &&
+        this->n_cols() <= std::numeric_limits<types::blas_int>::max())
       {
         // In case we have the LAPACK functions
         // getrf and getri detected by CMake,
@@ -1715,15 +1793,15 @@ FullMatrix<number>::gauss_jordan ()
         // we just got ((A^T)^{-1})^T, which is
         // A^{-1}.
 
-        const int nn = this->n();
+        const types::blas_int nn = static_cast<types::blas_int>(this->n());
 
         // workspace for permutations
-        std::vector<int> ipiv(nn);
-        int info;
+        std::vector<types::blas_int> ipiv(nn);
+        types::blas_int info;
 
         // Use the LAPACK function getrf for
         // calculating the LU factorization.
-        getrf(&nn, &nn, &this->values[0], &nn, &ipiv[0], &info);
+        getrf(&nn, &nn, &this->values[0], &nn, ipiv.data(), &info);
 
         Assert(info >= 0, ExcInternalError());
         Assert(info == 0, LACExceptions::ExcSingular());
@@ -1734,7 +1812,7 @@ FullMatrix<number>::gauss_jordan ()
         // Use the LAPACK function getri for
         // calculating the actual inverse using
         // the LU factorization.
-        getri(&nn, &this->values[0], &nn, &ipiv[0], &inv_work[0], &nn, &info);
+        getri(&nn, &this->values[0], &nn, ipiv.data(), inv_work.data(), &nn, &info);
 
         Assert(info >= 0, ExcInternalError());
         Assert(info == 0, LACExceptions::ExcSingular());
