@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 2008 - 2015 by the deal.II authors
+// Copyright (C) 2008 - 2017 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -13,8 +13,8 @@
 //
 // ---------------------------------------------------------------------
 
-#ifndef dealii__generic_linear_algebra_h
-#define dealii__generic_linear_algebra_h
+#ifndef dealii_generic_linear_algebra_h
+#define dealii_generic_linear_algebra_h
 
 #include <deal.II/base/config.h>
 
