@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 1999 - 2017 by the deal.II authors
+// Copyright (C) 1999 - 2018 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -13,8 +13,8 @@
 //
 // ---------------------------------------------------------------------
 
-#ifndef dealii__la_parallel_block_vector_h
-#define dealii__la_parallel_block_vector_h
+#ifndef dealii_la_parallel_block_vector_h
+#define dealii_la_parallel_block_vector_h
 
 
 #include <deal.II/base/config.h>
@@ -22,9 +22,9 @@
 #include <deal.II/lac/block_indices.h>
 #include <deal.II/lac/block_vector_base.h>
 #include <deal.II/lac/la_parallel_vector.h>
+#include <deal.II/lac/vector_operation.h>
 #include <deal.II/lac/vector_type_traits.h>
 
-
 #include <cstdio>
 #include <vector>
 
@@ -81,7 +81,20 @@ namespace LinearAlgebra
     class BlockVector : public BlockVectorBase<Vector<Number> >,
       public VectorSpaceVector<Number>
     {
+      /**
+       * The chunks size to split communication in update_ghost_values()
+       * and compress() calls.
+       *
+       * Most common MPI implementations will get slow when too many
+       * messages/requests are outstanding. Even when messages are small,
+       * say 1 kB only, we should collect enough data with @p communication_block_size
+       * to cover typical infiniband latencies which are around a few microseconds.
+       * Sending 20 kB at a throughput of 5 GB/s takes 4 microseconds,
+       * so we should arrive at the bandwidth dominated regime then which is good enough.
+       */
+      static constexpr unsigned int communication_block_size = 20;
     public:
+
       /**
        * Typedef the base class for simpler access to its own typedefs.
        */
@@ -129,8 +142,6 @@ namespace LinearAlgebra
        */
       BlockVector (const BlockVector<Number> &V);
 
-
-#ifndef DEAL_II_EXPLICIT_CONSTRUCTOR_BUG
       /**
        * Copy constructor taking a BlockVector of another data type. This will
        * fail if there is no conversion path from <tt>OtherNumber</tt> to
@@ -146,7 +157,6 @@ namespace LinearAlgebra
       template <typename OtherNumber>
       explicit
       BlockVector (const BlockVector<OtherNumber> &v);
-#endif
 
       /**
        * Constructor. Set the number of blocks to <tt>block_sizes.size()</tt>
@@ -172,7 +182,7 @@ namespace LinearAlgebra
        * Copy operator: fill all components of the vector with the given
        * scalar value.
        */
-      BlockVector &operator = (const value_type s);
+      virtual BlockVector &operator = (const value_type s) override;
 
       /**
        * Copy operator for arguments of the same type. Resize the present
@@ -301,7 +311,7 @@ namespace LinearAlgebra
        * elements do not agree.
        *
        */
-      void compress (::dealii::VectorOperation::values operation);
+      virtual void compress (::dealii::VectorOperation::values operation) override;
 
       /**
        * Fills the data field for ghost indices with the values stored in the
@@ -321,7 +331,7 @@ namespace LinearAlgebra
        * vector is forbidden and an exception is thrown. Only write access to
        * ghost elements is allowed in this state.
        */
-      void zero_out_ghosts ();
+      void zero_out_ghosts () const;
 
       /**
        * Return if this Vector contains ghost elements.
@@ -348,31 +358,33 @@ namespace LinearAlgebra
        *
        * This function is deprecated.
        */
+      DEAL_II_DEPRECATED
       void equ (const Number a, const BlockVector<Number> &u,
-                const Number b, const BlockVector<Number> &v) DEAL_II_DEPRECATED;
+                const Number b, const BlockVector<Number> &v);
 
       /**
        * Scaling and multiple addition.
        *
        * This function is deprecated.
        */
+      DEAL_II_DEPRECATED
       void sadd (const Number               s,
                  const Number               a,
                  const BlockVector<Number> &V,
                  const Number               b,
-                 const BlockVector<Number> &W) DEAL_II_DEPRECATED;
+                 const BlockVector<Number> &W);
 
       /**
        * Return whether the vector contains only elements with value zero.
        * This function is mainly for internal consistency checks and should
        * seldom be used when not in debug mode since it uses quite some time.
        */
-      bool all_zero () const;
+      virtual bool all_zero () const override;
 
       /**
        * Compute the mean value of all the entries in the vector.
        */
-      Number mean_value () const;
+      virtual Number mean_value () const override;
 
       /**
        * $l_p$-norm of the vector. The pth root of the sum of the pth powers
@@ -392,7 +404,7 @@ namespace LinearAlgebra
        * the same number of blocks. If needed, the numbers of blocks should be
        * exchanged, too.
        *
-       * This function is analogous to the the swap() function of all C++
+       * This function is analogous to the swap() function of all C++
        * standard containers. Also, there is a global function swap(u,v) that
        * simply calls <tt>u.swap(v)</tt>, again in analogy to standard
        * functions.
@@ -405,25 +417,32 @@ namespace LinearAlgebra
        */
       //@{
 
+      /**
+       * Change the dimension to that of the vector V. The elements of V are not
+       * copied.
+       */
+      virtual void reinit(const VectorSpaceVector<Number> &V,
+                          const bool omit_zeroing_entries = false) override;
+
       /**
        * Multiply the entire vector by a fixed factor.
        */
-      virtual BlockVector<Number> &operator*= (const Number factor);
+      virtual BlockVector<Number> &operator*= (const Number factor) override;
 
       /**
        * Divide the entire vector by a fixed factor.
        */
-      virtual BlockVector<Number> &operator/= (const Number factor);
+      virtual BlockVector<Number> &operator/= (const Number factor) override;
 
       /**
        * Add the vector @p V to the present one.
        */
-      virtual BlockVector<Number> &operator+= (const VectorSpaceVector<Number> &V);
+      virtual BlockVector<Number> &operator+= (const VectorSpaceVector<Number> &V) override;
 
       /**
        * Subtract the vector @p V from the present one.
        */
-      virtual BlockVector<Number> &operator-= (const VectorSpaceVector<Number> &V);
+      virtual BlockVector<Number> &operator-= (const VectorSpaceVector<Number> &V) override;
 
       /**
        * Import all the elements present in the vector's IndexSet from the input
@@ -435,29 +454,84 @@ namespace LinearAlgebra
        */
       virtual void import(const LinearAlgebra::ReadWriteVector<Number> &V,
                           VectorOperation::values operation,
-                          std_cxx11::shared_ptr<const CommunicationPatternBase> communication_pattern =
-                            std_cxx11::shared_ptr<const CommunicationPatternBase> ());
+                          std::shared_ptr<const CommunicationPatternBase> communication_pattern =
+                            std::shared_ptr<const CommunicationPatternBase> ()) override;
 
       /**
        * Return the scalar product of two vectors.
        */
-      virtual Number operator* (const VectorSpaceVector<Number> &V) const;
+      virtual Number operator* (const VectorSpaceVector<Number> &V) const override;
+
+      /**
+       * Calculate the scalar product between each block of this vector and @p V
+       * and store the result in a full matrix @p matrix. This function
+       * computes the result by forming $A_{ij}=U_i \cdot V_j$ where $U_i$
+       * and $V_j$ indicate the $i$th block (not element!) of $U$ and the
+       * $j$th block of $V$, respectively. If @p symmetric is
+       * <code>true</code>, it is assumed that inner product results in a
+       * square symmetric matrix and almost half of the scalar products can be avoided.
+       *
+       * Obviously, this function can only be used if all blocks of both vectors
+       * are of the same size.
+       *
+       * @note Internally, a single global reduction will be called to
+       * accumulate scalar product between locally owned degrees of freedom.
+       */
+      template <typename FullMatrixType>
+      void multivector_inner_product(FullMatrixType &matrix,
+                                     const BlockVector<Number> &V,
+                                     const bool symmetric = false) const;
+
+      /**
+       * Calculate the scalar product between each block of this vector and @p V
+       * using a metric tensor @p matrix. This function
+       * computes the result of $ \sum_{ij} A^{ij} U_i \cdot V_j$ where $U_i$
+       * and $V_j$ indicate the $i$th block (not element) of $U$ and the
+       * $j$th block of $V$, respectively. If @p symmetric is
+       * <code>true</code>, it is assumed that $U_i \cdot V_j$ and $A^{ij}$ are
+       * symmetric matrices and almost half of the scalar products can be avoided.
+       *
+       * Obviously, this function can only be used if all blocks of both vectors
+       * are of the same size.
+       *
+       * @note Internally, a single global reduction will be called to
+       * accumulate the scalar product between locally owned degrees of freedom.
+       */
+      template <typename FullMatrixType>
+      Number multivector_inner_product_with_metric(const FullMatrixType &matrix,
+                                                   const BlockVector<Number> &V,
+                                                   const bool symmetric = false) const;
+
+      /**
+       * Set each block of this vector as follows:
+       * $V^i = s V^i + b \sum_{j} U_j A^{ji}$ where $V^i$
+       * and $U_j$ indicate the $i$th block (not element) of $V$ and the
+       * $j$th block of $U$, respectively.
+       *
+       * Obviously, this function can only be used if all blocks of both vectors
+       * are of the same size.
+       */
+      template <typename FullMatrixType>
+      void mmult(BlockVector<Number> &V,
+                 const FullMatrixType &matrix,
+                 const Number s = Number(0.),
+                 const Number b = Number(1.)) const;
 
       /**
        * Add @p a to all components. Note that @p a is a scalar not a vector.
        */
-      virtual void add(const Number a);
+      virtual void add(const Number a) override;
 
       /**
        * Simple addition of a multiple of a vector, i.e. <tt>*this += a*V</tt>.
        */
-      virtual void add(const Number a, const VectorSpaceVector<Number> &V);
+      virtual void add(const Number a, const VectorSpaceVector<Number> &V) override;
 
       /**
        * Multiple addition of scaled vectors, i.e. <tt>*this += a*V+b*W</tt>.
        */
       virtual void add(const Number a, const VectorSpaceVector<Number> &V,
-                       const Number b, const VectorSpaceVector<Number> &W);
+                       const Number b, const VectorSpaceVector<Number> &W) override;
 
       /**
        * A collective add operation: This function adds a whole set of values
@@ -471,37 +545,42 @@ namespace LinearAlgebra
        * s*(*this)+a*V</tt>.
        */
       virtual void sadd(const Number s, const Number a,
-                        const VectorSpaceVector<Number> &V);
+                        const VectorSpaceVector<Number> &V) override;
 
       /**
        * Scale each element of this vector by the corresponding element in the
        * argument. This function is mostly meant to simulate multiplication (and
        * immediate re-assignment) by a diagonal scaling matrix.
        */
-      virtual void scale(const VectorSpaceVector<Number> &scaling_factors);
+      virtual void scale(const VectorSpaceVector<Number> &scaling_factors) override;
 
       /**
        * Assignment <tt>*this = a*V</tt>.
        */
-      virtual void equ(const Number a, const VectorSpaceVector<Number> &V);
+      virtual void equ(const Number a, const VectorSpaceVector<Number> &V) override;
 
       /**
        * Return the l<sub>1</sub> norm of the vector (i.e., the sum of the
        * absolute values of all entries among all processors).
        */
-      virtual real_type l1_norm() const;
+      virtual real_type l1_norm() const override;
 
       /**
-       * Return the l<sub>2</sub> norm of the vector (i.e., the square root of
+       * Return the $l_2$ norm of the vector (i.e., the square root of
        * the sum of the square of all entries among all processors).
        */
-      virtual real_type l2_norm() const;
+      virtual real_type l2_norm() const override;
+
+      /**
+       * Return the square of the $l_2$ norm of the vector.
+       */
+      real_type norm_sqr() const;
 
       /**
        * Return the maximum norm of the vector (i.e., the maximum absolute value
        * among all entries and among all processors).
        */
-      virtual real_type linfty_norm() const;
+      virtual real_type linfty_norm() const override;
 
       /**
        * Perform a combined operation of a vector addition and a subsequent
@@ -518,21 +597,24 @@ namespace LinearAlgebra
        * separate methods means to load the calling vector @p this twice. Since
        * most vector operations are memory transfer limited, this reduces the
        * time by 25\% (or 50\% if @p W equals @p this).
+       *
+       * For complex-valued vectors, the scalar product in the second step is implemented as
+       * $\left<v,w\right>=\sum_i v_i \bar{w_i}$.
        */
       virtual Number add_and_dot(const Number a,
                                  const VectorSpaceVector<Number> &V,
-                                 const VectorSpaceVector<Number> &W);
+                                 const VectorSpaceVector<Number> &W) override;
 
       /**
        * Return the global size of the vector, equal to the sum of the number of
        * locally owned indices among all processors.
        */
-      virtual size_type size() const;
+      virtual size_type size() const override;
 
       /**
        * Return an index set that describes which elements of this vector are
        * owned by the current processor. As a consequence, the index sets
-       * returned on different procesors if this is a distributed vector will
+       * returned on different processors if this is a distributed vector will
        * form disjoint sets that add up to the complete index set. Obviously, if
        * a vector is created on only one processor, then the result would
        * satisfy
@@ -540,7 +622,7 @@ namespace LinearAlgebra
        *  vec.locally_owned_elements() == complete_index_set(vec.size())
        * @endcode
        */
-      virtual dealii::IndexSet locally_owned_elements() const;
+      virtual dealii::IndexSet locally_owned_elements() const override;
 
       /**
        * Print the vector to the output stream @p out.
@@ -548,12 +630,12 @@ namespace LinearAlgebra
       virtual void print(std::ostream &out,
                          const unsigned int precision=3,
                          const bool scientific=true,
-                         const bool across=true) const;
+                         const bool across=true) const override;
 
       /**
        * Return the memory consumption of this class in bytes.
        */
-      virtual std::size_t memory_consumption() const;
+      virtual std::size_t memory_consumption() const override;
       //@}
 
       /**
@@ -587,7 +669,7 @@ namespace LinearAlgebra
  * standard library which uses a temporary object. The function simply
  * exchanges the data of the two vectors.
  *
- * @relates BlockVector
+ * @relatesalso BlockVector
  * @author Katharina Kormann, Martin Kronbichler, 2011
  */
 template <typename Number>
@@ -605,7 +687,7 @@ void swap (LinearAlgebra::distributed::BlockVector<Number> &u,
  * @author Uwe Koecher, 2017
  */
 template <typename Number>
-struct is_serial_vector< LinearAlgebra::distributed::BlockVector< Number > > : std_cxx11::false_type
+struct is_serial_vector< LinearAlgebra::distributed::BlockVector< Number > > : std::false_type
 {
 };
 
