@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 1999 - 2017 by the deal.II authors
+// Copyright (C) 1999 - 2018 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -13,18 +13,23 @@
 //
 // ---------------------------------------------------------------------
 
-#ifndef dealii__parallel_block_vector_templates_h
-#define dealii__parallel_block_vector_templates_h
+#ifndef dealii_parallel_block_vector_templates_h
+#define dealii_parallel_block_vector_templates_h
 
 
 #include <deal.II/base/config.h>
 #include <deal.II/lac/la_parallel_block_vector.h>
 #include <deal.II/lac/petsc_parallel_block_vector.h>
 #include <deal.II/lac/trilinos_parallel_block_vector.h>
+#include <deal.II/lac/lapack_support.h>
+#include <deal.II/lac/vector.h>
 
 
 DEAL_II_NAMESPACE_OPEN
 
+//Forward type declaration to have special treatment of LAPACKFullMatrix<number>
+// in multivector_inner_product()
+template <typename Number> class LAPACKFullMatrix;
 
 namespace LinearAlgebra
 {
@@ -93,7 +98,6 @@ namespace LinearAlgebra
     }
 
 
-#ifndef DEAL_II_EXPLICIT_CONSTRUCTOR_BUG
 
     template <typename Number>
     template <typename OtherNumber>
@@ -103,8 +107,6 @@ namespace LinearAlgebra
       *this = v;
     }
 
-#endif
-
 
 
     template <typename Number>
@@ -239,15 +241,22 @@ namespace LinearAlgebra
     void
     BlockVector<Number>::compress (::dealii::VectorOperation::values operation)
     {
-      // start all requests for all blocks before finishing the transfers as
-      // this saves repeated synchronizations. In order to avoid conflict with
-      // possible other ongoing communication requests (from
-      // LA::distributed::Vector that supports unfinished requests), add an
-      // arbitrary number 8273 to the communication tag
-      for (unsigned int block=0; block<this->n_blocks(); ++block)
-        this->block(block).compress_start(block + 8273, operation);
-      for (unsigned int block=0; block<this->n_blocks(); ++block)
-        this->block(block).compress_finish(operation);
+      const unsigned int n_chunks = (this->n_blocks()+communication_block_size-1)/communication_block_size;
+      for (unsigned int c = 0; c < n_chunks; ++c)
+        {
+          const unsigned int start = c*communication_block_size;
+          const unsigned int end   = std::min(start+communication_block_size, this->n_blocks());
+
+          // start all requests for all blocks before finishing the transfers as
+          // this saves repeated synchronizations. In order to avoid conflict with
+          // possible other ongoing communication requests (from
+          // LA::distributed::Vector that supports unfinished requests), add an
+          // arbitrary number 8273 to the communication tag
+          for (unsigned int block=start; block<end; ++block)
+            this->block(block).compress_start(block + 8273 - start, operation);
+          for (unsigned int block=start; block<end; ++block)
+            this->block(block).compress_finish(operation);
+        }
     }
 
 
@@ -256,20 +265,27 @@ namespace LinearAlgebra
     void
     BlockVector<Number>::update_ghost_values () const
     {
-      // In order to avoid conflict with possible other ongoing communication
-      // requests (from LA::distributed::Vector that supports unfinished
-      // requests), add an arbitrary number 9923 to the communication tag
-      for (unsigned int block=0; block<this->n_blocks(); ++block)
-        this->block(block).update_ghost_values_start(block + 9923);
-      for (unsigned int block=0; block<this->n_blocks(); ++block)
-        this->block(block).update_ghost_values_finish();
+      const unsigned int n_chunks = (this->n_blocks()+communication_block_size-1)/communication_block_size;
+      for (unsigned int c = 0; c < n_chunks; ++c)
+        {
+          const unsigned int start = c*communication_block_size;
+          const unsigned int end   = std::min(start+communication_block_size, this->n_blocks());
+
+          // In order to avoid conflict with possible other ongoing communication
+          // requests (from LA::distributed::Vector that supports unfinished
+          // requests), add an arbitrary number 9923 to the communication tag
+          for (unsigned int block=start; block<end; ++block)
+            this->block(block).update_ghost_values_start(block - start + 9923);
+          for (unsigned int block=start; block<end; ++block)
+            this->block(block).update_ghost_values_finish();
+        }
     }
 
 
 
     template <typename Number>
     void
-    BlockVector<Number>::zero_out_ghosts ()
+    BlockVector<Number>::zero_out_ghosts () const
     {
       for (unsigned int block=0; block<this->n_blocks(); ++block)
         this->block(block).zero_out_ghosts();
@@ -290,6 +306,18 @@ namespace LinearAlgebra
 
 
 
+    template <typename Number>
+    void
+    BlockVector<Number>::reinit(const VectorSpaceVector<Number> &V,
+                                const bool omit_zeroing_entries)
+    {
+      Assert(dynamic_cast<const BlockVector<Number> *>(&V)!=nullptr,
+             ExcVectorTypeNotCompatible());
+      const BlockVector<Number> &down_V = dynamic_cast<const BlockVector<Number> &>(V);
+      reinit(down_V, omit_zeroing_entries);
+    }
+
+
     template <typename Number>
     BlockVector<Number> &
     BlockVector<Number>::operator *= (const Number factor)
@@ -316,7 +344,7 @@ namespace LinearAlgebra
     BlockVector<Number>::scale (const VectorSpaceVector<Number> &vv)
     {
       // Downcast. Throws an exception if invalid.
-      Assert(dynamic_cast<const BlockVector<Number> *>(&vv)!=NULL,
+      Assert(dynamic_cast<const BlockVector<Number> *>(&vv)!=nullptr,
              ExcVectorTypeNotCompatible());
       const BlockVector<Number> &v = dynamic_cast<const BlockVector<Number> &>(vv);
       AssertDimension(this->n_blocks(), v.n_blocks());
@@ -332,7 +360,7 @@ namespace LinearAlgebra
                               const VectorSpaceVector<Number> &vv)
     {
       // Downcast. Throws an exception if invalid.
-      Assert(dynamic_cast<const BlockVector<Number> *>(&vv)!=NULL,
+      Assert(dynamic_cast<const BlockVector<Number> *>(&vv)!=nullptr,
              ExcVectorTypeNotCompatible());
       const BlockVector<Number> &v = dynamic_cast<const BlockVector<Number> &>(vv);
       AssertDimension(this->n_blocks(), v.n_blocks());
@@ -362,7 +390,7 @@ namespace LinearAlgebra
     BlockVector<Number>::operator += (const VectorSpaceVector<Number> &vv)
     {
       // Downcast. Throws an exception if invalid.
-      Assert(dynamic_cast<const BlockVector<Number> *>(&vv)!=NULL,
+      Assert(dynamic_cast<const BlockVector<Number> *>(&vv)!=nullptr,
              ExcVectorTypeNotCompatible());
       const BlockVector<Number> &v = dynamic_cast<const BlockVector<Number> &>(vv);
       AssertDimension(this->n_blocks(), v.n_blocks());
@@ -379,7 +407,7 @@ namespace LinearAlgebra
     BlockVector<Number>::operator -= (const VectorSpaceVector<Number> &vv)
     {
       // Downcast. Throws an exception if invalid.
-      Assert(dynamic_cast<const BlockVector<Number> *>(&vv)!=NULL,
+      Assert(dynamic_cast<const BlockVector<Number> *>(&vv)!=nullptr,
              ExcVectorTypeNotCompatible());
       const BlockVector<Number> &v = dynamic_cast<const BlockVector<Number> &>(vv);
       AssertDimension(this->n_blocks(), v.n_blocks());
@@ -407,7 +435,7 @@ namespace LinearAlgebra
                               const VectorSpaceVector<Number> &vv)
     {
       // Downcast. Throws an exception if invalid.
-      Assert(dynamic_cast<const BlockVector<Number> *>(&vv)!=NULL,
+      Assert(dynamic_cast<const BlockVector<Number> *>(&vv)!=nullptr,
              ExcVectorTypeNotCompatible());
       const BlockVector<Number> &v = dynamic_cast<const BlockVector<Number> &>(vv);
       AssertDimension(this->n_blocks(), v.n_blocks());
@@ -425,11 +453,11 @@ namespace LinearAlgebra
                               const VectorSpaceVector<Number> &ww)
     {
       // Downcast. Throws an exception if invalid.
-      Assert(dynamic_cast<const BlockVector<Number> *>(&vv)!=NULL,
+      Assert(dynamic_cast<const BlockVector<Number> *>(&vv)!=nullptr,
              ExcVectorTypeNotCompatible());
       const BlockVector<Number> &v = dynamic_cast<const BlockVector<Number> &>(vv);
       AssertDimension(this->n_blocks(), v.n_blocks());
-      Assert(dynamic_cast<const BlockVector<Number> *>(&ww)!=NULL,
+      Assert(dynamic_cast<const BlockVector<Number> *>(&ww)!=nullptr,
              ExcVectorTypeNotCompatible());
       const BlockVector<Number> &w = dynamic_cast<const BlockVector<Number> &>(ww);
       AssertDimension(this->n_blocks(), v.n_blocks());
@@ -447,7 +475,7 @@ namespace LinearAlgebra
                                const VectorSpaceVector<Number> &vv)
     {
       // Downcast. Throws an exception if invalid.
-      Assert(dynamic_cast<const BlockVector<Number> *>(&vv)!=NULL,
+      Assert(dynamic_cast<const BlockVector<Number> *>(&vv)!=nullptr,
              ExcVectorTypeNotCompatible());
       const BlockVector<Number> &v = dynamic_cast<const BlockVector<Number> &>(vv);
       AssertDimension(this->n_blocks(), v.n_blocks());
@@ -541,7 +569,7 @@ namespace LinearAlgebra
       Assert (this->n_blocks() > 0, ExcEmptyObject());
 
       // Downcast. Throws an exception if invalid.
-      Assert(dynamic_cast<const BlockVector<Number> *>(&vv)!=NULL,
+      Assert(dynamic_cast<const BlockVector<Number> *>(&vv)!=nullptr,
              ExcVectorTypeNotCompatible());
       const BlockVector<Number> &v = dynamic_cast<const BlockVector<Number> &>(vv);
       AssertDimension(this->n_blocks(), v.n_blocks());
@@ -603,7 +631,7 @@ namespace LinearAlgebra
     template <typename Number>
     inline
     typename BlockVector<Number>::real_type
-    BlockVector<Number>::l2_norm () const
+    BlockVector<Number>::norm_sqr () const
     {
       Assert (this->n_blocks() > 0, ExcEmptyObject());
 
@@ -612,10 +640,20 @@ namespace LinearAlgebra
         local_result += this->block(i).norm_sqr_local();
 
       if (this->block(0).partitioner->n_mpi_processes() > 1)
-        return std::sqrt(Utilities::MPI::sum (local_result,
-                                              this->block(0).partitioner->get_communicator()));
+        return Utilities::MPI::sum (local_result,
+                                    this->block(0).partitioner->get_communicator());
       else
-        return std::sqrt(local_result);
+        return local_result;
+    }
+
+
+
+    template <typename Number>
+    inline
+    typename BlockVector<Number>::real_type
+    BlockVector<Number>::l2_norm () const
+    {
+      return std::sqrt(norm_sqr());
     }
 
 
@@ -669,13 +707,13 @@ namespace LinearAlgebra
                                       const VectorSpaceVector<Number> &ww)
     {
       // Downcast. Throws an exception if invalid.
-      Assert(dynamic_cast<const BlockVector<Number> *>(&vv)!=NULL,
+      Assert(dynamic_cast<const BlockVector<Number> *>(&vv)!=nullptr,
              ExcVectorTypeNotCompatible());
       const BlockVector<Number> &v = dynamic_cast<const BlockVector<Number> &>(vv);
       AssertDimension(this->n_blocks(), v.n_blocks());
 
       // Downcast. Throws an exception if invalid.
-      Assert(dynamic_cast<const BlockVector<Number> *>(&ww)!=NULL,
+      Assert(dynamic_cast<const BlockVector<Number> *>(&ww)!=nullptr,
              ExcVectorTypeNotCompatible());
       const BlockVector<Number> &w = dynamic_cast<const BlockVector<Number> &>(ww);
       AssertDimension(this->n_blocks(), w.n_blocks());
@@ -722,7 +760,7 @@ namespace LinearAlgebra
     void
     BlockVector<Number>::import(const LinearAlgebra::ReadWriteVector<Number> &,
                                 VectorOperation::values,
-                                std_cxx11::shared_ptr<const CommunicationPatternBase>)
+                                std::shared_ptr<const CommunicationPatternBase>)
     {
       AssertThrow(false, ExcNotImplemented());
     }
@@ -770,6 +808,175 @@ namespace LinearAlgebra
               MemoryConsumption::memory_consumption (this->components));
     }
 
+
+
+    namespace
+    {
+      template <typename FullMatrixType>
+      inline
+      void set_symmetric(FullMatrixType &, const bool)
+      {
+      }
+
+      template <typename NumberType>
+      inline
+      void set_symmetric(LAPACKFullMatrix<NumberType> &matrix, const bool symmetric)
+      {
+        if (symmetric)
+          matrix.set_property (LAPACKSupport::symmetric);
+        else
+          matrix.set_property (LAPACKSupport::general);
+      }
+    }
+
+
+
+    template <typename Number>
+    template <typename FullMatrixType>
+    void
+    BlockVector<Number>::multivector_inner_product(FullMatrixType &matrix,
+                                                   const BlockVector<Number> &V,
+                                                   const bool symmetric) const
+    {
+      const unsigned int m = this->n_blocks();
+      const unsigned int n = V.n_blocks();
+
+      // in case one vector is empty and the second one is not, the
+      // FullMatrix resized to (m,n) will have 0 both in m() and n()
+      // which is how TableBase<N,T>::reinit() works as of deal.ii@8.5.0.
+      // Since in this case there is nothing to do anyway -- return immediately.
+      if (n==0 || m==0)
+        return;
+
+      Assert (matrix.m() == m,
+              ExcDimensionMismatch(matrix.m(),m));
+      Assert (matrix.n() == n,
+              ExcDimensionMismatch(matrix.n(),n));
+
+      // reset the matrix
+      matrix = typename FullMatrixType::value_type(0.0);
+
+      set_symmetric(matrix, symmetric);
+      if (symmetric)
+        {
+          Assert (m == n,
+                  ExcDimensionMismatch(m,n));
+
+          for (unsigned int i = 0; i < m; i++)
+            for (unsigned int j = i; j < n; j++)
+              matrix(i,j) = this->block(i).inner_product_local(V.block(j));
+
+          for (unsigned int i = 0; i < m; i++)
+            for (unsigned int j = i+1; j < n; j++)
+              matrix(j,i) = matrix(i,j);
+        }
+      else
+        {
+          for (unsigned int i = 0; i < m; ++i)
+            for (unsigned int j = 0; j < n; ++j)
+              matrix(i,j) = this->block(i).inner_product_local(V.block(j));
+        }
+
+      Utilities::MPI::sum(matrix, this->block(0).get_mpi_communicator(), matrix);
+    }
+
+
+
+    template <typename Number>
+    template <typename FullMatrixType>
+    Number
+    BlockVector<Number>::multivector_inner_product_with_metric(const FullMatrixType &matrix,
+                                                               const BlockVector<Number> &V,
+                                                               const bool symmetric) const
+    {
+      Number res = Number(0.);
+
+      const unsigned int m = this->n_blocks();
+      const unsigned int n = V.n_blocks();
+
+      // in case one vector is empty and the second one is not, the
+      // FullMatrix resized to (m,n) will have 0 both in m() and n()
+      // which is how TableBase<N,T>::reinit() works.
+      // Since in this case there is nothing to do anyway -- return immediately.
+      if (n==0 || m==0)
+        return res;
+
+      Assert (matrix.m() == m,
+              ExcDimensionMismatch(matrix.m(),m));
+      Assert (matrix.n() == n,
+              ExcDimensionMismatch(matrix.n(),n));
+
+      if (symmetric)
+        {
+          Assert (m == n,
+                  ExcDimensionMismatch(m,n));
+
+          for (unsigned int i = 0; i < m; i++)
+            {
+              res += matrix(i,i) * this->block(i).inner_product_local(V.block(i));
+              for (unsigned int j = i+1; j < n; j++)
+                res += 2. * matrix(i,j) * this->block(i).inner_product_local(V.block(j));
+            }
+        }
+      else
+        {
+          for (unsigned int i = 0; i < m; i++)
+            for (unsigned int j = 0; j < n; j++)
+              res += matrix(i,j) * this->block(i).inner_product_local(V.block(j));
+        }
+
+      return Utilities::MPI::sum(res, this->block(0).get_mpi_communicator());
+    }
+
+
+
+    template <typename Number>
+    template <typename FullMatrixType>
+    void
+    BlockVector<Number>::mmult(BlockVector<Number> &V,
+                               const FullMatrixType &matrix,
+                               const Number s,
+                               const Number b) const
+    {
+      const unsigned int m = this->n_blocks();
+      const unsigned int n = V.n_blocks();
+
+      // in case one vector is empty and the second one is not, the
+      // FullMatrix resized to (m,n) will have 0 both in m() and n()
+      // which is how TableBase<N,T>::reinit() works.
+      // Since in this case there is nothing to do anyway -- return immediately.
+      if (n==0 || m==0)
+        return;
+
+      Assert (matrix.m() == m,
+              ExcDimensionMismatch(matrix.m(),m));
+      Assert (matrix.n() == n,
+              ExcDimensionMismatch(matrix.n(),n));
+
+      for (unsigned int i = 0; i < n; i++)
+        {
+          // below we make this work gracefully for identity-like matrices in
+          // which case the two loops over j won't do any work as A(j,i)==0
+          const unsigned int k = std::min(i,m-1);
+          V.block(i).sadd_local(s,matrix(k,i)*b, this->block(k));
+          for (unsigned int j = 0  ; j < k; j++)
+            V.block(i).add_local (matrix(j,i)*b, this->block(j));
+          for (unsigned int j = k+1; j < m; j++)
+            V.block(i).add_local (matrix(j,i)*b, this->block(j));
+        }
+
+      if (V.block(0).vector_is_ghosted)
+        {
+          for (unsigned int i = 0; i < n; i++)
+            Assert (V.block(i).vector_is_ghosted,
+                    ExcMessage("All blocks should be either in ghosted state or not."));
+
+          V.update_ghost_values();
+        }
+    }
+
+
+
   } // end of namespace distributed
 
 } // end of namespace parallel
