@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 2011 - 2017 by the deal.II authors
+// Copyright (C) 2011 - 2018 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -13,19 +13,20 @@
 //
 // ---------------------------------------------------------------------
 
-#ifndef dealii__la_parallel_vector_h
-#define dealii__la_parallel_vector_h
+#ifndef dealii_la_parallel_vector_h
+#define dealii_la_parallel_vector_h
 
 #include <deal.II/base/config.h>
 #include <deal.II/base/mpi.h>
 #include <deal.II/base/numbers.h>
 #include <deal.II/base/partitioner.h>
-#include <deal.II/base/std_cxx11/shared_ptr.h>
 #include <deal.II/base/thread_management.h>
+#include <deal.II/lac/vector_operation.h>
 #include <deal.II/lac/vector_space_vector.h>
 #include <deal.II/lac/vector_type_traits.h>
 
 #include <iomanip>
+#include <memory>
 
 DEAL_II_NAMESPACE_OPEN
 
@@ -187,21 +188,6 @@ namespace LinearAlgebra
       typedef types::global_dof_index                           size_type;
       typedef typename numbers::NumberTraits<Number>::real_type real_type;
 
-      /**
-       * A variable that indicates whether this vector supports distributed
-       * data storage. If true, then this vector also needs an appropriate
-       * compress() function that allows communicating recent set or add
-       * operations to individual elements to be communicated to other
-       * processors.
-       *
-       * For the current class, the variable equals true, since it does
-       * support parallel data storage.
-       *
-       * @deprecated instead of using this variable, please use the type trait
-       * value <code>is_serial_vector< VectorType >::value</code>
-       */
-      static const bool supports_distributed_data DEAL_II_DEPRECATED = true;
-
       /**
        * @name 1: Basic Object-handling
        */
@@ -254,12 +240,12 @@ namespace LinearAlgebra
        * partitioner data only once and share it between several vectors with
        * the same layout.
        */
-      Vector (const std_cxx11::shared_ptr<const Utilities::MPI::Partitioner> &partitioner);
+      Vector (const std::shared_ptr<const Utilities::MPI::Partitioner> &partitioner);
 
       /**
        * Destructor.
        */
-      virtual ~Vector ();
+      virtual ~Vector () override;
 
       /**
        * Set the global size of the vector to @p size without any actual
@@ -314,7 +300,7 @@ namespace LinearAlgebra
        * the partitioner data only once and share it between several vectors
        * with the same layout.
        */
-      void reinit (const std_cxx11::shared_ptr<const Utilities::MPI::Partitioner> &partitioner);
+      void reinit (const std::shared_ptr<const Utilities::MPI::Partitioner> &partitioner);
 
       /**
        * Swap the contents of this vector and the other vector @p v. One could
@@ -323,7 +309,7 @@ namespace LinearAlgebra
        * only swaps the pointers to the data of the two vectors and therefore
        * does not need to allocate temporary storage and move data around.
        *
-       * This function is analogous to the the @p swap function of all C++
+       * This function is analogous to the @p swap function of all C++
        * standard containers. Also, there is a global function
        * <tt>swap(u,v)</tt> that simply calls <tt>u.swap(v)</tt>, again in
        * analogy to standard functions.
@@ -373,8 +359,9 @@ namespace LinearAlgebra
        * This function is deprecated. Use the interface through
        * ReadWriteVector instead.
        */
+      DEAL_II_DEPRECATED
       Vector<Number> &
-      operator = (const PETScWrappers::MPI::Vector &petsc_vec) DEAL_II_DEPRECATED;
+      operator = (const PETScWrappers::MPI::Vector &petsc_vec);
 #endif
 
 #ifdef DEAL_II_WITH_TRILINOS
@@ -389,8 +376,9 @@ namespace LinearAlgebra
        * This function is deprecated. Use the interface through
        * ReadWriteVector instead.
        */
+      DEAL_II_DEPRECATED
       Vector<Number> &
-      operator = (const TrilinosWrappers::MPI::Vector &trilinos_vec) DEAL_II_DEPRECATED;
+      operator = (const TrilinosWrappers::MPI::Vector &trilinos_vec);
 #endif
       //@}
 
@@ -420,7 +408,7 @@ namespace LinearAlgebra
        * ghost element is found, it is compared to the value on the owning
        * processor and an exception is thrown if these elements do not agree.
        */
-      void compress (::dealii::VectorOperation::values operation);
+      virtual void compress (::dealii::VectorOperation::values operation) override;
 
       /**
        * Fills the data field for ghost indices with the values stored in the
@@ -513,7 +501,7 @@ namespace LinearAlgebra
        * vector is forbidden and an exception is thrown. Only write access to
        * ghost elements is allowed in this state.
        */
-      void zero_out_ghosts ();
+      void zero_out_ghosts () const;
 
       /**
        * Return whether the vector currently is in a state where ghost values
@@ -527,6 +515,24 @@ namespace LinearAlgebra
        * @ref GlossGhostedVector "vectors with ghost elements"
        */
       bool has_ghost_elements() const;
+
+      /**
+       * This method copies the data in the locally owned range from another
+       * distributed vector @p src into the calling vector. As opposed to
+       * operator= that also includes ghost entries, this operation ignores
+       * the ghost range. The only prerequisite is that the local range on the
+       * calling vector and the given vector @p src are the same on all
+       * processors. It is explicitly allowed that the two vectors have
+       * different ghost elements that might or might not be related to each
+       * other.
+       *
+       * Since no data exchange is performed, make sure that neither @p src
+       * nor the calling vector have pending communications in order to obtain
+       * correct results.
+       */
+      template <typename Number2>
+      void copy_locally_owned_data_from(const Vector<Number2> &src);
+
       //@}
 
       /**
@@ -534,25 +540,32 @@ namespace LinearAlgebra
        */
       //@{
 
+      /**
+       * Change the dimension to that of the vector V. The elements of V are not
+       * copied.
+       */
+      virtual void reinit(const VectorSpaceVector<Number> &V,
+                          const bool omit_zeroing_entries = false) override;
+
       /**
        * Multiply the entire vector by a fixed factor.
        */
-      virtual Vector<Number> &operator*= (const Number factor);
+      virtual Vector<Number> &operator*= (const Number factor) override;
 
       /**
        * Divide the entire vector by a fixed factor.
        */
-      virtual Vector<Number> &operator/= (const Number factor);
+      virtual Vector<Number> &operator/= (const Number factor) override;
 
       /**
        * Add the vector @p V to the present one.
        */
-      virtual Vector<Number> &operator+= (const VectorSpaceVector<Number> &V);
+      virtual Vector<Number> &operator+= (const VectorSpaceVector<Number> &V) override;
 
       /**
        * Subtract the vector @p V from the present one.
        */
-      virtual Vector<Number> &operator-= (const VectorSpaceVector<Number> &V);
+      virtual Vector<Number> &operator-= (const VectorSpaceVector<Number> &V) override;
 
       /**
        * Import all the elements present in the vector's IndexSet from the input
@@ -564,29 +577,29 @@ namespace LinearAlgebra
        */
       virtual void import(const LinearAlgebra::ReadWriteVector<Number> &V,
                           VectorOperation::values operation,
-                          std_cxx11::shared_ptr<const CommunicationPatternBase> communication_pattern =
-                            std_cxx11::shared_ptr<const CommunicationPatternBase> ());
+                          std::shared_ptr<const CommunicationPatternBase> communication_pattern =
+                            std::shared_ptr<const CommunicationPatternBase> ()) override;
 
       /**
        * Return the scalar product of two vectors.
        */
-      virtual Number operator* (const VectorSpaceVector<Number> &V) const;
+      virtual Number operator* (const VectorSpaceVector<Number> &V) const override;
 
       /**
        * Add @p a to all components. Note that @p a is a scalar not a vector.
        */
-      virtual void add(const Number a);
+      virtual void add(const Number a) override;
 
       /**
        * Simple addition of a multiple of a vector, i.e. <tt>*this += a*V</tt>.
        */
-      virtual void add(const Number a, const VectorSpaceVector<Number> &V);
+      virtual void add(const Number a, const VectorSpaceVector<Number> &V) override;
 
       /**
        * Multiple addition of scaled vectors, i.e. <tt>*this += a*V+b*W</tt>.
        */
       virtual void add(const Number a, const VectorSpaceVector<Number> &V,
-                       const Number b, const VectorSpaceVector<Number> &W);
+                       const Number b, const VectorSpaceVector<Number> &W) override;
 
       /**
        * A collective add operation: This function adds a whole set of values
@@ -600,37 +613,42 @@ namespace LinearAlgebra
        * s*(*this)+a*V</tt>.
        */
       virtual void sadd(const Number s, const Number a,
-                        const VectorSpaceVector<Number> &V);
+                        const VectorSpaceVector<Number> &V) override;
 
       /**
        * Scale each element of this vector by the corresponding element in the
        * argument. This function is mostly meant to simulate multiplication (and
        * immediate re-assignment) by a diagonal scaling matrix.
        */
-      virtual void scale(const VectorSpaceVector<Number> &scaling_factors);
+      virtual void scale(const VectorSpaceVector<Number> &scaling_factors) override;
 
       /**
        * Assignment <tt>*this = a*V</tt>.
        */
-      virtual void equ(const Number a, const VectorSpaceVector<Number> &V);
+      virtual void equ(const Number a, const VectorSpaceVector<Number> &V) override;
 
       /**
        * Return the l<sub>1</sub> norm of the vector (i.e., the sum of the
        * absolute values of all entries among all processors).
        */
-      virtual real_type l1_norm() const;
+      virtual real_type l1_norm() const override;
 
       /**
-       * Return the l<sub>2</sub> norm of the vector (i.e., the square root of
+       * Return the $l_2$ norm of the vector (i.e., the square root of
        * the sum of the square of all entries among all processors).
        */
-      virtual real_type l2_norm() const;
+      virtual real_type l2_norm() const override;
+
+      /**
+       * Return the square of the $l_2$ norm of the vector.
+       */
+      real_type norm_sqr() const;
 
       /**
        * Return the maximum norm of the vector (i.e., the maximum absolute value
        * among all entries and among all processors).
        */
-      virtual real_type linfty_norm() const;
+      virtual real_type linfty_norm() const override;
 
       /**
        * Perform a combined operation of a vector addition and a subsequent
@@ -647,21 +665,24 @@ namespace LinearAlgebra
        * separate methods means to load the calling vector @p this twice. Since
        * most vector operations are memory transfer limited, this reduces the
        * time by 25\% (or 50\% if @p W equals @p this).
+       *
+       * For complex-valued vectors, the scalar product in the second step is implemented as
+       * $\left<v,w\right>=\sum_i v_i \bar{w_i}$.
        */
       virtual Number add_and_dot(const Number a,
                                  const VectorSpaceVector<Number> &V,
-                                 const VectorSpaceVector<Number> &W);
+                                 const VectorSpaceVector<Number> &W) override;
 
       /**
        * Return the global size of the vector, equal to the sum of the number of
        * locally owned indices among all processors.
        */
-      virtual size_type size() const;
+      virtual size_type size() const override;
 
       /**
        * Return an index set that describes which elements of this vector are
        * owned by the current processor. As a consequence, the index sets
-       * returned on different procesors if this is a distributed vector will
+       * returned on different processors if this is a distributed vector will
        * form disjoint sets that add up to the complete index set. Obviously, if
        * a vector is created on only one processor, then the result would
        * satisfy
@@ -669,7 +690,7 @@ namespace LinearAlgebra
        *  vec.locally_owned_elements() == complete_index_set(vec.size())
        * @endcode
        */
-      virtual dealii::IndexSet locally_owned_elements() const;
+      virtual dealii::IndexSet locally_owned_elements() const override;
 
       /**
        * Print the vector to the output stream @p out.
@@ -677,12 +698,12 @@ namespace LinearAlgebra
       virtual void print(std::ostream &out,
                          const unsigned int precision=3,
                          const bool scientific=true,
-                         const bool across=true) const;
+                         const bool across=true) const override;
 
       /**
        * Return the memory consumption of this class in bytes.
        */
-      virtual std::size_t memory_consumption() const;
+      virtual std::size_t memory_consumption() const override;
       //@}
 
       /**
@@ -695,7 +716,7 @@ namespace LinearAlgebra
        * zero, also ghost elements are set to zero, otherwise they remain
        * unchanged.
        */
-      Vector<Number> &operator = (const Number s);
+      virtual Vector<Number> &operator = (const Number s) override;
 
       /**
        * This is a collective add operation that adds a whole set of values
@@ -726,19 +747,21 @@ namespace LinearAlgebra
        *
        * This function is deprecated.
        */
+      DEAL_II_DEPRECATED
       void sadd (const Number          s,
                  const Number          a,
                  const Vector<Number> &V,
                  const Number          b,
-                 const Vector<Number> &W) DEAL_II_DEPRECATED;
+                 const Vector<Number> &W);
 
       /**
        * Assignment <tt>*this = a*u + b*v</tt>.
        *
        * This function is deprecated.
        */
+      DEAL_II_DEPRECATED
       void equ (const Number a, const Vector<Number> &u,
-                const Number b, const Vector<Number> &v) DEAL_II_DEPRECATED;
+                const Number b, const Vector<Number> &v);
 
       //@}
 
@@ -761,7 +784,8 @@ namespace LinearAlgebra
        *
        * This function is deprecated.
        */
-      std::pair<size_type, size_type> local_range () const DEAL_II_DEPRECATED;
+      DEAL_II_DEPRECATED
+      std::pair<size_type, size_type> local_range () const;
 
       /**
        * Return true if the given global index is in the local range of this
@@ -769,14 +793,16 @@ namespace LinearAlgebra
        *
        * This function is deprecated.
        */
-      bool in_local_range (const size_type global_index) const DEAL_II_DEPRECATED;
+      DEAL_II_DEPRECATED
+      bool in_local_range (const size_type global_index) const;
 
       /**
        * Return the number of ghost elements present on the vector.
        *
        * This function is deprecated.
        */
-      size_type n_ghost_entries () const DEAL_II_DEPRECATED;
+      DEAL_II_DEPRECATED
+      size_type n_ghost_entries () const;
 
       /**
        * Return an index set that describes which elements of this vector are
@@ -785,7 +811,8 @@ namespace LinearAlgebra
        *
        * This function is deprecated.
        */
-      const IndexSet &ghost_elements() const DEAL_II_DEPRECATED;
+      DEAL_II_DEPRECATED
+      const IndexSet &ghost_elements() const;
 
       /**
        * Return whether the given global index is a ghost index on the
@@ -794,7 +821,8 @@ namespace LinearAlgebra
        *
        * This function is deprecated.
        */
-      bool is_ghost_entry (const types::global_dof_index global_index) const DEAL_II_DEPRECATED;
+      DEAL_II_DEPRECATED
+      bool is_ghost_entry (const types::global_dof_index global_index) const;
 
       /**
        * Make the @p Vector class a bit like the <tt>vector<></tt> class of
@@ -883,18 +911,50 @@ namespace LinearAlgebra
       Number &local_element (const size_type local_index);
 
       /**
-       * A collective get operation: instead of getting individual elements of
-       * a vector, this function allows to get a whole set of elements at
-       * once. The indices of the elements to be read are stated in the first
-       * argument, the corresponding values are returned in the second.
+       * Instead of getting individual elements of a vector via operator(),
+       * this function allows getting a whole set of elements at once. The
+       * indices of the elements to be read are stated in the first argument, the
+       * corresponding values are returned in the second.
+       *
+       * If the current vector is called @p v, then this function is the equivalent
+       * to the code
+       * @code
+       *   for (unsigned int i=0; i<indices.size(); ++i)
+       *     values[i] = v[indices[i]];
+       * @endcode
+       *
+       * @pre The sizes of the @p indices and @p values arrays must be identical.
        */
       template <typename OtherNumber>
       void extract_subvector_to (const std::vector<size_type> &indices,
                                  std::vector<OtherNumber> &values) const;
 
       /**
-       * Just as the above, but with pointers. Useful in minimizing copying of
-       * data around.
+       * Instead of getting individual elements of a vector via operator(),
+       * this function allows getting a whole set of elements at once. In
+       * contrast to the previous function, this function obtains the
+       * indices of the elements by dereferencing all elements of the iterator
+       * range provided by the first two arguments, and puts the vector
+       * values into memory locations obtained by dereferencing a range
+       * of iterators starting at the location pointed to by the third
+       * argument.
+       *
+       * If the current vector is called @p v, then this function is the equivalent
+       * to the code
+       * @code
+       *   ForwardIterator indices_p = indices_begin;
+       *   OutputIterator  values_p  = values_begin;
+       *   while (indices_p != indices_end)
+       *   {
+       *     *values_p = v[*indices_p];
+       *     ++indices_p;
+       *     ++values_p;
+       *   }
+       * @endcode
+       *
+       * @pre It must be possible to write into as many memory locations
+       *   starting at @p values_begin as there are iterators between
+       *   @p indices_begin and @p indices_end.
        */
       template <typename ForwardIterator, typename OutputIterator>
       void extract_subvector_to (ForwardIterator          indices_begin,
@@ -905,12 +965,12 @@ namespace LinearAlgebra
        * This is a collective operation. This function is expensive, because
        * potentially all elements have to be checked.
        */
-      bool all_zero () const;
+      virtual bool all_zero () const override;
 
       /**
        * Compute the mean value of all the entries in the vector.
        */
-      virtual Number mean_value () const;
+      virtual Number mean_value () const override;
 
       /**
        * $l_p$-norm of the vector. The pth root of the sum of the pth powers
@@ -936,7 +996,7 @@ namespace LinearAlgebra
        * respective reinit() call, for additional queries regarding the
        * parallel communication, or the compatibility of partitioners.
        */
-      const std_cxx11::shared_ptr<const Utilities::MPI::Partitioner> &
+      const std::shared_ptr<const Utilities::MPI::Partitioner> &
       get_partitioner () const;
 
       /**
@@ -1000,6 +1060,20 @@ namespace LinearAlgebra
                       << "that this vector can access.");
 
     private:
+
+      /**
+       * Simple addition of a multiple of a vector, i.e. <tt>*this += a*V</tt>
+       * without MPI communication.
+       */
+      void add_local(const Number a, const VectorSpaceVector<Number> &V);
+
+      /**
+       * Scaling and simple addition of a multiple of a vector, i.e. <tt>*this =
+       * s*(*this)+a*V</tt> without MPI communication.
+       */
+      void sadd_local(const Number s, const Number a,
+                      const VectorSpaceVector<Number> &V);
+
       /**
        * Local part of all_zero().
        */
@@ -1038,7 +1112,8 @@ namespace LinearAlgebra
 
       /**
        * Local part of the addition followed by an inner product of two
-       * vectors.
+       * vectors. The same applies for complex-valued vectors as for
+       * the add_and_dot() function.
        */
       Number add_and_dot_local (const Number          a,
                                 const Vector<Number> &V,
@@ -1049,7 +1124,7 @@ namespace LinearAlgebra
        * information can be shared between several vectors that have the same
        * partitioning.
        */
-      std_cxx11::shared_ptr<const Utilities::MPI::Partitioner> partitioner;
+      std::shared_ptr<const Utilities::MPI::Partitioner> partitioner;
 
       /**
        * The size that is currently allocated in the val array.
@@ -1058,21 +1133,25 @@ namespace LinearAlgebra
 
       /**
        * Pointer to the array of local elements of this vector.
+       *
+       * Because we allocate these arrays via Utilities::System::posix_memalign,
+       * we need to use a custom deleter for this object that does not call
+       * <code>delete[]</code>, but instead calls @p free().
        */
-      Number         *val;
+      std::unique_ptr<Number[], decltype(&free)> values;
 
       /**
        * For parallel loops with TBB, this member variable stores the affinity
        * information of loops.
        */
-      mutable std_cxx11::shared_ptr< ::dealii::parallel::internal::TBBPartitioner> thread_loop_partitioner;
+      mutable std::shared_ptr< ::dealii::parallel::internal::TBBPartitioner> thread_loop_partitioner;
 
       /**
        * Temporary storage that holds the data that is sent to this processor
-       * in @p compress() or sent from this processor in @p
-       * update_ghost_values.
+       * in @p compress() or sent from this processor in
+       * @p update_ghost_values.
        */
-      mutable Number *import_data;
+      mutable std::unique_ptr<Number[]> import_data;
 
       /**
        * Stores whether the vector currently allows for reading ghost elements
@@ -1239,7 +1318,7 @@ namespace LinearAlgebra
     typename Vector<Number>::iterator
     Vector<Number>::begin ()
     {
-      return val;
+      return values.get();
     }
 
 
@@ -1249,7 +1328,7 @@ namespace LinearAlgebra
     typename Vector<Number>::const_iterator
     Vector<Number>::begin () const
     {
-      return val;
+      return values.get();
     }
 
 
@@ -1259,7 +1338,7 @@ namespace LinearAlgebra
     typename Vector<Number>::iterator
     Vector<Number>::end ()
     {
-      return val+partitioner->local_size();
+      return values.get()+partitioner->local_size();
     }
 
 
@@ -1269,7 +1348,7 @@ namespace LinearAlgebra
     typename Vector<Number>::const_iterator
     Vector<Number>::end () const
     {
-      return val+partitioner->local_size();
+      return values.get()+partitioner->local_size();
     }
 
 
@@ -1288,7 +1367,7 @@ namespace LinearAlgebra
       Assert (partitioner->in_local_range (global_index) || vector_is_ghosted == true,
               ExcMessage("You tried to read a ghost element of this vector, "
                          "but it has not imported its ghost values."));
-      return val[partitioner->global_to_local(global_index)];
+      return values[partitioner->global_to_local(global_index)];
     }
 
 
@@ -1309,7 +1388,7 @@ namespace LinearAlgebra
       // (then, the compiler picks this method according to the C++ rule book
       // even if a human would pick the const method when this subsequent use
       // is just a read)
-      return val[partitioner->global_to_local (global_index)];
+      return values[partitioner->global_to_local (global_index)];
     }
 
 
@@ -1346,7 +1425,7 @@ namespace LinearAlgebra
       Assert (local_index < local_size() || vector_is_ghosted == true,
               ExcMessage("You tried to read a ghost element of this vector, "
                          "but it has not imported its ghost values."));
-      return val[local_index];
+      return values[local_index];
     }
 
 
@@ -1359,7 +1438,7 @@ namespace LinearAlgebra
       AssertIndexRange (local_index,
                         partitioner->local_size()+
                         partitioner->n_ghost_indices());
-      return val[local_index];
+      return values[local_index];
     }
 
 
@@ -1441,7 +1520,7 @@ namespace LinearAlgebra
 
     template <typename Number>
     inline
-    const std_cxx11::shared_ptr<const Utilities::MPI::Partitioner> &
+    const std::shared_ptr<const Utilities::MPI::Partitioner> &
     Vector<Number>::get_partitioner () const
     {
       return partitioner;
@@ -1458,7 +1537,7 @@ namespace LinearAlgebra
  * C++ standard library which uses a temporary object. The function simply
  * exchanges the data of the two vectors.
  *
- * @relates Vector
+ * @relatesalso Vector
  * @author Katharina Kormann, Martin Kronbichler, 2011
  */
 template <typename Number>
@@ -1476,11 +1555,52 @@ void swap (LinearAlgebra::distributed::Vector<Number> &u,
  * @author Uwe Koecher, 2017
  */
 template <typename Number>
-struct is_serial_vector< LinearAlgebra::distributed::Vector< Number > > : std_cxx11::false_type
+struct is_serial_vector< LinearAlgebra::distributed::Vector< Number > > : std::false_type
 {
 };
 
 
+namespace internal
+{
+  namespace LinearOperatorImplementation
+  {
+    template <typename> class ReinitHelper;
+
+    /**
+     * A helper class used internally in linear_operator.h. Specialization for
+     * LinearAlgebra::distributed::Vector<Number>.
+     */
+    template <typename Number>
+    class ReinitHelper<LinearAlgebra::distributed::Vector<Number>>
+    {
+    public:
+      template <typename Matrix>
+      static
+      void reinit_range_vector (const Matrix &matrix,
+                                LinearAlgebra::distributed::Vector<Number> &v,
+                                bool omit_zeroing_entries)
+      {
+        matrix.initialize_dof_vector(v);
+        if (!omit_zeroing_entries)
+          v = Number();
+      }
+
+      template <typename Matrix>
+      static
+      void reinit_domain_vector(const Matrix &matrix,
+                                LinearAlgebra::distributed::Vector<Number> &v,
+                                bool omit_zeroing_entries)
+      {
+        matrix.initialize_dof_vector(v);
+        if (!omit_zeroing_entries)
+          v = Number();
+      }
+    };
+
+  } /* namespace LinearOperator */
+} /* namespace internal */
+
+
 DEAL_II_NAMESPACE_CLOSE
 
 #endif
