@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 2015 - 2017 by the deal.II authors
+// Copyright (C) 2015 - 2018 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -13,8 +13,8 @@
 //
 // ---------------------------------------------------------------------
 
-#ifndef dealii__la_vector_h
-#define dealii__la_vector_h
+#ifndef dealii_la_vector_h
+#define dealii_la_vector_h
 
 
 #include <deal.II/base/config.h>
@@ -22,6 +22,7 @@
 #include <deal.II/base/exceptions.h>
 #include <deal.II/base/index_set.h>
 #include <deal.II/lac/read_write_vector.h>
+#include <deal.II/lac/vector_operation.h>
 #include <deal.II/lac/vector_space_vector.h>
 #include <deal.II/lac/vector_type_traits.h>
 
@@ -68,7 +69,7 @@ namespace LinearAlgebra
     /**
      * Constructor. Create a vector of dimension zero.
      */
-    Vector();
+    Vector() = default;
 
     /**
      * Copy constructor. Sets the dimension to that of the given vector and
@@ -95,6 +96,49 @@ namespace LinearAlgebra
     template <typename InputIterator>
     Vector(const InputIterator first, const InputIterator last);
 
+    /**
+     * Set the global size of the vector to @p size. The stored elements have
+     * their index in [0,size).
+     *
+     * If the flag @p omit_zeroing_entries is set to false, the memory will be
+     * initialized with zero, otherwise the memory will be untouched (and the
+     * user must make sure to fill it with reasonable data before using it).
+     */
+    virtual void reinit(const size_type size,
+                        const bool      omit_zeroing_entries = false) override;
+
+    /**
+     * Uses the same IndexSet as the one of the input vector @p in_vector and
+     * allocates memory for this vector.
+     *
+     * If the flag @p omit_zeroing_entries is set to false, the memory will be
+     * initialized with zero, otherwise the memory will be untouched (and the
+     * user must make sure to fill it with reasonable data before using it).
+     */
+    template <typename Number2>
+    void reinit(const ReadWriteVector<Number2> &in_vector,
+                const bool                      omit_zeroing_entries = false);
+
+    /**
+     * Initializes the vector. The indices are specified by @p
+     * locally_stored_indices.
+     *
+     * If the flag @p omit_zeroing_entries is set to false, the memory will be
+     * initialized with zero, otherwise the memory will be untouched (and the
+     * user must make sure to fill it with reasonable data before using it).
+     * locally_stored_indices.
+     */
+    virtual void reinit(const IndexSet &locally_stored_indices,
+                        const bool      omit_zeroing_entries = false) override;
+
+
+    /**
+     * Change the dimension to that of the vector V. The elements of V are not
+     * copied.
+     */
+    virtual void reinit(const VectorSpaceVector<Number> &V,
+                        const bool omit_zeroing_entries = false) override;
+
     /**
      * Copies the data of the input vector @p in_vector.
      */
@@ -110,100 +154,105 @@ namespace LinearAlgebra
      * Sets all elements of the vector to the scalar @p s. This operation is
      * only allowed if @p s is equal to zero.
      */
-    Vector<Number> &operator= (const Number s);
+    virtual Vector<Number> &operator= (const Number s) override;
 
     /**
      * Multiply the entire vector by a fixed factor.
      */
-    virtual Vector<Number> &operator*= (const Number factor);
+    virtual Vector<Number> &operator*= (const Number factor) override;
 
     /**
      * Divide the entire vector by a fixed factor.
      */
-    virtual Vector<Number> &operator/= (const Number factor);
+    virtual Vector<Number> &operator/= (const Number factor) override;
 
     /**
      * Add the vector @p V to the present one.
      */
-    virtual Vector<Number> &operator+= (const VectorSpaceVector<Number> &V);
+    virtual Vector<Number> &operator+= (const VectorSpaceVector<Number> &V) override;
 
     /**
      * Subtract the vector @p V from the present one.
      */
-    virtual Vector<Number> &operator-= (const VectorSpaceVector<Number> &V);
+    virtual Vector<Number> &operator-= (const VectorSpaceVector<Number> &V) override;
 
     /**
      * Return the scalar product of two vectors.
      */
-    virtual Number operator* (const VectorSpaceVector<Number> &V) const;
+    virtual Number operator* (const VectorSpaceVector<Number> &V) const override;
 
     /**
      * This function is not implemented and will throw an exception.
      */
     virtual void import(const ReadWriteVector<Number> &V,
                         VectorOperation::values operation,
-                        std_cxx11::shared_ptr<const CommunicationPatternBase>
+                        std::shared_ptr<const CommunicationPatternBase>
                         communication_pattern =
-                          std_cxx11::shared_ptr<const CommunicationPatternBase>());
+                          std::shared_ptr<const CommunicationPatternBase>()) override;
 
     /**
      * Add @p a to all components. Note that @p a is a scalar not a vector.
      */
-    virtual void add(const Number a);
+    virtual void add(const Number a) override;
 
     /**
      * Simple addition of a multiple of a vector, i.e. <tt>*this += a*V</tt>.
      */
-    virtual void add(const Number a, const VectorSpaceVector<Number> &V);
+    virtual void add(const Number a, const VectorSpaceVector<Number> &V) override;
 
     /**
      * Multiple addition of a multiple of a vector, i.e. <tt>*this +=
      * a*V+b*W</tt>.
      */
     virtual void add(const Number a, const VectorSpaceVector<Number> &V,
-                     const Number b, const VectorSpaceVector<Number> &W);
+                     const Number b, const VectorSpaceVector<Number> &W) override;
 
     /**
      * Scaling and simple addition of a multiple of a vector, i.e. <tt>*this =
      * s*(*this)+a*V</tt>.
      */
     virtual void sadd(const Number s, const Number a,
-                      const VectorSpaceVector<Number> &V);
+                      const VectorSpaceVector<Number> &V) override;
 
     /**
      * Scale each element of this vector by the corresponding element in the
      * argument. This function is mostly meant to simulate multiplication (and
      * immediate re-assignment) by a diagonal scaling matrix.
      */
-    virtual void scale(const VectorSpaceVector<Number> &scaling_factors);
+    virtual void scale(const VectorSpaceVector<Number> &scaling_factors) override;
 
     /**
      * Assignment <tt>*this = a*V</tt>.
      */
-    virtual void equ(const Number a, const VectorSpaceVector<Number> &V);
+    virtual void equ(const Number a, const VectorSpaceVector<Number> &V) override;
+
+    /**
+     * Return whether the vector contains only elements with value zero.
+     */
+    virtual bool all_zero() const override;
 
     /**
      * Return the mean value of all the entries of this vector.
      */
-    virtual value_type mean_value() const;
+    virtual value_type mean_value() const override;
 
     /**
      * Return the l<sub>1</sub> norm of the vector (i.e., the sum of the
      * absolute values of all entries).
      */
-    virtual typename VectorSpaceVector<Number>::real_type l1_norm() const;
+    virtual typename VectorSpaceVector<Number>::real_type l1_norm() const override;
 
     /**
      * Return the l<sub>2</sub> norm of the vector (i.e., the square root of
      * the sum of the square of all entries among all processors).
      */
-    virtual typename VectorSpaceVector<Number>::real_type l2_norm() const;
+    virtual typename VectorSpaceVector<Number>::real_type l2_norm() const override;
 
     /**
      * Return the maximum norm of the vector (i.e., the maximum absolute value
      * among all entries and among all processors).
      */
-    virtual typename VectorSpaceVector<Number>::real_type linfty_norm() const;
+    virtual typename VectorSpaceVector<Number>::real_type linfty_norm() const override;
 
     /**
      * Perform a combined operation of a vector addition and a subsequent
@@ -213,21 +262,31 @@ namespace LinearAlgebra
      * this->add(a, V);
      * return_value = *this * W;
      * @endcode
+     *
+     * The reason this function exists is that this operation involves less
+     * memory transfer than calling the two functions separately. This method
+     * only needs to load three vectors, @p this, @p V, @p W, whereas calling
+     * separate methods means to load the calling vector @p this twice. Since
+     * most vector operations are memory transfer limited, this reduces the time
+     * by 25\% (or 50\% if @p W equals @p this).
+     *
+     * For complex-valued vectors, the scalar product in the second step is implemented as
+     * $\left<v,w\right>=\sum_i v_i \bar{w_i}$.
      */
     virtual Number add_and_dot(const Number a,
                                const VectorSpaceVector<Number> &V,
-                               const VectorSpaceVector<Number> &W);
+                               const VectorSpaceVector<Number> &W) override;
 
     /**
      * Return the global size of the vector, equal to the sum of the number of
      * locally owned indices among all processors.
      */
-    virtual size_type size() const;
+    virtual size_type size() const override;
 
     /**
      * Return an index set that describes which elements of this vector are
      * owned by the current processor. As a consequence, the index sets
-     * returned on different procesors if this is a distributed vector will
+     * returned on different processors if this is a distributed vector will
      * form disjoint sets that add up to the complete index set. Obviously, if
      * a vector is created on only one processor, then the result would
      * satisfy
@@ -235,7 +294,7 @@ namespace LinearAlgebra
      *  vec.locally_owned_elements() == complete_index_set(vec.size())
      * @endcode
      */
-    virtual dealii::IndexSet locally_owned_elements() const;
+    virtual dealii::IndexSet locally_owned_elements() const override;
 
     /**
      * Prints the vector to the output stream @p out.
@@ -243,7 +302,8 @@ namespace LinearAlgebra
     virtual void print(std::ostream &out,
                        const unsigned int precision=3,
                        const bool scientific=true,
-                       const bool across=true) const;
+                       const bool across=true) const override;
+
     /**
      * Write the vector en bloc to a file. This is done in a binary mode, so
      * the output is neither readable by humans nor (probably) by other
@@ -267,7 +327,7 @@ namespace LinearAlgebra
     /**
      * Return the memory consumption of this class in bytes.
      */
-    virtual std::size_t memory_consumption() const;
+    virtual std::size_t memory_consumption() const override;
 
     /**
      * Attempt to perform an operation between two incompatible vector types.
@@ -296,12 +356,6 @@ namespace LinearAlgebra
   /*@}*/
   /*----------------------- Inline functions ----------------------------------*/
 
-  template <typename Number>
-  inline
-  Vector<Number>::Vector() {}
-
-
-
   template <typename Number>
   inline
   Vector<Number>::Vector(const Vector<Number> &V)
@@ -372,7 +426,7 @@ namespace LinearAlgebra
     // If necessary, resize the vector during a read operation
     if (this->size()!=current_size)
       this->reinit(this->size());
-    ar &boost::serialization::make_array(this->val,this->size());
+    ar &boost::serialization::make_array(this->values.get(),this->size());
   }
 
 
@@ -392,7 +446,7 @@ namespace LinearAlgebra
  * @author Uwe Koecher, 2017
  */
 template <typename Number>
-struct is_serial_vector< LinearAlgebra::Vector< Number > > : std_cxx11::true_type
+struct is_serial_vector< LinearAlgebra::Vector< Number > > : std::true_type
 {
 };
 
