@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 2005 - 2017 by the deal.II authors
+// Copyright (C) 2005 - 2018 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -13,8 +13,8 @@
 //
 // ---------------------------------------------------------------------
 
-#ifndef dealii__lapack_full_matrix_h
-#define dealii__lapack_full_matrix_h
+#ifndef dealii_lapack_full_matrix_h
+#define dealii_lapack_full_matrix_h
 
 
 #include <deal.II/base/config.h>
@@ -22,18 +22,19 @@
 #include <deal.II/base/table.h>
 #include <deal.II/lac/lapack_support.h>
 #include <deal.II/lac/vector_memory.h>
+#include <deal.II/base/thread_management.h>
 
-#include <deal.II/base/std_cxx11/shared_ptr.h>
+#include <memory>
 #include <vector>
 #include <complex>
 
 DEAL_II_NAMESPACE_OPEN
 
 // forward declarations
-template<typename number> class Vector;
-template<typename number> class BlockVector;
-template<typename number> class FullMatrix;
-template<typename number> class SparseMatrix;
+template <typename number> class Vector;
+template <typename number> class BlockVector;
+template <typename number> class FullMatrix;
+template <typename number> class SparseMatrix;
 
 
 /**
@@ -46,7 +47,7 @@ template<typename number> class SparseMatrix;
  * usually the names chosen for the arguments in the LAPACK documentation.
  *
  * @ingroup Matrix1
- * @author Guido Kanschat, 2005
+ * @author Guido Kanschat, 2005, Denis Davydov, 2017, 2018
  */
 template <typename number>
 class LAPACKFullMatrix : public TransposeTable<number>
@@ -56,11 +57,11 @@ public:
   /**
    * Declare type for container size.
    */
-  typedef types::global_dof_index size_type;
+  typedef std::make_unsigned<types::blas_int>::type size_type;
 
   /**
    * Constructor. Initialize the matrix as a square matrix with dimension
-   * <tt>n</tt>.
+   * @p size.
    *
    * In order to avoid the implicit conversion of integers and other types to
    * a matrix, this constructor is declared <tt>explicit</tt>.
@@ -71,7 +72,7 @@ public:
 
 
   /**
-   * Constructor. Initialize the matrix as a rectangular matrix.
+   * Constructor. Initialize the matrix as a rectangular matrix $\rm{rows} \times \rm{cols}$.
    */
   LAPACKFullMatrix (const size_type rows,
                     const size_type cols);
@@ -83,7 +84,7 @@ public:
    * function arguments are passed by value rather than by reference.
    * Unfortunately, we can't mark this copy constructor <tt>explicit</tt>,
    * since that prevents the use of this class in containers, such as
-   * <tt>std::vector</tt>. The responsibility to check performance of programs
+   * <code>std::vector</code>. The responsibility to check performance of programs
    * must therefore remain with the user of this class.
    */
   LAPACKFullMatrix (const LAPACKFullMatrix &);
@@ -116,23 +117,58 @@ public:
 
   /**
    * This operator assigns a scalar to a matrix. To avoid confusion with
-   * constructors, zero is the only value allowed for <tt>d</tt>
+   * constructors, zero (when cast to the @p number type) is the only
+   * value allowed for @p d.
    */
   LAPACKFullMatrix<number> &
-  operator = (const double d);
+  operator = (const number d);
 
   /**
-   * This operator multiplies all entries by a fixed factor.
+   * This operator multiplies all entries by a fixed @p factor.
    */
   LAPACKFullMatrix<number> &
   operator*= (const number factor);
 
   /**
-   * This operator divides all entries by a fixed factor.
+   * This operator divides all entries by a fixed @p factor.
    */
   LAPACKFullMatrix<number> &
   operator/= (const number factor);
 
+  /**
+   * Set a particular entry of the matrix to a @p value.
+   * Thus, calling <code>A.set(1,2,3.141);</code> is entirely equivalent to the operation <code>A(1,2) = 3.141;</code>.
+   * This function exists for compatibility with the various sparse matrix objects.
+   *
+   * @param i The row index of the element to be set.
+   * @param j The column index of the element to be set.
+   * @param value The value to be written into the element.
+   */
+  void set (const size_type i,
+            const size_type j,
+            const number value);
+
+  /**
+   * Simple addition of a scaled matrix, i.e. $\mathbf A \mathrel{+}= a \, \mathbf B$.
+   */
+  void add (const number                    a,
+            const LAPACKFullMatrix<number> &B);
+
+  /**
+   * Perform a rank-1 update of a symmetric matrix
+   * $ \mathbf A \leftarrow \mathbf A + a \, \mathbf v \mathbf v^T $.
+   *
+   * This function also works for Cholesky factorization.
+   * In that case, updating ($a>0$) is
+   * performed via Givens rotations, whereas downdating ($a<0$) via hyperbolic
+   * rotations. Note that the latter case might lead to a negative definite
+   * matrix in which case the error will be thrown (because Cholesky
+   * factorizations are only valid for symmetric and positive definite
+   * matrices).
+   */
+  void rank1_update(const number a,
+                    const Vector<number> &v);
+
   /**
    * Assignment from different matrix classes, performing the usual conversion
    * to the transposed format expected by LAPACK. This assignment operator
@@ -149,6 +185,54 @@ public:
    */
   void reinit (const size_type size);
 
+  /**
+   * Same as above but will preserve the values of matrix upon resizing.
+   * The original values
+   * of the matrix are kept on increasing the size
+   * \f[
+   * \mathbf A \rightarrow
+   * \left(
+   * \begin{array}{cc}
+   * \mathbf A & \mathbf 0 \\
+   * \mathbf 0 & \mathbf 0
+   * \end{array}
+   * \right)
+   * \f]
+   * Whereas if the new size is smaller, the matrix will contain the upper left block
+   * of the original one
+   * \f[
+   * \left(
+   * \begin{array}{cc}
+   * \mathbf A_{11} & \mathbf A_{12} \\
+   * \mathbf A_{21} & \mathbf A_{22}
+   * \end{array}
+   * \right)
+   * \rightarrow \mathbf A_{11}
+   * \f]
+   */
+  void grow_or_shrink (const size_type size);
+
+  /**
+   * Remove row @p row and column @p col from the matrix.
+   * \f[
+   * \left(
+   * \begin{array}{ccc}
+   * \mathbf A_{11}   & \mathbf a_{12} & \mathbf A_{13}   \\
+   * \mathbf a_{21}^T & a_{22}         & \mathbf a_{23}^T \\
+   * \mathbf A_{31}   & \mathbf a_{32} & \mathbf A_{33}
+   * \end{array}
+   * \right)
+   * \rightarrow
+   * \left(
+   * \begin{array}{cc}
+   * \mathbf A_{11} & \mathbf A_{13} \\
+   * \mathbf A_{31} & \mathbf A_{33}
+   * \end{array}
+   * \right)
+   * \f]
+   */
+  void remove_row_and_column (const size_type row, const size_type col);
+
   /**
    * Regenerate the current matrix by one that has the same properties as if
    * it were created by the constructor of this class with the same argument
@@ -157,19 +241,24 @@ public:
   void reinit (const size_type rows,
                const size_type cols);
 
+  /**
+   * Assign @p property to this matrix.
+   */
+  void set_property(const LAPACKSupport::Property property);
+
   /**
    * Return the dimension of the codomain (or range) space.
    *
    * @note The matrix is of dimension $m \times n$.
    */
-  unsigned int m () const;
+  size_type m () const;
 
   /**
    * Return the dimension of the domain space.
    *
    * @note The matrix is of dimension $m \times n$.
    */
-  unsigned int n () const;
+  size_type n () const;
 
   /**
    * Fill rectangular block.
@@ -184,7 +273,7 @@ public:
    * The final two arguments allow to enter a multiple of the source or its
    * transpose.
    */
-  template<typename MatrixType>
+  template <typename MatrixType>
   void fill (const MatrixType &src,
              const size_type dst_offset_i = 0,
              const size_type dst_offset_j = 0,
@@ -208,12 +297,11 @@ public:
    * finally with the left transformation matrix.
    * </ul>
    *
-   * The optional parameter <tt>adding</tt> determines, whether the result is
-   * stored in <tt>w</tt> or added to <tt>w</tt>.
-   *
-   * if (adding) <i>w += A*v</i>
-   *
-   * if (!adding) <i>w = A*v</i>
+   * The optional parameter @p adding determines, whether the result is
+   * stored in the vector
+   * $\mathbf w = \mathbf A \cdot \mathbf v$
+   * or added to it
+   * $\mathbf w \mathrel{+}= \mathbf A \cdot \mathbf v$.
    *
    * @note Source and destination must not be the same vector.
    *
@@ -235,7 +323,7 @@ public:
               const bool            adding = false) const;
 
   /**
-   * Adding Matrix-vector-multiplication.  <i>w += A*v</i>
+   * Adding Matrix-vector-multiplication $\mathbf w \mathrel{+}= \mathbf A \cdot \mathbf v$.
    *
    * See the documentation of vmult() for details on the implementation.
    */
@@ -252,12 +340,11 @@ public:
   /**
    * Transpose matrix-vector-multiplication.
    *
-   * The optional parameter <tt>adding</tt> determines, whether the result is
-   * stored in <tt>w</tt> or added to <tt>w</tt>.
-   *
-   * if (adding) <i>w += A<sup>T</sup>*v</i>
-   *
-   * if (!adding) <i>w = A<sup>T</sup>*v</i>
+   * The optional parameter @p adding determines, whether the result is
+   * stored in the vector
+   * $\mathbf w = \mathbf A^T \cdot \mathbf v$
+   * or added to it
+   * $\mathbf w \mathrel{+}= \mathbf A^T \cdot \mathbf v$.
    *
    * See the documentation of vmult() for details on the implementation.
    */
@@ -274,8 +361,7 @@ public:
                const bool            adding=false) const;
 
   /**
-   * Adding transpose matrix-vector-multiplication. <i>w +=
-   * A<sup>T</sup>*v</i>
+   * Adding transpose matrix-vector-multiplication $\mathbf w \mathrel{+}= \mathbf A^T \cdot \mathbf v$.
    *
    * See the documentation of vmult() for details on the implementation.
    */
@@ -293,17 +379,16 @@ public:
   /**
    * Matrix-matrix-multiplication.
    *
-   * The optional parameter <tt>adding</tt> determines, whether the result is
-   * stored in <tt>C</tt> or added to <tt>C</tt>.
+   * The optional parameter @p adding determines, whether the result is
+   * stored in the matrix
+   * $\mathbf C            = \mathbf A \cdot \mathbf B$
+   * or added to it
+   * $\mathbf C \mathrel{+}= \mathbf A \cdot \mathbf B$.
    *
-   * if (adding) <i>C += A*B</i>
+   * @note It is assumed that @p A and @p B have compatible sizes and that
+   * @p C already has the right size.
    *
-   * if (!adding) <i>C = A*B</i>
-   *
-   * Assumes that <tt>A</tt> and <tt>B</tt> have compatible sizes and that
-   * <tt>C</tt> already has the right size.
-   *
-   * This function uses the BLAS function Xgemm.
+   * @p This function uses the BLAS function Xgemm.
    */
   void mmult (LAPACKFullMatrix<number>       &C,
               const LAPACKFullMatrix<number> &B,
@@ -320,17 +405,16 @@ public:
   /**
    * Matrix-matrix-multiplication using transpose of <tt>this</tt>.
    *
-   * The optional parameter <tt>adding</tt> determines, whether the result is
-   * stored in <tt>C</tt> or added to <tt>C</tt>.
+   * The optional parameter @p adding determines, whether the result is
+   * stored in the matrix
+   * $\mathbf C = \mathbf A^T \cdot \mathbf B$
+   * or added to it
+   * $\mathbf C \mathrel{+}= \mathbf A^T \cdot \mathbf B$.
    *
-   * if (adding) <i>C += A<sup>T</sup>*B</i>
+   * @note It is assumed that @p A and @p B have compatible sizes and that
+   * @p C already has the right size.
    *
-   * if (!adding) <i>C = A<sup>T</sup>*B</i>
-   *
-   * Assumes that <tt>A</tt> and <tt>B</tt> have compatible sizes and that
-   * <tt>C</tt> already has the right size.
-   *
-   * This function uses the BLAS function Xgemm.
+   * @note This function uses the BLAS function Xgemm.
    */
   void Tmmult (LAPACKFullMatrix<number>       &C,
                const LAPACKFullMatrix<number> &B,
@@ -345,19 +429,37 @@ public:
                const bool                      adding=false) const;
 
   /**
-   * Matrix-matrix-multiplication using transpose of <tt>B</tt>.
+   * Matrix-matrix-multiplication using transpose of <tt>this</tt> and a
+   * diagonal vector @p V.
    *
-   * The optional parameter <tt>adding</tt> determines, whether the result is
-   * stored in <tt>C</tt> or added to <tt>C</tt>.
+   * If the <code>adding=false</code> then the result is stored in the matrix
+   * $\mathbf C = \mathbf A^T \cdot \rm{diag}(\mathbf V) \cdot \mathbf B$
+   * otherwise it is added $\mathbf C \mathrel{+}= \mathbf A^T \cdot \rm{diag}(\mathbf V) \cdot \mathbf B$.
    *
-   * if (adding) <i>C += A*B<sup>T</sup></i>
+   * @note It is assumed that @p A, @p B and @p V have compatible sizes and that
+   * @p C already has the right size.
    *
-   * if (!adding) <i>C = A*B<sup>T</sup></i>
+   * @note This function is not provided by LAPACK. The function first forms $\rm{diag}(\mathbf V) \cdot \mathbf B$ product and
+   * then uses Xgemm function.
+   */
+  void Tmmult (LAPACKFullMatrix<number>       &C,
+               const LAPACKFullMatrix<number> &B,
+               const Vector<number>           &V,
+               const bool                      adding=false) const;
+
+  /**
+   * Matrix-matrix-multiplication using transpose of @p B.
    *
-   * Assumes that <tt>A</tt> and <tt>B</tt> have compatible sizes and that
-   * <tt>C</tt> already has the right size.
+   * The optional parameter @p adding determines, whether the result is
+   * stored in the matrix
+   * $\mathbf C = \mathbf A \cdot \mathbf B^T$
+   * or added to it
+   * $\mathbf C \mathrel{+}= \mathbf A \cdot \mathbf B^T$.
    *
-   * This function uses the BLAS function Xgemm.
+   * @note It is assumed that @p A and @p B have compatible sizes and that
+   * @p C already has the right size.
+   *
+   * @note This function uses the BLAS function Xgemm.
    */
   void mTmult (LAPACKFullMatrix<number>       &C,
                const LAPACKFullMatrix<number> &B,
@@ -373,19 +475,18 @@ public:
 
   /**
    * Matrix-matrix-multiplication using transpose of <tt>this</tt> and
-   * <tt>B</tt>.
+   * @p B.
    *
-   * The optional parameter <tt>adding</tt> determines, whether the result is
-   * stored in <tt>C</tt> or added to <tt>C</tt>.
+   * The optional parameter @p adding determines, whether the result is
+   * stored in the matrix
+   * $\mathbf C = \mathbf A^T \cdot \mathbf B^T$
+   * or added to it
+   * $\mathbf C \mathrel{+}= \mathbf A^T \cdot \mathbf B^T$.
    *
-   * if (adding) <i>C += A<sup>T</sup>*B<sup>T</sup></i>
+   * @note It is assumed that @p A and @p B have compatible sizes and that
+   * @p C already has the right size.
    *
-   * if (!adding) <i>C = A<sup>T</sup>*B<sup>T</sup></i>
-   *
-   * Assumes that <tt>A</tt> and <tt>B</tt> have compatible sizes and that
-   * <tt>C</tt> already has the right size.
-   *
-   * This function uses the BLAS function Xgemm.
+   * @note This function uses the BLAS function Xgemm.
    */
   void TmTmult (LAPACKFullMatrix<number>       &C,
                 const LAPACKFullMatrix<number> &B,
@@ -399,17 +500,104 @@ public:
                 const LAPACKFullMatrix<number> &B,
                 const bool                      adding=false) const;
 
+  /**
+   * Scale rows of this matrix by @p V . This is equivalent to premultiplication
+   * with a diagonal matrix $\mathbf A\leftarrow {\rm diag}(\mathbf V)\mathbf A$.
+   */
+  void scale_rows(const Vector<number> &V);
+
   /**
    * Compute the LU factorization of the matrix using LAPACK function Xgetrf.
    */
   void compute_lu_factorization ();
 
   /**
-   * Invert the matrix by first computing an LU factorization with the LAPACK
-   * function Xgetrf and then building the actual inverse using Xgetri.
+   * Compute the Cholesky factorization of the matrix using LAPACK function Xpotrf.
+   *
+   * @note The factorization is stored in the lower-triangular part of the matrix.
+   */
+  void compute_cholesky_factorization ();
+
+  /**
+   * Estimate the reciprocal of the condition number $1/k(\mathbf A)$ in $L_1$ norm ($1/(||\mathbf A||_1 \, ||\mathbf A^{-1}||_1)$)
+   * of a symmetric positive definite matrix using Cholesky factorization. This function can only
+   * be called if the matrix is already factorized.
+   *
+   * @note The condition number $k(\mathbf A)$ can be used to estimate the numerical
+   * error related to the matrix inversion or the solution of the
+   * system of linear algebraic equations as
+   * <code>error = std::numeric_limits<Number>::epsilon * k</code>.
+   * Alternatively one can get the number of accurate digits
+   * <code>std::floor(std::log10(k))</code>.
+   *
+   * @note The function computes reciprocal of the condition number to
+   * avoid possible overflow if the matrix is nearly singular.
+   *
+   * @param[in] l1_norm Is the $L_1$ norm of the matrix before calling Cholesky
+   * factorization. It can be obtained by calling l1_norm().
+   */
+  number reciprocal_condition_number(const number l1_norm) const;
+
+  /**
+   * Estimate the reciprocal of the condition number $1/k(\mathbf A)$ in $L_1$ norm
+   * for triangular matrices. The matrix has to have
+   * the LAPACKSupport::Property set to either LAPACKSupport::Property::upper_triangular
+   * or LAPACKSupport::Property::lower_triangular, see set_property().
+   */
+  number reciprocal_condition_number() const;
+
+  /**
+   * Compute the determinant of a matrix. As it requires the LU factorization of
+   * the matrix, this function can only be called after
+   * compute_lu_factorization() has been called.
+   */
+  number determinant () const;
+
+  /**
+   * Compute $L_1$ norm.
+   */
+  number l1_norm() const;
+
+  /**
+   * Compute $L_\infty$ norm.
+   */
+  number linfty_norm() const;
+
+  /**
+   * Compute Frobenius norm
+   */
+  number frobenius_norm() const;
+
+  /**
+   * Compute trace of the matrix, i.e. the sum of the diagonal values.
+   * Obviously, the matrix needs to be quadratic for this function.
+   */
+  number trace() const;
+
+  /**
+   * Invert the matrix by first computing an LU/Cholesky factorization with the LAPACK
+   * function Xgetrf/Xpotrf and then building the actual inverse using Xgetri/Xpotri.
    */
   void invert ();
 
+  /**
+   * Solve the linear system with right hand side @p v and put the solution
+   * back to @p v. The matrix should be either triangular or LU/Cholesky
+   * factorization should be previously computed.
+   *
+   * The flag transposed indicates whether the solution of the transposed
+   * system is to be performed.
+   */
+  void solve(Vector<number> &v,
+             const bool transposed = false) const;
+
+  /**
+   * Same as above but for multiple right hand sides (as many as there
+   * are columns in the matrix @p B).
+   */
+  void solve(LAPACKFullMatrix<number> &B,
+             const bool transposed = false) const;
+
   /**
    * Solve the linear system with right hand side given by applying
    * forward/backward substitution to the previously computed LU
@@ -417,7 +605,10 @@ public:
    *
    * The flag transposed indicates whether the solution of the transposed
    * system is to be performed.
+   *
+   * @deprecated use solve() instead.
    */
+  DEAL_II_DEPRECATED
   void apply_lu_factorization (Vector<number> &v,
                                const bool      transposed) const;
 
@@ -429,7 +620,10 @@ public:
    *
    * The flag transposed indicates whether the solution of the transposed
    * system is to be performed.
+   *
+   * @deprecated use solve() instead.
    */
+  DEAL_II_DEPRECATED
   void apply_lu_factorization (LAPACKFullMatrix<number> &B,
                                const bool                transposed) const;
 
@@ -456,21 +650,20 @@ public:
 
   /**
    * Compute eigenvalues and eigenvectors of a real symmetric matrix. Only
-   * eigenvalues in the interval (lower_bound, upper_bound] are computed with
-   * the absolute tolerance abs_accuracy. An approximate eigenvalue is
-   * accepted as converged when it is determined to lie in an interval [a,b]
-   * of width less than or equal to abs_accuracy + eps * max( |a|,|b| ), where
-   * eps is the machine precision.  If abs_accuracy is less than or equal to
-   * zero, then eps*|t| will be used in its place, where |t| is the 1-norm of
-   * the tridiagonal matrix obtained by reducing A to tridiagonal form.
-   * Eigenvalues will be computed most accurately when abs_accuracy is set to
+   * eigenvalues in the interval $(\rm{lower\_bound}, \rm{upper\_bound}]$ are computed with
+   * the absolute tolerance $\rm abs\_accuracy$. An approximate eigenvalue is
+   * accepted as converged when it is determined to lie in an interval $[a,b]$
+   * of width less than or equal to $\rm{abs\_accuracy} + eps * \rm{max}(|a|,|b|)$, where
+   * $eps$ is the machine precision.  If $\rm{abs\_accuracy}$ is less than or equal to
+   * zero, then $eps\,|\mathbf{T}|_1$ will be used in its place, where $|\mathbf{T}|_1$ is the 1-norm of
+   * the tridiagonal matrix obtained by reducing $\mathbf A$ to tridiagonal form.
+   * Eigenvalues will be computed most accurately when $\rm{abs\_accuracy}$ is set to
    * twice the underflow threshold, not zero.  After this routine has been
-   * called, all eigenvalues in (lower_bound, upper_bound] will be stored in
+   * called, all eigenvalues in $(\rm{lower\_bound}, \rm{upper\_bound}]$ will be stored in
    * eigenvalues and the corresponding eigenvectors will be stored in the
    * columns of eigenvectors, whose dimension is set accordingly.
    *
-   * @note Calls the LAPACK function Xsyevx. For this to work, deal.II must be
-   * configured to use LAPACK.
+   * @note Calls the LAPACK function Xsyevx.
    */
   void compute_eigenvalues_symmetric (const number        lower_bound,
                                       const number        upper_bound,
@@ -480,24 +673,27 @@ public:
 
   /**
    * Compute generalized eigenvalues and eigenvectors of a real generalized
-   * symmetric eigenproblem of the form itype = 1: $Ax=\lambda B x$ itype = 2:
-   * $ABx=\lambda x$ itype = 3: $BAx=\lambda x$, where A is this matrix.  A
-   * and B are assumed to be symmetric, and B has to be positive definite.
-   * Only eigenvalues in the interval (lower_bound, upper_bound] are computed
-   * with the absolute tolerance abs_accuracy.  An approximate eigenvalue is
-   * accepted as converged when it is determined to lie in an interval [a,b]
-   * of width less than or equal to abs_accuracy + eps * max( |a|,|b| ), where
-   * eps is the machine precision.  If abs_accuracy is less than or equal to
-   * zero, then eps*|t| will be used in its place, where |t| is the 1-norm of
-   * the tridiagonal matrix obtained by reducing A to tridiagonal form.
-   * Eigenvalues will be computed most accurately when abs_accuracy is set to
-   * twice the underflow threshold, not zero.  After this routine has been
-   * called, all eigenvalues in (lower_bound, upper_bound] will be stored in
+   * symmetric eigenproblem of the form
+   * - itype = 1: $\mathbf A \cdot \mathbf x=\lambda \mathbf B \cdot \mathbf x$
+   * - itype = 2: $\mathbf A \cdot \mathbf B \cdot \mathbf x=\lambda \mathbf x$
+   * - itype = 3: $\mathbf B \cdot \mathbf A \cdot \mathbf x=\lambda \mathbf x$
+   *
+   * where $\mathbf A$ is this matrix. $\mathbf A$
+   * and $\mathbf B$ are assumed to be symmetric, and $\mathbf B$ has to be positive definite.
+   * Only eigenvalues in the interval $(\rm{lower\_bound}, \rm{upper\_bound}]$ are computed
+   * with the absolute tolerance $\rm{abs\_accuracy}$.  An approximate eigenvalue is
+   * accepted as converged when it is determined to lie in an interval $[a,b]$
+   * of width less than or equal to $\rm{abs\_accuracy} + eps * \rm{max}( |a|,|b| )$, where
+   * $eps$ is the machine precision. If $\rm{abs\_accuracy}$ is less than or equal to
+   * zero, then $eps \, |\mathbf{T}|_1$ will be used in its place, where $|\mathbf{T}|_1$ is the 1-norm of
+   * the tridiagonal matrix obtained by reducing $\mathbf A$ to tridiagonal form.
+   * Eigenvalues will be computed most accurately when $\rm{abs\_accuracy}$ is set to
+   * twice the underflow threshold, not zero. After this routine has been
+   * called, all eigenvalues in $(\rm{lower\_bound}, \rm{upper\_bound}]$ will be stored in
    * eigenvalues and the corresponding eigenvectors will be stored in
    * eigenvectors, whose dimension is set accordingly.
    *
-   * @note Calls the LAPACK function Xsygvx. For this to work, deal.II must be
-   * configured to use LAPACK.
+   * @note Calls the LAPACK function Xsygvx.
    */
   void compute_generalized_eigenvalues_symmetric (LAPACKFullMatrix<number>     &B,
                                                   const number                  lower_bound,
@@ -505,7 +701,7 @@ public:
                                                   const number                  abs_accuracy,
                                                   Vector<number>               &eigenvalues,
                                                   std::vector<Vector<number> > &eigenvectors,
-                                                  const int                     itype = 1);
+                                                  const types::blas_int             itype = 1);
 
   /**
    * Same as the other compute_generalized_eigenvalues_symmetric function
@@ -520,12 +716,11 @@ public:
    * be retrieved using the eigenvalue() function.  The number of computed
    * eigenvectors is equal to eigenvectors.size()
    *
-   * @note Calls the LAPACK function Xsygv. For this to work, deal.II must be
-   * configured to use LAPACK.
+   * @note Calls the LAPACK function Xsygv.
    */
   void compute_generalized_eigenvalues_symmetric (LAPACKFullMatrix<number>     &B,
                                                   std::vector<Vector<number> > &eigenvectors,
-                                                  const int                     itype = 1);
+                                                  const types::blas_int             itype = 1);
 
   /**
    * Compute the singular value decomposition of the matrix using LAPACK
@@ -550,14 +745,20 @@ public:
    * does not have maximal rank, singular values 0 are not touched, thus
    * computing the minimal norm right inverse of the matrix.
    *
-   * The parameter <tt>threshold</tt> determines, when a singular value should
+   * The parameter @p threshold determines, when a singular value should
    * be considered zero. It is the ratio of the smallest to the largest
-   * nonzero singular value <i>s</i><sub>max</sub>. Thus, the inverses of all
-   * singular values less than <i>s</i><sub>max</sub>/<tt>threshold</tt> will
+   * nonzero singular value $s_{max}$. Thus, the inverses of all
+   * singular values less than  $s_{max}/\rm{threshold}$ will
    * be set to zero.
    */
   void compute_inverse_svd (const double threshold = 0.);
 
+  /**
+   * Same as above but provide the size of the kernel instead of a threshold,
+   * i.e. the @p kernel_size smallest eigenvalues.
+   */
+  void compute_inverse_svd_with_kernel (const unsigned int kernel_size);
+
   /**
    * Retrieve eigenvalue after compute_eigenvalues() was called.
    */
@@ -576,21 +777,21 @@ public:
    *
    * The parameters allow for a flexible setting of the output format:
    *
-   * @arg <tt>precision</tt> denotes the number of trailing digits.
+   * @param precision denotes the number of trailing digits.
    *
-   * @arg <tt>scientific</tt> is used to determine the number format, where
-   * <tt>scientific</tt> = <tt>false</tt> means fixed point notation.
+   * @param scientific is used to determine the number format, where
+   * <code>scientific = false</code> means fixed point notation.
    *
-   * @arg <tt>width</tt> denotes the with of each column. A zero entry for
-   * <tt>width</tt> makes the function compute a width, but it may be changed
+   * @param width denotes the with of each column. A zero entry for
+   * @p width makes the function compute a width, but it may be changed
    * to a positive value, if output is crude.
    *
-   * @arg <tt>zero_string</tt> specifies a string printed for zero entries.
+   * @param zero_string specifies a string printed for zero entries.
    *
-   * @arg <tt>denominator</tt> Multiply the whole matrix by this common
+   * @param denominator Multiply the whole matrix by this common
    * denominator to get nicer numbers.
    *
-   * @arg <tt>threshold</tt>: all entries with absolute value smaller than
+   * @param threshold all entries with absolute value smaller than
    * this are considered zero.
    *
    * @note The entries stored resemble a matrix only if the state is either
@@ -606,6 +807,10 @@ public:
                         const double        threshold   = 0.) const;
 
 private:
+  /**
+   * Internal function to compute various norms.
+   */
+  number norm(const char type) const;
 
   /**
    * Since LAPACK operations notoriously change the meaning of the matrix
@@ -614,23 +819,28 @@ private:
   LAPACKSupport::State state;
 
   /**
-   * Additional properties of the matrix which may help to select more
+   * Additional property of the matrix which may help to select more
    * efficient LAPACK functions.
    */
-  LAPACKSupport::Properties properties;
+  LAPACKSupport::Property property;
 
   /**
    * The working array used for some LAPACK functions.
    */
   mutable std::vector<number> work;
 
+  /**
+   * Integer working array used for some LAPACK functions.
+   */
+  mutable std::vector<types::blas_int> iwork;
+
   /**
    * The vector storing the permutations applied for pivoting in the LU-
    * factorization.
    *
    * Also used as the scratch array IWORK for LAPACK functions needing it.
    */
-  std::vector<int> ipiv;
+  std::vector<types::blas_int> ipiv;
 
   /**
    * Workspace for calculating the inverse matrix from an LU factorization.
@@ -659,16 +869,21 @@ private:
   std::vector<number> vr;
 
   /**
-   * The matrix <i>U</i> in the singular value decomposition
-   * <i>USV<sup>T</sup></i>.
+   * The matrix $\mathbf U$ in the singular value decomposition
+   * $\mathbf U \cdot \mathbf S \cdot \mathbf V^T$.
    */
-  std_cxx11::shared_ptr<LAPACKFullMatrix<number> > svd_u;
+  std::unique_ptr<LAPACKFullMatrix<number> > svd_u;
 
   /**
-   * The matrix <i>V<sup>T</sup></i> in the singular value decomposition
-   * <i>USV<sup>T</sup></i>.
+   * The matrix $\mathbf V^T$  in the singular value decomposition
+   * $\mathbf U \cdot \mathbf S \cdot \mathbf V^T$.
    */
-  std_cxx11::shared_ptr<LAPACKFullMatrix<number> > svd_vt;
+  std::unique_ptr<LAPACKFullMatrix<number> > svd_vt;
+
+  /**
+   * Thread mutex.
+   */
+  mutable Threads::Mutex mutex;
 };
 
 
@@ -703,18 +918,28 @@ private:
 
 template <typename number>
 inline
-unsigned int
+void LAPACKFullMatrix<number>::set (const size_type i,
+                                    const size_type j,
+                                    const number value)
+{
+  (*this)(i,j) = value;
+}
+
+
+template <typename number>
+inline
+typename LAPACKFullMatrix<number>::size_type
 LAPACKFullMatrix<number>::m () const
 {
-  return this->n_rows ();
+  return static_cast<size_type>(this->n_rows ());
 }
 
 template <typename number>
 inline
-unsigned int
+typename LAPACKFullMatrix<number>::size_type
 LAPACKFullMatrix<number>::n () const
 {
-  return this->n_cols ();
+  return static_cast<size_type>(this->n_cols ());
 }
 
 template <typename number>
@@ -830,7 +1055,7 @@ LAPACKFullMatrix<number>::eigenvalue (const size_type i) const
   Assert (state & LAPACKSupport::eigenvalues, ExcInvalidState());
   Assert (wr.size() == this->n_rows(), ExcInternalError());
   Assert (wi.size() == this->n_rows(), ExcInternalError());
-  Assert (i<this->n_rows(), ExcIndexRange(i,0,this->n_rows()));
+  AssertIndexRange (i,this->n_rows());
 
   return std::complex<number>(wr[i], wi[i]);
 }
