@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 2014 - 2017 by the deal.II authors
+// Copyright (C) 2014 - 2018 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -13,15 +13,13 @@
 //
 // ---------------------------------------------------------------------
 
-#ifndef dealii__linear_operator_h
-#define dealii__linear_operator_h
+#ifndef dealii_linear_operator_h
+#define dealii_linear_operator_h
 
 #include <deal.II/base/config.h>
 #include <deal.II/base/exceptions.h>
 #include <deal.II/lac/vector_memory.h>
 
-#ifdef DEAL_II_WITH_CXX11
-
 #include <array>
 #include <functional>
 #include <type_traits>
@@ -32,7 +30,7 @@ DEAL_II_NAMESPACE_OPEN
 
 namespace internal
 {
-  namespace LinearOperator
+  namespace LinearOperatorImplementation
   {
     class EmptyPayload;
   }
@@ -42,12 +40,12 @@ template <typename Number> class Vector;
 
 template <typename Range = Vector<double>,
           typename Domain = Range,
-          typename Payload = internal::LinearOperator::EmptyPayload>
+          typename Payload = internal::LinearOperatorImplementation::EmptyPayload>
 class LinearOperator;
 
 template <typename Range = Vector<double>,
           typename Domain = Range,
-          typename Payload = internal::LinearOperator::EmptyPayload,
+          typename Payload = internal::LinearOperatorImplementation::EmptyPayload,
           typename OperatorExemplar,
           typename Matrix>
 LinearOperator<Range, Domain, Payload> linear_operator (const OperatorExemplar &,
@@ -55,13 +53,13 @@ LinearOperator<Range, Domain, Payload> linear_operator (const OperatorExemplar &
 
 template <typename Range = Vector<double>,
           typename Domain = Range,
-          typename Payload = internal::LinearOperator::EmptyPayload,
+          typename Payload = internal::LinearOperatorImplementation::EmptyPayload,
           typename Matrix>
 LinearOperator<Range, Domain, Payload> linear_operator (const Matrix &);
 
 template <typename Range = Vector<double>,
           typename Domain = Range,
-          typename Payload = internal::LinearOperator::EmptyPayload>
+          typename Payload = internal::LinearOperatorImplementation::EmptyPayload>
 LinearOperator<Range, Domain, Payload>
 null_operator(const LinearOperator<Range, Domain, Payload> &);
 
@@ -121,12 +119,12 @@ null_operator(const LinearOperator<Range, Domain, Payload> &);
  * in conjunction with the LinearOperator class, it is necessary to extend the
  * functionality of the LinearOperator class by means of an additional Payload.
  *
- * For example, in order to construct an InverseOperator a call to a solver is
- * required. Naturally these solvers don't have an interface to the
- * LinearOperator (which, for example, may represent a composite operation).
- * The TrilinosWrappers::internal::LinearOperator::TrilinosPayload therefore
- * provides an interface extension to the LinearOperator so that it can be
- * passed to the solver and used by the solver as if it were a Trilinos
+ * For example: LinearOperator instances representing matrix inverses usually
+ * require calling some linear solver. These solvers may not have interfaces
+ * to the LinearOperator (which, for example, may represent a composite
+ * operation). The TrilinosWrappers::internal::LinearOperatorImplementation::TrilinosPayload
+ * therefore provides an interface extension to the LinearOperator so that it
+ * can be passed to the solver and used by the solver as if it were a Trilinos
  * operator. This implies that all of the necessary functionality of the
  * specific Trilinos operator has been overloaded within the Payload class.
  * This includes operator-vector multiplication and inverse operator-vector
@@ -142,11 +140,7 @@ null_operator(const LinearOperator<Range, Domain, Payload> &);
  *
  * @note To ensure that the correct payload is provided, wrapper functions
  * for linear operators have been provided within the respective
- * TrilinosWrappers (and, in the future, PetscWrappers) namespaces.
- *
- * @note This class is only available if deal.II was configured with C++11
- * support, i.e., if <code>DEAL_II_WITH_CXX11</code> is enabled during cmake
- * configure.
+ * TrilinosWrappers (and, in the future, PETScWrappers) namespaces.
  *
  * @author Luca Heltai, Matthias Maier, 2015; Jean-Paul Pelteret, 2016
  *
@@ -219,8 +213,8 @@ public:
    * object @p op for which the conversion function
    * <code>linear_operator</code> is defined.
    */
-  template<typename Op,
-           typename = typename std::enable_if<!std::is_base_of<LinearOperator<Range, Domain, Payload>, Op>::value>::type>
+  template <typename Op,
+            typename = typename std::enable_if<!std::is_base_of<LinearOperator<Range, Domain, Payload>, Op>::value>::type>
   LinearOperator (const Op &op)
   {
     *this = linear_operator<Range, Domain, Payload, Op>(op);
@@ -272,7 +266,7 @@ public:
    * Initializes a vector v of the Range space to be directly usable as the
    * destination parameter in an application of vmult. Similar to the reinit
    * functions of the vector classes, the boolean determines whether a fast
-   * initalization is done, i.e., if it is set to false the content of the
+   * initialization is done, i.e., if it is set to false the content of the
    * vector is set to 0.
    */
   std::function<void(Range &v, bool omit_zeroing_entries)> reinit_range_vector;
@@ -281,7 +275,7 @@ public:
    * Initializes a vector of the Domain space to be directly usable as the
    * source parameter in an application of vmult. Similar to the reinit
    * functions of the vector classes, the boolean determines whether a fast
-   * initalization is done, i.e., if it is set to false the content of the
+   * initialization is done, i.e., if it is set to false the content of the
    * vector is set to 0.
    */
   std::function<void(Domain &v, bool omit_zeroing_entries)> reinit_domain_vector;
@@ -352,7 +346,7 @@ public:
 //@{
 
 /**
- * @relates LinearOperator
+ * @relatesalso LinearOperator
  *
  * Addition of two linear operators @p first_op and @p second_op given by
  * $(\text{first\_op}+\text{second\_op})x := \text{first\_op}(x) +
@@ -415,7 +409,7 @@ operator+(const LinearOperator<Range, Domain, Payload> &first_op,
 
 
 /**
- * @relates LinearOperator
+ * @relatesalso LinearOperator
  *
  * Subtraction of two linear operators @p first_op and @p second_op given by
  * $(\text{first\_op}-\text{second\_op})x := \text{first\_op}(x) -
@@ -445,7 +439,7 @@ operator-(const LinearOperator<Range, Domain, Payload> &first_op,
 
 
 /**
- * @relates LinearOperator
+ * @relatesalso LinearOperator
  *
  * Scalar multiplication of a ScalarOperator object @p op with @p number from
  * the left.
@@ -517,7 +511,7 @@ operator*(typename Range::value_type  number,
 
 
 /**
- * @relates LinearOperator
+ * @relatesalso LinearOperator
  *
  * Scalar multiplication of a ScalarOperator object from the right.
  *
@@ -552,7 +546,7 @@ operator*(const LinearOperator<Range, Domain, Payload> &op,
 //@{
 
 /**
- * @relates LinearOperator
+ * @relatesalso LinearOperator
  *
  * Composition of two linear operators @p first_op and @p second_op given by
  * $(\text{first\_op}*\text{second\_op})x :=
@@ -586,46 +580,42 @@ operator*(const LinearOperator<Range, Intermediate, Payload> &first_op,
 
       return_op.vmult = [first_op, second_op](Range &v, const Domain &u)
       {
-        static GrowingVectorMemory<Intermediate> vector_memory;
+        GrowingVectorMemory<Intermediate> vector_memory;
 
-        Intermediate *i = vector_memory.alloc();
+        typename VectorMemory<Intermediate>::Pointer i (vector_memory);
         second_op.reinit_range_vector(*i, /*bool omit_zeroing_entries =*/ true);
         second_op.vmult(*i, u);
         first_op.vmult(v, *i);
-        vector_memory.free(i);
       };
 
       return_op.vmult_add = [first_op, second_op](Range &v, const Domain &u)
       {
-        static GrowingVectorMemory<Intermediate> vector_memory;
+        GrowingVectorMemory<Intermediate> vector_memory;
 
-        Intermediate *i = vector_memory.alloc();
+        typename VectorMemory<Intermediate>::Pointer i (vector_memory);
         second_op.reinit_range_vector(*i, /*bool omit_zeroing_entries =*/ true);
         second_op.vmult(*i, u);
         first_op.vmult_add(v, *i);
-        vector_memory.free(i);
       };
 
       return_op.Tvmult = [first_op, second_op](Domain &v, const Range &u)
       {
-        static GrowingVectorMemory<Intermediate> vector_memory;
+        GrowingVectorMemory<Intermediate> vector_memory;
 
-        Intermediate *i = vector_memory.alloc();
+        typename VectorMemory<Intermediate>::Pointer i (vector_memory);
         first_op.reinit_domain_vector(*i, /*bool omit_zeroing_entries =*/ true);
         first_op.Tvmult(*i, u);
         second_op.Tvmult(v, *i);
-        vector_memory.free(i);
       };
 
       return_op.Tvmult_add = [first_op, second_op](Domain &v, const Range &u)
       {
-        static GrowingVectorMemory<Intermediate> vector_memory;
+        GrowingVectorMemory<Intermediate> vector_memory;
 
-        Intermediate *i = vector_memory.alloc();
+        typename VectorMemory<Intermediate>::Pointer i (vector_memory);
         first_op.reinit_domain_vector(*i, /*bool omit_zeroing_entries =*/ true);
         first_op.Tvmult(*i, u);
         second_op.Tvmult_add(v, *i);
-        vector_memory.free(i);
       };
 
       return return_op;
@@ -633,7 +623,7 @@ operator*(const LinearOperator<Range, Intermediate, Payload> &first_op,
 }
 
 /**
- * @relates LinearOperator
+ * @relatesalso LinearOperator
  *
  * Return the transpose linear operations of @p op.
  *
@@ -662,7 +652,7 @@ transpose_operator(const LinearOperator<Range, Domain, Payload> &op)
 
 
 /**
- * @relates LinearOperator
+ * @relatesalso LinearOperator
  *
  * Return an object representing the inverse of the LinearOperator @p op.
  *
@@ -705,13 +695,12 @@ inverse_operator(const LinearOperator<Range, Domain, Payload> &op,
   return_op.vmult_add =
     [op, &solver, &preconditioner](Range &v, const Domain &u)
   {
-    static GrowingVectorMemory<Range> vector_memory;
+    GrowingVectorMemory<Range> vector_memory;
 
-    Range *v2 = vector_memory.alloc();
+    typename VectorMemory<Range>::Pointer v2 (vector_memory);
     op.reinit_range_vector(*v2, /*bool omit_zeroing_entries =*/ false);
     solver.solve(op, *v2, u, preconditioner);
     v += *v2;
-    vector_memory.free(v2);
   };
 
   return_op.Tvmult = [op, &solver, &preconditioner](Range &v, const Domain &u)
@@ -723,13 +712,12 @@ inverse_operator(const LinearOperator<Range, Domain, Payload> &op,
   return_op.Tvmult_add =
     [op, &solver, &preconditioner](Range &v, const Domain &u)
   {
-    static GrowingVectorMemory<Range> vector_memory;
+    GrowingVectorMemory<Range> vector_memory;
 
-    Range *v2 = vector_memory.alloc();
+    typename VectorMemory<Range>::Pointer v2 (vector_memory);
     op.reinit_range_vector(*v2, /*bool omit_zeroing_entries =*/ false);
     solver.solve(transpose_operator(op), *v2, u, preconditioner);
     v += *v2;
-    vector_memory.free(v2);
   };
 
   return return_op;
@@ -744,7 +732,7 @@ inverse_operator(const LinearOperator<Range, Domain, Payload> &op,
 //@{
 
 /**
- * @relates LinearOperator
+ * @relatesalso LinearOperator
  *
  * Return a LinearOperator that is the identity of the vector space @p Range.
  *
@@ -755,7 +743,7 @@ inverse_operator(const LinearOperator<Range, Domain, Payload> &op,
  * @ingroup LAOperators
  */
 template <typename Range,
-          typename Payload = internal::LinearOperator::EmptyPayload>
+          typename Payload = internal::LinearOperatorImplementation::EmptyPayload>
 LinearOperator<Range, Range, Payload>
 identity_operator(const std::function<void(Range &, bool)> &reinit_vector)
 {
@@ -789,12 +777,12 @@ identity_operator(const std::function<void(Range &, bool)> &reinit_vector)
 
 
 /**
- * @relates LinearOperator
+ * @relatesalso LinearOperator
  *
  * Return a LinearOperator that is the identity of the vector space @p Range.
  *
  * The function takes a LinearOperator @p op and uses its range initializer
- * to create an identiy operator. In contrast to the function above, this
+ * to create an identity operator. In contrast to the function above, this
  * function also ensures that the underlying Payload matches that of the
  * input @p op.
  *
@@ -804,25 +792,15 @@ template <typename Range, typename Domain, typename Payload>
 LinearOperator<Range, Domain, Payload>
 identity_operator(const LinearOperator<Range, Domain, Payload> &op)
 {
-  const LinearOperator<Range, Domain, Payload> id_op
-    = identity_operator<Range,Payload>(op.reinit_range_vector);
-  LinearOperator<Range, Domain, Payload> return_op (
-    op.identity_payload()
-  );
-
-  return_op.reinit_range_vector = id_op.reinit_range_vector;
-  return_op.reinit_domain_vector = id_op.reinit_domain_vector;
-  return_op.vmult = id_op.vmult;
-  return_op.vmult_add = id_op.vmult_add;
-  return_op.Tvmult = id_op.Tvmult;
-  return_op.Tvmult_add = id_op.Tvmult_add;
+  auto return_op = identity_operator<Range, Payload>(op.reinit_range_vector);
+  static_cast<Payload &>(return_op) = op.identity_payload();
 
   return return_op;
 }
 
 
 /**
- * @relates LinearOperator
+ * @relatesalso LinearOperator
  *
  * Return a nulled variant of the LinearOperator @p op, i.e. with optimized
  * LinearOperator::vmult, LinearOperator::vmult_add, etc. functions and with
@@ -863,9 +841,77 @@ null_operator(const LinearOperator<Range, Domain, Payload> &op)
 }
 
 
+/**
+ * @relatesalso LinearOperator
+ *
+ * Return a LinearOperator that acts as a mean value filter. The vmult()
+ * functions of this matrix subtract the mean values of the vector.
+ *
+ * The function takes an <code>std::function</code> object @p reinit_vector as
+ * an argument to initialize the <code>reinit_range_vector</code> and
+ * <code>reinit_domain_vector</code> objects of the LinearOperator object.
+ *
+ * @ingroup LAOperators
+ */
+template <typename Range,
+          typename Payload = internal::LinearOperatorImplementation::EmptyPayload>
+LinearOperator<Range, Range, Payload>
+mean_value_filter(const std::function<void(Range &, bool)> &reinit_vector)
+{
+  LinearOperator<Range, Range, Payload> return_op ((Payload()));
+
+  return_op.reinit_range_vector = reinit_vector;
+  return_op.reinit_domain_vector = reinit_vector;
+
+  return_op.vmult = [](Range &v, const Range &u)
+  {
+    const auto mean = u.mean_value();
+
+    v = u;
+    v.add(-mean);
+  };
+
+  return_op.vmult_add = [](Range &v, const Range &u)
+  {
+    const auto mean = u.mean_value();
+
+    v += u;
+    v.add(-mean);
+  };
+
+  return_op.Tvmult = return_op.vmult_add;
+  return_op.Tvmult_add = return_op.vmult_add;
+
+  return return_op;
+}
+
+
+/**
+ * @relatesalso LinearOperator
+ *
+ * Return a LinearOperator that acts as a mean value filter. The vmult()
+ * functions of this matrix subtract the mean values of the vector.
+ *
+ * The function takes a LinearOperator @p op and uses its range initializer
+ * to create an mean value filter operator. The function also ensures that
+ * the underlying Payload matches that of the input @p op.
+ *
+ * @ingroup LAOperators
+ */
+template <typename Range, typename Domain, typename Payload>
+LinearOperator<Range, Domain, Payload>
+mean_value_filter(const LinearOperator<Range, Domain, Payload> &op)
+{
+  auto return_op = mean_value_filter<Range, Payload>(op.reinit_range_vector);
+  static_cast<Payload &>(return_op) = op.identity_payload();
+
+  return return_op;
+}
+
+
 namespace internal
 {
-  namespace LinearOperator
+  namespace LinearOperatorImplementation
   {
     /**
      * A helper class that is responsible for the initialization of a vector
@@ -878,7 +924,7 @@ namespace internal
      * This class is specialized for more complicated data structures, such as
      * TrilinosWrappers::MPI::Vector, etc.
      */
-    template<typename Vector>
+    template <typename Vector>
     class ReinitHelper
     {
     public:
@@ -886,7 +932,7 @@ namespace internal
        * Initializes a vector v of the Range space to be directly usable as
        * the destination parameter in an application of vmult. Similar to the
        * reinit functions of the vector classes, the boolean determines
-       * whether a fast initalization is done, i.e., if it is set to false the
+       * whether a fast initialization is done, i.e., if it is set to false the
        * content of the vector is set to 0.
        *
        * The generic version of this class just calls
@@ -904,7 +950,7 @@ namespace internal
        * Initializes a vector of the Domain space to be directly usable as the
        * source parameter in an application of vmult. Similar to the reinit
        * functions of the vector classes, the boolean determines whether a
-       * fast initalization is done, i.e., if it is set to false the content
+       * fast initialization is done, i.e., if it is set to false the content
        * of the vector is set to 0.
        *
        * The generic version of this class just calls
@@ -925,8 +971,8 @@ namespace internal
      * facilitate the operations of the matrix.
      *
      * This is the Payload class typically associated with deal.II's native
-     * SparseMatrix. To use Trilinos and PETSc SparseMatrices it is necessary
-     * to initialize a LinearOperator with their associated Payload.
+     * SparseMatrix. To use Trilinos and PETSc sparse matrix classes it is
+     * necessary to initialize a LinearOperator with their associated Payload.
      *
      * @author Jean-Paul Pelteret, Matthias Maier, 2016
      *
@@ -949,7 +995,7 @@ namespace internal
 
 
       /**
-       * Returns a payload configured for identity operations
+       * Return a payload configured for identity operations
        */
       EmptyPayload
       identity_payload () const
@@ -959,7 +1005,7 @@ namespace internal
 
 
       /**
-       * Returns a payload configured for null operations
+       * Return a payload configured for null operations
        */
       EmptyPayload
       null_payload () const
@@ -969,7 +1015,7 @@ namespace internal
 
 
       /**
-       * Returns a payload configured for transpose operations
+       * Return a payload configured for transpose operations
        */
       EmptyPayload
       transpose_payload () const
@@ -979,7 +1025,7 @@ namespace internal
 
 
       /**
-       * Returns a payload configured for inverse operations
+       * Return a payload configured for inverse operations
        */
       template <typename Solver, typename Preconditioner>
       EmptyPayload
@@ -1045,9 +1091,9 @@ namespace
   void apply_with_intermediate_storage(Function function, Range &v,
                                        const Domain &u, bool add)
   {
-    static GrowingVectorMemory<Range> vector_memory;
+    GrowingVectorMemory<Range> vector_memory;
 
-    Range *i = vector_memory.alloc();
+    typename VectorMemory<Range>::Pointer i (vector_memory);
     i->reinit(v, /*bool omit_zeroing_entries =*/true);
 
     function(*i, u);
@@ -1056,8 +1102,6 @@ namespace
       v += *i;
     else
       v = *i;
-
-    vector_memory.free(i);
   }
 
 
@@ -1180,7 +1224,7 @@ namespace
 
 
 /**
- * @relates LinearOperator
+ * @relatesalso LinearOperator
  *
  * A function that encapsulates generic @p matrix objects that act on a
  * compatible Vector type into a LinearOperator. The LinearOperator object
@@ -1247,7 +1291,7 @@ linear_operator(const Matrix &matrix)
 
 
 /**
- * @relates LinearOperator
+ * @relatesalso LinearOperator
  *
  * Variant of above function that takes an operator object @p
  * operator_exemplar as an additional reference. This object is used to
@@ -1269,7 +1313,7 @@ template <typename Range,
 LinearOperator<Range, Domain, Payload>
 linear_operator(const OperatorExemplar &operator_exemplar, const Matrix &matrix)
 {
-  // Initialise the payload based on the input exemplar matrix
+  // Initialize the payload based on the input exemplar matrix
   LinearOperator<Range, Domain, Payload> return_op (Payload(operator_exemplar,matrix));
 
   // Always store a reference to matrix and operator_exemplar in the lambda
@@ -1279,12 +1323,12 @@ linear_operator(const OperatorExemplar &operator_exemplar, const Matrix &matrix)
 
   return_op.reinit_range_vector = [&operator_exemplar](Range &v, bool omit_zeroing_entries)
   {
-    internal::LinearOperator::ReinitHelper<Range>::reinit_range_vector(operator_exemplar, v, omit_zeroing_entries);
+    internal::LinearOperatorImplementation::ReinitHelper<Range>::reinit_range_vector(operator_exemplar, v, omit_zeroing_entries);
   };
 
   return_op.reinit_domain_vector = [&operator_exemplar](Domain &v, bool omit_zeroing_entries)
   {
-    internal::LinearOperator::ReinitHelper<Domain>::reinit_domain_vector(operator_exemplar, v, omit_zeroing_entries);
+    internal::LinearOperatorImplementation::ReinitHelper<Domain>::reinit_domain_vector(operator_exemplar, v, omit_zeroing_entries);
   };
 
   typename std::conditional<
@@ -1297,10 +1341,48 @@ linear_operator(const OperatorExemplar &operator_exemplar, const Matrix &matrix)
 }
 
 
+
+/**
+ * @relatesalso LinearOperator
+ *
+ * Variant of above function that takes a LinearOperator @p
+ * operator_exemplar as an additional reference.
+ * The reinit_domain_vector and reinit_range_vector function are copied
+ * from the @p operator_exemplar object.
+ *
+ * The reference @p matrix is used to construct vmult, Tvmult, etc.
+ *
+ * This variant can, for example, be used to encapsulate preconditioners (that
+ * typically do not expose any information about the underlying matrix).
+ *
+ * @author Matthias Maier, 2017
+ *
+ * @ingroup LAOperators
+ */
+template <typename Range,
+          typename Domain,
+          typename Payload,
+          typename Matrix>
+LinearOperator<Range, Domain, Payload>
+linear_operator(const LinearOperator<Range, Domain, Payload> &operator_exemplar,
+                const Matrix &matrix)
+{
+  // Initialize the payload based on the LinearOperator exemplar
+  auto return_op = operator_exemplar;
+
+  typename std::conditional<
+  has_vmult_add_and_Tvmult_add<Range, Domain, Matrix>::type::value,
+                               MatrixInterfaceWithVmultAdd<Range, Domain, Payload>,
+                               MatrixInterfaceWithoutVmultAdd<Range, Domain, Payload>>::type().
+                               operator()(return_op, matrix);
+
+  return return_op;
+}
+
+
 //@}
 
 
 DEAL_II_NAMESPACE_CLOSE
 
-#endif // DEAL_II_WITH_CXX11
 #endif
