@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 2007 - 2016 by the deal.II authors
+// Copyright (C) 2007 - 2018 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -13,12 +13,11 @@
 //
 // ---------------------------------------------------------------------
 
-#ifndef dealii__matrix_block_h
-#define dealii__matrix_block_h
+#ifndef dealii_matrix_block_h
+#define dealii_matrix_block_h
 
 #include <deal.II/base/config.h>
 #include <deal.II/base/smartpointer.h>
-#include <deal.II/base/std_cxx11/shared_ptr.h>
 #include <deal.II/base/memory_consumption.h>
 #include <deal.II/base/mg_level_object.h>
 #include <deal.II/lac/block_indices.h>
@@ -27,6 +26,8 @@
 #include <deal.II/lac/full_matrix.h>
 #include <deal.II/algorithms/any_data.h>
 
+#include <memory>
+
 
 DEAL_II_NAMESPACE_OPEN
 
@@ -235,7 +236,7 @@ public:
    * MatrixType. No index computations are done, thus, the vectors need to
    * have sizes matching #matrix.
    */
-  template<class VectorType>
+  template <class VectorType>
   void vmult (VectorType &w, const VectorType &v) const;
 
   /**
@@ -243,7 +244,7 @@ public:
    * MatrixType. No index computations are done, thus, the vectors need to
    * have sizes matching #matrix.
    */
-  template<class VectorType>
+  template <class VectorType>
   void vmult_add (VectorType &w, const VectorType &v) const;
 
   /**
@@ -251,7 +252,7 @@ public:
    * MatrixType. No index computations are done, thus, the vectors need to
    * have sizes matching #matrix.
    */
-  template<class VectorType>
+  template <class VectorType>
   void Tvmult (VectorType &w, const VectorType &v) const;
 
   /**
@@ -259,7 +260,7 @@ public:
    * MatrixType. No index computations are done, thus, the vectors need to
    * have sizes matching #matrix.
    */
-  template<class VectorType>
+  template <class VectorType>
   void Tvmult_add (VectorType &w, const VectorType &v) const;
 
   /**
@@ -320,7 +321,7 @@ private:
  * order to allow for copying and rearranging. Each matrix block can be
  * identified by name.
  *
- * @relates MatrixBlock
+ * @relatesalso MatrixBlock
  * @ingroup vector_valued
  * @author Baerbel Janssen, Guido Kanschat, 2010
  */
@@ -344,7 +345,7 @@ public:
    * The pointer type used for storing the objects. We use a shard pointer,
    * such that they get deleted automatically when not used anymore.
    */
-  typedef std_cxx11::shared_ptr<value_type> ptr_type;
+  typedef std::shared_ptr<value_type> ptr_type;
 
   /**
    * Add a new matrix block at the position <tt>(row,column)</tt> in the block
@@ -405,7 +406,7 @@ public:
  * pointers, in order to allow for copying and rearranging. Each matrix block
  * can be identified by name.
  *
- * @relates MatrixBlock
+ * @relatesalso MatrixBlock
  * @ingroup vector_valued
  * @author Baerbel Janssen, Guido Kanschat, 2010
  */
@@ -681,7 +682,7 @@ MatrixBlock<MatrixType>::add (const std::vector<size_type> &r_indices,
   AssertDimension (c_indices.size(), values.n());
 
   for (size_type i=0; i<row_indices.size(); ++i)
-    add (r_indices[i], c_indices.size(), &c_indices[0], &values(i,0),
+    add (r_indices[i], c_indices.size(), c_indices.data(), &values(i,0),
          elide_zero_values);
 }
 
@@ -741,7 +742,7 @@ MatrixBlock<MatrixType>::add (const std::vector<size_type> &indices,
   Assert (values.n() == values.m(), ExcNotQuadratic());
 
   for (size_type i=0; i<indices.size(); ++i)
-    add (indices[i], indices.size(), &indices[0], &values(i,0),
+    add (indices[i], indices.size(), indices.data(), &values(i,0),
          elide_zero_values);
 }
 
@@ -760,7 +761,7 @@ MatrixBlock<MatrixType>::add (const size_type               row,
   Assert(column_indices.size() != 0, ExcNotInitialized());
 
   AssertDimension (col_indices.size(), values.size());
-  add (row, col_indices.size(), &col_indices[0], &values[0],
+  add (row, col_indices.size(), col_indices.data(), values.data(),
        elide_zero_values);
 }
 
