@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 2014 - 2017 by the deal.II authors
+// Copyright (C) 2014 - 2018 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -13,15 +13,13 @@
 //
 // ---------------------------------------------------------------------
 
-#ifndef dealii__packaged_operation_h
-#define dealii__packaged_operation_h
+#ifndef dealii_packaged_operation_h
+#define dealii_packaged_operation_h
 
 #include <deal.II/base/config.h>
 #include <deal.II/base/exceptions.h>
 #include <deal.II/lac/vector_memory.h>
 
-#ifdef DEAL_II_WITH_CXX11
-
 #include <functional>
 
 DEAL_II_NAMESPACE_OPEN
@@ -277,7 +275,7 @@ public:
 //@{
 
 /**
- * @relates PackagedOperation
+ * @relatesalso PackagedOperation
  *
  * Addition of two PackagedOperation objects @p first_comp and @p second_comp
  * given by vector space addition of the corresponding results.
@@ -312,7 +310,7 @@ operator+(const PackagedOperation<Range> &first_comp,
 }
 
 /**
- * @relates PackagedOperation
+ * @relatesalso PackagedOperation
  *
  * Subtraction of two PackagedOperation objects @p first_comp and @p
  * second_comp given by vector space addition of the corresponding results.
@@ -350,7 +348,7 @@ operator-(const PackagedOperation<Range> &first_comp,
 }
 
 /**
- * @relates PackagedOperation
+ * @relatesalso PackagedOperation
  *
  * Scalar multiplication of a PackagedOperation objects @p comp with a scalar
  * @p number given by a scaling PackagedOperation result with @p number.
@@ -398,7 +396,7 @@ operator*(const PackagedOperation<Range> &comp,
 }
 
 /**
- * @relates PackagedOperation
+ * @relatesalso PackagedOperation
  *
  * Scalar multiplication of a PackagedOperation objects @p comp with a scalar
  * @p number given by a scaling PackagedOperation result with @p number.
@@ -414,7 +412,7 @@ operator*(typename Range::value_type number,
 }
 
 /**
- * @relates PackagedOperation
+ * @relatesalso PackagedOperation
  *
  * Add a constant @p offset (of the @p Range space) to the result of a
  * PackagedOperation.
@@ -429,7 +427,7 @@ PackagedOperation<Range> operator+(const PackagedOperation<Range> &comp,
 }
 
 /**
- * @relates PackagedOperation
+ * @relatesalso PackagedOperation
  *
  * Add a constant @p offset (of the @p Range space) to the result of a
  * PackagedOperation.
@@ -444,7 +442,7 @@ PackagedOperation<Range> operator+(const Range &offset,
 }
 
 /**
- * @relates PackagedOperation
+ * @relatesalso PackagedOperation
  *
  * Subtract a constant @p offset (of the @p Range space) from the result of a
  * PackagedOperation.
@@ -460,7 +458,7 @@ PackagedOperation<Range> operator-(const PackagedOperation<Range> &comp,
 
 
 /**
- * @relates PackagedOperation
+ * @relatesalso PackagedOperation
  *
  * Subtract a computational result from a constant @p offset (of the @p Range
  * space). The result is a PackagedOperation object that applies this
@@ -504,13 +502,13 @@ namespace
     // type is std::true_type if Matrix provides vmult_add and Tvmult_add,
     // otherwise it is std::false_type
 
-    typedef decltype(test<T>(0, 0, 0)) type;
+    typedef decltype(test<T>(nullptr, nullptr, nullptr)) type;
   };
 }
 
 
 /**
- * @relates PackagedOperation
+ * @relatesalso PackagedOperation
  *
  * Create a PackagedOperation object that stores the addition of two vectors.
  *
@@ -554,7 +552,7 @@ PackagedOperation<Range> operator+(const Range &u, const Range &v)
 
 
 /**
- * @relates PackagedOperation
+ * @relatesalso PackagedOperation
  *
  * Create a PackagedOperation object that stores the subtraction of two
  * vectors.
@@ -575,7 +573,7 @@ PackagedOperation<Range> operator-(const Range &u, const Range &v)
   PackagedOperation<Range> return_comp;
 
   // ensure to have valid PackagedOperation objects by catching op by value
-  // u is catched by reference
+  // u is caught by reference
 
   return_comp.reinit_vector = [&u](Range &x, bool omit_zeroing_entries)
   {
@@ -599,7 +597,7 @@ PackagedOperation<Range> operator-(const Range &u, const Range &v)
 
 
 /**
- * @relates PackagedOperation
+ * @relatesalso PackagedOperation
  *
  * Create a PackagedOperation object that stores the scaling of a vector with
  * a @p number.
@@ -622,7 +620,7 @@ PackagedOperation<Range> operator*(const Range &u,
 
 
 /**
- * @relates PackagedOperation
+ * @relatesalso PackagedOperation
  *
  * Create a PackagedOperation object that stores the scaling of a vector with
  * a @p number.
@@ -645,7 +643,7 @@ PackagedOperation<Range> operator*(typename Range::value_type number,
 
 
 /**
- * @relates PackagedOperation
+ * @relatesalso PackagedOperation
  *
  * Create a PackagedOperation object from a LinearOperator and a reference to
  * a vector @p u of the Domain space. The object stores the PackagedOperation
@@ -687,7 +685,7 @@ operator*(const LinearOperator<Range, Domain, Payload> &op,
 
 
 /**
- * @relates PackagedOperation
+ * @relatesalso PackagedOperation
  *
  * Create a PackagedOperation object from a LinearOperator and a reference to
  * a vector @p u of the Range space. The object stores the PackagedOperation
@@ -729,7 +727,7 @@ operator*(const Range &u,
 
 
 /**
- * @relates PackagedOperation
+ * @relatesalso PackagedOperation
  *
  * Composition of a PackagedOperation object with a LinearOperator. The object
  * stores the computation $\text{op} \,comp$ (in matrix notation).
@@ -750,28 +748,24 @@ operator*(const LinearOperator<Range, Domain, Payload> &op,
 
   return_comp.apply = [op, comp](Domain &v)
   {
-    static GrowingVectorMemory<Range> vector_memory;
+    GrowingVectorMemory<Range> vector_memory;
 
-    Range *i = vector_memory.alloc();
+    typename VectorMemory<Range>::Pointer i (vector_memory);
     op.reinit_domain_vector(*i, /*bool omit_zeroing_entries =*/ true);
 
     comp.apply(*i);
     op.vmult(v, *i);
-
-    vector_memory.free(i);
   };
 
   return_comp.apply_add = [op, comp](Domain &v)
   {
-    static GrowingVectorMemory<Range> vector_memory;
+    GrowingVectorMemory<Range> vector_memory;
 
-    Range *i = vector_memory.alloc();
+    typename VectorMemory<Range>::Pointer i (vector_memory);
     op.reinit_range_vector(*i, /*bool omit_zeroing_entries =*/ true);
 
     comp.apply(*i);
     op.vmult_add(v, *i);
-
-    vector_memory.free(i);
   };
 
   return return_comp;
@@ -779,7 +773,7 @@ operator*(const LinearOperator<Range, Domain, Payload> &op,
 
 
 /**
- * @relates PackagedOperation
+ * @relatesalso PackagedOperation
  *
  * Composition of a PackagedOperation object with a LinearOperator. The object
  * stores the computation $\text{op}^T \,comp$ (in matrix notation).
@@ -800,28 +794,24 @@ operator*(const PackagedOperation<Range> &comp,
 
   return_comp.apply = [op, comp](Domain &v)
   {
-    static GrowingVectorMemory<Range> vector_memory;
+    GrowingVectorMemory<Range> vector_memory;
 
-    Range *i = vector_memory.alloc();
+    typename VectorMemory<Range>::Pointer i (vector_memory);
     op.reinit_range_vector(*i, /*bool omit_zeroing_entries =*/ true);
 
     comp.apply(*i);
     op.Tvmult(v, *i);
-
-    vector_memory.free(i);
   };
 
   return_comp.apply_add = [op, comp](Domain &v)
   {
-    static GrowingVectorMemory<Range> vector_memory;
+    GrowingVectorMemory<Range> vector_memory;
 
-    Range *i = vector_memory.alloc();
+    typename VectorMemory<Range>::Pointer i (vector_memory);
     op.reinit_range_vector(*i, /*bool omit_zeroing_entries =*/ true);
 
     comp.apply(*i);
     op.Tvmult_add(v, *i);
-
-    vector_memory.free(i);
   };
 
   return return_comp;
@@ -831,5 +821,4 @@ operator*(const PackagedOperation<Range> &comp,
 
 DEAL_II_NAMESPACE_CLOSE
 
-#endif // DEAL_II_WITH_CXX11
 #endif
