@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 2016 by the deal.II authors
+// Copyright (C) 2016 - 2017 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -17,8 +17,8 @@
  * Rather than using ifdefs everywhere, try to wrap older versions of PETSc
  * functions in one place.
  */
-#ifndef dealii__petsc_compatibility_h
-#define dealii__petsc_compatibility_h
+#ifndef dealii_petsc_compatibility_h
+#define dealii_petsc_compatibility_h
 
 #include <deal.II/base/config.h>
 #include <deal.II/lac/exceptions.h>
@@ -36,11 +36,14 @@ DEAL_II_NAMESPACE_OPEN
 
 namespace PETScWrappers
 {
-#if DEAL_II_PETSC_VERSION_LT(3,2,0)
-  typedef PetscTruth PetscBooleanType;
-#else
-  typedef PetscBool PetscBooleanType;
-#endif
+  /**
+   * a compatibility typedef for older versions of PETSc.
+   *
+   * @deprecated The name of this type changed in PETSc 3.2: deal.II does not
+   * support versions of PETSc older than this so this typedef will be removed
+   * in a future release.
+   */
+  typedef PetscBool PetscBooleanType DEAL_II_DEPRECATED;
 
   /**
    * Set an option in the global PETSc database. This function just wraps
@@ -53,7 +56,7 @@ namespace PETScWrappers
 #if DEAL_II_PETSC_VERSION_LT(3, 7, 0)
     const PetscErrorCode ierr = PetscOptionsSetValue (name.c_str (), value.c_str ());
 #else
-    const PetscErrorCode ierr = PetscOptionsSetValue (NULL, name.c_str (), value.c_str ());
+    const PetscErrorCode ierr = PetscOptionsSetValue (nullptr, name.c_str (), value.c_str ());
 #endif
     AssertThrow (ierr == 0, ExcPETScError(ierr));
   }
@@ -72,12 +75,8 @@ namespace PETScWrappers
    */
   inline PetscErrorCode destroy_matrix (Mat &matrix)
   {
-    // PETSc will check whether or not matrix is NULL.
-#if DEAL_II_PETSC_VERSION_LT(3, 2, 0)
-    return MatDestroy (matrix);
-#else
+    // PETSc will check whether or not matrix is nullptr.
     return MatDestroy (&matrix);
-#endif
   }
 
 
@@ -94,12 +93,8 @@ namespace PETScWrappers
    */
   inline PetscErrorCode destroy_krylov_solver (KSP &krylov_solver)
   {
-    // PETSc will check whether or not matrix is NULL.
-#if DEAL_II_PETSC_VERSION_LT(3, 2, 0)
-    return KSPDestroy (krylov_solver);
-#else
+    // PETSc will check whether or not matrix is nullptr.
     return KSPDestroy (&krylov_solver);
-#endif
   }
 
 
@@ -113,14 +108,9 @@ namespace PETScWrappers
    */
   inline void set_matrix_option (Mat &matrix,
                                  const MatOption option_name,
-                                 const PetscBooleanType option_value = PETSC_FALSE)
+                                 const PetscBool option_value = PETSC_FALSE)
   {
-#if DEAL_II_PETSC_VERSION_LT(3,0,0)
-    const PetscErrorCode ierr = MatSetOption (matrix, option_name);
-#else
     const PetscErrorCode ierr = MatSetOption (matrix, option_name, option_value);
-#endif
-
     AssertThrow (ierr == 0, ExcPETScError(ierr));
   }
 
@@ -132,19 +122,11 @@ namespace PETScWrappers
    */
   inline void close_matrix (Mat &matrix)
   {
-#if DEAL_II_PETSC_VERSION_LT(3, 0, 0)
-#  ifdef DEBUG
-    set_matrix_option (matrix, MAT_NEW_NONZERO_LOCATION_ERR);
-#  else
-    set_matrix_option (matrix, MAT_NO_NEW_NONZERO_LOCATIONS);
-#  endif
-#else
 #  ifdef DEBUG
     set_matrix_option (matrix, MAT_NEW_NONZERO_LOCATION_ERR, PETSC_TRUE);
 #  else
     set_matrix_option (matrix, MAT_NEW_NONZERO_LOCATIONS, PETSC_FALSE);
 #  endif
-#endif
   }
 
 
@@ -156,15 +138,11 @@ namespace PETScWrappers
    */
   inline void set_keep_zero_rows (Mat &matrix)
   {
-#if DEAL_II_PETSC_VERSION_LT(3, 1, 0)
-    set_matrix_option (matrix, MAT_KEEP_ZEROED_ROWS, PETSC_TRUE);
-#else
     set_matrix_option (matrix, MAT_KEEP_NONZERO_PATTERN, PETSC_TRUE);
-#endif
   }
 }
 
 DEAL_II_NAMESPACE_CLOSE
 
 #endif // DEAL_II_WITH_PETSC
-#endif // dealii__petsc_compatibility_h
+#endif // dealii_petsc_compatibility_h
