@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 2004 - 2015 by the deal.II authors
+// Copyright (C) 2004 - 2017 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -13,8 +13,8 @@
 //
 // ---------------------------------------------------------------------
 
-#ifndef dealii__petsc_full_matrix_h
-#define dealii__petsc_full_matrix_h
+#ifndef dealii_petsc_full_matrix_h
+#define dealii_petsc_full_matrix_h
 
 
 #include <deal.II/base/config.h>
@@ -35,7 +35,7 @@ namespace PETScWrappers
    */
 
   /**
-   * Implementation of a sequential dense matrix class based on PETSC. All the
+   * Implementation of a sequential dense matrix class based on PETSc. All the
    * functionality is actually in the base class, except for the calls to
    * generate a sequential dense matrix. This is possible since PETSc only
    * works on an abstract matrix type and internally distributes to functions
