@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 2004 - 2017 by the deal.II authors
+// Copyright (C) 2004 - 2018 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -13,8 +13,8 @@
 //
 // ---------------------------------------------------------------------
 
-#ifndef dealii__petsc_matrix_base_h
-#define dealii__petsc_matrix_base_h
+#ifndef dealii_petsc_matrix_base_h
+#define dealii_petsc_matrix_base_h
 
 
 #include <deal.II/base/config.h>
@@ -25,13 +25,14 @@
 #  include <deal.II/lac/exceptions.h>
 #  include <deal.II/lac/full_matrix.h>
 #  include <deal.II/lac/petsc_compatibility.h>
-#  include <deal.II/lac/vector.h>
+#  include <deal.II/lac/vector_operation.h>
+#  include <deal.II/lac/petsc_vector_base.h>
 
 #  include <petscmat.h>
-#  include <deal.II/base/std_cxx11/shared_ptr.h>
 
 #  include <vector>
 #  include <cmath>
+#  include <memory>
 
 DEAL_II_NAMESPACE_OPEN
 
@@ -41,7 +42,6 @@ template <typename Matrix> class BlockMatrixBase;
 namespace PETScWrappers
 {
   // forward declarations
-  class VectorBase;
   class MatrixBase;
 
   namespace MatrixIterators
@@ -144,12 +144,12 @@ namespace PETScWrappers
          * performance, we keep a shared pointer to these entries so that more
          * than one accessor can access this data if necessary.
          */
-        std_cxx11::shared_ptr<const std::vector<size_type> > colnum_cache;
+        std::shared_ptr<const std::vector<size_type> > colnum_cache;
 
         /**
          * Similar cache for the values of this row.
          */
-        std_cxx11::shared_ptr<const std::vector<PetscScalar> > value_cache;
+        std::shared_ptr<const std::vector<PetscScalar> > value_cache;
 
         /**
          * Discard the old row caches (they may still be used by other
@@ -289,6 +289,20 @@ namespace PETScWrappers
      */
     MatrixBase ();
 
+    /**
+     * Copy constructor. It is deleted as copying this base class
+     * without knowing the concrete kind of matrix stored may both
+     * miss important details and be expensive if the matrix is large.
+     */
+    MatrixBase(const MatrixBase &) = delete;
+
+    /**
+     * Copy operator. It is deleted as copying this base class
+     * without knowing the concrete kind of matrix stored may both
+     * miss important details and be expensive if the matrix is large.
+     */
+    MatrixBase &operator=(const MatrixBase &) = delete;
+
     /**
      * Destructor. Made virtual so that one can use pointers to this class.
      */
@@ -678,14 +692,11 @@ namespace PETScWrappers
     PetscScalar matrix_scalar_product (const VectorBase &u,
                                        const VectorBase &v) const;
 
-
-#if DEAL_II_PETSC_VERSION_GTE(3,1,0)
     /**
      * Return the trace of the matrix, i.e. the sum of all diagonal entries in
      * the matrix.
      */
     PetscScalar trace () const;
-#endif
 
     /**
      * Multiply the entire matrix by a fixed factor.
@@ -711,8 +722,9 @@ namespace PETScWrappers
      * matrix.
      * @deprecated Use the function with order of arguments reversed instead.
      */
+    DEAL_II_DEPRECATED
     MatrixBase &add (const MatrixBase &other,
-                     const PetscScalar factor) DEAL_II_DEPRECATED;
+                     const PetscScalar factor);
 
     /**
      * Matrix-vector multiplication: let <i>dst = M*src</i> with <i>M</i>
@@ -772,7 +784,6 @@ namespace PETScWrappers
     void Tvmult_add (VectorBase       &dst,
                      const VectorBase &src) const;
 
-
     /**
      * Compute the residual of an equation <i>Mx=b</i>, where the residual is
      * defined to be <i>r=b-Mx</i>. Write the residual into @p dst. The
@@ -828,6 +839,13 @@ namespace PETScWrappers
      */
     operator Mat () const;
 
+    /**
+     * Return a reference to the underlying PETSc type. It can be used to
+     * modify the underlying data, so use it only when you know what you
+     * are doing.
+     */
+    Mat &petsc_matrix ();
+
     /**
      * Make an in-place transpose of a matrix.
      */
@@ -837,7 +855,7 @@ namespace PETScWrappers
      * Test whether a matrix is symmetric.  Default tolerance is
      * $1000\times32$-bit machine precision.
      */
-    PetscBooleanType
+    PetscBool
     is_symmetric (const double tolerance = 1.e-12);
 
     /**
@@ -845,7 +863,7 @@ namespace PETScWrappers
      * its transpose. Default tolerance is $1000\times32$-bit machine
      * precision.
      */
-    PetscBooleanType
+    PetscBool
     is_hermitian (const double tolerance = 1.e-12);
 
     /**
@@ -874,7 +892,10 @@ namespace PETScWrappers
     /**
      * Exception
      */
-    DeclException0 (ExcSourceEqualsDestination);
+    DeclExceptionMsg (ExcSourceEqualsDestination,
+                      "You are attempting an operation on two matrices that "
+                      "are the same object, but the operation requires that the "
+                      "two objects are in fact different.");
 
     /**
      * Exception.
@@ -937,32 +958,72 @@ namespace PETScWrappers
      */
     void prepare_set();
 
+    /**
+     * Base function to perform the matrix-matrix multiplication $C = AB$,
+     * or, if a vector $V$ whose size is compatible with B is given,
+     * $C = A \text{diag}(V) B$, where $\text{diag}(V)$ defines a
+     * diagonal matrix with the vector entries.
+     *
+     * This function assumes that the calling matrix $A$ and $B$
+     * have compatible sizes. The size of $C$ will be set within this
+     * function.
+     *
+     * The content as well as the sparsity pattern of the matrix $C$ will be
+     * reset by this function, so make sure that the sparsity pattern is not
+     * used somewhere else in your program. This is an expensive operation, so
+     * think twice before you use this function.
+     */
+    void mmult (MatrixBase       &C,
+                const MatrixBase &B,
+                const VectorBase &V) const;
 
+    /**
+     * Base function to perform the matrix-matrix multiplication with
+     * the transpose of <tt>this</tt>, i.e., $C = A^T B$, or,
+     * if an optional vector $V$ whose size is compatible with $B$ is given,
+     * $C = A^T \text{diag}(V) B$, where $\text{diag}(V)$ defines a
+     * diagonal matrix with the vector entries.
+     *
+     * This function assumes that the calling matrix $A$ and $B$
+     * have compatible sizes. The size of $C$ will be set within this
+     * function.
+     *
+     * The content as well as the sparsity pattern of the matrix $C$ will be
+     * changed by this function, so make sure that the sparsity pattern is not
+     * used somewhere else in your program. This is an expensive operation, so
+     * think twice before you use this function.
+     */
+    void Tmmult (MatrixBase       &C,
+                 const MatrixBase &B,
+                 const VectorBase &V) const;
 
   private:
 
-    /**
-     * purposefully not implemented
-     */
-    MatrixBase(const MatrixBase &);
-    /**
-     * purposefully not implemented
-     */
-    MatrixBase &operator=(const MatrixBase &);
-
     /**
      * An internal array of integer values that is used to store the column
      * indices when adding/inserting local data into the (large) sparse
      * matrix.
+     *
+     * This variable does not store any "state" of the matrix
+     * object. Rather, it is only used as a temporary buffer by some
+     * of the member functions of this class. As with all @p mutable
+     * member variables, the use of this variable is not thread-safe
+     * unless guarded by a mutex. However, since PETSc matrix
+     * operations are not thread-safe anyway, there is no need to
+     * attempt to make things thread-safe, and so there is no mutex
+     * associated with this variable.
      */
-    std::vector<PetscInt> column_indices;
+    mutable std::vector<PetscInt> column_indices;
 
     /**
      * An internal array of double values that is used to store the column
      * indices when adding/inserting local data into the (large) sparse
      * matrix.
+     *
+     * The same comment as for the @p column_indices variable above
+     * applies.
      */
-    std::vector<PetscScalar> column_values;
+    mutable std::vector<PetscScalar> column_values;
 
 
     /**
@@ -1050,15 +1111,16 @@ namespace PETScWrappers
 
       ++accessor.a_index;
 
-      // if at end of line: do one step, then
-      // cycle until we find a row with a
-      // nonzero number of entries
+      // if at end of line: do one step, then cycle until we find a
+      // row with a nonzero number of entries
       if (accessor.a_index >= accessor.colnum_cache->size())
         {
           accessor.a_index = 0;
           ++accessor.a_row;
 
           while ((accessor.a_row < accessor.matrix->m())
+                 &&
+                 (accessor.a_row < accessor.matrix->local_range().second)
                  &&
                  (accessor.matrix->row_length(accessor.a_row) == 0))
             ++accessor.a_row;
@@ -1159,7 +1221,7 @@ namespace PETScWrappers
     Assert (values.m() == values.n(), ExcNotQuadratic());
 
     for (size_type i=0; i<indices.size(); ++i)
-      set (indices[i], indices.size(), &indices[0], &values(i,0),
+      set (indices[i], indices.size(), indices.data(), &values(i,0),
            elide_zero_values);
   }
 
@@ -1178,7 +1240,7 @@ namespace PETScWrappers
             ExcDimensionMismatch(col_indices.size(), values.n()));
 
     for (size_type i=0; i<row_indices.size(); ++i)
-      set (row_indices[i], col_indices.size(), &col_indices[0], &values(i,0),
+      set (row_indices[i], col_indices.size(), col_indices.data(), &values(i,0),
            elide_zero_values);
   }
 
@@ -1194,7 +1256,7 @@ namespace PETScWrappers
     Assert (col_indices.size() == values.size(),
             ExcDimensionMismatch(col_indices.size(), values.size()));
 
-    set (row, col_indices.size(), &col_indices[0], &values[0],
+    set (row, col_indices.size(), col_indices.data(), values.data(),
          elide_zero_values);
   }
 
@@ -1208,26 +1270,22 @@ namespace PETScWrappers
                    const PetscScalar  *values,
                    const bool         elide_zero_values)
   {
-    (void)elide_zero_values;
-
     prepare_action(VectorOperation::insert);
 
     const PetscInt petsc_i = row;
-    PetscInt *col_index_ptr;
+    PetscInt const *col_index_ptr;
 
     PetscScalar const *col_value_ptr;
     int n_columns;
 
     // If we don't elide zeros, the pointers are already available...
-#ifndef PETSC_USE_64BIT_INDICES
     if (elide_zero_values == false)
       {
-        col_index_ptr = (int *)col_indices;
+        col_index_ptr = reinterpret_cast<const PetscInt *>(col_indices);
         col_value_ptr = values;
         n_columns = n_cols;
       }
     else
-#endif
       {
         // Otherwise, extract nonzero values in each row and get the
         // respective index.
@@ -1251,8 +1309,8 @@ namespace PETScWrappers
           }
         Assert(n_columns <= (int)n_cols, ExcInternalError());
 
-        col_index_ptr = &column_indices[0];
-        col_value_ptr = &column_values[0];
+        col_index_ptr = column_indices.data();
+        col_value_ptr = column_values.data();
       }
 
     const PetscErrorCode ierr = MatSetValues (matrix, 1, &petsc_i, n_columns,
@@ -1275,8 +1333,7 @@ namespace PETScWrappers
     if (value == PetscScalar())
       {
         // we have to check after using Insert/Add in any case to be
-        // consistent with the MPI communication model (see the comments in
-        // the documentation of TrilinosWrappers::Vector), but we can save
+        // consistent with the MPI communication model, but we can save
         // some work if the addend is zero. However, these actions are done
         // in case we pass on to the other function.
         prepare_action(VectorOperation::add);
@@ -1300,7 +1357,7 @@ namespace PETScWrappers
     Assert (values.m() == values.n(), ExcNotQuadratic());
 
     for (size_type i=0; i<indices.size(); ++i)
-      add (indices[i], indices.size(), &indices[0], &values(i,0),
+      add (indices[i], indices.size(), indices.data(), &values(i,0),
            elide_zero_values);
   }
 
@@ -1319,7 +1376,7 @@ namespace PETScWrappers
             ExcDimensionMismatch(col_indices.size(), values.n()));
 
     for (size_type i=0; i<row_indices.size(); ++i)
-      add (row_indices[i], col_indices.size(), &col_indices[0], &values(i,0),
+      add (row_indices[i], col_indices.size(), col_indices.data(), &values(i,0),
            elide_zero_values);
   }
 
@@ -1335,7 +1392,7 @@ namespace PETScWrappers
     Assert (col_indices.size() == values.size(),
             ExcDimensionMismatch(col_indices.size(), values.size()));
 
-    add (row, col_indices.size(), &col_indices[0], &values[0],
+    add (row, col_indices.size(), col_indices.data(), values.data(),
          elide_zero_values);
   }
 
@@ -1355,21 +1412,19 @@ namespace PETScWrappers
     prepare_action(VectorOperation::add);
 
     const PetscInt petsc_i = row;
-    PetscInt *col_index_ptr;
+    PetscInt const *col_index_ptr;
 
     PetscScalar const *col_value_ptr;
     int n_columns;
 
     // If we don't elide zeros, the pointers are already available...
-#ifndef PETSC_USE_64BIT_INDICES
     if (elide_zero_values == false)
       {
-        col_index_ptr = (int *)col_indices;
+        col_index_ptr = reinterpret_cast<const PetscInt *>(col_indices);
         col_value_ptr = values;
         n_columns = n_cols;
       }
     else
-#endif
       {
         // Otherwise, extract nonzero values in each row and get the
         // respective index.
@@ -1393,8 +1448,8 @@ namespace PETScWrappers
           }
         Assert(n_columns <= (int)n_cols, ExcInternalError());
 
-        col_index_ptr = &column_indices[0];
-        col_value_ptr = &column_values[0];
+        col_index_ptr = column_indices.data();
+        col_value_ptr = column_values.data();
       }
 
     const PetscErrorCode ierr = MatSetValues (matrix, 1, &petsc_i, n_columns,
@@ -1438,7 +1493,9 @@ namespace PETScWrappers
   MatrixBase::const_iterator
   MatrixBase::begin(const size_type r) const
   {
-    Assert (r < m(), ExcIndexRange(r, 0, m()));
+    Assert (in_local_range(r),
+            ExcIndexRange(r, local_range().first, local_range().second));
+
     if (row_length(r) > 0)
       return const_iterator(this, r, 0);
     else
@@ -1450,13 +1507,21 @@ namespace PETScWrappers
   MatrixBase::const_iterator
   MatrixBase::end(const size_type r) const
   {
-    Assert (r < m(), ExcIndexRange(r, 0, m()));
+    Assert (in_local_range(r),
+            ExcIndexRange(r, local_range().first, local_range().second));
 
-    // place the iterator on the first entry
-    // past this line, or at the end of the
-    // matrix
+    // place the iterator on the first entry past this line, or at the
+    // end of the matrix
+    //
+    // in the parallel case, we need to put it on the first entry of
+    // the first row after the locally owned range. this of course
+    // doesn't exist, but we can nevertheless create such an
+    // iterator. we need to check whether 'i' is past the locally
+    // owned range of rows first, before we ask for the length of the
+    // row since the latter query leads to an exception in case we ask
+    // for a row that is not locally owned
     for (size_type i=r+1; i<m(); ++i)
-      if (row_length(i) > 0)
+      if (i == local_range().second || (row_length(i) > 0))
         return const_iterator(this, i, 0);
 
     // if there is no such line, then take the
