@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 2004 - 2017 by the deal.II authors
+// Copyright (C) 2004 - 2018 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -13,8 +13,8 @@
 //
 // ---------------------------------------------------------------------
 
-#ifndef dealii__petsc_parallel_block_vector_h
-#define dealii__petsc_parallel_block_vector_h
+#ifndef dealii_petsc_parallel_block_vector_h
+#define dealii_petsc_parallel_block_vector_h
 
 
 #include <deal.II/base/config.h>
@@ -87,7 +87,7 @@ namespace PETScWrappers
       /**
        * Default constructor. Generate an empty vector without any blocks.
        */
-      BlockVector ();
+      BlockVector () = default;
 
       /**
        * Constructor. Generate a block vector with @p n_blocks blocks, each of
@@ -136,7 +136,7 @@ namespace PETScWrappers
       /**
        * Destructor. Clears memory
        */
-      ~BlockVector ();
+      ~BlockVector () = default;
 
       /**
        * Copy operator: fill all components of the vector that are locally
@@ -150,25 +150,6 @@ namespace PETScWrappers
       BlockVector &
       operator= (const BlockVector &V);
 
-      /**
-       * Copy the given sequential (non-distributed) block vector into the
-       * present parallel block vector. It is assumed that they have the same
-       * size, and this operation does not change the partitioning of the
-       * parallel vectors by which its elements are distributed across several
-       * MPI processes. What this operation therefore does is to copy that
-       * chunk of the given vector @p v that corresponds to elements of the
-       * target vector that are stored locally, and copies them, for each of
-       * the individual blocks of this object. Elements that are not stored
-       * locally are not touched.
-       *
-       * This being a parallel vector, you must make sure that @em all
-       * processes call this function at the same time. It is not possible to
-       * change the local part of a parallel vector on only one process,
-       * independent of what other processes do, with this function.
-       */
-      BlockVector &
-      operator= (const PETScWrappers::BlockVector &v);
-
       /**
        * Reinitialize the BlockVector to contain @p n_blocks of size @p
        * block_size, each of which stores @p local_size elements locally. The
@@ -271,7 +252,7 @@ namespace PETScWrappers
        * the same number of blocks. If needed, the numbers of blocks should be
        * exchanged, too.
        *
-       * This function is analogous to the the swap() function of all C++
+       * This function is analogous to the swap() function of all C++
        * standard containers. Also, there is a global function swap(u,v) that
        * simply calls <tt>u.swap(v)</tt>, again in analogy to standard
        * functions.
@@ -300,13 +281,6 @@ namespace PETScWrappers
 
     /*----------------------- Inline functions ----------------------------------*/
 
-
-    inline
-    BlockVector::BlockVector ()
-    {}
-
-
-
     inline
     BlockVector::BlockVector (const unsigned int  n_blocks,
                               const MPI_Comm     &communicator,
@@ -382,9 +356,6 @@ namespace PETScWrappers
       return *this;
     }
 
-    inline
-    BlockVector::~BlockVector ()
-    {}
 
 
     inline
@@ -525,7 +496,7 @@ namespace PETScWrappers
      * standard library which uses a temporary object. The function simply
      * exchanges the data of the two vectors.
      *
-     * @relates PETScWrappers::MPI::BlockVector
+     * @relatesalso PETScWrappers::MPI::BlockVector
      * @author Wolfgang Bangerth, 2000
      */
     inline
@@ -541,7 +512,7 @@ namespace PETScWrappers
 
 namespace internal
 {
-  namespace LinearOperator
+  namespace LinearOperatorImplementation
   {
     template <typename> class ReinitHelper;
 
@@ -549,7 +520,7 @@ namespace internal
      * A helper class used internally in linear_operator.h. Specialization for
      * PETScWrappers::MPI::BlockVector.
      */
-    template<>
+    template <>
     class ReinitHelper<PETScWrappers::MPI::BlockVector>
     {
     public:
@@ -582,7 +553,7 @@ namespace internal
  * @author Uwe Koecher, 2017
  */
 template <>
-struct is_serial_vector< PETScWrappers::MPI::BlockVector > : std_cxx11::false_type
+struct is_serial_vector< PETScWrappers::MPI::BlockVector > : std::false_type
 {
 };
 
