@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 2004 - 2016 by the deal.II authors
+// Copyright (C) 2004 - 2018 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -13,8 +13,8 @@
 //
 // ---------------------------------------------------------------------
 
-#ifndef dealii__petsc_parallel_sparse_matrix_h
-#define dealii__petsc_parallel_sparse_matrix_h
+#ifndef dealii_petsc_parallel_sparse_matrix_h
+#define dealii_petsc_parallel_sparse_matrix_h
 
 #include <deal.II/base/config.h>
 
@@ -40,7 +40,7 @@ namespace PETScWrappers
 
 
     /**
-     * Implementation of a parallel sparse matrix class based on PETSC, with
+     * Implementation of a parallel sparse matrix class based on PETSc, with
      * rows of the matrix distributed across an MPI network. All the
      * functionality is actually in the base class, except for the calls to
      * generate a parallel sparse matrix. This is possible since PETSc only
@@ -136,7 +136,7 @@ namespace PETScWrappers
         /**
          * It is not safe to elide additions of zeros to individual elements
          * of this matrix. The reason is that additions to the matrix may
-         * trigger collective operations synchronising buffers on multiple
+         * trigger collective operations synchronizing buffers on multiple
          * processes. If an addition is elided on one process, this may lead
          * to other processes hanging in an infinite waiting loop.
          */
@@ -171,7 +171,11 @@ namespace PETScWrappers
        * documentation states that one cannot form an ILU decomposition of a
        * matrix for which this flag has been set to @p true, only an ICC. The
        * default value of this flag is @p false.
+       *
+       * @deprecated This constructor is deprecated: please use the
+       * constructor with a sparsity pattern argument instead.
        */
+      DEAL_II_DEPRECATED
       SparseMatrix (const MPI_Comm  &communicator,
                     const size_type  m,
                     const size_type  n,
@@ -200,7 +204,11 @@ namespace PETScWrappers
        * documentation states that one cannot form an ILU decomposition of a
        * matrix for which this flag has been set to @p true, only an ICC. The
        * default value of this flag is @p false.
+       *
+       * @deprecated This constructor is deprecated: please use the
+       * constructor with a sparsity pattern argument instead.
        */
+      DEAL_II_DEPRECATED
       SparseMatrix (const MPI_Comm               &communicator,
                     const size_type               m,
                     const size_type               n,
@@ -262,7 +270,11 @@ namespace PETScWrappers
        * Throw away the present matrix and generate one that has the same
        * properties as if it were created by the constructor of this class
        * with the same argument list as the present function.
+       *
+       * @deprecated This overload of <code>reinit</code> is deprecated:
+       * please use the overload with a sparsity pattern argument instead.
        */
+      DEAL_II_DEPRECATED
       void reinit (const MPI_Comm     &communicator,
                    const size_type m,
                    const size_type n,
@@ -276,7 +288,11 @@ namespace PETScWrappers
        * Throw away the present matrix and generate one that has the same
        * properties as if it were created by the constructor of this class
        * with the same argument list as the present function.
+       *
+       * @deprecated This overload of <code>reinit</code> is deprecated:
+       * please use the overload with a sparsity pattern argument instead.
        */
+      DEAL_II_DEPRECATED
       void reinit (const MPI_Comm               &communicator,
                    const size_type               m,
                    const size_type               n,
@@ -392,6 +408,26 @@ namespace PETScWrappers
        */
       IndexSet locally_owned_range_indices() const;
 
+      /**
+       * Perform the matrix-matrix multiplication $C = AB$, or,
+       * $C = A \text{diag}(V) B$ given a compatible vector $V$.
+       *
+       * This function calls MatrixBase::mmult() to do the actual work.
+       */
+      void mmult (SparseMatrix       &C,
+                  const SparseMatrix &B,
+                  const MPI::Vector  &V = MPI::Vector()) const;
+
+      /**
+       * Perform the matrix-matrix multiplication with the transpose of
+       * <tt>this</tt>, i.e., $C = A^T B$, or,
+       * $C = A^T \text{diag}(V) B$ given a compatible vector $V$.
+       *
+       * This function calls MatrixBase::Tmmult() to do the actual work.
+       */
+      void Tmmult (SparseMatrix       &C,
+                   const SparseMatrix &B,
+                   const MPI::Vector       &V = MPI::Vector()) const;
     private:
 
       /**
@@ -403,7 +439,11 @@ namespace PETScWrappers
        * Do the actual work for the respective reinit() function and the
        * matching constructor, i.e. create a matrix. Getting rid of the
        * previous matrix is left to the caller.
+       *
+       * @deprecated This overload of <code>do_reinit</code> is deprecated:
+       * please use the overload with a sparsity pattern argument instead.
        */
+      DEAL_II_DEPRECATED
       void do_reinit (const size_type m,
                       const size_type n,
                       const size_type local_rows,
@@ -414,7 +454,11 @@ namespace PETScWrappers
 
       /**
        * Same as previous function.
+       *
+       * @deprecated This overload of <code>do_reinit</code> is deprecated:
+       * please use the overload with a sparsity pattern argument instead.
        */
+      DEAL_II_DEPRECATED
       void do_reinit (const size_type               m,
                       const size_type               n,
                       const size_type               local_rows,
