@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 2004 - 2017 by the deal.II authors
+// Copyright (C) 2004 - 2018 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -13,8 +13,8 @@
 //
 // ---------------------------------------------------------------------
 
-#ifndef dealii__petsc_parallel_vector_h
-#define dealii__petsc_parallel_vector_h
+#ifndef dealii_petsc_parallel_vector_h
+#define dealii_petsc_parallel_vector_h
 
 
 #include <deal.II/base/config.h>
@@ -24,6 +24,7 @@
 #  include <deal.II/base/subscriptor.h>
 #  include <deal.II/lac/exceptions.h>
 #  include <deal.II/lac/vector.h>
+#  include <deal.II/lac/vector_operation.h>
 #  include <deal.II/lac/petsc_vector_base.h>
 #  include <deal.II/base/index_set.h>
 #  include <deal.II/lac/vector_type_traits.h>
@@ -48,7 +49,7 @@ namespace PETScWrappers
 
     /**
      * Implementation of a parallel vector class based on PETSC and using MPI
-     * communication to synchronise distributed operations. All the
+     * communication to synchronize distributed operations. All the
      * functionality is actually in the base class, except for the calls to
      * generate a parallel vector. This is possible since PETSc only works on
      * an abstract vector type and internally distributes to functions that do
@@ -105,7 +106,7 @@ namespace PETScWrappers
      * time, all these additions are executed. However, if one process adds to
      * an element, and another overwrites to it, the order of execution would
      * yield non-deterministic behavior if we don't make sure that a
-     * synchronisation with compress() happens in between.
+     * synchronization with compress() happens in between.
      *
      * In order to make sure these calls to compress() happen at the
      * appropriate time, the deal.II wrappers keep a state variable that store
@@ -162,21 +163,6 @@ namespace PETScWrappers
        */
       typedef types::global_dof_index size_type;
 
-      /**
-       * A variable that indicates whether this vector supports distributed
-       * data storage. If true, then this vector also needs an appropriate
-       * compress() function that allows communicating recent set or add
-       * operations to individual elements to be communicated to other
-       * processors.
-       *
-       * For the current class, the variable equals true, since it does
-       * support parallel data storage.
-       *
-       * @deprecated instead of using this variable, please use the type trait
-       * value <code>is_serial_vector< VectorType >::value</code>
-       */
-      static const bool supports_distributed_data DEAL_II_DEPRECATED = true;
-
       /**
        * Default constructor. Initialize the vector as empty.
        */
@@ -231,23 +217,30 @@ namespace PETScWrappers
        * @deprecated The use of objects that are explicitly of type VectorBase
        * is deprecated: use PETScWrappers::MPI::Vector instead.
        */
+      DEAL_II_DEPRECATED
       explicit Vector (const MPI_Comm     &communicator,
                        const VectorBase   &v,
-                       const size_type     local_size) DEAL_II_DEPRECATED;
+                       const size_type     local_size);
 
       /**
-       * Construct a new parallel ghosted PETSc vector from an IndexSet.
-       * Note that @p local must be ascending and 1:1.
-       * The global size of the vector is determined by local.size().
-       * The global indices in @p ghost are supplied as ghost indices
-       * that can also be read locally.
+       * Construct a new parallel ghosted PETSc vector from IndexSets.
+       *
+       * Note that @p local must be ascending and 1:1, see
+       * IndexSet::is_ascending_and_one_to_one().  In particular, the DoFs in
+       * @p local need to be contiguous, meaning you can only create vectors
+       * from a DoFHandler with several finite element components if they are
+       * not reordered by component (use a PETScWrappers::BlockVector
+       * otherwise).  The global size of the vector is determined by
+       * local.size(). The global indices in @p ghost are supplied as ghost
+       * indices so that they can be read locally.
        *
        * Note that the @p ghost IndexSet may be empty and that any indices
        * already contained in @p local are ignored during construction. That
        * way, the ghost parameter can equal the set of locally relevant
        * degrees of freedom, see step-32.
        *
-       * @note This operation always creates a ghosted vector.
+       * @note This operation always creates a ghosted vector, which is considered
+       * read-only.
        *
        * @see
        * @ref GlossGhostedVector "vectors with ghost elements"
@@ -257,8 +250,15 @@ namespace PETScWrappers
               const MPI_Comm &communicator);
 
       /**
-       * Construct a new parallel PETSc vector from an IndexSet. This creates
-       * a non ghosted vector.
+       * Construct a new parallel PETSc vector without ghost elements from an
+       * IndexSet.
+       *
+       * Note that @p local must be ascending and 1:1, see
+       * IndexSet::is_ascending_and_one_to_one().  In particular, the DoFs in
+       * @p local need to be contiguous, meaning you can only create vectors
+       * from a DoFHandler with several finite element components if they are
+       * not reordered by component (use a PETScWrappers::BlockVector
+       * otherwise).
        */
       explicit Vector (const IndexSet &local,
                        const MPI_Comm &communicator);
@@ -275,23 +275,6 @@ namespace PETScWrappers
        */
       Vector &operator= (const Vector &v);
 
-      /**
-       * Copy the given sequential (non-distributed) vector into the present
-       * parallel vector. It is assumed that they have the same size, and this
-       * operation does not change the partitioning of the parallel vector by
-       * which its elements are distributed across several MPI processes. What
-       * this operation therefore does is to copy that chunk of the given
-       * vector @p v that corresponds to elements of the target vector that
-       * are stored locally, and copies them. Elements that are not stored
-       * locally are not touched.
-       *
-       * This being a parallel vector, you must make sure that @em all
-       * processes call this function at the same time. It is not possible to
-       * change the local part of a parallel vector on only one process,
-       * independent of what other processes do, with this function.
-       */
-      Vector &operator= (const PETScWrappers::Vector &v);
-
       /**
        * Set all components of the vector to the given number @p s. Simply
        * pass this down to the base class, but we still need to declare this
@@ -434,7 +417,7 @@ namespace PETScWrappers
      * the C++ standard library which uses a temporary object. The function
      * simply exchanges the data of the two vectors.
      *
-     * @relates PETScWrappers::MPI::Vector
+     * @relatesalso PETScWrappers::MPI::Vector
      * @author Wolfgang Bangerth, 2004
      */
     inline
@@ -471,67 +454,6 @@ namespace PETScWrappers
 
 
 
-    inline
-    Vector &
-    Vector::operator= (const Vector &v)
-    {
-      // make sure left- and right-hand side of the assignment are compress()'ed:
-      Assert(v.last_action == VectorOperation::unknown,
-             internal::VectorReference::ExcWrongMode (VectorOperation::unknown,
-                                                      v.last_action));
-      Assert(last_action == VectorOperation::unknown,
-             internal::VectorReference::ExcWrongMode (VectorOperation::unknown,
-                                                      last_action));
-
-
-      if (v.size()==0)
-        {
-          // this happens if v has not been initialized to something useful:
-          //   Vector x,v;
-          //   x=v;
-          // we skip the code below and create a simple serial vector of
-          // length 0
-
-#if DEAL_II_PETSC_VERSION_LT(3,2,0)
-          PetscErrorCode ierr = VecDestroy (vector);
-#else
-          PetscErrorCode ierr = VecDestroy (&vector);
-#endif
-          AssertThrow (ierr == 0, ExcPETScError(ierr));
-
-          const int n = 0;
-          ierr = VecCreateSeq (PETSC_COMM_SELF, n, &vector);
-          AssertThrow (ierr == 0, ExcPETScError(ierr));
-          ghosted = false;
-          ghost_indices.clear();
-          return *this;
-        }
-
-      // if the vectors have different sizes,
-      // then first resize the present one
-      if (size() != v.size())
-        {
-          if (v.has_ghost_elements())
-            reinit( v.locally_owned_elements(), v.ghost_indices, v.communicator);
-          else
-            reinit (v.communicator, v.size(), v.local_size(), true);
-        }
-
-      PetscErrorCode ierr = VecCopy (v.vector, vector);
-      AssertThrow (ierr == 0, ExcPETScError(ierr));
-
-      if (has_ghost_elements())
-        {
-          ierr = VecGhostUpdateBegin(vector, INSERT_VALUES, SCATTER_FORWARD);
-          AssertThrow (ierr == 0, ExcPETScError(ierr));
-          ierr = VecGhostUpdateEnd(vector, INSERT_VALUES, SCATTER_FORWARD);
-          AssertThrow (ierr == 0, ExcPETScError(ierr));
-        }
-      return *this;
-    }
-
-
-
     template <typename number>
     inline
     Vector &
@@ -589,7 +511,7 @@ namespace PETScWrappers
 
 namespace internal
 {
-  namespace LinearOperator
+  namespace LinearOperatorImplementation
   {
     template <typename> class ReinitHelper;
 
@@ -597,7 +519,7 @@ namespace internal
      * A helper class used internally in linear_operator.h. Specialization for
      * PETScWrappers::MPI::Vector.
      */
-    template<>
+    template <>
     class ReinitHelper<PETScWrappers::MPI::Vector>
     {
     public:
@@ -632,7 +554,7 @@ namespace internal
  * @author Uwe Koecher, 2017
  */
 template <>
-struct is_serial_vector< PETScWrappers::MPI::Vector > : std_cxx11::false_type
+struct is_serial_vector< PETScWrappers::MPI::Vector > : std::false_type
 {
 };
 
