@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 2004 - 2016 by the deal.II authors
+// Copyright (C) 2004 - 2018 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -13,8 +13,8 @@
 //
 // ---------------------------------------------------------------------
 
-#ifndef dealii__petsc_precondition_h
-#define dealii__petsc_precondition_h
+#ifndef dealii_petsc_precondition_h
+#define dealii_petsc_precondition_h
 
 
 #include <deal.II/base/config.h>
@@ -79,7 +79,7 @@ namespace PETScWrappers
 
 
     /**
-     * Gives access to the underlying PETSc object.
+     * Give access to the underlying PETSc object.
      */
     const PC &get_pc () const;
 
@@ -141,7 +141,7 @@ namespace PETScWrappers
      * Empty Constructor. You need to call initialize() before using this
      * object.
      */
-    PreconditionJacobi ();
+    PreconditionJacobi () = default;
 
 
     /**
@@ -159,7 +159,7 @@ namespace PETScWrappers
                         const AdditionalData &additional_data = AdditionalData());
 
     /**
-     * Initializes the preconditioner object and calculate all data that is
+     * Initialize the preconditioner object and calculate all data that is
      * necessary for applying it in a solver. This function is automatically
      * called when calling the constructor with the same arguments and is only
      * used if you create the preconditioner without arguments.
@@ -174,7 +174,7 @@ namespace PETScWrappers
     AdditionalData additional_data;
 
     /**
-     * Initializes the preconditioner object without knowing a particular
+     * Initialize the preconditioner object without knowing a particular
      * matrix. This function sets up appropriate parameters to the underlying
      * PETSc object after it has been created.
      */
@@ -185,9 +185,15 @@ namespace PETScWrappers
 
   /**
    * A class that implements the interface to use the PETSc Block Jacobi
-   * preconditioner. The blocking structure of the matrix is determined by the
-   * association of degrees of freedom to the individual processors in an MPI-
-   * parallel job. If you use this preconditioner on a sequential job (or an
+   * preconditioner. PETSc defines the term "block Jacobi" as a preconditioner
+   * in which it looks at a number of diagonal blocks of the matrix and then
+   * defines a preconditioner in which the preconditioner matrix has the same
+   * block structure as only these diagonal blocks, and each diagonal block
+   * of the preconditioner is an approximation of the inverse of the
+   * corresponding block of the original matrix.
+   * The blocking structure of the matrix is determined by the
+   * association of degrees of freedom to the individual processors in an
+   * MPI-parallel job. If you use this preconditioner on a sequential job (or an
    * MPI job with only one process) then the entire matrix is the only block.
    *
    * By default, PETSc uses an ILU(0) decomposition of each diagonal block of
@@ -215,7 +221,7 @@ namespace PETScWrappers
      * Empty Constructor. You need to call initialize() before using this
      * object.
      */
-    PreconditionBlockJacobi ();
+    PreconditionBlockJacobi () = default;
 
     /**
      * Constructor. Take the matrix which is used to form the preconditioner,
@@ -233,7 +239,7 @@ namespace PETScWrappers
 
 
     /**
-     * Initializes the preconditioner object and calculate all data that is
+     * Initialize the preconditioner object and calculate all data that is
      * necessary for applying it in a solver. This function is automatically
      * called when calling the constructor with the same arguments and is only
      * used if you create the preconditioner without arguments.
@@ -248,7 +254,7 @@ namespace PETScWrappers
     AdditionalData additional_data;
 
     /**
-     * Initializes the preconditioner object without knowing a particular
+     * Initialize the preconditioner object without knowing a particular
      * matrix. This function sets up appropriate parameters to the underlying
      * PETSc object after it has been created.
      */
@@ -291,7 +297,7 @@ namespace PETScWrappers
      * Empty Constructor. You need to call initialize() before using this
      * object.
      */
-    PreconditionSOR ();
+    PreconditionSOR () = default;
 
     /**
      * Constructor. Take the matrix which is used to form the preconditioner,
@@ -301,7 +307,7 @@ namespace PETScWrappers
                      const AdditionalData &additional_data = AdditionalData());
 
     /**
-     * Initializes the preconditioner object and calculate all data that is
+     * Initialize the preconditioner object and calculate all data that is
      * necessary for applying it in a solver. This function is automatically
      * called when calling the constructor with the same arguments and is only
      * used if you create the preconditioner without arguments.
@@ -351,7 +357,7 @@ namespace PETScWrappers
      * Empty Constructor. You need to call initialize() before using this
      * object.
      */
-    PreconditionSSOR ();
+    PreconditionSSOR () = default;
 
     /**
      * Constructor. Take the matrix which is used to form the preconditioner,
@@ -361,7 +367,7 @@ namespace PETScWrappers
                       const AdditionalData &additional_data = AdditionalData());
 
     /**
-     * Initializes the preconditioner object and calculate all data that is
+     * Initialize the preconditioner object and calculate all data that is
      * necessary for applying it in a solver. This function is automatically
      * called when calling the constructor with the same arguments and is only
      * used if you create the preconditioner without arguments.
@@ -414,7 +420,7 @@ namespace PETScWrappers
      * Empty Constructor. You need to call initialize() before using this
      * object.
      */
-    PreconditionEisenstat ();
+    PreconditionEisenstat () = default;
 
     /**
      * Constructor. Take the matrix which is used to form the preconditioner,
@@ -424,7 +430,7 @@ namespace PETScWrappers
                            const AdditionalData &additional_data = AdditionalData());
 
     /**
-     * Initializes the preconditioner object and calculate all data that is
+     * Initialize the preconditioner object and calculate all data that is
      * necessary for applying it in a solver. This function is automatically
      * called when calling the constructor with the same arguments and is only
      * used if you create the preconditioner without arguments.
@@ -474,7 +480,7 @@ namespace PETScWrappers
      * Empty Constructor. You need to call initialize() before using this
      * object.
      */
-    PreconditionICC ();
+    PreconditionICC () = default;
 
     /**
      * Constructor. Take the matrix which is used to form the preconditioner,
@@ -484,7 +490,7 @@ namespace PETScWrappers
                      const AdditionalData &additional_data = AdditionalData());
 
     /**
-     * Initializes the preconditioner object and calculate all data that is
+     * Initialize the preconditioner object and calculate all data that is
      * necessary for applying it in a solver. This function is automatically
      * called when calling the constructor with the same arguments and is only
      * used if you create the preconditioner without arguments.
@@ -534,7 +540,7 @@ namespace PETScWrappers
      * Empty Constructor. You need to call initialize() before using this
      * object.
      */
-    PreconditionILU ();
+    PreconditionILU () = default;
 
     /**
      * Constructor. Take the matrix which is used to form the preconditioner,
@@ -544,7 +550,7 @@ namespace PETScWrappers
                      const AdditionalData &additional_data = AdditionalData());
 
     /**
-     * Initializes the preconditioner object and calculate all data that is
+     * Initialize the preconditioner object and calculate all data that is
      * necessary for applying it in a solver. This function is automatically
      * called when calling the constructor with the same arguments and is only
      * used if you create the preconditioner without arguments.
@@ -562,11 +568,15 @@ namespace PETScWrappers
 
 
   /**
-   * A class that implements the interface to use the PETSc LU preconditioner.
-   * The LU decomposition is only implemented for single processor machines.
-   * It should provide a convenient interface to another direct solver.
+   * A class that implements the interface to use the PETSc LU preconditioner
+   * (@p PCLU). Unlike classes like PreconditionILU, this class usually
+   * (depending on the settings) performs an exact factorization of the
+   * matrix, so it is not necessary to wrap it in an iterative solver. This
+   * class is typically used with SolverPreOnly to get a direct
+   * solver. Alternatively, you can use PreconditionerBase::vmult() directly.
    *
-   * @note Only works in serial with a PETScWrappers::SparseMatrix.
+   * @note This is not a parallel preconditioner so it only works in serial
+   * computations with a single processor.
    *
    * @ingroup PETScWrappers
    * @author Oliver Kayser-Herold, 2004
@@ -582,7 +592,7 @@ namespace PETScWrappers
     {
       /**
        * Constructor. (Default values taken from function PCCreate_LU of the
-       * PetSC lib.)
+       * PETSc lib.)
        */
       AdditionalData (const double pivoting = 1.e-6,
                       const double zero_pivot = 1.e-12,
@@ -590,20 +600,20 @@ namespace PETScWrappers
 
       /**
        * Determines, when Pivoting is done during LU decomposition. 0.0
-       * indicates no pivoting, and 1.0 complete pivoting. Confer PetSC manual
+       * indicates no pivoting, and 1.0 complete pivoting. Confer PETSc manual
        * for more details.
        */
       double pivoting;
 
       /**
-       * Size at which smaller pivots are declared to be zero. Confer PetSC
+       * Size at which smaller pivots are declared to be zero. Confer PETSc
        * manual for more details.
        */
       double zero_pivot;
 
       /**
        * This quantity is added to the diagonal of the matrix during
-       * factorisation.
+       * factorization.
        */
       double damping;
     };
@@ -612,7 +622,7 @@ namespace PETScWrappers
      * Empty Constructor. You need to call initialize() before using this
      * object.
      */
-    PreconditionLU ();
+    PreconditionLU () = default;
 
     /**
      * Constructor. Take the matrix which is used to form the preconditioner,
@@ -622,7 +632,7 @@ namespace PETScWrappers
                     const AdditionalData &additional_data = AdditionalData());
 
     /**
-     * Initializes the preconditioner object and calculate all data that is
+     * Initialize the preconditioner object and calculate all data that is
      * necessary for applying it in a solver. This function is automatically
      * called when calling the constructor with the same arguments and is only
      * used if you create the preconditioner without arguments.
@@ -718,7 +728,7 @@ namespace PETScWrappers
      * Empty Constructor. You need to call initialize() before using this
      * object.
      */
-    PreconditionBoomerAMG ();
+    PreconditionBoomerAMG () = default;
 
     /**
      * Constructor. Take the matrix which is used to form the preconditioner,
@@ -736,7 +746,7 @@ namespace PETScWrappers
 
 
     /**
-     * Initializes the preconditioner object and calculate all data that is
+     * Initialize the preconditioner object and calculate all data that is
      * necessary for applying it in a solver. This function is automatically
      * called when calling the constructor with the same arguments and is only
      * used if you create the preconditioner without arguments.
@@ -751,7 +761,7 @@ namespace PETScWrappers
     AdditionalData additional_data;
 
     /**
-     * Initializes the preconditioner object without knowing a particular
+     * Initialize the preconditioner object without knowing a particular
      * matrix. This function sets up appropriate parameters to the underlying
      * PETSc object after it has been created.
      */
@@ -859,7 +869,7 @@ namespace PETScWrappers
      * Empty Constructor. You need to call initialize() before using this
      * object.
      */
-    PreconditionParaSails ();
+    PreconditionParaSails () = default;
 
     /**
      * Constructor. Take the matrix which is used to form the preconditioner,
@@ -869,7 +879,7 @@ namespace PETScWrappers
                            const AdditionalData &additional_data = AdditionalData());
 
     /**
-     * Initializes the preconditioner object and calculate all data that is
+     * Initialize the preconditioner object and calculate all data that is
      * necessary for applying it in a solver. This function is automatically
      * called when calling the constructor with the same arguments and is only
      * used if you create the preconditioner without arguments.
@@ -906,7 +916,7 @@ namespace PETScWrappers
      * Empty Constructor. You need to call initialize() before using this
      * object.
      */
-    PreconditionNone ();
+    PreconditionNone () = default;
 
     /**
      * Constructor. Take the matrix which is used to form the preconditioner,
@@ -917,7 +927,7 @@ namespace PETScWrappers
                       const AdditionalData &additional_data = AdditionalData());
 
     /**
-     * Initializes the preconditioner object and calculate all data that is
+     * Initialize the preconditioner object and calculate all data that is
      * necessary for applying it in a solver. This function is automatically
      * called when calling the constructor with the same arguments and is only
      * used if you create the preconditioner without arguments. The matrix is
