@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 2004 - 2017 by the deal.II authors
+// Copyright (C) 2004 - 2018 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -13,8 +13,8 @@
 //
 // ---------------------------------------------------------------------
 
-#ifndef dealii__petsc_vector_base_h
-#define dealii__petsc_vector_base_h
+#ifndef dealii_petsc_vector_base_h
+#define dealii_petsc_vector_base_h
 
 
 #include <deal.II/base/config.h>
@@ -24,6 +24,7 @@
 #  include <deal.II/base/subscriptor.h>
 #  include <deal.II/lac/exceptions.h>
 #  include <deal.II/lac/vector.h>
+#  include <deal.II/lac/vector_operation.h>
 
 #  include <vector>
 #  include <utility>
@@ -209,8 +210,10 @@ namespace PETScWrappers
    * parallel ones) are built by filling the contents of an abstract object
    * that is only referenced through a pointer of a type that is independent
    * of the actual vector type, we can implement almost all functionality of
-   * vectors in this base class. Derived classes will then only have to
-   * provide the functionality to create one or the other kind of vector.
+   * vectors in this base class. As such, this class can also be used as a
+   * deal.II-compatible wrapper for a PETSc <code>Vec</code> object of any
+   * type. Derived classes will then only have to provide the functionality to
+   * create one or the other kind of vector.
    *
    * The interface of this class is modeled after the existing Vector class in
    * deal.II. It has almost the same member functions, and is often
@@ -255,7 +258,7 @@ namespace PETScWrappers
 
     /**
      * Initialize a Vector from a PETSc Vec object. Note that we do not copy
-     * the vector and we do not attain ownership, so we do not destroy the
+     * the vector and we do not obtain ownership, so we do not destroy the
      * PETSc object in the destructor.
      */
     explicit VectorBase (const Vec &v);
@@ -367,6 +370,14 @@ namespace PETScWrappers
      */
     bool has_ghost_elements() const;
 
+    /**
+     * This function only exists for compatibility with the @p
+     * LinearAlgebra::distributed::Vector class and does nothing: this class
+     * implements ghost value updates in a different way that is a better fit
+     * with the underlying PETSc vector object.
+     */
+    void update_ghost_values () const;
+
     /**
      * Provide access to a given element, both read and write.
      */
@@ -405,17 +416,49 @@ namespace PETScWrappers
               const std::vector<PetscScalar>  &values);
 
     /**
-     * A collective get operation: instead of getting individual elements of a
-     * vector, this function allows to get a whole set of elements at once.
-     * The indices of the elements to be read are stated in the first
-     * argument, the corresponding values are returned in the second.
+     * Instead of getting individual elements of a vector via operator(),
+     * this function allows getting a whole set of elements at once. The
+     * indices of the elements to be read are stated in the first argument, the
+     * corresponding values are returned in the second.
+     *
+     * If the current vector is called @p v, then this function is the equivalent
+     * to the code
+     * @code
+     *   for (unsigned int i=0; i<indices.size(); ++i)
+     *     values[i] = v[indices[i]];
+     * @endcode
+     *
+     * @pre The sizes of the @p indices and @p values arrays must be identical.
      */
     void extract_subvector_to (const std::vector<size_type> &indices,
                                std::vector<PetscScalar> &values) const;
 
     /**
-     * Just as the above, but with pointers. Useful in minimizing copying of
-     * data around.
+     * Instead of getting individual elements of a vector via operator(),
+     * this function allows getting a whole set of elements at once. In
+     * contrast to the previous function, this function obtains the
+     * indices of the elements by dereferencing all elements of the iterator
+     * range provided by the first two arguments, and puts the vector
+     * values into memory locations obtained by dereferencing a range
+     * of iterators starting at the location pointed to by the third
+     * argument.
+     *
+     * If the current vector is called @p v, then this function is the equivalent
+     * to the code
+     * @code
+     *   ForwardIterator indices_p = indices_begin;
+     *   OutputIterator  values_p  = values_begin;
+     *   while (indices_p != indices_end)
+     *   {
+     *     *values_p = v[*indices_p];
+     *     ++indices_p;
+     *     ++values_p;
+     *   }
+     * @endcode
+     *
+     * @pre It must be possible to write into as many memory locations
+     *   starting at @p values_begin as there are iterators between
+     *   @p indices_begin and @p indices_end.
      */
     template <typename ForwardIterator, typename OutputIterator>
     void extract_subvector_to (const ForwardIterator    indices_begin,
@@ -454,7 +497,7 @@ namespace PETScWrappers
     PetscScalar operator * (const VectorBase &vec) const;
 
     /**
-     * Return square of the $l_2$-norm.
+     * Return the square of the $l_2$-norm.
      */
     real_type norm_sqr () const;
 
@@ -504,19 +547,14 @@ namespace PETScWrappers
      * transfer. However, for PETSc vectors such a combined operation is not
      * natively supported and thus the cost is completely equivalent as
      * calling the two methods separately.
+     *
+     * For complex-valued vectors, the scalar product in the second step is implemented as
+     * $\left<v,w\right>=\sum_i v_i \bar{w_i}$.
      */
     PetscScalar add_and_dot (const PetscScalar a,
                              const VectorBase &V,
                              const VectorBase &W);
 
-    /**
-     * Normalize vector by dividing by the $l_2$-norm of the vector. Return
-     * the vector norm before normalization.
-     *
-     * This function is deprecated.
-     */
-    real_type normalize () const DEAL_II_DEPRECATED;
-
     /**
      * Return the value of the vector element with the largest negative value.
      */
@@ -527,46 +565,6 @@ namespace PETScWrappers
      */
     real_type max () const;
 
-    /**
-     * Replace every element in a vector with its absolute value.
-     *
-     * This function is deprecated.
-     */
-    VectorBase &abs () DEAL_II_DEPRECATED;
-
-    /**
-     * Conjugate a vector.
-     *
-     * This function is deprecated.
-     */
-    VectorBase &conjugate () DEAL_II_DEPRECATED;
-
-    /**
-     * A collective piecewise multiply operation on <code>this</code> vector
-     * with itself. TODO: The model for this function should be similar to add
-     * ().
-     *
-     * This function is deprecated.
-     */
-    VectorBase &mult () DEAL_II_DEPRECATED;
-
-    /**
-     * Same as above, but a collective piecewise multiply operation of
-     * <code>this</code> vector with <b>v</b>.
-     *
-     * This function is deprecated.
-     */
-    VectorBase &mult (const VectorBase &v) DEAL_II_DEPRECATED;
-
-    /**
-     * Same as above, but a collective piecewise multiply operation of
-     * <b>u</b> with <b>v</b>.
-     *
-     * This function is deprecated.
-     */
-    VectorBase &mult (const VectorBase &u,
-                      const VectorBase &v) DEAL_II_DEPRECATED;
-
     /**
      * Return whether the vector contains only elements with value zero. This
      * is a collective operation. This function is expensive, because
@@ -607,13 +605,6 @@ namespace PETScWrappers
      */
     void add (const PetscScalar s);
 
-    /**
-     * Simple vector addition, equal to the <tt>operator +=</tt>.
-     *
-     * @deprecated Use the <tt>operator +=</tt> instead.
-     */
-    void add (const VectorBase &V) DEAL_II_DEPRECATED;
-
     /**
      * Simple addition of a multiple of a vector, i.e. <tt>*this += a*V</tt>.
      */
@@ -638,31 +629,6 @@ namespace PETScWrappers
                const PetscScalar a,
                const VectorBase     &V);
 
-    /**
-     * Scaling and multiple addition.
-     *
-     * This function is deprecated.
-     */
-    void sadd (const PetscScalar s,
-               const PetscScalar a,
-               const VectorBase     &V,
-               const PetscScalar b,
-               const VectorBase     &W) DEAL_II_DEPRECATED;
-
-    /**
-     * Scaling and multiple addition. <tt>*this = s*(*this)+a*V + b*W +
-     * c*X</tt>.
-     *
-     * This function is deprecated.
-     */
-    void sadd (const PetscScalar s,
-               const PetscScalar a,
-               const VectorBase     &V,
-               const PetscScalar b,
-               const VectorBase     &W,
-               const PetscScalar c,
-               const VectorBase     &X) DEAL_II_DEPRECATED;
-
     /**
      * Scale each element of this vector by the corresponding element in the
      * argument. This function is mostly meant to simulate multiplication (and
@@ -675,14 +641,6 @@ namespace PETScWrappers
      */
     void equ (const PetscScalar a, const VectorBase &V);
 
-    /**
-     * Assignment <tt>*this = a*V + b*W</tt>.
-     *
-     * This function is deprecated.
-     */
-    void equ (const PetscScalar a, const VectorBase &V,
-              const PetscScalar b, const VectorBase &W) DEAL_II_DEPRECATED;
-
     /**
      * Compute the elementwise ratio of the two given vectors, that is let
      * <tt>this[i] = a[i]/b[i]</tt>. This is useful for example if you want to
@@ -693,8 +651,9 @@ namespace PETScWrappers
      * If any of the <tt>b[i]</tt> is zero, the result is undefined. No
      * attempt is made to catch such situations.
      */
+    DEAL_II_DEPRECATED
     void ratio (const VectorBase &a,
-                const VectorBase &b) DEAL_II_DEPRECATED;
+                const VectorBase &b);
 
     /**
      * Prints the PETSc vector object values using PETSc internal vector
@@ -724,7 +683,7 @@ namespace PETScWrappers
      * only swaps the pointers to the data of the two vectors and therefore
      * does not need to allocate temporary storage and move data around.
      *
-     * This function is analogous to the the @p swap function of all C++
+     * This function is analogous to the @p swap function of all C++
      * standard containers. Also, there is a global function
      * <tt>swap(u,v)</tt> that simply calls <tt>u.swap(v)</tt>, again in
      * analogy to standard functions.
@@ -786,10 +745,10 @@ namespace PETScWrappers
 
     /**
      * Specifies if the vector is the owner of the PETSc Vec. This is true if
-     * it got created by this class and determines if it gets destructed in
+     * it got created by this class and determines if it gets destroyed in
      * the destructor.
      */
-    bool attained_ownership;
+    bool obtained_ownership;
 
     /**
      * Collective set or add operation: This function is invoked by the
@@ -807,7 +766,7 @@ namespace PETScWrappers
      * deliberately left as private (and undefined) to prevent accidental
      * usage.
      */
-    VectorBase &operator=(const VectorBase &);
+    VectorBase &operator=(const VectorBase &) = delete;
   };
 
 
@@ -819,7 +778,7 @@ namespace PETScWrappers
    * C++ standard library which uses a temporary object. The function simply
    * exchanges the data of the two vectors.
    *
-   * @relates PETScWrappers::VectorBase
+   * @relatesalso PETScWrappers::VectorBase
    * @author Wolfgang Bangerth, 2004
    */
   inline
@@ -1105,6 +1064,13 @@ namespace PETScWrappers
 
 
 
+  inline
+  void
+  VectorBase::update_ghost_values () const
+  {}
+
+
+
   inline
   internal::VectorReference
   VectorBase::operator () (const size_type index)
@@ -1190,7 +1156,7 @@ namespace PETScWrappers
         PetscErrorCode ierr = VecGetOwnershipRange (vector, &begin, &end);
         AssertThrow (ierr == 0, ExcPETScError(ierr));
 
-        Vec locally_stored_elements = PETSC_NULL;
+        Vec locally_stored_elements = nullptr;
         ierr = VecGhostGetLocalForm(vector, &locally_stored_elements);
         AssertThrow (ierr == 0, ExcPETScError(ierr));
 
