@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 1999 - 2017 by the deal.II authors
+// Copyright (C) 1999 - 2018 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -13,8 +13,8 @@
 //
 // ---------------------------------------------------------------------
 
-#ifndef dealii__precondition_h
-#define dealii__precondition_h
+#ifndef dealii_precondition_h
+#define dealii_precondition_h
 
 // This file contains simple preconditioners.
 
@@ -87,7 +87,7 @@ public:
     /**
      * Constructor.
      */
-    AdditionalData () {}
+    AdditionalData () = default;
   };
 
   /**
@@ -106,27 +106,27 @@ public:
   /**
    * Apply preconditioner.
    */
-  template<class VectorType>
+  template <class VectorType>
   void vmult (VectorType &, const VectorType &) const;
 
   /**
    * Apply transpose preconditioner. Since this is the identity, this function
    * is the same as vmult().
    */
-  template<class VectorType>
+  template <class VectorType>
   void Tvmult (VectorType &, const VectorType &) const;
 
   /**
    * Apply preconditioner, adding to the previous value.
    */
-  template<class VectorType>
+  template <class VectorType>
   void vmult_add (VectorType &, const VectorType &) const;
 
   /**
    * Apply transpose preconditioner, adding. Since this is the identity, this
    * function is the same as vmult_add().
    */
-  template<class VectorType>
+  template <class VectorType>
   void Tvmult_add (VectorType &, const VectorType &) const;
 
   /**
@@ -229,26 +229,26 @@ public:
   /**
    * Apply preconditioner.
    */
-  template<class VectorType>
+  template <class VectorType>
   void vmult (VectorType &, const VectorType &) const;
 
   /**
    * Apply transpose preconditioner. Since this is the identity, this function
    * is the same as vmult().
    */
-  template<class VectorType>
+  template <class VectorType>
   void Tvmult (VectorType &, const VectorType &) const;
   /**
    * Apply preconditioner, adding to the previous value.
    */
-  template<class VectorType>
+  template <class VectorType>
   void vmult_add (VectorType &, const VectorType &) const;
 
   /**
    * Apply transpose preconditioner, adding. Since this is the identity, this
    * function is the same as vmult_add().
    */
-  template<class VectorType>
+  template <class VectorType>
   void Tvmult_add (VectorType &, const VectorType &) const;
 
   /**
@@ -334,7 +334,7 @@ private:
  *
  * @author Guido Kanschat, Wolfgang Bangerth, 1999
  */
-template<typename MatrixType = SparseMatrix<double>, class VectorType = Vector<double> >
+template <typename MatrixType = SparseMatrix<double>, class VectorType = Vector<double> >
 class PreconditionUseMatrix : public Subscriptor
 {
 public:
@@ -380,7 +380,7 @@ private:
  * @author Guido Kanschat, 2000; extension for full compatibility with
  * LinearOperator class: Jean-Paul Pelteret, 2015
  */
-template<typename MatrixType = SparseMatrix<double> >
+template <typename MatrixType = SparseMatrix<double> >
 class PreconditionRelaxation : public Subscriptor
 {
 public:
@@ -484,26 +484,26 @@ public:
   /**
    * Apply preconditioner.
    */
-  template<class VectorType>
+  template <class VectorType>
   void vmult (VectorType &, const VectorType &) const;
 
   /**
    * Apply transpose preconditioner. Since this is a symmetric preconditioner,
    * this function is the same as vmult().
    */
-  template<class VectorType>
+  template <class VectorType>
   void Tvmult (VectorType &, const VectorType &) const;
 
   /**
    * Perform one step of the preconditioned Richardson iteration.
    */
-  template<class VectorType>
+  template <class VectorType>
   void step (VectorType &x, const VectorType &rhs) const;
 
   /**
    * Perform one transposed step of the preconditioned Richardson iteration.
    */
-  template<class VectorType>
+  template <class VectorType>
   void Tstep (VectorType &x, const VectorType &rhs) const;
 };
 
@@ -566,25 +566,25 @@ public:
   /**
    * Apply preconditioner.
    */
-  template<class VectorType>
+  template <class VectorType>
   void vmult (VectorType &, const VectorType &) const;
 
   /**
    * Apply transpose preconditioner.
    */
-  template<class VectorType>
+  template <class VectorType>
   void Tvmult (VectorType &, const VectorType &) const;
 
   /**
    * Perform one step of the preconditioned Richardson iteration.
    */
-  template<class VectorType>
+  template <class VectorType>
   void step (VectorType &x, const VectorType &rhs) const;
 
   /**
    * Perform one transposed step of the preconditioned Richardson iteration.
    */
-  template<class VectorType>
+  template <class VectorType>
   void Tstep (VectorType &x, const VectorType &rhs) const;
 };
 
@@ -648,27 +648,27 @@ public:
   /**
    * Apply preconditioner.
    */
-  template<class VectorType>
+  template <class VectorType>
   void vmult (VectorType &, const VectorType &) const;
 
   /**
    * Apply transpose preconditioner. Since this is a symmetric preconditioner,
    * this function is the same as vmult().
    */
-  template<class VectorType>
+  template <class VectorType>
   void Tvmult (VectorType &, const VectorType &) const;
 
 
   /**
    * Perform one step of the preconditioned Richardson iteration
    */
-  template<class VectorType>
+  template <class VectorType>
   void step (VectorType &x, const VectorType &rhs) const;
 
   /**
    * Perform one transposed step of the preconditioned Richardson iteration.
    */
-  template<class VectorType>
+  template <class VectorType>
   void Tstep (VectorType &x, const VectorType &rhs) const;
 
 private:
@@ -789,13 +789,13 @@ public:
   /**
    * Apply preconditioner.
    */
-  template<class VectorType>
+  template <class VectorType>
   void vmult (VectorType &, const VectorType &) const;
 
   /**
    * Apply transpose preconditioner.
    */
-  template<class VectorType>
+  template <class VectorType>
   void Tvmult (VectorType &, const VectorType &) const;
 private:
   /**
@@ -908,7 +908,6 @@ public:
   typedef types::global_dof_index size_type;
 
   // avoid warning about use of deprecated variables
-  DEAL_II_DISABLE_EXTRA_DIAGNOSTICS
 
   /**
    * Standardized data struct to pipe additional parameters to the
@@ -996,10 +995,9 @@ public:
     /**
      * Stores the preconditioner object that the Chebyshev is wrapped around.
      */
-    std_cxx11::shared_ptr<PreconditionerType> preconditioner;
+    std::shared_ptr<PreconditionerType> preconditioner;
   };
 
-  DEAL_II_ENABLE_EXTRA_DIAGNOSTICS
 
   PreconditionChebyshev ();
 
@@ -1103,7 +1101,7 @@ private:
    * Stores whether the preconditioner has been set up and eigenvalues have
    * been computed.
    */
-  bool is_initialized;
+  bool eigenvalues_are_initialized;
 
   /**
    * A mutex to avoid that multiple vmult() invocations by different threads
@@ -1160,7 +1158,7 @@ PreconditionIdentity::initialize (const MatrixType &matrix,
 }
 
 
-template<class VectorType>
+template <class VectorType>
 inline void
 PreconditionIdentity::vmult (VectorType &dst, const VectorType &src) const
 {
@@ -1169,14 +1167,14 @@ PreconditionIdentity::vmult (VectorType &dst, const VectorType &src) const
 
 
 
-template<class VectorType>
+template <class VectorType>
 inline void
 PreconditionIdentity::Tvmult (VectorType &dst, const VectorType &src) const
 {
   dst = src;
 }
 
-template<class VectorType>
+template <class VectorType>
 inline void
 PreconditionIdentity::vmult_add (VectorType &dst, const VectorType &src) const
 {
@@ -1185,7 +1183,7 @@ PreconditionIdentity::vmult_add (VectorType &dst, const VectorType &src) const
 
 
 
-template<class VectorType>
+template <class VectorType>
 inline void
 PreconditionIdentity::Tvmult_add (VectorType &dst, const VectorType &src) const
 {
@@ -1250,58 +1248,50 @@ PreconditionRichardson::initialize
 
 
 
-template<class VectorType>
+template <class VectorType>
 inline void
 PreconditionRichardson::vmult (VectorType &dst, const VectorType &src) const
 {
-#ifdef DEAL_II_WITH_CXX11
   static_assert(
     std::is_same<size_type, typename VectorType::size_type>::value,
     "PreconditionRichardson and VectorType must have the same size_type.");
-#endif // DEAL_II_WITH_CXX11
 
   dst.equ(relaxation,src);
 }
 
 
 
-template<class VectorType>
+template <class VectorType>
 inline void
 PreconditionRichardson::Tvmult (VectorType &dst, const VectorType &src) const
 {
-#ifdef DEAL_II_WITH_CXX11
   static_assert(
     std::is_same<size_type, typename VectorType::size_type>::value,
     "PreconditionRichardson and VectorType must have the same size_type.");
-#endif // DEAL_II_WITH_CXX11
 
   dst.equ(relaxation,src);
 }
 
-template<class VectorType>
+template <class VectorType>
 inline void
 PreconditionRichardson::vmult_add (VectorType &dst, const VectorType &src) const
 {
-#ifdef DEAL_II_WITH_CXX11
   static_assert(
     std::is_same<size_type, typename VectorType::size_type>::value,
     "PreconditionRichardson and VectorType must have the same size_type.");
-#endif // DEAL_II_WITH_CXX11
 
   dst.add(relaxation,src);
 }
 
 
 
-template<class VectorType>
+template <class VectorType>
 inline void
 PreconditionRichardson::Tvmult_add (VectorType &dst, const VectorType &src) const
 {
-#ifdef DEAL_II_WITH_CXX11
   static_assert(
     std::is_same<size_type, typename VectorType::size_type>::value,
     "PreconditionRichardson and VectorType must have the same size_type.");
-#endif // DEAL_II_WITH_CXX11
 
   dst.add(relaxation,src);
 }
@@ -1336,14 +1326,14 @@ template <typename MatrixType>
 inline void
 PreconditionRelaxation<MatrixType>::clear ()
 {
-  A = 0;
+  A = nullptr;
 }
 
 template <typename MatrixType>
 inline typename PreconditionRelaxation<MatrixType>::size_type
 PreconditionRelaxation<MatrixType>::m () const
 {
-  Assert (A!=0, ExcNotInitialized());
+  Assert (A!=nullptr, ExcNotInitialized());
   return A->m();
 }
 
@@ -1351,73 +1341,65 @@ template <typename MatrixType>
 inline typename PreconditionRelaxation<MatrixType>::size_type
 PreconditionRelaxation<MatrixType>::n () const
 {
-  Assert (A!=0, ExcNotInitialized());
+  Assert (A!=nullptr, ExcNotInitialized());
   return A->n();
 }
 
 //---------------------------------------------------------------------------
 
 template <typename MatrixType>
-template<class VectorType>
+template <class VectorType>
 inline void
 PreconditionJacobi<MatrixType>::vmult (VectorType &dst, const VectorType &src) const
 {
-#ifdef DEAL_II_WITH_CXX11
   static_assert(
     std::is_same<typename PreconditionJacobi<MatrixType>::size_type, typename VectorType::size_type>::value,
     "PreconditionJacobi and VectorType must have the same size_type.");
-#endif // DEAL_II_WITH_CXX11
 
-  Assert (this->A!=0, ExcNotInitialized());
+  Assert (this->A!=nullptr, ExcNotInitialized());
   this->A->precondition_Jacobi (dst, src, this->relaxation);
 }
 
 
 
 template <typename MatrixType>
-template<class VectorType>
+template <class VectorType>
 inline void
 PreconditionJacobi<MatrixType>::Tvmult (VectorType &dst, const VectorType &src) const
 {
-#ifdef DEAL_II_WITH_CXX11
   static_assert(
     std::is_same<typename PreconditionJacobi<MatrixType>::size_type, typename VectorType::size_type>::value,
     "PreconditionJacobi and VectorType must have the same size_type.");
-#endif // DEAL_II_WITH_CXX11
 
-  Assert (this->A!=0, ExcNotInitialized());
+  Assert (this->A!=nullptr, ExcNotInitialized());
   this->A->precondition_Jacobi (dst, src, this->relaxation);
 }
 
 
 
 template <typename MatrixType>
-template<class VectorType>
+template <class VectorType>
 inline void
 PreconditionJacobi<MatrixType>::step (VectorType &dst, const VectorType &src) const
 {
-#ifdef DEAL_II_WITH_CXX11
   static_assert(
     std::is_same<typename PreconditionJacobi<MatrixType>::size_type, typename VectorType::size_type>::value,
     "PreconditionJacobi and VectorType must have the same size_type.");
-#endif // DEAL_II_WITH_CXX11
 
-  Assert (this->A!=0, ExcNotInitialized());
+  Assert (this->A!=nullptr, ExcNotInitialized());
   this->A->Jacobi_step (dst, src, this->relaxation);
 }
 
 
 
 template <typename MatrixType>
-template<class VectorType>
+template <class VectorType>
 inline void
 PreconditionJacobi<MatrixType>::Tstep (VectorType &dst, const VectorType &src) const
 {
-#ifdef DEAL_II_WITH_CXX11
   static_assert(
     std::is_same<typename PreconditionJacobi<MatrixType>::size_type, typename VectorType::size_type>::value,
     "PreconditionJacobi and VectorType must have the same size_type.");
-#endif // DEAL_II_WITH_CXX11
 
   step (dst, src);
 }
@@ -1427,68 +1409,60 @@ PreconditionJacobi<MatrixType>::Tstep (VectorType &dst, const VectorType &src) c
 //---------------------------------------------------------------------------
 
 template <typename MatrixType>
-template<class VectorType>
+template <class VectorType>
 inline void
 PreconditionSOR<MatrixType>::vmult (VectorType &dst, const VectorType &src) const
 {
-#ifdef DEAL_II_WITH_CXX11
   static_assert(
     std::is_same<typename PreconditionSOR<MatrixType>::size_type, typename VectorType::size_type>::value,
     "PreconditionSOR and VectorType must have the same size_type.");
-#endif // DEAL_II_WITH_CXX11
 
-  Assert (this->A!=0, ExcNotInitialized());
+  Assert (this->A!=nullptr, ExcNotInitialized());
   this->A->precondition_SOR (dst, src, this->relaxation);
 }
 
 
 
 template <typename MatrixType>
-template<class VectorType>
+template <class VectorType>
 inline void
 PreconditionSOR<MatrixType>::Tvmult (VectorType &dst, const VectorType &src) const
 {
-#ifdef DEAL_II_WITH_CXX11
   static_assert(
     std::is_same<typename PreconditionSOR<MatrixType>::size_type, typename VectorType::size_type>::value,
     "PreconditionSOR and VectorType must have the same size_type.");
-#endif // DEAL_II_WITH_CXX11
 
-  Assert (this->A!=0, ExcNotInitialized());
+  Assert (this->A!=nullptr, ExcNotInitialized());
   this->A->precondition_TSOR (dst, src, this->relaxation);
 }
 
 
 
 template <typename MatrixType>
-template<class VectorType>
+template <class VectorType>
 inline void
 PreconditionSOR<MatrixType>::step (VectorType &dst, const VectorType &src) const
 {
-#ifdef DEAL_II_WITH_CXX11
   static_assert(
     std::is_same<typename PreconditionSOR<MatrixType>::size_type, typename VectorType::size_type>::value,
     "PreconditionSOR and VectorType must have the same size_type.");
-#endif // DEAL_II_WITH_CXX11
 
-  Assert (this->A!=0, ExcNotInitialized());
+  Assert (this->A!=nullptr, ExcNotInitialized());
   this->A->SOR_step (dst, src, this->relaxation);
 }
 
 
 
 template <typename MatrixType>
-template<class VectorType>
+template <class VectorType>
 inline void
 PreconditionSOR<MatrixType>::Tstep (VectorType &dst, const VectorType &src) const
 {
-#ifdef DEAL_II_WITH_CXX11
   static_assert(
     std::is_same<typename PreconditionSOR<MatrixType>::size_type, typename VectorType::size_type>::value,
     "PreconditionSOR and VectorType must have the same size_type.");
-#endif // DEAL_II_WITH_CXX11
 
-  Assert (this->A!=0, ExcNotInitialized());
+  Assert (this->A!=nullptr, ExcNotInitialized());
   this->A->TSOR_step (dst, src, this->relaxation);
 }
 
@@ -1509,7 +1483,7 @@ PreconditionSSOR<MatrixType>::initialize (const MatrixType                     &
     dynamic_cast<const SparseMatrix<typename MatrixType::value_type> *>(&*this->A);
 
   // calculate the positions first after the diagonal.
-  if (mat != 0)
+  if (mat != nullptr)
     {
       const size_type n = this->A->n();
       pos_right_of_diagonal.resize(n, static_cast<std::size_t>(-1));
@@ -1531,66 +1505,58 @@ PreconditionSSOR<MatrixType>::initialize (const MatrixType                     &
 
 
 template <typename MatrixType>
-template<class VectorType>
+template <class VectorType>
 inline void
 PreconditionSSOR<MatrixType>::vmult (VectorType &dst, const VectorType &src) const
 {
-#ifdef DEAL_II_WITH_CXX11
   static_assert(
     std::is_same<typename PreconditionSSOR<MatrixType>::size_type, typename VectorType::size_type>::value,
     "PreconditionSSOR and VectorType must have the same size_type.");
-#endif // DEAL_II_WITH_CXX11
 
-  Assert (this->A!=0, ExcNotInitialized());
+  Assert (this->A!=nullptr, ExcNotInitialized());
   this->A->precondition_SSOR (dst, src, this->relaxation, pos_right_of_diagonal);
 }
 
 
 
 template <typename MatrixType>
-template<class VectorType>
+template <class VectorType>
 inline void
 PreconditionSSOR<MatrixType>::Tvmult (VectorType &dst, const VectorType &src) const
 {
-#ifdef DEAL_II_WITH_CXX11
   static_assert(
     std::is_same<typename PreconditionSSOR<MatrixType>::size_type, typename VectorType::size_type>::value,
     "PreconditionSSOR and VectorType must have the same size_type.");
-#endif // DEAL_II_WITH_CXX11
 
-  Assert (this->A!=0, ExcNotInitialized());
+  Assert (this->A!=nullptr, ExcNotInitialized());
   this->A->precondition_SSOR (dst, src, this->relaxation, pos_right_of_diagonal);
 }
 
 
 
 template <typename MatrixType>
-template<class VectorType>
+template <class VectorType>
 inline void
 PreconditionSSOR<MatrixType>::step (VectorType &dst, const VectorType &src) const
 {
-#ifdef DEAL_II_WITH_CXX11
   static_assert(
     std::is_same<typename PreconditionSSOR<MatrixType>::size_type, typename VectorType::size_type>::value,
     "PreconditionSSOR and VectorType must have the same size_type.");
-#endif // DEAL_II_WITH_CXX11
 
-  Assert (this->A!=0, ExcNotInitialized());
+  Assert (this->A!=nullptr, ExcNotInitialized());
   this->A->SSOR_step (dst, src, this->relaxation);
 }
 
 
 
 template <typename MatrixType>
-template<class VectorType>
+template <class VectorType>
 inline void
 PreconditionSSOR<MatrixType>::Tstep (VectorType &dst, const VectorType &src) const
 {
-#ifdef DEAL_II_WITH_CXX11
   static_assert(
     std::is_same<typename PreconditionSSOR<MatrixType>::size_type, typename VectorType::size_type>::value,
     "PreconditionSSOR and VectorType must have the same size_type.");
-#endif // DEAL_II_WITH_CXX11
 
   step (dst, src);
 }
@@ -1630,13 +1596,11 @@ template <typename VectorType>
 inline void
 PreconditionPSOR<MatrixType>::vmult (VectorType &dst, const VectorType &src) const
 {
-#ifdef DEAL_II_WITH_CXX11
   static_assert(
     std::is_same<typename PreconditionPSOR<MatrixType>::size_type, typename VectorType::size_type>::value,
     "PreconditionPSOR and VectorType must have the same size_type.");
-#endif // DEAL_II_WITH_CXX11
 
-  Assert (this->A!=0, ExcNotInitialized());
+  Assert (this->A!=nullptr, ExcNotInitialized());
   dst = src;
   this->A->PSOR (dst, *permutation, *inverse_permutation, this->relaxation);
 }
@@ -1644,17 +1608,15 @@ PreconditionPSOR<MatrixType>::vmult (VectorType &dst, const VectorType &src) con
 
 
 template <typename MatrixType>
-template<class VectorType>
+template <class VectorType>
 inline void
 PreconditionPSOR<MatrixType>::Tvmult (VectorType &dst, const VectorType &src) const
 {
-#ifdef DEAL_II_WITH_CXX11
   static_assert(
     std::is_same<typename PreconditionPSOR<MatrixType>::size_type, typename VectorType::size_type>::value,
     "PreconditionPSOR and VectorType must have the same size_type.");
-#endif // DEAL_II_WITH_CXX11
 
-  Assert (this->A!=0, ExcNotInitialized());
+  Assert (this->A!=nullptr, ExcNotInitialized());
   dst = src;
   this->A->TPSOR (dst, *permutation, *inverse_permutation, this->relaxation);
 }
@@ -1676,7 +1638,7 @@ PreconditionPSOR<MatrixType>::AdditionalData::AdditionalData
 //---------------------------------------------------------------------------
 
 
-template<typename MatrixType, class VectorType>
+template <typename MatrixType, class VectorType>
 PreconditionUseMatrix<MatrixType,VectorType>::PreconditionUseMatrix(const MatrixType   &M,
     const function_ptr method)
   :
@@ -1685,7 +1647,7 @@ PreconditionUseMatrix<MatrixType,VectorType>::PreconditionUseMatrix(const Matrix
 
 
 
-template<typename MatrixType, class VectorType>
+template <typename MatrixType, class VectorType>
 void
 PreconditionUseMatrix<MatrixType,VectorType>::vmult (VectorType       &dst,
                                                      const VectorType &src) const
@@ -1695,7 +1657,7 @@ PreconditionUseMatrix<MatrixType,VectorType>::vmult (VectorType       &dst,
 
 //---------------------------------------------------------------------------
 
-template<typename MatrixType>
+template <typename MatrixType>
 inline
 PreconditionRelaxation<MatrixType>::AdditionalData::
 AdditionalData (const double relaxation)
@@ -1709,7 +1671,7 @@ AdditionalData (const double relaxation)
 
 namespace internal
 {
-  namespace PreconditionChebyshev
+  namespace PreconditionChebyshevImplementation
   {
     // for deal.II vectors, perform updates for Chebyshev preconditioner all
     // at once to reduce memory transfer. Here, we select between general
@@ -1826,7 +1788,7 @@ namespace internal
       mutable Number *dst;
     };
 
-    template<typename Number>
+    template <typename Number>
     struct VectorUpdatesRange : public parallel::ParallelForInteger
     {
       VectorUpdatesRange(const VectorUpdater<Number> &updater,
@@ -1834,14 +1796,14 @@ namespace internal
         :
         updater (updater)
       {
-        if (size < internal::Vector::minimum_parallel_grain_size)
+        if (size < internal::VectorImplementation::minimum_parallel_grain_size)
           apply_to_subrange (0, size);
         else
           apply_parallel (0, size,
-                          internal::Vector::minimum_parallel_grain_size);
+                          internal::VectorImplementation::minimum_parallel_grain_size);
       }
 
-      ~VectorUpdatesRange() {}
+      ~VectorUpdatesRange() = default;
 
       virtual void
       apply_to_subrange (const std::size_t begin,
@@ -1897,26 +1859,26 @@ namespace internal
     inline
     void
     initialize_preconditioner(const MatrixType                          &matrix,
-                              std_cxx11::shared_ptr<PreconditionerType> &preconditioner,
+                              std::shared_ptr<PreconditionerType> &preconditioner,
                               VectorType &)
     {
       (void)matrix;
       (void)preconditioner;
-      AssertThrow(preconditioner.get() != NULL, ExcNotInitialized());
+      AssertThrow(preconditioner.get() != nullptr, ExcNotInitialized());
     }
 
     template <typename MatrixType, typename VectorType>
     inline
     void
     initialize_preconditioner(const MatrixType                                   &matrix,
-                              std_cxx11::shared_ptr<DiagonalMatrix<VectorType> > &preconditioner,
+                              std::shared_ptr<DiagonalMatrix<VectorType> > &preconditioner,
                               VectorType                                         &diagonal_inverse)
     {
-      if (preconditioner.get() == NULL ||
+      if (preconditioner.get() == nullptr ||
           preconditioner->m() != matrix.m())
         {
-          if (preconditioner.get() == NULL)
-            preconditioner.reset(new DiagonalMatrix<VectorType>());
+          if (preconditioner.get() == nullptr)
+            preconditioner = std::make_shared<DiagonalMatrix<VectorType>>();
 
           Assert(preconditioner->m() == 0,
                  ExcMessage("Preconditioner appears to be initialized but not sized correctly"));
@@ -1956,6 +1918,9 @@ namespace internal
       // a period of 11
       for (unsigned int i=0; i<vector.size(); ++i)
         vector(i) = i%11;
+
+      const Number mean_value = vector.mean_value();
+      vector.add(-mean_value);
     }
 
     template <typename Number>
@@ -1964,9 +1929,17 @@ namespace internal
       // Choose a high-frequency mode consisting of numbers between 0 and 1
       // that is cheap to compute (cheaper than random numbers) but avoids
       // obviously re-occurring numbers in multi-component systems by choosing
-      // a period of 11
+      // a period of 11.
+      // Make initial guess robust with respect to number of processors
+      // by operating on the global index.
+      types::global_dof_index first_local_range = 0;
+      if (!vector.locally_owned_elements().is_empty())
+        first_local_range = vector.locally_owned_elements().nth_index_in_set(0);
       for (unsigned int i=0; i<vector.local_size(); ++i)
-        vector.local_element(i) = i%11;
+        vector.local_element(i) = (i+first_local_range)%11;
+
+      const Number mean_value = vector.mean_value();
+      vector.add(-mean_value);
     }
 
     struct EigenvalueTracker
@@ -1985,7 +1958,6 @@ namespace internal
 
 
 // avoid warning about deprecated variable nonzero_starting
-DEAL_II_DISABLE_EXTRA_DIAGNOSTICS
 
 template <typename MatrixType, class VectorType, typename PreconditionerType>
 inline
@@ -2005,27 +1977,23 @@ AdditionalData (const unsigned int degree,
   max_eigenvalue (max_eigenvalue)
 {}
 
-DEAL_II_ENABLE_EXTRA_DIAGNOSTICS
 
 
 template <typename MatrixType, typename VectorType, typename PreconditionerType>
 inline
 PreconditionChebyshev<MatrixType,VectorType,PreconditionerType>::PreconditionChebyshev ()
   :
-  theta          (1.),
-  delta          (1.),
-  is_initialized (false)
+  theta                       (1.),
+  delta                       (1.),
+  eigenvalues_are_initialized (false)
 {
-#ifdef DEAL_II_WITH_CXX11
   static_assert(
     std::is_same<size_type, typename VectorType::size_type>::value,
     "PreconditionChebyshev and VectorType must have the same size_type.");
-#endif // DEAL_II_WITH_CXX11
 }
 
 
 // avoid warning about deprecated variable AdditionalData::matrix_diagonal_inverse
-DEAL_II_DISABLE_EXTRA_DIAGNOSTICS
 
 template <typename MatrixType, typename VectorType, typename PreconditionerType>
 inline
@@ -2036,10 +2004,10 @@ PreconditionChebyshev<MatrixType,VectorType,PreconditionerType>::initialize
 {
   matrix_ptr = &matrix;
   data = additional_data;
-  internal::PreconditionChebyshev::initialize_preconditioner(matrix,
-                                                             data.preconditioner,
-                                                             data.matrix_diagonal_inverse);
-  is_initialized = false;
+  internal::PreconditionChebyshevImplementation::initialize_preconditioner(matrix,
+      data.preconditioner,
+      data.matrix_diagonal_inverse);
+  eigenvalues_are_initialized = false;
 }
 
 
@@ -2049,9 +2017,9 @@ inline
 void
 PreconditionChebyshev<MatrixType,VectorType,PreconditionerType>::clear ()
 {
-  is_initialized = false;
+  eigenvalues_are_initialized = false;
   theta = delta = 1.0;
-  matrix_ptr = 0;
+  matrix_ptr = nullptr;
   {
     VectorType empty_vector;
     data.matrix_diagonal_inverse.reinit(empty_vector);
@@ -2062,7 +2030,6 @@ PreconditionChebyshev<MatrixType,VectorType,PreconditionerType>::clear ()
   data.preconditioner.reset();
 }
 
-DEAL_II_ENABLE_EXTRA_DIAGNOSTICS
 
 
 
@@ -2072,8 +2039,8 @@ void
 PreconditionChebyshev<MatrixType,VectorType,PreconditionerType>::estimate_eigenvalues
 (const VectorType &src) const
 {
-  Assert(is_initialized == false, ExcInternalError());
-  Assert(data.preconditioner.get() != NULL, ExcNotInitialized());
+  Assert(eigenvalues_are_initialized == false, ExcInternalError());
+  Assert(data.preconditioner.get() != nullptr, ExcNotInitialized());
 
   update1.reinit(src);
   update2.reinit(src, true);
@@ -2092,15 +2059,15 @@ PreconditionChebyshev<MatrixType,VectorType,PreconditionerType>::estimate_eigenv
                                 std::sqrt(std::numeric_limits<typename VectorType::value_type>::epsilon()),
                                 1e-10, false, false);
 
-      internal::PreconditionChebyshev::EigenvalueTracker eigenvalue_tracker;
+      internal::PreconditionChebyshevImplementation::EigenvalueTracker eigenvalue_tracker;
       SolverCG<VectorType> solver (control);
-      solver.connect_eigenvalues_slot(std_cxx11::bind(&internal::PreconditionChebyshev::EigenvalueTracker::slot,
-                                                      &eigenvalue_tracker,
-                                                      std_cxx11::_1));
+      solver.connect_eigenvalues_slot(std::bind(&internal::PreconditionChebyshevImplementation::EigenvalueTracker::slot,
+                                                &eigenvalue_tracker,
+                                                std::placeholders::_1));
 
       // set an initial guess which is close to the constant vector but where
       // one entry is different to trigger high frequencies
-      internal::PreconditionChebyshev::set_initial_guess(update2);
+      internal::PreconditionChebyshevImplementation::set_initial_guess(update2);
 
       try
         {
@@ -2151,14 +2118,14 @@ PreconditionChebyshev<MatrixType,VectorType,PreconditionerType>::estimate_eigenv
 
   // We do not need the third auxiliary vector in case we have a
   // DiagonalMatrix as preconditioner and use deal.II's own vectors
-  if (types_are_equal<PreconditionerType,DiagonalMatrix<VectorType> >::value == false ||
-      (types_are_equal<VectorType,dealii::Vector<typename VectorType::value_type> >::value == false
+  if (std::is_same<PreconditionerType,DiagonalMatrix<VectorType> >::value == false ||
+      (std::is_same<VectorType,dealii::Vector<typename VectorType::value_type> >::value == false
        &&
-       types_are_equal<VectorType,LinearAlgebra::distributed::Vector<typename VectorType::value_type> >::value == false
+       std::is_same<VectorType,LinearAlgebra::distributed::Vector<typename VectorType::value_type> >::value == false
       ))
     update3.reinit (src, true);
 
-  const_cast<PreconditionChebyshev<MatrixType,VectorType,PreconditionerType> *>(this)->is_initialized = true;
+  const_cast<PreconditionChebyshev<MatrixType,VectorType,PreconditionerType> *>(this)->eigenvalues_are_initialized = true;
 }
 
 
@@ -2182,7 +2149,7 @@ PreconditionChebyshev<MatrixType,VectorType,PreconditionerType>
       const double rhokp = 1./(2.*sigma-rhok);
       const double factor1 = rhokp * rhok, factor2 = 2.*rhokp/delta;
       rhok = rhokp;
-      internal::PreconditionChebyshev::vector_updates
+      internal::PreconditionChebyshevImplementation::vector_updates
       (src, *data.preconditioner, false, factor1, factor2, update1, update2, update3, dst);
     }
 }
@@ -2203,7 +2170,7 @@ PreconditionChebyshev<MatrixType,VectorType,PreconditionerType>
       const double rhokp = 1./(2.*sigma-rhok);
       const double factor1 = rhokp * rhok, factor2 = 2.*rhokp/delta;
       rhok = rhokp;
-      internal::PreconditionChebyshev::vector_updates
+      internal::PreconditionChebyshevImplementation::vector_updates
       (src, *data.preconditioner, false, factor1, factor2, update1, update2, update3, dst);
     }
 }
@@ -2218,10 +2185,10 @@ PreconditionChebyshev<MatrixType,VectorType,PreconditionerType>
          const VectorType &src) const
 {
   Threads::Mutex::ScopedLock lock(mutex);
-  if (is_initialized == false)
+  if (eigenvalues_are_initialized == false)
     estimate_eigenvalues(src);
 
-  internal::PreconditionChebyshev::vector_updates
+  internal::PreconditionChebyshevImplementation::vector_updates
   (src, *data.preconditioner, true, 0., 1./theta, update1, update2, update3, dst);
 
   do_chebyshev_loop(dst, src);
@@ -2237,10 +2204,10 @@ PreconditionChebyshev<MatrixType,VectorType,PreconditionerType>
           const VectorType &src) const
 {
   Threads::Mutex::ScopedLock lock(mutex);
-  if (is_initialized == false)
+  if (eigenvalues_are_initialized == false)
     estimate_eigenvalues(src);
 
-  internal::PreconditionChebyshev::vector_updates
+  internal::PreconditionChebyshevImplementation::vector_updates
   (src, *data.preconditioner, true, 0., 1./theta, update1, update2, update3, dst);
 
   do_transpose_chebyshev_loop(dst, src);
@@ -2256,18 +2223,12 @@ PreconditionChebyshev<MatrixType,VectorType,PreconditionerType>
         const VectorType &src) const
 {
   Threads::Mutex::ScopedLock lock(mutex);
-  if (is_initialized == false)
+  if (eigenvalues_are_initialized == false)
     estimate_eigenvalues(src);
 
-  if (!dst.all_zero())
-    {
-      matrix_ptr->vmult (update2, dst);
-      internal::PreconditionChebyshev::vector_updates
-      (src, *data.preconditioner, false, 0., 1./theta, update1, update2, update3, dst);
-    }
-  else
-    internal::PreconditionChebyshev::vector_updates
-    (src, *data.preconditioner, true, 0., 1./theta, update1, update2, update3, dst);
+  matrix_ptr->vmult (update2, dst);
+  internal::PreconditionChebyshevImplementation::vector_updates
+  (src, *data.preconditioner, false, 0., 1./theta, update1, update2, update3, dst);
 
   do_chebyshev_loop(dst, src);
 }
@@ -2282,18 +2243,12 @@ PreconditionChebyshev<MatrixType,VectorType,PreconditionerType>
          const VectorType &src) const
 {
   Threads::Mutex::ScopedLock lock(mutex);
-  if (is_initialized == false)
+  if (eigenvalues_are_initialized == false)
     estimate_eigenvalues(src);
 
-  if (!dst.all_zero())
-    {
-      matrix_ptr->Tvmult (update2, dst);
-      internal::PreconditionChebyshev::vector_updates
-      (src, *data.preconditioner, false, 0., 1./theta, update1, update2, update3, dst);
-    }
-  else
-    internal::PreconditionChebyshev::vector_updates
-    (src, *data.preconditioner, true, 0., 1./theta, update1, update2, update3, dst);
+  matrix_ptr->Tvmult (update2, dst);
+  internal::PreconditionChebyshevImplementation::vector_updates
+  (src, *data.preconditioner, false, 0., 1./theta, update1, update2, update3, dst);
 
   do_transpose_chebyshev_loop(dst, src);
 }
@@ -2305,7 +2260,7 @@ inline
 typename PreconditionChebyshev<MatrixType,VectorType,PreconditionerType>::size_type
 PreconditionChebyshev<MatrixType,VectorType,PreconditionerType>::m () const
 {
-  Assert (matrix_ptr!=0, ExcNotInitialized());
+  Assert (matrix_ptr!=nullptr, ExcNotInitialized());
   return matrix_ptr->m();
 }
 
@@ -2316,7 +2271,7 @@ inline
 typename PreconditionChebyshev<MatrixType,VectorType,PreconditionerType>::size_type
 PreconditionChebyshev<MatrixType,VectorType,PreconditionerType>::n () const
 {
-  Assert (matrix_ptr!=0, ExcNotInitialized());
+  Assert (matrix_ptr!=nullptr, ExcNotInitialized());
   return matrix_ptr->n();
 }
 
