@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 1999 - 2016 by the deal.II authors
+// Copyright (C) 1999 - 2017 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -13,8 +13,8 @@
 //
 // ---------------------------------------------------------------------
 
-#ifndef dealii__precondition_block_h
-#define dealii__precondition_block_h
+#ifndef dealii_precondition_block_h
+#define dealii_precondition_block_h
 
 
 #include <deal.II/base/config.h>
@@ -77,7 +77,7 @@ DEAL_II_NAMESPACE_OPEN
  * @author Ralf Hartmann, Guido Kanschat
  * @date 1999, 2000, 2010
  */
-template<typename MatrixType, typename inverse_type = typename MatrixType::value_type>
+template <typename MatrixType, typename inverse_type = typename MatrixType::value_type>
 class PreconditionBlock
   : public virtual Subscriptor,
     protected PreconditionBlockBase<inverse_type>
@@ -155,7 +155,7 @@ public:
   /**
    * Destructor.
    */
-  ~PreconditionBlock();
+  ~PreconditionBlock() = default;
 
   /**
    * Initialize matrix and block size.  We store the matrix and the block size
@@ -363,7 +363,7 @@ protected:
  *
  * @author Ralf Hartmann, Guido Kanschat, 1999, 2000, 2003
  */
-template<typename MatrixType, typename inverse_type = typename MatrixType::value_type>
+template <typename MatrixType, typename inverse_type = typename MatrixType::value_type>
 class PreconditionBlockJacobi : public virtual Subscriptor,
   private PreconditionBlock<MatrixType, inverse_type>
 {
@@ -457,11 +457,6 @@ public:
      */
     const_iterator &operator++ ();
 
-    /**
-     * Postfix increment.
-     */
-    const_iterator &operator++ (int);
-
     /**
      * Dereferencing operator.
      */
@@ -502,7 +497,6 @@ public:
   using PreconditionBlock<MatrixType, inverse_type>::clear;
   using PreconditionBlock<MatrixType, inverse_type>::empty;
   using PreconditionBlock<MatrixType, inverse_type>::el;
-  using PreconditionBlock<MatrixType, inverse_type>::set_same_diagonal;
   using PreconditionBlock<MatrixType, inverse_type>::invert_diagblocks;
   using PreconditionBlock<MatrixType, inverse_type>::block_size;
   using PreconditionBlockBase<inverse_type>::size;
@@ -629,7 +623,7 @@ private:
  *
  * @author Ralf Hartmann, Guido Kanschat, 1999, 2000, 2001, 2002, 2003
  */
-template<typename MatrixType, typename inverse_type = typename MatrixType::value_type>
+template <typename MatrixType, typename inverse_type = typename MatrixType::value_type>
 class PreconditionBlockSOR : public virtual Subscriptor,
   protected PreconditionBlock<MatrixType, inverse_type>
 {
@@ -655,13 +649,10 @@ public:
   using typename PreconditionBlock<MatrixType,inverse_type>::AdditionalData;
   using PreconditionBlock<MatrixType, inverse_type>::initialize;
   using PreconditionBlock<MatrixType, inverse_type>::clear;
-  using PreconditionBlock<MatrixType, inverse_type>::empty;
   using PreconditionBlockBase<inverse_type>::size;
   using PreconditionBlockBase<inverse_type>::inverse;
   using PreconditionBlockBase<inverse_type>::inverse_householder;
   using PreconditionBlockBase<inverse_type>::inverse_svd;
-  using PreconditionBlock<MatrixType, inverse_type>::el;
-  using PreconditionBlock<MatrixType, inverse_type>::set_same_diagonal;
   using PreconditionBlock<MatrixType, inverse_type>::invert_diagblocks;
   using PreconditionBlock<MatrixType, inverse_type>::set_permutation;
   using PreconditionBlockBase<inverse_type>::log_statistics;
@@ -787,7 +778,7 @@ protected:
  *
  * @author Ralf Hartmann, Guido Kanschat, 1999, 2000
  */
-template<typename MatrixType, typename inverse_type = typename MatrixType::value_type>
+template <typename MatrixType, typename inverse_type = typename MatrixType::value_type>
 class PreconditionBlockSSOR : public virtual Subscriptor,
   private PreconditionBlockSOR<MatrixType, inverse_type>
 {
@@ -827,7 +818,6 @@ public:
   using PreconditionBlockSOR<MatrixType,inverse_type>::set_permutation;
   using PreconditionBlockSOR<MatrixType, inverse_type>::empty;
   using PreconditionBlockSOR<MatrixType, inverse_type>::el;
-  using PreconditionBlockSOR<MatrixType,inverse_type>::set_same_diagonal;
   using PreconditionBlockSOR<MatrixType,inverse_type>::invert_diagblocks;
 
   /**
@@ -864,17 +854,17 @@ public:
 
 #ifndef DOXYGEN
 
-template<typename MatrixType, typename inverse_type>
+template <typename MatrixType, typename inverse_type>
 inline bool
 PreconditionBlock<MatrixType, inverse_type>::empty () const
 {
-  if (A == 0)
+  if (A == nullptr)
     return true;
   return A->empty();
 }
 
 
-template<typename MatrixType, typename inverse_type>
+template <typename MatrixType, typename inverse_type>
 inline inverse_type
 PreconditionBlock<MatrixType, inverse_type>::el (
   size_type i,
@@ -898,7 +888,7 @@ PreconditionBlock<MatrixType, inverse_type>::el (
 
 //---------------------------------------------------------------------------
 
-template<typename MatrixType, typename inverse_type>
+template <typename MatrixType, typename inverse_type>
 inline
 PreconditionBlockJacobi<MatrixType, inverse_type>::const_iterator::Accessor::
 Accessor (const PreconditionBlockJacobi<MatrixType, inverse_type> *matrix,
@@ -926,7 +916,7 @@ Accessor (const PreconditionBlockJacobi<MatrixType, inverse_type> *matrix,
 }
 
 
-template<typename MatrixType, typename inverse_type>
+template <typename MatrixType, typename inverse_type>
 inline
 typename PreconditionBlockJacobi<MatrixType, inverse_type>::size_type
 PreconditionBlockJacobi<MatrixType, inverse_type>::const_iterator::Accessor::row() const
@@ -938,7 +928,7 @@ PreconditionBlockJacobi<MatrixType, inverse_type>::const_iterator::Accessor::row
 }
 
 
-template<typename MatrixType, typename inverse_type>
+template <typename MatrixType, typename inverse_type>
 inline
 typename PreconditionBlockJacobi<MatrixType, inverse_type>::size_type
 PreconditionBlockJacobi<MatrixType, inverse_type>::const_iterator::Accessor::column() const
@@ -950,7 +940,7 @@ PreconditionBlockJacobi<MatrixType, inverse_type>::const_iterator::Accessor::col
 }
 
 
-template<typename MatrixType, typename inverse_type>
+template <typename MatrixType, typename inverse_type>
 inline
 inverse_type
 PreconditionBlockJacobi<MatrixType, inverse_type>::const_iterator::Accessor::value() const
@@ -962,7 +952,7 @@ PreconditionBlockJacobi<MatrixType, inverse_type>::const_iterator::Accessor::val
 }
 
 
-template<typename MatrixType, typename inverse_type>
+template <typename MatrixType, typename inverse_type>
 inline
 PreconditionBlockJacobi<MatrixType, inverse_type>::const_iterator::
 const_iterator(const PreconditionBlockJacobi<MatrixType, inverse_type> *matrix,
@@ -972,7 +962,7 @@ const_iterator(const PreconditionBlockJacobi<MatrixType, inverse_type> *matrix,
 {}
 
 
-template<typename MatrixType, typename inverse_type>
+template <typename MatrixType, typename inverse_type>
 inline
 typename PreconditionBlockJacobi<MatrixType, inverse_type>::const_iterator &
 PreconditionBlockJacobi<MatrixType, inverse_type>::const_iterator::operator++ ()
@@ -994,7 +984,7 @@ PreconditionBlockJacobi<MatrixType, inverse_type>::const_iterator::operator++ ()
 }
 
 
-template<typename MatrixType, typename inverse_type>
+template <typename MatrixType, typename inverse_type>
 inline
 const typename PreconditionBlockJacobi<MatrixType, inverse_type>::const_iterator::Accessor &
 PreconditionBlockJacobi<MatrixType, inverse_type>::const_iterator::operator* () const
@@ -1003,7 +993,7 @@ PreconditionBlockJacobi<MatrixType, inverse_type>::const_iterator::operator* ()
 }
 
 
-template<typename MatrixType, typename inverse_type>
+template <typename MatrixType, typename inverse_type>
 inline
 const typename PreconditionBlockJacobi<MatrixType, inverse_type>::const_iterator::Accessor *
 PreconditionBlockJacobi<MatrixType, inverse_type>::const_iterator::operator-> () const
@@ -1012,7 +1002,7 @@ PreconditionBlockJacobi<MatrixType, inverse_type>::const_iterator::operator-> ()
 }
 
 
-template<typename MatrixType, typename inverse_type>
+template <typename MatrixType, typename inverse_type>
 inline
 bool
 PreconditionBlockJacobi<MatrixType, inverse_type>::const_iterator::
@@ -1030,7 +1020,7 @@ operator == (const const_iterator &other) const
 }
 
 
-template<typename MatrixType, typename inverse_type>
+template <typename MatrixType, typename inverse_type>
 inline
 bool
 PreconditionBlockJacobi<MatrixType, inverse_type>::const_iterator::
@@ -1040,7 +1030,7 @@ operator != (const const_iterator &other) const
 }
 
 
-template<typename MatrixType, typename inverse_type>
+template <typename MatrixType, typename inverse_type>
 inline
 bool
 PreconditionBlockJacobi<MatrixType, inverse_type>::const_iterator::
@@ -1052,7 +1042,7 @@ operator < (const const_iterator &other) const
 }
 
 
-template<typename MatrixType, typename inverse_type>
+template <typename MatrixType, typename inverse_type>
 inline
 typename PreconditionBlockJacobi<MatrixType, inverse_type>::const_iterator
 PreconditionBlockJacobi<MatrixType, inverse_type>::begin () const
@@ -1061,7 +1051,7 @@ PreconditionBlockJacobi<MatrixType, inverse_type>::begin () const
 }
 
 
-template<typename MatrixType, typename inverse_type>
+template <typename MatrixType, typename inverse_type>
 inline
 typename PreconditionBlockJacobi<MatrixType, inverse_type>::const_iterator
 PreconditionBlockJacobi<MatrixType, inverse_type>::end () const
@@ -1070,7 +1060,7 @@ PreconditionBlockJacobi<MatrixType, inverse_type>::end () const
 }
 
 
-template<typename MatrixType, typename inverse_type>
+template <typename MatrixType, typename inverse_type>
 inline
 typename PreconditionBlockJacobi<MatrixType, inverse_type>::const_iterator
 PreconditionBlockJacobi<MatrixType, inverse_type>::begin (
@@ -1082,7 +1072,7 @@ PreconditionBlockJacobi<MatrixType, inverse_type>::begin (
 
 
 
-template<typename MatrixType, typename inverse_type>
+template <typename MatrixType, typename inverse_type>
 inline
 typename PreconditionBlockJacobi<MatrixType, inverse_type>::const_iterator
 PreconditionBlockJacobi<MatrixType, inverse_type>::end (
