@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 1999 - 2016 by the deal.II authors
+// Copyright (C) 1999 - 2017 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -13,8 +13,8 @@
 //
 // ---------------------------------------------------------------------
 
-#ifndef dealii__precondition_block_base_h
-#define dealii__precondition_block_base_h
+#ifndef dealii_precondition_block_base_h
+#define dealii_precondition_block_base_h
 
 
 #include <deal.II/base/config.h>
@@ -89,7 +89,7 @@ public:
   /**
    * The virtual destructor
    */
-  ~PreconditionBlockBase();
+  ~PreconditionBlockBase() = default;
 
   /**
    * Deletes the inverse diagonal block matrices if existent hence leaves the
@@ -109,16 +109,6 @@ public:
    */
   void inverses_computed(bool are_they);
 
-  /**
-   * Use only the inverse of the first diagonal block to save memory and
-   * computation time.
-   *
-   * Possible applications: computing on a cartesian grid, all diagonal blocks
-   * are the same or all diagonal blocks are at least similar and inversion of
-   * one of them still yields a preconditioner.
-   */
-  void set_same_diagonal ();
-
   /**
    * Does the matrix use only one diagonal block?
    */
@@ -134,22 +124,11 @@ public:
    */
   bool inverses_ready () const;
 
-  /**
-   * Check whether the object is empty.
-   */
-  bool empty () const;
-
   /**
    * The number of blocks.
    */
   unsigned int size() const;
 
-  /**
-   * Read-only access to entries. This function is only possible if the
-   * inverse diagonal blocks are stored.
-   */
-  number el(size_type i, size_type j) const;
-
   /**
    * Multiply with the inverse block at position <tt>i</tt>.
    */
