@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 2015 - 2017 by the deal.II authors
+// Copyright (C) 2015 - 2018 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -13,8 +13,8 @@
 //
 // ---------------------------------------------------------------------
 
-#ifndef dealii__read_write_vector_h
-#define dealii__read_write_vector_h
+#ifndef dealii_read_write_vector_h
+#define dealii_read_write_vector_h
 
 #include <deal.II/base/config.h>
 #include <deal.II/base/index_set.h>
@@ -24,14 +24,18 @@
 #include <deal.II/base/utilities.h>
 #include <deal.II/base/memory_consumption.h>
 #include <deal.II/base/thread_management.h>
-#include <deal.II/lac/vector_view.h>
+#include <deal.II/lac/vector_operation.h>
 
 #include <cstring>
+#include <cstdlib>
 #include <iomanip>
 
 #ifdef DEAL_II_WITH_TRILINOS
-#include <deal.II/lac/trilinos_epetra_communication_pattern.h>
-#include "Epetra_MultiVector.h"
+#  include <deal.II/lac/trilinos_epetra_communication_pattern.h>
+#  include <deal.II/lac/trilinos_epetra_vector.h>
+
+#  include <Epetra_MultiVector.h>
+
 #endif
 
 DEAL_II_NAMESPACE_OPEN
@@ -63,14 +67,6 @@ namespace TrilinosWrappers
     class Vector;
   }
 }
-
-namespace LinearAlgebra
-{
-  namespace EpetraWrappers
-  {
-    class Vector;
-  }
-}
 #endif
 
 #ifdef DEAL_II_WITH_CUDA
@@ -172,7 +168,7 @@ namespace LinearAlgebra
     /**
      * Destructor.
      */
-    ~ReadWriteVector ();
+    ~ReadWriteVector () = default;
 
     /**
      * Set the global size of the vector to @p size. The stored elements have
@@ -182,8 +178,8 @@ namespace LinearAlgebra
      * initialized with zero, otherwise the memory will be untouched (and the
      * user must make sure to fill it with reasonable data before using it).
      */
-    void reinit (const size_type size,
-                 const bool      omit_zeroing_entries = false);
+    virtual void reinit (const size_type size,
+                         const bool      omit_zeroing_entries = false);
 
     /**
      * Uses the same IndexSet as the one of the input vector @p in_vector and
@@ -206,10 +202,27 @@ namespace LinearAlgebra
      * user must make sure to fill it with reasonable data before using it).
      * locally_stored_indices.
      */
-    void reinit (const IndexSet &locally_stored_indices,
-                 const bool      omit_zeroing_entries = false);
+    virtual void reinit (const IndexSet &locally_stored_indices,
+                         const bool      omit_zeroing_entries = false);
+
+
+#ifdef DEAL_II_WITH_TRILINOS
+#ifdef DEAL_II_WITH_MPI
+    /**
+     * Initialize this ReadWriteVector by supplying access to all locally available
+     * entries in the given ghosted or non-ghosted vector.
+     *
+     * @note This function currently copies the values from the argument into
+     * the ReadWriteVector, so modifications here will not modify @p trilinos_vec.
+     *
+     * This function is mainly written for backwards-compatibility to get
+     * element access to a ghosted TrilinosWrappers::MPI::Vector inside the
+     * library.
+     */
+    void reinit(const TrilinosWrappers::MPI::Vector &trilinos_vec);
+#endif
+#endif
 
-#ifdef DEAL_II_WITH_CXX11
     /**
      * Apply the functor @p func to each element of the vector. The functor
      * should look like
@@ -220,12 +233,11 @@ namespace LinearAlgebra
      * };
      * @endcode
      *
-     * @note This function requires C++11 and read_write_vector.templates.h
-     * needs to be included.
+     * @note This function requires that the header read_write_vector.templates.h
+     * be included.
      */
     template <typename Functor>
     void apply(const Functor &func);
-#endif
 
     /**
      * Swap the contents of this vector and the other vector @p v. One could
@@ -234,7 +246,7 @@ namespace LinearAlgebra
      * only swaps the pointers to the data of the two vectors and therefore
      * does not need to allocate temporary storage and move data around.
      *
-     * This function is analogous to the the @p swap function of all C++
+     * This function is analogous to the @p swap function of all C++
      * standard containers. Also, there is a global function
      * <tt>swap(u,v)</tt> that simply calls <tt>u.swap(v)</tt>, again in
      * analogy to standard functions.
@@ -270,8 +282,8 @@ namespace LinearAlgebra
      */
     void import(const distributed::Vector<Number> &vec,
                 VectorOperation::values operation,
-                std_cxx11::shared_ptr<const CommunicationPatternBase> communication_pattern =
-                  std_cxx11::shared_ptr<const CommunicationPatternBase> ());
+                const std::shared_ptr<const CommunicationPatternBase> &communication_pattern =
+                  std::shared_ptr<const CommunicationPatternBase> ());
 
 #ifdef DEAL_II_WITH_PETSC
     /**
@@ -284,8 +296,8 @@ namespace LinearAlgebra
      */
     void import(const PETScWrappers::MPI::Vector &petsc_vec,
                 VectorOperation::values operation,
-                std_cxx11::shared_ptr<const CommunicationPatternBase> communication_pattern =
-                  std_cxx11::shared_ptr<const CommunicationPatternBase> ());
+                const std::shared_ptr<const CommunicationPatternBase> &communication_pattern =
+                  std::shared_ptr<const CommunicationPatternBase> ());
 #endif
 
 #ifdef DEAL_II_WITH_TRILINOS
@@ -296,12 +308,15 @@ namespace LinearAlgebra
      * replace the current elements. The last parameter can be used if the same
      * communication pattern is used multiple times. This can be used to improve
      * performance.
+     *
+     * @note: The @p trilinos_vec is not allowed to have ghost entries.
      */
     void import(const TrilinosWrappers::MPI::Vector &trilinos_vec,
                 VectorOperation::values operation,
-                std_cxx11::shared_ptr<const CommunicationPatternBase> communication_pattern =
-                  std_cxx11::shared_ptr<const CommunicationPatternBase> ());
+                std::shared_ptr<const CommunicationPatternBase> communication_pattern =
+                  std::shared_ptr<const CommunicationPatternBase> ());
 
+#ifdef DEAL_II_WITH_MPI
     /**
      * Imports all the elements present in the vector's IndexSet from the input
      * vector @p epetra_vec. VectorOperation::values @p operation is used to
@@ -312,8 +327,9 @@ namespace LinearAlgebra
      */
     void import(const EpetraWrappers::Vector &epetra_vec,
                 VectorOperation::values operation,
-                std_cxx11::shared_ptr<const CommunicationPatternBase> communication_pattern =
-                  std_cxx11::shared_ptr<const CommunicationPatternBase> ());
+                std::shared_ptr<const CommunicationPatternBase> communication_pattern =
+                  std::shared_ptr<const CommunicationPatternBase> ());
+#endif
 #endif
 
 #ifdef DEAL_II_WITH_CUDA
@@ -325,8 +341,8 @@ namespace LinearAlgebra
      */
     void import(const CUDAWrappers::Vector<Number> &cuda_vec,
                 VectorOperation::values operation,
-                std_cxx11::shared_ptr<const CommunicationPatternBase> communication_pattern =
-                  std_cxx11::shared_ptr<const CommunicationPatternBase> ());
+                std::shared_ptr<const CommunicationPatternBase> communication_pattern =
+                  std::shared_ptr<const CommunicationPatternBase> ());
 #endif
 
     /**
@@ -416,18 +432,50 @@ namespace LinearAlgebra
     Number &operator [] (const size_type global_index);
 
     /**
-     * Instead of getting individual elements of a vector, this function
-     * allows to get a whole set of elements at once. The indices of the
-     * elements to be read are stated in the first argument, the corresponding
-     * values are returned in the second.
+     * Instead of getting individual elements of a vector via operator(),
+     * this function allows getting a whole set of elements at once. The
+     * indices of the elements to be read are stated in the first argument, the
+     * corresponding values are returned in the second.
+     *
+     * If the current vector is called @p v, then this function is the equivalent
+     * to the code
+     * @code
+     *   for (unsigned int i=0; i<indices.size(); ++i)
+     *     values[i] = v[indices[i]];
+     * @endcode
+     *
+     * @pre The sizes of the @p indices and @p values arrays must be identical.
      */
     template <typename Number2>
     void extract_subvector_to (const std::vector<size_type> &indices,
                                std::vector<Number2> &values) const;
 
     /**
-     * Just as the above, but with pointers. Useful in minimizing copying of
-     * data around.
+     * Instead of getting individual elements of a vector via operator(),
+     * this function allows getting a whole set of elements at once. In
+     * contrast to the previous function, this function obtains the
+     * indices of the elements by dereferencing all elements of the iterator
+     * range provided by the first two arguments, and puts the vector
+     * values into memory locations obtained by dereferencing a range
+     * of iterators starting at the location pointed to by the third
+     * argument.
+     *
+     * If the current vector is called @p v, then this function is the equivalent
+     * to the code
+     * @code
+     *   ForwardIterator indices_p = indices_begin;
+     *   OutputIterator  values_p  = values_begin;
+     *   while (indices_p != indices_end)
+     *   {
+     *     *values_p = v[*indices_p];
+     *     ++indices_p;
+     *     ++values_p;
+     *   }
+     * @endcode
+     *
+     * @pre It must be possible to write into as many memory locations
+     *   starting at @p values_begin as there are iterators between
+     *   @p indices_begin and @p indices_end.
      */
     template <typename ForwardIterator, typename OutputIterator>
     void extract_subvector_to (ForwardIterator          indices_begin,
@@ -515,7 +563,7 @@ namespace LinearAlgebra
                 const IndexSet                                 &locally_owned_elements,
                 VectorOperation::values                         operation,
                 const MPI_Comm                                 &mpi_comm,
-                std_cxx11::shared_ptr<const CommunicationPatternBase> communication_pattern);
+                const std::shared_ptr<const CommunicationPatternBase> &communication_pattern);
 #endif
 
     /**
@@ -558,25 +606,24 @@ namespace LinearAlgebra
      * CommunicationPattern for the communication between the
      * source_stored_elements IndexSet and the current vector.
      */
-    std_cxx11::shared_ptr<CommunicationPatternBase> comm_pattern;
+    std::shared_ptr<CommunicationPatternBase> comm_pattern;
 
     /**
      * Pointer to the array of local elements of this vector.
      */
-    Number *val;
+    std::unique_ptr<Number[], decltype (free) *> values;
 
     /**
      * For parallel loops with TBB, this member variable stores the affinity
      * information of loops.
      */
-    mutable std_cxx11::shared_ptr< ::dealii::parallel::internal::TBBPartitioner> thread_loop_partitioner;
+    mutable std::shared_ptr< ::dealii::parallel::internal::TBBPartitioner> thread_loop_partitioner;
 
     /**
      * Make all other ReadWriteVector types friends.
      */
     template <typename Number2> friend class ReadWriteVector;
 
-#ifdef DEAL_II_WITH_CXX11
   private:
     /**
      * This class provides a wrapper around a Functor which acts on
@@ -610,7 +657,6 @@ namespace LinearAlgebra
        */
       const Functor &functor;
     };
-#endif
   };
 
   /*@}*/
@@ -624,9 +670,13 @@ namespace LinearAlgebra
   inline
   ReadWriteVector<Number>::ReadWriteVector ()
     :
-    val(NULL)
+    Subscriptor(),
+    values(nullptr, free)
   {
-    reinit(0, true);
+    // virtual functions called in constructors and destructors never use the
+    // override in a derived class
+    // for clarity be explicit on which function is called
+    ReadWriteVector<Number>::reinit(0, true);
   }
 
 
@@ -636,7 +686,7 @@ namespace LinearAlgebra
   ReadWriteVector<Number>::ReadWriteVector (const ReadWriteVector<Number> &v)
     :
     Subscriptor(),
-    val(NULL)
+    values(nullptr, free)
   {
     this->operator=(v);
   }
@@ -647,9 +697,13 @@ namespace LinearAlgebra
   inline
   ReadWriteVector<Number>::ReadWriteVector (const size_type size)
     :
-    val(NULL)
+    Subscriptor(),
+    values(nullptr, free)
   {
-    reinit (size, false);
+    // virtual functions called in constructors and destructors never use the
+    // override in a derived class
+    // for clarity be explicit on which function is called
+    ReadWriteVector<Number>::reinit (size, false);
   }
 
 
@@ -658,18 +712,13 @@ namespace LinearAlgebra
   inline
   ReadWriteVector<Number>::ReadWriteVector (const IndexSet &locally_stored_indices)
     :
-    val(NULL)
+    Subscriptor(),
+    values(nullptr, free)
   {
-    reinit (locally_stored_indices);
-  }
-
-
-
-  template <typename Number>
-  inline
-  ReadWriteVector<Number>::~ReadWriteVector ()
-  {
-    resize_val(0);
+    // virtual functions called in constructors and destructors never use the
+    // override in a derived class
+    // for clarity be explicit on which function is called
+    ReadWriteVector<Number>::reinit (locally_stored_indices);
   }
 
 
@@ -709,7 +758,7 @@ namespace LinearAlgebra
   typename ReadWriteVector<Number>::iterator
   ReadWriteVector<Number>::begin ()
   {
-    return &val[0];
+    return values.get();
   }
 
 
@@ -719,7 +768,7 @@ namespace LinearAlgebra
   typename ReadWriteVector<Number>::const_iterator
   ReadWriteVector<Number>::begin () const
   {
-    return &val[0];
+    return values.get();
   }
 
 
@@ -729,7 +778,7 @@ namespace LinearAlgebra
   typename ReadWriteVector<Number>::iterator
   ReadWriteVector<Number>::end ()
   {
-    return &val[this->n_elements()];
+    return values.get() + this->n_elements();
   }
 
 
@@ -739,7 +788,7 @@ namespace LinearAlgebra
   typename ReadWriteVector<Number>::const_iterator
   ReadWriteVector<Number>::end () const
   {
-    return &val[this->n_elements()];
+    return values.get() + this->n_elements();
   }
 
 
@@ -749,7 +798,7 @@ namespace LinearAlgebra
   Number
   ReadWriteVector<Number>::operator() (const size_type global_index) const
   {
-    return val[global_to_local(global_index)];
+    return values[global_to_local(global_index)];
   }
 
 
@@ -759,7 +808,7 @@ namespace LinearAlgebra
   Number &
   ReadWriteVector<Number>::operator() (const size_type global_index)
   {
-    return val[global_to_local (global_index)];
+    return values[global_to_local (global_index)];
   }
 
 
@@ -788,10 +837,10 @@ namespace LinearAlgebra
   template <typename Number2>
   inline
   void ReadWriteVector<Number>::extract_subvector_to (const std::vector<size_type> &indices,
-                                                      std::vector<Number2> &values) const
+                                                      std::vector<Number2> &extracted_values) const
   {
     for (size_type i = 0; i < indices.size(); ++i)
-      values[i] = operator()(indices[i]);
+      extracted_values[i] = operator()(indices[i]);
   }
 
 
@@ -820,7 +869,7 @@ namespace LinearAlgebra
   {
     AssertIndexRange (local_index, this->n_elements());
 
-    return val[local_index];
+    return values[local_index];
   }
 
 
@@ -832,7 +881,7 @@ namespace LinearAlgebra
   {
     AssertIndexRange (local_index, this->n_elements());
 
-    return val[local_index];
+    return values[local_index];
   }
 
 
@@ -845,7 +894,7 @@ namespace LinearAlgebra
                                 const std::vector<Number2>   &values)
   {
     AssertDimension (indices.size(), values.size());
-    add (indices.size(), &indices[0], &values[0]);
+    add (indices.size(), indices.data(), values.data());
   }
 
 
@@ -874,19 +923,18 @@ namespace LinearAlgebra
   void
   ReadWriteVector<Number>::add (const size_type    n_indices,
                                 const size_type   *indices,
-                                const Number2     *values)
+                                const Number2     *values_to_add)
   {
     for (size_type i=0; i<n_indices; ++i)
       {
         Assert (numbers::is_finite(values[i]),
                 ExcMessage("The given value is not finite but either infinite or Not A Number (NaN)"));
-        this->operator()(indices[i]) += values[i];
+        this->operator()(indices[i]) += values_to_add[i];
       }
   }
 
 
 
-#ifdef DEAL_II_WITH_CXX11
   template <typename Number>
   template <typename Functor>
   inline
@@ -907,9 +955,8 @@ namespace LinearAlgebra
       const size_type end)
   {
     for (size_type i=begin; i<end; ++i)
-      functor(parent.val[i]);
+      functor(parent.values[i]);
   }
-#endif
 
 #endif  // ifndef DOXYGEN
 
@@ -923,7 +970,7 @@ namespace LinearAlgebra
  * C++ standard library which uses a temporary object. The function simply
  * exchanges the data of the two vectors.
  *
- * @relates Vector
+ * @relatesalso Vector
  */
 template <typename Number>
 inline
