@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 2015 - 2017-2016 by the deal.II authors
+// Copyright (C) 2015 - 2018-2016 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -13,8 +13,8 @@
 //
 // ---------------------------------------------------------------------
 
-#ifndef dealii__parallel_vector_templates_h
-#define dealii__parallel_vector_templates_h
+#ifndef dealii_parallel_vector_templates_h
+#define dealii_parallel_vector_templates_h
 
 
 #include <deal.II/base/config.h>
@@ -25,6 +25,8 @@
 #include <deal.II/lac/vector_operations_internal.h>
 #include <deal.II/lac/la_parallel_vector.h>
 
+#include <boost/io/ios_state.hpp>
+
 #ifdef DEAL_II_WITH_PETSC
 #  include <deal.II/lac/petsc_parallel_vector.h>
 #endif
@@ -33,7 +35,7 @@
 #  include <deal.II/lac/trilinos_vector.h>
 #  include <deal.II/lac/trilinos_epetra_vector.h>
 #  include <deal.II/lac/trilinos_epetra_communication_pattern.h>
-#  include "Epetra_Import.h"
+#  include <Epetra_Import.h>
 #endif
 
 #ifdef DEAL_II_WITH_CUDA
@@ -52,19 +54,17 @@ namespace LinearAlgebra
   {
     if (new_alloc_size == 0)
       {
-        if (val != NULL)
-          free(val);
-        val = NULL;
-        thread_loop_partitioner.reset(new parallel::internal::TBBPartitioner());
+        values.reset ();
+        thread_loop_partitioner = std::make_shared<parallel::internal::TBBPartitioner>();
       }
     else
       {
-        if (val != NULL)
-          free(val);
+        Number *new_values;
+        Utilities::System::posix_memalign ((void **)&new_values, 64, sizeof(Number)*new_alloc_size);
+        values.reset (new_values);
 
-        Utilities::System::posix_memalign ((void **)&val, 64, sizeof(Number)*new_alloc_size);
-        if (new_alloc_size >= 4*dealii::internal::Vector::minimum_parallel_grain_size)
-          thread_loop_partitioner.reset(new parallel::internal::TBBPartitioner());
+        if (new_alloc_size >= 4*dealii::internal::VectorImplementation::minimum_parallel_grain_size)
+          thread_loop_partitioner = std::make_shared<parallel::internal::TBBPartitioner>();
       }
   }
 
@@ -133,7 +133,35 @@ namespace LinearAlgebra
 
 
 
-#ifdef DEAL_II_WITH_CXX11
+#if defined(DEAL_II_WITH_TRILINOS) && defined(DEAL_II_WITH_MPI)
+  template <typename Number>
+  void
+  ReadWriteVector<Number>::reinit(const TrilinosWrappers::MPI::Vector &trilinos_vec)
+  {
+    // TODO: We could avoid copying the data by just using a view into the
+    // trilinos data but only if Number=double. Also update documentation that
+    // the argument's lifetime needs to be longer then. If we do this, we need
+    // to think about whether the view should be read/write.
+
+    stored_elements = IndexSet(trilinos_vec.vector_partitioner());
+
+    resize_val(stored_elements.n_elements());
+
+    TrilinosScalar *start_ptr;
+    int leading_dimension;
+    int ierr = trilinos_vec.trilinos_vector().ExtractView (&start_ptr, &leading_dimension);
+    AssertThrow (ierr == 0, ExcTrilinosError(ierr));
+
+    std::copy(start_ptr, start_ptr + leading_dimension, values.get());
+
+    // reset the communication pattern
+    source_stored_elements.clear();
+    comm_pattern.reset();
+  }
+#endif
+
+
+
   template <typename Number>
   template <typename Functor>
   void
@@ -142,7 +170,6 @@ namespace LinearAlgebra
     FunctorTemplate<Functor> functor(*this, func);
     internal::VectorOperations::parallel_for(functor, 0, n_elements(), thread_loop_partitioner);
   }
-#endif
 
 
 
@@ -157,7 +184,7 @@ namespace LinearAlgebra
     if (n_elements() != in_vector.n_elements())
       reinit(in_vector, true);
 
-    dealii::internal::VectorOperations::Vector_copy<Number,Number> copier(in_vector.val, val);
+    dealii::internal::VectorOperations::Vector_copy<Number,Number> copier(in_vector.values.get(), values.get());
     dealii::internal::VectorOperations::parallel_for(copier, 0, n_elements(), thread_loop_partitioner);
 
     return *this;
@@ -174,7 +201,7 @@ namespace LinearAlgebra
     if (n_elements() != in_vector.n_elements())
       reinit(in_vector, true);
 
-    dealii::internal::VectorOperations::Vector_copy<Number,Number2> copier(in_vector.val, val);
+    dealii::internal::VectorOperations::Vector_copy<Number,Number2> copier(in_vector.values.get(), values.get());
     dealii::internal::VectorOperations::parallel_for(copier, 0, n_elements(), thread_loop_partitioner);
 
     return *this;
@@ -189,8 +216,12 @@ namespace LinearAlgebra
     Assert(s==static_cast<Number>(0), ExcMessage("Only 0 can be assigned to a vector."));
     (void)s;
 
-    dealii::internal::VectorOperations::Vector_set<Number> setter(Number(), val);
-    dealii::internal::VectorOperations::parallel_for(setter, 0, n_elements(), thread_loop_partitioner);
+    const size_type this_size = n_elements();
+    if (this_size>0)
+      {
+        dealii::internal::VectorOperations::Vector_set<Number> setter(Number(), values.get());
+        dealii::internal::VectorOperations::parallel_for(setter, 0, this_size, thread_loop_partitioner);
+      }
 
     return *this;
   }
@@ -201,22 +232,23 @@ namespace LinearAlgebra
   void
   ReadWriteVector<Number>::import(const distributed::Vector<Number> &vec,
                                   VectorOperation::values operation,
-                                  std_cxx11::shared_ptr<const CommunicationPatternBase> communication_pattern)
+                                  const std::shared_ptr<const CommunicationPatternBase> &communication_pattern)
   {
     // If no communication pattern is given, create one. Otherwise, use the
     // given one.
-    std_cxx11::shared_ptr<const Utilities::MPI::Partitioner> comm_pattern;
-    if (communication_pattern.get() == NULL)
+    std::shared_ptr<const Utilities::MPI::Partitioner> comm_pattern;
+    if (communication_pattern.get() == nullptr)
       {
-        comm_pattern.reset(new Utilities::MPI::Partitioner(vec.locally_owned_elements(),
-                                                           get_stored_elements(),
-                                                           vec.get_mpi_communicator()));
+        comm_pattern = std::make_shared<Utilities::MPI::Partitioner>
+                       (vec.locally_owned_elements(),
+                        get_stored_elements(),
+                        vec.get_mpi_communicator());
       }
     else
       {
         comm_pattern =
-          std_cxx11::dynamic_pointer_cast<const Utilities::MPI::Partitioner> (communication_pattern);
-        AssertThrow(comm_pattern != NULL,
+          std::dynamic_pointer_cast<const Utilities::MPI::Partitioner> (communication_pattern);
+        AssertThrow(comm_pattern != nullptr,
                     ExcMessage("The communication pattern is not of type "
                                "Utilities::MPI::Partitioner."));
       }
@@ -270,7 +302,7 @@ namespace LinearAlgebra
   void
   ReadWriteVector<Number>::import(const PETScWrappers::MPI::Vector &petsc_vec,
                                   VectorOperation::values /*operation*/,
-                                  std_cxx11::shared_ptr<const CommunicationPatternBase> /*communication_pattern*/)
+                                  const std::shared_ptr<const CommunicationPatternBase> & /*communication_pattern*/)
   {
     //TODO: this works only if no communication is needed.
     Assert(petsc_vec.locally_owned_elements() == stored_elements,
@@ -299,13 +331,13 @@ namespace LinearAlgebra
                                   const IndexSet                  &source_elements,
                                   VectorOperation::values          operation,
                                   const MPI_Comm                  &mpi_comm,
-                                  std_cxx11::shared_ptr<const CommunicationPatternBase> communication_pattern)
+                                  const std::shared_ptr<const CommunicationPatternBase> &communication_pattern)
   {
-    std_cxx11::shared_ptr<const EpetraWrappers::CommunicationPattern> epetra_comm_pattern;
+    std::shared_ptr<const EpetraWrappers::CommunicationPattern> epetra_comm_pattern;
 
     // If no communication pattern is given, create one. Otherwise, use the one
     // given.
-    if (communication_pattern == NULL)
+    if (communication_pattern == nullptr)
       {
         // The first time import is called, we create a communication pattern.
         // Check if the communication pattern already exists and if it can be
@@ -314,20 +346,20 @@ namespace LinearAlgebra
             (source_elements == source_stored_elements))
           {
             epetra_comm_pattern =
-              std_cxx11::dynamic_pointer_cast<const EpetraWrappers::CommunicationPattern> (comm_pattern);
-            if (epetra_comm_pattern == NULL)
-              epetra_comm_pattern = std_cxx11::make_shared<const EpetraWrappers::CommunicationPattern>(
+              std::dynamic_pointer_cast<const EpetraWrappers::CommunicationPattern> (comm_pattern);
+            if (epetra_comm_pattern == nullptr)
+              epetra_comm_pattern = std::make_shared<const EpetraWrappers::CommunicationPattern>(
                                       create_epetra_comm_pattern(source_elements, mpi_comm));
           }
         else
-          epetra_comm_pattern = std_cxx11::make_shared<const EpetraWrappers::CommunicationPattern>(
+          epetra_comm_pattern = std::make_shared<const EpetraWrappers::CommunicationPattern>(
                                   create_epetra_comm_pattern(source_elements, mpi_comm));
       }
     else
       {
-        epetra_comm_pattern = std_cxx11::dynamic_pointer_cast<const EpetraWrappers::CommunicationPattern> (
+        epetra_comm_pattern = std::dynamic_pointer_cast<const EpetraWrappers::CommunicationPattern> (
                                 communication_pattern);
-        AssertThrow(epetra_comm_pattern != NULL,
+        AssertThrow(epetra_comm_pattern != nullptr,
                     ExcMessage(std::string("The communication pattern is not of type ") +
                                "LinearAlgebra::EpetraWrappers::CommunicationPattern."));
       }
@@ -338,20 +370,32 @@ namespace LinearAlgebra
 
     if (operation==VectorOperation::insert)
       {
-        target_vector.Import(multivector, import, Insert);
-        double *values = target_vector.Values();
+        const int err = target_vector.Import(multivector, import, Insert);
+        AssertThrow(err == 0, ExcMessage("Epetra Import() failed with error code: "
+                                         + Utilities::to_string(err)));
+
+        const double *new_values = target_vector.Values();
         const int size = target_vector.MyLength();
+        Assert(size==0 || values != nullptr, ExcInternalError("Import failed."));
+
         for (int i=0; i<size; ++i)
-          val[i] = values[i];
+          values[i] = new_values[i];
+      }
+    else if (operation==VectorOperation::add)
+      {
+        const int err = target_vector.Import(multivector, import, Add);
+        AssertThrow(err == 0, ExcMessage("Epetra Import() failed with error code: "
+                                         + Utilities::to_string(err)));
+
+        const double *new_values = target_vector.Values();
+        const int size = target_vector.MyLength();
+        Assert(size==0 || values != nullptr, ExcInternalError("Import failed."));
+
+        for (int i=0; i<size; ++i)
+          values[i] += new_values[i];
       }
     else
-      {
-        target_vector.Import(multivector, import, Add);
-        double *values = target_vector.Values();
-        const int size = target_vector.MyLength();
-        for (int i=0; i<size; ++i)
-          val[i] += values[i];
-      }
+      AssertThrow(false, ExcNotImplemented());
   }
 
 
@@ -359,8 +403,13 @@ namespace LinearAlgebra
   void
   ReadWriteVector<Number>::import(const TrilinosWrappers::MPI::Vector            &trilinos_vec,
                                   VectorOperation::values                         operation,
-                                  std_cxx11::shared_ptr<const CommunicationPatternBase> communication_pattern)
+                                  std::shared_ptr<const CommunicationPatternBase> communication_pattern)
   {
+    // While the import does work with Trilinos 12.8.x, it fails with 12.4.x. To be safe,
+    // we disable it here. Note that it would be a useful case, as ReadWriteVector is
+    // supposed to replace ghosted vectors anyways.
+    AssertThrow(!trilinos_vec.has_ghost_elements(),
+                ExcMessage("Import() from TrilinosWrappers::MPI::Vector with ghost entries is not supported!"));
     import(trilinos_vec.trilinos_vector(), trilinos_vec.locally_owned_elements(),
            operation, trilinos_vec.get_mpi_communicator(), communication_pattern);
   }
@@ -371,7 +420,7 @@ namespace LinearAlgebra
   void
   ReadWriteVector<Number>::import(const LinearAlgebra::EpetraWrappers::Vector    &trilinos_vec,
                                   VectorOperation::values                         operation,
-                                  std_cxx11::shared_ptr<const CommunicationPatternBase> communication_pattern)
+                                  std::shared_ptr<const CommunicationPatternBase> communication_pattern)
   {
     import(trilinos_vec.trilinos_vector(), trilinos_vec.locally_owned_elements(),
            operation, trilinos_vec.get_mpi_communicator(), communication_pattern);
@@ -385,12 +434,12 @@ namespace LinearAlgebra
   void
   ReadWriteVector<Number>::import(const LinearAlgebra::CUDAWrappers::Vector<Number> &cuda_vec,
                                   VectorOperation::values                            operation,
-                                  std_cxx11::shared_ptr<const CommunicationPatternBase> )
+                                  std::shared_ptr<const CommunicationPatternBase> )
   {
     const unsigned int n_elements = stored_elements.n_elements();
     if (operation == VectorOperation::insert)
       {
-        cudaError_t error_code = cudaMemcpy(&val[0], cuda_vec.get_values(),
+        cudaError_t error_code = cudaMemcpy(values.get(), cuda_vec.get_values(),
                                             n_elements*sizeof(Number),
                                             cudaMemcpyDeviceToHost);
         AssertCuda(error_code);
@@ -399,14 +448,14 @@ namespace LinearAlgebra
       {
         // Copy the vector from the device to a temporary vector on the host
         std::vector<Number> tmp(n_elements);
-        cudaError_t error_code = cudaMemcpy(&tmp[0], cuda_vec.get_values(),
+        cudaError_t error_code = cudaMemcpy(tmp.data(), cuda_vec.get_values(),
                                             n_elements*sizeof(Number),
                                             cudaMemcpyDeviceToHost);
         AssertCuda(error_code);
 
         // Add the two vectors
         for (unsigned int i=0; i<n_elements; ++i)
-          val[i] += tmp[i];
+          values[i] += tmp[i];
       }
   }
 #endif
@@ -418,7 +467,7 @@ namespace LinearAlgebra
   ReadWriteVector<Number>::swap (ReadWriteVector<Number> &v)
   {
     std::swap(stored_elements, v.stored_elements);
-    std::swap(val, v.val);
+    std::swap(values, v.values);
   }
 
 
@@ -444,8 +493,7 @@ namespace LinearAlgebra
                                   const bool         scientific) const
   {
     AssertThrow (out, ExcIO());
-    std::ios::fmtflags old_flags = out.flags();
-    unsigned int old_precision = out.precision (precision);
+    boost::io::ios_flags_saver restore_flags(out);
 
     out.precision (precision);
     if (scientific)
@@ -456,15 +504,12 @@ namespace LinearAlgebra
     out << "IndexSet: ";
     stored_elements.print(out);
     out << std::endl;
-    for (unsigned int i=0; i<this->n_elements(); ++i)
-      out << val[i] << std::endl;
+    unsigned int i=0;
+    for (const auto &idx : this->stored_elements)
+      out << "[" << idx << "]: " << values[i++] << '\n';
     out << std::flush;
 
-
     AssertThrow (out, ExcIO());
-    // reset output format
-    out.flags (old_flags);
-    out.precision(old_precision);
   }
 
 
@@ -478,8 +523,8 @@ namespace LinearAlgebra
     source_stored_elements = source_index_set;
     EpetraWrappers::CommunicationPattern epetra_comm_pattern(
       source_stored_elements, stored_elements, mpi_comm);
-    comm_pattern.reset(new EpetraWrappers::CommunicationPattern(
-                         source_stored_elements, stored_elements, mpi_comm));
+    comm_pattern = std::make_shared<EpetraWrappers::CommunicationPattern>
+                   (source_stored_elements, stored_elements, mpi_comm);
 
     return epetra_comm_pattern;
   }
