@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 2010 - 2016 by the deal.II authors
+// Copyright (C) 2010 - 2018 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -13,8 +13,8 @@
 //
 // ---------------------------------------------------------------------
 
-#ifndef dealii__relaxation_block_h
-#define dealii__relaxation_block_h
+#ifndef dealii_relaxation_block_h
+#define dealii_relaxation_block_h
 
 #include <deal.II/base/subscriptor.h>
 #include <deal.II/base/smartpointer.h>
@@ -91,7 +91,7 @@ public:
                     const typename PreconditionBlockBase<InverseNumberType>::Inversion inversion
                     = PreconditionBlockBase<InverseNumberType>::gauss_jordan,
                     const double threshold = 0.,
-                    VectorType *temp_ghost_vector = NULL);
+                    VectorType *temp_ghost_vector = nullptr);
 
     /**
      * The mapping from indices to blocks. Each row of this pattern enumerates
@@ -131,10 +131,23 @@ public:
      * If #inversion is SVD, we can compute the Penrose-Moore inverse of the
      * blocks. In order to do so, we can specify here the threshold below
      * which a singular value will be considered zero and thus not inverted.
+     * Setting this parameter to a value greater than zero takes precedence over
+     * threshold, i.e. kernel_size must be zero if you want to use threshold.
      * This parameter is used in the call to
      * LAPACKFullMatrix::compute_inverse_svd().
      */
-    double threshold;
+    double threshold = 0.;
+
+    /**
+     * If #inversion is SVD, we can compute the Penrose-Moore inverse of the
+     * blocks. In order to do so, we can specify here the size of the kernel
+     * that will not be inverted but considered zero. Setting this parameter
+     * to a value greater than zero takes precedence over threshold, i.e.
+     * kernel_size must be zero if you want to use threshold.
+     * This parameter is used in the call to
+     * LAPACKFullMatrix::compute_inverse_svd().
+     */
+    unsigned int kernel_size = 0;
 
     /**
      * The order in which blocks should be traversed. This vector can initiate
@@ -193,18 +206,6 @@ public:
    */
   void clear();
 
-  /**
-   * Check whether the object is empty.
-   */
-  bool empty () const;
-
-  /**
-   * Read-only access to entries.  This function is only possible if the
-   * inverse diagonal blocks are stored.
-   */
-  value_type el(size_type i,
-                size_type j) const;
-
   /**
    * Stores the inverse of the diagonal blocks in @p inverse. This costs some
    * additional memory - for DG methods about 1/3 (for double inverses) or 1/6
@@ -227,9 +228,9 @@ protected:
    * Perform one block relaxation step.
    *
    * Depending on the arguments @p dst and @p pref, this performs an SOR step
-   * (both reference the same vector) of a Jacobi step (both are different
+   * (both reference the same vector) or a Jacobi step (both are different
    * vectors). For the Jacobi step, the calling function must copy @p dst to
-   * @p pref after this.
+   * @p prev after this.
    */
   void do_step (
     VectorType       &dst,
@@ -273,9 +274,9 @@ private:
  * @author Guido Kanschat
  * @date 2010
  */
-template<typename MatrixType,
-         typename InverseNumberType = typename MatrixType::value_type,
-         typename VectorType = Vector<double> >
+template <typename MatrixType,
+          typename InverseNumberType = typename MatrixType::value_type,
+          typename VectorType = Vector<double> >
 class RelaxationBlockJacobi : public virtual Subscriptor,
   protected RelaxationBlock<MatrixType, InverseNumberType, VectorType>
 {
@@ -305,10 +306,6 @@ public:
    */
   using RelaxationBlock<MatrixType, InverseNumberType, VectorType>::clear;
 
-  /**
-   * Make function of base class public again.
-   */
-  using RelaxationBlock<MatrixType, InverseNumberType, VectorType>::empty;
   /**
    * Make function of base class public again.
    */
@@ -340,9 +337,16 @@ public:
   void Tstep (VectorType &dst, const VectorType &rhs) const;
 
   /**
-   * Return the memory allocated in this object.
+   * Implements a vmult() operation, which for this class first sets the dst()
+   * vector to zero before calling the step() method.
    */
-  std::size_t memory_consumption() const;
+  void vmult (VectorType &dst, const VectorType &rhs) const;
+
+  /**
+   * Implements a transpose vmult operation, which for this class first sets
+   * the dst() vector to zero before calling the Tstep() method.
+   */
+  void Tvmult (VectorType &dst, const VectorType &rhs) const;
 };
 
 
@@ -361,9 +365,9 @@ public:
  * @author Guido Kanschat
  * @date 2010
  */
-template<typename MatrixType,
-         typename InverseNumberType = typename MatrixType::value_type,
-         typename VectorType = Vector<double> >
+template <typename MatrixType,
+          typename InverseNumberType = typename MatrixType::value_type,
+          typename VectorType = Vector<double> >
 class RelaxationBlockSOR : public virtual Subscriptor,
   protected RelaxationBlock<MatrixType, InverseNumberType, VectorType>
 {
@@ -393,10 +397,6 @@ public:
    */
   using RelaxationBlock<MatrixType, InverseNumberType, VectorType>::clear;
 
-  /**
-   * Make function of base class public again.
-   */
-  using RelaxationBlock<MatrixType, InverseNumberType, VectorType>::empty;
   /**
    * Make function of base class public again.
    */
@@ -426,6 +426,18 @@ public:
    * Perform one step of the transposed SOR iteration.
    */
   void Tstep (VectorType &dst, const VectorType &rhs) const;
+
+  /**
+   * Implements a vmult() operation, which for this class first sets the dst()
+   * vector to zero before calling the step() method.
+   */
+  void vmult (VectorType &dst, const VectorType &rhs) const;
+
+  /**
+   * Implements a transpose vmult operation, which for this class first sets
+   * the dst() vector to zero before calling the Tstep() method.
+   */
+  void Tvmult (VectorType &dst, const VectorType &rhs) const;
 };
 
 
@@ -444,9 +456,9 @@ public:
  * @author Guido Kanschat
  * @date 2010
  */
-template<typename MatrixType,
-         typename InverseNumberType = typename MatrixType::value_type,
-         typename VectorType = Vector<double> >
+template <typename MatrixType,
+          typename InverseNumberType = typename MatrixType::value_type,
+          typename VectorType = Vector<double> >
 class RelaxationBlockSSOR : public virtual Subscriptor,
   protected RelaxationBlock<MatrixType, InverseNumberType, VectorType>
 {
@@ -471,11 +483,6 @@ public:
    */
   using RelaxationBlock<MatrixType, InverseNumberType, VectorType>::clear;
 
-  /**
-   * Make function of base class public again.
-   */
-  using RelaxationBlock<MatrixType, InverseNumberType, VectorType>::empty;
-
   /**
    * Make function of base class public again.
    */
@@ -505,6 +512,18 @@ public:
    * Perform one step of the transposed SSOR iteration.
    */
   void Tstep (VectorType &dst, const VectorType &rhs) const;
+
+  /**
+   * Implements a vmult() operation, which for this class first sets the dst()
+   * vector to zero before calling the step() method.
+   */
+  void vmult (VectorType &dst, const VectorType &rhs) const;
+
+  /**
+   * Implements a transpose vmult operation, which for this class first sets
+   * the dst() vector to zero before calling the Tstep() method.
+   */
+  void Tvmult (VectorType &dst, const VectorType &rhs) const;
 };
 
 
