@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 1999 - 2016 by the deal.II authors
+// Copyright (C) 1999 - 2018 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -13,8 +13,8 @@
 //
 // ---------------------------------------------------------------------
 
-#ifndef dealii__relaxation_block_templates_h
-#define dealii__relaxation_block_templates_h
+#ifndef dealii_relaxation_block_templates_h
+#define dealii_relaxation_block_templates_h
 
 #include <deal.II/lac/relaxation_block.h>
 #include <deal.II/lac/full_matrix.h>
@@ -83,8 +83,8 @@ inline
 void
 RelaxationBlock<MatrixType, InverseNumberType, VectorType>::clear ()
 {
-  A = 0;
-  additional_data = 0;
+  A = nullptr;
+  additional_data = nullptr;
   PreconditionBlockBase<InverseNumberType>::clear ();
 }
 
@@ -102,8 +102,8 @@ RelaxationBlock<MatrixType, InverseNumberType, VectorType>::invert_diagblocks ()
     {
       // compute blocks in parallel
       parallel::apply_to_subranges(0, this->additional_data->block_list.n_rows(),
-                                   std_cxx11::bind(&RelaxationBlock<MatrixType, InverseNumberType, VectorType>::block_kernel, this,
-                                                   std_cxx11::_1, std_cxx11::_2),
+                                   std::bind(&RelaxationBlock<MatrixType, InverseNumberType, VectorType>::block_kernel, this,
+                                             std::placeholders::_1, std::placeholders::_2),
                                    16);
     }
   this->inverses_computed(true);
@@ -156,7 +156,10 @@ RelaxationBlock<MatrixType, InverseNumberType, VectorType>::block_kernel (const
         case PreconditionBlockBase<InverseNumberType>::svd:
           this->inverse_svd(block).reinit(bs, bs);
           this->inverse_svd(block) = M_cell;
-          this->inverse_svd(block).compute_inverse_svd(this->additional_data->threshold);
+          if (this->additional_data->kernel_size>0)
+            this->inverse_svd(block).compute_inverse_svd_with_kernel(this->additional_data->kernel_size);
+          else
+            this->inverse_svd(block).compute_inverse_svd(this->additional_data->threshold);
           break;
         default:
           Assert(false, ExcNotImplemented());
@@ -179,7 +182,7 @@ namespace internal
     // If the following Assertion triggers, you either set temp_ghost_vector
     // for a serial computation (don't!), or nobody implemented, instantiated, and
     // tested the parallel version for your vector type.
-    Assert(other==NULL, ExcNotImplemented());
+    Assert(other==nullptr, ExcNotImplemented());
     (void)other;
     return prev;
   }
@@ -194,7 +197,7 @@ namespace internal
     const TrilinosWrappers::MPI::Vector &prev,
     TrilinosWrappers::MPI::Vector *other)
   {
-    Assert(other!=NULL,
+    Assert(other!=nullptr,
            ExcMessage("You need to provide a ghosted vector in RelaxationBlock::AdditionalData::temp_trilinos_ghost_vector."));
     Assert(other->size()==prev.size(), ExcInternalError());
 
@@ -279,7 +282,7 @@ void RelaxationBlockJacobi<MatrixType, InverseNumberType, VectorType>::step
  const VectorType &src) const
 {
   GrowingVectorMemory<VectorType> mem;
-  typename VectorMemory<VectorType>::Pointer aux = mem;
+  typename VectorMemory<VectorType>::Pointer aux(mem);
   aux->reinit(dst, false);
   *aux = dst;
   this->do_step(dst, *aux, src, false);
@@ -292,13 +295,39 @@ void RelaxationBlockJacobi<MatrixType, InverseNumberType, VectorType>::Tstep
  const VectorType &src) const
 {
   GrowingVectorMemory<VectorType> mem;
-  typename VectorMemory<VectorType>::Pointer aux = mem;
+  typename VectorMemory<VectorType>::Pointer aux(mem);
   aux->reinit(dst, false);
   *aux = dst;
   this->do_step(dst, *aux, src, true);
 }
 
 
+template <typename MatrixType, typename InverseNumberType, typename VectorType>
+void RelaxationBlockJacobi<MatrixType, InverseNumberType, VectorType>::vmult
+(VectorType       &dst,
+ const VectorType &src) const
+{
+  GrowingVectorMemory<VectorType> mem;
+  typename VectorMemory<VectorType>::Pointer aux(mem);
+  dst = 0;
+  aux->reinit(dst);
+  this->do_step(dst, *aux, src, false);
+}
+
+
+template <typename MatrixType, typename InverseNumberType, typename VectorType>
+void RelaxationBlockJacobi<MatrixType, InverseNumberType, VectorType>::Tvmult
+(VectorType       &dst,
+ const VectorType &src) const
+{
+  GrowingVectorMemory<VectorType> mem;
+  typename VectorMemory<VectorType>::Pointer aux(mem);
+  dst = 0;
+  aux->reinit(dst);
+  this->do_step(dst, *aux, src, true);
+}
+
+
 //----------------------------------------------------------------------//
 
 template <typename MatrixType, typename InverseNumberType, typename VectorType>
@@ -319,6 +348,26 @@ void RelaxationBlockSOR<MatrixType, InverseNumberType, VectorType>::Tstep
 }
 
 
+template <typename MatrixType, typename InverseNumberType, typename VectorType>
+void RelaxationBlockSOR<MatrixType, InverseNumberType, VectorType>::vmult
+(VectorType &dst,
+ const VectorType &src) const
+{
+  dst = 0;
+  this->do_step(dst, dst, src, false);
+}
+
+
+template <typename MatrixType, typename InverseNumberType, typename VectorType>
+void RelaxationBlockSOR<MatrixType, InverseNumberType, VectorType>::Tvmult
+(VectorType       &dst,
+ const VectorType &src) const
+{
+  dst = 0;
+  this->do_step(dst, dst, src, true);
+}
+
+
 //----------------------------------------------------------------------//
 
 template <typename MatrixType, typename InverseNumberType, typename VectorType>
@@ -341,6 +390,28 @@ void RelaxationBlockSSOR<MatrixType, InverseNumberType, VectorType>::Tstep
 }
 
 
+template <typename MatrixType, typename InverseNumberType, typename VectorType>
+void RelaxationBlockSSOR<MatrixType, InverseNumberType, VectorType>::vmult
+(VectorType       &dst,
+ const VectorType &src) const
+{
+  dst = 0;
+  this->do_step(dst, dst, src, false);
+  this->do_step(dst, dst, src, true);
+}
+
+
+template <typename MatrixType, typename InverseNumberType, typename VectorType>
+void RelaxationBlockSSOR<MatrixType, InverseNumberType, VectorType>::Tvmult
+(VectorType       &dst,
+ const VectorType &src) const
+{
+  dst = 0;
+  this->do_step(dst, dst, src, true);
+  this->do_step(dst, dst, src, false);
+}
+
+
 
 DEAL_II_NAMESPACE_CLOSE
 
