@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 2015 - 2017 by the deal.II authors
+// Copyright (C) 2015 - 2018 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -13,8 +13,8 @@
 //
 // ---------------------------------------------------------------------
 
-#ifndef dealii__schur_complement_h
-#define dealii__schur_complement_h
+#ifndef dealii_schur_complement_h
+#define dealii_schur_complement_h
 
 #include <deal.II/base/config.h>
 #include <deal.II/base/exceptions.h>
@@ -22,7 +22,6 @@
 #include <deal.II/lac/linear_operator.h>
 #include <deal.II/lac/packaged_operation.h>
 
-#ifdef DEAL_II_WITH_CXX11
 
 DEAL_II_NAMESPACE_OPEN
 
@@ -32,7 +31,7 @@ DEAL_II_NAMESPACE_OPEN
 //@{
 
 /**
- * @relates LinearOperator
+ * @relatesalso LinearOperator
  *
  * Return a LinearOperator that performs the operations associated with the
  * Schur complement. There are two additional helper functions,
@@ -252,7 +251,7 @@ schur_complement(const LinearOperator<Domain_1, Range_1, Payload> &A_inv,
 //@{
 
 /**
- * @relates PackagedOperation
+ * @relatesalso PackagedOperation
  *
  * For the system of equations
  * @f{eqnarray*}{
@@ -290,7 +289,7 @@ condense_schur_rhs (const LinearOperator<Range_1, Domain_1, Payload> &A_inv,
 }
 
 /**
- * @relates PackagedOperation
+ * @relatesalso PackagedOperation
  *
  * For the system of equations
  * @f{eqnarray*}{
@@ -330,5 +329,4 @@ postprocess_schur_solution (const LinearOperator<Range_1, Domain_1, Payload> &A_
 
 DEAL_II_NAMESPACE_CLOSE
 
-#endif // DEAL_II_WITH_CXX11
 #endif
