@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 2009 - 2016 by the deal.II authors
+// Copyright (C) 2009 - 2017 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -14,15 +14,13 @@
 // ---------------------------------------------------------------------
 
 
-#ifndef dealii__slepc_solver_h
-#define dealii__slepc_solver_h
+#ifndef dealii_slepc_solver_h
+#define dealii_slepc_solver_h
 
 #include <deal.II/base/config.h>
 
 #ifdef DEAL_II_WITH_SLEPC
 
-#  include <deal.II/base/std_cxx11/shared_ptr.h>
-#  include <deal.II/lac/exceptions.h>
 #  include <deal.II/lac/solver_control.h>
 #  include <deal.II/lac/petsc_matrix_base.h>
 #  include <deal.II/lac/slepc_spectral_transformation.h>
@@ -31,6 +29,8 @@
 #  include <petscksp.h>
 #  include <slepceps.h>
 
+#  include <memory>
+
 DEAL_II_NAMESPACE_OPEN
 
 /**
@@ -208,9 +208,10 @@ namespace SLEPcWrappers
     /**
      * Set the initial vector for the solver.
      */
+    DEAL_II_DEPRECATED
     void
     set_initial_vector
-    (const PETScWrappers::VectorBase &this_initial_vector) DEAL_II_DEPRECATED;
+    (const PETScWrappers::VectorBase &this_initial_vector);
 
     /**
      * Set the initial vector space for the solver.
@@ -803,11 +804,7 @@ namespace SLEPcWrappers
     // One could still build a vector that is rich in the directions of all guesses,
     // by taking a linear combination of them. (TODO: make function virtual?)
 
-#if DEAL_II_PETSC_VERSION_LT(3,1,0)
-    const PetscErrorCode ierr = EPSSetInitialVector (eps, &vecs[0]);
-#else
-    const PetscErrorCode ierr = EPSSetInitialSpace (eps, vecs.size(), &vecs[0]);
-#endif
+    const PetscErrorCode ierr = EPSSetInitialSpace (eps, vecs.size(), vecs.data());
     AssertThrow (ierr == 0, ExcSLEPcError(ierr));
   }
 
