@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 1999 - 2015 by the deal.II authors
+// Copyright (C) 1999 - 2017 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -14,8 +14,8 @@
 // ---------------------------------------------------------------------
 
 
-#ifndef dealii__sparse_ilu_h
-#define dealii__sparse_ilu_h
+#ifndef dealii_sparse_ilu_h
+#define dealii_sparse_ilu_h
 
 
 #include <deal.II/base/config.h>
@@ -71,7 +71,7 @@ public:
    * Call the @p initialize function before using this object as
    * preconditioner.
    */
-  SparseILU ();
+  SparseILU () = default;
 
   /**
    * Make SparseLUDecomposition::AdditionalData accessible to this class as
@@ -162,4 +162,4 @@ public:
 
 DEAL_II_NAMESPACE_CLOSE
 
-#endif // dealii__sparse_ilu_h
+#endif // dealii_sparse_ilu_h
