@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 1999 - 2015 by the deal.II authors
+// Copyright (C) 1999 - 2017 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -13,8 +13,8 @@
 //
 // ---------------------------------------------------------------------
 
-#ifndef dealii__sparse_ilu_templates_h
-#define dealii__sparse_ilu_templates_h
+#ifndef dealii_sparse_ilu_templates_h
+#define dealii_sparse_ilu_templates_h
 
 
 
@@ -28,11 +28,6 @@
 
 DEAL_II_NAMESPACE_OPEN
 
-template <typename number>
-SparseILU<number>::SparseILU ()
-{}
-
-
 
 template <typename number>
 template <typename somenumber>
@@ -59,10 +54,10 @@ void SparseILU<number>::initialize (const SparseMatrix<somenumber> &matrix,
   // translating in essence the algorithm given at the end of section 10.3.2,
   // using the names of variables used there
   const SparsityPattern     &sparsity = this->get_sparsity_pattern();
-  const std::size_t *const ia    = sparsity.rowstart;
-  const size_type *const ja      = sparsity.colnums;
+  const std::size_t *const ia    = sparsity.rowstart.get();
+  const size_type *const ja      = sparsity.colnums.get();
 
-  number *luval = this->SparseMatrix<number>::val;
+  number *luval = this->SparseMatrix<number>::val.get();
 
   const size_type N = this->m();
   size_type jrow = 0;
@@ -147,9 +142,9 @@ void SparseILU<number>::vmult (Vector<somenumber>       &dst,
 
   const size_type N=dst.size();
   const std::size_t *const rowstart_indices
-    = this->get_sparsity_pattern().rowstart;
+    = this->get_sparsity_pattern().rowstart.get();
   const size_type *const column_numbers
-    = this->get_sparsity_pattern().colnums;
+    = this->get_sparsity_pattern().colnums.get();
 
   // solve LUx=b in two steps:
   // first Ly = b, then
@@ -174,7 +169,7 @@ void SparseILU<number>::vmult (Vector<somenumber>       &dst,
       const size_type *const first_after_diagonal = this->prebuilt_lower_bound[row];
 
       somenumber dst_row = dst(row);
-      const number *luval = this->SparseMatrix<number>::val +
+      const number *luval = this->SparseMatrix<number>::val.get() +
                             (rowstart - column_numbers);
       for (const size_type *col=rowstart; col!=first_after_diagonal; ++col, ++luval)
         dst_row -= *luval * dst(*col);
@@ -198,7 +193,7 @@ void SparseILU<number>::vmult (Vector<somenumber>       &dst,
       const size_type *const first_after_diagonal = this->prebuilt_lower_bound[row];
 
       somenumber dst_row = dst(row);
-      const number *luval = this->SparseMatrix<number>::val +
+      const number *luval = this->SparseMatrix<number>::val.get() +
                             (first_after_diagonal - column_numbers);
       for (const size_type *col=first_after_diagonal; col!=rowend; ++col, ++luval)
         dst_row -= *luval * dst(*col);
@@ -221,9 +216,9 @@ void SparseILU<number>::Tvmult (Vector<somenumber>       &dst,
 
   const size_type N=dst.size();
   const std::size_t *const rowstart_indices
-    = this->get_sparsity_pattern().rowstart;
+    = this->get_sparsity_pattern().rowstart.get();
   const size_type *const column_numbers
-    = this->get_sparsity_pattern().colnums;
+    = this->get_sparsity_pattern().colnums.get();
 
   // solve (LU)'x=b in two steps:
   // first U'y = b, then
@@ -251,7 +246,7 @@ void SparseILU<number>::Tvmult (Vector<somenumber>       &dst,
       const size_type *const first_after_diagonal = this->prebuilt_lower_bound[row];
 
       const somenumber dst_row = dst (row);
-      const number *luval = this->SparseMatrix<number>::val +
+      const number *luval = this->SparseMatrix<number>::val.get() +
                             (first_after_diagonal - column_numbers);
       for (const size_type *col=first_after_diagonal; col!=rowend; ++col, ++luval)
         tmp(*col) += *luval * dst_row;
@@ -278,7 +273,7 @@ void SparseILU<number>::Tvmult (Vector<somenumber>       &dst,
       const size_type *const first_after_diagonal = this->prebuilt_lower_bound[row];
 
       const somenumber dst_row = dst (row);
-      const number *luval = this->SparseMatrix<number>::val +
+      const number *luval = this->SparseMatrix<number>::val.get() +
                             (rowstart - column_numbers);
       for (const size_type *col=rowstart; col!=first_after_diagonal; ++col, ++luval)
         tmp(*col) += *luval * dst_row;
