@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 1999 - 2017 by the deal.II authors
+// Copyright (C) 1999 - 2018 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -13,8 +13,8 @@
 //
 // ---------------------------------------------------------------------
 
-#ifndef dealii__sparse_matrix_h
-#define dealii__sparse_matrix_h
+#ifndef dealii_sparse_matrix_h
+#define dealii_sparse_matrix_h
 
 
 #include <deal.II/base/config.h>
@@ -23,7 +23,13 @@
 #include <deal.II/lac/sparsity_pattern.h>
 #include <deal.II/lac/identity_matrix.h>
 #include <deal.II/lac/exceptions.h>
-#include <deal.II/lac/vector.h>
+#include <deal.II/lac/vector_operation.h>
+#ifdef DEAL_II_WITH_MPI
+#include <mpi.h>
+#endif
+
+#include <memory>
+
 
 DEAL_II_NAMESPACE_OPEN
 
@@ -31,6 +37,15 @@ template <typename number> class Vector;
 template <typename number> class FullMatrix;
 template <typename Matrix> class BlockMatrixBase;
 template <typename number> class SparseILU;
+#ifdef DEAL_II_WITH_MPI
+namespace Utilities
+{
+  namespace MPI
+  {
+    template <typename Number> void sum (const SparseMatrix<Number> &, const MPI_Comm &, SparseMatrix<Number> &);
+  }
+}
+#endif
 
 #ifdef DEAL_II_WITH_TRILINOS
 namespace TrilinosWrappers
@@ -537,19 +552,14 @@ public:
    */
   SparseMatrix (const SparseMatrix &);
 
-#ifdef DEAL_II_WITH_CXX11
   /**
    * Move constructor. Construct a new sparse matrix by transferring the
    * internal data of the matrix @p m into a new object.
    *
    * Move construction allows an object to be returned from a function or
    * packed into a tuple even when the class cannot be copy-constructed.
-   *
-   * @note this constructor is only available if deal.II is configured with
-   * C++11 support.
    */
-  SparseMatrix (SparseMatrix<number> &&m);
-#endif
+  SparseMatrix (SparseMatrix<number> &&m) noexcept;
 
   /**
    * Constructor. Takes the given matrix sparsity structure to represent the
@@ -592,16 +602,11 @@ public:
    */
   SparseMatrix<number> &operator = (const SparseMatrix<number> &);
 
-#ifdef DEAL_II_WITH_CXX11
   /**
    * Move assignment operator. This operator replaces the present matrix with
    * @p m by transferring the internal data of @p m.
-   *
-   * @note This operator is only available if deal.II is configured with C++11
-   * support.
    */
-  SparseMatrix<number> &operator = (SparseMatrix<number> &&m);
-#endif
+  SparseMatrix<number> &operator = (SparseMatrix<number> &&m) noexcept;
 
   /**
    * Copy operator: initialize the matrix with the identity matrix. This
@@ -981,8 +986,14 @@ public:
    * classes instead, since they are tailored better to a sparse matrix
    * structure.
    */
-  number operator () (const size_type i,
-                      const size_type j) const;
+  const number &operator () (const size_type i,
+                             const size_type j) const;
+
+  /**
+   * In contrast to the one above, this function allows modifying the object.
+   */
+  number &operator () (const size_type i,
+                       const size_type j);
 
   /**
    * This function is mostly like operator()() in that it returns the value of
@@ -1570,7 +1581,10 @@ public:
   /**
    * Exception
    */
-  DeclException0 (ExcSourceEqualsDestination);
+  DeclExceptionMsg (ExcSourceEqualsDestination,
+                    "You are attempting an operation on two matrices that "
+                    "are the same object, but the operation requires that the "
+                    "two objects are in fact different.");
   //@}
 
 protected:
@@ -1601,12 +1615,13 @@ private:
   SmartPointer<const SparsityPattern,SparseMatrix<number> > cols;
 
   /**
-   * Array of values for all the nonzero entries. The position within the
-   * matrix, i.e.  the row and column number for a given entry can only be
-   * deduced using the sparsity pattern. The same holds for the more common
-   * operation of finding an entry by its coordinates.
+   * Array of values for all the nonzero entries. The position of an
+   * entry within the matrix, i.e., the row and column number for a
+   * given value in this array can only be deduced using the sparsity
+   * pattern. The same holds for the more common operation of finding
+   * an entry by its coordinates.
    */
-  number *val;
+  std::unique_ptr<number[]> val;
 
   /**
    * Allocated size of #val. This can be larger than the actually used part if
@@ -1631,6 +1646,13 @@ private:
    */
   template <typename,bool> friend class SparseMatrixIterators::Iterator;
   template <typename,bool> friend class SparseMatrixIterators::Accessor;
+
+#ifdef DEAL_II_WITH_MPI
+  /**
+   * Give access to internal datastructures to perform MPI operations.
+   */
+  template <typename Number> friend void Utilities::MPI::sum (const SparseMatrix<Number> &, const MPI_Comm &, SparseMatrix<Number> &);
+#endif
 };
 
 #ifndef DOXYGEN
@@ -1642,7 +1664,7 @@ template <typename number>
 inline
 typename SparseMatrix<number>::size_type SparseMatrix<number>::m () const
 {
-  Assert (cols != 0, ExcNotInitialized());
+  Assert (cols != nullptr, ExcNotInitialized());
   return cols->rows;
 }
 
@@ -1651,7 +1673,7 @@ template <typename number>
 inline
 typename SparseMatrix<number>::size_type SparseMatrix<number>::n () const
 {
-  Assert (cols != 0, ExcNotInitialized());
+  Assert (cols != nullptr, ExcNotInitialized());
   return cols->cols;
 }
 
@@ -1696,7 +1718,7 @@ SparseMatrix<number>::set (const std::vector<size_type> &indices,
   Assert (values.m() == values.n(), ExcNotQuadratic());
 
   for (size_type i=0; i<indices.size(); ++i)
-    set (indices[i], indices.size(), &indices[0], &values(i,0),
+    set (indices[i], indices.size(), indices.data(), &values(i,0),
          elide_zero_values);
 }
 
@@ -1717,7 +1739,7 @@ SparseMatrix<number>::set (const std::vector<size_type> &row_indices,
           ExcDimensionMismatch(col_indices.size(), values.n()));
 
   for (size_type i=0; i<row_indices.size(); ++i)
-    set (row_indices[i], col_indices.size(), &col_indices[0], &values(i,0),
+    set (row_indices[i], col_indices.size(), col_indices.data(), &values(i,0),
          elide_zero_values);
 }
 
@@ -1735,7 +1757,7 @@ SparseMatrix<number>::set (const size_type               row,
   Assert (col_indices.size() == values.size(),
           ExcDimensionMismatch(col_indices.size(), values.size()));
 
-  set (row, col_indices.size(), &col_indices[0], &values[0],
+  set (row, col_indices.size(), col_indices.data(), values.data(),
        elide_zero_values);
 }
 
@@ -1783,7 +1805,7 @@ SparseMatrix<number>::add (const std::vector<size_type> &indices,
   Assert (values.m() == values.n(), ExcNotQuadratic());
 
   for (size_type i=0; i<indices.size(); ++i)
-    add (indices[i], indices.size(), &indices[0], &values(i,0),
+    add (indices[i], indices.size(), indices.data(), &values(i,0),
          elide_zero_values);
 }
 
@@ -1804,7 +1826,7 @@ SparseMatrix<number>::add (const std::vector<size_type> &row_indices,
           ExcDimensionMismatch(col_indices.size(), values.n()));
 
   for (size_type i=0; i<row_indices.size(); ++i)
-    add (row_indices[i], col_indices.size(), &col_indices[0], &values(i,0),
+    add (row_indices[i], col_indices.size(), col_indices.data(), &values(i,0),
          elide_zero_values);
 }
 
@@ -1822,7 +1844,7 @@ SparseMatrix<number>::add (const size_type               row,
   Assert (col_indices.size() == values.size(),
           ExcDimensionMismatch(col_indices.size(), values.size()));
 
-  add (row, col_indices.size(), &col_indices[0], &values[0],
+  add (row, col_indices.size(), col_indices.data(), values.data(),
        elide_zero_values);
 }
 
@@ -1833,11 +1855,11 @@ inline
 SparseMatrix<number> &
 SparseMatrix<number>::operator *= (const number factor)
 {
-  Assert (cols != 0, ExcNotInitialized());
-  Assert (val != 0, ExcNotInitialized());
+  Assert (cols != nullptr, ExcNotInitialized());
+  Assert (val != nullptr, ExcNotInitialized());
 
-  number             *val_ptr    = &val[0];
-  const number *const end_ptr    = &val[cols->n_nonzero_elements()];
+  number             *val_ptr    = val.get();
+  const number *const end_ptr    = val.get() + cols->n_nonzero_elements();
 
   while (val_ptr != end_ptr)
     *val_ptr++ *= factor;
@@ -1852,14 +1874,14 @@ inline
 SparseMatrix<number> &
 SparseMatrix<number>::operator /= (const number factor)
 {
-  Assert (cols != 0, ExcNotInitialized());
-  Assert (val != 0, ExcNotInitialized());
+  Assert (cols != nullptr, ExcNotInitialized());
+  Assert (val != nullptr, ExcNotInitialized());
   Assert (factor != number(), ExcDivideByZero());
 
   const number factor_inv = number(1.) / factor;
 
-  number             *val_ptr    = &val[0];
-  const number *const end_ptr    = &val[cols->n_nonzero_elements()];
+  number             *val_ptr    = val.get();
+  const number *const end_ptr    = val.get() + cols->n_nonzero_elements();
 
   while (val_ptr != end_ptr)
     *val_ptr++ *= factor_inv;
@@ -1871,10 +1893,23 @@ SparseMatrix<number>::operator /= (const number factor)
 
 template <typename number>
 inline
-number SparseMatrix<number>::operator () (const size_type i,
-                                          const size_type j) const
+const number &SparseMatrix<number>::operator () (const size_type i,
+                                                 const size_type j) const
 {
-  Assert (cols != 0, ExcNotInitialized());
+  Assert (cols != nullptr, ExcNotInitialized());
+  Assert (cols->operator()(i,j) != SparsityPattern::invalid_entry,
+          ExcInvalidIndex(i,j));
+  return val[cols->operator()(i,j)];
+}
+
+
+
+template <typename number>
+inline
+number &SparseMatrix<number>::operator () (const size_type i,
+                                           const size_type j)
+{
+  Assert (cols != nullptr, ExcNotInitialized());
   Assert (cols->operator()(i,j) != SparsityPattern::invalid_entry,
           ExcInvalidIndex(i,j));
   return val[cols->operator()(i,j)];
@@ -1887,7 +1922,7 @@ inline
 number SparseMatrix<number>::el (const size_type i,
                                  const size_type j) const
 {
-  Assert (cols != 0, ExcNotInitialized());
+  Assert (cols != nullptr, ExcNotInitialized());
   const size_type index = cols->operator()(i,j);
 
   if (index != SparsityPattern::invalid_entry)
@@ -1902,7 +1937,7 @@ template <typename number>
 inline
 number SparseMatrix<number>::diag_element (const size_type i) const
 {
-  Assert (cols != 0, ExcNotInitialized());
+  Assert (cols != nullptr, ExcNotInitialized());
   Assert (m() == n(),  ExcNotQuadratic());
   AssertIndexRange(i, m());
 
@@ -1917,7 +1952,7 @@ template <typename number>
 inline
 number &SparseMatrix<number>::diag_element (const size_type i)
 {
-  Assert (cols != 0, ExcNotInitialized());
+  Assert (cols != nullptr, ExcNotInitialized());
   Assert (m() == n(),  ExcNotQuadratic());
   AssertIndexRange(i, m());
 
@@ -2372,8 +2407,8 @@ void SparseMatrix<number>::print (StreamType &out,
                                   const bool  across,
                                   const bool  diagonal_first) const
 {
-  Assert (cols != 0, ExcNotInitialized());
-  Assert (val != 0, ExcNotInitialized());
+  Assert (cols != nullptr, ExcNotInitialized());
+  Assert (val != nullptr, ExcNotInitialized());
 
   bool hanging_diagonal = false;
   number diagonal = number();
