@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 1999 - 2016 by the deal.II authors
+// Copyright (C) 1999 - 2018 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -14,8 +14,8 @@
 // ---------------------------------------------------------------------
 
 
-#ifndef dealii__sparse_matrix_templates_h
-#define dealii__sparse_matrix_templates_h
+#ifndef dealii_sparse_matrix_templates_h
+#define dealii_sparse_matrix_templates_h
 
 
 #include <deal.II/base/config.h>
@@ -37,7 +37,7 @@
 #include <cmath>
 #include <vector>
 #include <numeric>
-#include <deal.II/base/std_cxx11/bind.h>
+#include <functional>
 
 
 
@@ -47,8 +47,8 @@ DEAL_II_NAMESPACE_OPEN
 template <typename number>
 SparseMatrix<number>::SparseMatrix ()
   :
-  cols(0, "SparseMatrix"),
-  val(0),
+  cols(nullptr, "SparseMatrix"),
+  val(nullptr),
   max_len(0)
 {}
 
@@ -58,11 +58,11 @@ template <typename number>
 SparseMatrix<number>::SparseMatrix (const SparseMatrix &m)
   :
   Subscriptor (m),
-  cols(0, "SparseMatrix"),
-  val(0),
+  cols(nullptr, "SparseMatrix"),
+  val(nullptr),
   max_len(0)
 {
-  Assert (m.cols==0 && m.val==0 && m.max_len==0,
+  Assert (m.cols==nullptr && m.val==nullptr && m.max_len==0,
           ExcMessage("This constructor can only be called if the provided argument "
                      "is an empty matrix. This constructor can not be used to "
                      "copy-construct a non-empty matrix. Use the "
@@ -71,20 +71,18 @@ SparseMatrix<number>::SparseMatrix (const SparseMatrix &m)
 
 
 
-#ifdef DEAL_II_WITH_CXX11
 template <typename number>
-SparseMatrix<number>::SparseMatrix (SparseMatrix<number> &&m)
-  :
-  Subscriptor(std::move(m)),
-  cols(m.cols),
-  val(m.val),
-  max_len(m.max_len)
+SparseMatrix<number>::SparseMatrix (SparseMatrix<number> &&m) noexcept
+:
+Subscriptor(std::move(m)),
+            cols(m.cols),
+            val(std::move(m.val)),
+            max_len(m.max_len)
 {
   m.cols = nullptr;
   m.val = nullptr;
   m.max_len = 0;
 }
-#endif
 
 
 
@@ -93,7 +91,7 @@ SparseMatrix<number> &
 SparseMatrix<number>::operator = (const SparseMatrix<number> &m)
 {
   (void)m;
-  Assert (m.cols==0 && m.val==0 && m.max_len==0,
+  Assert (m.cols==nullptr && m.val==nullptr && m.max_len==0,
           ExcMessage("This operator can only be called if the provided right "
                      "hand side is an empty matrix. This operator can not be "
                      "used to copy a non-empty matrix. Use the "
@@ -104,13 +102,12 @@ SparseMatrix<number>::operator = (const SparseMatrix<number> &m)
 
 
 
-#ifdef DEAL_II_WITH_CXX11
 template <typename number>
 SparseMatrix<number> &
-SparseMatrix<number>::operator = (SparseMatrix<number> &&m)
+SparseMatrix<number>::operator = (SparseMatrix<number> &&m) noexcept
 {
   cols = m.cols;
-  val = m.val;
+  val = std::move(m.val);
   max_len = m.max_len;
 
   m.cols = nullptr;
@@ -119,18 +116,20 @@ SparseMatrix<number>::operator = (SparseMatrix<number> &&m)
 
   return *this;
 }
-#endif
 
 
 
 template <typename number>
 SparseMatrix<number>::SparseMatrix (const SparsityPattern &c)
   :
-  cols(0, "SparseMatrix"),
-  val(0),
+  cols(nullptr, "SparseMatrix"),
+  val(nullptr),
   max_len(0)
 {
-  reinit (c);
+  // virtual functions called in constructors and destructors never use the
+  // override in a derived class
+  // for clarity be explicit on which function is called
+  SparseMatrix<number>::reinit (c);
 }
 
 
@@ -139,15 +138,18 @@ template <typename number>
 SparseMatrix<number>::SparseMatrix (const SparsityPattern &c,
                                     const IdentityMatrix  &id)
   :
-  cols(0, "SparseMatrix"),
-  val(0),
+  cols(nullptr, "SparseMatrix"),
+  val(nullptr),
   max_len(0)
 {
   (void)id;
   Assert (c.n_rows() == id.m(), ExcDimensionMismatch (c.n_rows(), id.m()));
   Assert (c.n_cols() == id.n(), ExcDimensionMismatch (c.n_cols(), id.n()));
 
-  reinit (c);
+  // virtual functions called in constructors and destructors never use the
+  // override in a derived class
+  // for clarity be explicit on which function is called
+  SparseMatrix<number>::reinit (c);
   for (size_type i=0; i<n(); ++i)
     this->set(i,i,1.);
 }
@@ -157,27 +159,34 @@ SparseMatrix<number>::SparseMatrix (const SparsityPattern &c,
 template <typename number>
 SparseMatrix<number>::~SparseMatrix ()
 {
-  cols = 0;
-
-  if (val != 0)
-    delete[] val;
+  cols = nullptr;
 }
 
 
 
 namespace internal
 {
-  namespace SparseMatrix
+  namespace SparseMatrixImplementation
   {
     typedef types::global_dof_index size_type;
 
-    template<typename T>
-    void zero_subrange (const size_type begin,
-                        const size_type end,
-                        T *dst)
+    template <typename T>
+    typename std::enable_if<std::is_trivial<T>::value>::type
+    zero_subrange (const size_type begin,
+                   const size_type end,
+                   T              *dst)
     {
       std::memset (dst+begin,0,(end-begin)*sizeof(T));
     }
+
+    template <typename T>
+    typename std::enable_if<!std::is_trivial<T>::value>::type
+    zero_subrange (const size_type begin,
+                   const size_type end,
+                   T              *dst)
+    {
+      std::fill (dst+begin,dst+end,0);
+    }
   }
 }
 
@@ -190,7 +199,7 @@ SparseMatrix<number>::operator = (const double d)
   (void)d;
   Assert (d==0, ExcScalarAssignmentOnlyForZeroValue());
 
-  Assert (cols != 0, ExcNotInitialized());
+  Assert (cols != nullptr, ExcNotInitialized());
   Assert (cols->compressed || cols->empty(), SparsityPattern::ExcNotCompressed());
 
   // do initial zeroing of elements in parallel. Try to achieve a similar
@@ -202,17 +211,26 @@ SparseMatrix<number>::operator = (const double d)
   // per row on average.
   const std::size_t matrix_size = cols->n_nonzero_elements();
   const size_type grain_size =
-    internal::SparseMatrix::minimum_parallel_grain_size *
+    internal::SparseMatrixImplementation::minimum_parallel_grain_size *
     (cols->n_nonzero_elements()+m()) / m();
   if (matrix_size>grain_size)
     parallel::apply_to_subranges (0U, matrix_size,
-                                  std_cxx11::bind(&internal::SparseMatrix::template
-                                                  zero_subrange<number>,
-                                                  std_cxx11::_1, std_cxx11::_2,
-                                                  val),
+                                  std::bind(&internal::SparseMatrixImplementation::template
+                                            zero_subrange<number>,
+                                            std::placeholders::_1, std::placeholders::_2,
+                                            val.get()),
                                   grain_size);
   else if (matrix_size > 0)
-    std::memset (&val[0], 0, matrix_size*sizeof(number));
+    {
+#ifdef DEAL_II_WITH_CXX17
+      if constexpr(std::is_trivial<number>::value)
+#else
+      if (std::is_trivial<number>::value)
+#endif
+        std::memset (val.get(), 0, matrix_size*sizeof(number));
+      else
+        std::fill (val.get(), val.get()+matrix_size, 0);
+    }
 
   return *this;
 }
@@ -246,9 +264,7 @@ SparseMatrix<number>::reinit (const SparsityPattern &sparsity)
 
   if (cols->empty())
     {
-      if (val != 0)
-        delete[] val;
-      val = 0;
+      val.reset ();
       max_len = 0;
       return;
     }
@@ -256,9 +272,7 @@ SparseMatrix<number>::reinit (const SparsityPattern &sparsity)
   const std::size_t N = cols->n_nonzero_elements();
   if (N > max_len || max_len == 0)
     {
-      if (val != 0)
-        delete[] val;
-      val = new number[N];
+      val = std_cxx14::make_unique<number[]>(N);
       max_len = N;
     }
 
@@ -271,9 +285,8 @@ template <typename number>
 void
 SparseMatrix<number>::clear ()
 {
-  cols = 0;
-  if (val) delete[] val;
-  val = 0;
+  cols = nullptr;
+  val.reset ();
   max_len = 0;
 }
 
@@ -283,7 +296,7 @@ template <typename number>
 bool
 SparseMatrix<number>::empty () const
 {
-  if (cols == 0)
+  if (cols == nullptr)
     return true;
   else
     return cols->empty();
@@ -295,7 +308,7 @@ template <typename number>
 typename SparseMatrix<number>::size_type
 SparseMatrix<number>::get_row_length (const size_type row) const
 {
-  Assert (cols != 0, ExcNotInitialized());
+  Assert (cols != nullptr, ExcNotInitialized());
   return cols->row_length(row);
 }
 
@@ -305,7 +318,7 @@ template <typename number>
 std::size_t
 SparseMatrix<number>::n_nonzero_elements () const
 {
-  Assert (cols != 0, ExcNotInitialized());
+  Assert (cols != nullptr, ExcNotInitialized());
   return cols->n_nonzero_elements ();
 }
 
@@ -315,7 +328,7 @@ template <typename number>
 std::size_t
 SparseMatrix<number>::n_actually_nonzero_elements (const double threshold) const
 {
-  Assert (cols != 0, ExcNotInitialized());
+  Assert (cols != nullptr, ExcNotInitialized());
   Assert (threshold >= 0, ExcMessage ("Negative threshold!"));
   size_type nnz = 0;
   const std::size_t nnz_alloc = n_nonzero_elements();
@@ -331,7 +344,7 @@ template <typename number>
 void
 SparseMatrix<number>::symmetrize ()
 {
-  Assert (cols != 0, ExcNotInitialized());
+  Assert (cols != nullptr, ExcNotInitialized());
   Assert (cols->rows == cols->cols, ExcNotQuadratic());
 
   const size_type n_rows = m();
@@ -371,12 +384,12 @@ template <typename somenumber>
 SparseMatrix<number> &
 SparseMatrix<number>::copy_from (const SparseMatrix<somenumber> &matrix)
 {
-  Assert (cols != 0, ExcNotInitialized());
-  Assert (val != 0, ExcNotInitialized());
+  Assert (cols != nullptr, ExcNotInitialized());
+  Assert (val != nullptr, ExcNotInitialized());
   Assert (cols == matrix.cols, ExcDifferentSparsityPatterns());
 
-  std::copy (&matrix.val[0], &matrix.val[cols->n_nonzero_elements()],
-             &val[0]);
+  std::copy (matrix.val.get(), matrix.val.get() + cols->n_nonzero_elements(),
+             val.get());
 
   return *this;
 }
@@ -453,13 +466,13 @@ void
 SparseMatrix<number>::add (const number factor,
                            const SparseMatrix<somenumber> &matrix)
 {
-  Assert (cols != 0, ExcNotInitialized());
-  Assert (val != 0, ExcNotInitialized());
+  Assert (cols != nullptr, ExcNotInitialized());
+  Assert (val != nullptr, ExcNotInitialized());
   Assert (cols == matrix.cols, ExcDifferentSparsityPatterns());
 
-  number             *val_ptr    = &val[0];
-  const somenumber   *matrix_ptr = &matrix.val[0];
-  const number *const end_ptr    = &val[cols->n_nonzero_elements()];
+  number             *val_ptr    = val.get();
+  const somenumber   *matrix_ptr = matrix.val.get();
+  const number *const end_ptr    = val.get() + cols->n_nonzero_elements();
 
   while (val_ptr != end_ptr)
     *val_ptr++ += factor * number(*matrix_ptr++);
@@ -469,7 +482,7 @@ SparseMatrix<number>::add (const number factor,
 
 namespace internal
 {
-  namespace SparseMatrix
+  namespace SparseMatrixImplementation
   {
     /**
      * Perform a vmult using the SparseMatrix data structures, but only using
@@ -528,7 +541,7 @@ SparseMatrix<number>::add (const size_type  row,
                            const bool       elide_zero_values,
                            const bool       col_indices_are_sorted)
 {
-  Assert (cols != 0, ExcNotInitialized());
+  Assert (cols != nullptr, ExcNotInitialized());
 
   // if we have sufficiently many columns
   // and sorted indices it is faster to
@@ -624,7 +637,7 @@ SparseMatrix<number>::add (const size_type  row,
   // unsorted case: first, search all the
   // indices to find out which values we
   // actually need to add.
-  const size_type *const my_cols = cols->colnums;
+  const size_type *const my_cols = cols->colnums.get();
   size_type index = cols->rowstart[row];
   const size_type next_row_index = cols->rowstart[row+1];
 
@@ -677,13 +690,13 @@ SparseMatrix<number>::set (const size_type  row,
                            const number2   *values,
                            const bool       elide_zero_values)
 {
-  Assert (cols != 0, ExcNotInitialized());
+  Assert (cols != nullptr, ExcNotInitialized());
   AssertIndexRange(row, m());
 
   // First, search all the indices to find
   // out which values we actually need to
   // set.
-  const size_type *my_cols = cols->colnums;
+  const size_type *my_cols = cols->colnums.get();
   std::size_t index = cols->rowstart[row], next_index = index;
   const std::size_t next_row_index = cols->rowstart[row+1];
 
@@ -757,24 +770,24 @@ void
 SparseMatrix<number>::vmult (OutVector &dst,
                              const InVector &src) const
 {
-  Assert (cols != 0, ExcNotInitialized());
-  Assert (val != 0, ExcNotInitialized());
+  Assert (cols != nullptr, ExcNotInitialized());
+  Assert (val != nullptr, ExcNotInitialized());
   Assert(m() == dst.size(), ExcDimensionMismatch(m(),dst.size()));
   Assert(n() == src.size(), ExcDimensionMismatch(n(),src.size()));
 
   Assert (!PointerComparison::equal(&src, &dst), ExcSourceEqualsDestination());
 
   parallel::apply_to_subranges (0U, m(),
-                                std_cxx11::bind (&internal::SparseMatrix::vmult_on_subrange
-                                                 <number,InVector,OutVector>,
-                                                 std_cxx11::_1, std_cxx11::_2,
-                                                 val,
-                                                 cols->rowstart,
-                                                 cols->colnums,
-                                                 std_cxx11::cref(src),
-                                                 std_cxx11::ref(dst),
-                                                 false),
-                                internal::SparseMatrix::minimum_parallel_grain_size);
+                                std::bind (&internal::SparseMatrixImplementation::vmult_on_subrange
+                                           <number,InVector,OutVector>,
+                                           std::placeholders::_1, std::placeholders::_2,
+                                           val.get(),
+                                           cols->rowstart.get(),
+                                           cols->colnums.get(),
+                                           std::cref(src),
+                                           std::ref(dst),
+                                           false),
+                                internal::SparseMatrixImplementation::minimum_parallel_grain_size);
 }
 
 
@@ -785,8 +798,8 @@ void
 SparseMatrix<number>::Tvmult (OutVector &dst,
                               const InVector &src) const
 {
-  Assert (val != 0, ExcNotInitialized());
-  Assert (cols != 0, ExcNotInitialized());
+  Assert (val != nullptr, ExcNotInitialized());
+  Assert (cols != nullptr, ExcNotInitialized());
   Assert(n() == dst.size(), ExcDimensionMismatch(n(),dst.size()));
   Assert(m() == src.size(), ExcDimensionMismatch(m(),src.size()));
 
@@ -812,24 +825,24 @@ void
 SparseMatrix<number>::vmult_add (OutVector &dst,
                                  const InVector &src) const
 {
-  Assert (cols != 0, ExcNotInitialized());
-  Assert (val != 0, ExcNotInitialized());
+  Assert (cols != nullptr, ExcNotInitialized());
+  Assert (val != nullptr, ExcNotInitialized());
   Assert(m() == dst.size(), ExcDimensionMismatch(m(),dst.size()));
   Assert(n() == src.size(), ExcDimensionMismatch(n(),src.size()));
 
   Assert (!PointerComparison::equal(&src, &dst), ExcSourceEqualsDestination());
 
   parallel::apply_to_subranges (0U, m(),
-                                std_cxx11::bind (&internal::SparseMatrix::vmult_on_subrange
-                                                 <number,InVector,OutVector>,
-                                                 std_cxx11::_1, std_cxx11::_2,
-                                                 val,
-                                                 cols->rowstart,
-                                                 cols->colnums,
-                                                 std_cxx11::cref(src),
-                                                 std_cxx11::ref(dst),
-                                                 true),
-                                internal::SparseMatrix::minimum_parallel_grain_size);
+                                std::bind (&internal::SparseMatrixImplementation::vmult_on_subrange
+                                           <number,InVector,OutVector>,
+                                           std::placeholders::_1, std::placeholders::_2,
+                                           val.get(),
+                                           cols->rowstart.get(),
+                                           cols->colnums.get(),
+                                           std::cref(src),
+                                           std::ref(dst),
+                                           true),
+                                internal::SparseMatrixImplementation::minimum_parallel_grain_size);
 }
 
 
@@ -840,8 +853,8 @@ void
 SparseMatrix<number>::Tvmult_add (OutVector &dst,
                                   const InVector &src) const
 {
-  Assert (val != 0, ExcNotInitialized());
-  Assert (cols != 0, ExcNotInitialized());
+  Assert (val != nullptr, ExcNotInitialized());
+  Assert (cols != nullptr, ExcNotInitialized());
   Assert(n() == dst.size(), ExcDimensionMismatch(n(),dst.size()));
   Assert(m() == src.size(), ExcDimensionMismatch(m(),src.size()));
 
@@ -858,7 +871,7 @@ SparseMatrix<number>::Tvmult_add (OutVector &dst,
 
 namespace internal
 {
-  namespace SparseMatrix
+  namespace SparseMatrixImplementation
   {
     /**
      * Perform a vmult using the SparseMatrix data structures, but only using
@@ -899,27 +912,29 @@ template <typename somenumber>
 somenumber
 SparseMatrix<number>::matrix_norm_square (const Vector<somenumber> &v) const
 {
-  Assert (cols != 0, ExcNotInitialized());
-  Assert (val != 0, ExcNotInitialized());
+  Assert (cols != nullptr, ExcNotInitialized());
+  Assert (val != nullptr, ExcNotInitialized());
   Assert(m() == v.size(), ExcDimensionMismatch(m(),v.size()));
   Assert(n() == v.size(), ExcDimensionMismatch(n(),v.size()));
 
   return
     parallel::accumulate_from_subranges<somenumber>
-    (std_cxx11::bind (&internal::SparseMatrix::matrix_norm_sqr_on_subrange
-                      <number,Vector<somenumber> >,
-                      std_cxx11::_1, std_cxx11::_2,
-                      val, cols->rowstart, cols->colnums,
-                      std_cxx11::cref(v)),
+    (std::bind (&internal::SparseMatrixImplementation::matrix_norm_sqr_on_subrange
+                <number,Vector<somenumber> >,
+                std::placeholders::_1, std::placeholders::_2,
+                val.get(),
+                cols->rowstart.get(),
+                cols->colnums.get(),
+                std::cref(v)),
      0, m(),
-     internal::SparseMatrix::minimum_parallel_grain_size);
+     internal::SparseMatrixImplementation::minimum_parallel_grain_size);
 }
 
 
 
 namespace internal
 {
-  namespace SparseMatrix
+  namespace SparseMatrixImplementation
   {
     /**
      * Perform a vmult using the SparseMatrix data structures, but only using
@@ -962,21 +977,23 @@ somenumber
 SparseMatrix<number>::matrix_scalar_product (const Vector<somenumber> &u,
                                              const Vector<somenumber> &v) const
 {
-  Assert (cols != 0, ExcNotInitialized());
-  Assert (val != 0, ExcNotInitialized());
+  Assert (cols != nullptr, ExcNotInitialized());
+  Assert (val != nullptr, ExcNotInitialized());
   Assert(m() == u.size(), ExcDimensionMismatch(m(),u.size()));
   Assert(n() == v.size(), ExcDimensionMismatch(n(),v.size()));
 
   return
     parallel::accumulate_from_subranges<somenumber>
-    (std_cxx11::bind (&internal::SparseMatrix::matrix_scalar_product_on_subrange
-                      <number,Vector<somenumber> >,
-                      std_cxx11::_1, std_cxx11::_2,
-                      val, cols->rowstart, cols->colnums,
-                      std_cxx11::cref(u),
-                      std_cxx11::cref(v)),
+    (std::bind (&internal::SparseMatrixImplementation::matrix_scalar_product_on_subrange
+                <number,Vector<somenumber> >,
+                std::placeholders::_1, std::placeholders::_2,
+                val.get(),
+                cols->rowstart.get(),
+                cols->colnums.get(),
+                std::cref(u),
+                std::cref(v)),
      0, m(),
-     internal::SparseMatrix::minimum_parallel_grain_size);
+     internal::SparseMatrixImplementation::minimum_parallel_grain_size);
 }
 
 
@@ -991,9 +1008,9 @@ SparseMatrix<number>::mmult (SparseMatrix<numberC>       &C,
 {
   const bool use_vector = V.size() == n() ? true : false;
   Assert (n() == B.m(), ExcDimensionMismatch(n(), B.m()));
-  Assert (cols != 0, ExcNotInitialized());
-  Assert (B.cols != 0, ExcNotInitialized());
-  Assert (C.cols != 0, ExcNotInitialized());
+  Assert (cols != nullptr, ExcNotInitialized());
+  Assert (B.cols != nullptr, ExcNotInitialized());
+  Assert (C.cols != nullptr, ExcNotInitialized());
 
   const SparsityPattern &sp_A = *cols;
   const SparsityPattern &sp_B = *B.cols;
@@ -1016,37 +1033,10 @@ SparseMatrix<number>::mmult (SparseMatrix<numberC>       &C,
       C.clear();
       sp_C.reinit (0,0,0);
 
-      // create a sparsity pattern for the matrix. we will go through all the
-      // rows in the matrix A, and for each column in a row we add the whole
-      // row of matrix B with that row number. This means that we will insert
-      // a lot of entries to each row, which is best handled by the
-      // DynamicSparsityPattern class.
+      // create a sparsity pattern for the matrix C.
       {
-        DynamicSparsityPattern dsp (m(), B.n());
-        for (size_type i = 0; i < dsp.n_rows(); ++i)
-          {
-            const size_type *rows = &sp_A.colnums[sp_A.rowstart[i]];
-            const size_type *const end_rows =
-              &sp_A.colnums[sp_A.rowstart[i+1]];
-            for (; rows != end_rows; ++rows)
-              {
-                const size_type col = *rows;
-                size_type *new_cols = const_cast<size_type *>
-                                      (&sp_B.colnums[sp_B.rowstart[col]]);
-                size_type *end_new_cols = const_cast<size_type *>
-                                          (&sp_B.colnums[sp_B.rowstart[col+1]]);
-
-                // if B has a diagonal, need to add that manually. this way,
-                // we maintain sortedness.
-                if (sp_B.n_rows() == sp_B.n_cols())
-                  {
-                    ++new_cols;
-                    dsp.add(i, col);
-                  }
-
-                dsp.add_entries (i, new_cols, end_new_cols, true);
-              }
-          }
+        DynamicSparsityPattern dsp ;
+        dsp.compute_mmult_pattern(sp_A,sp_B);
         sp_C.copy_from (dsp);
       }
 
@@ -1092,14 +1082,14 @@ SparseMatrix<number>::mmult (SparseMatrix<numberC>       &C,
           // now the innermost loop that goes over all the elements in row
           // 'col' of matrix B. Cache the elements, and then write them into C
           // at once
-          numberC *new_ptr = &new_entries[0];
+          numberC *new_ptr = new_entries.data();
           const numberB *B_val_ptr =
             &B.val[new_cols-&sp_B.colnums[sp_B.rowstart[0]]];
           const numberB *const end_cols = &B.val[sp_B.rowstart[col+1]];
           for (; B_val_ptr != end_cols; ++B_val_ptr)
             *new_ptr++ = numberC(A_val) * numberC(*B_val_ptr) * numberC(use_vector ? V(col) : 1);
 
-          C.add (i, new_ptr-&new_entries[0], new_cols, &new_entries[0],
+          C.add (i, new_ptr-new_entries.data(), new_cols, new_entries.data(),
                  false, true);
         }
     }
@@ -1118,9 +1108,9 @@ SparseMatrix<number>::Tmmult (SparseMatrix<numberC>       &C,
 {
   const bool use_vector = V.size() == m() ? true : false;
   Assert (m() == B.m(), ExcDimensionMismatch(m(), B.m()));
-  Assert (cols != 0, ExcNotInitialized());
-  Assert (B.cols != 0, ExcNotInitialized());
-  Assert (C.cols != 0, ExcNotInitialized());
+  Assert (cols != nullptr, ExcNotInitialized());
+  Assert (B.cols != nullptr, ExcNotInitialized());
+  Assert (C.cols != nullptr, ExcNotInitialized());
 
   const SparsityPattern &sp_A = *cols;
   const SparsityPattern &sp_B = *B.cols;
@@ -1223,13 +1213,13 @@ SparseMatrix<number>::Tmmult (SparseMatrix<numberC>       &C,
           // now the innermost loop that goes over all the elements in row
           // 'col' of matrix B. Cache the elements, and then write them into C
           // at once
-          numberC *new_ptr = &new_entries[0];
+          numberC *new_ptr = new_entries.data();
           const numberB *B_val_ptr =
             &B.val[new_cols-&sp_B.colnums[sp_B.rowstart[0]]];
           for (; B_val_ptr != end_cols; ++B_val_ptr)
             *new_ptr++ = numberC(A_val) * numberC(*B_val_ptr) * numberC(use_vector ? V(i) : 1);
 
-          C.add (row, new_ptr-&new_entries[0], new_cols, &new_entries[0],
+          C.add (row, new_ptr-new_entries.data(), new_cols, new_entries.data(),
                  false, true);
         }
     }
@@ -1241,8 +1231,8 @@ template <typename number>
 typename SparseMatrix<number>::real_type
 SparseMatrix<number>::l1_norm () const
 {
-  Assert (cols != 0, ExcNotInitialized());
-  Assert (val != 0, ExcNotInitialized());
+  Assert (cols != nullptr, ExcNotInitialized());
+  Assert (val != nullptr, ExcNotInitialized());
 
   Vector<real_type> column_sums(n());
   const size_type n_rows = m();
@@ -1259,8 +1249,8 @@ template <typename number>
 typename SparseMatrix<number>::real_type
 SparseMatrix<number>::linfty_norm () const
 {
-  Assert (cols != 0, ExcNotInitialized());
-  Assert (val != 0, ExcNotInitialized());
+  Assert (cols != nullptr, ExcNotInitialized());
+  Assert (val != nullptr, ExcNotInitialized());
 
   const number *val_ptr = &val[cols->rowstart[0]];
 
@@ -1289,8 +1279,8 @@ SparseMatrix<number>::frobenius_norm () const
   // reference to rows or columns
   real_type norm_sqr = 0;
   const size_type n_rows = m();
-  for (const number *ptr = &val[0];
-       ptr != &val[cols->rowstart[n_rows]]; ++ptr)
+  for (const number *ptr = val.get();
+       ptr != val.get() + cols->rowstart[n_rows]; ++ptr)
     norm_sqr +=  numbers::NumberTraits<number>::abs_square(*ptr);
 
   return std::sqrt (norm_sqr);
@@ -1300,7 +1290,7 @@ SparseMatrix<number>::frobenius_norm () const
 
 namespace internal
 {
-  namespace SparseMatrix
+  namespace SparseMatrixImplementation
   {
     /**
      * Perform a vmult using the SparseMatrix data structures, but only using
@@ -1346,8 +1336,8 @@ SparseMatrix<number>::residual (Vector<somenumber>       &dst,
                                 const Vector<somenumber> &u,
                                 const Vector<somenumber> &b) const
 {
-  Assert (cols != 0, ExcNotInitialized());
-  Assert (val != 0, ExcNotInitialized());
+  Assert (cols != nullptr, ExcNotInitialized());
+  Assert (val != nullptr, ExcNotInitialized());
   Assert(m() == dst.size(), ExcDimensionMismatch(m(),dst.size()));
   Assert(m() == b.size(), ExcDimensionMismatch(m(),b.size()));
   Assert(n() == u.size(), ExcDimensionMismatch(n(),u.size()));
@@ -1356,15 +1346,17 @@ SparseMatrix<number>::residual (Vector<somenumber>       &dst,
 
   return
     std::sqrt (parallel::accumulate_from_subranges<somenumber>
-               (std_cxx11::bind (&internal::SparseMatrix::residual_sqr_on_subrange
-                                 <number,Vector<somenumber>,Vector<somenumber> >,
-                                 std_cxx11::_1, std_cxx11::_2,
-                                 val, cols->rowstart, cols->colnums,
-                                 std_cxx11::cref(u),
-                                 std_cxx11::cref(b),
-                                 std_cxx11::ref(dst)),
+               (std::bind (&internal::SparseMatrixImplementation::residual_sqr_on_subrange
+                           <number,Vector<somenumber>,Vector<somenumber> >,
+                           std::placeholders::_1, std::placeholders::_2,
+                           val.get(),
+                           cols->rowstart.get(),
+                           cols->colnums.get(),
+                           std::cref(u),
+                           std::cref(b),
+                           std::ref(dst)),
                 0, m(),
-                internal::SparseMatrix::minimum_parallel_grain_size));
+                internal::SparseMatrixImplementation::minimum_parallel_grain_size));
 }
 
 
@@ -1406,8 +1398,8 @@ SparseMatrix<number>::precondition_Jacobi (Vector<somenumber>       &dst,
                                            const Vector<somenumber> &src,
                                            const number              om) const
 {
-  Assert (cols != 0, ExcNotInitialized());
-  Assert (val != 0, ExcNotInitialized());
+  Assert (cols != nullptr, ExcNotInitialized());
+  Assert (val != nullptr, ExcNotInitialized());
   AssertDimension (m(), n());
   AssertDimension (dst.size(), n());
   AssertDimension (src.size(), n());
@@ -1452,8 +1444,8 @@ SparseMatrix<number>::precondition_SSOR (Vector<somenumber>              &dst,
   // you may want to take a look at the CVS
   // archives to see the original version
   // which is much clearer...
-  Assert (cols != 0, ExcNotInitialized());
-  Assert (val != 0, ExcNotInitialized());
+  Assert (cols != nullptr, ExcNotInitialized());
+  Assert (val != nullptr, ExcNotInitialized());
   AssertDimension (m(), n());
   AssertDimension (dst.size(), n());
   AssertDimension (src.size(), n());
@@ -1576,8 +1568,8 @@ SparseMatrix<number>::precondition_SOR (Vector<somenumber> &dst,
                                         const Vector<somenumber> &src,
                                         const number om) const
 {
-  Assert (cols != 0, ExcNotInitialized());
-  Assert (val != 0, ExcNotInitialized());
+  Assert (cols != nullptr, ExcNotInitialized());
+  Assert (val != nullptr, ExcNotInitialized());
 
   dst = src;
   SOR(dst,om);
@@ -1591,8 +1583,8 @@ SparseMatrix<number>::precondition_TSOR (Vector<somenumber> &dst,
                                          const Vector<somenumber> &src,
                                          const number om) const
 {
-  Assert (cols != 0, ExcNotInitialized());
-  Assert (val != 0, ExcNotInitialized());
+  Assert (cols != nullptr, ExcNotInitialized());
+  Assert (val != nullptr, ExcNotInitialized());
 
   dst = src;
   TSOR(dst,om);
@@ -1605,8 +1597,8 @@ void
 SparseMatrix<number>::SOR (Vector<somenumber> &dst,
                            const number om) const
 {
-  Assert (cols != 0, ExcNotInitialized());
-  Assert (val != 0, ExcNotInitialized());
+  Assert (cols != nullptr, ExcNotInitialized());
+  Assert (val != nullptr, ExcNotInitialized());
   AssertDimension (m(), n());
   AssertDimension (dst.size(), n());
 
@@ -1633,8 +1625,8 @@ void
 SparseMatrix<number>::TSOR (Vector<somenumber> &dst,
                             const number om) const
 {
-  Assert (cols != 0, ExcNotInitialized());
-  Assert (val != 0, ExcNotInitialized());
+  Assert (cols != nullptr, ExcNotInitialized());
+  Assert (val != nullptr, ExcNotInitialized());
   AssertDimension (m(), n());
   AssertDimension (dst.size(), n());
 
@@ -1666,8 +1658,8 @@ SparseMatrix<number>::PSOR (Vector<somenumber> &dst,
                             const std::vector<size_type> &inverse_permutation,
                             const number om) const
 {
-  Assert (cols != 0, ExcNotInitialized());
-  Assert (val != 0, ExcNotInitialized());
+  Assert (cols != nullptr, ExcNotInitialized());
+  Assert (val != nullptr, ExcNotInitialized());
   AssertDimension (m(), n());
 
   Assert (m() == dst.size(), ExcDimensionMismatch(m(),dst.size()));
@@ -1705,8 +1697,8 @@ SparseMatrix<number>::TPSOR (Vector<somenumber> &dst,
                              const std::vector<size_type> &inverse_permutation,
                              const number om) const
 {
-  Assert (cols != 0, ExcNotInitialized());
-  Assert (val != 0, ExcNotInitialized());
+  Assert (cols != nullptr, ExcNotInitialized());
+  Assert (val != nullptr, ExcNotInitialized());
   AssertDimension (m(), n());
 
   Assert (m() == dst.size(), ExcDimensionMismatch(m(),dst.size()));
@@ -1742,8 +1734,8 @@ SparseMatrix<number>::Jacobi_step (Vector<somenumber> &v,
                                    const Vector<somenumber> &b,
                                    const number        om) const
 {
-  Assert (cols != 0, ExcNotInitialized());
-  Assert (val != 0, ExcNotInitialized());
+  Assert (cols != nullptr, ExcNotInitialized());
+  Assert (val != nullptr, ExcNotInitialized());
   AssertDimension (m(), n());
 
   Assert (m() == v.size(), ExcDimensionMismatch(m(),v.size()));
@@ -1773,8 +1765,8 @@ SparseMatrix<number>::SOR_step (Vector<somenumber> &v,
                                 const Vector<somenumber> &b,
                                 const number        om) const
 {
-  Assert (cols != 0, ExcNotInitialized());
-  Assert (val != 0, ExcNotInitialized());
+  Assert (cols != nullptr, ExcNotInitialized());
+  Assert (val != nullptr, ExcNotInitialized());
   AssertDimension (m(), n());
   Assert (m() == v.size(), ExcDimensionMismatch(m(),v.size()));
   Assert (m() == b.size(), ExcDimensionMismatch(m(),b.size()));
@@ -1801,8 +1793,8 @@ SparseMatrix<number>::TSOR_step (Vector<somenumber> &v,
                                  const Vector<somenumber> &b,
                                  const number        om) const
 {
-  Assert (cols != 0, ExcNotInitialized());
-  Assert (val != 0, ExcNotInitialized());
+  Assert (cols != nullptr, ExcNotInitialized());
+  Assert (val != nullptr, ExcNotInitialized());
   AssertDimension (m(), n());
   Assert (m() == v.size(), ExcDimensionMismatch(m(),v.size()));
   Assert (m() == b.size(), ExcDimensionMismatch(m(),b.size()));
@@ -1844,8 +1836,8 @@ SparseMatrix<number>::SSOR (Vector<somenumber> &dst,
 //TODO: Is this called anywhere? If so, multiplication with om(2-om)D is missing
   Assert(false, ExcNotImplemented());
 
-  Assert (cols != 0, ExcNotInitialized());
-  Assert (val != 0, ExcNotInitialized());
+  Assert (cols != nullptr, ExcNotInitialized());
+  Assert (val != nullptr, ExcNotInitialized());
   AssertDimension (m(), n());
   Assert (m() == dst.size(), ExcDimensionMismatch(m(),dst.size()));
 
@@ -1893,7 +1885,7 @@ template <typename number>
 const SparsityPattern &
 SparseMatrix<number>::get_sparsity_pattern () const
 {
-  Assert (cols != 0, ExcNotInitialized());
+  Assert (cols != nullptr, ExcNotInitialized());
   return *cols;
 }
 
@@ -1907,8 +1899,8 @@ void SparseMatrix<number>::print_formatted (std::ostream &out,
                                             const char *zero_string,
                                             const double denominator) const
 {
-  Assert (cols != 0, ExcNotInitialized());
-  Assert (val != 0, ExcNotInitialized());
+  Assert (cols != nullptr, ExcNotInitialized());
+  Assert (val != nullptr, ExcNotInitialized());
 
   unsigned int width = width_;
 
@@ -1951,8 +1943,8 @@ template <typename number>
 void SparseMatrix<number>::print_pattern (std::ostream &out,
                                           const double threshold) const
 {
-  Assert (cols != 0, ExcNotInitialized());
-  Assert (val != 0, ExcNotInitialized());
+  Assert (cols != nullptr, ExcNotInitialized());
+  Assert (val != nullptr, ExcNotInitialized());
 
   for (size_type i=0; i<m(); ++i)
     {
@@ -1980,9 +1972,9 @@ SparseMatrix<number>::block_write (std::ostream &out) const
   // bracketed in [...]
   out << '[' << max_len << "][";
   // then write out real data
-  out.write (reinterpret_cast<const char *>(&val[0]),
-             reinterpret_cast<const char *>(&val[max_len])
-             - reinterpret_cast<const char *>(&val[0]));
+  out.write (reinterpret_cast<const char *>(val.get()),
+             reinterpret_cast<const char *>(val.get() +  max_len)
+             - reinterpret_cast<const char *>(val.get()));
   out << ']';
 
   AssertThrow (out, ExcIO());
@@ -2009,13 +2001,12 @@ SparseMatrix<number>::block_read (std::istream &in)
   AssertThrow (c == '[', ExcIO());
 
   // reallocate space
-  delete[] val;
-  val  = new number[max_len];
+  val = std_cxx14::make_unique<number[]> (max_len);
 
   // then read data
-  in.read (reinterpret_cast<char *>(&val[0]),
-           reinterpret_cast<char *>(&val[max_len])
-           - reinterpret_cast<char *>(&val[0]));
+  in.read (reinterpret_cast<char *>(val.get()),
+           reinterpret_cast<char *>(val.get() + max_len)
+           - reinterpret_cast<char *>(val.get()));
   in >> c;
   AssertThrow (c == ']', ExcIO());
 }
