@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 2002 - 2016 by the deal.II authors
+// Copyright (C) 2002 - 2018 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -13,8 +13,8 @@
 //
 // ---------------------------------------------------------------------
 
-#ifndef dealii__sparse_matrix_ez_templates_h
-#define dealii__sparse_matrix_ez_templates_h
+#ifndef dealii_sparse_matrix_ez_templates_h
+#define dealii_sparse_matrix_ez_templates_h
 
 
 #include <deal.II/lac/sparse_matrix_ez.h>
@@ -63,10 +63,6 @@ SparseMatrixEZ<number>::SparseMatrixEZ(const size_type    n_rows,
 }
 
 
-template <typename number>
-SparseMatrixEZ<number>::~SparseMatrixEZ()
-{}
-
 
 template <typename number>
 SparseMatrixEZ<number> &
@@ -519,7 +515,7 @@ SparseMatrixEZ<number>::print_formatted ( std::ostream          &out,
         width = precision+2;
     }
 
-  // TODO: Skip nonexistant entries
+  // TODO: Skip nonexistent entries
   for (size_type i=0; i<m(); ++i)
     {
       for (size_type j=0; j<n(); ++j)
@@ -603,13 +599,13 @@ SparseMatrixEZ<number>::block_read (std::istream &in)
   DEAL_II_CHECK_INPUT(in,'[',c);
 
   // then read data
-  in.read(reinterpret_cast<char *>(&row_info[0]),
+  in.read(reinterpret_cast<char *>(row_info.data()),
           sizeof(RowInfo) * row_info.size());
 
   DEAL_II_CHECK_INPUT(in,']',c);
   DEAL_II_CHECK_INPUT(in,'[',c);
 
-  in.read(reinterpret_cast<char *>(&data[0]),
+  in.read(reinterpret_cast<char *>(data.data()),
           sizeof(Entry) * data.size());
 
   DEAL_II_CHECK_INPUT(in,']',c);
@@ -621,4 +617,4 @@ SparseMatrixEZ<number>::block_read (std::istream &in)
 
 DEAL_II_NAMESPACE_CLOSE
 
-#endif // dealii__sparse_matrix_ez_templates_h
+#endif // dealii_sparse_matrix_ez_templates_h
