@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 2002 - 2015 by the deal.II authors
+// Copyright (C) 2002 - 2018 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -13,8 +13,8 @@
 //
 // ---------------------------------------------------------------------
 
-#ifndef dealii__sparse_mic_templates_h
-#define dealii__sparse_mic_templates_h
+#ifndef dealii_sparse_mic_templates_h
+#define dealii_sparse_mic_templates_h
 
 
 #include <deal.II/base/memory_consumption.h>
@@ -36,7 +36,10 @@ SparseMIC<number>::SparseMIC ()
 template <typename number>
 SparseMIC<number>::~SparseMIC()
 {
-  clear();
+  // virtual functions called in constructors and destructors never use the
+  // override in a derived class
+  // for clarity be explicit on which function is called
+  SparseMIC<number>::clear();
 }
 
 
@@ -213,4 +216,4 @@ SparseMIC<number>::memory_consumption () const
 
 DEAL_II_NAMESPACE_CLOSE
 
-#endif // dealii__sparse_mic_templates_h
+#endif // dealii_sparse_mic_templates_h
