@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 2000 - 2016 by the deal.II authors
+// Copyright (C) 2000 - 2018 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -13,12 +13,13 @@
 //
 // ---------------------------------------------------------------------
 
-#ifndef dealii__sparsity_pattern_h
-#define dealii__sparsity_pattern_h
+#ifndef dealii_sparsity_pattern_h
+#define dealii_sparsity_pattern_h
 
 
 #include <deal.II/base/config.h>
 #include <deal.II/base/exceptions.h>
+#include <deal.II/base/std_cxx14/memory.h>
 #include <deal.II/base/subscriptor.h>
 
 // boost::serialization::make_array used to be in array.hpp, but was
@@ -38,6 +39,7 @@
 DEAL_II_NAMESPACE_OPEN
 
 class SparsityPattern;
+class DynamicSparsityPattern;
 class ChunkSparsityPattern;
 template <typename number> class FullMatrix;
 template <typename number> class SparseMatrix;
@@ -400,7 +402,7 @@ public:
    * amounts of computing time. However, copy constructors are needed if one
    * wants to place a SparsityPattern in a container, e.g., to write such
    * statements like <tt>v.push_back (SparsityPattern());</tt>, with
-   * <tt>v</tt> a vector of SparsityPattern objects.
+   * <tt>v</tt> a std::vector of SparsityPattern objects.
    *
    * Usually, it is sufficient to use the explicit keyword to disallow
    * unwanted temporaries, but this does not work for <tt>std::vector</tt>s.
@@ -478,13 +480,13 @@ public:
    * compressed after this function finishes.
    */
   SparsityPattern (const SparsityPattern  &original,
-                   const unsigned int        max_per_row,
-                   const size_type        extra_off_diagonals);
+                   const unsigned int      max_per_row,
+                   const size_type         extra_off_diagonals);
 
   /**
    * Destructor.
    */
-  ~SparsityPattern ();
+  ~SparsityPattern () = default;
 
   /**
    * Copy operator. For this the same holds as for the copy constructor: it is
@@ -494,11 +496,11 @@ public:
   SparsityPattern &operator = (const SparsityPattern &);
 
   /**
-   * Reallocate memory and set up data structures for a new matrix with <tt>m
-   * </tt>rows and <tt>n</tt> columns, with at most <tt>max_per_row</tt>
+   * Reallocate memory and set up data structures for a new matrix with @p m
+   * rows and @p n columns, with at most @p max_per_row
    * nonzero entries per row.
    *
-   * This function simply maps its operations to the other <tt>reinit</tt>
+   * This function simply maps its operations to the other reinit()
    * function.
    */
   void reinit (const size_type m,
@@ -507,9 +509,9 @@ public:
 
 
   /**
-   * Reallocate memory for a matrix of size <tt>m x n</tt>. The number of
-   * entries for each row is taken from the array <tt>row_lengths</tt> which
-   * has to give this number of each row <tt>i=1...m</tt>.
+   * Reallocate memory for a matrix of size @p m times @p n. The number of
+   * entries for each row is taken from the array @p row_lengths which
+   * has to give this number of each row $i=1\ldots m$.
    *
    * If <tt>m*n==0</tt> all memory is freed, resulting in a total
    * reinitialization of the object. If it is nonzero, new memory is only
@@ -630,15 +632,16 @@ public:
                   const ForwardIterator end);
 
   /**
-   * Copy data from an object of type DynamicSparsityPattern. Although not a
-   * compressed sparsity pattern, this function is also instantiated if the
-   * argument is of type SparsityPattern (i.e., the current class). Previous
-   * content of this object is lost, and the sparsity pattern is in compressed
-   * mode afterwards.
+   * Copy data from a DynamicSparsityPattern. Previous content of this object
+   * is lost, and the sparsity pattern is in compressed mode afterwards.
    */
-  template <typename SparsityPatternType>
-  void copy_from (const SparsityPatternType &dsp);
+  void copy_from (const DynamicSparsityPattern &dsp);
 
+  /**
+   * Copy data from a SparsityPattern. Previous content of this object is
+   * lost, and the sparsity pattern is in compressed mode afterwards.
+   */
+  void copy_from (const SparsityPattern &sp);
 
   /**
    * Take a full matrix and use its nonzero entries to generate a sparse
@@ -1062,7 +1065,7 @@ private:
    * region that is used. The actual number of elements that was allocated is
    * stored in #max_dim.
    */
-  std::size_t *rowstart;
+  std::unique_ptr<std::size_t[]> rowstart;
 
   /**
    * Array of column numbers. In this array, we store for each non-zero
@@ -1086,7 +1089,7 @@ private:
    * sorted, such that finding whether an element exists and determining its
    * position can be done by a binary search.
    */
-  size_type *colnums;
+  std::unique_ptr<size_type[]> colnums;
 
   /**
    * Store whether the compress() function was called for this object.
@@ -1162,10 +1165,10 @@ namespace SparsityPatternIterators
     Assert (is_valid_entry() == true, ExcInvalidIterator());
 
     const std::size_t *insert_point =
-      std::upper_bound(sparsity_pattern->rowstart,
-                       sparsity_pattern->rowstart + sparsity_pattern->rows + 1,
+      std::upper_bound(sparsity_pattern->rowstart.get(),
+                       sparsity_pattern->rowstart.get() + sparsity_pattern->rows + 1,
                        index_within_sparsity);
-    return insert_point - sparsity_pattern->rowstart - 1;
+    return insert_point - sparsity_pattern->rowstart.get() - 1;
   }
 
 
@@ -1406,9 +1409,13 @@ inline
 std::size_t
 SparsityPattern::n_nonzero_elements () const
 {
-  Assert ((rowstart!=0) && (colnums!=0), ExcEmptyObject());
   Assert (compressed, ExcNotCompressed());
-  return rowstart[rows]-rowstart[0];
+
+  if ((rowstart!=nullptr) && (colnums!=nullptr))
+    return rowstart[rows]-rowstart[0];
+  else
+    // the object is empty or has zero size
+    return 0;
 }
 
 
@@ -1423,8 +1430,8 @@ SparsityPattern::save (Archive &ar, const unsigned int) const
 
   ar &max_dim &rows &cols &max_vec_len &max_row_length &compressed &store_diagonal_first_in_row;
 
-  ar &boost::serialization::make_array(rowstart, max_dim + 1);
-  ar &boost::serialization::make_array(colnums, max_vec_len);
+  ar &boost::serialization::make_array(rowstart.get(), max_dim + 1);
+  ar &boost::serialization::make_array(colnums.get(), max_vec_len);
 }
 
 
@@ -1439,16 +1446,11 @@ SparsityPattern::load (Archive &ar, const unsigned int)
 
   ar &max_dim &rows &cols &max_vec_len &max_row_length &compressed &store_diagonal_first_in_row;
 
-  if (rowstart != 0)
-    delete[] rowstart;
-  rowstart = new std::size_t[max_dim + 1];
+  rowstart = std_cxx14::make_unique<std::size_t[]>(max_dim + 1);
+  colnums = std_cxx14::make_unique<size_type[]>(max_vec_len);
 
-  if (colnums != 0)
-    delete[] colnums;
-  colnums = new size_type[max_vec_len];
-
-  ar &boost::serialization::make_array(rowstart, max_dim + 1);
-  ar &boost::serialization::make_array(colnums, max_vec_len);
+  ar &boost::serialization::make_array(rowstart.get(), max_dim + 1);
+  ar &boost::serialization::make_array(colnums.get(), max_vec_len);
 }
 
 
