@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 2008 - 2017 by the deal.II authors
+// Copyright (C) 2008 - 2018 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -13,8 +13,8 @@
 //
 // ---------------------------------------------------------------------
 
-#ifndef dealii__trilinos_block_sparse_matrix_h
-#define dealii__trilinos_block_sparse_matrix_h
+#ifndef dealii_trilinos_block_sparse_matrix_h
+#define dealii_trilinos_block_sparse_matrix_h
 
 
 #include <deal.II/base/config.h>
@@ -25,7 +25,7 @@
 #  include <deal.II/base/template_constraints.h>
 #  include <deal.II/lac/block_matrix_base.h>
 #  include <deal.II/lac/trilinos_sparse_matrix.h>
-#  include <deal.II/lac/trilinos_block_vector.h>
+#  include <deal.II/lac/trilinos_parallel_block_vector.h>
 #  include <deal.II/lac/full_matrix.h>
 #  include <deal.II/lac/exceptions.h>
 
@@ -104,7 +104,7 @@ namespace TrilinosWrappers
      * reinit(BlockSparsityPattern). The number of blocks per row and column
      * are then determined by that function.
      */
-    BlockSparseMatrix ();
+    BlockSparseMatrix () = default;
 
     /**
      * Destructor.
@@ -116,7 +116,7 @@ namespace TrilinosWrappers
      * matrices need to be the same.
      */
     BlockSparseMatrix &
-    operator = (const BlockSparseMatrix &);
+    operator = (const BlockSparseMatrix &) = default;
 
     /**
      * This operator assigns a scalar to a matrix. Since this does usually not
@@ -183,9 +183,10 @@ namespace TrilinosWrappers
      *
      * @deprecated Use the respective method with IndexSet arguments instead.
      */
+    DEAL_II_DEPRECATED
     void reinit (const std::vector<Epetra_Map>             &input_maps,
                  const ::dealii::BlockSparseMatrix<double> &deal_ii_sparse_matrix,
-                 const double                               drop_tolerance=1e-13) DEAL_II_DEPRECATED;
+                 const double                               drop_tolerance=1e-13);
 
     /**
      * This function initializes the Trilinos matrix using the deal.II sparse
@@ -224,6 +225,11 @@ namespace TrilinosWrappers
      */
     size_type n_nonzero_elements () const;
 
+    /**
+     * Return the MPI communicator object in use with this matrix.
+     */
+    MPI_Comm get_mpi_communicator () const;
+
     /**
      * Return a vector of the underlying Trilinos Epetra_Map that sets the
      * partitioning of the domain space of this block matrix, i.e., the
@@ -233,7 +239,8 @@ namespace TrilinosWrappers
      * @deprecated Use the methods of the individual matrices based on
      * IndexSet arguments.
      */
-    std::vector<Epetra_Map> domain_partitioner () const DEAL_II_DEPRECATED;
+    DEAL_II_DEPRECATED
+    std::vector<Epetra_Map> domain_partitioner () const;
 
     /**
      * Return a vector of the underlying Trilinos Epetra_Map that sets the
@@ -244,8 +251,22 @@ namespace TrilinosWrappers
      * @deprecated Use the methods of the individual matrices based on
      * IndexSet arguments.
      */
-    std::vector<Epetra_Map> range_partitioner () const DEAL_II_DEPRECATED;
+    DEAL_II_DEPRECATED
+    std::vector<Epetra_Map> range_partitioner () const;
 
+    /**
+     * Return the partitioning of the domain space for the individual blocks of
+     * this matrix, i.e., the partitioning of the block vectors this matrix has
+     * to be multiplied with.
+     */
+    std::vector<IndexSet> locally_owned_domain_indices() const;
+
+    /**
+     * Return the partitioning of the range space for the individual blocks of
+     * this matrix, i.e., the partitioning of the block vectors that result
+     * from matrix-vector products.
+     */
+    std::vector<IndexSet> locally_owned_range_indices() const;
 
     /**
      * Matrix-vector multiplication: let $dst = M*src$ with $M$ being this
@@ -274,9 +295,9 @@ namespace TrilinosWrappers
      * Source <i>x</i> and destination <i>dst</i> must not be the same vector.
      *
      * Note that both vectors have to be distributed vectors generated using
-     * the same Map as was used for the matrix in case you work on a
-     * distributed memory architecture, using the interface in the
-     * TrilinosWrappers::MPI::BlockVector class.
+     * the same Map as was used for the matrix.
+     *
+     * This function only applicable if the matrix only has one block row.
      */
     TrilinosScalar residual (MPI::BlockVector       &dst,
                              const MPI::BlockVector &x,
@@ -287,25 +308,7 @@ namespace TrilinosWrappers
      * defined to be <i>r=b-Mx</i>. Write the residual into @p dst. The
      * <i>l<sub>2</sub></i> norm of the residual vector is returned.
      *
-     * Source <i>x</i> and destination <i>dst</i> must not be the same vector.
-     *
-     * Note that both vectors have to be distributed vectors generated using
-     * the same Map as was used for the matrix in case you work on a
-     * distributed memory architecture, using the interface in the
-     * TrilinosWrappers::BlockVector class. Since the block matrix is in
-     * general distributed among processes, this function only works when
-     * running the program on one processor.
-     */
-    TrilinosScalar residual (BlockVector       &dst,
-                             const BlockVector &x,
-                             const BlockVector &b) const;
-
-    /**
-     * Compute the residual of an equation <i>Mx=b</i>, where the residual is
-     * defined to be <i>r=b-Mx</i>. Write the residual into @p dst. The
-     * <i>l<sub>2</sub></i> norm of the residual vector is returned. Just like
-     * the previous function, but only applicable if the matrix only has one
-     * block row.
+     * This function is only applicable if the matrix only has one block row.
      */
     TrilinosScalar residual (MPI::BlockVector       &dst,
                              const MPI::Vector      &x,
@@ -314,20 +317,9 @@ namespace TrilinosWrappers
     /**
      * Compute the residual of an equation <i>Mx=b</i>, where the residual is
      * defined to be <i>r=b-Mx</i>. Write the residual into @p dst. The
-     * <i>l<sub>2</sub></i> norm of the residual vector is returned. Just like
-     * the previous function, but only applicable if the matrix only has one
-     * block row.
-     */
-    TrilinosScalar residual (BlockVector       &dst,
-                             const Vector      &x,
-                             const BlockVector &b) const;
-
-    /**
-     * Compute the residual of an equation <i>Mx=b</i>, where the residual is
-     * defined to be <i>r=b-Mx</i>. Write the residual into @p dst. The
-     * <i>l<sub>2</sub></i> norm of the residual vector is returned. Just like
-     * the previous function, but only applicable if the matrix only has one
-     * block column.
+     * <i>l<sub>2</sub></i> norm of the residual vector is returned.
+     *
+     * This function is only applicable if the matrix only has one block column.
      */
     TrilinosScalar residual (MPI::Vector            &dst,
                              const MPI::BlockVector &x,
@@ -336,24 +328,13 @@ namespace TrilinosWrappers
     /**
      * Compute the residual of an equation <i>Mx=b</i>, where the residual is
      * defined to be <i>r=b-Mx</i>. Write the residual into @p dst. The
-     * <i>l<sub>2</sub></i> norm of the residual vector is returned. Just like
-     * the previous function, but only applicable if the matrix only has one
-     * block column.
+     * <i>l<sub>2</sub></i> norm of the residual vector is returned.
+     *
+     * This function is only applicable if the matrix only has one block.
      */
-    TrilinosScalar residual (Vector            &dst,
-                             const BlockVector &x,
-                             const Vector      &b) const;
-
-    /**
-     * Compute the residual of an equation <i>Mx=b</i>, where the residual is
-     * defined to be <i>r=b-Mx</i>. Write the residual into @p dst. The
-     * <i>l<sub>2</sub></i> norm of the residual vector is returned. Just like
-     * the previous function, but only applicable if the matrix only has one
-     * block.
-     */
-    TrilinosScalar residual (VectorBase       &dst,
-                             const VectorBase &x,
-                             const VectorBase &b) const;
+    TrilinosScalar residual (MPI::Vector       &dst,
+                             const MPI::Vector &x,
+                             const MPI::Vector &b) const;
 
     /**
      * Make the clear() function in the base class visible, though it is
@@ -391,8 +372,8 @@ namespace TrilinosWrappers
     void vmult (VectorType1       &dst,
                 const VectorType2 &src,
                 const bool         transpose,
-                const dealii::internal::bool2type<true>,
-                const dealii::internal::bool2type<true>) const;
+                const std::integral_constant<bool, true>,
+                const std::integral_constant<bool, true>) const;
 
     /**
      * Internal version of (T)vmult where the source vector is a block vector
@@ -402,8 +383,8 @@ namespace TrilinosWrappers
     void vmult (VectorType1       &dst,
                 const VectorType2 &src,
                 const bool         transpose,
-                const dealii::internal::bool2type<false>,
-                const dealii::internal::bool2type<true>) const;
+                const std::integral_constant<bool, false>,
+                const std::integral_constant<bool, true>) const;
 
     /**
      * Internal version of (T)vmult where the source vector is a non-block
@@ -413,8 +394,8 @@ namespace TrilinosWrappers
     void vmult (VectorType1       &dst,
                 const VectorType2 &src,
                 const bool         transpose,
-                const dealii::internal::bool2type<true>,
-                const dealii::internal::bool2type<false>) const;
+                const std::integral_constant<bool, true>,
+                const std::integral_constant<bool, false>) const;
 
     /**
      * Internal version of (T)vmult where both source vector and the
@@ -425,8 +406,8 @@ namespace TrilinosWrappers
     void vmult (VectorType1       &dst,
                 const VectorType2 &src,
                 const bool         transpose,
-                const dealii::internal::bool2type<false>,
-                const dealii::internal::bool2type<false>) const;
+                const std::integral_constant<bool, false>,
+                const std::integral_constant<bool, false>) const;
   };
 
 
@@ -477,8 +458,8 @@ namespace TrilinosWrappers
                             const VectorType2 &src) const
   {
     vmult(dst, src, false,
-          dealii::internal::bool2type<IsBlockVector<VectorType1>::value>(),
-          dealii::internal::bool2type<IsBlockVector<VectorType2>::value>());
+          std::integral_constant<bool, IsBlockVector<VectorType1>::value>(),
+          std::integral_constant<bool, IsBlockVector<VectorType2>::value>());
   }
 
 
@@ -490,8 +471,8 @@ namespace TrilinosWrappers
                              const VectorType2 &src) const
   {
     vmult(dst, src, true,
-          dealii::internal::bool2type<IsBlockVector<VectorType1>::value>(),
-          dealii::internal::bool2type<IsBlockVector<VectorType2>::value>());
+          std::integral_constant<bool, IsBlockVector<VectorType1>::value>(),
+          std::integral_constant<bool, IsBlockVector<VectorType2>::value>());
   }
 
 
@@ -502,8 +483,8 @@ namespace TrilinosWrappers
   BlockSparseMatrix::vmult (VectorType1       &dst,
                             const VectorType2 &src,
                             const bool         transpose,
-                            dealii::internal::bool2type<true>,
-                            dealii::internal::bool2type<true>) const
+                            std::integral_constant<bool, true>,
+                            std::integral_constant<bool, true>) const
   {
     if (transpose == true)
       BaseClass::Tvmult_block_block (dst, src);
@@ -520,8 +501,8 @@ namespace TrilinosWrappers
   BlockSparseMatrix::vmult (VectorType1       &dst,
                             const VectorType2 &src,
                             const bool         transpose,
-                            dealii::internal::bool2type<false>,
-                            dealii::internal::bool2type<true>) const
+                            std::integral_constant<bool, false>,
+                            std::integral_constant<bool, true>) const
   {
     if (transpose == true)
       BaseClass::Tvmult_nonblock_block (dst, src);
@@ -537,8 +518,8 @@ namespace TrilinosWrappers
   BlockSparseMatrix::vmult (VectorType1       &dst,
                             const VectorType2 &src,
                             const bool         transpose,
-                            dealii::internal::bool2type<true>,
-                            dealii::internal::bool2type<false>) const
+                            std::integral_constant<bool, true>,
+                            std::integral_constant<bool, false>) const
   {
     if (transpose == true)
       BaseClass::Tvmult_block_nonblock (dst, src);
@@ -554,8 +535,8 @@ namespace TrilinosWrappers
   BlockSparseMatrix::vmult (VectorType1       &dst,
                             const VectorType2 &src,
                             const bool         transpose,
-                            dealii::internal::bool2type<false>,
-                            dealii::internal::bool2type<false>) const
+                            std::integral_constant<bool, false>,
+                            std::integral_constant<bool, false>) const
   {
     if (transpose == true)
       BaseClass::Tvmult_nonblock_nonblock (dst, src);
@@ -564,12 +545,42 @@ namespace TrilinosWrappers
   }
 
 
-#ifdef DEAL_II_WITH_CXX11
+
+  inline
+  std::vector<IndexSet>
+  BlockSparseMatrix::locally_owned_domain_indices () const
+  {
+    Assert (this->n_block_cols() != 0, ExcNotInitialized());
+    Assert (this->n_block_rows() != 0, ExcNotInitialized());
+
+    std::vector<IndexSet> domain_indices;
+    for (size_type c = 0; c < this->n_block_cols(); ++c)
+      domain_indices.push_back(this->sub_objects[0][c]->locally_owned_domain_indices());
+
+    return domain_indices;
+  }
+
+
+
+  inline
+  std::vector<IndexSet>
+  BlockSparseMatrix::locally_owned_range_indices () const
+  {
+    Assert (this->n_block_cols() != 0, ExcNotInitialized());
+    Assert (this->n_block_rows() != 0, ExcNotInitialized());
+
+    std::vector<IndexSet> range_indices;
+    for (size_type r = 0; r < this->n_block_rows(); ++r)
+      range_indices.push_back(this->sub_objects[r][0]->locally_owned_range_indices());
+
+    return range_indices;
+  }
+
 
 
   namespace internal
   {
-    namespace BlockLinearOperator
+    namespace BlockLinearOperatorImplementation
     {
 
       /**
@@ -580,14 +591,14 @@ namespace TrilinosWrappers
        * that the correct Payload type for each subblock has been chosen
        * correctly. Further extensions to the class may be necessary in the
        * future in order to add further functionality to BlockLinearOperators
-       * while retaining compatability with the Trilinos sparse matrix and
+       * while retaining compatibility with the Trilinos sparse matrix and
        * preconditioner classes.
        *
        * @author Jean-Paul Pelteret, 2016
        *
        * @ingroup TrilinosWrappers
        */
-      template<typename PayloadBlockType>
+      template <typename PayloadBlockType>
       class TrilinosBlockPayload
       {
       public:
@@ -608,7 +619,7 @@ namespace TrilinosWrappers
         template <typename... Args>
         TrilinosBlockPayload (const Args &...)
         {
-          static_assert(typeid(PayloadBlockType)==typeid(internal::LinearOperator::TrilinosPayload),
+          static_assert(typeid(PayloadBlockType)==typeid(internal::LinearOperatorImplementation::TrilinosPayload),
                         "TrilinosBlockPayload can only accept a payload of type TrilinosPayload.");
         }
       };
@@ -616,7 +627,6 @@ namespace TrilinosWrappers
     } /*namespace BlockLinearOperator*/
   } /* namespace internal */
 
-#endif // DEAL_II_WITH_CXX11
 
 }/* namespace TrilinosWrappers */
 
@@ -625,4 +635,4 @@ DEAL_II_NAMESPACE_CLOSE
 
 #endif    // DEAL_II_WITH_TRILINOS
 
-#endif    // dealii__trilinos_block_sparse_matrix_h
+#endif    // dealii_trilinos_block_sparse_matrix_h
