@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 2015 - 2016 by the deal.II authors
+// Copyright (C) 2015 - 2018 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -13,23 +13,24 @@
 //
 // ---------------------------------------------------------------------
 
-#ifndef dealii__trilinos_epetra_vector_h
-#define dealii__trilinos_epetra_vector_h
+#ifndef dealii_trilinos_epetra_vector_h
+#define dealii_trilinos_epetra_vector_h
 
 
 #include <deal.II/base/config.h>
 
-#ifdef DEAL_II_WITH_TRILINOS
-
-#ifdef DEAL_II_WITH_MPI
+#if defined(DEAL_II_WITH_TRILINOS) && defined(DEAL_II_WITH_MPI)
 
 #include <deal.II/base/index_set.h>
-#include <deal.II/base/std_cxx11/shared_ptr.h>
-#include <deal.II/lac/read_write_vector.h>
+#include <deal.II/base/subscriptor.h>
+#include <deal.II/lac/trilinos_epetra_communication_pattern.h>
+#include <deal.II/lac/vector_operation.h>
 #include <deal.II/lac/vector_space_vector.h>
+#include <deal.II/lac/vector_type_traits.h>
 #include <memory>
-#include "mpi.h"
-#include "Epetra_FEVector.h"
+
+#  include <mpi.h>
+#  include <Epetra_FEVector.h>
 
 DEAL_II_NAMESPACE_OPEN
 
@@ -51,7 +52,7 @@ namespace LinearAlgebra
      * @ingroup Vectors
      * @author Bruno Turcksin, 2015
      */
-    class Vector : public VectorSpaceVector<double>
+    class Vector : public VectorSpaceVector<double>, public Subscriptor
     {
     public:
       /**
@@ -69,7 +70,7 @@ namespace LinearAlgebra
        * This constructor takes an IndexSet that defines how to distribute the
        * individual components among the MPI processors. Since it also
        * includes information about the size of the vector, this is all we
-       * need to genereate a %parallel vector.
+       * need to generate a %parallel vector.
        */
       explicit Vector(const IndexSet &parallel_partitioner,
                       const MPI_Comm &communicator);
@@ -84,6 +85,13 @@ namespace LinearAlgebra
                    const MPI_Comm &communicator,
                    const bool      omit_zeroing_entries = false);
 
+      /**
+       * Change the dimension to that of the vector V. The elements of V are not
+       * copied.
+       */
+      virtual void reinit(const VectorSpaceVector<double> &V,
+                          const bool omit_zeroing_entries = false) override;
+
       /**
        * Copy function. This function takes a Vector and copies all the
        * elements. The Vector will have the same parallel distribution as @p
@@ -91,6 +99,12 @@ namespace LinearAlgebra
        */
       Vector &operator= (const Vector &V);
 
+      /**
+       * Sets all elements of the vector to the scalar @p s. This operation is
+       * only allowed if @p s is equal to zero.
+       */
+      virtual Vector &operator= (const double s) override;
+
       /**
        * Imports all the elements present in the vector's IndexSet from the input
        * vector @p V. VectorOperation::values @p operation is used to decide if
@@ -101,95 +115,100 @@ namespace LinearAlgebra
        */
       virtual void import(const ReadWriteVector<double>  &V,
                           VectorOperation::values         operation,
-                          std_cxx11::shared_ptr<const CommunicationPatternBase> communication_pattern =
-                            std_cxx11::shared_ptr<const CommunicationPatternBase> ());
+                          std::shared_ptr<const CommunicationPatternBase> communication_pattern =
+                            std::shared_ptr<const CommunicationPatternBase> ()) override;
 
       /**
        * Multiply the entire vector by a fixed factor.
        */
-      virtual Vector &operator*= (const double factor);
+      virtual Vector &operator*= (const double factor) override;
 
       /**
        * Divide the entire vector by a fixed factor.
        */
-      virtual Vector &operator/= (const double factor);
+      virtual Vector &operator/= (const double factor) override;
 
       /**
        * Add the vector @p V to the present one.
        */
-      virtual Vector &operator+= (const VectorSpaceVector<double> &V);
+      virtual Vector &operator+= (const VectorSpaceVector<double> &V) override;
 
       /**
-       * Substract the vector @p V from the present one.
+       * Subtract the vector @p V from the present one.
        */
-      virtual Vector &operator-= (const VectorSpaceVector<double> &V);
+      virtual Vector &operator-= (const VectorSpaceVector<double> &V) override;
 
       /**
        * Return the scalar product of two vectors. The vectors need to have the
        * same layout.
        */
-      virtual double operator* (const VectorSpaceVector<double> &V) const;
+      virtual double operator* (const VectorSpaceVector<double> &V) const override;
 
       /**
        * Add @p a to all components. Note that @p is a scalar not a vector.
        */
-      virtual void add(const double a);
+      virtual void add(const double a) override;
 
       /**
        * Simple addition of a multiple of a vector, i.e. <tt>*this +=
        * a*V</tt>. The vectors need to have the same layout.
        */
-      virtual void add(const double a, const VectorSpaceVector<double> &V);
+      virtual void add(const double a, const VectorSpaceVector<double> &V) override;
 
       /**
        * Multiple addition of multiple of a vector, i.e. <tt>*this> +=
        * a*V+b*W</tt>. The vectors need to have the same layout.
        */
       virtual void add(const double a, const VectorSpaceVector<double> &V,
-                       const double b, const VectorSpaceVector<double> &W);
+                       const double b, const VectorSpaceVector<double> &W) override;
 
       /**
        * Scaling and simple addition of a multiple of a vector, i.e. <tt>*this
        * = s*(*this)+a*V</tt>.
        */
       virtual void sadd(const double s, const double a,
-                        const VectorSpaceVector<double> &V);
+                        const VectorSpaceVector<double> &V) override;
 
       /**
        * Scale each element of this vector by the corresponding element in the
        * argument. This function is mostly meant to simulate multiplication
-       * (and immediate re-assignement) by a diagonal scaling matrix. The
+       * (and immediate re-assignment) by a diagonal scaling matrix. The
        * vectors need to have the same layout.
        */
-      virtual void scale(const VectorSpaceVector<double> &scaling_factors);
+      virtual void scale(const VectorSpaceVector<double> &scaling_factors) override;
 
       /**
-       * Assignement <tt>*this = a*V</tt>.
+       * Assignment <tt>*this = a*V</tt>.
        */
-      virtual void equ(const double a, const VectorSpaceVector<double> &V);
+      virtual void equ(const double a, const VectorSpaceVector<double> &V) override;
+
+      /**
+       * Return whether the vector contains only elements with value zero.
+       */
+      virtual bool all_zero() const override;
 
       /**
        * Return the mean value of the element of this vector.
        */
-      virtual double mean_value() const;
+      virtual double mean_value() const override;
 
       /**
        * Return the l<sub>1</sub> norm of the vector (i.e., the sum of the
        * absolute values of all entries among all processors).
        */
-      virtual double l1_norm() const;
+      virtual double l1_norm() const override;
 
       /**
        * Return the l<sub>2</sub> norm of the vector (i.e., the square root of
        * the sum of the square of all entries among all processors).
        */
-      virtual double l2_norm() const;
+      virtual double l2_norm() const override;
 
       /**
        * Return the maximum norm of the vector (i.e., the maximum absolute value
        * among all entries and among all processors).
        */
-      virtual double linfty_norm() const;
+      virtual double linfty_norm() const override;
 
       /**
        * Performs a combined operation of a vector addition and a subsequent
@@ -208,16 +227,24 @@ namespace LinearAlgebra
        * reduces the time by 25\% (or 50\% if @p W equals @p this).
        *
        * The vectors need to have the same layout.
+       *
+       * For complex-valued vectors, the scalar product in the second step is implemented as
+       * $\left<v,w\right>=\sum_i v_i \bar{w_i}$.
        */
       virtual double add_and_dot(const double a,
                                  const VectorSpaceVector<double> &V,
-                                 const VectorSpaceVector<double> &W);
+                                 const VectorSpaceVector<double> &W) override;
+      /**
+       * This function always returns false and is present only for backward
+       * compatibility.
+       */
+      bool has_ghost_elements() const;
 
       /**
        * Return the global size of the vector, equal to the sum of the number of
        * locally owned indices among all processors.
        */
-      virtual size_type size() const;
+      virtual size_type size() const override;
 
       /**
        * Return the MPI communicator object in use with this object.
@@ -227,14 +254,14 @@ namespace LinearAlgebra
       /**
        * Return an index set that describes which elements of this vector are
        * owned by the current processor. As a consequence, the index sets returned
-       * on different procesors if this is a distributed vector will form disjoint
+       * on different processors if this is a distributed vector will form disjoint
        * sets that add up to the complete index set. Obviously, if a vector is
        * created on only one processor, then the result would satisfy
        * @code
        *  vec.locally_owned_elements() == complete_index_set(vec.size())
        * @endcode
        */
-      virtual ::dealii::IndexSet locally_owned_elements() const;
+      virtual ::dealii::IndexSet locally_owned_elements() const override;
 
       /**
        * Return a const reference to the underlying Trilinos
@@ -254,12 +281,12 @@ namespace LinearAlgebra
       virtual void print(std::ostream &out,
                          const unsigned int precision=3,
                          const bool scientific=true,
-                         const bool across=true) const;
+                         const bool across=true) const override;
 
       /**
        * Return the memory consumption of this class in bytes.
        */
-      virtual std::size_t memory_consumption() const;
+      virtual std::size_t memory_consumption() const override;
 
       /**
        * The vectors have different partitioning, i.e. they have use different
@@ -296,7 +323,7 @@ namespace LinearAlgebra
       /**
        * Pointer to the actual Epetra vector object.
        */
-      std_cxx11::shared_ptr<Epetra_FEVector> vector;
+      std::unique_ptr<Epetra_FEVector> vector;
 
       /**
        * IndexSet of the elements of the last imported vector.
@@ -307,15 +334,29 @@ namespace LinearAlgebra
        * CommunicationPattern for the communication between the
        * source_stored_elements IndexSet and the current vector.
        */
-      std_cxx11::shared_ptr<const CommunicationPattern> epetra_comm_pattern;
+      std::shared_ptr<const CommunicationPattern> epetra_comm_pattern;
     };
+
+
+    inline
+    bool Vector::has_ghost_elements() const
+    {
+      return false;
+    }
   }
 }
 
+
+/**
+ * Declare dealii::LinearAlgebra::EpetraWrappers::Vector as distributed vector.
+ */
+template <>
+struct is_serial_vector<LinearAlgebra::EpetraWrappers::Vector> : std::false_type
+{
+};
+
 DEAL_II_NAMESPACE_CLOSE
 
 #endif
 
 #endif
-
-#endif
