@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 2008 - 2017 by the deal.II authors
+// Copyright (C) 2008 - 2018 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -13,8 +13,8 @@
 //
 // ---------------------------------------------------------------------
 
-#ifndef dealii__trilinos_precondition_h
-#define dealii__trilinos_precondition_h
+#ifndef dealii_trilinos_precondition_h
+#define dealii_trilinos_precondition_h
 
 
 #include <deal.II/base/config.h>
@@ -22,12 +22,12 @@
 #ifdef DEAL_II_WITH_TRILINOS
 
 #  include <deal.II/base/subscriptor.h>
-#  include <deal.II/base/std_cxx11/shared_ptr.h>
 
-#  include <deal.II/lac/trilinos_vector_base.h>
+#  include <deal.II/lac/trilinos_vector.h>
 #  include <deal.II/lac/la_parallel_vector.h>
 
-DEAL_II_DISABLE_EXTRA_DIAGNOSTICS
+#  include <memory>
+
 #  ifdef DEAL_II_WITH_MPI
 #    include <Epetra_MpiComm.h>
 #  else
@@ -38,7 +38,6 @@ DEAL_II_DISABLE_EXTRA_DIAGNOSTICS
 #  include <Teuchos_ParameterList.hpp>
 #  include <Epetra_RowMatrix.h>
 #  include <Epetra_Vector.h>
-DEAL_II_ENABLE_EXTRA_DIAGNOSTICS
 
 // forward declarations
 class Ifpack_Preconditioner;
@@ -105,7 +104,7 @@ namespace TrilinosWrappers
     /**
      * Destructor.
      */
-    ~PreconditionBase ();
+    ~PreconditionBase () = default;
 
     /**
      * Destroys the preconditioner, leaving an object like just after having
@@ -132,14 +131,14 @@ namespace TrilinosWrappers
     /**
      * Apply the preconditioner.
      */
-    virtual void vmult (VectorBase       &dst,
-                        const VectorBase &src) const;
+    virtual void vmult (MPI::Vector       &dst,
+                        const MPI::Vector &src) const;
 
     /**
      * Apply the transpose preconditioner.
      */
-    virtual void Tvmult (VectorBase       &dst,
-                         const VectorBase &src) const;
+    virtual void Tvmult (MPI::Vector       &dst,
+                         const MPI::Vector &src) const;
 
     /**
      * Apply the preconditioner on deal.II data structures instead of the ones
@@ -224,7 +223,7 @@ namespace TrilinosWrappers
      * This is a pointer to the preconditioner object that is used when
      * applying the preconditioner.
      */
-    std_cxx11::shared_ptr<Epetra_Operator> preconditioner;
+    std::shared_ptr<Epetra_Operator> preconditioner;
 
     /**
      * Internal communication pattern in case the matrix needs to be copied
@@ -240,7 +239,7 @@ namespace TrilinosWrappers
      * Internal Trilinos map in case the matrix needs to be copied from
      * deal.II format.
      */
-    std_cxx11::shared_ptr<Epetra_Map>   vector_distributor;
+    std::shared_ptr<Epetra_Map>   vector_distributor;
   };
 
 
@@ -1526,7 +1525,7 @@ namespace TrilinosWrappers
      *
      * This initialization routine is useful in cases where the operator to be
      * preconditioned is not a TrilinosWrappers::SparseMatrix object but still
-     * allows to get a copy of the entries in each of the locally owned matrix
+     * allows getting a copy of the entries in each of the locally owned matrix
      * rows (method ExtractMyRowCopy) and implements a matrix-vector product
      * (methods Multiply or Apply). An example are operators which provide
      * faster matrix-vector multiplications than possible with matrix entries
@@ -1573,7 +1572,7 @@ namespace TrilinosWrappers
     void initialize (const ::dealii::SparseMatrix<number> &deal_ii_sparse_matrix,
                      const AdditionalData                 &additional_data = AdditionalData(),
                      const double                          drop_tolerance = 1e-13,
-                     const ::dealii::SparsityPattern      *use_this_sparsity = 0);
+                     const ::dealii::SparsityPattern      *use_this_sparsity = nullptr);
 
     /**
      * This function can be used for a faster recalculation of the
@@ -1604,7 +1603,7 @@ namespace TrilinosWrappers
     /**
      * A copy of the deal.II matrix into Trilinos format.
      */
-    std_cxx11::shared_ptr<SparseMatrix> trilinos_matrix;
+    std::shared_ptr<SparseMatrix> trilinos_matrix;
   };
 
 
@@ -1822,7 +1821,7 @@ namespace TrilinosWrappers
     void initialize (const ::dealii::SparseMatrix<number> &deal_ii_sparse_matrix,
                      const AdditionalData                 &additional_data = AdditionalData(),
                      const double                          drop_tolerance = 1e-13,
-                     const ::dealii::SparsityPattern      *use_this_sparsity = 0);
+                     const ::dealii::SparsityPattern      *use_this_sparsity = nullptr);
 
     /**
      * Destroys the preconditioner, leaving an object like just after having
@@ -1839,7 +1838,7 @@ namespace TrilinosWrappers
     /**
      * A copy of the deal.II matrix into Trilinos format.
      */
-    std_cxx11::shared_ptr<SparseMatrix> trilinos_matrix;
+    std::shared_ptr<SparseMatrix> trilinos_matrix;
   };
 #endif
 
@@ -1862,12 +1861,7 @@ namespace TrilinosWrappers
      * preconditioner to be handed to a smoother.  This does nothing.
      */
     struct AdditionalData
-    {
-      /**
-       * Constructor.
-       */
-      AdditionalData () {}
-    };
+    {};
 
     /**
      * The matrix argument is ignored and here just for compatibility with more
@@ -1881,14 +1875,14 @@ namespace TrilinosWrappers
     /**
      * Apply the preconditioner, i.e., dst = src.
      */
-    void vmult (VectorBase       &dst,
-                const VectorBase &src) const;
+    void vmult (MPI::Vector       &dst,
+                const MPI::Vector &src) const;
 
     /**
      * Apply the transport conditioner, i.e., dst = src.
      */
-    void Tvmult (VectorBase       &dst,
-                 const VectorBase &src) const;
+    void Tvmult (MPI::Vector       &dst,
+                 const MPI::Vector &src) const;
 
     /**
      * Apply the preconditioner on deal.II data structures instead of the ones
@@ -1954,8 +1948,8 @@ namespace TrilinosWrappers
 
   inline
   void
-  PreconditionBase::vmult (VectorBase       &dst,
-                           const VectorBase &src) const
+  PreconditionBase::vmult (MPI::Vector       &dst,
+                           const MPI::Vector &src) const
   {
     Assert (dst.vector_partitioner().SameAs(preconditioner->OperatorRangeMap()),
             ExcNonMatchingMaps("dst"));
@@ -1969,8 +1963,8 @@ namespace TrilinosWrappers
 
   inline
   void
-  PreconditionBase::Tvmult (VectorBase       &dst,
-                            const VectorBase &src) const
+  PreconditionBase::Tvmult (MPI::Vector       &dst,
+                            const MPI::Vector &src) const
   {
     Assert (dst.vector_partitioner().SameAs(preconditioner->OperatorRangeMap()),
             ExcNonMatchingMaps("dst"));
