@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 2008 - 2017 by the deal.II authors
+// Copyright (C) 2008 - 2018 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -13,26 +13,25 @@
 //
 // ---------------------------------------------------------------------
 
-#ifndef dealii__trilinos_solver_h
-#define dealii__trilinos_solver_h
+#ifndef dealii_trilinos_solver_h
+#define dealii_trilinos_solver_h
 
 
 #include <deal.II/base/config.h>
 
 #ifdef DEAL_II_WITH_TRILINOS
 
-#  include <deal.II/base/std_cxx11/shared_ptr.h>
 #  include <deal.II/lac/exceptions.h>
 #  include <deal.II/lac/solver_control.h>
 #  include <deal.II/lac/vector.h>
 #  include <deal.II/lac/la_parallel_vector.h>
 
-DEAL_II_DISABLE_EXTRA_DIAGNOSTICS
+#  include <memory>
+
 #  include <Epetra_LinearProblem.h>
 #  include <AztecOO.h>
 #  include <Epetra_Operator.h>
 #  include <Amesos.h>
-DEAL_II_ENABLE_EXTRA_DIAGNOSTICS
 
 
 DEAL_II_NAMESPACE_OPEN
@@ -41,7 +40,6 @@ namespace TrilinosWrappers
 {
   // forward declarations
   class SparseMatrix;
-  class VectorBase;
   class PreconditionBase;
 
 
@@ -132,19 +130,21 @@ namespace TrilinosWrappers
     /**
      * Constructor. Takes the solver control object and creates the solver.
      */
-    SolverBase (SolverControl  &cn);
+    SolverBase (SolverControl        &cn,
+                const AdditionalData &data = AdditionalData());
 
     /**
      * Second constructor. This constructor takes an enum object that
      * specifies the solver name and sets the appropriate Krylov method.
      */
     SolverBase (const enum SolverName  solver_name,
-                SolverControl         &cn);
+                SolverControl         &cn,
+                const AdditionalData  &data = AdditionalData());
 
     /**
      * Destructor.
      */
-    virtual ~SolverBase ();
+    virtual ~SolverBase () = default;
 
     /**
      * Solve the linear system <tt>Ax=b</tt>. Depending on the information
@@ -153,8 +153,8 @@ namespace TrilinosWrappers
      */
     void
     solve (const SparseMatrix     &A,
-           VectorBase             &x,
-           const VectorBase       &b,
+           MPI::Vector            &x,
+           const MPI::Vector      &b,
            const PreconditionBase &preconditioner);
 
     /**
@@ -166,13 +166,13 @@ namespace TrilinosWrappers
      */
     void
     solve (const Epetra_Operator  &A,
-           VectorBase             &x,
-           const VectorBase       &b,
+           MPI::Vector            &x,
+           const MPI::Vector      &b,
            const PreconditionBase &preconditioner);
 
     /**
      * Solve the linear system <tt>Ax=b</tt> where both <tt>A</tt> and its
-     * @p precondtioner are an operator.
+     * @p preconditioner are an operator.
      * This function can be used when both <tt>A</tt> and the @p preconditioner
      * are LinearOperators derived from a TrilinosPayload.
      * Depending on the information provided by derived classes and the object
@@ -181,8 +181,8 @@ namespace TrilinosWrappers
      */
     void
     solve (const Epetra_Operator  &A,
-           VectorBase             &x,
-           const VectorBase       &b,
+           MPI::Vector            &x,
+           const MPI::Vector      &b,
            const Epetra_Operator  &preconditioner);
 
     /**
@@ -202,7 +202,7 @@ namespace TrilinosWrappers
 
     /**
      * Solve the linear system <tt>Ax=b</tt> where both <tt>A</tt> and its
-     * @p precondtioner are an operator, and the vectors @p x and @p b are
+     * @p preconditioner are an operator, and the vectors @p x and @p b are
      * native Trilinos vector types.
      * This function can be used when both <tt>A</tt> and the @p preconditioner
      * are LinearOperators derived from a TrilinosPayload.
@@ -306,13 +306,13 @@ namespace TrilinosWrappers
      * The solve function is used to set properly the Epetra_LinearProblem,
      * once it is done this function solves the linear problem.
      */
-    template<typename Preconditioner>
+    template <typename Preconditioner>
     void do_solve(const Preconditioner &preconditioner);
 
     /**
      * A function that sets the preconditioner that the solver will apply
      */
-    template<typename Preconditioner>
+    template <typename Preconditioner>
     void set_preconditioner (AztecOO              &solver,
                              const Preconditioner &preconditioner);
 
@@ -321,13 +321,13 @@ namespace TrilinosWrappers
      * side vector and the solution vector, which is passed down to the
      * Trilinos solver.
      */
-    std_cxx11::shared_ptr<Epetra_LinearProblem> linear_problem;
+    std::unique_ptr<Epetra_LinearProblem> linear_problem;
 
     /**
      * A structure that contains a Trilinos object that can query the linear
      * solver and determine whether the convergence criterion have been met.
      */
-    std_cxx11::shared_ptr<AztecOO_StatusTest> status_test;
+    std::unique_ptr<AztecOO_StatusTest> status_test;
 
     /**
      * A structure that contains the Trilinos solver and preconditioner
@@ -368,19 +368,13 @@ namespace TrilinosWrappers
      * Standardized data struct to pipe additional data to the solver.
      */
 
-    struct AdditionalData
+    struct AdditionalData : public SolverBase::AdditionalData
     {
       /**
        * Set the additional data field to the desired output format.
        */
       explicit
       AdditionalData (const bool output_solver_details = false);
-
-      /**
-       * Enables/disables the output of solver details (residual in each
-       * iterations etc.).
-       */
-      bool output_solver_details;
     };
 
     /**
@@ -414,19 +408,13 @@ namespace TrilinosWrappers
     /**
      * Standardized data struct to pipe additional data to the solver.
      */
-    struct AdditionalData
+    struct AdditionalData : public SolverBase::AdditionalData
     {
       /**
        * Set the additional data field to the desired output format.
        */
       explicit
       AdditionalData (const bool output_solver_details = false);
-
-      /**
-       * Enables/disables the output of solver details (residual in each
-       * iterations etc.).
-       */
-      bool output_solver_details;
     };
 
     /**
@@ -460,7 +448,7 @@ namespace TrilinosWrappers
     /**
      * Standardized data struct to pipe additional data to the solver.
      */
-    struct AdditionalData
+    struct AdditionalData : public SolverBase::AdditionalData
     {
       /**
        * Constructor. By default, set the number of temporary vectors to 30,
@@ -469,17 +457,6 @@ namespace TrilinosWrappers
       explicit
       AdditionalData (const bool         output_solver_details = false,
                       const unsigned int restart_parameter = 30);
-
-      /**
-       * Enables/disables the output of solver details (residual in each
-       * iterations etc.).
-       */
-      bool output_solver_details;
-
-      /**
-       * Maximum number of tmp vectors.
-       */
-      unsigned int restart_parameter;
     };
 
     /**
@@ -514,19 +491,13 @@ namespace TrilinosWrappers
     /**
      * Standardized data struct to pipe additional data to the solver.
      */
-    struct AdditionalData
+    struct AdditionalData : public SolverBase::AdditionalData
     {
       /**
        * Set the additional data field to the desired output format.
        */
       explicit
       AdditionalData (const bool output_solver_details = false);
-
-      /**
-       * Enables/disables the output of solver details (residual in each
-       * iterations etc.).
-       */
-      bool output_solver_details;
     };
 
     /**
@@ -561,19 +532,13 @@ namespace TrilinosWrappers
     /**
      * Standardized data struct to pipe additional data to the solver.
      */
-    struct AdditionalData
+    struct AdditionalData : public SolverBase::AdditionalData
     {
       /**
        * Set the additional data field to the desired output format.
        */
       explicit
       AdditionalData (const bool output_solver_details = false);
-
-      /**
-       * Enables/disables the output of solver details (residual in each
-       * iterations etc.).
-       */
-      bool output_solver_details;
     };
 
     /**
@@ -661,7 +626,7 @@ namespace TrilinosWrappers
     /**
      * Destructor.
      */
-    virtual ~SolverDirect ();
+    virtual ~SolverDirect () = default;
 
     /**
      * Initializes the direct solver for the matrix <tt>A</tt> and creates a
@@ -673,10 +638,19 @@ namespace TrilinosWrappers
 
     /**
      * Solve the linear system <tt>Ax=b</tt> based on the
-     * package set in intialize(). Note the matrix is not refactorized during
+     * package set in initialize(). Note the matrix is not refactorized during
      * this call.
      */
-    void solve (VectorBase &x, const VectorBase &b);
+    void solve (MPI::Vector &x, const MPI::Vector &b);
+
+    /**
+     * Solve the linear system <tt>Ax=b</tt> based on the package set in
+     * initialize() for deal.II's own parallel vectors. Note the matrix is not
+     * refactorized during this call.
+     */
+    void
+    solve (dealii::LinearAlgebra::distributed::Vector<double>       &x,
+           const dealii::LinearAlgebra::distributed::Vector<double> &b);
 
     /**
      * Solve the linear system <tt>Ax=b</tt>. Creates a factorization of the
@@ -686,8 +660,8 @@ namespace TrilinosWrappers
      */
     void
     solve (const SparseMatrix     &A,
-           VectorBase             &x,
-           const VectorBase       &b);
+           MPI::Vector            &x,
+           const MPI::Vector      &b);
 
     /**
      * Solve the linear system <tt>Ax=b</tt>. This class works with Trilinos
@@ -745,13 +719,13 @@ namespace TrilinosWrappers
      * side vector and the solution vector, which is passed down to the
      * Trilinos solver.
      */
-    std_cxx11::shared_ptr<Epetra_LinearProblem> linear_problem;
+    std::unique_ptr<Epetra_LinearProblem> linear_problem;
 
     /**
      * A structure that contains the Trilinos solver and preconditioner
      * objects.
      */
-    std_cxx11::shared_ptr<Amesos_BaseSolver> solver;
+    std::unique_ptr<Amesos_BaseSolver> solver;
 
     /**
      * Store a copy of the flags for this particular solver.
