@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 2008 - 2016 by the deal.II authors
+// Copyright (C) 2008 - 2018 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -13,8 +13,8 @@
 //
 // ---------------------------------------------------------------------
 
-#ifndef dealii__trilinos_sparsity_pattern_h
-#define dealii__trilinos_sparsity_pattern_h
+#ifndef dealii_trilinos_sparsity_pattern_h
+#define dealii_trilinos_sparsity_pattern_h
 
 
 #include <deal.II/base/config.h>
@@ -29,18 +29,15 @@
 #  include <cmath>
 #  include <memory>
 
-#  include <deal.II/base/std_cxx11/shared_ptr.h>
 
-DEAL_II_DISABLE_EXTRA_DIAGNOSTICS
 #  include <Epetra_FECrsGraph.h>
 #  include <Epetra_Map.h>
 #  ifdef DEAL_II_WITH_MPI
 #    include <Epetra_MpiComm.h>
-#    include "mpi.h"
+#    include <mpi.h>
 #  else
-#    include "Epetra_SerialComm.h"
+#    include <Epetra_SerialComm.h>
 #  endif
-DEAL_II_ENABLE_EXTRA_DIAGNOSTICS
 
 
 DEAL_II_NAMESPACE_OPEN
@@ -146,7 +143,7 @@ namespace TrilinosWrappers
        * performance, we keep a shared pointer to these entries so that more
        * than one accessor can access this data if necessary.
        */
-      std_cxx11::shared_ptr<const std::vector<size_type> > colnum_cache;
+      std::shared_ptr<const std::vector<size_type> > colnum_cache;
 
       /**
        * Discard the old row caches (they may still be used by other
@@ -317,6 +314,12 @@ namespace TrilinosWrappers
                      const size_type               n,
                      const std::vector<size_type> &n_entries_per_row);
 
+    /**
+     * Move constructor. Create a new sparse matrix by stealing the internal
+     * data.
+     */
+    SparsityPattern (SparsityPattern &&other) noexcept;
+
     /**
      * Copy constructor. Sets the calling sparsity pattern to be the same as
      * the input sparsity pattern.
@@ -326,7 +329,7 @@ namespace TrilinosWrappers
     /**
      * Destructor. Made virtual so that one can use pointers to this class.
      */
-    virtual ~SparsityPattern ();
+    virtual ~SparsityPattern () = default;
 
     /**
      * Initialize a sparsity pattern that is completely stored locally, having
@@ -369,7 +372,7 @@ namespace TrilinosWrappers
      * parallel, this function uses an ad-hoc partitioning of the rows and
      * columns.
      */
-    template<typename SparsityPatternType>
+    template <typename SparsityPatternType>
     void
     copy_from (const SparsityPatternType &nontrilinos_sparsity_pattern);
 
@@ -420,8 +423,9 @@ namespace TrilinosWrappers
      *
      * @deprecated Use the respective method with IndexSet argument instead.
      */
+    DEAL_II_DEPRECATED
     SparsityPattern (const Epetra_Map &parallel_partitioning,
-                     const size_type   n_entries_per_row = 0) DEAL_II_DEPRECATED;
+                     const size_type   n_entries_per_row = 0);
 
     /**
      * Same as before, but now use the exact number of nonzeros in each m row.
@@ -435,8 +439,9 @@ namespace TrilinosWrappers
      *
      * @deprecated Use the respective method with IndexSet argument instead.
      */
+    DEAL_II_DEPRECATED
     SparsityPattern (const Epetra_Map             &parallel_partitioning,
-                     const std::vector<size_type> &n_entries_per_row) DEAL_II_DEPRECATED;
+                     const std::vector<size_type> &n_entries_per_row);
 
     /**
      * This constructor is similar to the one above, but it now takes two
@@ -456,9 +461,10 @@ namespace TrilinosWrappers
      *
      * @deprecated Use the respective method with IndexSet argument instead.
      */
+    DEAL_II_DEPRECATED
     SparsityPattern (const Epetra_Map   &row_parallel_partitioning,
                      const Epetra_Map   &col_parallel_partitioning,
-                     const size_type     n_entries_per_row = 0) DEAL_II_DEPRECATED;
+                     const size_type     n_entries_per_row = 0);
 
     /**
      * This constructor is similar to the one above, but it now takes two
@@ -473,9 +479,10 @@ namespace TrilinosWrappers
      *
      * @deprecated Use the respective method with IndexSet argument instead.
      */
+    DEAL_II_DEPRECATED
     SparsityPattern (const Epetra_Map             &row_parallel_partitioning,
                      const Epetra_Map             &col_parallel_partitioning,
-                     const std::vector<size_type> &n_entries_per_row) DEAL_II_DEPRECATED;
+                     const std::vector<size_type> &n_entries_per_row);
 
     /**
      * Reinitialization function for generating a square sparsity pattern
@@ -493,9 +500,10 @@ namespace TrilinosWrappers
      *
      * @deprecated Use the respective method with IndexSet argument instead.
      */
+    DEAL_II_DEPRECATED
     void
     reinit (const Epetra_Map &parallel_partitioning,
-            const size_type   n_entries_per_row = 0) DEAL_II_DEPRECATED;
+            const size_type   n_entries_per_row = 0);
 
     /**
      * Same as before, but now use the exact number of nonzeros in each m row.
@@ -509,9 +517,10 @@ namespace TrilinosWrappers
      *
      * @deprecated Use the respective method with IndexSet argument instead.
      */
+    DEAL_II_DEPRECATED
     void
     reinit (const Epetra_Map             &parallel_partitioning,
-            const std::vector<size_type> &n_entries_per_row) DEAL_II_DEPRECATED;
+            const std::vector<size_type> &n_entries_per_row);
 
     /**
      * This reinit function is similar to the one above, but it now takes two
@@ -531,10 +540,11 @@ namespace TrilinosWrappers
      *
      * @deprecated Use the respective method with IndexSet argument instead.
      */
+    DEAL_II_DEPRECATED
     void
     reinit (const Epetra_Map   &row_parallel_partitioning,
             const Epetra_Map   &col_parallel_partitioning,
-            const size_type     n_entries_per_row = 0) DEAL_II_DEPRECATED;
+            const size_type     n_entries_per_row = 0);
 
     /**
      * This reinit function is similar to the one above, but it now takes two
@@ -549,10 +559,11 @@ namespace TrilinosWrappers
      *
      * @deprecated Use the respective method with IndexSet argument instead.
      */
+    DEAL_II_DEPRECATED
     void
     reinit (const Epetra_Map             &row_parallel_partitioning,
             const Epetra_Map             &col_parallel_partitioning,
-            const std::vector<size_type> &n_entries_per_row) DEAL_II_DEPRECATED;
+            const std::vector<size_type> &n_entries_per_row);
 
     /**
      * Reinit function. Takes one of the deal.II sparsity patterns and a
@@ -564,12 +575,13 @@ namespace TrilinosWrappers
      *
      * @deprecated Use the respective method with IndexSet argument instead.
      */
-    template<typename SparsityPatternType>
+    template <typename SparsityPatternType>
+    DEAL_II_DEPRECATED
     void
     reinit (const Epetra_Map          &row_parallel_partitioning,
             const Epetra_Map          &col_parallel_partitioning,
             const SparsityPatternType &nontrilinos_sparsity_pattern,
-            const bool                 exchange_data = false) DEAL_II_DEPRECATED;
+            const bool                 exchange_data = false);
 
     /**
      * Reinit function. Takes one of the deal.II sparsity patterns and a
@@ -581,11 +593,12 @@ namespace TrilinosWrappers
      *
      * @deprecated Use the respective method with IndexSet argument instead.
      */
-    template<typename SparsityPatternType>
+    template <typename SparsityPatternType>
+    DEAL_II_DEPRECATED
     void
     reinit (const Epetra_Map          &parallel_partitioning,
             const SparsityPatternType &nontrilinos_sparsity_pattern,
-            const bool                 exchange_data = false) DEAL_II_DEPRECATED;
+            const bool                 exchange_data = false);
 //@}
     /**
      * @name Constructors and initialization using an IndexSet description
@@ -796,7 +809,7 @@ namespace TrilinosWrappers
      * constructed. This feature is only implemented for input sparsity
      * patterns of type DynamicSparsityPattern.
      */
-    template<typename SparsityPatternType>
+    template <typename SparsityPatternType>
     void
     reinit (const IndexSet            &row_parallel_partitioning,
             const IndexSet            &col_parallel_partitioning,
@@ -812,7 +825,7 @@ namespace TrilinosWrappers
      * that is not fully constructed. This feature is only implemented for
      * input sparsity patterns of type DynamicSparsityPattern.
      */
-    template<typename SparsityPatternType>
+    template <typename SparsityPatternType>
     void
     reinit (const IndexSet            &parallel_partitioning,
             const SparsityPatternType &nontrilinos_sparsity_pattern,
@@ -831,7 +844,7 @@ namespace TrilinosWrappers
     bool is_compressed () const;
 
     /**
-     * Gives the maximum number of entries per row on the current processor.
+     * Return the maximum number of entries per row on the current processor.
      */
     unsigned int max_entries_per_row () const;
 
@@ -950,7 +963,8 @@ namespace TrilinosWrappers
      *
      * @deprecated Use locally_owned_domain_indices() instead.
      */
-    const Epetra_Map &domain_partitioner () const DEAL_II_DEPRECATED;
+    DEAL_II_DEPRECATED
+    const Epetra_Map &domain_partitioner () const;
 
     /**
      * Return a const reference to the underlying Trilinos Epetra_Map that
@@ -960,7 +974,8 @@ namespace TrilinosWrappers
      *
      * @deprecated Use locally_owned_range_indices() instead.
      */
-    const Epetra_Map &range_partitioner () const DEAL_II_DEPRECATED;
+    DEAL_II_DEPRECATED
+    const Epetra_Map &range_partitioner () const;
 
     /**
      * Return a const reference to the underlying Trilinos Epetra_Map that
@@ -969,7 +984,8 @@ namespace TrilinosWrappers
      *
      * @deprecated Use locally_owned_range_indices() instead.
      */
-    const Epetra_Map &row_partitioner () const DEAL_II_DEPRECATED;
+    DEAL_II_DEPRECATED
+    const Epetra_Map &row_partitioner () const;
 
     /**
      * Return a const reference to the underlying Trilinos Epetra_Map that
@@ -980,14 +996,16 @@ namespace TrilinosWrappers
      * @deprecated Usually not necessary. If desired, access via the
      * Epetra_FECrsGraph.
      */
-    const Epetra_Map &col_partitioner () const DEAL_II_DEPRECATED;
+    DEAL_II_DEPRECATED
+    const Epetra_Map &col_partitioner () const;
 
     /**
      * Return a const reference to the communicator used for this object.
      *
      * @deprecated Use get_mpi_communicator instead.
      */
-    const Epetra_Comm &trilinos_communicator () const DEAL_II_DEPRECATED;
+    DEAL_II_DEPRECATED
+    const Epetra_Comm &trilinos_communicator () const;
 
     /**
      * Return the MPI communicator object in use with this matrix.
@@ -1114,7 +1132,10 @@ namespace TrilinosWrappers
     /**
      * Exception
      */
-    DeclException0 (ExcSourceEqualsDestination);
+    DeclExceptionMsg (ExcSourceEqualsDestination,
+                      "You are attempting an operation on two sparsity patterns that "
+                      "are the same object, but the operation requires that the "
+                      "two objects are in fact different.");
 
     /**
      * Exception
@@ -1143,14 +1164,14 @@ namespace TrilinosWrappers
      * Pointer to the user-supplied Epetra Trilinos mapping of the matrix
      * columns that assigns parts of the matrix to the individual processes.
      */
-    std_cxx11::shared_ptr<Epetra_Map> column_space_map;
+    std::unique_ptr<Epetra_Map> column_space_map;
 
     /**
      * A sparsity pattern object in Trilinos to be used for finite element
      * based problems which allows for adding non-local elements to the
      * pattern.
      */
-    std_cxx11::shared_ptr<Epetra_FECrsGraph> graph;
+    std::unique_ptr<Epetra_FECrsGraph> graph;
 
     /**
      * A sparsity pattern object for the non-local part of the sparsity
@@ -1158,7 +1179,7 @@ namespace TrilinosWrappers
      * when the particular constructor or reinit method with writable_rows
      * argument is set
      */
-    std_cxx11::shared_ptr<Epetra_CrsGraph> nonlocal_graph;
+    std::unique_ptr<Epetra_CrsGraph> nonlocal_graph;
 
     friend class TrilinosWrappers::SparseMatrix;
     friend class SparsityPatternIterators::Accessor;
@@ -1229,10 +1250,7 @@ namespace TrilinosWrappers
 
 
     inline
-    Iterator::Iterator(const Iterator &i)
-      :
-      accessor(i.accessor)
-    {}
+    Iterator::Iterator(const Iterator &) = default;
 
 
 
@@ -1455,7 +1473,7 @@ namespace TrilinosWrappers
     int ierr;
     if ( graph->RowMap().LID(static_cast<TrilinosWrappers::types::int_type>(row)) != -1)
       ierr = graph->InsertGlobalIndices (row, n_cols, col_index_ptr);
-    else if (nonlocal_graph.get() != 0)
+    else if (nonlocal_graph.get() != nullptr)
       {
         // this is the case when we have explicitly set the off-processor rows
         // and want to create a separate matrix object for them (to retain
