@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 1999 - 2017 by the deal.II authors
+// Copyright (C) 1999 - 2018 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -13,8 +13,8 @@
 //
 // ---------------------------------------------------------------------
 
-#ifndef dealii__vector_h
-#define dealii__vector_h
+#ifndef dealii_vector_h
+#define dealii_vector_h
 
 
 #include <deal.II/base/config.h>
@@ -22,6 +22,7 @@
 #include <deal.II/base/exceptions.h>
 #include <deal.II/base/subscriptor.h>
 #include <deal.II/base/index_set.h>
+#include <deal.II/lac/vector_operation.h>
 #include <deal.II/lac/vector_type_traits.h>
 
 // boost::serialization::make_array used to be in array.hpp, but was
@@ -45,11 +46,7 @@ DEAL_II_NAMESPACE_OPEN
 #ifdef DEAL_II_WITH_PETSC
 namespace PETScWrappers
 {
-  class Vector;
-  namespace MPI
-  {
-    class Vector;
-  }
+  class VectorBase;
 }
 #endif
 
@@ -60,11 +57,10 @@ namespace TrilinosWrappers
   {
     class Vector;
   }
-  class Vector;
 }
 #endif
 
-template<typename number> class LAPACKFullMatrix;
+template <typename number> class LAPACKFullMatrix;
 
 template <typename> class BlockVector;
 
@@ -84,37 +80,6 @@ namespace parallel
  *@{
  */
 
-/**
- * This enum keeps track of the current operation in parallel linear algebra
- * objects like Vectors and Matrices.
- *
- * It is used in the various compress() functions. They also exist in serial
- * codes for compatibility and are empty there.
- *
- * See
- * @ref GlossCompress "Compressing distributed objects"
- * for more information.
- */
-struct VectorOperation
-{
-  enum values
-  {
-    /**
-     * The current operation is unknown.
-     */
-    unknown,
-    /**
-     * The current operation is an insertion.
-     */
-    insert,
-    /**
-     * The current operation is an addition.
-     */
-    add
-  };
-};
-
-
 /**
  * Numerical vector of data.  For this class there are different types of
  * functions available. The first type of function initializes the vector,
@@ -162,20 +127,6 @@ public:
    */
   typedef typename numbers::NumberTraits<Number>::real_type real_type;
 
-  /**
-   * A variable that indicates whether this vector supports distributed data
-   * storage. If true, then this vector also needs an appropriate compress()
-   * function that allows communicating recent set or add operations to
-   * individual elements to be communicated to other processors.
-   *
-   * For the current class, the variable equals false, since it does not
-   * support parallel data storage.
-   *
-   * @deprecated instead of using this variable, please use the type trait value
-   * <code>is_serial_vector< VectorType >::value</code>
-   */
-  static const bool supports_distributed_data DEAL_II_DEPRECATED = false;
-
 public:
 
   /**
@@ -196,19 +147,12 @@ public:
    */
   Vector (const Vector<Number> &v);
 
-#ifdef DEAL_II_WITH_CXX11
   /**
    * Move constructor. Creates a new vector by stealing the internal data of
    * the vector @p v.
-   *
-   * @note This constructor is only available if deal.II is configured with
-   * C++11 support.
    */
-  Vector (Vector<Number> &&v);
-#endif
+  Vector (Vector<Number> &&v) noexcept;
 
-
-#ifndef DEAL_II_EXPLICIT_CONSTRUCTOR_BUG
   /**
    * Copy constructor taking a vector of another data type. This will fail if
    * there is no conversion path from @p OtherNumber to @p Number. Note that
@@ -224,27 +168,20 @@ public:
   template <typename OtherNumber>
   explicit
   Vector (const Vector<OtherNumber> &v);
-#endif
 
 #ifdef DEAL_II_WITH_PETSC
   /**
-   * Another copy constructor: copy the values from a sequential PETSc wrapper
-   * vector class. This copy constructor is only available if PETSc was
-   * detected during configuration time.
-   */
-  explicit Vector (const PETScWrappers::Vector &v);
-
-  /**
-   * Another copy constructor: copy the values from a parallel PETSc wrapper
-   * vector class. This copy constructor is only available if PETSc was
-   * detected during configuration time.
+   * Another copy constructor: copy the values from a PETSc vector class. This
+   * copy constructor is only available if PETSc was detected during
+   * configuration time.
    *
    * Note that due to the communication model used in MPI, this operation can
-   * only succeed if all processes do it at the same time. I.e., it is not
-   * possible for only one process to obtain a copy of a parallel vector while
-   * the other jobs do something else.
+   * only succeed if all processes do it at the same time when <code>v</code>
+   * is a distributed vector: It is not possible for only one process to
+   * obtain a copy of a parallel vector while the other jobs do something
+   * else.
    */
-  explicit Vector (const PETScWrappers::MPI::Vector &v);
+  explicit Vector (const PETScWrappers::VectorBase &v);
 #endif
 
 #ifdef DEAL_II_WITH_TRILINOS
@@ -253,20 +190,16 @@ public:
    * This copy constructor is only available if Trilinos was detected during
    * configuration time.
    *
-   * Note that due to the communication model used in MPI, this operation can
-   * only succeed if all processes do it at the same time. This means that it
-   * is not possible for only one process to obtain a copy of a parallel
-   * vector while the other jobs do something else. This call will rather
-   * result in a copy of the vector on all processors.
+   * @note Due to the communication model used in MPI, this operation can
+   * only succeed if all processes that have knowledge of @p v
+   * (i.e. those given by <code>v.get_mpi_communicator()</code>) do it at
+   * the same time. This means that unless you use a split MPI communicator
+   * then it is not normally possible for only one or a subset of processes
+   * to obtain a copy of a parallel vector while the other jobs do something
+   * else. In other words, calling this function is a 'collective operation'
+   * that needs to be executed by all MPI processes that jointly share @p v.
    */
   explicit Vector (const TrilinosWrappers::MPI::Vector &v);
-
-  /**
-   * Another copy constructor: copy the values from a localized Trilinos
-   * wrapper vector. This copy constructor is only available if Trilinos was
-   * detected during configuration time.
-   */
-  explicit Vector (const TrilinosWrappers::Vector &v);
 #endif
 
   /**
@@ -292,17 +225,17 @@ public:
    * Destructor, deallocates memory. Made virtual to allow for derived classes
    * to behave properly.
    */
-  virtual ~Vector ();
+  virtual ~Vector () = default;
 
   /**
-   * This function does nothing but is there for compatibility with the @p
-   * PETScWrappers::Vector class.
+   * This function does nothing but exists for compatibility with the parallel
+   * vector classes.
    *
-   * For the PETSc vector wrapper class, this function compresses the
-   * underlying representation of the PETSc object, i.e. flushes the buffers
-   * of the vector object if it has any. This function is necessary after
-   * writing into a vector element-by-element and before anything else can be
-   * done on it.
+   * For the parallel vector wrapper class, this function compresses the
+   * underlying representation of the vector, i.e. flushes the buffers of the
+   * vector object if it has any. This function is necessary after writing
+   * into a vector element-by-element and before anything else can be done on
+   * it.
    *
    * However, for the implementation of this class, it is immaterial and thus
    * an empty function.
@@ -328,6 +261,31 @@ public:
   virtual void reinit (const size_type N,
                        const bool      omit_zeroing_entries=false);
 
+  /**
+   * Same as above, but will preserve the values of vector upon resizing.
+   * If we new size is bigger, we will have
+   * \f[
+   * \mathbf V \rightarrow
+   * \left(
+   * \begin{array}{c}
+   * \mathbf V   \\
+   * \mathbf 0
+   * \end{array}
+   * \right)
+   * \f]
+   * whereas if the desired size is smaller, then
+   * \f[
+   * \left(
+   * \begin{array}{c}
+   * \mathbf V_1   \\
+   * \mathbf V_2
+   * \end{array}
+   * \right)
+   * \rightarrow \mathbf V_1
+   * \f]
+   */
+  void grow_or_shrink (const size_type N);
+
   /**
    * Change the dimension to that of the vector @p V. The same applies as for
    * the other @p reinit function.
@@ -346,7 +304,7 @@ public:
    * swaps the pointers to the data of the two vectors and therefore does not
    * need to allocate temporary storage and move data around.
    *
-   * This function is analogous to the the @p swap function of all C++
+   * This function is analogous to the @p swap function of all C++
    * standard containers. Also, there is a global function <tt>swap(u,v)</tt>
    * that simply calls <tt>u.swap(v)</tt>, again in analogy to standard
    * functions.
@@ -378,17 +336,12 @@ public:
    */
   Vector<Number> &operator= (const Vector<Number> &v);
 
-#ifdef DEAL_II_WITH_CXX11
   /**
    * Move the given vector. This operator replaces the present vector with
    * the internal data of the vector @p v and resets @p v to the state it would
    * have after being newly default-constructed.
-   *
-   * @note This operator is only available if deal.II is configured with C++11
-   * support.
    */
-  Vector<Number> &operator= (Vector<Number> &&v);
-#endif
+  Vector<Number> &operator= (Vector<Number> &&v) noexcept;
 
   /**
    * Copy the given vector. Resize the present vector if necessary.
@@ -405,25 +358,18 @@ public:
 
 #ifdef DEAL_II_WITH_PETSC
   /**
-   * Another copy operator: copy the values from a sequential PETSc wrapper
-   * vector class. This operator is only available if PETSc was detected
-   * during configuration time.
-   */
-  Vector<Number> &
-  operator= (const PETScWrappers::Vector &v);
-
-  /**
-   * Another copy operator: copy the values from a parallel PETSc wrapper
-   * vector class. This operator is only available if PETSc was detected
-   * during configuration time.
+   * Another copy operator: copy the values from a PETSc wrapper vector
+   * class. This operator is only available if PETSc was detected during
+   * configuration time.
    *
    * Note that due to the communication model used in MPI, this operation can
-   * only succeed if all processes do it at the same time. I.e., it is not
-   * possible for only one process to obtain a copy of a parallel vector while
-   * the other jobs do something else.
+   * only succeed if all processes do it at the same time when <code>v</code>
+   * is a distributed vector: It is not possible for only one process to
+   * obtain a copy of a parallel vector while the other jobs do something
+   * else.
    */
   Vector<Number> &
-  operator= (const PETScWrappers::MPI::Vector &v);
+  operator= (const PETScWrappers::VectorBase &v);
 #endif
 
 
@@ -434,21 +380,17 @@ public:
    * operator is only available if Trilinos was detected during configuration
    * time.
    *
-   * Note that due to the communication model used in MPI, this operation can
-   * only succeed if all processes do it at the same time. I.e., it is not
-   * possible for only one process to obtain a copy of a parallel vector while
-   * the other jobs do something else.
+   * @note Due to the communication model used in MPI, this operation can
+   * only succeed if all processes that have knowledge of @p v
+   * (i.e. those given by <code>v.get_mpi_communicator()</code>) do it at
+   * the same time. This means that unless you use a split MPI communicator
+   * then it is not normally possible for only one or a subset of processes
+   * to obtain a copy of a parallel vector while the other jobs do something
+   * else. In other words, calling this function is a 'collective operation'
+   * that needs to be executed by all MPI processes that jointly share @p v.
    */
   Vector<Number> &
   operator= (const TrilinosWrappers::MPI::Vector &v);
-
-  /**
-   * Another copy operator: copy the values from a sequential Trilinos wrapper
-   * vector class. This operator is only available if Trilinos was detected
-   * during configuration time.
-   */
-  Vector<Number> &
-  operator= (const TrilinosWrappers::Vector &v);
 #endif
 
   /**
@@ -492,7 +434,7 @@ public:
   Number operator * (const Vector<Number2> &V) const;
 
   /**
-   * Return square of the $l_2$-norm.
+   * Return the square of the $l_2$-norm.
    *
    * @dealiiOperationIsMultithreaded The algorithm uses pairwise summation
    * with the same order of summation in every run, which gives fully
@@ -559,13 +501,17 @@ public:
    * most vector operations are memory transfer limited, this reduces the time
    * by 25\% (or 50\% if @p W equals @p this).
    *
+   * For complex-valued vectors, the scalar product in the second step is implemented as
+   * $\left<v,w\right>=\sum_i v_i \bar{w_i}$.
+   *
    * @dealiiOperationIsMultithreaded The algorithm uses pairwise summation
    * with the same order of summation in every run, which gives fully
    * repeatable results from one run to another.
    */
-  Number add_and_dot (const Number          a,
-                      const Vector<Number> &V,
-                      const Vector<Number> &W);
+  Number
+  add_and_dot (const Number          a,
+               const Vector<Number> &V,
+               const Vector<Number> &W);
 
   //@}
 
@@ -623,18 +569,50 @@ public:
   Number &operator[] (const size_type i);
 
   /**
-   * A collective get operation: instead of getting individual elements of a
-   * vector, this function allows to get a whole set of elements at once. The
+   * Instead of getting individual elements of a vector via operator(),
+   * this function allows getting a whole set of elements at once. The
    * indices of the elements to be read are stated in the first argument, the
    * corresponding values are returned in the second.
+   *
+   * If the current vector is called @p v, then this function is the equivalent
+   * to the code
+   * @code
+   *   for (unsigned int i=0; i<indices.size(); ++i)
+   *     values[i] = v[indices[i]];
+   * @endcode
+   *
+   * @pre The sizes of the @p indices and @p values arrays must be identical.
    */
   template <typename OtherNumber>
   void extract_subvector_to (const std::vector<size_type> &indices,
                              std::vector<OtherNumber> &values) const;
 
   /**
-   * Just as the above, but with pointers. Useful in minimizing copying of
-   * data around.
+   * Instead of getting individual elements of a vector via operator(),
+   * this function allows getting a whole set of elements at once. In
+   * contrast to the previous function, this function obtains the
+   * indices of the elements by dereferencing all elements of the iterator
+   * range provided by the first two arguments, and puts the vector
+   * values into memory locations obtained by dereferencing a range
+   * of iterators starting at the location pointed to by the third
+   * argument.
+   *
+   * If the current vector is called @p v, then this function is the equivalent
+   * to the code
+   * @code
+   *   ForwardIterator indices_p = indices_begin;
+   *   OutputIterator  values_p  = values_begin;
+   *   while (indices_p != indices_end)
+   *   {
+   *     *values_p = v[*indices_p];
+   *     ++indices_p;
+   *     ++values_p;
+   *   }
+   * @endcode
+   *
+   * @pre It must be possible to write into as many memory locations
+   *   starting at @p values_begin as there are iterators between
+   *   @p indices_begin and @p indices_end.
    */
   template <typename ForwardIterator, typename OutputIterator>
   void extract_subvector_to (ForwardIterator       indices_begin,
@@ -696,16 +674,6 @@ public:
    */
   void add (const Number s);
 
-  /**
-   * Simple vector addition, equal to the <tt>operator +=</tt>.
-   *
-   * @deprecated Use the <tt>operator +=</tt> instead.
-   *
-   * @dealiiOperationIsMultithreaded
-   */
-  void add (const Vector<Number> &V) DEAL_II_DEPRECATED;
-
-
   /**
    * Multiple addition of scaled vectors, i.e. <tt>*this += a*V+b*W</tt>.
    *
@@ -738,35 +706,6 @@ public:
              const Number          a,
              const Vector<Number> &V);
 
-  /**
-   * Scaling and multiple addition.
-   *
-   * This function is deprecated.
-   *
-   * @dealiiOperationIsMultithreaded
-   */
-  void sadd (const Number          s,
-             const Number          a,
-             const Vector<Number> &V,
-             const Number          b,
-             const Vector<Number> &W) DEAL_II_DEPRECATED;
-
-  /**
-   * Scaling and multiple addition.  <tt>*this = s*(*this)+a*V + b*W +
-   * c*X</tt>.
-   *
-   * This function is deprecated.
-   *
-   * @dealiiOperationIsMultithreaded
-   */
-  void sadd (const Number          s,
-             const Number          a,
-             const Vector<Number> &V,
-             const Number          b,
-             const Vector<Number> &W,
-             const Number          c,
-             const Vector<Number> &X) DEAL_II_DEPRECATED;
-
   /**
    * Scale each element of the vector by a constant value.
    *
@@ -811,27 +750,6 @@ public:
   template <typename Number2>
   void equ (const Number a, const Vector<Number2> &u);
 
-  /**
-   * Assignment <tt>*this = a*u + b*v</tt>.
-   *
-   * This function is deprecated.
-   *
-   * @dealiiOperationIsMultithreaded
-   */
-  void equ (const Number a, const Vector<Number> &u,
-            const Number b, const Vector<Number> &v) DEAL_II_DEPRECATED;
-
-  /**
-   * Assignment <tt>*this = a*u + b*v + b*w</tt>.
-   *
-   * This function is deprecated.
-   *
-   * @dealiiOperationIsMultithreaded
-   */
-  void equ (const Number a, const Vector<Number> &u,
-            const Number b, const Vector<Number> &v,
-            const Number c, const Vector<Number> &w) DEAL_II_DEPRECATED;
-
   /**
    * Compute the elementwise ratio of the two given vectors, that is let
    * <tt>this[i] = a[i]/b[i]</tt>. This is useful for example if you want to
@@ -844,19 +762,13 @@ public:
    *
    * @dealiiOperationIsMultithreaded
    */
+  DEAL_II_DEPRECATED
   void ratio (const Vector<Number> &a,
-              const Vector<Number> &b) DEAL_II_DEPRECATED;
+              const Vector<Number> &b);
 
   /**
-   * This function does nothing but is there for compatibility with the @p
-   * PETScWrappers::Vector class.
-   *
-   * For the PETSc vector wrapper class, this function updates the ghost
-   * values of the PETSc vector. This is necessary after any modification
-   * before reading ghost values.
-   *
-   * However, for the implementation of this class, it is immaterial and thus
-   * an empty function.
+   * This function does nothing but exists for compatibility with the @p
+   * parallel vector classes (e.g., LinearAlgebra::distributed::Vector class).
    */
   void update_ghost_values () const;
   //@}
@@ -872,7 +784,8 @@ public:
    *
    * This function is deprecated.
    */
-  void print (const char *format = 0) const DEAL_II_DEPRECATED;
+  DEAL_II_DEPRECATED
+  void print (const char *format = nullptr) const;
 
   /**
    * Print to a stream. @p precision denotes the desired precision with which
@@ -893,9 +806,10 @@ public:
    *
    * This function is deprecated.
    */
+  DEAL_II_DEPRECATED
   void print (LogStream &out,
               const unsigned int width = 6,
-              const bool across = true) const DEAL_II_DEPRECATED;
+              const bool across = true) const;
 
   /**
    * Write the vector en bloc to a file. This is done in a binary mode, so the
@@ -969,7 +883,7 @@ public:
   /**
    * Return dimension of the vector.
    */
-  std::size_t size () const;
+  size_type size () const;
 
   /**
    * Return whether the vector contains only elements with value zero. This
@@ -1014,14 +928,18 @@ protected:
 
   /**
    * Pointer to the array of elements of this vector.
+   *
+   * Because we allocate these arrays via Utilities::System::posix_memalign,
+   * we need to use a custom deleter for this object that does not call
+   * <code>delete[]</code>, but instead calls @p free().
    */
-  Number *val;
+  std::unique_ptr<Number[], decltype(&free)> values;
 
   /**
    * For parallel loops with TBB, this member variable stores the affinity
    * information of loops.
    */
-  mutable std_cxx11::shared_ptr<parallel::internal::TBBPartitioner> thread_loop_partitioner;
+  mutable std::shared_ptr<parallel::internal::TBBPartitioner> thread_loop_partitioner;
 
   /**
    * Make all other vector types friends.
@@ -1041,15 +959,11 @@ protected:
 private:
 
   /**
-   * Allocate and align @p val along 64-byte boundaries. The size of the
-   * allocated memory is determined by @p max_vec_size .
+   * Allocate and align @p values along 64-byte boundaries. The size of the
+   * allocated memory is determined by @p max_vec_size . Copy first
+   * @p copy_n_el from the old values.
    */
-  void allocate();
-
-  /**
-   * Deallocate @p val.
-   */
-  void deallocate();
+  void allocate(const size_type copy_n_el = 0);
 };
 
 /*@}*/
@@ -1060,14 +974,6 @@ private:
 
 
 //------------------------ declarations for explicit specializations
-
-#ifdef DEAL_II_BOOST_BIND_COMPILER_BUG
-template <>
-Vector<std::complex<float> > &
-Vector<std::complex<float> >::operator= (const std::complex<float> s);
-#endif
-
-
 template <>
 Vector<int>::real_type
 Vector<int>::lp_norm (const real_type) const;
@@ -1081,9 +987,12 @@ Vector<Number>::Vector ()
   :
   vec_size(0),
   max_vec_size(0),
-  val(0)
+  values(nullptr, &free)
 {
-  reinit(0);
+  // virtual functions called in constructors and destructors never use the
+  // override in a derived class
+  // for clarity be explicit on which function is called
+  Vector<Number>::reinit(0);
 }
 
 
@@ -1094,7 +1003,7 @@ Vector<Number>::Vector (const InputIterator first, const InputIterator last)
   :
   vec_size (0),
   max_vec_size (0),
-  val (0)
+  values (nullptr, &free)
 {
   // allocate memory. do not initialize it, as we will copy over to it in a
   // second
@@ -1110,29 +1019,20 @@ Vector<Number>::Vector (const size_type n)
   :
   vec_size(0),
   max_vec_size(0),
-  val(0)
+  values(nullptr, &free)
 {
-  reinit (n, false);
+  // virtual functions called in constructors and destructors never use the
+  // override in a derived class
+  // for clarity be explicit on which function is called
+  Vector<Number>::reinit (n, false);
 }
 
 
 
 template <typename Number>
 inline
-Vector<Number>::~Vector ()
-{
-  if (val)
-    {
-      deallocate();
-      val=0;
-    }
-}
-
-
-
-template <typename Number>
-inline
-std::size_t Vector<Number>::size () const
+typename Vector<Number>::size_type
+Vector<Number>::size () const
 {
   return vec_size;
 }
@@ -1153,7 +1053,7 @@ inline
 typename Vector<Number>::iterator
 Vector<Number>::begin ()
 {
-  return &val[0];
+  return values.get();
 }
 
 
@@ -1163,7 +1063,7 @@ inline
 typename Vector<Number>::const_iterator
 Vector<Number>::begin () const
 {
-  return &val[0];
+  return values.get();
 }
 
 
@@ -1173,7 +1073,7 @@ inline
 typename Vector<Number>::iterator
 Vector<Number>::end ()
 {
-  return &val[vec_size];
+  return values.get() + vec_size;
 }
 
 
@@ -1183,7 +1083,7 @@ inline
 typename Vector<Number>::const_iterator
 Vector<Number>::end () const
 {
-  return &val[vec_size];
+  return values.get() + vec_size;
 }
 
 
@@ -1193,7 +1093,7 @@ inline
 Number Vector<Number>::operator() (const size_type i) const
 {
   Assert (i<vec_size, ExcIndexRange(i,0,vec_size));
-  return val[i];
+  return values[i];
 }
 
 
@@ -1203,7 +1103,7 @@ inline
 Number &Vector<Number>::operator() (const size_type i)
 {
   Assert (i<vec_size, ExcIndexRangeType<size_type>(i,0,vec_size));
-  return val[i];
+  return values[i];
 }
 
 
@@ -1278,7 +1178,7 @@ Vector<Number>::add (const std::vector<size_type> &indices,
 {
   Assert (indices.size() == values.size(),
           ExcDimensionMismatch(indices.size(), values.size()));
-  add (indices.size(), &indices[0], &values[0]);
+  add (indices.size(), indices.data(), values.data());
 }
 
 
@@ -1292,7 +1192,7 @@ Vector<Number>::add (const std::vector<size_type> &indices,
 {
   Assert (indices.size() == values.size(),
           ExcDimensionMismatch(indices.size(), values.size()));
-  add (indices.size(), &indices[0], values.val);
+  add (indices.size(), indices.data(), values.values.get());
 }
 
 
@@ -1311,7 +1211,7 @@ Vector<Number>::add (const size_type  n_indices,
       Assert (numbers::is_finite(values[i]),
               ExcMessage("The given value is not finite but either infinite or Not A Number (NaN)"));
 
-      val[indices[i]] += values[i];
+      this->values[indices[i]] += values[i];
     }
 }
 
@@ -1354,7 +1254,7 @@ Vector<Number>::swap (Vector<Number> &v)
 {
   std::swap (vec_size,     v.vec_size);
   std::swap (max_vec_size, v.max_vec_size);
-  std::swap (val,          v.val);
+  std::swap (values,       v.values);
 }
 
 
@@ -1369,7 +1269,7 @@ Vector<Number>::save (Archive &ar, const unsigned int) const
   ar &static_cast<const Subscriptor &>(*this);
 
   ar &vec_size &max_vec_size ;
-  ar &boost::serialization::make_array(val, max_vec_size);
+  ar &boost::serialization::make_array(values.get(), max_vec_size);
 }
 
 
@@ -1381,14 +1281,14 @@ void
 Vector<Number>::load (Archive &ar, const unsigned int)
 {
   // get rid of previous content
-  deallocate();
+  values.reset();
 
   // the load stuff again from the archive
   ar &static_cast<Subscriptor &>(*this);
   ar &vec_size &max_vec_size ;
 
   allocate();
-  ar &boost::serialization::make_array(val, max_vec_size);
+  ar &boost::serialization::make_array(values.get(), max_vec_size);
 }
 
 #endif
@@ -1404,7 +1304,7 @@ Vector<Number>::load (Archive &ar, const unsigned int)
  * C++ standard library which uses a temporary object. The function simply
  * exchanges the data of the two vectors.
  *
- * @relates Vector
+ * @relatesalso Vector
  * @author Wolfgang Bangerth, 2000
  */
 template <typename Number>
@@ -1448,7 +1348,7 @@ operator << (LogStream &os, const Vector<number> &v)
  * @author Uwe Koecher, 2017
  */
 template <typename Number>
-struct is_serial_vector< Vector<Number> > : std_cxx11::true_type
+struct is_serial_vector< Vector<Number> > : std::true_type
 {
 };
 
