@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 1999 - 2017 by the deal.II authors
+// Copyright (C) 1999 - 2018 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -13,8 +13,8 @@
 //
 // ---------------------------------------------------------------------
 
-#ifndef dealii__vector_templates_h
-#define dealii__vector_templates_h
+#ifndef dealii_vector_templates_h
+#define dealii_vector_templates_h
 
 
 #include <deal.II/base/template_constraints.h>
@@ -25,8 +25,7 @@
 #include <deal.II/lac/vector_operations_internal.h>
 
 #ifdef DEAL_II_WITH_PETSC
-#  include <deal.II/lac/petsc_vector.h>
-#  include <deal.II/lac/petsc_parallel_vector.h>
+#  include <deal.II/lac/petsc_vector_base.h>
 #endif
 
 #ifdef DEAL_II_WITH_TRILINOS
@@ -34,11 +33,12 @@
 #endif
 
 
+#include <algorithm>
 #include <cmath>
 #include <cstring>
-#include <algorithm>
-#include <iostream>
 #include <iomanip>
+#include <iostream>
+#include <memory>
 
 DEAL_II_NAMESPACE_OPEN
 
@@ -48,7 +48,7 @@ Vector<Number>::Vector (const Vector<Number> &v)
   Subscriptor(),
   vec_size(v.size()),
   max_vec_size(v.size()),
-  val(0)
+  values(nullptr, &free)
 {
   if (vec_size != 0)
     {
@@ -59,25 +59,22 @@ Vector<Number>::Vector (const Vector<Number> &v)
 
 
 
-#ifdef DEAL_II_WITH_CXX11
 template <typename Number>
-Vector<Number>::Vector (Vector<Number> &&v)
-  :
-  Subscriptor(std::move(v)),
-  vec_size(v.vec_size),
-  max_vec_size(v.max_vec_size),
-  val(v.val),
-  thread_loop_partitioner(std::move(v.thread_loop_partitioner))
+Vector<Number>::Vector (Vector<Number> &&v) noexcept
+:
+Subscriptor(std::move(v)),
+            vec_size(v.vec_size),
+            max_vec_size(v.max_vec_size),
+            values(std::move(v.values)),
+            thread_loop_partitioner(std::move(v.thread_loop_partitioner))
 {
   v.vec_size = 0;
   v.max_vec_size = 0;
-  v.val = nullptr;
+  v.values = nullptr;
 }
-#endif
 
 
 
-#ifndef DEAL_II_EXPLICIT_CONSTRUCTOR_BUG
 template <typename Number>
 template <typename OtherNumber>
 Vector<Number>::Vector (const Vector<OtherNumber> &v)
@@ -85,7 +82,7 @@ Vector<Number>::Vector (const Vector<OtherNumber> &v)
   Subscriptor(),
   vec_size(v.size()),
   max_vec_size(v.size()),
-  val(0)
+  values(nullptr, &free)
 {
   if (vec_size != 0)
     {
@@ -93,58 +90,68 @@ Vector<Number>::Vector (const Vector<OtherNumber> &v)
       *this = v;
     }
 }
-#endif
+
 
 
 #ifdef DEAL_II_WITH_PETSC
-
-template <typename Number>
-Vector<Number>::Vector (const PETScWrappers::Vector &v)
-  :
-  Subscriptor(),
-  vec_size(v.size()),
-  max_vec_size(v.size()),
-  val(0)
+namespace internal
 {
-  if (vec_size != 0)
-    {
-      allocate();
+  template <typename Number>
+  void
+  copy_petsc_vector(const PETScWrappers::VectorBase &v,
+                    ::dealii::Vector<Number> &out)
+  {
+    // Create a sequential PETSc vector and then copy over the entries into
+    // the deal.II vector.
+    Vec sequential_vector;
+    VecScatter scatter_context;
 
-      // get a representation of the vector
-      // and copy it
-      PetscScalar *start_ptr;
-      PetscErrorCode ierr = VecGetArray (static_cast<const Vec &>(v), &start_ptr);
-      AssertThrow (ierr == 0, ExcPETScError(ierr));
+    PetscErrorCode ierr = VecScatterCreateToAll(v, &scatter_context, &sequential_vector);
+    AssertThrow (ierr == 0, ExcPETScError(ierr));
 
-      internal::VectorOperations::copy (start_ptr, start_ptr+vec_size, begin());
+    ierr = VecScatterBegin(scatter_context, v, sequential_vector, INSERT_VALUES,
+                           SCATTER_FORWARD);
+    AssertThrow (ierr == 0, ExcPETScError(ierr));
+    ierr = VecScatterEnd(scatter_context, v, sequential_vector, INSERT_VALUES,
+                         SCATTER_FORWARD);
+    AssertThrow (ierr == 0, ExcPETScError(ierr));
 
-      // restore the representation of the
-      // vector
-      ierr = VecRestoreArray (static_cast<const Vec &>(v), &start_ptr);
-      AssertThrow (ierr == 0, ExcPETScError(ierr));
-    }
+    PetscScalar *start_ptr;
+    ierr = VecGetArray(sequential_vector, &start_ptr);
+    AssertThrow (ierr == 0, ExcPETScError(ierr));
+
+    const PETScWrappers::VectorBase::size_type v_size = v.size();
+    if (out.size() != v_size)
+      out.reinit (v_size, true);
+
+    internal::VectorOperations::copy (start_ptr,
+                                      start_ptr + out.size(),
+                                      out.begin());
+    ierr = VecRestoreArray (sequential_vector, &start_ptr);
+    AssertThrow (ierr == 0, ExcPETScError(ierr));
+
+    ierr = VecScatterDestroy(&scatter_context);
+    AssertNothrow (ierr == 0, ExcPETScError(ierr));
+    ierr = VecDestroy (&sequential_vector);
+    AssertNothrow (ierr == 0, ExcPETScError(ierr));
+  }
 }
 
 
 
 template <typename Number>
-Vector<Number>::Vector (const PETScWrappers::MPI::Vector &v)
+Vector<Number>::Vector (const PETScWrappers::VectorBase &v)
   :
   Subscriptor(),
   vec_size(0),
   max_vec_size(0),
-  val(0)
+  values(nullptr, &free)
 {
   if (v.size() != 0)
     {
-      // do this in a two-stage process:
-      // first convert to a sequential petsc
-      // vector, then copy that
-      PETScWrappers::Vector seq (v);
-      *this = seq;
+      internal::copy_petsc_vector(v, *this);
     }
 }
-
 #endif
 
 
@@ -156,19 +163,25 @@ Vector<Number>::Vector (const TrilinosWrappers::MPI::Vector &v)
   Subscriptor(),
   vec_size(v.size()),
   max_vec_size(v.size()),
-  val(0)
+  values(nullptr, &free)
 {
   if (vec_size != 0)
     {
       allocate();
 
       // Copy the distributed vector to
-      // a local one at all
-      // processors. TODO: There could
+      // a local one at all processors
+      // that know about the original vector.
+      // TODO: There could
       // be a better solution than
       // this, but it has not yet been
       // found.
-      TrilinosWrappers::Vector localized_vector (v);
+      TrilinosWrappers::MPI::Vector localized_vector;
+      localized_vector.reinit(complete_index_set(vec_size), v.get_mpi_communicator());
+      localized_vector.reinit (v, false, true);
+
+      Assert(localized_vector.size() == vec_size,
+             ExcDimensionMismatch(localized_vector.size(), vec_size));
 
       // get a representation of the vector
       // and copy it
@@ -181,31 +194,6 @@ Vector<Number>::Vector (const TrilinosWrappers::MPI::Vector &v)
     }
 }
 
-
-
-template <typename Number>
-Vector<Number>::Vector (const TrilinosWrappers::Vector &v)
-  :
-  Subscriptor(),
-  vec_size(v.size()),
-  max_vec_size(v.size()),
-  val(0)
-{
-  if (vec_size != 0)
-    {
-      allocate();
-
-      // get a representation of the vector
-      // and copy it
-      TrilinosScalar **start_ptr;
-
-      int ierr = v.trilinos_vector().ExtractView (&start_ptr);
-      AssertThrow (ierr == 0, ExcTrilinosError(ierr));
-
-      std::copy (start_ptr[0], start_ptr[0]+vec_size, begin());
-    }
-}
-
 #endif
 
 
@@ -221,36 +209,34 @@ Vector<Number>::operator= (const Vector<Number> &v)
   if (vec_size != v.vec_size)
     reinit (v, true);
 
-  dealii::internal::VectorOperations::Vector_copy<Number,Number> copier(v.val, val);
-  internal::VectorOperations::parallel_for(copier,0,vec_size,thread_loop_partitioner);
+  if (vec_size>0)
+    {
+      dealii::internal::VectorOperations::Vector_copy<Number,Number> copier(v.values.get(), values.get());
+      internal::VectorOperations::parallel_for(copier,0,vec_size,thread_loop_partitioner);
+    }
 
   return *this;
 }
 
 
 
-#ifdef DEAL_II_WITH_CXX11
 template <typename Number>
 inline
 Vector<Number> &
-Vector<Number>::operator= (Vector<Number> &&v)
+Vector<Number>::operator= (Vector<Number> &&v) noexcept
 {
   Subscriptor::operator=(std::move(v));
 
-  if (val) deallocate();
-
   vec_size = v.vec_size;
   max_vec_size = v.max_vec_size;
-  val = v.val;
+  values = std::move(v.values);
   thread_loop_partitioner = std::move(v.thread_loop_partitioner);
 
   v.vec_size = 0;
   v.max_vec_size = 0;
-  v.val = nullptr;
 
   return *this;
 }
-#endif
 
 
 
@@ -264,7 +250,7 @@ Vector<Number>::operator= (const Vector<Number2> &v)
   if (vec_size != v.vec_size)
     reinit (v, true);
 
-  dealii::internal::VectorOperations::Vector_copy<Number,Number2> copier(v.val, val);
+  dealii::internal::VectorOperations::Vector_copy<Number,Number2> copier(v.values.get(), values.get());
   internal::VectorOperations::parallel_for(copier,0,vec_size,thread_loop_partitioner);
 
   return *this;
@@ -279,19 +265,17 @@ void Vector<Number>::reinit (const size_type n,
 {
   if (n==0)
     {
-      if (val) deallocate();
-      val = 0;
+      values.reset();
       max_vec_size = vec_size = 0;
-      thread_loop_partitioner.reset(new parallel::internal::TBBPartitioner());
+      thread_loop_partitioner = std::make_shared<parallel::internal::TBBPartitioner>();
       return;
-    };
+    }
 
   if (n>max_vec_size)
     {
-      if (val) deallocate();
       max_vec_size = n;
       allocate();
-    };
+    }
 
   if (vec_size != n)
     {
@@ -299,8 +283,8 @@ void Vector<Number>::reinit (const size_type n,
 
       // only reset the partitioner if we actually expect a significant vector
       // size
-      if (vec_size >= 4*internal::Vector::minimum_parallel_grain_size)
-        thread_loop_partitioner.reset(new parallel::internal::TBBPartitioner());
+      if (vec_size >= 4*internal::VectorImplementation::minimum_parallel_grain_size)
+        thread_loop_partitioner = std::make_shared<parallel::internal::TBBPartitioner>();
     }
 
   if (omit_zeroing_entries == false)
@@ -309,6 +293,43 @@ void Vector<Number>::reinit (const size_type n,
 
 
 
+template <typename Number>
+inline
+void Vector<Number>::grow_or_shrink (const size_type n)
+{
+  if (n==0)
+    {
+      values.reset();
+      max_vec_size = vec_size = 0;
+      thread_loop_partitioner = std::make_shared<parallel::internal::TBBPartitioner>();
+      return;
+    }
+
+  const size_type s = std::min(vec_size,n);
+  if (n>max_vec_size)
+    {
+      max_vec_size = n;
+      allocate(s);
+    }
+
+  if (vec_size != n)
+    {
+      vec_size = n;
+
+      // only reset the partitioner if we actually expect a significant vector
+      // size
+      if (vec_size >= 4*internal::VectorImplementation::minimum_parallel_grain_size)
+        thread_loop_partitioner = std::make_shared<parallel::internal::TBBPartitioner>();
+    }
+
+  // pad with zeroes
+  for (size_type i = s; i < vec_size; ++i)
+    values[i] = Number();
+
+}
+
+
+
 template <typename Number>
 template <typename Number2>
 void Vector<Number>::reinit (const Vector<Number2> &v,
@@ -318,18 +339,16 @@ void Vector<Number>::reinit (const Vector<Number2> &v,
 
   if (v.vec_size==0)
     {
-      if (val) deallocate();
-      val = 0;
+      values.reset();
       max_vec_size = vec_size = 0;
       return;
-    };
+    }
 
   if (v.vec_size>max_vec_size)
     {
-      if (val) deallocate();
       max_vec_size = v.vec_size;
       allocate();
-    };
+    }
   vec_size = v.vec_size;
   if (omit_zeroing_entries == false)
     *this = Number();
@@ -344,7 +363,7 @@ Vector<Number>::all_zero () const
   Assert (vec_size!=0, ExcEmptyObject());
 
   for (size_type i=0; i<vec_size; ++i)
-    if (val[i] != Number())
+    if (values[i] != Number())
       return false;
   return true;
 }
@@ -358,7 +377,7 @@ Vector<Number>::is_non_negative () const
   Assert (vec_size!=0, ExcEmptyObject());
 
   for (size_type i=0; i<vec_size; ++i)
-    if ( ! internal::VectorOperations::is_non_negative (val[i]))
+    if ( ! internal::VectorOperations::is_non_negative (values[i]))
       return false;
 
   return true;
@@ -374,9 +393,11 @@ Vector<Number>::operator= (const Number s)
   if (s != Number())
     Assert (vec_size!=0, ExcEmptyObject());
 
-  internal::VectorOperations::Vector_set<Number> setter(s, val);
-
-  internal::VectorOperations::parallel_for(setter,0,vec_size,thread_loop_partitioner);
+  if (vec_size>0)
+    {
+      internal::VectorOperations::Vector_set<Number> setter(s, values.get());
+      internal::VectorOperations::parallel_for(setter,0,vec_size,thread_loop_partitioner);
+    }
 
   return *this;
 }
@@ -390,7 +411,7 @@ Vector<Number> &Vector<Number>::operator *= (const Number factor)
 
   Assert (vec_size!=0, ExcEmptyObject());
 
-  internal::VectorOperations::Vectorization_multiply_factor<Number> vector_multiply(val, factor);
+  internal::VectorOperations::Vectorization_multiply_factor<Number> vector_multiply(values.get(), factor);
 
   internal::VectorOperations::parallel_for(vector_multiply,0,vec_size,thread_loop_partitioner);
 
@@ -409,7 +430,7 @@ Vector<Number>::add (const Number a,
   Assert (vec_size!=0, ExcEmptyObject());
   Assert (vec_size == v.vec_size, ExcDimensionMismatch(vec_size, v.vec_size));
 
-  internal::VectorOperations::Vectorization_add_av<Number> vector_add_av(val, v.val, a);
+  internal::VectorOperations::Vectorization_add_av<Number> vector_add_av(values.get(), v.values.get(), a);
   internal::VectorOperations::parallel_for(vector_add_av,0,vec_size,thread_loop_partitioner);
 }
 
@@ -427,7 +448,7 @@ Vector<Number>::sadd (const Number x,
   Assert (vec_size!=0, ExcEmptyObject());
   Assert (vec_size == v.vec_size, ExcDimensionMismatch(vec_size, v.vec_size));
 
-  internal::VectorOperations::Vectorization_sadd_xav<Number> vector_sadd_xav(val, v.val, a, x);
+  internal::VectorOperations::Vectorization_sadd_xav<Number> vector_sadd_xav(values.get(), v.values.get(), a, x);
   internal::VectorOperations::parallel_for(vector_sadd_xav,0,vec_size,thread_loop_partitioner);
 }
 
@@ -446,7 +467,7 @@ Number Vector<Number>::operator * (const Vector<Number2> &v) const
           ExcDimensionMismatch(vec_size, v.size()));
 
   Number sum;
-  internal::VectorOperations::Dot<Number,Number2> dot(val, v.val);
+  internal::VectorOperations::Dot<Number,Number2> dot(values.get(), v.values.get());
   internal::VectorOperations::parallel_reduce (dot, 0, vec_size, sum, thread_loop_partitioner);
   AssertIsFinite(sum);
 
@@ -462,7 +483,7 @@ Vector<Number>::norm_sqr () const
   Assert (vec_size!=0, ExcEmptyObject());
 
   real_type sum;
-  internal::VectorOperations::Norm2<Number,real_type> norm2(val);
+  internal::VectorOperations::Norm2<Number,real_type> norm2(values.get());
   internal::VectorOperations::parallel_reduce (norm2, 0, vec_size, sum, thread_loop_partitioner);
 
   AssertIsFinite(sum);
@@ -478,7 +499,7 @@ Number Vector<Number>::mean_value () const
   Assert (vec_size!=0, ExcEmptyObject());
 
   Number sum;
-  internal::VectorOperations::MeanValue<Number> mean(val);
+  internal::VectorOperations::MeanValue<Number> mean(values.get());
   internal::VectorOperations::parallel_reduce (mean, 0, vec_size, sum, thread_loop_partitioner);
 
   return sum / real_type(size());
@@ -493,7 +514,7 @@ Vector<Number>::l1_norm () const
   Assert (vec_size!=0, ExcEmptyObject());
 
   real_type sum;
-  internal::VectorOperations::Norm1<Number, real_type> norm1(val);
+  internal::VectorOperations::Norm1<Number, real_type> norm1(values.get());
   internal::VectorOperations::parallel_reduce (norm1, 0, vec_size, sum, thread_loop_partitioner);
 
   return sum;
@@ -513,7 +534,7 @@ Vector<Number>::l2_norm () const
   Assert (vec_size!=0, ExcEmptyObject());
 
   real_type norm_square;
-  internal::VectorOperations::Norm2<Number, real_type> norm2(val);
+  internal::VectorOperations::Norm2<Number, real_type> norm2(values.get());
   internal::VectorOperations::parallel_reduce (norm2, 0, vec_size, norm_square,
                                                thread_loop_partitioner);
   if (numbers::is_finite(norm_square) &&
@@ -525,10 +546,10 @@ Vector<Number>::l2_norm () const
       real_type sum = 1.;
       for (size_type i=0; i<vec_size; ++i)
         {
-          if (val[i] != Number())
+          if (values[i] != Number())
             {
               const real_type abs_x =
-                numbers::NumberTraits<Number>::abs(val[i]);
+                numbers::NumberTraits<Number>::abs(values[i]);
               if (scale < abs_x)
                 {
                   sum = 1. + sum * (scale/abs_x) * (scale/abs_x);
@@ -557,7 +578,7 @@ Vector<Number>::lp_norm (const real_type p) const
     return l2_norm();
 
   real_type sum;
-  internal::VectorOperations::NormP<Number, real_type> normp(val, p);
+  internal::VectorOperations::NormP<Number, real_type> normp(values.get(), p);
   internal::VectorOperations::parallel_reduce (normp, 0, vec_size, sum, thread_loop_partitioner);
 
   if (numbers::is_finite(sum) && sum >= std::numeric_limits<real_type>::min())
@@ -568,10 +589,10 @@ Vector<Number>::lp_norm (const real_type p) const
       real_type sum = 1.;
       for (size_type i=0; i<vec_size; ++i)
         {
-          if (val[i] != Number())
+          if (values[i] != Number())
             {
               const real_type abs_x =
-                numbers::NumberTraits<Number>::abs(val[i]);
+                numbers::NumberTraits<Number>::abs(values[i]);
               if (scale < abs_x)
                 {
                   sum = 1. + sum * std::pow(scale/abs_x, p);
@@ -596,7 +617,7 @@ Vector<Number>::linfty_norm () const
   real_type max = 0.;
 
   for (size_type i=0; i<vec_size; ++i)
-    max = std::max (numbers::NumberTraits<Number>::abs(val[i]), max);
+    max = std::max (numbers::NumberTraits<Number>::abs(values[i]), max);
 
   return max;
 }
@@ -614,7 +635,7 @@ Vector<Number>::add_and_dot (const Number          a,
   AssertDimension (vec_size, W.size());
 
   Number sum;
-  internal::VectorOperations::AddAndDot<Number> adder(this->val, V.val, W.val, a);
+  internal::VectorOperations::AddAndDot<Number> adder(this->values.get(), V.values.get(), W.values.get(), a);
   internal::VectorOperations::parallel_reduce (adder, 0, vec_size, sum, thread_loop_partitioner);
   AssertIsFinite(sum);
 
@@ -627,8 +648,10 @@ template <typename Number>
 Vector<Number> &Vector<Number>::operator += (const Vector<Number> &v)
 {
   Assert (vec_size!=0, ExcEmptyObject());
+  Assert (vec_size == v.vec_size, ExcDimensionMismatch(vec_size, v.vec_size));
 
-  add (v);
+  internal::VectorOperations::Vectorization_add_v<Number> vector_add(values.get(), v.values.get());
+  internal::VectorOperations::parallel_for(vector_add,0,vec_size,thread_loop_partitioner);
   return *this;
 }
 
@@ -640,7 +663,7 @@ Vector<Number> &Vector<Number>::operator -= (const Vector<Number> &v)
   Assert (vec_size!=0, ExcEmptyObject());
   Assert (vec_size == v.vec_size, ExcDimensionMismatch(vec_size, v.vec_size));
 
-  internal::VectorOperations::Vectorization_subtract_v<Number> vector_subtract(val, v.val);
+  internal::VectorOperations::Vectorization_subtract_v<Number> vector_subtract(values.get(), v.values.get());
   internal::VectorOperations::parallel_for(vector_subtract,0,vec_size,thread_loop_partitioner);
 
   return *this;
@@ -653,19 +676,7 @@ void Vector<Number>::add (const Number v)
 {
   Assert (vec_size!=0, ExcEmptyObject());
 
-  internal::VectorOperations::Vectorization_add_factor<Number> vector_add(val, v);
-  internal::VectorOperations::parallel_for(vector_add,0,vec_size,thread_loop_partitioner);
-}
-
-
-
-template <typename Number>
-void Vector<Number>::add (const Vector<Number> &v)
-{
-  Assert (vec_size!=0, ExcEmptyObject());
-  Assert (vec_size == v.vec_size, ExcDimensionMismatch(vec_size, v.vec_size));
-
-  internal::VectorOperations::Vectorization_add_v<Number> vector_add(val, v.val);
+  internal::VectorOperations::Vectorization_add_factor<Number> vector_add(values.get(), v);
   internal::VectorOperations::parallel_for(vector_add,0,vec_size,thread_loop_partitioner);
 }
 
@@ -682,7 +693,7 @@ void Vector<Number>::add (const Number a, const Vector<Number> &v,
   Assert (vec_size == v.vec_size, ExcDimensionMismatch(vec_size, v.vec_size));
   Assert (vec_size == w.vec_size, ExcDimensionMismatch(vec_size, w.vec_size));
 
-  internal::VectorOperations::Vectorization_add_avpbw<Number> vector_add(val, v.val, w.val, a, b);
+  internal::VectorOperations::Vectorization_add_avpbw<Number> vector_add(values.get(), v.values.get(), w.values.get(), a, b);
   internal::VectorOperations::parallel_for(vector_add,0,vec_size,thread_loop_partitioner);
 }
 
@@ -697,50 +708,19 @@ void Vector<Number>::sadd (const Number x,
   Assert (vec_size!=0, ExcEmptyObject());
   Assert (vec_size == v.vec_size, ExcDimensionMismatch(vec_size, v.vec_size));
 
-  internal::VectorOperations::Vectorization_sadd_xv<Number> vector_sadd(val, v.val, x);
+  internal::VectorOperations::Vectorization_sadd_xv<Number> vector_sadd(values.get(), v.values.get(), x);
   internal::VectorOperations::parallel_for(vector_sadd,0,vec_size,thread_loop_partitioner);
 }
 
 
 
-template <typename Number>
-void Vector<Number>::sadd (const Number x, const Number a,
-                           const Vector<Number> &v, const Number b,
-                           const Vector<Number> &w)
-{
-  AssertIsFinite(x);
-  AssertIsFinite(a);
-  AssertIsFinite(b);
-
-  Assert (vec_size!=0, ExcEmptyObject());
-  Assert (vec_size == v.vec_size, ExcDimensionMismatch(vec_size, v.vec_size));
-  Assert (vec_size == w.vec_size, ExcDimensionMismatch(vec_size, w.vec_size));
-
-  internal::VectorOperations::Vectorization_sadd_xavbw<Number> vector_sadd(val, v.val, w.val, x,
-      a, b);
-  internal::VectorOperations::parallel_for(vector_sadd,0,vec_size,thread_loop_partitioner);
-}
-
-
-template <typename Number>
-void Vector<Number>::sadd (const Number x, const Number a,
-                           const Vector<Number> &v, const Number b,
-                           const Vector<Number> &w, const Number c,
-                           const Vector<Number> &y)
-{
-  sadd (x, a, v, b, w);
-  add (c, y);
-}
-
-
-
 template <typename Number>
 void Vector<Number>::scale (const Vector<Number> &s)
 {
   Assert (vec_size!=0, ExcEmptyObject());
   Assert (vec_size == s.vec_size, ExcDimensionMismatch(vec_size, s.vec_size));
 
-  internal::VectorOperations::Vectorization_scale<Number> vector_scale(val, s.val);
+  internal::VectorOperations::Vectorization_scale<Number> vector_scale(values.get(), s.values.get());
   internal::VectorOperations::parallel_for(vector_scale,0,vec_size,thread_loop_partitioner);
 }
 
@@ -754,7 +734,7 @@ void Vector<Number>::scale (const Vector<Number2> &s)
   Assert (vec_size == s.vec_size, ExcDimensionMismatch(vec_size, s.vec_size));
 
   for (size_type i=0; i<vec_size; ++i)
-    val[i] *= Number(s.val[i]);
+    values[i] *= Number(s.values[i]);
 }
 
 
@@ -768,7 +748,7 @@ void Vector<Number>::equ (const Number a,
   Assert (vec_size!=0, ExcEmptyObject());
   Assert (vec_size == u.vec_size, ExcDimensionMismatch(vec_size, u.vec_size));
 
-  internal::VectorOperations::Vectorization_equ_au<Number> vector_equ(val, u.val, a);
+  internal::VectorOperations::Vectorization_equ_au<Number> vector_equ(values.get(), u.values.get(), a);
   internal::VectorOperations::parallel_for(vector_equ,0,vec_size,thread_loop_partitioner);
 }
 
@@ -791,43 +771,11 @@ void Vector<Number>::equ (const Number a,
   // operator*(complex<float>,complex<double>)
   // is not defined by default
   for (size_type i=0; i<vec_size; ++i)
-    val[i] = a * Number(u.val[i]);
+    values[i] = a * Number(u.values[i]);
 }
 
 
 
-template <typename Number>
-void Vector<Number>::equ (const Number a, const Vector<Number> &u,
-                          const Number b, const Vector<Number> &v)
-{
-  AssertIsFinite(a);
-  AssertIsFinite(b);
-
-  Assert (vec_size!=0, ExcEmptyObject());
-  Assert (vec_size == u.vec_size, ExcDimensionMismatch(vec_size, u.vec_size));
-  Assert (vec_size == v.vec_size, ExcDimensionMismatch(vec_size, v.vec_size));
-
-  internal::VectorOperations::Vectorization_equ_aubv<Number> vector_equ(val, u.val, v.val, a, b);
-  internal::VectorOperations::parallel_for(vector_equ,0,vec_size,thread_loop_partitioner);
-}
-
-
-template <typename Number>
-void Vector<Number>::equ (const Number a, const Vector<Number> &u,
-                          const Number b, const Vector<Number> &v,
-                          const Number c, const Vector<Number> &w)
-{
-  Assert (vec_size!=0, ExcEmptyObject());
-  Assert (vec_size == u.vec_size, ExcDimensionMismatch(vec_size, u.vec_size));
-  Assert (vec_size == v.vec_size, ExcDimensionMismatch(vec_size, v.vec_size));
-  Assert (vec_size == w.vec_size, ExcDimensionMismatch(vec_size, w.vec_size));
-
-  internal::VectorOperations::Vectorization_equ_aubvcw<Number> vector_equ(val, u.val, v.val, w.val,
-      a, b, c);
-  internal::VectorOperations::parallel_for(vector_equ,0,vec_size,thread_loop_partitioner);
-}
-
-
 template <typename Number>
 void Vector<Number>::ratio (const Vector<Number> &a,
                             const Vector<Number> &b)
@@ -840,7 +788,7 @@ void Vector<Number>::ratio (const Vector<Number> &a,
   // we overwrite them anyway
   reinit (a.size(), true);
 
-  internal::VectorOperations::Vectorization_ratio<Number> vector_ratio(val, a.val, b.val);
+  internal::VectorOperations::Vectorization_ratio<Number> vector_ratio(values.get(), a.values.get(), b.values.get());
   internal::VectorOperations::parallel_for(vector_ratio,0,vec_size,thread_loop_partitioner);
 }
 
@@ -856,7 +804,7 @@ Vector<Number>::operator= (const BlockVector<Number> &v)
   size_type this_index = 0;
   for (size_type b=0; b<v.n_blocks(); ++b)
     for (size_type i=0; i<v.block(b).size(); ++i, ++this_index)
-      val[this_index] = v.block(b)(i);
+      values[this_index] = v.block(b)(i);
 
   return *this;
 }
@@ -864,47 +812,13 @@ Vector<Number>::operator= (const BlockVector<Number> &v)
 
 
 #ifdef DEAL_II_WITH_PETSC
-
 template <typename Number>
 Vector<Number> &
-Vector<Number>::operator= (const PETScWrappers::Vector &v)
+Vector<Number>::operator= (const PETScWrappers::VectorBase &v)
 {
-  if (v.size() != vec_size)
-    reinit (v.size(), true);
-  if (vec_size != 0)
-    {
-      // get a representation of the vector
-      // and copy it
-      PetscScalar *start_ptr;
-      PetscErrorCode ierr = VecGetArray (static_cast<const Vec &>(v), &start_ptr);
-      AssertThrow (ierr == 0, ExcPETScError(ierr));
-
-      internal::VectorOperations::copy (start_ptr, start_ptr+vec_size, begin());
-
-      // restore the representation of the
-      // vector
-      ierr = VecRestoreArray (static_cast<const Vec &>(v), &start_ptr);
-      AssertThrow (ierr == 0, ExcPETScError(ierr));
-    }
-
+  internal::copy_petsc_vector(v, *this);
   return *this;
 }
-
-
-
-template <typename Number>
-Vector<Number> &
-Vector<Number>::operator= (const PETScWrappers::MPI::Vector &v)
-{
-  // do this in a two-stage process:
-  // first convert to a sequential petsc
-  // vector, then copy that
-  PETScWrappers::Vector seq (v);
-  *this = seq;
-
-  return *this;
-}
-
 #endif
 
 
@@ -913,30 +827,30 @@ Vector<Number>::operator= (const PETScWrappers::MPI::Vector &v)
 template <typename Number>
 Vector<Number> &
 Vector<Number>::operator= (const TrilinosWrappers::MPI::Vector &v)
-{
-  // Generate a localized version
-  // of the Trilinos vectors and
-  // then call the other =
-  // operator.
-  TrilinosWrappers::Vector localized_vector (v);
-  *this = localized_vector;
-  return *this;
-}
-
-
-
-template <typename Number>
-Vector<Number> &
-Vector<Number>::operator= (const TrilinosWrappers::Vector &v)
 {
   if (v.size() != vec_size)
     reinit (v.size(), true);
   if (vec_size != 0)
     {
+      // Copy the distributed vector to
+      // a local one at all processors
+      // that know about the original vector.
+      // TODO: There could
+      // be a better solution than
+      // this, but it has not yet been
+      // found.
+      TrilinosWrappers::MPI::Vector localized_vector;
+      localized_vector.reinit(complete_index_set(vec_size), v.get_mpi_communicator());
+      localized_vector.reinit (v, false, true);
+
+      Assert(localized_vector.size() == vec_size,
+             ExcDimensionMismatch(localized_vector.size(), vec_size));
+
       // get a representation of the vector
       // and copy it
       TrilinosScalar **start_ptr;
-      int ierr = v.trilinos_vector().ExtractView (&start_ptr);
+
+      int ierr = localized_vector.trilinos_vector().ExtractView (&start_ptr);
       AssertThrow (ierr == 0, ExcTrilinosError(ierr));
 
       std::copy (start_ptr[0], start_ptr[0]+vec_size, begin());
@@ -962,7 +876,7 @@ Vector<Number>::operator== (const Vector<Number2> &v) const
   // operator==(complex<float>,complex<double>)
   // is not defined by default
   for (size_type i=0; i<vec_size; ++i)
-    if (val[i] != Number(v.val[i]))
+    if (values[i] != Number(v.values[i]))
       return false;
 
   return true;
@@ -976,7 +890,7 @@ void Vector<Number>::print (const char *format) const
   Assert (vec_size!=0, ExcEmptyObject());
 
   for (size_type j=0; j<size(); ++j)
-    internal::VectorOperations::print (val[j], format);
+    internal::VectorOperations::print (values[j], format);
   std::printf ("\n");
 }
 
@@ -1002,10 +916,10 @@ void Vector<Number>::print (std::ostream      &out,
 
   if (across)
     for (size_type i=0; i<size(); ++i)
-      out << val[i] << ' ';
+      out << values[i] << ' ';
   else
     for (size_type i=0; i<size(); ++i)
-      out << val[i] << std::endl;
+      out << values[i] << std::endl;
   out << std::endl;
 
   AssertThrow (out, ExcIO());
@@ -1024,10 +938,10 @@ Vector<Number>::print (LogStream &out, const unsigned int width, const bool acro
 
   if (across)
     for (size_type i=0; i<size(); ++i)
-      out << std::setw(width) << val[i] << ' ';
+      out << std::setw(width) << values[i] << ' ';
   else
     for (size_type i=0; i<size(); ++i)
-      out << val[i] << std::endl;
+      out << values[i] << std::endl;
   out << std::endl;
 }
 
@@ -1120,25 +1034,19 @@ Vector<Number>::memory_consumption () const
 
 template <typename Number>
 void
-Vector<Number>::allocate()
+Vector<Number>::allocate(const size_type copy_n_el)
 {
-  // make sure that we don't create a memory leak
-  Assert (val == 0, ExcInternalError());
-
-  // then allocate memory with the proper alignment requirements of 64 bytes
-  Utilities::System::posix_memalign ((void **)&val, 64, sizeof(Number)*max_vec_size);
+  // allocate memory with the proper alignment requirements of 64 bytes
+  Number *new_values;
+  Utilities::System::posix_memalign ((void **)&new_values, 64, sizeof(Number)*max_vec_size);
+  // copy:
+  for (size_type i = 0; i < copy_n_el; ++i)
+    new_values[i] = values[i];
+  values.reset (new_values);
 }
 
 
 
-template <typename Number>
-void
-Vector<Number>::deallocate()
-{
-  free(val);
-  val = 0;
-}
-
 DEAL_II_NAMESPACE_CLOSE
 
 #endif
