@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 2007 - 2015 by the deal.II authors
+// Copyright (C) 2007 - 2017 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -13,10 +13,13 @@
 //
 // ---------------------------------------------------------------------
 
-#ifndef dealii__vector_memory_templates_h
-#define dealii__vector_memory_templates_h
+#ifndef dealii_vector_memory_templates_h
+#define dealii_vector_memory_templates_h
 
 
+#include <deal.II/base/std_cxx14/memory.h>
+#include <deal.II/base/memory_consumption.h>
+
 #include <deal.II/lac/vector_memory.h>
 
 DEAL_II_NAMESPACE_OPEN
@@ -32,7 +35,7 @@ template <typename VectorType>
 inline
 GrowingVectorMemory<VectorType>::Pool::Pool()
   :
-  data(0)
+  data(nullptr)
 {}
 
 
@@ -42,18 +45,12 @@ inline
 GrowingVectorMemory<VectorType>::Pool::~Pool()
 {
   // Nothing to do if memory was unused.
-  if (data == 0) return;
+  if (data == nullptr)
+    return;
 
-  // First, delete all remaining
-  // vectors. Actually, there should
-  // be none, if there is no memory
-  // leak
-  for (typename std::vector<entry_type>::iterator i=data->begin();
-       i != data->end();
-       ++i)
-    {
-      delete i->second;
-    }
+  // delete the 'data' object. this also releases all vectors
+  // that are pointed to by the std::unique_ptrs
+  data->clear();
   delete data;
 }
 
@@ -63,7 +60,7 @@ inline
 void
 GrowingVectorMemory<VectorType>::Pool::initialize(const size_type size)
 {
-  if (data == 0)
+  if (data == nullptr)
     {
       data = new std::vector<entry_type>(size);
 
@@ -72,7 +69,7 @@ GrowingVectorMemory<VectorType>::Pool::initialize(const size_type size)
            ++i)
         {
           i->first = false;
-          i->second = new VectorType;
+          i->second = std_cxx14::make_unique<VectorType>();
         }
     }
 }
@@ -93,7 +90,7 @@ GrowingVectorMemory<VectorType>::GrowingVectorMemory (const size_type initial_si
 }
 
 
-template<typename VectorType>
+template <typename VectorType>
 inline
 GrowingVectorMemory<VectorType>::~GrowingVectorMemory()
 {
@@ -110,12 +107,13 @@ GrowingVectorMemory<VectorType>::~GrowingVectorMemory()
 
 
 
-template<typename VectorType>
+template <typename VectorType>
 inline
 VectorType *
 GrowingVectorMemory<VectorType>::alloc ()
 {
   Threads::Mutex::ScopedLock lock(mutex);
+
   ++total_alloc;
   ++current_alloc;
   // see if there is a free vector
@@ -126,30 +124,29 @@ GrowingVectorMemory<VectorType>::alloc ()
       if (i->first == false)
         {
           i->first = true;
-          return (i->second);
+          return i->second.get();
         }
     }
 
-  // no free vector found, so let's
-  // just allocate a new one
-  const entry_type t (true, new VectorType);
-  pool.data->push_back(t);
+  // no free vector found, so let's just allocate a new one
+  pool.data->emplace_back(entry_type (true, std_cxx14::make_unique<VectorType>()));
 
-  return t.second;
+  return pool.data->back().second.get();
 }
 
 
 
-template<typename VectorType>
+template <typename VectorType>
 inline
 void
 GrowingVectorMemory<VectorType>::free(const VectorType *const v)
 {
   Threads::Mutex::ScopedLock lock(mutex);
+
   for (typename std::vector<entry_type>::iterator i=pool.data->begin();
        i != pool.data->end(); ++i)
     {
-      if (v == (i->second))
+      if (v == i->second.get())
         {
           i->first = false;
           --current_alloc;
@@ -161,33 +158,20 @@ GrowingVectorMemory<VectorType>::free(const VectorType *const v)
 
 
 
-template<typename VectorType>
+template <typename VectorType>
 inline
 void
 GrowingVectorMemory<VectorType>::release_unused_memory ()
 {
   Threads::Mutex::ScopedLock lock(mutex);
 
-  std::vector<entry_type> new_data;
-
-  if (pool.data != 0)
-    {
-      const typename std::vector<entry_type>::const_iterator
-      end = pool.data->end();
-      for (typename std::vector<entry_type>::const_iterator
-           i = pool.data->begin(); i != end ; ++i)
-        if (i->first == false)
-          delete i->second;
-        else
-          new_data.push_back (*i);
-
-      *pool.data = new_data;
-    }
+  if (pool.data != nullptr)
+    pool.data->clear();
 }
 
 
 
-template<typename VectorType>
+template <typename VectorType>
 inline
 std::size_t
 GrowingVectorMemory<VectorType>::memory_consumption () const
@@ -199,7 +183,7 @@ GrowingVectorMemory<VectorType>::memory_consumption () const
   end = pool.data->end();
   for (typename std::vector<entry_type>::const_iterator
        i = pool.data->begin(); i != end ; ++i)
-    result += sizeof (*i) + i->second->memory_consumption();
+    result += sizeof (*i) + MemoryConsumption::memory_consumption(i->second);
 
   return result;
 }
