@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 2016 - 2017 by the deal.II authors
+// Copyright (C) 2016 - 2018 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -14,8 +14,8 @@
 // ---------------------------------------------------------------------
 
 
-#ifndef dealii__vector_operations_internal_h
-#define dealii__vector_operations_internal_h
+#ifndef dealii_vector_operations_internal_h
+#define dealii_vector_operations_internal_h
 
 #include <deal.II/base/multithread_info.h>
 #include <deal.II/base/parallel.h>
@@ -54,7 +54,7 @@ namespace internal
     void print (const T    &t,
                 const char *format)
     {
-      if (format != 0)
+      if (format != nullptr)
         std::printf (format, t);
       else
         std::printf (" %5.2f", double(t));
@@ -66,7 +66,7 @@ namespace internal
     void print (const std::complex<T> &t,
                 const char            *format)
     {
-      if (format != 0)
+      if (format != nullptr)
         std::printf (format, t.real(), t.imag());
       else
         std::printf (" %5.2f+%5.2fi",
@@ -126,7 +126,7 @@ namespace internal
       {
         const size_type vec_size = end-start;
         // set chunk size for sub-tasks
-        const unsigned int gs = internal::Vector::minimum_parallel_grain_size;
+        const unsigned int gs = internal::VectorImplementation::minimum_parallel_grain_size;
         n_chunks = std::min(static_cast<size_type>(4*MultithreadInfo::n_threads()),
                             vec_size / gs);
         chunk_size = vec_size / n_chunks;
@@ -140,7 +140,7 @@ namespace internal
         n_chunks = (vec_size + chunk_size - 1) / chunk_size;
         AssertIndexRange((n_chunks-1)*chunk_size, vec_size);
         AssertIndexRange(vec_size, n_chunks*chunk_size+1);
-      };
+      }
 
       void operator() (const tbb::blocked_range<size_type> &range) const
       {
@@ -161,19 +161,19 @@ namespace internal
     void parallel_for(Functor &functor,
                       size_type start,
                       size_type end,
-                      std_cxx11::shared_ptr<parallel::internal::TBBPartitioner> &partitioner)
+                      std::shared_ptr<parallel::internal::TBBPartitioner> &partitioner)
     {
 #ifdef DEAL_II_WITH_THREADS
       size_type vec_size = end-start;
       // only go to the parallel function in case there are at least 4 parallel
       // items, otherwise the overhead is too large
-      if (vec_size >= 4*internal::Vector::minimum_parallel_grain_size &&
+      if (vec_size >= 4*internal::VectorImplementation::minimum_parallel_grain_size &&
           MultithreadInfo::n_threads() > 1)
         {
-          Assert(partitioner.get() != NULL,
+          Assert(partitioner.get() != nullptr,
                  ExcInternalError("Unexpected initialization of Vector that does "
                                   "not set the TBB partitioner to a usable state."));
-          std_cxx11::shared_ptr<tbb::affinity_partitioner> tbb_partitioner =
+          std::shared_ptr<tbb::affinity_partitioner> tbb_partitioner =
             partitioner->acquire_one_partitioner();
 
           TBBForFunctor<Functor> generic_functor(functor, start, end);
@@ -203,18 +203,31 @@ namespace internal
         :
         value(value),
         dst(dst)
-      {}
+      {
+        Assert(dst != nullptr, ExcInternalError());
+      }
 
       void operator() (const size_type begin, const size_type end) const
       {
+        Assert (end>=begin, ExcInternalError());
+
         if (value == Number())
-          std::memset (dst+begin,0,(end-begin)*sizeof(Number));
-        else
-          std::fill (dst+begin, dst+end, value);
+          {
+#ifdef DEAL_II_WITH_CXX17
+            if constexpr (std::is_trivial<Number>::value)
+#else
+            if (std::is_trivial<Number>::value)
+#endif
+              {
+                std::memset(dst+begin, 0, sizeof(Number)*(end-begin));
+                return;
+              }
+          }
+        std::fill (dst+begin, dst+end, value);
       }
 
-      Number value;
-      Number *dst;
+      const Number value;
+      Number *const dst;
     };
 
     template <typename Number, typename OtherNumber>
@@ -224,11 +237,24 @@ namespace internal
         :
         src(src),
         dst(dst)
-      {}
+      {
+        Assert(src!=nullptr, ExcInternalError());
+        Assert(dst!=nullptr, ExcInternalError());
+      }
 
       void operator() (const size_type begin, const size_type end) const
       {
-        if (types_are_equal<Number,OtherNumber>::value)
+        Assert (end>=begin, ExcInternalError());
+
+#if __GNUG__ && __GNUC__ < 5
+        if (  __has_trivial_copy(Number) && std::is_same<Number,OtherNumber>::value)
+#else
+#  ifdef DEAL_II_WITH_CXX17
+        if constexpr (std::is_trivially_copyable<Number>()  && std::is_same<Number,OtherNumber>::value)
+#  else
+        if (std::is_trivially_copyable<Number>() && std::is_same<Number,OtherNumber>::value)
+#  endif
+#endif
           std::memcpy(dst+begin, src+begin, (end-begin)*sizeof(Number));
         else
           {
@@ -238,8 +264,8 @@ namespace internal
           }
       }
 
-      const OtherNumber *src;
-      Number *dst;
+      const OtherNumber *const src;
+      Number *const dst;
     };
 
     template <typename Number>
@@ -689,7 +715,7 @@ namespace internal
     template <typename Number, typename Number2>
     struct Dot
     {
-      static const bool vectorizes = types_are_equal<Number,Number2>::value &&
+      static const bool vectorizes = std::is_same<Number,Number2>::value &&
                                      (VectorizedArray<Number>::n_array_elements > 1);
 
       Dot(const Number *X, const Number2 *Y)
@@ -710,6 +736,17 @@ namespace internal
         VectorizedArray<Number> x, y;
         x.load(X+i);
         y.load(Y+i);
+
+        // the following operation in VectorizedArray does an element-wise
+        // scalar product without taking into account complex values and
+        // the need to take the complex-conjugate of one argument. this
+        // may be a bug, but because all VectorizedArray classes only
+        // work on real scalars, it doesn't really matter very much.
+        // in any case, assert that we really don't get here for
+        // complex-valued objects
+        static_assert (numbers::NumberTraits<Number>::is_complex == false,
+                       "This operation is not correctly implemented for "
+                       "complex-valued objects.");
         return x * y;
       }
 
@@ -858,6 +895,17 @@ namespace internal
         // may only load from W after storing in X because the pointers might
         // point to the same memory
         w.load(W+i);
+
+        // the following operation in VectorizedArray does an element-wise
+        // scalar product without taking into account complex values and
+        // the need to take the complex-conjugate of one argument. this
+        // may be a bug, but because all VectorizedArray classes only
+        // work on real scalars, it doesn't really matter very much.
+        // in any case, assert that we really don't get here for
+        // complex-valued objects
+        static_assert (numbers::NumberTraits<Number>::is_complex == false,
+                       "This operation is not correctly implemented for "
+                       "complex-valued objects.");
         return x * w;
       }
 
@@ -942,7 +990,7 @@ namespace internal
           // version often enough, we need to have all tasks but the last one
           // to be divisible by the vectorization length
           accumulate_regular(op, n_chunks, index, outer_results,
-                             internal::bool2type<Operation::vectorizes>());
+                             std::integral_constant<bool, Operation::vectorizes>());
 
           // now work on the remainder, i.e., the last up to 32 values. Use
           // switch statement with fall-through to work on these values.
@@ -965,18 +1013,18 @@ namespace internal
                   r2 = op(index++);
                   for (size_type j=1; j<8; ++j)
                     r2 += op(index++);
-                // no break
+                  DEAL_II_FALLTHROUGH;
                 case 2:
                   r1 = op(index++);
                   for (size_type j=1; j<8; ++j)
                     r1 += op(index++);
                   r1 += r2;
-                // no break
+                  DEAL_II_FALLTHROUGH;
                 case 1:
                   r2 = op(index++);
                   for (size_type j=1; j<8; ++j)
                     r2 += op(index++);
-                // no break
+                  DEAL_II_FALLTHROUGH;
                 default:
                   for (size_type j=0; j<remainder_inner; ++j)
                     r0 += op(index++);
@@ -1039,7 +1087,7 @@ namespace internal
                        size_type       &n_chunks,
                        size_type       &index,
                        ResultType (&outer_results)[vector_accumulation_recursion_threshold],
-                       internal::bool2type<false>)
+                       std::integral_constant<bool, false>)
     {
       // note that each chunk is chosen to have a width of 32, thereby the index
       // is incremented by 4*8 for each @p i.
@@ -1076,7 +1124,7 @@ namespace internal
                        size_type       &n_chunks,
                        size_type       &index,
                        Number (&outer_results)[vector_accumulation_recursion_threshold],
-                       internal::bool2type<true>)
+                       std::integral_constant<bool, true>)
     {
       // we start from @p index and workout @p n_chunks each of size 32.
       // in order employ SIMD and work on @p nvecs at a time, we split this
@@ -1180,7 +1228,7 @@ namespace internal
       {
         const size_type vec_size = end-start;
         // set chunk size for sub-tasks
-        const unsigned int gs = internal::Vector::minimum_parallel_grain_size;
+        const unsigned int gs = internal::VectorImplementation::minimum_parallel_grain_size;
         n_chunks = std::min(static_cast<size_type>(4*MultithreadInfo::n_threads()),
                             vec_size / gs);
         chunk_size = vec_size / n_chunks;
@@ -1200,11 +1248,11 @@ namespace internal
             // make sure we allocate an even number of elements,
             // access to the new last element is needed in do_sum()
             large_array.resize(2*((n_chunks+1)/2));
-            array_ptr = &large_array[0];
+            array_ptr = large_array.data();
           }
         else
           array_ptr = &small_array[0];
-      };
+      }
 
       /**
        * An operator used by TBB to work on a given @p range of chunks
@@ -1255,19 +1303,19 @@ namespace internal
                           const size_type    start,
                           const size_type    end,
                           ResultType        &result,
-                          std_cxx11::shared_ptr<parallel::internal::TBBPartitioner> &partitioner)
+                          std::shared_ptr<parallel::internal::TBBPartitioner> &partitioner)
     {
 #ifdef DEAL_II_WITH_THREADS
       size_type vec_size = end-start;
       // only go to the parallel function in case there are at least 4 parallel
       // items, otherwise the overhead is too large
-      if (vec_size >= 4*internal::Vector::minimum_parallel_grain_size &&
+      if (vec_size >= 4*internal::VectorImplementation::minimum_parallel_grain_size &&
           MultithreadInfo::n_threads() > 1)
         {
-          Assert(partitioner.get() != NULL,
+          Assert(partitioner.get() != nullptr,
                  ExcInternalError("Unexpected initialization of Vector that does "
                                   "not set the TBB partitioner to a usable state."));
-          std_cxx11::shared_ptr<tbb::affinity_partitioner> tbb_partitioner =
+          std::shared_ptr<tbb::affinity_partitioner> tbb_partitioner =
             partitioner->acquire_one_partitioner();
 
           TBBReduceFunctor<Operation,ResultType> generic_functor(op, start, end);
