@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 2015 - 2016 by the deal.II authors
+// Copyright (C) 2015 - 2018 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -13,13 +13,14 @@
 //
 // ---------------------------------------------------------------------
 
-#ifndef dealii__vector_space_vector_h
-#define dealii__vector_space_vector_h
+#ifndef dealii_vector_space_vector_h
+#define dealii_vector_space_vector_h
 
 #include <deal.II/base/config.h>
 #include <deal.II/base/numbers.h>
-#include <deal.II/base/std_cxx11/shared_ptr.h>
-#include <deal.II/lac/vector.h>
+#include <deal.II/lac/vector_operation.h>
+
+#include <memory>
 
 
 DEAL_II_NAMESPACE_OPEN
@@ -54,6 +55,18 @@ namespace LinearAlgebra
     typedef types::global_dof_index                           size_type;
     typedef typename numbers::NumberTraits<Number>::real_type real_type;
 
+    /**
+     * Change the dimension to that of the vector V. The elements of V are not
+     * copied.
+     */
+    virtual void reinit(const VectorSpaceVector<Number> &V,
+                        const bool omit_zeroing_entries = false) = 0;
+
+    /**
+     * Sets all elements of the vector to the scalar @p s. This operation is
+     * only allowed if @p s is equal to zero.
+     */
+    virtual VectorSpaceVector<Number> &operator= (const Number s) = 0;
 
     /**
      * Multiply the entire vector by a fixed factor.
@@ -85,8 +98,8 @@ namespace LinearAlgebra
      */
     virtual void import(const ReadWriteVector<Number> &V,
                         VectorOperation::values operation,
-                        std_cxx11::shared_ptr<const CommunicationPatternBase> communication_pattern =
-                          std_cxx11::shared_ptr<const CommunicationPatternBase> ()) = 0;
+                        std::shared_ptr<const CommunicationPatternBase> communication_pattern =
+                          std::shared_ptr<const CommunicationPatternBase> ()) = 0;
 
     /**
      * Return the scalar product of two vectors.
@@ -128,6 +141,11 @@ namespace LinearAlgebra
      */
     virtual void equ(const Number a, const VectorSpaceVector<Number> &V) = 0;
 
+    /**
+     * Return whether the vector contains only elements with value zero.
+     */
+    virtual bool all_zero() const = 0;
+
     /**
      * Return the mean value of all the entries of this vector.
      */
@@ -166,6 +184,9 @@ namespace LinearAlgebra
      * separate methods means to load the calling vector @p this twice. Since
      * most vector operations are memory transfer limited, this reduces the
      * time by 25\% (or 50\% if @p W equals @p this).
+     *
+     * For complex-valued vectors, the scalar product in the second step is implemented as
+     * $\left<v,w\right>=\sum_i v_i \bar{w_i}$.
      */
     virtual Number add_and_dot(const Number a,
                                const VectorSpaceVector<Number> &V,
@@ -185,7 +206,7 @@ namespace LinearAlgebra
     /**
      * Return an index set that describes which elements of this vector are
      * owned by the current processor. As a consequence, the index sets
-     * returned on different procesors if this is a distributed vector will
+     * returned on different processors if this is a distributed vector will
      * form disjoint sets that add up to the complete index set. Obviously, if
      * a vector is created on only one processor, then the result would
      * satisfy
@@ -212,20 +233,9 @@ namespace LinearAlgebra
      * Destructor. Declared as virtual so that inheriting classes (which may
      * manage their own memory) are destroyed correctly.
      */
-    virtual ~VectorSpaceVector();
+    virtual ~VectorSpaceVector() = default;
   };
   /*@}*/
-
-
-
-#ifndef DOXYGEN
-  // TODO we can get rid of this when we require C++11 by using '=default'
-  // above
-  template <typename Number>
-  inline
-  VectorSpaceVector<Number>::~VectorSpaceVector()
-  {}
-#endif // DOXYGEN
 }
 
 DEAL_II_NAMESPACE_CLOSE
