@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 2011 - 2017 by the deal.II authors
+// Copyright (C) 2011 - 2018 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -14,8 +14,8 @@
 // ---------------------------------------------------------------------
 
 
-#ifndef dealii__matrix_free_operators_h
-#define dealii__matrix_free_operators_h
+#ifndef dealii_matrix_free_operators_h
+#define dealii_matrix_free_operators_h
 
 
 #include <deal.II/base/exceptions.h>
@@ -24,7 +24,6 @@
 
 #include <deal.II/lac/diagonal_matrix.h>
 #include <deal.II/lac/la_parallel_vector.h>
-#include <deal.II/lac/vector_view.h>
 #include <deal.II/multigrid/mg_constrained_dofs.h>
 #include <deal.II/matrix_free/matrix_free.h>
 #include <deal.II/matrix_free/fe_evaluation.h>
@@ -41,7 +40,7 @@ namespace MatrixFreeOperators
     // workaroud for unifying non-block vector and block vector implementations
     // a non-block vector has one block and the only subblock is the vector itself
     template <typename VectorType>
-    typename std_cxx11::enable_if<IsBlockVector<VectorType>::value,
+    typename std::enable_if<IsBlockVector<VectorType>::value,
              unsigned int>::type
              n_blocks(const VectorType &vector)
     {
@@ -49,7 +48,7 @@ namespace MatrixFreeOperators
     }
 
     template <typename VectorType>
-    typename std_cxx11::enable_if<!IsBlockVector<VectorType>::value,
+    typename std::enable_if<!IsBlockVector<VectorType>::value,
              unsigned int>::type
              n_blocks(const VectorType &)
     {
@@ -57,7 +56,7 @@ namespace MatrixFreeOperators
     }
 
     template <typename VectorType>
-    typename std_cxx11::enable_if<IsBlockVector<VectorType>::value,
+    typename std::enable_if<IsBlockVector<VectorType>::value,
              typename VectorType::BlockType &>::type
              subblock(VectorType &vector, unsigned int block_no)
     {
@@ -65,7 +64,7 @@ namespace MatrixFreeOperators
     }
 
     template <typename VectorType>
-    typename std_cxx11::enable_if<IsBlockVector<VectorType>::value,
+    typename std::enable_if<IsBlockVector<VectorType>::value,
              const typename VectorType::BlockType &>::type
              subblock(const VectorType &vector, unsigned int block_no)
     {
@@ -74,7 +73,7 @@ namespace MatrixFreeOperators
     }
 
     template <typename VectorType>
-    typename std_cxx11::enable_if<!IsBlockVector<VectorType>::value,
+    typename std::enable_if<!IsBlockVector<VectorType>::value,
              VectorType &>::type
              subblock(VectorType &vector, unsigned int)
     {
@@ -82,7 +81,7 @@ namespace MatrixFreeOperators
     }
 
     template <typename VectorType>
-    typename std_cxx11::enable_if<!IsBlockVector<VectorType>::value,
+    typename std::enable_if<!IsBlockVector<VectorType>::value,
              const VectorType &>::type
              subblock(const VectorType &vector, unsigned int)
     {
@@ -90,7 +89,7 @@ namespace MatrixFreeOperators
     }
 
     template <typename VectorType>
-    typename std_cxx11::enable_if<IsBlockVector<VectorType>::value,
+    typename std::enable_if<IsBlockVector<VectorType>::value,
              void>::type
              collect_sizes(VectorType &vector)
     {
@@ -98,7 +97,7 @@ namespace MatrixFreeOperators
     }
 
     template <typename VectorType>
-    typename std_cxx11::enable_if<!IsBlockVector<VectorType>::value,
+    typename std::enable_if<!IsBlockVector<VectorType>::value,
              void>::type
              collect_sizes(const VectorType &)
     {}
@@ -109,7 +108,8 @@ namespace MatrixFreeOperators
    * the finest mesh or at a certain level in geometric multigrid.
    *
    * A derived class has to implement apply_add() method as well as
-   * compute_diagonal() to initialize the protected member inverse_diagonal_entries.
+   * compute_diagonal() to initialize the protected member inverse_diagonal_entries
+   * and/or diagonal_entries.
    * In case of a non-symmetric operator, Tapply_add() should be additionally
    * implemented.
    *
@@ -119,30 +119,13 @@ namespace MatrixFreeOperators
    * <h4>Selective use of blocks in MatrixFree</h4>
    *
    * MatrixFree allows to use several DoFHandler/ConstraintMatrix combinations
-   * by passing an std::vector with pointers to the respective objects into
+   * by passing a std::vector with pointers to the respective objects into
    * the MatrixFree::reinit function. This class supports to select only some
    * of the blocks in the underlying MatrixFree object by optional integer
    * lists that specify the chosen blocks.
    *
-   * One application of selection is in problems with a Newton-type iteration
-   * or problems with inhomogeneous boundary conditions. In such a case, one
-   * has to deal with two different sets of constraints: One set of
-   * constraints applies to the solution vector which might include hanging
-   * node constraints or periodicity constraints but no constraints on
-   * inhomogeneous Dirichlet boundaries. Before the nonlinear iteration, the
-   * boundary values are set to the expected value in the vector representing
-   * the initial guess. In each iteration of the Newton method, a linear
-   * system subject to zero Dirichlet boundary conditions is solved that is
-   * then added to the initial guess. This setup can be realized by using a
-   * vector of two pointers pointing to the same DoFHandler object and a
-   * vector of two pointers to the two ConstraintMatrix objects. If the first
-   * constraint matrix is the one including the zero Dirichlet constraints,
-   * one would give an std::vector<unsigned int>(1, 0) to the initialize()
-   * function, i.e., a vector of length 1 that selects exactly the first
-   * constraint matrix with index 0.
-   *
-   * A second application of using selectively constructing a matrix-free
-   * operator would be the setting of the step-32 tutorial program: This
+   * One application of constructing a matrix-free operator only on selected
+   * blocks would be the setting of the step-32 tutorial program: This
    * problem has three <i>blocks</i>, one for the velocity, one for the
    * pressure, and one for temperature. The time lag scheme used for temporal
    * evolution splits the temperature equation away from the Stokes system in
@@ -150,7 +133,7 @@ namespace MatrixFreeOperators
    * that enters the temperature advection-diffusion equation or the
    * temperature that enters the right hand side of the velocity. In order to
    * be sure that MatrixFree uses the same integer indexing to the different
-   * blocks, one needs to put all three blocks into the same MatrixFree
+   * blocks, one needs to put all the three blocks into the same MatrixFree
    * object. However, when solving a linear system the operators involved
    * either address the first two in the Stokes solver, or the last one for
    * the temperature solver. In the former case, a BlockVector of two
@@ -158,6 +141,44 @@ namespace MatrixFreeOperators
    * MatrixFree, whereas in the latter, a non-block vector selecting the block
    * {2} would be used.
    *
+   * A second application of selection is in problems with a Newton-type iteration
+   * or problems with inhomogeneous boundary conditions. In such a case, one
+   * has to deal with two different sets of constraints: One set of
+   * constraints applies to the solution vector which might include hanging
+   * node constraints or periodicity constraints but no constraints on
+   * inhomogeneous Dirichlet boundaries. Before the nonlinear iteration, the
+   * boundary values are set to the expected value in the vector, representing
+   * the initial guess. In each iteration of the Newton method, a linear
+   * system subject to zero Dirichlet boundary conditions is solved that is
+   * then added to the initial guess. This setup can be realized by using a
+   * vector of two pointers pointing to the same DoFHandler object and a
+   * vector of two pointers to the two ConstraintMatrix objects. If the first
+   * constraint matrix is the one including the zero Dirichlet constraints,
+   * one would give a std::vector<unsigned int>(1, 0) to the initialize()
+   * function, i.e., a vector of length 1 that selects exactly the first
+   * constraint matrix with index 0.
+   *
+   * For systems of PDEs where the different blocks of MatrixFree are
+   * associated with different physical components of the equations, adding
+   * another block with a different ConstraintMatrix argument solely for the
+   * purpose of boundary conditions might lead to cumbersome index
+   * handling. Instead, one could set up a second MatrixFree instance with the
+   * different constraint matrix but the same interpretation of blocks, and
+   * use that for interpolating inhomogeneous boundary conditions (see also
+   * the discussion in the results section of the step-37 tutorial program):
+   *
+   * @code
+   * matrix_free_inhomogeneous.reinit(dof_handler, constraints_no_dirichlet,
+   *                                  quadrature, additional_data);
+   * operator_inhomogeneous.initialize(matrix_free_inhomogeneous, selected_blocks);
+   * LinearAlgebra::distributed::Vector<double> inhomogeneity;
+   * matrix_free_inhomogeneous.initialize_dof_vector(inhomogeneity);
+   * constraints_with_dirichlet.distribute(inhomogeneity);
+   * operator_inhomogeneous.vmult(system_rhs, inhomogeneity);
+   * system_rhs *= -1.;
+   * // proceed with other terms from right hand side...
+   * @endcode
+   *
    * @author Denis Davydov, Daniel Arndt, Martin Kronbichler, 2016, 2017
    */
   template <int dim, typename VectorType = LinearAlgebra::distributed::Vector<double> >
@@ -182,7 +203,7 @@ namespace MatrixFreeOperators
     /**
      * Virtual destructor.
      */
-    virtual ~Base();
+    virtual ~Base() = default;
 
     /**
      * Release all memory and return to a state just like after having called
@@ -206,7 +227,7 @@ namespace MatrixFreeOperators
      * column selection vector is empty, it is taken the same as the row
      * selection, defining a diagonal block.
      */
-    void initialize (std_cxx11::shared_ptr<const MatrixFree<dim,value_type> > data,
+    void initialize (std::shared_ptr<const MatrixFree<dim,value_type> > data,
                      const std::vector<unsigned int> &selected_row_blocks = std::vector<unsigned int>(),
                      const std::vector<unsigned int> &selected_column_blocks = std::vector<unsigned int>());
 
@@ -223,7 +244,7 @@ namespace MatrixFreeOperators
      * columns is used as opposed to the non-level initialization function. If
      * empty, all components are selected.
      */
-    void initialize (std_cxx11::shared_ptr<const MatrixFree<dim,value_type> > data,
+    void initialize (std::shared_ptr<const MatrixFree<dim,value_type> > data,
                      const MGConstrainedDoFs &mg_constrained_dofs,
                      const unsigned int level,
                      const std::vector<unsigned int> &selected_row_blocks = std::vector<unsigned int>());
@@ -243,7 +264,7 @@ namespace MatrixFreeOperators
      * empty, all components are selected.
      */
     void
-    initialize(std_cxx11::shared_ptr<const MatrixFree<dim, value_type> > data_,
+    initialize(std::shared_ptr<const MatrixFree<dim, value_type> > data_,
                const std::vector<MGConstrainedDoFs> &mg_constrained_dofs,
                const unsigned int level,
                const std::vector<unsigned int> &selected_row_blocks = std::vector<unsigned int>());
@@ -295,7 +316,7 @@ namespace MatrixFreeOperators
                      const VectorType &src) const;
 
     /**
-     * Returns the value of the matrix entry (row,col). In matrix-free context
+     * Return the value of the matrix entry (row,col). In matrix-free context
      * this function is valid only for row==col when diagonal is initialized.
      */
     value_type el (const unsigned int row,
@@ -315,22 +336,29 @@ namespace MatrixFreeOperators
      * Compute diagonal of this operator.
      *
      * A derived class needs to implement this function and resize and fill
-     * the protected member inverse_diagonal_entries accordingly.
+     * the protected member inverse_diagonal_entries and/or diagonal_entries accordingly.
      */
     virtual void compute_diagonal () = 0;
 
     /**
      * Get read access to the MatrixFree object stored with this operator.
      */
-    std_cxx11::shared_ptr<const MatrixFree<dim,value_type> >
+    std::shared_ptr<const MatrixFree<dim,value_type> >
     get_matrix_free () const;
 
     /**
      * Get read access to the inverse diagonal of this operator.
      */
-    const std_cxx11::shared_ptr<DiagonalMatrix<VectorType> > &
+    const std::shared_ptr<DiagonalMatrix<VectorType> > &
     get_matrix_diagonal_inverse() const;
 
+    /**
+     * Get read access to the diagonal of this operator.
+     */
+    const std::shared_ptr<DiagonalMatrix<VectorType> > &
+    get_matrix_diagonal() const;
+
+
     /**
      * Apply the Jacobi preconditioner, which multiplies every element of the
      * <tt>src</tt> vector by the inverse of the respective diagonal element and
@@ -342,6 +370,18 @@ namespace MatrixFreeOperators
 
   protected:
 
+    /**
+     * Perform necessary operations related to constraints before calling
+     * apply_add() or Tapply_add() inside mult_add().
+     */
+    void preprocess_constraints(VectorType &dst, const VectorType &src) const;
+
+    /**
+     * Perform necessary operations related to constraints after calling
+     * apply_add() or Tapply_add() inside mult_add().
+     */
+    void postprocess_constraints(VectorType &dst, const VectorType &src) const;
+
     /**
      * Set constrained entries (both from hanging nodes and edge constraints)
      * of @p dst to one.
@@ -365,13 +405,19 @@ namespace MatrixFreeOperators
     /**
      * MatrixFree object to be used with this operator.
      */
-    std_cxx11::shared_ptr<const MatrixFree<dim,value_type> > data;
+    std::shared_ptr<const MatrixFree<dim,value_type> > data;
+
+    /**
+     * A shared pointer to a diagonal matrix that stores the
+     * diagonal elements as a vector.
+     */
+    std::shared_ptr<DiagonalMatrix<VectorType > > diagonal_entries;
 
     /**
      * A shared pointer to a diagonal matrix that stores the inverse of
      * diagonal elements as a vector.
      */
-    std_cxx11::shared_ptr<DiagonalMatrix<VectorType > > inverse_diagonal_entries;
+    std::shared_ptr<DiagonalMatrix<VectorType > > inverse_diagonal_entries;
 
     /**
      * A vector which defines the selection of sub-components of MatrixFree
@@ -470,6 +516,11 @@ namespace MatrixFreeOperators
      */
     typedef typename OperatorType::value_type value_type;
 
+    /**
+     * Size type.
+     */
+    typedef typename OperatorType::size_type size_type;
+
     /**
      * Default constructor.
      */
@@ -499,6 +550,13 @@ namespace MatrixFreeOperators
     void Tvmult (VectorType &dst,
                  const VectorType &src) const;
 
+    /**
+     * A wrapper for initialize_dof_vector() of OperatorType object.
+     */
+    template <typename VectorType>
+    void initialize_dof_vector (VectorType &vec) const;
+
+
   private:
     /**
      * Const pointer to the operator class.
@@ -675,8 +733,8 @@ namespace MatrixFreeOperators
      *
      * Such tables can be initialized by
      * @code
-     * std_cxx11::shared_ptr<Table<2, VectorizedArray<double> > > coefficient;
-     * coefficient = std_cxx11::make_shared<Table<2, VectorizedArray<double> > >();
+     * std::shared_ptr<Table<2, VectorizedArray<double> > > coefficient;
+     * coefficient = std::make_shared<Table<2, VectorizedArray<double> > >();
      * {
      *   FEEvaluation<dim,fe_degree,n_q_points_1d,1,double> fe_eval(mf_data);
      *   const unsigned int n_cells = mf_data.n_macro_cells();
@@ -705,7 +763,7 @@ namespace MatrixFreeOperators
      * of scope in user code and the clear() command or destructor of this class
      * will delete the table.
      */
-    void set_coefficient(const std_cxx11::shared_ptr<Table<2, VectorizedArray<value_type> > > &scalar_coefficient );
+    void set_coefficient(const std::shared_ptr<Table<2, VectorizedArray<value_type> > > &scalar_coefficient );
 
     virtual void clear();
 
@@ -715,7 +773,7 @@ namespace MatrixFreeOperators
      * The function will throw an error if coefficients are not previously set
      * by set_coefficient() function.
      */
-    std_cxx11::shared_ptr< Table<2, VectorizedArray<value_type> > > get_coefficient();
+    std::shared_ptr< Table<2, VectorizedArray<value_type> > > get_coefficient();
 
   private:
     /**
@@ -751,7 +809,7 @@ namespace MatrixFreeOperators
     /**
      * User-provided heterogeneity coefficient.
      */
-    std_cxx11::shared_ptr< Table<2, VectorizedArray<value_type> > > scalar_coefficient;
+    std::shared_ptr< Table<2, VectorizedArray<value_type> > > scalar_coefficient;
   };
 
 
@@ -768,7 +826,7 @@ namespace MatrixFreeOperators
     FullMatrix<double> shapes_1d(fe_degree+1, fe_degree+1);
     for (unsigned int i=0, c=0; i<shapes_1d.m(); ++i)
       for (unsigned int j=0; j<shapes_1d.n(); ++j, ++c)
-        shapes_1d(i,j) = fe_eval.get_shape_info().shape_values_number[c];
+        shapes_1d(i,j) = fe_eval.get_shape_info().shape_values[c][0];
     shapes_1d.gauss_jordan();
     const unsigned int stride = (fe_degree+2)/2;
     inverse_shape.resize(stride*(fe_degree+1));
@@ -793,7 +851,7 @@ namespace MatrixFreeOperators
   CellwiseInverseMassMatrix<dim,fe_degree,n_components,Number>
   ::fill_inverse_JxW_values(AlignedVector<VectorizedArray<Number> > &inverse_jxw) const
   {
-    const unsigned int dofs_per_cell = Utilities::fixed_int_power<fe_degree+1,dim>::value;
+    constexpr unsigned int dofs_per_cell = Utilities::pow(fe_degree + 1, dim);
     Assert(inverse_jxw.size() > 0 &&
            inverse_jxw.size() % dofs_per_cell == 0,
            ExcMessage("Expected diagonal to be a multiple of scalar dof per cells"));
@@ -825,7 +883,7 @@ namespace MatrixFreeOperators
           const VectorizedArray<Number> *in_array,
           VectorizedArray<Number>       *out_array) const
   {
-    const unsigned int dofs_per_cell = Utilities::fixed_int_power<fe_degree+1,dim>::value;
+    constexpr unsigned int dofs_per_cell = Utilities::pow(fe_degree + 1, dim);
     Assert(inverse_coefficients.size() > 0 &&
            inverse_coefficients.size() % dofs_per_cell == 0,
            ExcMessage("Expected diagonal to be a multiple of scalar dof per cells"));
@@ -834,7 +892,7 @@ namespace MatrixFreeOperators
 
     Assert(dim == 2 || dim == 3, ExcNotImplemented());
 
-    internal::EvaluatorTensorProduct<internal::evaluate_evenodd,dim,fe_degree,
+    internal::EvaluatorTensorProduct<internal::evaluate_evenodd,dim,fe_degree+1,
              fe_degree+1, VectorizedArray<Number> >
              evaluator(inverse_shape, inverse_shape, inverse_shape);
 
@@ -870,23 +928,12 @@ namespace MatrixFreeOperators
   }
 
   //----------------- Base operator -----------------------------
-  template <int dim, typename VectorType>
-  Base<dim,VectorType>::~Base ()
-  {
-  }
-
-
-
   template <int dim, typename VectorType>
   Base<dim,VectorType>::Base ()
     :
     Subscriptor(),
     have_interface_matrices(false)
-  {
-    // boost-1.62.0 doesn't allow initializing a shared_ptr
-    // with NULL. Make sure the default constructor does that.
-    Assert(data.get() == NULL, ExcInternalError());
-  }
+  {}
 
 
 
@@ -894,7 +941,7 @@ namespace MatrixFreeOperators
   typename Base<dim,VectorType>::size_type
   Base<dim,VectorType>::m () const
   {
-    Assert(data.get() != NULL,
+    Assert(data.get() != nullptr,
            ExcNotInitialized());
     typename Base<dim, VectorType>::size_type total_size = 0;
     for (unsigned int i=0; i<selected_rows.size(); ++i)
@@ -908,7 +955,7 @@ namespace MatrixFreeOperators
   typename Base<dim,VectorType>::size_type
   Base<dim,VectorType>::n () const
   {
-    Assert(data.get() != NULL,
+    Assert(data.get() != nullptr,
            ExcNotInitialized());
     typename Base<dim, VectorType>::size_type total_size = 0;
     for (unsigned int i=0; i<selected_columns.size(); ++i)
@@ -935,7 +982,7 @@ namespace MatrixFreeOperators
   {
     (void) col;
     Assert (row == col, ExcNotImplemented());
-    Assert (inverse_diagonal_entries.get() != NULL &&
+    Assert (inverse_diagonal_entries.get() != nullptr &&
             inverse_diagonal_entries->m() > 0, ExcNotInitialized());
     return 1.0/(*inverse_diagonal_entries)(row,row);
   }
@@ -946,7 +993,7 @@ namespace MatrixFreeOperators
   void
   Base<dim,VectorType>::initialize_dof_vector (VectorType &vec) const
   {
-    Assert(data.get() != NULL,
+    Assert(data.get() != nullptr,
            ExcNotInitialized());
     AssertDimension(n_blocks(vec), selected_rows.size());
     for (unsigned int i = 0; i < n_blocks(vec); ++i)
@@ -954,7 +1001,8 @@ namespace MatrixFreeOperators
         const unsigned int index = selected_rows[i];
         if (!subblock(vec,index).partitioners_are_compatible
             (*data->get_dof_info(index).vector_partitioner))
-          data->initialize_dof_vector(subblock(vec,index));
+          data->initialize_dof_vector(subblock(vec,index),index);
+
         Assert(subblock(vec,index).partitioners_are_globally_compatible
                (*data->get_dof_info(index).vector_partitioner),
                ExcInternalError());
@@ -967,7 +1015,7 @@ namespace MatrixFreeOperators
   template <int dim, typename VectorType>
   void
   Base<dim,VectorType>::
-  initialize (std_cxx11::shared_ptr<const MatrixFree<dim,typename Base<dim,VectorType>::value_type> > data_,
+  initialize (std::shared_ptr<const MatrixFree<dim,typename Base<dim,VectorType>::value_type> > data_,
               const std::vector<unsigned int> &given_row_selection,
               const std::vector<unsigned int> &given_column_selection)
   {
@@ -1019,7 +1067,7 @@ namespace MatrixFreeOperators
   template <int dim, typename VectorType>
   void
   Base<dim,VectorType>::
-  initialize (std_cxx11::shared_ptr<const MatrixFree<dim,typename Base<dim,VectorType>::value_type> > data_,
+  initialize (std::shared_ptr<const MatrixFree<dim,typename Base<dim,VectorType>::value_type> > data_,
               const MGConstrainedDoFs         &mg_constrained_dofs,
               const unsigned int               level,
               const std::vector<unsigned int> &given_row_selection)
@@ -1033,7 +1081,7 @@ namespace MatrixFreeOperators
   template <int dim, typename VectorType>
   void
   Base<dim, VectorType>::
-  initialize (std_cxx11::shared_ptr<const MatrixFree<dim, value_type> > data_,
+  initialize (std::shared_ptr<const MatrixFree<dim, value_type> > data_,
               const std::vector<MGConstrainedDoFs> &mg_constrained_dofs,
               const unsigned int                    level,
               const std::vector<unsigned int>      &given_row_selection)
@@ -1169,14 +1217,10 @@ namespace MatrixFreeOperators
 
         // copy the vector content to a temporary vector so that it does not get
         // lost
-        VectorView<Number> view_src_in(subblock(src,i).local_size(),
-                                       subblock(src,i).begin());
-        const Vector<Number> copy_vec = view_src_in;
+        LinearAlgebra::distributed::Vector<Number> copy_vec(subblock(src,i));
         subblock(const_cast<VectorType &>(src),i).
         reinit(data->get_dof_info(mf_component).vector_partitioner);
-        VectorView<Number> view_src_out(subblock(src,i).local_size(),
-                                        subblock(src,i).begin());
-        static_cast<Vector<Number>&>(view_src_out) = copy_vec;
+        subblock(const_cast<VectorType &>(src),i).copy_locally_owned_data_from(copy_vec);
       }
   }
 
@@ -1184,13 +1228,9 @@ namespace MatrixFreeOperators
 
   template <int dim, typename VectorType>
   void
-  Base<dim,VectorType>::mult_add (VectorType &dst,
-                                  const VectorType &src,
-                                  const bool transpose) const
+  Base<dim,VectorType>::preprocess_constraints (VectorType &dst,
+                                                const VectorType &src) const
   {
-    AssertDimension(dst.size(), src.size());
-    AssertDimension(n_blocks(dst), n_blocks(src));
-    AssertDimension(n_blocks(dst), selected_rows.size());
     typedef typename Base<dim,VectorType>::value_type Number;
     adjust_ghost_range_if_necessary(src, false);
     adjust_ghost_range_if_necessary(dst, true);
@@ -1208,12 +1248,34 @@ namespace MatrixFreeOperators
             subblock(const_cast<VectorType &>(src),j).local_element(edge_constrained_indices[j][i]) = 0.;
           }
       }
+  }
 
+
+
+  template <int dim, typename VectorType>
+  void
+  Base<dim,VectorType>::mult_add (VectorType &dst,
+                                  const VectorType &src,
+                                  const bool transpose) const
+  {
+    AssertDimension(dst.size(), src.size());
+    AssertDimension(n_blocks(dst), n_blocks(src));
+    AssertDimension(n_blocks(dst), selected_rows.size());
+    preprocess_constraints (dst,src);
     if (transpose)
       Tapply_add(dst,src);
     else
       apply_add(dst,src);
+    postprocess_constraints(dst,src);
+  }
 
+
+
+  template <int dim, typename VectorType>
+  void
+  Base<dim,VectorType>::postprocess_constraints (VectorType &dst,
+                                                 const VectorType &src) const
+  {
     for (unsigned int j=0; j<n_blocks(dst); ++j)
       {
         const std::vector<unsigned int> &
@@ -1286,7 +1348,7 @@ namespace MatrixFreeOperators
             .local_element(edge_constrained_indices[j][i])
               = edge_constrained_values[j][i].first;
           }
-        for ( ; c<dst.local_size(); ++c)
+        for ( ; c<subblock(dst,j).local_size(); ++c)
           subblock(dst,j).local_element(c) = 0.;
       }
   }
@@ -1348,13 +1410,14 @@ namespace MatrixFreeOperators
   std::size_t
   Base<dim,VectorType>::memory_consumption () const
   {
-    return inverse_diagonal_entries.get() != NULL ? inverse_diagonal_entries->memory_consumption() : sizeof(*this);
+    return inverse_diagonal_entries.get() != nullptr ?
+           inverse_diagonal_entries->memory_consumption() : sizeof(*this);
   }
 
 
 
   template <int dim, typename VectorType>
-  std_cxx11::shared_ptr<const MatrixFree<dim,typename Base<dim,VectorType>::value_type> >
+  std::shared_ptr<const MatrixFree<dim,typename Base<dim,VectorType>::value_type> >
   Base<dim,VectorType>::get_matrix_free() const
   {
     return data;
@@ -1363,16 +1426,27 @@ namespace MatrixFreeOperators
 
 
   template <int dim, typename VectorType>
-  const std_cxx11::shared_ptr<DiagonalMatrix<VectorType> > &
+  const std::shared_ptr<DiagonalMatrix<VectorType> > &
   Base<dim,VectorType>::get_matrix_diagonal_inverse() const
   {
-    Assert(inverse_diagonal_entries.get() != NULL &&
+    Assert(inverse_diagonal_entries.get() != nullptr &&
            inverse_diagonal_entries->m() > 0, ExcNotInitialized());
     return inverse_diagonal_entries;
   }
 
 
 
+  template <int dim, typename VectorType>
+  const std::shared_ptr<DiagonalMatrix<VectorType> > &
+  Base<dim,VectorType>::get_matrix_diagonal() const
+  {
+    Assert(diagonal_entries.get() != nullptr &&
+           diagonal_entries->m() > 0, ExcNotInitialized());
+    return diagonal_entries;
+  }
+
+
+
   template <int dim, typename VectorType>
   void
   Base<dim,VectorType>::Tapply_add(VectorType &dst,
@@ -1403,7 +1477,7 @@ namespace MatrixFreeOperators
   MGInterfaceOperator<OperatorType>::MGInterfaceOperator ()
     :
     Subscriptor(),
-    mf_base_operator(NULL)
+    mf_base_operator(nullptr)
   {
   }
 
@@ -1413,7 +1487,7 @@ namespace MatrixFreeOperators
   void
   MGInterfaceOperator<OperatorType>::clear ()
   {
-    mf_base_operator = NULL;
+    mf_base_operator = nullptr;
   }
 
 
@@ -1433,15 +1507,13 @@ namespace MatrixFreeOperators
   MGInterfaceOperator<OperatorType>::vmult (VectorType &dst,
                                             const VectorType &src) const
   {
-#ifdef DEAL_II_WITH_CXX11
 #ifndef DEAL_II_MSVC
     static_assert (std::is_same<typename VectorType::value_type,value_type>::value,
                    "The vector type must be based on the same value type as this"
                    "operator");
-#endif
 #endif
 
-    Assert(mf_base_operator != NULL,
+    Assert(mf_base_operator != nullptr,
            ExcNotInitialized());
 
     mf_base_operator->vmult_interface_down(dst, src);
@@ -1455,15 +1527,13 @@ namespace MatrixFreeOperators
   MGInterfaceOperator<OperatorType>::Tvmult (VectorType &dst,
                                              const VectorType &src) const
   {
-#ifdef DEAL_II_WITH_CXX11
 #ifndef DEAL_II_MSVC
     static_assert (std::is_same<typename VectorType::value_type,value_type>::value,
                    "The vector type must be based on the same value type as this"
                    "operator");
-#endif
 #endif
 
-    Assert(mf_base_operator != NULL,
+    Assert(mf_base_operator != nullptr,
            ExcNotInitialized());
 
     mf_base_operator->vmult_interface_up(dst, src);
@@ -1471,6 +1541,19 @@ namespace MatrixFreeOperators
 
 
 
+  template <typename OperatorType>
+  template <typename VectorType>
+  void
+  MGInterfaceOperator<OperatorType>::initialize_dof_vector (VectorType &vec) const
+  {
+    Assert(mf_base_operator != nullptr,
+           ExcNotInitialized());
+
+    mf_base_operator->initialize_dof_vector(vec);
+  }
+
+
+
   //-----------------------------MassOperator----------------------------------
 
   template <int dim, int fe_degree, int n_q_points_1d, int n_components, typename VectorType>
@@ -1488,18 +1571,21 @@ namespace MatrixFreeOperators
   compute_diagonal()
   {
     typedef typename Base<dim,VectorType>::value_type Number;
-    Assert((Base<dim, VectorType>::data.get() != NULL), ExcNotInitialized());
+    Assert((Base<dim, VectorType>::data.get() != nullptr), ExcNotInitialized());
 
     this->inverse_diagonal_entries.
     reset(new DiagonalMatrix<VectorType>());
+    this->diagonal_entries.
+    reset(new DiagonalMatrix<VectorType>());
     VectorType &inverse_diagonal_vector = this->inverse_diagonal_entries->get_vector();
-    VectorType ones;
-    this->initialize_dof_vector(ones);
+    VectorType &diagonal_vector         = this->diagonal_entries->get_vector();
     this->initialize_dof_vector(inverse_diagonal_vector);
-    ones = Number(1.);
-    apply_add(inverse_diagonal_vector, ones);
+    this->initialize_dof_vector(diagonal_vector);
+    inverse_diagonal_vector = Number(1.);
+    apply_add(diagonal_vector, inverse_diagonal_vector);
 
-    this->set_constrained_entries_to_one(inverse_diagonal_vector);
+    this->set_constrained_entries_to_one(diagonal_vector);
+    inverse_diagonal_vector = diagonal_vector;
 
     const unsigned int local_size = inverse_diagonal_vector.local_size();
     for (unsigned int i=0; i<local_size; ++i)
@@ -1507,6 +1593,7 @@ namespace MatrixFreeOperators
         =1./inverse_diagonal_vector.local_element(i);
 
     inverse_diagonal_vector.update_ghost_values();
+    diagonal_vector.update_ghost_values();
   }
 
 
@@ -1572,7 +1659,7 @@ namespace MatrixFreeOperators
   template <int dim, int fe_degree, int n_q_points_1d, int n_components, typename VectorType>
   void
   LaplaceOperator<dim, fe_degree, n_q_points_1d, n_components, VectorType>::
-  set_coefficient(const std_cxx11::shared_ptr<Table<2, VectorizedArray<typename Base<dim,VectorType>::value_type> > > &scalar_coefficient_ )
+  set_coefficient(const std::shared_ptr<Table<2, VectorizedArray<typename Base<dim,VectorType>::value_type> > > &scalar_coefficient_ )
   {
     scalar_coefficient = scalar_coefficient_;
   }
@@ -1580,7 +1667,7 @@ namespace MatrixFreeOperators
 
 
   template <int dim, int fe_degree, int n_q_points_1d, int n_components, typename VectorType>
-  std_cxx11::shared_ptr< Table<2, VectorizedArray< typename LaplaceOperator<dim, fe_degree, n_q_points_1d, n_components, VectorType>::value_type> > >
+  std::shared_ptr< Table<2, VectorizedArray< typename LaplaceOperator<dim, fe_degree, n_q_points_1d, n_components, VectorType>::value_type> > >
   LaplaceOperator<dim, fe_degree, n_q_points_1d, n_components, VectorType>::
   get_coefficient()
   {
@@ -1597,17 +1684,23 @@ namespace MatrixFreeOperators
   compute_diagonal()
   {
     typedef typename Base<dim,VectorType>::value_type Number;
-    Assert((Base<dim, VectorType>::data.get() != NULL), ExcNotInitialized());
+    Assert((Base<dim, VectorType>::data.get() != nullptr), ExcNotInitialized());
 
     unsigned int dummy = 0;
     this->inverse_diagonal_entries.
     reset(new DiagonalMatrix<VectorType>());
+    this->diagonal_entries.
+    reset(new DiagonalMatrix<VectorType>());
     VectorType &inverse_diagonal_vector = this->inverse_diagonal_entries->get_vector();
+    VectorType &diagonal_vector         = this->diagonal_entries->get_vector();
     this->initialize_dof_vector(inverse_diagonal_vector);
+    this->initialize_dof_vector(diagonal_vector);
 
     this->data->cell_loop (&LaplaceOperator::local_diagonal_cell,
-                           this, inverse_diagonal_vector, dummy);
-    this->set_constrained_entries_to_one(inverse_diagonal_vector);
+                           this, diagonal_vector, dummy);
+    this->set_constrained_entries_to_one(diagonal_vector);
+
+    inverse_diagonal_vector = diagonal_vector;
 
     for (unsigned int i=0; i<inverse_diagonal_vector.local_size(); ++i)
       if (std::abs(inverse_diagonal_vector.local_element(i)) > std::sqrt(std::numeric_limits<Number>::epsilon()))
@@ -1616,6 +1709,7 @@ namespace MatrixFreeOperators
         inverse_diagonal_vector.local_element(i) = 1.;
 
     inverse_diagonal_vector.update_ghost_values();
+    diagonal_vector.update_ghost_values();
   }
 
 
@@ -1632,7 +1726,7 @@ namespace MatrixFreeOperators
 
   namespace
   {
-    template<typename Number>
+    template <typename Number>
     bool
     non_negative(const VectorizedArray<Number> &n)
     {
@@ -1708,17 +1802,18 @@ namespace MatrixFreeOperators
     for (unsigned int cell=cell_range.first; cell<cell_range.second; ++cell)
       {
         phi.reinit (cell);
-        VectorizedArray<Number> local_diagonal_vector[phi.tensor_dofs_per_cell];
-        for (unsigned int i=0; i<phi.dofs_per_cell; ++i)
+        VectorizedArray<Number> local_diagonal_vector[phi.static_dofs_per_cell];
+        for (unsigned int i=0; i<phi.dofs_per_component; ++i)
           {
-            for (unsigned int j=0; j<phi.dofs_per_cell; ++j)
+            for (unsigned int j=0; j<phi.dofs_per_component; ++j)
               phi.begin_dof_values()[j] = VectorizedArray<Number>();
             phi.begin_dof_values()[i] = 1.;
             do_operation_on_cell(phi,cell);
             local_diagonal_vector[i] = phi.begin_dof_values()[i];
           }
-        for (unsigned int i=0; i<phi.tensor_dofs_per_cell; ++i)
-          phi.begin_dof_values()[i] = local_diagonal_vector[i];
+        for (unsigned int i=0; i<phi.dofs_per_component; ++i)
+          for (unsigned int c=0; c<phi.n_components; ++c)
+            phi.begin_dof_values()[i+c*phi.dofs_per_component] = local_diagonal_vector[i];
         phi.distribute_local_to_global (dst);
       }
   }
