@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 2009 - 2016 by the deal.II authors
+// Copyright (C) 2009 - 2017 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -13,8 +13,8 @@
 //
 // ---------------------------------------------------------------------
 
-#ifndef dealii__integration_info_templates_h
-#define dealii__integration_info_templates_h
+#ifndef dealii_integration_info_templates_h
+#define dealii_integration_info_templates_h
 
 
 #include <deal.II/meshworker/dof_info.h>
@@ -26,10 +26,10 @@ DEAL_II_NAMESPACE_OPEN
 
 namespace MeshWorker
 {
-  template<int dim, int sdim>
+  template <int dim, int sdim>
   void
   IntegrationInfo<dim,sdim>::initialize_data(
-    const std_cxx11::shared_ptr<VectorDataBase<dim,sdim> > &data)
+    const std::shared_ptr<VectorDataBase<dim,sdim> > &data)
   {
     global_data = data;
     const unsigned int nqp = fevalv[0]->n_quadrature_points;
@@ -75,7 +75,7 @@ namespace MeshWorker
   }
 
 
-  template<int dim, int sdim>
+  template <int dim, int sdim>
   void
   IntegrationInfo<dim,sdim>::clear()
   {
@@ -84,7 +84,7 @@ namespace MeshWorker
 
 
 
-  template<int dim, int sdim>
+  template <int dim, int sdim>
   template <typename number>
   void
   IntegrationInfo<dim,sdim>::fill_local_data(const DoFInfo<dim, sdim, number> &info, bool split_fevalues)
@@ -124,7 +124,7 @@ namespace MeshWorker
   }
 
 
-  template<int dim, int sdim>
+  template <int dim, int sdim>
   std::size_t
   IntegrationInfo<dim,sdim>::memory_consumption () const
   {
@@ -138,7 +138,7 @@ namespace MeshWorker
 
 //----------------------------------------------------------------------//
 
-  template<int dim, int sdim>
+  template <int dim, int sdim>
   IntegrationInfoBox<dim,sdim>::IntegrationInfoBox()
   {
     cell_flags = update_default;
@@ -148,7 +148,7 @@ namespace MeshWorker
   }
 
 
-  template<int dim, int sdim>
+  template <int dim, int sdim>
   void
   IntegrationInfoBox<dim,sdim>::initialize_update_flags (bool neighbor_geometry)
   {
@@ -193,7 +193,7 @@ namespace MeshWorker
   }
 
 
-  template<int dim, int sdim>
+  template <int dim, int sdim>
   std::size_t
   IntegrationInfoBox<dim,sdim>::memory_consumption () const
   {
