@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 2006 - 2016 by the deal.II authors
+// Copyright (C) 2006 - 2018 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -14,16 +14,17 @@
 // ---------------------------------------------------------------------
 
 
-#ifndef dealii__mesh_worker_local_results_h
-#define dealii__mesh_worker_local_results_h
+#ifndef dealii_mesh_worker_local_results_h
+#define dealii_mesh_worker_local_results_h
 
 #include <deal.II/base/config.h>
-#include <deal.II/base/std_cxx11/function.h>
 #include <deal.II/base/geometry_info.h>
 #include <deal.II/lac/matrix_block.h>
 #include <deal.II/lac/block_vector.h>
 #include <deal.II/meshworker/vector_selector.h>
 
+#include <functional>
+
 DEAL_II_NAMESPACE_OPEN
 
 class BlockIndices;
@@ -39,7 +40,7 @@ class BlockIndices;
  * integration_loop() instead.
  *
  * The loop() depends on certain objects handed to it as arguments. These
- * objects are of two types, info objects like DoFInfo and IntegrationInfo and
+ * objects are of two types, @p info objects like DoFInfo and IntegrationInfo and
  * worker objects like LocalWorker and IntegrationWorker.
  *
  * Worker objects usually do two different jobs: first, they compute the local
@@ -197,7 +198,7 @@ namespace MeshWorker
    * with matrix(), using the second argument <tt>true</tt>.
    * </ol>
    *
-   * The local matrices initialized by reinit() of the info object and then
+   * The local matrices initialized by reinit() of the @p info object and then
    * assembled into the global system by Assembler classes.
    *
    * @ingroup MeshWorker
@@ -356,14 +357,6 @@ namespace MeshWorker
     std::size_t memory_consumption () const;
 
   private:
-    /**
-     * Initialize a single local matrix block. A helper function for
-     * initialize()
-     */
-    void initialize_local(MatrixBlock<FullMatrix<number> > &M,
-                          const unsigned int row,
-                          const unsigned int col);
-
     /**
      * The local numbers, computed on a cell or on a face.
      */
