@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 2010 - 2015 by the deal.II authors
+// Copyright (C) 2010 - 2018 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -14,8 +14,8 @@
 // ---------------------------------------------------------------------
 
 
-#ifndef dealii__mesh_worker_output_h
-#define dealii__mesh_worker_output_h
+#ifndef dealii_mesh_worker_output_h
+#define dealii_mesh_worker_output_h
 
 #include <deal.II/meshworker/dof_info.h>
 #include <deal.II/base/smartpointer.h>
@@ -84,7 +84,7 @@ namespace MeshWorker
        * Initialize the local data in the DoFInfo object used later for
        * assembling.
        *
-       * The info object refers to a cell if <code>!face</code>, or else to an
+       * The @p info object refers to a cell if <code>!face</code>, or else to an
        * interior or boundary face.
        */
       template <int dim>
@@ -93,13 +93,13 @@ namespace MeshWorker
       /**
        * Write the patch to the output stream.
        */
-      template<int dim>
+      template <int dim>
       void assemble(const DoFInfo<dim> &info);
 
       /**
        * @warning Not implemented yet
        */
-      template<int dim>
+      template <int dim>
       void assemble(const DoFInfo<dim> &info1,
                     const DoFInfo<dim> &info2);
 
@@ -108,7 +108,7 @@ namespace MeshWorker
        * Write the object T either to the stream #os, if initialize_stream()
        * has been called, or to @p deallog if no pointer has been set.
        */
-      template<typename T>
+      template <typename T>
       void write(const T &t) const;
 
       /**
@@ -139,7 +139,7 @@ namespace MeshWorker
     inline void
     GnuplotPatch::write(const T &d) const
     {
-      if (os == 0)
+      if (os == nullptr)
         deallog << d;
       else
         (*os) << d;
@@ -149,7 +149,7 @@ namespace MeshWorker
     inline void
     GnuplotPatch::write_endl() const
     {
-      if (os == 0)
+      if (os == nullptr)
         deallog << std::endl;
       else
         (*os) << std::endl;
@@ -159,7 +159,9 @@ namespace MeshWorker
     inline
     GnuplotPatch::GnuplotPatch()
       :
-      os(0)
+      n_vectors(numbers::invalid_unsigned_int),
+      n_points(numbers::invalid_unsigned_int),
+      os(nullptr)
     {}
 
 
