@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 2010 - 2016 by the deal.II authors
+// Copyright (C) 2010 - 2018 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -14,8 +14,8 @@
 // ---------------------------------------------------------------------
 
 
-#ifndef dealii__mesh_worker_simple_h
-#define dealii__mesh_worker_simple_h
+#ifndef dealii_mesh_worker_simple_h
+#define dealii_mesh_worker_simple_h
 
 #include <deal.II/algorithms/any_data.h>
 #include <deal.II/base/smartpointer.h>
@@ -86,7 +86,7 @@ namespace MeshWorker
        * Initialize the local data in the DoFInfo object used later for
        * assembling.
        *
-       * The info object refers to a cell if <code>!face</code>, or else to an
+       * The @p info object refers to a cell if <code>!face</code>, or else to an
        * interior or boundary face.
        */
       template <class DOFINFO>
@@ -104,7 +104,7 @@ namespace MeshWorker
       /**
        * Assemble both local residuals into the global residuals.
        */
-      template<class DOFINFO>
+      template <class DOFINFO>
       void assemble(const DOFINFO &info1,
                     const DOFINFO &info2);
     protected:
@@ -184,7 +184,7 @@ namespace MeshWorker
        * Initialize the local data in the DoFInfo object used later for
        * assembling.
        *
-       * The info object refers to a cell if <code>!face</code>, or else to an
+       * The @p info object refers to a cell if <code>!face</code>, or else to an
        * interior or boundary face.
        */
       template <class DOFINFO>
@@ -194,14 +194,14 @@ namespace MeshWorker
        * Assemble the local matrices associated with a single cell into the
        * global matrix.
        */
-      template<class DOFINFO>
+      template <class DOFINFO>
       void assemble(const DOFINFO &info);
 
       /**
        * Assemble all local matrices associated with an interior face in the
-       * info objects into the global matrix.
+       * @p info1 and @p info2 objects into the global matrix.
        */
-      template<class DOFINFO>
+      template <class DOFINFO>
       void assemble(const DOFINFO &info1,
                     const DOFINFO &info2);
     protected:
@@ -282,7 +282,7 @@ namespace MeshWorker
        * Initialize the local data in the DoFInfo object used later for
        * assembling.
        *
-       * The info object refers to a cell if <code>!face</code>, or else to an
+       * The @p info object refers to a cell if <code>!face</code>, or else to an
        * interior or boundary face.
        */
       template <class DOFINFO>
@@ -291,14 +291,14 @@ namespace MeshWorker
       /**
        * Assemble the matrix DoFInfo::M1[0] into the global matrix.
        */
-      template<class DOFINFO>
+      template <class DOFINFO>
       void assemble(const DOFINFO &info);
 
       /**
-       * Assemble both local matrices in the info objects into the global
-       * matrices.
+       * Assemble both local matrices in the @p info1 and @p info2
+       * objects into the global matrices.
        */
-      template<class DOFINFO>
+      template <class DOFINFO>
       void assemble(const DOFINFO &info1,
                     const DOFINFO &info2);
     private:
@@ -440,7 +440,7 @@ namespace MeshWorker
        * Initialize the local data in the DoFInfo object used later for
        * assembling.
        *
-       * The info object refers to a cell if <code>!face</code>, or else to an
+       * The @p info object refers to a cell if <code>!face</code>, or else to an
        * interior or boundary face.
        */
       template <class DOFINFO>
@@ -449,14 +449,14 @@ namespace MeshWorker
       /**
        * Assemble the matrix DoFInfo::M1[0] into the global matrix.
        */
-      template<class DOFINFO>
+      template <class DOFINFO>
       void assemble(const DOFINFO &info);
 
       /**
-       * Assemble both local matrices in the info objects into the global
-       * matrix.
+       * Assemble both local matrices in the @p info1 and @p info2
+       * objects into the global matrix.
        */
-      template<class DOFINFO>
+      template <class DOFINFO>
       void assemble(const DOFINFO &info1,
                     const DOFINFO &info2);
 
@@ -524,7 +524,7 @@ namespace MeshWorker
                                                                  info.indices:
                                                                  info.indices_by_block[i];
 
-              if (constraints !=0)
+              if (constraints != nullptr)
                 constraints->distribute_local_to_global(info.vector(k).block(i), ldi, *v);
               else
                 v->add(ldi, info.vector(k).block(i));
@@ -622,7 +622,7 @@ namespace MeshWorker
       AssertDimension(M.m(), i1.size());
       AssertDimension(M.n(), i2.size());
 
-      if (constraints == 0)
+      if (constraints == nullptr)
         {
           for (unsigned int j=0; j<i1.size(); ++j)
             for (unsigned int k=0; k<i2.size(); ++k)
@@ -805,7 +805,7 @@ namespace MeshWorker
       AssertDimension(M.m(), i1.size());
       AssertDimension(M.n(), i2.size());
 
-      if (mg_constrained_dofs == 0)
+      if (mg_constrained_dofs == nullptr)
         {
           for (unsigned int j=0; j<i1.size(); ++j)
             for (unsigned int k=0; k<i2.size(); ++k)
@@ -858,7 +858,7 @@ namespace MeshWorker
       AssertDimension(M.n(), i1.size());
       AssertDimension(M.m(), i2.size());
 
-      if (mg_constrained_dofs == 0)
+      if (mg_constrained_dofs == nullptr)
         {
           for (unsigned int j=0; j<i1.size(); ++j)
             for (unsigned int k=0; k<i2.size(); ++k)
@@ -887,7 +887,7 @@ namespace MeshWorker
       AssertDimension(M.m(), i1.size());
       AssertDimension(M.n(), i2.size());
 
-      if (mg_constrained_dofs == 0)
+      if (mg_constrained_dofs == nullptr)
         {
           for (unsigned int j=0; j<i1.size(); ++j)
             for (unsigned int k=0; k<i2.size(); ++k)
@@ -915,7 +915,7 @@ namespace MeshWorker
     {
       AssertDimension(M.m(), i1.size());
       AssertDimension(M.n(), i2.size());
-      Assert(mg_constrained_dofs != 0, ExcInternalError());
+      Assert(mg_constrained_dofs != nullptr, ExcInternalError());
 
       for (unsigned int j=0; j<i1.size(); ++j)
         for (unsigned int k=0; k<i2.size(); ++k)
@@ -956,7 +956,7 @@ namespace MeshWorker
     {
       AssertDimension(M.n(), i1.size());
       AssertDimension(M.m(), i2.size());
-      Assert(mg_constrained_dofs != 0, ExcInternalError());
+      Assert(mg_constrained_dofs != nullptr, ExcInternalError());
 
       for (unsigned int j=0; j<i1.size(); ++j)
         for (unsigned int k=0; k<i2.size(); ++k)
@@ -987,7 +987,7 @@ namespace MeshWorker
         {
           assemble((*matrix)[level], info.matrix(0,false).matrix,
                    info.indices, info.indices, level);
-          if (mg_constrained_dofs != 0)
+          if (mg_constrained_dofs != nullptr)
             {
               assemble_in((*interface_in)[level], info.matrix(0,false).matrix,
                           info.indices, info.indices, level);
@@ -1004,7 +1004,7 @@ namespace MeshWorker
             assemble((*matrix)[level], info.matrix(k,false).matrix,
                      info.indices_by_block[row], info.indices_by_block[column], level);
 
-            if (mg_constrained_dofs != 0)
+            if (mg_constrained_dofs != nullptr)
               {
                 assemble_in((*interface_in)[level], info.matrix(k,false).matrix,
                             info.indices_by_block[row], info.indices_by_block[column], level);
@@ -1130,7 +1130,7 @@ namespace MeshWorker
       AnyData residuals = ResidualSimple<VectorType>::residuals;
       VectorType *v = residuals.entry<VectorType *>(index);
 
-      if (ResidualSimple<VectorType>::constraints == 0)
+      if (ResidualSimple<VectorType>::constraints == nullptr)
         {
           for (unsigned int i=0; i<indices.size(); ++i)
             (*v)(indices[i]) += vector(i);
@@ -1160,7 +1160,7 @@ namespace MeshWorker
       AnyData residuals = ResidualSimple<VectorType>::residuals;
       VectorType *v = residuals.entry<VectorType *>(index);
 
-      if (ResidualSimple<VectorType>::constraints == 0)
+      if (ResidualSimple<VectorType>::constraints == nullptr)
         {
           for (unsigned int j=0; j<i1.size(); ++j)
             for (unsigned int k=0; k<i2.size(); ++k)
