@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 2005 - 2016 by the deal.II authors
+// Copyright (C) 2005 - 2018 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -13,8 +13,8 @@
 //
 // ---------------------------------------------------------------------
 
-#ifndef dealii__mg_tools_h
-#define dealii__mg_tools_h
+#ifndef dealii_mg_tools_h
+#define dealii_mg_tools_h
 
 #include <deal.II/base/config.h>
 #include <deal.II/base/index_set.h>
@@ -28,6 +28,7 @@
 DEAL_II_NAMESPACE_OPEN
 
 template <int dim, int spacedim> class DoFHandler;
+class MGConstrainedDoFs;
 
 /* !@addtogroup mg */
 /* @{ */
@@ -137,6 +138,20 @@ namespace MGTools
                                    const unsigned int                 level,
                                    const Table<2,DoFTools::Coupling> &flux_mask);
 
+
+  /**
+   * Create sparsity pattern for interface_in/out matrices used in a multigrid computation.
+   * These matrices contain an entry representing the coupling of degrees of
+   * freedom on a refinement edge to those not on the refinement edge of a certain level.
+   */
+  template <typename DoFHandlerType, typename SparsityPatternType>
+  void
+  make_interface_sparsity_pattern (const DoFHandlerType    &dof_handler,
+                                   const MGConstrainedDoFs &mg_constrained_dofs,
+                                   SparsityPatternType     &sparsity,
+                                   const unsigned int      level);
+
+
   /**
    * Count the dofs block-wise on each level.
    *
@@ -232,6 +247,18 @@ namespace MGTools
   void
   extract_non_interface_dofs (const DoFHandler<dim,spacedim> &mg_dof_handler,
                               std::vector<std::set<types::global_dof_index> > &non_interface_dofs);
+
+  /**
+   * Return the highest possible level that can be used as the coarsest level in
+   * a Multigrid computation, that is, the highest level in the hierarchy whose mesh
+   * covers the entire domain. This corresponds to the minimum level of a cell on
+   * the active mesh. Since each processor only has a local view of the mesh, each
+   * processor must call this function. Note that this is a global minimum over the
+   * entire mesh and therefore each processor will return the same value.
+   */
+  template <int dim, int spacedim>
+  unsigned int
+  max_level_for_coarse_mesh (const Triangulation<dim,spacedim> &tria);
 }
 
 /* @} */
