@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 2001 - 2017 by the deal.II authors
+// Copyright (C) 2001 - 2018 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -13,8 +13,8 @@
 //
 // ---------------------------------------------------------------------
 
-#ifndef dealii__mg_transfer_h
-#define dealii__mg_transfer_h
+#ifndef dealii_mg_transfer_h
+#define dealii_mg_transfer_h
 
 #include <deal.II/base/config.h>
 
@@ -33,7 +33,7 @@
 
 #include <deal.II/dofs/dof_handler.h>
 
-#include <deal.II/base/std_cxx11/shared_ptr.h>
+#include <memory>
 
 
 DEAL_II_NAMESPACE_OPEN
@@ -66,12 +66,20 @@ namespace internal
     template <typename SparsityPatternType, typename DoFHandlerType>
     static void reinit(Matrix &matrix, Sparsity &, int level, const SparsityPatternType &sp, DoFHandlerType &dh)
     {
+      const parallel::Triangulation<DoFHandlerType::dimension,DoFHandlerType::space_dimension> *dist_tria =
+        dynamic_cast<const parallel::Triangulation<DoFHandlerType::dimension,DoFHandlerType::space_dimension>*>
+        (&(dh.get_triangulation()));
+      MPI_Comm communicator = dist_tria != nullptr ?
+                              dist_tria->get_communicator() :
+                              MPI_COMM_SELF;
+
       matrix.reinit(dh.locally_owned_mg_dofs(level+1),
                     dh.locally_owned_mg_dofs(level),
-                    sp, MPI_COMM_WORLD, true);
+                    sp, communicator, true);
     }
 
   };
+
   template <>
   struct MatrixSelector<dealii::TrilinosWrappers::MPI::Vector>
   {
@@ -81,24 +89,42 @@ namespace internal
     template <typename SparsityPatternType, typename DoFHandlerType>
     static void reinit(Matrix &matrix, Sparsity &, int level, const SparsityPatternType &sp, DoFHandlerType &dh)
     {
+      const parallel::Triangulation<DoFHandlerType::dimension,DoFHandlerType::space_dimension> *dist_tria =
+        dynamic_cast<const parallel::Triangulation<DoFHandlerType::dimension,DoFHandlerType::space_dimension>*>
+        (&(dh.get_triangulation()));
+      MPI_Comm communicator = dist_tria != nullptr ?
+                              dist_tria->get_communicator() :
+                              MPI_COMM_SELF;
       matrix.reinit(dh.locally_owned_mg_dofs(level+1),
                     dh.locally_owned_mg_dofs(level),
-                    sp, MPI_COMM_WORLD, true);
+                    sp, communicator, true);
     }
 
   };
 
+#ifdef DEAL_II_WITH_MPI
   template <>
-  struct MatrixSelector<dealii::TrilinosWrappers::Vector>
+  struct MatrixSelector<dealii::LinearAlgebra::EpetraWrappers::Vector>
   {
     typedef ::dealii::TrilinosWrappers::SparsityPattern Sparsity;
     typedef ::dealii::TrilinosWrappers::SparseMatrix Matrix;
 
     template <typename SparsityPatternType, typename DoFHandlerType>
-    static void reinit(Matrix &, Sparsity &, int /*level*/, const SparsityPatternType &, DoFHandlerType &)
+    static void reinit(Matrix &matrix, Sparsity &, int level, const SparsityPatternType &sp, DoFHandlerType &dh)
     {
+      const parallel::Triangulation<DoFHandlerType::dimension,DoFHandlerType::space_dimension> *dist_tria =
+        dynamic_cast<const parallel::Triangulation<DoFHandlerType::dimension,DoFHandlerType::space_dimension>*>
+        (&(dh.get_triangulation()));
+      MPI_Comm communicator = dist_tria != nullptr ?
+                              dist_tria->get_communicator() :
+                              MPI_COMM_SELF;
+      matrix.reinit(dh.locally_owned_mg_dofs(level+1),
+                    dh.locally_owned_mg_dofs(level),
+                    sp, communicator, true);
     }
   };
+#endif
+
 #else
   // ! DEAL_II_WITH_TRILINOS
   template <typename Number>
@@ -147,7 +173,9 @@ public:
 
   /**
    * Transfer from a vector on the global grid to vectors defined on each of
-   * the levels separately, i.a. an @p MGVector.
+   * the levels separately for the active degrees of freedom. In particular,
+   * for a globally refined mesh only the finest level in @p dst is filled as a
+   * plain copy of @p src. All the other level objects are left untouched.
    */
   template <int dim, class InVector, int spacedim>
   void
@@ -295,7 +323,9 @@ public:
 
   /**
    * Transfer from a vector on the global grid to vectors defined on each of
-   * the levels separately, i.a. an @p MGVector.
+   * the levels separately for the active degrees of freedom. In particular, for
+   * a globally refined mesh only the finest level in @p dst is filled as a
+   * plain copy of @p src. All the other level objects are left untouched.
    */
   template <int dim, typename Number2, int spacedim>
   void
@@ -357,6 +387,17 @@ public:
 
 protected:
 
+  /**
+   * Internal function to perform transfer of residuals or solutions
+   * basesd on the flag @p solution_transfer.
+   */
+  template <int dim, typename Number2, int spacedim>
+  void
+  copy_to_mg (const DoFHandler<dim,spacedim>                        &mg_dof,
+              MGLevelObject<LinearAlgebra::distributed::Vector<Number> > &dst,
+              const LinearAlgebra::distributed::Vector<Number2>          &src,
+              const bool solution_transfer) const;
+
   /**
    * Internal function to @p fill copy_indices*. Called by derived classes.
    */
@@ -378,6 +419,13 @@ protected:
   std::vector<std::vector<std::pair<unsigned int, unsigned int> > >
   copy_indices;
 
+
+  /**
+   * Same as above, but used to transfer solution vectors.
+   */
+  std::vector<std::vector<std::pair<unsigned int, unsigned int> > >
+  solution_copy_indices;
+
   /**
    * Additional degrees of freedom for the copy_to_mg() function. These are
    * the ones where the global degree of freedom is locally owned and the
@@ -388,6 +436,12 @@ protected:
   std::vector<std::vector<std::pair<unsigned int, unsigned int> > >
   copy_indices_global_mine;
 
+  /**
+   * Same as above, but used to transfer solution vectors.
+   */
+  std::vector<std::vector<std::pair<unsigned int, unsigned int> > >
+  solution_copy_indices_global_mine;
+
   /**
    * Additional degrees of freedom for the copy_from_mg() function. These are
    * the ones where the level degree of freedom is locally owned and the
@@ -398,6 +452,12 @@ protected:
   std::vector<std::vector<std::pair<unsigned int, unsigned int> > >
   copy_indices_level_mine;
 
+  /**
+   * Same as above, but used to transfer solution vectors.
+   */
+  std::vector<std::vector<std::pair<unsigned int, unsigned int> > >
+  solution_copy_indices_level_mine;
+
   /**
    * Stores whether the copy operation from the global to the level vector is
    * actually a plain copy to the finest level. This means that the grid has
@@ -406,6 +466,14 @@ protected:
    */
   bool perform_plain_copy;
 
+  /**
+   * Stores whether the copy operation from the global to the level vector is
+   * actually a plain copy to the finest level except for a renumbering within
+   * the finest level of the degrees of freedom. This means that the grid has
+   * no adaptive refinement.
+   */
+  bool perform_renumbered_plain_copy;
+
   /**
    * The vector that stores what has been given to the
    * set_component_to_block_map() function.
@@ -424,11 +492,22 @@ protected:
    */
   mutable LinearAlgebra::distributed::Vector<Number> ghosted_global_vector;
 
+  /**
+   * Same as above but used when working with solution vectors.
+   */
+  mutable LinearAlgebra::distributed::Vector<Number> solution_ghosted_global_vector;
+
   /**
    * In the function copy_from_mg, we access all level vectors with certain
    * ghost entries for inserting the result into a global vector.
    */
   mutable MGLevelObject<LinearAlgebra::distributed::Vector<Number> > ghosted_level_vector;
+
+  /**
+   * Same as above but used when working with solution vectors.
+   */
+  mutable MGLevelObject<LinearAlgebra::distributed::Vector<Number> > solution_ghosted_level_vector;
+
 };
 
 
@@ -454,7 +533,7 @@ public:
    * Constructor without constraint matrices. Use this constructor only with
    * discontinuous finite elements or with no local refinement.
    */
-  MGTransferPrebuilt ();
+  MGTransferPrebuilt () = default;
 
   /**
    * Constructor with constraints. Equivalent to the default constructor
@@ -468,13 +547,14 @@ public:
    *
    * @deprecated @p constraints is unused.
    */
+  DEAL_II_DEPRECATED
   MGTransferPrebuilt (const ConstraintMatrix &constraints,
-                      const MGConstrainedDoFs &mg_constrained_dofs) DEAL_II_DEPRECATED;
+                      const MGConstrainedDoFs &mg_constrained_dofs);
 
   /**
    * Destructor.
    */
-  virtual ~MGTransferPrebuilt ();
+  virtual ~MGTransferPrebuilt () = default;
 
   /**
    * Initialize the constraints to be used in build_matrices().
@@ -486,8 +566,9 @@ public:
    *
    * @deprecated @p constraints is unused.
    */
+  DEAL_II_DEPRECATED
   void initialize_constraints (const ConstraintMatrix &constraints,
-                               const MGConstrainedDoFs &mg_constrained_dofs) DEAL_II_DEPRECATED;
+                               const MGConstrainedDoFs &mg_constrained_dofs);
 
   /**
    * Reset the object to the state it had right after the default constructor.
@@ -559,14 +640,14 @@ private:
   /**
    * Sparsity patterns for transfer matrices.
    */
-  std::vector<std_cxx11::shared_ptr<typename internal::MatrixSelector<VectorType>::Sparsity> > prolongation_sparsities;
+  std::vector<std::shared_ptr<typename internal::MatrixSelector<VectorType>::Sparsity> > prolongation_sparsities;
 
   /**
    * The actual prolongation matrix.  column indices belong to the dof indices
    * of the mother cell, i.e. the coarse level.  while row indices belong to
    * the child cell, i.e. the fine level.
    */
-  std::vector<std_cxx11::shared_ptr<typename internal::MatrixSelector<VectorType>::Matrix> > prolongation_matrices;
+  std::vector<std::shared_ptr<typename internal::MatrixSelector<VectorType>::Matrix> > prolongation_matrices;
 
   /**
    * Degrees of freedom on the refinement edge excluding those on the
