@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 2003 - 2016 by the deal.II authors
+// Copyright (C) 2003 - 2017 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -14,10 +14,11 @@
 // ---------------------------------------------------------------------
 
 
-#ifndef dealii__mg_transfer_templates_h
-#define dealii__mg_transfer_templates_h
+#ifndef dealii_mg_transfer_templates_h
+#define dealii_mg_transfer_templates_h
 
 #include <deal.II/lac/trilinos_vector.h>
+#include <deal.II/lac/trilinos_epetra_vector.h>
 #include <deal.II/lac/sparse_matrix.h>
 #include <deal.II/lac/vector_view.h>
 #include <deal.II/grid/tria_iterator.h>
@@ -119,7 +120,7 @@ namespace
       {
         if (v[level].size() != mg_dof.locally_owned_mg_dofs(level).size() ||
             v[level].local_size() != mg_dof.locally_owned_mg_dofs(level).n_elements())
-          v[level].reinit(mg_dof.locally_owned_mg_dofs(level), tria != 0 ?
+          v[level].reinit(mg_dof.locally_owned_mg_dofs(level), tria != nullptr ?
                           tria->get_communicator() : MPI_COMM_SELF);
         else
           v[level] = 0.;
@@ -139,20 +140,16 @@ namespace
                  const std::vector<unsigned int> &,
                  MGLevelObject<TrilinosWrappers::MPI::Vector> &v)
   {
-    const dealii::parallel::distributed::Triangulation<dim,spacedim> *tria =
-      (dynamic_cast<const parallel::distributed::Triangulation<dim,spacedim>*>
+    const dealii::parallel::Triangulation<dim,spacedim> *tria =
+      (dynamic_cast<const parallel::Triangulation<dim,spacedim>*>
        (&mg_dof.get_triangulation()));
-    AssertThrow(tria!=NULL, ExcMessage("multigrid with Trilinos vectors only works with distributed Triangulation!"));
+    AssertThrow(tria!=nullptr, ExcMessage("multigrid with Trilinos vectors only works with a parallel Triangulation!"));
 
-#ifdef DEAL_II_WITH_P4EST
     for (unsigned int level=v.min_level();
          level<=v.max_level(); ++level)
       {
         v[level].reinit(mg_dof.locally_owned_mg_dofs(level), tria->get_communicator());
       }
-#else
-    (void)v;
-#endif
   }
 #endif
 }
@@ -368,18 +365,46 @@ MGLevelGlobalTransfer<LinearAlgebra::distributed::Vector<Number> >::copy_to_mg
  MGLevelObject<LinearAlgebra::distributed::Vector<Number> > &dst,
  const LinearAlgebra::distributed::Vector<Number2>          &src) const
 {
+  copy_to_mg (mg_dof_handler,
+              dst,
+              src,
+              false);
+}
+
+
+template <typename Number>
+template <int dim, typename Number2, int spacedim>
+void
+MGLevelGlobalTransfer<LinearAlgebra::distributed::Vector<Number> >::copy_to_mg
+(const DoFHandler<dim,spacedim>                             &mg_dof_handler,
+ MGLevelObject<LinearAlgebra::distributed::Vector<Number> > &dst,
+ const LinearAlgebra::distributed::Vector<Number2>          &src,
+ const bool solution_transfer) const
+{
+  LinearAlgebra::distributed::Vector<Number> &this_ghosted_global_vector =
+    solution_transfer ? solution_ghosted_global_vector : ghosted_global_vector;
+  const std::vector<std::vector<std::pair<unsigned int, unsigned int> > > &this_copy_indices =
+    solution_transfer ? solution_copy_indices : copy_indices;
+  const std::vector<std::vector<std::pair<unsigned int, unsigned int> > > &this_copy_indices_level_mine =
+    solution_transfer ? solution_copy_indices_level_mine : copy_indices_level_mine;
+
   AssertIndexRange(dst.max_level(), mg_dof_handler.get_triangulation().n_global_levels());
   AssertIndexRange(dst.min_level(), dst.max_level()+1);
   reinit_vector(mg_dof_handler, component_to_block_map, dst);
 
   if (perform_plain_copy)
     {
-      // In this case, we can simply copy the local range (in parallel by
-      // VectorView)
+      // In this case, we can simply copy the local range.
       AssertDimension(dst[dst.max_level()].local_size(), src.local_size());
-      VectorView<Number>  dst_view (src.local_size(), dst[dst.max_level()].begin());
-      VectorView<Number2> src_view (src.local_size(), src.begin());
-      static_cast<Vector<Number> &>(dst_view) = static_cast<Vector<Number2> &>(src_view);
+      dst[dst.max_level()].copy_locally_owned_data_from(src);
+      return;
+    }
+  else if (perform_renumbered_plain_copy)
+    {
+      Assert(copy_indices_level_mine.back().empty(), ExcInternalError());
+      LinearAlgebra::distributed::Vector<Number> &dst_level = dst[dst.max_level()];
+      for (std::pair<unsigned int,unsigned int> i : this_copy_indices.back())
+        dst_level.local_element(i.second) = src.local_element(i.first);
       return;
     }
 
@@ -389,8 +414,8 @@ MGLevelGlobalTransfer<LinearAlgebra::distributed::Vector<Number> >::copy_to_mg
 
   // copy the source vector to the temporary vector that we hold for the
   // purpose of data exchange
-  ghosted_global_vector = src;
-  ghosted_global_vector.update_ghost_values();
+  this_ghosted_global_vector = src;
+  this_ghosted_global_vector.update_ghost_values();
 
   for (unsigned int level=dst.max_level()+1; level != dst.min_level();)
     {
@@ -400,15 +425,15 @@ MGLevelGlobalTransfer<LinearAlgebra::distributed::Vector<Number> >::copy_to_mg
       LinearAlgebra::distributed::Vector<Number> &dst_level = dst[level];
 
       // first copy local unknowns
-      for (dof_pair_iterator i = copy_indices[level].begin();
-           i != copy_indices[level].end(); ++i)
-        dst_level.local_element(i->second) = ghosted_global_vector.local_element(i->first);
+      for (dof_pair_iterator i = this_copy_indices[level].begin();
+           i != this_copy_indices[level].end(); ++i)
+        dst_level.local_element(i->second) = this_ghosted_global_vector.local_element(i->first);
 
       // Do the same for the indices where the level index is local, but the
       // global index is not
-      for (dof_pair_iterator i = copy_indices_level_mine[level].begin();
-           i != copy_indices_level_mine[level].end(); ++i)
-        dst_level.local_element(i->second) = ghosted_global_vector.local_element(i->first);
+      for (dof_pair_iterator i = this_copy_indices_level_mine[level].begin();
+           i != this_copy_indices_level_mine[level].end(); ++i)
+        dst_level.local_element(i->second) = this_ghosted_global_vector.local_element(i->first);
 
       dst_level.compress(VectorOperation::insert);
     }
@@ -420,7 +445,7 @@ template <typename Number>
 template <int dim, typename Number2, int spacedim>
 void
 MGLevelGlobalTransfer<LinearAlgebra::distributed::Vector<Number> >::copy_from_mg
-(const DoFHandler<dim,spacedim>                              &mg_dof_handler,
+(const DoFHandler<dim,spacedim>                                   &mg_dof_handler,
  LinearAlgebra::distributed::Vector<Number2>                      &dst,
  const MGLevelObject<LinearAlgebra::distributed::Vector<Number> > &src) const
 {
@@ -429,14 +454,21 @@ MGLevelGlobalTransfer<LinearAlgebra::distributed::Vector<Number> >::copy_from_mg
   AssertIndexRange(src.min_level(), src.max_level()+1);
   if (perform_plain_copy)
     {
-      // In this case, we can simply copy the local range (in parallel by
-      // VectorView). To avoid having stray data in ghost entries of the
-      // destination, make sure to clear them here.
+      // In this case, we can simply copy the local range. To avoid having
+      // stray data in ghost entries of the destination, make sure to clear
+      // them here.
       dst.zero_out_ghosts();
-      AssertDimension(dst.local_size(), src[src.max_level()].local_size());
-      VectorView<Number2> dst_view (dst.local_size(), dst.begin());
-      VectorView<Number>  src_view (dst.local_size(), src[src.max_level()].begin());
-      static_cast<Vector<Number2> &>(dst_view) = static_cast<Vector<Number> &>(src_view);
+      AssertDimension(src[src.max_level()].local_size(), dst.local_size());
+      dst.copy_locally_owned_data_from(src[src.max_level()]);
+      return;
+    }
+  else if (perform_renumbered_plain_copy)
+    {
+      Assert(copy_indices_global_mine.back().empty(), ExcInternalError());
+      const LinearAlgebra::distributed::Vector<Number> &src_level = src[src.max_level()];
+      dst.zero_out_ghosts();
+      for (std::pair<unsigned int,unsigned int> i : copy_indices.back())
+        dst.local_element(i.first) = src_level.local_element(i.second);
       return;
     }
 
@@ -481,7 +513,7 @@ template <typename Number>
 template <int dim, typename Number2, int spacedim>
 void
 MGLevelGlobalTransfer<LinearAlgebra::distributed::Vector<Number> >::copy_from_mg_add
-(const DoFHandler<dim,spacedim>                              &/*mg_dof_handler*/,
+(const DoFHandler<dim,spacedim>                                   &/*mg_dof_handler*/,
  LinearAlgebra::distributed::Vector<Number2>                      &dst,
  const MGLevelObject<LinearAlgebra::distributed::Vector<Number> > &src) const
 {
