@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 1999 - 2016 by the deal.II authors
+// Copyright (C) 1999 - 2018 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -13,8 +13,8 @@
 //
 // ---------------------------------------------------------------------
 
-#ifndef dealii__multigrid_h
-#define dealii__multigrid_h
+#ifndef dealii_multigrid_h
+#define dealii_multigrid_h
 
 
 #include <deal.II/base/config.h>
@@ -34,14 +34,73 @@ DEAL_II_NAMESPACE_OPEN
 /*!@addtogroup mg */
 /*@{*/
 
+namespace mg
+{
+  /**
+   * A structure containing boost::signal objects for optional processing in multigrid solvers.
+   *
+   * Each of these signals is called twice, once before and once after
+   * the action is performed. The two function calls differ in the
+   * boolean argument @p before, which is true the first time and
+   * false the second.
+   */
+  struct Signals
+  {
+    /**
+     * This signal is triggered before (@p before is true) and after (@p before is
+     * false) the call to MGTransfer::copy_to_mg which transfers the vector
+     * given to it to a multi-level vector.
+     */
+    boost::signals2::signal<void (const bool before)> transfer_to_mg;
+
+    /**
+     * This signal is triggered before (@p before is true) and after (@p before is
+     * false) the call to MGTransfer::copy_from_mg which transfers the
+     * multi-level vector given to it to a normal vector.
+     */
+    boost::signals2::signal<void (const bool before)> transfer_to_global;
+
+    /**
+     * This signal is triggered before (@p before is true) and after (@p before is
+     * false) the call to the coarse solver on @p level.
+     */
+    boost::signals2::signal<void (const bool before, const unsigned int level)> coarse_solve;
+
+    /**
+     * This signal is triggered before (@p before is true) and after (@p before is
+     * false) the call to MGTransfer::restrict_and_add() which restricts a
+     * vector from @p level to the next coarser one (@p level - 1).
+     */
+    boost::signals2::signal<void (const bool before, const unsigned int level)> restriction;
+
+    /**
+     * This signal is triggered before (@p before is true) and after (@p before is
+     * false) the call to MGTransfer::prolongate() which prolongs a vector to
+     * @p level from the next coarser one (@p level - 1).
+     */
+    boost::signals2::signal<void (const bool before, const unsigned int level)> prolongation;
+
+    /**
+     * This signal is triggered before (@p before is true) and after (@p before is
+     * false) the call to a pre-smoothing step via MGPreSmoother::apply() on
+     * @p level.
+     */
+    boost::signals2::signal<void (const bool before, const unsigned int level)> pre_smoother_step;
+
+    /**
+     * This signal is triggered before (@p before is true) and after (@p before is
+     * false) the call to a post-smoothing step via MGPostSmoother::apply() on
+     * @p level.
+     */
+    boost::signals2::signal<void (const bool before, const unsigned int level)> post_smoother_step;
+  };
+}
+
 /**
- * Implementation of the multigrid method.
- *
- * @warning multigrid on locally refined meshes only works with
- * <b>discontinuous finite elements</b> right now. It is not clear, whether
- * the paradigm of local smoothing we use is applicable to continuous elements
- * with hanging nodes; in fact, most people you meet on conferences seem to
- * deny this.
+ * Implementation of the multigrid method. The implementation supports both
+ * continuous and discontinuous elements and follows the procedure described in
+ * the
+ * @ref mg_paper "multigrid paper by Janssen and Kanschat".
  *
  * The function which starts a multigrid cycle on the finest level is cycle().
  * Depending on the cycle type chosen with the constructor (see enum Cycle),
@@ -97,6 +156,9 @@ public:
    * The DoFHandler is actually not needed.
    */
   template <int dim>
+#ifndef _MSC_VER
+  DEAL_II_DEPRECATED
+#endif
   Multigrid(const DoFHandler<dim>              &mg_dof_handler,
             const MGMatrixBase<VectorType>     &matrix,
             const MGCoarseGridBase<VectorType> &coarse,
@@ -105,7 +167,7 @@ public:
             const MGSmootherBase<VectorType>   &post_smooth,
             const unsigned int                 minlevel = 0,
             const unsigned int                 maxlevel = numbers::invalid_unsigned_int,
-            Cycle                              cycle = v_cycle) DEAL_II_DEPRECATED;
+            Cycle                              cycle = v_cycle);
 
   /**
    * Constructor. <tt>transfer</tt> is an object performing prolongation and
@@ -220,13 +282,51 @@ public:
   void set_cycle(Cycle);
 
   /**
+   * @deprecated Debug output will go away. Use signals instead.
+   *
    * Set the debug level. Higher values will create more debugging output
    * during the multigrid cycles.
    */
+  DEAL_II_DEPRECATED
   void set_debug (const unsigned int);
 
+  /**
+   * Connect a function to mg::Signals::coarse_solve.
+   */
+  boost::signals2::connection
+  connect_coarse_solve(const std::function<void (const bool, const unsigned int)> &slot);
+
+  /**
+   * Connect a function to mg::Signals::restriction.
+   */
+  boost::signals2::connection
+  connect_restriction(const std::function<void (const bool, const unsigned int)> &slot);
+
+  /**
+   * Connect a function to mg::Signals::prolongation.
+   */
+  boost::signals2::connection
+  connect_prolongation(const std::function<void (const bool, const unsigned int)> &slot);
+
+  /**
+   * Connect a function to mg::Signals::pre_smoother_step.
+   */
+  boost::signals2::connection
+  connect_pre_smoother_step(const std::function<void (const bool, const unsigned int)> &slot);
+
+  /**
+   * Connect a function to mg::Signals::post_smoother_step.
+   */
+  boost::signals2::connection
+  connect_post_smoother_step(const std::function<void (const bool, const unsigned int)> &slot);
+
 private:
 
+  /**
+   * Signals for the various actions that the Multigrid algorithm uses.
+    */
+  mg::Signals signals;
+
   /**
    * The V-cycle multigrid method. <tt>level</tt> is the level the function
    * starts on. It will usually be called for the highest level from outside,
@@ -294,7 +394,7 @@ private:
   SmartPointer<const MGCoarseGridBase<VectorType>,Multigrid<VectorType> > coarse;
 
   /**
-   * Object for grid tranfer.
+   * Object for grid transfer.
    */
   SmartPointer<const MGTransferBase<VectorType>,Multigrid<VectorType> > transfer;
 
@@ -342,7 +442,7 @@ private:
    */
   unsigned int debug;
 
-  template<int dim, class OtherVectorType, class TRANSFER> friend class PreconditionMG;
+  template <int dim, class OtherVectorType, class TRANSFER> friend class PreconditionMG;
 };
 
 
@@ -356,9 +456,13 @@ private:
  * <tt>void copy_from_mg(VectorType&)</tt> to store the result of the v-cycle
  * in @p dst.
  *
- * @author Guido Kanschat, 1999, 2000, 2001, 2002
+ * If VectorType is in fact a block vector and the TRANSFER object supports
+ * use of a separate DoFHandler for each block, this class also allows
+ * to be initialized with a separate DoFHandler for each block.
+ *
+ * @author Guido Kanschat, Daniel Arndt, 1999, 2000, 2001, 2002, 2017
  */
-template<int dim, typename VectorType, class TRANSFER>
+template <int dim, typename VectorType, class TRANSFER>
 class PreconditionMG : public Subscriptor
 {
 public:
@@ -370,6 +474,14 @@ public:
                  Multigrid<VectorType> &mg,
                  const TRANSFER        &transfer);
 
+  /**
+   * Same as above in case every component of a block vector
+   * uses its own DoFHandler.
+   */
+  PreconditionMG(const std::vector<const DoFHandler<dim>*> &dof_handler,
+                 Multigrid<VectorType> &mg,
+                 const TRANSFER        &transfer);
+
   /**
    * Dummy function needed by other classes.
    */
@@ -381,7 +493,7 @@ public:
    *
    * This is the operator used by LAC iterative solvers.
    */
-  template<class OtherVectorType>
+  template <class OtherVectorType>
   void vmult (OtherVectorType       &dst,
               const OtherVectorType &src) const;
 
@@ -389,48 +501,72 @@ public:
    * Preconditioning operator. Calls the @p vcycle function of the @p MG
    * object passed to the constructor.
    */
-  template<class OtherVectorType>
+  template <class OtherVectorType>
   void vmult_add (OtherVectorType       &dst,
                   const OtherVectorType &src) const;
 
   /**
-   * Tranposed preconditioning operator.
+   * Transposed preconditioning operator.
    *
    * Not implemented, but the definition may be needed.
    */
-  template<class OtherVectorType>
+  template <class OtherVectorType>
   void Tvmult (OtherVectorType       &dst,
                const OtherVectorType &src) const;
 
   /**
-   * Tranposed preconditioning operator.
+   * Transposed preconditioning operator.
    *
    * Not implemented, but the definition may be needed.
    */
-  template<class OtherVectorType>
+  template <class OtherVectorType>
   void Tvmult_add (OtherVectorType       &dst,
                    const OtherVectorType &src) const;
 
   /**
-   * Return the partitioning of the range space of this preconditioner, i.e., the partitioning of the vectors that are result from matrix-vector products.
+   * Return the partitioning of the range space of this preconditioner, i.e.,
+   * the partitioning of the vectors that are result from matrix-vector products.
+   * By default, the respective information for the first DoFHandler object
+   * are returned.
    */
-  IndexSet locally_owned_range_indices() const;
+  IndexSet locally_owned_range_indices(const unsigned int block=0) const;
 
   /**
-   * Return the partitioning of the domain space of this preconditioner, i.e., the partitioning of the vectors this matrix has to be multiplied with.
+   * Return the partitioning of the domain space of this preconditioner, i.e.,
+   * the partitioning of the vectors this matrix has to be multiplied with.
+   * By default, the respective information for the first DoFHandler object
+   * are returned.
    */
-  IndexSet locally_owned_domain_indices() const;
+  IndexSet locally_owned_domain_indices(const unsigned int block=0) const;
 
   /**
    * Return the MPI communicator object in use with this preconditioner.
    */
   MPI_Comm get_mpi_communicator() const;
 
+  /**
+   * Connect a function to mg::Signals::transfer_to_mg.
+   */
+  boost::signals2::connection
+  connect_transfer_to_mg(const std::function<void (bool)> &slot);
+
+  /**
+   * Connect a function to mg::Signals::transfer_to_global.
+   */
+  boost::signals2::connection
+  connect_transfer_to_global(const std::function<void (bool)> &slot);
+
 private:
   /**
    * Associated @p DoFHandler.
    */
-  SmartPointer<const DoFHandler<dim>,PreconditionMG<dim,VectorType,TRANSFER> > dof_handler;
+  std::vector<SmartPointer<const DoFHandler<dim>,PreconditionMG<dim,VectorType,TRANSFER> > > dof_handler_vector;
+
+  /**
+   * Storage for the pointers to the DoFHandler objects
+   * without SmartPointer wrapper.
+   */
+  std::vector<const DoFHandler<dim>*> dof_handler_vector_raw;
 
   /**
    * The multigrid object.
@@ -438,9 +574,20 @@ private:
   SmartPointer<Multigrid<VectorType>,PreconditionMG<dim,VectorType,TRANSFER> > multigrid;
 
   /**
-   * Object for grid tranfer.
+   * Object for grid transfer.
    */
   SmartPointer<const TRANSFER,PreconditionMG<dim,VectorType,TRANSFER> > transfer;
+
+  /**
+   * Flag to indicate if the object is initialized with a single DoFHandler
+   * or with one for each block.
+   */
+  const bool uses_dof_handler_vector;
+
+  /**
+   * Signals used by this object
+   */
+  mg::Signals signals;
 };
 
 /*@}*/
@@ -468,8 +615,8 @@ Multigrid<VectorType>::Multigrid (const DoFHandler<dim>              &mg_dof_han
   transfer(&transfer, typeid(*this).name()),
   pre_smooth(&pre_smooth, typeid(*this).name()),
   post_smooth(&post_smooth, typeid(*this).name()),
-  edge_down(0, typeid(*this).name()),
-  edge_up(0, typeid(*this).name()),
+  edge_down(nullptr, typeid(*this).name()),
+  edge_up(nullptr, typeid(*this).name()),
   debug(0)
 {
   const unsigned int dof_handler_max_level
@@ -500,10 +647,10 @@ Multigrid<VectorType>::Multigrid (const MGMatrixBase<VectorType>        &matrix,
   transfer(&transfer, typeid(*this).name()),
   pre_smooth(&pre_smooth, typeid(*this).name()),
   post_smooth(&post_smooth, typeid(*this).name()),
-  edge_out(0, typeid(*this).name()),
-  edge_in(0, typeid(*this).name()),
-  edge_down(0, typeid(*this).name()),
-  edge_up(0, typeid(*this).name()),
+  edge_out(nullptr, typeid(*this).name()),
+  edge_in(nullptr, typeid(*this).name()),
+  edge_down(nullptr, typeid(*this).name()),
+  edge_up(nullptr, typeid(*this).name()),
   debug(0)
 {
   if (max_level == numbers::invalid_unsigned_int)
@@ -537,90 +684,259 @@ Multigrid<VectorType>::get_minlevel () const
 /* --------------------------- inline functions --------------------- */
 
 
-template<int dim, typename VectorType, class TRANSFER>
+namespace internal
+{
+  namespace PreconditionMGImplementation
+  {
+    template <int dim, typename VectorType, class TRANSFER, typename OtherVectorType>
+    typename std::enable_if<TRANSFER::supports_dof_handler_vector>::type
+    vmult(const std::vector<const dealii::DoFHandler<dim>*> &dof_handler_vector,
+          dealii::Multigrid<VectorType>  &multigrid,
+          const TRANSFER &transfer,
+          OtherVectorType       &dst,
+          const OtherVectorType &src,
+          const bool uses_dof_handler_vector,
+          const typename dealii::mg::Signals &signals, int)
+    {
+      signals.transfer_to_mg(true);
+      if (uses_dof_handler_vector)
+        transfer.copy_to_mg(dof_handler_vector,
+                            multigrid.defect,
+                            src);
+      else
+        transfer.copy_to_mg(*dof_handler_vector[0],
+                            multigrid.defect,
+                            src);
+      signals.transfer_to_mg(false);
+
+      multigrid.cycle();
+
+      signals.transfer_to_global(true);
+      if (uses_dof_handler_vector)
+        transfer.copy_from_mg(dof_handler_vector,
+                              dst,
+                              multigrid.solution);
+      else
+        transfer.copy_from_mg(*dof_handler_vector[0],
+                              dst,
+                              multigrid.solution);
+      signals.transfer_to_global(false);
+    }
+
+    template <int dim, typename VectorType, class TRANSFER, typename OtherVectorType>
+    void
+    vmult(const std::vector<const dealii::DoFHandler<dim>*> &dof_handler_vector,
+          dealii::Multigrid<VectorType>  &multigrid,
+          const TRANSFER &transfer,
+          OtherVectorType       &dst,
+          const OtherVectorType &src,
+          const bool uses_dof_handler_vector,
+          const typename dealii::mg::Signals &signals, ...)
+    {
+      (void) uses_dof_handler_vector;
+      Assert (!uses_dof_handler_vector, ExcInternalError());
+
+      signals.transfer_to_mg(true);
+      transfer.copy_to_mg(*dof_handler_vector[0],
+                          multigrid.defect,
+                          src);
+      signals.transfer_to_mg(false);
+
+      multigrid.cycle();
+
+      signals.transfer_to_global(true);
+      transfer.copy_from_mg(*dof_handler_vector[0],
+                            dst,
+                            multigrid.solution);
+      signals.transfer_to_global(false);
+    }
+
+    template <int dim, typename VectorType, class TRANSFER, typename OtherVectorType>
+    typename std::enable_if<TRANSFER::supports_dof_handler_vector>::type
+    vmult_add(const std::vector<const dealii::DoFHandler<dim>*> &dof_handler_vector,
+              dealii::Multigrid<VectorType>  &multigrid,
+              const TRANSFER &transfer,
+              OtherVectorType       &dst,
+              const OtherVectorType &src,
+              const bool uses_dof_handler_vector,
+              const typename dealii::mg::Signals &signals, int)
+    {
+      signals.transfer_to_mg(true);
+      if (uses_dof_handler_vector)
+        transfer.copy_to_mg(dof_handler_vector,
+                            multigrid.defect,
+                            src);
+      else
+        transfer.copy_to_mg(*dof_handler_vector[0],
+                            multigrid.defect,
+                            src);
+      signals.transfer_to_mg(false);
+
+      multigrid.cycle();
+
+      signals.transfer_to_global(true);
+      if (uses_dof_handler_vector)
+        transfer.copy_from_mg_add(dof_handler_vector,
+                                  dst,
+                                  multigrid.solution);
+      else
+        transfer.copy_from_mg_add(*dof_handler_vector[0],
+                                  dst,
+                                  multigrid.solution);
+      signals.transfer_to_global(false);
+    }
+
+    template <int dim, typename VectorType, class TRANSFER, typename OtherVectorType>
+    void
+    vmult_add(const std::vector<const dealii::DoFHandler<dim>*> &dof_handler_vector,
+              dealii::Multigrid<VectorType>  &multigrid,
+              const TRANSFER &transfer,
+              OtherVectorType       &dst,
+              const OtherVectorType &src,
+              const bool uses_dof_handler_vector,
+              const typename dealii::mg::Signals &signals, ...)
+    {
+      (void) uses_dof_handler_vector;
+      Assert (!uses_dof_handler_vector, ExcInternalError());
+
+      signals.transfer_to_mg(true);
+      transfer.copy_to_mg(*dof_handler_vector[0],
+                          multigrid.defect,
+                          src);
+      signals.transfer_to_mg(false);
+
+      multigrid.cycle();
+
+      signals.transfer_to_global(true);
+      transfer.copy_from_mg_add(*dof_handler_vector[0],
+                                dst,
+                                multigrid.solution);
+      signals.transfer_to_global(false);
+    }
+  }
+}
+
+template <int dim, typename VectorType, class TRANSFER>
 PreconditionMG<dim, VectorType, TRANSFER>
 ::PreconditionMG(const DoFHandler<dim>  &dof_handler,
                  Multigrid<VectorType>  &mg,
                  const TRANSFER         &transfer)
   :
-  dof_handler(&dof_handler),
+  dof_handler_vector(1,&dof_handler),
+  dof_handler_vector_raw(1,&dof_handler),
   multigrid(&mg),
-  transfer(&transfer)
+  transfer(&transfer),
+  uses_dof_handler_vector(false)
 {}
 
-template<int dim, typename VectorType, class TRANSFER>
+template <int dim, typename VectorType, class TRANSFER>
+PreconditionMG<dim, VectorType, TRANSFER>
+::PreconditionMG(const std::vector<const DoFHandler<dim>*>  &dof_handler,
+                 Multigrid<VectorType>  &mg,
+                 const TRANSFER         &transfer)
+  :
+  dof_handler_vector(dof_handler.size()),
+  dof_handler_vector_raw(dof_handler.size()),
+  multigrid(&mg),
+  transfer(&transfer),
+  uses_dof_handler_vector(true)
+{
+  for (unsigned int i = 0; i< dof_handler.size() ; ++i)
+    {
+      dof_handler_vector[i] = dof_handler[i];
+      dof_handler_vector_raw[i] = dof_handler[i];
+    }
+}
+
+template <int dim, typename VectorType, class TRANSFER>
 inline bool
 PreconditionMG<dim, VectorType, TRANSFER>::empty () const
 {
   return false;
 }
 
-template<int dim, typename VectorType, class TRANSFER>
-template<class OtherVectorType>
+template <int dim, typename VectorType, class TRANSFER>
+template <class OtherVectorType>
 void
 PreconditionMG<dim, VectorType, TRANSFER>::vmult
 (OtherVectorType       &dst,
  const OtherVectorType &src) const
 {
-  transfer->copy_to_mg(*dof_handler,
-                       multigrid->defect,
-                       src);
-  multigrid->cycle();
-
-  transfer->copy_from_mg(*dof_handler,
-                         dst,
-                         multigrid->solution);
+  internal::PreconditionMGImplementation::vmult(dof_handler_vector_raw,*multigrid,*transfer,
+                                                dst,src,uses_dof_handler_vector,
+                                                this->signals, 0);
 }
 
 
-template<int dim, typename VectorType, class TRANSFER>
+template <int dim, typename VectorType, class TRANSFER>
 IndexSet
-PreconditionMG<dim, VectorType, TRANSFER>::locally_owned_range_indices() const
+PreconditionMG<dim, VectorType, TRANSFER>::locally_owned_range_indices(const unsigned int block) const
 {
-  return dof_handler->locally_owned_dofs();
+  AssertIndexRange(block,dof_handler_vector.size());
+  return dof_handler_vector[block]->locally_owned_dofs();
 }
 
 
-template<int dim, typename VectorType, class TRANSFER>
+template <int dim, typename VectorType, class TRANSFER>
 IndexSet
-PreconditionMG<dim, VectorType, TRANSFER>::locally_owned_domain_indices() const
+PreconditionMG<dim, VectorType, TRANSFER>::locally_owned_domain_indices(const unsigned int block) const
 {
-  return dof_handler->locally_owned_dofs();
+  AssertIndexRange(block,dof_handler_vector.size());
+  return dof_handler_vector[block]->locally_owned_dofs();
 }
 
 
-template<int dim, typename VectorType, class TRANSFER>
+
+template <int dim, typename VectorType, class TRANSFER>
 MPI_Comm
 PreconditionMG<dim, VectorType, TRANSFER>::get_mpi_communicator() const
 {
   // currently parallel GMG works with distributed Triangulation only,
   // so it should be a safe bet to use it to query MPI communicator:
-  const Triangulation<dim> &tria = dof_handler->get_triangulation();
+  const Triangulation<dim> &tria = dof_handler_vector[0]->get_triangulation();
   const parallel::distributed::Triangulation<dim> *ptria = dynamic_cast<const parallel::distributed::Triangulation<dim> *>(&tria);
-  Assert (ptria != NULL, ExcInternalError());
+  Assert (ptria != nullptr, ExcInternalError());
   return ptria->get_communicator ();
 }
 
 
-template<int dim, typename VectorType, class TRANSFER>
-template<class OtherVectorType>
+
+template <int dim, typename VectorType, class TRANSFER>
+boost::signals2::connection
+PreconditionMG<dim, VectorType, TRANSFER>::
+connect_transfer_to_mg(const std::function<void (bool)> &slot)
+{
+  return this->signals.transfer_to_mg.connect(slot);
+}
+
+
+
+template <int dim, typename VectorType, class TRANSFER>
+boost::signals2::connection
+PreconditionMG<dim, VectorType, TRANSFER>::
+connect_transfer_to_global(
+  const std::function<void (bool)> &slot)
+{
+  return this->signals.transfer_to_global.connect(slot);
+}
+
+
+
+template <int dim, typename VectorType, class TRANSFER>
+template <class OtherVectorType>
 void
 PreconditionMG<dim, VectorType, TRANSFER>::vmult_add
 (OtherVectorType       &dst,
  const OtherVectorType &src) const
 {
-  transfer->copy_to_mg(*dof_handler,
-                       multigrid->defect,
-                       src);
-  multigrid->cycle();
-  transfer->copy_from_mg_add(*dof_handler,
-                             dst,
-                             multigrid->solution);
+  internal::PreconditionMGImplementation::vmult_add(dof_handler_vector_raw,*multigrid,*transfer,
+                                                    dst,src,uses_dof_handler_vector,
+                                                    this->signals, 0);
 }
 
 
-template<int dim, typename VectorType, class TRANSFER>
-template<class OtherVectorType>
+template <int dim, typename VectorType, class TRANSFER>
+template <class OtherVectorType>
 void
 PreconditionMG<dim, VectorType, TRANSFER>::Tvmult
 (OtherVectorType &,
@@ -630,8 +946,8 @@ PreconditionMG<dim, VectorType, TRANSFER>::Tvmult
 }
 
 
-template<int dim, typename VectorType, class TRANSFER>
-template<class OtherVectorType>
+template <int dim, typename VectorType, class TRANSFER>
+template <class OtherVectorType>
 void
 PreconditionMG<dim, VectorType, TRANSFER>::Tvmult_add
 (OtherVectorType &,
