@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 1999 - 2016 by the deal.II authors
+// Copyright (C) 1999 - 2018 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -13,8 +13,8 @@
 //
 // ---------------------------------------------------------------------
 
-#ifndef dealii__multigrid_templates_h
-#define dealii__multigrid_templates_h
+#ifndef dealii_multigrid_templates_h
+#define dealii_multigrid_templates_h
 #include <deal.II/multigrid/multigrid.h>
 
 #include <deal.II/base/logstream.h>
@@ -116,16 +116,21 @@ Multigrid<VectorType>::level_v_step (const unsigned int level)
 
   if (level == minlevel)
     {
+      this->signals.coarse_solve(true, level);
       if (debug>0)
         deallog << "Coarse level           " << level << std::endl;
       (*coarse)(level, solution[level], defect[level]);
+      this->signals.coarse_solve(false, level);
       return;
     }
 
   // smoothing of the residual
   if (debug>1)
     deallog << "Smoothing on     level " << level << std::endl;
-  pre_smooth->smooth(level, solution[level], defect[level]);
+
+  this->signals.pre_smoother_step(true, level);
+  pre_smooth->apply(level, solution[level], defect[level]);
+  this->signals.pre_smoother_step(false, level);
 
   if (debug>2)
     deallog << "Solution norm          " << solution[level].l2_norm()
@@ -135,7 +140,7 @@ Multigrid<VectorType>::level_v_step (const unsigned int level)
   if (debug>1)
     deallog << "Residual on      level " << level << std::endl;
   matrix->vmult(level, t[level], solution[level]);
-  if (edge_out != 0)
+  if (edge_out != nullptr)
     {
       edge_out->vmult_add(level, t[level], solution[level]);
       if (debug>2)
@@ -149,29 +154,35 @@ Multigrid<VectorType>::level_v_step (const unsigned int level)
 
   // Get the defect on the next coarser level as part of the (DG) edge matrix
   // and then the main part by the restriction of the transfer
-  if (edge_down != 0)
+  if (edge_down != nullptr)
     {
       edge_down->vmult(level, t[level-1], solution[level]);
       defect[level-1] -= t[level-1];
     }
+
+  this->signals.restriction(true, level);
   transfer->restrict_and_add(level, defect[level-1], t[level]);
+  this->signals.restriction(false, level);
 
   // do recursion
   level_v_step(level-1);
 
   // do coarse grid correction
+  this->signals.prolongation(true, level);
   transfer->prolongate(level, t[level], solution[level-1]);
+  this->signals.prolongation(false, level);
+
   if (debug>2)
     deallog << "Prolongate norm        " << t[level].l2_norm() << std::endl;
   solution[level] += t[level];
 
   // get in contribution from edge matrices to the defect
-  if (edge_in != 0)
+  if (edge_in != nullptr)
     {
       edge_in->Tvmult(level, t[level], solution[level]);
       defect[level] -= t[level];
     }
-  if (edge_up != 0)
+  if (edge_up != nullptr)
     {
       edge_up->Tvmult(level, t[level], solution[level-1]);
       defect[level] -= t[level];
@@ -184,7 +195,9 @@ Multigrid<VectorType>::level_v_step (const unsigned int level)
   // post-smoothing
   if (debug>1)
     deallog << "Smoothing on     level " << level << std::endl;
+  this->signals.post_smoother_step(true, level);
   post_smooth->smooth(level, solution[level], defect[level]);
+  this->signals.post_smoother_step(false, level);
 
   if (debug>2)
     deallog << "Solution norm          " << solution[level].l2_norm()
@@ -223,7 +236,7 @@ Multigrid<VectorType>::level_step(const unsigned int level,
   // Combine the defect from the initial copy_to_mg with the one that has come
   // from the finer level by the transfer
   defect2[level] += defect[level];
-  defect[level] = 0;
+  defect[level] = typename VectorType::value_type(0.);
 
   if (debug>2)
     deallog << cychar << "-cycle defect norm     " << defect2[level].l2_norm()
@@ -241,7 +254,7 @@ Multigrid<VectorType>::level_step(const unsigned int level,
   // smoothing of the residual
   if (debug>1)
     deallog << cychar << "-cycle smoothing level " << level << std::endl;
-  pre_smooth->smooth(level, solution[level], defect2[level]);
+  pre_smooth->apply(level, solution[level], defect2[level]);
 
   if (debug>2)
     deallog << cychar << "-cycle solution norm   "
@@ -251,7 +264,7 @@ Multigrid<VectorType>::level_step(const unsigned int level,
   if (debug>1)
     deallog << cychar << "-cycle residual level  " << level << std::endl;
   matrix->vmult(level, t[level], solution[level]);
-  if (edge_out != 0)
+  if (edge_out != nullptr)
     edge_out->vmult_add(level, t[level], solution[level]);
   t[level].sadd(-1.0, 1.0, defect2[level]);
 
@@ -261,18 +274,14 @@ Multigrid<VectorType>::level_step(const unsigned int level,
 
   // Get the defect on the next coarser level as part of the (DG) edge matrix
   // and then the main part by the restriction of the transfer
-  if (edge_down != 0)
+  if (edge_down != nullptr)
     edge_down->vmult(level, defect2[level-1], solution[level]);
   else
-    defect2[level-1] = 0;
+    defect2[level-1] = typename VectorType::value_type(0.);
 
   transfer->restrict_and_add (level, defect2[level-1], t[level]);
 
-  // Every cycle starts with a recursion of its type. As opposed to the
-  // V-cycle above where we do not need to set the solution vector to zero (as
-  // that has been done in the init functions), we need to zero out the values
-  // from previous cycles.
-  solution[level-1] = 0;
+  // Every cycle starts with a recursion of its type.
   level_step(level-1, cycle);
 
   // For W and F-cycle, repeat the process on the next coarser level except
@@ -293,13 +302,13 @@ Multigrid<VectorType>::level_step(const unsigned int level,
   solution[level] += t[level];
 
   // get in contribution from edge matrices to the defect
-  if (edge_in != 0)
+  if (edge_in != nullptr)
     {
       edge_in->Tvmult(level, t[level], solution[level]);
       defect2[level] -= t[level];
     }
 
-  if (edge_up != 0)
+  if (edge_up != nullptr)
     {
       edge_up->Tvmult(level, t[level], solution[level-1]);
       defect2[level] -= t[level];
@@ -337,8 +346,10 @@ Multigrid<VectorType>::cycle()
 
   for (unsigned int level=minlevel; level<=maxlevel; ++level)
     {
-      solution[level].reinit(defect[level]);
-      t[level].reinit(defect[level]);
+      // the vectors for level>minlevel will be overwritten by the apply()
+      // method of the smoother -> do not force them to be zeroed out here
+      solution[level].reinit(defect[level], level>minlevel);
+      t[level].reinit(defect[level], level>minlevel);
       if (cycle_type != v_cycle)
         defect2[level].reinit(defect[level]);
     }
@@ -362,13 +373,63 @@ Multigrid<VectorType>::vcycle()
 
   for (unsigned int level=minlevel; level<=maxlevel; ++level)
     {
-      solution[level].reinit(defect[level]);
-      t[level].reinit(defect[level]);
+      solution[level].reinit(defect[level], level>minlevel);
+      t[level].reinit(defect[level], level>minlevel);
     }
   level_v_step (maxlevel);
 }
 
 
+
+template <typename VectorType>
+boost::signals2::connection
+Multigrid<VectorType>::
+connect_coarse_solve(const std::function<void (const bool, const unsigned int)> &slot)
+{
+  return this->signals.coarse_solve.connect(slot);
+}
+
+
+
+template <typename VectorType>
+boost::signals2::connection
+Multigrid<VectorType>::
+connect_restriction(const std::function<void (const bool, const unsigned int)> &slot)
+{
+  return this->signals.restriction.connect(slot);
+}
+
+
+
+template <typename VectorType>
+boost::signals2::connection
+Multigrid<VectorType>::
+connect_prolongation(const std::function<void (const bool, const unsigned int)> &slot)
+{
+  return this->signals.prolongation.connect(slot);
+}
+
+
+
+template <typename VectorType>
+boost::signals2::connection
+Multigrid<VectorType>::
+connect_pre_smoother_step(const std::function<void (const bool, const unsigned int)> &slot)
+{
+  return this->signals.pre_smoother_step.connect(slot);
+}
+
+
+
+template <typename VectorType>
+boost::signals2::connection
+Multigrid<VectorType>::
+connect_post_smoother_step(const std::function<void (const bool, const unsigned int)> &slot)
+{
+  return this->signals.post_smoother_step.connect(slot);
+}
+
+
 DEAL_II_NAMESPACE_CLOSE
 
 #endif
