@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 1999 - 2016 by the deal.II authors
+// Copyright (C) 1999 - 2018 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -13,8 +13,8 @@
 //
 // ---------------------------------------------------------------------
 
-#ifndef dealii__data_out_dof_data_h
-#define dealii__data_out_dof_data_h
+#ifndef dealii_data_out_dof_data_h
+#define dealii_data_out_dof_data_h
 
 
 
@@ -31,20 +31,17 @@
 #include <deal.II/numerics/data_postprocessor.h>
 #include <deal.II/numerics/data_component_interpretation.h>
 
-#include <deal.II/base/std_cxx11/shared_ptr.h>
+#include <memory>
 
 DEAL_II_NAMESPACE_OPEN
 
-template <int, int> class FEValuesBase;
-
-
 namespace Exceptions
 {
   /**
    * A namespace for exceptions that are used throughout the DataOut*
    * collection of classes.
    */
-  namespace DataOut
+  namespace DataOutImplementation
   {
     /**
      * Exception
@@ -146,8 +143,37 @@ namespace Exceptions
 
 namespace internal
 {
-  namespace DataOut
+  namespace DataOutImplementation
   {
+    /**
+     * The DataEntry classes abstract away the concrete data type of vectors
+     * users can attach to DataOut (and similar) objects and allow the underlying
+     * DataOut functions to query for individual elements of solution vectors
+     * without having to know the concrete vector type. This avoids that
+     * DataOut has to know what vectors are being used, but it has the downside
+     * that DataOut also doesn't know the underlying scalar type of these vectors.
+     *
+     * If the underlying scalar types all represent real numbers (in the
+     * mathematical sense -- i.e., the scalar type would be @p float,
+     * @p double, etc) then that is not a problem -- DataOut simply
+     * receives the values of individual vector components as @p double
+     * objects. On the other hand, if the vector type uses a std::complex
+     * scalar type, then DataEntry returning a @p double for a vector
+     * entry is not sufficient -- we need to provide DataOut with a way
+     * to query both the real and the imaginary part, so that they can
+     * be written into output files separately.
+     *
+     * This enum allows DataOut to tell a DataEntry function which component
+     * of a vector entry it wants to query, i.e., whether it wants the real
+     * or the imaginary part of a vector entry.
+     */
+    enum class ComponentExtractor
+    {
+      real_part,
+      imaginary_part
+    };
+
+
     /**
      * For each vector that has been added through the add_data_vector()
      * functions, we need to keep track of a pointer to it, and allow data
@@ -185,7 +211,7 @@ namespace internal
       /**
        * Destructor made virtual.
        */
-      virtual ~DataEntryBase ();
+      virtual ~DataEntryBase () = default;
 
       /**
        * Assuming that the stored vector is a cell vector, extract the given
@@ -193,7 +219,8 @@ namespace internal
        */
       virtual
       double
-      get_cell_data_value (const unsigned int cell_number) const = 0;
+      get_cell_data_value (const unsigned int cell_number,
+                           const ComponentExtractor extract_component) const = 0;
 
       /**
        * Given a FEValuesBase object, extract the values on the present cell
@@ -203,6 +230,7 @@ namespace internal
       void
       get_function_values
       (const FEValuesBase<DoFHandlerType::dimension,DoFHandlerType::space_dimension> &fe_patch_values,
+       const ComponentExtractor extract_component,
        std::vector<double> &patch_values) const = 0;
 
       /**
@@ -214,6 +242,7 @@ namespace internal
       void
       get_function_values
       (const FEValuesBase<DoFHandlerType::dimension,DoFHandlerType::space_dimension> &fe_patch_values,
+       const ComponentExtractor extract_component,
        std::vector<dealii::Vector<double> > &patch_values_system) const = 0;
 
       /**
@@ -224,6 +253,7 @@ namespace internal
       void
       get_function_gradients
       (const FEValuesBase<DoFHandlerType::dimension,DoFHandlerType::space_dimension> &fe_patch_values,
+       const ComponentExtractor extract_component,
        std::vector<Tensor<1,DoFHandlerType::space_dimension> > &patch_gradients) const = 0;
 
       /**
@@ -235,6 +265,7 @@ namespace internal
       void
       get_function_gradients
       (const FEValuesBase<DoFHandlerType::dimension,DoFHandlerType::space_dimension> &fe_patch_values,
+       const ComponentExtractor extract_component,
        std::vector<std::vector<Tensor<1,DoFHandlerType::space_dimension> > > &patch_gradients_system) const = 0;
 
       /**
@@ -245,6 +276,7 @@ namespace internal
       void
       get_function_hessians
       (const FEValuesBase<DoFHandlerType::dimension,DoFHandlerType::space_dimension> &fe_patch_values,
+       const ComponentExtractor extract_component,
        std::vector<Tensor<2,DoFHandlerType::space_dimension> > &patch_hessians) const = 0;
 
       /**
@@ -256,8 +288,15 @@ namespace internal
       void
       get_function_hessians
       (const FEValuesBase<DoFHandlerType::dimension,DoFHandlerType::space_dimension> &fe_patch_values,
+       const ComponentExtractor extract_component,
        std::vector<std::vector< Tensor<2,DoFHandlerType::space_dimension> > > &patch_hessians_system) const = 0;
 
+      /**
+       * Return whether the data represented by (a derived class of) this object
+       * represents a complex-valued (as opposed to real-valued) information.
+       */
+      virtual bool is_complex_valued () const = 0;
+
       /**
        * Clear all references to the vectors.
        */
@@ -326,14 +365,14 @@ namespace internal
                         const unsigned int n_subdivisions,
                         const std::vector<unsigned int> &n_postprocessor_outputs,
                         const Mapping<dim,spacedim> &mapping,
-                        const std::vector<std_cxx11::shared_ptr<dealii::hp::FECollection<dim,spacedim> > > &finite_elements,
+                        const std::vector<std::shared_ptr<dealii::hp::FECollection<dim,spacedim> > > &finite_elements,
                         const UpdateFlags update_flags,
                         const bool        use_face_values);
 
       ParallelDataBase (const ParallelDataBase &data);
 
       template <typename DoFHandlerType>
-      void reinit_all_fe_values(std::vector<std_cxx11::shared_ptr<DataEntryBase<DoFHandlerType> > > &dof_data,
+      void reinit_all_fe_values(std::vector<std::shared_ptr<DataEntryBase<DoFHandlerType> > > &dof_data,
                                 const typename dealii::Triangulation<dim,spacedim>::cell_iterator &cell,
                                 const unsigned int face = numbers::invalid_unsigned_int);
 
@@ -350,11 +389,11 @@ namespace internal
       std::vector<std::vector<dealii::Vector<double> > > postprocessed_values;
 
       const dealii::hp::MappingCollection<dim,spacedim> mapping_collection;
-      const std::vector<std_cxx11::shared_ptr<dealii::hp::FECollection<dim,spacedim> > > finite_elements;
+      const std::vector<std::shared_ptr<dealii::hp::FECollection<dim,spacedim> > > finite_elements;
       const UpdateFlags update_flags;
 
-      std::vector<std_cxx11::shared_ptr<dealii::hp::FEValues<dim,spacedim> > > x_fe_values;
-      std::vector<std_cxx11::shared_ptr<dealii::hp::FEFaceValues<dim,spacedim> > > x_fe_face_values;
+      std::vector<std::shared_ptr<dealii::hp::FEValues<dim,spacedim> > > x_fe_values;
+      std::vector<std::shared_ptr<dealii::hp::FEFaceValues<dim,spacedim> > > x_fe_face_values;
     };
   }
 }
@@ -388,8 +427,8 @@ namespace internal
  *   ...   // compute error_estimator, which contains one value per cell
  *
  *   std::vector<std::string> solution_names;
- *   solution_names.push_back ("x-displacement");
- *   solution_names.push_back ("y-displacement");
+ *   solution_names.emplace_back ("x-displacement");
+ *   solution_names.emplace_back ("y-displacement");
  *
  *   DataOut<dim> data_out;
  *   data_out.attach_dof_handler (dof_handler);
@@ -826,12 +865,12 @@ protected:
   /**
    * List of data elements with vectors of values for each degree of freedom.
    */
-  std::vector<std_cxx11::shared_ptr<internal::DataOut::DataEntryBase<DoFHandlerType> > >  dof_data;
+  std::vector<std::shared_ptr<internal::DataOutImplementation::DataEntryBase<DoFHandlerType> > >  dof_data;
 
   /**
    * List of data elements with vectors of values for each cell.
    */
-  std::vector<std_cxx11::shared_ptr<internal::DataOut::DataEntryBase<DoFHandlerType> > >  cell_data;
+  std::vector<std::shared_ptr<internal::DataOutImplementation::DataEntryBase<DoFHandlerType> > >  cell_data;
 
   /**
    * This is a list of patches that is created each time build_patches() is
@@ -858,15 +897,15 @@ protected:
    * Extracts the finite elements stored in the dof_data object, including a
    * dummy object of FE_DGQ<dim>(0) in case only the triangulation is used.
    */
-  std::vector<std_cxx11::shared_ptr<dealii::hp::FECollection<DoFHandlerType::dimension,DoFHandlerType::space_dimension> > >
-  get_finite_elements() const;
+  std::vector<std::shared_ptr<dealii::hp::FECollection<DoFHandlerType::dimension,DoFHandlerType::space_dimension> > >
+  get_fes() const;
 
   /**
    * Overload of the respective DataOutInterface::get_vector_data_ranges()
    * function. See there for a more extensive documentation.
    */
   virtual
-  std::vector<std_cxx11::tuple<unsigned int, unsigned int, std::string> >
+  std::vector<std::tuple<unsigned int, unsigned int, std::string> >
   get_vector_data_ranges () const;
 
   /**
@@ -875,11 +914,99 @@ protected:
    */
   template <class, int, int>
   friend class DataOut_DoFData;
+private:
+  /**
+   * Common function called by the four public add_data_vector methods.
+   */
+  template <class VectorType>
+  void
+  add_data_vector_internal
+  (const DoFHandlerType           *dof_handler,
+   const VectorType               &data,
+   const std::vector<std::string> &names,
+   const DataVectorType            type,
+   const std::vector<DataComponentInterpretation::DataComponentInterpretation> &data_component_interpretation,
+   const bool                      deduce_output_names);
 };
 
 
 
 // -------------------- template and inline functions ------------------------
+template <typename DoFHandlerType, int patch_dim, int patch_space_dim>
+template <typename VectorType>
+void
+DataOut_DoFData<DoFHandlerType,patch_dim,patch_space_dim>::
+add_data_vector
+(const VectorType                         &vec,
+ const std::string                        &name,
+ const DataVectorType                      type,
+ const std::vector<DataComponentInterpretation::DataComponentInterpretation> &data_component_interpretation)
+{
+  Assert (triangulation != nullptr, Exceptions::DataOutImplementation::ExcNoTriangulationSelected ());
+  std::vector<std::string> names(1, name);
+  add_data_vector_internal (dofs, vec, names, type, data_component_interpretation, true);
+}
+
+
+
+template <typename DoFHandlerType, int patch_dim, int patch_space_dim>
+template <typename VectorType>
+void
+DataOut_DoFData<DoFHandlerType,patch_dim,patch_space_dim>::
+add_data_vector
+(const VectorType                         &vec,
+ const std::vector<std::string>           &names,
+ const DataVectorType                      type,
+ const std::vector<DataComponentInterpretation::DataComponentInterpretation> &data_component_interpretation)
+{
+  Assert (triangulation != nullptr, Exceptions::DataOutImplementation::ExcNoTriangulationSelected ());
+  add_data_vector_internal(dofs, vec, names, type, data_component_interpretation, false);
+}
+
+
+
+template <typename DoFHandlerType, int patch_dim, int patch_space_dim>
+template <typename VectorType>
+void
+DataOut_DoFData<DoFHandlerType,patch_dim,patch_space_dim>::
+add_data_vector
+(const DoFHandlerType           &dof_handler,
+ const VectorType               &data,
+ const std::string              &name,
+ const std::vector<DataComponentInterpretation::DataComponentInterpretation> &data_component_interpretation)
+{
+  std::vector<std::string> names(1, name);
+  add_data_vector_internal (&dof_handler, data, names, type_dof_data, data_component_interpretation, true);
+}
+
+
+
+template <typename DoFHandlerType, int patch_dim, int patch_space_dim>
+template <typename VectorType>
+void
+DataOut_DoFData<DoFHandlerType,patch_dim,patch_space_dim>::
+add_data_vector
+(const DoFHandlerType           &dof_handler,
+ const VectorType               &data,
+ const std::vector<std::string> &names,
+ const std::vector<DataComponentInterpretation::DataComponentInterpretation> &data_component_interpretation)
+{
+  add_data_vector_internal(&dof_handler, data, names, type_dof_data, data_component_interpretation, false);
+}
+
+
+
+template <typename DoFHandlerType, int patch_dim, int patch_space_dim>
+template <typename VectorType>
+void
+DataOut_DoFData<DoFHandlerType,patch_dim,patch_space_dim>::
+add_data_vector (const VectorType                       &vec,
+                 const DataPostprocessor<DoFHandlerType::space_dimension> &data_postprocessor)
+{
+  Assert (dofs != nullptr, Exceptions::DataOutImplementation::ExcNoDoFHandlerSelected ());
+  add_data_vector(*dofs, vec, data_postprocessor);
+}
+
 
 
 template <typename DoFHandlerType, int patch_dim, int patch_space_dim>
@@ -889,7 +1016,7 @@ DataOut_DoFData<DoFHandlerType,patch_dim,patch_space_dim>::
 merge_patches (const DataOut_DoFData<DoFHandlerType2,patch_dim,patch_space_dim> &source,
                const Point<patch_space_dim> &shift)
 {
-  const std::vector<Patch> source_patches = source.get_patches ();
+  const std::vector<Patch> &source_patches = source.get_patches ();
   Assert ((patches.size () != 0) &&
           (source_patches.size () != 0),
           ExcMessage ("When calling this function, both the current "
@@ -901,17 +1028,17 @@ merge_patches (const DataOut_DoFData<DoFHandlerType2,patch_dim,patch_space_dim>
   // check equality of component
   // names
   Assert (get_dataset_names() == source.get_dataset_names(),
-          Exceptions::DataOut::ExcIncompatibleDatasetNames());
+          Exceptions::DataOutImplementation::ExcIncompatibleDatasetNames());
   // make sure patches are compatible. we'll
   // assume that if the first respective
   // patches are ok that all the other ones
   // are ok as well
   Assert (patches[0].n_subdivisions == source_patches[0].n_subdivisions,
-          Exceptions::DataOut::ExcIncompatiblePatchLists());
+          Exceptions::DataOutImplementation::ExcIncompatiblePatchLists());
   Assert (patches[0].data.n_rows() == source_patches[0].data.n_rows(),
-          Exceptions::DataOut::ExcIncompatiblePatchLists());
+          Exceptions::DataOutImplementation::ExcIncompatiblePatchLists());
   Assert (patches[0].data.n_cols() == source_patches[0].data.n_cols(),
-          Exceptions::DataOut::ExcIncompatiblePatchLists());
+          Exceptions::DataOutImplementation::ExcIncompatiblePatchLists());
 
   // check equality of the vector data
   // specifications
@@ -921,16 +1048,16 @@ merge_patches (const DataOut_DoFData<DoFHandlerType2,patch_dim,patch_space_dim>
                       "as vectors."));
   for (unsigned int i=0; i<get_vector_data_ranges().size(); ++i)
     {
-      Assert (std_cxx11::get<0>(get_vector_data_ranges()[i]) ==
-              std_cxx11::get<0>(source.get_vector_data_ranges()[i]),
+      Assert (std::get<0>(get_vector_data_ranges()[i]) ==
+              std::get<0>(source.get_vector_data_ranges()[i]),
               ExcMessage ("Both sources need to declare the same components "
                           "as vectors."));
-      Assert (std_cxx11::get<1>(get_vector_data_ranges()[i]) ==
-              std_cxx11::get<1>(source.get_vector_data_ranges()[i]),
+      Assert (std::get<1>(get_vector_data_ranges()[i]) ==
+              std::get<1>(source.get_vector_data_ranges()[i]),
               ExcMessage ("Both sources need to declare the same components "
                           "as vectors."));
-      Assert (std_cxx11::get<2>(get_vector_data_ranges()[i]) ==
-              std_cxx11::get<2>(source.get_vector_data_ranges()[i]),
+      Assert (std::get<2>(get_vector_data_ranges()[i]) ==
+              std::get<2>(source.get_vector_data_ranges()[i]),
               ExcMessage ("Both sources need to declare the same components "
                           "as vectors."));
     }
