@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 2000 - 2016 by the deal.II authors
+// Copyright (C) 2000 - 2018 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -13,8 +13,8 @@
 //
 // ---------------------------------------------------------------------
 
-#ifndef dealii__data_out_rotation_h
-#define dealii__data_out_rotation_h
+#ifndef dealii_data_out_rotation_h
+#define dealii_data_out_rotation_h
 
 
 #include <deal.II/base/config.h>
@@ -28,7 +28,7 @@ DEAL_II_NAMESPACE_OPEN
 
 namespace internal
 {
-  namespace DataOutRotation
+  namespace DataOutRotationImplementation
   {
     /**
      * A derived class for use in the DataOutFaces class. This is a class for
@@ -36,14 +36,14 @@ namespace internal
      * documentation of the WorkStream class.
      */
     template <int dim, int spacedim>
-    struct ParallelData : public internal::DataOut::ParallelDataBase<dim,spacedim>
+    struct ParallelData : public internal::DataOutImplementation::ParallelDataBase<dim,spacedim>
     {
       ParallelData (const unsigned int n_datasets,
                     const unsigned int n_subdivisions,
                     const unsigned int n_patches_per_circle,
                     const std::vector<unsigned int> &n_postprocessor_outputs,
                     const Mapping<dim,spacedim> &mapping,
-                    const std::vector<std_cxx11::shared_ptr<dealii::hp::FECollection<dim,spacedim> > > &finite_elements,
+                    const std::vector<std::shared_ptr<dealii::hp::FECollection<dim,spacedim> > > &finite_elements,
                     const UpdateFlags update_flags);
 
       const unsigned int n_patches_per_circle;
@@ -199,7 +199,7 @@ private:
    */
   void
   build_one_patch (const cell_iterator *cell,
-                   internal::DataOutRotation::ParallelData<dimension, space_dimension> &data,
+                   internal::DataOutRotationImplementation::ParallelData<dimension, space_dimension> &data,
                    std::vector<DataOutBase::Patch<dimension+1,space_dimension+1> > &my_patches);
 };
 
