@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 2000 - 2017 by the deal.II authors
+// Copyright (C) 2000 - 2018 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -13,12 +13,11 @@
 //
 // ---------------------------------------------------------------------
 
-#ifndef dealii__derivative_approximation_h
-#define dealii__derivative_approximation_h
+#ifndef dealii_derivative_approximation_h
+#define dealii_derivative_approximation_h
 
 #include <deal.II/base/config.h>
 #include <deal.II/base/exceptions.h>
-#include <deal.II/base/std_cxx11/tuple.h>
 #include <deal.II/base/synchronous_iterator.h>
 #include <deal.II/fe/fe_update_flags.h>
 #include <deal.II/fe/mapping.h>
@@ -144,7 +143,7 @@ DEAL_II_NAMESPACE_OPEN
  * in the main function @p approximate.
  *
  * Due to this way of operation, the class may be easily extended for higher
- * oder derivatives than are presently implemented. Basically, only an
+ * order derivatives than are presently implemented. Basically, only an
  * additional class along the lines of the derivative descriptor classes @p
  * Gradient and @p SecondDerivative has to be implemented, with the respective
  * typedefs and functions replaced by the appropriate analogues for the
