@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 2000 - 2015 by the deal.II authors
+// Copyright (C) 2000 - 2017 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -13,8 +13,8 @@
 //
 // ---------------------------------------------------------------------
 
-#ifndef dealii__dof_print_solver_step_h
-#define dealii__dof_print_solver_step_h
+#ifndef dealii_dof_print_solver_step_h
+#define dealii_dof_print_solver_step_h
 
 #include <deal.II/base/config.h>
 #include <deal.II/base/logstream.h>
@@ -49,7 +49,7 @@ DEAL_II_NAMESPACE_OPEN
  * @ingroup output
  * @author Guido Kanschat, 2000
  */
-template<int dim, typename SolverType, class VectorType = Vector<double> >
+template <int dim, typename SolverType, class VectorType = Vector<double> >
 class DoFPrintSolverStep : public SolverType
 {
 public:
@@ -87,7 +87,7 @@ private:
 
 /* ----------------------- template functions --------------- */
 
-template<int dim, typename SolverType, class VectorType>
+template <int dim, typename SolverType, class VectorType>
 DoFPrintSolverStep<dim, SolverType, VectorType>::DoFPrintSolverStep
 (SolverControl            &control,
  VectorMemory<VectorType> &mem,
@@ -99,7 +99,7 @@ DoFPrintSolverStep<dim, SolverType, VectorType>::DoFPrintSolverStep
 {}
 
 
-template<int dim, typename SolverType, class VectorType>
+template <int dim, typename SolverType, class VectorType>
 void
 DoFPrintSolverStep<dim, SolverType, VectorType>::print_vectors
 (const unsigned int  step,
