@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 1998 - 2016 by the deal.II authors
+// Copyright (C) 1998 - 2018 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -13,8 +13,8 @@
 //
 // ---------------------------------------------------------------------
 
-#ifndef dealii__error_estimator_h
-#define dealii__error_estimator_h
+#ifndef dealii_error_estimator_h
+#define dealii_error_estimator_h
 
 
 #include <deal.II/base/config.h>
@@ -253,7 +253,7 @@ class KellyErrorEstimator
 public:
   /**
    * The enum type given to the class functions to decide on the scaling
-   * factors of the facial integrals.
+   * factors of the face integrals.
    */
   enum Strategy
   {
@@ -330,11 +330,11 @@ public:
   (const Mapping<dim, spacedim>               &mapping,
    const DoFHandlerType                       &dof,
    const Quadrature<dim-1>                    &quadrature,
-   const typename FunctionMap<spacedim>::type &neumann_bc,
+   const typename FunctionMap<spacedim,typename InputVector::value_type>::type &neumann_bc,
    const InputVector                          &solution,
    Vector<float>                              &error,
    const ComponentMask                        &component_mask = ComponentMask(),
-   const Function<spacedim>                   *coefficients   = 0,
+   const Function<spacedim>                   *coefficients   = nullptr,
    const unsigned int                          n_threads      = numbers::invalid_unsigned_int,
    const types::subdomain_id                   subdomain_id   = numbers::invalid_subdomain_id,
    const types::material_id                    material_id    = numbers::invalid_material_id,
@@ -348,11 +348,11 @@ public:
   static void estimate
   (const DoFHandlerType                       &dof,
    const Quadrature<dim-1>                    &quadrature,
-   const typename FunctionMap<spacedim>::type &neumann_bc,
+   const typename FunctionMap<spacedim,typename InputVector::value_type>::type &neumann_bc,
    const InputVector                          &solution,
    Vector<float>                              &error,
    const ComponentMask                        &component_mask = ComponentMask(),
-   const Function<spacedim>                   *coefficients   = 0,
+   const Function<spacedim>                   *coefficients   = nullptr,
    const unsigned int                          n_threads      = numbers::invalid_unsigned_int,
    const types::subdomain_id                   subdomain_id   = numbers::invalid_subdomain_id,
    const types::material_id                    material_id    = numbers::invalid_material_id,
@@ -376,7 +376,7 @@ public:
   (const Mapping<dim, spacedim>               &mapping,
    const DoFHandlerType                       &dof,
    const Quadrature<dim-1>                    &quadrature,
-   const typename FunctionMap<spacedim>::type &neumann_bc,
+   const typename FunctionMap<spacedim,typename InputVector::value_type>::type &neumann_bc,
    const std::vector<const InputVector *>     &solutions,
    std::vector<Vector<float>*>                &errors,
    const ComponentMask                        &component_mask = ComponentMask(),
@@ -394,7 +394,7 @@ public:
   static void estimate
   (const DoFHandlerType                       &dof,
    const Quadrature<dim-1>                    &quadrature,
-   const typename FunctionMap<spacedim>::type &neumann_bc,
+   const typename FunctionMap<spacedim,typename InputVector::value_type>::type &neumann_bc,
    const std::vector<const InputVector *>     &solutions,
    std::vector<Vector<float>*>                &errors,
    const ComponentMask                        &component_mask = ComponentMask(),
@@ -414,7 +414,7 @@ public:
   (const Mapping<dim, spacedim>               &mapping,
    const DoFHandlerType                       &dof,
    const hp::QCollection<dim-1>               &quadrature,
-   const typename FunctionMap<spacedim>::type &neumann_bc,
+   const typename FunctionMap<spacedim,typename InputVector::value_type>::type &neumann_bc,
    const InputVector                          &solution,
    Vector<float>                              &error,
    const ComponentMask                        &component_mask = ComponentMask(),
@@ -433,11 +433,11 @@ public:
   static void estimate
   (const DoFHandlerType                       &dof,
    const hp::QCollection<dim-1>               &quadrature,
-   const typename FunctionMap<spacedim>::type &neumann_bc,
+   const typename FunctionMap<spacedim,typename InputVector::value_type>::type &neumann_bc,
    const InputVector                          &solution,
    Vector<float>                              &error,
    const ComponentMask                        &component_mask = ComponentMask(),
-   const Function<spacedim>                   *coefficients   = 0,
+   const Function<spacedim>                   *coefficients   = nullptr,
    const unsigned int                          n_threads      = numbers::invalid_unsigned_int,
    const types::subdomain_id                   subdomain_id   = numbers::invalid_subdomain_id,
    const types::material_id                    material_id    = numbers::invalid_material_id,
@@ -453,7 +453,7 @@ public:
   (const Mapping<dim, spacedim>               &mapping,
    const DoFHandlerType                       &dof,
    const hp::QCollection<dim-1>               &quadrature,
-   const typename FunctionMap<spacedim>::type &neumann_bc,
+   const typename FunctionMap<spacedim,typename InputVector::value_type>::type &neumann_bc,
    const std::vector<const InputVector *>     &solutions,
    std::vector<Vector<float>*>                &errors,
    const ComponentMask                        &component_mask = ComponentMask(),
@@ -472,11 +472,11 @@ public:
   static void estimate
   (const DoFHandlerType                       &dof,
    const hp::QCollection<dim-1>               &quadrature,
-   const typename FunctionMap<spacedim>::type &neumann_bc,
+   const typename FunctionMap<spacedim,typename InputVector::value_type>::type &neumann_bc,
    const std::vector<const InputVector *>     &solutions,
    std::vector<Vector<float>*>                &errors,
    const ComponentMask                        &component_mask = ComponentMask(),
-   const Function<spacedim>                   *coefficients   = 0,
+   const Function<spacedim>                   *coefficients   = nullptr,
    const unsigned int                          n_threads      = numbers::invalid_unsigned_int,
    const types::subdomain_id                   subdomain_id   = numbers::invalid_subdomain_id,
    const types::material_id                    material_id    = numbers::invalid_material_id,
@@ -549,6 +549,20 @@ template <int spacedim>
 class KellyErrorEstimator<1,spacedim>
 {
 public:
+  /**
+   * The enum type given to the class functions to decide on the scaling
+   * factors of the face integrals.
+   */
+  enum Strategy
+  {
+    //! Kelly error estimator with the factor $\frac {h_K}{24}$.
+    cell_diameter_over_24 = 0,
+    //! the boundary residual estimator with the factor $\frac {h_F}{2 max(p^+,p^-)}$.
+    face_diameter_over_twice_max_degree,
+    //! Kelly error estimator with the factor $h_K$.
+    cell_diameter
+  };
+
   /**
    * Implementation of the error estimator described above. You may give a
    * coefficient, but there is a default value which denotes the constant
@@ -576,14 +590,15 @@ public:
   (const Mapping<1,spacedim>                  &mapping,
    const DoFHandlerType                       &dof,
    const Quadrature<0>                        &quadrature,
-   const typename FunctionMap<spacedim>::type &neumann_bc,
+   const typename FunctionMap<spacedim,typename InputVector::value_type>::type &neumann_bc,
    const InputVector                          &solution,
    Vector<float>                              &error,
    const ComponentMask                        &component_mask = ComponentMask(),
-   const Function<spacedim>                   *coefficients   = 0,
+   const Function<spacedim>                   *coefficient    = nullptr,
    const unsigned int                          n_threads      = numbers::invalid_unsigned_int,
    const types::subdomain_id                   subdomain_id   = numbers::invalid_subdomain_id,
-   const types::material_id                    material_id    = numbers::invalid_material_id);
+   const types::material_id                    material_id    = numbers::invalid_material_id,
+   const Strategy                              strategy       = cell_diameter_over_24);
 
   /**
    * Call the @p estimate function, see above, with
@@ -593,14 +608,15 @@ public:
   static void estimate
   (const DoFHandlerType                       &dof,
    const Quadrature<0>                        &quadrature,
-   const typename FunctionMap<spacedim>::type &neumann_bc,
+   const typename FunctionMap<spacedim,typename InputVector::value_type>::type &neumann_bc,
    const InputVector                          &solution,
    Vector<float>                              &error,
    const ComponentMask                        &component_mask = ComponentMask(),
-   const Function<spacedim>                   *coefficients   = 0,
+   const Function<spacedim>                   *coefficients   = nullptr,
    const unsigned int                          n_threads      = numbers::invalid_unsigned_int,
    const types::subdomain_id                   subdomain_id   = numbers::invalid_subdomain_id,
-   const types::material_id                    material_id    = numbers::invalid_material_id);
+   const types::material_id                    material_id    = numbers::invalid_material_id,
+   const Strategy                              strategy       = cell_diameter_over_24);
 
   /**
    * Same function as above, but accepts more than one solution vectors and
@@ -620,14 +636,15 @@ public:
   (const Mapping<1,spacedim>                  &mapping,
    const DoFHandlerType                       &dof,
    const Quadrature<0>                        &quadrature,
-   const typename FunctionMap<spacedim>::type &neumann_bc,
+   const typename FunctionMap<spacedim,typename InputVector::value_type>::type &neumann_bc,
    const std::vector<const InputVector *>     &solutions,
    std::vector<Vector<float>*>                &errors,
    const ComponentMask                        &component_mask = ComponentMask(),
    const Function<spacedim>                   *coefficients   = 0,
    const unsigned int                          n_threads      = numbers::invalid_unsigned_int,
    const types::subdomain_id                   subdomain_id   = numbers::invalid_subdomain_id,
-   const types::material_id                    material_id    = numbers::invalid_material_id);
+   const types::material_id                    material_id    = numbers::invalid_material_id,
+   const Strategy                              strategy       = cell_diameter_over_24);
 
   /**
    * Call the @p estimate function, see above, with
@@ -637,14 +654,15 @@ public:
   static void estimate
   (const DoFHandlerType                       &dof,
    const Quadrature<0>                        &quadrature,
-   const typename FunctionMap<spacedim>::type &neumann_bc,
+   const typename FunctionMap<spacedim,typename InputVector::value_type>::type &neumann_bc,
    const std::vector<const InputVector *>     &solutions,
    std::vector<Vector<float>*>                &errors,
    const ComponentMask                        &component_mask = ComponentMask(),
    const Function<spacedim>                   *coefficients   = 0,
    const unsigned int                          n_threads      = numbers::invalid_unsigned_int,
    const types::subdomain_id                   subdomain_id   = numbers::invalid_subdomain_id,
-   const types::material_id                    material_id    = numbers::invalid_material_id);
+   const types::material_id                    material_id    = numbers::invalid_material_id,
+   const Strategy                              strategy       = cell_diameter_over_24);
 
 
   /**
@@ -656,14 +674,15 @@ public:
   (const Mapping<1,spacedim>                  &mapping,
    const DoFHandlerType                       &dof,
    const hp::QCollection<0>                   &quadrature,
-   const typename FunctionMap<spacedim>::type &neumann_bc,
+   const typename FunctionMap<spacedim,typename InputVector::value_type>::type &neumann_bc,
    const InputVector                          &solution,
    Vector<float>                              &error,
    const ComponentMask                        &component_mask = ComponentMask(),
    const Function<spacedim>                   *coefficients   = 0,
    const unsigned int                          n_threads      = numbers::invalid_unsigned_int,
    const types::subdomain_id                   subdomain_id   = numbers::invalid_subdomain_id,
-   const types::material_id                    material_id    = numbers::invalid_material_id);
+   const types::material_id                    material_id    = numbers::invalid_material_id,
+   const Strategy                              strategy       = cell_diameter_over_24);
 
 
   /**
@@ -674,14 +693,15 @@ public:
   static void estimate
   (const DoFHandlerType                       &dof,
    const hp::QCollection<0>                   &quadrature,
-   const typename FunctionMap<spacedim>::type &neumann_bc,
+   const typename FunctionMap<spacedim,typename InputVector::value_type>::type &neumann_bc,
    const InputVector                          &solution,
    Vector<float>                              &error,
    const ComponentMask                        &component_mask = ComponentMask(),
    const Function<spacedim>                   *coefficients   = 0,
    const unsigned int                          n_threads      = numbers::invalid_unsigned_int,
    const types::subdomain_id                   subdomain_id   = numbers::invalid_subdomain_id,
-   const types::material_id                    material_id    = numbers::invalid_material_id);
+   const types::material_id                    material_id    = numbers::invalid_material_id,
+   const Strategy                              strategy       = cell_diameter_over_24);
 
 
   /**
@@ -693,14 +713,15 @@ public:
   (const Mapping<1,spacedim>                  &mapping,
    const DoFHandlerType                       &dof,
    const hp::QCollection<0>                   &quadrature,
-   const typename FunctionMap<spacedim>::type &neumann_bc,
+   const typename FunctionMap<spacedim,typename InputVector::value_type>::type &neumann_bc,
    const std::vector<const InputVector *>     &solutions,
    std::vector<Vector<float>*>                &errors,
    const ComponentMask                        &component_mask = ComponentMask(),
    const Function<spacedim>                   *coefficients   = 0,
    const unsigned int                          n_threads      = numbers::invalid_unsigned_int,
    const types::subdomain_id                   subdomain_id   = numbers::invalid_subdomain_id,
-   const types::material_id                    material_id    = numbers::invalid_material_id);
+   const types::material_id                    material_id    = numbers::invalid_material_id,
+   const Strategy                              strategy       = cell_diameter_over_24);
 
 
   /**
@@ -711,14 +732,15 @@ public:
   static void estimate
   (const DoFHandlerType                       &dof,
    const hp::QCollection<0>                   &quadrature,
-   const typename FunctionMap<spacedim>::type &neumann_bc,
+   const typename FunctionMap<spacedim,typename InputVector::value_type>::type &neumann_bc,
    const std::vector<const InputVector *>     &solutions,
    std::vector<Vector<float>*>                &errors,
    const ComponentMask                        &component_mask = ComponentMask(),
    const Function<spacedim>                   *coefficients   = 0,
    const unsigned int                          n_threads      = numbers::invalid_unsigned_int,
    const types::subdomain_id                   subdomain_id   = numbers::invalid_subdomain_id,
-   const types::material_id                    material_id    = numbers::invalid_material_id);
+   const types::material_id                    material_id    = numbers::invalid_material_id,
+   const Strategy                              strategy       = cell_diameter_over_24);
 
   /**
    * Exception
