@@ -13,8 +13,8 @@
 //
 // ---------------------------------------------------------------------
 
-#ifndef dealii__error_estimator_templates_h
-#define dealii__error_estimator_templates_h
+#ifndef dealii_error_estimator_templates_h
+#define dealii_error_estimator_templates_h
 
 #include <deal.II/base/numbers.h>
 #include <deal.II/base/thread_management.h>
@@ -26,10 +26,10 @@
 #include <deal.II/lac/la_parallel_vector.h>
 #include <deal.II/lac/block_vector.h>
 #include <deal.II/lac/la_parallel_block_vector.h>
-#include <deal.II/lac/petsc_vector.h>
-#include <deal.II/lac/petsc_block_vector.h>
+#include <deal.II/lac/petsc_parallel_vector.h>
+#include <deal.II/lac/petsc_parallel_block_vector.h>
 #include <deal.II/lac/trilinos_vector.h>
-#include <deal.II/lac/trilinos_block_vector.h>
+#include <deal.II/lac/trilinos_parallel_block_vector.h>
 #include <deal.II/grid/tria_iterator.h>
 #include <deal.II/base/geometry_info.h>
 #include <deal.II/dofs/dof_handler.h>
@@ -44,12 +44,12 @@
 #include <deal.II/numerics/error_estimator.h>
 #include <deal.II/distributed/tria.h>
 
-#include <deal.II/base/std_cxx11/bind.h>
 
 #include <numeric>
 #include <algorithm>
 #include <cmath>
 #include <vector>
+#include <functional>
 
 DEAL_II_NAMESPACE_OPEN
 
@@ -69,14 +69,14 @@ namespace internal
      * which we found can take a significant amount of time if it happens
      * often even in the single threaded case (10-20 per cent in our
      * measurements); however, most importantly, memory allocation requires
-     * synchronisation in multithreaded mode. While that is done by the C++
+     * synchronization in multithreaded mode. While that is done by the C++
      * library and has not to be handcoded, it nevertheless seriously damages
      * the ability to efficiently run the functions of this class in parallel,
-     * since they are quite often blocked by these synchronisation points,
+     * since they are quite often blocked by these synchronization points,
      * slowing everything down by a factor of two or three.
      *
      * Thus, every thread gets an instance of this class to work with and
-     * needs not allocate memory itself, or synchronise with other threads.
+     * needs not allocate memory itself, or synchronize with other threads.
      *
      * The sizes of the arrays are initialized with the maximal number of
      * entries necessary for the hp case. Within the loop over individual
@@ -113,7 +113,7 @@ namespace internal
        * points for each of the solution vectors (i.e. a temporary value).
        * This vector is not allocated inside the functions that use it, but
        * rather globally, since memory allocation is slow, in particular in
-       * presence of multiple threads where synchronisation makes things even
+       * presence of multiple threads where synchronization makes things even
        * slower.
        */
       std::vector<std::vector<std::vector<number> > > phi;
@@ -169,9 +169,9 @@ namespace internal
        * Some more references to input data to the
        * KellyErrorEstimator::estimate() function.
        */
-      const typename FunctionMap<spacedim>::type *neumann_bc;
-      const ComponentMask                component_mask;
-      const Function<spacedim>                   *coefficients;
+      const typename FunctionMap<spacedim,number>::type *neumann_bc;
+      const ComponentMask                                component_mask;
+      const Function<spacedim>                          *coefficients;
 
       /**
        * Constructor.
@@ -184,8 +184,8 @@ namespace internal
                     const unsigned int n_solution_vectors,
                     const types::subdomain_id subdomain_id,
                     const types::material_id material_id,
-                    const typename FunctionMap<spacedim>::type *neumann_bc,
-                    const ComponentMask                component_mask,
+                    const typename FunctionMap<spacedim,number>::type *neumann_bc,
+                    const ComponentMask                &component_mask,
                     const Function<spacedim>                   *coefficients);
 
       /**
@@ -208,8 +208,8 @@ namespace internal
      const unsigned int                                  n_solution_vectors,
      const types::subdomain_id                           subdomain_id,
      const types::material_id                            material_id,
-     const typename FunctionMap<spacedim>::type         *neumann_bc,
-     const ComponentMask                                 component_mask,
+     const typename FunctionMap<spacedim,number>::type         *neumann_bc,
+     const ComponentMask                                 &component_mask,
      const Function<spacedim>                           *coefficients)
       :
       finite_element (fe),
@@ -377,7 +377,7 @@ namespace internal
 
       // if a coefficient was given: use that to scale the jump in the
       // gradient
-      if (parallel_data.coefficients != 0)
+      if (parallel_data.coefficients != nullptr)
         {
           // scalar coefficient
           if (parallel_data.coefficients->n_components == 1)
@@ -420,7 +420,7 @@ namespace internal
           // get the values of the boundary function at the quadrature points
           if (n_components == 1)
             {
-              std::vector<double> g(n_q_points);
+              std::vector<number> g(n_q_points);
               parallel_data.neumann_bc->find(boundary_id)->second
               ->value_list (fe_face_values_cell.get_present_fe_values()
                             .get_quadrature_points(), g);
@@ -431,8 +431,8 @@ namespace internal
             }
           else
             {
-              std::vector<dealii::Vector<double> >
-              g(n_q_points, dealii::Vector<double>(n_components));
+              std::vector<dealii::Vector<number> >
+              g(n_q_points, dealii::Vector<number>(n_components));
               parallel_data.neumann_bc->find(boundary_id)->second
               ->vector_value_list (fe_face_values_cell.get_present_fe_values()
                                    .get_quadrature_points(),
@@ -971,7 +971,7 @@ KellyErrorEstimator<dim, spacedim>::
 estimate (const Mapping<dim, spacedim>               &mapping,
           const DoFHandlerType                       &dof_handler,
           const Quadrature<dim-1>                    &quadrature,
-          const typename FunctionMap<spacedim>::type &neumann_bc,
+          const typename FunctionMap<spacedim,typename InputVector::value_type>::type &neumann_bc,
           const InputVector                          &solution,
           Vector<float>                              &error,
           const ComponentMask                        &component_mask,
@@ -995,7 +995,7 @@ void
 KellyErrorEstimator<dim,spacedim>::
 estimate (const DoFHandlerType                       &dof_handler,
           const Quadrature<dim-1>                    &quadrature,
-          const typename FunctionMap<spacedim>::type &neumann_bc,
+          const typename FunctionMap<spacedim,typename InputVector::value_type>::type &neumann_bc,
           const InputVector                          &solution,
           Vector<float>                              &error,
           const ComponentMask                        &component_mask,
@@ -1018,7 +1018,7 @@ KellyErrorEstimator<dim, spacedim>::
 estimate (const Mapping<dim, spacedim>               &mapping,
           const DoFHandlerType                       &dof_handler,
           const hp::QCollection<dim-1>               &quadrature,
-          const typename FunctionMap<spacedim>::type &neumann_bc,
+          const typename FunctionMap<spacedim,typename InputVector::value_type>::type &neumann_bc,
           const InputVector                          &solution,
           Vector<float>                              &error,
           const ComponentMask                        &component_mask,
@@ -1042,7 +1042,7 @@ void
 KellyErrorEstimator<dim, spacedim>::
 estimate (const DoFHandlerType                       &dof_handler,
           const hp::QCollection<dim-1>               &quadrature,
-          const typename FunctionMap<spacedim>::type &neumann_bc,
+          const typename FunctionMap<spacedim,typename InputVector::value_type>::type &neumann_bc,
           const InputVector                          &solution,
           Vector<float>                              &error,
           const ComponentMask                        &component_mask,
@@ -1067,7 +1067,7 @@ KellyErrorEstimator<dim, spacedim>::
 estimate (const Mapping<dim, spacedim>               &mapping,
           const DoFHandlerType                       &dof_handler,
           const hp::QCollection<dim-1>               &face_quadratures,
-          const typename FunctionMap<spacedim>::type &neumann_bc,
+          const typename FunctionMap<spacedim,typename InputVector::value_type>::type &neumann_bc,
           const std::vector<const InputVector *>     &solutions,
           std::vector<Vector<float>*>                &errors,
           const ComponentMask                        &component_mask,
@@ -1080,7 +1080,7 @@ estimate (const Mapping<dim, spacedim>               &mapping,
 #ifdef DEAL_II_WITH_P4EST
   if (dynamic_cast<const parallel::distributed::Triangulation<dim,spacedim>*>
       (&dof_handler.get_triangulation())
-      != 0)
+      != nullptr)
     Assert ((subdomain_id_ == numbers::invalid_subdomain_id)
             ||
             (subdomain_id_ ==
@@ -1093,7 +1093,7 @@ estimate (const Mapping<dim, spacedim>               &mapping,
   const types::subdomain_id subdomain_id
     = ((dynamic_cast<const parallel::distributed::Triangulation<dim,spacedim>*>
         (&dof_handler.get_triangulation())
-        != 0)
+        != nullptr)
        ?
        dynamic_cast<const parallel::distributed::Triangulation<dim,spacedim>&>
        (dof_handler.get_triangulation()).locally_owned_subdomain()
@@ -1104,7 +1104,7 @@ estimate (const Mapping<dim, spacedim>               &mapping,
     = subdomain_id_;
 #endif
 
-  const unsigned int n_components = dof_handler.get_fe().n_components();
+  const unsigned int n_components = dof_handler.get_fe(0).n_components();
   (void)n_components;
 
   // sanity checks
@@ -1113,7 +1113,7 @@ estimate (const Mapping<dim, spacedim>               &mapping,
   Assert (solutions.size() == errors.size(),
           ExcIncompatibleNumberOfElements(solutions.size(), errors.size()));
 
-  for (typename FunctionMap<spacedim>::type::const_iterator i=neumann_bc.begin();
+  for (typename FunctionMap<spacedim,typename InputVector::value_type>::type::const_iterator i=neumann_bc.begin();
        i!=neumann_bc.end(); ++i)
     Assert (i->second->n_components == n_components,
             ExcInvalidBoundaryFunction(i->first,
@@ -1125,7 +1125,7 @@ estimate (const Mapping<dim, spacedim>               &mapping,
   Assert (component_mask.n_selected_components(n_components) > 0,
           ExcInvalidComponentMask());
 
-  Assert ((coefficients == 0) ||
+  Assert ((coefficients == nullptr) ||
           (coefficients->n_components == n_components) ||
           (coefficients->n_components == 1),
           ExcInvalidCoefficient());
@@ -1147,10 +1147,10 @@ estimate (const Mapping<dim, spacedim>               &mapping,
   // gathered in the following structures
   const hp::MappingCollection<dim,spacedim> mapping_collection(mapping);
   const internal::ParallelData<DoFHandlerType,typename InputVector::value_type>
-  parallel_data (dof_handler.get_fe(),
+  parallel_data (dof_handler.get_fe_collection(),
                  face_quadratures,
                  mapping_collection,
-                 (!neumann_bc.empty() || (coefficients != 0)),
+                 (!neumann_bc.empty() || (coefficients != nullptr)),
                  solutions.size(),
                  subdomain_id,
                  material_id,
@@ -1162,10 +1162,10 @@ estimate (const Mapping<dim, spacedim>               &mapping,
   // now let's work on all those cells:
   WorkStream::run (dof_handler.begin_active(),
                    static_cast<typename DoFHandlerType::active_cell_iterator>(dof_handler.end()),
-                   std_cxx11::bind (&internal::estimate_one_cell<InputVector,DoFHandlerType>,
-                                    std_cxx11::_1, std_cxx11::_2, std_cxx11::_3, std_cxx11::ref(solutions),strategy),
-                   std_cxx11::bind (&internal::copy_local_to_global<DoFHandlerType>,
-                                    std_cxx11::_1, std_cxx11::ref(face_integrals)),
+                   std::bind (&internal::estimate_one_cell<InputVector,DoFHandlerType>,
+                              std::placeholders::_1, std::placeholders::_2, std::placeholders::_3, std::ref(solutions),strategy),
+                   std::bind (&internal::copy_local_to_global<DoFHandlerType>,
+                              std::placeholders::_1, std::ref(face_integrals)),
                    parallel_data,
                    sample_local_face_integrals);
 
@@ -1231,7 +1231,7 @@ KellyErrorEstimator<dim, spacedim>::
 estimate (const Mapping<dim, spacedim>               &mapping,
           const DoFHandlerType                       &dof_handler,
           const Quadrature<dim-1>                    &quadrature,
-          const typename FunctionMap<spacedim>::type &neumann_bc,
+          const typename FunctionMap<spacedim,typename InputVector::value_type>::type &neumann_bc,
           const std::vector<const InputVector *>     &solutions,
           std::vector<Vector<float>*>                &errors,
           const ComponentMask                        &component_mask,
@@ -1255,7 +1255,7 @@ template <typename InputVector, typename DoFHandlerType>
 void KellyErrorEstimator<dim, spacedim>::estimate
 (const DoFHandlerType                       &dof_handler,
  const Quadrature<dim-1>                    &quadrature,
- const typename FunctionMap<spacedim>::type &neumann_bc,
+ const typename FunctionMap<spacedim,typename InputVector::value_type>::type &neumann_bc,
  const std::vector<const InputVector *>     &solutions,
  std::vector<Vector<float>*>                &errors,
  const ComponentMask                        &component_mask,
@@ -1277,7 +1277,7 @@ template <typename InputVector, typename DoFHandlerType>
 void KellyErrorEstimator<dim, spacedim>::estimate
 (const DoFHandlerType                       &dof_handler,
  const hp::QCollection<dim-1>               &quadrature,
- const typename FunctionMap<spacedim>::type &neumann_bc,
+ const typename FunctionMap<spacedim,typename InputVector::value_type>::type &neumann_bc,
  const std::vector<const InputVector *>     &solutions,
  std::vector<Vector<float>*>                &errors,
  const ComponentMask                        &component_mask,
