@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 2007 - 2016 by the deal.II authors
+// Copyright (C) 2007 - 2018 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -13,8 +13,8 @@
 //
 // ---------------------------------------------------------------------
 
-#ifndef dealii__fe_function_h
-#define dealii__fe_function_h
+#ifndef dealii_fe_function_h
+#define dealii_fe_function_h
 
 #include <deal.II/base/function.h>
 #include <deal.II/dofs/dof_handler.h>
@@ -24,6 +24,7 @@
 #include <deal.II/base/point.h>
 #include <deal.II/base/tensor.h>
 #include <deal.II/base/thread_local_storage.h>
+#include <deal.II/grid/grid_tools_cache.h>
 
 #include <deal.II/lac/vector.h>
 
@@ -122,16 +123,10 @@ namespace Functions
    * @ref GlossArtificialCell
    * and
    * @ref GlossGhostCell).
-   * If the cell is artificial, we have no access to the solution there and
+   * The solution can be evaluated on ghost cells, but for artificial cells
+   * we have no access to the solution there and
    * functions that evaluate the solution at such a point will trigger an
-   * exception of type VectorTools::ExcPointNotAvailableHere. The same kind of
-   * exception will also be produced if the cell is a ghost cell: On such
-   * cells, one could in principle evaluate the solution, but it becomes
-   * easier if we do not allow to do so because then there is exactly one
-   * processor in a parallel distributed computation that can indeed evaluate
-   * the solution. Consequently, it is clear which processor is responsible
-   * for producing output if the point evaluation is done as a postprocessing
-   * step.
+   * exception of type VectorTools::ExcPointNotAvailableHere.
    *
    * To deal with this situation, you will want to use code as follows when,
    * for example, evaluating the solution at the origin (here using a parallel
@@ -193,8 +188,8 @@ namespace Functions
      *
      * @note When using this function on a
      * parallel::distributed::Triangulation you may get an exception when
-     * trying to evaluate the solution at a point that does not lie in a
-     * locally owned cell (see
+     * trying to evaluate the solution at a point that lies on an artificial
+     * cell (see
      * @ref GlossLocallyOwnedCell).
      * See the section in the general documentation of this class for more
      * information.
@@ -213,8 +208,8 @@ namespace Functions
      *
      * @note When using this function on a
      * parallel::distributed::Triangulation you may get an exception when
-     * trying to evaluate the solution at a point that does not lie in a
-     * locally owned cell (see
+     * trying to evaluate the solution at a point that lies on an artificial
+     * cell (see
      * @ref GlossLocallyOwnedCell).
      * See the section in the general documentation of this class for more
      * information.
@@ -231,8 +226,8 @@ namespace Functions
      *
      * @note When using this function on a
      * parallel::distributed::Triangulation you may get an exception when
-     * trying to evaluate the solution at a point that does not lie in a
-     * locally owned cell (see
+     * trying to evaluate the solution at a point that lies on an artificial
+     * cell (see
      * @ref GlossLocallyOwnedCell).
      * See the section in the general documentation of this class for more
      * information.
@@ -251,8 +246,8 @@ namespace Functions
      *
      * @note When using this function on a
      * parallel::distributed::Triangulation you may get an exception when
-     * trying to evaluate the solution at a point that does not lie in a
-     * locally owned cell (see
+     * trying to evaluate the solution at a point that lies on an artificial
+     * cell (see
      * @ref GlossLocallyOwnedCell).
      * See the section in the general documentation of this class for more
      * information.
@@ -269,8 +264,8 @@ namespace Functions
      *
      * @note When using this function on a
      * parallel::distributed::Triangulation you may get an exception when
-     * trying to evaluate the solution at a point that does not lie in a
-     * locally owned cell (see
+     * trying to evaluate the solution at a point that lies on an artificial
+     * cell (see
      * @ref GlossLocallyOwnedCell).
      * See the section in the general documentation of this class for more
      * information.
@@ -288,8 +283,8 @@ namespace Functions
      *
      * @note When using this function on a
      * parallel::distributed::Triangulation you may get an exception when
-     * trying to evaluate the solution at a point that does not lie in a
-     * locally owned cell (see
+     * trying to evaluate the solution at a point that lies on an artificial
+     * cell (see
      * @ref GlossLocallyOwnedCell).
      * See the section in the general documentation of this class for more
      * information.
@@ -304,8 +299,8 @@ namespace Functions
      *
      * @note When using this function on a
      * parallel::distributed::Triangulation you may get an exception when
-     * trying to evaluate the solution at a point that does not lie in a
-     * locally owned cell (see
+     * trying to evaluate the solution at a point that lies on an artificial
+     * cell (see
      * @ref GlossLocallyOwnedCell).
      * See the section in the general documentation of this class for more
      * information.
@@ -322,8 +317,8 @@ namespace Functions
      *
      * @note When using this function on a
      * parallel::distributed::Triangulation you may get an exception when
-     * trying to evaluate the solution at a point that does not lie in a
-     * locally owned cell (see
+     * trying to evaluate the solution at a point that lies on an artificial
+     * cell (see
      * @ref GlossLocallyOwnedCell).
      * See the section in the general documentation of this class for more
      * information.
@@ -339,8 +334,8 @@ namespace Functions
      *
      * @note When using this function on a
      * parallel::distributed::Triangulation you may get an exception when
-     * trying to evaluate the solution at a point that does not lie in a
-     * locally owned cell (see
+     * trying to evaluate the solution at a point that lies on an artificial
+     * cell (see
      * @ref GlossLocallyOwnedCell).
      * See the section in the general documentation of this class for more
      * information.
@@ -355,8 +350,8 @@ namespace Functions
      *
      * @note When using this function on a
      * parallel::distributed::Triangulation you may get an exception when
-     * trying to evaluate the solution at a point that does not lie in a
-     * locally owned cell (see
+     * trying to evaluate the solution at a point that lies on an artificial
+     * cell (see
      * @ref GlossLocallyOwnedCell).
      * See the section in the general documentation of this class for more
      * information.
@@ -370,8 +365,8 @@ namespace Functions
      *
      * @note When using this function on a
      * parallel::distributed::Triangulation you may get an exception when
-     * trying to evaluate the solution at a point that does not lie in a
-     * locally owned cell (see
+     * trying to evaluate the solution at a point that lies on an artificial
+     * cell (see
      * @ref GlossLocallyOwnedCell).
      * See the section in the general documentation of this class for more
      * information.
@@ -386,8 +381,8 @@ namespace Functions
      *
      * @note When using this function on a
      * parallel::distributed::Triangulation you may get an exception when
-     * trying to evaluate the solution at a point that does not lie in a
-     * locally owned cell (see
+     * trying to evaluate the solution at a point that lies on an artificial
+     * cell (see
      * @ref GlossLocallyOwnedCell).
      * See the section in the general documentation of this class for more
      * information.
@@ -397,15 +392,26 @@ namespace Functions
                            std::vector<Vector<typename VectorType::value_type> >   &values) const;
 
     /**
-     * Create quadrature rules. This function groups the points into blocks
-     * that live in the same cell, and fills up three vectors: @p cells, @p
-     * qpoints and @p maps. The first is a list of the cells that contain the
-     * points, the second is a list of quadrature points matching each cell of
-     * the first list, and the third contains the index of the given
-     * quadrature points, i.e., @p points[maps[3][4]] ends up as the 5th
-     * quadrature point in the 4th cell. This is where optimization would
-     * help. This function returns the number of cells that contain the given
-     * set of points.
+     * Given a set of points located in the domain (or, in the case of
+     * a parallel Triangulation, in the locally owned part of the domain
+     * or on the ghost cells for the current processor), sort these
+     * points into buckets for each of the cells on which at least
+     * one of the points is located.
+     *
+     * This function fills three output vectors: @p cells, @p qpoints
+     * and @p maps. The first is a list of the cells that contain the
+     * points, the second is a list of quadrature points matching each
+     * cell of the first list, and the third contains the index of the
+     * given quadrature points, i.e., @p points[maps[3][4]] ends up as
+     * the 5th quadrature point in the 4th cell.
+     *
+     * @return This function returns the number of cells that
+     *   collectively contain the set of points give as @p
+     *   points. This also equals the lengths of the output arrays.
+     *
+     * This function simply calls GridTools::compute_point_locations :
+     * using the original function avoids computing a
+     * new Cache at every function call.
      */
     unsigned int
     compute_point_locations
@@ -414,11 +420,6 @@ namespace Functions
      std::vector<std::vector<Point<dim> > >                      &qpoints,
      std::vector<std::vector<unsigned int> >                     &maps) const;
 
-    /**
-     * @deprecated Use VectorTools::ExcPointNotAvailableHere instead.
-     */
-    typedef VectorTools::ExcPointNotAvailableHere ExcPointNotAvailableHere DEAL_II_DEPRECATED;
-
   private:
     /**
      * Typedef holding the local cell_hint.
@@ -442,6 +443,11 @@ namespace Functions
      */
     const Mapping<dim> &mapping;
 
+    /**
+     * The Cache object
+     */
+    GridTools::Cache<dim,DoFHandlerType::space_dimension> cache;
+
     /**
      * The latest cell hint.
      */
