@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 2009 - 2016 by the deal.II authors
+// Copyright (C) 2009 - 2018 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -14,8 +14,8 @@
 // ---------------------------------------------------------------------
 
 
-#ifndef dealii__point_value_history_h
-#define dealii__point_value_history_h
+#ifndef dealii_point_value_history_h
+#define dealii_point_value_history_h
 
 #include <deal.II/base/point.h>
 #include <deal.II/base/smartpointer.h>
@@ -44,7 +44,7 @@ DEAL_II_NAMESPACE_OPEN
 
 namespace internal
 {
-  namespace PointValueHistory
+  namespace PointValueHistoryImplementation
   {
     /**
      * A class that stores the data needed to reference the support points
@@ -393,7 +393,7 @@ public:
 
   /**
    * Write out a series of .gpl files named base_name + "-00.gpl", base_name +
-   * "-01.gpl" etc. The data file gives info about where the support points
+   * "-01.gpl" etc. The data file gives information about where the support points
    * selected and interpreting the data. If @p n_indep != 0 an additional file
    * base_name + "_indep.gpl" containing key and independent data. The file
    * name agrees with the order the points were added to the class. The names
@@ -560,14 +560,14 @@ private:
   std::vector <std::vector <double> > independent_values;
 
   /**
-   * Saves a vector listing component names associated with a
+   * Save a vector listing component names associated with a
    * independent_values. This will be an empty vector if the user does not
    * supplies names.
    */
   std::vector<std::string> indep_names;
 
   /**
-   * Saves data for each mnemonic entry. data_store: mnemonic ->
+   * Save data for each mnemonic entry. data_store: mnemonic ->
    * [point_0_components point_1_components ... point_n-1_components][key]
    * This format facilitates scalar mnemonics in a vector space, because
    * scalar mnemonics will only have one component per point. Vector
@@ -576,21 +576,21 @@ private:
   std::map <std::string, std::vector <std::vector <double> > > data_store;
 
   /**
-   * Saves a component mask for each mnemonic.
+   * Save a component mask for each mnemonic.
    */
   std::map <std::string, ComponentMask> component_mask;
 
 
   /**
-   * Saves a vector listing component names associated with a mnemonic. This
+   * Save a vector listing component names associated with a mnemonic. This
    * will be an empty vector if the user does not supplies names.
    */
   std::map <std::string, std::vector<std::string> > component_names_map;
 
   /**
-   * Saves the location and other mesh info about support points.
+   * Save the location and other mesh information about support points.
    */
-  std::vector <internal::PointValueHistory::PointGeometryData <dim> >
+  std::vector <internal::PointValueHistoryImplementation::PointGeometryData <dim> >
   point_geometry_data;
 
 
@@ -648,4 +648,4 @@ private:
 
 
 DEAL_II_NAMESPACE_CLOSE
-#endif /* dealii__point_value_history_h */
+#endif /* dealii_point_value_history_h */
