@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 1999 - 2016 by the deal.II authors
+// Copyright (C) 1999 - 2018 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -13,8 +13,8 @@
 //
 // ---------------------------------------------------------------------
 
-#ifndef dealii__solution_transfer_h
-#define dealii__solution_transfer_h
+#ifndef dealii_solution_transfer_h
+#define dealii_solution_transfer_h
 
 
 /*----------------------------   solutiontransfer.h     ----------------------*/
@@ -243,7 +243,7 @@ DEAL_II_NAMESPACE_OPEN
  *
  * @note This situation can only happen if you do coarsening. If all cells
  * remain as they are or are refined, then SolutionTransfer::interpolate()
- * computes a new vector of nodel values, but the function represented is of
+ * computes a new vector of nodal values, but the function represented is of
  * course exactly the same because the old finite element space is a subspace
  * of the new one. Thus, if the old function was conforming (i.e., satisfied
  * hanging node constraints), then so does the new one, and it is not
@@ -282,16 +282,23 @@ DEAL_II_NAMESPACE_OPEN
  * space is not a subspace of the new space and you may run into the same
  * situation as discussed above with hanging nodes. You may want to consider
  * calling ConstraintMatrix::distribute() on the vector obtained by
- * transfering the solution.
+ * transferring the solution.
  *
  * @ingroup numerics
  * @author Ralf Hartmann, 1999, Oliver Kayser-Herold and Wolfgang Bangerth,
  * 2006, Wolfgang Bangerth 2014
  */
-template<int dim, typename VectorType=Vector<double>,
-         typename DoFHandlerType=DoFHandler<dim> >
+template <int dim,
+          typename VectorType = Vector<double>,
+          typename DoFHandlerType = DoFHandler<dim> >
 class SolutionTransfer
 {
+#ifndef DEAL_II_MSVC
+  static_assert (dim == DoFHandlerType::dimension,
+                 "The dimension explicitly provided as a template "
+                 "argument, and the dimension of the DoFHandlerType "
+                 "template argument must match.");
+#endif
 public:
 
   /**
@@ -473,18 +480,18 @@ private:
    */
   struct Pointerstruct
   {
-    Pointerstruct() : indices_ptr(0), dof_values_ptr(0), active_fe_index(0) {};
+    Pointerstruct() : indices_ptr(nullptr), dof_values_ptr(nullptr), active_fe_index(0) {}
     Pointerstruct(std::vector<types::global_dof_index> *indices_ptr_in,
                   const unsigned int active_fe_index_in = 0)
       :
       indices_ptr(indices_ptr_in),
-      dof_values_ptr (0),
-      active_fe_index(active_fe_index_in) {};
+      dof_values_ptr (nullptr),
+      active_fe_index(active_fe_index_in) {}
     Pointerstruct(std::vector<Vector<typename VectorType::value_type> > *dof_values_ptr_in,
                   const unsigned int active_fe_index_in = 0) :
-      indices_ptr (0),
+      indices_ptr (nullptr),
       dof_values_ptr(dof_values_ptr_in),
-      active_fe_index(active_fe_index_in) {};
+      active_fe_index(active_fe_index_in) {}
     std::size_t memory_consumption () const;
 
     std::vector<types::global_dof_index>    *indices_ptr;
