@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 2014 - 2017 by the deal.II authors
+// Copyright (C) 2014 - 2018 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -14,15 +14,14 @@
 // ---------------------------------------------------------------------
 
 
-#ifndef dealii__occ_boundary_lib_h
-#define dealii__occ_boundary_lib_h
+#ifndef dealii_occ_boundary_lib_h
+#define dealii_occ_boundary_lib_h
 
 #include <deal.II/base/config.h>
 
 #ifdef DEAL_II_WITH_OPENCASCADE
 
 #include <deal.II/opencascade/utilities.h>
-#include <deal.II/grid/tria_boundary.h>
 #include <deal.II/grid/manifold.h>
 
 // opencascade needs "HAVE_CONFIG_H" to be exported...
@@ -42,10 +41,10 @@ DEAL_II_NAMESPACE_OPEN
 namespace OpenCASCADE
 {
   /**
-   * A Boundary object based on OpenCASCADE TopoDS_Shape where where new
-   * points are first computed by averaging the surrounding points in the same
-   * way as FlatManifold does, and are then projected in the normal direction
-   * using OpenCASCADE utilities.
+   * A Manifold object based on OpenCASCADE TopoDS_Shape where new points are
+   * first computed by averaging the surrounding points in the same way as
+   * FlatManifold does, and are then projected in the normal direction using
+   * OpenCASCADE utilities.
    *
    * This class makes no assumptions on the shape you pass to it, and the
    * topological dimension of the Manifold is inferred from the TopoDS_Shape
@@ -63,7 +62,7 @@ namespace OpenCASCADE
    * @author Luca Heltai, Andrea Mola, 2011--2014.
    */
   template <int dim, int spacedim>
-  class NormalProjectionBoundary : public Boundary<dim,spacedim>
+  class NormalProjectionBoundary : public FlatManifold<dim,spacedim>
   {
   public:
 
@@ -77,6 +76,11 @@ namespace OpenCASCADE
     NormalProjectionBoundary(const TopoDS_Shape &sh,
                              const double tolerance=1e-7);
 
+    /**
+     * Clone the current Manifold.
+     */
+    virtual std::unique_ptr<Manifold<dim,spacedim> > clone() const override;
+
     /**
      * Perform the actual projection onto the manifold. This function, in
      * debug mode, checks that each of the @p surrounding_points is within
@@ -87,8 +91,8 @@ namespace OpenCASCADE
      * algorithms.
      */
     virtual Point<spacedim>
-    project_to_manifold (const std::vector<Point<spacedim> > &surrounding_points,
-                         const Point<spacedim> &candidate) const;
+    project_to_manifold (const ArrayView<const Point<spacedim>> &surrounding_points,
+                         const Point<spacedim>                  &candidate) const override;
 
 
   private:
@@ -107,7 +111,7 @@ namespace OpenCASCADE
   };
 
   /**
-   * A Boundary object based on OpenCASCADE TopoDS_Shape where new points are
+   * A Manifold object based on OpenCASCADE TopoDS_Shape where new points are
    * first computed by averaging the surrounding points in the same way as
    * FlatManifold does, and then projecting them onto the manifold along the
    * direction specified at construction time using OpenCASCADE utilities.
@@ -120,7 +124,7 @@ namespace OpenCASCADE
    * points leaves them untouched. If this is not the case, an
    * ExcPointNotOnManifold is thrown.
    *
-   * Notice that this type of Boundary descriptor may fail to give results if
+   * Notice that this type of Manifold descriptor may fail to give results if
    * the triangulation to be refined is close to the boundary of the given
    * TopoDS_Shape, or when the direction you use at construction time does not
    * intersect the shape. An exception is thrown when this happens.
@@ -128,17 +132,22 @@ namespace OpenCASCADE
    * @author Luca Heltai, Andrea Mola, 2011--2014.
    */
   template <int dim, int spacedim>
-  class DirectionalProjectionBoundary : public Boundary<dim,spacedim>
+  class DirectionalProjectionBoundary : public FlatManifold<dim,spacedim>
   {
   public:
     /**
-     * Construct a Boundary object which will project points on the
+     * Construct a Manifold object which will project points on the
      * TopoDS_Shape @p sh, along the given @p direction.
      */
     DirectionalProjectionBoundary(const TopoDS_Shape &sh,
                                   const Tensor<1,spacedim> &direction,
                                   const double tolerance=1e-7);
 
+    /**
+     * Clone the current Manifold.
+     */
+    virtual std::unique_ptr<Manifold<dim,spacedim> > clone() const override;
+
     /**
      * Perform the actual projection onto the manifold. This function, in
      * debug mode, checks that each of the @p surrounding_points is within
@@ -149,8 +158,8 @@ namespace OpenCASCADE
      * projection algorithms.
      */
     virtual Point<spacedim>
-    project_to_manifold (const std::vector<Point<spacedim> > &surrounding_points,
-                         const Point<spacedim> &candidate) const;
+    project_to_manifold (const ArrayView<const Point<spacedim>> &surrounding_points,
+                         const Point<spacedim>                  &candidate) const override;
 
   private:
     /**
@@ -164,7 +173,7 @@ namespace OpenCASCADE
     /**
      * Direction used to project new points on the shape.
      */
-    const Point<3> direction;
+    const Tensor<1, spacedim> direction;
 
     /**
      * Relative tolerance used by this class to compute distances.
@@ -174,7 +183,7 @@ namespace OpenCASCADE
 
 
   /**
-   * A Boundary object based on OpenCASCADE TopoDS_Shape where new points are
+   * A Manifold object based on OpenCASCADE TopoDS_Shape where new points are
    * first computed by averaging the surrounding points in the same way as
    * FlatManifold does, and then projecting them using OpenCASCADE utilities
    * onto the manifold along a direction which is an estimation of the
@@ -209,7 +218,7 @@ namespace OpenCASCADE
    * ExcPointNotOnManifold is thrown.
    *
    *
-   * Notice that this type of Boundary descriptor may fail to give results if
+   * Notice that this type of Manifold descriptor may fail to give results if
    * the triangulation to be refined is close to the boundary of the given
    * TopoDS_Shape, or when the normal direction estimated from the surrounding
    * points does not intersect the shape.  An exception is thrown when this
@@ -218,17 +227,22 @@ namespace OpenCASCADE
    * @author Luca Heltai, Andrea Mola, 2011--2014.
    */
   template <int dim, int spacedim>
-  class NormalToMeshProjectionBoundary : public Boundary<dim,spacedim>
+  class NormalToMeshProjectionBoundary : public FlatManifold<dim,spacedim>
   {
   public:
     /**
-     * Construct a Boundary object which will project points on the
+     * Construct a Manifold object which will project points on the
      * TopoDS_Shape @p sh, along a direction which is approximately normal to
      * the mesh cell.
      */
     NormalToMeshProjectionBoundary(const TopoDS_Shape &sh,
                                    const double tolerance=1e-7);
 
+    /**
+     * Clone the current Manifold.
+     */
+    virtual std::unique_ptr<Manifold<dim,spacedim> > clone() const override;
+
     /**
      * Perform the actual projection onto the manifold. This function, in
      * debug mode, checks that each of the @p surrounding_points is within
@@ -236,8 +250,8 @@ namespace OpenCASCADE
      * exception is thrown.
      */
     virtual Point<spacedim>
-    project_to_manifold (const std::vector<Point<spacedim> > &surrounding_points,
-                         const Point<spacedim> &candidate) const;
+    project_to_manifold (const ArrayView<const Point<spacedim>> &surrounding_points,
+                         const Point<spacedim>                  &candidate) const override;
 
   private:
     /**
@@ -248,11 +262,6 @@ namespace OpenCASCADE
      */
     const TopoDS_Shape sh;
 
-    /**
-     * Direction used to project new points on the shape.
-     */
-    const Point<3> direction;
-
     /**
      * Relative tolerance used by this class to compute distances.
      */
@@ -260,7 +269,7 @@ namespace OpenCASCADE
   };
 
   /**
-   * A Boundary object based on OpenCASCADE TopoDS_Shape objects which have
+   * A Manifold object based on OpenCASCADE TopoDS_Shape objects which have
    * topological dimension equal to one (TopoDS_Edge or TopoDS_Wire) where new
    * points are located at the arclength average of the surrounding points. If
    * the given TopoDS_Shape can be casted to a periodic (closed) curve, then
@@ -288,21 +297,31 @@ namespace OpenCASCADE
     ArclengthProjectionLineManifold(const TopoDS_Shape &sh,
                                     const double tolerance=1e-7);
 
+    /**
+     * Clone the current Manifold.
+     */
+    virtual std::unique_ptr<Manifold<dim,spacedim> > clone() const override;
+
     /**
      * Given a point on real space, find its arclength parameter. Throws an
      * error in debug mode, if the point is not on the TopoDS_Edge given at
      * construction time.
      */
     virtual Point<1>
-    pull_back(const Point<spacedim> &space_point) const;
+    pull_back(const Point<spacedim> &space_point) const override;
 
     /**
      * Given an arclength parameter, find its image in real space.
      */
     virtual Point<spacedim>
-    push_forward(const Point<1> &chart_point) const;
+    push_forward(const Point<1> &chart_point) const override;
 
   private:
+    /**
+     * The actual shape used to build this object.
+     */
+    const TopoDS_Shape sh;
+
     /**
      * A Curve adaptor. This is the one which is used in the computations, and
      * it points to the right one above.
@@ -339,19 +358,24 @@ namespace OpenCASCADE
      */
     NURBSPatchManifold(const TopoDS_Face &face, const double tolerance = 1e-7);
 
+    /**
+     * Clone the current Manifold.
+     */
+    virtual std::unique_ptr<Manifold<dim,spacedim> > clone() const override;
+
     /**
      * Pull back the given point from the Euclidean space. Will return the uv
      * coordinates associated with the point @p space_point.
      */
     virtual Point<2>
-    pull_back(const Point<spacedim> &space_point) const;
+    pull_back(const Point<spacedim> &space_point) const override;
 
     /**
      * Given a @p chart_point in the uv coordinate system, this method returns the
      * Euclidean coordinates associated.
      */
     virtual Point<spacedim>
-    push_forward(const Point<2> &chart_point) const;
+    push_forward(const Point<2> &chart_point) const override;
 
     /**
      * Given a point in the spacedim dimensional Euclidean space, this
@@ -367,14 +391,14 @@ namespace OpenCASCADE
      */
     virtual
     DerivativeForm<1,2,spacedim>
-    push_forward_gradient(const Point<2> &chart_point) const;
+    push_forward_gradient(const Point<2> &chart_point) const override;
 
   private:
     /**
      * Return a tuple representing the minimum and maximum values of u
      * and v.  Precisely, it returns (u_min, u_max, v_min, v_max)
      */
-    std_cxx11::tuple<double, double, double, double>
+    std::tuple<double, double, double, double>
     get_uv_bounds() const;
 
     /**
