@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 2014 - 2017 by the deal.II authors
+// Copyright (C) 2014 - 2018 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -14,8 +14,8 @@
 // ---------------------------------------------------------------------
 
 
-#ifndef dealii__occ_utilities_h
-#define dealii__occ_utilities_h
+#ifndef dealii_occ_utilities_h
+#define dealii_occ_utilities_h
 
 #include <deal.II/base/config.h>
 
@@ -23,11 +23,11 @@
 
 #include <deal.II/grid/tria.h>
 #include <deal.II/base/point.h>
+#include <deal.II/fe/mapping_q1.h>
 
 #include <string>
 
 // opencascade needs "HAVE_CONFIG_H" to be exported...
-DEAL_II_DISABLE_EXTRA_DIAGNOSTICS
 #define HAVE_CONFIG_H
 #include <TopoDS_Shape.hxx>
 #include <TopoDS_Face.hxx>
@@ -41,7 +41,6 @@ DEAL_II_DISABLE_EXTRA_DIAGNOSTICS
 #include <IFSelect_ReturnStatus.hxx>
 #include <gp_Pnt.hxx>
 #undef HAVE_CONFIG_H
-DEAL_II_ENABLE_EXTRA_DIAGNOSTICS
 
 
 
@@ -88,10 +87,16 @@ DEAL_II_NAMESPACE_OPEN
  * TopoDS_Shapes of one type or another, and provide interfaces to common
  * deal.II objects, like Triangulation, Manifold, and so on.
  *
- * Notice that these tools are only useful when spacedim is equal to three,
- * since OpenCASCADE only operates in three-dimensional mode.
+ * Notice that most of these tools are only useful when spacedim is equal to
+ * three, since OpenCASCADE only operates in three-dimensional mode. In some
+ * cases they can be used in two dimensions as well, and the third dimension
+ * will be set to zero.
  *
- * @author Luca Heltai, Andrea Mola, 2011--2014.
+ * If you wish to use these tools when the dimension of the space is two, then
+ * make sure your CAD files are actually flat and that all z coordinates are equal
+ * to zero, as otherwise you will get many exceptions.
+ *
+ * @author Luca Heltai, Andrea Mola, 2011--2017.
  */
 namespace OpenCASCADE
 {
@@ -102,7 +107,7 @@ namespace OpenCASCADE
    * associated with actual geometries) which are contained in the given
    * shape.
    */
-  std_cxx11::tuple<unsigned int, unsigned int, unsigned int>
+  std::tuple<unsigned int, unsigned int, unsigned int>
   count_elements(const TopoDS_Shape &shape);
 
   /**
@@ -176,9 +181,9 @@ namespace OpenCASCADE
                          const double tolerance=1e-7);
 
   /**
-   * Creates a 3D smooth BSpline curve passing through the points in the
-   * assigned vector, and store it in the returned TopoDS_Shape (which is of
-   * type TopoDS_Edge). The points are reordered internally according to their
+   * Creates a smooth BSpline curve passing through the points in the assigned
+   * vector, and store it in the returned TopoDS_Shape (which is of type
+   * TopoDS_Edge). The points are reordered internally according to their
    * scalar product with the direction, if direction is different from zero,
    * otherwise they are used as passed. Notice that this function changes the
    * input points if required by the algorithm.
@@ -188,12 +193,13 @@ namespace OpenCASCADE
    * set to true, then the curve will be C2 at all points except the first
    * (where only C1 continuity will be given), and it will be a closed curve.
    *
-   * The curve is garanteed to be at distance @p tolerance from the input
+   * The curve is guaranteed to be at distance @p tolerance from the input
    * points. If the algorithm fails in generating such a curve, an exception
    * is thrown.
    */
-  TopoDS_Edge interpolation_curve(std::vector<Point<3> >  &curve_points,
-                                  const Tensor<1,3> &direction=Tensor<1,3>(),
+  template<int dim>
+  TopoDS_Edge interpolation_curve(std::vector<Point<dim> > &curve_points,
+                                  const Tensor<1, dim> &direction=Tensor<1,dim>(),
                                   const bool closed=false,
                                   const double tolerance=1e-7);
 
@@ -208,16 +214,46 @@ namespace OpenCASCADE
                                   std::vector<TopoDS_Vertex> &vertices);
 
   /**
-   * Create a triangulation from a single face. This class extract the first u
-   * and v parameter of the parametric surface making up this face, and
-   * creates a Triangulation<2,3> containing a single coarse cell reflecting
+   * Create a triangulation from a single face. This class extracts the first u
+   * and v parameter of the parametric surface making up this face, and creates
+   * a Triangulation<2,spacedim> containing a single coarse cell reflecting
    * this face. If the surface is not a trimmed surface, the vertices of this
    * cell will coincide with the TopoDS_Vertex vertices of the original
    * TopoDS_Face. This, however, is often not the case, and the user should be
    * careful on how this mesh is used.
+   *
+   * If you call this function with a Triangulation<2,2>, make sure that the
+   * input face has all z coordinates set to zero, or you'll get an exception.
    */
+  template <int spacedim>
   void create_triangulation(const TopoDS_Face &face,
-                            Triangulation<2,3> &tria);
+                            Triangulation<2,spacedim> &tria);
+
+
+  /**
+   * Given a Triangulation and an optional Mapping, create a vector of smooth
+   * curves that interpolate the connected parts of the boundary vertices of
+   * the Triangulation and return them as a vector of TopoDS_Edge objects.
+   *
+   * This function constructs closed Bspline curve objects passing through all
+   * vertices of the boundary of the triangulation, with $C^2$ Continuity on
+   * each vertex except the first, where only $C^1$ continuity is guaranteed.
+   *
+   * The returned curves are ordered with respect to the indices of the faces
+   * that make up the triangulation boundary, i.e., the first curve is the one
+   * extracted starting from the face with the lowest index, and so on.
+   *
+   * @param[in] triangulation Input triangulation
+   * @param[in] mapping Optional input mapping
+   * @return An std::vector of TopoDS_Edge objects representing the smooth
+   *  interpolation of the boundary of the `triangulation`
+   *
+   * @author Dirk Peschka, Luca Heltai, 2017.
+   */
+  template <int spacedim>
+  std::vector<TopoDS_Edge> create_curves_from_triangulation_boundary
+  (const Triangulation<2,spacedim> &triangulation,
+   const Mapping<2,spacedim> &mapping=StaticMappingQ1<2,spacedim>::mapping);
 
   /**
    * Extract all compound shapes from a TopoDS_Shape, and store the results
@@ -245,9 +281,10 @@ namespace OpenCASCADE
    * the u coordinate and the v coordinate (which is different from zero only
    * if the resulting shape is a face).
    */
-  std_cxx11::tuple<Point<3>, TopoDS_Shape, double, double>
+  template <int dim>
+  std::tuple<Point<dim>, TopoDS_Shape, double, double>
   project_point_and_pull_back(const TopoDS_Shape &in_shape,
-                              const Point<3> &origin,
+                              const Point<dim> &origin,
                               const double tolerance=1e-7);
 
   /**
@@ -256,9 +293,10 @@ namespace OpenCASCADE
    * are iterated, faces first, then edges, and the returned point is the
    * closest one to the @p in_shape, regardless of its type.
    */
-  Point<3> closest_point(const TopoDS_Shape &in_shape,
-                         const Point<3> &origin,
-                         const double tolerance=1e-7);
+  template <int dim>
+  Point<dim> closest_point(const TopoDS_Shape &in_shape,
+                           const Point<dim> &origin,
+                           const double tolerance=1e-7);
 
   /**
    * Given an elementary shape @p in_shape and the reference coordinates
@@ -268,9 +306,10 @@ namespace OpenCASCADE
    * used as input to this function. If this is not the case, an Exception is
    * thrown.
    */
-  Point<3> push_forward(const TopoDS_Shape &in_shape,
-                        const double u,
-                        const double v);
+  template <int dim>
+  Point<dim> push_forward(const TopoDS_Shape &in_shape,
+                          const double u,
+                          const double v);
 
 
   /**
@@ -278,7 +317,7 @@ namespace OpenCASCADE
    * face, returns the corresponding point in real space, the normal to the
    * surface at that point and the min and max curvatures as a tuple.
    */
-  std_cxx11::tuple<Point<3>,  Tensor<1,3>, double, double>
+  std::tuple<Point<3>,  Tensor<1,3>, double, double>
   push_forward_and_differential_forms(const TopoDS_Face &face,
                                       const double u,
                                       const double v,
@@ -292,7 +331,7 @@ namespace OpenCASCADE
    * and only the closest point is returned. This function will throw an
    * exception if the @p in_shape does not contain at least one face.
    */
-  std_cxx11::tuple<Point<3>,  Tensor<1,3>, double, double>
+  std::tuple<Point<3>,  Tensor<1,3>, double, double>
   closest_point_and_differential_forms(const TopoDS_Shape &in_shape,
                                        const Point<3> &origin,
                                        const double tolerance=1e-7);
@@ -305,22 +344,29 @@ namespace OpenCASCADE
    *
    * The optional @p tolerance parameter is used to compute distances.
    */
-  Point<3> line_intersection(const TopoDS_Shape &in_shape,
-                             const Point<3> &origin,
-                             const Tensor<1,3> &direction,
-                             const double tolerance=1e-7);
+  template <int dim>
+  Point<dim> line_intersection(const TopoDS_Shape &in_shape,
+                               const Point<dim> &origin,
+                               const Tensor<1,dim> &direction,
+                               const double tolerance=1e-7);
 
 
   /**
-   * Convert OpenCASCADE point into a Point<3>.
+   * Convert OpenCASCADE point into a Point<spacedim>.
+   *
+   * The tolerance argument is used to check if the non used components of the
+   * OpenCASCADE point are close to zero. If this is not the case, an assertion
+   * is thrown in debug mode.
    */
-  Point<3> point(const gp_Pnt &p);
+  template <int spacedim>
+  Point<spacedim> point(const gp_Pnt &p, const double &tolerance=1e-10);
 
 
   /**
    * Convert Point<3> into OpenCASCADE point.
    */
-  gp_Pnt point(const Point<3> &p);
+  template <int spacedim>
+  gp_Pnt point(const Point<spacedim> &p);
 
 
   /**
@@ -329,25 +375,29 @@ namespace OpenCASCADE
    * optional parameter is used as a relative tolerance when comparing
    * objects.
    */
-  bool point_compare(const Point<3> &p1, const Point<3> &p2,
-                     const Tensor<1,3> &direction=Tensor<1,3>(),
-                     const double tolerance=1e-10);
+  template <int dim>
+  bool point_compare(const Point<dim> &p1,
+                     const Point<dim> &p2,
+                     const Tensor<1,dim> &direction = Tensor<1,dim>(),
+                     const double       tolerance = 1e-10);
 
 
   /**
    * Exception thrown when the point specified as argument does not lie
    * between @p tolerance from the given TopoDS_Shape.
    */
+  template <int dim>
   DeclException1 (ExcPointNotOnManifold,
-                  Point<3>,
+                  Point<dim>,
                   <<"The point [ "<<arg1<<" ] is not on the manifold.");
 
   /**
    * Exception thrown when the point specified as argument cannot be projected
    * to the manifold.
    */
+  template <int dim>
   DeclException1 (ExcProjectionFailed,
-                  Point<3>,
+                  Point<dim>,
                   <<"Projection of point [ "<< arg1
                   << " ] failed.");
 
