@@ -13,8 +13,8 @@
 //
 // ---------------------------------------------------------------------
 
-#ifndef dealii__transformations_h
-#define dealii__transformations_h
+#ifndef dealii_transformations_h
+#define dealii_transformations_h
 
 #include <deal.II/base/point.h>
 #include <deal.II/base/tensor.h>
@@ -43,7 +43,7 @@ namespace Physics
 //@{
 
       /**
-       * Returns the rotation matrix for 2-d Euclidean space, namely
+       * Return the rotation matrix for 2-d Euclidean space, namely
        * @f[
        *  \mathbf{R} := \left[ \begin{array}{cc}
        *  cos(\theta) & sin(\theta) \\
@@ -57,13 +57,13 @@ namespace Physics
        *
        * @param[in] angle The rotation angle (about the z-axis) in radians
        */
-      template<typename Number>
+      template <typename Number>
       Tensor<2,2,Number>
       rotation_matrix_2d (const Number &angle);
 
 
       /**
-       * Returns the rotation matrix for 3-d Euclidean space. Most concisely
+       * Return the rotation matrix for 3-d Euclidean space. Most concisely
        * stated using the Rodrigues' rotation formula, this function returns
        * the equivalent of
        * @f[
@@ -90,7 +90,7 @@ namespace Physics
        * @param[in] axis  A unit vector that defines the axis of rotation
        * @param[in] angle The rotation angle in radians
        */
-      template<typename Number>
+      template <typename Number>
       Tensor<2,3,Number>
       rotation_matrix_3d (const Point<3,Number> &axis,
                           const Number          &angle);
@@ -126,7 +126,7 @@ namespace Physics
 //@{
 
       /**
-       * Returns the result of the push forward transformation on a
+       * Return the result of the push forward transformation on a
        * contravariant vector, i.e.
        * @f[
        *  \chi\left(\bullet\right)^{\sharp}
@@ -144,7 +144,7 @@ namespace Physics
                     const Tensor<2,dim,Number> &F);
 
       /**
-       * Returns the result of the push forward transformation on a rank-2
+       * Return the result of the push forward transformation on a rank-2
        * contravariant tensor, i.e.
        * @f[
        *  \chi\left(\bullet\right)^{\sharp}
@@ -162,7 +162,7 @@ namespace Physics
                     const Tensor<2,dim,Number> &F);
 
       /**
-       * Returns the result of the push forward transformation on a rank-2
+       * Return the result of the push forward transformation on a rank-2
        * contravariant symmetric tensor, i.e.
        * @f[
        *  \chi\left(\bullet\right)^{\sharp}
@@ -181,7 +181,7 @@ namespace Physics
                     const Tensor<2,dim,Number>          &F);
 
       /**
-       * Returns the result of the push forward transformation on a rank-4
+       * Return the result of the push forward transformation on a rank-4
        * contravariant tensor, i.e. (in index notation)
        * @f[
        *  \left[ \chi\left(\bullet\right)^{\sharp} \right]_{ijkl}
@@ -199,7 +199,7 @@ namespace Physics
                     const Tensor<2,dim,Number> &F);
 
       /**
-       * Returns the result of the push forward transformation on a rank-4
+       * Return the result of the push forward transformation on a rank-4
        * contravariant symmetric tensor, i.e. (in index notation)
        * @f[
        *  \left[ \chi\left(\bullet\right)^{\sharp} \right]_{ijkl}
@@ -225,7 +225,7 @@ namespace Physics
 //@{
 
       /**
-       * Returns the result of the pull back transformation on a contravariant
+       * Return the result of the pull back transformation on a contravariant
        * vector, i.e.
        * @f[
        *  \chi^{-1}\left(\bullet\right)^{\sharp}
@@ -243,7 +243,7 @@ namespace Physics
                  const Tensor<2,dim,Number> &F);
 
       /**
-       * Returns the result of the pull back transformation on a rank-2
+       * Return the result of the pull back transformation on a rank-2
        * contravariant tensor, i.e.
        * @f[
        *  \chi^{-1}\left(\bullet\right)^{\sharp}
@@ -261,7 +261,7 @@ namespace Physics
                  const Tensor<2,dim,Number> &F);
 
       /**
-       * Returns the result of the pull back transformation on a rank-2
+       * Return the result of the pull back transformation on a rank-2
        * contravariant symmetric tensor, i.e.
        * @f[
        *  \chi^{-1}\left(\bullet\right)^{\sharp}
@@ -279,7 +279,7 @@ namespace Physics
                  const Tensor<2,dim,Number>          &F);
 
       /**
-       * Returns the result of the pull back transformation on a rank-4
+       * Return the result of the pull back transformation on a rank-4
        * contravariant tensor, i.e. (in index notation)
        * @f[
        *  \left[ \chi^{-1}\left(\bullet\right)^{\sharp} \right]_{IJKL}
@@ -297,7 +297,7 @@ namespace Physics
                  const Tensor<2,dim,Number> &F);
 
       /**
-       * Returns the result of the pull back transformation on a rank-4
+       * Return the result of the pull back transformation on a rank-4
        * contravariant symmetric tensor, i.e. (in index notation)
        * @f[
        *  \left[ \chi^{-1}\left(\bullet\right)^{\sharp} \right]_{IJKL}
@@ -345,7 +345,7 @@ namespace Physics
 //@{
 
       /**
-       * Returns the result of the push forward transformation on a covariant
+       * Return the result of the push forward transformation on a covariant
        * vector, i.e.
        * @f[
        *  \chi\left(\bullet\right)^{\flat}
@@ -363,7 +363,7 @@ namespace Physics
                     const Tensor<2,dim,Number> &F);
 
       /**
-       * Returns the result of the push forward transformation on a rank-2
+       * Return the result of the push forward transformation on a rank-2
        * covariant tensor, i.e.
        * @f[
        *  \chi\left(\bullet\right)^{\flat}
@@ -381,7 +381,7 @@ namespace Physics
                     const Tensor<2,dim,Number> &F);
 
       /**
-       * Returns the result of the push forward transformation on a rank-2
+       * Return the result of the push forward transformation on a rank-2
        * covariant symmetric tensor, i.e.
        * @f[
        *  \chi\left(\bullet\right)^{\flat}
@@ -400,7 +400,7 @@ namespace Physics
                     const Tensor<2,dim,Number>          &F);
 
       /**
-       * Returns the result of the push forward transformation on a rank-4
+       * Return the result of the push forward transformation on a rank-4
        * covariant tensor, i.e. (in index notation)
        * @f[
        *  \left[ \chi\left(\bullet\right)^{\flat} \right]_{ijkl}
@@ -418,7 +418,7 @@ namespace Physics
                     const Tensor<2,dim,Number> &F);
 
       /**
-       * Returns the result of the push forward transformation on a rank-4
+       * Return the result of the push forward transformation on a rank-4
        * covariant symmetric tensor, i.e. (in index notation)
        * @f[
        *  \left[ \chi\left(\bullet\right)^{\flat} \right]_{ijkl}
@@ -444,7 +444,7 @@ namespace Physics
 //@{
 
       /**
-       * Returns the result of the pull back transformation on a covariant
+       * Return the result of the pull back transformation on a covariant
        * vector, i.e.
        * @f[
        *  \chi^{-1}\left(\bullet\right)^{\flat}
@@ -462,7 +462,7 @@ namespace Physics
                  const Tensor<2,dim,Number> &F);
 
       /**
-       * Returns the result of the pull back transformation on a rank-2
+       * Return the result of the pull back transformation on a rank-2
        * covariant tensor, i.e.
        * @f[
        *  \chi^{-1}\left(\bullet\right)^{\flat}
@@ -480,7 +480,7 @@ namespace Physics
                  const Tensor<2,dim,Number> &F);
 
       /**
-       * Returns the result of the pull back transformation on a rank-2
+       * Return the result of the pull back transformation on a rank-2
        * covariant symmetric tensor, i.e.
        * @f[
        *  \chi^{-1}\left(\bullet\right)^{\flat}
@@ -498,7 +498,7 @@ namespace Physics
                  const Tensor<2,dim,Number>          &F);
 
       /**
-       * Returns the result of the pull back transformation on a rank-4
+       * Return the result of the pull back transformation on a rank-4
        * contravariant tensor, i.e. (in index notation)
        * @f[
        *  \left[ \chi^{-1}\left(\bullet\right)^{\flat} \right]_{IJKL}
@@ -516,7 +516,7 @@ namespace Physics
                  const Tensor<2,dim,Number> &F);
 
       /**
-       * Returns the result of the pull back transformation on a rank-4
+       * Return the result of the pull back transformation on a rank-4
        * contravariant symmetric tensor, i.e. (in index notation)
        * @f[
        *  \left[ \chi^{-1}\left(\bullet\right)^{\flat} \right]_{IJKL}
@@ -552,7 +552,7 @@ namespace Physics
 //@{
 
       /**
-       * Returns the result of the push forward transformation on a
+       * Return the result of the push forward transformation on a
        * contravariant vector, i.e.
        * @f[
        *  \textrm{det} \mathbf{F}^{-1} \; \chi\left(\bullet\right)^{\sharp}
@@ -571,7 +571,7 @@ namespace Physics
                     const Tensor<2,dim,Number> &F);
 
       /**
-       * Returns the result of the push forward transformation on a rank-2
+       * Return the result of the push forward transformation on a rank-2
        * contravariant tensor, i.e.
        * @f[
        *  \textrm{det} \mathbf{F}^{-1} \; \chi\left(\bullet\right)^{\sharp}
@@ -590,7 +590,7 @@ namespace Physics
                     const Tensor<2,dim,Number> &F);
 
       /**
-       * Returns the result of the push forward transformation on a rank-2
+       * Return the result of the push forward transformation on a rank-2
        * contravariant symmetric tensor, i.e.
        * @f[
        *  \textrm{det} \mathbf{F}^{-1} \; \chi\left(\bullet\right)^{\sharp}
@@ -610,7 +610,7 @@ namespace Physics
                     const Tensor<2,dim,Number>          &F);
 
       /**
-       * Returns the result of the push forward transformation on a rank-4
+       * Return the result of the push forward transformation on a rank-4
        * contravariant tensor, i.e. (in index notation)
        * @f[
        *  \textrm{det} \mathbf{F}^{-1} \; \left[ \chi\left(\bullet\right)^{\sharp} \right]_{ijkl}
@@ -629,7 +629,7 @@ namespace Physics
                     const Tensor<2,dim,Number> &F);
 
       /**
-       * Returns the result of the push forward transformation on a rank-4
+       * Return the result of the push forward transformation on a rank-4
        * contravariant symmetric tensor, i.e. (in index notation)
        * @f[
        *  \textrm{det} \mathbf{F}^{-1} \; \left[ \chi\left(\bullet\right)^{\sharp} \right]_{ijkl}
@@ -656,7 +656,7 @@ namespace Physics
 //@{
 
       /**
-       * Returns the result of the pull back transformation on a contravariant
+       * Return the result of the pull back transformation on a contravariant
        * vector, i.e.
        * @f[
        *  \textrm{det} \mathbf{F} \; \chi^{-1}\left(\bullet\right)^{\sharp}
@@ -675,7 +675,7 @@ namespace Physics
                  const Tensor<2,dim,Number> &F);
 
       /**
-       * Returns the result of the pull back transformation on a rank-2
+       * Return the result of the pull back transformation on a rank-2
        * contravariant tensor, i.e.
        * @f[
        *  \textrm{det} \mathbf{F} \; \chi^{-1}\left(\bullet\right)^{\sharp}
@@ -694,7 +694,7 @@ namespace Physics
                  const Tensor<2,dim,Number> &F);
 
       /**
-       * Returns the result of the pull back transformation on a rank-2
+       * Return the result of the pull back transformation on a rank-2
        * contravariant symmetric tensor, i.e.
        * @f[
        *  \textrm{det} \mathbf{F} \; \chi^{-1}\left(\bullet\right)^{\sharp}
@@ -713,7 +713,7 @@ namespace Physics
                  const Tensor<2,dim,Number>          &F);
 
       /**
-       * Returns the result of the pull back transformation on a rank-4
+       * Return the result of the pull back transformation on a rank-4
        * contravariant tensor, i.e. (in index notation)
        * @f[
        *  \textrm{det} \mathbf{F} \; \left[ \chi^{-1}\left(\bullet\right)^{\sharp} \right]_{IJKL}
@@ -732,7 +732,7 @@ namespace Physics
                  const Tensor<2,dim,Number> &F);
 
       /**
-       * Returns the result of the pull back transformation on a rank-4
+       * Return the result of the pull back transformation on a rank-4
        * contravariant symmetric tensor, i.e. (in index notation)
        * @f[
        *  \textrm{det} \mathbf{F} \; \left[ \chi^{-1}\left(\bullet\right)^{\sharp} \right]_{IJKL}
@@ -759,7 +759,7 @@ namespace Physics
 //@{
 
     /**
-     * Returns the result of applying Nanson's formula for the transformation
+     * Return the result of applying Nanson's formula for the transformation
      * of the material surface area element $d\mathbf{A}$ to the current
      * surfaces area element $d\mathbf{a}$ under the nonlinear transformation
      * map $\mathbf{x} = \boldsymbol{\varphi} \left( \mathbf{X} \right)$.
@@ -780,7 +780,7 @@ namespace Physics
      *
      * @dealiiHolzapfelA{75,2.55} @dealiiWriggersA{23,3.11}
      */
-    template<int dim, typename Number>
+    template <int dim, typename Number>
     Tensor<1,dim,Number>
     nansons_formula (const Tensor<1,dim,Number> &N,
                      const Tensor<2,dim,Number> &F);
@@ -818,7 +818,7 @@ namespace internal
       transformation_contraction (const Tensor<2,dim,Number> &T,
                                   const Tensor<2,dim,Number> &F)
       {
-        return contract<1,1>(F,contract<1,0>(F, T));
+        return contract<1,0>(F,contract<1,1>(T, F));
       }
 
 
@@ -852,9 +852,26 @@ namespace internal
       transformation_contraction (const Tensor<4,dim,Number> &H,
                                   const Tensor<2,dim,Number> &F)
       {
-        // Its significantly quicker (in 3d) to push forward
+        // This contraction order and indexing might look a bit dubious, so a
+        // quick explanation as to what's going on is probably in order:
+        //
+        // When the contract() function operates on the inner indices, the
+        // result has the inner index and outer index transposed, i.e.
+        // contract<2,1>(H,F) implies
+        // T_{IJLk} = (H_{IJMN} F_{mM}) \delta_{mL} \delta_{Nk}
+        // rather than T_{IJkL} (the desired result).
+        // So, in effect, contraction of the 3rd (inner) index with F as the
+        // second argument results in its transposition with respect to its
+        // adjacent neighbor. This is due to the position of the argument F,
+        // leading to the free index being on the right hand side of the result.
+        // However, given that we can do two transformations from the LHS of H
+        // and two from the right we can undo the otherwise erroneous
+        // swapping of the outer indices upon application of the second
+        // sets of contractions.
+        //
+        // Note: Its significantly quicker (in 3d) to push forward
         // each index individually
-        return contract<1,3>(F,contract<1,2>(F,contract<1,1>(F,contract<1,0>(F, H))));
+        return contract<1,1>(F,contract<1,1>(F,contract<2,1>(contract<2,1>(H,F), F)));
       }
 
 
@@ -865,40 +882,41 @@ namespace internal
       transformation_contraction (const dealii::SymmetricTensor<4,dim,Number> &H,
                                   const Tensor<2,dim,Number>                  &F)
       {
-        // Its significantly quicker (in 3d) to push forward
+        // The first and last transformation operations respectively
+        // break and recover the symmetry properties of the tensors.
+        // We also want to perform a minimal number of operations here
+        // and avoid some complications related to the transposition of
+        // tensor indices when contracting inner indices using the contract()
+        // function. (For an explanation of the contraction operations,
+        // please see the note in the equivalent function for standard
+        // Tensors.) So what we'll do here is manually perform the first
+        // and last contractions that break/recover the tensor symmetries
+        // on the inner indices, and use the contract() function only on
+        // the outer indices.
+        //
+        // Note: Its significantly quicker (in 3d) to push forward
         // each index individually
 
-        Tensor<4,dim,Number> tmp_1;
-        for (unsigned int i=0; i<dim; ++i)
-          for (unsigned int J=0; J<dim; ++J)
-            for (unsigned int K=0; K<dim; ++K)
-              for (unsigned int L=0; L<dim; ++L)
-                for (unsigned int I=0; I<dim; ++I)
-                  tmp_1[i][J][K][L] += F[i][I] * H[I][J][K][L];
-
-        Tensor<4,dim,Number> tmp_2;
-        for (unsigned int i=0; i<dim; ++i)
+        // Push forward (inner) index 1
+        Tensor<4,dim,Number> tmp;
+        for (unsigned int I=0; I<dim; ++I)
           for (unsigned int j=0; j<dim; ++j)
             for (unsigned int K=0; K<dim; ++K)
               for (unsigned int L=0; L<dim; ++L)
                 for (unsigned int J=0; J<dim; ++J)
-                  tmp_2[i][j][K][L] += F[j][J] * tmp_1[i][J][K][L];
+                  tmp[I][j][K][L] += F[j][J] * H[I][J][K][L];
 
-        tmp_1 = 0.0;
-        for (unsigned int i=0; i<dim; ++i)
-          for (unsigned int j=0; j<dim; ++j)
-            for (unsigned int k=0; k<dim; ++k)
-              for (unsigned int L=0; L<dim; ++L)
-                for (unsigned int K=0; K<dim; ++K)
-                  tmp_1[i][j][k][L] += F[k][K] * tmp_2[i][j][K][L];
+        // Push forward (outer) indices 0 and 3
+        tmp = contract<1,0>(F, contract<3,1>(tmp,F));
 
+        // Push forward (inner) index 2
         dealii::SymmetricTensor<4,dim,Number> out;
         for (unsigned int i=0; i<dim; ++i)
           for (unsigned int j=i; j<dim; ++j)
             for (unsigned int k=0; k<dim; ++k)
               for (unsigned int l=k; l<dim; ++l)
-                for (unsigned int L=0; L<dim; ++L)
-                  out[i][j][k][l] += F[l][L] * tmp_1[i][j][k][L];
+                for (unsigned int K=0; K<dim; ++K)
+                  out[i][j][k][l] += F[k][K] * tmp[i][j][K][l];
 
         return out;
       }
@@ -908,16 +926,16 @@ namespace internal
 
 
 
-template<typename Number>
+template <typename Number>
 Tensor<2,2,Number>
 Physics::Transformations::Rotations::rotation_matrix_2d (const Number &angle)
 {
   const double rotation[2][2]
   = {{
-      std::cos(angle) , -std::sin(angle)
+      std::cos(angle), -std::sin(angle)
     },
     {
-      std::sin(angle) , std::cos(angle)
+      std::sin(angle), std::cos(angle)
     }
   };
   return Tensor<2,2> (rotation);
@@ -925,7 +943,7 @@ Physics::Transformations::Rotations::rotation_matrix_2d (const Number &angle)
 
 
 
-template<typename Number>
+template <typename Number>
 Tensor<2,3,Number>
 Physics::Transformations::Rotations::rotation_matrix_3d (const Point<3,Number> &axis,
                                                          const Number          &angle)
@@ -1287,7 +1305,7 @@ Physics::Transformations::Piola::pull_back (const SymmetricTensor<4,dim,Number>
 
 
 
-template<int dim, typename Number>
+template <int dim, typename Number>
 inline Tensor<1,dim,Number>
 Physics::Transformations::nansons_formula (const Tensor<1,dim,Number> &N,
                                            const Tensor<2,dim,Number> &F)
