@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 2010 - 2016 by the deal.II authors
+// Copyright (C) 2010 - 2017 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -27,31 +27,29 @@
 #include <deal.II/lac/la_vector.h>
 #include <deal.II/lac/la_parallel_vector.h>
 #include <deal.II/lac/la_parallel_block_vector.h>
-#include <deal.II/lac/petsc_vector.h>
-#include <deal.II/lac/petsc_block_vector.h>
+#include <deal.II/lac/petsc_parallel_vector.h>
+#include <deal.II/lac/petsc_parallel_block_vector.h>
 #include <deal.II/lac/trilinos_vector.h>
-#include <deal.II/lac/trilinos_block_vector.h>
+#include <deal.II/lac/trilinos_parallel_block_vector.h>
 
 
 DEAL_II_NAMESPACE_OPEN
 
 namespace Algorithms
 {
-  OperatorBase::~OperatorBase()
-  {}
-
   void OperatorBase::notify(const Event &e)
   {
     notifications += e;
   }
 
+
+
   void
   OperatorBase::clear_events ()
   {
     notifications.clear();
   }
 
-
 #include "operator.inst"
 }
 
