@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 2001 - 2016 by the deal.II authors
+// Copyright (C) 2001 - 2017 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -37,11 +37,6 @@ AutoDerivativeFunction (const double hh,
 }
 
 
-template <int dim>
-AutoDerivativeFunction<dim>::~AutoDerivativeFunction ()
-{}
-
-
 
 template <int dim>
 void
