@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 1998 - 2017 by the deal.II authors
+// Copyright (C) 1998 - 2018 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -74,13 +74,13 @@ ExceptionBase::ExceptionBase ()
   function(""),
   cond(""),
   exc(""),
-  stacktrace (NULL),
+  stacktrace (nullptr),
   n_stacktrace_frames (0),
   what_str("")
 {
 #ifdef DEAL_II_HAVE_GLIBC_STACKTRACE
   for (unsigned int i=0; i<sizeof(raw_stacktrace)/sizeof(raw_stacktrace[0]); ++i)
-    raw_stacktrace[i] = NULL;
+    raw_stacktrace[i] = nullptr;
 #endif
 }
 
@@ -93,22 +93,22 @@ ExceptionBase::ExceptionBase (const ExceptionBase &exc)
   function(exc.function),
   cond(exc.cond),
   exc(exc.exc),
-  stacktrace (NULL), // don't copy stacktrace to avoid double de-allocation problem
+  stacktrace (nullptr), // don't copy stacktrace to avoid double de-allocation problem
   n_stacktrace_frames (0),
   what_str("") // don't copy the error message, it gets generated dynamically by what()
 {
 #ifdef DEAL_II_HAVE_GLIBC_STACKTRACE
   for (unsigned int i=0; i<sizeof(raw_stacktrace)/sizeof(raw_stacktrace[0]); ++i)
-    raw_stacktrace[i] = NULL;
+    raw_stacktrace[i] = nullptr;
 #endif
 }
 
 
 
-ExceptionBase::~ExceptionBase () DEAL_II_NOEXCEPT
+ExceptionBase::~ExceptionBase () noexcept
 {
   free (stacktrace); // free(NULL) is allowed
-  stacktrace = NULL;
+  stacktrace = nullptr;
 }
 
 
@@ -135,7 +135,7 @@ void ExceptionBase::set_fields (const char *f,
 #endif
 }
 
-const char *ExceptionBase::what() const DEAL_II_NOEXCEPT
+const char *ExceptionBase::what() const noexcept
 {
   // If no error c_string was generated so far, do it now:
   if (what_str == "")
@@ -186,7 +186,7 @@ void ExceptionBase::print_exc_data (std::ostream &out) const
   // between numerous "..."-enclosed strings that the preprocessor
   // collates into a single string, making it awkward to read. Consequently,
   // elide this text if the message was generated via an ExcMessage object
-  if (std::strstr(cond, "dealii::ExcMessage") != NULL)
+  if (std::strstr(cond, "dealii::ExcMessage") != nullptr)
     out << "The name and call sequence of the exception was:" << std::endl
         << "    " << exc  << std::endl;
 
@@ -244,13 +244,16 @@ void ExceptionBase::print_stack_trace (std::ostream &out) const
       std::string functionname = stacktrace_entry.substr (pos_start+1,
                                                           pos_end-pos_start-1);
 
+      stacktrace_entry = stacktrace_entry.substr(0, pos_start);
+      stacktrace_entry += ": ";
+
       // demangle, and if successful replace old mangled string by
       // unmangled one (skipping address and offset). treat "main"
       // differently, since it is apparently demangled as "unsigned int"
       // for unknown reasons :-) if we can, demangle the function name
 #ifdef DEAL_II_HAVE_LIBSTDCXX_DEMANGLER
       int         status;
-      char *p = abi::__cxa_demangle(functionname.c_str(), 0, 0, &status);
+      char *p = abi::__cxa_demangle(functionname.c_str(), nullptr, nullptr, &status);
 
       if ((status == 0) && (functionname != "main"))
         {
@@ -266,28 +269,16 @@ void ExceptionBase::print_stack_trace (std::ostream &out) const
             realname.erase (realname.find (", boost::tuples::null_type>"),
                             std::string (", boost::tuples::null_type").size());
 
-          stacktrace_entry = stacktrace_entry.substr(0, pos_start)
-                             +
-                             ": "
-                             +
-                             realname;
+          stacktrace_entry += realname;
         }
       else
-        stacktrace_entry = stacktrace_entry.substr(0, pos_start)
-                           +
-                           ": "
-                           +
-                           functionname;
+        stacktrace_entry += functionname;
 
       free (p);
 
 #else
 
-      stacktrace_entry = stacktrace_entry.substr(0, pos_start)
-                         +
-                         ": "
-                         +
-                         functionname;
+      stacktrace_entry += functionname;
 #endif
 
       // then output what we have
@@ -399,37 +390,173 @@ namespace StandardExceptions
 }
 #endif // DEAL_II_WITH_MPI
 
+namespace
+{
+  [[noreturn]]
+  void internal_abort (const ExceptionBase &exc) noexcept
+  {
+    // first print the error
+    std::cerr << exc.what() << std::endl;
 
+    // then bail out. if in MPI mode, bring down the entire
+    // house by asking the MPI system to do a best-effort
+    // operation at also terminating all of the other MPI
+    // processes. this is useful because if only one process
+    // runs into an assertion, then that may lead to deadlocks
+    // if the others don't recognize this, or at the very least
+    // delay their termination until they realize that their
+    // communication with the job that died times out.
+    //
+    // Unlike std::abort(), MPI_Abort() unfortunately doesn't break when
+    // running inside a debugger like GDB, so only use this strategy if
+    // absolutely necessary and inform the user how to use a debugger.
+#ifdef DEAL_II_WITH_MPI
+    int is_initialized;
+    MPI_Initialized(&is_initialized);
+    if (is_initialized)
+      {
+        // do the same as in Utilities::MPI::n_mpi_processes() here,
+        // but without error checking to not throw again.
+        int n_proc=1;
+        MPI_Comm_size (MPI_COMM_WORLD, &n_proc);
+        if (n_proc>1)
+          {
+            std::cerr << "Calling MPI_Abort now.\n"
+            << "To break execution in a GDB session, execute 'break MPI_Abort' before "
+            << "running. You can also put the following into your ~/.gdbinit:\n"
+            << "  set breakpoint pending on\n"
+            << "  break MPI_Abort\n"
+            << "  set breakpoint pending auto" << std::endl;
+
+            MPI_Abort (MPI_COMM_WORLD,
+            /* return code = */ 255);
+          }
+      }
+#endif
+    std::abort();
+  }
+}
 
 namespace deal_II_exceptions
 {
   namespace internals
   {
-
-    void abort (const ExceptionBase &exc, bool nothrow /*= false*/)
+    void do_issue_error_nothrow (const ExceptionBase &exc) noexcept
     {
       if (dealii::deal_II_exceptions::abort_on_exception)
-        {
-          // Print the error message and bail out:
-          std::cerr << exc.what() << std::endl;
-          std::abort();
-        }
-      else if (nothrow)
-        {
-          // We are not allowed to throw, and not allowed to abort.
-          // Just print the exception name to deallog and continue
-          // normally:
-          deallog << "Exception: " << exc.get_exc_name() << std::endl;
-        }
+        internal_abort(exc);
       else
         {
-          // We are not allowed to abort, so just throw the error so just
-          // throw the error so just throw the error so just throw the
-          // error:
+          // We are not allowed to throw, and not allowed to abort.
+          // Just print the exception name to deallog and continue normally:
+          deallog << "Exception: " << exc.get_exc_name() << std::endl;
+          deallog << exc.what() << std::endl;
+        }
+    }
+
+
+
+    [[noreturn]]
+    void abort (const ExceptionBase &exc)
+    {
+      if (dealii::deal_II_exceptions::abort_on_exception)
+        internal_abort(exc);
+      else
+        {
+          // We are not allowed to abort, so just throw the error:
           throw exc;
         }
     }
 
+
+
+#ifdef DEAL_II_WITH_CUDA
+    std::string get_cusparse_error_string(const cusparseStatus_t error_code)
+    {
+      switch (error_code)
+        {
+        case CUSPARSE_STATUS_NOT_INITIALIZED:
+        {
+          return "The cuSPARSE library was not initialized";
+        }
+        case CUSPARSE_STATUS_ALLOC_FAILED:
+        {
+          return "Resource allocation failed inside the cuSPARSE library";
+        }
+        case CUSPARSE_STATUS_INVALID_VALUE:
+        {
+          return "An unsupported value of parameter was passed to the function";
+        }
+        case CUSPARSE_STATUS_ARCH_MISMATCH:
+        {
+          return "The function requires a feature absent from the device architecture";
+        }
+        case CUSPARSE_STATUS_MAPPING_ERROR:
+        {
+          return "An access to GPU memory space failed";
+        }
+        case CUSPARSE_STATUS_EXECUTION_FAILED:
+        {
+          return "The GPU program failed to execute";
+        }
+        case CUSPARSE_STATUS_INTERNAL_ERROR:
+        {
+          return "An internal cuSPARSE operation failed";
+        }
+        case CUSPARSE_STATUS_MATRIX_TYPE_NOT_SUPPORTED:
+        {
+          return "The matrix type is not supported by this function";
+        }
+        default:
+        {
+          return "Unknown error";
+        }
+        }
+    }
+
+
+
+    std::string get_cusolver_error_string(cusolverStatus_t error_code)
+    {
+      std::string message;
+      switch (error_code)
+        {
+        case CUSOLVER_STATUS_NOT_INITIALIZED:
+        {
+          return "The cuSolver library was not initialized";
+        }
+        case CUSOLVER_STATUS_ALLOC_FAILED:
+        {
+          return "Resource allocation failed inside the cuSolver library";
+        }
+        case CUSOLVER_STATUS_INVALID_VALUE:
+        {
+          return "An unsupported value of a parameter was passed to the function";
+        }
+        case CUSOLVER_STATUS_ARCH_MISMATCH:
+        {
+          return "The function requires a feature absent from the device architecture";
+        }
+        case CUSOLVER_STATUS_EXECUTION_FAILED:
+        {
+          return "The GPU program failed to execute";
+        }
+        case CUSOLVER_STATUS_INTERNAL_ERROR:
+        {
+          return "An internal cuSolver operation failed";
+        }
+        case CUSOLVER_STATUS_MATRIX_TYPE_NOT_SUPPORTED:
+        {
+          return "The matrix type is not supported by this function";
+        }
+        default:
+        {
+          return "Unknown error";
+        }
+        }
+    }
+#endif
+
   } /*namespace internals*/
 } /*namespace deal_II_exceptions*/
 
