@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 2007 - 2015 by the deal.II authors
+// Copyright (C) 2007 - 2017 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -27,7 +27,7 @@ DEAL_II_NAMESPACE_OPEN
 namespace Functions
 {
 
-  template<int dim>
+  template <int dim>
   FlowFunction<dim>::FlowFunction()
     :
     Function<dim>(dim+1),
@@ -37,12 +37,8 @@ namespace Functions
   {}
 
 
-  template<int dim>
-  FlowFunction<dim>::~FlowFunction()
-  {}
 
-
-  template<int dim>
+  template <int dim>
   void
   FlowFunction<dim>::pressure_adjustment(double p)
   {
@@ -50,7 +46,7 @@ namespace Functions
   }
 
 
-  template<int dim>
+  template <int dim>
   void FlowFunction<dim>::vector_value_list (
     const std::vector<Point<dim> > &points,
     std::vector<Vector<double> >   &values) const
@@ -75,7 +71,7 @@ namespace Functions
   }
 
 
-  template<int dim>
+  template <int dim>
   void FlowFunction<dim>::vector_value (
     const Point<dim> &point,
     Vector<double> &value) const
@@ -99,7 +95,7 @@ namespace Functions
   }
 
 
-  template<int dim>
+  template <int dim>
   double FlowFunction<dim>::value (
     const Point<dim> &point,
     const unsigned int comp) const
@@ -121,7 +117,7 @@ namespace Functions
   }
 
 
-  template<int dim>
+  template <int dim>
   void FlowFunction<dim>::vector_gradient_list (
     const std::vector<Point<dim> > &points,
     std::vector<std::vector<Tensor<1,dim> > > &values) const
@@ -146,7 +142,7 @@ namespace Functions
   }
 
 
-  template<int dim>
+  template <int dim>
   void FlowFunction<dim>::vector_laplacian_list (
     const std::vector<Point<dim> > &points,
     std::vector<Vector<double> > &values) const
@@ -171,7 +167,7 @@ namespace Functions
   }
 
 
-  template<int dim>
+  template <int dim>
   std::size_t
   FlowFunction<dim>::memory_consumption () const
   {
@@ -182,7 +178,7 @@ namespace Functions
 
 //----------------------------------------------------------------------//
 
-  template<int dim>
+  template <int dim>
   PoisseuilleFlow<dim>::PoisseuilleFlow(const double r,
                                         const double Re)
     :
@@ -192,12 +188,8 @@ namespace Functions
   }
 
 
-  template<int dim>
-  PoisseuilleFlow<dim>::~PoisseuilleFlow()
-  {}
 
-
-  template<int dim>
+  template <int dim>
   void PoisseuilleFlow<dim>::vector_values (
     const std::vector<Point<dim> > &points,
     std::vector<std::vector<double> > &values) const
@@ -232,7 +224,7 @@ namespace Functions
 
 
 
-  template<int dim>
+  template <int dim>
   void PoisseuilleFlow<dim>::vector_gradients (
     const std::vector<Point<dim> > &points,
     std::vector<std::vector<Tensor<1,dim> > > &values) const
@@ -263,7 +255,7 @@ namespace Functions
 
 
 
-  template<int dim>
+  template <int dim>
   void PoisseuilleFlow<dim>::vector_laplacians (
     const std::vector<Point<dim> > &points,
     std::vector<std::vector<double> > &values) const
@@ -281,19 +273,15 @@ namespace Functions
 
 //----------------------------------------------------------------------//
 
-  template<int dim>
+  template <int dim>
   StokesCosine<dim>::StokesCosine(const double nu, const double r)
     :
     viscosity(nu), reaction(r)
   {}
 
 
-  template<int dim>
-  StokesCosine<dim>::~StokesCosine()
-  {}
 
-
-  template<int dim>
+  template <int dim>
   void
   StokesCosine<dim>::set_parameters(const double nu, const double r)
   {
@@ -302,7 +290,7 @@ namespace Functions
   }
 
 
-  template<int dim>
+  template <int dim>
   void StokesCosine<dim>::vector_values (
     const std::vector<Point<dim> > &points,
     std::vector<std::vector<double> > &values) const
@@ -349,7 +337,7 @@ namespace Functions
 
 
 
-  template<int dim>
+  template <int dim>
   void StokesCosine<dim>::vector_gradients (
     const std::vector<Point<dim> > &points,
     std::vector<std::vector<Tensor<1,dim> > > &values) const
@@ -413,7 +401,7 @@ namespace Functions
 
 
 
-  template<int dim>
+  template <int dim>
   void StokesCosine<dim>::vector_laplacians (
     const std::vector<Point<dim> > &points,
     std::vector<std::vector<double> > &values) const
@@ -659,9 +647,6 @@ namespace Functions
   }
 
 
-  Kovasznay::~Kovasznay()
-  {}
-
 
   void Kovasznay::vector_values (
     const std::vector<Point<2> > &points,
