@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 2016 - 2017 by the deal.II authors
+// Copyright (C) 2016 - 2018 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -40,10 +40,10 @@ namespace GeometricUtilities
 
 
     template <int dim>
-    std_cxx11::array<double,dim>
+    std::array<double,dim>
     to_spherical(const Point<dim> &position)
     {
-      std_cxx11::array<double,dim> scoord;
+      std::array<double,dim> scoord;
 
       // radius
       scoord[0] = position.norm();
@@ -67,7 +67,7 @@ namespace GeometricUtilities
 
     template <std::size_t dim>
     Point<dim>
-    from_spherical(const std_cxx11::array<double,dim> &scoord)
+    from_spherical(const std::array<double,dim> &scoord)
     {
       Point<dim> ccoord;
 
@@ -103,11 +103,8 @@ namespace GeometricUtilities
       return ccoord;
     }
 
-
-    template Point<2> from_spherical<2>(const std_cxx11::array<double,2> &);
-    template Point<3> from_spherical<3>(const std_cxx11::array<double,3> &);
-    template std_cxx11::array<double,2> to_spherical<2>(const Point<2> &);
-    template std_cxx11::array<double,3> to_spherical<3>(const Point<3> &);
+    // explicit instantiations
+#include "geometric_utilities.inst"
 
   }
 }
