@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 2005 - 2017 by the deal.II authors
+// Copyright (C) 2005 - 2018 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -41,16 +41,21 @@ DEAL_II_NAMESPACE_OPEN
 IndexSet::IndexSet (const Epetra_Map &map)
   :
   is_compressed (true),
-  index_space_size (map.NumGlobalElements64()),
+  index_space_size (1+map.MaxAllGID64()),
   largest_range (numbers::invalid_unsigned_int)
 {
+  Assert(
+    map.MinAllGID64() == 0,
+    ExcMessage("The Epetra_Map does not contain the global index 0, which "
+               "means some entries are not present on any processor."));
+
   // For a contiguous map, we do not need to go through the whole data...
   if (map.LinearMap())
     add_range(size_type(map.MinMyGID64()), size_type(map.MaxMyGID64()+1));
   else
     {
       const size_type n_indices = map.NumMyElements();
-      size_type *indices = (size_type *)map.MyGlobalElements64();
+      size_type *indices = reinterpret_cast<size_type *>(map.MyGlobalElements64());
       add_indices(indices, indices+n_indices);
     }
   compress();
@@ -63,16 +68,21 @@ IndexSet::IndexSet (const Epetra_Map &map)
 IndexSet::IndexSet (const Epetra_Map &map)
   :
   is_compressed (true),
-  index_space_size (map.NumGlobalElements()),
+  index_space_size (1+map.MaxAllGID()),
   largest_range (numbers::invalid_unsigned_int)
 {
+  Assert(
+    map.MinAllGID() == 0,
+    ExcMessage("The Epetra_Map does not contain the global index 0, which "
+               "means some entries are not present on any processor."));
+
   // For a contiguous map, we do not need to go through the whole data...
   if (map.LinearMap())
     add_range(size_type(map.MinMyGID()), size_type(map.MaxMyGID()+1));
   else
     {
       const size_type n_indices = map.NumMyElements();
-      unsigned int *indices = (unsigned int *)map.MyGlobalElements();
+      unsigned int *indices = reinterpret_cast<unsigned int *>(map.MyGlobalElements());
       add_indices(indices, indices+n_indices);
     }
   compress();
@@ -419,7 +429,7 @@ IndexSet::add_indices(const IndexSet &other,
         }
       else if (r1 == ranges.end() || (r2->end+offset) < r1->begin)
         {
-          new_ranges.push_back(Range(r2->begin+offset,r2->end+offset));
+          new_ranges.emplace_back(r2->begin+offset, r2->end+offset);
           ++r2;
         }
       else
@@ -459,14 +469,18 @@ IndexSet::write(std::ostream &out) const
 void
 IndexSet::read(std::istream &in)
 {
-  size_type s;
-  unsigned int numranges;
+  AssertThrow (in, ExcIO());
 
-  in >> s >> numranges;
+  size_type s;
+  unsigned int n_ranges;
+
+  in >> s >> n_ranges;
   ranges.clear();
   set_size(s);
-  for (unsigned int i=0; i<numranges; ++i)
+  for (unsigned int i=0; i<n_ranges; ++i)
     {
+      AssertThrow (in, ExcIO());
+
       size_type b, e;
       in >> b >> e;
       add_range(b,e);
@@ -562,7 +576,7 @@ IndexSet::make_trilinos_map (const MPI_Comm &communicator,
     }
 #endif
 
-  // Find out if the IndexSet is ascending and 1:1. This correspinds to a
+  // Find out if the IndexSet is ascending and 1:1. This corresponds to a
   // linear EpetraMap. Overlapping IndexSets are never 1:1.
   const bool linear = overlapping ? false : is_ascending_and_one_to_one(communicator);
 
@@ -584,9 +598,9 @@ IndexSet::make_trilinos_map (const MPI_Comm &communicator,
                          TrilinosWrappers::types::int_type(n_elements()),
                          (n_elements() > 0
                           ?
-                          reinterpret_cast<TrilinosWrappers::types::int_type *>(&indices[0])
+                          reinterpret_cast<TrilinosWrappers::types::int_type *>(indices.data())
                           :
-                          0),
+                          nullptr),
                          0,
 #ifdef DEAL_II_WITH_MPI
                          Epetra_MpiComm(communicator)
@@ -623,24 +637,15 @@ IndexSet::is_ascending_and_one_to_one (const MPI_Comm &communicator) const
       : numbers::invalid_dof_index ;
 
   const unsigned int my_rank = Utilities::MPI::this_mpi_process(communicator);
-  const unsigned int n_ranks = Utilities::MPI::n_mpi_processes(communicator);
-  // first gather all information on process 0
-  const unsigned int gather_size = (my_rank==0)?n_ranks:1;
-  std::vector<types::global_dof_index> global_dofs(gather_size);
-
-  int ierr = MPI_Gather(&first_local_dof, 1, DEAL_II_DOF_INDEX_MPI_TYPE,
-                        &(global_dofs[0]), 1, DEAL_II_DOF_INDEX_MPI_TYPE, 0,
-                        communicator);
-  AssertThrowMPI(ierr);
+  const std::vector<types::global_dof_index> global_dofs = Utilities::MPI::gather(communicator,first_local_dof,0);
 
   if (my_rank == 0)
     {
       // find out if the received std::vector is ascending
-      types::global_dof_index old_dof = global_dofs[0];
       types::global_dof_index index = 0;
       while (global_dofs[index] == numbers::invalid_dof_index)
         ++index;
-      old_dof = global_dofs[index++];
+      types::global_dof_index old_dof = global_dofs[index++];
       for (; index<global_dofs.size(); ++index)
         {
           const types::global_dof_index new_dof = global_dofs[index];
@@ -659,7 +664,7 @@ IndexSet::is_ascending_and_one_to_one (const MPI_Comm &communicator) const
 
   // now broadcast the result
   int is_ascending = is_globally_ascending ? 1 : 0;
-  ierr = MPI_Bcast(&is_ascending, 1, MPI_INT, 0, communicator);
+  int ierr = MPI_Bcast(&is_ascending, 1, MPI_INT, 0, communicator);
   AssertThrowMPI(ierr);
 
   return (is_ascending==1);
