@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 2000 - 2015 by the deal.II authors
+// Copyright (C) 2000 - 2018 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -25,6 +25,7 @@
 #  include <sys/sysctl.h>
 #endif
 
+#include <algorithm>
 
 #ifdef DEAL_II_WITH_THREADS
 #  include <deal.II/base/thread_management.h>
@@ -112,7 +113,7 @@ void MultithreadInfo::set_thread_limit(const unsigned int max_threads)
   // then also see if something was given in the environment
   {
     const char *penv = getenv ("DEAL_II_NUM_THREADS");
-    if (penv!=NULL)
+    if (penv!=nullptr)
       {
         unsigned int max_threads_env = numbers::invalid_unsigned_int;
         try
@@ -188,11 +189,6 @@ bool MultithreadInfo::is_running_single_threaded()
 }
 
 
-MultithreadInfo::MultithreadInfo ()
-{}
-
-
-
 std::size_t
 MultithreadInfo::memory_consumption ()
 {
@@ -202,6 +198,19 @@ MultithreadInfo::memory_consumption ()
 }
 
 
+void
+MultithreadInfo::initialize_multithreading()
+{
+  static bool done = false;
+  if (done)
+    return;
+
+  MultithreadInfo::set_thread_limit (numbers::invalid_unsigned_int);
+  done = true;
+}
+
+
+
 const unsigned int MultithreadInfo::n_cpus = MultithreadInfo::get_n_cpus();
 unsigned int MultithreadInfo::n_max_threads = numbers::invalid_unsigned_int;
 
@@ -214,7 +223,7 @@ namespace
   {
     DoOnce ()
     {
-      MultithreadInfo::set_thread_limit (numbers::invalid_unsigned_int);
+      MultithreadInfo::initialize_multithreading();
     }
   } do_once;
 }
