@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 1999 - 2016 by the deal.II authors
+// Copyright (C) 1999 - 2017 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -14,6 +14,7 @@
 // ---------------------------------------------------------------------
 
 #include <deal.II/base/partitioner.h>
+#include <deal.II/base/partitioner.templates.h>
 
 DEAL_II_NAMESPACE_OPEN
 
@@ -27,6 +28,7 @@ namespace Utilities
       local_range_data (std::pair<types::global_dof_index, types::global_dof_index> (0, 0)),
       n_ghost_indices_data (0),
       n_import_indices_data (0),
+      n_ghost_indices_in_larger_set(0),
       my_pid (0),
       n_procs (1),
       communicator (MPI_COMM_SELF),
@@ -43,6 +45,7 @@ namespace Utilities
       local_range_data (std::pair<types::global_dof_index, types::global_dof_index> (0, size)),
       n_ghost_indices_data (0),
       n_import_indices_data (0),
+      n_ghost_indices_in_larger_set(0),
       my_pid (0),
       n_procs (1),
       communicator (MPI_COMM_SELF),
@@ -62,6 +65,7 @@ namespace Utilities
       global_size (static_cast<types::global_dof_index>(locally_owned_indices.size())),
       n_ghost_indices_data (0),
       n_import_indices_data (0),
+      n_ghost_indices_in_larger_set(0),
       my_pid (0),
       n_procs (1),
       communicator (communicator_in),
@@ -79,6 +83,7 @@ namespace Utilities
       global_size (static_cast<types::global_dof_index>(locally_owned_indices.size())),
       n_ghost_indices_data (0),
       n_import_indices_data (0),
+      n_ghost_indices_in_larger_set(0),
       my_pid (0),
       n_procs (1),
       communicator (communicator_in),
@@ -105,7 +110,7 @@ namespace Utilities
     void
     Partitioner::set_owned_indices (const IndexSet &locally_owned_indices)
     {
-      if (Utilities::System::job_supports_mpi() == true)
+      if (Utilities::MPI::job_supports_mpi() == true)
         {
           my_pid = Utilities::MPI::this_mpi_process(communicator);
           n_procs = Utilities::MPI::n_mpi_processes(communicator);
@@ -139,7 +144,8 @@ namespace Utilities
 
 
     void
-    Partitioner::set_ghost_indices (const IndexSet &ghost_indices_in)
+    Partitioner::set_ghost_indices (const IndexSet &ghost_indices_in,
+                                    const IndexSet &larger_ghost_index_set)
     {
       // Set ghost indices from input. To be sure that no entries from the
       // locally owned range are present, subtract the locally owned indices
@@ -183,7 +189,7 @@ namespace Utilities
       // Allow non-zero start index for the vector. send this data to all
       // processors
       first_index[0] = local_range_data.first;
-      int ierr = MPI_Bcast(&first_index[0], 1, DEAL_II_DOF_INDEX_MPI_TYPE,
+      int ierr = MPI_Bcast(first_index.data(), 1, DEAL_II_DOF_INDEX_MPI_TYPE,
                            0, communicator);
       AssertThrowMPI(ierr);
 
@@ -227,7 +233,7 @@ namespace Utilities
           // indices and then push back new values. When we are done, copy the
           // data to that field of the partitioner. This way, the variable
           // ghost_targets will have exactly the size we need, whereas the
-          // vector filled with push_back might actually be too long.
+          // vector filled with emplace_back might actually be too long.
           unsigned int current_proc = 0;
           ghost_indices_data.fill_index_vector (expanded_ghost_indices);
           types::global_dof_index current_index = expanded_ghost_indices[0];
@@ -247,8 +253,7 @@ namespace Utilities
                 {
                   ghost_targets_temp[n_ghost_targets-1].second =
                     iterator - ghost_targets_temp[n_ghost_targets-1].second;
-                  ghost_targets_temp.push_back(std::pair<unsigned int,
-                                               unsigned int>(current_proc,iterator));
+                  ghost_targets_temp.emplace_back (current_proc, iterator);
                   n_ghost_targets++;
                 }
             }
@@ -263,7 +268,7 @@ namespace Utilities
         for (unsigned int i=0; i<n_ghost_targets; i++)
           send_buffer[ghost_targets_data[i].first] = ghost_targets_data[i].second;
 
-        const int ierr = MPI_Alltoall (&send_buffer[0], 1, MPI_INT, &receive_buffer[0], 1,
+        const int ierr = MPI_Alltoall (send_buffer.data(), 1, MPI_INT, receive_buffer.data(), 1,
                                        MPI_INT, communicator);
         AssertThrowMPI(ierr);
 
@@ -274,9 +279,9 @@ namespace Utilities
           if (receive_buffer[i] > 0)
             {
               n_import_indices_data += receive_buffer[i];
-              import_targets_temp.push_back(std::pair<unsigned int,
-                                            unsigned int> (i, receive_buffer[i]));
+              import_targets_temp.emplace_back(i, receive_buffer[i]);
             }
+        // copy, don't move, to get deterministic memory usage.
         import_targets_data = import_targets_temp;
       }
 
@@ -315,7 +320,7 @@ namespace Utilities
         if (import_requests.size()>0)
           {
             const int ierr = MPI_Waitall (import_requests.size(),
-                                          &import_requests[0],
+                                          import_requests.data(),
                                           MPI_STATUSES_IGNORE);
             AssertThrowMPI(ierr);
           }
@@ -323,26 +328,32 @@ namespace Utilities
         // transform import indices to local index space and compress
         // contiguous indices in form of ranges
         {
-          types::global_dof_index last_index = numbers::invalid_dof_index-1;
+          import_indices_chunks_by_rank_data.resize(import_targets_data.size()+1);
+          import_indices_chunks_by_rank_data[0] = 0;
           std::vector<std::pair<unsigned int,unsigned int> > compressed_import_indices;
-          for (unsigned int i=0; i<n_import_indices_data; i++)
+          unsigned int shift = 0;
+          for (unsigned int p=0; p<import_targets_data.size(); ++p)
             {
-              Assert (expanded_import_indices[i] >= local_range_data.first &&
-                      expanded_import_indices[i] < local_range_data.second,
-                      ExcIndexRange(expanded_import_indices[i], local_range_data.first,
-                                    local_range_data.second));
-              types::global_dof_index new_index = (expanded_import_indices[i] -
-                                                   local_range_data.first);
-              Assert(new_index<numbers::invalid_unsigned_int,
-                     ExcNotImplemented());
-              if (new_index == last_index+1)
-                compressed_import_indices.back().second++;
-              else
+              types::global_dof_index last_index = numbers::invalid_dof_index-1;
+              for (unsigned int ii=0; ii<import_targets_data[p].second; ++ii)
                 {
-                  compressed_import_indices.push_back
-                  (std::pair<unsigned int,unsigned int>(new_index,new_index+1));
+                  const unsigned int i = shift + ii;
+                  Assert (expanded_import_indices[i] >= local_range_data.first &&
+                          expanded_import_indices[i] < local_range_data.second,
+                          ExcIndexRange(expanded_import_indices[i], local_range_data.first,
+                                        local_range_data.second));
+                  types::global_dof_index new_index = (expanded_import_indices[i] -
+                                                       local_range_data.first);
+                  Assert(new_index<numbers::invalid_unsigned_int,
+                         ExcNotImplemented());
+                  if (new_index == last_index+1)
+                    compressed_import_indices.back().second++;
+                  else
+                    compressed_import_indices.emplace_back (new_index,new_index+1);
+                  last_index = new_index;
                 }
-              last_index = new_index;
+              shift += import_targets_data[p].second;
+              import_indices_chunks_by_rank_data[p+1] = compressed_import_indices.size();
             }
           import_indices_data = compressed_import_indices;
 
@@ -357,7 +368,58 @@ namespace Utilities
 #endif
         }
       }
-#endif
+#endif // #ifdef DEAL_II_WITH_MPI
+
+      if (larger_ghost_index_set.size() == 0)
+        {
+          ghost_indices_subset_chunks_by_rank_data.clear();
+          ghost_indices_subset_data.emplace_back(local_size(),
+                                                 local_size()+n_ghost_indices());
+          n_ghost_indices_in_larger_set = n_ghost_indices_data;
+        }
+      else
+        {
+          AssertDimension(larger_ghost_index_set.size(), ghost_indices_data.size());
+          Assert((larger_ghost_index_set & locally_owned_range_data).n_elements()==0,
+                 ExcMessage("Ghost index set should not overlap with owned set."));
+          Assert((larger_ghost_index_set & ghost_indices_data) == ghost_indices_data,
+                 ExcMessage("Larger ghost index set must contain the tight "
+                            "ghost index set."));
+
+          n_ghost_indices_in_larger_set = larger_ghost_index_set.n_elements();
+
+          std::vector<unsigned int> expanded_numbering;
+          for (IndexSet::ElementIterator it=ghost_indices_data.begin();
+               it != ghost_indices_data.end(); ++it)
+            {
+              Assert(larger_ghost_index_set.is_element(*it),
+                     ExcMessage("The given larger ghost index set must contain"
+                                "all indices in the actual index set."));
+              expanded_numbering.push_back(larger_ghost_index_set.index_within_set(*it));
+            }
+
+          std::vector<std::pair<unsigned int,unsigned int> > ghost_indices_subset;
+          ghost_indices_subset_chunks_by_rank_data.resize(ghost_targets_data.size()+1);
+          ghost_indices_subset_chunks_by_rank_data[0] = 0;
+          unsigned int shift = 0;
+          for (unsigned int p=0; p<ghost_targets_data.size(); ++p)
+            {
+              unsigned int last_index = numbers::invalid_unsigned_int-1;
+              for (unsigned int ii=0; ii<ghost_targets_data[p].second; ii++)
+                {
+                  const unsigned int i = shift + ii;
+                  if (expanded_numbering[i] == last_index+1)
+                    ghost_indices_subset.back().second++;
+                  else
+                    ghost_indices_subset.emplace_back(expanded_numbering[i],
+                                                      expanded_numbering[i]+1);
+                  last_index = expanded_numbering[i];
+                }
+              shift += ghost_targets_data[p].second;
+              ghost_indices_subset_chunks_by_rank_data[p+1] = ghost_indices_subset.size();
+            }
+          ghost_indices_subset_data = ghost_indices_subset;
+        }
     }
 
 
@@ -406,6 +468,9 @@ namespace Utilities
       memory += MemoryConsumption::memory_consumption(ghost_targets_data);
       memory += MemoryConsumption::memory_consumption(import_targets_data);
       memory += MemoryConsumption::memory_consumption(import_indices_data);
+      memory += MemoryConsumption::memory_consumption(import_indices_chunks_by_rank_data);
+      memory += MemoryConsumption::memory_consumption(ghost_indices_subset_chunks_by_rank_data);
+      memory += MemoryConsumption::memory_consumption(ghost_indices_subset_data);
       memory += MemoryConsumption::memory_consumption(ghost_indices_data);
       return memory;
     }
@@ -415,4 +480,8 @@ namespace Utilities
 } // end of namespace Utilities
 
 
+
+// explicit instantiations from .templates.h file
+#include "partitioner.inst"
+
 DEAL_II_NAMESPACE_CLOSE
