@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 2000 - 2017 by the deal.II authors
+// Copyright (C) 2000 - 2018 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -13,11 +13,12 @@
 //
 // ---------------------------------------------------------------------
 
-#include <deal.II/base/polynomial.h>
-#include <deal.II/base/point.h>
 #include <deal.II/base/exceptions.h>
-#include <deal.II/base/thread_management.h>
+#include <deal.II/base/point.h>
+#include <deal.II/base/polynomial.h>
 #include <deal.II/base/quadrature_lib.h>
+#include <deal.II/base/std_cxx14/memory.h>
+#include <deal.II/base/thread_management.h>
 
 #include <cmath>
 #include <algorithm>
@@ -104,7 +105,7 @@ namespace Polynomials
   {
     Assert (values.size() > 0, ExcZero());
 
-    value(x,values.size()-1,&values[0]);
+    value(x,values.size()-1,values.data());
   }
 
 
@@ -370,11 +371,11 @@ namespace Polynomials
     // need to transform p into standard form as
     // well if necessary. copy the polynomial to
     // do this
-    std_cxx11::shared_ptr<Polynomial<number> > q_data;
-    const Polynomial<number> *q = 0;
+    std::unique_ptr<Polynomial<number> > q_data;
+    const Polynomial<number> *q = nullptr;
     if (p.in_lagrange_product_form == true)
       {
-        q_data.reset (new Polynomial<number>(p));
+        q_data = std_cxx14::make_unique<Polynomial<number>>(p);
         q_data->transform_into_standard_form();
         q = q_data.get();
       }
@@ -414,11 +415,11 @@ namespace Polynomials
     // need to transform p into standard form as
     // well if necessary. copy the polynomial to
     // do this
-    std_cxx11::shared_ptr<Polynomial<number> > q_data;
-    const Polynomial<number> *q = 0;
+    std::unique_ptr<Polynomial<number> > q_data;
+    const Polynomial<number> *q = nullptr;
     if (p.in_lagrange_product_form == true)
       {
-        q_data.reset (new Polynomial<number>(p));
+        q_data = std_cxx14::make_unique<Polynomial<number>>(p);
         q_data->transform_into_standard_form();
         q = q_data.get();
       }
@@ -450,11 +451,11 @@ namespace Polynomials
     // need to transform p into standard form as
     // well if necessary. copy the polynomial to
     // do this
-    std_cxx11::shared_ptr<Polynomial<number> > q_data;
-    const Polynomial<number> *q = 0;
+    std::unique_ptr<Polynomial<number> > q_data;
+    const Polynomial<number> *q = nullptr;
     if (p.in_lagrange_product_form == true)
       {
-        q_data.reset (new Polynomial<number>(p));
+        q_data = std_cxx14::make_unique<Polynomial<number>>(p);
         q_data->transform_into_standard_form();
         q = q_data.get();
       }
@@ -597,11 +598,11 @@ namespace Polynomials
     if (degree() == 0)
       return Monomial<number>(0, 0.);
 
-    std_cxx11::shared_ptr<Polynomial<number> > q_data;
-    const Polynomial<number> *q = 0;
+    std::unique_ptr<Polynomial<number> > q_data;
+    const Polynomial<number> *q = nullptr;
     if (in_lagrange_product_form == true)
       {
-        q_data.reset (new Polynomial<number>(*this));
+        q_data = std_cxx14::make_unique<Polynomial<number>>(*this);
         q_data->transform_into_standard_form();
         q = q_data.get();
       }
@@ -623,11 +624,11 @@ namespace Polynomials
   {
     // no simple form possible for Lagrange
     // polynomial on product form
-    std_cxx11::shared_ptr<Polynomial<number> > q_data;
-    const Polynomial<number> *q = 0;
+    std::unique_ptr<Polynomial<number> > q_data;
+    const Polynomial<number> *q = nullptr;
     if (in_lagrange_product_form == true)
       {
-        q_data.reset (new Polynomial<number>(*this));
+        q_data = std_cxx14::make_unique<Polynomial<number>>(*this);
         q_data->transform_into_standard_form();
         q = q_data.get();
       }
@@ -703,7 +704,7 @@ namespace Polynomials
 
   namespace internal
   {
-    namespace LagrangeEquidistant
+    namespace LagrangeEquidistantImplementation
     {
       std::vector<Point<1> >
       generate_equidistant_unit_points (const unsigned int n)
@@ -722,7 +723,7 @@ namespace Polynomials
   LagrangeEquidistant::LagrangeEquidistant (const unsigned int n,
                                             const unsigned int support_point)
     :
-    Polynomial<double> (internal::LagrangeEquidistant::
+    Polynomial<double> (internal::LagrangeEquidistantImplementation::
                         generate_equidistant_unit_points (n),
                         support_point)
   {
@@ -753,7 +754,7 @@ namespace Polynomials
     unsigned int n_functions=n+1;
     Assert(support_point<n_functions,
            ExcIndexRange(support_point, 0, n_functions));
-    double const *x=0;
+    double const *x=nullptr;
 
     switch (n)
       {
@@ -794,7 +795,7 @@ namespace Polynomials
         Assert(false, ExcInternalError())
       }
 
-    Assert(x!=0, ExcInternalError());
+    Assert(x!=nullptr, ExcInternalError());
     for (unsigned int i=0; i<n_functions; ++i)
       a[i]=x[support_point*n_functions+i];
   }
@@ -832,7 +833,7 @@ namespace Polynomials
     p.reserve (points.size());
 
     for (unsigned int i=0; i<points.size(); ++i)
-      p.push_back (Polynomial<double> (points, i));
+      p.emplace_back (points, i);
     return p;
   }
 
@@ -973,7 +974,7 @@ namespace Polynomials
 
 // Reserve space for polynomials up to degree 19. Should be sufficient
 // for the start.
-  std::vector<std_cxx11::shared_ptr<const std::vector<double> > >
+  std::vector<std::unique_ptr<const std::vector<double> > >
   Hierarchical::recursive_coefficients(20);
 
 
@@ -1004,8 +1005,7 @@ namespace Polynomials
     // check: does the information
     // already exist?
     if (  (recursive_coefficients.size() < k+1) ||
-          ((recursive_coefficients.size() >= k+1) &&
-           (recursive_coefficients[k].get() == 0)) )
+          (recursive_coefficients[k].get() == nullptr))
       // no, then generate the
       // respective coefficients
       {
@@ -1039,20 +1039,20 @@ namespace Polynomials
             // later assign it to the
             // coefficients array to
             // make it const
-            std::vector<double> *c0 = new std::vector<double>(2);
-            (*c0)[0] =  1.;
-            (*c0)[1] = -1.;
+            std::vector<double> c0(2);
+            c0[0] =  1.;
+            c0[1] = -1.;
 
-            std::vector<double> *c1 = new std::vector<double>(2);
-            (*c1)[0] = 0.;
-            (*c1)[1] = 1.;
+            std::vector<double> c1(2);
+            c1[0] = 0.;
+            c1[1] = 1.;
 
             // now make these arrays
             // const
             recursive_coefficients[0] =
-              std_cxx11::shared_ptr<const std::vector<double> >(c0);
+              std_cxx14::make_unique<const std::vector<double> >(std::move(c0));
             recursive_coefficients[1] =
-              std_cxx11::shared_ptr<const std::vector<double> >(c1);
+              std_cxx14::make_unique<const std::vector<double> >(std::move(c1));
           }
         else if (k==2)
           {
@@ -1060,16 +1060,16 @@ namespace Polynomials
             compute_coefficients(1);
             coefficients_lock.acquire ();
 
-            std::vector<double> *c2 = new std::vector<double>(3);
+            std::vector<double> c2(3);
 
             const double a = 1.; //1./8.;
 
-            (*c2)[0] =   0.*a;
-            (*c2)[1] =  -4.*a;
-            (*c2)[2] =   4.*a;
+            c2[0] =   0.*a;
+            c2[1] =  -4.*a;
+            c2[2] =   4.*a;
 
             recursive_coefficients[2] =
-              std_cxx11::shared_ptr<const std::vector<double> >(c2);
+              std_cxx14::make_unique<const std::vector<double> >(std::move(c2));
           }
         else
           {
@@ -1085,17 +1085,17 @@ namespace Polynomials
             compute_coefficients(k-1);
             coefficients_lock.acquire ();
 
-            std::vector<double> *ck = new std::vector<double>(k+1);
+            std::vector<double> ck (k+1);
 
             const double a = 1.; //1./(2.*k);
 
-            (*ck)[0] = - a*(*recursive_coefficients[k-1])[0];
+            ck[0] = - a*(*recursive_coefficients[k-1])[0];
 
             for (unsigned int i=1; i<=k-1; ++i)
-              (*ck)[i] = a*( 2.*(*recursive_coefficients[k-1])[i-1]
-                             - (*recursive_coefficients[k-1])[i] );
+              ck[i] = a*( 2.*(*recursive_coefficients[k-1])[i-1]
+                          - (*recursive_coefficients[k-1])[i] );
 
-            (*ck)[k] = a*2.*(*recursive_coefficients[k-1])[k-1];
+            ck[k] = a*2.*(*recursive_coefficients[k-1])[k-1];
             // for even degrees, we need
             // to add a multiple of
             // basis fcn phi_2
@@ -1105,15 +1105,15 @@ namespace Polynomials
                 //for (unsigned int i=1; i<=k; i++)
                 //  b /= 2.*i;
 
-                (*ck)[1] += b*(*recursive_coefficients[2])[1];
-                (*ck)[2] += b*(*recursive_coefficients[2])[2];
+                ck[1] += b*(*recursive_coefficients[2])[1];
+                ck[2] += b*(*recursive_coefficients[2])[2];
               }
             // finally assign the newly
             // created vector to the
             // const pointer in the
             // coefficients array
             recursive_coefficients[k] =
-              std_cxx11::shared_ptr<const std::vector<double> >(ck);
+              std_cxx14::make_unique<const std::vector<double> >(std::move(ck));
           };
       };
   }
@@ -1220,6 +1220,9 @@ namespace Polynomials
   std::vector<Polynomial<double> >
   HermiteInterpolation::generate_complete_basis (const unsigned int n)
   {
+    Assert(n>=3,
+           ExcNotImplemented("Hermite interpolation makes no sense for "
+                             "degrees less than three"));
     std::vector<Polynomial<double> > basis (n + 1);
 
     for (unsigned int i = 0; i <= n; ++i)
@@ -1228,6 +1231,283 @@ namespace Polynomials
     return basis;
   }
 
+
+// ------------------ HermiteLikeInterpolation --------------- //
+  namespace
+  {
+    // Finds the zero position x_star such that the mass matrix entry (0,1)
+    // with the Hermite polynomials evaluates to zero. The function has
+    // originally been derived by a secant method for the integral entry
+    // l_0(x) * l_1(x) but we only need to do one iteration because the zero
+    // x_star is linear in the integral value.
+    double find_support_point_x_star (const std::vector<double> &jacobi_roots)
+    {
+      // Initial guess for the support point position values: The zero turns
+      // out to be between zero and the first root of the Jacobi polynomial,
+      // but the algorithm is agnostic about that, so simply choose two points
+      // that are sufficiently far apart.
+      double guess_left = 0;
+      double guess_right = 0.5;
+      const unsigned int degree = jacobi_roots.size() + 3;
+
+      // Compute two integrals of the product of l_0(x) * l_1(x)
+      // l_0(x) = (x-y)*(x-jacobi_roots(0))*...*(x-jacobi_roos(degree-4))*(x-1)*(x-1)
+      // l_1(x) = (x-0)*(x-jacobi_roots(0))*...*(x-jacobi_roots(degree-4))*(x-1)*(x-1)
+      // where y is either guess_left or guess_right for the two integrals.
+      // Note that the polynomials are not yet normalized here, which is not
+      // necessary because we are only looking for the x_star where the matrix
+      // entry is zero, for which the constants do not matter.
+      QGauss<1> gauss(degree + 1);
+      double integral_left = 0, integral_right = 0;
+      for (unsigned int q=0; q<gauss.size(); ++q)
+        {
+          const double x = gauss.point(q)[0];
+          double poly_val_common = x;
+          for (unsigned int j=0; j<degree-3; ++j)
+            poly_val_common *= Utilities::fixed_power<2>(x-jacobi_roots[j]);
+          poly_val_common *= Utilities::fixed_power<4>(x - 1.);
+          integral_left += gauss.weight(q)*(poly_val_common*(x - guess_left));
+          integral_right += gauss.weight(q)*(poly_val_common*(x - guess_right));
+        }
+
+      // compute guess by secant method. Due to linearity in the root x_star,
+      // this is the correct position after this single step
+      return guess_right - (guess_right-guess_left)/(integral_right-integral_left)*integral_right;
+    }
+  }
+
+
+
+  HermiteLikeInterpolation::HermiteLikeInterpolation (const unsigned int degree,
+                                                      const unsigned int index)
+    :
+    Polynomial<double>(0)
+  {
+    AssertIndexRange(index, degree+1);
+
+    this->coefficients.clear();
+    this->in_lagrange_product_form = true;
+
+    this->lagrange_support_points.resize(degree);
+
+    if (degree == 0)
+      this->lagrange_weight = 1.;
+    else if (degree == 1)
+      {
+        if (index == 0)
+          {
+            this->lagrange_support_points[0] = 1.;
+            this->lagrange_weight = -1.;
+          }
+        else
+          {
+            this->lagrange_support_points[0] = 0.;
+            this->lagrange_weight = 1.;
+          }
+      }
+    else if (degree == 2)
+      {
+        if (index == 0)
+          {
+            this->lagrange_support_points[0] = 1.;
+            this->lagrange_support_points[1] = 1.;
+            this->lagrange_weight = 1.;
+          }
+        else if (index == 1)
+          {
+            this->lagrange_support_points[0] = 0;
+            this->lagrange_support_points[1] = 1;
+            this->lagrange_weight = -2.;
+          }
+        else
+          {
+            this->lagrange_support_points[0] = 0.;
+            this->lagrange_support_points[1] = 0.;
+            this->lagrange_weight = 1.;
+          }
+      }
+    else if (degree==3)
+      {
+        // 4 Polynomials with degree 3
+        // entries (1,0) and (3,2) of the mass matrix will be equal to 0
+        //
+        //     | x  0  x  x |
+        //     | 0  x  x  x |
+        // M = | x  x  x  0 |
+        //     | x  x  0  x |
+        //
+        if (index==0)
+          {
+            this->lagrange_support_points[0] = 2./7.;
+            this->lagrange_support_points[1] = 1.;
+            this->lagrange_support_points[2] = 1.;
+            this->lagrange_weight = -3.5;
+          }
+        else if (index==1)
+          {
+            this->lagrange_support_points[0] = 0.;
+            this->lagrange_support_points[1] = 1.;
+            this->lagrange_support_points[2] = 1.;
+
+            // this magic value 5.5 is obtained when evaluating the general
+            // formula below for the degree=3 case
+            this->lagrange_weight = 5.5;
+          }
+        else if (index==2)
+          {
+            this->lagrange_support_points[0] = 0.;
+            this->lagrange_support_points[1] = 0.;
+            this->lagrange_support_points[2] = 1.;
+            this->lagrange_weight = -5.5;
+          }
+        else if (index==3)
+          {
+            this->lagrange_support_points[0] = 0.;
+            this->lagrange_support_points[1] = 0.;
+            this->lagrange_support_points[2] = 5./7.;
+            this->lagrange_weight = 3.5;
+          }
+      }
+    else
+      {
+        // Higher order Polynomials degree>=4: the entries (1,0) and
+        // (degree,degree-1) of the mass matrix will be equal to 0
+        //
+        //     | x  0  x  x         x  x  x |
+        //     | 0  x  x  x  . . .  x  x  x |
+        //     | x  x  x  x         x  x  x |
+        //     | x  x  x  x         x  x  x |
+        //     |     .       .         .    |
+        // M = |     .         .       .    |
+        //     |     .           .     .    |
+        //     | x  x  x  x         x  x  x |
+        //     | x  x  x  x  . . .  x  x  0 |
+        //     | x  x  x  x         x  0  x |
+        //
+        // We find the inner points as the zeros of the Jacobi polynomials
+        // with alpha = beta = 2 which is the polynomial with the kernel
+        // (1-x)^2 (1+x)^2, the two polynomials achieving zero value and zero
+        // derivative at the boundary.
+
+        std::vector<double> jacobi_roots = jacobi_polynomial_roots<double>(degree-3, 2, 2);
+        AssertDimension(jacobi_roots.size(), degree-3);
+
+        // iteration from variable support point N with secant method
+        // initial values
+
+        this->lagrange_support_points.resize(degree);
+        if (index==0)
+          {
+            const double auxiliary_zero = find_support_point_x_star(jacobi_roots);
+            this->lagrange_support_points[0] = auxiliary_zero;
+            for (unsigned int m=0; m<degree-3; m++)
+              this->lagrange_support_points[m+1] = jacobi_roots[m];
+            this->lagrange_support_points[degree-2] = 1.;
+            this->lagrange_support_points[degree-1] = 1.;
+
+            // ensure that the polynomial evaluates to one at x=0
+            this->lagrange_weight = 1./this->value(0);
+          }
+        else if (index==1)
+          {
+            this->lagrange_support_points[0] = 0.;
+            for (unsigned int m=0; m<degree-3; m++)
+              this->lagrange_support_points[m+1] = jacobi_roots[m];
+            this->lagrange_support_points[degree-2] = 1.;
+            this->lagrange_support_points[degree-1] = 1.;
+
+            // Select the weight to make the derivative of the sum of P_0 and
+            // P_1 in zero to be 0. The derivative in x=0 is simply given by
+            // p~(0)/auxiliary_zero+p~'(0) + a*p~(0), where p~(x) is the
+            // Lagrange polynomial in all points except the first one which is
+            // the same for P_0 and P_1, and a is the weight we seek here. If
+            // we solve this for a, we obtain the desired property. Since the
+            // basis is nodal for all interior points, this property ensures
+            // that the sum of all polynomials with weight 1 is one.
+            std::vector<Point<1>> points(degree);
+            double ratio = 1.;
+            for (unsigned int i=0; i<degree; ++i)
+              {
+                points[i][0] = this->lagrange_support_points[i];
+                if (i>0)
+                  ratio *= -this->lagrange_support_points[i];
+              }
+            Polynomial<double> helper(points, 0);
+            std::vector<double> value_and_grad(2);
+            helper.value(0., value_and_grad);
+            Assert(std::abs(value_and_grad[0]) > 1e-10,
+                   ExcInternalError("There should not be a zero at x=0."));
+
+            const double auxiliary_zero = find_support_point_x_star(jacobi_roots);
+            this->lagrange_weight = (1./auxiliary_zero - value_and_grad[1]/value_and_grad[0])/ratio;
+          }
+        else if (index>=2 && index<degree-1)
+          {
+            this->lagrange_support_points[0] = 0.;
+            this->lagrange_support_points[1] = 0.;
+            for (unsigned int m=0, c=2; m<degree-3; m++)
+              if (m+2 != index)
+                this->lagrange_support_points[c++] = jacobi_roots[m];
+            this->lagrange_support_points[degree-2] = 1.;
+            this->lagrange_support_points[degree-1] = 1.;
+
+            // ensure that the polynomial evaluates to one at the respective
+            // nodal point
+            this->lagrange_weight = 1./this->value(jacobi_roots[index-2]);
+          }
+        else if (index==degree-1)
+          {
+            this->lagrange_support_points[0] = 0.;
+            this->lagrange_support_points[1] = 0.;
+            for (unsigned int m=0; m<degree-3; m++)
+              this->lagrange_support_points[m+2] = jacobi_roots[m];
+            this->lagrange_support_points[degree-1] = 1.;
+
+            std::vector<Point<1>> points(degree);
+            double ratio = 1.;
+            for (unsigned int i=0; i<degree; ++i)
+              {
+                points[i][0] = this->lagrange_support_points[i];
+                if (i<degree-1)
+                  ratio *= 1.-this->lagrange_support_points[i];
+              }
+            Polynomial<double> helper(points, degree-1);
+            std::vector<double> value_and_grad(2);
+            helper.value(1., value_and_grad);
+            Assert(std::abs(value_and_grad[0]) > 1e-10,
+                   ExcInternalError("There should not be a zero at x=1."));
+
+            const double auxiliary_zero = find_support_point_x_star(jacobi_roots);
+            this->lagrange_weight = (-1./auxiliary_zero - value_and_grad[1]/value_and_grad[0])/ratio;
+          }
+        else if (index==degree)
+          {
+            const double auxiliary_zero = find_support_point_x_star(jacobi_roots);
+            this->lagrange_support_points[0] = 0.;
+            this->lagrange_support_points[1] = 0.;
+            for (unsigned int m=0; m<degree-3; m++)
+              this->lagrange_support_points[m+2] = jacobi_roots[m];
+            this->lagrange_support_points[degree-1] = 1.-auxiliary_zero;
+
+            // ensure that the polynomial evaluates to one at x=1
+            this->lagrange_weight = 1./this->value(1.);
+          }
+      }
+  }
+
+
+
+  std::vector<Polynomial<double> >
+  HermiteLikeInterpolation::generate_complete_basis (const unsigned int degree)
+  {
+    std::vector<Polynomial<double> > basis (degree + 1);
+
+    for (unsigned int i = 0; i <= degree; ++i)
+      basis[i] = HermiteLikeInterpolation (degree, i);
+
+    return basis;
+  }
+
 }
 
 // ------------------ explicit instantiations --------------- //
