@@ -1,7 +1,6 @@
 // ---------------------------------------------------------------------
-// $Id: polynomials_bdm.cc 30037 2013-07-18 16:55:40Z maier $
 //
-// Copyright (C) 2004 - 2013 by the deal.II authors
+// Copyright (C) 2004 - 2015 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -15,7 +14,9 @@
 // ---------------------------------------------------------------------
 
 
+#include <deal.II/base/geometry_info.h>
 #include <deal.II/base/polynomials_bdm.h>
+#include <deal.II/base/polynomial_space.h>
 #include <deal.II/base/quadrature_lib.h>
 #include <iostream>
 #include <iomanip>
@@ -54,7 +55,9 @@ void
 PolynomialsBDM<dim>::compute (const Point<dim>            &unit_point,
                               std::vector<Tensor<1,dim> > &values,
                               std::vector<Tensor<2,dim> > &grads,
-                              std::vector<Tensor<3,dim> > &grad_grads) const
+                              std::vector<Tensor<3,dim> > &grad_grads,
+                              std::vector<Tensor<4,dim> > &third_derivatives,
+                              std::vector<Tensor<5,dim> > &fourth_derivatives) const
 {
   Assert(values.size()==n_pols || values.size()==0,
          ExcDimensionMismatch(values.size(), n_pols));
@@ -62,6 +65,18 @@ PolynomialsBDM<dim>::compute (const Point<dim>            &unit_point,
          ExcDimensionMismatch(grads.size(), n_pols));
   Assert(grad_grads.size()==n_pols|| grad_grads.size()==0,
          ExcDimensionMismatch(grad_grads.size(), n_pols));
+  Assert(third_derivatives.size()==n_pols|| third_derivatives.size()==0,
+         ExcDimensionMismatch(third_derivatives.size(), n_pols));
+  Assert(fourth_derivatives.size()==n_pols|| fourth_derivatives.size()==0,
+         ExcDimensionMismatch(fourth_derivatives.size(), n_pols));
+
+  // third and fourth derivatives not implemented
+  (void)third_derivatives;
+  Assert(third_derivatives.size()==0,
+         ExcNotImplemented());
+  (void)fourth_derivatives;
+  Assert(fourth_derivatives.size()==0,
+         ExcNotImplemented());
 
   const unsigned int n_sub = polynomial_space.n();
 
@@ -82,7 +97,8 @@ PolynomialsBDM<dim>::compute (const Point<dim>            &unit_point,
     // will have first all polynomials
     // in the x-component, then y and
     // z.
-    polynomial_space.compute (unit_point, p_values, p_grads, p_grad_grads);
+    polynomial_space.compute (unit_point, p_values, p_grads, p_grad_grads,
+                              p_third_derivatives, p_fourth_derivatives);
 
     std::fill(values.begin(), values.end(), Tensor<1,dim>());
     for (unsigned int i=0; i<p_values.size(); ++i)
@@ -118,8 +134,8 @@ PolynomialsBDM<dim>::compute (const Point<dim>            &unit_point,
         {
           values[start][0] = monovali[0][0];
           values[start][1] = -unit_point(1) * monovali[0][1];
-          values[start+1][0] = -unit_point(0) * monovali[1][1];
-          values[start+1][1] = monovali[1][0];
+          values[start+1][0] = unit_point(0) * monovali[1][1];
+          values[start+1][1] = -monovali[1][0];
         }
       if (grads.size() != 0)
         {
@@ -127,10 +143,10 @@ PolynomialsBDM<dim>::compute (const Point<dim>            &unit_point,
           grads[start][0][1] = 0.;
           grads[start][1][0] = -unit_point(1) * monovali[0][2];
           grads[start][1][1] = -monovali[0][1];
-          grads[start+1][0][0] = -monovali[1][1];
-          grads[start+1][0][1] = -unit_point(0) * monovali[1][2];
+          grads[start+1][0][0] = monovali[1][1];
+          grads[start+1][0][1] = unit_point(0) * monovali[1][2];
           grads[start+1][1][0] = 0.;
-          grads[start+1][1][1] = monovali[1][1];
+          grads[start+1][1][1] = -monovali[1][1];
         }
       if (grad_grads.size() != 0)
         {
@@ -143,13 +159,13 @@ PolynomialsBDM<dim>::compute (const Point<dim>            &unit_point,
           grad_grads[start][1][1][0] = -monovali[0][2];
           grad_grads[start][1][1][1] = 0.;
           grad_grads[start+1][0][0][0] = 0;
-          grad_grads[start+1][0][0][1] = -monovali[1][2];
-          grad_grads[start+1][0][1][0] = -monovali[1][2];
-          grad_grads[start+1][0][1][1] = -unit_point(0) * monovali[1][3];
+          grad_grads[start+1][0][0][1] = monovali[1][2];
+          grad_grads[start+1][0][1][0] = monovali[1][2];
+          grad_grads[start+1][0][1][1] = unit_point(0) * monovali[1][3];
           grad_grads[start+1][1][0][0] = 0.;
           grad_grads[start+1][1][0][1] = 0.;
           grad_grads[start+1][1][1][0] = 0.;
-          grad_grads[start+1][1][1][1] = monovali[1][2];
+          grad_grads[start+1][1][1][1] = -monovali[1][2];
         }
     }
   else // dim == 3
@@ -202,9 +218,9 @@ PolynomialsBDM<dim>::compute (const Point<dim>            &unit_point,
               grads[start][1][0] = 0.;
               grads[start][1][1] = -monovali[1][1] * monovalk[2][0];
               grads[start][1][2] = -monovali[1][0] * monovalk[2][1];
-              grads[start+2][2][0] = 0.;
-              grads[start+2][2][1] = 0.;
-              grads[start+2][2][2] = 0.;
+              grads[start][2][0] = 0.;
+              grads[start][2][1] = 0.;
+              grads[start][2][2] = 0.;
 
               grads[start+1][1][1] = monovali[2][1] * monovalk[0][0];
               grads[start+1][1][2] = unit_point(1) * monovali[2][2] * monovalk[0][0];
@@ -228,7 +244,90 @@ PolynomialsBDM<dim>::compute (const Point<dim>            &unit_point,
             }
           if (grad_grads.size() != 0)
             {
-              Assert(false,ExcNotImplemented());
+              grad_grads[start][0][0][0] = 0.;
+              grad_grads[start][0][0][1] = monovali[1][2]*monovalk[2][0];
+              grad_grads[start][0][0][2] = monovali[1][1]*monovalk[2][1];
+              grad_grads[start][0][1][0] = monovali[1][2]*monovalk[2][0];
+              grad_grads[start][0][1][1] = unit_point(0)*monovali[1][3]*monovalk[2][0];
+              grad_grads[start][0][1][2] = unit_point(0)*monovali[1][2]*monovalk[2][1];
+              grad_grads[start][0][2][0] = monovali[1][1]*monovalk[2][1];
+              grad_grads[start][0][2][1] = unit_point(0)*monovali[1][2]*monovalk[2][1];
+              grad_grads[start][0][2][2] = unit_point(0)*monovali[1][1]*monovalk[2][2];
+              grad_grads[start][1][0][0] = 0.;
+              grad_grads[start][1][0][1] = 0.;
+              grad_grads[start][1][0][2] = 0.;
+              grad_grads[start][1][1][0] = 0.;
+              grad_grads[start][1][1][1] = -monovali[1][2]*monovalk[2][0];
+              grad_grads[start][1][1][2] = -monovali[1][1]*monovalk[2][1];
+              grad_grads[start][1][2][0] = 0.;
+              grad_grads[start][1][2][1] = -monovali[1][1]*monovalk[2][1];
+              grad_grads[start][1][2][2] = -monovali[1][0]*monovalk[2][2];
+              grad_grads[start][2][0][0] = 0.;
+              grad_grads[start][2][0][1] = 0.;
+              grad_grads[start][2][0][2] = 0.;
+              grad_grads[start][2][1][0] = 0.;
+              grad_grads[start][2][1][1] = 0.;
+              grad_grads[start][2][1][2] = 0.;
+              grad_grads[start][2][2][0] = 0.;
+              grad_grads[start][2][2][1] = 0.;
+              grad_grads[start][2][2][2] = 0.;
+
+              grad_grads[start+1][0][0][0] = 0.;
+              grad_grads[start+1][0][0][1] = 0.;
+              grad_grads[start+1][0][0][2] = 0.;
+              grad_grads[start+1][0][1][0] = 0.;
+              grad_grads[start+1][0][1][1] = 0.;
+              grad_grads[start+1][0][1][2] = 0.;
+              grad_grads[start+1][0][2][0] = 0.;
+              grad_grads[start+1][0][2][1] = 0.;
+              grad_grads[start+1][0][2][2] = 0.;
+              grad_grads[start+1][1][0][0] = unit_point(1)*monovali[2][1]*monovalk[0][2];
+              grad_grads[start+1][1][0][1] = monovali[2][1]*monovalk[0][1];
+              grad_grads[start+1][1][0][2] = unit_point(1)*monovali[2][2]*monovalk[0][1];
+              grad_grads[start+1][1][1][0] = monovalk[0][1]*monovali[2][1];
+              grad_grads[start+1][1][1][1] = 0.;
+              grad_grads[start+1][1][1][2] = monovalk[0][0]*monovali[2][2];
+              grad_grads[start+1][1][2][0] = unit_point(1)*monovalk[0][1]*monovali[2][2];
+              grad_grads[start+1][1][2][1] = monovalk[0][0]*monovali[2][2];
+              grad_grads[start+1][1][2][2] = unit_point(1)*monovalk[0][0]*monovali[2][3];
+              grad_grads[start+1][2][0][0] = -monovalk[0][2]*monovali[2][0];
+              grad_grads[start+1][2][0][1] = 0.;
+              grad_grads[start+1][2][0][2] = -monovalk[0][1]*monovali[2][1];
+              grad_grads[start+1][2][1][0] = 0.;
+              grad_grads[start+1][2][1][1] = 0.;
+              grad_grads[start+1][2][1][2] = 0.;
+              grad_grads[start+1][2][2][0] = -monovalk[0][1]*monovali[2][1];
+              grad_grads[start+1][2][2][1] = 0.;
+              grad_grads[start+1][2][2][2] = -monovalk[0][0]*monovali[2][2];
+
+              grad_grads[start+2][0][0][0] = -monovali[0][2]*monovalk[1][0];
+              grad_grads[start+2][0][0][1] = -monovali[0][1]*monovalk[1][1];
+              grad_grads[start+2][0][0][2] = 0.;
+              grad_grads[start+2][0][1][0] = -monovali[0][1]*monovalk[1][1];
+              grad_grads[start+2][0][1][1] = -monovali[0][0]*monovalk[1][2];
+              grad_grads[start+2][0][1][2] = 0.;
+              grad_grads[start+2][0][2][0] = 0.;
+              grad_grads[start+2][0][2][1] = 0.;
+              grad_grads[start+2][0][2][2] = 0.;
+              grad_grads[start+2][1][0][0] = 0.;
+              grad_grads[start+2][1][0][1] = 0.;
+              grad_grads[start+2][1][0][2] = 0.;
+              grad_grads[start+2][1][1][0] = 0.;
+              grad_grads[start+2][1][1][1] = 0.;
+              grad_grads[start+2][1][1][2] = 0.;
+              grad_grads[start+2][1][2][0] = 0.;
+              grad_grads[start+2][1][2][1] = 0.;
+              grad_grads[start+2][1][2][2] = 0.;
+              grad_grads[start+2][2][0][0] = unit_point(2)*monovali[0][3]*monovalk[1][0];
+              grad_grads[start+2][2][0][1] = unit_point(2)*monovali[0][2]*monovalk[1][1];
+              grad_grads[start+2][2][0][2] = monovali[0][2]*monovalk[1][0];
+              grad_grads[start+2][2][1][0] = unit_point(2)*monovali[0][2]*monovalk[1][1];
+              grad_grads[start+2][2][1][1] = unit_point(2)*monovali[0][1]*monovalk[1][2];
+              grad_grads[start+2][2][1][2] = monovali[0][1]*monovalk[1][1];
+              grad_grads[start+2][2][2][0] = monovali[0][2]*monovalk[1][0];
+              grad_grads[start+2][2][2][1] = monovali[0][1]*monovalk[1][1];
+              grad_grads[start+2][2][2][2] = 0.;
+
             }
         }
       Assert(start == n_pols, ExcInternalError());
