@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 1998 - 2017 by the deal.II authors
+// Copyright (C) 1998 - 2018 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -15,12 +15,12 @@
 
 #include <deal.II/base/quadrature_lib.h>
 #include <deal.II/base/geometry_info.h>
-
-#include <deal.II/base/std_cxx11/bind.h>
+#include <deal.II/base/polynomial.h>
 
 #include <cmath>
 #include <limits>
 #include <algorithm>
+#include <functional>
 
 
 DEAL_II_NAMESPACE_OPEN
@@ -68,100 +68,74 @@ QGauss<1>::QGauss (const unsigned int n)
   if (n == 0)
     return;
 
-  const unsigned int m = (n+1)/2;
+  std::vector<long double> points
+    = Polynomials::jacobi_polynomial_roots<long double>(n, 0, 0);
 
-  // tolerance for the Newton
-  // iteration below. we need to make
-  // it adaptive since on some
-  // machines (for example PowerPC)
-  // long double is the same as
-  // double -- in that case we can
-  // only get to a certain multiple
-  // of the accuracy of double there,
-  // while on other machines we'd
-  // like to go further down
-  //
-  // the situation is complicated by
-  // the fact that even if long
-  // double exists and is described
-  // by std::numeric_limits, we may
-  // not actually get the additional
-  // precision. One case where this
-  // happens is on x86, where one can
-  // set hardware flags that disable
-  // long double precision even for
-  // long double variables. these
-  // flags are not usually set, but
-  // for example matlab sets them and
-  // this then breaks deal.II code
-  // that is run as a subroutine to
-  // matlab...
-  //
-  // a similar situation exists, btw,
-  // when running programs under
-  // valgrind up to and including at
-  // least version 3.3: valgrind's
-  // emulator only supports 64 bit
-  // arithmetic, even for 80 bit long
-  // doubles.
-  const long double
-  long_double_eps = static_cast<long double>(std::numeric_limits<long double>::epsilon()),
-  double_eps      = static_cast<long double>(std::numeric_limits<double>::epsilon());
-
-  // now check whether long double is more
-  // accurate than double, and set
-  // tolerances accordingly. generate a one
-  // that really is generated at run-time
-  // and is not optimized away by the
-  // compiler. that makes sure that the
-  // tolerance is set at run-time with the
-  // current behavior, not at compile-time
-  // (not doing so leads to trouble with
-  // valgrind for example).
-  volatile long double runtime_one = 1.0;
-  const long double tolerance
-    = (runtime_one + long_double_eps != runtime_one
-       ?
-       std::max (double_eps / 100, long_double_eps * 5)
-       :
-       double_eps * 5
-      );
-
-
-  for (unsigned int i=1; i<=m; ++i)
+  for (unsigned int i=0; i<(points.size()+1)/2; ++i)
     {
-      long double z = std::cos(numbers::PI * (i-.25)/(n+.5));
+      this->quadrature_points[i][0] = points[i];
+      this->quadrature_points[n-i-1][0] = 1.-points[i];
 
-      long double pp;
-      long double p1;
-
-      // Newton iteration
-      do
-        {
-          // compute L_n (z)
-          p1 = 1.;
-          long double p2 = 0.;
-          for (unsigned int j=0; j<n; ++j)
-            {
-              const long double p3 = p2;
-              p2 = p1;
-              p1 = ((2.*j+1.)*z*p2-j*p3)/(j+1);
-            }
-          pp = n*(z*p1-p2)/(z*z-1);
-          z = z-p1/pp;
-        }
-      while (std::abs(p1/pp) > tolerance);
-
-      double x = .5*z;
-      this->quadrature_points[i-1] = Point<1>(.5-x);
-      this->quadrature_points[n-i] = Point<1>(.5+x);
-
-      double w = 1./((1.-z*z)*pp*pp);
-      this->weights[i-1] = w;
-      this->weights[n-i] = w;
+      // derivative of Jacobi polynomial
+      const long double pp = 0.5*(n + 1)*Polynomials::jacobi_polynomial_value(n-1, 1, 1, points[i]);
+      const long double x = -1. + 2.*points[i];
+      const double w = 1./((1.-x*x)*pp*pp);
+      this->weights[i] = w;
+      this->weights[n-i-1] = w;
     }
 }
 
+namespace internal
+{
+  namespace QGaussLobatto
+  {
+    /**
+     * Evaluate the Gamma function $ \Gamma(n) = (n-1)! $.
+     * @param n  point of evaluation (integer).
+     */
+    long double gamma(const unsigned int n)
+    {
+      long double result = n - 1;
+      for (int i=n-2; i>1; --i)
+        result *= i;
+      return result;
+    }
+
+
+
+    /**
+     * Compute Legendre-Gauss-Lobatto quadrature weights. The quadrature points
+     * and weights are related to Jacobi polynomial specified by @p alpha, @p
+     * beta. @p x denotes the quadrature points.
+     *
+     * @return Vector containing weights.
+     */
+    std::vector<long double>
+    compute_quadrature_weights(const std::vector<long double> &x,
+                               const int alpha,
+                               const int beta)
+    {
+      const unsigned int q = x.size();
+      std::vector<long double> w(q);
+
+      const long double factor = std::pow(2., alpha+beta+1) *
+                                 gamma(alpha+q) *
+                                 gamma(beta+q) /
+                                 ((q-1)*gamma(q)*gamma(alpha+beta+q+1));
+      for (unsigned int i=0; i<q; ++i)
+        {
+          const long double s = Polynomials::jacobi_polynomial_value(q-1, alpha, beta, x[i]);
+          w[i] = factor/(s*s);
+        }
+      w[0]   *= (beta + 1);
+      w[q-1] *= (alpha + 1);
+
+      return w;
+    }
+  }
+}
+
+
 
 template <>
 QGaussLobatto<1>::QGaussLobatto (const unsigned int n)
@@ -170,166 +144,23 @@ QGaussLobatto<1>::QGaussLobatto (const unsigned int n)
 {
   Assert (n >= 2, ExcNotImplemented());
 
-  std::vector<long double> points  = compute_quadrature_points(n, 1, 1);
-  std::vector<long double> w       = compute_quadrature_weights(points, 0, 0);
+  std::vector<long double> points
+    = Polynomials::jacobi_polynomial_roots<long double>(n-2, 1, 1);
+  points.insert(points.begin(), 0);
+  points.push_back(1.);
+  std::vector<long double> w
+    = internal::QGaussLobatto::compute_quadrature_weights(points, 0, 0);
 
-  // scale points to the interval
-  // [0.0, 1.0]:
+  // scale weights to the interval [0.0, 1.0]:
   for (unsigned int i=0; i<points.size(); ++i)
     {
-      this->quadrature_points[i] = Point<1>(0.5 + 0.5*static_cast<double>(points[i]));
-      this->weights[i]           = 0.5*w[i];
+      this->quadrature_points[i][0] = points[i];
+      this->weights[i]              = 0.5*w[i];
     }
 }
 
 
 
-template <>
-std::vector<long double> QGaussLobatto<1>::
-compute_quadrature_points(const unsigned int q,
-                          const int alpha,
-                          const int beta) const
-{
-  const unsigned int m = q-2; // no. of inner points
-  std::vector<long double> x(m);
-
-  // compute quadrature points with
-  // a Newton algorithm.
-
-  // Set tolerance. See class QGauss
-  // for detailed explanation.
-  const long double
-  long_double_eps = static_cast<long double>(std::numeric_limits<long double>::epsilon()),
-  double_eps      = static_cast<long double>(std::numeric_limits<double>::epsilon());
-
-  // check whether long double is
-  // more accurate than double, and
-  // set tolerances accordingly
-  volatile long double runtime_one = 1.0;
-  const long double tolerance
-    = (runtime_one + long_double_eps != runtime_one
-       ?
-       std::max (double_eps / 100, long_double_eps * 5)
-       :
-       double_eps * 5
-      );
-
-  // The following implementation
-  // follows closely the one given in
-  // the appendix of the book by
-  // Karniadakis and Sherwin:
-  // Spectral/hp element methods for
-  // computational fluid dynamics
-  // (Oxford University Press, 2005)
-
-  // we take the zeros of the Chebyshev
-  // polynomial (alpha=beta=-0.5) as
-  // initial values:
-  for (unsigned int i=0; i<m; ++i)
-    x[i] = - std::cos( (long double) (2*i+1)/(2*m) * numbers::PI );
-
-  long double s, J_x, f, delta;
-
-  for (unsigned int k=0; k<m; ++k)
-    {
-      long double r = x[k];
-      if (k>0)
-        r = (r + x[k-1])/2;
-
-      do
-        {
-          s = 0.;
-          for (unsigned int i=0; i<k; ++i)
-            s += 1./(r - x[i]);
-
-          J_x   =  0.5*(alpha + beta + m + 1)*JacobiP(r, alpha+1, beta+1, m-1);
-          f     = JacobiP(r, alpha, beta, m);
-          delta = f/(f*s- J_x);
-          r += delta;
-        }
-      while (std::fabs(delta) >= tolerance);
-
-      x[k] = r;
-    } // for
-
-  // add boundary points:
-  x.insert(x.begin(), -1.L);
-  x.push_back(+1.L);
-
-  return x;
-}
-
-
-
-template <>
-std::vector<long double> QGaussLobatto<1>::
-compute_quadrature_weights(const std::vector<long double> &x,
-                           const int alpha,
-                           const int beta) const
-{
-  const unsigned int q = x.size();
-  std::vector<long double> w(q);
-  long double s = 0.L;
-
-  const long double factor = std::pow(2., alpha+beta+1) *
-                             gamma(alpha+q) *
-                             gamma(beta+q) /
-                             ((q-1)*gamma(q)*gamma(alpha+beta+q+1));
-  for (unsigned int i=0; i<q; ++i)
-    {
-      s = JacobiP(x[i], alpha, beta, q-1);
-      w[i] = factor/(s*s);
-    }
-  w[0]   *= (beta + 1);
-  w[q-1] *= (alpha + 1);
-
-  return w;
-}
-
-
-
-template <>
-long double QGaussLobatto<1>::JacobiP(const long double x,
-                                      const int alpha,
-                                      const int beta,
-                                      const unsigned int n) const
-{
-  // the Jacobi polynomial is evaluated
-  // using a recursion formula.
-  std::vector<long double> p(n+1);
-
-  // initial values P_0(x), P_1(x):
-  p[0] = 1.0L;
-  if (n==0) return p[0];
-  p[1] = ((alpha+beta+2)*x + (alpha-beta))/2;
-  if (n==1) return p[1];
-
-  for (unsigned int i=1; i<=(n-1); ++i)
-    {
-      const int v  = 2*i + alpha + beta;
-      const int a1 = 2*(i+1)*(i + alpha + beta + 1)*v;
-      const int a2 = (v + 1)*(alpha*alpha - beta*beta);
-      const int a3 = v*(v + 1)*(v + 2);
-      const int a4 = 2*(i+alpha)*(i+beta)*(v + 2);
-
-      p[i+1] = static_cast<long double>( (a2 + a3*x)*p[i] - a4*p[i-1])/a1;
-    } // for
-  return p[n];
-}
-
-
-
-template <>
-long double QGaussLobatto<1>::gamma(const unsigned int n)
-{
-  long double result = n - 1;
-  for (int i=n-2; i>1; --i)
-    result *= i;
-  return result;
-}
-
-
-
 template <>
 QMidpoint<1>::QMidpoint ()
   :
@@ -694,7 +525,7 @@ QGaussLog<1>::get_quadrature_weights(const unsigned int n)
 }
 
 
-template<>
+template <>
 QGaussLogR<1>::QGaussLogR(const unsigned int n,
                           const Point<1> origin,
                           const double alpha,
@@ -766,7 +597,7 @@ QGaussLogR<1>::QGaussLogR(const unsigned int n,
 }
 
 
-template<>
+template <>
 unsigned int QGaussOneOverR<2>::quad_size(const Point<2> singularity,
                                           const unsigned int n)
 {
@@ -785,7 +616,7 @@ unsigned int QGaussOneOverR<2>::quad_size(const Point<2> singularity,
   return (8*n*n);
 }
 
-template<>
+template <>
 QGaussOneOverR<2>::QGaussOneOverR(const unsigned int n,
                                   const Point<2> singularity,
                                   const bool factor_out_singularity) :
@@ -801,14 +632,14 @@ QGaussOneOverR<2>::QGaussOneOverR(const unsigned int n,
   std::vector<Point<2> > origins;
   // Id of the corner with a
   // singularity
-  quads.push_back(QGaussOneOverR(n, 3, factor_out_singularity));
-  quads.push_back(QGaussOneOverR(n, 2, factor_out_singularity));
-  quads.push_back(QGaussOneOverR(n, 1, factor_out_singularity));
-  quads.push_back(QGaussOneOverR(n, 0, factor_out_singularity));
+  quads.emplace_back(n, 3, factor_out_singularity);
+  quads.emplace_back(n, 2, factor_out_singularity);
+  quads.emplace_back(n, 1, factor_out_singularity);
+  quads.emplace_back(n, 0, factor_out_singularity);
 
-  origins.push_back(Point<2>(0.,0.));
-  origins.push_back(Point<2>(singularity[0],0.));
-  origins.push_back(Point<2>(0.,singularity[1]));
+  origins.emplace_back(0., 0.);
+  origins.emplace_back(singularity[0], 0.);
+  origins.emplace_back(0., singularity[1]);
   origins.push_back(singularity);
 
   // Lexicographical ordering.
@@ -835,7 +666,7 @@ QGaussOneOverR<2>::QGaussOneOverR(const unsigned int n,
 }
 
 
-template<>
+template <>
 QGaussOneOverR<2>::QGaussOneOverR(const unsigned int n,
                                   const unsigned int vertex_index,
                                   const bool factor_out_singularity) :
@@ -935,15 +766,24 @@ QSorted<dim>::QSorted(const Quadrature<dim> &quad) :
 
   std::sort(permutation.begin(),
             permutation.end(),
-            std_cxx11::bind(&QSorted<dim>::compare_weights,
-                            std_cxx11::ref(*this),
-                            std_cxx11::_1,
-                            std_cxx11::_2));
+            std::bind(&QSorted<dim>::compare_weights,
+                      std::ref(*this),
+                      std::placeholders::_1,
+                      std::placeholders::_2));
 
+  // At this point, the variable is_tensor_product_flag is set
+  // to the respective value of the given Quadrature in the base
+  // class copy constructor.
+  // We only call a quadrature formula 'tensor product'
+  // if the quadrature points are also sorted lexicographically.
+  // In particular, any reordering destroys that property
+  // and we might need to modify the variable accordingly.
   for (unsigned int i=0; i<quad.size(); ++i)
     {
       this->weights[i]           = quad.weight(permutation[i]);
       this->quadrature_points[i] = quad.point(permutation[i]);
+      if (permutation[i] != i)
+        this->is_tensor_product_flag = false;
     }
 }
 
@@ -1014,11 +854,9 @@ template <int dim>
 QTelles<dim>::QTelles (
   const Quadrature<1> &base_quad, const Point<dim> &singularity)
   :
-/**
-* We need the explicit implementation if dim == 1. If dim > 1 we use the
-* former implementation and apply a tensorial product to obtain the higher
-* dimensions.
-**/
+  // We need the explicit implementation if dim == 1. If dim > 1 we use the
+  // former implementation and apply a tensorial product to obtain the higher
+  // dimensions.
   Quadrature<dim>(
     dim == 2 ?
     QAnisotropic<dim>(
@@ -1037,10 +875,8 @@ template <int dim>
 QTelles<dim>::QTelles (
   const unsigned int n, const Point<dim> &singularity)
   :
-/**
-* In this case we map the standard Gauss Legendre formula using the given
-* singularity point coordinates.
-**/
+  // In this case we map the standard Gauss Legendre formula using the given
+  // singularity point coordinates.
   Quadrature<dim>(QTelles<dim>(QGauss<1>(n), singularity))
 {}
 
@@ -1050,15 +886,11 @@ template <>
 QTelles<1>::QTelles (
   const Quadrature<1> &base_quad, const Point<1> &singularity)
   :
-/**
-* We explicitly implement the Telles' variable change if dim == 1.
-**/
+  // We explicitly implement the Telles' variable change if dim == 1.
   Quadrature<1>(base_quad)
 {
-  /**
-  * We define all the constants to be used in the implementation of
-  * Telles' rule
-  **/
+  // We define all the constants to be used in the implementation of
+  // Telles' rule
   const double eta_bar = singularity[0] * 2. - 1.;
   const double eta_star = eta_bar * eta_bar - 1.;
   double gamma_bar;
@@ -1086,7 +918,7 @@ QTelles<1>::QTelles (
     {
       quadrature_points.resize(quadrature_points_dummy.size()-1);
       weights.resize(weights_dummy.size()-1);
-      for (unsigned int d = 0; d < quadrature_points.size()-1; ++d)
+      for (unsigned int d = 0; d < quadrature_points.size(); ++d)
         {
           quadrature_points[d] = quadrature_points_dummy[d];
           weights[d] = weights_dummy[d];
@@ -1123,40 +955,49 @@ QTelles<1>::QTelles (
     }
 }
 
-
-
-template <>
-std::vector<double>
-QGaussChebyshev<1>::get_quadrature_points(const unsigned int n)
+namespace internal
 {
-
-  std::vector<double> points(n);
-  // n point quadrature: index from 0 to n-1
-  for (unsigned short i=0; i<n; ++i)
-    // would be cos((2i+1)Pi/(2N+2))
-    // put + Pi so we start from the smallest point
-    // then map from [-1,1] to [0,1]
-    points[i] = 1./2.*(1.+std::cos(numbers::PI*(1.+double(2*i+1)/double(2*(n-1)+2))));
-
-  return points;
-}
-
-
-template <>
-std::vector<double>
-QGaussChebyshev<1>::get_quadrature_weights(const unsigned int n)
-{
-
-  std::vector<double> weights(n);
-
-  for (unsigned short i=0; i<n; ++i)
+  namespace QGaussChebyshev
+  {
+    /**
+     * Computes the points of the quadrature formula.
+     */
+    std::vector<double>
+    get_quadrature_points(const unsigned int n)
     {
-      // same weights as on [-1,1]
-      weights[i] = numbers::PI/double(n);
+
+      std::vector<double> points(n);
+      // n point quadrature: index from 0 to n-1
+      for (unsigned short i=0; i<n; ++i)
+        // would be cos((2i+1)Pi/(2N+2))
+        // put + Pi so we start from the smallest point
+        // then map from [-1,1] to [0,1]
+        points[i] = 1./2.*(1.+std::cos(numbers::PI*(1.+double(2*i+1)/double(2*(n-1)+2))));
+
+      return points;
     }
 
-  return weights;
 
+
+    /**
+    * Computes the weights of the quadrature formula.
+    */
+    std::vector<double>
+    get_quadrature_weights(const unsigned int n)
+    {
+
+      std::vector<double> weights(n);
+
+      for (unsigned short i=0; i<n; ++i)
+        {
+          // same weights as on [-1,1]
+          weights[i] = numbers::PI/double(n);
+        }
+
+      return weights;
+
+    }
+  }
 }
 
 
@@ -1167,98 +1008,100 @@ QGaussChebyshev<1>::QGaussChebyshev(const unsigned int n)
 {
 
   Assert(n>0,ExcMessage("Need at least one point for the quadrature rule"));
-  std::vector<double> p=get_quadrature_points(n);
-  std::vector<double> w=get_quadrature_weights(n);
+  std::vector<double> p=internal::QGaussChebyshev::get_quadrature_points(n);
+  std::vector<double> w=internal::QGaussChebyshev::get_quadrature_weights(n);
 
   for (unsigned int i=0; i<this->size(); ++i)
     {
       this->quadrature_points[i] = Point<1>(p[i]);
       this->weights[i]           = w[i];
     }
-
 }
 
 
 template <int dim>
 QGaussChebyshev<dim>::QGaussChebyshev (const unsigned int n)
   :
-  Quadrature<dim> (QGaussChebyshev<dim-1>(n), QGaussChebyshev<1>(n))
+  Quadrature<dim> (QGaussChebyshev<1>(n))
 {}
 
 
-
-
-
-template <>
-std::vector<double>
-QGaussRadauChebyshev<1>::get_quadrature_points(const unsigned int n,
-                                               EndPoint ep)
+namespace internal
 {
-
-  std::vector<double> points(n);
-  // n point quadrature: index from 0 to n-1
-  for (unsigned short i=0; i<n; ++i)
-    // would be -cos(2i Pi/(2N+1))
-    // put + Pi so we start from the smallest point
-    // then map from [-1,1] to [0,1]
-    switch (ep)
-      {
-      case QGaussRadauChebyshev::left:
-      {
-        points[i] = 1./2.*(1.-std::cos(numbers::PI*(1+2*double(i)/(2*double(n-1)+1.))));
-        break;
-      }
-
-      case QGaussRadauChebyshev::right:
-      {
-        points[i] = 1./2.*(1.-std::cos(numbers::PI*(2*double(n-1-i)/(2*double(n-1)+1.))));
-        break;
-      }
-
-      default:
-        Assert (false, ExcMessage ("This constructor can only be called with either "
-                                   "QGaussRadauChebyshev::left or QGaussRadauChebyshev::right as "
-                                   "second argument."));
-      }
-
-  return points;
-}
-
-
-template <>
-std::vector<double>
-QGaussRadauChebyshev<1>::get_quadrature_weights(const unsigned int n,
-                                                EndPoint ep)
-{
-
-  std::vector<double> weights(n);
-
-  for (unsigned short i=0; i<n; ++i)
+  namespace QGaussRadauChebyshev
+  {
+    // Computes the points of the quadrature formula.
+    std::vector<double>
+    get_quadrature_points(const unsigned int n,
+                          ::dealii::QGaussRadauChebyshev<1>::EndPoint ep)
     {
-      // same weights as on [-1,1]
-      weights[i] = 2.*numbers::PI/double(2*(n-1)+1.);
-      if (ep==left && i==0)
-        weights[i] /= 2.;
-      else if (ep==right && i==(n-1))
-        weights[i] /= 2.;
+
+      std::vector<double> points(n);
+      // n point quadrature: index from 0 to n-1
+      for (unsigned short i=0; i<n; ++i)
+        // would be -cos(2i Pi/(2N+1))
+        // put + Pi so we start from the smallest point
+        // then map from [-1,1] to [0,1]
+        switch (ep)
+          {
+          case ::dealii::QGaussRadauChebyshev<1>::left:
+          {
+            points[i] = 1./2.*(1.-std::cos(numbers::PI*(1+2*double(i)/(2*double(n-1)+1.))));
+            break;
+          }
+
+          case ::dealii::QGaussRadauChebyshev<1>::right:
+          {
+            points[i] = 1./2.*(1.-std::cos(numbers::PI*(2*double(n-1-i)/(2*double(n-1)+1.))));
+            break;
+          }
+
+          default:
+            Assert (false, ExcMessage ("This constructor can only be called with either "
+                                       "QGaussRadauChebyshev::left or QGaussRadauChebyshev::right as "
+                                       "second argument."));
+          }
+
+      return points;
     }
 
-  return weights;
 
+
+    // Computes the weights of the quadrature formula.
+    std::vector<double>
+    get_quadrature_weights(const unsigned int n,
+                           ::dealii::QGaussRadauChebyshev<1>::EndPoint ep)
+    {
+
+      std::vector<double> weights(n);
+
+      for (unsigned short i=0; i<n; ++i)
+        {
+          // same weights as on [-1,1]
+          weights[i] = 2.*numbers::PI/double(2*(n-1)+1.);
+          if (ep==::dealii::QGaussRadauChebyshev<1>::left && i==0)
+            weights[i] /= 2.;
+          else if (ep==::dealii::QGaussRadauChebyshev<1>::right && i==(n-1))
+            weights[i] /= 2.;
+        }
+
+      return weights;
+    }
+  }
 }
 
 
 template <>
 QGaussRadauChebyshev<1>::QGaussRadauChebyshev(const unsigned int n,
-                                              QGaussRadauChebyshev<1>::EndPoint ep)
+                                              EndPoint ep)
   :
   Quadrature<1> (n),
   ep (ep)
 {
 
   Assert(n>0,ExcMessage("Need at least one point for quadrature rules"));
-  std::vector<double> p=get_quadrature_points(n,ep);
-  std::vector<double> w=get_quadrature_weights(n,ep);
+  std::vector<double> p=internal::QGaussRadauChebyshev::get_quadrature_points(n,ep);
+  std::vector<double> w=internal::QGaussRadauChebyshev::get_quadrature_weights(n,ep);
 
   for (unsigned int i=0; i<this->size(); ++i)
     {
@@ -1268,63 +1111,58 @@ QGaussRadauChebyshev<1>::QGaussRadauChebyshev(const unsigned int n,
 }
 
 
-template <>
-QGaussRadauChebyshev<2>::QGaussRadauChebyshev (const unsigned int n,
-                                               EndPoint ep)
-  :
-  Quadrature<2> (QGaussRadauChebyshev<1>(n, static_cast<QGaussRadauChebyshev<1>::EndPoint>(ep)),
-                 QGaussRadauChebyshev<1>(n, static_cast<QGaussRadauChebyshev<1>::EndPoint>(ep))),
-  ep (ep)
-{}
-
-
 template <int dim>
 QGaussRadauChebyshev<dim>::QGaussRadauChebyshev (const unsigned int n,
                                                  EndPoint ep)
   :
-  Quadrature<dim> (QGaussRadauChebyshev<dim-1>(n,static_cast<typename QGaussRadauChebyshev<dim-1>::EndPoint>(ep)),
-                   QGaussRadauChebyshev<1>(n,static_cast<QGaussRadauChebyshev<1>::EndPoint>(ep))),
+  Quadrature<dim> (QGaussRadauChebyshev<1>(n,static_cast<QGaussRadauChebyshev<1>::EndPoint>(ep))),
   ep (ep)
 {}
 
 
-template <>
-std::vector<double>
-QGaussLobattoChebyshev<1>::get_quadrature_points(const unsigned int n)
+
+namespace internal
 {
-
-  std::vector<double> points(n);
-  // n point quadrature: index from 0 to n-1
-  for (unsigned short i=0; i<n; ++i)
-    // would be cos(i Pi/N)
-    // put + Pi so we start from the smallest point
-    // then map from [-1,1] to [0,1]
-    points[i] = 1./2.*(1.+std::cos(numbers::PI*(1+double(i)/double(n-1))));
-
-  return points;
-}
-
-
-template <>
-std::vector<double>
-QGaussLobattoChebyshev<1>::get_quadrature_weights(const unsigned int n)
-{
-
-  std::vector<double> weights(n);
-
-  for (unsigned short i=0; i<n; ++i)
+  namespace QGaussLobattoChebyshev
+  {
+    // Computes the points of the quadrature formula.
+    std::vector<double>
+    get_quadrature_points(const unsigned int n)
     {
-      // same weights as on [-1,1]
-      weights[i] = numbers::PI/double((n-1));
-      if (i==0 || i==(n-1))
-        weights[i] /= 2.;
+
+      std::vector<double> points(n);
+      // n point quadrature: index from 0 to n-1
+      for (unsigned short i=0; i<n; ++i)
+        // would be cos(i Pi/N)
+        // put + Pi so we start from the smallest point
+        // then map from [-1,1] to [0,1]
+        points[i] = 1./2.*(1.+std::cos(numbers::PI*(1+double(i)/double(n-1))));
+
+      return points;
     }
 
-  return weights;
+    // Computes the weights of the quadrature formula.
+    std::vector<double>
+    get_quadrature_weights(const unsigned int n)
+    {
 
+      std::vector<double> weights(n);
+
+      for (unsigned short i=0; i<n; ++i)
+        {
+          // same weights as on [-1,1]
+          weights[i] = numbers::PI/double((n-1));
+          if (i==0 || i==(n-1))
+            weights[i] /= 2.;
+        }
+
+      return weights;
+    }
+  }
 }
 
 
+
 template <>
 QGaussLobattoChebyshev<1>::QGaussLobattoChebyshev(const unsigned int n)
   :
@@ -1332,24 +1170,184 @@ QGaussLobattoChebyshev<1>::QGaussLobattoChebyshev(const unsigned int n)
 {
 
   Assert(n>1,ExcMessage("Need at least two points for Gauss-Lobatto quadrature rule"));
-  std::vector<double> p=get_quadrature_points(n);
-  std::vector<double> w=get_quadrature_weights(n);
+  std::vector<double> p=internal::QGaussLobattoChebyshev::get_quadrature_points(n);
+  std::vector<double> w=internal::QGaussLobattoChebyshev::get_quadrature_weights(n);
 
   for (unsigned int i=0; i<this->size(); ++i)
     {
       this->quadrature_points[i] = Point<1>(p[i]);
       this->weights[i]           = w[i];
     }
-
 }
 
 
 template <int dim>
 QGaussLobattoChebyshev<dim>::QGaussLobattoChebyshev (const unsigned int n)
   :
-  Quadrature<dim> (QGaussLobattoChebyshev<dim-1>(n), QGaussLobattoChebyshev<1>(n))
+  Quadrature<dim> (QGaussLobattoChebyshev<1>(n))
 {}
 
+
+
+template<int dim>
+QSimplex<dim>::QSimplex(const Quadrature<dim> &quad)
+{
+  std::vector<Point<dim> > qpoints;
+  std::vector<double > weights;
+
+  for (unsigned int i=0; i<quad.size(); ++i)
+    {
+      double r=0;
+      for (unsigned int d=0; d<dim; ++d)
+        r += quad.point(i)[d];
+      if (r <= 1+1e-10)
+        {
+          this->quadrature_points.push_back(quad.point(i));
+          this->weights.push_back(quad.weight(i));
+        }
+    }
+}
+
+
+
+template<int dim>
+Quadrature<dim>
+QSimplex<dim>::compute_affine_transformation(const std::array<Point<dim>, dim+1>& vertices) const
+{
+  Tensor<2,dim> B;
+  for (unsigned int d=0; d<dim; ++d)
+    B[d] = vertices[d+1]-vertices[0];
+
+  B = transpose(B);
+  const double J = std::abs(determinant(B));
+
+  // if the determinant is zero, we return an empty quadrature
+  if (J < 1e-12)
+    return Quadrature<dim>();
+
+  std::vector<Point<dim> > qp(this->size());
+  std::vector<double> w(this->size());
+
+  for (unsigned int i=0; i<this->size(); ++i)
+    {
+      qp[i] = Point<dim>(vertices[0]+B*this->point(i));
+      w[i] = J*this->weight(i);
+    }
+
+  return Quadrature<dim>(qp, w);
+}
+
+
+
+QTrianglePolar::QTrianglePolar(const Quadrature<1> &radial_quadrature,
+                               const Quadrature<1> &angular_quadrature) :
+  QSimplex<2>(Quadrature<2>())
+{
+  const QAnisotropic<2> base(radial_quadrature, angular_quadrature);
+  this->quadrature_points.resize(base.size());
+  this->weights.resize(base.size());
+  for (unsigned int i=0; i<base.size(); ++i)
+    {
+      const auto q = base.point(i);
+      const auto w = base.weight(i);
+
+      const auto xhat = q[0];
+      const auto yhat = q[1];
+
+      const double t = numbers::PI_2*yhat;
+      const double pi = numbers::PI;
+      const double st = std::sin(t);
+      const double ct = std::cos(t);
+      const double r = xhat/(st+ct);
+
+      const double J = pi*xhat/(2*(std::sin(pi*yhat) + 1));
+
+      this->quadrature_points[i] = Point<2>(r*ct, r*st);
+      this->weights[i] = w*J;
+    }
+}
+
+
+
+QTrianglePolar::QTrianglePolar(const unsigned int &n)
+  :QTrianglePolar(QGauss<1>(n),QGauss<1>(n))
+{}
+
+
+
+QDuffy::QDuffy(const Quadrature<1> &radial_quadrature,
+               const Quadrature<1> &angular_quadrature,
+               const double beta) :
+  QSimplex<2>(Quadrature<2>())
+{
+  const QAnisotropic<2> base(radial_quadrature, angular_quadrature);
+  this->quadrature_points.resize(base.size());
+  this->weights.resize(base.size());
+  for (unsigned int i=0; i<base.size(); ++i)
+    {
+      const auto q = base.point(i);
+      const auto w = base.weight(i);
+
+      const auto xhat = q[0];
+      const auto yhat = q[1];
+
+      const double x = std::pow(xhat, beta)*(1-yhat);
+      const double y = std::pow(xhat, beta)*yhat;
+
+      const double J = beta * std::pow(xhat, 2.*beta-1.);
+
+      this->quadrature_points[i] = Point<2>(x,y);
+      this->weights[i] = w*J;
+    }
+}
+
+
+
+QDuffy::QDuffy(const unsigned int n, const double beta)
+  :QDuffy(QGauss<1>(n), QGauss<1>(n), beta)
+{}
+
+
+
+template<int dim>
+QSplit<dim>::QSplit(const QSimplex<dim> &base,
+                    const Point<dim> &split_point)
+{
+  AssertThrow(GeometryInfo<dim>::is_inside_unit_cell(split_point, 1e-12),
+              ExcMessage("The split point should be inside the unit reference cell."));
+
+  std::array<Point<dim>, dim+1> vertices;
+  vertices[0] = split_point;
+
+  // Make a simplex from the split_point and the first dim vertices of each
+  // face. In dimension three, we need to split the face in two triangles, so
+  // we use once the first dim vertices of each face, and the second time the
+  // the dim vertices of each face starting from 1.
+  for (unsigned int f=0; f< GeometryInfo<dim>::faces_per_cell; ++f)
+    for (unsigned int start=0; start < (dim >2 ? 2 : 1); ++start)
+      {
+        for (unsigned int i=0; i<dim; ++i)
+          vertices[i+1] =
+            GeometryInfo<dim>::unit_cell_vertex(
+              GeometryInfo<dim>::face_to_cell_vertices(f, i+start)
+            );
+        const auto quad = base.compute_affine_transformation(vertices);
+        if (quad.size())
+          {
+            this->quadrature_points.insert(
+              this->quadrature_points.end(),
+              quad.get_points().begin(),
+              quad.get_points().end());
+            this->weights.insert(
+              this->weights.end(),
+              quad.get_weights().begin(),
+              quad.get_weights().end());
+          }
+      }
+}
+
+
+
 // explicit specialization
 // note that 1d formulae are specialized by implementation above
 template class QGauss<2>;
@@ -1388,4 +1386,12 @@ template class QGaussLobattoChebyshev<1>;
 template class QGaussLobattoChebyshev<2>;
 template class QGaussLobattoChebyshev<3>;
 
+template class QSimplex<1>;
+template class QSimplex<2>;
+template class QSimplex<3>;
+
+template class QSplit<1>;
+template class QSplit<2>;
+template class QSplit<3>;
+
 DEAL_II_NAMESPACE_CLOSE
