@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 1998 - 2016 by the deal.II authors
+// Copyright (C) 1998 - 2018 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -24,30 +24,13 @@
 DEAL_II_NAMESPACE_OPEN
 
 
-#ifdef DEBUG
-namespace
-{
-// create a lock that might be used to control subscription to and
-// unsubscription from objects, as that might happen in parallel.
-// since it should happen rather seldom that several threads try to
-// operate on different objects at the same time (the usual case is
-// that they subscribe to the same object right after thread
-// creation), a global lock should be sufficient, rather than one that
-// operates on a per-object base (in which case we would have to
-// include the huge <thread_management.h> file into the
-// <subscriptor.h> file).
-  Threads::Mutex subscription_lock;
-}
-#endif
-
-
 static const char *unknown_subscriber = "unknown subscriber";
 
 
 Subscriptor::Subscriptor ()
   :
   counter (0),
-  object_info (0)
+  object_info (nullptr)
 {
   // this has to go somewhere to avoid an extra warning.
   (void)unknown_subscriber;
@@ -58,36 +41,29 @@ Subscriptor::Subscriptor ()
 Subscriptor::Subscriptor (const Subscriptor &)
   :
   counter (0),
-  object_info (0)
+  object_info (nullptr)
 {}
 
 
 
-#ifdef DEAL_II_WITH_CXX11
-Subscriptor::Subscriptor (Subscriptor &&subscriptor)
-  :
-  counter(0),
-  object_info (subscriptor.object_info)
+Subscriptor::Subscriptor (Subscriptor &&subscriptor) noexcept
+:
+counter(0),
+        object_info (subscriptor.object_info)
 {
   subscriptor.check_no_subscribers();
 }
-#endif
 
 
 
 Subscriptor::~Subscriptor ()
 {
   check_no_subscribers();
-
-#ifdef DEAL_II_WITH_CXX11
   object_info = nullptr;
-#else
-  object_info = 0;
-#endif
 }
 
 
-void Subscriptor::check_no_subscribers () const
+void Subscriptor::check_no_subscribers () const noexcept
 {
   // Check whether there are still subscriptions to this object. If so, output
   // the actual name of the class to which this object belongs, i.e. the most
@@ -125,7 +101,7 @@ void Subscriptor::check_no_subscribers () const
             infostring = "<none>";
 
           AssertNothrow (counter == 0,
-                         ExcInUse (counter, object_info->name(), infostring));
+                         ExcInUse (counter.load(), object_info->name(), infostring));
         }
       else
         {
@@ -145,7 +121,6 @@ void Subscriptor::check_no_subscribers () const
                     << std::endl;
         }
     }
-
 #endif
 }
 
@@ -153,31 +128,33 @@ void Subscriptor::check_no_subscribers () const
 
 Subscriptor &Subscriptor::operator = (const Subscriptor &s)
 {
+  check_no_subscribers();
   object_info = s.object_info;
   return *this;
 }
 
 
 
-#ifdef DEAL_II_WITH_CXX11
-Subscriptor &Subscriptor::operator = (Subscriptor &&s)
+Subscriptor &Subscriptor::operator = (Subscriptor &&s) noexcept
 {
+  check_no_subscribers();
   s.check_no_subscribers();
   object_info = s.object_info;
   return *this;
 }
-#endif
+
 
 
 void
 Subscriptor::subscribe(const char *id) const
 {
 #ifdef DEBUG
-  if (object_info == 0)
+  if (object_info == nullptr)
     object_info = &typeid(*this);
-  Threads::Mutex::ScopedLock lock (subscription_lock);
   ++counter;
 
+  // This feature is disabled when we compile with threads: see the
+  // documentation of this class.
 #  ifndef DEAL_II_WITH_THREADS
   const char *const name = (id != 0) ? id : unknown_subscriber;
 
@@ -200,16 +177,17 @@ void
 Subscriptor::unsubscribe(const char *id) const
 {
 #ifdef DEBUG
-  const char *name = (id != 0) ? id : unknown_subscriber;
+  const char *name = (id != nullptr) ? id : unknown_subscriber;
   AssertNothrow (counter>0, ExcNoSubscriber(object_info->name(), name));
   // This is for the case that we do
   // not abort after the exception
   if (counter == 0)
     return;
 
-  Threads::Mutex::ScopedLock lock (subscription_lock);
   --counter;
 
+  // This feature is disabled when we compile with threads: see the
+  // documentation of this class.
 #  ifndef DEAL_II_WITH_THREADS
   map_iterator it = counter_map.find(name);
   AssertNothrow (it != counter_map.end(), ExcNoSubscriber(object_info->name(), name));
