@@ -31,10 +31,6 @@ DEAL_II_NAMESPACE_OPEN
 // inline and template functions
 namespace internal
 {
-  TableEntry::TableEntry ()
-  {}
-
-
   double TableEntry::get_numeric_value () const
   {
     // we don't quite know the data type in 'value', but
@@ -59,6 +55,14 @@ namespace internal
     catch (...)
       {}
 
+    // ... then with unsigned long long int...
+    try
+      {
+        return boost::get<unsigned long long int>(value);
+      }
+    catch (...)
+      {}
+
     // ...and finally with double precision:
     try
       {
@@ -129,7 +133,7 @@ TableHandler::Column::Column(const std::string &tex_caption)
   tex_caption(tex_caption),
   tex_format("c"),
   precision(4),
-  scientific(0),
+  scientific(false),
   flag(0),
   max_length(0)
 {}
@@ -141,7 +145,7 @@ TableHandler::Column::Column()
   tex_caption(),
   tex_format("c"),
   precision(4),
-  scientific(0),
+  scientific(false),
   flag(0),
   max_length(0)
 {}
@@ -189,6 +193,42 @@ TableHandler::TableHandler()
 
 
 
+void TableHandler::declare_column (const std::string &key)
+{
+  // see if the column already exists; insert it if not
+  Assert (columns.find(key) == columns.end(),
+          ExcMessage ("You are trying to declare a column with key <" + key +
+                      "> but such a column already exists."));
+
+  columns.insert(std::make_pair(key, Column(key)));
+  column_order.push_back(key);
+}
+
+
+
+void TableHandler::start_new_row ()
+{
+  // figure out the longest current column
+  unsigned int max_col_length = 0;
+  for (std::map< std::string, Column >::iterator p = columns.begin(); p != columns.end(); ++p)
+    max_col_length = std::max(max_col_length,
+                              static_cast<unsigned int>(p->second.entries.size()));
+
+
+  // then pad all columns to that length with empty strings
+  for (std::map<std::string,Column>::iterator col=columns.begin(); col!=columns.end(); ++col)
+    while (col->second.entries.size() < max_col_length)
+      {
+        col->second.entries.emplace_back("");
+        internal::TableEntry &entry = col->second.entries.back();
+        entry.cache_string(col->second.scientific, col->second.precision);
+        col->second.max_length = std::max(col->second.max_length,
+                                          static_cast<unsigned int>(entry.get_cached_string().length()));
+      }
+}
+
+
+
 void
 TableHandler::set_auto_fill_mode (const bool state)
 {
