@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 2000 - 2015 by the deal.II authors
+// Copyright (C) 2000 - 2017 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -15,10 +15,9 @@
 
 #include <deal.II/base/thread_management.h>
 
-#include <cerrno>
+#include <atomic>
 #include <cstdlib>
 #include <iostream>
-#include <list>
 
 #ifdef DEAL_II_HAVE_UNISTD_H
 #  include <unistd.h>
@@ -32,15 +31,11 @@ namespace Threads
 {
   namespace internal
   {
-    // counter and access mutex for the
-    // number of threads
-    volatile unsigned int n_existing_threads_counter = 1;
-    Mutex  n_existing_threads_mutex;
+    static std::atomic<unsigned int> n_existing_threads_counter(1);
 
 
     void register_thread ()
     {
-      Mutex::ScopedLock lock (n_existing_threads_mutex);
       ++n_existing_threads_counter;
     }
 
@@ -48,7 +43,6 @@ namespace Threads
 
     void deregister_thread ()
     {
-      Mutex::ScopedLock lock (n_existing_threads_mutex);
       --n_existing_threads_counter;
       Assert (n_existing_threads_counter >= 1,
               ExcInternalError());
@@ -56,6 +50,7 @@ namespace Threads
 
 
 
+    [[noreturn]]
     void handle_std_exception (const std::exception &exc)
     {
       // lock the following context
@@ -93,6 +88,7 @@ namespace Threads
 
 
 
+    [[noreturn]]
     void handle_unknown_exception ()
     {
       // lock the following context
@@ -128,7 +124,6 @@ namespace Threads
 
   unsigned int n_existing_threads ()
   {
-    Mutex::ScopedLock lock (internal::n_existing_threads_mutex);
     return internal::n_existing_threads_counter;
   }
 
@@ -167,7 +162,7 @@ namespace Threads
                                           const char *,
                                           void *)
   {
-    pthread_barrier_init (&barrier, 0, count);
+    pthread_barrier_init (&barrier, nullptr, count);
   }
 
 #else
