@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 2005 - 2017 by the deal.II authors
+// Copyright (C) 2005 - 2018 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -13,17 +13,23 @@
 //
 // ---------------------------------------------------------------------
 
+#include <deal.II/base/config.h>
+
+// It's necessary to include winsock2.h before thread_local_storage.h,
+// because Intel implementation of TBB includes winsock.h,
+// and we'll get a conflict between winsock.h and winsock2.h otherwise.
+#ifdef DEAL_II_MSVC
+#  include <winsock2.h>
+#endif
 
 #include <deal.II/base/utilities.h>
 #include <deal.II/base/mpi.h>
 #include <deal.II/base/exceptions.h>
 #include <deal.II/base/thread_local_storage.h>
 
-DEAL_II_DISABLE_EXTRA_DIAGNOSTICS
 #include <boost/math/special_functions/erf.hpp>
 #include <boost/lexical_cast.hpp>
 #include <boost/random.hpp>
-DEAL_II_ENABLE_EXTRA_DIAGNOSTICS
 
 #include <algorithm>
 #include <cctype>
@@ -38,27 +44,25 @@ DEAL_II_ENABLE_EXTRA_DIAGNOSTICS
 #include <limits>
 #include <sstream>
 
+#if defined(DEAL_II_HAVE_UNISTD_H) && defined(DEAL_II_HAVE_GETHOSTNAME)
+#  include <unistd.h>
+#endif
+
 #ifndef DEAL_II_MSVC
 #  include <stdlib.h>
 #endif
 
-#ifdef DEAL_II_MSVC
-#  include <winsock2.h>
-#endif
 
-
-DEAL_II_DISABLE_EXTRA_DIAGNOSTICS
 #ifdef DEAL_II_WITH_TRILINOS
 #  ifdef DEAL_II_WITH_MPI
 #    include <Epetra_MpiComm.h>
 #    include <deal.II/lac/vector_memory.h>
 #    include <deal.II/lac/trilinos_vector.h>
-#    include <deal.II/lac/trilinos_block_vector.h>
+#    include <deal.II/lac/trilinos_parallel_block_vector.h>
 #  endif
-#  include "Teuchos_RCP.hpp"
-#  include "Epetra_SerialComm.h"
+#  include <Teuchos_RCP.hpp>
+#  include <Epetra_SerialComm.h>
 #endif
-DEAL_II_ENABLE_EXTRA_DIAGNOSTICS
 
 
 
@@ -81,12 +85,23 @@ namespace Utilities
                   << "Can't convert the string " << arg1
                   << " to the desired type");
 
+
+  std::string
+  dealii_version_string ()
+  {
+    return DEAL_II_PACKAGE_NAME " version " DEAL_II_PACKAGE_VERSION;
+  }
+
+
+
   std::string
   int_to_string (const unsigned int value, const unsigned int digits)
   {
     return to_string(value,digits);
   }
 
+
+
   template <typename number>
   std::string
   to_string (const number value, const unsigned int digits)
@@ -266,7 +281,7 @@ namespace Utilities
 
   std::vector<std::string>
   split_string_list (const std::string &s,
-                     const char         delimiter)
+                     const std::string &delimiter)
   {
     // keep the currently remaining part of the input string in 'tmp' and
     // keep chopping elements of the list off the front
@@ -284,7 +299,6 @@ namespace Utilities
     // there was space after the last delimiter. this matches what's
     // discussed in the documentation
     std::vector<std::string> split_list;
-    split_list.reserve (std::count (tmp.begin(), tmp.end(), delimiter)+1);
     while (tmp.length() != 0)
       {
         std::string name;
@@ -293,7 +307,7 @@ namespace Utilities
         if (name.find(delimiter) != std::string::npos)
           {
             name.erase (name.find(delimiter), std::string::npos);
-            tmp.erase (0, tmp.find(delimiter)+1);
+            tmp.erase (0, tmp.find(delimiter)+delimiter.size());
           }
         else
           tmp = "";
@@ -311,6 +325,15 @@ namespace Utilities
   }
 
 
+  std::vector<std::string>
+  split_string_list (const std::string &s,
+                     const char delimiter)
+  {
+    std::string d = ",";
+    d[0] = delimiter;
+    return split_string_list(s,d);
+  }
+
 
   std::vector<std::string>
   break_text_into_lines (const std::string &original_text,
@@ -332,7 +355,7 @@ namespace Utilities
         while ((text.length() != 0) && (text[0] == delimiter))
           text.erase(0, 1);
 
-        std::size_t pos_newline = text.find_first_of("\n", 0);
+        std::size_t pos_newline = text.find_first_of('\n', 0);
         if (pos_newline != std::string::npos && pos_newline <= width)
           {
             std::string line (text, 0, pos_newline);
@@ -608,6 +631,24 @@ namespace Utilities
 
 #endif
 
+    const std::string
+    get_current_vectorization_level()
+    {
+      switch (DEAL_II_COMPILER_VECTORIZATION_LEVEL)
+        {
+        case 0:
+          return "disabled";
+        case 1:
+          return "SSE2";
+        case 2:
+          return "AVX";
+        case 3:
+          return "AVX512";
+        default:
+          AssertThrow(false, ExcInternalError("Invalid DEAL_II_COMPILER_VECTORIZATION_LEVEL."));
+          return "ERROR";
+        }
+    }
 
 
     void get_memory_stats (MemoryStats &stats)
@@ -659,7 +700,7 @@ namespace Utilities
 
     std::string get_time ()
     {
-      std::time_t  time1= std::time (0);
+      std::time_t  time1= std::time (nullptr);
       std::tm     *time = std::localtime(&time1);
 
       std::ostringstream o;
@@ -674,7 +715,7 @@ namespace Utilities
 
     std::string get_date ()
     {
-      std::time_t  time1= std::time (0);
+      std::time_t  time1= std::time (nullptr);
       std::tm     *time = std::localtime(&time1);
 
       std::ostringstream o;
@@ -693,7 +734,7 @@ namespace Utilities
       const int ierr = ::posix_memalign (memptr, alignment, size);
 
       AssertThrow (ierr == 0, ExcOutOfMemory());
-      AssertThrow (*memptr != 0, ExcOutOfMemory());
+      AssertThrow (*memptr != nullptr, ExcOutOfMemory());
 #else
       // Windows does not appear to have posix_memalign. just use the
       // regular malloc in that case
@@ -758,7 +799,7 @@ namespace Utilities
       // return a duplicate of it
       const Epetra_MpiComm
       *mpi_comm = dynamic_cast<const Epetra_MpiComm *>(&communicator);
-      if (mpi_comm != 0)
+      if (mpi_comm != nullptr)
         return new Epetra_MpiComm(Utilities::MPI::
                                   duplicate_communicator(mpi_comm->GetMpiComm()));
 #endif
@@ -768,7 +809,7 @@ namespace Utilities
       // not an MPI communicator, return a
       // copy of the same object again
       Assert (dynamic_cast<const Epetra_SerialComm *>(&communicator)
-              != 0,
+              != nullptr,
               ExcInternalError());
       return new Epetra_SerialComm(dynamic_cast<const Epetra_SerialComm &>(communicator));
     }
@@ -782,7 +823,7 @@ namespace Utilities
 #ifdef DEAL_II_WITH_MPI
       Epetra_MpiComm
       *mpi_comm = dynamic_cast<Epetra_MpiComm *>(&communicator);
-      if (mpi_comm != 0)
+      if (mpi_comm != nullptr)
         {
           MPI_Comm comm = mpi_comm->GetMpiComm();
           *mpi_comm = Epetra_MpiComm(MPI_COMM_SELF);
