@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 2015 - 2017 by the deal.II authors
+// Copyright (C) 2015 - 2018 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -76,7 +76,7 @@ namespace parallel
 
         // release unused vector memory because we will have very different
         // vectors now
-        ::dealii::internal::GrowingVectorMemory::release_all_unused_memory();
+        ::dealii::internal::GrowingVectorMemoryImplementation::release_all_unused_memory();
       }
 #endif
   }
@@ -103,7 +103,7 @@ namespace parallel
   {
     // release unused vector memory because the vector layout is going to look
     // very different now
-    ::dealii::internal::GrowingVectorMemory::release_all_unused_memory();
+    ::dealii::internal::GrowingVectorMemoryImplementation::release_all_unused_memory();
   }
 
   template <int dim, int spacedim>
@@ -202,7 +202,7 @@ namespace parallel
     const int ierr = MPI_Allgather (&send_value,
                                     1,
                                     MPI_UNSIGNED,
-                                    &number_cache.n_locally_owned_active_cells[0],
+                                    number_cache.n_locally_owned_active_cells.data(),
                                     1,
                                     MPI_UNSIGNED,
                                     this->mpi_communicator);
@@ -215,7 +215,77 @@ namespace parallel
                          static_cast<types::global_dof_index>(0));
     number_cache.n_global_levels = Utilities::MPI::max(this->n_levels(), this->mpi_communicator);
   }
+
+
+
+  template <int dim, int spacedim>
+  void
+  Triangulation<dim,spacedim>::fill_level_ghost_owners ()
+  {
+    number_cache.level_ghost_owners.clear ();
+
+    // if there is nothing to do, then do nothing
+    if (this->n_levels() == 0)
+      return;
+
+    // find level ghost owners
+    for (typename Triangulation<dim,spacedim>::cell_iterator
+         cell = this->begin();
+         cell != this->end();
+         ++cell)
+      if (cell->level_subdomain_id() != numbers::artificial_subdomain_id
+          && cell->level_subdomain_id() != this->locally_owned_subdomain())
+        this->number_cache.level_ghost_owners.insert(cell->level_subdomain_id());
+
+#ifdef DEBUG
+    // Check that level_ghost_owners is symmetric by sending a message to everyone
+    {
+      int ierr = MPI_Barrier(this->mpi_communicator);
+      AssertThrowMPI(ierr);
+
+      // important: preallocate to avoid (re)allocation:
+      std::vector<MPI_Request> requests (this->number_cache.level_ghost_owners.size());
+      unsigned int dummy = 0;
+      unsigned int req_counter = 0;
+
+      for (std::set<types::subdomain_id>::iterator it = this->number_cache.level_ghost_owners.begin();
+           it != this->number_cache.level_ghost_owners.end();
+           ++it, ++req_counter)
+        {
+          ierr = MPI_Isend(&dummy, 1, MPI_UNSIGNED,
+                           *it, 9001, this->mpi_communicator,
+                           &requests[req_counter]);
+          AssertThrowMPI(ierr);
+        }
+
+      for (std::set<types::subdomain_id>::iterator it = this->number_cache.level_ghost_owners.begin();
+           it != this->number_cache.level_ghost_owners.end();
+           ++it)
+        {
+          unsigned int dummy;
+          ierr = MPI_Recv(&dummy, 1, MPI_UNSIGNED,
+                          *it, 9001, this->mpi_communicator,
+                          MPI_STATUS_IGNORE);
+          AssertThrowMPI(ierr);
+        }
+
+      if (requests.size() > 0)
+        {
+          ierr = MPI_Waitall(requests.size(), requests.data(), MPI_STATUSES_IGNORE);
+          AssertThrowMPI(ierr);
+        }
+
+      ierr = MPI_Barrier(this->mpi_communicator);
+      AssertThrowMPI(ierr);
+    }
+#endif
+
+    Assert(this->number_cache.level_ghost_owners.size() < Utilities::MPI::n_mpi_processes(this->mpi_communicator),
+           ExcInternalError());
+  }
+
 #else
+
   template <int dim, int spacedim>
   void
   Triangulation<dim,spacedim>::update_number_cache ()
@@ -223,6 +293,13 @@ namespace parallel
     Assert (false, ExcNotImplemented());
   }
 
+  template <int dim, int spacedim>
+  void
+  Triangulation<dim, spacedim>::fill_level_ghost_owners()
+  {
+    Assert(false, ExcNotImplemented());
+  }
+
 #endif
 
   template <int dim, int spacedim>
@@ -232,6 +309,8 @@ namespace parallel
     return my_subdomain;
   }
 
+
+
   template <int dim, int spacedim>
   const std::set<types::subdomain_id> &
   Triangulation<dim,spacedim>::
@@ -240,6 +319,8 @@ namespace parallel
     return number_cache.ghost_owners;
   }
 
+
+
   template <int dim, int spacedim>
   const std::set<types::subdomain_id> &
   Triangulation<dim,spacedim>::
@@ -248,6 +329,43 @@ namespace parallel
     return number_cache.level_ghost_owners;
   }
 
+
+
+  template <int dim, int spacedim>
+  std::map<unsigned int, std::set<dealii::types::subdomain_id> >
+  Triangulation<dim,spacedim>::
+  compute_vertices_with_ghost_neighbors () const
+  {
+    // TODO: we are not treating periodic neighbors correctly here. If we do
+    // we can remove the overriding implementation for p::d::Triangulation
+    // that is currently using a p4est callback to get correct ghost neighbors
+    // over periodic faces.
+    Assert(this->get_periodic_face_map().size()==0,
+           ExcNotImplemented());
+
+
+    std::vector<bool> vertex_of_own_cell(this->n_vertices(), false);
+
+    for (auto cell : this->active_cell_iterators())
+      if (cell->is_locally_owned())
+        for (unsigned int v=0; v<GeometryInfo<dim>::vertices_per_cell; ++v)
+          vertex_of_own_cell[cell->vertex_index(v)] = true;
+
+    std::map<unsigned int, std::set<dealii::types::subdomain_id> > result;
+    for (auto cell : this->active_cell_iterators())
+      if (cell->is_ghost())
+        {
+          const types::subdomain_id owner = cell->subdomain_id();
+          for (unsigned int v=0; v<GeometryInfo<dim>::vertices_per_cell; ++v)
+            {
+              if (vertex_of_own_cell[cell->vertex_index(v)])
+                result[cell->vertex_index(v)].insert(owner);
+            }
+        }
+
+    return result;
+  }
+
 } // end namespace parallel
 
 
