@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 2006 - 2015 by the deal.II authors
+// Copyright (C) 2006 - 2018 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -23,7 +23,7 @@ DEAL_II_NAMESPACE_OPEN
 
 namespace internal
 {
-  namespace DoFHandler
+  namespace DoFHandlerImplementation
   {
     template <int dim>
     std::size_t
@@ -64,7 +64,7 @@ namespace internal
 // explicit instantiations
 namespace internal
 {
-  namespace DoFHandler
+  namespace DoFHandlerImplementation
   {
 #include "dof_objects.inst"
   }
