@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 1999 - 2016 by the deal.II authors
+// Copyright (C) 1999 - 2017 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -125,10 +125,10 @@ namespace DoFTools
             ExcDimensionMismatch (sparsity.n_rows(), n_dofs));
     Assert (sparsity.n_cols() == n_dofs,
             ExcDimensionMismatch (sparsity.n_cols(), n_dofs));
-    Assert (couplings.n_rows() == dof.get_fe().n_components(),
-            ExcDimensionMismatch(couplings.n_rows(), dof.get_fe().n_components()));
-    Assert (couplings.n_cols() == dof.get_fe().n_components(),
-            ExcDimensionMismatch(couplings.n_cols(), dof.get_fe().n_components()));
+    Assert (couplings.n_rows() == dof.get_fe(0).n_components(),
+            ExcDimensionMismatch(couplings.n_rows(), dof.get_fe(0).n_components()));
+    Assert (couplings.n_cols() == dof.get_fe(0).n_components(),
+            ExcDimensionMismatch(couplings.n_cols(), dof.get_fe(0).n_components()));
 
     // If we have a distributed::Triangulation only allow locally_owned
     // subdomain. Not setting a subdomain is also okay, because we skip
@@ -143,7 +143,7 @@ namespace DoFTools
                   "associated DoF handler objects, asking for any subdomain other "
                   "than the locally owned one does not make sense."));
 
-    const hp::FECollection<DoFHandlerType::dimension,DoFHandlerType::space_dimension> fe_collection (dof.get_fe());
+    const hp::FECollection<DoFHandlerType::dimension,DoFHandlerType::space_dimension> &fe_collection = dof.get_fe_collection();
 
     const std::vector<Table<2,Coupling> > dof_mask //(fe_collection.size())
       = dof_couplings_from_component_couplings (fe_collection,
@@ -310,8 +310,8 @@ namespace DoFTools
         // there are only 2 boundary indicators in 1d, so it is no
         // performance problem to call the other function
         std::map<types::boundary_id, const Function<DoFHandlerType::space_dimension,double>*> boundary_ids;
-        boundary_ids[0] = 0;
-        boundary_ids[1] = 0;
+        boundary_ids[0] = nullptr;
+        boundary_ids[1] = nullptr;
         make_boundary_sparsity_pattern<DoFHandlerType, SparsityPatternType>
         (dof,
          boundary_ids,
@@ -332,7 +332,7 @@ namespace DoFTools
         types::global_dof_index max_element = 0;
         for (std::vector<types::global_dof_index>::const_iterator i=dof_to_boundary_mapping.begin();
              i!=dof_to_boundary_mapping.end(); ++i)
-          if ((*i != DoFHandlerType::invalid_dof_index) &&
+          if ((*i != numbers::invalid_dof_index) &&
               (*i > max_element))
             max_element = *i;
         AssertDimension (max_element, sparsity.n_rows()-1);
@@ -425,7 +425,7 @@ namespace DoFTools
         types::global_dof_index max_element = 0;
         for (std::vector<types::global_dof_index>::const_iterator i=dof_to_boundary_mapping.begin();
              i!=dof_to_boundary_mapping.end(); ++i)
-          if ((*i != DoFHandlerType::invalid_dof_index) &&
+          if ((*i != numbers::invalid_dof_index) &&
               (*i > max_element))
             max_element = *i;
         AssertDimension (max_element, sparsity.n_rows()-1);
@@ -978,7 +978,7 @@ namespace DoFTools
         // consequently, the implementation here is simpler and probably
         // less efficient but at least readable...
 
-        const dealii::hp::FECollection<dim,spacedim> &fe = dof.get_fe();
+        const dealii::hp::FECollection<dim,spacedim> &fe = dof.get_fe_collection();
 
         std::vector<types::global_dof_index> dofs_on_this_cell(DoFTools::max_dofs_per_cell(dof));
         std::vector<types::global_dof_index> dofs_on_other_cell(DoFTools::max_dofs_per_cell(dof));
@@ -1032,16 +1032,34 @@ namespace DoFTools
                   if (cell->at_boundary (face) && (!periodic_neighbor))
                     {
                       for (unsigned int i=0; i<cell->get_fe().dofs_per_cell; ++i)
-                        for (unsigned int j=0; j<cell->get_fe().dofs_per_cell; ++j)
-                          if ((flux_mask(cell->get_fe().system_to_component_index(i).first,
-                                         cell->get_fe().system_to_component_index(j).first)
-                               == always)
-                              ||
-                              (flux_mask(cell->get_fe().system_to_component_index(i).first,
-                                         cell->get_fe().system_to_component_index(j).first)
-                               == nonzero))
-                            sparsity.add (dofs_on_this_cell[i],
-                                          dofs_on_this_cell[j]);
+                        {
+                          const unsigned int ii
+                            = (cell->get_fe().is_primitive(i) ?
+                               cell->get_fe().system_to_component_index(i).first
+                               :
+                               cell->get_fe().get_nonzero_components(i).first_selected_component()
+                              );
+
+                          Assert (ii < cell->get_fe().n_components(), ExcInternalError());
+
+                          for (unsigned int j=0; j<cell->get_fe().dofs_per_cell; ++j)
+                            {
+                              const unsigned int jj
+                                = (cell->get_fe().is_primitive(j) ?
+                                   cell->get_fe().system_to_component_index(j).first
+                                   :
+                                   cell->get_fe().get_nonzero_components(j).first_selected_component()
+                                  );
+
+                              Assert (jj < cell->get_fe().n_components(), ExcInternalError());
+
+                              if ((flux_mask(ii,jj) == always)
+                                  ||
+                                  (flux_mask(ii,jj) == nonzero))
+                                sparsity.add (dofs_on_this_cell[i],
+                                              dofs_on_this_cell[j]);
+                            }
+                        }
                     }
                   else
                     {
@@ -1096,16 +1114,30 @@ namespace DoFTools
                               sub_neighbor->get_dof_indices (dofs_on_other_cell);
                               for (unsigned int i=0; i<cell->get_fe().dofs_per_cell; ++i)
                                 {
+                                  const unsigned int ii
+                                    = (cell->get_fe().is_primitive(i) ?
+                                       cell->get_fe().system_to_component_index(i).first
+                                       :
+                                       cell->get_fe().get_nonzero_components(i).first_selected_component()
+                                      );
+
+                                  Assert (ii < cell->get_fe().n_components(), ExcInternalError());
+
                                   for (unsigned int j=0; j<sub_neighbor->get_fe().dofs_per_cell;
                                        ++j)
                                     {
-                                      if ((flux_mask(cell->get_fe().system_to_component_index(i).first,
-                                                     sub_neighbor->get_fe().system_to_component_index(j).first)
-                                           == always)
+                                      const unsigned int jj
+                                        = (sub_neighbor->get_fe().is_primitive(j) ?
+                                           sub_neighbor->get_fe().system_to_component_index(j).first
+                                           :
+                                           sub_neighbor->get_fe().get_nonzero_components(j).first_selected_component()
+                                          );
+
+                                      Assert (jj < sub_neighbor->get_fe().n_components(), ExcInternalError());
+
+                                      if ((flux_mask(ii,jj) == always)
                                           ||
-                                          (flux_mask(cell->get_fe().system_to_component_index(i).first,
-                                                     sub_neighbor->get_fe().system_to_component_index(j).first)
-                                           == nonzero))
+                                          (flux_mask(ii,jj) == nonzero))
                                         {
                                           sparsity.add (dofs_on_this_cell[i],
                                                         dofs_on_other_cell[j]);
@@ -1117,13 +1149,9 @@ namespace DoFTools
                                                         dofs_on_other_cell[j]);
                                         }
 
-                                      if ((flux_mask(sub_neighbor->get_fe().system_to_component_index(j).first,
-                                                     cell->get_fe().system_to_component_index(i).first)
-                                           == always)
+                                      if ((flux_mask(jj,ii) == always)
                                           ||
-                                          (flux_mask(sub_neighbor->get_fe().system_to_component_index(j).first,
-                                                     cell->get_fe().system_to_component_index(i).first)
-                                           == nonzero))
+                                          (flux_mask(jj,ii) == nonzero))
                                         {
                                           sparsity.add (dofs_on_this_cell[j],
                                                         dofs_on_other_cell[i]);
@@ -1144,15 +1172,29 @@ namespace DoFTools
                           neighbor->get_dof_indices (dofs_on_other_cell);
                           for (unsigned int i=0; i<cell->get_fe().dofs_per_cell; ++i)
                             {
+                              const unsigned int ii
+                                = (cell->get_fe().is_primitive(i) ?
+                                   cell->get_fe().system_to_component_index(i).first
+                                   :
+                                   cell->get_fe().get_nonzero_components(i).first_selected_component()
+                                  );
+
+                              Assert (ii < cell->get_fe().n_components(), ExcInternalError());
+
                               for (unsigned int j=0; j<neighbor->get_fe().dofs_per_cell; ++j)
                                 {
-                                  if ((flux_mask(cell->get_fe().system_to_component_index(i).first,
-                                                 neighbor->get_fe().system_to_component_index(j).first)
-                                       == always)
+                                  const unsigned int jj
+                                    = (neighbor->get_fe().is_primitive(j) ?
+                                       neighbor->get_fe().system_to_component_index(j).first
+                                       :
+                                       neighbor->get_fe().get_nonzero_components(j).first_selected_component()
+                                      );
+
+                                  Assert (jj < neighbor->get_fe().n_components(), ExcInternalError());
+
+                                  if ((flux_mask(ii,jj) == always)
                                       ||
-                                      (flux_mask(cell->get_fe().system_to_component_index(i).first,
-                                                 neighbor->get_fe().system_to_component_index(j).first)
-                                       == nonzero))
+                                      (flux_mask(ii,jj) == nonzero))
                                     {
                                       sparsity.add (dofs_on_this_cell[i],
                                                     dofs_on_other_cell[j]);
@@ -1164,13 +1206,9 @@ namespace DoFTools
                                                     dofs_on_other_cell[j]);
                                     }
 
-                                  if ((flux_mask(neighbor->get_fe().system_to_component_index(j).first,
-                                                 cell->get_fe().system_to_component_index(i).first)
-                                       == always)
+                                  if ((flux_mask(jj,ii) == always)
                                       ||
-                                      (flux_mask(neighbor->get_fe().system_to_component_index(j).first,
-                                                 cell->get_fe().system_to_component_index(i).first)
-                                       == nonzero))
+                                      (flux_mask(jj,ii) == nonzero))
                                     {
                                       sparsity.add (dofs_on_this_cell[j],
                                                     dofs_on_other_cell[i]);
@@ -1225,7 +1263,7 @@ namespace DoFTools
     // specialized functions in the internal namespace
     const types::global_dof_index n_dofs = dof.n_dofs();
     (void)n_dofs;
-    const unsigned int n_comp = dof.get_fe().n_components();
+    const unsigned int n_comp = dof.get_fe(0).n_components();
     (void)n_comp;
 
     Assert (sparsity.n_rows() == n_dofs,
