@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 1998 - 2017 by the deal.II authors
+// Copyright (C) 1998 - 2018 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -27,6 +27,7 @@
 #include <algorithm>
 #include <functional>
 #include <numeric>
+#include <typeinfo>
 
 DEAL_II_NAMESPACE_OPEN
 
@@ -41,12 +42,6 @@ FiniteElement<dim, spacedim>::InternalDataBase::InternalDataBase ():
 
 
 
-template <int dim, int spacedim>
-FiniteElement<dim,spacedim>::InternalDataBase::~InternalDataBase ()
-{}
-
-
-
 template <int dim, int spacedim>
 std::size_t
 FiniteElement<dim, spacedim>::InternalDataBase::memory_consumption () const
@@ -64,7 +59,7 @@ FiniteElement (const FiniteElementData<dim> &fe_data,
   :
   FiniteElementData<dim> (fe_data),
   adjust_quad_dof_index_for_face_orientation_table (dim == 3 ?
-                                                    this->dofs_per_quad : 0 ,
+                                                    this->dofs_per_quad : 0,
                                                     dim==3 ? 8 : 0),
   adjust_line_dof_index_for_line_orientation_table (dim == 3 ?
                                                     this->dofs_per_line : 0),
@@ -88,9 +83,9 @@ FiniteElement (const FiniteElementData<dim> &fe_data,
   n_nonzero_components_table (compute_n_nonzero_components(nonzero_components)),
   cached_primitivity (std::find_if (n_nonzero_components_table.begin(),
                                     n_nonzero_components_table.end(),
-                                    std_cxx11::bind (std::not_equal_to<unsigned int>(),
-                                                     std_cxx11::_1,
-                                                     1U))
+                                    std::bind (std::not_equal_to<unsigned int>(),
+                                               std::placeholders::_1,
+                                               1U))
                       == n_nonzero_components_table.end())
 {
   Assert (restriction_is_additive_flags.size() == this->dofs_per_cell,
@@ -152,9 +147,11 @@ FiniteElement (const FiniteElementData<dim> &fe_data,
 
 
 template <int dim, int spacedim>
-FiniteElement<dim,spacedim>::~FiniteElement ()
-{}
-
+std::pair<std::unique_ptr<FiniteElement<dim, spacedim> >, unsigned int>
+FiniteElement<dim, spacedim>::operator^ (const unsigned int multiplicity) const
+{
+  return {this->clone(), multiplicity};
+}
 
 
 
@@ -853,10 +850,9 @@ get_interpolation_matrix (const FiniteElement<dim,spacedim> &,
   // by default, no interpolation
   // implemented. so throw exception,
   // as documentation says
-  typedef FiniteElement<dim,spacedim> FEE;
   AssertThrow (false,
-               typename FEE::
-               ExcInterpolationNotImplemented());
+               (typename FiniteElement<dim,spacedim>::
+                ExcInterpolationNotImplemented()));
 }
 
 
@@ -870,10 +866,9 @@ get_face_interpolation_matrix (const FiniteElement<dim,spacedim> &,
   // by default, no interpolation
   // implemented. so throw exception,
   // as documentation says
-  typedef    FiniteElement<dim,spacedim> FEE;
   AssertThrow (false,
-               typename FEE::
-               ExcInterpolationNotImplemented());
+               (typename FiniteElement<dim,spacedim>::
+                ExcInterpolationNotImplemented()));
 }
 
 
@@ -888,9 +883,9 @@ get_subface_interpolation_matrix (const FiniteElement<dim,spacedim> &,
   // by default, no interpolation
   // implemented. so throw exception,
   // as documentation says
-  typedef    FiniteElement<dim,spacedim> FEE;
   AssertThrow (false,
-               typename FEE::ExcInterpolationNotImplemented());
+               (typename FiniteElement<dim,spacedim>::
+                ExcInterpolationNotImplemented()));
 }
 
 
@@ -943,13 +938,29 @@ template <int dim, int spacedim>
 bool
 FiniteElement<dim,spacedim>::operator == (const FiniteElement<dim,spacedim> &f) const
 {
-  return ((static_cast<const FiniteElementData<dim>&>(*this) ==
-           static_cast<const FiniteElementData<dim>&>(f)) &&
+  // Compare fields in roughly increasing order of how expensive the
+  // comparison is
+  return ((typeid(*this) == typeid(f))
+          &&
+          (this->get_name() == f.get_name())
+          &&
+          (static_cast<const FiniteElementData<dim>&>(*this) ==
+           static_cast<const FiniteElementData<dim>&>(f))
+          &&
           (interface_constraints == f.interface_constraints));
 }
 
 
 
+template <int dim, int spacedim>
+bool
+FiniteElement<dim,spacedim>::operator != (const FiniteElement<dim,spacedim> &f) const
+{
+  return !(*this == f);
+}
+
+
+
 template <int dim, int spacedim>
 const std::vector<Point<dim> > &
 FiniteElement<dim,spacedim>::get_unit_support_points () const
@@ -979,10 +990,8 @@ template <int dim, int spacedim>
 const std::vector<Point<dim> > &
 FiniteElement<dim,spacedim>::get_generalized_support_points () const
 {
-  // a finite element may define
-  // support points, but only if
-  // there are as many as there are
-  // degrees of freedom
+  // If the finite element implements generalized support points, return
+  // those. Otherwise fall back to unit support points.
   return ((generalized_support_points.size() == 0)
           ? unit_support_points
           : generalized_support_points);
@@ -1085,6 +1094,57 @@ FiniteElement<dim,spacedim>::has_support_on_face (
 
 
 
+template <int dim, int spacedim>
+const FiniteElement<dim,spacedim> &
+FiniteElement<dim,spacedim>::
+get_sub_fe (const ComponentMask &mask) const
+{
+  // Translate the ComponentMask into first_selected and n_components after
+  // some error checking:
+  const unsigned int n_total_components = this->n_components();
+  Assert((n_total_components == mask.size()) || (mask.size()==0),
+         ExcMessage("The given ComponentMask has the wrong size."));
+
+  const unsigned int n_selected = mask.n_selected_components(n_total_components);
+  Assert(n_selected>0,
+         ExcMessage("You need at least one selected component."));
+
+  const unsigned int first_selected = mask.first_selected_component(n_total_components);
+
+#ifdef DEBUG
+  // check that it is contiguous:
+  for (unsigned int c=0; c<n_total_components; ++c)
+    Assert((c<first_selected && (!mask[c]))
+           ||
+           (c>=first_selected && c<first_selected+n_selected && mask[c])
+           ||
+           (c>=first_selected+n_selected && !mask[c]),
+           ExcMessage("Error: the given ComponentMask is not contiguous!"));
+#endif
+
+  return get_sub_fe(first_selected, n_selected);
+}
+
+
+
+template <int dim, int spacedim>
+const FiniteElement<dim,spacedim> &
+FiniteElement<dim,spacedim>::
+get_sub_fe (const unsigned int first_component,
+            const unsigned int n_selected_components) const
+{
+  // No complicated logic is needed here, because it is overridden in
+  // FESystem<dim,spacedim>. Just make sure that what the user chose is valid:
+  Assert(first_component == 0 && n_selected_components == this->n_components(),
+         ExcMessage("You can only select a whole FiniteElement, not a part of one."));
+
+  (void)first_component;
+  (void)n_selected_components;
+  return *this;
+}
+
+
+
 template <int dim, int spacedim>
 std::pair<Table<2,bool>, std::vector<unsigned int> >
 FiniteElement<dim,spacedim>::get_constant_modes () const
@@ -1100,8 +1160,8 @@ FiniteElement<dim,spacedim>::get_constant_modes () const
 template <int dim, int spacedim>
 void
 FiniteElement<dim,spacedim>::
-convert_generalized_support_point_values_to_nodal_values (const std::vector<Vector<double> > &,
-                                                          std::vector<double> &) const
+convert_generalized_support_point_values_to_dof_values (const std::vector<Vector<double> > &,
+                                                        std::vector<double> &) const
 {
   Assert (has_generalized_support_points(),
           ExcMessage ("The element for which you are calling the current "
@@ -1113,75 +1173,6 @@ convert_generalized_support_point_values_to_nodal_values (const std::vector<Vect
 }
 
 
-template <int dim, int spacedim>
-void
-FiniteElement<dim,spacedim>::interpolate(
-  std::vector<double>       &local_dofs,
-  const std::vector<double> &values) const
-{
-  Assert (has_support_points(), ExcFEHasNoSupportPoints());
-  Assert (values.size() == unit_support_points.size(),
-          ExcDimensionMismatch(values.size(), unit_support_points.size()));
-  Assert (local_dofs.size() == this->dofs_per_cell,
-          ExcDimensionMismatch(local_dofs.size(),this->dofs_per_cell));
-  Assert (this->n_components() == 1,
-          ExcDimensionMismatch(this->n_components(), 1));
-
-  std::copy(values.begin(), values.end(), local_dofs.begin());
-}
-
-
-
-
-template <int dim, int spacedim>
-void
-FiniteElement<dim,spacedim>::interpolate(
-  std::vector<double>                &local_dofs,
-  const std::vector<Vector<double> > &values,
-  const unsigned int                  offset) const
-{
-  Assert (has_support_points(), ExcFEHasNoSupportPoints());
-  Assert (values.size() == unit_support_points.size(),
-          ExcDimensionMismatch(values.size(), unit_support_points.size()));
-  Assert (local_dofs.size() == this->dofs_per_cell,
-          ExcDimensionMismatch(local_dofs.size(),this->dofs_per_cell));
-  Assert (values[0].size() >= offset+this->n_components(),
-          ExcDimensionMismatch(values[0].size(),offset+this->n_components()));
-
-  for (unsigned int i=0; i<this->dofs_per_cell; ++i)
-    {
-      const std::pair<unsigned int, unsigned int> index
-        = this->system_to_component_index(i);
-      local_dofs[i] = values[i](offset+index.first);
-    }
-}
-
-
-
-
-template <int dim, int spacedim>
-void
-FiniteElement<dim,spacedim>::interpolate(
-  std::vector<double> &local_dofs,
-  const VectorSlice<const std::vector<std::vector<double> > > &values) const
-{
-  Assert (has_support_points(), ExcFEHasNoSupportPoints());
-  Assert (values[0].size() == unit_support_points.size(),
-          ExcDimensionMismatch(values.size(), unit_support_points.size()));
-  Assert (local_dofs.size() == this->dofs_per_cell,
-          ExcDimensionMismatch(local_dofs.size(),this->dofs_per_cell));
-  Assert (values.size() == this->n_components(),
-          ExcDimensionMismatch(values.size(), this->n_components()));
-
-  for (unsigned int i=0; i<this->dofs_per_cell; ++i)
-    {
-      const std::pair<unsigned int, unsigned int> index
-        = this->system_to_component_index(i);
-      local_dofs[i] = values[index.first][i];
-    }
-}
-
-
 
 template <int dim, int spacedim>
 std::size_t
@@ -1219,11 +1210,11 @@ FiniteElement<dim,spacedim>::compute_n_nonzero_components (
 /*------------------------------- FiniteElement ----------------------*/
 
 template <int dim, int spacedim>
-typename FiniteElement<dim,spacedim>::InternalDataBase *
+std::unique_ptr<typename FiniteElement<dim,spacedim>::InternalDataBase>
 FiniteElement<dim,spacedim>::get_face_data (const UpdateFlags       flags,
                                             const Mapping<dim,spacedim>      &mapping,
                                             const Quadrature<dim-1> &quadrature,
-                                            dealii::internal::FEValues::FiniteElementRelatedData<dim, spacedim> &output_data) const
+                                            dealii::internal::FEValuesImplementation::FiniteElementRelatedData<dim, spacedim> &output_data) const
 {
   return get_data (flags, mapping,
                    QProjector<dim>::project_to_all_faces(quadrature),
@@ -1233,11 +1224,11 @@ FiniteElement<dim,spacedim>::get_face_data (const UpdateFlags       flags,
 
 
 template <int dim, int spacedim>
-typename FiniteElement<dim,spacedim>::InternalDataBase *
+std::unique_ptr<typename FiniteElement<dim,spacedim>::InternalDataBase>
 FiniteElement<dim,spacedim>::get_subface_data (const UpdateFlags        flags,
                                                const Mapping<dim,spacedim>      &mapping,
                                                const Quadrature<dim-1> &quadrature,
-                                               dealii::internal::FEValues::FiniteElementRelatedData<dim, spacedim> &output_data) const
+                                               dealii::internal::FEValuesImplementation::FiniteElementRelatedData<dim, spacedim> &output_data) const
 {
   return get_data (flags, mapping,
                    QProjector<dim>::project_to_all_subfaces(quadrature),
