@@ -30,6 +30,8 @@
 
 #include <sstream>
 #include <iostream>
+#include <deal.II/base/std_cxx14/memory.h>
+
 
 //TODO: implement the adjust_quad_dof_index_for_face_orientation_table and
 //adjust_line_dof_index_for_line_orientation_table fields, and write tests
@@ -90,7 +92,7 @@ FE_ABF<dim>::FE_ABF (const unsigned int deg)
                                                  this->dofs_per_face));
   // TODO: Something goes wrong there. The error of the least squares fit
   // is to large ...
-  // FETools::compute_face_embedding_matrices(*this, &face_embeddings[0], 0, 0);
+  // FETools::compute_face_embedding_matrices(*this, face_embeddings.data(), 0, 0);
   this->interface_constraints.reinit((1<<(dim-1)) * this->dofs_per_face,
                                      this->dofs_per_face);
   unsigned int target_row=0;
@@ -126,10 +128,10 @@ FE_ABF<dim>::get_name () const
 
 
 template <int dim>
-FiniteElement<dim> *
+std::unique_ptr<FiniteElement<dim,dim> >
 FE_ABF<dim>::clone() const
 {
-  return new FE_ABF<dim>(rt_order);
+  return std_cxx14::make_unique<FE_ABF<dim>>(rt_order);
 }
 
 
@@ -158,7 +160,7 @@ FE_ABF<dim>::initialize_support_points (const unsigned int deg)
 
 
   // These might be required when the faces contribution is computed
-  // Therefore they will be initialised at this point.
+  // Therefore they will be initialized at this point.
   std::vector<AnisotropicPolynomials<dim>* > polynomials_abf(dim);
 
   // Generate x_1^{i} x_2^{r+1} ...
@@ -210,7 +212,7 @@ FE_ABF<dim>::initialize_support_points (const unsigned int deg)
         }
 
 
-      // Now initialise edge interior weights for the ABF elements.
+      // Now initialize edge interior weights for the ABF elements.
       // These are completely independent from the usual edge moments. They
       // stem from applying the Gauss theorem to the nodal values, which
       // was necessary to cast the ABF elements into the deal.II framework
@@ -511,8 +513,8 @@ FE_ABF<dim>::has_support_on_face (const unsigned int shape_index,
 template <int dim>
 void
 FE_ABF<dim>::
-convert_generalized_support_point_values_to_nodal_values (const std::vector<Vector<double> > &support_point_values,
-                                                          std::vector<double>                &nodal_values) const
+convert_generalized_support_point_values_to_dof_values (const std::vector<Vector<double> > &support_point_values,
+                                                        std::vector<double>                &nodal_values) const
 {
   Assert (support_point_values.size() == this->generalized_support_points.size(),
           ExcDimensionMismatch(support_point_values.size(), this->generalized_support_points.size()));
@@ -570,142 +572,6 @@ convert_generalized_support_point_values_to_nodal_values (const std::vector<Vect
 
 
 
-template <int dim>
-void
-FE_ABF<dim>::interpolate(
-  std::vector<double> &,
-  const std::vector<double> &) const
-{
-  Assert(false, ExcNotImplemented());
-}
-
-
-
-template <int dim>
-void
-FE_ABF<dim>::interpolate(
-  std::vector<double>    &local_dofs,
-  const std::vector<Vector<double> > &values,
-  const unsigned int offset) const
-{
-  Assert (values.size() == this->generalized_support_points.size(),
-          ExcDimensionMismatch(values.size(), this->generalized_support_points.size()));
-  Assert (local_dofs.size() == this->dofs_per_cell,
-          ExcDimensionMismatch(local_dofs.size(),this->dofs_per_cell));
-  Assert (values[0].size() >= offset+this->n_components(),
-          ExcDimensionMismatch(values[0].size(),offset+this->n_components()));
-
-  std::fill(local_dofs.begin(), local_dofs.end(), 0.);
-
-  const unsigned int n_face_points = boundary_weights.size(0);
-  for (unsigned int face=0; face<GeometryInfo<dim>::faces_per_cell; ++face)
-    for (unsigned int k=0; k<n_face_points; ++k)
-      for (unsigned int i=0; i<boundary_weights.size(1); ++i)
-        {
-          local_dofs[i+face*this->dofs_per_face] += boundary_weights(k,i)
-                                                    * values[face*n_face_points+k](GeometryInfo<dim>::unit_normal_direction[face]+offset);
-        }
-
-  const unsigned int start_cell_dofs = GeometryInfo<dim>::faces_per_cell*this->dofs_per_face;
-  const unsigned int start_cell_points = GeometryInfo<dim>::faces_per_cell*n_face_points;
-
-  for (unsigned int k=0; k<interior_weights.size(0); ++k)
-    for (unsigned int i=0; i<interior_weights.size(1); ++i)
-      for (unsigned int d=0; d<dim; ++d)
-        local_dofs[start_cell_dofs+i*dim+d] += interior_weights(k,i,d) * values[k+start_cell_points](d+offset);
-
-  const unsigned int start_abf_dofs = start_cell_dofs + interior_weights.size(1) * dim;
-
-  // Cell integral of ABF terms
-  for (unsigned int k=0; k<interior_weights_abf.size(0); ++k)
-    for (unsigned int i=0; i<interior_weights_abf.size(1); ++i)
-      for (unsigned int d=0; d<dim; ++d)
-        local_dofs[start_abf_dofs+i] += interior_weights_abf(k,i,d) * values[k+start_cell_points](d+offset);
-
-  // Face integral of ABF terms
-  for (unsigned int face=0; face<GeometryInfo<dim>::faces_per_cell; ++face)
-    {
-      double n_orient = (double) GeometryInfo<dim>::unit_normal_orientation[face];
-      for (unsigned int fp=0; fp < n_face_points; ++fp)
-        {
-          // TODO: Check what the face_orientation, face_flip and face_rotation  have to be in 3D
-          unsigned int k = QProjector<dim>::DataSetDescriptor::face (face, false, false, false, n_face_points);
-          for (unsigned int i=0; i<boundary_weights_abf.size(1); ++i)
-            local_dofs[start_abf_dofs+i] += n_orient * boundary_weights_abf(k + fp, i)
-                                            * values[face*n_face_points+fp](GeometryInfo<dim>::unit_normal_direction[face]+offset);
-        }
-    }
-
-  // TODO: Check if this "correction" can be removed.
-  for (unsigned int i=0; i<boundary_weights_abf.size(1); ++i)
-    if (std::fabs (local_dofs[start_abf_dofs+i]) < 1.0e-16)
-      local_dofs[start_abf_dofs+i] = 0.0;
-}
-
-
-
-template <int dim>
-void
-FE_ABF<dim>::interpolate(
-  std::vector<double> &local_dofs,
-  const VectorSlice<const std::vector<std::vector<double> > > &values) const
-{
-  Assert (values.size() == this->n_components(),
-          ExcDimensionMismatch(values.size(), this->n_components()));
-  Assert (values[0].size() == this->generalized_support_points.size(),
-          ExcDimensionMismatch(values[0].size(), this->generalized_support_points.size()));
-  Assert (local_dofs.size() == this->dofs_per_cell,
-          ExcDimensionMismatch(local_dofs.size(),this->dofs_per_cell));
-
-  std::fill(local_dofs.begin(), local_dofs.end(), 0.);
-
-  const unsigned int n_face_points = boundary_weights.size(0);
-  for (unsigned int face=0; face<GeometryInfo<dim>::faces_per_cell; ++face)
-    for (unsigned int k=0; k<n_face_points; ++k)
-      for (unsigned int i=0; i<boundary_weights.size(1); ++i)
-        {
-          local_dofs[i+face*this->dofs_per_face] += boundary_weights(k,i)
-                                                    * values[GeometryInfo<dim>::unit_normal_direction[face]][face*n_face_points+k];
-        }
-
-  const unsigned int start_cell_dofs = GeometryInfo<dim>::faces_per_cell*this->dofs_per_face;
-  const unsigned int start_cell_points = GeometryInfo<dim>::faces_per_cell*n_face_points;
-
-  for (unsigned int k=0; k<interior_weights.size(0); ++k)
-    for (unsigned int i=0; i<interior_weights.size(1); ++i)
-      for (unsigned int d=0; d<dim; ++d)
-        local_dofs[start_cell_dofs+i*dim+d] += interior_weights(k,i,d) * values[d][k+start_cell_points];
-
-  const unsigned int start_abf_dofs = start_cell_dofs + interior_weights.size(1) * dim;
-
-  // Cell integral of ABF terms
-  for (unsigned int k=0; k<interior_weights_abf.size(0); ++k)
-    for (unsigned int i=0; i<interior_weights_abf.size(1); ++i)
-      for (unsigned int d=0; d<dim; ++d)
-        local_dofs[start_abf_dofs+i] += interior_weights_abf(k,i,d) * values[d][k+start_cell_points];
-
-  // Face integral of ABF terms
-  for (unsigned int face=0; face<GeometryInfo<dim>::faces_per_cell; ++face)
-    {
-      double n_orient = (double) GeometryInfo<dim>::unit_normal_orientation[face];
-      for (unsigned int fp=0; fp < n_face_points; ++fp)
-        {
-          // TODO: Check what the face_orientation, face_flip and face_rotation have to be in 3D
-          unsigned int k = QProjector<dim>::DataSetDescriptor::face (face, false, false, false, n_face_points);
-          for (unsigned int i=0; i<boundary_weights_abf.size(1); ++i)
-            local_dofs[start_abf_dofs+i] += n_orient * boundary_weights_abf(k + fp, i)
-                                            * values[GeometryInfo<dim>::unit_normal_direction[face]][face*n_face_points+fp];
-        }
-    }
-
-  // TODO: Check if this "correction" can be removed.
-  for (unsigned int i=0; i<boundary_weights_abf.size(1); ++i)
-    if (std::fabs (local_dofs[start_abf_dofs+i]) < 1.0e-16)
-      local_dofs[start_abf_dofs+i] = 0.0;
-}
-
-
-
 template <int dim>
 std::size_t
 FE_ABF<dim>::memory_consumption () const
