@@ -1,7 +1,6 @@
 // ---------------------------------------------------------------------
-// $Id: fe_q.cc 30037 2013-07-18 16:55:40Z maier $
 //
-// Copyright (C) 2000 - 2016 by the deal.II authors
+// Copyright (C) 2000 - 2018 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -26,6 +25,8 @@
 
 #include <vector>
 #include <sstream>
+#include <deal.II/base/std_cxx14/memory.h>
+
 
 DEAL_II_NAMESPACE_OPEN
 
@@ -51,9 +52,9 @@ get_interpolation_matrix (const FiniteElement<dim,spacedim> &,
                           FullMatrix<double> &) const
 {
   // no interpolation possible. throw exception, as documentation says
-  typedef FiniteElement<dim,spacedim> FEE;
   AssertThrow (false,
-               typename FEE::ExcInterpolationNotImplemented());
+               (typename FiniteElement<dim,spacedim>::
+                ExcInterpolationNotImplemented()));
 }
 
 
@@ -63,9 +64,8 @@ const FullMatrix<double> &
 FE_Bernstein<dim,spacedim>::get_restriction_matrix (const unsigned int,
                                                     const RefinementCase<dim> &) const
 {
-  typedef FiniteElement<dim,spacedim> FEE;
   AssertThrow (false,
-               typename FEE::ExcProjectionVoid());
+               (typename FiniteElement<dim,spacedim>::ExcProjectionVoid()));
   // return dummy, nothing will happen because the base class FE_Q_Base
   // implements lazy evaluation of those matrices
   return this->restriction[0][0];
@@ -78,9 +78,8 @@ const FullMatrix<double> &
 FE_Bernstein<dim,spacedim>::get_prolongation_matrix (const unsigned int,
                                                      const RefinementCase<dim> &) const
 {
-  typedef FiniteElement<dim,spacedim> FEE;
   AssertThrow (false,
-               typename FEE::ExcEmbeddingVoid());
+               (typename FiniteElement<dim,spacedim>::ExcEmbeddingVoid()));
   // return dummy, nothing will happen because the base class FE_Q_Base
   // implements lazy evaluation of those matrices
   return this->prolongation[0][0];
@@ -181,7 +180,7 @@ get_subface_interpolation_matrix (const FiniteElement<dim,spacedim> &x_source_fe
           Assert (std::fabs(sum-1) < eps, ExcInternalError());
         }
     }
-  else if (dynamic_cast<const FE_Nothing<dim> *>(&x_source_fe) != 0)
+  else if (dynamic_cast<const FE_Nothing<dim> *>(&x_source_fe) != nullptr)
     {
       // nothing to do here, the FE_Nothing has no degrees of freedom anyway
     }
@@ -208,13 +207,13 @@ FE_Bernstein<dim,spacedim>::hp_vertex_dof_identities (const FiniteElement<dim,sp
   // or if the other one is an FE_Nothing. in the first case, there should be
   // exactly one single DoF of each FE at a vertex, and they should have
   // identical value
-  if (dynamic_cast<const FE_Bernstein<dim,spacedim>*>(&fe_other) != 0)
+  if (dynamic_cast<const FE_Bernstein<dim,spacedim>*>(&fe_other) != nullptr)
     {
       return
         std::vector<std::pair<unsigned int, unsigned int> >
         (1, std::make_pair (0U, 0U));
     }
-  else if (dynamic_cast<const FE_Nothing<dim>*>(&fe_other) != 0)
+  else if (dynamic_cast<const FE_Nothing<dim>*>(&fe_other) != nullptr)
     {
       // the FE_Nothing has no degrees of freedom, so there are no
       // equivalencies to be recorded
@@ -315,10 +314,10 @@ FE_Bernstein<dim,spacedim>::get_name () const
 
 
 template <int dim, int spacedim>
-FiniteElement<dim,spacedim> *
+std::unique_ptr<FiniteElement<dim,spacedim> >
 FE_Bernstein<dim,spacedim>::clone() const
 {
-  return new FE_Bernstein<dim,spacedim>(*this);
+  return std_cxx14::make_unique<FE_Bernstein<dim,spacedim>>(*this);
 }
 
 
