@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 2016 - 2017 by the deal.II authors
+// Copyright (C) 2016 - 2018 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -15,100 +15,106 @@
 
 
 #include <deal.II/fe/fe_enriched.h>
-
-#ifdef DEAL_II_WITH_CXX14
-
 #include <deal.II/fe/fe_tools.h>
 
+#include <deal.II/base/std_cxx14/memory.h>
+
+
 DEAL_II_NAMESPACE_OPEN
 
-namespace
+namespace internal
 {
-  /**
-   * Auxiliary function to create multiplicity vector from input enrichment functions.
-   */
-  template <typename T>
-  std::vector<unsigned int>
-  build_multiplicities(const std::vector<std::vector<T > > &functions )
+  namespace FE_Enriched
   {
-    std::vector<unsigned int> multiplicities;
-    multiplicities.push_back(1); // the first one is non-enriched FE
-    for (unsigned int i = 0; i < functions.size(); i++)
-      multiplicities.push_back(functions[i].size());
-
-    return multiplicities;
-  }
-
-
-  /**
-   * Auxiliary function to build FiniteElement's vector
-   */
-  template <int dim, int spacedim>
-  std::vector< const FiniteElement< dim, spacedim > * >
-  build_fes(const FiniteElement<dim,spacedim> *fe_base,
-            const std::vector<const FiniteElement<dim,spacedim> * > &fe_enriched)
-  {
-    std::vector< const FiniteElement< dim, spacedim > * > fes;
-    fes.push_back(fe_base);
-    for (unsigned int i = 0; i < fe_enriched.size(); i++)
-      fes.push_back(fe_enriched[i]);
-
-    return fes;
-  }
-
-
-  /**
-   * Auxiliary function which check consistency of the input parameters.
-   * Returns true if everything is ok.
-   */
-  template <int dim, int spacedim>
-  bool
-  consistency_check (const std::vector< const FiniteElement< dim, spacedim > * > &fes,
-                     const std::vector< unsigned int > &multiplicities,
-                     const std::vector<std::vector<std::function<const Function<spacedim> *(const typename Triangulation<dim, spacedim>::cell_iterator &) > > > &functions)
-  {
-    AssertThrow(fes.size() > 0,
-                ExcMessage("FEs size should be >=1"));
-    AssertThrow(fes.size() == multiplicities.size(),
-                ExcMessage("FEs and multiplicities should have the same size"));
-
-    AssertThrow (functions.size() == fes.size() - 1,
-                 ExcDimensionMismatch(functions.size(), fes.size()-1));
-
-    AssertThrow(multiplicities[0] == 1,
-                ExcMessage("First multiplicity should be 1"));
-
-    const unsigned int n_comp_base = fes[0]->n_components();
-
-    // start from fe=1 as 0th is always non-enriched FE.
-    for (unsigned int fe=1; fe < fes.size(); fe++)
+    namespace
+    {
+      /**
+       * Auxiliary function to create multiplicity vector from input enrichment functions.
+       */
+      template <typename T>
+      std::vector<unsigned int>
+      build_multiplicities(const std::vector<std::vector<T > > &functions )
       {
-        const FE_Nothing<dim> *fe_nothing = dynamic_cast<const FE_Nothing<dim>*>(fes[fe]);
-        if (fe_nothing)
-          AssertThrow (fe_nothing->is_dominating(),
-                       ExcMessage("Only dominating FE_Nothing can be used in FE_Enriched"));
+        std::vector<unsigned int> multiplicities;
+        multiplicities.push_back(1); // the first one is non-enriched FE
+        for (unsigned int i = 0; i < functions.size(); i++)
+          multiplicities.push_back(functions[i].size());
 
-        AssertThrow (fes[fe]->n_components() == n_comp_base,
-                     ExcMessage("All elements must have the same number of components"));
+        return multiplicities;
       }
-    return true;
-  }
 
 
-  /**
-   * Auxiliary function which determines whether the FiniteElement will be enriched.
-   */
-  template <int dim, int spacedim>
-  bool
-  check_if_enriched (const std::vector< const FiniteElement< dim, spacedim > * > &fes)
-  {
-    // start from fe=1 as 0th is always non-enriched FE.
-    for (unsigned int fe=1; fe < fes.size(); fe++)
-      if (dynamic_cast<const FE_Nothing<dim>*>(fes[fe]) == 0)
-        // this is not FE_Nothing => there will be enrichment
+      /**
+       * Auxiliary function to build FiniteElement's vector
+       */
+      template <int dim, int spacedim>
+      std::vector< const FiniteElement< dim, spacedim > * >
+      build_fes(const FiniteElement<dim,spacedim> *fe_base,
+                const std::vector<const FiniteElement<dim,spacedim> * > &fe_enriched)
+      {
+        std::vector< const FiniteElement< dim, spacedim > * > fes;
+        fes.push_back(fe_base);
+        for (unsigned int i = 0; i < fe_enriched.size(); i++)
+          fes.push_back(fe_enriched[i]);
+
+        return fes;
+      }
+
+
+      /**
+       * Auxiliary function which check consistency of the input parameters.
+       * Returns true if everything is ok.
+       */
+      template <int dim, int spacedim>
+      bool
+      consistency_check (const std::vector< const FiniteElement< dim, spacedim > * > &fes,
+                         const std::vector< unsigned int > &multiplicities,
+                         const std::vector<std::vector<std::function<const Function<spacedim> *(const typename dealii::Triangulation<dim, spacedim>::cell_iterator &) > > > &functions)
+      {
+        AssertThrow(fes.size() > 0,
+                    ExcMessage("FEs size should be >=1"));
+        AssertThrow(fes.size() == multiplicities.size(),
+                    ExcMessage("FEs and multiplicities should have the same size"));
+
+        AssertThrow (functions.size() == fes.size() - 1,
+                     ExcDimensionMismatch(functions.size(), fes.size()-1));
+
+        AssertThrow(multiplicities[0] == 1,
+                    ExcMessage("First multiplicity should be 1"));
+
+        const unsigned int n_comp_base = fes[0]->n_components();
+
+        // start from fe=1 as 0th is always non-enriched FE.
+        for (unsigned int fe=1; fe < fes.size(); fe++)
+          {
+            const FE_Nothing<dim> *fe_nothing = dynamic_cast<const FE_Nothing<dim>*>(fes[fe]);
+            if (fe_nothing)
+              AssertThrow (fe_nothing->is_dominating(),
+                           ExcMessage("Only dominating FE_Nothing can be used in FE_Enriched"));
+
+            AssertThrow (fes[fe]->n_components() == n_comp_base,
+                         ExcMessage("All elements must have the same number of components"));
+          }
         return true;
+      }
 
-    return false;
+
+      /**
+       * Auxiliary function which determines whether the FiniteElement will be enriched.
+       */
+      template <int dim, int spacedim>
+      bool
+      check_if_enriched (const std::vector< const FiniteElement< dim, spacedim > * > &fes)
+      {
+        // start from fe=1 as 0th is always non-enriched FE.
+        for (unsigned int fe=1; fe < fes.size(); fe++)
+          if (dynamic_cast<const FE_Nothing<dim>*>(fes[fe]) == nullptr)
+            // this is not FE_Nothing => there will be enrichment
+            return true;
+
+        return false;
+      }
+    }
   }
 }
 
@@ -118,7 +124,7 @@ FE_Enriched<dim,spacedim>::FE_Enriched (const FiniteElement<dim,spacedim> &fe_ba
   :
   FE_Enriched<dim,spacedim>(fe_base,
                             FE_Nothing<dim,spacedim>(fe_base.n_components(),true),
-                            NULL)
+                            nullptr)
 {
 }
 
@@ -149,8 +155,8 @@ FE_Enriched<dim,spacedim>::FE_Enriched (const FiniteElement<dim,spacedim> *fe_ba
                                         const std::vector<const FiniteElement<dim,spacedim> * > &fe_enriched,
                                         const std::vector<std::vector<std::function<const Function<spacedim> *(const typename Triangulation<dim, spacedim>::cell_iterator &) > > > &functions)
   :
-  FE_Enriched<dim,spacedim> (build_fes(fe_base,fe_enriched),
-                             build_multiplicities(functions),
+  FE_Enriched<dim,spacedim> (internal::FE_Enriched::build_fes(fe_base,fe_enriched),
+                             internal::FE_Enriched::build_multiplicities(functions),
                              functions)
 {}
 
@@ -164,11 +170,11 @@ FE_Enriched<dim,spacedim>::FE_Enriched (const std::vector< const FiniteElement<
                                FETools::Compositing::compute_restriction_is_additive_flags(fes,multiplicities),
                                FETools::Compositing::compute_nonzero_components(fes,multiplicities,false)),
   enrichments(functions),
-  is_enriched(check_if_enriched(fes)),
-  fe_system(fes,multiplicities)
+  is_enriched(internal::FE_Enriched::check_if_enriched(fes)),
+  fe_system(std_cxx14::make_unique<FESystem<dim, spacedim> >(fes,multiplicities))
 {
   // descriptive error are thrown within the function.
-  Assert(consistency_check(fes,multiplicities,functions),
+  Assert(internal::FE_Enriched::consistency_check(fes,multiplicities,functions),
          ExcInternalError());
 
   initialize(fes, multiplicities);
@@ -233,12 +239,12 @@ FE_Enriched<dim,spacedim>::shape_value(const unsigned int   i,
                                        const Point< dim > &p) const
 {
   Assert(!is_enriched, ExcMessage("For enriched finite elements shape_value() can not be defined on the reference element."));
-  return fe_system.shape_value(i,p);
+  return fe_system->shape_value(i,p);
 }
 
 
 template <int dim, int spacedim>
-FiniteElement<dim,spacedim> *
+std::unique_ptr<FiniteElement<dim,spacedim> >
 FE_Enriched<dim,spacedim>::clone() const
 {
   std::vector< const FiniteElement< dim, spacedim > * > fes;
@@ -250,7 +256,9 @@ FE_Enriched<dim,spacedim>::clone() const
       multiplicities.push_back(this->element_multiplicity(i) );
     }
 
-  return new FE_Enriched<dim,spacedim>(fes, multiplicities, get_enrichments());
+  return std::unique_ptr<FE_Enriched<dim,spacedim>>(new FE_Enriched<dim,spacedim>(fes,
+                                                    multiplicities,
+                                                    get_enrichments()));
 }
 
 
@@ -258,7 +266,7 @@ template <int dim, int spacedim>
 UpdateFlags
 FE_Enriched<dim,spacedim>::requires_update_flags (const UpdateFlags flags) const
 {
-  UpdateFlags out = fe_system.requires_update_flags(flags);
+  UpdateFlags out = fe_system->requires_update_flags(flags);
 
   if (is_enriched)
     {
@@ -280,23 +288,18 @@ FE_Enriched<dim,spacedim>::requires_update_flags (const UpdateFlags flags) const
 
 template <int dim, int spacedim>
 template <int dim_1>
-typename FiniteElement<dim,spacedim>::InternalDataBase *
-FE_Enriched<dim,spacedim>::setup_data (std::unique_ptr<typename FiniteElement<dim,spacedim>::InternalDataBase> fes_data,
+std::unique_ptr<typename FiniteElement<dim,spacedim>::InternalDataBase>
+FE_Enriched<dim,spacedim>::setup_data (std::unique_ptr<typename FESystem<dim,spacedim>::InternalData> fes_data,
                                        const UpdateFlags      flags,
                                        const Quadrature<dim_1> &quadrature) const
 {
-  Assert ((dynamic_cast<typename FESystem<dim,spacedim>::InternalData *> (fes_data.get()) != NULL),
-          ExcInternalError());
-  typename FESystem<dim,spacedim>::InternalData *data_fesystem =
-    static_cast<typename FESystem<dim,spacedim>::InternalData *> (fes_data.get());
-
-  // FESystem::InternalData will be aggregated (owned) by
-  // our InternalData.
-  fes_data.release();
-  InternalData *data = new InternalData(std::unique_ptr<typename FESystem<dim,spacedim>::InternalData>(data_fesystem));
+  // Pass ownership of the FiniteElement::InternalDataBase object
+  // that fes_data points to, to the new InternalData object.
+  auto update_each_flags = fes_data->update_each;
+  auto data = std_cxx14::make_unique<InternalData>(std::move(fes_data));
 
   // copy update_each from FESystem data:
-  data->update_each = data_fesystem->update_each;
+  data->update_each = update_each_flags;
 
   // resize cache array according to requested flags
   data->enrichment.resize(this->n_base_elements());
@@ -319,44 +322,47 @@ FE_Enriched<dim,spacedim>::setup_data (std::unique_ptr<typename FiniteElement<di
         }
     }
 
-  return data;
+  return std::move(data);
 }
 
 
 template <int dim, int spacedim>
-typename FiniteElement<dim,spacedim>::InternalDataBase *
+std::unique_ptr<typename FiniteElement<dim,spacedim>::InternalDataBase>
 FE_Enriched<dim,spacedim>::get_face_data (const UpdateFlags      update_flags,
                                           const Mapping<dim,spacedim>    &mapping,
                                           const Quadrature<dim-1> &quadrature,
-                                          internal::FEValues::FiniteElementRelatedData< dim, spacedim >        &output_data) const
+                                          internal::FEValuesImplementation::FiniteElementRelatedData< dim, spacedim >        &output_data) const
 {
-  return setup_data(std::unique_ptr<typename FiniteElement<dim,spacedim>::InternalDataBase>(fe_system.get_face_data(update_flags,mapping,quadrature,output_data)),
+  auto data = fe_system->get_face_data(update_flags,mapping,quadrature,output_data);
+  return setup_data(Utilities::dynamic_unique_cast<typename FESystem<dim,spacedim>::InternalData>(std::move(data)),
                     update_flags,
                     quadrature);
 }
 
 
 template <int dim, int spacedim>
-typename FiniteElement<dim,spacedim>::InternalDataBase *
+std::unique_ptr<typename FiniteElement<dim,spacedim>::InternalDataBase>
 FE_Enriched<dim,spacedim>::get_subface_data (const UpdateFlags      update_flags,
                                              const Mapping<dim,spacedim>    &mapping,
                                              const Quadrature<dim-1> &quadrature,
-                                             dealii::internal::FEValues::FiniteElementRelatedData<dim, spacedim> &output_data) const
+                                             dealii::internal::FEValuesImplementation::FiniteElementRelatedData<dim, spacedim> &output_data) const
 {
-  return setup_data(std::unique_ptr<typename FiniteElement<dim,spacedim>::InternalDataBase>(fe_system.get_subface_data(update_flags,mapping,quadrature,output_data)),
+  auto data = fe_system->get_subface_data(update_flags,mapping,quadrature,output_data);
+  return setup_data(Utilities::dynamic_unique_cast<typename FESystem<dim,spacedim>::InternalData>(std::move(data)),
                     update_flags,
                     quadrature);
 }
 
 
 template <int dim, int spacedim>
-typename FiniteElement<dim,spacedim>::InternalDataBase *
+std::unique_ptr<typename FiniteElement<dim,spacedim>::InternalDataBase>
 FE_Enriched<dim,spacedim>::get_data (const UpdateFlags      flags,
                                      const Mapping<dim,spacedim>    &mapping,
                                      const Quadrature<dim> &quadrature,
-                                     internal::FEValues::FiniteElementRelatedData< dim, spacedim >   &output_data) const
+                                     internal::FEValuesImplementation::FiniteElementRelatedData< dim, spacedim >   &output_data) const
 {
-  return setup_data(std::unique_ptr<typename FiniteElement<dim,spacedim>::InternalDataBase>(fe_system.get_data(flags,mapping,quadrature,output_data)),
+  auto data = fe_system->get_data(flags,mapping,quadrature,output_data);
+  return setup_data(Utilities::dynamic_unique_cast<typename FESystem<dim,spacedim>::InternalData>(std::move(data)),
                     flags,
                     quadrature);
 }
@@ -398,7 +404,7 @@ void FE_Enriched<dim,spacedim>::initialize (const std::vector<const FiniteElemen
 
   // now set up the interface constraints for h-refinement.
   // take them from fe_system:
-  this->interface_constraints = fe_system.interface_constraints;
+  this->interface_constraints = fe_system->interface_constraints;
 
   // if we just wrap another FE (i.e. use FE_Nothing as a second FE)
   // then it makes sense to have support points.
@@ -407,13 +413,13 @@ void FE_Enriched<dim,spacedim>::initialize (const std::vector<const FiniteElemen
   // this FE sits on the boundary or not. Thus for moment just copy support
   // points from fe system:
   {
-    this->unit_support_points      = fe_system.unit_support_points;
-    this->unit_face_support_points = fe_system.unit_face_support_points;
+    this->unit_support_points      = fe_system->unit_support_points;
+    this->unit_face_support_points = fe_system->unit_face_support_points;
   }
 
   // take adjust_quad_dof_index_for_face_orientation_table from FESystem:
   {
-    this->adjust_line_dof_index_for_line_orientation_table = fe_system.adjust_line_dof_index_for_line_orientation_table;
+    this->adjust_line_dof_index_for_line_orientation_table = fe_system->adjust_line_dof_index_for_line_orientation_table;
   }
 }
 
@@ -445,7 +451,7 @@ template <int dim, int spacedim>
 const FiniteElement<dim,spacedim> &
 FE_Enriched<dim,spacedim>::base_element (const unsigned int index) const
 {
-  return fe_system.base_element(index);
+  return fe_system->base_element(index);
 }
 
 
@@ -456,24 +462,24 @@ FE_Enriched<dim,spacedim>::fill_fe_values (const typename Triangulation< dim, sp
                                            const Quadrature< dim > &quadrature,
                                            const Mapping< dim, spacedim > &mapping,
                                            const typename Mapping< dim, spacedim >::InternalDataBase &mapping_internal,
-                                           const dealii::internal::FEValues::MappingRelatedData< dim, spacedim > &mapping_data,
+                                           const dealii::internal::FEValuesImplementation::MappingRelatedData< dim, spacedim > &mapping_data,
                                            const typename FiniteElement<dim,spacedim>::InternalDataBase &fe_internal,
-                                           internal::FEValues::FiniteElementRelatedData< dim, spacedim > &output_data
+                                           internal::FEValuesImplementation::FiniteElementRelatedData< dim, spacedim > &output_data
                                           ) const
 {
-  Assert (dynamic_cast<const InternalData *> (&fe_internal) != NULL,
+  Assert (dynamic_cast<const InternalData *> (&fe_internal) != nullptr,
           ExcInternalError());
   const InternalData &fe_data = static_cast<const InternalData &> (fe_internal);
 
   // call FESystem's method to fill everything without enrichment function
-  fe_system.fill_fe_values(cell,
-                           cell_similarity,
-                           quadrature,
-                           mapping,
-                           mapping_internal,
-                           mapping_data,
-                           *fe_data.fesystem_data,
-                           output_data);
+  fe_system->fill_fe_values(cell,
+                            cell_similarity,
+                            quadrature,
+                            mapping,
+                            mapping_internal,
+                            mapping_data,
+                            *fe_data.fesystem_data,
+                            output_data);
 
   if (is_enriched)
     multiply_by_enrichment(quadrature,
@@ -492,24 +498,24 @@ FE_Enriched<dim,spacedim>::fill_fe_face_values
  const Quadrature< dim-1 > &quadrature,
  const Mapping< dim, spacedim > &mapping,
  const typename Mapping< dim, spacedim >::InternalDataBase &mapping_internal,
- const dealii::internal::FEValues::MappingRelatedData< dim, spacedim > &mapping_data,
+ const dealii::internal::FEValuesImplementation::MappingRelatedData< dim, spacedim > &mapping_data,
  const typename FiniteElement<dim,spacedim>::InternalDataBase &fe_internal,
- internal::FEValues::FiniteElementRelatedData< dim, spacedim > &output_data
+ internal::FEValuesImplementation::FiniteElementRelatedData< dim, spacedim > &output_data
 ) const
 {
-  Assert (dynamic_cast<const InternalData *> (&fe_internal) != NULL,
+  Assert (dynamic_cast<const InternalData *> (&fe_internal) != nullptr,
           ExcInternalError());
   const InternalData &fe_data = static_cast<const InternalData &> (fe_internal);
 
   // call FESystem's method to fill everything without enrichment function
-  fe_system.fill_fe_face_values(cell,
-                                face_no,
-                                quadrature,
-                                mapping,
-                                mapping_internal,
-                                mapping_data,
-                                *fe_data.fesystem_data,
-                                output_data);
+  fe_system->fill_fe_face_values(cell,
+                                 face_no,
+                                 quadrature,
+                                 mapping,
+                                 mapping_internal,
+                                 mapping_data,
+                                 *fe_data.fesystem_data,
+                                 output_data);
 
   if (is_enriched)
     multiply_by_enrichment(quadrature,
@@ -529,25 +535,25 @@ FE_Enriched<dim,spacedim>::fill_fe_subface_values
  const Quadrature< dim-1 > &quadrature,
  const Mapping< dim, spacedim > &mapping,
  const typename Mapping< dim, spacedim >::InternalDataBase &mapping_internal,
- const dealii::internal::FEValues::MappingRelatedData< dim, spacedim > &mapping_data,
+ const dealii::internal::FEValuesImplementation::MappingRelatedData< dim, spacedim > &mapping_data,
  const typename FiniteElement<dim,spacedim>::InternalDataBase &fe_internal,
- internal::FEValues::FiniteElementRelatedData< dim, spacedim > &output_data
+ internal::FEValuesImplementation::FiniteElementRelatedData< dim, spacedim > &output_data
 ) const
 {
-  Assert (dynamic_cast<const InternalData *> (&fe_internal) != NULL,
+  Assert (dynamic_cast<const InternalData *> (&fe_internal) != nullptr,
           ExcInternalError());
   const InternalData &fe_data = static_cast<const InternalData &> (fe_internal);
 
   // call FESystem's method to fill everything without enrichment function
-  fe_system.fill_fe_subface_values(cell,
-                                   face_no,
-                                   sub_no,
-                                   quadrature,
-                                   mapping,
-                                   mapping_internal,
-                                   mapping_data,
-                                   *fe_data.fesystem_data,
-                                   output_data);
+  fe_system->fill_fe_subface_values(cell,
+                                    face_no,
+                                    sub_no,
+                                    quadrature,
+                                    mapping,
+                                    mapping_internal,
+                                    mapping_data,
+                                    *fe_data.fesystem_data,
+                                    output_data);
 
   if (is_enriched)
     multiply_by_enrichment(quadrature,
@@ -565,9 +571,9 @@ void
 FE_Enriched<dim,spacedim>::multiply_by_enrichment
 (const Quadrature<dim_1> &quadrature,
  const InternalData &fe_data,
- const internal::FEValues::MappingRelatedData<dim,spacedim> &mapping_data,
+ const internal::FEValuesImplementation::MappingRelatedData<dim,spacedim> &mapping_data,
  const typename Triangulation< dim, spacedim >::cell_iterator &cell,
- internal::FEValues::FiniteElementRelatedData<dim,spacedim> &output_data) const
+ internal::FEValuesImplementation::FiniteElementRelatedData<dim,spacedim> &output_data) const
 {
   // mapping_data will contain quadrature points on the real element.
   // fe_internal is needed to get update flags
@@ -595,7 +601,7 @@ FE_Enriched<dim,spacedim>::multiply_by_enrichment
         base_fe      = base_element(base_no);
         typename FiniteElement<dim,spacedim>::InternalDataBase &
         base_fe_data = fe_data.get_fe_data(base_no);
-        internal::FEValues::FiniteElementRelatedData<dim,spacedim> &
+        internal::FEValuesImplementation::FiniteElementRelatedData<dim,spacedim> &
         base_data    = fe_data.get_fe_output_object(base_no);
 
         const UpdateFlags base_flags = base_fe_data.update_each;
@@ -656,7 +662,12 @@ FE_Enriched<dim,spacedim>::multiply_by_enrichment
                                    fe_data.enrichment[base_no].size()));
       for (unsigned int m=0; m < base_no_mult_local_enriched_dofs[base_no].size(); m++)
         {
-          Assert (enrichments[base_no-1][m](cell) !=NULL,
+          //Avoid evaluating quadrature points if no dofs are assigned. This
+          //happens when FE_Nothing is used together with other FE (i.e. FE_Q) as enrichments.
+          if (base_no_mult_local_enriched_dofs[base_no][m].size()==0)
+            continue;
+
+          Assert (enrichments[base_no-1][m](cell) != nullptr,
                   ExcMessage("The pointer to the enrichment function is NULL"));
 
           Assert (enrichments[base_no-1][m](cell)->n_components == 1,
@@ -756,7 +767,7 @@ const FESystem<dim,spacedim> &
 FE_Enriched<dim,spacedim>::
 get_fe_system() const
 {
-  return fe_system;
+  return *fe_system;
 }
 
 
@@ -778,13 +789,13 @@ get_face_interpolation_matrix (const FiniteElement<dim,spacedim> &source,
   if (const FE_Enriched<dim,spacedim> *fe_enr_other
       = dynamic_cast<const FE_Enriched<dim,spacedim>*>(&source))
     {
-      fe_system.get_face_interpolation_matrix(fe_enr_other->get_fe_system(),
-                                              matrix);
+      fe_system->get_face_interpolation_matrix(fe_enr_other->get_fe_system(),
+                                               matrix);
     }
   else
     {
-      typedef FiniteElement<dim,spacedim> FEL;
-      AssertThrow(false,typename FEL::ExcInterpolationNotImplemented());
+      AssertThrow(false, (typename FiniteElement<dim,spacedim>::
+                          ExcInterpolationNotImplemented()));
     }
 }
 
@@ -799,14 +810,14 @@ get_subface_interpolation_matrix (const FiniteElement<dim,spacedim> &source,
   if (const FE_Enriched<dim,spacedim> *fe_enr_other
       = dynamic_cast<const FE_Enriched<dim,spacedim>*>(&source))
     {
-      fe_system.get_subface_interpolation_matrix(fe_enr_other->get_fe_system(),
-                                                 subface,
-                                                 matrix);
+      fe_system->get_subface_interpolation_matrix(fe_enr_other->get_fe_system(),
+                                                  subface,
+                                                  matrix);
     }
   else
     {
-      typedef FiniteElement<dim,spacedim> FEL;
-      AssertThrow(false,typename FEL::ExcInterpolationNotImplemented());
+      AssertThrow(false,(typename FiniteElement<dim,spacedim>::
+                         ExcInterpolationNotImplemented()));
     }
 }
 
@@ -819,7 +830,7 @@ hp_vertex_dof_identities (const FiniteElement<dim,spacedim> &fe_other) const
   if (const FE_Enriched<dim,spacedim> *fe_enr_other
       = dynamic_cast<const FE_Enriched<dim,spacedim>*>(&fe_other))
     {
-      return fe_system.hp_vertex_dof_identities(fe_enr_other->get_fe_system());
+      return fe_system->hp_vertex_dof_identities(fe_enr_other->get_fe_system());
     }
   else
     {
@@ -837,7 +848,7 @@ hp_line_dof_identities (const FiniteElement<dim,spacedim> &fe_other) const
   if (const FE_Enriched<dim,spacedim> *fe_enr_other
       = dynamic_cast<const FE_Enriched<dim,spacedim>*>(&fe_other))
     {
-      return fe_system.hp_line_dof_identities(fe_enr_other->get_fe_system());
+      return fe_system->hp_line_dof_identities(fe_enr_other->get_fe_system());
     }
   else
     {
@@ -855,7 +866,7 @@ hp_quad_dof_identities (const FiniteElement<dim,spacedim> &fe_other) const
   if (const FE_Enriched<dim,spacedim> *fe_enr_other
       = dynamic_cast<const FE_Enriched<dim,spacedim>*>(&fe_other))
     {
-      return fe_system.hp_quad_dof_identities(fe_enr_other->get_fe_system());
+      return fe_system->hp_quad_dof_identities(fe_enr_other->get_fe_system());
     }
   else
     {
@@ -883,7 +894,7 @@ compare_for_face_domination (const FiniteElement<dim,spacedim> &fe_other) const
   if (const FE_Enriched<dim,spacedim> *fe_enr_other
       = dynamic_cast<const FE_Enriched<dim,spacedim>*>(&fe_other))
     {
-      return fe_system.compare_for_face_domination(fe_enr_other->get_fe_system());
+      return fe_system->compare_for_face_domination(fe_enr_other->get_fe_system());
     }
   else
     {
@@ -897,7 +908,7 @@ const FullMatrix<double> &
 FE_Enriched<dim,spacedim>::get_prolongation_matrix (const unsigned int child,
                                                     const RefinementCase<dim> &refinement_case) const
 {
-  return fe_system.get_prolongation_matrix(child, refinement_case);
+  return fe_system->get_prolongation_matrix(child, refinement_case);
 }
 
 
@@ -906,7 +917,7 @@ const FullMatrix<double> &
 FE_Enriched<dim,spacedim>::get_restriction_matrix (const unsigned int child,
                                                    const RefinementCase<dim> &refinement_case) const
 {
-  return fe_system.get_restriction_matrix(child, refinement_case);
+  return fe_system->get_restriction_matrix(child, refinement_case);
 }
 
 
@@ -930,7 +941,7 @@ InternalData::get_fe_data (const unsigned int base_no) const
 
 
 template <int dim, int spacedim>
-internal::FEValues::FiniteElementRelatedData<dim,spacedim> &
+internal::FEValuesImplementation::FiniteElementRelatedData<dim,spacedim> &
 FE_Enriched<dim,spacedim>::
 InternalData::get_fe_output_object (const unsigned int base_no) const
 {
@@ -942,5 +953,3 @@ InternalData::get_fe_output_object (const unsigned int base_no) const
 #include "fe_enriched.inst"
 
 DEAL_II_NAMESPACE_CLOSE
-
-#endif // CXX14
