@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 2009 - 2016 by the deal.II authors
+// Copyright (C) 2009 - 2018 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -20,30 +20,38 @@
 #include <deal.II/fe/fe_tools.h>
 #include <deal.II/base/quadrature_lib.h>
 #include <deal.II/lac/householder.h>
+
 #include <sstream>
+#include <deal.II/base/std_cxx14/memory.h>
 
 DEAL_II_NAMESPACE_OPEN
 
 
-namespace
+namespace internal
 {
-  std::vector<Point<1> >
-  get_QGaussLobatto_points (const unsigned int degree)
+  namespace FE_FaceQImplementation
   {
-    if (degree > 0)
-      return QGaussLobatto<1>(degree+1).get_points();
-    else
-      return std::vector<Point<1> >(1, Point<1>(0.5));
+    namespace
+    {
+      std::vector<Point<1> >
+      get_QGaussLobatto_points (const unsigned int degree)
+      {
+        if (degree > 0)
+          return QGaussLobatto<1>(degree+1).get_points();
+        else
+          return std::vector<Point<1> >(1, Point<1>(0.5));
+      }
+    }
   }
 }
 
 template <int dim, int spacedim>
 FE_FaceQ<dim,spacedim>::FE_FaceQ (const unsigned int degree)
   :
-  FE_PolyFace<TensorProductPolynomials<dim-1>, dim, spacedim> (
-    TensorProductPolynomials<dim-1>(Polynomials::generate_complete_Lagrange_basis(get_QGaussLobatto_points(degree))),
-    FiniteElementData<dim>(get_dpo_vector(degree), 1, degree, FiniteElementData<dim>::L2),
-    std::vector<bool>(1,true))
+  FE_PolyFace<TensorProductPolynomials<dim-1>, dim, spacedim>
+  (TensorProductPolynomials<dim-1>(Polynomials::generate_complete_Lagrange_basis(internal::FE_FaceQImplementation::get_QGaussLobatto_points(degree))),
+   FiniteElementData<dim>(get_dpo_vector(degree), 1, degree, FiniteElementData<dim>::L2),
+   std::vector<bool>(1,true))
 {
   // initialize unit face support points
   const unsigned int codim = dim-1;
@@ -54,7 +62,7 @@ FE_FaceQ<dim,spacedim>::FE_FaceQ (const unsigned int degree)
       this->unit_face_support_points[0][d] = 0.5;
   else
     {
-      std::vector<Point<1> > points = get_QGaussLobatto_points(degree);
+      std::vector<Point<1> > points = internal::FE_FaceQImplementation::get_QGaussLobatto_points(degree);
 
       unsigned int k=0;
       for (unsigned int iz=0; iz <= ((codim>2) ? this->degree : 0) ; ++iz)
@@ -102,10 +110,10 @@ FE_FaceQ<dim,spacedim>::FE_FaceQ (const unsigned int degree)
 
 
 template <int dim, int spacedim>
-FiniteElement<dim,spacedim> *
+std::unique_ptr<FiniteElement<dim,spacedim> >
 FE_FaceQ<dim,spacedim>::clone() const
 {
-  return new FE_FaceQ<dim,spacedim>(this->degree);
+  return std_cxx14::make_unique<FE_FaceQ<dim,spacedim>>(this->degree);
 }
 
 
@@ -217,7 +225,7 @@ get_subface_interpolation_matrix (const FiniteElement<dim,spacedim> &x_source_fe
           Assert (std::fabs(sum-1) < eps, ExcInternalError());
         }
     }
-  else if (dynamic_cast<const FE_Nothing<dim> *>(&x_source_fe) != 0)
+  else if (dynamic_cast<const FE_Nothing<dim> *>(&x_source_fe) != nullptr)
     {
       // nothing to do here, the FE_Nothing has no degrees of freedom anyway
     }
@@ -261,6 +269,162 @@ FE_FaceQ<dim,spacedim>::hp_constraints_are_implemented () const
 
 
 
+template <int dim, int spacedim>
+std::vector<std::pair<unsigned int, unsigned int> >
+FE_FaceQ<dim, spacedim>::
+hp_vertex_dof_identities (const FiniteElement<dim, spacedim> &/*fe_other*/) const
+{
+  // this element is always discontinuous at vertices
+  return
+    std::vector<std::pair<unsigned int, unsigned int> > ();
+}
+
+
+
+template <int dim, int spacedim>
+std::vector<std::pair<unsigned int, unsigned int> >
+FE_FaceQ<dim, spacedim>::
+hp_line_dof_identities (const FiniteElement<dim, spacedim> &fe_other) const
+{
+  Assert (dim >= 2, ExcInternalError());
+
+  // this element is continuous only for the highest dimensional bounding object
+  if (dim > 2)
+    return
+      std::vector<std::pair<unsigned int, unsigned int> > ();
+  else
+    {
+      // this is similar to the FE_Q_Base class
+      if (const FE_FaceQ<dim,spacedim> *fe_q_other
+          = dynamic_cast<const FE_FaceQ<dim,spacedim>*>(&fe_other))
+        {
+          // dofs are located along lines, so two dofs are identical if they are
+          // located at identical positions.
+          // Therefore, read the points in unit_support_points for the
+          // first coordinate direction. We take the lexicographic ordering of the
+          // points in the second direction (i.e., y-direction) since we know
+          // that the first p+1 dofs are located at the left (x=0) face.
+          const unsigned int p = this->degree;
+          const unsigned int q = fe_q_other->degree;
+
+          std::vector<std::pair<unsigned int, unsigned int> > identities;
+
+          const std::vector<unsigned int> &index_map_inverse=
+            this->poly_space.get_numbering_inverse();
+          const std::vector<unsigned int> &index_map_inverse_other=
+            fe_q_other->poly_space.get_numbering_inverse();
+
+          for (unsigned int i=0; i<p+1; ++i)
+            for (unsigned int j=0; j<q+1; ++j)
+              if (std::fabs(this->unit_support_points[index_map_inverse[i]][dim-1]-
+                            fe_q_other->unit_support_points[index_map_inverse_other[j]][dim-1])
+                  < 1e-14)
+                identities.emplace_back (i, j);
+
+          return identities;
+        }
+      else if (dynamic_cast<const FE_Nothing<dim>*>(&fe_other) != nullptr)
+        {
+          // the FE_Nothing has no degrees of freedom, so there are no
+          // equivalencies to be recorded
+          return std::vector<std::pair<unsigned int, unsigned int> > ();
+        }
+      else if (fe_other.dofs_per_face == 0)
+        {
+          // if the other element has no elements on faces at all,
+          // then it would be impossible to enforce any kind of
+          // continuity even if we knew exactly what kind of element
+          // we have -- simply because the other element declares
+          // that it is discontinuous because it has no DoFs on
+          // its faces. in that case, just state that we have no
+          // constraints to declare
+          return std::vector<std::pair<unsigned int, unsigned int> > ();
+        }
+      else
+        {
+          Assert (false, ExcNotImplemented());
+          return std::vector<std::pair<unsigned int, unsigned int> > ();
+        }
+    }
+}
+
+
+
+template <int dim, int spacedim>
+std::vector<std::pair<unsigned int, unsigned int> >
+FE_FaceQ<dim, spacedim>::
+hp_quad_dof_identities (const FiniteElement<dim, spacedim> &fe_other) const
+{
+  Assert (dim >= 3, ExcInternalError());
+
+  // this element is continuous only for the highest dimensional bounding object
+  if (dim > 3)
+    return
+      std::vector<std::pair<unsigned int, unsigned int> > ();
+  else
+    {
+      // this is similar to the FE_Q_Base class
+      if (const FE_FaceQ<dim,spacedim> *fe_q_other
+          = dynamic_cast<const FE_FaceQ<dim,spacedim>*>(&fe_other))
+        {
+          // this works exactly like the line case above, except that now we have
+          // to have two indices i1, i2 and j1, j2 to characterize the dofs on the
+          // face of each of the finite elements. since they are ordered
+          // lexicographically along the first line and we have a tensor product,
+          // the rest is rather straightforward
+          const unsigned int p = this->degree;
+          const unsigned int q = fe_q_other->degree;
+
+          std::vector<std::pair<unsigned int, unsigned int> > identities;
+
+          const std::vector<unsigned int> &index_map_inverse=
+            this->poly_space.get_numbering_inverse();
+          const std::vector<unsigned int> &index_map_inverse_other=
+            fe_q_other->poly_space.get_numbering_inverse();
+
+          std::vector<std::pair<unsigned int, unsigned int> > identities_1d;
+
+          for (unsigned int i=0; i<p+1; ++i)
+            for (unsigned int j=0; j<q+1; ++j)
+              if (std::fabs(this->unit_support_points[index_map_inverse[i]][dim-2]-
+                            fe_q_other->unit_support_points[index_map_inverse_other[j]][dim-2])
+                  < 1e-14)
+                identities_1d.emplace_back (i, j);
+
+          for (unsigned int n1=0; n1<identities_1d.size(); ++n1)
+            for (unsigned int n2=0; n2<identities_1d.size(); ++n2)
+              identities.emplace_back (identities_1d[n1].first*(p+1)+identities_1d[n2].first,
+                                       identities_1d[n1].second*(q+1)+identities_1d[n2].second);
+
+          return identities;
+        }
+      else if (dynamic_cast<const FE_Nothing<dim>*>(&fe_other) != nullptr)
+        {
+          // the FE_Nothing has no degrees of freedom, so there are no
+          // equivalencies to be recorded
+          return std::vector<std::pair<unsigned int, unsigned int> > ();
+        }
+      else if (fe_other.dofs_per_face == 0)
+        {
+          // if the other element has no elements on faces at all,
+          // then it would be impossible to enforce any kind of
+          // continuity even if we knew exactly what kind of element
+          // we have -- simply because the other element declares
+          // that it is discontinuous because it has no DoFs on
+          // its faces. in that case, just state that we have no
+          // constraints to declare
+          return std::vector<std::pair<unsigned int, unsigned int> > ();
+        }
+      else
+        {
+          Assert (false, ExcNotImplemented());
+          return std::vector<std::pair<unsigned int, unsigned int> > ();
+        }
+    }
+}
+
+
+
 template <int dim, int spacedim>
 FiniteElementDomination::Domination
 FE_FaceQ<dim,spacedim>::
@@ -307,7 +471,26 @@ FE_FaceQ<dim,spacedim>::get_constant_modes () const
          (constant_modes, std::vector<unsigned int>(1, 0));
 }
 
+template <int dim, int spacedim>
+void
+FE_FaceQ<dim,spacedim>::
+convert_generalized_support_point_values_to_dof_values (const std::vector<Vector<double> > &support_point_values,
+                                                        std::vector<double>                &nodal_values) const
+{
+  AssertDimension (support_point_values.size(),
+                   this->get_unit_support_points().size());
+  AssertDimension (support_point_values.size(),
+                   nodal_values.size());
+  AssertDimension (this->dofs_per_cell,
+                   nodal_values.size());
 
+  for (unsigned int i=0; i<this->dofs_per_cell; ++i)
+    {
+      AssertDimension (support_point_values[i].size(), 1);
+
+      nodal_values[i] = support_point_values[i](0);
+    }
+}
 
 // ----------------------------- FE_FaceQ<1,spacedim> ------------------------
 
@@ -329,10 +512,10 @@ FE_FaceQ<1,spacedim>::FE_FaceQ (const unsigned int degree)
 
 
 template <int spacedim>
-FiniteElement<1,spacedim> *
-FE_FaceQ<1,spacedim>::clone() const
+std::unique_ptr<FiniteElement<1,spacedim>>
+                                        FE_FaceQ<1,spacedim>::clone() const
 {
-  return new FE_FaceQ<1,spacedim>(this->degree);
+  return std_cxx14::make_unique<FE_FaceQ<1,spacedim>>(this->degree);
 }
 
 
@@ -415,6 +598,40 @@ FE_FaceQ<1,spacedim>::hp_constraints_are_implemented () const
   return true;
 }
 
+template <int spacedim>
+std::vector<std::pair<unsigned int, unsigned int> >
+FE_FaceQ<1, spacedim>::
+hp_vertex_dof_identities (const FiniteElement<1, spacedim> &/*fe_other*/) const
+{
+  // this element is always discontinuous at vertices
+  return
+    std::vector<std::pair<unsigned int, unsigned int> > (1,std::make_pair(0U,0U));
+}
+
+
+
+template <int spacedim>
+std::vector<std::pair<unsigned int, unsigned int> >
+FE_FaceQ<1, spacedim>::
+hp_line_dof_identities (const FiniteElement<1, spacedim> &) const
+{
+  // this element is continuous only for the highest dimensional bounding object
+  return
+    std::vector<std::pair<unsigned int, unsigned int> > ();
+}
+
+
+
+template <int spacedim>
+std::vector<std::pair<unsigned int, unsigned int> >
+FE_FaceQ<1, spacedim>::
+hp_quad_dof_identities (const FiniteElement<1, spacedim> &) const
+{
+  // this element is continuous only for the highest dimensional bounding object
+  return
+    std::vector<std::pair<unsigned int, unsigned int> > ();
+}
+
 
 
 template <int spacedim>
@@ -460,13 +677,13 @@ template <int spacedim>
 void
 FE_FaceQ<1,spacedim>::
 fill_fe_values(const typename Triangulation<1,spacedim>::cell_iterator &,
-               const CellSimilarity::Similarity                                   ,
+               const CellSimilarity::Similarity,
                const Quadrature<1> &,
                const Mapping<1,spacedim> &,
                const typename Mapping<1,spacedim>::InternalDataBase &,
-               const dealii::internal::FEValues::MappingRelatedData<1, spacedim> &,
+               const dealii::internal::FEValuesImplementation::MappingRelatedData<1, spacedim> &,
                const typename FiniteElement<1,spacedim>::InternalDataBase &,
-               dealii::internal::FEValues::FiniteElementRelatedData<1, spacedim> &) const
+               dealii::internal::FEValuesImplementation::FiniteElementRelatedData<1, spacedim> &) const
 {
   // Do nothing, since we do not have values in the interior
 }
@@ -481,9 +698,9 @@ fill_fe_face_values (const typename Triangulation<1,spacedim>::cell_iterator &,
                      const Quadrature<0> &,
                      const Mapping<1,spacedim> &,
                      const typename Mapping<1,spacedim>::InternalDataBase &,
-                     const dealii::internal::FEValues::MappingRelatedData<1, spacedim> &,
+                     const dealii::internal::FEValuesImplementation::MappingRelatedData<1, spacedim> &,
                      const typename FiniteElement<1,spacedim>::InternalDataBase        &fe_internal,
-                     dealii::internal::FEValues::FiniteElementRelatedData<1, spacedim> &output_data) const
+                     dealii::internal::FEValuesImplementation::FiniteElementRelatedData<1, spacedim> &output_data) const
 {
   const unsigned int foffset = face;
   if (fe_internal.update_each & update_values)
@@ -499,14 +716,14 @@ template <int spacedim>
 void
 FE_FaceQ<1,spacedim>::
 fill_fe_subface_values (const typename Triangulation<1,spacedim>::cell_iterator &,
-                        const unsigned int                                                 ,
-                        const unsigned int                                                 ,
+                        const unsigned int,
+                        const unsigned int,
                         const Quadrature<0> &,
                         const Mapping<1,spacedim> &,
                         const typename Mapping<1,spacedim>::InternalDataBase &,
-                        const dealii::internal::FEValues::MappingRelatedData<1, spacedim> &,
+                        const dealii::internal::FEValuesImplementation::MappingRelatedData<1, spacedim> &,
                         const typename FiniteElement<1,spacedim>::InternalDataBase &,
-                        dealii::internal::FEValues::FiniteElementRelatedData<1, spacedim> &) const
+                        dealii::internal::FEValuesImplementation::FiniteElementRelatedData<1, spacedim> &) const
 {
   Assert(false, ExcMessage("There are no sub-face values to fill in 1D!"));
 }
@@ -527,10 +744,10 @@ FE_FaceP<dim,spacedim>::FE_FaceP (const unsigned int degree)
 
 
 template <int dim, int spacedim>
-FiniteElement<dim,spacedim> *
+std::unique_ptr<FiniteElement<dim,spacedim> >
 FE_FaceP<dim,spacedim>::clone() const
 {
-  return new FE_FaceP<dim,spacedim>(this->degree);
+  return std_cxx14::make_unique<FE_FaceP<dim,spacedim>>(this->degree);
 }
 
 
@@ -579,7 +796,6 @@ FE_FaceP<dim,spacedim>::get_dpo_vector (const unsigned int deg)
 
 
 
-
 template <int dim, int spacedim>
 bool
 FE_FaceP<dim,spacedim>::hp_constraints_are_implemented () const
@@ -687,7 +903,7 @@ get_subface_interpolation_matrix (const FiniteElement<dim,spacedim> &x_source_fe
                                                             subface);
 
           for (unsigned int j = 0; j < source_fe->dofs_per_face; ++j)
-            mass (k , j) = source_fe->poly_space.compute_value(j, p);
+            mass (k, j) = source_fe->poly_space.compute_value(j, p);
         }
 
       Householder<double> H(mass);
@@ -727,7 +943,7 @@ get_subface_interpolation_matrix (const FiniteElement<dim,spacedim> &x_source_fe
             }
         }
     }
-  else if (dynamic_cast<const FE_Nothing<dim> *>(&x_source_fe) != 0)
+  else if (dynamic_cast<const FE_Nothing<dim> *>(&x_source_fe) != nullptr)
     {
       // nothing to do here, the FE_Nothing has no degrees of freedom anyway
     }
