@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 2009 - 2015 by the deal.II authors
+// Copyright (C) 2009 - 2018 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -15,13 +15,20 @@
 
 
 #include <deal.II/fe/fe_nothing.h>
+#include <deal.II/base/std_cxx14/memory.h>
 
 DEAL_II_NAMESPACE_OPEN
 
-namespace
+namespace internal
 {
-  const char *
-  zero_dof_message = "This element has no shape functions.";
+  namespace FE_Nothing
+  {
+    namespace
+    {
+      const char *
+      zero_dof_message = "This element has no shape functions.";
+    }
+  }
 }
 
 
@@ -48,10 +55,10 @@ FE_Nothing<dim,spacedim>::FE_Nothing (const unsigned int n_components,
 
 
 template <int dim, int spacedim>
-FiniteElement<dim,spacedim> *
+std::unique_ptr<FiniteElement<dim,spacedim> >
 FE_Nothing<dim,spacedim>::clone() const
 {
-  return new FE_Nothing<dim,spacedim>(*this);
+  return std_cxx14::make_unique<FE_Nothing<dim,spacedim>>(*this);
 }
 
 
@@ -63,7 +70,13 @@ FE_Nothing<dim,spacedim>::get_name () const
   std::ostringstream namebuf;
   namebuf << "FE_Nothing<" << dim << ">(";
   if (this->n_components() > 1)
-    namebuf << this->n_components();
+    {
+      namebuf << this->n_components();
+      if (dominate)
+        namebuf << ", dominating";
+    }
+  else if (dominate)
+    namebuf << "dominating";
   namebuf << ")";
   return namebuf.str();
 }
@@ -84,26 +97,24 @@ double
 FE_Nothing<dim,spacedim>::shape_value (const unsigned int /*i*/,
                                        const Point<dim> & /*p*/) const
 {
-  (void)zero_dof_message;
-  Assert(false,ExcMessage(zero_dof_message));
+  (void)internal::FE_Nothing::zero_dof_message;
+  Assert(false,ExcMessage(internal::FE_Nothing::zero_dof_message));
   return 0;
 }
 
 
 
 template <int dim, int spacedim>
-typename FiniteElement<dim,spacedim>::InternalDataBase *
+std::unique_ptr<typename FiniteElement<dim,spacedim>::InternalDataBase>
 FE_Nothing<dim,spacedim>::get_data (const UpdateFlags                                                    /*update_flags*/,
                                     const Mapping<dim,spacedim>                                         &/*mapping*/,
                                     const Quadrature<dim>                                               &/*quadrature*/,
-                                    dealii::internal::FEValues::FiniteElementRelatedData<dim, spacedim> &/*output_data*/) const
+                                    dealii::internal::FEValuesImplementation::FiniteElementRelatedData<dim, spacedim> &/*output_data*/) const
 {
   // Create a default data object.  Normally we would then
   // need to resize things to hold the appropriate numbers
   // of dofs, but in this case all data fields are empty.
-  typename FiniteElement<dim,spacedim>::InternalDataBase *data
-    = new typename FiniteElement<dim,spacedim>::InternalDataBase();
-  return data;
+  return std_cxx14::make_unique<typename FiniteElement<dim,spacedim>::InternalDataBase>();
 }
 
 
@@ -112,13 +123,13 @@ template <int dim, int spacedim>
 void
 FE_Nothing<dim,spacedim>::
 fill_fe_values (const typename Triangulation<dim,spacedim>::cell_iterator &,
-                const CellSimilarity::Similarity                                     ,
+                const CellSimilarity::Similarity,
                 const Quadrature<dim> &,
                 const Mapping<dim,spacedim> &,
                 const typename Mapping<dim,spacedim>::InternalDataBase &,
-                const dealii::internal::FEValues::MappingRelatedData<dim, spacedim> &,
+                const dealii::internal::FEValuesImplementation::MappingRelatedData<dim, spacedim> &,
                 const typename FiniteElement<dim,spacedim>::InternalDataBase &,
-                dealii::internal::FEValues::FiniteElementRelatedData<dim, spacedim> &) const
+                dealii::internal::FEValuesImplementation::FiniteElementRelatedData<dim, spacedim> &) const
 {
   // leave data fields empty
 }
@@ -129,13 +140,13 @@ template <int dim, int spacedim>
 void
 FE_Nothing<dim,spacedim>::
 fill_fe_face_values (const typename Triangulation<dim,spacedim>::cell_iterator &,
-                     const unsigned int                                                   ,
+                     const unsigned int,
                      const Quadrature<dim-1>                                             &,
                      const Mapping<dim,spacedim> &,
                      const typename Mapping<dim,spacedim>::InternalDataBase &,
-                     const dealii::internal::FEValues::MappingRelatedData<dim, spacedim> &,
+                     const dealii::internal::FEValuesImplementation::MappingRelatedData<dim, spacedim> &,
                      const typename FiniteElement<dim,spacedim>::InternalDataBase &,
-                     dealii::internal::FEValues::FiniteElementRelatedData<dim, spacedim> &) const
+                     dealii::internal::FEValuesImplementation::FiniteElementRelatedData<dim, spacedim> &) const
 {
   // leave data fields empty
 }
@@ -146,14 +157,14 @@ template <int dim, int spacedim>
 void
 FE_Nothing<dim,spacedim>::
 fill_fe_subface_values (const typename Triangulation<dim,spacedim>::cell_iterator &,
-                        const unsigned int                                                   ,
-                        const unsigned int                                                   ,
+                        const unsigned int,
+                        const unsigned int,
                         const Quadrature<dim-1>                                             &,
                         const Mapping<dim,spacedim> &,
                         const typename Mapping<dim,spacedim>::InternalDataBase &,
-                        const dealii::internal::FEValues::MappingRelatedData<dim, spacedim> &,
+                        const dealii::internal::FEValuesImplementation::MappingRelatedData<dim, spacedim> &,
                         const typename FiniteElement<dim,spacedim>::InternalDataBase &,
-                        dealii::internal::FEValues::FiniteElementRelatedData<dim, spacedim> &) const
+                        dealii::internal::FEValuesImplementation::FiniteElementRelatedData<dim, spacedim> &) const
 {
   // leave data fields empty
 }
@@ -168,6 +179,29 @@ FE_Nothing<dim,spacedim>::is_dominating() const
 }
 
 
+
+template <int dim, int spacedim>
+bool
+FE_Nothing<dim,spacedim>::operator == (const FiniteElement<dim,spacedim> &f) const
+{
+  // Compare fields stored in the base class
+  if (! (this->FiniteElement<dim,spacedim>::operator== (f)))
+    return false;
+
+  // Then make sure the other object is really of type FE_Nothing,
+  // and compare the data that has been passed to both objects'
+  // constructors.
+  if (const FE_Nothing<dim,spacedim> *f_nothing
+      = dynamic_cast<const FE_Nothing<dim,spacedim> *>(&f))
+    return ((dominate == f_nothing->dominate)
+            &&
+            (this->components == f_nothing->components));
+  else
+    return false;
+}
+
+
+
 template <int dim, int spacedim>
 FiniteElementDomination::Domination
 FE_Nothing<dim,spacedim> ::
@@ -179,7 +213,7 @@ compare_for_face_domination (const FiniteElement<dim,spacedim> &fe) const
       return FiniteElementDomination::no_requirements;
     }
   // if it does and the other is FE_Nothing, either can dominate
-  else if (dynamic_cast<const FE_Nothing<dim>*>(&fe) != 0)
+  else if (dynamic_cast<const FE_Nothing<dim>*>(&fe) != nullptr)
     {
       return FiniteElementDomination::either_element_can_dominate;
     }
@@ -239,6 +273,27 @@ hp_constraints_are_implemented () const
 }
 
 
+
+template <int dim, int spacedim>
+void
+FE_Nothing<dim,spacedim>::
+get_interpolation_matrix (const FiniteElement<dim,spacedim> &/*source_fe*/,
+                          FullMatrix<double>       &interpolation_matrix) const
+{
+  // Since this element has no dofs,
+  // the interpolation matrix is necessarily empty.
+  (void)interpolation_matrix;
+
+  Assert (interpolation_matrix.m() == 0,
+          ExcDimensionMismatch (interpolation_matrix.m(),
+                                0));
+  Assert (interpolation_matrix.n() == 0,
+          ExcDimensionMismatch (interpolation_matrix.n(),
+                                0));
+}
+
+
+
 template <int dim, int spacedim>
 void
 FE_Nothing<dim,spacedim>::
@@ -284,4 +339,3 @@ get_subface_interpolation_matrix (const FiniteElement<dim,spacedim> & /*source_f
 
 
 DEAL_II_NAMESPACE_CLOSE
-
