@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 2004 - 2015 by the deal.II authors
+// Copyright (C) 2004 - 2018 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -36,15 +36,15 @@ fill_fe_values (const Triangulation<1,2>::cell_iterator &,
                 const Quadrature<1>                                       &quadrature,
                 const Mapping<1,2>                                        &mapping,
                 const Mapping<1,2>::InternalDataBase                      &mapping_internal,
-                const dealii::internal::FEValues::MappingRelatedData<1,2> &mapping_data,
+                const dealii::internal::FEValuesImplementation::MappingRelatedData<1,2> &mapping_data,
                 const FiniteElement<1,2>::InternalDataBase                &fe_internal,
-                dealii::internal::FEValues::FiniteElementRelatedData<1,2> &output_data) const
+                dealii::internal::FEValuesImplementation::FiniteElementRelatedData<1,2> &output_data) const
 {
   // convert data object to internal
   // data for this class. fails with
   // an exception if that is not
   // possible
-  Assert (dynamic_cast<const InternalData *> (&fe_internal) != 0, ExcInternalError());
+  Assert (dynamic_cast<const InternalData *> (&fe_internal) != nullptr, ExcInternalError());
   const InternalData &fe_data = static_cast<const InternalData &> (fe_internal);
 
   // transform gradients and higher derivatives. there is nothing to do
@@ -96,14 +96,14 @@ fill_fe_values (const Triangulation<2,3>::cell_iterator &,
                 const Quadrature<2>                                       &quadrature,
                 const Mapping<2,3>                                        &mapping,
                 const Mapping<2,3>::InternalDataBase                      &mapping_internal,
-                const dealii::internal::FEValues::MappingRelatedData<2,3> &mapping_data,
+                const dealii::internal::FEValuesImplementation::MappingRelatedData<2,3> &mapping_data,
                 const FiniteElement<2,3>::InternalDataBase                &fe_internal,
-                dealii::internal::FEValues::FiniteElementRelatedData<2,3> &output_data) const
+                dealii::internal::FEValuesImplementation::FiniteElementRelatedData<2,3> &output_data) const
 {
 
   // assert that the following dynamics
   // cast is really well-defined.
-  Assert (dynamic_cast<const InternalData *> (&fe_internal) != 0, ExcInternalError());
+  Assert (dynamic_cast<const InternalData *> (&fe_internal) != nullptr, ExcInternalError());
   const InternalData &fe_data = static_cast<const InternalData &> (fe_internal);
 
   // transform gradients and higher derivatives. there is nothing to do
@@ -154,16 +154,16 @@ fill_fe_values (const Triangulation<1,2>::cell_iterator &,
                 const Quadrature<1>                                       &quadrature,
                 const Mapping<1,2>                                        &mapping,
                 const Mapping<1,2>::InternalDataBase                      &mapping_internal,
-                const dealii::internal::FEValues::MappingRelatedData<1,2> &mapping_data,
+                const dealii::internal::FEValuesImplementation::MappingRelatedData<1,2> &mapping_data,
                 const FiniteElement<1,2>::InternalDataBase                &fe_internal,
-                dealii::internal::FEValues::FiniteElementRelatedData<1,2> &output_data) const
+                dealii::internal::FEValuesImplementation::FiniteElementRelatedData<1,2> &output_data) const
 {
   // convert data object to internal
   // data for this class. fails with
   // an exception if that is not
   // possible
 
-  Assert (dynamic_cast<const InternalData *> (&fe_internal) != 0, ExcInternalError());
+  Assert (dynamic_cast<const InternalData *> (&fe_internal) != nullptr, ExcInternalError());
   const InternalData &fe_data = static_cast<const InternalData &> (fe_internal);
 
   // transform gradients and higher derivatives. there is nothing to do
@@ -214,11 +214,11 @@ fill_fe_values (const Triangulation<2,3>::cell_iterator &,
                 const Quadrature<2>                                       &quadrature,
                 const Mapping<2,3>                                        &mapping,
                 const Mapping<2,3>::InternalDataBase                      &mapping_internal,
-                const dealii::internal::FEValues::MappingRelatedData<2,3> &mapping_data,
+                const dealii::internal::FEValuesImplementation::MappingRelatedData<2,3> &mapping_data,
                 const FiniteElement<2,3>::InternalDataBase                &fe_internal,
-                dealii::internal::FEValues::FiniteElementRelatedData<2,3> &output_data) const
+                dealii::internal::FEValuesImplementation::FiniteElementRelatedData<2,3> &output_data) const
 {
-  Assert (dynamic_cast<const InternalData *> (&fe_internal) != 0, ExcInternalError());
+  Assert (dynamic_cast<const InternalData *> (&fe_internal) != nullptr, ExcInternalError());
   const InternalData &fe_data = static_cast<const InternalData &> (fe_internal);
 
   // transform gradients and higher derivatives. there is nothing to do
