@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 2000 - 2017 by the deal.II authors
+// Copyright (C) 2000 - 2018 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -30,70 +30,74 @@
 
 #include <vector>
 #include <sstream>
+#include <deal.II/base/std_cxx14/memory.h>
 
 DEAL_II_NAMESPACE_OPEN
 
 
-namespace FE_Q_Helper
+namespace internal
 {
-  namespace
+  namespace FE_Q_Base
   {
-    // get the renumbering for faces
-    template <int dim>
-    inline
-    std::vector<unsigned int>
-    face_lexicographic_to_hierarchic_numbering (const unsigned int degree)
+    namespace
     {
-      std::vector<unsigned int> dpo(dim, 1U);
-      for (unsigned int i=1; i<dpo.size(); ++i)
-        dpo[i]=dpo[i-1]*(degree-1);
-      const dealii::FiniteElementData<dim-1> face_data(dpo,1,degree);
-      std::vector<unsigned int> face_renumber (face_data.dofs_per_cell);
-      FETools::lexicographic_to_hierarchic_numbering (face_data, face_renumber);
-      return face_renumber;
-    }
+      // get the renumbering for faces
+      template <int dim>
+      inline
+      std::vector<unsigned int>
+      face_lexicographic_to_hierarchic_numbering (const unsigned int degree)
+      {
+        std::vector<unsigned int> dpo(dim, 1U);
+        for (unsigned int i=1; i<dpo.size(); ++i)
+          dpo[i]=dpo[i-1]*(degree-1);
+        const dealii::FiniteElementData<dim-1> face_data(dpo,1,degree);
+        std::vector<unsigned int> face_renumber (face_data.dofs_per_cell);
+        FETools::lexicographic_to_hierarchic_numbering (face_data, face_renumber);
+        return face_renumber;
+      }
 
-    // dummy specialization for dim == 1 to avoid linker errors
-    template <>
-    inline
-    std::vector<unsigned int>
-    face_lexicographic_to_hierarchic_numbering<1> (const unsigned int)
-    {
-      return std::vector<unsigned int>();
-    }
+      // dummy specialization for dim == 1 to avoid linker errors
+      template <>
+      inline
+      std::vector<unsigned int>
+      face_lexicographic_to_hierarchic_numbering<1> (const unsigned int)
+      {
+        return std::vector<unsigned int>();
+      }
 
 
 
-    // in get_restriction_matrix() and get_prolongation_matrix(), want to undo
-    // tensorization on inner loops for performance reasons. this clears a
-    // dim-array
-    template <int dim>
-    inline
-    void
-    zero_indices (unsigned int (&indices)[dim])
-    {
-      for (unsigned int d=0; d<dim; ++d)
-        indices[d] = 0;
-    }
+      // in get_restriction_matrix() and get_prolongation_matrix(), want to undo
+      // tensorization on inner loops for performance reasons. this clears a
+      // dim-array
+      template <int dim>
+      inline
+      void
+      zero_indices (unsigned int (&indices)[dim])
+      {
+        for (unsigned int d=0; d<dim; ++d)
+          indices[d] = 0;
+      }
 
 
 
-    // in get_restriction_matrix() and get_prolongation_matrix(), want to undo
-    // tensorization on inner loops for performance reasons. this increments
-    // tensor product indices
-    template <int dim>
-    inline
-    void
-    increment_indices (unsigned int       (&indices)[dim],
-                       const unsigned int   dofs1d)
-    {
-      ++indices[0];
-      for (int d=0; d<dim-1; ++d)
-        if (indices[d]==dofs1d)
-          {
-            indices[d] = 0;
-            indices[d+1]++;
-          }
+      // in get_restriction_matrix() and get_prolongation_matrix(), want to undo
+      // tensorization on inner loops for performance reasons. this increments
+      // tensor product indices
+      template <int dim>
+      inline
+      void
+      increment_indices (unsigned int       (&indices)[dim],
+                         const unsigned int   dofs1d)
+      {
+        ++indices[0];
+        for (int d=0; d<dim-1; ++d)
+          if (indices[d]==dofs1d)
+            {
+              indices[d] = 0;
+              indices[d+1]++;
+            }
+      }
     }
   }
 }
@@ -128,7 +132,7 @@ struct FE_Q_Base<PolynomialType,xdim,xspacedim>::Implementation
     const unsigned int dim = 2;
 
     unsigned int q_deg = fe.degree;
-    if (types_are_equal<PolynomialType, TensorProductPolynomialsBubbles<dim> >::value)
+    if (std::is_same<PolynomialType, TensorProductPolynomialsBubbles<dim> >::value)
       q_deg = fe.degree-1;
 
     // restricted to each face, the traces of the shape functions is an
@@ -177,7 +181,7 @@ struct FE_Q_Base<PolynomialType,xdim,xspacedim>::Implementation
     // 0 and on subface 1
     std::vector<Point<dim-1> > constraint_points;
     // Add midpoint
-    constraint_points.push_back (Point<dim-1> (0.5));
+    constraint_points.emplace_back(0.5);
 
     if (q_deg>1)
       {
@@ -204,7 +208,7 @@ struct FE_Q_Base<PolynomialType,xdim,xspacedim>::Implementation
     const std::vector<unsigned int> &index_map_inverse =
       fe.poly_space.get_numbering_inverse();
     const std::vector<unsigned int> face_index_map =
-      FE_Q_Helper::face_lexicographic_to_hierarchic_numbering<dim>(q_deg);
+      internal::FE_Q_Base::face_lexicographic_to_hierarchic_numbering<dim>(q_deg);
     Assert(std::abs(fe.poly_space.compute_value(index_map_inverse[0],Point<dim>())
                     - 1.) < 1e-14,
            ExcInternalError());
@@ -235,7 +239,7 @@ struct FE_Q_Base<PolynomialType,xdim,xspacedim>::Implementation
     const unsigned int dim = 3;
 
     unsigned int q_deg = fe.degree;
-    if (types_are_equal<PolynomialType,TensorProductPolynomialsBubbles<dim> >::value)
+    if (std::is_same<PolynomialType,TensorProductPolynomialsBubbles<dim> >::value)
       q_deg = fe.degree-1;
 
     // For a detailed documentation of the interpolation see the
@@ -256,13 +260,13 @@ struct FE_Q_Base<PolynomialType,xdim,xspacedim>::Implementation
     std::vector<Point<dim-1> > constraint_points;
 
     // Add midpoint
-    constraint_points.push_back (Point<dim-1> (0.5, 0.5));
+    constraint_points.emplace_back(0.5, 0.5);
 
     // Add midpoints of lines of "mother-face"
-    constraint_points.push_back (Point<dim-1> (0, 0.5));
-    constraint_points.push_back (Point<dim-1> (1, 0.5));
-    constraint_points.push_back (Point<dim-1> (0.5, 0));
-    constraint_points.push_back (Point<dim-1> (0.5, 1));
+    constraint_points.emplace_back(0, 0.5);
+    constraint_points.emplace_back(1, 0.5);
+    constraint_points.emplace_back(0.5, 0);
+    constraint_points.emplace_back(0.5, 1);
 
     if (q_deg>1)
       {
@@ -328,7 +332,7 @@ struct FE_Q_Base<PolynomialType,xdim,xspacedim>::Implementation
     const std::vector<unsigned int> &index_map_inverse =
       fe.poly_space.get_numbering_inverse();
     const std::vector<unsigned int> face_index_map =
-      FE_Q_Helper::face_lexicographic_to_hierarchic_numbering<dim>(q_deg);
+      internal::FE_Q_Base::face_lexicographic_to_hierarchic_numbering<dim>(q_deg);
     Assert(std::abs(fe.poly_space.compute_value(index_map_inverse[0],Point<dim>())
                     - 1.) < 1e-14,
            ExcInternalError());
@@ -418,7 +422,7 @@ FE_Q_Base<PolynomialType,dim,spacedim>::FE_Q_Base
   :
   FE_Poly<PolynomialType,dim,spacedim>(poly_space, fe_data, restriction_is_additive_flags,
                                        std::vector<ComponentMask>(1, std::vector<bool>(1,true))),
-  q_degree (types_are_equal<PolynomialType, TensorProductPolynomialsBubbles<dim> >::value
+  q_degree (std::is_same<PolynomialType, TensorProductPolynomialsBubbles<dim> >::value
             ?this->degree-1
             :this->degree)
 {}
@@ -440,7 +444,7 @@ FE_Q_Base<PolynomialType,dim,spacedim>::initialize (const std::vector<Point<1> >
   const unsigned int q_dofs_per_cell = Utilities::fixed_power<dim>(q_degree+1);
   Assert(q_dofs_per_cell == this->dofs_per_cell ||
          q_dofs_per_cell+1 == this->dofs_per_cell ||
-         q_dofs_per_cell+dim == this->dofs_per_cell , ExcInternalError());
+         q_dofs_per_cell+dim == this->dofs_per_cell, ExcInternalError());
 
   {
     std::vector<unsigned int> renumber(q_dofs_per_cell);
@@ -452,18 +456,18 @@ FE_Q_Base<PolynomialType,dim,spacedim>::initialize (const std::vector<Point<1> >
     this->poly_space.set_numbering(renumber);
   }
 
-  // finally fill in support points on cell and face
-  initialize_unit_support_points (points);
-  initialize_unit_face_support_points (points);
-
-  // reinit constraints
-  initialize_constraints (points);
+  // Finally fill in support points on cell and face and initialize
+  // constraints. All of this can happen in parallel
+  Threads::TaskGroup<> tasks;
+  tasks += Threads::new_task ([&]() { initialize_unit_support_points (points); });
+  tasks += Threads::new_task ([&]() { initialize_unit_face_support_points (points); });
+  tasks += Threads::new_task ([&]() { initialize_constraints (points); });
+  tasks += Threads::new_task ([&]() { this->initialize_quad_dof_index_permutation(); });
+  tasks.join_all();
 
   // do not initialize embedding and restriction here. these matrices are
   // initialized on demand in get_restriction_matrix and
   // get_prolongation_matrix
-
-  this->initialize_quad_dof_index_permutation();
 }
 
 
@@ -657,7 +661,7 @@ get_subface_interpolation_matrix (const FiniteElement<dim,spacedim> &x_source_fe
           Assert (std::fabs(sum-1) < eps, ExcInternalError());
         }
     }
-  else if (dynamic_cast<const FE_Nothing<dim> *>(&x_source_fe) != 0)
+  else if (dynamic_cast<const FE_Nothing<dim> *>(&x_source_fe) != nullptr)
     {
       // nothing to do here, the FE_Nothing has no degrees of freedom anyway
     }
@@ -687,13 +691,13 @@ hp_vertex_dof_identities (const FiniteElement<dim,spacedim> &fe_other) const
   // if the other one is an FE_Nothing. in the first case, there should be
   // exactly one single DoF of each FE at a vertex, and they should have
   // identical value
-  if (dynamic_cast<const FE_Q_Base<PolynomialType,dim,spacedim>*>(&fe_other) != 0)
+  if (dynamic_cast<const FE_Q_Base<PolynomialType,dim,spacedim>*>(&fe_other) != nullptr)
     {
       return
         std::vector<std::pair<unsigned int, unsigned int> >
         (1, std::make_pair (0U, 0U));
     }
-  else if (dynamic_cast<const FE_Nothing<dim>*>(&fe_other) != 0)
+  else if (dynamic_cast<const FE_Nothing<dim>*>(&fe_other) != nullptr)
     {
       // the FE_Nothing has no degrees of freedom, so there are no
       // equivalencies to be recorded
@@ -752,11 +756,11 @@ hp_line_dof_identities (const FiniteElement<dim,spacedim> &fe_other) const
           if (std::fabs(this->unit_support_points[index_map_inverse[i+1]][0]-
                         fe_q_other->unit_support_points[index_map_inverse_other[j+1]][0])
               < 1e-14)
-            identities.push_back (std::make_pair(i,j));
+            identities.emplace_back (i, j);
 
       return identities;
     }
-  else if (dynamic_cast<const FE_Nothing<dim>*>(&fe_other) != 0)
+  else if (dynamic_cast<const FE_Nothing<dim>*>(&fe_other) != nullptr)
     {
       // the FE_Nothing has no degrees of freedom, so there are no
       // equivalencies to be recorded
@@ -818,12 +822,11 @@ hp_quad_dof_identities (const FiniteElement<dim,spacedim>        &fe_other) cons
                   (std::fabs(this->unit_support_points[index_map_inverse[i2+1]][0]-
                              fe_q_other->unit_support_points[index_map_inverse_other[j2+1]][0])
                    < 1e-14))
-                identities.push_back (std::make_pair(i1*(p-1)+i2,
-                                                     j1*(q-1)+j2));
+                identities.emplace_back (i1*(p-1)+i2, j1*(q-1)+j2);
 
       return identities;
     }
-  else if (dynamic_cast<const FE_Nothing<dim>*>(&fe_other) != 0)
+  else if (dynamic_cast<const FE_Nothing<dim>*>(&fe_other) != nullptr)
     {
       // the FE_Nothing has no degrees of freedom, so there are no
       // equivalencies to be recorded
@@ -878,9 +881,9 @@ compare_for_face_domination (const FiniteElement<dim,spacedim> &fe_other) const
           return FiniteElementDomination::no_requirements;
         }
     }
-  else if ((dynamic_cast<const FE_DGQ<dim,spacedim>*>(&fe_other) != 0)
+  else if ((dynamic_cast<const FE_DGQ<dim,spacedim>*>(&fe_other) != nullptr)
            ||
-           (dynamic_cast<const FE_DGP<dim,spacedim>*>(&fe_other) != 0))
+           (dynamic_cast<const FE_DGP<dim,spacedim>*>(&fe_other) != nullptr))
     {
       // there are no requirements between continuous and
       // discontinuous elements
@@ -899,17 +902,24 @@ compare_for_face_domination (const FiniteElement<dim,spacedim> &fe_other) const
 
 
 template <class PolynomialType, int dim, int spacedim>
-void FE_Q_Base<PolynomialType,dim,spacedim>::initialize_unit_support_points
-(const std::vector<Point<1> > &points)
+void
+FE_Q_Base<PolynomialType,dim,spacedim>::
+initialize_unit_support_points (const std::vector<Point<1> > &points)
 {
   const std::vector<unsigned int> &index_map_inverse=
     this->poly_space.get_numbering_inverse();
 
-  Quadrature<1> support_1d(points);
-  Quadrature<dim> support_quadrature(support_1d);
-  this->unit_support_points.resize(support_quadrature.size());
+  // We can compute the support points by computing the tensor
+  // product of the 1d set of points. We could do this by hand, but it's
+  // easier to just re-use functionality that's already been implemented
+  // for quadrature formulas.
+  const Quadrature<1>   support_1d(points);
+  const Quadrature<dim> support_quadrature(support_1d); // NOLINT
 
-  for (unsigned int k=0; k<support_quadrature.size(); k++)
+  // The only thing we have to do is reorder the points from tensor
+  // product order to the order in which we enumerate DoFs on cells
+  this->unit_support_points.resize(support_quadrature.size());
+  for (unsigned int k=0; k<support_quadrature.size(); ++k)
     this->unit_support_points[index_map_inverse[k]] = support_quadrature.point(k);
 }
 
@@ -923,17 +933,23 @@ void FE_Q_Base<PolynomialType,dim,spacedim>::initialize_unit_face_support_points
   if (dim == 1)
     return;
 
-  const unsigned int codim = dim-1;
-  this->unit_face_support_points.resize(Utilities::fixed_power<codim>(q_degree+1));
+  this->unit_face_support_points.resize(Utilities::fixed_power<dim-1>(q_degree+1));
 
   // find renumbering of faces and assign from values of quadrature
-  std::vector<unsigned int> face_index_map =
-    FE_Q_Helper::face_lexicographic_to_hierarchic_numbering<dim>(q_degree);
-  Quadrature<1> support_1d(points);
-  Quadrature<codim> support_quadrature(support_1d);
-  this->unit_face_support_points.resize(support_quadrature.size());
+  const std::vector<unsigned int> face_index_map =
+    internal::FE_Q_Base::face_lexicographic_to_hierarchic_numbering<dim>(q_degree);
 
-  for (unsigned int k=0; k<support_quadrature.size(); k++)
+  // We can compute the support points by computing the tensor
+  // product of the 1d set of points. We could do this by hand, but it's
+  // easier to just re-use functionality that's already been implemented
+  // for quadrature formulas.
+  const Quadrature<1>     support_1d(points);
+  const Quadrature<dim-1> support_quadrature(support_1d); // NOLINT
+
+  // The only thing we have to do is reorder the points from tensor
+  // product order to the order in which we enumerate DoFs on cells
+  this->unit_face_support_points.resize(support_quadrature.size());
+  for (unsigned int k=0; k<support_quadrature.size(); ++k)
     this->unit_face_support_points[face_index_map[k]] = support_quadrature.point(k);
 }
 
@@ -953,9 +969,6 @@ FE_Q_Base<PolynomialType,dim,spacedim>::initialize_quad_dof_index_permutation ()
   const unsigned int n=q_degree-1;
   Assert(n*n==this->dofs_per_quad, ExcInternalError());
 
-  // alias for the table to fill
-  Table<2,int> &data=this->adjust_quad_dof_index_for_face_orientation_table;
-
   // the dofs on a face are connected to a n x n matrix. for example, for
   // degree==4 we have the following dofs on a quad
 
@@ -982,26 +995,26 @@ FE_Q_Base<PolynomialType,dim,spacedim>::initialize_quad_dof_index_permutation ()
                    j=local/n;
 
       // face_orientation=false, face_flip=false, face_rotation=false
-      data(local,0)=j       + i      *n - local;
+      this->adjust_quad_dof_index_for_face_orientation_table(local,0)=j       + i      *n - local;
       // face_orientation=false, face_flip=false, face_rotation=true
-      data(local,1)=i       + (n-1-j)*n - local;
+      this->adjust_quad_dof_index_for_face_orientation_table(local,1)=i       + (n-1-j)*n - local;
       // face_orientation=false, face_flip=true,  face_rotation=false
-      data(local,2)=(n-1-j) + (n-1-i)*n - local;
+      this->adjust_quad_dof_index_for_face_orientation_table(local,2)=(n-1-j) + (n-1-i)*n - local;
       // face_orientation=false, face_flip=true,  face_rotation=true
-      data(local,3)=(n-1-i) + j      *n - local;
+      this->adjust_quad_dof_index_for_face_orientation_table(local,3)=(n-1-i) + j      *n - local;
       // face_orientation=true,  face_flip=false, face_rotation=false
-      data(local,4)=0;
+      this->adjust_quad_dof_index_for_face_orientation_table(local,4)=0;
       // face_orientation=true,  face_flip=false, face_rotation=true
-      data(local,5)=j       + (n-1-i)*n - local;
+      this->adjust_quad_dof_index_for_face_orientation_table(local,5)=j       + (n-1-i)*n - local;
       // face_orientation=true,  face_flip=true,  face_rotation=false
-      data(local,6)=(n-1-i) + (n-1-j)*n - local;
+      this->adjust_quad_dof_index_for_face_orientation_table(local,6)=(n-1-i) + (n-1-j)*n - local;
       // face_orientation=true,  face_flip=true,  face_rotation=true
-      data(local,7)=(n-1-j) + i      *n - local;
+      this->adjust_quad_dof_index_for_face_orientation_table(local,7)=(n-1-j) + i      *n - local;
     }
 
   // additionally initialize reordering of line dofs
   for (unsigned int i=0; i<this->dofs_per_line; ++i)
-    this->adjust_line_dof_index_for_line_orientation_table[i]=this->dofs_per_line-1-i - i;
+    this->adjust_line_dof_index_for_line_orientation_table[i] = this->dofs_per_line-1-i - i;
 }
 
 
@@ -1061,11 +1074,12 @@ face_to_cell_index (const unsigned int face_index,
 
       // we now also need to adjust the line index for the case of
       // face orientation, face flips, etc
-      unsigned int adjusted_dof_index_on_line;
+      unsigned int adjusted_dof_index_on_line = 0;
       switch (dim)
         {
         case 1:
           Assert (false, ExcInternalError());
+          break;
 
         case 2:
           // in 2d, only face_flip has a meaning. if it is set, consider
@@ -1091,6 +1105,9 @@ face_to_cell_index (const unsigned int face_index,
                   ExcNotImplemented());
           adjusted_dof_index_on_line = dof_index_on_line;
           break;
+
+        default:
+          Assert (false, ExcInternalError());
         }
 
       return (this->first_line_index
@@ -1280,11 +1297,11 @@ FE_Q_Base<PolynomialType,dim,spacedim>
       // now expand from 1D info. block innermost dimension (x_0) in order to
       // avoid difficult checks at innermost loop
       unsigned int j_indices[dim];
-      FE_Q_Helper::zero_indices<dim> (j_indices);
+      internal::FE_Q_Base::zero_indices<dim> (j_indices);
       for (unsigned int j=0; j<q_dofs_per_cell; j+=dofs1d)
         {
           unsigned int i_indices[dim];
-          FE_Q_Helper::zero_indices<dim> (i_indices);
+          internal::FE_Q_Base::zero_indices<dim> (i_indices);
           for (unsigned int i=0; i<q_dofs_per_cell; i+=dofs1d)
             {
               double val_extra_dim = 1.;
@@ -1304,10 +1321,10 @@ FE_Q_Base<PolynomialType,dim,spacedim>
 
               // update indices that denote the tensor product position. a bit
               // fuzzy and therefore not done for innermost x_0 direction
-              FE_Q_Helper::increment_indices<dim> (i_indices, dofs1d);
+              internal::FE_Q_Base::increment_indices<dim> (i_indices, dofs1d);
             }
           Assert (i_indices[dim-1] == 1, ExcInternalError());
-          FE_Q_Helper::increment_indices<dim> (j_indices, dofs1d);
+          internal::FE_Q_Base::increment_indices<dim> (j_indices, dofs1d);
         }
 
       // the discontinuous node is simply mapped on the discontinuous node on
@@ -1424,7 +1441,7 @@ FE_Q_Base<PolynomialType,dim,spacedim>
                       this->poly_space.compute_value(index_map_inverse[j], point);
                   }
               unsigned int j_indices[dim];
-              FE_Q_Helper::zero_indices<dim> (j_indices);
+              internal::FE_Q_Base::zero_indices<dim> (j_indices);
               double sum_check = 0;
               for (unsigned int j = 0; j<q_dofs_per_cell; j += dofs1d)
                 {
@@ -1450,7 +1467,7 @@ FE_Q_Base<PolynomialType,dim,spacedim>
                         my_restriction(mother_dof,child_dof)=val;
                       sum_check += val;
                     }
-                  FE_Q_Helper::increment_indices<dim> (j_indices, dofs1d);
+                  internal::FE_Q_Base::increment_indices<dim> (j_indices, dofs1d);
                 }
               Assert (std::fabs(sum_check-1) <
                       std::max(eps, 5e-16*std::sqrt(this->dofs_per_cell)),
@@ -1601,6 +1618,7 @@ FE_Q_Base<PolynomialType,dim,spacedim>::get_constant_modes () const
 }
 
 
+
 // explicit instantiations
 #include "fe_q_base.inst"
 
