@@ -20,6 +20,7 @@
 #include <algorithm>
 
 #include <sstream>
+#include <deal.II/base/std_cxx14/memory.h>
 
 
 DEAL_II_NAMESPACE_OPEN
@@ -70,9 +71,10 @@ std::string FE_RannacherTurek<dim>::get_name() const
 
 
 template <int dim>
-FiniteElement<dim> *FE_RannacherTurek<dim>::clone() const
+std::unique_ptr<FiniteElement<dim,dim> >
+FE_RannacherTurek<dim>::clone() const
 {
-  return new FE_RannacherTurek<dim>(this->order, this->n_face_support_points);
+  return std_cxx14::make_unique<FE_RannacherTurek<dim>>(this->order, this->n_face_support_points);
 }
 
 
@@ -104,35 +106,16 @@ void FE_RannacherTurek<dim>::initialize_support_points()
 template <int dim>
 void
 FE_RannacherTurek<dim>::
-convert_generalized_support_point_values_to_nodal_values(const std::vector<Vector<double> > &support_point_values,
-                                                         std::vector<double> &nodal_dofs) const
+convert_generalized_support_point_values_to_dof_values(const std::vector<Vector<double> > &support_point_values,
+                                                       std::vector<double> &nodal_values) const
 {
   AssertDimension(support_point_values.size(), this->generalized_support_points.size());
-  AssertDimension(nodal_dofs.size(), this->dofs_per_cell);
-
-  // extract component and call scalar version of this function
-  std::vector<double> scalar_values(support_point_values.size());
-  for (unsigned int q = 0; q < support_point_values.size(); ++q)
-    {
-      scalar_values[q] = support_point_values[q][0];
-    }
-  this->interpolate(nodal_dofs, scalar_values);
-}
-
-
-
-template <int dim>
-void FE_RannacherTurek<dim>::interpolate(
-  std::vector<double> &local_dofs,
-  const std::vector<double> &values) const
-{
-  AssertDimension(values.size(), this->generalized_support_points.size());
-  AssertDimension(local_dofs.size(), this->dofs_per_cell);
+  AssertDimension(nodal_values.size(), this->dofs_per_cell);
 
   const unsigned int q_points_per_face = this->weights.size();
-  std::fill(local_dofs.begin(), local_dofs.end(), 0.0);
+  std::fill(nodal_values.begin(), nodal_values.end(), 0.0);
 
-  std::vector<double>::const_iterator value = values.begin();
+  std::vector<Vector<double> >::const_iterator value = support_point_values.begin();
   for (unsigned int face = 0;
        face < dealii::GeometryInfo<dim>::faces_per_cell;
        ++face)
@@ -141,7 +124,7 @@ void FE_RannacherTurek<dim>::interpolate(
            q < q_points_per_face;
            ++q)
         {
-          local_dofs[face] += (*value) * this->weights[q];
+          nodal_values[face] += (*value)[0] * this->weights[q];
           ++value;
         }
     }
@@ -149,46 +132,6 @@ void FE_RannacherTurek<dim>::interpolate(
 
 
 
-template <int dim>
-void FE_RannacherTurek<dim>::interpolate(
-  std::vector<double> &local_dofs,
-  const std::vector<Vector<double> > &values,
-  const unsigned int offset) const
-{
-  AssertDimension(values.size(), this->generalized_support_points.size());
-  AssertDimension(local_dofs.size(), this->dofs_per_cell);
-
-  // extract component at offset and call scalar version of this function
-  std::vector<double> scalar_values(values.size());
-  for (unsigned int q = 0; q < values.size(); ++q)
-    {
-      scalar_values[q] = values[q][offset];
-    }
-  this->interpolate(local_dofs, scalar_values);
-}
-
-
-
-template <int dim>
-void FE_RannacherTurek<dim>::interpolate(
-  std::vector<double> &local_dofs,
-  const VectorSlice<const std::vector<std::vector<double> > > &values) const
-{
-  AssertDimension(values.size(), 1);
-  AssertDimension(values[0].size(), this->generalized_support_points.size());
-  AssertDimension(local_dofs.size(), this->dofs_per_cell);
-
-  // convert data structure to use scalar version of this function
-  std::vector<double> scalar_values(values[0].size());
-  for (unsigned int q = 0; q < values[0].size(); ++q)
-    {
-      scalar_values[q] = values[0][q];
-    }
-  this->interpolate(local_dofs, scalar_values);
-}
-
-
-
 // explicit instantiations
 #include "fe_rannacher_turek.inst"
 
