@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 2003 - 2017 by the deal.II authors
+// Copyright (C) 2003 - 2018 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -30,6 +30,7 @@
 
 #include <sstream>
 #include <iostream>
+#include <deal.II/base/std_cxx14/memory.h>
 
 //TODO: implement the adjust_quad_dof_index_for_face_orientation_table and
 //adjust_line_dof_index_for_line_orientation_table fields, and write tests
@@ -122,10 +123,10 @@ FE_RaviartThomas<dim>::get_name () const
 
 
 template <int dim>
-FiniteElement<dim> *
+std::unique_ptr<FiniteElement<dim,dim> >
 FE_RaviartThomas<dim>::clone() const
 {
-  return new FE_RaviartThomas<dim>(*this);
+  return std_cxx14::make_unique<FE_RaviartThomas<dim>>(*this);
 }
 
 
@@ -192,9 +193,8 @@ FE_RaviartThomas<dim>::initialize_support_points (const unsigned int deg)
 
   if (deg==0) return;
 
-  // Create Legendre basis for the
-  // space D_xi Q_k
-  std::vector<AnisotropicPolynomials<dim>* > polynomials(dim);
+  // Create Legendre basis for the space D_xi Q_k
+  std::unique_ptr<AnisotropicPolynomials<dim>> polynomials[dim];
   for (unsigned int dd=0; dd<dim; ++dd)
     {
       std::vector<std::vector<Polynomials::Polynomial<double> > > poly(dim);
@@ -202,7 +202,7 @@ FE_RaviartThomas<dim>::initialize_support_points (const unsigned int deg)
         poly[d] = Polynomials::Legendre::generate_complete_basis(deg);
       poly[dd] = Polynomials::Legendre::generate_complete_basis(deg-1);
 
-      polynomials[dd] = new AnisotropicPolynomials<dim>(poly);
+      polynomials[dd] = std_cxx14::make_unique<AnisotropicPolynomials<dim>>(poly);
     }
 
   interior_weights.reinit(TableIndices<3>(n_interior_points, polynomials[0]->n(), dim));
@@ -216,9 +216,6 @@ FE_RaviartThomas<dim>::initialize_support_points (const unsigned int deg)
                                     * polynomials[d]->compute_value(i,cell_quadrature.point(k));
     }
 
-  for (unsigned int d=0; d<dim; ++d)
-    delete polynomials[d];
-
   Assert (current == this->generalized_support_points.size(),
           ExcInternalError());
 }
@@ -322,10 +319,9 @@ FE_RaviartThomas<dim>::initialize_restriction()
 
   if (this->degree == 1) return;
 
-  // Create Legendre basis for the
-  // space D_xi Q_k. Here, we cannot
+  // Create Legendre basis for the space D_xi Q_k. Here, we cannot
   // use the shape functions
-  std::vector<AnisotropicPolynomials<dim>* > polynomials(dim);
+  std::unique_ptr<AnisotropicPolynomials<dim>> polynomials[dim];
   for (unsigned int dd=0; dd<dim; ++dd)
     {
       std::vector<std::vector<Polynomials::Polynomial<double> > > poly(dim);
@@ -333,7 +329,7 @@ FE_RaviartThomas<dim>::initialize_restriction()
         poly[d] = Polynomials::Legendre::generate_complete_basis(this->degree-1);
       poly[dd] = Polynomials::Legendre::generate_complete_basis(this->degree-2);
 
-      polynomials[dd] = new AnisotropicPolynomials<dim>(poly);
+      polynomials[dd] = std_cxx14::make_unique<AnisotropicPolynomials<dim>>(poly);
     }
 
   QGauss<dim> q_cell(this->degree);
@@ -365,9 +361,6 @@ FE_RaviartThomas<dim>::initialize_restriction()
                    * polynomials[d]->compute_value(i_weight, q_sub.point(k));
               }
     }
-
-  for (unsigned int d=0; d<dim; ++d)
-    delete polynomials[d];
 }
 
 
@@ -464,8 +457,8 @@ FE_RaviartThomas<dim>::has_support_on_face (
 template <int dim>
 void
 FE_RaviartThomas<dim>::
-convert_generalized_support_point_values_to_nodal_values(const std::vector<Vector<double> > &support_point_values,
-                                                         std::vector<double>    &nodal_values) const
+convert_generalized_support_point_values_to_dof_values(const std::vector<Vector<double> > &support_point_values,
+                                                       std::vector<double>    &nodal_values) const
 {
   Assert (support_point_values.size() == this->generalized_support_points.size(),
           ExcDimensionMismatch(support_point_values.size(), this->generalized_support_points.size()));
@@ -496,88 +489,6 @@ convert_generalized_support_point_values_to_nodal_values(const std::vector<Vecto
 
 
 
-// Since this is a vector valued element, we cannot interpolate a
-// scalar function
-template <int dim>
-void
-FE_RaviartThomas<dim>::interpolate(
-  std::vector<double> &,
-  const std::vector<double> &) const
-{
-  Assert(false, ExcNotImplemented());
-}
-
-
-template <int dim>
-void
-FE_RaviartThomas<dim>::interpolate(
-  std::vector<double>    &local_dofs,
-  const std::vector<Vector<double> > &values,
-  const unsigned int offset) const
-{
-  Assert (values.size() == this->generalized_support_points.size(),
-          ExcDimensionMismatch(values.size(), this->generalized_support_points.size()));
-  Assert (local_dofs.size() == this->dofs_per_cell,
-          ExcDimensionMismatch(local_dofs.size(),this->dofs_per_cell));
-  Assert (values[0].size() >= offset+this->n_components(),
-          ExcDimensionMismatch(values[0].size(),offset+this->n_components()));
-
-  std::fill(local_dofs.begin(), local_dofs.end(), 0.);
-
-  const unsigned int n_face_points = boundary_weights.size(0);
-  for (unsigned int face=0; face<GeometryInfo<dim>::faces_per_cell; ++face)
-    for (unsigned int k=0; k<n_face_points; ++k)
-      for (unsigned int i=0; i<boundary_weights.size(1); ++i)
-        {
-          local_dofs[i+face*this->dofs_per_face] += boundary_weights(k,i)
-                                                    * values[face*n_face_points+k](GeometryInfo<dim>::unit_normal_direction[face]+offset);
-        }
-
-  const unsigned int start_cell_dofs = GeometryInfo<dim>::faces_per_cell*this->dofs_per_face;
-  const unsigned int start_cell_points = GeometryInfo<dim>::faces_per_cell*n_face_points;
-
-  for (unsigned int k=0; k<interior_weights.size(0); ++k)
-    for (unsigned int i=0; i<interior_weights.size(1); ++i)
-      for (unsigned int d=0; d<dim; ++d)
-        local_dofs[start_cell_dofs+i*dim+d] += interior_weights(k,i,d) * values[k+start_cell_points](d+offset);
-}
-
-
-template <int dim>
-void
-FE_RaviartThomas<dim>::interpolate(
-  std::vector<double> &local_dofs,
-  const VectorSlice<const std::vector<std::vector<double> > > &values) const
-{
-  Assert (values.size() == this->n_components(),
-          ExcDimensionMismatch(values.size(), this->n_components()));
-  Assert (values[0].size() == this->generalized_support_points.size(),
-          ExcDimensionMismatch(values[0].size(), this->generalized_support_points.size()));
-  Assert (local_dofs.size() == this->dofs_per_cell,
-          ExcDimensionMismatch(local_dofs.size(),this->dofs_per_cell));
-
-  std::fill(local_dofs.begin(), local_dofs.end(), 0.);
-
-  const unsigned int n_face_points = boundary_weights.size(0);
-  for (unsigned int face=0; face<GeometryInfo<dim>::faces_per_cell; ++face)
-    for (unsigned int k=0; k<n_face_points; ++k)
-      for (unsigned int i=0; i<boundary_weights.size(1); ++i)
-        {
-          local_dofs[i+face*this->dofs_per_face] += boundary_weights(k,i)
-                                                    * values[GeometryInfo<dim>::unit_normal_direction[face]][face*n_face_points+k];
-        }
-
-  const unsigned int start_cell_dofs = GeometryInfo<dim>::faces_per_cell*this->dofs_per_face;
-  const unsigned int start_cell_points = GeometryInfo<dim>::faces_per_cell*n_face_points;
-
-  for (unsigned int k=0; k<interior_weights.size(0); ++k)
-    for (unsigned int i=0; i<interior_weights.size(1); ++i)
-      for (unsigned int d=0; d<dim; ++d)
-        local_dofs[start_cell_dofs+i*dim+d] += interior_weights(k,i,d) * values[d][k+start_cell_points];
-}
-
-
-
 template <int dim>
 std::size_t
 FE_RaviartThomas<dim>::memory_consumption () const
