@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 2000 - 2016 by the deal.II authors
+// Copyright (C) 2000 - 2017 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -26,6 +26,7 @@
 #include <sstream>
 #include <fstream>
 #include <iostream>
+#include <deal.II/base/std_cxx14/memory.h>
 
 DEAL_II_NAMESPACE_OPEN
 
@@ -64,10 +65,10 @@ FE_TraceQ<dim,spacedim>::FE_TraceQ (const unsigned int degree)
 
 
 template <int dim, int spacedim>
-FiniteElement<dim,spacedim> *
+std::unique_ptr<FiniteElement<dim,spacedim> >
 FE_TraceQ<dim,spacedim>::clone() const
 {
-  return new FE_TraceQ<dim,spacedim>(this->degree);
+  return std_cxx14::make_unique<FE_TraceQ<dim,spacedim>>(this->degree);
 }
 
 
@@ -121,6 +122,26 @@ FE_TraceQ<dim,spacedim>::get_constant_modes () const
          (constant_modes, std::vector<unsigned int>(1, 0));
 }
 
+template <int dim, int spacedim>
+void
+FE_TraceQ<dim,spacedim>::
+convert_generalized_support_point_values_to_dof_values (const std::vector<Vector<double> > &support_point_values,
+                                                        std::vector<double>                &nodal_values) const
+{
+  AssertDimension (support_point_values.size(),
+                   this->get_unit_support_points().size());
+  AssertDimension (support_point_values.size(),
+                   nodal_values.size());
+  AssertDimension (this->dofs_per_cell,
+                   nodal_values.size());
+
+  for (unsigned int i=0; i<this->dofs_per_cell; ++i)
+    {
+      AssertDimension (support_point_values[i].size(), 1);
+
+      nodal_values[i] = support_point_values[i](0);
+    }
+}
 
 
 template <int dim, int spacedim>
@@ -217,7 +238,7 @@ get_subface_interpolation_matrix (const FiniteElement<dim,spacedim> &x_source_fe
     {
       fe_q.get_subface_interpolation_matrix (source_fe->fe_q, subface, interpolation_matrix);
     }
-  else if (dynamic_cast<const FE_Nothing<dim> *>(&x_source_fe) != 0)
+  else if (dynamic_cast<const FE_Nothing<dim> *>(&x_source_fe) != nullptr)
     {
       // nothing to do here, the FE_Nothing has no degrees of freedom anyway
     }
