@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 2001 - 2015 by the deal.II authors
+// Copyright (C) 2001 - 2017 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -21,16 +21,11 @@ DEAL_II_NAMESPACE_OPEN
 
 
 template <int dim, int spacedim>
-Mapping<dim, spacedim>::~Mapping ()
-{}
-
-
-template <int dim, int spacedim>
-std_cxx11::array<Point<spacedim>, GeometryInfo<dim>::vertices_per_cell>
+std::array<Point<spacedim>, GeometryInfo<dim>::vertices_per_cell>
 Mapping<dim, spacedim>::get_vertices (
   const typename Triangulation<dim,spacedim>::cell_iterator &cell) const
 {
-  std_cxx11::array<Point<spacedim>, GeometryInfo<dim>::vertices_per_cell> vertices;
+  std::array<Point<spacedim>, GeometryInfo<dim>::vertices_per_cell> vertices;
   for (unsigned int i = 0; i < GeometryInfo<dim>::vertices_per_cell; ++i)
     {
       vertices[i] = cell->vertex(i);
@@ -39,7 +34,7 @@ Mapping<dim, spacedim>::get_vertices (
 }
 
 
-template<int dim, int spacedim>
+template <int dim, int spacedim>
 Point<dim-1>
 Mapping<dim,spacedim>::
 project_real_point_to_unit_point_on_face (
@@ -86,12 +81,6 @@ Mapping<dim, spacedim>::InternalDataBase::InternalDataBase ():
 
 
 
-template <int dim, int spacedim>
-Mapping<dim, spacedim>::InternalDataBase::~InternalDataBase ()
-{}
-
-
-
 template <int dim, int spacedim>
 std::size_t
 Mapping<dim, spacedim>::InternalDataBase::memory_consumption () const
