@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 2001 - 2015 by the deal.II authors
+// Copyright (C) 2001 - 2018 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -13,10 +13,14 @@
 //
 // ---------------------------------------------------------------------
 
+#include <deal.II/base/std_cxx14/memory.h>
+
 #include <deal.II/grid/tria_accessor.h>
 #include <deal.II/grid/tria_iterator.h>
 #include <deal.II/grid/tria_boundary.h>
+
 #include <deal.II/fe/mapping_c1.h>
+
 #include <cmath>
 
 DEAL_II_NAMESPACE_OPEN
@@ -43,7 +47,7 @@ MappingC1<dim,spacedim>::MappingC1 ()
   //
   // we only need to replace the Qp mapping because that's the one that's
   // used on boundary cells where it matters
-  this->qp_mapping.reset (new MappingC1<dim,spacedim>::MappingC1Generic());
+  this->qp_mapping = std::make_shared<MappingC1<dim,spacedim>::MappingC1Generic>();
 }
 
 
@@ -66,7 +70,8 @@ MappingC1<2>::MappingC1Generic::add_line_support_points (const Triangulation<2>:
                                                          std::vector<Point<2> > &a) const
 {
   const unsigned int dim = 2;
-  std::vector<Point<dim> > line_points (2);
+  const std::array<double, 2> interior_gl_points
+  {{0.5 - 0.5*std::sqrt(1.0/5.0), 0.5 + 0.5*std::sqrt(1.0/5.0)}};
 
   // loop over each of the lines, and if it is at the boundary, then first get
   // the boundary description and second compute the points on it. if not at
@@ -79,11 +84,10 @@ MappingC1<2>::MappingC1Generic::add_line_support_points (const Triangulation<2>:
         {
           // first get the normal vectors at the two vertices of this line
           // from the boundary description
-          const Boundary<dim> &boundary
-            = line->get_triangulation().get_boundary(line->boundary_id());
+          const Manifold<dim> &manifold = line->get_manifold();
 
-          Boundary<dim>::FaceVertexNormals face_vertex_normals;
-          boundary.get_normals_at_vertices (line, face_vertex_normals);
+          Manifold<dim>::FaceVertexNormals face_vertex_normals;
+          manifold.get_normals_at_vertices (line, face_vertex_normals);
 
           // then transform them into interpolation points for a cubic
           // polynomial
@@ -124,9 +128,8 @@ MappingC1<2>::MappingC1Generic::add_line_support_points (const Triangulation<2>:
                              -face_vertex_normals[1][0] * std::sin(alpha)))
                            -2*c;
 
-          QGaussLobatto<1> quad_points(4);
-          const double t1 = quad_points.point(1)[0];
-          const double t2 = quad_points.point(2)[0];
+          const double t1 = interior_gl_points[0];
+          const double t2 = interior_gl_points[1];
           const double s_t1 = (((-b-c)*t1+b)*t1+c)*t1;
           const double s_t2 = (((-b-c)*t2+b)*t2+c)*t2;
 
@@ -149,18 +152,22 @@ MappingC1<2>::MappingC1Generic::add_line_support_points (const Triangulation<2>:
             };
         }
       else
-        // not at boundary
+        // not at boundary, so just use scaled Gauss-Lobatto points (i.e., use
+        // plain straight lines).
         {
-          static const StraightBoundary<dim> straight_boundary;
-          straight_boundary.get_intermediate_points_on_line (line, line_points);
-          a.insert (a.end(), line_points.begin(), line_points.end());
+          // Note that the zeroth Gauss-Lobatto point is a boundary point, so
+          // we push back mapped versions of the first and second.
+          a.push_back((1.0 - interior_gl_points[0])*line->vertex(0) +
+                      (interior_gl_points[0]*line->vertex(1)));
+          a.push_back((1.0 - interior_gl_points[1])*line->vertex(0) +
+                      (interior_gl_points[1]*line->vertex(1)));
         }
     }
 }
 
 
 
-template<int dim, int spacedim>
+template <int dim, int spacedim>
 void
 MappingC1<dim,spacedim>::MappingC1Generic::add_line_support_points (const typename Triangulation<dim>::cell_iterator &,
     std::vector<Point<dim> > &) const
@@ -194,7 +201,7 @@ MappingC1<2>::MappingC1Generic::add_quad_support_points (const Triangulation<2>:
 
 
 
-template<int dim, int spacedim>
+template <int dim, int spacedim>
 void
 MappingC1<dim,spacedim>::MappingC1Generic::add_quad_support_points (const typename Triangulation<dim>::cell_iterator &,
     std::vector<Point<dim> > &) const
@@ -204,11 +211,11 @@ MappingC1<dim,spacedim>::MappingC1Generic::add_quad_support_points (const typena
 
 
 
-template<int dim, int spacedim>
-Mapping<dim, spacedim> *
+template <int dim, int spacedim>
+std::unique_ptr<Mapping<dim,spacedim> >
 MappingC1<dim,spacedim>::clone () const
 {
-  return new MappingC1<dim,spacedim>();
+  return std_cxx14::make_unique<MappingC1<dim,spacedim>>();
 }
 
 
