@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 2001 - 2016 by the deal.II authors
+// Copyright (C) 2001 - 2018 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -13,7 +13,8 @@
 //
 // ---------------------------------------------------------------------
 
-
+#include <deal.II/base/std_cxx14/memory.h>
+#include <deal.II/base/array_view.h>
 #include <deal.II/base/tensor.h>
 #include <deal.II/base/quadrature.h>
 #include <deal.II/base/qprojector.h>
@@ -33,12 +34,12 @@
 DEAL_II_NAMESPACE_OPEN
 
 
-template<int dim, int spacedim>
+template <int dim, int spacedim>
 const unsigned int MappingCartesian<dim,spacedim>::invalid_face_number;
 
 
 
-template<int dim, int spacedim>
+template <int dim, int spacedim>
 MappingCartesian<dim, spacedim>::InternalData::InternalData (const Quadrature<dim> &q)
   :
   cell_extents (numbers::signaling_nan<Tensor<1,dim> >()),
@@ -48,7 +49,7 @@ MappingCartesian<dim, spacedim>::InternalData::InternalData (const Quadrature<di
 
 
 
-template<int dim, int spacedim>
+template <int dim, int spacedim>
 std::size_t
 MappingCartesian<dim, spacedim>::InternalData::memory_consumption () const
 {
@@ -69,7 +70,7 @@ MappingCartesian<dim,spacedim>::preserves_vertex_locations () const
 
 
 
-template<int dim, int spacedim>
+template <int dim, int spacedim>
 UpdateFlags
 MappingCartesian<dim, spacedim>::requires_update_flags (const UpdateFlags in) const
 {
@@ -87,30 +88,29 @@ MappingCartesian<dim, spacedim>::requires_update_flags (const UpdateFlags in) co
 
 
 
-template<int dim, int spacedim>
-typename Mapping<dim, spacedim>::InternalDataBase *
+template <int dim, int spacedim>
+std::unique_ptr<typename Mapping<dim, spacedim>::InternalDataBase>
 MappingCartesian<dim, spacedim>::get_data (const UpdateFlags      update_flags,
                                            const Quadrature<dim> &q) const
 {
-  InternalData *data = new InternalData (q);
+  auto data = std_cxx14::make_unique<InternalData> (q);
 
   // store the flags in the internal data object so we can access them
   // in fill_fe_*_values(). use the transitive hull of the required
   // flags
   data->update_each = requires_update_flags(update_flags);
 
-  return data;
+  return std::move(data);
 }
 
 
 
-template<int dim, int spacedim>
-typename Mapping<dim, spacedim>::InternalDataBase *
+template <int dim, int spacedim>
+std::unique_ptr<typename Mapping<dim, spacedim>::InternalDataBase>
 MappingCartesian<dim, spacedim>::get_face_data (const UpdateFlags update_flags,
                                                 const Quadrature<dim-1>& quadrature) const
 {
-  InternalData *data
-    = new InternalData (QProjector<dim>::project_to_all_faces(quadrature));
+  auto data = std_cxx14::make_unique<InternalData> (QProjector<dim>::project_to_all_faces(quadrature));
 
   // verify that we have computed the transitive hull of the required
   // flags and that FEValues has faithfully passed them on to us
@@ -121,18 +121,17 @@ MappingCartesian<dim, spacedim>::get_face_data (const UpdateFlags update_flags,
   // in fill_fe_*_values()
   data->update_each = update_flags;
 
-  return data;
+  return std::move(data);
 }
 
 
 
-template<int dim, int spacedim>
-typename Mapping<dim, spacedim>::InternalDataBase *
+template <int dim, int spacedim>
+std::unique_ptr<typename Mapping<dim, spacedim>::InternalDataBase>
 MappingCartesian<dim, spacedim>::get_subface_data (const UpdateFlags update_flags,
                                                    const Quadrature<dim-1> &quadrature) const
 {
-  InternalData *data
-    = new InternalData (QProjector<dim>::project_to_all_subfaces(quadrature));
+  auto data = std_cxx14::make_unique<InternalData> (QProjector<dim>::project_to_all_subfaces(quadrature));
 
   // verify that we have computed the transitive hull of the required
   // flags and that FEValues has faithfully passed them on to us
@@ -143,13 +142,13 @@ MappingCartesian<dim, spacedim>::get_subface_data (const UpdateFlags update_flag
   // in fill_fe_*_values()
   data->update_each = update_flags;
 
-  return data;
+  return std::move(data);
 }
 
 
 
 
-template<int dim, int spacedim>
+template <int dim, int spacedim>
 void
 MappingCartesian<dim, spacedim>::compute_fill (const typename Triangulation<dim,spacedim>::cell_iterator &cell,
                                                const unsigned int        face_no,
@@ -333,18 +332,18 @@ MappingCartesian<dim, spacedim>::compute_fill (const typename Triangulation<dim,
 
 
 
-template<int dim, int spacedim>
+template <int dim, int spacedim>
 CellSimilarity::Similarity
 MappingCartesian<dim, spacedim>::
 fill_fe_values (const typename Triangulation<dim,spacedim>::cell_iterator &cell,
                 const CellSimilarity::Similarity                           cell_similarity,
                 const Quadrature<dim>                                     &quadrature,
                 const typename Mapping<dim,spacedim>::InternalDataBase    &internal_data,
-                internal::FEValues::MappingRelatedData<dim,spacedim>      &output_data) const
+                internal::FEValuesImplementation::MappingRelatedData<dim,spacedim>      &output_data) const
 {
   // convert data object to internal data for this class. fails with
   // an exception if that is not possible
-  Assert (dynamic_cast<const InternalData *> (&internal_data) != 0, ExcInternalError());
+  Assert (dynamic_cast<const InternalData *> (&internal_data) != nullptr, ExcInternalError());
   const InternalData &data = static_cast<const InternalData &> (internal_data);
 
   std::vector<Tensor<1,dim> > dummy;
@@ -419,7 +418,7 @@ fill_fe_values (const typename Triangulation<dim,spacedim>::cell_iterator &cell,
         {
           output_data.inverse_jacobians[i] = Tensor<2,dim>();
           for (unsigned int j=0; j<dim; ++j)
-            output_data.inverse_jacobians[j][j]=1./data.cell_extents[j];
+            output_data.inverse_jacobians[i][j][j]=1./data.cell_extents[j];
         }
 
   return cell_similarity;
@@ -427,20 +426,20 @@ fill_fe_values (const typename Triangulation<dim,spacedim>::cell_iterator &cell,
 
 
 
-template<int dim, int spacedim>
+template <int dim, int spacedim>
 void
 MappingCartesian<dim, spacedim>::
 fill_fe_face_values (const typename Triangulation<dim,spacedim>::cell_iterator &cell,
                      const unsigned int                                         face_no,
                      const Quadrature<dim-1>                                   &quadrature,
                      const typename Mapping<dim,spacedim>::InternalDataBase    &internal_data,
-                     internal::FEValues::MappingRelatedData<dim,spacedim>      &output_data) const
+                     internal::FEValuesImplementation::MappingRelatedData<dim,spacedim>      &output_data) const
 {
   // convert data object to internal
   // data for this class. fails with
   // an exception if that is not
   // possible
-  Assert (dynamic_cast<const InternalData *> (&internal_data) != 0,
+  Assert (dynamic_cast<const InternalData *> (&internal_data) != nullptr,
           ExcInternalError());
   const InternalData &data = static_cast<const InternalData &> (internal_data);
 
@@ -516,7 +515,7 @@ fill_fe_face_values (const typename Triangulation<dim,spacedim>::cell_iterator &
 
 
 
-template<int dim, int spacedim>
+template <int dim, int spacedim>
 void
 MappingCartesian<dim, spacedim>::
 fill_fe_subface_values (const typename Triangulation<dim,spacedim>::cell_iterator &cell,
@@ -524,11 +523,11 @@ fill_fe_subface_values (const typename Triangulation<dim,spacedim>::cell_iterato
                         const unsigned int                                         subface_no,
                         const Quadrature<dim-1>                                   &quadrature,
                         const typename Mapping<dim,spacedim>::InternalDataBase    &internal_data,
-                        internal::FEValues::MappingRelatedData<dim,spacedim>      &output_data) const
+                        internal::FEValuesImplementation::MappingRelatedData<dim,spacedim>      &output_data) const
 {
   // convert data object to internal data for this class. fails with
   // an exception if that is not possible
-  Assert (dynamic_cast<const InternalData *> (&internal_data) != 0, ExcInternalError());
+  Assert (dynamic_cast<const InternalData *> (&internal_data) != nullptr, ExcInternalError());
   const InternalData &data = static_cast<const InternalData &> (internal_data);
 
   compute_fill (cell, face_no, subface_no, CellSimilarity::none,
@@ -613,7 +612,7 @@ fill_fe_subface_values (const typename Triangulation<dim,spacedim>::cell_iterato
 
 
 
-template<int dim, int spacedim>
+template <int dim, int spacedim>
 void
 MappingCartesian<dim,spacedim>::
 transform (const ArrayView<const Tensor<1,dim> >                  &input,
@@ -622,7 +621,7 @@ transform (const ArrayView<const Tensor<1,dim> >                  &input,
            const ArrayView<Tensor<1,spacedim> >                   &output) const
 {
   AssertDimension (input.size(), output.size());
-  Assert (dynamic_cast<const InternalData *>(&mapping_data) != 0,
+  Assert (dynamic_cast<const InternalData *>(&mapping_data) != nullptr,
           ExcInternalError());
   const InternalData &data = static_cast<const InternalData &> (mapping_data);
 
@@ -668,7 +667,7 @@ transform (const ArrayView<const Tensor<1,dim> >                  &input,
 
 
 
-template<int dim, int spacedim>
+template <int dim, int spacedim>
 void
 MappingCartesian<dim,spacedim>::
 transform (const ArrayView<const DerivativeForm<1, dim,spacedim> > &input,
@@ -677,7 +676,7 @@ transform (const ArrayView<const DerivativeForm<1, dim,spacedim> > &input,
            const ArrayView<Tensor<2,spacedim> >                    &output) const
 {
   AssertDimension (input.size(), output.size());
-  Assert (dynamic_cast<const InternalData *>(&mapping_data) != 0,
+  Assert (dynamic_cast<const InternalData *>(&mapping_data) != nullptr,
           ExcInternalError());
   const InternalData &data = static_cast<const InternalData &>(mapping_data);
 
@@ -769,7 +768,7 @@ transform (const ArrayView<const DerivativeForm<1, dim,spacedim> > &input,
 
 
 
-template<int dim, int spacedim>
+template <int dim, int spacedim>
 void
 MappingCartesian<dim,spacedim>::
 transform (const ArrayView<const Tensor<2, dim> >                 &input,
@@ -779,7 +778,7 @@ transform (const ArrayView<const Tensor<2, dim> >                 &input,
 {
 
   AssertDimension (input.size(), output.size());
-  Assert (dynamic_cast<const InternalData *>(&mapping_data) != 0,
+  Assert (dynamic_cast<const InternalData *>(&mapping_data) != nullptr,
           ExcInternalError());
   const InternalData &data = static_cast<const InternalData &>(mapping_data);
 
@@ -870,7 +869,7 @@ transform (const ArrayView<const Tensor<2, dim> >                 &input,
 }
 
 
-template<int dim, int spacedim>
+template <int dim, int spacedim>
 void
 MappingCartesian<dim,spacedim>::
 transform (const ArrayView<const  DerivativeForm<2, dim, spacedim> > &input,
@@ -880,7 +879,7 @@ transform (const ArrayView<const  DerivativeForm<2, dim, spacedim> > &input,
 {
 
   AssertDimension (input.size(), output.size());
-  Assert (dynamic_cast<const InternalData *>(&mapping_data) != 0,
+  Assert (dynamic_cast<const InternalData *>(&mapping_data) != nullptr,
           ExcInternalError());
   const InternalData &data = static_cast<const InternalData &>(mapping_data);
 
@@ -908,7 +907,7 @@ transform (const ArrayView<const  DerivativeForm<2, dim, spacedim> > &input,
 
 }
 
-template<int dim, int spacedim>
+template <int dim, int spacedim>
 void
 MappingCartesian<dim,spacedim>::
 transform (const ArrayView<const  Tensor<3,dim> >                 &input,
@@ -918,7 +917,7 @@ transform (const ArrayView<const  Tensor<3,dim> >                 &input,
 {
 
   AssertDimension (input.size(), output.size());
-  Assert (dynamic_cast<const InternalData *>(&mapping_data) != 0,
+  Assert (dynamic_cast<const InternalData *>(&mapping_data) != nullptr,
           ExcInternalError());
   const InternalData &data = static_cast<const InternalData &>(mapping_data);
 
@@ -1028,7 +1027,7 @@ MappingCartesian<dim, spacedim>::transform_unit_to_real_cell (
 
 
 
-template<int dim, int spacedim>
+template <int dim, int spacedim>
 Point<dim>
 MappingCartesian<dim, spacedim>::transform_real_to_unit_cell (
   const typename Triangulation<dim,spacedim>::cell_iterator &cell,
@@ -1062,11 +1061,11 @@ MappingCartesian<dim, spacedim>::transform_real_to_unit_cell (
 }
 
 
-template<int dim, int spacedim>
-Mapping<dim, spacedim> *
+template <int dim, int spacedim>
+std::unique_ptr<Mapping<dim, spacedim> >
 MappingCartesian<dim, spacedim>::clone () const
 {
-  return new MappingCartesian<dim, spacedim>(*this);
+  return std_cxx14::make_unique<MappingCartesian<dim,spacedim>>(*this);
 }
 
 
