@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 2001 - 2016 by the deal.II authors
+// Copyright (C) 2001 - 2018 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -13,13 +13,14 @@
 //
 // ---------------------------------------------------------------------
 
-#include <deal.II/base/utilities.h>
+#include <deal.II/base/array_view.h>
 #include <deal.II/base/polynomial.h>
 #include <deal.II/base/quadrature.h>
 #include <deal.II/base/quadrature_lib.h>
 #include <deal.II/base/memory_consumption.h>
+#include <deal.II/base/std_cxx14/memory.h>
 #include <deal.II/base/tensor_product_polynomials.h>
-#include <deal.II/base/std_cxx11/unique_ptr.h>
+#include <deal.II/base/utilities.h>
 #include <deal.II/lac/full_matrix.h>
 #include <deal.II/grid/tria_iterator.h>
 #include <deal.II/dofs/dof_accessor.h>
@@ -28,11 +29,12 @@
 #include <deal.II/fe/fe_values.h>
 
 #include <numeric>
+#include <memory>
 
 DEAL_II_NAMESPACE_OPEN
 
 
-template<int dim, int spacedim>
+template <int dim, int spacedim>
 MappingQ<dim,spacedim>::InternalData::InternalData ()
   :
   use_mapping_q1_on_current_cell(false)
@@ -40,7 +42,7 @@ MappingQ<dim,spacedim>::InternalData::InternalData ()
 
 
 
-template<int dim, int spacedim>
+template <int dim, int spacedim>
 std::size_t
 MappingQ<dim,spacedim>::InternalData::memory_consumption () const
 {
@@ -52,7 +54,7 @@ MappingQ<dim,spacedim>::InternalData::memory_consumption () const
 
 
 
-template<int dim, int spacedim>
+template <int dim, int spacedim>
 MappingQ<dim,spacedim>::MappingQ (const unsigned int degree,
                                   const bool use_mapping_q_on_all_cells)
   :
@@ -70,39 +72,48 @@ MappingQ<dim,spacedim>::MappingQ (const unsigned int degree,
                               (dim != spacedim)),
   // create a Q1 mapping for use on interior cells (if necessary)
   // or to create a good initial guess in transform_real_to_unit_cell()
-  q1_mapping (new MappingQGeneric<dim,spacedim>(1)),
+  q1_mapping (std::make_shared<MappingQGeneric<dim,spacedim> >(1)),
 
   // create a Q_p mapping; if p=1, simply share the Q_1 mapping already
   // created via the shared_ptr objects
   qp_mapping (this->polynomial_degree>1
               ?
-              std_cxx11::shared_ptr<const MappingQGeneric<dim,spacedim> >(new MappingQGeneric<dim,spacedim>(degree))
+              std::make_shared<const MappingQGeneric<dim,spacedim> >(degree)
               :
               q1_mapping)
 {}
 
 
 
-template<int dim, int spacedim>
+template <int dim, int spacedim>
 MappingQ<dim,spacedim>::MappingQ (const MappingQ<dim,spacedim> &mapping)
   :
   polynomial_degree (mapping.polynomial_degree),
-  use_mapping_q_on_all_cells (mapping.use_mapping_q_on_all_cells),
-  // clone the Q1 mapping for use on interior cells (if necessary)
-  // or to create a good initial guess in transform_real_to_unit_cell()
-  q1_mapping (dynamic_cast<MappingQGeneric<dim,spacedim>*>(mapping.q1_mapping->clone())),
-  // create a Q_p mapping; if p=1, simply share the Q_1 mapping already
-  // created via the shared_ptr objects
-  qp_mapping (this->polynomial_degree>1
-              ?
-              std_cxx11::shared_ptr<const MappingQGeneric<dim,spacedim> >(dynamic_cast<MappingQGeneric<dim,spacedim>*>(mapping.qp_mapping->clone()))
-              :
-              q1_mapping)
-{}
+  use_mapping_q_on_all_cells (mapping.use_mapping_q_on_all_cells)
+{
+  // Note that we really do have to use clone() here, since mapping.q1_mapping
+  // may be MappingQ1Eulerian and mapping.qp_mapping may be MappingQEulerian.
+  std::shared_ptr<const Mapping<dim,spacedim>> other_q1_map = mapping.q1_mapping->clone();
+  q1_mapping = std::dynamic_pointer_cast<const MappingQGeneric<dim,spacedim>>(other_q1_map);
+  Assert(q1_mapping != nullptr, ExcInternalError());
+  Assert(q1_mapping->get_degree() == 1, ExcInternalError());
+
+  // Same as the other constructor: if possible reuse the Q1 mapping
+  if (this->polynomial_degree == 1)
+    {
+      qp_mapping = q1_mapping;
+    }
+  else
+    {
+      std::shared_ptr<const Mapping<dim,spacedim>> other_qp_map = mapping.qp_mapping->clone();
+      qp_mapping = std::dynamic_pointer_cast<const MappingQGeneric<dim,spacedim>>(other_qp_map);
+      Assert(qp_mapping != nullptr, ExcInternalError());
+    }
+}
 
 
 
-template<int dim, int spacedim>
+template <int dim, int spacedim>
 unsigned int
 MappingQ<dim,spacedim>::get_degree() const
 {
@@ -121,7 +132,7 @@ MappingQ<dim,spacedim>::preserves_vertex_locations () const
 
 
 
-template<int dim, int spacedim>
+template <int dim, int spacedim>
 UpdateFlags
 MappingQ<dim,spacedim>::requires_update_flags (const UpdateFlags in) const
 {
@@ -132,91 +143,97 @@ MappingQ<dim,spacedim>::requires_update_flags (const UpdateFlags in) const
 
 
 
-template<int dim, int spacedim>
-typename MappingQ<dim,spacedim>::InternalData *
+template <int dim, int spacedim>
+std::unique_ptr<typename Mapping<dim,spacedim>::InternalDataBase>
 MappingQ<dim,spacedim>::get_data (const UpdateFlags update_flags,
                                   const Quadrature<dim> &quadrature) const
 {
-  InternalData *data = new InternalData;
+  auto data = std_cxx14::make_unique<InternalData>();
 
   // build the Q1 and Qp internal data objects in parallel
-  Threads::Task<typename MappingQGeneric<dim,spacedim>::InternalData *>
-  do_get_data = Threads::new_task (&MappingQGeneric<dim,spacedim>::get_data,
-                                   *qp_mapping,
-                                   update_flags,
-                                   quadrature);
+  Threads::Task<std::unique_ptr<typename Mapping<dim,spacedim>::InternalDataBase>>
+      do_get_data = Threads::new_task (&MappingQGeneric<dim,spacedim>::get_data,
+                                       *qp_mapping,
+                                       update_flags,
+                                       quadrature);
 
   if (!use_mapping_q_on_all_cells)
-    data->mapping_q1_data.reset (q1_mapping->get_data (update_flags, quadrature));
+    data->mapping_q1_data = Utilities::dynamic_unique_cast<typename MappingQGeneric<dim,spacedim>::InternalData>
+                            (std::move(q1_mapping->get_data (update_flags, quadrature)));
 
   // wait for the task above to finish and use returned value
-  data->mapping_qp_data.reset (do_get_data.return_value());
-  return data;
+  data->mapping_qp_data = Utilities::dynamic_unique_cast<typename MappingQGeneric<dim,spacedim>::InternalData>
+                          (std::move(do_get_data.return_value()));
+  return std::move(data);
 }
 
 
 
-template<int dim, int spacedim>
-typename MappingQ<dim,spacedim>::InternalData *
+template <int dim, int spacedim>
+std::unique_ptr<typename Mapping<dim,spacedim>::InternalDataBase>
 MappingQ<dim,spacedim>::get_face_data (const UpdateFlags update_flags,
                                        const Quadrature<dim-1>& quadrature) const
 {
-  InternalData *data = new InternalData;
+  auto data = std_cxx14::make_unique<InternalData>();
 
   // build the Q1 and Qp internal data objects in parallel
-  Threads::Task<typename MappingQGeneric<dim,spacedim>::InternalData *>
-  do_get_data = Threads::new_task (&MappingQGeneric<dim,spacedim>::get_face_data,
-                                   *qp_mapping,
-                                   update_flags,
-                                   quadrature);
+  Threads::Task<std::unique_ptr<typename Mapping<dim,spacedim>::InternalDataBase>>
+      do_get_data = Threads::new_task (&MappingQGeneric<dim,spacedim>::get_face_data,
+                                       *qp_mapping,
+                                       update_flags,
+                                       quadrature);
 
   if (!use_mapping_q_on_all_cells)
-    data->mapping_q1_data.reset (q1_mapping->get_face_data (update_flags, quadrature));
+    data->mapping_q1_data = Utilities::dynamic_unique_cast<typename MappingQGeneric<dim,spacedim>::InternalData>
+                            (std::move(q1_mapping->get_face_data (update_flags, quadrature)));
 
   // wait for the task above to finish and use returned value
-  data->mapping_qp_data.reset (do_get_data.return_value());
-  return data;
+  data->mapping_qp_data = Utilities::dynamic_unique_cast<typename MappingQGeneric<dim,spacedim>::InternalData>
+                          (std::move(do_get_data.return_value()));
+  return std::move(data);
 }
 
 
 
-template<int dim, int spacedim>
-typename MappingQ<dim,spacedim>::InternalData *
+template <int dim, int spacedim>
+std::unique_ptr<typename Mapping<dim,spacedim>::InternalDataBase>
 MappingQ<dim,spacedim>::get_subface_data (const UpdateFlags update_flags,
                                           const Quadrature<dim-1>& quadrature) const
 {
-  InternalData *data = new InternalData;
+  auto data = std_cxx14::make_unique<InternalData>();
 
   // build the Q1 and Qp internal data objects in parallel
-  Threads::Task<typename MappingQGeneric<dim,spacedim>::InternalData *>
-  do_get_data = Threads::new_task (&MappingQGeneric<dim,spacedim>::get_subface_data,
-                                   *qp_mapping,
-                                   update_flags,
-                                   quadrature);
+  Threads::Task<std::unique_ptr<typename Mapping<dim,spacedim>::InternalDataBase>>
+      do_get_data = Threads::new_task (&MappingQGeneric<dim,spacedim>::get_subface_data,
+                                       *qp_mapping,
+                                       update_flags,
+                                       quadrature);
 
   if (!use_mapping_q_on_all_cells)
-    data->mapping_q1_data.reset (q1_mapping->get_subface_data (update_flags, quadrature));
+    data->mapping_q1_data = Utilities::dynamic_unique_cast<typename MappingQGeneric<dim,spacedim>::InternalData>
+                            (std::move(q1_mapping->get_subface_data (update_flags, quadrature)));
 
   // wait for the task above to finish and use returned value
-  data->mapping_qp_data.reset (do_get_data.return_value());
-  return data;
+  data->mapping_qp_data = Utilities::dynamic_unique_cast<typename MappingQGeneric<dim,spacedim>::InternalData>
+                          (std::move(do_get_data.return_value()));
+  return std::move(data);
 }
 
 
 // Note that the CellSimilarity flag is modifiable, since MappingQ can need to
 // recalculate data even when cells are similar.
-template<int dim, int spacedim>
+template <int dim, int spacedim>
 CellSimilarity::Similarity
 MappingQ<dim,spacedim>::
 fill_fe_values (const typename Triangulation<dim,spacedim>::cell_iterator &cell,
                 const CellSimilarity::Similarity                           cell_similarity,
                 const Quadrature<dim>                                     &quadrature,
                 const typename Mapping<dim,spacedim>::InternalDataBase    &internal_data,
-                internal::FEValues::MappingRelatedData<dim,spacedim>      &output_data) const
+                internal::FEValuesImplementation::MappingRelatedData<dim,spacedim>      &output_data) const
 {
   // convert data object to internal data for this class. fails with an
   // exception if that is not possible
-  Assert (dynamic_cast<const InternalData *> (&internal_data) != 0, ExcInternalError());
+  Assert (dynamic_cast<const InternalData *> (&internal_data) != nullptr, ExcInternalError());
   const InternalData &data = static_cast<const InternalData &> (internal_data);
 
   // check whether this cell needs the full mapping or can be treated by a
@@ -260,18 +277,18 @@ fill_fe_values (const typename Triangulation<dim,spacedim>::cell_iterator &cell,
 
 
 
-template<int dim, int spacedim>
+template <int dim, int spacedim>
 void
 MappingQ<dim,spacedim>::
 fill_fe_face_values (const typename Triangulation<dim,spacedim>::cell_iterator &cell,
                      const unsigned int                                         face_no,
                      const Quadrature<dim-1>                                   &quadrature,
                      const typename Mapping<dim,spacedim>::InternalDataBase    &internal_data,
-                     internal::FEValues::MappingRelatedData<dim,spacedim>      &output_data) const
+                     internal::FEValuesImplementation::MappingRelatedData<dim,spacedim>      &output_data) const
 {
   // convert data object to internal data for this class. fails with an
   // exception if that is not possible
-  Assert (dynamic_cast<const InternalData *> (&internal_data) != 0,
+  Assert (dynamic_cast<const InternalData *> (&internal_data) != nullptr,
           ExcInternalError());
   const InternalData &data = static_cast<const InternalData &> (internal_data);
 
@@ -301,7 +318,7 @@ fill_fe_face_values (const typename Triangulation<dim,spacedim>::cell_iterator &
 }
 
 
-template<int dim, int spacedim>
+template <int dim, int spacedim>
 void
 MappingQ<dim,spacedim>::
 fill_fe_subface_values (const typename Triangulation<dim,spacedim>::cell_iterator &cell,
@@ -309,11 +326,11 @@ fill_fe_subface_values (const typename Triangulation<dim,spacedim>::cell_iterato
                         const unsigned int                                         subface_no,
                         const Quadrature<dim-1>                                   &quadrature,
                         const typename Mapping<dim,spacedim>::InternalDataBase    &internal_data,
-                        internal::FEValues::MappingRelatedData<dim,spacedim>      &output_data) const
+                        internal::FEValuesImplementation::MappingRelatedData<dim,spacedim>      &output_data) const
 {
   // convert data object to internal data for this class. fails with an
   // exception if that is not possible
-  Assert (dynamic_cast<const InternalData *> (&internal_data) != 0,
+  Assert (dynamic_cast<const InternalData *> (&internal_data) != nullptr,
           ExcInternalError());
   const InternalData &data = static_cast<const InternalData &> (internal_data);
 
@@ -346,7 +363,7 @@ fill_fe_subface_values (const typename Triangulation<dim,spacedim>::cell_iterato
 
 
 
-template<int dim, int spacedim>
+template <int dim, int spacedim>
 void
 MappingQ<dim,spacedim>::
 transform (const ArrayView<const Tensor<1,dim> >                  &input,
@@ -355,10 +372,9 @@ transform (const ArrayView<const Tensor<1,dim> >                  &input,
            const ArrayView<Tensor<1,spacedim> >                   &output) const
 {
   AssertDimension (input.size(), output.size());
-  Assert ((dynamic_cast<const typename MappingQ<dim,spacedim>::InternalData *> (&mapping_data)
-           != 0),
-          ExcInternalError());
+
   const InternalData *data = dynamic_cast<const InternalData *>(&mapping_data);
+  Assert (data != nullptr, ExcInternalError());
 
   // check whether we should in fact work on the Q1 portion of it
   if (data->use_mapping_q1_on_current_cell)
@@ -370,17 +386,17 @@ transform (const ArrayView<const Tensor<1,dim> >                  &input,
 
 
 
-template<int dim, int spacedim>
+template <int dim, int spacedim>
 void
 MappingQ<dim,spacedim>::
-transform (const ArrayView<const DerivativeForm<1, dim ,spacedim> >  &input,
+transform (const ArrayView<const DerivativeForm<1, dim, spacedim> >  &input,
            const MappingType                                          mapping_type,
            const typename Mapping<dim,spacedim>::InternalDataBase    &mapping_data,
            const ArrayView<Tensor<2,spacedim> >                      &output) const
 {
   AssertDimension (input.size(), output.size());
   Assert ((dynamic_cast<const typename MappingQ<dim,spacedim>::InternalData *> (&mapping_data)
-           != 0),
+           != nullptr),
           ExcInternalError());
   const InternalData *data = dynamic_cast<const InternalData *>(&mapping_data);
 
@@ -394,7 +410,7 @@ transform (const ArrayView<const DerivativeForm<1, dim ,spacedim> >  &input,
 
 
 
-template<int dim, int spacedim>
+template <int dim, int spacedim>
 void
 MappingQ<dim,spacedim>::
 transform (const ArrayView<const Tensor<2, dim> >                 &input,
@@ -404,7 +420,7 @@ transform (const ArrayView<const Tensor<2, dim> >                 &input,
 {
   AssertDimension (input.size(), output.size());
   Assert ((dynamic_cast<const typename MappingQ<dim,spacedim>::InternalData *> (&mapping_data)
-           != 0),
+           != nullptr),
           ExcInternalError());
   const InternalData *data = dynamic_cast<const InternalData *>(&mapping_data);
 
@@ -418,17 +434,17 @@ transform (const ArrayView<const Tensor<2, dim> >                 &input,
 
 
 
-template<int dim, int spacedim>
+template <int dim, int spacedim>
 void
 MappingQ<dim,spacedim>::
-transform (const ArrayView<const DerivativeForm<2, dim ,spacedim> >  &input,
+transform (const ArrayView<const DerivativeForm<2, dim,spacedim> >  &input,
            const MappingType                                          mapping_type,
            const typename Mapping<dim,spacedim>::InternalDataBase    &mapping_data,
            const ArrayView<Tensor<3,spacedim> >                      &output) const
 {
   AssertDimension (input.size(), output.size());
   Assert ((dynamic_cast<const typename MappingQ<dim,spacedim>::InternalData *> (&mapping_data)
-           != 0),
+           != nullptr),
           ExcInternalError());
   const InternalData *data = dynamic_cast<const InternalData *>(&mapping_data);
 
@@ -442,7 +458,7 @@ transform (const ArrayView<const DerivativeForm<2, dim ,spacedim> >  &input,
 
 
 
-template<int dim, int spacedim>
+template <int dim, int spacedim>
 void MappingQ<dim,spacedim>::
 transform (const ArrayView<const Tensor<3, dim> >                 &input,
            const MappingType                                       mapping_type,
@@ -451,7 +467,7 @@ transform (const ArrayView<const Tensor<3, dim> >                 &input,
 {
   AssertDimension (input.size(), output.size());
   Assert ((dynamic_cast<const typename MappingQ<dim,spacedim>::InternalData *> (&mapping_data)
-           != 0),
+           != nullptr),
           ExcInternalError());
   const InternalData *data = dynamic_cast<const InternalData *>(&mapping_data);
 
@@ -465,7 +481,7 @@ transform (const ArrayView<const Tensor<3, dim> >                 &input,
 
 
 
-template<int dim, int spacedim>
+template <int dim, int spacedim>
 Point<spacedim>
 MappingQ<dim,spacedim>::
 transform_unit_to_real_cell (const typename Triangulation<dim,spacedim>::cell_iterator &cell,
@@ -482,7 +498,7 @@ transform_unit_to_real_cell (const typename Triangulation<dim,spacedim>::cell_it
 
 
 
-template<int dim, int spacedim>
+template <int dim, int spacedim>
 Point<dim>
 MappingQ<dim,spacedim>::
 transform_real_to_unit_cell (const typename Triangulation<dim,spacedim>::cell_iterator &cell,
@@ -500,12 +516,12 @@ transform_real_to_unit_cell (const typename Triangulation<dim,spacedim>::cell_it
 
 
 
-template<int dim, int spacedim>
-Mapping<dim,spacedim> *
+template <int dim, int spacedim>
+std::unique_ptr<Mapping<dim,spacedim> >
 MappingQ<dim,spacedim>::clone () const
 {
-  return new MappingQ<dim,spacedim>(this->polynomial_degree,
-                                    this->use_mapping_q_on_all_cells);
+  return std_cxx14::make_unique<MappingQ<dim,spacedim>>(this->polynomial_degree,
+                                                        this->use_mapping_q_on_all_cells);
 }
 
 
