@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 2001 - 2016 by the deal.II authors
+// Copyright (C) 2001 - 2018 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -15,15 +15,16 @@
 
 #include <deal.II/base/utilities.h>
 #include <deal.II/base/quadrature_lib.h>
+#include <deal.II/base/std_cxx14/memory.h>
 #include <deal.II/lac/vector.h>
 #include <deal.II/lac/block_vector.h>
 #include <deal.II/lac/la_vector.h>
 #include <deal.II/lac/parallel_vector.h>
 #include <deal.II/lac/parallel_block_vector.h>
-#include <deal.II/lac/petsc_vector.h>
-#include <deal.II/lac/petsc_block_vector.h>
+#include <deal.II/lac/petsc_parallel_vector.h>
+#include <deal.II/lac/petsc_parallel_block_vector.h>
 #include <deal.II/lac/trilinos_vector.h>
-#include <deal.II/lac/trilinos_block_vector.h>
+#include <deal.II/lac/trilinos_parallel_block_vector.h>
 #include <deal.II/lac/trilinos_parallel_block_vector.h>
 #include <deal.II/grid/tria_iterator.h>
 #include <deal.II/dofs/dof_handler.h>
@@ -52,57 +53,38 @@ MappingQEulerianGeneric (const unsigned int                                 degr
             update_values | update_quadrature_points)
 {}
 
-template <int dim, class VectorType, int spacedim>
-MappingQEulerian<dim, VectorType, spacedim>::
-MappingQEulerian (const unsigned int              degree,
-                  const VectorType               &euler_vector,
-                  const DoFHandler<dim,spacedim> &euler_dof_handler)
-  :
-  MappingQ<dim,spacedim>(degree, true),
-  euler_vector(&euler_vector),
-  euler_dof_handler(&euler_dof_handler)
-{
-  // reset the q1 mapping we use for interior cells (and previously
-  // set by the MappingQ constructor) to a MappingQ1Eulerian with the
-  // current vector
-  this->q1_mapping.reset (new MappingQ1Eulerian<dim,VectorType,spacedim>(euler_vector,
-                          euler_dof_handler));
-
-  // also reset the qp mapping pointer with our own class
-  this->qp_mapping.reset (new MappingQEulerianGeneric(degree,*this));
-}
-
 
 
 template <int dim, class VectorType, int spacedim>
 MappingQEulerian<dim, VectorType, spacedim>::
 MappingQEulerian (const unsigned int              degree,
                   const DoFHandler<dim,spacedim> &euler_dof_handler,
-                  const VectorType               &euler_vector)
+                  const VectorType               &euler_vector,
+                  const unsigned int              level)
   :
   MappingQ<dim,spacedim>(degree, true),
   euler_vector(&euler_vector),
-  euler_dof_handler(&euler_dof_handler)
+  euler_dof_handler(&euler_dof_handler),
+  level(level)
 {
   // reset the q1 mapping we use for interior cells (and previously
   // set by the MappingQ constructor) to a MappingQ1Eulerian with the
   // current vector
-  this->q1_mapping.reset (new MappingQ1Eulerian<dim,VectorType,spacedim>(euler_vector,
-                          euler_dof_handler));
+  this->q1_mapping = std::make_shared<MappingQ1Eulerian<dim,VectorType,spacedim>>
+                     (euler_dof_handler, euler_vector);
 
   // also reset the qp mapping pointer with our own class
-  this->qp_mapping.reset (new MappingQEulerianGeneric(degree,*this));
+  this->qp_mapping = std::make_shared<MappingQEulerianGeneric>(degree,*this);
 }
 
 
 
 template <int dim, class VectorType, int spacedim>
-Mapping<dim,spacedim> *
+std::unique_ptr<Mapping<dim,spacedim> >
 MappingQEulerian<dim, VectorType, spacedim>::clone () const
 {
-  return new MappingQEulerian<dim,VectorType,spacedim>(this->get_degree(),
-                                                       *euler_vector,
-                                                       *euler_dof_handler);
+  return std_cxx14::make_unique<MappingQEulerian<dim,VectorType,spacedim>>
+         (this->get_degree(), *euler_dof_handler, *euler_vector);
 }
 
 
@@ -143,7 +125,7 @@ SupportQuadrature (const unsigned int map_degree)
 // .... COMPUTE MAPPING SUPPORT POINTS
 
 template <int dim, class VectorType, int spacedim>
-std_cxx11::array<Point<spacedim>, GeometryInfo<dim>::vertices_per_cell>
+std::array<Point<spacedim>, GeometryInfo<dim>::vertices_per_cell>
 MappingQEulerian<dim, VectorType, spacedim>::
 get_vertices
 (const typename Triangulation<dim,spacedim>::cell_iterator &cell) const
@@ -152,7 +134,7 @@ get_vertices
   const std::vector<Point<spacedim> > a
     = dynamic_cast<const MappingQEulerianGeneric &>(*this->qp_mapping).compute_mapping_support_points(cell);
 
-  std_cxx11::array<Point<spacedim>, GeometryInfo<dim>::vertices_per_cell> vertex_locations;
+  std::array<Point<spacedim>, GeometryInfo<dim>::vertices_per_cell> vertex_locations;
   std::copy (a.begin(),
              a.begin()+GeometryInfo<dim>::vertices_per_cell,
              vertex_locations.begin());
@@ -163,7 +145,7 @@ get_vertices
 
 
 template <int dim, class VectorType, int spacedim>
-std_cxx11::array<Point<spacedim>, GeometryInfo<dim>::vertices_per_cell>
+std::array<Point<spacedim>, GeometryInfo<dim>::vertices_per_cell>
 MappingQEulerian<dim, VectorType, spacedim>::MappingQEulerianGeneric::
 get_vertices
 (const typename Triangulation<dim,spacedim>::cell_iterator &cell) const
@@ -173,16 +155,28 @@ get_vertices
 
 
 
+template <int dim, class VectorType, int spacedim>
+bool
+MappingQEulerian<dim, VectorType, spacedim>::MappingQEulerianGeneric::
+preserves_vertex_locations () const
+{
+  return false;
+}
+
+
 
 template <int dim, class VectorType, int spacedim>
 std::vector<Point<spacedim> >
 MappingQEulerian<dim, VectorType, spacedim>::MappingQEulerianGeneric::
 compute_mapping_support_points (const typename Triangulation<dim,spacedim>::cell_iterator &cell) const
 {
-  // first, basic assertion with respect to vector size,
+  const bool mg_vector = mapping_q_eulerian.level != numbers::invalid_unsigned_int;
 
-  const types::global_dof_index n_dofs  = mapping_q_eulerian.euler_dof_handler->n_dofs();
+  const types::global_dof_index n_dofs  = mg_vector ?
+                                          mapping_q_eulerian.euler_dof_handler->n_dofs(mapping_q_eulerian.level) :
+                                          mapping_q_eulerian.euler_dof_handler->n_dofs();
   const types::global_dof_index vector_size = mapping_q_eulerian.euler_vector->size();
+
   (void)n_dofs;
   (void)vector_size;
 
@@ -193,22 +187,21 @@ compute_mapping_support_points (const typename Triangulation<dim,spacedim>::cell
   typename DoFHandler<dim,spacedim>::cell_iterator dof_cell(*cell,
                                                             mapping_q_eulerian.euler_dof_handler);
 
-  Assert (dof_cell->active() == true, ExcInactiveCell());
+  Assert (mg_vector || dof_cell->active() == true, ExcInactiveCell());
 
   // our quadrature rule is chosen so that each quadrature point corresponds
-  // to a support point in the undeformed configuration.  we can then query
+  // to a support point in the undeformed configuration. We can then query
   // the given displacement field at these points to determine the shift
   // vector that maps the support points to the deformed configuration.
 
-  // we assume that the given field contains dim displacement components, but
+  // We assume that the given field contains dim displacement components, but
   // that there may be other solution components as well (e.g. pressures).
   // this class therefore assumes that the first dim components represent the
   // actual shift vector we need, and simply ignores any components after
-  // that.  this implies that the user should order components appropriately,
+  // that. This implies that the user should order components appropriately,
   // or create a separate dof handler for the displacements.
-
   const unsigned int n_support_pts = support_quadrature.size();
-  const unsigned int n_components  = mapping_q_eulerian.euler_dof_handler->get_fe().n_components();
+  const unsigned int n_components  = mapping_q_eulerian.euler_dof_handler->get_fe(0).n_components();
 
   Assert (n_components >= spacedim, ExcDimensionMismatch(n_components, spacedim) );
 
@@ -216,12 +209,21 @@ compute_mapping_support_points (const typename Triangulation<dim,spacedim>::cell
   shift_vector(n_support_pts,
                Vector<typename VectorType::value_type>(n_components));
 
+  std::vector<types::global_dof_index> dof_indices(mapping_q_eulerian.euler_dof_handler->get_fe(0).dofs_per_cell);
   // fill shift vector for each support point using an fe_values object. make
   // sure that the fe_values variable isn't used simultaneously from different
   // threads
   Threads::Mutex::ScopedLock lock(fe_values_mutex);
   fe_values.reinit(dof_cell);
-  fe_values.get_function_values(*mapping_q_eulerian.euler_vector, shift_vector);
+  if (mg_vector)
+    {
+      dof_cell->get_mg_dof_indices(dof_indices);
+      fe_values.get_function_values(*mapping_q_eulerian.euler_vector,
+                                    dof_indices,
+                                    shift_vector);
+    }
+  else
+    fe_values.get_function_values(*mapping_q_eulerian.euler_vector, shift_vector);
 
   // and finally compute the positions of the support points in the deformed
   // configuration.
@@ -238,14 +240,14 @@ compute_mapping_support_points (const typename Triangulation<dim,spacedim>::cell
 
 
 
-template<int dim, class VectorType, int spacedim>
+template <int dim, class VectorType, int spacedim>
 CellSimilarity::Similarity
 MappingQEulerian<dim,VectorType,spacedim>::
 fill_fe_values (const typename Triangulation<dim,spacedim>::cell_iterator &cell,
-                const CellSimilarity::Similarity                           ,
+                const CellSimilarity::Similarity,
                 const Quadrature<dim>                                     &quadrature,
                 const typename Mapping<dim,spacedim>::InternalDataBase    &internal_data,
-                internal::FEValues::MappingRelatedData<dim,spacedim>      &output_data) const
+                internal::FEValuesImplementation::MappingRelatedData<dim,spacedim>      &output_data) const
 {
   // call the function of the base class, but ignoring
   // any potentially detected cell similarity between
