@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 2000 - 2016 by the deal.II authors
+// Copyright (C) 2000 - 2017 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -17,7 +17,6 @@
 #include <deal.II/grid/grid_reordering.h>
 #include <deal.II/grid/grid_tools.h>
 #include <deal.II/base/utilities.h>
-#include <deal.II/base/std_cxx11/bind.h>
 #include <deal.II/base/timer.h>
 
 #include <algorithm>
@@ -231,7 +230,7 @@ namespace
      */
     const_iterator begin () const
     {
-      return &adjacent_cells[0];
+      return adjacent_cells;
     }
 
 
@@ -246,11 +245,11 @@ namespace
       // adjacent cells, and use this to point to the element past the
       // last valid one
       if (adjacent_cells[0].cell_index == numbers::invalid_unsigned_int)
-        return &adjacent_cells[0];
+        return adjacent_cells;
       else if (adjacent_cells[1].cell_index == numbers::invalid_unsigned_int)
-        return &adjacent_cells[0]+1;
+        return adjacent_cells + 1;
       else
-        return &adjacent_cells[0]+2;
+        return adjacent_cells + 2;
     }
 
   private:
@@ -290,17 +289,6 @@ namespace
   class Edge
   {
   public:
-    /**
-     * Default constructor. Creates an invalid edge.
-     */
-    Edge ()
-      :
-      orientation_status (not_oriented)
-    {
-      for (unsigned int i=0; i<2; ++i)
-        vertex_indices[i] = numbers::invalid_unsigned_int;
-    }
-
     /**
      * Constructor. Create the edge based on the information given
      * in @p cell, and selecting the edge with number @p edge_number
@@ -378,17 +366,6 @@ namespace
   template <int dim>
   struct Cell
   {
-    /**
-     * Default construct a cell.
-     */
-    Cell ()
-    {
-      for (unsigned int i=0; i<GeometryInfo<dim>::vertices_per_cell; ++i)
-        vertex_indices[i] = numbers::invalid_unsigned_int;
-      for (unsigned int i=0; i<GeometryInfo<dim>::lines_per_cell; ++i)
-        edge_indices[i] = numbers::invalid_unsigned_int;
-    }
-
     /**
      * Construct a Cell object from a CellData object. Also take a
      * (sorted) list of edges and to point the edges of the current
@@ -487,7 +464,7 @@ namespace
      */
     const_iterator begin () const
     {
-      return &edge_indices[0];
+      return edge_indices;
     }
 
 
@@ -500,11 +477,11 @@ namespace
       // indices, and use this to point to the element past the
       // last valid one
       if (edge_indices[0] == numbers::invalid_unsigned_int)
-        return &edge_indices[0];
+        return edge_indices;
       else if (edge_indices[1] == numbers::invalid_unsigned_int)
-        return &edge_indices[0]+1;
+        return edge_indices + 1;
       else
-        return &edge_indices[0]+2;
+        return edge_indices + 2;
     }
 
   private:
@@ -550,7 +527,7 @@ namespace
     edge_list.reserve(cells.size()*GeometryInfo<dim>::lines_per_cell);
     for (unsigned int i=0; i<cells.size(); ++i)
       for (unsigned int l=0; l<GeometryInfo<dim>::lines_per_cell; ++l)
-        edge_list.push_back (Edge<dim>(cells[i], l));
+        edge_list.emplace_back (cells[i], l);
 
     // next sort the edge list and then remove duplicates
     std::sort (edge_list.begin(), edge_list.end());
@@ -577,7 +554,7 @@ namespace
       {
         // create our own data structure for the cells and let it
         // connect to the edges array
-        cell_list.push_back (Cell<dim>(cells[i], edges));
+        cell_list.emplace_back (cells[i], edges);
 
         // then also inform the edges that they are adjacent
         // to the current cell, and where within this cell
@@ -848,8 +825,8 @@ namespace
         for (origin_vertex_of_cell=0;
              origin_vertex_of_cell<GeometryInfo<dim>::vertices_per_cell;
              ++origin_vertex_of_cell)
-          if (std::count (&starting_vertex_of_edge[0],
-                          &starting_vertex_of_edge[0]+GeometryInfo<dim>::lines_per_cell,
+          if (std::count (starting_vertex_of_edge,
+                          starting_vertex_of_edge + GeometryInfo<dim>::lines_per_cell,
                           cell_list[cell_index].vertex_indices[origin_vertex_of_cell])
               == dim)
             break;
@@ -1101,7 +1078,7 @@ GridReordering<dim,spacedim>::reorder_cells (std::vector<CellData<dim> > &cells,
 
 
 
-template<>
+template <>
 void
 GridReordering<1>::invert_all_cells_of_negative_grid(const std::vector<Point<1> > &,
                                                      std::vector<CellData<1> > &)
@@ -1111,7 +1088,7 @@ GridReordering<1>::invert_all_cells_of_negative_grid(const std::vector<Point<1>
 
 
 
-template<>
+template <>
 void
 GridReordering<1,2>::invert_all_cells_of_negative_grid(const std::vector<Point<2> > &,
                                                        std::vector<CellData<1> > &)
@@ -1121,7 +1098,7 @@ GridReordering<1,2>::invert_all_cells_of_negative_grid(const std::vector<Point<2
 
 
 
-template<>
+template <>
 void
 GridReordering<1,3>::invert_all_cells_of_negative_grid(const std::vector<Point<3> > &,
                                                        std::vector<CellData<1> > &)
@@ -1130,7 +1107,7 @@ GridReordering<1,3>::invert_all_cells_of_negative_grid(const std::vector<Point<3
 }
 
 
-template<>
+template <>
 void
 GridReordering<2>::invert_all_cells_of_negative_grid(const std::vector<Point<2> > &all_vertices,
                                                      std::vector<CellData<2> >    &cells)
@@ -1186,7 +1163,7 @@ GridReordering<2>::invert_all_cells_of_negative_grid(const std::vector<Point<2>
 
 
 
-template<>
+template <>
 void
 GridReordering<2,3>::invert_all_cells_of_negative_grid(const std::vector<Point<3> > &,
                                                        std::vector<CellData<2> > &)
@@ -1196,7 +1173,7 @@ GridReordering<2,3>::invert_all_cells_of_negative_grid(const std::vector<Point<3
 
 
 
-template<>
+template <>
 void
 GridReordering<3>::invert_all_cells_of_negative_grid(const std::vector<Point<3> > &all_vertices,
                                                      std::vector<CellData<3> > &cells)
@@ -1270,4 +1247,3 @@ template class GridReordering<2,3>;
 template class GridReordering<3,3>;
 
 DEAL_II_NAMESPACE_CLOSE
-
