@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 1998 - 2017 by the deal.II authors
+// Copyright (C) 1998 - 2018 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -15,48 +15,26 @@
 
 #include <deal.II/base/tensor.h>
 #include <deal.II/base/table.h>
+#include <deal.II/base/std_cxx14/memory.h>
 #include <deal.II/grid/manifold.h>
 #include <deal.II/grid/tria.h>
 #include <deal.II/grid/tria_iterator.h>
 #include <deal.II/grid/tria_accessor.h>
 #include <deal.II/fe/fe_q.h>
+
 #include <cmath>
 
+#include <boost/container/small_vector.hpp>
+
 DEAL_II_NAMESPACE_OPEN
 
 using namespace Manifolds;
 
-// This structure is used as comparison function for std::sort when sorting
-// points according to their weight.
-struct CompareWeights
-{
-public:
-  CompareWeights(const std::vector<double> &weights)
-    :
-    compare_weights(weights)
-  {}
-
-  bool operator() (unsigned int a, unsigned int b) const
-  {
-    return compare_weights[a] < compare_weights[b];
-  }
-
-private:
-  const std::vector<double> &compare_weights;
-};
-
 /* -------------------------- Manifold --------------------- */
-
-template <int dim, int spacedim>
-Manifold<dim, spacedim>::~Manifold ()
-{}
-
-
-
 template <int dim, int spacedim>
 Point<spacedim>
 Manifold<dim, spacedim>::
-project_to_manifold (const std::vector<Point<spacedim> > &,
+project_to_manifold (const ArrayView<const Point<spacedim>> &,
                      const Point<spacedim> &) const
 {
   Assert (false, ExcPureFunctionCalled());
@@ -72,10 +50,9 @@ get_intermediate_point (const Point<spacedim> &p1,
                         const Point<spacedim> &p2,
                         const double w) const
 {
-  std::vector<Point<spacedim> > vertices;
-  vertices.push_back(p1);
-  vertices.push_back(p2);
-  return project_to_manifold(vertices, w * p2 + (1-w)*p1 );
+  const std::array<Point<spacedim>, 2> vertices {{p1, p2}};
+  return project_to_manifold(make_array_view(vertices.begin(), vertices.end()),
+                             w * p2 + (1-w)*p1);
 }
 
 
@@ -83,18 +60,8 @@ get_intermediate_point (const Point<spacedim> &p1,
 template <int dim, int spacedim>
 Point<spacedim>
 Manifold<dim, spacedim>::
-get_new_point (const Quadrature<spacedim> &quad) const
-{
-  return get_new_point(quad.get_points(),quad.get_weights());
-}
-
-
-
-template <int dim, int spacedim>
-Point<spacedim>
-Manifold<dim, spacedim>::
-get_new_point (const std::vector<Point<spacedim> > &surrounding_points,
-               const std::vector<double>           &weights) const
+get_new_point (const ArrayView<const Point<spacedim>> &surrounding_points,
+               const ArrayView<const double>          &weights) const
 {
   const double tol = 1e-10;
   const unsigned int n_points = surrounding_points.size();
@@ -111,23 +78,13 @@ get_new_point (const std::vector<Point<spacedim> > &surrounding_points,
   // First sort points in the order of their weights. This is done to
   // produce unique points even if get_intermediate_points is not
   // associative (as for the SphericalManifold).
-  unsigned int permutation_short[30];
-  std::vector<unsigned int> permutation_long;
-  unsigned int *permutation;
-  if (n_points > 30)
-    {
-      permutation_long.resize(n_points);
-      permutation = &permutation_long[0];
-    }
-  else
-    permutation = &permutation_short[0];
-
-  for (unsigned int i=0; i<n_points; ++i)
-    permutation[i] = i;
-
-  std::sort(permutation,
-            permutation + n_points,
-            CompareWeights(weights));
+  boost::container::small_vector<unsigned int, 100> permutation(n_points);
+  std::iota(permutation.begin(), permutation.end(), 0u);
+  std::sort(permutation.begin(), permutation.end(),
+            [&weights](const std::size_t a, const std::size_t b)
+  {
+    return weights[a] < weights[b];
+  });
 
   // Now loop over points in the order of their associated weight
   Point<spacedim> p = surrounding_points[permutation[0]];
@@ -154,22 +111,18 @@ get_new_point (const std::vector<Point<spacedim> > &surrounding_points,
 template <int dim, int spacedim>
 void
 Manifold<dim, spacedim>::
-add_new_points (const std::vector<Point<spacedim> > &surrounding_points,
-                const Table<2,double>               &weights,
-                std::vector<Point<spacedim> >       &new_points) const
+get_new_points (const ArrayView<const Point<spacedim>> &surrounding_points,
+                const Table<2,double>                  &weights,
+                ArrayView<Point<spacedim>>              new_points) const
 {
   AssertDimension(surrounding_points.size(), weights.size(1));
-  Assert(&surrounding_points != &new_points,
-         ExcMessage("surrounding_points and new_points cannot be the same "
-                    "array"));
 
-  const unsigned int n_points = surrounding_points.size();
-  std::vector<double> local_weights(n_points);
   for (unsigned int row=0; row<weights.size(0); ++row)
     {
-      for (unsigned int i=0; i<n_points; ++i)
-        local_weights[i] = weights(row,i);
-      new_points.push_back(get_new_point(surrounding_points, local_weights));
+      new_points[row] = get_new_point(make_array_view(surrounding_points.begin(),
+                                                      surrounding_points.end()),
+                                      make_array_view(weights,
+                                                      row));
     }
 }
 
@@ -197,94 +150,72 @@ normal_vector (const Triangulation<2, 2>::face_iterator &face,
 
 
 
-template<>
+template <>
 Tensor<1,3>
 Manifold<3, 3>::
 normal_vector (const Triangulation<3, 3>::face_iterator &face,
                const Point<3> &p) const
 {
   const int spacedim=3;
-  Tensor<1,spacedim> t1,t2;
 
-  // Take the difference between p and all four vertices
-  unsigned int min_index=0;
-  double       min_distance = (p-face->vertex(0)).norm_square();
+  const std::array<Point<spacedim>, 4> vertices
+  {{face->vertex(0), face->vertex(1), face->vertex(2), face->vertex(3)}};
+  const std::array<double, 4> distances
+  {{vertices[0].distance(p), vertices[1].distance(p), vertices[2].distance(p), vertices[3].distance(p)}};
+  const double max_distance = std::max(std::max(distances[0], distances[1]),
+                                       std::max(distances[2], distances[3]));
 
-  for (unsigned int i=1; i<4; ++i)
-    {
-      const Tensor<1,spacedim> dp = p-face->vertex(i);
-      double distance = dp.norm_square();
-      if (distance < min_distance)
-        {
-          min_index = i;
-          min_distance = distance;
-        }
-    }
-  // Verify we have a valid vertex index
-  AssertIndexRange(min_index, 4);
+  // We need to find two tangential vectors to the given point p, but we do
+  // not know how the point is oriented against the face. We guess the two
+  // directions by assuming a flat topology and take the two directions that
+  // indicate the angle closest to a perpendicular one (i.e., cos(theta) close
+  // to zero). We start with an invalid value but the loops should always find
+  // a value.
+  double abs_cos_angle = std::numeric_limits<double>::max();
+  unsigned int first_index = numbers::invalid_unsigned_int,
+               second_index = numbers::invalid_unsigned_int;
+  for (unsigned int i=0; i<3; ++i)
+    if (distances[i] > 1e-8*max_distance)
+      for (unsigned int j=i+1; j<4; ++j)
+        if (distances[j] > 1e-8*max_distance)
+          {
+            const double new_angle = (p-vertices[i]) * (p-vertices[j]) /
+                                     (distances[i]*distances[j]);
+            // multiply by factor 0.999 to bias the search in a way that
+            // avoids trouble with roundoff
+            if (std::abs(new_angle) < 0.999 * abs_cos_angle)
+              {
+                abs_cos_angle = std::abs(new_angle);
+                first_index = i;
+                second_index = j;
+              }
+          }
+  Assert(first_index != numbers::invalid_unsigned_int,
+         ExcMessage("The search for possible directions did not succeed."));
 
-  // Now figure out which vertices are best to compute tangent vectors.
-  // We split the cell in a central diamond of points closer to the
-  // center than to any of the vertices, and the 4 triangles in the
-  // corner. The central diamond is split into its upper and lower
-  // half. For each of these 6 cases, the following encodes a list
-  // of two vertices each to which we compute the tangent vectors,
-  // and then take the cross product. See the documentation of this
-  // function for exact details.
-  if ((p-face->center()).norm_square() < min_distance)
-    {
-      // we are close to the face center: pick two consecutive vertices,
-      // but not the closest one. We make sure the direction is always
-      // the same.
-      if (min_index < 2)
-        {
-          t1 = get_tangent_vector(p, face->vertex(3));
-          t2 = get_tangent_vector(p, face->vertex(2));
-        }
-      else
-        {
-          t1 = get_tangent_vector(p, face->vertex(0));
-          t2 = get_tangent_vector(p, face->vertex(1));
-        }
-    }
-  else
-    {
-      // we are closer to one of the vertices than to the
-      // center of the face
-      switch (min_index)
-        {
-        case 0:
-        {
-          t1 = get_tangent_vector(p, face->vertex(1));
-          t2 = get_tangent_vector(p, face->vertex(2));
-          break;
-        }
-        case 1:
-        {
-          t1 = get_tangent_vector(p, face->vertex(3));
-          t2 = get_tangent_vector(p, face->vertex(0));
-          break;
-        }
-        case 2:
-        {
-          t1 = get_tangent_vector(p, face->vertex(0));
-          t2 = get_tangent_vector(p, face->vertex(3));
-          break;
-        }
-        case 3:
-        {
-          t1 = get_tangent_vector(p, face->vertex(2));
-          t2 = get_tangent_vector(p, face->vertex(1));
-          break;
-        }
-        default:
-          Assert(false, ExcInternalError());
-          break;
-        }
-    }
+  // Compute tangents and normal for selected vertices
+  Tensor<1,spacedim> t1 = get_tangent_vector(p, vertices[first_index]);
+  Tensor<1,spacedim> t2 = get_tangent_vector(p, vertices[second_index]);
+  Tensor<1,spacedim> normal = cross_product_3d(t1,t2);
 
-  const Tensor<1,spacedim> normal = cross_product_3d(t1,t2);
-  return normal/normal.norm();
+  Assert(normal.norm_square() > 1e4 * std::numeric_limits<double>::epsilon() *
+         t1.norm_square() * t2.norm_square(),
+         ExcMessage("Manifold::normal_vector was unable to find a suitable combination "
+                    "of vertices to compute a normal on this face. We chose the secants "
+                    "that are as orthogonal as possible, but tangents appear to be "
+                    "linearly dependent. Check for distorted faces in your triangulation."));
+
+  // Now figure out if we need to flip the direction, we do this by comparing
+  // to a reference normal that would be the correct result if all vertices
+  // would lie in a plane
+  const Tensor<1,spacedim> rt1 = vertices[3] - vertices[0];
+  const Tensor<1,spacedim> rt2 = vertices[2] - vertices[1];
+  const Tensor<1,spacedim> reference_normal = cross_product_3d(rt1,rt2);
+
+  if (reference_normal * normal < 0.0)
+    normal *= -1.0;
+
+  return normal / normal.norm();
 }
 
 
@@ -374,8 +305,11 @@ Point<spacedim>
 Manifold<dim, spacedim>::
 get_new_point_on_line (const typename Triangulation<dim, spacedim>::line_iterator &line) const
 {
-  const std::pair<std::vector<Point<spacedim> >, std::vector<double> > points_weights(get_default_points_and_weights(line));
-  return get_new_point (points_weights.first,points_weights.second);
+  const auto points_weights = get_default_points_and_weights(line);
+  return get_new_point (make_array_view(points_weights.first.begin(),
+                                        points_weights.first.end()),
+                        make_array_view(points_weights.second.begin(),
+                                        points_weights.second.end()));
 }
 
 
@@ -385,8 +319,11 @@ Point<spacedim>
 Manifold<dim, spacedim>::
 get_new_point_on_quad (const typename Triangulation<dim, spacedim>::quad_iterator &quad) const
 {
-  const std::pair<std::vector<Point<spacedim> >, std::vector<double> > points_weights(get_default_points_and_weights(quad));
-  return get_new_point (points_weights.first,points_weights.second);
+  const auto points_weights = get_default_points_and_weights(quad);
+  return get_new_point (make_array_view(points_weights.first.begin(),
+                                        points_weights.first.end()),
+                        make_array_view(points_weights.second.begin(),
+                                        points_weights.second.end()));
 }
 
 
@@ -513,8 +450,11 @@ Point<3>
 Manifold<3,3>::
 get_new_point_on_hex (const Triangulation<3, 3>::hex_iterator &hex) const
 {
-  const std::pair<std::vector<Point<3> >, std::vector<double> > points_weights(get_default_points_and_weights(hex,true));
-  return get_new_point (points_weights.first,points_weights.second);
+  const auto points_weights = get_default_points_and_weights(hex, true);
+  return get_new_point (make_array_view(points_weights.first.begin(),
+                                        points_weights.first.end()),
+                        make_array_view(points_weights.second.begin(),
+                                        points_weights.second.end()));
 }
 
 
@@ -526,20 +466,42 @@ Manifold<dim,spacedim>::get_tangent_vector(const Point<spacedim> &x1,
 {
   const double epsilon = 1e-8;
 
-  std::vector<Point<spacedim> > q;
-  q.push_back(x1);
-  q.push_back(x2);
-
-  std::vector<double> w;
-  w.push_back(epsilon);
-  w.push_back(1.0-epsilon);
-
-  const Tensor<1,spacedim> neighbor_point = get_new_point (q, w);
+  const std::array<Point<spacedim>, 2> points {{x1, x2}};
+  const std::array<double, 2> weights {{epsilon, 1.0 - epsilon}};
+  const Point<spacedim> neighbor_point = get_new_point (make_array_view(points.begin(),
+                                                        points.end()),
+                                                        make_array_view(weights.begin(),
+                                                            weights.end()));
   return (neighbor_point-x1)/epsilon;
 }
 
 /* -------------------------- FlatManifold --------------------- */
 
+namespace internal
+{
+  namespace
+  {
+    Tensor<1,3>
+    normalized_alternating_product (const Tensor<1,3> ( &)[1])
+    {
+      // we get here from FlatManifold<2,3>::normal_vector, but
+      // the implementation below is bogus for this case anyway
+      // (see the assert at the beginning of that function).
+      Assert (false, ExcNotImplemented());
+      return Tensor<1,3>();
+    }
+
+
+
+    Tensor<1,3>
+    normalized_alternating_product (const Tensor<1,3> (&basis_vectors)[2])
+    {
+      Tensor<1,3> tmp = cross_product_3d (basis_vectors[0], basis_vectors[1]);
+      return tmp/tmp.norm();
+    }
+
+  }
+}
 
 template <int dim, int spacedim>
 FlatManifold<dim,spacedim>::FlatManifold (const Tensor<1,spacedim> &periodicity,
@@ -551,12 +513,11 @@ FlatManifold<dim,spacedim>::FlatManifold (const Tensor<1,spacedim> &periodicity,
 
 
 
-template <int dim, int spacedim>
-Point<spacedim>
-FlatManifold<dim, spacedim>::
-get_new_point (const Quadrature<spacedim> &quad) const
+template<int dim, int spacedim>
+std::unique_ptr<Manifold<dim, spacedim> >
+FlatManifold<dim,spacedim>::clone() const
 {
-  return get_new_point(quad.get_points(),quad.get_weights());
+  return std_cxx14::make_unique<FlatManifold<dim,spacedim> >(periodicity, tolerance);
 }
 
 
@@ -564,8 +525,8 @@ get_new_point (const Quadrature<spacedim> &quad) const
 template <int dim, int spacedim>
 Point<spacedim>
 FlatManifold<dim, spacedim>::
-get_new_point (const std::vector<Point<spacedim> > &surrounding_points,
-               const std::vector<double>           &weights) const
+get_new_point (const ArrayView<const Point<spacedim>> &surrounding_points,
+               const ArrayView<const double>          &weights) const
 {
   Assert(std::abs(std::accumulate(weights.begin(), weights.end(), 0.0)-1.0) < 1e-10,
          ExcMessage("The weights for the individual points should sum to 1!"));
@@ -589,7 +550,7 @@ get_new_point (const std::vector<Point<spacedim> > &surrounding_points,
               minP[d] = std::min(minP[d], surrounding_points[i][d]);
               Assert( (surrounding_points[i][d] < periodicity[d]+tolerance*periodicity[d]) ||
                       (surrounding_points[i][d] >= -tolerance*periodicity[d]),
-                      ExcPeriodicBox(d, surrounding_points[i], periodicity[i]));
+                      ExcPeriodicBox(d, surrounding_points[i], periodicity[d]));
             }
 
       // compute the weighted average point, possibly taking into account periodicity
@@ -619,15 +580,15 @@ get_new_point (const std::vector<Point<spacedim> > &surrounding_points,
 template <int dim, int spacedim>
 void
 FlatManifold<dim, spacedim>::
-add_new_points (const std::vector<Point<spacedim> > &surrounding_points,
-                const Table<2,double>               &weights,
-                std::vector<Point<spacedim> >       &new_points) const
+get_new_points (const ArrayView<const Point<spacedim>> &surrounding_points,
+                const Table<2,double>                  &weights,
+                ArrayView<Point<spacedim>>              new_points) const
 {
   AssertDimension(surrounding_points.size(), weights.size(1));
   if (weights.size(0) == 0)
     return;
 
-  const unsigned int n_points = surrounding_points.size();
+  const std::size_t n_points = surrounding_points.size();
 
   Tensor<1,spacedim> minP = periodicity;
   for (unsigned int d=0; d<spacedim; ++d)
@@ -642,8 +603,9 @@ add_new_points (const std::vector<Point<spacedim> > &surrounding_points,
 
   // check whether periodicity shifts some of the points. Only do this if
   // necessary to avoid memory allocation
-  const Point<spacedim> *surrounding_points_start = &surrounding_points[0];
-  std::vector<Point<spacedim> > modified_points;
+  const Point<spacedim> *surrounding_points_start = surrounding_points.data();
+
+  boost::container::small_vector<Point<spacedim>, 200> modified_points;
   bool adjust_periodicity = false;
   for (unsigned int d=0; d<spacedim; ++d)
     if (periodicity[d] > 0)
@@ -655,13 +617,15 @@ add_new_points (const std::vector<Point<spacedim> > &surrounding_points,
           }
   if (adjust_periodicity == true)
     {
-      modified_points = surrounding_points;
+      modified_points.resize(surrounding_points.size());
+      std::copy(surrounding_points.begin(), surrounding_points.end(),
+                modified_points.begin());
       for (unsigned int d=0; d<spacedim; ++d)
         if (periodicity[d] > 0)
           for (unsigned int i=0; i<n_points; ++i)
             if ((surrounding_points[i][d]-minP[d]) > periodicity[d]/2.0)
               modified_points[i][d] -= periodicity[d];
-      surrounding_points_start = &modified_points[0];
+      surrounding_points_start = modified_points.data();
     }
 
   // Now perform the interpolation
@@ -679,7 +643,11 @@ add_new_points (const std::vector<Point<spacedim> > &surrounding_points,
           if (new_point[d] < 0)
             new_point[d] += periodicity[d];
 
-      new_points.push_back(project_to_manifold(surrounding_points, new_point));
+      // TODO should this use surrounding_points_start or surrounding_points?
+      // The older version used surrounding_points
+      new_points[row] = project_to_manifold(make_array_view(surrounding_points.begin(),
+                                                            surrounding_points.end()),
+                                            new_point);
     }
 }
 
@@ -687,8 +655,9 @@ add_new_points (const std::vector<Point<spacedim> > &surrounding_points,
 
 template <int dim, int spacedim>
 Point<spacedim>
-FlatManifold<dim, spacedim>::project_to_manifold (const std::vector<Point<spacedim> > &/*vertices*/,
-                                                  const Point<spacedim> &candidate) const
+FlatManifold<dim, spacedim>::project_to_manifold
+(const ArrayView<const Point<spacedim>> &/*vertices*/,
+ const Point<spacedim>                  &candidate) const
 {
   return candidate;
 }
@@ -729,14 +698,237 @@ FlatManifold<dim, spacedim>::get_tangent_vector (const Point<spacedim> &x1,
 
 
 
+template <>
+void
+FlatManifold<1>::
+get_normals_at_vertices (const Triangulation<1>::face_iterator &,
+                         Manifold<1,1>::FaceVertexNormals &) const
+{
+  Assert (false, ExcImpossibleInDim(1));
+}
+
+
+
+template <>
+void
+FlatManifold<1,2>::
+get_normals_at_vertices (const Triangulation<1,2>::face_iterator &,
+                         Manifold<1,2>::FaceVertexNormals &) const
+{
+  Assert (false, ExcNotImplemented());
+}
+
+
+
+template <>
+void
+FlatManifold<1,3>::
+get_normals_at_vertices (const Triangulation<1,3>::face_iterator &,
+                         Manifold<1,3>::FaceVertexNormals &) const
+{
+  Assert (false, ExcNotImplemented());
+}
+
+
+
+template <>
+void
+FlatManifold<2>::
+get_normals_at_vertices (const Triangulation<2>::face_iterator &face,
+                         Manifold<2,2>::FaceVertexNormals &face_vertex_normals) const
+{
+  const Tensor<1,2> tangent = face->vertex(1) - face->vertex(0);
+  for (unsigned int vertex=0; vertex<GeometryInfo<2>::vertices_per_face; ++vertex)
+    // compute normals from tangent
+    face_vertex_normals[vertex] = Point<2>(tangent[1],
+                                           -tangent[0]);
+}
+
+
+
+template <>
+void
+FlatManifold<2,3>::
+get_normals_at_vertices (const Triangulation<2,3>::face_iterator &/*face*/,
+                         Manifold<2,3>::FaceVertexNormals &/*face_vertex_normals*/) const
+{
+  Assert(false, ExcNotImplemented());
+}
+
+
+
+template <>
+void
+FlatManifold<3>::
+get_normals_at_vertices (const Triangulation<3>::face_iterator &face,
+                         Manifold<3,3>::FaceVertexNormals &face_vertex_normals) const
+{
+  const unsigned int vertices_per_face = GeometryInfo<3>::vertices_per_face;
+
+  static const unsigned int neighboring_vertices[4][2]=
+  { {1,2},{3,0},{0,3},{2,1}};
+  for (unsigned int vertex=0; vertex<vertices_per_face; ++vertex)
+    {
+      // first define the two tangent vectors at the vertex by using the
+      // two lines radiating away from this vertex
+      const Tensor<1,3> tangents[2]
+        = { face->vertex(neighboring_vertices[vertex][0])
+            - face->vertex(vertex),
+            face->vertex(neighboring_vertices[vertex][1])
+            - face->vertex(vertex)
+          };
+
+      // then compute the normal by taking the cross product. since the
+      // normal is not required to be normalized, no problem here
+      face_vertex_normals[vertex] = cross_product_3d(tangents[0], tangents[1]);
+    }
+}
+
+
+
+template <>
+Tensor<1,1>
+FlatManifold<1,1>::
+normal_vector (const Triangulation<1,1>::face_iterator &,
+               const Point<1> &) const
+{
+  Assert (false, ExcNotImplemented());
+  return Tensor<1,1>();
+}
+
+
+
+template <>
+Tensor<1,2>
+FlatManifold<1,2>::
+normal_vector (const Triangulation<1,2>::face_iterator &,
+               const Point<2> &) const
+{
+  Assert (false, ExcNotImplemented());
+  return Tensor<1,2>();
+}
+
+
+
+template <>
+Tensor<1,3>
+FlatManifold<1,3>::
+normal_vector (const Triangulation<1,3>::face_iterator &,
+               const Point<3> &) const
+{
+  Assert (false, ExcNotImplemented());
+  return Tensor<1,3>();
+}
+
+
+
+template <>
+Tensor<1,2>
+FlatManifold<2,2>::
+normal_vector (const Triangulation<2,2>::face_iterator &face,
+               const Point<2> &p) const
+{
+  // In 2d, a face is just a straight line and
+  // we can use the 'standard' implementation.
+  return Manifold<2,2>::normal_vector(face, p);
+}
+
+
+
+template <int dim, int spacedim>
+Tensor<1, spacedim>
+FlatManifold< dim, spacedim >::
+normal_vector (const typename Triangulation<dim, spacedim>::face_iterator &face,
+               const Point<spacedim>                                      &p) const
+{
+  // I don't think the implementation below will work when dim!=spacedim;
+  // in fact, I believe that we don't even have enough information here,
+  // because we would need to know not only about the tangent vectors
+  // of the face, but also of the cell, to compute the normal vector.
+  // Someone will have to think about this some more.
+  Assert (dim == spacedim, ExcNotImplemented());
+
+  // in order to find out what the normal vector is, we first need to
+  // find the reference coordinates of the point p on the given face,
+  // or at least the reference coordinates of the closest point on the
+  // face
+  //
+  // in other words, we need to find a point xi so that f(xi)=||F(xi)-p||^2->min
+  // where F(xi) is the mapping. this algorithm is implemented in
+  // MappingQ1<dim,spacedim>::transform_real_to_unit_cell but only for cells,
+  // while we need it for faces here. it's also implemented in somewhat
+  // more generality there using the machinery of the MappingQ1 class
+  // while we really only need it for a specific case here
+  //
+  // in any case, the iteration we use here is a Gauss-Newton's iteration with
+  //   xi^{n+1} = xi^n - H(xi^n)^{-1} J(xi^n)
+  // where
+  //   J(xi) = (grad F(xi))^T (F(xi)-p)
+  // and
+  //   H(xi) = [grad F(xi)]^T [grad F(xi)]
+  // In all this,
+  //   F(xi) = sum_v vertex[v] phi_v(xi)
+  // We get the shape functions phi_v from an object of type FE_Q<dim-1>(1)
+
+  // we start with the point xi=1/2, xi=(1/2,1/2), ...
+  const unsigned int facedim = dim-1;
+
+  Point<facedim> xi;
+  for (unsigned int i=0; i<facedim; ++i)
+    xi[i] = 1./2;
+
+  const double eps = 1e-12;
+  Tensor<1,spacedim> grad_F[facedim];
+  unsigned int iteration = 0;
+  while (true)
+    {
+      Point<spacedim> F;
+      for (unsigned int v=0; v<GeometryInfo<facedim>::vertices_per_cell; ++v)
+        F += face->vertex(v) * GeometryInfo<facedim>::d_linear_shape_function(xi, v);
+
+      for (unsigned int i=0; i<facedim; ++i)
+        {
+          grad_F[i] = 0;
+          for (unsigned int v=0; v<GeometryInfo<facedim>::vertices_per_cell; ++v)
+            grad_F[i] += face->vertex(v) *
+                         GeometryInfo<facedim>::d_linear_shape_function_gradient(xi, v)[i];
+        }
+
+      Tensor<1,facedim> J;
+      for (unsigned int i=0; i<facedim; ++i)
+        for (unsigned int j=0; j<spacedim; ++j)
+          J[i] += grad_F[i][j] * (F-p)[j];
+
+      Tensor<2,facedim> H;
+      for (unsigned int i=0; i<facedim; ++i)
+        for (unsigned int j=0; j<facedim; ++j)
+          for (unsigned int k=0; k<spacedim; ++k)
+            H[i][j] += grad_F[i][k] * grad_F[j][k];
+
+      const Tensor<1,facedim> delta_xi = -invert(H) * J;
+      xi += delta_xi;
+      ++iteration;
+
+      Assert (iteration<10,
+              ExcMessage("The Newton iteration to find the reference point "
+                         "did not converge in 10 iterations. Do you have a "
+                         "deformed cell? (See the glossary for a definition "
+                         "of what a deformed cell is. You may want to output "
+                         "the vertices of your cell."));
+
+      if (delta_xi.norm() < eps)
+        break;
+    }
+
+  // so now we have the reference coordinates xi of the point p.
+  // we then have to compute the normal vector, which we can do
+  // by taking the (normalize) alternating product of all the tangent
+  // vectors given by grad_F
+  return internal::normalized_alternating_product(grad_F);
+}
+
+
 /* -------------------------- ChartManifold --------------------- */
-
-template <int dim, int spacedim, int chartdim>
-ChartManifold<dim,spacedim,chartdim>::~ChartManifold ()
-{}
-
-
-
 template <int dim, int spacedim, int chartdim>
 ChartManifold<dim,spacedim,chartdim>::ChartManifold (const Tensor<1,chartdim> &periodicity)
   :
@@ -748,9 +940,14 @@ ChartManifold<dim,spacedim,chartdim>::ChartManifold (const Tensor<1,chartdim> &p
 template <int dim, int spacedim, int chartdim>
 Point<spacedim>
 ChartManifold<dim,spacedim,chartdim>::
-get_new_point (const Quadrature<spacedim> &quad) const
+get_intermediate_point (const Point<spacedim> &p1,
+                        const Point<spacedim> &p2,
+                        const double w) const
 {
-  return get_new_point(quad.get_points(),quad.get_weights());
+  const std::array<Point<spacedim>, 2> points {{p1, p2}};
+  const std::array<double, 2> weights {{1.-w, w}};
+  return get_new_point(make_array_view(points.begin(), points.end()),
+                       make_array_view(weights.begin(), weights.end()));
 }
 
 
@@ -758,15 +955,19 @@ get_new_point (const Quadrature<spacedim> &quad) const
 template <int dim, int spacedim, int chartdim>
 Point<spacedim>
 ChartManifold<dim,spacedim,chartdim>::
-get_new_point (const std::vector<Point<spacedim> > &surrounding_points,
-               const std::vector<double>           &weights) const
+get_new_point (const ArrayView<const Point<spacedim>> &surrounding_points,
+               const ArrayView<const double>          &weights) const
 {
-  std::vector<Point<chartdim> > chart_points(surrounding_points.size());
+  const std::size_t n_points = surrounding_points.size();
 
-  for (unsigned int i=0; i<surrounding_points.size(); ++i)
+  boost::container::small_vector<Point<chartdim>, 200> chart_points(n_points);
+
+  for (unsigned int i=0; i<n_points; ++i)
     chart_points[i] = pull_back(surrounding_points[i]);
 
-  const Point<chartdim> p_chart = sub_manifold.get_new_point(chart_points,weights);
+  const Point<chartdim> p_chart = sub_manifold.get_new_point
+                                  (make_array_view(chart_points.begin(), chart_points.end()),
+                                   weights);
 
   return push_forward(p_chart);
 }
@@ -776,25 +977,28 @@ get_new_point (const std::vector<Point<spacedim> > &surrounding_points,
 template <int dim, int spacedim, int chartdim>
 void
 ChartManifold<dim,spacedim,chartdim>::
-add_new_points (const std::vector<Point<spacedim> > &surrounding_points,
-                const Table<2,double>               &weights,
-                std::vector<Point<spacedim> >       &new_points) const
+get_new_points (const ArrayView<const Point<spacedim>> &surrounding_points,
+                const Table<2,double>                  &weights,
+                ArrayView<Point<spacedim>>              new_points) const
 {
   Assert(weights.size(0) > 0, ExcEmptyObject());
   AssertDimension(surrounding_points.size(), weights.size(1));
 
-  const unsigned int n_points = surrounding_points.size();
+  const std::size_t n_points = surrounding_points.size();
 
-  std::vector<Point<chartdim> > chart_points(n_points);
-  for (unsigned int i=0; i<n_points; ++i)
+  boost::container::small_vector<Point<chartdim>, 200> chart_points(n_points);
+  for (std::size_t i=0; i<n_points; ++i)
     chart_points[i] = pull_back(surrounding_points[i]);
 
-  std::vector<Point<chartdim> > new_points_on_chart;
-  new_points_on_chart.reserve(weights.size(0));
-  sub_manifold.add_new_points(chart_points, weights, new_points_on_chart);
+  boost::container::small_vector<Point<chartdim>, 200> new_points_on_chart(weights.size(0));
+  sub_manifold.get_new_points(make_array_view(chart_points.begin(),
+                                              chart_points.end()),
+                              weights,
+                              make_array_view(new_points_on_chart.begin(),
+                                              new_points_on_chart.end()));
 
-  for (unsigned int row=0; row<weights.size(0); ++row)
-    new_points.push_back(push_forward(new_points_on_chart[row]));
+  for (std::size_t row=0; row<weights.size(0); ++row)
+    new_points[row] = push_forward(new_points_on_chart[row]);
 }
 
 
