@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 1999 - 2014 by the deal.II authors
+// Copyright (C) 1999 - 2017 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -56,19 +56,13 @@ PersistentTriangulation (const PersistentTriangulation<dim,spacedim> &old_tria)
 
 
 
-template <int dim, int spacedim>
-PersistentTriangulation<dim,spacedim>::~PersistentTriangulation ()
-{}
-
-
-
 template <int dim, int spacedim>
 void
 PersistentTriangulation<dim,spacedim>::execute_coarsening_and_refinement ()
 {
   // first save flags
-  refine_flags.push_back (std::vector<bool>());
-  coarsen_flags.push_back (std::vector<bool>());
+  refine_flags.emplace_back ();
+  coarsen_flags.emplace_back ();
   this->save_refine_flags (refine_flags.back());
   this->save_coarsen_flags (coarsen_flags.back());
 
@@ -206,8 +200,8 @@ PersistentTriangulation<dim,spacedim>::read_flags(std::istream &in)
   in >> n_flag_levels;
   for (unsigned int i=0; i<n_flag_levels; ++i)
     {
-      refine_flags.push_back (std::vector<bool>());
-      coarsen_flags.push_back (std::vector<bool>());
+      refine_flags.emplace_back ();
+      coarsen_flags.emplace_back ();
       this->read_bool_vector (mn_tria_refine_flags_begin, refine_flags.back(),
                               mn_tria_refine_flags_end, in);
       this->read_bool_vector (mn_tria_coarsen_flags_begin, coarsen_flags.back(),
@@ -252,4 +246,3 @@ template class PersistentTriangulation<1,2>;
 template class PersistentTriangulation<2,3>;
 
 DEAL_II_NAMESPACE_CLOSE
-
