@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 1998 - 2016 by the deal.II authors
+// Copyright (C) 1998 - 2018 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -14,6 +14,7 @@
 // ---------------------------------------------------------------------
 
 #include <deal.II/base/tensor.h>
+#include <deal.II/base/std_cxx14/memory.h>
 #include <deal.II/grid/tria_boundary.h>
 #include <deal.II/grid/tria.h>
 #include <deal.II/grid/tria_iterator.h>
@@ -29,8 +30,13 @@ DEAL_II_NAMESPACE_OPEN
 
 
 template <int dim, int spacedim>
-Boundary<dim, spacedim>::~Boundary ()
-{}
+std::unique_ptr<Manifold<dim, spacedim> >
+Boundary<dim,spacedim>::clone() const
+{
+  AssertThrow(false, ExcPureFunctionCalled());
+  return std::unique_ptr<Manifold<dim,spacedim> >();
+}
+
 
 
 template <int dim, int spacedim>
@@ -163,19 +169,16 @@ Boundary<dim,spacedim>::
 get_line_support_points (const unsigned int n_intermediate_points) const
 {
   if (points.size() <= n_intermediate_points ||
-      points[n_intermediate_points].get() == 0)
+      points[n_intermediate_points].get() == nullptr)
     {
       Threads::Mutex::ScopedLock lock(mutex);
       if (points.size() <= n_intermediate_points)
         points.resize(n_intermediate_points+1);
 
       // another thread might have created points in the meantime
-      if (points[n_intermediate_points].get() == 0)
-        {
-          std_cxx11::shared_ptr<QGaussLobatto<1> >
-          quadrature (new QGaussLobatto<1>(n_intermediate_points+2));
-          points[n_intermediate_points] = quadrature;
-        }
+      if (points[n_intermediate_points].get() == nullptr)
+        points[n_intermediate_points] = std_cxx14::make_unique<QGaussLobatto<1> >
+                                        (n_intermediate_points+2);
     }
   return points[n_intermediate_points]->get_points();
 }
@@ -185,10 +188,11 @@ get_line_support_points (const unsigned int n_intermediate_points) const
 
 /* -------------------------- StraightBoundary --------------------- */
 
-
+// At least clang < 3.9.0 complains if we move this definition to its
+// declaration when a 'const StraightBoundary' object is built.
 template <int dim, int spacedim>
-StraightBoundary<dim, spacedim>::StraightBoundary ()
-{}
+StraightBoundary<dim, spacedim>::StraightBoundary () = default;
+
 
 
 template <int dim, int spacedim>
@@ -200,6 +204,14 @@ get_new_point_on_line (const typename Triangulation<dim, spacedim>::line_iterato
 }
 
 
+template <int dim, int spacedim>
+std::unique_ptr<Manifold<dim, spacedim> >
+StraightBoundary<dim,spacedim>::clone() const
+{
+  return std_cxx14::make_unique<StraightBoundary<dim,spacedim> >();
+}
+
+
 namespace
 {
   // compute the new midpoint of a quad --
@@ -422,259 +434,24 @@ get_intermediate_points_on_quad (const Triangulation<2,3>::quad_iterator &quad,
 
 
 
-template <>
-Tensor<1,1>
-StraightBoundary<1,1>::
-normal_vector (const Triangulation<1,1>::face_iterator &,
-               const Point<1> &) const
-{
-  Assert (false, ExcNotImplemented());
-  return Tensor<1,1>();
-}
-
-
-template <>
-Tensor<1,2>
-StraightBoundary<1,2>::
-normal_vector (const Triangulation<1,2>::face_iterator &,
-               const Point<2> &) const
-{
-  Assert (false, ExcNotImplemented());
-  return Tensor<1,2>();
-}
-
-
-template <>
-Tensor<1,3>
-StraightBoundary<1,3>::
-normal_vector (const Triangulation<1,3>::face_iterator &,
-               const Point<3> &) const
-{
-  Assert (false, ExcNotImplemented());
-  return Tensor<1,3>();
-}
-
-
-namespace internal
-{
-  namespace
-  {
-    /**
-     * Compute the normalized cross product of a set of dim-1 basis
-     * vectors.
-     */
-    Tensor<1,2>
-    normalized_alternating_product (const Tensor<1,2> (&basis_vectors)[1])
-    {
-      Tensor<1,2> tmp = cross_product_2d (basis_vectors[0]);
-      return tmp/tmp.norm();
-    }
-
-
-
-    Tensor<1,3>
-    normalized_alternating_product (const Tensor<1,3> ( &)[1])
-    {
-      // we get here from StraightBoundary<2,3>::normal_vector, but
-      // the implementation below is bogus for this case anyway
-      // (see the assert at the beginning of that function).
-      Assert (false, ExcNotImplemented());
-      return Tensor<1,3>();
-    }
-
-
-
-    Tensor<1,3>
-    normalized_alternating_product (const Tensor<1,3> (&basis_vectors)[2])
-    {
-      Tensor<1,3> tmp = cross_product_3d (basis_vectors[0], basis_vectors[1]);
-      return tmp/tmp.norm();
-    }
-
-  }
-}
-
-
 template <int dim, int spacedim>
 Tensor<1,spacedim>
 StraightBoundary<dim,spacedim>::
 normal_vector (const typename Triangulation<dim,spacedim>::face_iterator &face,
-               const Point<spacedim> &p) const
+               const Point<spacedim>                                     &p) const
 {
-  // I don't think the implementation below will work when dim!=spacedim;
-  // in fact, I believe that we don't even have enough information here,
-  // because we would need to know not only about the tangent vectors
-  // of the face, but also of the cell, to compute the normal vector.
-  // Someone will have to think about this some more.
-  Assert (dim == spacedim, ExcNotImplemented());
-
-  // in order to find out what the normal vector is, we first need to
-  // find the reference coordinates of the point p on the given face,
-  // or at least the reference coordinates of the closest point on the
-  // face
-  //
-  // in other words, we need to find a point xi so that f(xi)=||F(xi)-p||^2->min
-  // where F(xi) is the mapping. this algorithm is implemented in
-  // MappingQ1<dim,spacedim>::transform_real_to_unit_cell but only for cells,
-  // while we need it for faces here. it's also implemented in somewhat
-  // more generality there using the machinery of the MappingQ1 class
-  // while we really only need it for a specific case here
-  //
-  // in any case, the iteration we use here is a Gauss-Newton's iteration with
-  //   xi^{n+1} = xi^n - H(xi^n)^{-1} J(xi^n)
-  // where
-  //   J(xi) = (grad F(xi))^T (F(xi)-p)
-  // and
-  //   H(xi) = [grad F(xi)]^T [grad F(xi)]
-  // In all this,
-  //   F(xi) = sum_v vertex[v] phi_v(xi)
-  // We get the shape functions phi_v from an object of type FE_Q<dim-1>(1)
-
-  // we start with the point xi=1/2, xi=(1/2,1/2), ...
-  const unsigned int facedim = dim-1;
-
-  Point<facedim> xi;
-  for (unsigned int i=0; i<facedim; ++i)
-    xi[i] = 1./2;
-
-  FE_Q<facedim> linear_fe(1);
-
-  const double eps = 1e-12;
-  Tensor<1,spacedim> grad_F[facedim];
-  unsigned int iteration = 0;
-  while (true)
-    {
-      Point<spacedim> F;
-      for (unsigned int v=0; v<GeometryInfo<facedim>::vertices_per_cell; ++v)
-        F += face->vertex(v) * linear_fe.shape_value(v, xi);
-
-      for (unsigned int i=0; i<facedim; ++i)
-        {
-          grad_F[i] = 0;
-          for (unsigned int v=0; v<GeometryInfo<facedim>::vertices_per_cell; ++v)
-            grad_F[i] += face->vertex(v) * linear_fe.shape_grad(v, xi)[i];
-        }
-
-      Tensor<1,facedim> J;
-      for (unsigned int i=0; i<facedim; ++i)
-        for (unsigned int j=0; j<spacedim; ++j)
-          J[i] += grad_F[i][j] * (F-p)[j];
-
-      Tensor<2,facedim> H;
-      for (unsigned int i=0; i<facedim; ++i)
-        for (unsigned int j=0; j<facedim; ++j)
-          for (unsigned int k=0; k<spacedim; ++k)
-            H[i][j] += grad_F[i][k] * grad_F[j][k];
-
-      const Tensor<1,facedim> delta_xi = -invert(H) * J;
-      xi += delta_xi;
-      ++iteration;
-
-      Assert (iteration<10,
-              ExcMessage("The Newton iteration to find the reference point "
-                         "did not converge in 10 iterations. Do you have a "
-                         "deformed cell? (See the glossary for a definition "
-                         "of what a deformed cell is. You may want to output "
-                         "the vertices of your cell."));
-
-      if (delta_xi.norm() < eps)
-        break;
-    }
-
-  // so now we have the reference coordinates xi of the point p.
-  // we then have to compute the normal vector, which we can do
-  // by taking the (normalize) alternating product of all the tangent
-  // vectors given by grad_F
-  return internal::normalized_alternating_product(grad_F);
+  return FlatManifold<dim, spacedim>::normal_vector(face, p);
 }
 
 
 
-template <>
+template <int dim, int spacedim>
 void
-StraightBoundary<1>::
-get_normals_at_vertices (const Triangulation<1>::face_iterator &,
-                         Boundary<1,1>::FaceVertexNormals &) const
+StraightBoundary<dim, spacedim>::
+get_normals_at_vertices (const typename Triangulation<dim, spacedim>::face_iterator &face,
+                         typename Boundary<dim,spacedim>::FaceVertexNormals         &face_vertex_normals) const
 {
-  Assert (false, ExcImpossibleInDim(1));
-}
-
-template <>
-void
-StraightBoundary<1,2>::
-get_normals_at_vertices (const Triangulation<1,2>::face_iterator &,
-                         Boundary<1,2>::FaceVertexNormals &) const
-{
-  Assert (false, ExcNotImplemented());
-}
-
-
-template <>
-void
-StraightBoundary<1,3>::
-get_normals_at_vertices (const Triangulation<1,3>::face_iterator &,
-                         Boundary<1,3>::FaceVertexNormals &) const
-{
-  Assert (false, ExcNotImplemented());
-}
-
-
-
-template <>
-void
-StraightBoundary<2>::
-get_normals_at_vertices (const Triangulation<2>::face_iterator &face,
-                         Boundary<2,2>::FaceVertexNormals &face_vertex_normals) const
-{
-  const Tensor<1,2> tangent = face->vertex(1) - face->vertex(0);
-  for (unsigned int vertex=0; vertex<GeometryInfo<2>::vertices_per_face; ++vertex)
-    // compute normals from tangent
-    face_vertex_normals[vertex] = Point<2>(tangent[1],
-                                           -tangent[0]);
-}
-
-template <>
-void
-StraightBoundary<2,3>::
-get_normals_at_vertices (const Triangulation<2,3>::face_iterator &face,
-                         Boundary<2,3>::FaceVertexNormals &face_vertex_normals) const
-{
-  const Tensor<1,3> tangent = face->vertex(1) - face->vertex(0);
-  for (unsigned int vertex=0; vertex<GeometryInfo<2>::vertices_per_face; ++vertex)
-    // compute normals from tangent
-    face_vertex_normals[vertex] = Point<3>(tangent[1],
-                                           -tangent[0],0);
-  Assert(false, ExcNotImplemented());
-}
-
-
-
-
-template <>
-void
-StraightBoundary<3>::
-get_normals_at_vertices (const Triangulation<3>::face_iterator &face,
-                         Boundary<3,3>::FaceVertexNormals &face_vertex_normals) const
-{
-  const unsigned int vertices_per_face = GeometryInfo<3>::vertices_per_face;
-
-  static const unsigned int neighboring_vertices[4][2]=
-  { {1,2},{3,0},{0,3},{2,1}};
-  for (unsigned int vertex=0; vertex<vertices_per_face; ++vertex)
-    {
-      // first define the two tangent vectors at the vertex by using the
-      // two lines radiating away from this vertex
-      const Tensor<1,3> tangents[2]
-        = { face->vertex(neighboring_vertices[vertex][0])
-            - face->vertex(vertex),
-            face->vertex(neighboring_vertices[vertex][1])
-            - face->vertex(vertex)
-          };
-
-      // then compute the normal by taking the cross product. since the
-      // normal is not required to be normalized, no problem here
-      face_vertex_normals[vertex] = cross_product_3d(tangents[0], tangents[1]);
-    };
+  FlatManifold<dim, spacedim>::get_normals_at_vertices(face, face_vertex_normals);
 }
 
 
@@ -710,7 +487,7 @@ namespace internal
   Point<spacedim>
   compute_projection (const Iterator        &object,
                       const Point<spacedim> &y,
-                      internal::int2type<dim>)
+                      std::integral_constant<int, dim>)
   {
     // let's look at this for
     // simplicity for a quad (dim==2)
@@ -807,7 +584,7 @@ namespace internal
   Point<1>
   compute_projection (const Iterator &,
                       const Point<1> &y,
-                      /* it's a quad: */internal::int2type<2>)
+                      /* it's a quad: */std::integral_constant<int, 2>)
   {
     return y;
   }
@@ -817,7 +594,7 @@ namespace internal
   Point<2>
   compute_projection (const Iterator &,
                       const Point<2> &y,
-                      /* it's a quad: */internal::int2type<2>)
+                      /* it's a quad: */std::integral_constant<int, 2>)
   {
     return y;
   }
@@ -847,7 +624,7 @@ project_to_surface (const typename Triangulation<dim, spacedim>::quad_iterator &
     return y;
   else
     return internal::compute_projection (quad, y,
-                                         /* it's a quad */internal::int2type<2>());
+                                         /* it's a quad */std::integral_constant<int, 2>());
 }
 
 
@@ -879,4 +656,3 @@ project_to_surface (const typename Triangulation<dim, spacedim>::hex_iterator &,
 #include "tria_boundary.inst"
 
 DEAL_II_NAMESPACE_CLOSE
-
