@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 1999 - 2016 by the deal.II authors
+// Copyright (C) 1999 - 2018 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -18,6 +18,7 @@
 #include <deal.II/grid/tria_iterator.h>
 #include <deal.II/grid/tria_accessor.h>
 #include <deal.II/base/tensor.h>
+#include <deal.II/base/std_cxx14/memory.h>
 #include <cmath>
 
 
@@ -47,6 +48,16 @@ CylinderBoundary<dim,spacedim>::CylinderBoundary (const double           radius,
 {}
 
 
+
+template<int dim, int spacedim>
+std::unique_ptr<Manifold<dim, spacedim> >
+CylinderBoundary<dim,spacedim>::clone() const
+{
+  return std_cxx14::make_unique<CylinderBoundary<dim, spacedim> >(radius, direction, point_on_axis);
+}
+
+
+
 template <int dim, int spacedim>
 Point<spacedim>
 CylinderBoundary<dim,spacedim>::get_axis_vector (const unsigned int axis)
@@ -90,7 +101,7 @@ get_new_point_on_line (const typename Triangulation<dim,spacedim>::line_iterator
 
 
 
-template<>
+template <>
 Point<3>
 CylinderBoundary<3>::
 get_new_point_on_quad (const Triangulation<3>::quad_iterator &quad) const
@@ -110,7 +121,7 @@ get_new_point_on_quad (const Triangulation<3>::quad_iterator &quad) const
                     point_on_axis);
 }
 
-template<>
+template <>
 Point<3>
 CylinderBoundary<2,3>::
 get_new_point_on_quad (const Triangulation<2,3>::quad_iterator &quad) const
@@ -271,7 +282,7 @@ CylinderBoundary<dim,spacedim>::get_radius () const
 
 //======================================================================//
 
-template<int dim>
+template <int dim>
 ConeBoundary<dim>::ConeBoundary (const double radius_0,
                                  const double radius_1,
                                  const Point<dim> x_0,
@@ -286,6 +297,14 @@ ConeBoundary<dim>::ConeBoundary (const double radius_0,
 
 
 template<int dim>
+std::unique_ptr<Manifold<dim,dim> > ConeBoundary<dim>::clone() const
+{
+  return std_cxx14::make_unique<ConeBoundary<dim> >(radius_0,radius_1,x_0,x_1);
+}
+
+
+
+template <int dim>
 double ConeBoundary<dim>::get_radius (Point<dim> x) const
 {
   for (unsigned int i = 0; i < dim; ++i)
@@ -297,7 +316,7 @@ double ConeBoundary<dim>::get_radius (Point<dim> x) const
 
 
 
-template<int dim>
+template <int dim>
 void
 ConeBoundary<dim>::
 get_intermediate_points_between_points (const Point<dim> &p0,
@@ -327,7 +346,7 @@ get_intermediate_points_between_points (const Point<dim> &p0,
     }
 }
 
-template<int dim>
+template <int dim>
 Point<dim>
 ConeBoundary<dim>::
 get_new_point_on_line (const typename Triangulation<dim>::line_iterator &line) const
@@ -366,7 +385,7 @@ get_new_point_on_quad (const Triangulation<3>::quad_iterator &quad) const
 
 
 
-template<int dim>
+template <int dim>
 Point<dim>
 ConeBoundary<dim>::
 get_new_point_on_quad (const typename Triangulation<dim>::quad_iterator &) const
@@ -378,7 +397,7 @@ get_new_point_on_quad (const typename Triangulation<dim>::quad_iterator &) const
 
 
 
-template<int dim>
+template <int dim>
 void
 ConeBoundary<dim>::
 get_intermediate_points_on_line (const typename Triangulation<dim>::line_iterator &line,
@@ -393,7 +412,7 @@ get_intermediate_points_on_line (const typename Triangulation<dim>::line_iterato
 
 
 
-template<>
+template <>
 void
 ConeBoundary<3>::
 get_intermediate_points_on_quad (const Triangulation<3>::quad_iterator &quad,
@@ -441,7 +460,7 @@ get_intermediate_points_on_quad (const typename Triangulation<dim>::quad_iterato
 
 
 
-template<>
+template <>
 void
 ConeBoundary<1>::
 get_normals_at_vertices (const Triangulation<1>::face_iterator &,
@@ -452,7 +471,7 @@ get_normals_at_vertices (const Triangulation<1>::face_iterator &,
 
 
 
-template<int dim>
+template <int dim>
 void
 ConeBoundary<dim>::
 get_normals_at_vertices (const typename Triangulation<dim>::face_iterator &face,
@@ -475,6 +494,33 @@ get_normals_at_vertices (const typename Triangulation<dim>::face_iterator &face,
 }
 
 
+template <int dim>
+inline
+Tensor<1,dim>
+ConeBoundary<dim>::
+normal_vector (const typename Triangulation<dim>::face_iterator &,
+               const Point<dim> &p) const
+{
+// TODO only for cone opening along z-axis
+  Assert (dim == 3, ExcNotImplemented());
+  Assert (this->radius_0 == 0., ExcNotImplemented());
+  Assert (this->x_0 == Point<dim>(), ExcNotImplemented());
+  for (unsigned int d=0; d<dim; ++d)
+    if (d != dim-1)  // don't test the last component of the vector
+      Assert (this->x_1[d] == 0., ExcNotImplemented());
+  Assert (this->x_1[dim-1] > 0, ExcNotImplemented());
+
+  const double c_squared = (this->radius_1 / this->x_1[dim-1])*(this->radius_1 / this->x_1[dim-1]);
+  Tensor<1,dim> normal = p;
+  normal[0] *= -2.0/c_squared;
+  normal[1] *= -2.0/c_squared;
+  normal[dim-1] *= 2.0;
+
+  return normal/normal.norm();
+}
+
+
+
 //======================================================================//
 
 template <int dim, int spacedim>
@@ -488,6 +534,15 @@ HyperBallBoundary<dim,spacedim>::HyperBallBoundary (const Point<spacedim> p,
 
 
 
+template<int dim, int spacedim>
+std::unique_ptr<Manifold<dim, spacedim> >
+HyperBallBoundary<dim,spacedim>::clone() const
+{
+  return std_cxx14::make_unique<HyperBallBoundary<dim,spacedim> >(center, radius);
+}
+
+
+
 template <int dim, int spacedim>
 Point<spacedim>
 HyperBallBoundary<dim,spacedim>::get_new_point_on_line (const typename Triangulation<dim,spacedim>::line_iterator &line) const
@@ -793,6 +848,15 @@ HalfHyperBallBoundary<dim>::HalfHyperBallBoundary (const Point<dim> center,
 
 
 
+template<int dim>
+std::unique_ptr<Manifold<dim,dim> > HalfHyperBallBoundary<dim>::clone() const
+{
+  return std_cxx14::make_unique<HalfHyperBallBoundary<dim> >(this->get_center(),
+                                                             this->get_radius());
+}
+
+
+
 template <int dim>
 Point<dim>
 HalfHyperBallBoundary<dim>::
@@ -933,6 +997,16 @@ HyperShellBoundary<dim>::HyperShellBoundary (const Point<dim> &center)
 }
 
 
+
+template<int dim>
+std::unique_ptr<Manifold<dim,dim> >
+HyperShellBoundary<dim>::clone() const
+{
+  return std_cxx14::make_unique<HyperShellBoundary<dim> >(this->get_center());
+}
+
+
+
 /* ---------------------------------------------------------------------- */
 
 
@@ -956,6 +1030,17 @@ HalfHyperShellBoundary<dim>::HalfHyperShellBoundary (const Point<dim> &center,
 
 
 
+template<int dim>
+std::unique_ptr<Manifold<dim,dim> >
+HalfHyperShellBoundary<dim>::clone() const
+{
+  return std_cxx14::make_unique<HalfHyperShellBoundary<dim> >(this->get_center(),
+                                                              inner_radius,
+                                                              outer_radius);
+}
+
+
+
 template <int dim>
 Point<dim>
 HalfHyperShellBoundary<dim>::
@@ -1192,11 +1277,11 @@ get_normals_at_vertices (const typename Triangulation<dim>::face_iterator &face,
 
 
 template <int dim, int spacedim>
-TorusBoundary<dim,spacedim>::TorusBoundary (const double R__,
-                                            const double r__)
+TorusBoundary<dim,spacedim>::TorusBoundary (const double R_,
+                                            const double r_)
   :
-  R(R__),
-  r(r__)
+  R(R_),
+  r(r_)
 {
   Assert (false, ExcNotImplemented());
 }
@@ -1204,15 +1289,22 @@ TorusBoundary<dim,spacedim>::TorusBoundary (const double R__,
 
 
 template <>
-TorusBoundary<2,3>::TorusBoundary (const double R__,
-                                   const double r__)
+TorusBoundary<2,3>::TorusBoundary (const double R_,
+                                   const double r_)
   :
-  R(R__),
-  r(r__)
+  R(R_),
+  r(r_)
 {
   Assert (R>r, ExcMessage("Outer radius must be greater than inner radius."));
 }
 
+template<int dim, int spacedim>
+std::unique_ptr<Manifold<dim, spacedim> >
+TorusBoundary<dim,spacedim>::clone() const
+{
+  return std_cxx14::make_unique<TorusBoundary<dim,spacedim> >(R,r);
+}
+
 
 
 template <int dim, int spacedim>
@@ -1373,7 +1465,7 @@ TorusBoundary<2,3>::get_surf_norm(const Point<3> &p) const
 
 
 
-template<>
+template <>
 void
 TorusBoundary<2,3>::
 get_intermediate_points_on_line (const Triangulation<2, 3>::line_iterator   &line,
@@ -1393,7 +1485,7 @@ get_intermediate_points_on_line (const Triangulation<2, 3>::line_iterator   &lin
   offset[1]=0;
 
   //Take care for periodic conditions & negative angles, see
-  //get_new_point_on_line() above. Because we dont have a symmetric
+  //get_new_point_on_line() above. Because we don't have a symmetric
   //interpolation (just the middle) we need to add 2*Pi to each almost zero
   //and negative angles.
   for (unsigned int i=0; i<2; i++)
@@ -1424,7 +1516,7 @@ get_intermediate_points_on_line (const Triangulation<2, 3>::line_iterator   &lin
 
 
 
-template<>
+template <>
 void
 TorusBoundary<2,3>::
 get_intermediate_points_on_quad (const Triangulation< 2, 3 >::quad_iterator &quad,
@@ -1447,7 +1539,7 @@ get_intermediate_points_on_quad (const Triangulation< 2, 3 >::quad_iterator &qua
   offset[1]=0;
 
   //Take care for periodic conditions & negative angles, see
-  //get_new_point_on_line() above.  Because we dont have a symmetric
+  //get_new_point_on_line() above.  Because we don't have a symmetric
   //interpolation (just the middle) we need to add 2*Pi to each almost zero
   //and negative angles.
   for (unsigned int i=0; i<2; i++)
@@ -1484,7 +1576,7 @@ get_intermediate_points_on_quad (const Triangulation< 2, 3 >::quad_iterator &qua
 
 
 
-template<>
+template <>
 void
 TorusBoundary<2,3>::
 get_normals_at_vertices (const Triangulation<2,3 >::face_iterator &face,
