@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 2006 - 2016 by the deal.II authors
+// Copyright (C) 2006 - 2018 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -14,7 +14,6 @@
 // ---------------------------------------------------------------------
 
 #include <deal.II/base/memory_consumption.h>
-#include <deal.II/base/std_cxx11/bind.h>
 #include <deal.II/grid/tria_objects.h>
 #include <deal.II/grid/tria.h>
 #include <deal.II/grid/tria_iterator.h>
@@ -29,9 +28,9 @@ DEAL_II_NAMESPACE_OPEN
 
 namespace internal
 {
-  namespace Triangulation
+  namespace TriangulationImplementation
   {
-    template<class G>
+    template <class G>
     void
     TriaObjects<G>::reserve_space (const unsigned int new_objects_in_pairs,
                                    const unsigned int new_objects_single)
@@ -171,7 +170,7 @@ namespace internal
       const unsigned int new_size = new_hexes +
                                     std::count_if (used.begin(),
                                                    used.end(),
-                                                   std_cxx11::bind (std::equal_to<bool>(), std_cxx11::_1, true));
+                                                   std::bind (std::equal_to<bool>(), std::placeholders::_1, true));
 
       // see above...
       if (new_size>cells.size())
@@ -308,7 +307,7 @@ namespace internal
     }
 
 
-    template<>
+    template <>
     void
     TriaObjects<TriaObject<1> >::monitor_memory (const unsigned int) const
     {
@@ -327,7 +326,7 @@ namespace internal
     }
 
 
-    template<>
+    template <>
     void
     TriaObjects<TriaObject<2> >::monitor_memory (const unsigned int) const
     {
@@ -427,7 +426,7 @@ namespace internal
     }
 
 
-    template<typename G>
+    template <typename G>
     std::size_t
     TriaObjects<G>::memory_consumption () const
     {
