@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 2003 - 2016 by the deal.II authors
+// Copyright (C) 2003 - 2018 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -86,10 +86,6 @@ namespace hp
     return numbers::invalid_unsigned_int;
   }
 
-  template <int dim, int spacedim>
-  FECollection<dim,spacedim>::FECollection ()
-  {}
-
 
 
   template <int dim, int spacedim>
@@ -100,58 +96,6 @@ namespace hp
 
 
 
-  template <int dim, int spacedim>
-  FECollection<dim,spacedim>::FECollection (const FiniteElement<dim,spacedim> &fe1,
-                                            const FiniteElement<dim,spacedim> &fe2)
-  {
-    push_back(fe1);
-    push_back(fe2);
-  }
-
-
-
-  template <int dim, int spacedim>
-  FECollection<dim,spacedim>::FECollection (const FiniteElement<dim,spacedim> &fe1,
-                                            const FiniteElement<dim,spacedim> &fe2,
-                                            const FiniteElement<dim,spacedim> &fe3)
-  {
-    push_back(fe1);
-    push_back(fe2);
-    push_back(fe3);
-  }
-
-
-
-  template <int dim, int spacedim>
-  FECollection<dim,spacedim>::FECollection (const FiniteElement<dim,spacedim> &fe1,
-                                            const FiniteElement<dim,spacedim> &fe2,
-                                            const FiniteElement<dim,spacedim> &fe3,
-                                            const FiniteElement<dim,spacedim> &fe4)
-  {
-    push_back(fe1);
-    push_back(fe2);
-    push_back(fe3);
-    push_back(fe4);
-  }
-
-
-
-  template <int dim, int spacedim>
-  FECollection<dim,spacedim>::FECollection (const FiniteElement<dim,spacedim> &fe1,
-                                            const FiniteElement<dim,spacedim> &fe2,
-                                            const FiniteElement<dim,spacedim> &fe3,
-                                            const FiniteElement<dim,spacedim> &fe4,
-                                            const FiniteElement<dim,spacedim> &fe5)
-  {
-    push_back(fe1);
-    push_back(fe2);
-    push_back(fe3);
-    push_back(fe4);
-    push_back(fe5);
-  }
-
-
-
   template <int dim, int spacedim>
   FECollection<dim,spacedim>::
   FECollection (const std::vector<const FiniteElement<dim,spacedim>*>  &fes)
@@ -165,27 +109,6 @@ namespace hp
 
 
 
-  template <int dim, int spacedim>
-  FECollection<dim,spacedim>::
-  FECollection (const FECollection<dim,spacedim> &fe_collection)
-    :
-    Subscriptor (),
-    // copy the array
-    // of shared
-    // pointers. nothing
-    // bad should
-    // happen -- they
-    // simply all point
-    // to the same
-    // objects, and the
-    // last one to die
-    // will delete the
-    // mappings
-    finite_elements (fe_collection.finite_elements)
-  {}
-
-
-
   template <int dim, int spacedim>
   void FECollection<dim,spacedim>::push_back (const FiniteElement<dim,spacedim> &new_fe)
   {
@@ -199,8 +122,7 @@ namespace hp
               ExcMessage ("All elements inside a collection need to have the "
                           "same number of vector components!"));
 
-    finite_elements
-    .push_back (std_cxx11::shared_ptr<const FiniteElement<dim,spacedim> >(new_fe.clone()));
+    finite_elements.push_back (new_fe.clone());
   }
 
 
