@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 2003 - 2015 by the deal.II authors
+// Copyright (C) 2003 - 2018 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -91,14 +91,14 @@ namespace internal
       // already have an object for
       // this particular combination
       // of indices
-      if (fe_values_table(present_fe_values_index).get() == 0)
+      if (fe_values_table(present_fe_values_index).get() == nullptr)
         fe_values_table(present_fe_values_index)
           =
-            std_cxx11::shared_ptr<FEValuesType>
-            (new FEValuesType ((*mapping_collection)[mapping_index],
-                               (*fe_collection)[fe_index],
-                               q_collection[q_index],
-                               update_flags));
+            std::make_shared<FEValuesType>
+            ((*mapping_collection)[mapping_index],
+             (*fe_collection)[fe_index],
+             q_collection[q_index],
+             update_flags);
 
       // now there definitely is one!
       return *fe_values_table(present_fe_values_index);
