@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 2003 - 2015 by the deal.II authors
+// Copyright (C) 2003 - 2017 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -22,24 +22,17 @@ DEAL_II_NAMESPACE_OPEN
 
 namespace hp
 {
-
-  template<int dim, int spacedim>
-  MappingCollection<dim,spacedim>::MappingCollection ()
-  {}
-
-
-
-  template<int dim, int spacedim>
+  template <int dim, int spacedim>
   MappingCollection<dim,spacedim>::
   MappingCollection (const Mapping<dim,spacedim> &mapping)
   {
     mappings
-    .push_back (std_cxx11::shared_ptr<const Mapping<dim,spacedim> >(mapping.clone()));
+    .push_back (std::shared_ptr<const Mapping<dim,spacedim> >(mapping.clone()));
   }
 
 
 
-  template<int dim, int spacedim>
+  template <int dim, int spacedim>
   MappingCollection<dim,spacedim>::
   MappingCollection (const MappingCollection<dim,spacedim> &mapping_collection)
     :
@@ -60,7 +53,7 @@ namespace hp
 
 
 
-  template<int dim, int spacedim>
+  template <int dim, int spacedim>
   std::size_t
   MappingCollection<dim,spacedim>::memory_consumption () const
   {
@@ -70,12 +63,12 @@ namespace hp
 
 
 
-  template<int dim, int spacedim>
+  template <int dim, int spacedim>
   void
   MappingCollection<dim,spacedim>::push_back (const Mapping<dim,spacedim> &new_mapping)
   {
     mappings
-    .push_back (std_cxx11::shared_ptr<const Mapping<dim,spacedim> >(new_mapping.clone()));
+    .push_back (std::shared_ptr<const Mapping<dim,spacedim> >(new_mapping.clone()));
   }
 
 //---------------------------------------------------------------------------
@@ -92,7 +85,7 @@ namespace hp
      * reference to a static object that will be constructed the first time
      * this function is called.
      */
-    template<int dim, int spacedim>
+    template <int dim, int spacedim>
     MappingQGeneric<dim,spacedim> &
     get_static_mapping_q1()
     {
@@ -101,7 +94,7 @@ namespace hp
     }
   }
 
-  template<int dim, int spacedim>
+  template <int dim, int spacedim>
   MappingCollection<dim,spacedim>
   StaticMappingQ1<dim,spacedim>::mapping_collection
     = MappingCollection<dim,spacedim>(get_static_mapping_q1<dim,spacedim>());
