@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 2008 - 2016 by the deal.II authors
+// Copyright (C) 2008 - 2017 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -40,7 +40,7 @@ ChunkSparsityPattern::ChunkSparsityPattern (const ChunkSparsityPattern &s)
                      "is the sparsity pattern for an empty matrix. This constructor can "
                      "not be used to copy-construct a non-empty sparsity pattern."));
 
-  reinit (0,0,0,0);
+  reinit (0,0,0,chunk_size);
 }
 
 
@@ -91,11 +91,6 @@ ChunkSparsityPattern::ChunkSparsityPattern (
 
 
 
-ChunkSparsityPattern::~ChunkSparsityPattern ()
-{}
-
-
-
 ChunkSparsityPattern &
 ChunkSparsityPattern::operator = (const ChunkSparsityPattern &s)
 {
@@ -477,7 +472,7 @@ ChunkSparsityPattern::n_nonzero_elements () const
 void
 ChunkSparsityPattern::print (std::ostream &out) const
 {
-  Assert ((sparsity_pattern.rowstart!=0) && (sparsity_pattern.colnums!=0),
+  Assert ((sparsity_pattern.rowstart!=nullptr) && (sparsity_pattern.colnums!=nullptr),
           ExcEmptyObject());
 
   AssertThrow (out, ExcIO());
@@ -507,8 +502,8 @@ ChunkSparsityPattern::print (std::ostream &out) const
 void
 ChunkSparsityPattern::print_gnuplot (std::ostream &out) const
 {
-  Assert ((sparsity_pattern.rowstart!=0) &&
-          (sparsity_pattern.colnums!=0), ExcEmptyObject());
+  Assert ((sparsity_pattern.rowstart!=nullptr) &&
+          (sparsity_pattern.colnums!=nullptr), ExcEmptyObject());
 
   AssertThrow (out, ExcIO());
 
