@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 1998 - 2017 by the deal.II authors
+// Copyright (C) 1998 - 2018 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -26,16 +26,13 @@
 #include <deal.II/lac/la_vector.h>
 #include <deal.II/lac/la_parallel_vector.h>
 #include <deal.II/lac/la_parallel_block_vector.h>
-#include <deal.II/lac/petsc_vector.h>
-#include <deal.II/lac/petsc_block_vector.h>
 #include <deal.II/lac/petsc_sparse_matrix.h>
-#include <deal.II/lac/petsc_block_sparse_matrix.h>
 #include <deal.II/lac/petsc_parallel_vector.h>
 #include <deal.II/lac/petsc_parallel_block_vector.h>
 #include <deal.II/lac/petsc_parallel_sparse_matrix.h>
 #include <deal.II/lac/petsc_parallel_block_sparse_matrix.h>
 #include <deal.II/lac/trilinos_vector.h>
-#include <deal.II/lac/trilinos_block_vector.h>
+#include <deal.II/lac/trilinos_parallel_block_vector.h>
 #include <deal.II/lac/trilinos_sparse_matrix.h>
 #include <deal.II/lac/trilinos_block_sparse_matrix.h>
 #include <deal.II/lac/matrix_block.h>
@@ -45,6 +42,7 @@
 #include <numeric>
 #include <set>
 #include <ostream>
+#include <boost/serialization/utility.hpp>
 
 DEAL_II_NAMESPACE_OPEN
 
@@ -55,6 +53,17 @@ const Table<2,bool> ConstraintMatrix::default_empty_table = Table<2,bool>();
 
 
 
+void
+ConstraintMatrix::copy_from (const ConstraintMatrix &other)
+{
+  lines       = other.lines;
+  lines_cache = other.lines_cache;
+  local_lines = other.local_lines;
+  sorted      = other.sorted;
+}
+
+
+
 bool
 ConstraintMatrix::check_zero_weight (const std::pair<size_type, double> &p)
 {
@@ -66,7 +75,7 @@ ConstraintMatrix::check_zero_weight (const std::pair<size_type, double> &p)
 bool
 ConstraintMatrix::ConstraintLine::operator < (const ConstraintLine &a) const
 {
-  return line < a.line;
+  return index < a.index;
 }
 
 
@@ -74,7 +83,7 @@ ConstraintMatrix::ConstraintLine::operator < (const ConstraintLine &a) const
 bool
 ConstraintMatrix::ConstraintLine::operator == (const ConstraintLine &a) const
 {
-  return line == a.line;
+  return index == a.index;
 }
 
 
@@ -82,13 +91,107 @@ ConstraintMatrix::ConstraintLine::operator == (const ConstraintLine &a) const
 std::size_t
 ConstraintMatrix::ConstraintLine::memory_consumption () const
 {
-  return (MemoryConsumption::memory_consumption (line) +
+  return (MemoryConsumption::memory_consumption (index) +
           MemoryConsumption::memory_consumption (entries) +
           MemoryConsumption::memory_consumption (inhomogeneity));
 }
 
 
 
+const ConstraintMatrix::LineRange ConstraintMatrix::get_lines() const
+{
+  return boost::make_iterator_range(lines.begin(), lines.end());
+}
+
+
+
+bool ConstraintMatrix::is_consistent_in_parallel(const std::vector<IndexSet> &locally_owned_dofs,
+                                                 const IndexSet &locally_active_dofs,
+                                                 const MPI_Comm mpi_communicator,
+                                                 const bool verbose) const
+{
+  ConstraintLine empty;
+  empty.inhomogeneity = 0.0;
+
+  // Helper to return a reference to the ConstraintLine object that belongs to row @p row.
+  // We don't want to make copies but to return a reference, we need an empty object that
+  // we store above.
+  auto get_line = [&] (const size_type row) -> const ConstraintLine&
+  {
+    const size_type line_index = calculate_line_index(row);
+    if (line_index >= lines_cache.size() ||
+        lines_cache[line_index] == numbers::invalid_size_type)
+      {
+        empty.index = row;
+        return empty;
+      }
+    else
+      return lines[lines_cache[line_index]];
+  };
+
+  // identify non-owned rows and send to owner:
+  std::map< unsigned int, std::vector<ConstraintLine> > to_send;
+
+  const unsigned int myid = dealii::Utilities::MPI::this_mpi_process(mpi_communicator);
+  const unsigned int nproc = dealii::Utilities::MPI::n_mpi_processes(mpi_communicator);
+
+  // We will send all locally active dofs that are not locally owned for checking. Note
+  // that we allow constraints to differ on locally_relevant (and not active) DoFs.
+  IndexSet non_owned = locally_active_dofs;
+  non_owned.subtract_set(locally_owned_dofs[myid]);
+  for (unsigned int owner=0; owner<nproc; ++owner)
+    {
+      // find all lines to send to @p owner
+      IndexSet indices_to_send = non_owned & locally_owned_dofs[owner];
+      for (const auto &row_idx : indices_to_send)
+        {
+          to_send[owner].push_back(get_line(row_idx));
+        }
+    }
+
+  std::map<unsigned int, std::vector<ConstraintLine> > received = Utilities::MPI::some_to_some (mpi_communicator, to_send);
+
+  unsigned int inconsistent = 0;
+
+  // from each processor:
+  for (const auto &kv : received)
+    {
+      // for each incoming line:
+      for (auto &lineit : kv.second)
+        {
+          const ConstraintLine &reference = get_line(lineit.index);
+
+          if (lineit.inhomogeneity != reference.inhomogeneity)
+            {
+              ++inconsistent;
+
+              if (verbose)
+                std::cout << "Proc " << myid
+                          << " got line " << lineit.index
+                          << " from " << kv.first
+                          << " inhomogeneity " << lineit.inhomogeneity << " != " << reference.inhomogeneity << std::endl;
+            }
+          else if (lineit.entries != reference.entries)
+            {
+              ++inconsistent;
+              if (verbose)
+                std::cout << "Proc " << myid
+                          << " got line " << lineit.index
+                          << " from " << kv.first
+                          << " wrong values!"
+                          << std::endl;
+            }
+        }
+    }
+
+  const unsigned int total = Utilities::MPI::sum(inconsistent, mpi_communicator);
+  if (verbose && total>0 && myid==0)
+    std::cout << total << " inconsistent lines discovered!" << std::endl;
+  return total==0;
+}
+
+
+
 void
 ConstraintMatrix::add_lines (const std::set<size_type> &lines)
 {
@@ -127,7 +230,7 @@ ConstraintMatrix::add_entries
   Assert (is_constrained(line), ExcLineInexistant(line));
 
   ConstraintLine *line_ptr = &lines[lines_cache[calculate_line_index(line)]];
-  Assert (line_ptr->line == line, ExcInternalError());
+  Assert (line_ptr->index == line, ExcInternalError());
 
   // if in debug mode, check whether an entry for this column already
   // exists and if its the same as the one entered at present
@@ -166,13 +269,13 @@ void ConstraintMatrix::add_selected_constraints
   if (constraints.n_constraints() == 0)
     return;
 
-  Assert (filter.size() > constraints.lines.back().line,
+  Assert (filter.size() > constraints.lines.back().index,
           ExcMessage ("Filter needs to be larger than constraint matrix size."));
   for (std::vector<ConstraintLine>::const_iterator line=constraints.lines.begin();
        line!=constraints.lines.end(); ++line)
-    if (filter.is_element(line->line))
+    if (filter.is_element(line->index))
       {
-        const size_type row = filter.index_within_set (line->line);
+        const size_type row = filter.index_within_set (line->index);
         add_line (row);
         set_inhomogeneity (row, line->inhomogeneity);
         for (size_type i=0; i<line->entries.size(); ++i)
@@ -200,14 +303,14 @@ void ConstraintMatrix::close ()
     size_type counter = 0;
     for (std::vector<ConstraintLine>::const_iterator line=lines.begin();
          line!=lines.end(); ++line, ++counter)
-      new_lines[calculate_line_index(line->line)] = counter;
+      new_lines[calculate_line_index(line->index)] = counter;
     std::swap (lines_cache, new_lines);
   }
 
   // in debug mode: check whether we really set the pointers correctly.
   for (size_type i=0; i<lines_cache.size(); ++i)
     if (lines_cache[i] != numbers::invalid_size_type)
-      Assert (i == calculate_line_index(lines[lines_cache[i]].line),
+      Assert (i == calculate_line_index(lines[lines_cache[i]].index),
               ExcInternalError());
 
   // first, strip zero entries, as we have to do that only once
@@ -287,12 +390,12 @@ void ConstraintMatrix::close ()
                 const size_type  dof_index = line->entries[entry].first;
                 const double     weight = line->entries[entry].second;
 
-                Assert (dof_index != line->line,
+                Assert (dof_index != line->index,
                         ExcMessage ("Cycle in constraints detected!"));
 
                 const ConstraintLine *constrained_line =
                   &lines[lines_cache[calculate_line_index(dof_index)]];
-                Assert (constrained_line->line == dof_index,
+                Assert (constrained_line->index == dof_index,
                         ExcInternalError());
 
                 // now we have to replace an entry by its expansion. we do
@@ -317,10 +420,9 @@ void ConstraintMatrix::close ()
                                       weight);
 
                     for (size_type i=1; i<constrained_line->entries.size(); ++i)
-                      line->entries
-                      .push_back (std::make_pair (constrained_line->entries[i].first,
-                                                  constrained_line->entries[i].second *
-                                                  weight));
+                      line->entries.emplace_back (constrained_line->entries[i].first,
+                                                  constrained_line->entries[i].second
+                                                  * weight);
 
 #ifdef DEBUG
                     // keep track of how many entries we replace in this
@@ -464,7 +566,7 @@ void ConstraintMatrix::close ()
           // make sure that entry->first is not the index of a line itself
           const bool is_circle = is_constrained(entry->first);
           Assert (is_circle == false,
-                  ExcDoFConstrainedToConstrainedDoF(line->line, entry->first));
+                  ExcDoFConstrainedToConstrainedDoF(line->index, entry->first));
         }
 #endif
 
@@ -521,15 +623,14 @@ ConstraintMatrix::merge (const ConstraintMatrix &other_constraints,
             {
               const ConstraintLine::Entries *other_line
                 = other_constraints.get_constraint_entries (line->entries[i].first);
-              Assert (other_line != 0,
+              Assert (other_line != nullptr,
                       ExcInternalError());
 
               const double weight = line->entries[i].second;
 
               for (ConstraintLine::Entries::const_iterator j=other_line->begin();
                    j!=other_line->end(); ++j)
-                tmp.push_back (std::pair<size_type,double>(j->first,
-                                                           j->second*weight));
+                tmp.emplace_back(j->first, j->second*weight);
 
               line->inhomogeneity
               += other_constraints.get_inhomogeneity(line->entries[i].first) *
@@ -553,7 +654,7 @@ ConstraintMatrix::merge (const ConstraintMatrix &other_constraints,
     for (std::vector<ConstraintLine>::const_iterator line = lines.begin();
          line != lines.end(); ++line)
       {
-        size_type local_line_no = calculate_line_index(line->line);
+        size_type local_line_no = calculate_line_index(line->index);
         if (local_line_no >= lines_cache.size())
           lines_cache.resize(local_line_no+1, numbers::invalid_size_type);
         lines_cache[local_line_no] = index++;
@@ -563,7 +664,7 @@ ConstraintMatrix::merge (const ConstraintMatrix &other_constraints,
     for (std::vector<ConstraintLine>::const_iterator line = other_constraints.lines.begin();
          line != other_constraints.lines.end(); ++line)
       {
-        const size_type local_line_no = calculate_line_index(line->line);
+        const size_type local_line_no = calculate_line_index(line->index);
         if (local_line_no >= lines_cache.size())
           {
             lines_cache.resize(local_line_no+1, numbers::invalid_size_type);
@@ -584,7 +685,7 @@ ConstraintMatrix::merge (const ConstraintMatrix &other_constraints,
               {
               case no_conflicts_allowed:
                 AssertThrow (false,
-                             ExcDoFIsConstrainedFromBothObjects (line->line));
+                             ExcDoFIsConstrainedFromBothObjects (line->index));
                 break;
 
               case left_object_wins:
@@ -605,7 +706,7 @@ ConstraintMatrix::merge (const ConstraintMatrix &other_constraints,
     // check that we set the pointers correctly
     for (size_type i=0; i<lines_cache.size(); ++i)
       if (lines_cache[i] != numbers::invalid_size_type)
-        Assert (i == calculate_line_index(lines[lines_cache[i]].line),
+        Assert (i == calculate_line_index(lines[lines_cache[i]].index),
                 ExcInternalError());
   }
 
@@ -633,7 +734,7 @@ void ConstraintMatrix::shift (const size_type offset)
   for (std::vector<ConstraintLine>::iterator i = lines.begin();
        i != lines.end(); ++i)
     {
-      i->line += offset;
+      i->index += offset;
       for (ConstraintLine::Entries::iterator
            j = i->entries.begin();
            j != i->entries.end(); ++j)
@@ -645,7 +746,7 @@ void ConstraintMatrix::shift (const size_type offset)
   // are still linked correctly
   for (size_type i=0; i<lines_cache.size(); ++i)
     Assert(lines_cache[i] == numbers::invalid_size_type ||
-           calculate_line_index(lines[lines_cache[i]].line) == i,
+           calculate_line_index(lines[lines_cache[i]].index) == i,
            ExcInternalError());
 #endif
 }
@@ -697,7 +798,7 @@ void ConstraintMatrix::condense (SparsityPattern &sparsity) const
                                     numbers::invalid_size_type);
 
   for (size_type c=0; c<lines.size(); ++c)
-    distribute[lines[c].line] = c;
+    distribute[lines[c].index] = c;
 
   const size_type n_rows = sparsity.n_rows();
   for (size_type row=0; row<n_rows; ++row)
@@ -781,7 +882,7 @@ void ConstraintMatrix::condense (DynamicSparsityPattern &sparsity) const
                                     numbers::invalid_size_type);
 
   for (size_type c=0; c<lines.size(); ++c)
-    distribute[lines[c].line] = c;
+    distribute[lines[c].index] = c;
 
   const size_type n_rows = sparsity.n_rows();
   for (size_type row=0; row<n_rows; ++row)
@@ -825,8 +926,8 @@ void ConstraintMatrix::condense (DynamicSparsityPattern &sparsity) const
                     if ((new_col < column) && (old_rowlength != new_rowlength))
                       ++j;
                     old_rowlength = new_rowlength;
-                  };
-              };
+                  }
+              }
           }
       else
         // row must be distributed
@@ -851,8 +952,8 @@ void ConstraintMatrix::condense (DynamicSparsityPattern &sparsity) const
                   sparsity.add (lines[distribute[row]].entries[p].first,
                                 lines[distribute[sparsity.column_number(row,j)]]
                                 .entries[q].first);
-          };
-    };
+          }
+    }
 }
 
 
@@ -881,7 +982,7 @@ void ConstraintMatrix::condense (BlockSparsityPattern &sparsity) const
                                      numbers::invalid_size_type);
 
   for (size_type c=0; c<lines.size(); ++c)
-    distribute[lines[c].line] = c;
+    distribute[lines[c].index] = c;
 
   const size_type n_rows = sparsity.n_rows();
   for (size_type row=0; row<n_rows; ++row)
@@ -992,7 +1093,7 @@ void ConstraintMatrix::condense (BlockDynamicSparsityPattern &sparsity) const
                                      numbers::invalid_size_type);
 
   for (size_type c=0; c<lines.size(); ++c)
-    distribute[lines[c].line] = static_cast<signed int>(c);
+    distribute[lines[c].index] = static_cast<signed int>(c);
 
   const size_type n_rows = sparsity.n_rows();
   for (size_type row=0; row<n_rows; ++row)
@@ -1036,9 +1137,9 @@ void ConstraintMatrix::condense (BlockDynamicSparsityPattern &sparsity) const
                            .entries.size(); ++q)
                         sparsity.add (row,
                                       lines[distribute[global_col]].entries[q].first);
-                    };
-                };
-            };
+                    }
+                }
+            }
         }
       else
         {
@@ -1073,11 +1174,11 @@ void ConstraintMatrix::condense (BlockDynamicSparsityPattern &sparsity) const
                         for (size_type q=0; q!=lines[distribute[global_col]].entries.size(); ++q)
                           sparsity.add (lines[distribute[row]].entries[p].first,
                                         lines[distribute[global_col]].entries[q].first);
-                    };
-                };
-            };
-        };
-    };
+                    }
+                }
+            }
+        }
+    }
 }
 
 
@@ -1088,7 +1189,7 @@ bool ConstraintMatrix::is_identity_constrained (const size_type index) const
     return false;
 
   const ConstraintLine &p = lines[lines_cache[calculate_line_index(index)]];
-  Assert (p.line == index, ExcInternalError());
+  Assert (p.index == index, ExcInternalError());
 
   // return if an entry for this line was found and if it has only one
   // entry equal to 1.0
@@ -1103,7 +1204,7 @@ bool ConstraintMatrix::are_identity_constrained (const size_type index1,
   if (is_constrained(index1) == true)
     {
       const ConstraintLine &p = lines[lines_cache[calculate_line_index(index1)]];
-      Assert (p.line == index1, ExcInternalError());
+      Assert (p.index == index1, ExcInternalError());
 
       // return if an entry for this line was found and if it has only one
       // entry equal to 1.0 and that one is index2
@@ -1114,7 +1215,7 @@ bool ConstraintMatrix::are_identity_constrained (const size_type index1,
   else if (is_constrained(index2) == true)
     {
       const ConstraintLine &p = lines[lines_cache[calculate_line_index(index2)]];
-      Assert (p.line == index2, ExcInternalError());
+      Assert (p.index == index2, ExcInternalError());
 
       // return if an entry for this line was found and if it has only one
       // entry equal to 1.0 and that one is index1
@@ -1163,13 +1264,13 @@ void ConstraintMatrix::print (std::ostream &out) const
       if (lines[i].entries.size() > 0)
         {
           for (size_type j=0; j<lines[i].entries.size(); ++j)
-            out << "    " << lines[i].line
+            out << "    " << lines[i].index
                 << " " << lines[i].entries[j].first
                 << ":  " << lines[i].entries[j].second << "\n";
 
           // print out inhomogeneity.
           if (lines[i].inhomogeneity != 0)
-            out << "    " << lines[i].line
+            out << "    " << lines[i].index
                 << ": " << lines[i].inhomogeneity << "\n";
         }
       else
@@ -1178,11 +1279,11 @@ void ConstraintMatrix::print (std::ostream &out) const
         // combination of other dofs
         {
           if (lines[i].inhomogeneity != 0)
-            out << "    " << lines[i].line
+            out << "    " << lines[i].index
                 << " = " << lines[i].inhomogeneity
                 << "\n";
           else
-            out << "    " << lines[i].line << " = 0\n";
+            out << "    " << lines[i].index << " = 0\n";
         }
     }
 
@@ -1201,12 +1302,12 @@ ConstraintMatrix::write_dot (std::ostream &out) const
       // same concept as in the previous function
       if (lines[i].entries.size() > 0)
         for (size_type j=0; j<lines[i].entries.size(); ++j)
-          out << "  " << lines[i].line << "->" << lines[i].entries[j].first
+          out << "  " << lines[i].index << "->" << lines[i].entries[j].first
               << "; // weight: "
               << lines[i].entries[j].second
               << "\n";
       else
-        out << "  " << lines[i].line << "\n";
+        out << "  " << lines[i].index << "\n";
     }
   out << "}" << std::endl;
 }
@@ -1234,7 +1335,7 @@ ConstraintMatrix::resolve_indices (std::vector<types::global_dof_index> &indices
       line_ptr = get_constraint_entries(indices[i]);
       // if the index is constraint, the constraints indices are added to the
       // indices vector
-      if (line_ptr!=NULL)
+      if (line_ptr!=nullptr)
         {
           const unsigned int line_size = line_ptr->size();
           for (unsigned int j=0; j<line_size; ++j)
@@ -1310,16 +1411,16 @@ PARALLEL_VECTOR_FUNCTIONS(TrilinosWrappers::MPI::BlockVector);
                                                       MatrixType                      &, \
                                                       VectorType                      &, \
                                                       bool                             , \
-                                                      internal::bool2type<false>) const
-#define MATRIX_FUNCTIONS(MatrixType) \
+                                                      std::integral_constant<bool, false>) const
+#define MATRIX_FUNCTIONS(MatrixType,VectorScalar) \
   template void ConstraintMatrix:: \
-  distribute_local_to_global<MatrixType,Vector<MatrixType::value_type> > (const FullMatrix<MatrixType::value_type>        &, \
-      const Vector<MatrixType::value_type>            &, \
+  distribute_local_to_global<MatrixType,Vector<VectorScalar> > (const FullMatrix<MatrixType::value_type>        &, \
+      const Vector<VectorScalar>            &, \
       const std::vector<ConstraintMatrix::size_type> &, \
       MatrixType                      &, \
-      Vector<MatrixType::value_type>                  &, \
+      Vector<VectorScalar>                  &, \
       bool                             , \
-      internal::bool2type<false>) const
+      std::integral_constant<bool, false>) const
 #define BLOCK_MATRIX_VECTOR_FUNCTIONS(MatrixType, VectorType)   \
   template void ConstraintMatrix:: \
   distribute_local_to_global<MatrixType,VectorType > (const FullMatrix<MatrixType::value_type>        &, \
@@ -1328,7 +1429,7 @@ PARALLEL_VECTOR_FUNCTIONS(TrilinosWrappers::MPI::BlockVector);
                                                       MatrixType                      &, \
                                                       VectorType                      &, \
                                                       bool                             , \
-                                                      internal::bool2type<true>) const
+                                                      std::integral_constant<bool, true>) const
 #define BLOCK_MATRIX_FUNCTIONS(MatrixType)      \
   template void ConstraintMatrix:: \
   distribute_local_to_global<MatrixType,Vector<MatrixType::value_type> > (const FullMatrix<MatrixType::value_type>        &, \
@@ -1337,45 +1438,46 @@ PARALLEL_VECTOR_FUNCTIONS(TrilinosWrappers::MPI::BlockVector);
       MatrixType                      &, \
       Vector<MatrixType::value_type>                  &, \
       bool                             , \
-      internal::bool2type<true>) const
+      std::integral_constant<bool, true>) const
+
+MATRIX_FUNCTIONS(FullMatrix<double>,double);
+MATRIX_FUNCTIONS(FullMatrix<float>,float);
+MATRIX_FUNCTIONS(FullMatrix<double>,std::complex<double>);
+MATRIX_FUNCTIONS(FullMatrix<std::complex<double> >,std::complex<double>);
+
+MATRIX_FUNCTIONS(SparseMatrix<double>,double);
+MATRIX_FUNCTIONS(SparseMatrix<float>,float);
+MATRIX_FUNCTIONS(SparseMatrix<double>,std::complex<double>);
+MATRIX_FUNCTIONS(SparseMatrix<float>,std::complex<float>);
+MATRIX_FUNCTIONS(SparseMatrix<std::complex<double> >,std::complex<double>);
+MATRIX_FUNCTIONS(SparseMatrix<std::complex<float> >,std::complex<float>);
+
+MATRIX_FUNCTIONS(SparseMatrixEZ<double>,double);
+MATRIX_FUNCTIONS(SparseMatrixEZ<float>,float);
+MATRIX_FUNCTIONS(ChunkSparseMatrix<double>,double);
+MATRIX_FUNCTIONS(ChunkSparseMatrix<float>,float);
 
-MATRIX_FUNCTIONS(SparseMatrix<double>);
-MATRIX_FUNCTIONS(SparseMatrix<float>);
-MATRIX_FUNCTIONS(FullMatrix<double>);
-MATRIX_FUNCTIONS(FullMatrix<float>);
-MATRIX_FUNCTIONS(FullMatrix<std::complex<double> >);
-MATRIX_FUNCTIONS(SparseMatrix<std::complex<double> >);
-MATRIX_FUNCTIONS(SparseMatrix<std::complex<float> >);
 
 BLOCK_MATRIX_FUNCTIONS(BlockSparseMatrix<double>);
 BLOCK_MATRIX_FUNCTIONS(BlockSparseMatrix<float>);
 BLOCK_MATRIX_VECTOR_FUNCTIONS(BlockSparseMatrix<double>, BlockVector<double>);
 BLOCK_MATRIX_VECTOR_FUNCTIONS(BlockSparseMatrix<float>,  BlockVector<float>);
 
-MATRIX_FUNCTIONS(SparseMatrixEZ<double>);
-MATRIX_FUNCTIONS(SparseMatrixEZ<float>);
-MATRIX_FUNCTIONS(ChunkSparseMatrix<double>);
-MATRIX_FUNCTIONS(ChunkSparseMatrix<float>);
-
 // BLOCK_MATRIX_FUNCTIONS(BlockSparseMatrixEZ<double>);
 // BLOCK_MATRIX_VECTOR_FUNCTIONS(BlockSparseMatrixEZ<float>,  Vector<float>);
 
 #ifdef DEAL_II_WITH_PETSC
-MATRIX_FUNCTIONS(PETScWrappers::SparseMatrix);
-BLOCK_MATRIX_FUNCTIONS(PETScWrappers::BlockSparseMatrix);
-MATRIX_FUNCTIONS(PETScWrappers::MPI::SparseMatrix);
+MATRIX_FUNCTIONS(PETScWrappers::SparseMatrix,PetscScalar);
+MATRIX_FUNCTIONS(PETScWrappers::MPI::SparseMatrix,PetscScalar);
 BLOCK_MATRIX_FUNCTIONS(PETScWrappers::MPI::BlockSparseMatrix);
-MATRIX_VECTOR_FUNCTIONS(PETScWrappers::SparseMatrix, PETScWrappers::Vector);
-BLOCK_MATRIX_VECTOR_FUNCTIONS(PETScWrappers::BlockSparseMatrix, PETScWrappers::BlockVector);
 MATRIX_VECTOR_FUNCTIONS(PETScWrappers::MPI::SparseMatrix, PETScWrappers::MPI::Vector);
-BLOCK_MATRIX_VECTOR_FUNCTIONS(PETScWrappers::MPI::BlockSparseMatrix ,PETScWrappers::MPI::BlockVector);
+MATRIX_VECTOR_FUNCTIONS(PETScWrappers::SparseMatrix, PETScWrappers::MPI::Vector);
+BLOCK_MATRIX_VECTOR_FUNCTIONS(PETScWrappers::MPI::BlockSparseMatrix,PETScWrappers::MPI::BlockVector);
 #endif
 
 #ifdef DEAL_II_WITH_TRILINOS
-MATRIX_FUNCTIONS(TrilinosWrappers::SparseMatrix);
+MATRIX_FUNCTIONS(TrilinosWrappers::SparseMatrix,double);
 BLOCK_MATRIX_FUNCTIONS(TrilinosWrappers::BlockSparseMatrix);
-MATRIX_VECTOR_FUNCTIONS(TrilinosWrappers::SparseMatrix, TrilinosWrappers::Vector);
-BLOCK_MATRIX_VECTOR_FUNCTIONS(TrilinosWrappers::BlockSparseMatrix, TrilinosWrappers::BlockVector);
 MATRIX_VECTOR_FUNCTIONS(TrilinosWrappers::SparseMatrix, TrilinosWrappers::MPI::Vector);
 BLOCK_MATRIX_VECTOR_FUNCTIONS(TrilinosWrappers::BlockSparseMatrix, TrilinosWrappers::MPI::BlockVector);
 #endif
@@ -1387,7 +1489,7 @@ BLOCK_MATRIX_VECTOR_FUNCTIONS(TrilinosWrappers::BlockSparseMatrix, TrilinosWrapp
       SparsityPatternType &,                                                         \
       const bool,                                                                    \
       const Table<2,bool> &,                                                         \
-      internal::bool2type<false>) const;                                             \
+      std::integral_constant<bool, false>) const;                                             \
   template void ConstraintMatrix::add_entries_local_to_global<SparsityPatternType> ( \
       const std::vector<ConstraintMatrix::size_type> &,                              \
       const std::vector<ConstraintMatrix::size_type> &,                              \
@@ -1400,7 +1502,7 @@ BLOCK_MATRIX_VECTOR_FUNCTIONS(TrilinosWrappers::BlockSparseMatrix, TrilinosWrapp
       SparsityPatternType &,                                                         \
       const bool,                                                                    \
       const Table<2,bool> &,                                                         \
-      internal::bool2type<true>) const;                                              \
+      std::integral_constant<bool, true>) const;                                              \
   template void ConstraintMatrix::add_entries_local_to_global<SparsityPatternType> ( \
       const std::vector<ConstraintMatrix::size_type> &,                              \
       const std::vector<ConstraintMatrix::size_type> &,                              \
@@ -1419,11 +1521,17 @@ BLOCK_SPARSITY_FUNCTIONS(TrilinosWrappers::BlockSparsityPattern);
 #endif
 
 
-#define ONLY_MATRIX_FUNCTIONS(MatrixType) \
-  template void ConstraintMatrix::distribute_local_to_global<MatrixType > (\
-      const FullMatrix<MatrixType::value_type>        &, \
-      const std::vector<ConstraintMatrix::size_type> &, \
-      const std::vector<ConstraintMatrix::size_type> &, \
+#define ONLY_MATRIX_FUNCTIONS(MatrixType)                                   \
+  template void ConstraintMatrix::distribute_local_to_global<MatrixType > ( \
+      const FullMatrix<MatrixType::value_type>        &,                    \
+      const std::vector<ConstraintMatrix::size_type> &,                     \
+      const std::vector<ConstraintMatrix::size_type> &,                     \
+      MatrixType                      &) const;                             \
+  template void ConstraintMatrix::distribute_local_to_global<MatrixType > ( \
+      const FullMatrix<MatrixType::value_type>        &,                      \
+      const std::vector<ConstraintMatrix::size_type> &,                       \
+      const ConstraintMatrix &,                                               \
+      const std::vector<ConstraintMatrix::size_type> &,                       \
       MatrixType                      &) const
 
 ONLY_MATRIX_FUNCTIONS(FullMatrix<float>);
@@ -1442,7 +1550,6 @@ ONLY_MATRIX_FUNCTIONS(TrilinosWrappers::BlockSparseMatrix);
 
 #ifdef DEAL_II_WITH_PETSC
 ONLY_MATRIX_FUNCTIONS(PETScWrappers::SparseMatrix);
-ONLY_MATRIX_FUNCTIONS(PETScWrappers::BlockSparseMatrix);
 ONLY_MATRIX_FUNCTIONS(PETScWrappers::MPI::SparseMatrix);
 ONLY_MATRIX_FUNCTIONS(PETScWrappers::MPI::BlockSparseMatrix);
 #endif
@@ -1455,15 +1562,17 @@ ONLY_MATRIX_FUNCTIONS(PETScWrappers::MPI::BlockSparseMatrix);
 // constructor of scratch_data (it won't allow one to be constructed in place).
 namespace internals
 {
-#define SCRATCH_INITIALIZER(Number,Name)                                \
-  ConstraintMatrixData<Number>::ScratchData scratch_data_initializer_##Name; \
-  template<> Threads::ThreadLocalStorage<ConstraintMatrixData<Number>::ScratchData> \
-  ConstraintMatrixData<Number>::scratch_data(scratch_data_initializer_##Name)
+#define SCRATCH_INITIALIZER(MatrixScalar,VectorScalar,Name)                  \
+  ConstraintMatrixData<MatrixScalar,VectorScalar>::ScratchData scratch_data_initializer_##Name; \
+  template <> Threads::ThreadLocalStorage<ConstraintMatrixData<MatrixScalar,VectorScalar>::ScratchData> \
+  ConstraintMatrixData<MatrixScalar,VectorScalar>::scratch_data(scratch_data_initializer_##Name)
 
-  SCRATCH_INITIALIZER(double,double);
-  SCRATCH_INITIALIZER(float,float);
-  SCRATCH_INITIALIZER(std::complex<double>,cdouble);
-  SCRATCH_INITIALIZER(std::complex<float>,cfloat);
+  SCRATCH_INITIALIZER(double,double,dd);
+  SCRATCH_INITIALIZER(float,float,ff);
+  SCRATCH_INITIALIZER(std::complex<double>,std::complex<double>,zz);
+  SCRATCH_INITIALIZER(std::complex<float>,std::complex<float>,cc);
+  SCRATCH_INITIALIZER(double,std::complex<double>,dz);
+  SCRATCH_INITIALIZER(float,std::complex<float>,fc);
 #undef SCRATCH_INITIALIZER
 }
 
