@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 2004 - 2016 by the deal.II authors
+// Copyright (C) 2004 - 2017 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -56,7 +56,7 @@ namespace PETScWrappers
     // use the call sequence indicating only a maximal number of
     // elements per row for all rows globally
     const PetscErrorCode ierr = MatCreateSeqDense (PETSC_COMM_SELF, m, n,
-                                                   PETSC_NULL, &matrix);
+                                                   nullptr, &matrix);
     AssertThrow (ierr == 0, ExcPETScError(ierr));
   }
 
