@@ -32,7 +32,7 @@ namespace PETScWrappers
 {
   PreconditionerBase::PreconditionerBase ()
     :
-    pc(NULL), matrix(NULL)
+    pc(nullptr), matrix(nullptr)
   {}
 
   PreconditionerBase::~PreconditionerBase ()
@@ -48,16 +48,12 @@ namespace PETScWrappers
   void
   PreconditionerBase::clear ()
   {
-    matrix = NULL;
+    matrix = nullptr;
 
-    if (pc!=NULL)
+    if (pc!=nullptr)
       {
-#if DEAL_II_PETSC_VERSION_LT(3,2,0)
-        PetscErrorCode ierr = PCDestroy(pc);
-#else
         PetscErrorCode ierr = PCDestroy(&pc);
-#endif
-        pc = NULL;
+        pc = nullptr;
         AssertThrow (ierr == 0, ExcPETScError(ierr));
       }
   }
@@ -67,7 +63,7 @@ namespace PETScWrappers
   PreconditionerBase::vmult (VectorBase       &dst,
                              const VectorBase &src) const
   {
-    AssertThrow (pc != NULL, StandardExceptions::ExcInvalidState ());
+    AssertThrow (pc != nullptr, StandardExceptions::ExcInvalidState ());
 
     const PetscErrorCode ierr = PCApply(pc, src, dst);
     AssertThrow (ierr == 0, ExcPETScError(ierr));
@@ -79,7 +75,7 @@ namespace PETScWrappers
   {
     // only allow the creation of the
     // preconditioner once
-    AssertThrow (pc == NULL, StandardExceptions::ExcInvalidState ());
+    AssertThrow (pc == nullptr, StandardExceptions::ExcInvalidState ());
 
     MPI_Comm comm;
     // this ugly cast is necessary because the
@@ -93,9 +89,9 @@ namespace PETScWrappers
     AssertThrow (ierr == 0, ExcPETScError(ierr));
 
 #if DEAL_II_PETSC_VERSION_LT(3, 5, 0)
-    ierr = PCSetOperators(pc , matrix, matrix, SAME_PRECONDITIONER);
+    ierr = PCSetOperators(pc, matrix, matrix, SAME_PRECONDITIONER);
 #else
-    ierr = PCSetOperators(pc , matrix, matrix);
+    ierr = PCSetOperators(pc, matrix, matrix);
 #endif
     AssertThrow (ierr == 0, ExcPETScError(ierr));
   }
@@ -127,9 +123,6 @@ namespace PETScWrappers
   }
 
 
-  PreconditionJacobi::PreconditionJacobi ()
-  {}
-
 
   PreconditionJacobi::PreconditionJacobi (const MatrixBase     &matrix,
                                           const AdditionalData &additional_data)
@@ -140,7 +133,7 @@ namespace PETScWrappers
   void
   PreconditionJacobi::initialize()
   {
-    AssertThrow (pc != NULL, StandardExceptions::ExcInvalidState ());
+    AssertThrow (pc != nullptr, StandardExceptions::ExcInvalidState ());
 
     PetscErrorCode ierr = PCSetType (pc, const_cast<char *>(PCJACOBI));
     AssertThrow (ierr == 0, ExcPETScError(ierr));
@@ -179,9 +172,6 @@ namespace PETScWrappers
   }
 
 
-  PreconditionBlockJacobi::PreconditionBlockJacobi ()
-  {}
-
 
   PreconditionBlockJacobi::
   PreconditionBlockJacobi (const MatrixBase     &matrix,
@@ -227,9 +217,6 @@ namespace PETScWrappers
   {}
 
 
-  PreconditionSOR::PreconditionSOR ()
-  {}
-
 
   PreconditionSOR::PreconditionSOR (const MatrixBase     &matrix,
                                     const AdditionalData &additional_data)
@@ -273,9 +260,6 @@ namespace PETScWrappers
   {}
 
 
-  PreconditionSSOR::PreconditionSSOR ()
-  {}
-
 
   PreconditionSSOR::PreconditionSSOR (const MatrixBase     &matrix,
                                       const AdditionalData &additional_data)
@@ -323,9 +307,6 @@ namespace PETScWrappers
   {}
 
 
-  PreconditionEisenstat::PreconditionEisenstat ()
-  {}
-
 
   PreconditionEisenstat::PreconditionEisenstat (const MatrixBase     &matrix,
                                                 const AdditionalData &additional_data)
@@ -370,9 +351,6 @@ namespace PETScWrappers
   {}
 
 
-  PreconditionICC::PreconditionICC ()
-  {}
-
 
   PreconditionICC::PreconditionICC (const MatrixBase     &matrix,
                                     const AdditionalData &additional_data)
@@ -416,9 +394,6 @@ namespace PETScWrappers
   {}
 
 
-  PreconditionILU::PreconditionILU ()
-  {}
-
 
   PreconditionILU::PreconditionILU (const MatrixBase     &matrix,
                                     const AdditionalData &additional_data)
@@ -471,8 +446,6 @@ namespace PETScWrappers
   {}
 
 
-  PreconditionBoomerAMG::PreconditionBoomerAMG ()
-  {}
 
   PreconditionBoomerAMG::PreconditionBoomerAMG (const MPI_Comm comm,
                                                 const AdditionalData &additional_data_)
@@ -482,9 +455,9 @@ namespace PETScWrappers
     PetscErrorCode ierr = PCCreate(comm, &pc);
     AssertThrow (ierr == 0, ExcPETScError(ierr));
 
-#ifdef PETSC_HAVE_HYPRE
+#ifdef DEAL_II_PETSC_WITH_HYPRE
     initialize();
-#else // PETSC_HAVE_HYPRE
+#else // DEAL_II_PETSC_WITH_HYPRE
     (void)pc;
     Assert (false,
             ExcMessage ("Your PETSc installation does not include a copy of "
@@ -502,7 +475,7 @@ namespace PETScWrappers
   void
   PreconditionBoomerAMG::initialize ()
   {
-#ifdef PETSC_HAVE_HYPRE
+#ifdef DEAL_II_PETSC_WITH_HYPRE
     PetscErrorCode ierr = PCSetType (pc, const_cast<char *>(PCHYPRE));
     AssertThrow (ierr == 0, ExcPETScError(ierr));
 
@@ -551,7 +524,7 @@ namespace PETScWrappers
   PreconditionBoomerAMG::initialize (const MatrixBase     &matrix_,
                                      const AdditionalData &additional_data_)
   {
-#ifdef PETSC_HAVE_HYPRE
+#ifdef DEAL_II_PETSC_WITH_HYPRE
     clear ();
 
     matrix = static_cast<Mat>(matrix_);
@@ -563,7 +536,7 @@ namespace PETScWrappers
     PetscErrorCode ierr = PCSetUp (pc);
     AssertThrow (ierr == 0, ExcPETScError(ierr));
 
-#else // PETSC_HAVE_HYPRE
+#else // DEAL_II_PETSC_WITH_HYPRE
     (void)matrix_;
     (void)additional_data_;
     Assert (false,
@@ -590,9 +563,6 @@ namespace PETScWrappers
   {}
 
 
-  PreconditionParaSails::PreconditionParaSails ()
-  {}
-
 
   PreconditionParaSails::PreconditionParaSails (const MatrixBase     &matrix,
                                                 const AdditionalData &additional_data)
@@ -610,7 +580,7 @@ namespace PETScWrappers
     matrix = static_cast<Mat>(matrix_);
     additional_data = additional_data_;
 
-#ifdef PETSC_HAVE_HYPRE
+#ifdef DEAL_II_PETSC_WITH_HYPRE
     create_pc();
 
     PetscErrorCode ierr = PCSetType (pc, const_cast<char *>(PCHYPRE));
@@ -675,7 +645,7 @@ namespace PETScWrappers
     ierr = PCSetUp (pc);
     AssertThrow (ierr == 0, ExcPETScError(ierr));
 
-#else // PETSC_HAVE_HYPRE
+#else // DEAL_II_PETSC_WITH_HYPRE
     (void)pc;
     Assert (false,
             ExcMessage ("Your PETSc installation does not include a copy of "
@@ -686,10 +656,6 @@ namespace PETScWrappers
 
   /* ----------------- PreconditionNone ------------------------- */
 
-  PreconditionNone::PreconditionNone ()
-  {}
-
-
   PreconditionNone::PreconditionNone (const MatrixBase     &matrix,
                                       const AdditionalData &additional_data)
   {
@@ -732,9 +698,6 @@ namespace PETScWrappers
   {}
 
 
-  PreconditionLU::PreconditionLU ()
-  {}
-
 
   PreconditionLU::PreconditionLU (const MatrixBase     &matrix,
                                   const AdditionalData &additional_data)
@@ -758,21 +721,13 @@ namespace PETScWrappers
     AssertThrow (ierr == 0, ExcPETScError(ierr));
 
     // set flags as given
-#if DEAL_II_PETSC_VERSION_LT(3,0,1)
-    ierr = PCFactorSetPivoting (pc, additional_data.pivoting);
-#else
     ierr = PCFactorSetColumnPivot (pc, additional_data.pivoting);
-#endif
     AssertThrow (ierr == 0, ExcPETScError(ierr));
 
     ierr = PCFactorSetZeroPivot (pc, additional_data.zero_pivot);
     AssertThrow (ierr == 0, ExcPETScError(ierr));
 
-#if DEAL_II_PETSC_VERSION_LT(3,0,1)
-    ierr = PCFactorSetShiftNonzero (pc, additional_data.damping);
-#else
     ierr = PCFactorSetShiftAmount (pc, additional_data.damping);
-#endif
     AssertThrow (ierr == 0, ExcPETScError(ierr));
 
     ierr = PCSetFromOptions (pc);
