@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 2004 - 2016 by the deal.II authors
+// Copyright (C) 2004 - 2018 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -15,6 +15,7 @@
 
 
 #include <deal.II/base/logstream.h>
+#include <deal.II/base/std_cxx14/memory.h>
 #include <deal.II/lac/petsc_solver.h>
 
 #ifdef DEAL_II_WITH_PETSC
@@ -49,11 +50,6 @@ namespace PETScWrappers
 
 
 
-  SolverBase::~SolverBase ()
-  {}
-
-
-
   void
   SolverBase::solve (const MatrixBase         &A,
                      VectorBase               &x,
@@ -69,9 +65,9 @@ namespace PETScWrappers
 
     // first create a solver object if this
     // is necessary
-    if (solver_data.get() == 0)
+    if (solver_data.get() == nullptr)
       {
-        solver_data.reset (new SolverData());
+        solver_data = std_cxx14::make_unique<SolverData>();
 
         PetscErrorCode ierr = KSPCreate (mpi_communicator, &solver_data->ksp);
         AssertThrow (ierr == 0, ExcPETScError(ierr));
@@ -87,7 +83,7 @@ namespace PETScWrappers
 
         // make sure the preconditioner has an associated matrix set
         const Mat B = preconditioner;
-        AssertThrow (B != NULL,
+        AssertThrow (B != nullptr,
                      ExcMessage("PETSc preconditioner should have an"
                                 "associated matrix set to be used in solver."));
 
@@ -110,7 +106,7 @@ namespace PETScWrappers
         // convergence
         ierr = KSPSetConvergenceTest (solver_data->ksp, &convergence_test,
                                       reinterpret_cast<void *>(&solver_control),
-                                      PETSC_NULL);
+                                      nullptr);
         AssertThrow (ierr == 0, ExcPETScError(ierr));
       }
 
@@ -207,7 +203,7 @@ namespace PETScWrappers
   {
     PetscErrorCode ierr;
 
-    solver_data.reset (new SolverData());
+    solver_data = std_cxx14::make_unique<SolverData>();
 
     ierr = KSPCreate (mpi_communicator, &solver_data->ksp);
     AssertThrow (ierr == 0, ExcPETScError(ierr));
@@ -228,7 +224,7 @@ namespace PETScWrappers
     // convergence
     ierr = KSPSetConvergenceTest (solver_data->ksp, &convergence_test,
                                   reinterpret_cast<void *>(&solver_control),
-                                  PETSC_NULL);
+                                  nullptr);
     AssertThrow (ierr == 0, ExcPETScError(ierr));
 
     // set the command line options provided
@@ -307,15 +303,7 @@ namespace PETScWrappers
   void
   SolverChebychev::set_solver_type (KSP &ksp) const
   {
-    // set the type of solver. note the
-    // completely pointless change in
-    // spelling Chebyshev between PETSc 3.2
-    // and 3.3...
-#if DEAL_II_PETSC_VERSION_LT(3,3,0)
-    PetscErrorCode ierr = KSPSetType (ksp, KSPCHEBYCHEV);
-#else
     PetscErrorCode ierr = KSPSetType (ksp, KSPCHEBYSHEV);
-#endif
     AssertThrow (ierr == 0, ExcPETScError(ierr));
 
     // in the deal.II solvers, we always
@@ -436,12 +424,7 @@ namespace PETScWrappers
     // right
     if (additional_data.right_preconditioning)
       {
-#if DEAL_II_PETSC_VERSION_LT(3,2,0)
-        ierr = KSPSetPreconditionerSide(ksp, PC_RIGHT);
-#else
         ierr = KSPSetPCSide(ksp, PC_RIGHT);
-#endif
-
         AssertThrow (ierr == 0, ExcPETScError(ierr));
       }
 
@@ -643,6 +626,8 @@ namespace PETScWrappers
   SparseDirectMUMPS::SolverDataMUMPS::~SolverDataMUMPS ()
   {
     destroy_krylov_solver (ksp);
+    // the 'pc' object is owned by the 'ksp' object, and consequently
+    // does not have to be destroyed explicitly here
   }
 
 
@@ -688,7 +673,7 @@ namespace PETScWrappers
                             VectorBase       &x,
                             const VectorBase &b)
   {
-#ifdef PETSC_HAVE_MUMPS
+#ifdef DEAL_II_PETSC_WITH_MUMPS
     /**
      * factorization matrix to be obtained from MUMPS
      */
@@ -712,9 +697,9 @@ namespace PETScWrappers
     /**
      * creating a solver object if this is necessary
      */
-    if (solver_data.get() == 0)
+    if (solver_data == nullptr)
       {
-        solver_data.reset (new SolverDataMUMPS ());
+        solver_data = std_cxx14::make_unique<SolverDataMUMPS >();
 
         /**
          * creates the default KSP context and puts it in the location
@@ -770,22 +755,26 @@ namespace PETScWrappers
          * factorization here we start to see differences with the base
          * class solve function
          */
-#if DEAL_II_PETSC_VERSION_GTE(3,2,0)
+#if DEAL_II_PETSC_VERSION_LT(3, 9, 0)
         ierr = PCFactorSetMatSolverPackage (solver_data->pc, MATSOLVERMUMPS);
 #else
-        ierr = PCFactorSetMatSolverPackage (solver_data->pc, MAT_SOLVER_MUMPS);
+        ierr = PCFactorSetMatSolverType (solver_data->pc, MATSOLVERMUMPS);
 #endif
         AssertThrow (ierr == 0, ExcPETScError (ierr));
 
         /**
          * set up the package to call for the factorization
          */
+#if DEAL_II_PETSC_VERSION_LT(3, 9, 0)
         ierr = PCFactorSetUpMatSolverPackage (solver_data->pc);
+#else
+        ierr = PCFactorSetUpMatSolverType (solver_data->pc);
+#endif
         AssertThrow (ierr == 0, ExcPETScError(ierr));
 
         /**
          * get the factored matrix F from the preconditioner context.  This
-         * routine is valid only for LU, ILU, Cholesky, and imcomplete
+         * routine is valid only for LU, ILU, Cholesky, and incomplete
          * Cholesky
          */
         ierr = PCFactorGetMatrix(solver_data->pc, &F);
@@ -838,7 +827,7 @@ namespace PETScWrappers
         AssertThrow (ierr == 0, ExcPETScError(ierr));
       }
 
-#else  // PETSC_HAVE_MUMPS
+#else  // DEAL_II_PETSC_WITH_MUMPS
     Assert (false,
             ExcMessage ("Your PETSc installation does not include a copy of "
                         "the MUMPS package necessary for this solver. You will need to configure "
