@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 2004 - 2016 by the deal.II authors
+// Copyright (C) 2004 - 2018 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -19,7 +19,7 @@
 
 #  include <deal.II/lac/exceptions.h>
 #  include <deal.II/lac/petsc_compatibility.h>
-#  include <deal.II/lac/petsc_vector.h>
+#  include <deal.II/lac/petsc_vector_base.h>
 #  include <deal.II/lac/sparsity_pattern.h>
 #  include <deal.II/lac/dynamic_sparsity_pattern.h>
 
@@ -32,7 +32,7 @@ namespace PETScWrappers
   {
     const int m=0, n=0, n_nonzero_per_row=0;
     const PetscErrorCode ierr = MatCreateSeqAIJ(PETSC_COMM_SELF, m, n,
-                                                n_nonzero_per_row, 0, &matrix);
+                                                n_nonzero_per_row, nullptr, &matrix);
     AssertThrow (ierr == 0, ExcPETScError(ierr));
   }
 
@@ -147,7 +147,7 @@ namespace PETScWrappers
     // for all rows globally
     const PetscErrorCode ierr = MatCreateSeqAIJ(PETSC_COMM_SELF, m, n,
                                                 n_nonzero_per_row,
-                                                0, &matrix);
+                                                nullptr, &matrix);
     AssertThrow (ierr == 0, ExcPETScError(ierr));
 
     // set symmetric flag, if so requested
@@ -180,7 +180,7 @@ namespace PETScWrappers
     int_row_lengths (row_lengths.begin(), row_lengths.end());
 
     const PetscErrorCode ierr = MatCreateSeqAIJ(PETSC_COMM_SELF, m, n, 0,
-                                                &int_row_lengths[0], &matrix);
+                                                int_row_lengths.data(), &matrix);
     AssertThrow (ierr == 0, ExcPETScError(ierr));
 
     // set symmetric flag, if so requested
@@ -231,8 +231,8 @@ namespace PETScWrappers
 
             const PetscInt int_row = i;
             const PetscErrorCode ierr = MatSetValues (matrix, 1, &int_row,
-                                                      row_lengths[i], &row_entries[0],
-                                                      &row_values[0], INSERT_VALUES);
+                                                      row_lengths[i], row_entries.data(),
+                                                      row_values.data(), INSERT_VALUES);
             AssertThrow (ierr == 0, ExcPETScError(ierr));
           }
         compress (VectorOperation::insert);
@@ -262,6 +262,28 @@ namespace PETScWrappers
     return n;
   }
 
+  void
+  SparseMatrix::mmult (SparseMatrix &C,
+                       const SparseMatrix &B,
+                       const MPI::Vector &V) const
+  {
+    // Simply forward to the protected member function of the base class
+    // that takes abstract matrix and vector arguments (to which the compiler
+    // automatically casts the arguments).
+    MatrixBase::mmult (C, B, V);
+  }
+
+  void
+  SparseMatrix::Tmmult (SparseMatrix &C,
+                        const SparseMatrix &B,
+                        const MPI::Vector &V) const
+  {
+    // Simply forward to the protected member function of the base class
+    // that takes abstract matrix and vector arguments (to which the compiler
+    // automatically casts the arguments).
+    MatrixBase::Tmmult (C, B, V);
+  }
+
   // Explicit instantiations
   //
   template
@@ -284,23 +306,6 @@ namespace PETScWrappers
   template void
   SparseMatrix::do_reinit (const DynamicSparsityPattern &,
                            const bool);
-
-  PetscScalar
-  SparseMatrix::matrix_norm_square (const VectorBase &v) const
-  {
-    Vector tmp (v.size());
-    vmult (tmp, v);
-    return tmp*v;
-  }
-
-  PetscScalar
-  SparseMatrix::matrix_scalar_product (const VectorBase &u,
-                                       const VectorBase &v) const
-  {
-    Vector tmp (v.size());
-    vmult (tmp, v);
-    return u*tmp;
-  }
 }
 
 
