@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 2004 - 2017 by the deal.II authors
+// Copyright (C) 2004 - 2018 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -20,7 +20,6 @@
 #  include <deal.II/base/memory_consumption.h>
 #  include <deal.II/lac/exceptions.h>
 #  include <deal.II/lac/petsc_compatibility.h>
-#  include <deal.II/lac/petsc_vector.h>
 #  include <deal.II/lac/petsc_parallel_vector.h>
 #  include <cmath>
 #  include <deal.II/base/multithread_info.h>
@@ -36,123 +35,86 @@ namespace PETScWrappers
       Assert (index < vector.size(),
               ExcIndexRange (index, 0, vector.size()));
 
-      // if the vector is local, then
-      // simply access the element we
-      // are interested in
-      if (dynamic_cast<const PETScWrappers::Vector *>(&vector) != 0)
+      // The vector may have ghost entries. In that case, we first need to
+      // figure out which elements we own locally, then get a pointer to the
+      // elements that are stored here (both the ones we own as well as the
+      // ghost elements). In this array, the locally owned elements come first
+      // followed by the ghost elements whose position we can get from an
+      // index set.
+      if (vector.ghosted)
         {
-          PetscInt idx = index;
-          PetscScalar value;
-          PetscErrorCode ierr = VecGetValues(vector.vector, 1, &idx, &value);
+          PetscInt begin, end;
+          PetscErrorCode ierr = VecGetOwnershipRange (vector.vector, &begin,
+                                                      &end);
           AssertThrow (ierr == 0, ExcPETScError(ierr));
-          return value;
-        }
-      // else see if we are dealing
-      // with a parallel vector
-      else if (dynamic_cast<const PETScWrappers::MPI::Vector *>(&vector) != 0)
-        {
-          // there is the possibility
-          // that the vector has
-          // ghost elements. in that
-          // case, we first need to
-          // figure out which
-          // elements we own locally,
-          // then get a pointer to
-          // the elements that are
-          // stored here (both the
-          // ones we own as well as
-          // the ghost elements). in
-          // this array, the locally
-          // owned elements come
-          // first followed by the
-          // ghost elements whose
-          // position we can get from
-          // an index set
-          if (vector.ghosted)
+
+          Vec locally_stored_elements = PETSC_NULL;
+          ierr = VecGhostGetLocalForm(vector.vector, &locally_stored_elements);
+          AssertThrow (ierr == 0, ExcPETScError(ierr));
+
+          PetscInt lsize;
+          ierr = VecGetSize(locally_stored_elements, &lsize);
+          AssertThrow (ierr == 0, ExcPETScError(ierr));
+
+          PetscScalar *ptr;
+          ierr = VecGetArray(locally_stored_elements, &ptr);
+          AssertThrow (ierr == 0, ExcPETScError(ierr));
+
+          PetscScalar value;
+
+          if ( index>=static_cast<size_type>(begin)
+               && index<static_cast<size_type>(end) )
             {
-              PetscInt begin, end;
-              PetscErrorCode ierr = VecGetOwnershipRange (vector.vector, &begin,
-                                                          &end);
-              AssertThrow (ierr == 0, ExcPETScError(ierr));
+              //local entry
+              value = *(ptr+index-begin);
+            }
+          else
+            {
+              //ghost entry
+              const size_type ghostidx
+                = vector.ghost_indices.index_within_set(index);
 
-              Vec locally_stored_elements = PETSC_NULL;
-              ierr = VecGhostGetLocalForm(vector.vector, &locally_stored_elements);
-              AssertThrow (ierr == 0, ExcPETScError(ierr));
-
-              PetscInt lsize;
-              ierr = VecGetSize(locally_stored_elements, &lsize);
-              AssertThrow (ierr == 0, ExcPETScError(ierr));
-
-              PetscScalar *ptr;
-              ierr = VecGetArray(locally_stored_elements, &ptr);
-              AssertThrow (ierr == 0, ExcPETScError(ierr));
-
-              PetscScalar value;
-
-              if ( index>=static_cast<size_type>(begin)
-                   && index<static_cast<size_type>(end) )
-                {
-                  //local entry
-                  value = *(ptr+index-begin);
-                }
-              else
-                {
-                  //ghost entry
-                  const size_type ghostidx
-                    = vector.ghost_indices.index_within_set(index);
-
-                  Assert(ghostidx+end-begin<(size_type)lsize, ExcInternalError());
-                  value = *(ptr+ghostidx+end-begin);
-                }
-
-              ierr = VecRestoreArray(locally_stored_elements, &ptr);
-              AssertThrow (ierr == 0, ExcPETScError(ierr));
-
-              ierr = VecGhostRestoreLocalForm(vector.vector, &locally_stored_elements);
-              AssertThrow (ierr == 0, ExcPETScError(ierr));
-
-              return value;
+              Assert(ghostidx+end-begin<(size_type)lsize, ExcInternalError());
+              value = *(ptr+ghostidx+end-begin);
             }
 
-
-          // first verify that the requested
-          // element is actually locally
-          // available
-          PetscInt begin, end;
-
-          PetscErrorCode ierr = VecGetOwnershipRange (vector.vector, &begin, &end);
+          ierr = VecRestoreArray(locally_stored_elements, &ptr);
           AssertThrow (ierr == 0, ExcPETScError(ierr));
 
-
-
-          AssertThrow ((index >= static_cast<size_type>(begin)) &&
-                       (index < static_cast<size_type>(end)),
-                       ExcAccessToNonlocalElement (index, begin, end-1));
-
-          // old version which only work with
-          // VecGetArray()...
-          PetscInt idx = index;
-          PetscScalar value;
-          ierr = VecGetValues(vector.vector, 1, &idx, &value);
+          ierr = VecGhostRestoreLocalForm(vector.vector, &locally_stored_elements);
           AssertThrow (ierr == 0, ExcPETScError(ierr));
 
           return value;
         }
-      else
-        // what? what other kind of vector
-        // exists there?
-        Assert (false, ExcInternalError());
 
-      return -1e20;
+
+      // first verify that the requested
+      // element is actually locally
+      // available
+      PetscInt begin, end;
+
+      PetscErrorCode ierr = VecGetOwnershipRange (vector.vector, &begin, &end);
+      AssertThrow (ierr == 0, ExcPETScError(ierr));
+
+      AssertThrow ((index >= static_cast<size_type>(begin)) &&
+                   (index < static_cast<size_type>(end)),
+                   ExcAccessToNonlocalElement (index, begin, end-1));
+
+      PetscInt idx = index;
+      PetscScalar value;
+      ierr = VecGetValues(vector.vector, 1, &idx, &value);
+      AssertThrow (ierr == 0, ExcPETScError(ierr));
+
+      return value;
     }
   }
 
   VectorBase::VectorBase ()
     :
-    vector (NULL),
+    vector (nullptr),
     ghosted(false),
     last_action (::dealii::VectorOperation::unknown),
-    attained_ownership(true)
+    obtained_ownership(true)
   {
     Assert( MultithreadInfo::is_running_single_threaded(),
             ExcMessage("PETSc does not support multi-threaded access, set "
@@ -167,7 +129,7 @@ namespace PETScWrappers
     ghosted(v.ghosted),
     ghost_indices(v.ghost_indices),
     last_action (::dealii::VectorOperation::unknown),
-    attained_ownership(true)
+    obtained_ownership(true)
   {
     Assert( MultithreadInfo::is_running_single_threaded(),
             ExcMessage("PETSc does not support multi-threaded access, set "
@@ -188,7 +150,7 @@ namespace PETScWrappers
     vector(v),
     ghosted(false),
     last_action (::dealii::VectorOperation::unknown),
-    attained_ownership(false)
+    obtained_ownership(false)
   {
     Assert( MultithreadInfo::is_running_single_threaded(),
             ExcMessage("PETSc does not support multi-threaded access, set "
@@ -199,13 +161,9 @@ namespace PETScWrappers
 
   VectorBase::~VectorBase ()
   {
-    if (attained_ownership)
+    if (obtained_ownership)
       {
-#if DEAL_II_PETSC_VERSION_LT(3,2,0)
-        const PetscErrorCode ierr = VecDestroy (vector);
-#else
         const PetscErrorCode ierr = VecDestroy (&vector);
-#endif
         AssertNothrow (ierr == 0, ExcPETScError(ierr));
         (void) ierr;
       }
@@ -216,20 +174,16 @@ namespace PETScWrappers
   void
   VectorBase::clear ()
   {
-    if (attained_ownership)
+    if (obtained_ownership)
       {
-#if DEAL_II_PETSC_VERSION_LT(3,2,0)
-        const PetscErrorCode ierr = VecDestroy (vector);
-#else
         const PetscErrorCode ierr = VecDestroy (&vector);
-#endif
         AssertThrow (ierr == 0, ExcPETScError(ierr));
       }
 
     ghosted = false;
     ghost_indices.clear ();
     last_action = ::dealii::VectorOperation::unknown;
-    attained_ownership = true;
+    obtained_ownership = true;
   }
 
 
@@ -268,7 +222,7 @@ namespace PETScWrappers
     Assert (size() == v.size(),
             ExcDimensionMismatch(size(), v.size()));
 
-    PetscBooleanType flag;
+    PetscBool flag;
     const PetscErrorCode ierr = VecEqual (vector, v.vector, &flag);
     AssertThrow (ierr == 0, ExcPETScError(ierr));
 
@@ -283,7 +237,7 @@ namespace PETScWrappers
     Assert (size() == v.size(),
             ExcDimensionMismatch(size(), v.size()));
 
-    PetscBooleanType flag;
+    PetscBool flag;
     const PetscErrorCode ierr = VecEqual (vector, v.vector, &flag);
     AssertThrow (ierr == 0, ExcPETScError(ierr));
 
@@ -335,7 +289,7 @@ namespace PETScWrappers
   {
     Assert (indices.size() == values.size(),
             ExcMessage ("Function called with arguments of different sizes"));
-    do_set_add_operation(indices.size(), &indices[0], &values[0], false);
+    do_set_add_operation(indices.size(), indices.data(), values.data(), false);
   }
 
 
@@ -346,7 +300,7 @@ namespace PETScWrappers
   {
     Assert (indices.size() == values.size(),
             ExcMessage ("Function called with arguments of different sizes"));
-    do_set_add_operation(indices.size(), &indices[0], &values[0], true);
+    do_set_add_operation(indices.size(), indices.data(), values.data(), true);
   }
 
 
@@ -357,7 +311,7 @@ namespace PETScWrappers
   {
     Assert (indices.size() == values.size(),
             ExcMessage ("Function called with arguments of different sizes"));
-    do_set_add_operation(indices.size(), &indices[0], values.begin(), true);
+    do_set_add_operation(indices.size(), indices.data(), values.begin(), true);
   }
 
 
@@ -380,10 +334,13 @@ namespace PETScWrappers
 
     PetscScalar result;
 
-    //For complex vectors, VecDot() computes
+    // For complex vectors, VecDot() computes
     //    val = (x,y) = y^H x,
-    //where y^H denotes the conjugate transpose of y.
-    //Note that this corresponds to the usual "mathematicians" complex inner product where the SECOND argument gets the complex conjugate.
+    // where y^H denotes the conjugate transpose of y.
+    // Note that this corresponds to the usual "mathematicians'"
+    // complex inner product where the SECOND argument gets the
+    // complex conjugate, which is also how we document this
+    // operation.
     const PetscErrorCode ierr = VecDot (vec.vector, vector, &result);
     AssertThrow (ierr == 0, ExcPETScError(ierr));
 
@@ -472,7 +429,7 @@ namespace PETScWrappers
   {
     // We can only use our more efficient
     // routine in the serial case.
-    if (dynamic_cast<const PETScWrappers::MPI::Vector *>(this) != 0)
+    if (dynamic_cast<const PETScWrappers::MPI::Vector *>(this) != nullptr)
       {
         PetscScalar sum;
         const PetscErrorCode ierr = VecSum(vector, &sum);
@@ -606,18 +563,6 @@ namespace PETScWrappers
 
 
 
-  VectorBase::real_type
-  VectorBase::normalize () const
-  {
-    real_type d;
-    Assert (!has_ghost_elements(), ExcGhostsPresent());
-    const PetscErrorCode ierr = VecNormalize (vector, &d);
-    AssertThrow (ierr == 0, ExcPETScError(ierr));
-
-    return d;
-  }
-
-
   VectorBase::real_type
   VectorBase::min ()  const
   {
@@ -644,65 +589,6 @@ namespace PETScWrappers
   }
 
 
-  VectorBase &
-  VectorBase::abs ()
-  {
-    Assert (!has_ghost_elements(), ExcGhostsPresent());
-
-    const PetscErrorCode ierr = VecAbs (vector);
-    AssertThrow (ierr == 0, ExcPETScError(ierr));
-
-    return *this;
-  }
-
-
-
-  VectorBase &
-  VectorBase::conjugate ()
-  {
-    Assert (!has_ghost_elements(), ExcGhostsPresent());
-
-    const PetscErrorCode ierr = VecConjugate (vector);
-    AssertThrow (ierr == 0, ExcPETScError(ierr));
-
-    return *this;
-  }
-
-
-
-  VectorBase &
-  VectorBase::mult ()
-  {
-    Assert (!has_ghost_elements(), ExcGhostsPresent());
-
-    const PetscErrorCode ierr = VecPointwiseMult (vector,vector,vector);
-    AssertThrow (ierr == 0, ExcPETScError(ierr));
-
-    return *this;
-  }
-
-
-  VectorBase &
-  VectorBase::mult (const VectorBase &v)
-  {
-    Assert (!has_ghost_elements(), ExcGhostsPresent());
-    const PetscErrorCode ierr = VecPointwiseMult (vector,vector,v);
-    AssertThrow (ierr == 0, ExcPETScError(ierr));
-
-    return *this;
-  }
-
-
-  VectorBase &
-  VectorBase::mult (const VectorBase &u,
-                    const VectorBase &v)
-  {
-    Assert (!has_ghost_elements(), ExcGhostsPresent());
-    const PetscErrorCode ierr = VecPointwiseMult (vector,u,v);
-    AssertThrow (ierr == 0, ExcPETScError(ierr));
-
-    return *this;
-  }
 
   bool
   VectorBase::all_zero () const
@@ -856,14 +742,6 @@ namespace PETScWrappers
 
 
 
-  void
-  VectorBase::add (const VectorBase &v)
-  {
-    *this += v;
-  }
-
-
-
   void
   VectorBase::add (const PetscScalar a,
                    const VectorBase &v)
@@ -927,59 +805,6 @@ namespace PETScWrappers
 
 
 
-  void
-  VectorBase::sadd (const PetscScalar s,
-                    const PetscScalar a,
-                    const VectorBase     &v,
-                    const PetscScalar b,
-                    const VectorBase     &w)
-  {
-    Assert (!has_ghost_elements(), ExcGhostsPresent());
-    AssertIsFinite(s);
-    AssertIsFinite(a);
-    AssertIsFinite(b);
-
-    // there is no operation like MAXPAY, so
-    // do it in two steps
-    *this *= s;
-
-    const PetscScalar weights[2] = {a,b};
-    Vec               addends[2] = {v.vector,w.vector};
-
-    const PetscErrorCode ierr = VecMAXPY (vector, 2, weights, addends);
-    AssertThrow (ierr == 0, ExcPETScError(ierr));
-  }
-
-
-
-  void
-  VectorBase::sadd (const PetscScalar s,
-                    const PetscScalar a,
-                    const VectorBase     &v,
-                    const PetscScalar b,
-                    const VectorBase     &w,
-                    const PetscScalar c,
-                    const VectorBase     &x)
-  {
-    Assert (!has_ghost_elements(), ExcGhostsPresent());
-    AssertIsFinite(s);
-    AssertIsFinite(a);
-    AssertIsFinite(b);
-    AssertIsFinite(c);
-
-    // there is no operation like MAXPAY, so
-    // do it in two steps
-    *this *= s;
-
-    const PetscScalar weights[3] = {a,b,c};
-    Vec               addends[3] = {v.vector, w.vector, x.vector};
-
-    const PetscErrorCode ierr = VecMAXPY (vector, 3, weights, addends);
-    AssertThrow (ierr == 0, ExcPETScError(ierr));
-  }
-
-
-
   void
   VectorBase::scale (const VectorBase &factors)
   {
@@ -1012,30 +837,6 @@ namespace PETScWrappers
 
 
 
-  void
-  VectorBase::equ (const PetscScalar a,
-                   const VectorBase &v,
-                   const PetscScalar b,
-                   const VectorBase &w)
-  {
-    Assert (!has_ghost_elements(), ExcGhostsPresent());
-    AssertIsFinite(a);
-    AssertIsFinite(b);
-
-    Assert (size() == v.size(),
-            ExcDimensionMismatch (size(), v.size()));
-
-    // there is no simple operation for this
-    // in PETSc. there are multiple ways to
-    // emulate it, we choose this one:
-    const PetscErrorCode ierr = VecCopy (v.vector, vector);
-    AssertThrow (ierr == 0, ExcPETScError(ierr));
-
-    sadd (a, b, w);
-  }
-
-
-
   void
   VectorBase::ratio (const VectorBase &a,
                      const VectorBase &b)
@@ -1171,14 +972,7 @@ namespace PETScWrappers
     // (unlike the above calls)
     if (n_elements != 0)
       {
-#ifdef PETSC_USE_64BIT_INDICES
-        std::vector<PetscInt> petsc_ind (n_elements);
-        for (size_type i=0; i<n_elements; ++i)
-          petsc_ind[i] = indices[i];
-        const PetscInt *petsc_indices = &petsc_ind[0];
-#else
-        const int *petsc_indices = (const int *)indices;
-#endif
+        const PetscInt *petsc_indices = reinterpret_cast<const PetscInt *>(indices);
 
         const InsertMode mode = (add_values ? ADD_VALUES : INSERT_VALUES);
         const PetscErrorCode ierr = VecSetValues (vector, n_elements, petsc_indices,
