@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 2009 - 2016 by the deal.II authors
+// Copyright (C) 2009 - 2018 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -19,7 +19,6 @@
 
 #  include <deal.II/lac/petsc_matrix_base.h>
 #  include <deal.II/lac/petsc_vector_base.h>
-#  include <deal.II/lac/petsc_vector.h>
 #  include <deal.II/lac/slepc_spectral_transformation.h>
 
 #  include <cmath>
@@ -37,7 +36,8 @@ namespace SLEPcWrappers
                           const MPI_Comm &mpi_communicator)
     :
     solver_control (cn),
-    mpi_communicator (mpi_communicator)
+    mpi_communicator (mpi_communicator),
+    reason (EPS_CONVERGED_ITERATING)
   {
     // create eigensolver context
     PetscErrorCode ierr = EPSCreate (mpi_communicator, &eps);
@@ -59,15 +59,13 @@ namespace SLEPcWrappers
 
   SolverBase::~SolverBase ()
   {
-    if (eps != NULL)
+    if (eps != nullptr)
       {
         // Destroy the solver object.
-#if DEAL_II_PETSC_VERSION_LT(3,2,0)
-        const PetscErrorCode ierr = EPSDestroy (eps);
-#else
         const PetscErrorCode ierr = EPSDestroy (&eps);
-#endif
-        AssertThrow (ierr == 0, ExcSLEPcError(ierr));
+
+        (void)ierr;
+        AssertNothrow (ierr == 0, ExcSLEPcError(ierr));
       }
   }
 
@@ -75,7 +73,7 @@ namespace SLEPcWrappers
   SolverBase::set_matrices (const PETScWrappers::MatrixBase &A)
   {
     // standard eigenspectrum problem
-    const PetscErrorCode ierr = EPSSetOperators (eps, A, PETSC_NULL);
+    const PetscErrorCode ierr = EPSSetOperators (eps, A, nullptr);
     AssertThrow (ierr == 0, ExcSLEPcError(ierr));
   }
 
@@ -93,8 +91,21 @@ namespace SLEPcWrappers
   {
     // set transformation type if any
     // STSetShift is called inside
-    const PetscErrorCode ierr = EPSSetST(eps,transformation.st);
+    PetscErrorCode ierr = EPSSetST(eps,transformation.st);
     AssertThrow (ierr == 0, SolverBase::ExcSLEPcError(ierr));
+
+#if DEAL_II_SLEPC_VERSION_GTE(3, 8, 0)
+    // see https://lists.mcs.anl.gov/mailman/htdig/petsc-users/2017-October/033649.html
+    // From 3.8.0 SLEPc insists that when looking for smallest eigenvalues with shift-and-invert
+    // users should (a) set target (b) use EPS_TARGET_MAGNITUDE
+    // The former, however, needs to be applied to eps object and not spectral transformation.
+    if (SLEPcWrappers::TransformationShiftInvert *sinv = dynamic_cast<SLEPcWrappers::TransformationShiftInvert *>(&transformation))
+      {
+        ierr = EPSSetTarget (eps, sinv->additional_data.shift_parameter);
+        AssertThrow (ierr == 0, SolverBase::ExcSLEPcError(ierr));
+      }
+#endif
+
   }
 
   void
@@ -104,11 +115,8 @@ namespace SLEPcWrappers
            ExcMessage("Initial vector should be nonzero."));
 
     Vec vec = this_initial_vector;
-#if DEAL_II_PETSC_VERSION_LT(3,1,0)
-    const PetscErrorCode ierr = EPSSetInitialVector (eps, &vec);
-#else
     const PetscErrorCode ierr = EPSSetInitialSpace (eps, 1, &vec);
-#endif
+
     AssertThrow (ierr == 0, ExcSLEPcError(ierr));
   }
 
@@ -187,22 +195,29 @@ namespace SLEPcWrappers
       for (unsigned int i = 0; i < *n_converged; i++)
         {
           double residual_norm_i = 0.0;
-          // EPSComputeResidualNorm is L2-norm and is not consistent with the stopping criteria
-          // used during the solution process.
-          // Yet, this is the norm which gives error bounds (Saad, 1992, ch3):
-          //   | \lambda - \widehat\lambda | <= ||r||_2
-          ierr = EPSComputeResidualNorm (eps, i, &residual_norm_i);
-
-          // EPSComputeRelativeError may not be consistent with the stopping criteria
-          // used during the solution process. Given EPS_CONV_ABS set above,
-          // this can be either the l2 norm or the mass-matrix induced norm
-          // when EPS_GHEP is set.
-          // ierr = EPSComputeRelativeError (solver_data->eps, i, &residual_norm_i);
-
-          // EPSGetErrorEstimate is consistent with the residual norm
-          // used during the solution process. However, it is not guaranteed to
-          // be derived from the residual even when EPSSetTrueResidual is set.
-          // ierr = EPSGetErrorEstimate (solver_data->eps, i, &residual_norm_i);
+          // EPSComputeError (or, in older versions of SLEPc,
+          // EPSComputeResidualNorm) uses an L2-norm and is not consistent
+          // with the stopping criterion used during the solution process (see
+          // the SLEPC manual, section 2.5). However, the norm that gives error
+          // bounds (Saad, 1992, ch3) is (for Hermitian problems)
+          // | \lambda - \widehat\lambda | <= ||r||_2
+          //
+          // Similarly, EPSComputeRelativeError may not be consistent with the
+          // stopping criterion used in the solution process.
+          //
+          // EPSGetErrorEstimate is (according to the SLEPc manual) consistent
+          // with the residual norm used during the solution process. However,
+          // it is not guaranteed to be derived from the residual even when
+          // EPSSetTrueResidual is set: see the discussion in the thread
+          //
+          // https://lists.mcs.anl.gov/pipermail/petsc-users/2014-November/023509.html
+          //
+          // for more information.
+#if DEAL_II_SLEPC_VERSION_GTE(3, 6, 0)
+          ierr = EPSComputeError(eps, i, EPS_ERROR_ABSOLUTE, &residual_norm_i);
+#else
+          ierr = EPSComputeResidualNorm(eps, i, &residual_norm_i);
+#endif
 
           AssertThrow (ierr == 0, ExcSLEPcError(ierr));
           residual_norm = std::max (residual_norm, residual_norm_i);
@@ -231,8 +246,8 @@ namespace SLEPcWrappers
   {
     // get converged eigenpair
     const PetscErrorCode ierr = EPSGetEigenpair (eps, index,
-                                                 &eigenvalues, PETSC_NULL,
-                                                 eigenvectors, PETSC_NULL);
+                                                 &eigenvalues, nullptr,
+                                                 eigenvectors, nullptr);
     AssertThrow (ierr == 0, ExcSLEPcError(ierr));
   }
 
@@ -399,7 +414,6 @@ namespace SLEPcWrappers
     SolverBase (cn, mpi_communicator),
     additional_data (data)
   {
-#if DEAL_II_PETSC_VERSION_GTE(3,1,0)
     PetscErrorCode ierr = EPSSetType (eps, const_cast<char *>(EPSGD));
     AssertThrow (ierr == 0, ExcSLEPcError(ierr));
 
@@ -408,12 +422,6 @@ namespace SLEPcWrappers
         ierr = EPSGDSetDoubleExpansion (eps, PETSC_TRUE);
         AssertThrow (ierr == 0, ExcSLEPcError(ierr));
       }
-#else
-    // PETSc/SLEPc version must be > 3.1.0.
-    Assert ((false),
-            ExcMessage ("Your SLEPc installation does not include a copy of the "
-                        "Generalized Davidson solver. A SLEPc version > 3.1.0 is required."));
-#endif
   }
 
   /* ------------------ Jacobi Davidson -------------------- */
@@ -424,15 +432,8 @@ namespace SLEPcWrappers
     SolverBase (cn, mpi_communicator),
     additional_data (data)
   {
-#if DEAL_II_PETSC_VERSION_GTE(3,1,0)
     const PetscErrorCode ierr = EPSSetType (eps, const_cast<char *>(EPSJD));
     AssertThrow (ierr == 0, ExcSLEPcError(ierr));
-#else
-    // PETSc/SLEPc version must be > 3.1.0.
-    Assert ((false),
-            ExcMessage ("Your SLEPc installation does not include a copy of the "
-                        "Jacobi-Davidson solver. A SLEPc version > 3.1.0 is required."));
-#endif
   }
 
   /* ---------------------- LAPACK ------------------------- */
@@ -459,4 +460,3 @@ namespace SLEPcWrappers
 DEAL_II_NAMESPACE_CLOSE
 
 #endif // DEAL_II_WITH_SLEPC
-
