@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 2009 - 2016 by the deal.II authors
+// Copyright (C) 2009 - 2017 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -19,8 +19,6 @@
 
 #  include <deal.II/lac/slepc_solver.h>
 #  include <deal.II/lac/petsc_matrix_base.h>
-#  include <deal.II/lac/petsc_vector_base.h>
-#  include <deal.II/lac/petsc_vector.h>
 
 #  include <cmath>
 #  include <vector>
@@ -39,10 +37,11 @@ namespace SLEPcWrappers
 
   TransformationBase::~TransformationBase ()
   {
-    if (st!=NULL)
+    if (st!=nullptr)
       {
         const PetscErrorCode ierr = STDestroy(&st);
-        AssertThrow (ierr == 0, SolverBase::ExcSLEPcError(ierr));
+        (void)ierr;
+        AssertNothrow (ierr == 0, SolverBase::ExcSLEPcError(ierr));
       }
   }
 
@@ -93,11 +92,7 @@ namespace SLEPcWrappers
     TransformationBase(mpi_communicator),
     additional_data (data)
   {
-#if DEAL_II_PETSC_VERSION_LT(3,1,0)
-    PetscErrorCode ierr = STSetType (st, const_cast<char *>(STSINV));
-#else
     PetscErrorCode ierr = STSetType (st, const_cast<char *>(STSINVERT));
-#endif
     AssertThrow (ierr == 0, SolverBase::ExcSLEPcError(ierr));
 
     ierr = STSetShift (st, additional_data.shift_parameter);
