@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 1998 - 2016 by the deal.II authors
+// Copyright (C) 1998 - 2017 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -36,7 +36,7 @@ SolverControl::SolverControl (const unsigned int maxiter,
   lcheck(failure),
   initial_val(numbers::signaling_nan<double>()),
   lvalue(numbers::signaling_nan<double>()),
-  lstep(0),
+  lstep(numbers::invalid_unsigned_int),
   check_failure(false),
   relative_failure_residual(0),
   failure_residual(0),
@@ -48,11 +48,6 @@ SolverControl::SolverControl (const unsigned int maxiter,
 
 
 
-SolverControl::~SolverControl()
-{}
-
-
-
 SolverControl::State
 SolverControl::check (const unsigned int step,
                       const double check_value)
@@ -63,8 +58,6 @@ SolverControl::check (const unsigned int step,
   if (step==0)
     {
       initial_val = check_value;
-      if (history_data_enabled)
-        history_data.resize(maxsteps);
     }
 
   if (m_log_history && ((step % m_log_frequency) == 0))
@@ -83,7 +76,7 @@ SolverControl::check (const unsigned int step,
     }
 
   if (history_data_enabled)
-    history_data[step] = check_value;
+    history_data.push_back(check_value);
 
   if (check_value <= tol)
     {
@@ -95,7 +88,7 @@ SolverControl::check (const unsigned int step,
     }
 
   if ((step >= maxsteps) ||
-      numbers::is_nan(check_value) ||
+      std::isnan(check_value) ||
       (check_failure && (check_value > failure_residual))
      )
     {
@@ -158,6 +151,20 @@ SolverControl::enable_history_data ()
 }
 
 
+
+const std::vector<double> &SolverControl::get_history_data() const
+{
+  Assert (history_data_enabled, ExcHistoryDataRequired());
+  Assert (history_data.size() > 0,
+          ExcMessage("The SolverControl object was asked for the solver history "
+                     "data, but there is no data. Possibly you requested the data before the "
+                     "solver was run."));
+
+  return history_data;
+}
+
+
+
 double
 SolverControl::average_reduction() const
 {
@@ -247,9 +254,6 @@ ReductionControl::operator= (const SolverControl &c)
 }
 
 
-ReductionControl::~ReductionControl()
-{}
-
 
 SolverControl::State
 ReductionControl::check (const unsigned int step,
@@ -275,8 +279,11 @@ ReductionControl::check (const unsigned int step,
                 << " value " << check_value << std::endl;
       lstep  = step;
       lvalue = check_value;
-
       lcheck = success;
+
+      if (history_data_enabled)
+        history_data.push_back(check_value);
+
       return success;
     }
   else
@@ -312,8 +319,6 @@ IterationNumberControl::IterationNumberControl(const unsigned int n,
   SolverControl (n, tolerance, m_log_history, m_log_result) {}
 
 
-IterationNumberControl::~IterationNumberControl()
-{}
 
 
 SolverControl::State
@@ -337,4 +342,81 @@ IterationNumberControl::check (const unsigned int step,
     return SolverControl::check(step, check_value);
 }
 
+/*------------------------ ConsecutiveControl -------------------------------*/
+
+
+ConsecutiveControl::ConsecutiveControl(const unsigned int n,
+                                       const double       tolerance,
+                                       const unsigned int n_consecutive_iterations,
+                                       const bool m_log_history,
+                                       const bool m_log_result)
+  :
+  SolverControl (n, tolerance, m_log_history, m_log_result),
+  n_consecutive_iterations(n_consecutive_iterations),
+  n_converged_iterations(0)
+{
+  AssertThrow(n_consecutive_iterations>0,
+              ExcMessage("n_consecutive_iterations should be positive"));
+}
+
+
+
+ConsecutiveControl::ConsecutiveControl (const SolverControl &c)
+  :
+  SolverControl(c),
+  n_consecutive_iterations(1),
+  n_converged_iterations(0)
+{}
+
+
+
+ConsecutiveControl &
+ConsecutiveControl::operator= (const SolverControl &c)
+{
+  SolverControl::operator=(c);
+  n_consecutive_iterations = 1;
+  n_converged_iterations = 0;
+  return *this;
+}
+
+
+
+SolverControl::State
+ConsecutiveControl::check (const unsigned int step,
+                           const double check_value)
+{
+  // reset the counter if ConsecutiveControl is being reused
+  if (step==0)
+    n_converged_iterations = 0;
+  else
+    {
+      // check two things:
+      // (i)  steps are ascending without repetitions
+      // (ii) user started from zero even when solver is being reused.
+      Assert (step-1 == lstep,
+              ExcMessage("steps should be ascending integers."));
+    }
+
+  SolverControl::State state = SolverControl::check(step, check_value);
+  // check if we need to override the success:
+  if (state == success)
+    {
+      n_converged_iterations++;
+      if (n_converged_iterations == n_consecutive_iterations)
+        {
+          return success;
+        }
+      else
+        {
+          lcheck = iterate;
+          return iterate;
+        }
+    }
+  else
+    {
+      n_converged_iterations = 0;
+      return state;
+    }
+}
+
 DEAL_II_NAMESPACE_CLOSE
