@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 2000 - 2017 by the deal.II authors
+// Copyright (C) 2000 - 2018 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -14,6 +14,7 @@
 // ---------------------------------------------------------------------
 
 
+#include <deal.II/base/std_cxx14/memory.h>
 #include <deal.II/base/vector_slice.h>
 #include <deal.II/base/utilities.h>
 #include <deal.II/lac/sparsity_pattern.h>
@@ -41,8 +42,8 @@ SparsityPattern::SparsityPattern ()
   :
   max_dim(0),
   max_vec_len(0),
-  rowstart(0),
-  colnums(0),
+  rowstart(nullptr),
+  colnums(nullptr),
   compressed(false),
   store_diagonal_first_in_row(false)
 {
@@ -56,8 +57,8 @@ SparsityPattern::SparsityPattern (const SparsityPattern &s)
   Subscriptor(),
   max_dim(0),
   max_vec_len(0),
-  rowstart(0),
-  colnums(0),
+  rowstart(nullptr),
+  colnums(nullptr),
   compressed(false),
   store_diagonal_first_in_row(false)
 {
@@ -78,8 +79,8 @@ SparsityPattern::SparsityPattern (const size_type m,
   :
   max_dim(0),
   max_vec_len(0),
-  rowstart(0),
-  colnums(0),
+  rowstart(nullptr),
+  colnums(nullptr),
   compressed(false),
   store_diagonal_first_in_row(m == n)
 {
@@ -94,8 +95,8 @@ SparsityPattern::SparsityPattern (const size_type m,
   :
   max_dim(0),
   max_vec_len(0),
-  rowstart(0),
-  colnums(0),
+  rowstart(nullptr),
+  colnums(nullptr),
   store_diagonal_first_in_row(m == n)
 {
   reinit (m, n, row_lengths);
@@ -108,8 +109,8 @@ SparsityPattern::SparsityPattern (const size_type m,
   :
   max_dim(0),
   max_vec_len(0),
-  rowstart(0),
-  colnums(0)
+  rowstart(nullptr),
+  colnums(nullptr)
 {
   reinit (m, m, max_per_row);
 }
@@ -121,8 +122,8 @@ SparsityPattern::SparsityPattern (const size_type               m,
   :
   max_dim(0),
   max_vec_len(0),
-  rowstart(0),
-  colnums(0)
+  rowstart(nullptr),
+  colnums(nullptr)
 {
   reinit (m, m, row_lengths);
 }
@@ -135,8 +136,8 @@ SparsityPattern::SparsityPattern (const SparsityPattern &original,
   :
   max_dim(0),
   max_vec_len(0),
-  rowstart(0),
-  colnums(0)
+  rowstart(nullptr),
+  colnums(nullptr)
 {
   Assert (original.rows==original.cols, ExcNotQuadratic());
   Assert (original.is_compressed(), ExcNotCompressed());
@@ -214,14 +215,6 @@ SparsityPattern::SparsityPattern (const SparsityPattern &original,
 
 
 
-SparsityPattern::~SparsityPattern ()
-{
-  if (rowstart != 0)  delete[] rowstart;
-  if (colnums != 0)   delete[] colnums;
-}
-
-
-
 SparsityPattern &
 SparsityPattern::operator = (const SparsityPattern &s)
 {
@@ -265,14 +258,14 @@ SparsityPattern::reinit (const size_type m,
   // delete empty matrices
   if ((m==0) || (n==0))
     {
-      if (rowstart)  delete[] rowstart;
-      if (colnums)   delete[] colnums;
-      rowstart = 0;
-      colnums = 0;
+      rowstart.reset();
+      colnums.reset();
+
       max_vec_len = max_dim = rows = cols = 0;
       // if dimension is zero: ignore max_per_row
       max_row_length = 0;
       compressed = false;
+
       return;
     }
 
@@ -301,14 +294,8 @@ SparsityPattern::reinit (const size_type m,
   if (vec_len == 0)
     {
       vec_len = 1;
-      if (colnums)
-        {
-          delete[] colnums;
-          colnums = 0;
-        }
-
       max_vec_len = vec_len;
-      colnums = new size_type[max_vec_len];
+      colnums = std_cxx14::make_unique<size_type[]> (max_vec_len);
     }
 
   max_row_length = (row_lengths.size() == 0 ?
@@ -328,27 +315,15 @@ SparsityPattern::reinit (const size_type m,
   // and try to delete the memory a second time.
   if (rows > max_dim)
     {
-      if (rowstart)
-        {
-          delete[] rowstart;
-          rowstart = 0;
-        }
-
       max_dim = rows;
-      rowstart = new std::size_t[max_dim+1];
+      rowstart = std_cxx14::make_unique<std::size_t[]> (max_dim+1);
     }
 
   // allocate memory for the column numbers if necessary
   if (vec_len > max_vec_len)
     {
-      if (colnums)
-        {
-          delete[] colnums;
-          colnums = 0;
-        }
-
       max_vec_len = vec_len;
-      colnums = new size_type[max_vec_len];
+      colnums = std_cxx14::make_unique<size_type[]> (max_vec_len);
     }
 
   // set the rowstart array
@@ -365,7 +340,7 @@ SparsityPattern::reinit (const size_type m,
           ExcInternalError());
 
   // preset the column numbers by a value indicating it is not in use
-  std::fill_n (&colnums[0], vec_len, invalid_entry);
+  std::fill_n (colnums.get(), vec_len, invalid_entry);
 
   // if diagonal elements are special: let the first entry in each row be the
   // diagonal value
@@ -381,7 +356,12 @@ SparsityPattern::reinit (const size_type m,
 void
 SparsityPattern::compress ()
 {
-  Assert ((rowstart!=0) && (colnums!=0), ExcEmptyObject());
+  // nothing to do if the object corresponds to an empty matrix
+  if ((rowstart==nullptr) && (colnums==nullptr))
+    {
+      compressed = true;
+      return;
+    }
 
   // do nothing if already compressed
   if (compressed)
@@ -396,9 +376,9 @@ SparsityPattern::compress ()
   const std::size_t nonzero_elements
     = std::count_if (&colnums[rowstart[0]],
                      &colnums[rowstart[rows]],
-                     std_cxx11::bind(std::not_equal_to<size_type>(), std_cxx11::_1, invalid_entry));
+                     std::bind(std::not_equal_to<size_type>(), std::placeholders::_1, invalid_entry));
   // now allocate the respective memory
-  size_type *new_colnums = new size_type[nonzero_elements];
+  std::unique_ptr<size_type[]> new_colnums (new size_type[nonzero_elements]);
 
 
   // reserve temporary storage to store the entries of one row
@@ -470,9 +450,9 @@ SparsityPattern::compress ()
   // set iterator-past-the-end
   rowstart[rows] = next_row_start;
 
-  // set colnums to the newly allocated array and delete the old one
-  delete[] colnums;
-  colnums = new_colnums;
+  // set colnums to the newly allocated array and delete previous content
+  // in the process
+  colnums = std::move(new_colnums);
 
   // store the size
   max_vec_len = nonzero_elements;
@@ -482,33 +462,32 @@ SparsityPattern::compress ()
 
 
 
-template <typename SparsityPatternType>
 void
-SparsityPattern::copy_from (const SparsityPatternType &dsp)
+SparsityPattern::copy_from (const SparsityPattern &sp)
 {
   // first determine row lengths for each row. if the matrix is quadratic,
   // then we might have to add an additional entry for the diagonal, if that
   // is not yet present. as we have to call compress anyway later on, don't
   // bother to check whether that diagonal entry is in a certain row or not
-  const bool do_diag_optimize = (dsp.n_rows() == dsp.n_cols());
-  std::vector<unsigned int> row_lengths (dsp.n_rows());
-  for (size_type i=0; i<dsp.n_rows(); ++i)
+  const bool do_diag_optimize = (sp.n_rows() == sp.n_cols());
+  std::vector<unsigned int> row_lengths (sp.n_rows());
+  for (size_type i=0; i<sp.n_rows(); ++i)
     {
-      row_lengths[i] = dsp.row_length(i);
-      if (do_diag_optimize && !dsp.exists(i,i))
+      row_lengths[i] = sp.row_length(i);
+      if (do_diag_optimize && !sp.exists(i,i))
         ++row_lengths[i];
     }
-  reinit (dsp.n_rows(), dsp.n_cols(), row_lengths);
+  reinit (sp.n_rows(), sp.n_cols(), row_lengths);
 
   // now enter all the elements into the matrix, if there are any. note that
   // if the matrix is quadratic, then we already have the diagonal element
   // preallocated
   if (n_rows() != 0 && n_cols() != 0)
-    for (size_type row = 0; row<dsp.n_rows(); ++row)
+    for (size_type row = 0; row<sp.n_rows(); ++row)
       {
         size_type *cols = &colnums[rowstart[row]] + (do_diag_optimize ? 1 : 0);
-        typename SparsityPatternType::iterator col_num = dsp.begin (row),
-                                               end_row = dsp.end (row);
+        typename SparsityPattern::iterator col_num = sp.begin (row),
+                                           end_row = sp.end (row);
 
         for (; col_num != end_row; ++col_num)
           {
@@ -519,8 +498,8 @@ SparsityPattern::copy_from (const SparsityPatternType &dsp)
       }
 
   // do not need to compress the sparsity pattern since we already have
-  // allocated the right amount of data, and the SparsityPatternType data is sorted,
-  // too.
+  // allocated the right amount of data, and the SparsityPattern data is
+  // sorted, too.
   compressed = true;
 }
 
@@ -531,7 +510,6 @@ SparsityPattern::copy_from (const SparsityPatternType &dsp)
 // entries. DynamicSparsityPattern::iterator can show quadratic complexity in
 // case many rows are empty and the begin() method needs to jump to the next
 // free row. Otherwise, the code is exactly the same as above.
-template <>
 void
 SparsityPattern::copy_from (const DynamicSparsityPattern &dsp)
 {
@@ -572,13 +550,15 @@ void SparsityPattern::copy_from (const FullMatrix<number> &matrix)
   // first init with the number of entries per row. if this matrix is square
   // then we also have to allocate memory for the diagonal entry, unless we
   // have already counted it
+  const bool matrix_is_square = (matrix.m() == matrix.n());
+
   std::vector<unsigned int> entries_per_row (matrix.m(), 0);
   for (size_type row=0; row<matrix.m(); ++row)
     {
       for (size_type col=0; col<matrix.n(); ++col)
         if (matrix(row,col) != 0)
           ++entries_per_row[row];
-      if ((matrix.m() == matrix.n())
+      if (matrix_is_square
           &&
           (matrix(row,row) == 0))
         ++entries_per_row[row];
@@ -586,11 +566,47 @@ void SparsityPattern::copy_from (const FullMatrix<number> &matrix)
 
   reinit (matrix.m(), matrix.n(), entries_per_row);
 
-  // now set entries
+  // now set entries. if we enter entries row by row, then we'll get
+  // quadratic complexity in the number of entries per row. this is
+  // not usually a problem (we don't usually create dense matrices),
+  // but there are cases where it matters -- so we may as well be
+  // gentler and hand over a whole row of entries at a time
+  std::vector<size_type> column_indices;
+  column_indices.reserve (entries_per_row.size() > 0
+                          ?
+                          *std::max_element (entries_per_row.begin(),
+                                             entries_per_row.end())
+                          :
+                          0);
   for (size_type row=0; row<matrix.m(); ++row)
-    for (size_type col=0; col<matrix.n(); ++col)
-      if (matrix(row,col) != 0)
-        add (row,col);
+    {
+      column_indices.resize(entries_per_row[row]);
+
+      size_type current_index = 0;
+      for (size_type col=0; col<matrix.n(); ++col)
+        if (matrix(row,col) != 0)
+          {
+            column_indices[current_index] = col;
+            ++current_index;
+          }
+        else
+          // the (row,col) entry is zero; check if we need to add it
+          // anyway because it's the diagonal entry of a square
+          // matrix
+          if (matrix_is_square
+              &&
+              (col == row))
+            {
+              column_indices[current_index] = row;
+              ++current_index;
+            }
+
+      // check that we really added the correct number of indices
+      Assert (current_index == entries_per_row[row], ExcInternalError());
+
+      // now bulk add all of these entries
+      add_entries(row, column_indices.begin(), column_indices.end(), true);
+    }
 
   // finally compress
   compress ();
@@ -616,12 +632,12 @@ SparsityPattern::empty () const
   // and freeing memory was not present in the original implementation and I
   // don't know at how many places I missed something in adding it, so I try
   // to be cautious. wb)
-  if ((rowstart==0) || (rows==0) || (cols==0))
+  if ((rowstart==nullptr) || (rows==0) || (cols==0))
     {
-      Assert (rowstart==0, ExcInternalError());
+      Assert (rowstart==nullptr, ExcInternalError());
       Assert (rows==0, ExcInternalError());
       Assert (cols==0, ExcInternalError());
-      Assert (colnums==0, ExcInternalError());
+      Assert (colnums==nullptr, ExcInternalError());
       Assert (max_vec_len==0, ExcInternalError());
 
       return true;
@@ -654,7 +670,7 @@ SparsityPattern::size_type
 SparsityPattern::operator () (const size_type i,
                               const size_type j) const
 {
-  Assert ((rowstart!=0) && (colnums!=0), ExcEmptyObject());
+  Assert ((rowstart!=nullptr) && (colnums!=nullptr), ExcEmptyObject());
   Assert (i<rows, ExcIndexRange(i,0,rows));
   Assert (j<cols, ExcIndexRange(j,0,cols));
   Assert (compressed, ExcNotCompressed());
@@ -682,7 +698,7 @@ SparsityPattern::operator () (const size_type i,
                                                  &colnums[rowstart[i+1]],
                                                  j);
   if ((p != &colnums[rowstart[i+1]])  &&  (*p == j))
-    return (p - &colnums[0]);
+    return (p - colnums.get());
   else
     return invalid_entry;
 }
@@ -693,7 +709,7 @@ void
 SparsityPattern::add (const size_type i,
                       const size_type j)
 {
-  Assert ((rowstart!=0) && (colnums!=0), ExcEmptyObject());
+  Assert ((rowstart!=nullptr) && (colnums!=nullptr), ExcEmptyObject());
   Assert (i<rows, ExcIndexRange(i,0,rows));
   Assert (j<cols, ExcIndexRange(j,0,cols));
   Assert (compressed==false, ExcMatrixIsCompressed());
@@ -768,7 +784,7 @@ SparsityPattern::add_entries (const size_type row,
 bool
 SparsityPattern::exists (const size_type i, const size_type j) const
 {
-  Assert ((rowstart!=0) && (colnums!=0), ExcEmptyObject());
+  Assert ((rowstart!=nullptr) && (colnums!=nullptr), ExcEmptyObject());
   Assert (i<rows, ExcIndexRange(i,0,rows));
   Assert (j<cols, ExcIndexRange(j,0,cols));
 
@@ -785,7 +801,7 @@ SparsityPattern::exists (const size_type i, const size_type j) const
 SparsityPattern::size_type
 SparsityPattern::row_position (const size_type i, const size_type j) const
 {
-  Assert ((rowstart!=0) && (colnums!=0), ExcEmptyObject());
+  Assert ((rowstart!=nullptr) && (colnums!=nullptr), ExcEmptyObject());
   Assert (i<rows, ExcIndexRange(i,0,rows));
   Assert (j<cols, ExcIndexRange(j,0,cols));
 
@@ -811,8 +827,8 @@ SparsityPattern::matrix_position (const std::size_t global_index) const
   // it is sorted, and since there is an element for the one-past-last row, we
   // can simply use a bisection search on it
   const size_type row
-    = (std::upper_bound (&rowstart[0], &rowstart[rows], global_index)
-       - &rowstart[0] - 1);
+    = (std::upper_bound (rowstart.get(), rowstart.get() + rows, global_index)
+       - rowstart.get() - 1);
 
   // now, the column index is simple since that is what the colnums array
   // stores:
@@ -827,7 +843,7 @@ SparsityPattern::matrix_position (const std::size_t global_index) const
 void
 SparsityPattern::symmetrize ()
 {
-  Assert ((rowstart!=0) && (colnums!=0), ExcEmptyObject());
+  Assert ((rowstart!=nullptr) && (colnums!=nullptr), ExcEmptyObject());
   Assert (compressed==false, ExcMatrixIsCompressed());
   // Note that we only require a quadratic matrix here, no special treatment
   // of diagonals
@@ -861,7 +877,7 @@ SparsityPattern::symmetrize ()
 void
 SparsityPattern::print (std::ostream &out) const
 {
-  Assert ((rowstart!=0) && (colnums!=0), ExcEmptyObject());
+  Assert ((rowstart!=nullptr) && (colnums!=nullptr), ExcEmptyObject());
 
   AssertThrow (out, ExcIO());
 
@@ -882,7 +898,7 @@ SparsityPattern::print (std::ostream &out) const
 void
 SparsityPattern::print_gnuplot (std::ostream &out) const
 {
-  Assert ((rowstart!=0) && (colnums!=0), ExcEmptyObject());
+  Assert ((rowstart!=nullptr) && (colnums!=nullptr), ExcEmptyObject());
 
   AssertThrow (out, ExcIO());
 
@@ -912,7 +928,7 @@ SparsityPattern::print_svg (std::ostream &out) const
       "    ]]>\n"
       "  </style>\n\n"
       "   <rect width=\"" << n+2 << "\" height=\"" << m+2 << "\" fill=\"rgb(128, 128, 128)\"/>\n"
-      "   <rect x=\"1\" y=\"1\" width=\"" << n << "\" height=\"" << m
+      "   <rect x=\"1\" y=\"1\" width=\"" << n+0.1 << "\" height=\"" << m+0.1
       << "\" fill=\"rgb(255, 255, 255)\"/>\n\n";
 
   SparsityPattern::iterator
@@ -934,7 +950,7 @@ SparsityPattern::print_svg (std::ostream &out) const
 SparsityPattern::size_type
 SparsityPattern::bandwidth () const
 {
-  Assert ((rowstart!=0) && (colnums!=0), ExcEmptyObject());
+  Assert ((rowstart!=nullptr) && (colnums!=nullptr), ExcEmptyObject());
   size_type b=0;
   for (size_type i=0; i<rows; ++i)
     for (size_type j=rowstart[i]; j<rowstart[i+1]; ++j)
@@ -964,13 +980,13 @@ SparsityPattern::block_write (std::ostream &out) const
       << compressed << ' '
       << store_diagonal_first_in_row << "][";
   // then write out real data
-  out.write (reinterpret_cast<const char *>(&rowstart[0]),
-             reinterpret_cast<const char *>(&rowstart[max_dim+1])
-             - reinterpret_cast<const char *>(&rowstart[0]));
+  out.write (reinterpret_cast<const char *>(rowstart.get()),
+             reinterpret_cast<const char *>(rowstart.get() + max_dim + 1)
+             - reinterpret_cast<const char *>(rowstart.get()));
   out << "][";
-  out.write (reinterpret_cast<const char *>(&colnums[0]),
-             reinterpret_cast<const char *>(&colnums[max_vec_len])
-             - reinterpret_cast<const char *>(&colnums[0]));
+  out.write (reinterpret_cast<const char *>(colnums.get()),
+             reinterpret_cast<const char *>(colnums.get() + max_vec_len)
+             - reinterpret_cast<const char *>(colnums.get()));
   out << ']';
 
   AssertThrow (out, ExcIO());
@@ -1002,25 +1018,20 @@ SparsityPattern::block_read (std::istream &in)
   AssertThrow (c == '[', ExcIO());
 
   // reallocate space
-  if (rowstart)
-    delete[] rowstart;
-  if (colnums)
-    delete[] colnums;
-
-  rowstart = new std::size_t[max_dim+1];
-  colnums  = new size_type[max_vec_len];
+  rowstart = std_cxx14::make_unique<std::size_t[]> (max_dim+1);
+  colnums = std_cxx14::make_unique<size_type[]> (max_vec_len);
 
   // then read data
-  in.read (reinterpret_cast<char *>(&rowstart[0]),
-           reinterpret_cast<char *>(&rowstart[max_dim+1])
-           - reinterpret_cast<char *>(&rowstart[0]));
+  in.read (reinterpret_cast<char *>(rowstart.get()),
+           reinterpret_cast<char *>(rowstart.get() + max_dim + 1)
+           - reinterpret_cast<char *>(rowstart.get()));
   in >> c;
   AssertThrow (c == ']', ExcIO());
   in >> c;
   AssertThrow (c == '[', ExcIO());
-  in.read (reinterpret_cast<char *>(&colnums[0]),
-           reinterpret_cast<char *>(&colnums[max_vec_len])
-           - reinterpret_cast<char *>(&colnums[0]));
+  in.read (reinterpret_cast<char *>(colnums.get()),
+           reinterpret_cast<char *>(colnums.get() + max_vec_len)
+           - reinterpret_cast<char *>(colnums.get()));
   in >> c;
   AssertThrow (c == ']', ExcIO());
 }
@@ -1038,11 +1049,10 @@ SparsityPattern::memory_consumption () const
 
 
 // explicit instantiations
-template void SparsityPattern::copy_from<SparsityPattern> (const SparsityPattern &);
 template void SparsityPattern::copy_from<float> (const FullMatrix<float> &);
 template void SparsityPattern::copy_from<double> (const FullMatrix<double> &);
 
-template void SparsityPattern::add_entries<const SparsityPattern::size_type *> (const size_type ,
+template void SparsityPattern::add_entries<const SparsityPattern::size_type *> (const size_type,
     const size_type *,
     const size_type *,
     const bool);
