@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 2008 - 2016 by the deal.II authors
+// Copyright (C) 2008 - 2018 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -22,6 +22,7 @@
 #include <algorithm>
 #include <functional>
 #include <set>
+#include <deal.II/base/std_cxx14/memory.h>
 
 #ifdef DEAL_II_WITH_MPI
 #include <deal.II/base/utilities.h>
@@ -31,23 +32,334 @@
 #endif
 
 #ifdef DEAL_II_WITH_METIS
-DEAL_II_DISABLE_EXTRA_DIAGNOSTICS
 extern "C"
 {
 #include <metis.h>
 }
-DEAL_II_ENABLE_EXTRA_DIAGNOSTICS
 #endif
 
+#ifdef DEAL_II_TRILINOS_WITH_ZOLTAN
+#include <zoltan_cpp.h>
+#endif
+
+#include <string>
+
 
 DEAL_II_NAMESPACE_OPEN
 
 namespace SparsityTools
 {
 
+  namespace
+  {
+    void partition_metis (const SparsityPattern           &sparsity_pattern,
+                          const std::vector<unsigned int> &cell_weights,
+                          const unsigned int               n_partitions,
+                          std::vector<unsigned int>       &partition_indices)
+    {
+      // Make sure that METIS is actually
+      // installed and detected
+#ifndef DEAL_II_WITH_METIS
+      (void) sparsity_pattern;
+      (void) cell_weights;
+      (void) n_partitions;
+      (void) partition_indices;
+      AssertThrow (false, ExcMETISNotInstalled());
+#else
+
+      // generate the data structures for
+      // METIS. Note that this is particularly
+      // simple, since METIS wants exactly our
+      // compressed row storage format. we only
+      // have to set up a few auxiliary arrays
+      idx_t
+      n       = static_cast<signed int>(sparsity_pattern.n_rows()),
+      ncon    = 1,                              // number of balancing constraints (should be >0)
+      nparts  = static_cast<int>(n_partitions), // number of subdomains to create
+      dummy;                                    // the numbers of edges cut by the
+      // resulting partition
+
+      // We can not partition n items into more than n parts. METIS will
+      // generate non-sensical output (everything is owned by a single process)
+      // and complain with a message (but won't return an error code!):
+      // ***Cannot bisect a graph with 0 vertices!
+      // ***You are trying to partition a graph into too many parts!
+      nparts = std::min(n, nparts);
+
+      // use default options for METIS
+      idx_t options[METIS_NOPTIONS];
+      METIS_SetDefaultOptions (options);
+
+      // one more nuisance: we have to copy our own data to arrays that store
+      // signed integers :-(
+      std::vector<idx_t> int_rowstart(1);
+      int_rowstart.reserve(sparsity_pattern.n_rows()+1);
+      std::vector<idx_t> int_colnums;
+      int_colnums.reserve(sparsity_pattern.n_nonzero_elements());
+      for (SparsityPattern::size_type row=0; row<sparsity_pattern.n_rows(); ++row)
+        {
+          for (SparsityPattern::iterator col=sparsity_pattern.begin(row);
+               col < sparsity_pattern.end(row); ++col)
+            int_colnums.push_back(col->column());
+          int_rowstart.push_back(int_colnums.size());
+        }
+
+      std::vector<idx_t> int_partition_indices (sparsity_pattern.n_rows());
+
+      // Setup cell weighting option
+      std::vector<idx_t> int_cell_weights;
+      if (cell_weights.size() > 0)
+        {
+          Assert(cell_weights.size() == sparsity_pattern.n_rows(),
+                 ExcDimensionMismatch(cell_weights.size(),sparsity_pattern.n_rows()));
+          int_cell_weights.resize(cell_weights.size());
+          std::copy (cell_weights.begin(),
+                     cell_weights.end(),
+                     int_cell_weights.begin());
+        }
+      // Set a pointer to the optional cell weighting information.
+      // METIS expects a null pointer if there are no weights to be considered.
+      idx_t *const p_int_cell_weights = (cell_weights.size() > 0 ? int_cell_weights.data() : nullptr);
+
+
+      // Make use of METIS' error code.
+      int ierr;
+
+      // Select which type of partitioning to create
+
+      // Use recursive if the number of partitions is less than or equal to 8
+      if (nparts <= 8)
+        ierr = METIS_PartGraphRecursive(&n, &ncon, int_rowstart.data(), int_colnums.data(),
+                                        p_int_cell_weights, nullptr, nullptr,
+                                        &nparts,nullptr,nullptr,options,
+                                        &dummy,int_partition_indices.data());
+
+      // Otherwise use kway
+      else
+        ierr = METIS_PartGraphKway(&n, &ncon, int_rowstart.data(), int_colnums.data(),
+                                   p_int_cell_weights, nullptr, nullptr,
+                                   &nparts,nullptr,nullptr,options,
+                                   &dummy,int_partition_indices.data());
+
+      // If metis returns normally, an error code METIS_OK=1 is returned from
+      // the above functions (see metish.h)
+      AssertThrow (ierr == 1, ExcMETISError (ierr));
+
+      // now copy back generated indices into the output array
+      std::copy (int_partition_indices.begin(),
+                 int_partition_indices.end(),
+                 partition_indices.begin());
+#endif
+    }
+
+
+//Query functions unused if zoltan is not installed
+#ifdef DEAL_II_TRILINOS_WITH_ZOLTAN
+    //Query functions for partition_zoltan
+    int get_number_of_objects(void *data, int *ierr)
+    {
+      SparsityPattern *graph = reinterpret_cast<SparsityPattern *>(data);
+
+      *ierr = ZOLTAN_OK;
+
+      return graph->n_rows();
+    }
+
+
+    void get_object_list(void *data, int /*sizeGID*/, int /*sizeLID*/,
+                         ZOLTAN_ID_PTR globalID, ZOLTAN_ID_PTR localID,
+                         int /*wgt_dim*/, float * /*obj_wgts*/, int *ierr)
+    {
+      SparsityPattern *graph = reinterpret_cast<SparsityPattern *>(data);
+      *ierr = ZOLTAN_OK;
+
+      Assert( globalID != nullptr , ExcInternalError() );
+      Assert( localID != nullptr , ExcInternalError() );
+
+      //set global degrees of freedom
+      auto n_dofs = graph->n_rows();
+
+      for (unsigned int i=0; i < n_dofs; i++)
+        {
+          globalID[i] = i;
+          localID[i] = i;     //Same as global ids.
+        }
+    }
+
+
+    void get_num_edges_list(void *data, int /*sizeGID*/, int /*sizeLID*/,
+                            int num_obj,
+                            ZOLTAN_ID_PTR globalID, ZOLTAN_ID_PTR /*localID*/,
+                            int *numEdges, int *ierr)
+    {
+      SparsityPattern *graph = reinterpret_cast<SparsityPattern *>(data);
+
+      *ierr = ZOLTAN_OK;
+
+      Assert ( numEdges != nullptr , ExcInternalError() );
+
+      for (int i=0; i<num_obj; ++i)
+        {
+          if ( graph->exists(i,i) )     //Check if diagonal element is present
+            numEdges[i] = graph->row_length(globalID[i])-1;
+          else
+            numEdges[i] = graph->row_length(globalID[i]);
+        }
+    }
+
+
+
+    void get_edge_list(void *data, int /*sizeGID*/, int /*sizeLID*/,
+                       int num_obj, ZOLTAN_ID_PTR /*globalID*/, ZOLTAN_ID_PTR /*localID*/,
+                       int * /*num_edges*/,
+                       ZOLTAN_ID_PTR nborGID, int *nborProc,
+                       int /*wgt_dim*/, float * /*ewgts*/, int *ierr)
+    {
+      SparsityPattern *graph = reinterpret_cast<SparsityPattern *>(data);
+      *ierr = ZOLTAN_OK;
+
+      ZOLTAN_ID_PTR nextNborGID = nborGID;
+      int *nextNborProc = nborProc;
+
+      //Loop through rows corresponding to indices in globalID implicitly
+      for (SparsityPattern::size_type i=0; i < static_cast<SparsityPattern::size_type>(num_obj); ++i)
+        {
+          //Loop through each column to find neighbours
+          for ( SparsityPattern::iterator col=graph->begin(i);
+                col < graph->end(i); ++col )
+            //Ignore diagonal entries. Not needed for partitioning.
+            if ( i != col->column() )
+              {
+                Assert( nextNborGID != nullptr , ExcInternalError() );
+                Assert( nextNborProc != nullptr , ExcInternalError() );
+
+                *nextNborGID++ = col->column();
+                *nextNborProc++ = 0;    //All the vertices on processor 0
+              }
+        }
+    }
+#endif
+
+
+    void partition_zoltan (const SparsityPattern           &sparsity_pattern,
+                           const std::vector<unsigned int> &cell_weights,
+                           const unsigned int               n_partitions,
+                           std::vector<unsigned int>       &partition_indices)
+    {
+      // Make sure that ZOLTAN is actually
+      // installed and detected
+#ifndef DEAL_II_TRILINOS_WITH_ZOLTAN
+      (void) sparsity_pattern;
+      (void) cell_weights;
+      (void) n_partitions;
+      (void) partition_indices;
+      AssertThrow (false, ExcZOLTANNotInstalled());
+#else
+
+      Assert(cell_weights.size() == 0,
+             ExcMessage("The cell weighting functionality for Zoltan has not yet been implemented."));
+      (void) cell_weights;
+
+      //MPI environment must have been initialized by this point.
+      std::unique_ptr<Zoltan> zz = std_cxx14::make_unique<Zoltan>(MPI_COMM_SELF);
+
+      //General parameters
+      // DEBUG_LEVEL call must precede the call to LB_METHOD
+      zz->Set_Param( "DEBUG_LEVEL", "0" );   //set level of debug info
+      zz->Set_Param( "LB_METHOD", "GRAPH" );  //graph based partition method (LB-load balancing)
+      zz->Set_Param( "NUM_LOCAL_PARTS", std::to_string(n_partitions) ); //set number of partitions
+
+      // The PHG partitioner is a hypergraph partitioner that Zoltan could use
+      // for graph partitioning.
+      // If number of vertices in hyperedge divided by total vertices in
+      // hypergraph exceeds PHG_EDGE_SIZE_THRESHOLD,
+      // then the hyperedge will be omitted as such (dense) edges will likely
+      // incur high communication costs regardless of the partition.
+      // PHG_EDGE_SIZE_THRESHOLD value is raised to 0.5 from the default
+      // value of 0.25 so that the PHG partitioner doesn't throw warning saying
+      // "PHG_EDGE_SIZE_THRESHOLD is low ..." after removing all dense edges.
+      // For instance, in two dimensions if the triangulation being partitioned
+      // is two quadrilaterals sharing an edge and if PHG_EDGE_SIZE_THRESHOLD
+      // value is set to 0.25, PHG will remove all the edges throwing the
+      // above warning.
+      zz->Set_Param( "PHG_EDGE_SIZE_THRESHOLD", "0.5" );
+
+      //Need a non-const object equal to sparsity_pattern
+      SparsityPattern graph;
+      graph.copy_from(sparsity_pattern);
+
+      //Set query functions
+      zz->Set_Num_Obj_Fn(get_number_of_objects, &graph);
+      zz->Set_Obj_List_Fn(get_object_list, &graph);
+      zz->Set_Num_Edges_Multi_Fn(get_num_edges_list, &graph);
+      zz->Set_Edge_List_Multi_Fn(get_edge_list, &graph);
+
+      //Variables needed by partition function
+      int changes=0;
+      int num_gid_entries=1;
+      int num_lid_entries=1;
+      int num_import=0;
+      ZOLTAN_ID_PTR import_global_ids=nullptr;
+      ZOLTAN_ID_PTR import_local_ids=nullptr;
+      int *import_procs=nullptr;
+      int *import_to_part=nullptr;
+      int num_export=0;
+      ZOLTAN_ID_PTR export_global_ids=nullptr;
+      ZOLTAN_ID_PTR export_local_ids=nullptr;
+      int *export_procs=nullptr;
+      int *export_to_part=nullptr;
+
+      //call partitioner
+      const int rc = zz->LB_Partition (changes,
+                                       num_gid_entries,
+                                       num_lid_entries,
+                                       num_import,
+                                       import_global_ids,
+                                       import_local_ids,
+                                       import_procs,
+                                       import_to_part,
+                                       num_export,
+                                       export_global_ids,
+                                       export_local_ids,
+                                       export_procs,
+                                       export_to_part);
+      (void)rc;
+
+      //check for error code in partitioner
+      Assert( rc == ZOLTAN_OK , ExcInternalError() );
+
+      //By default, all indices belong to part 0. After zoltan partition
+      //some are migrated to different part ID, which is stored in export_to_part array.
+      std::fill(partition_indices.begin(), partition_indices.end(), 0);
+
+      //copy from export_to_part to partition_indices, whose part_ids != 0.
+      for (int i=0; i < num_export; i++)
+        partition_indices[export_local_ids[i]] = export_to_part[i];
+#endif
+    }
+  }
+
+
   void partition (const SparsityPattern     &sparsity_pattern,
                   const unsigned int         n_partitions,
-                  std::vector<unsigned int> &partition_indices)
+                  std::vector<unsigned int> &partition_indices,
+                  const Partitioner partitioner
+                 )
+  {
+    std::vector<unsigned int> cell_weights;
+
+    // Call the other more general function
+    partition(sparsity_pattern, cell_weights, n_partitions,
+              partition_indices, partitioner);
+  }
+
+
+  void partition (const SparsityPattern           &sparsity_pattern,
+                  const std::vector<unsigned int> &cell_weights,
+                  const unsigned int               n_partitions,
+                  std::vector<unsigned int>       &partition_indices,
+                  const Partitioner                partitioner
+                 )
   {
     Assert (sparsity_pattern.n_rows()==sparsity_pattern.n_cols(),
             ExcNotQuadratic());
@@ -66,79 +378,81 @@ namespace SparsityTools
         return;
       }
 
-    // Make sure that METIS is actually
-    // installed and detected
-#ifndef DEAL_II_WITH_METIS
-    (void)sparsity_pattern;
-    AssertThrow (false, ExcMETISNotInstalled());
-#else
-
-    // generate the data structures for
-    // METIS. Note that this is particularly
-    // simple, since METIS wants exactly our
-    // compressed row storage format. we only
-    // have to set up a few auxiliary arrays
-    idx_t
-    n       = static_cast<signed int>(sparsity_pattern.n_rows()),
-    ncon    = 1,                              // number of balancing constraints (should be >0)
-    nparts  = static_cast<int>(n_partitions), // number of subdomains to create
-    dummy;                                    // the numbers of edges cut by the
-    // resulting partition
-
-    // We can not partition n items into more than n parts. METIS will
-    // generate non-sensical output (everything is owned by a single process)
-    // and complain with a message (but won't return an error code!):
-    // ***Cannot bisect a graph with 0 vertices!
-    // ***You are trying to partition a graph into too many parts!
-    nparts = std::min(n, nparts);
-
-    // use default options for METIS
-    idx_t options[METIS_NOPTIONS];
-    METIS_SetDefaultOptions (options);
-
-    // one more nuisance: we have to copy our own data to arrays that store
-    // signed integers :-(
-    std::vector<idx_t> int_rowstart(1);
-    int_rowstart.reserve(sparsity_pattern.n_rows()+1);
-    std::vector<idx_t> int_colnums;
-    int_colnums.reserve(sparsity_pattern.n_nonzero_elements());
-    for (SparsityPattern::size_type row=0; row<sparsity_pattern.n_rows(); ++row)
-      {
-        for (SparsityPattern::iterator col=sparsity_pattern.begin(row);
-             col < sparsity_pattern.end(row); ++col)
-          int_colnums.push_back(col->column());
-        int_rowstart.push_back(int_colnums.size());
-      }
-
-    std::vector<idx_t> int_partition_indices (sparsity_pattern.n_rows());
-
-    // Make use of METIS' error code.
-    int ierr;
-
-    // Select which type of partitioning to create
-
-    // Use recursive if the number of partitions is less than or equal to 8
-    if (nparts <= 8)
-      ierr = METIS_PartGraphRecursive(&n, &ncon, &int_rowstart[0], &int_colnums[0],
-                                      NULL, NULL, NULL,
-                                      &nparts,NULL,NULL,&options[0],
-                                      &dummy,&int_partition_indices[0]);
-
-    // Otherwise use kway
+    if (partitioner == Partitioner::metis)
+      partition_metis(sparsity_pattern, cell_weights, n_partitions, partition_indices);
+    else if (partitioner == Partitioner::zoltan)
+      partition_zoltan(sparsity_pattern, cell_weights, n_partitions, partition_indices);
     else
-      ierr = METIS_PartGraphKway(&n, &ncon, &int_rowstart[0], &int_colnums[0],
-                                 NULL, NULL, NULL,
-                                 &nparts,NULL,NULL,&options[0],
-                                 &dummy,&int_partition_indices[0]);
+      AssertThrow(false, ExcInternalError());
+  }
 
-    // If metis returns normally, an error code METIS_OK=1 is returned from
-    // the above functions (see metish.h)
-    AssertThrow (ierr == 1, ExcMETISError (ierr));
 
-    // now copy back generated indices into the output array
-    std::copy (int_partition_indices.begin(),
-               int_partition_indices.end(),
-               partition_indices.begin());
+  unsigned int color_sparsity_pattern (const SparsityPattern     &sparsity_pattern,
+                                       std::vector<unsigned int> &color_indices)
+  {
+    // Make sure that ZOLTAN is actually
+    // installed and detected
+#ifndef DEAL_II_TRILINOS_WITH_ZOLTAN
+    (void) sparsity_pattern;
+    (void) color_indices;
+    AssertThrow (false, ExcZOLTANNotInstalled());
+    return 0;
+#else
+    //coloring algorithm is run in serial by each processor.
+    std::unique_ptr<Zoltan> zz = std_cxx14::make_unique<Zoltan> (MPI_COMM_SELF);
+
+    //Coloring parameters
+    // DEBUG_LEVEL must precede all other calls
+    zz->Set_Param ("DEBUG_LEVEL", "0" );     //level of debug info
+    zz->Set_Param ("COLORING_PROBLEM", "DISTANCE-1");  //Standard coloring
+    zz->Set_Param ("NUM_GID_ENTRIES", "1");  // 1 entry represents global ID
+    zz->Set_Param ("NUM_LID_ENTRIES", "1");  // 1 entry represents local ID
+    zz->Set_Param ("OBJ_WEIGHT_DIM", "0");   // object weights not used
+    zz->Set_Param ("RECOLORING_NUM_OF_ITERATIONS", "0");
+
+    //Zoltan::Color function requires a non-const SparsityPattern object
+    SparsityPattern graph;
+    graph.copy_from (sparsity_pattern);
+
+    //Set query functions required by coloring function
+    zz->Set_Num_Obj_Fn (get_number_of_objects, &graph);
+    zz->Set_Obj_List_Fn (get_object_list, &graph);
+    zz->Set_Num_Edges_Multi_Fn (get_num_edges_list, &graph);
+    zz->Set_Edge_List_Multi_Fn (get_edge_list, &graph);
+
+    //Variables needed by coloring function
+    int num_gid_entries=1;
+    const int num_objects = graph.n_rows();
+
+    //Preallocate input variables. Element type fixed by ZOLTAN.
+    std::vector<ZOLTAN_ID_TYPE> global_ids (num_objects);
+    std::vector<int> color_exp (num_objects);
+
+    //Set ids for which coloring needs to be done
+    for (int i=0; i < num_objects; i++)
+      global_ids[i] = i;
+
+    //Call ZOLTAN coloring algorithm
+    int rc = zz->Color (num_gid_entries,
+                        num_objects,
+                        global_ids.data(),
+                        color_exp.data());
+
+    (void) rc;
+    //Check for error code
+    Assert (rc == ZOLTAN_OK ,
+            ExcInternalError());
+
+    //Allocate and assign color indices
+    color_indices.resize(num_objects);
+    Assert (color_exp.size() == color_indices.size(),
+            ExcDimensionMismatch (color_exp.size(),color_indices.size()));
+
+    std::copy (color_exp.begin(), color_exp.end(), color_indices.begin());
+
+    unsigned int n_colors = *(std::max_element (color_indices.begin(),
+                                                color_indices.end()));
+    return n_colors;
 #endif
   }
 
@@ -328,23 +642,6 @@ namespace SparsityTools
 
 
 
-  void
-  reorder_Cuthill_McKee (const SparsityPattern                   &sparsity,
-                         std::vector<SparsityPattern::size_type> &new_indices,
-                         const std::vector<SparsityPattern::size_type> &starting_indices)
-  {
-    DynamicSparsityPattern dsp(sparsity.n_rows(), sparsity.n_cols());
-    for (unsigned int row=0; row<sparsity.n_rows(); ++row)
-      {
-        for (SparsityPattern::iterator it=sparsity.begin(row); it!=sparsity.end(row)
-             && it->is_valid_entry() ; ++it)
-          dsp.add(row, it->column());
-      }
-    reorder_Cuthill_McKee(dsp, new_indices, starting_indices);
-  }
-
-
-
   namespace internal
   {
     void
@@ -431,7 +728,7 @@ namespace SparsityTools
 
               // Add the pivot and all direct neighbors of the pivot node not
               // yet touched to the list of new entries.
-              groups.push_back(std::vector<types::global_dof_index>());
+              groups.emplace_back();
               std::vector<types::global_dof_index> &next_group = groups.back();
 
               next_group.push_back(min_neighbors.first);
@@ -621,7 +918,7 @@ namespace SparsityTools
           ierr = MPI_Get_count(&status, DEAL_II_DOF_INDEX_MPI_TYPE, &len);
           AssertThrowMPI(ierr);
           recv_buf.resize(len);
-          ierr = MPI_Recv(&recv_buf[0], len, DEAL_II_DOF_INDEX_MPI_TYPE, status.MPI_SOURCE,
+          ierr = MPI_Recv(recv_buf.data(), len, DEAL_II_DOF_INDEX_MPI_TYPE, status.MPI_SOURCE,
                           status.MPI_TAG, mpi_comm, &status);
           AssertThrowMPI(ierr);
 
@@ -646,7 +943,7 @@ namespace SparsityTools
     // complete all sends, so that we can safely destroy the buffers.
     if (requests.size())
       {
-        const int ierr = MPI_Waitall(requests.size(), &requests[0], MPI_STATUSES_IGNORE);
+        const int ierr = MPI_Waitall(requests.size(), requests.data(), MPI_STATUSES_IGNORE);
         AssertThrowMPI(ierr);
       }
 
@@ -752,7 +1049,7 @@ namespace SparsityTools
           ierr = MPI_Get_count(&status, DEAL_II_DOF_INDEX_MPI_TYPE, &len);
           AssertThrowMPI(ierr);
           recv_buf.resize(len);
-          ierr = MPI_Recv(&recv_buf[0], len, DEAL_II_DOF_INDEX_MPI_TYPE, status.MPI_SOURCE,
+          ierr = MPI_Recv(recv_buf.data(), len, DEAL_II_DOF_INDEX_MPI_TYPE, status.MPI_SOURCE,
                           status.MPI_TAG, mpi_comm, &status);
           AssertThrowMPI(ierr);
 
@@ -777,7 +1074,7 @@ namespace SparsityTools
     // complete all sends, so that we can safely destroy the buffers.
     if (requests.size())
       {
-        const int ierr = MPI_Waitall(requests.size(), &requests[0], MPI_STATUSES_IGNORE);
+        const int ierr = MPI_Waitall(requests.size(), requests.data(), MPI_STATUSES_IGNORE);
         AssertThrowMPI(ierr);
       }
   }
