@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 2005 - 2014 by the deal.II authors
+// Copyright (C) 2005 - 2017 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -22,7 +22,7 @@ DEAL_II_NAMESPACE_OPEN
 
 using namespace LAPACKSupport;
 
-template<typename number>
+template <typename number>
 TridiagonalMatrix<number>::TridiagonalMatrix(
   size_type size,
   bool symmetric)
@@ -35,7 +35,8 @@ TridiagonalMatrix<number>::TridiagonalMatrix(
 {}
 
 
-template<typename number>
+
+template <typename number>
 void
 TridiagonalMatrix<number>::reinit(
   size_type size,
@@ -49,7 +50,8 @@ TridiagonalMatrix<number>::reinit(
 }
 
 
-template<typename number>
+
+template <typename number>
 bool
 TridiagonalMatrix<number>::all_zero() const
 {
@@ -73,7 +75,8 @@ TridiagonalMatrix<number>::all_zero() const
 }
 
 
-template<typename number>
+
+template <typename number>
 void
 TridiagonalMatrix<number>::vmult (
   Vector<number>       &w,
@@ -127,7 +130,7 @@ TridiagonalMatrix<number>::vmult (
 }
 
 
-template<typename number>
+template <typename number>
 void
 TridiagonalMatrix<number>::vmult_add (
   Vector<number>       &w,
@@ -137,7 +140,8 @@ TridiagonalMatrix<number>::vmult_add (
 }
 
 
-template<typename number>
+
+template <typename number>
 void
 TridiagonalMatrix<number>::Tvmult (
   Vector<number>       &w,
@@ -181,7 +185,8 @@ TridiagonalMatrix<number>::Tvmult (
 }
 
 
-template<typename number>
+
+template <typename number>
 void
 TridiagonalMatrix<number>::Tvmult_add (
   Vector<number>       &w,
@@ -191,7 +196,8 @@ TridiagonalMatrix<number>::Tvmult_add (
 }
 
 
-template<typename number>
+
+template <typename number>
 number
 TridiagonalMatrix<number>::matrix_scalar_product(
   const Vector<number> &w,
@@ -218,7 +224,8 @@ TridiagonalMatrix<number>::matrix_scalar_product(
 }
 
 
-template<typename number>
+
+template <typename number>
 number
 TridiagonalMatrix<number>::matrix_norm_square(
   const Vector<number> &v) const
@@ -227,38 +234,40 @@ TridiagonalMatrix<number>::matrix_norm_square(
 }
 
 
-template<>
+
+template <typename number>
 void
-TridiagonalMatrix<double>::compute_eigenvalues()
+TridiagonalMatrix<number>::compute_eigenvalues()
 {
 #ifdef DEAL_II_WITH_LAPACK
   Assert(state == matrix, ExcState(state));
   Assert(is_symmetric, ExcNotImplemented());
 
-  const int nn = n();
-  int info;
-  stev (&N, &nn, &*diagonal.begin(), &*right.begin(), 0, &one, 0, &info);
+  const types::blas_int nn = n();
+  types::blas_int info;
+  stev (&N, &nn, diagonal.data(), right.data(), nullptr, &one, nullptr, &info);
   Assert(info == 0, ExcInternalError());
 
-  state = eigenvalues;
+  state = LAPACKSupport::eigenvalues;
 #else
   Assert(false, ExcNeedsLAPACK());
 #endif
 }
 
 
-template<typename number>
+
+template <typename number>
 number
 TridiagonalMatrix<number>::eigenvalue(const size_type i) const
 {
-  Assert(state == eigenvalues, ExcState(state));
+  Assert(state == LAPACKSupport::eigenvalues, ExcState(state));
   Assert(i<n(), ExcIndexRange(i,0,n()));
   return diagonal[i];
 }
 
 
 /*
-template<typename number>
+template <typename number>
 TridiagonalMatrix<number>::
 {
 }
