@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 2008 - 2017 by the deal.II authors
+// Copyright (C) 2008 - 2018 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -21,13 +21,11 @@
 #  include <deal.II/lac/sparse_matrix.h>
 #  include <deal.II/lac/trilinos_sparse_matrix.h>
 
-DEAL_II_DISABLE_EXTRA_DIAGNOSTICS
 #  include <Ifpack.h>
 #  include <Ifpack_Chebyshev.h>
 #  include <Teuchos_ParameterList.hpp>
 #  include <Teuchos_RCP.hpp>
 #  include <Epetra_MultiVector.h>
-DEAL_II_ENABLE_EXTRA_DIAGNOSTICS
 
 DEAL_II_NAMESPACE_OPEN
 
@@ -55,11 +53,6 @@ namespace TrilinosWrappers
 
 
 
-  PreconditionBase::~PreconditionBase()
-  {}
-
-
-
   void PreconditionBase::clear ()
   {
     preconditioner.reset();
@@ -129,8 +122,8 @@ namespace TrilinosWrappers
 
     Ifpack_Preconditioner *ifpack = static_cast<Ifpack_Preconditioner *>
                                     (preconditioner.get());
-    Assert (ifpack != 0, ExcMessage ("Trilinos could not create this "
-                                     "preconditioner"));
+    Assert (ifpack != nullptr, ExcMessage ("Trilinos could not create this "
+                                           "preconditioner"));
 
     int ierr;
 
@@ -181,8 +174,8 @@ namespace TrilinosWrappers
 
     Ifpack_Preconditioner *ifpack = static_cast<Ifpack_Preconditioner *>
                                     (preconditioner.get());
-    Assert (ifpack != 0, ExcMessage ("Trilinos could not create this "
-                                     "preconditioner"));
+    Assert (ifpack != nullptr, ExcMessage ("Trilinos could not create this "
+                                           "preconditioner"));
 
     int ierr;
 
@@ -234,8 +227,8 @@ namespace TrilinosWrappers
 
     Ifpack_Preconditioner *ifpack = static_cast<Ifpack_Preconditioner *>
                                     (preconditioner.get());
-    Assert (ifpack != 0, ExcMessage ("Trilinos could not create this "
-                                     "preconditioner"));
+    Assert (ifpack != nullptr, ExcMessage ("Trilinos could not create this "
+                                           "preconditioner"));
 
     int ierr;
 
@@ -283,15 +276,20 @@ namespace TrilinosWrappers
   {
     // release memory before reallocation
     preconditioner.reset ();
+
+    // Block relaxation setup fails if we have no locally owned rows. As a work-around
+    // we just pretend to use point relaxation on those processors:
     preconditioner.reset (Ifpack().Create
-                          ("block relaxation",
-                           const_cast<Epetra_CrsMatrix *>(&matrix.trilinos_matrix()),
-                           0));
+                          (
+                            (matrix.trilinos_matrix().NumMyRows()==0) ?
+                            "point relaxation" : "block relaxation",
+                            const_cast<Epetra_CrsMatrix *>(&matrix.trilinos_matrix()),
+                            0));
 
     Ifpack_Preconditioner *ifpack = static_cast<Ifpack_Preconditioner *>
                                     (preconditioner.get());
-    Assert (ifpack != 0, ExcMessage ("Trilinos could not create this "
-                                     "preconditioner"));
+    Assert (ifpack != nullptr, ExcMessage ("Trilinos could not create this "
+                                           "preconditioner"));
 
     int ierr;
 
@@ -343,15 +341,20 @@ namespace TrilinosWrappers
                                      const AdditionalData &additional_data)
   {
     preconditioner.reset ();
+
+    // Block relaxation setup fails if we have no locally owned rows. As a work-around
+    // we just pretend to use point relaxation on those processors:
     preconditioner.reset (Ifpack().Create
-                          ("block relaxation",
-                           const_cast<Epetra_CrsMatrix *>(&matrix.trilinos_matrix()),
-                           additional_data.overlap));
+                          (
+                            (matrix.trilinos_matrix().NumMyRows()==0) ?
+                            "point relaxation" : "block relaxation",
+                            const_cast<Epetra_CrsMatrix *>(&matrix.trilinos_matrix()),
+                            additional_data.overlap));
 
     Ifpack_Preconditioner *ifpack = static_cast<Ifpack_Preconditioner *>
                                     (preconditioner.get());
-    Assert (ifpack != 0, ExcMessage ("Trilinos could not create this "
-                                     "preconditioner"));
+    Assert (ifpack != nullptr, ExcMessage ("Trilinos could not create this "
+                                           "preconditioner"));
 
     int ierr;
 
@@ -404,15 +407,20 @@ namespace TrilinosWrappers
                                     const AdditionalData &additional_data)
   {
     preconditioner.reset ();
+
+    // Block relaxation setup fails if we have no locally owned rows. As a work-around
+    // we just pretend to use point relaxation on those processors:
     preconditioner.reset (Ifpack().Create
-                          ("block relaxation",
-                           const_cast<Epetra_CrsMatrix *>(&matrix.trilinos_matrix()),
-                           additional_data.overlap));
+                          (
+                            (matrix.trilinos_matrix().NumMyRows()==0) ?
+                            "point relaxation" : "block relaxation",
+                            const_cast<Epetra_CrsMatrix *>(&matrix.trilinos_matrix()),
+                            additional_data.overlap));
 
     Ifpack_Preconditioner *ifpack = static_cast<Ifpack_Preconditioner *>
                                     (preconditioner.get());
-    Assert (ifpack != 0, ExcMessage ("Trilinos could not create this "
-                                     "preconditioner"));
+    Assert (ifpack != nullptr, ExcMessage ("Trilinos could not create this "
+                                           "preconditioner"));
 
     int ierr;
 
@@ -468,8 +476,8 @@ namespace TrilinosWrappers
 
     Ifpack_Preconditioner *ifpack = static_cast<Ifpack_Preconditioner *>
                                     (preconditioner.get());
-    Assert (ifpack != 0, ExcMessage ("Trilinos could not create this "
-                                     "preconditioner"));
+    Assert (ifpack != nullptr, ExcMessage ("Trilinos could not create this "
+                                           "preconditioner"));
 
     int ierr;
 
@@ -519,8 +527,8 @@ namespace TrilinosWrappers
 
     Ifpack_Preconditioner *ifpack = static_cast<Ifpack_Preconditioner *>
                                     (preconditioner.get());
-    Assert (ifpack != 0, ExcMessage ("Trilinos could not create this "
-                                     "preconditioner"));
+    Assert (ifpack != nullptr, ExcMessage ("Trilinos could not create this "
+                                           "preconditioner"));
 
     int ierr;
 
@@ -572,8 +580,8 @@ namespace TrilinosWrappers
 
     Ifpack_Preconditioner *ifpack = static_cast<Ifpack_Preconditioner *>
                                     (preconditioner.get());
-    Assert (ifpack != 0, ExcMessage ("Trilinos could not create this "
-                                     "preconditioner"));
+    Assert (ifpack != nullptr, ExcMessage ("Trilinos could not create this "
+                                           "preconditioner"));
 
     int ierr;
 
@@ -618,8 +626,8 @@ namespace TrilinosWrappers
 
     Ifpack_Preconditioner *ifpack = static_cast<Ifpack_Preconditioner *>
                                     (preconditioner.get());
-    Assert (ifpack != 0, ExcMessage ("Trilinos could not create this "
-                                     "preconditioner"));
+    Assert (ifpack != nullptr, ExcMessage ("Trilinos could not create this "
+                                           "preconditioner"));
 
     int ierr;
 
@@ -663,12 +671,12 @@ namespace TrilinosWrappers
                                      const AdditionalData &additional_data)
   {
     preconditioner.reset ();
-    preconditioner.reset (new Ifpack_Chebyshev (&matrix.trilinos_matrix()));
+    preconditioner = std::make_shared<Ifpack_Chebyshev> (&matrix.trilinos_matrix());
 
     Ifpack_Chebyshev *ifpack = static_cast<Ifpack_Chebyshev *>
                                (preconditioner.get());
-    Assert (ifpack != 0, ExcMessage ("Trilinos could not create this "
-                                     "preconditioner"));
+    Assert (ifpack != nullptr, ExcMessage ("Trilinos could not create this "
+                                           "preconditioner"));
 
     int ierr;
 
@@ -712,7 +720,7 @@ namespace TrilinosWrappers
     // What follows just configures a dummy preconditioner that
     // sets up the domain and range maps, as well as the communicator.
     // It is never used as the vmult, Tvmult operations are
-    // given a custom defintion.
+    // given a custom definition.
     // Note: This is only required in order to wrap this
     // preconditioner in a LinearOperator without an exemplar
     // matrix.
@@ -727,8 +735,8 @@ namespace TrilinosWrappers
 
     Ifpack_Preconditioner *ifpack = static_cast<Ifpack_Preconditioner *>
                                     (preconditioner.get());
-    Assert (ifpack != 0, ExcMessage ("Trilinos could not create this "
-                                     "preconditioner"));
+    Assert (ifpack != nullptr, ExcMessage ("Trilinos could not create this "
+                                           "preconditioner"));
 
     int ierr;
 
@@ -749,15 +757,15 @@ namespace TrilinosWrappers
   }
 
   void
-  PreconditionIdentity::vmult(VectorBase       &dst,
-                              const VectorBase &src) const
+  PreconditionIdentity::vmult(MPI::Vector       &dst,
+                              const MPI::Vector &src) const
   {
     dst = src;
   }
 
   void
-  PreconditionIdentity::Tvmult(VectorBase       &dst,
-                               const VectorBase &src) const
+  PreconditionIdentity::Tvmult(MPI::Vector       &dst,
+                               const MPI::Vector &src) const
   {
     dst = src;
   }
