@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 2008 - 2017 by the deal.II authors
+// Copyright (C) 2008 - 2018 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -13,6 +13,7 @@
 //
 // ---------------------------------------------------------------------
 
+#include <deal.II/lac/trilinos_index_access.h>
 #include <deal.II/lac/trilinos_precondition.h>
 
 #ifdef DEAL_II_WITH_TRILINOS
@@ -20,8 +21,8 @@
 #  include <deal.II/lac/vector.h>
 #  include <deal.II/lac/sparse_matrix.h>
 #  include <deal.II/lac/trilinos_sparse_matrix.h>
+#  include <deal.II/lac/trilinos_index_access.h>
 
-DEAL_II_DISABLE_EXTRA_DIAGNOSTICS
 #  include <Ifpack.h>
 #  include <Ifpack_Chebyshev.h>
 #  include <Teuchos_ParameterList.hpp>
@@ -29,49 +30,11 @@ DEAL_II_DISABLE_EXTRA_DIAGNOSTICS
 #  include <Epetra_MultiVector.h>
 #  include <ml_include.h>
 #  include <ml_MultiLevelPreconditioner.h>
-DEAL_II_ENABLE_EXTRA_DIAGNOSTICS
 
 DEAL_II_NAMESPACE_OPEN
 
 namespace TrilinosWrappers
 {
-  namespace
-  {
-#ifndef DEAL_II_WITH_64BIT_INDICES
-    int n_global_rows (const Epetra_RowMatrix &matrix)
-    {
-      return matrix.NumGlobalRows();
-    }
-
-    int global_length (const Epetra_MultiVector &vector)
-    {
-      return vector.GlobalLength();
-    }
-
-    int gid(const Epetra_Map &map, unsigned int i)
-    {
-      return map.GID(i);
-    }
-#else
-    long long int n_global_rows (const Epetra_RowMatrix &matrix)
-    {
-      return matrix.NumGlobalRows64();
-    }
-
-    long long int global_length (const Epetra_MultiVector &vector)
-    {
-      return vector.GlobalLength64();
-    }
-
-    long long int gid(const Epetra_Map &map, dealii::types::global_dof_index i)
-    {
-      return map.GID64(i);
-    }
-#endif
-  }
-
-
-
   /* -------------------------- PreconditionAMG -------------------------- */
 
   PreconditionAMG::AdditionalData::
@@ -188,12 +151,12 @@ namespace TrilinosWrappers
 
     if (constant_modes_dimension > 0)
       {
-        const size_type global_size = n_global_rows(matrix);
+        const size_type global_size = TrilinosWrappers::n_global_rows(matrix);
         (void)global_length; // work around compiler warning about unused function in release mode
         Assert (global_size ==
-                static_cast<size_type>(global_length(distributed_constant_modes)),
+                static_cast<size_type>(TrilinosWrappers::global_length(distributed_constant_modes)),
                 ExcDimensionMismatch(global_size,
-                                     global_length(distributed_constant_modes)));
+                                     TrilinosWrappers::global_length(distributed_constant_modes)));
         const bool constant_modes_are_global
           = additional_data.constant_modes[0].size() == global_size;
         const size_type my_size = domain_map.NumMyElements();
@@ -209,7 +172,7 @@ namespace TrilinosWrappers
             for (size_type row=0; row<my_size; ++row)
               {
                 const TrilinosWrappers::types::int_type mode_index =
-                  constant_modes_are_global ? gid(domain_map,row) : row;
+                  constant_modes_are_global ? TrilinosWrappers::global_index(domain_map,row) : row;
                 distributed_constant_modes[d][row] =
                   additional_data.constant_modes[d][mode_index];
               }
@@ -226,7 +189,7 @@ namespace TrilinosWrappers
         // the current processor. Therefore, pass a dummy in that case
         else
           parameter_list.set("null space: vectors",
-                             &dummy[0]);
+                             dummy.data());
       }
 
     initialize (matrix, parameter_list);
@@ -235,7 +198,7 @@ namespace TrilinosWrappers
       {
         ML_Epetra::MultiLevelPreconditioner *multilevel_operator =
           dynamic_cast<ML_Epetra::MultiLevelPreconditioner *> (preconditioner.get());
-        Assert (multilevel_operator != 0,
+        Assert (multilevel_operator != nullptr,
                 ExcMessage ("Preconditioner setup failed."));
         multilevel_operator->PrintUnused(0);
       }
@@ -257,8 +220,8 @@ namespace TrilinosWrappers
                                const Teuchos::ParameterList &ml_parameters)
   {
     preconditioner.reset ();
-    preconditioner.reset (new ML_Epetra::MultiLevelPreconditioner
-                          (matrix, ml_parameters));
+    preconditioner = std::make_shared<ML_Epetra::MultiLevelPreconditioner>
+                     (matrix, ml_parameters);
   }
 
 
@@ -278,11 +241,12 @@ namespace TrilinosWrappers
     // equidistributed map; avoid
     // storing the nonzero
     // elements.
-    vector_distributor.reset (new Epetra_Map(static_cast<TrilinosWrappers::types::int_type>(n_rows),
-                                             0, communicator));
+    vector_distributor = std::make_shared<Epetra_Map>
+                         (static_cast<TrilinosWrappers::types::int_type>(n_rows),
+                          0, communicator);
 
-    if (trilinos_matrix.get() == 0)
-      trilinos_matrix.reset (new SparseMatrix());
+    if (trilinos_matrix.get() == nullptr)
+      trilinos_matrix = std::make_shared<SparseMatrix>();
 
     trilinos_matrix->reinit (*vector_distributor, *vector_distributor,
                              deal_ii_sparse_matrix, drop_tolerance, true,
@@ -317,7 +281,7 @@ namespace TrilinosWrappers
 
     // todo: find a way to read out ML's data
     // sizes
-    if (trilinos_matrix.get() != 0)
+    if (trilinos_matrix.get() != nullptr)
       memory += trilinos_matrix->memory_consumption();
     return memory;
   }
