@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 2008 - 2017 by the deal.II authors
+// Copyright (C) 2008 - 2018 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -13,6 +13,7 @@
 //
 // ---------------------------------------------------------------------
 
+#include <deal.II/lac/trilinos_index_access.h>
 #include <deal.II/lac/trilinos_precondition.h>
 
 #ifdef DEAL_II_WITH_TRILINOS
@@ -20,9 +21,9 @@
 
 #  include <deal.II/lac/vector.h>
 #  include <deal.II/lac/sparse_matrix.h>
+#  include <deal.II/lac/trilinos_index_access.h>
 #  include <deal.II/lac/trilinos_sparse_matrix.h>
 
-DEAL_II_DISABLE_EXTRA_DIAGNOSTICS
 #  include <Teuchos_ParameterList.hpp>
 #  include <Teuchos_RCP.hpp>
 #  include <Epetra_MultiVector.h>
@@ -32,49 +33,11 @@ DEAL_II_DISABLE_EXTRA_DIAGNOSTICS
 #  include <MueLu.hpp>
 #  include <MueLu_EpetraOperator.hpp>
 #  include <MueLu_MLParameterListInterpreter.hpp>
-DEAL_II_ENABLE_EXTRA_DIAGNOSTICS
 
 DEAL_II_NAMESPACE_OPEN
 
 namespace TrilinosWrappers
 {
-  namespace
-  {
-#ifndef DEAL_II_WITH_64BIT_INDICES
-    int n_global_rows (const Epetra_RowMatrix &matrix)
-    {
-      return matrix.NumGlobalRows();
-    }
-
-    int global_length (const Epetra_MultiVector &vector)
-    {
-      return vector.GlobalLength();
-    }
-
-    int gid(const Epetra_Map &map, unsigned int i)
-    {
-      return map.GID(i);
-    }
-#else
-    long long int n_global_rows (const Epetra_RowMatrix &matrix)
-    {
-      return matrix.NumGlobalRows64();
-    }
-
-    long long int global_length (const Epetra_MultiVector &vector)
-    {
-      return vector.GlobalLength64();
-    }
-
-    long long int gid(const Epetra_Map &map, dealii::types::global_dof_index i)
-    {
-      return map.GID64(i);
-    }
-#endif
-  }
-
-
-
   PreconditionAMGMueLu::AdditionalData::
   AdditionalData (const bool                             elliptic,
                   const unsigned int                     n_cycles,
@@ -181,7 +144,7 @@ namespace TrilinosWrappers
 
     if (constant_modes_dimension > 0)
       {
-        const size_type n_rows = n_global_rows(matrix);
+        const size_type n_rows = TrilinosWrappers::n_global_rows(matrix);
         const bool constant_modes_are_global =
           additional_data.constant_modes[0].size() == n_rows;
         const size_type n_relevant_rows =
@@ -191,9 +154,9 @@ namespace TrilinosWrappers
           Assert (n_relevant_rows == my_size,
                   ExcDimensionMismatch(n_relevant_rows, my_size));
         Assert (n_rows ==
-                static_cast<size_type>(global_length(distributed_constant_modes)),
+                static_cast<size_type>(TrilinosWrappers::global_length(distributed_constant_modes)),
                 ExcDimensionMismatch(n_rows,
-                                     global_length(distributed_constant_modes)));
+                                     TrilinosWrappers::global_length(distributed_constant_modes)));
 
         (void)n_relevant_rows;
         (void)global_length;
@@ -204,7 +167,7 @@ namespace TrilinosWrappers
           for (size_type row=0; row<my_size; ++row)
             {
               TrilinosWrappers::types::int_type global_row_id =
-                constant_modes_are_global ? gid(domain_map,row) : row;
+                constant_modes_are_global ? TrilinosWrappers::global_index(domain_map,row) : row;
               distributed_constant_modes[d][row] =
                 additional_data.constant_modes[d][global_row_id];
             }
@@ -219,7 +182,7 @@ namespace TrilinosWrappers
         // the current processor. Therefore, pass a dummy in that case
         else
           parameter_list.set("null space: vectors",
-                             &dummy[0]);
+                             dummy.data());
       }
 
     initialize (matrix, parameter_list);
@@ -267,7 +230,7 @@ namespace TrilinosWrappers
 
     // MueLu::EpetraOperator is just a wrapper around a "standard"
     // Epetra_Operator.
-    preconditioner.reset(new MueLu::EpetraOperator(hierarchy));
+    preconditioner = std::make_shared<MueLu::EpetraOperator>(hierarchy);
   }
 
 
@@ -287,11 +250,12 @@ namespace TrilinosWrappers
     // equidistributed map; avoid
     // storing the nonzero
     // elements.
-    vector_distributor.reset (new Epetra_Map(static_cast<TrilinosWrappers::types::int_type>(n_rows),
-                                             0, communicator));
+    vector_distributor = std::make_shared<Epetra_Map>
+                         (static_cast<TrilinosWrappers::types::int_type>(n_rows),
+                          0, communicator);
 
-    if (trilinos_matrix.get() == 0)
-      trilinos_matrix.reset (new SparseMatrix());
+    if (trilinos_matrix.get() == nullptr)
+      trilinos_matrix = std::make_shared<SparseMatrix>();
 
     trilinos_matrix->reinit (*vector_distributor, *vector_distributor,
                              deal_ii_sparse_matrix, drop_tolerance, true,
@@ -317,7 +281,7 @@ namespace TrilinosWrappers
 
     // todo: find a way to read out ML's data
     // sizes
-    if (trilinos_matrix.get() != 0)
+    if (trilinos_matrix.get() != nullptr)
       memory += trilinos_matrix->memory_consumption();
     return memory;
   }
