@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 2008 - 2017 by the deal.II authors
+// Copyright (C) 2008 - 2018 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -18,17 +18,16 @@
 #ifdef DEAL_II_WITH_TRILINOS
 
 #  include <deal.II/base/conditional_ostream.h>
+#  include <deal.II/base/std_cxx14/memory.h>
 #  include <deal.II/lac/trilinos_sparse_matrix.h>
-#  include <deal.II/lac/trilinos_vector_base.h>
+#  include <deal.II/lac/trilinos_vector.h>
 #  include <deal.II/lac/trilinos_precondition.h>
 
-DEAL_II_DISABLE_EXTRA_DIAGNOSTICS
 #  include <AztecOO_StatusTest.h>
 #  include <AztecOO_StatusTestMaxIters.h>
 #  include <AztecOO_StatusTestResNorm.h>
 #  include <AztecOO_StatusTestCombo.h>
 #  include <AztecOO_StatusType.h>
-DEAL_II_ENABLE_EXTRA_DIAGNOSTICS
 
 #  include <cmath>
 #  include <limits>
@@ -47,24 +46,23 @@ namespace TrilinosWrappers
 
 
 
-  SolverBase::SolverBase (SolverControl  &cn)
+  SolverBase::SolverBase (SolverControl        &cn,
+                          const AdditionalData &data)
     :
     solver_name    (gmres),
-    solver_control (cn)
+    solver_control (cn),
+    additional_data (data)
   {}
 
 
 
-  SolverBase::SolverBase (const enum SolverBase::SolverName  solver_name,
-                          SolverControl                     &cn)
+  SolverBase::SolverBase (const SolverBase::SolverName  solver_name,
+                          SolverControl                &cn,
+                          const AdditionalData         &data)
     :
     solver_name    (solver_name),
-    solver_control (cn)
-  {}
-
-
-
-  SolverBase::~SolverBase ()
+    solver_control (cn),
+    additional_data (data)
   {}
 
 
@@ -79,18 +77,16 @@ namespace TrilinosWrappers
 
   void
   SolverBase::solve (const SparseMatrix     &A,
-                     VectorBase             &x,
-                     const VectorBase       &b,
+                     MPI::Vector            &x,
+                     const MPI::Vector      &b,
                      const PreconditionBase &preconditioner)
   {
-    linear_problem.reset();
-
     // We need an Epetra_LinearProblem object to let the AztecOO solver know
     // about the matrix and vectors.
-    linear_problem.reset
-    (new Epetra_LinearProblem(const_cast<Epetra_CrsMatrix *>(&A.trilinos_matrix()),
-                              &x.trilinos_vector(),
-                              const_cast<Epetra_MultiVector *>(&b.trilinos_vector())));
+    linear_problem = std_cxx14::make_unique<Epetra_LinearProblem>
+                     (const_cast<Epetra_CrsMatrix *>(&A.trilinos_matrix()),
+                      &x.trilinos_vector(),
+                      const_cast<Epetra_MultiVector *>(&b.trilinos_vector()));
 
     do_solve(preconditioner);
   }
@@ -101,18 +97,16 @@ namespace TrilinosWrappers
   //       can be used by the inverse_operator of LinearOperator
   void
   SolverBase::solve (const Epetra_Operator  &A,
-                     VectorBase             &x,
-                     const VectorBase       &b,
+                     MPI::Vector            &x,
+                     const MPI::Vector      &b,
                      const PreconditionBase &preconditioner)
   {
-    linear_problem.reset();
-
     // We need an Epetra_LinearProblem object to let the AztecOO solver know
     // about the matrix and vectors.
-    linear_problem.reset
-    (new Epetra_LinearProblem(const_cast<Epetra_Operator *>(&A),
-                              &x.trilinos_vector(),
-                              const_cast<Epetra_MultiVector *>(&b.trilinos_vector())));
+    linear_problem = std_cxx14::make_unique<Epetra_LinearProblem>
+                     (const_cast<Epetra_Operator *>(&A),
+                      &x.trilinos_vector(),
+                      const_cast<Epetra_MultiVector *>(&b.trilinos_vector()));
 
     do_solve(preconditioner);
   }
@@ -123,18 +117,16 @@ namespace TrilinosWrappers
   //       can be used by the inverse_operator of LinearOperator
   void
   SolverBase::solve (const Epetra_Operator  &A,
-                     VectorBase             &x,
-                     const VectorBase       &b,
+                     MPI::Vector            &x,
+                     const MPI::Vector      &b,
                      const Epetra_Operator  &preconditioner)
   {
-    linear_problem.reset();
-
     // We need an Epetra_LinearProblem object to let the AztecOO solver know
     // about the matrix and vectors.
-    linear_problem.reset
-    (new Epetra_LinearProblem(const_cast<Epetra_Operator *>(&A),
-                              &x.trilinos_vector(),
-                              const_cast<Epetra_MultiVector *>(&b.trilinos_vector())));
+    linear_problem = std_cxx14::make_unique<Epetra_LinearProblem>
+                     (const_cast<Epetra_Operator *>(&A),
+                      &x.trilinos_vector(),
+                      const_cast<Epetra_MultiVector *>(&b.trilinos_vector()));
 
     do_solve(preconditioner);
   }
@@ -149,14 +141,12 @@ namespace TrilinosWrappers
                      const Epetra_MultiVector &b,
                      const PreconditionBase   &preconditioner)
   {
-    linear_problem.reset();
-
     // We need an Epetra_LinearProblem object to let the AztecOO solver know
     // about the matrix and vectors.
-    linear_problem.reset
-    (new Epetra_LinearProblem(const_cast<Epetra_Operator *>(&A),
-                              &x,
-                              const_cast<Epetra_MultiVector *>(&b)));
+    linear_problem = std_cxx14::make_unique<Epetra_LinearProblem>
+                     (const_cast<Epetra_Operator *>(&A),
+                      &x,
+                      const_cast<Epetra_MultiVector *>(&b));
 
     do_solve(preconditioner);
   }
@@ -171,14 +161,12 @@ namespace TrilinosWrappers
                      const Epetra_MultiVector &b,
                      const Epetra_Operator    &preconditioner)
   {
-    linear_problem.reset();
-
     // We need an Epetra_LinearProblem object to let the AztecOO solver know
     // about the matrix and vectors.
-    linear_problem.reset
-    (new Epetra_LinearProblem(const_cast<Epetra_Operator *>(&A),
-                              &x,
-                              const_cast<Epetra_MultiVector *>(&b)));
+    linear_problem = std_cxx14::make_unique<Epetra_LinearProblem>
+                     (const_cast<Epetra_Operator *>(&A),
+                      &x,
+                      const_cast<Epetra_MultiVector *>(&b));
 
     do_solve(preconditioner);
   }
@@ -191,8 +179,6 @@ namespace TrilinosWrappers
                      const dealii::Vector<double> &b,
                      const PreconditionBase       &preconditioner)
   {
-    linear_problem.reset();
-
     // In case we call the solver with deal.II vectors, we create views of the
     // vectors in Epetra format.
     Assert (x.size() == A.n(),
@@ -209,9 +195,9 @@ namespace TrilinosWrappers
 
     // We need an Epetra_LinearProblem object to let the AztecOO solver know
     // about the matrix and vectors.
-    linear_problem.reset (new Epetra_LinearProblem
-                          (const_cast<Epetra_CrsMatrix *>(&A.trilinos_matrix()),
-                           &ep_x, &ep_b));
+    linear_problem = std_cxx14::make_unique<Epetra_LinearProblem>
+                     (const_cast<Epetra_CrsMatrix *>(&A.trilinos_matrix()),
+                      &ep_x, &ep_b);
 
     do_solve(preconditioner);
   }
@@ -224,14 +210,12 @@ namespace TrilinosWrappers
                      const dealii::Vector<double> &b,
                      const PreconditionBase       &preconditioner)
   {
-    linear_problem.reset();
-
     Epetra_Vector ep_x (View, A.OperatorDomainMap(), x.begin());
     Epetra_Vector ep_b (View, A.OperatorRangeMap(), const_cast<double *>(b.begin()));
 
     // We need an Epetra_LinearProblem object to let the AztecOO solver know
     // about the matrix and vectors.
-    linear_problem.reset (new Epetra_LinearProblem(&A,&ep_x, &ep_b));
+    linear_problem = std_cxx14::make_unique<Epetra_LinearProblem>(&A,&ep_x, &ep_b);
 
     do_solve(preconditioner);
   }
@@ -244,8 +228,6 @@ namespace TrilinosWrappers
                      const dealii::LinearAlgebra::distributed::Vector<double> &b,
                      const PreconditionBase                              &preconditioner)
   {
-    linear_problem.reset();
-
     // In case we call the solver with deal.II vectors, we create views of the
     // vectors in Epetra format.
     AssertDimension (static_cast<TrilinosWrappers::types::int_type>(x.local_size()),
@@ -258,9 +240,9 @@ namespace TrilinosWrappers
 
     // We need an Epetra_LinearProblem object to let the AztecOO solver know
     // about the matrix and vectors.
-    linear_problem.reset (new Epetra_LinearProblem
-                          (const_cast<Epetra_CrsMatrix *>(&A.trilinos_matrix()),
-                           &ep_x, &ep_b));
+    linear_problem = std_cxx14::make_unique<Epetra_LinearProblem>
+                     (const_cast<Epetra_CrsMatrix *>(&A.trilinos_matrix()),
+                      &ep_x, &ep_b);
 
     do_solve(preconditioner);
   }
@@ -273,8 +255,6 @@ namespace TrilinosWrappers
                      const dealii::LinearAlgebra::distributed::Vector<double> &b,
                      const PreconditionBase                              &preconditioner)
   {
-    linear_problem.reset();
-
     AssertDimension (static_cast<TrilinosWrappers::types::int_type>(x.local_size()),
                      A.OperatorDomainMap().NumMyElements());
     AssertDimension (static_cast<TrilinosWrappers::types::int_type>(b.local_size()),
@@ -285,7 +265,7 @@ namespace TrilinosWrappers
 
     // We need an Epetra_LinearProblem object to let the AztecOO solver know
     // about the matrix and vectors.
-    linear_problem.reset (new Epetra_LinearProblem(&A,&ep_x, &ep_b));
+    linear_problem = std_cxx14::make_unique<Epetra_LinearProblem>(&A,&ep_x, &ep_b);
 
     do_solve(preconditioner);
   }
@@ -309,12 +289,12 @@ namespace TrilinosWrappers
       class TrilinosReductionControl : public AztecOO_StatusTest
       {
       public:
-        TrilinosReductionControl (const double               &max_steps,
+        TrilinosReductionControl (const int                  &max_steps,
                                   const double               &tolerance,
                                   const double               &reduction,
                                   const Epetra_LinearProblem &linear_problem);
 
-        virtual ~TrilinosReductionControl() {}
+        virtual ~TrilinosReductionControl() = default;
 
         virtual bool
         ResidualVectorRequired () const
@@ -371,15 +351,15 @@ namespace TrilinosWrappers
       private:
         double initial_residual;
         double current_residual;
-        std_cxx11::shared_ptr<AztecOO_StatusTestCombo>    status_test_collection;
-        std_cxx11::shared_ptr<AztecOO_StatusTestMaxIters> status_test_max_steps;
-        std_cxx11::shared_ptr<AztecOO_StatusTestResNorm>  status_test_abs_tol;
-        std_cxx11::shared_ptr<AztecOO_StatusTestResNorm>  status_test_rel_tol;
+        std::unique_ptr<AztecOO_StatusTestCombo>    status_test_collection;
+        std::unique_ptr<AztecOO_StatusTestMaxIters> status_test_max_steps;
+        std::unique_ptr<AztecOO_StatusTestResNorm>  status_test_abs_tol;
+        std::unique_ptr<AztecOO_StatusTestResNorm>  status_test_rel_tol;
       };
 
 
       TrilinosReductionControl::TrilinosReductionControl(
-        const double               &max_steps,
+        const int                  &max_steps,
         const double               &tolerance,
         const double               &reduction,
         const Epetra_LinearProblem &linear_problem )
@@ -389,23 +369,23 @@ namespace TrilinosWrappers
       {
         // Consider linear problem converged if any of the collection
         // of criterion are met
-        status_test_collection.reset(
-          new AztecOO_StatusTestCombo (AztecOO_StatusTestCombo::OR) );
+        status_test_collection = std_cxx14::make_unique<AztecOO_StatusTestCombo>
+                                 (AztecOO_StatusTestCombo::OR);
 
         // Maximum number of iterations
-        status_test_max_steps.reset(
-          new AztecOO_StatusTestMaxIters(max_steps) );
+        Assert (max_steps >=0, ExcInternalError());
+        status_test_max_steps = std_cxx14::make_unique<AztecOO_StatusTestMaxIters>(max_steps);
         status_test_collection->AddStatusTest(*status_test_max_steps);
 
         Assert(linear_problem.GetRHS()->NumVectors() == 1,
                ExcMessage("RHS multivector holds more than one vector"));
 
         // Residual norm is below some absolute value
-        status_test_abs_tol.reset(
-          new AztecOO_StatusTestResNorm(*linear_problem.GetOperator(),
-                                        *(linear_problem.GetLHS()->operator()(0)),
-                                        *(linear_problem.GetRHS()->operator()(0)),
-                                        tolerance) );
+        status_test_abs_tol = std_cxx14::make_unique<AztecOO_StatusTestResNorm>
+                              (*linear_problem.GetOperator(),
+                               *(linear_problem.GetLHS()->operator()(0)),
+                               *(linear_problem.GetRHS()->operator()(0)),
+                               tolerance);
         status_test_abs_tol->DefineResForm(AztecOO_StatusTestResNorm::Explicit,
                                            AztecOO_StatusTestResNorm::TwoNorm);
         status_test_abs_tol->DefineScaleForm(AztecOO_StatusTestResNorm::None,
@@ -413,11 +393,11 @@ namespace TrilinosWrappers
         status_test_collection->AddStatusTest(*status_test_abs_tol);
 
         // Residual norm, scaled by some initial value, is below some threshold
-        status_test_rel_tol.reset(
-          new AztecOO_StatusTestResNorm(*linear_problem.GetOperator(),
-                                        *(linear_problem.GetLHS()->operator()(0)),
-                                        *(linear_problem.GetRHS()->operator()(0)),
-                                        reduction) );
+        status_test_rel_tol = std_cxx14::make_unique<AztecOO_StatusTestResNorm>
+                              (*linear_problem.GetOperator(),
+                               *(linear_problem.GetLHS()->operator()(0)),
+                               *(linear_problem.GetRHS()->operator()(0)),
+                               reduction);
         status_test_rel_tol->DefineResForm(AztecOO_StatusTestResNorm::Explicit,
                                            AztecOO_StatusTestResNorm::TwoNorm);
         status_test_rel_tol->DefineScaleForm(AztecOO_StatusTestResNorm::NormOfInitRes,
@@ -429,7 +409,7 @@ namespace TrilinosWrappers
   }
 
 
-  template<typename Preconditioner>
+  template <typename Preconditioner>
   void
   SolverBase::do_solve(const Preconditioner &preconditioner)
   {
@@ -487,11 +467,11 @@ namespace TrilinosWrappers
         if (const ReductionControl* const reduction_control
             = dynamic_cast<const ReductionControl *const>(&solver_control))
           {
-            status_test.reset(new internal::TrilinosReductionControl(
-                                reduction_control->max_steps(),
-                                reduction_control->tolerance(),
-                                reduction_control->reduction(),
-                                *linear_problem) );
+            status_test = std_cxx14::make_unique<internal::TrilinosReductionControl>
+                          (reduction_control->max_steps(),
+                           reduction_control->tolerance(),
+                           reduction_control->reduction(),
+                           *linear_problem);
             solver.SetStatusTest(status_test.get());
           }
       }
@@ -508,15 +488,19 @@ namespace TrilinosWrappers
       case -1:
         AssertThrow (false, ExcMessage("AztecOO::Iterate error code -1: "
                                        "option not implemented"));
+        break;
       case -2:
         AssertThrow (false, ExcMessage("AztecOO::Iterate error code -2: "
                                        "numerical breakdown"));
+        break;
       case -3:
         AssertThrow (false, ExcMessage("AztecOO::Iterate error code -3: "
                                        "loss of precision"));
+        break;
       case -4:
         AssertThrow (false, ExcMessage("AztecOO::Iterate error code -4: "
                                        "GMRES Hessenberg ill-conditioned"));
+        break;
       default:
         AssertThrow (ierr >= 0, ExcTrilinosError(ierr));
       }
@@ -560,7 +544,7 @@ namespace TrilinosWrappers
 
 
 
-  template<>
+  template <>
   void
   SolverBase::set_preconditioner(AztecOO                &solver,
                                  const PreconditionBase &preconditioner)
@@ -578,7 +562,7 @@ namespace TrilinosWrappers
   }
 
 
-  template<>
+  template <>
   void
   SolverBase::set_preconditioner(AztecOO               &solver,
                                  const Epetra_Operator &preconditioner)
@@ -593,7 +577,7 @@ namespace TrilinosWrappers
   SolverCG::AdditionalData::
   AdditionalData (const bool output_solver_details)
     :
-    output_solver_details (output_solver_details)
+    SolverBase::AdditionalData (output_solver_details)
   {}
 
 
@@ -601,8 +585,8 @@ namespace TrilinosWrappers
   SolverCG::SolverCG (SolverControl        &cn,
                       const AdditionalData &data)
     :
-    SolverBase (cn),
-    additional_data (data.output_solver_details)
+    SolverBase (cn, data),
+    additional_data (data)
   {
     solver_name = cg;
   }
@@ -614,8 +598,8 @@ namespace TrilinosWrappers
   AdditionalData (const bool output_solver_details,
                   const unsigned int restart_parameter)
     :
-    output_solver_details (output_solver_details),
-    restart_parameter (restart_parameter)
+    SolverBase::AdditionalData (output_solver_details,
+                                restart_parameter)
   {}
 
 
@@ -623,9 +607,8 @@ namespace TrilinosWrappers
   SolverGMRES::SolverGMRES (SolverControl        &cn,
                             const AdditionalData &data)
     :
-    SolverBase (cn),
-    additional_data (data.output_solver_details,
-                     data.restart_parameter)
+    SolverBase (cn, data),
+    additional_data (data)
   {
     solver_name = gmres;
   }
@@ -636,7 +619,7 @@ namespace TrilinosWrappers
   SolverBicgstab::AdditionalData::
   AdditionalData (const bool output_solver_details)
     :
-    output_solver_details (output_solver_details)
+    SolverBase::AdditionalData (output_solver_details)
   {}
 
 
@@ -645,8 +628,8 @@ namespace TrilinosWrappers
   SolverBicgstab::SolverBicgstab (SolverControl        &cn,
                                   const AdditionalData &data)
     :
-    SolverBase (cn),
-    additional_data (data.output_solver_details)
+    SolverBase (cn, data),
+    additional_data (data)
   {
     solver_name = bicgstab;
   }
@@ -657,17 +640,16 @@ namespace TrilinosWrappers
   SolverCGS::AdditionalData::
   AdditionalData (const bool output_solver_details)
     :
-    output_solver_details (output_solver_details)
+    SolverBase::AdditionalData (output_solver_details)
   {}
 
 
 
-
   SolverCGS::SolverCGS (SolverControl        &cn,
                         const AdditionalData &data)
     :
-    SolverBase (cn),
-    additional_data (data.output_solver_details)
+    SolverBase (cn, data),
+    additional_data (data)
   {
     solver_name = cgs;
   }
@@ -678,7 +660,7 @@ namespace TrilinosWrappers
   SolverTFQMR::AdditionalData::
   AdditionalData (const bool output_solver_details)
     :
-    output_solver_details (output_solver_details)
+    SolverBase::AdditionalData (output_solver_details)
   {}
 
 
@@ -686,8 +668,8 @@ namespace TrilinosWrappers
   SolverTFQMR::SolverTFQMR (SolverControl        &cn,
                             const AdditionalData &data)
     :
-    SolverBase (cn),
-    additional_data (data.output_solver_details)
+    SolverBase (cn, data),
+    additional_data (data)
   {
     solver_name = tfqmr;
   }
@@ -716,11 +698,6 @@ namespace TrilinosWrappers
 
 
 
-  SolverDirect::~SolverDirect ()
-  {}
-
-
-
   SolverControl &
   SolverDirect::control() const
   {
@@ -733,7 +710,7 @@ namespace TrilinosWrappers
   {
     // We need an Epetra_LinearProblem object to let the Amesos solver know
     // about the matrix and vectors.
-    linear_problem.reset (new Epetra_LinearProblem ());
+    linear_problem = std_cxx14::make_unique<Epetra_LinearProblem >();
 
     // Assign the matrix operator to the Epetra_LinearProblem object
     linear_problem->SetOperator(const_cast<Epetra_CrsMatrix *>(&A.trilinos_matrix()));
@@ -746,8 +723,6 @@ namespace TrilinosWrappers
     ConditionalOStream  verbose_cout (std::cout,
                                       additional_data.output_solver_details);
 
-    solver.reset();
-
     // Next allocate the Amesos solver, this is done in two steps, first we
     // create a solver Factory and and generate with that the concrete Amesos
     // solver, if possible.
@@ -776,7 +751,7 @@ namespace TrilinosWrappers
   }
 
 
-  void SolverDirect::solve (VectorBase &x, const VectorBase &b)
+  void SolverDirect::solve (MPI::Vector &x, const MPI::Vector &b)
   {
     // Assign the empty LHS vector to the Epetra_LinearProblem object
     linear_problem->SetLHS(&x.trilinos_vector());
@@ -784,9 +759,6 @@ namespace TrilinosWrappers
     // Assign the RHS vector to the Epetra_LinearProblem object
     linear_problem->SetRHS(const_cast<Epetra_MultiVector *>(&b.trilinos_vector()));
 
-    // Fetch return value of Amesos Solver functions
-    int ierr;
-
     // First set whether we want to print the solver information to screen or
     // not.
     ConditionalOStream verbose_cout (std::cout,
@@ -794,7 +766,38 @@ namespace TrilinosWrappers
 
 
     verbose_cout << "Starting solve" << std::endl;
-    ierr = solver->Solve ();
+
+    // Fetch return value of Amesos Solver functions
+    int ierr = solver->Solve ();
+    AssertThrow (ierr == 0, ExcTrilinosError (ierr));
+
+    // Finally, force the SolverControl object to report convergence
+    solver_control.check (0, 0);
+  }
+
+
+
+  void SolverDirect::solve (dealii::LinearAlgebra::distributed::Vector<double>       &x,
+                            const dealii::LinearAlgebra::distributed::Vector<double> &b)
+  {
+    Epetra_Vector ep_x (View, linear_problem->GetOperator()->OperatorDomainMap(), x.begin());
+    Epetra_Vector ep_b (View, linear_problem->GetOperator()->OperatorRangeMap(), const_cast<double *>(b.begin()));
+
+    // Assign the empty LHS vector to the Epetra_LinearProblem object
+    linear_problem->SetLHS(&ep_x);
+
+    // Assign the RHS vector to the Epetra_LinearProblem object
+    linear_problem->SetRHS(&ep_b);
+
+    // First set whether we want to print the solver information to screen or
+    // not.
+    ConditionalOStream verbose_cout (std::cout,
+                                     additional_data.output_solver_details);
+
+    verbose_cout << "Starting solve" << std::endl;
+
+    // Fetch return value of Amesos Solver functions
+    int ierr = solver->Solve ();
     AssertThrow (ierr == 0, ExcTrilinosError (ierr));
 
     // Finally, force the SolverControl object to report convergence
@@ -814,8 +817,6 @@ namespace TrilinosWrappers
     ConditionalOStream verbose_cout (std::cout,
                                      additional_data.output_solver_details);
 
-    solver.reset ();
-
     // Next allocate the Amesos solver, this is done in two steps, first we
     // create a solver Factory and and generate with that the concrete Amesos
     // solver, if possible.
@@ -858,15 +859,15 @@ namespace TrilinosWrappers
 
   void
   SolverDirect::solve (const SparseMatrix     &A,
-                       VectorBase             &x,
-                       const VectorBase       &b)
+                       MPI::Vector            &x,
+                       const MPI::Vector      &b)
   {
     // We need an Epetra_LinearProblem object to let the Amesos solver know
     // about the matrix and vectors.
-    linear_problem.reset
-    (new Epetra_LinearProblem(const_cast<Epetra_CrsMatrix *>(&A.trilinos_matrix()),
-                              &x.trilinos_vector(),
-                              const_cast<Epetra_MultiVector *>(&b.trilinos_vector())));
+    linear_problem = std_cxx14::make_unique<Epetra_LinearProblem>
+                     (const_cast<Epetra_CrsMatrix *>(&A.trilinos_matrix()),
+                      &x.trilinos_vector(),
+                      const_cast<Epetra_MultiVector *>(&b.trilinos_vector()));
 
     do_solve();
   }
@@ -892,9 +893,9 @@ namespace TrilinosWrappers
 
     // We need an Epetra_LinearProblem object to let the Amesos solver know
     // about the matrix and vectors.
-    linear_problem.reset (new Epetra_LinearProblem
-                          (const_cast<Epetra_CrsMatrix *>(&A.trilinos_matrix()),
-                           &ep_x, &ep_b));
+    linear_problem = std_cxx14::make_unique<Epetra_LinearProblem>
+                     (const_cast<Epetra_CrsMatrix *>(&A.trilinos_matrix()),
+                      &ep_x, &ep_b);
 
     do_solve();
   }
@@ -915,13 +916,12 @@ namespace TrilinosWrappers
 
     // We need an Epetra_LinearProblem object to let the Amesos solver know
     // about the matrix and vectors.
-    linear_problem.reset (new Epetra_LinearProblem
-                          (const_cast<Epetra_CrsMatrix *>(&A.trilinos_matrix()),
-                           &ep_x, &ep_b));
+    linear_problem = std_cxx14::make_unique<Epetra_LinearProblem>
+                     (const_cast<Epetra_CrsMatrix *>(&A.trilinos_matrix()),
+                      &ep_x, &ep_b);
 
     do_solve();
   }
-
 }
 
 
