@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 2008 - 2017 by the deal.II authors
+// Copyright (C) 2008 - 2018 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -13,102 +13,24 @@
 //
 // ---------------------------------------------------------------------
 
+#include <deal.II/lac/trilinos_index_access.h>
 #include <deal.II/lac/trilinos_sparsity_pattern.h>
 
 #ifdef DEAL_II_WITH_TRILINOS
 
 #  include <deal.II/base/utilities.h>
 #  include <deal.II/base/mpi.h>
-#  include <deal.II/lac/sparsity_pattern.h>
-#  include <deal.II/lac/dynamic_sparsity_pattern.h>
 
-DEAL_II_DISABLE_EXTRA_DIAGNOSTICS
+#  include <deal.II/lac/dynamic_sparsity_pattern.h>
+#  include <deal.II/lac/sparsity_pattern.h>
+#  include <deal.II/lac/trilinos_index_access.h>
+
 #  include <Epetra_Export.h>
-DEAL_II_ENABLE_EXTRA_DIAGNOSTICS
 
 DEAL_II_NAMESPACE_OPEN
 
 namespace TrilinosWrappers
 {
-  namespace
-  {
-    // define a helper function that queries the size of an Epetra_Map object
-    // by calling either the 32- or 64-bit function necessary, and returns the
-    // result in the correct data type so that we can use it in calling other
-    // Epetra member functions that are overloaded by index type
-#ifndef DEAL_II_WITH_64BIT_INDICES
-    int n_global_elements (const Epetra_BlockMap &map)
-    {
-      return map.NumGlobalElements();
-    }
-
-    int min_my_gid(const Epetra_BlockMap &map)
-    {
-      return map.MinMyGID();
-    }
-
-    int max_my_gid(const Epetra_BlockMap &map)
-    {
-      return map.MaxMyGID();
-    }
-
-    int n_global_rows(const Epetra_CrsGraph &graph)
-    {
-      return graph.NumGlobalRows();
-    }
-
-    int n_global_cols(const Epetra_CrsGraph &graph)
-    {
-      return graph.NumGlobalCols();
-    }
-
-    int n_global_entries(const Epetra_CrsGraph &graph)
-    {
-      return graph.NumGlobalEntries();
-    }
-
-    int local_to_global_index(const Epetra_BlockMap &map, int i)
-    {
-      return map.GID(i);
-    }
-#else
-    long long int n_global_elements (const Epetra_BlockMap &map)
-    {
-      return map.NumGlobalElements64();
-    }
-
-    long long int min_my_gid(const Epetra_BlockMap &map)
-    {
-      return map.MinMyGID64();
-    }
-
-    long long int max_my_gid(const Epetra_BlockMap &map)
-    {
-      return map.MaxMyGID64();
-    }
-
-    long long int n_global_rows(const Epetra_CrsGraph &graph)
-    {
-      return graph.NumGlobalRows64();
-    }
-
-    long long int n_global_cols(const Epetra_CrsGraph &graph)
-    {
-      return graph.NumGlobalCols64();
-    }
-
-    long long int n_global_entries(const Epetra_CrsGraph &graph)
-    {
-      return graph.NumGlobalEntries64();
-    }
-
-    long long int local_to_global_index(const Epetra_BlockMap &map, int i)
-    {
-      return map.GID64(i);
-    }
-#endif
-  }
-
   namespace SparsityPatternIterators
   {
     void
@@ -126,8 +48,7 @@ namespace TrilinosWrappers
       // otherwise first flush Trilinos caches
       sparsity_pattern->compress ();
 
-      colnum_cache.reset (new std::vector<size_type> (sparsity_pattern->row_length(this->a_row)));
-
+      colnum_cache = std::make_shared<std::vector<size_type> >(sparsity_pattern->row_length(this->a_row));
 
       if (colnum_cache->size() > 0)
         {
@@ -159,13 +80,13 @@ namespace TrilinosWrappers
   // interface.
   SparsityPattern::SparsityPattern ()
   {
-    column_space_map.reset(new Epetra_Map (TrilinosWrappers::types::int_type(0),
-                                           TrilinosWrappers::types::int_type(0),
-                                           Utilities::Trilinos::comm_self()));
-    graph.reset (new Epetra_FECrsGraph(View,
-                                       *column_space_map,
-                                       *column_space_map,
-                                       0));
+    column_space_map = std_cxx14::make_unique<Epetra_Map> (TrilinosWrappers::types::int_type(0),
+                                                           TrilinosWrappers::types::int_type(0),
+                                                           Utilities::Trilinos::comm_self());
+    graph = std_cxx14::make_unique<Epetra_FECrsGraph>(View,
+                                                      *column_space_map,
+                                                      *column_space_map,
+                                                      0);
     graph->FillComplete();
   }
 
@@ -221,6 +142,16 @@ namespace TrilinosWrappers
   }
 
 
+
+  SparsityPattern::SparsityPattern (SparsityPattern &&other) noexcept
+:
+  Subscriptor(std::move(other)),
+              column_space_map(std::move(other.column_space_map)),
+              graph(std::move(other.graph)),
+              nonlocal_graph(std::move(other.nonlocal_graph))
+  {}
+
+
   // Copy function only works if the
   // sparsity pattern is empty.
   SparsityPattern::SparsityPattern (const SparsityPattern &input_sparsity)
@@ -297,11 +228,6 @@ namespace TrilinosWrappers
 
 
 
-  SparsityPattern::~SparsityPattern ()
-  {}
-
-
-
   void
   SparsityPattern::reinit (const size_type  m,
                            const size_type  n,
@@ -332,9 +258,9 @@ namespace TrilinosWrappers
     reinit_sp (const Epetra_Map                         &row_map,
                const Epetra_Map                         &col_map,
                const size_type                           n_entries_per_row,
-               std_cxx11::shared_ptr<Epetra_Map>        &column_space_map,
-               std_cxx11::shared_ptr<Epetra_FECrsGraph> &graph,
-               std_cxx11::shared_ptr<Epetra_CrsGraph>   &nonlocal_graph)
+               std::unique_ptr<Epetra_Map>        &column_space_map,
+               std::unique_ptr<Epetra_FECrsGraph> &graph,
+               std::unique_ptr<Epetra_CrsGraph>   &nonlocal_graph)
     {
       Assert(row_map.IsOneToOne(),
              ExcMessage("Row map must be 1-to-1, i.e., no overlap between "
@@ -345,7 +271,7 @@ namespace TrilinosWrappers
 
       nonlocal_graph.reset();
       graph.reset ();
-      column_space_map.reset (new Epetra_Map (col_map));
+      column_space_map = std_cxx14::make_unique<Epetra_Map >(col_map);
 
       // for more than one processor, need to specify only row map first and
       // let the matrix entries decide about the column map (which says which
@@ -356,19 +282,17 @@ namespace TrilinosWrappers
       // require building a non-local graph which gives us thread-safe
       // initialization.
       if (row_map.Comm().NumProc() > 1)
-        graph.reset (new Epetra_FECrsGraph(Copy, row_map,
-                                           n_entries_per_row, false
-                                           // TODO: Check which new Trilinos
-                                           // version supports this... Remember
-                                           // to change tests/trilinos/assemble_matrix_parallel_07
-                                           // too.
-                                           //#if DEAL_II_TRILINOS_VERSION_GTE(11,14,0)
-                                           //, true
-                                           //#endif
-                                          ));
+        graph = std_cxx14::make_unique<Epetra_FECrsGraph>
+                (Copy, row_map, n_entries_per_row, false
+                 // TODO: Check which new Trilinos version supports this...
+                 // Remember to change tests/trilinos/assemble_matrix_parallel_07, too.
+//#if DEAL_II_TRILINOS_VERSION_GTE(11,14,0)
+//                 , true
+//#endif
+                );
       else
-        graph.reset (new Epetra_FECrsGraph(Copy, row_map, col_map,
-                                           n_entries_per_row, false));
+        graph = std_cxx14::make_unique<Epetra_FECrsGraph>
+                (Copy, row_map, col_map, n_entries_per_row, false);
     }
 
 
@@ -377,9 +301,9 @@ namespace TrilinosWrappers
     reinit_sp (const Epetra_Map                         &row_map,
                const Epetra_Map                         &col_map,
                const std::vector<size_type>             &n_entries_per_row,
-               std_cxx11::shared_ptr<Epetra_Map>        &column_space_map,
-               std_cxx11::shared_ptr<Epetra_FECrsGraph> &graph,
-               std_cxx11::shared_ptr<Epetra_CrsGraph>   &nonlocal_graph)
+               std::unique_ptr<Epetra_Map>        &column_space_map,
+               std::unique_ptr<Epetra_FECrsGraph> &graph,
+               std::unique_ptr<Epetra_CrsGraph>   &nonlocal_graph)
     {
       Assert(row_map.IsOneToOne(),
              ExcMessage("Row map must be 1-to-1, i.e., no overlap between "
@@ -392,30 +316,26 @@ namespace TrilinosWrappers
       nonlocal_graph.reset();
       graph.reset ();
       AssertDimension (n_entries_per_row.size(),
-                       static_cast<size_type>(n_global_elements(row_map)));
+                       static_cast<size_type>(TrilinosWrappers::n_global_elements(row_map)));
 
-      column_space_map.reset (new Epetra_Map (col_map));
-      std::vector<int> local_entries_per_row(max_my_gid(row_map)-
-                                             min_my_gid(row_map));
+      column_space_map = std_cxx14::make_unique<Epetra_Map >(col_map);
+      std::vector<int> local_entries_per_row(TrilinosWrappers::max_my_gid(row_map)-
+                                             TrilinosWrappers::min_my_gid(row_map));
       for (unsigned int i=0; i<local_entries_per_row.size(); ++i)
-        local_entries_per_row[i] = n_entries_per_row[min_my_gid(row_map)+i];
+        local_entries_per_row[i] = n_entries_per_row[TrilinosWrappers::min_my_gid(row_map)+i];
 
       if (row_map.Comm().NumProc() > 1)
-        graph.reset(new Epetra_FECrsGraph(Copy, row_map,
-                                          &local_entries_per_row[0],
-                                          false
-                                          // TODO: Check which new Trilinos
-                                          // version supports this... Remember
-                                          // to change tests/trilinos/assemble_matrix_parallel_07
-                                          // too.
-                                          //#if DEAL_II_TRILINOS_VERSION_GTE(11,14,0)
-                                          //, true
-                                          //#endif
-                                         ));
+        graph = std_cxx14::make_unique<Epetra_FECrsGraph>
+                (Copy, row_map, local_entries_per_row.data(), false
+                 // TODO: Check which new Trilinos version supports this...
+                 // Remember to change tests/trilinos/assemble_matrix_parallel_07, too.
+//#if DEAL_II_TRILINOS_VERSION_GTE(11,14,0)
+//                , true
+//#endif
+                );
       else
-        graph.reset(new Epetra_FECrsGraph(Copy, row_map, col_map,
-                                          &local_entries_per_row[0],
-                                          false));
+        graph = std_cxx14::make_unique<Epetra_FECrsGraph>
+                (Copy, row_map, col_map, local_entries_per_row.data(), false);
     }
 
 
@@ -426,25 +346,25 @@ namespace TrilinosWrappers
                const Epetra_Map                         &col_map,
                const SparsityPatternType                &sp,
                const bool                                exchange_data,
-               std_cxx11::shared_ptr<Epetra_Map>        &column_space_map,
-               std_cxx11::shared_ptr<Epetra_FECrsGraph> &graph,
-               std_cxx11::shared_ptr<Epetra_CrsGraph>   &nonlocal_graph)
+               std::unique_ptr<Epetra_Map>        &column_space_map,
+               std::unique_ptr<Epetra_FECrsGraph> &graph,
+               std::unique_ptr<Epetra_CrsGraph>   &nonlocal_graph)
     {
       nonlocal_graph.reset ();
       graph.reset ();
 
       AssertDimension (sp.n_rows(),
-                       static_cast<size_type>(n_global_elements(row_map)));
+                       static_cast<size_type>(TrilinosWrappers::n_global_elements(row_map)));
       AssertDimension (sp.n_cols(),
-                       static_cast<size_type>(n_global_elements(col_map)));
+                       static_cast<size_type>(TrilinosWrappers::n_global_elements(col_map)));
 
-      column_space_map.reset (new Epetra_Map (col_map));
+      column_space_map = std_cxx14::make_unique<Epetra_Map >(col_map);
 
       Assert (row_map.LinearMap() == true,
               ExcMessage ("This function only works if the row map is contiguous."));
 
-      const size_type first_row = min_my_gid(row_map),
-                      last_row = max_my_gid(row_map)+1;
+      const size_type first_row = TrilinosWrappers::min_my_gid(row_map),
+                      last_row = TrilinosWrappers::max_my_gid(row_map)+1;
       std::vector<int> n_entries_per_row(last_row - first_row);
 
       // Trilinos wants the row length as an int this is hopefully never going
@@ -453,13 +373,13 @@ namespace TrilinosWrappers
         n_entries_per_row[row-first_row] = static_cast<int>(sp.row_length(row));
 
       if (row_map.Comm().NumProc() > 1)
-        graph.reset(new Epetra_FECrsGraph(Copy, row_map,
-                                          &n_entries_per_row[0],
-                                          false));
+        graph = std_cxx14::make_unique<Epetra_FECrsGraph>(Copy, row_map,
+                                                          n_entries_per_row.data(),
+                                                          false);
       else
-        graph.reset (new Epetra_FECrsGraph(Copy, row_map, col_map,
-                                           &n_entries_per_row[0],
-                                           false));
+        graph = std_cxx14::make_unique<Epetra_FECrsGraph>(Copy, row_map, col_map,
+                                                          n_entries_per_row.data(),
+                                                          false);
 
       AssertDimension (sp.n_rows(),
                        static_cast<size_type>(n_global_rows(*graph)));
@@ -488,7 +408,7 @@ namespace TrilinosWrappers
                 }
             }
             graph->Epetra_CrsGraph::InsertGlobalIndices (row, row_length,
-                                                         &row_indices[0]);
+                                                         row_indices.data());
           }
       else
         for (size_type row=0; row<sp.n_rows(); ++row)
@@ -511,7 +431,7 @@ namespace TrilinosWrappers
             }
             graph->InsertGlobalIndices (1,
                                         reinterpret_cast<TrilinosWrappers::types::int_type *>(&row),
-                                        row_length, &row_indices[0]);
+                                        row_length, row_indices.data());
           }
 
       int ierr =
@@ -653,7 +573,7 @@ namespace TrilinosWrappers
       {
         Epetra_Map nonlocal_map =
           nonlocal_partitioner.make_trilinos_map(communicator, true);
-        nonlocal_graph.reset(new Epetra_CrsGraph(Copy, nonlocal_map, 0));
+        nonlocal_graph = std_cxx14::make_unique<Epetra_CrsGraph>(Copy, nonlocal_map, 0);
       }
     else
       Assert(nonlocal_partitioner.n_elements() == 0, ExcInternalError());
@@ -661,7 +581,7 @@ namespace TrilinosWrappers
 
 
 
-  template<typename SparsityPatternType>
+  template <typename SparsityPatternType>
   void
   SparsityPattern::reinit (const IndexSet            &row_parallel_partitioning,
                            const IndexSet            &col_parallel_partitioning,
@@ -679,7 +599,7 @@ namespace TrilinosWrappers
 
 
 
-  template<typename SparsityPatternType>
+  template <typename SparsityPatternType>
   void
   SparsityPattern::reinit (const IndexSet            &parallel_partitioning,
                            const SparsityPatternType &nontrilinos_sparsity_pattern,
@@ -730,6 +650,20 @@ namespace TrilinosWrappers
 
 
 
+  void
+  SparsityPattern::copy_from (const SparsityPattern &sp)
+  {
+    column_space_map = std_cxx14::make_unique<Epetra_Map >(*sp.column_space_map);
+    graph = std_cxx14::make_unique<Epetra_FECrsGraph>(*sp.graph);
+
+    if (sp.nonlocal_graph.get()!=nullptr)
+      nonlocal_graph = std_cxx14::make_unique<Epetra_CrsGraph>(*sp.nonlocal_graph);
+    else
+      nonlocal_graph.reset();
+  }
+
+
+
   template <typename SparsityPatternType>
   void
   SparsityPattern::copy_from (const SparsityPatternType &sp)
@@ -751,11 +685,11 @@ namespace TrilinosWrappers
     // When we clear the matrix, reset
     // the pointer and generate an
     // empty sparsity pattern.
-    column_space_map.reset (new Epetra_Map (TrilinosWrappers::types::int_type(0),
-                                            TrilinosWrappers::types::int_type(0),
-                                            Utilities::Trilinos::comm_self()));
-    graph.reset (new Epetra_FECrsGraph(View, *column_space_map,
-                                       *column_space_map, 0));
+    column_space_map = std_cxx14::make_unique<Epetra_Map >(TrilinosWrappers::types::int_type(0),
+                                                           TrilinosWrappers::types::int_type(0),
+                                                           Utilities::Trilinos::comm_self());
+    graph = std_cxx14::make_unique<Epetra_FECrsGraph>(View, *column_space_map,
+                                                      *column_space_map, 0);
     graph->FillComplete();
 
     nonlocal_graph.reset();
@@ -767,8 +701,8 @@ namespace TrilinosWrappers
   SparsityPattern::compress ()
   {
     int ierr;
-    Assert (column_space_map.get() != 0, ExcInternalError());
-    if (nonlocal_graph.get() != 0)
+    Assert (column_space_map.get() != nullptr, ExcInternalError());
+    if (nonlocal_graph.get() != nullptr)
       {
         if (nonlocal_graph->IndicesAreGlobal() == false &&
             nonlocal_graph->RowMap().NumMyElements() > 0)
@@ -932,7 +866,7 @@ namespace TrilinosWrappers
     if (graph->Filled() == true)
       n_cols = n_global_cols(*graph);
     else
-      n_cols = n_global_elements(*column_space_map);
+      n_cols = TrilinosWrappers::n_global_elements(*column_space_map);
 
     return n_cols;
   }
@@ -953,8 +887,8 @@ namespace TrilinosWrappers
   SparsityPattern::local_range () const
   {
     size_type begin, end;
-    begin =  min_my_gid(graph->RowMap());
-    end = max_my_gid(graph->RowMap())+1;
+    begin =  TrilinosWrappers::min_my_gid(graph->RowMap());
+    end = TrilinosWrappers::max_my_gid(graph->RowMap())+1;
 
     return std::make_pair (begin, end);
   }
@@ -1051,7 +985,7 @@ namespace TrilinosWrappers
 
     const Epetra_MpiComm *mpi_comm
       = dynamic_cast<const Epetra_MpiComm *>(&graph->RangeMap().Comm());
-    Assert (mpi_comm != 0, ExcInternalError());
+    Assert (mpi_comm != nullptr, ExcInternalError());
     return mpi_comm->Comm();
 #else
 
@@ -1072,7 +1006,7 @@ namespace TrilinosWrappers
 
 
   // As of now, no particularly neat
-  // ouput is generated in case of
+  // output is generated in case of
   // multiple processors.
   void
   SparsityPattern::print (std::ostream &out,
@@ -1089,8 +1023,8 @@ namespace TrilinosWrappers
           {
             graph->ExtractMyRowView (i, num_entries, indices);
             for (int j=0; j<num_entries; ++j)
-              out << "(" << local_to_global_index(graph->RowMap(), i)
-                  << "," << local_to_global_index(graph->ColMap(), indices[j]) << ") "
+              out << "(" << TrilinosWrappers::global_index(graph->RowMap(), i)
+                  << "," << TrilinosWrappers::global_index(graph->ColMap(), indices[j]) << ") "
                   << std::endl;
           }
       }
@@ -1104,20 +1038,23 @@ namespace TrilinosWrappers
   SparsityPattern::print_gnuplot (std::ostream &out) const
   {
     Assert (graph->Filled() == true, ExcInternalError());
-    for (unsigned int row=0; row<local_size(); ++row)
+    for (dealii::types::global_dof_index row=0; row<local_size(); ++row)
       {
         int *indices;
         int num_entries;
         graph->ExtractMyRowView (row, num_entries, indices);
 
-        for (int j=0; j<num_entries; ++j)
+        Assert(num_entries >= 0, ExcInternalError());
+        // avoid sign comparison warning
+        const dealii::types::global_dof_index num_entries_ = num_entries;
+        for (dealii::types::global_dof_index j=0; j<num_entries_; ++j)
           // while matrix entries are usually
           // written (i,j), with i vertical and
           // j horizontal, gnuplot output is
           // x-y, that is we have to exchange
           // the order of output
-          out << static_cast<int>(local_to_global_index(graph->ColMap(), indices[j]))
-              << " " << -static_cast<int>(local_to_global_index(graph->RowMap(), row)) << std::endl;
+          out << static_cast<int>(TrilinosWrappers::global_index(graph->ColMap(), indices[j]))
+              << " " << -static_cast<int>(TrilinosWrappers::global_index(graph->RowMap(), row)) << std::endl;
       }
 
     AssertThrow (out, ExcIO());
