@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 2011 - 2016 by the deal.II authors
+// Copyright (C) 2011 - 2017 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -16,6 +16,7 @@
 
 #include <deal.II/matrix_free/matrix_free.templates.h>
 #include <deal.II/base/utilities.h>
+#include <deal.II/base/vectorization.h>
 #include <deal.II/base/conditional_ostream.h>
 
 #include <iostream>
@@ -26,5 +27,7 @@ DEAL_II_NAMESPACE_OPEN
 
 template struct internal::MatrixFreeFunctions::ShapeInfo<double>;
 template struct internal::MatrixFreeFunctions::ShapeInfo<float>;
+template struct internal::MatrixFreeFunctions::ShapeInfo<VectorizedArray<double>>;
+template struct internal::MatrixFreeFunctions::ShapeInfo<VectorizedArray<float>>;
 
 DEAL_II_NAMESPACE_CLOSE
