@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 1999 - 2016 by the deal.II authors
+// Copyright (C) 1999 - 2017 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -18,10 +18,9 @@
 #include <deal.II/lac/la_vector.h>
 #include <deal.II/lac/la_parallel_vector.h>
 #include <deal.II/lac/la_parallel_block_vector.h>
-#include <deal.II/lac/petsc_vector.h>
-#include <deal.II/lac/petsc_block_vector.h>
+#include <deal.II/lac/petsc_parallel_block_vector.h>
 #include <deal.II/lac/trilinos_vector.h>
-#include <deal.II/lac/trilinos_block_vector.h>
+#include <deal.II/lac/trilinos_parallel_block_vector.h>
 #include <deal.II/multigrid/mg_base.h>
 
 
@@ -29,23 +28,14 @@ DEAL_II_NAMESPACE_OPEN
 
 
 template <typename VectorType>
-MGTransferBase<VectorType>::~MGTransferBase()
-{}
-
-
-template <typename VectorType>
-MGMatrixBase<VectorType>::~MGMatrixBase()
-{}
-
-
-template <typename VectorType>
-MGSmootherBase<VectorType>::~MGSmootherBase()
-{}
-
-
-template <typename VectorType>
-MGCoarseGridBase<VectorType>::~MGCoarseGridBase()
-{}
+void
+MGSmootherBase<VectorType>::apply (const unsigned int level,
+                                   VectorType         &u,
+                                   const VectorType   &rhs) const
+{
+  u = typename VectorType::value_type(0.);
+  smooth(level, u, rhs);
+}
 
 
 // Explicit instantiations
