@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 1999 - 2016 by the deal.II authors
+// Copyright (C) 1999 - 2018 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -28,6 +28,7 @@
 #include <deal.II/dofs/dof_accessor.h>
 #include <deal.II/multigrid/mg_tools.h>
 #include <deal.II/multigrid/mg_base.h>
+#include <deal.II/multigrid/mg_constrained_dofs.h>
 #include <deal.II/base/mg_level_object.h>
 #include <deal.II/dofs/dof_handler.h>
 #include <deal.II/dofs/dof_tools.h>
@@ -318,7 +319,7 @@ namespace MGTools
     // We loop over cells and go from
     // cells to lower dimensional
     // objects. This is the only way to
-    // cope withthe fact, that an
+    // cope with the fact, that an
     // unknown number of cells may
     // share an object of dimension
     // smaller than dim-1.
@@ -982,6 +983,39 @@ namespace MGTools
 
 
 
+  template <typename DoFHandlerType, typename SparsityPatternType>
+  void
+  make_interface_sparsity_pattern (const DoFHandlerType    &dof,
+                                   const MGConstrainedDoFs &mg_constrained_dofs,
+                                   SparsityPatternType     &sparsity,
+                                   const unsigned int      level)
+  {
+    const types::global_dof_index n_dofs = dof.n_dofs(level);
+    (void)n_dofs;
+
+    Assert (sparsity.n_rows() == n_dofs,
+            ExcDimensionMismatch (sparsity.n_rows(), n_dofs));
+    Assert (sparsity.n_cols() == n_dofs,
+            ExcDimensionMismatch (sparsity.n_cols(), n_dofs));
+
+    const unsigned int dofs_per_cell = dof.get_fe().dofs_per_cell;
+    std::vector<types::global_dof_index> dofs_on_this_cell(dofs_per_cell);
+    typename DoFHandlerType::cell_iterator cell = dof.begin(level),
+                                           endc = dof.end(level);
+    for (; cell!=endc; ++cell)
+      if (cell->is_locally_owned_on_level())
+        {
+          cell->get_mg_dof_indices (dofs_on_this_cell);
+          for (unsigned int i=0; i<dofs_per_cell; ++i)
+            for (unsigned int j=0; j<dofs_per_cell; ++j)
+              if (mg_constrained_dofs.is_interface_matrix_entry(level,dofs_on_this_cell[i],dofs_on_this_cell[j]))
+                sparsity.add (dofs_on_this_cell[i],
+                              dofs_on_this_cell[j]);
+        }
+  }
+
+
+
   template <int dim, int spacedim>
   void
   count_dofs_per_component (const DoFHandler<dim,spacedim> &dof_handler,
@@ -1074,7 +1108,8 @@ namespace MGTools
             Assert (!dof_handler.get_fe().is_primitive()
                     ||
                     std::accumulate (result[l].begin(),
-                                     result[l].end(), 0U)
+                                     result[l].end(),
+                                     types::global_dof_index(0))
                     ==
                     dof_handler.n_dofs(l),
                     ExcInternalError());
@@ -1239,7 +1274,7 @@ namespace MGTools
     local_dofs.reserve (DoFTools::max_dofs_per_face(dof));
     std::fill (local_dofs.begin (),
                local_dofs.end (),
-               DoFHandler<dim,spacedim>::invalid_dof_index);
+               numbers::invalid_dof_index);
 
     // First, deal with the simpler case when we have to identify all boundary dofs
     if (component_mask.n_selected_components(n_components) == n_components)
@@ -1512,6 +1547,40 @@ namespace MGTools
       }
 
   }
+
+
+
+  template <int dim, int spacedim>
+  unsigned int
+  max_level_for_coarse_mesh (const Triangulation<dim,spacedim> &tria)
+  {
+    // Find minimum level for an active cell in
+    // this locally owned subdomain
+    // Note: with the way active cells are traversed,
+    // the first locally owned cell we find will have
+    // the lowest level in the particular subdomain.
+    unsigned int min_level = tria.n_global_levels();
+    typename Triangulation<dim,spacedim>::active_cell_iterator cell = tria.begin_active(),
+                                                               endc = tria.end();
+    for (; cell!=endc; ++cell)
+      if (cell->is_locally_owned())
+        {
+          min_level = cell->level();
+          break;
+        }
+
+    unsigned int global_min = min_level;
+    // If necessary, communicate to find minimum
+    // level for an active cell over all subdomains
+    if (const parallel::Triangulation<dim,spacedim> *tr
+        = dynamic_cast<const parallel::Triangulation<dim,spacedim> *>
+        (&tria))
+      global_min = Utilities::MPI::min(min_level, tr->get_communicator());
+
+    AssertIndexRange(global_min, tria.n_global_levels());
+
+    return global_min;
+  }
 }
 
 
