@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 2003 - 2016 by the deal.II authors
+// Copyright (C) 2003 - 2017 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -58,7 +58,7 @@ namespace
     const unsigned int n_selected
       = std::accumulate(selected.begin(),
                         selected.end(),
-                        0U);
+                        0u);
 
     if (ndofs.size() == 0)
       {
@@ -273,13 +273,13 @@ void MGTransferBlockBase::build_matrices (
   // by itself
   prolongation_matrices.resize (0);
   prolongation_sparsities.resize (0);
+  prolongation_matrices.reserve (n_levels - 1);
+  prolongation_sparsities.reserve (n_levels - 1);
 
   for (unsigned int i=0; i<n_levels-1; ++i)
     {
-      prolongation_sparsities
-      .push_back (std_cxx11::shared_ptr<BlockSparsityPattern> (new BlockSparsityPattern));
-      prolongation_matrices
-      .push_back (std_cxx11::shared_ptr<BlockSparseMatrix<double> > (new BlockSparseMatrix<double>));
+      prolongation_sparsities.emplace_back (new BlockSparsityPattern);
+      prolongation_matrices.emplace_back (new BlockSparseMatrix<double>);
     }
 
   // two fields which will store the
@@ -398,7 +398,7 @@ void MGTransferBlockBase::build_matrices (
   // impose boundary conditions
   // but only in the column of
   // the prolongation matrix
-  if (mg_constrained_dofs != 0 && mg_constrained_dofs->have_boundary_indices())
+  if (mg_constrained_dofs != nullptr && mg_constrained_dofs->have_boundary_indices())
     {
       std::vector<types::global_dof_index> constrain_indices;
       std::vector<std::vector<bool> > constraints_per_block (n_blocks);
@@ -425,7 +425,7 @@ void MGTransferBlockBase::build_matrices (
             {
               const types::global_dof_index n_dofs = prolongation_matrices[level]->block(block, block).m();
               constraints_per_block[block].resize(0);
-              constraints_per_block[block].resize(n_dofs, 0);
+              constraints_per_block[block].resize(n_dofs, false);
               for (types::global_dof_index i=0; i<n_dofs; ++i, ++index)
                 constraints_per_block[block][i] = (constrain_indices[index] == 1);
 
@@ -494,7 +494,7 @@ void MGTransferBlockSelect<number>::build_matrices (
               const unsigned int block = fe.system_to_block_index(i).first;
               if (selected[block])
                 {
-                  if (mg_constrained_dofs != 0)
+                  if (mg_constrained_dofs != nullptr)
                     {
                       if (!mg_constrained_dofs->at_refinement_edge(level,level_dof_indices[i]))
                         temp_copy_indices[level_dof_indices[i] - mg_block_start[level][block]]
@@ -515,9 +515,9 @@ void MGTransferBlockSelect<number>::build_matrices (
       // copy_indices.
       const types::global_dof_index n_active_dofs =
         std::count_if (temp_copy_indices.begin(), temp_copy_indices.end(),
-                       std_cxx11::bind (std::not_equal_to<types::global_dof_index>(),
-                                        std_cxx11::_1,
-                                        numbers::invalid_dof_index));
+                       std::bind (std::not_equal_to<types::global_dof_index>(),
+                                  std::placeholders::_1,
+                                  numbers::invalid_dof_index));
       copy_indices[selected_block][level].resize (n_active_dofs);
       types::global_dof_index counter = 0;
       for (types::global_dof_index i=0; i<temp_copy_indices.size(); ++i)
@@ -596,9 +596,9 @@ void MGTransferBlock<number>::build_matrices (
             const types::global_dof_index n_active_dofs =
               std::count_if (temp_copy_indices[block].begin(),
                              temp_copy_indices[block].end(),
-                             std_cxx11::bind (std::not_equal_to<types::global_dof_index>(),
-                                              std_cxx11::_1,
-                                              numbers::invalid_dof_index));
+                             std::bind (std::not_equal_to<types::global_dof_index>(),
+                                        std::placeholders::_1,
+                                        numbers::invalid_dof_index));
             copy_indices[block][level].resize (n_active_dofs);
             types::global_dof_index counter = 0;
             for (types::global_dof_index i=0; i<temp_copy_indices[block].size(); ++i)
