@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 2016 - 2017 by the deal.II authors
+// Copyright (C) 2016 - 2018 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -53,19 +53,21 @@ namespace internal
       {}
     };
 
-    // Internal function for filling the copy indices from global to level
-    // indices
+
+
+
     template <int dim, int spacedim>
     void fill_copy_indices(const dealii::DoFHandler<dim,spacedim>                                                  &mg_dof,
                            const MGConstrainedDoFs                                                                 *mg_constrained_dofs,
                            std::vector<std::vector<std::pair<types::global_dof_index, types::global_dof_index> > > &copy_indices,
                            std::vector<std::vector<std::pair<types::global_dof_index, types::global_dof_index> > > &copy_indices_global_mine,
-                           std::vector<std::vector<std::pair<types::global_dof_index, types::global_dof_index> > > &copy_indices_level_mine)
+                           std::vector<std::vector<std::pair<types::global_dof_index, types::global_dof_index> > > &copy_indices_level_mine,
+                           const bool skip_interface_dofs)
     {
       // Now we are filling the variables copy_indices*, which are essentially
       // maps from global to mgdof for each level stored as a std::vector of
       // pairs. We need to split this map on each level depending on the
-      // ownership of the global and mgdof, so that we later not access
+      // ownership of the global and mgdof, so that we later do not access
       // non-local elements in copy_to/from_mg.
       // We keep track in the bitfield dof_touched which global dof has been
       // processed already (on the current level). This is the same as the
@@ -114,9 +116,11 @@ namespace internal
               for (unsigned int i=0; i<dofs_per_cell; ++i)
                 {
                   // we need to ignore if the DoF is on a refinement edge (hanging node)
-                  if (mg_constrained_dofs != 0
+                  if (skip_interface_dofs &&
+                      mg_constrained_dofs != nullptr
                       && mg_constrained_dofs->at_refinement_edge(level, level_dof_indices[i]))
                     continue;
+
                   types::global_dof_index global_idx = globally_relevant.index_within_set(global_dof_indices[i]);
                   //skip if we did this global dof already (on this or a coarser level)
                   if (dof_touched[global_idx])
@@ -127,16 +131,18 @@ namespace internal
 
                   if (global_mine && level_mine)
                     {
-                      copy_indices[level].push_back(
-                        std::make_pair (global_dof_indices[i], level_dof_indices[i]));
+                      copy_indices[level].emplace_back(global_dof_indices[i],
+                                                       level_dof_indices[i]);
                     }
                   else if (global_mine)
                     {
-                      copy_indices_global_mine[level].push_back(
-                        std::make_pair (global_dof_indices[i], level_dof_indices[i]));
+                      copy_indices_global_mine[level].emplace_back(global_dof_indices[i],
+                                                                   level_dof_indices[i]);
 
                       //send this to the owner of the level_dof:
-                      send_data_temp.push_back(DoFPair(level, global_dof_indices[i], level_dof_indices[i]));
+                      send_data_temp.emplace_back(level,
+                                                  global_dof_indices[i],
+                                                  level_dof_indices[i]);
                     }
                   else
                     {
@@ -148,10 +154,10 @@ namespace internal
             }
         }
 
-      const dealii::parallel::distributed::Triangulation<dim,spacedim> *tria =
-        (dynamic_cast<const parallel::distributed::Triangulation<dim,spacedim>*>
+      const dealii::parallel::Triangulation<dim,spacedim> *tria =
+        (dynamic_cast<const parallel::Triangulation<dim,spacedim>*>
          (&mg_dof.get_triangulation()));
-      AssertThrow(send_data_temp.size()==0 || tria!=NULL, ExcMessage("parallel Multigrid only works with a distributed Triangulation!"));
+      AssertThrow(send_data_temp.size()==0 || tria!=nullptr, ExcMessage("We should only be sending information with a parallel Triangulation!"));
 
 #ifdef DEAL_II_WITH_MPI
       if (tria)
@@ -161,7 +167,7 @@ namespace internal
           // The list of neighbors is symmetric (our neighbors have us as a
           // neighbor), so we can use it to send and to know how many messages
           // we will get.
-          std::set<types::subdomain_id> neighbors = tria->level_ghost_owners();
+          const std::set<types::subdomain_id> &neighbors = tria->level_ghost_owners();
           std::map<int, std::vector<DoFPair> > send_data;
 
           // * find owners of the level dofs and insert into send_data accordingly
@@ -194,14 +200,14 @@ namespace internal
                 // just send an empty message.
                 if (data.size())
                   {
-                    const int ierr = MPI_Isend(&data[0], data.size()*sizeof(data[0]),
+                    const int ierr = MPI_Isend(data.data(), data.size()*sizeof(data[0]),
                                                MPI_BYTE, dest, 71, tria->get_communicator(),
                                                &*requests.rbegin());
                     AssertThrowMPI(ierr);
                   }
                 else
                   {
-                    const int ierr = MPI_Isend(NULL, 0, MPI_BYTE, dest, 71,
+                    const int ierr = MPI_Isend(nullptr, 0, MPI_BYTE, dest, 71,
                                                tria->get_communicator(), &*requests.rbegin());
                     AssertThrowMPI(ierr);
                   }
@@ -223,7 +229,7 @@ namespace internal
 
                 if (len==0)
                   {
-                    ierr = MPI_Recv(NULL, 0, MPI_BYTE, status.MPI_SOURCE, status.MPI_TAG,
+                    ierr = MPI_Recv(nullptr, 0, MPI_BYTE, status.MPI_SOURCE, status.MPI_TAG,
                                     tria->get_communicator(), &status);
                     AssertThrowMPI(ierr);
                     continue;
@@ -233,16 +239,15 @@ namespace internal
                 Assert(static_cast<int>(count * sizeof(DoFPair)) == len, ExcInternalError());
                 receive_buffer.resize(count);
 
-                void *ptr = &receive_buffer[0];
+                void *ptr = receive_buffer.data();
                 ierr = MPI_Recv(ptr, len, MPI_BYTE, status.MPI_SOURCE, status.MPI_TAG,
                                 tria->get_communicator(), &status);
                 AssertThrowMPI(ierr);
 
                 for (unsigned int i=0; i<receive_buffer.size(); ++i)
                   {
-                    copy_indices_level_mine[receive_buffer[i].level].push_back(
-                      std::make_pair (receive_buffer[i].global_dof_index, receive_buffer[i].level_dof_index)
-                    );
+                    copy_indices_level_mine[receive_buffer[i].level].emplace_back(
+                      receive_buffer[i].global_dof_index, receive_buffer[i].level_dof_index);
                   }
               }
           }
@@ -250,7 +255,7 @@ namespace internal
           // * wait for all MPI_Isend to complete
           if (requests.size() > 0)
             {
-              const int ierr = MPI_Waitall(requests.size(), &requests[0], MPI_STATUSES_IGNORE);
+              const int ierr = MPI_Waitall(requests.size(), requests.data(), MPI_STATUSES_IGNORE);
               AssertThrowMPI(ierr);
               requests.clear();
             }
@@ -300,8 +305,7 @@ namespace internal
         {
           // shift the local number of the copy indices according to the new
           // partitioner that we are going to use for the vector
-          const std_cxx11::shared_ptr<const Utilities::MPI::Partitioner> part
-            = ghosted_level_vector.get_partitioner();
+          const auto &part = ghosted_level_vector.get_partitioner();
           ghosted_dofs.add_indices(part->ghost_indices());
           for (unsigned int i=0; i<copy_indices_global_mine.size(); ++i)
             copy_indices_global_mine[i].second =
@@ -441,6 +445,27 @@ namespace internal
               fe.get_prolongation_matrix(c)(renumbering[j],renumbering[i]);
     }
 
+    namespace
+    {
+      /**
+       * Helper function for setup_transfer. Checks for identity constrained dofs
+       * and replace with the indices of the dofs to which they are constrained
+       */
+      void replace (const MGConstrainedDoFs              *mg_constrained_dofs,
+                    const unsigned int                   level,
+                    std::vector<types::global_dof_index> &dof_indices)
+      {
+        if (mg_constrained_dofs != nullptr &&
+            mg_constrained_dofs->get_level_constraint_matrix(level).n_constraints() > 0)
+          for (auto &ind : dof_indices)
+            if (mg_constrained_dofs->get_level_constraint_matrix(level).is_identity_constrained(ind))
+              {
+                Assert(mg_constrained_dofs->get_level_constraint_matrix(level).get_constraint_entries(ind)->size() == 1,
+                       ExcInternalError());
+                ind = mg_constrained_dofs->get_level_constraint_matrix(level).get_constraint_entries(ind)->front().first;
+              }
+      }
+    }
 
     // Sets up most of the internal data structures of the MGTransferMatrixFree
     // class
@@ -479,11 +504,9 @@ namespace internal
                 ExcInternalError());
         fe_name[template_starts+1] = '1';
       }
-      std_cxx11::shared_ptr<FiniteElement<1> > fe_1d
-      (FETools::get_fe_by_name<1,1>(fe_name));
-      const FiniteElement<1> &fe = *fe_1d;
+      const std::unique_ptr<FiniteElement<1> > fe (FETools::get_fe_by_name<1,1>(fe_name));
 
-      setup_element_info(elem_info,fe,mg_dof);
+      setup_element_info(elem_info,*fe,mg_dof);
 
 
       // -------------- 2. Extract and match dof indices between child and parent
@@ -562,13 +585,15 @@ namespace internal
                     continue;
                   cell->child(c)->get_mg_dof_indices(local_dof_indices);
 
+                  replace(mg_constrained_dofs, level, local_dof_indices);
+
                   const IndexSet &owned_level_dofs = mg_dof.locally_owned_mg_dofs(level);
                   for (unsigned int i=0; i<local_dof_indices.size(); ++i)
                     if (!owned_level_dofs.is_element(local_dof_indices[i]))
                       ghosted_level_dofs.push_back(local_dof_indices[i]);
 
-                  add_child_indices<dim>(c, fe.dofs_per_cell - fe.dofs_per_vertex,
-                                         fe.degree, elem_info.lexicographic_numbering,
+                  add_child_indices<dim>(c, fe->dofs_per_cell - fe->dofs_per_vertex,
+                                         fe->degree, elem_info.lexicographic_numbering,
                                          local_dof_indices,
                                          &next_indices[start_index]);
 
@@ -596,7 +621,7 @@ namespace internal
 
                       // set Dirichlet boundary conditions (as a list of
                       // constrained DoFs) for the child
-                      if (mg_constrained_dofs != 0)
+                      if (mg_constrained_dofs != nullptr)
                         for (unsigned int i=0; i<mg_dof.get_fe().dofs_per_cell; ++i)
                           if (mg_constrained_dofs->is_boundary_index(level,
                                                                      local_dof_indices[elem_info.lexicographic_numbering[i]]))
@@ -618,6 +643,8 @@ namespace internal
                 {
                   cell->get_mg_dof_indices(local_dof_indices);
 
+                  replace(mg_constrained_dofs, level-1, local_dof_indices);
+
                   const IndexSet &owned_level_dofs_l0 = mg_dof.locally_owned_mg_dofs(0);
                   for (unsigned int i=0; i<local_dof_indices.size(); ++i)
                     if (!owned_level_dofs_l0.is_element(local_dof_indices[i]))
@@ -626,13 +653,13 @@ namespace internal
                   const std::size_t start_index = global_level_dof_indices_l0.size();
                   global_level_dof_indices_l0.resize(start_index+elem_info.n_child_cell_dofs,
                                                      numbers::invalid_dof_index);
-                  add_child_indices<dim>(0, fe.dofs_per_cell - fe.dofs_per_vertex,
-                                         fe.degree, elem_info.lexicographic_numbering,
+                  add_child_indices<dim>(0, fe->dofs_per_cell - fe->dofs_per_vertex,
+                                         fe->degree, elem_info.lexicographic_numbering,
                                          local_dof_indices,
                                          &global_level_dof_indices_l0[start_index]);
 
-                  dirichlet_indices[0].push_back(std::vector<unsigned short>());
-                  if (mg_constrained_dofs != 0)
+                  dirichlet_indices[0].emplace_back();
+                  if (mg_constrained_dofs != nullptr)
                     for (unsigned int i=0; i<mg_dof.get_fe().dofs_per_cell; ++i)
                       if (mg_constrained_dofs->is_boundary_index(0, local_dof_indices[elem_info.lexicographic_numbering[i]]))
                         dirichlet_indices[0].back().push_back(i);
@@ -665,7 +692,7 @@ namespace internal
           const parallel::Triangulation<dim,dim> *ptria =
             (dynamic_cast<const parallel::Triangulation<dim,dim>*> (&tria));
           const MPI_Comm communicator =
-            ptria != 0 ? ptria->get_communicator() : MPI_COMM_SELF;
+            ptria != nullptr ? ptria->get_communicator() : MPI_COMM_SELF;
 
           reinit_ghosted_vector (mg_dof.locally_owned_mg_dofs(level),
                                  ghosted_level_dofs, communicator,
@@ -696,7 +723,7 @@ namespace internal
 
       // ---------------------- 3. compute weights to make restriction additive
 
-      const unsigned int n_child_dofs_1d = fe.degree + 1 + fe.dofs_per_cell - fe.dofs_per_vertex;
+      const unsigned int n_child_dofs_1d = fe->degree + 1 + fe->dofs_per_cell - fe->dofs_per_vertex;
 
       // get the valence of the individual components and compute the weights as
       // the inverse of the valence
