@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 2016 - 2017 by the deal.II authors
+// Copyright (C) 2016 - 2018 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -16,7 +16,7 @@
 
 #include <deal.II/base/logstream.h>
 #include <deal.II/base/function.h>
-
+#include <deal.II/base/vectorization.h>
 #include <deal.II/lac/la_parallel_vector.h>
 #include <deal.II/grid/tria.h>
 #include <deal.II/grid/tria_iterator.h>
@@ -27,14 +27,14 @@
 #include <deal.II/multigrid/mg_transfer_matrix_free.h>
 #include <deal.II/multigrid/mg_transfer_internal.h>
 
-#include <deal.II/matrix_free/tensor_product_kernels.h>
+#include <deal.II/matrix_free/evaluation_kernels.h>
 
 #include <algorithm>
 
 DEAL_II_NAMESPACE_OPEN
 
 
-template<int dim, typename Number>
+template <int dim, typename Number>
 MGTransferMatrixFree<dim,Number>::MGTransferMatrixFree ()
   :
   fe_degree(0),
@@ -45,7 +45,7 @@ MGTransferMatrixFree<dim,Number>::MGTransferMatrixFree ()
 
 
 
-template<int dim, typename Number>
+template <int dim, typename Number>
 MGTransferMatrixFree<dim,Number>::MGTransferMatrixFree (const MGConstrainedDoFs &mg_c)
   :
   fe_degree(0),
@@ -58,12 +58,6 @@ MGTransferMatrixFree<dim,Number>::MGTransferMatrixFree (const MGConstrainedDoFs
 
 
 
-template <int dim, typename Number>
-MGTransferMatrixFree<dim,Number>::~MGTransferMatrixFree ()
-{}
-
-
-
 template <int dim, typename Number>
 void MGTransferMatrixFree<dim,Number>::initialize_constraints
 (const MGConstrainedDoFs &mg_c)
@@ -144,7 +138,7 @@ void MGTransferMatrixFree<dim,Number>::build
         }
     }
 
-  evaluation_data.resize(3*n_child_cell_dofs);
+  evaluation_data.resize(n_child_cell_dofs);
 }
 
 
@@ -163,7 +157,7 @@ void MGTransferMatrixFree<dim,Number>
   AssertDimension(this->ghosted_level_vector[to_level-1].local_size(),
                   src.local_size());
 
-  this->ghosted_level_vector[to_level-1] = src;
+  this->ghosted_level_vector[to_level-1].copy_locally_owned_data_from(src);
   this->ghosted_level_vector[to_level-1].update_ghost_values();
   this->ghosted_level_vector[to_level] = 0.;
 
@@ -208,7 +202,7 @@ void MGTransferMatrixFree<dim,Number>
                           this->ghosted_level_vector[to_level-1]);
 
   this->ghosted_level_vector[to_level].compress(VectorOperation::add);
-  dst = this->ghosted_level_vector[to_level];
+  dst.copy_locally_owned_data_from(this->ghosted_level_vector[to_level]);
 }
 
 
@@ -227,7 +221,7 @@ void MGTransferMatrixFree<dim,Number>
   AssertDimension(this->ghosted_level_vector[from_level-1].local_size(),
                   dst.local_size());
 
-  this->ghosted_level_vector[from_level] = src;
+  this->ghosted_level_vector[from_level].copy_locally_owned_data_from(src);
   this->ghosted_level_vector[from_level].update_ghost_values();
   this->ghosted_level_vector[from_level-1] = 0.;
 
@@ -277,51 +271,6 @@ void MGTransferMatrixFree<dim,Number>
 
 namespace
 {
-  template <int dim, typename Eval, typename Number, bool prolongate>
-  void
-  perform_tensorized_op(const Eval &evaluator,
-                        const unsigned int n_child_cell_dofs,
-                        const unsigned int n_components,
-                        AlignedVector<VectorizedArray<Number> > &evaluation_data)
-  {
-    if (Eval::n_q_points != numbers::invalid_unsigned_int)
-      AssertDimension(n_components * Eval::n_q_points, n_child_cell_dofs);
-    VectorizedArray<Number> *t0 = &evaluation_data[0];
-    VectorizedArray<Number> *t1 = &evaluation_data[n_child_cell_dofs];
-    VectorizedArray<Number> *t2 = &evaluation_data[2*n_child_cell_dofs];
-
-    for (unsigned int c=0; c<n_components; ++c)
-      {
-        // for the prolongate case, we go from dofs (living on the parent cell) to
-        // quads (living on all children) in the FEEvaluation terminology
-        if (dim == 1)
-          evaluator.template values<0,prolongate,false>(t0, t2);
-        else if (dim == 2)
-          {
-            evaluator.template values<0,prolongate,false>(t0, t1);
-            evaluator.template values<1,prolongate,false>(t1, t2);
-          }
-        else if (dim == 3)
-          {
-            evaluator.template values<0,prolongate,false>(t0, t2);
-            evaluator.template values<1,prolongate,false>(t2, t1);
-            evaluator.template values<2,prolongate,false>(t1, t2);
-          }
-        else
-          Assert(false, ExcNotImplemented());
-        if (prolongate)
-          {
-            t0 += Eval::dofs_per_cell;
-            t2 += Eval::n_q_points;
-          }
-        else
-          {
-            t0 += Eval::n_q_points;
-            t2 += Eval::dofs_per_cell;
-          }
-      }
-  }
-
   template <int dim, int degree, typename Number>
   void weight_dofs_on_child (const VectorizedArray<Number> *weights,
                              const unsigned int n_components,
@@ -370,11 +319,11 @@ void MGTransferMatrixFree<dim,Number>
   for (unsigned int cell=0; cell < n_owned_level_cells[to_level-1];
        cell += vec_size)
     {
-      const unsigned int n_chunks = cell+vec_size > n_owned_level_cells[to_level-1] ?
-                                    n_owned_level_cells[to_level-1] - cell : vec_size;
+      const unsigned int n_lanes = cell+vec_size > n_owned_level_cells[to_level-1] ?
+                                   n_owned_level_cells[to_level-1] - cell : vec_size;
 
       // read from source vector
-      for (unsigned int v=0; v<n_chunks; ++v)
+      for (unsigned int v=0; v<n_lanes; ++v)
         {
           const unsigned int shift = internal::MGTransfer::compute_shift_within_children<dim>
                                      (parent_child_connect[to_level-1][cell+v].second,
@@ -401,41 +350,37 @@ void MGTransferMatrixFree<dim,Number>
       // perform tensorized operation
       if (element_is_continuous)
         {
-          typedef internal::EvaluatorTensorProduct<internal::evaluate_general,dim,degree,degree!=-1 ? 2*degree+1 : 0,VectorizedArray<Number> > Evaluator;
-          Evaluator evaluator(prolongation_matrix_1d,
-                              prolongation_matrix_1d,
-                              prolongation_matrix_1d,
-                              fe_degree,
-                              2*fe_degree+1);
-          perform_tensorized_op<dim,Evaluator,Number,true>(evaluator,
-                                                           n_child_cell_dofs,
-                                                           n_components,
-                                                           evaluation_data);
+          // must go through the components backwards because we want to write
+          // the output to the same array as the input
+          for (int c=n_components-1; c>=0; --c)
+            internal::FEEvaluationImplBasisChange<internal::evaluate_general,dim,degree+1,2*degree+1,
+                     1,VectorizedArray<Number>, VectorizedArray<Number> >
+                     ::do_forward(prolongation_matrix_1d,
+                                  evaluation_data.begin()+ c*Utilities::fixed_power<dim>(degree_size),
+                                  evaluation_data.begin()+ c*n_scalar_cell_dofs,
+                                  fe_degree+1, 2*fe_degree+1);
           weight_dofs_on_child<dim,degree,Number>(&weights_on_refined[to_level-1][(cell/vec_size)*three_to_dim],
                                                   n_components, fe_degree,
-                                                  &evaluation_data[2*n_child_cell_dofs]);
+                                                  evaluation_data.begin());
         }
       else
         {
-          typedef internal::EvaluatorTensorProduct<internal::evaluate_general,dim,degree,2*(degree+1),VectorizedArray<Number> > Evaluator;
-          Evaluator evaluator(prolongation_matrix_1d,
-                              prolongation_matrix_1d,
-                              prolongation_matrix_1d,
-                              fe_degree,
-                              2*(fe_degree+1));
-          perform_tensorized_op<dim,Evaluator,Number,true>(evaluator,
-                                                           n_child_cell_dofs,
-                                                           n_components,
-                                                           evaluation_data);
+          for (int c=n_components-1; c>=0; --c)
+            internal::FEEvaluationImplBasisChange<internal::evaluate_general,dim,degree+1,2*degree+2,
+                     1,VectorizedArray<Number>, VectorizedArray<Number> >
+                     ::do_forward(prolongation_matrix_1d,
+                                  evaluation_data.begin() + c*Utilities::fixed_power<dim>(degree_size),
+                                  evaluation_data.begin() + c*n_scalar_cell_dofs,
+                                  fe_degree+1, 2*fe_degree+2);
         }
 
       // write into dst vector
       const unsigned int *indices = &level_dof_indices[to_level][cell*
                                                                  n_child_cell_dofs];
-      for (unsigned int v=0; v<n_chunks; ++v)
+      for (unsigned int v=0; v<n_lanes; ++v)
         {
           for (unsigned int i=0; i<n_child_cell_dofs; ++i)
-            dst.local_element(indices[i]) += evaluation_data[2*n_child_cell_dofs+i][v];
+            dst.local_element(indices[i]) += evaluation_data[i][v];
           indices += n_child_cell_dofs;
         }
     }
@@ -459,14 +404,14 @@ void MGTransferMatrixFree<dim,Number>
   for (unsigned int cell=0; cell < n_owned_level_cells[from_level-1];
        cell += vec_size)
     {
-      const unsigned int n_chunks = cell+vec_size > n_owned_level_cells[from_level-1] ?
-                                    n_owned_level_cells[from_level-1] - cell : vec_size;
+      const unsigned int n_lanes = cell+vec_size > n_owned_level_cells[from_level-1] ?
+                                   n_owned_level_cells[from_level-1] - cell : vec_size;
 
       // read from source vector
       {
         const unsigned int *indices = &level_dof_indices[from_level][cell*
                                       n_child_cell_dofs];
-        for (unsigned int v=0; v<n_chunks; ++v)
+        for (unsigned int v=0; v<n_lanes; ++v)
           {
             for (unsigned int i=0; i<n_child_cell_dofs; ++i)
               evaluation_data[i][v] = src.local_element(indices[i]);
@@ -479,36 +424,30 @@ void MGTransferMatrixFree<dim,Number>
       // perform tensorized operation
       if (element_is_continuous)
         {
-          typedef internal::EvaluatorTensorProduct<internal::evaluate_general,dim,degree,degree!=-1 ? 2*degree+1 : 0,VectorizedArray<Number> > Evaluator;
-          Evaluator evaluator(prolongation_matrix_1d,
-                              prolongation_matrix_1d,
-                              prolongation_matrix_1d,
-                              fe_degree,
-                              2*fe_degree+1);
           weight_dofs_on_child<dim,degree,Number>(&weights_on_refined[from_level-1][(cell/vec_size)*three_to_dim],
                                                   n_components, fe_degree,
                                                   &evaluation_data[0]);
-          perform_tensorized_op<dim,Evaluator,Number,false>(evaluator,
-                                                            n_child_cell_dofs,
-                                                            n_components,
-                                                            evaluation_data);
+          for (unsigned int c=0; c<n_components; ++c)
+            internal::FEEvaluationImplBasisChange<internal::evaluate_general,dim,degree+1,2*degree+1,
+                     1,VectorizedArray<Number>,VectorizedArray<Number> >
+                     ::do_backward(prolongation_matrix_1d, false,
+                                   evaluation_data.begin() + c*n_scalar_cell_dofs,
+                                   evaluation_data.begin() + c*Utilities::fixed_power<dim>(degree_size),
+                                   fe_degree+1, 2*fe_degree+1);
         }
       else
         {
-          typedef internal::EvaluatorTensorProduct<internal::evaluate_general,dim,degree,2*(degree+1),VectorizedArray<Number> > Evaluator;
-          Evaluator evaluator(prolongation_matrix_1d,
-                              prolongation_matrix_1d,
-                              prolongation_matrix_1d,
-                              fe_degree,
-                              2*(fe_degree+1));
-          perform_tensorized_op<dim,Evaluator,Number,false>(evaluator,
-                                                            n_child_cell_dofs,
-                                                            n_components,
-                                                            evaluation_data);
+          for (unsigned int c=0; c<n_components; ++c)
+            internal::FEEvaluationImplBasisChange<internal::evaluate_general,dim,degree+1,2*degree+2,
+                     1,VectorizedArray<Number>,VectorizedArray<Number> >
+                     ::do_backward(prolongation_matrix_1d, false,
+                                   evaluation_data.begin() + c*n_scalar_cell_dofs,
+                                   evaluation_data.begin() + c*Utilities::fixed_power<dim>(degree_size),
+                                   fe_degree+1, 2*fe_degree+2);
         }
 
       // write into dst vector
-      for (unsigned int v=0; v<n_chunks; ++v)
+      for (unsigned int v=0; v<n_lanes; ++v)
         {
           const unsigned int shift = internal::MGTransfer::compute_shift_within_children<dim>
                                      (parent_child_connect[from_level-1][cell+v].second,
@@ -521,7 +460,7 @@ void MGTransferMatrixFree<dim,Number>
             {
               // apply Dirichlet boundary conditions on parent cell
               for (std::vector<unsigned short>::const_iterator i=dirichlet_indices[from_level-1][cell+v].begin(); i!=dirichlet_indices[from_level-1][cell+v].end(); ++i)
-                evaluation_data[2*n_child_cell_dofs+(*i)][v] = 0.;
+                evaluation_data[*i][v] = 0.;
 
               for (unsigned int k=0; k<(dim>2 ? degree_size : 1); ++k)
                 for (unsigned int j=0; j<(dim>1 ? degree_size : 1); ++j)
@@ -529,7 +468,7 @@ void MGTransferMatrixFree<dim,Number>
                     dst.local_element(indices[c*n_scalar_cell_dofs +
                                               k*n_child_dofs_1d*n_child_dofs_1d+
                                               j*n_child_dofs_1d+i])
-                    += evaluation_data[2*n_child_cell_dofs+m][v];
+                    += evaluation_data[m][v];
             }
         }
     }
@@ -553,6 +492,142 @@ MGTransferMatrixFree<dim,Number>::memory_consumption() const
 }
 
 
+
+template <int dim, typename Number>
+MGTransferBlockMatrixFree<dim,Number>::MGTransferBlockMatrixFree (const MGConstrainedDoFs &mg_c)
+  : same_for_all (true)
+{
+  matrix_free_transfer_vector.emplace_back(mg_c);
+}
+
+
+
+template <int dim, typename Number>
+MGTransferBlockMatrixFree<dim,Number>::MGTransferBlockMatrixFree
+(const std::vector<MGConstrainedDoFs> &mg_c)
+  : same_for_all (false)
+{
+  for (unsigned int i=0; i<mg_c.size(); ++i)
+    matrix_free_transfer_vector.emplace_back(mg_c[i]);
+}
+
+
+
+template <int dim, typename Number>
+void MGTransferBlockMatrixFree<dim,Number>::initialize_constraints
+(const MGConstrainedDoFs &mg_c)
+{
+  Assert(same_for_all,
+         ExcMessage("This object was initialized with support for usage with "
+                    "one DoFHandler for each block, but this method assumes "
+                    "that the same DoFHandler is used for all the blocks!"));
+  AssertDimension(matrix_free_transfer_vector.size(),1);
+
+  matrix_free_transfer_vector[0].initialize_constraints(mg_c);
+}
+
+
+
+template <int dim, typename Number>
+void MGTransferBlockMatrixFree<dim,Number>::initialize_constraints
+(const std::vector<MGConstrainedDoFs> &mg_c)
+{
+  Assert(!same_for_all,
+         ExcMessage("This object was initialized with support for using "
+                    "the same DoFHandler for all the blocks, but this "
+                    "method assumes that there is a separate DoFHandler "
+                    "for each block!"));
+  AssertDimension(matrix_free_transfer_vector.size(),mg_c.size());
+
+  for (unsigned int i = 0; i<mg_c.size(); ++i)
+    matrix_free_transfer_vector[i].initialize_constraints(mg_c[i]);
+}
+
+
+
+template <int dim, typename Number>
+void MGTransferBlockMatrixFree<dim,Number>::clear ()
+{
+  matrix_free_transfer_vector.clear();
+}
+
+
+
+template <int dim, typename Number>
+void MGTransferBlockMatrixFree<dim,Number>::build
+(const DoFHandler<dim,dim>  &mg_dof)
+{
+  AssertDimension(matrix_free_transfer_vector.size(),1);
+  matrix_free_transfer_vector[0].build(mg_dof);
+}
+
+
+
+template <int dim, typename Number>
+void MGTransferBlockMatrixFree<dim,Number>::build
+(const std::vector<const DoFHandler<dim,dim>*> &mg_dof)
+{
+  AssertDimension (matrix_free_transfer_vector.size(),mg_dof.size());
+  for (unsigned int i=0; i<mg_dof.size(); ++i)
+    matrix_free_transfer_vector[i].build(*mg_dof[i]);
+}
+
+
+
+template <int dim, typename Number>
+void MGTransferBlockMatrixFree<dim,Number>
+::prolongate (const unsigned int                           to_level,
+              LinearAlgebra::distributed::BlockVector<Number>       &dst,
+              const LinearAlgebra::distributed::BlockVector<Number> &src) const
+{
+  const unsigned int n_blocks = src.n_blocks();
+  AssertDimension(dst.n_blocks(), n_blocks);
+
+  if (!same_for_all)
+    AssertDimension (matrix_free_transfer_vector.size(), n_blocks);
+
+  for (unsigned int b = 0; b < n_blocks; ++b)
+    {
+      const unsigned int data_block = same_for_all ? 0 : b;
+      matrix_free_transfer_vector[data_block].prolongate(to_level, dst.block(b), src.block(b));
+    }
+}
+
+
+
+template <int dim, typename Number>
+void MGTransferBlockMatrixFree<dim,Number>
+::restrict_and_add (const unsigned int                           from_level,
+                    LinearAlgebra::distributed::BlockVector<Number>       &dst,
+                    const LinearAlgebra::distributed::BlockVector<Number> &src) const
+{
+  const unsigned int n_blocks = src.n_blocks();
+  AssertDimension(dst.n_blocks(), n_blocks);
+
+  if (!same_for_all)
+    AssertDimension (matrix_free_transfer_vector.size(), n_blocks);
+
+  for (unsigned int b = 0; b < n_blocks; ++b)
+    {
+      const unsigned int data_block = same_for_all ? 0 : b;
+      matrix_free_transfer_vector[data_block].restrict_and_add(from_level, dst.block(b), src.block(b));
+    }
+}
+
+
+
+template <int dim, typename Number>
+std::size_t
+MGTransferBlockMatrixFree<dim,Number>::memory_consumption() const
+{
+  std::size_t total_memory_consumption= 0;
+  for (const auto &el: matrix_free_transfer_vector)
+    total_memory_consumption += el.memory_consumption();
+  return total_memory_consumption;
+}
+
+
+
 // explicit instantiation
 #include "mg_transfer_matrix_free.inst"
 
