@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 2003 - 2016 by the deal.II authors
+// Copyright (C) 2003 - 2017 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -21,10 +21,9 @@
 #include <deal.II/lac/block_vector.h>
 #include <deal.II/lac/la_parallel_vector.h>
 #include <deal.II/lac/la_parallel_block_vector.h>
-#include <deal.II/lac/petsc_vector.h>
-#include <deal.II/lac/petsc_block_vector.h>
+#include <deal.II/lac/trilinos_epetra_vector.h>
 #include <deal.II/lac/trilinos_vector.h>
-#include <deal.II/lac/trilinos_block_vector.h>
+#include <deal.II/lac/trilinos_parallel_block_vector.h>
 #include <deal.II/lac/sparse_matrix.h>
 #include <deal.II/lac/block_sparse_matrix.h>
 #include <deal.II/lac/dynamic_sparsity_pattern.h>
@@ -41,13 +40,7 @@
 DEAL_II_NAMESPACE_OPEN
 
 
-template<typename VectorType>
-MGTransferPrebuilt<VectorType>::MGTransferPrebuilt ()
-{}
-
-
-
-template<typename VectorType>
+template <typename VectorType>
 MGTransferPrebuilt<VectorType>::MGTransferPrebuilt (const MGConstrainedDoFs &mg_c)
 {
   this->mg_constrained_dofs = &mg_c;
@@ -55,7 +48,7 @@ MGTransferPrebuilt<VectorType>::MGTransferPrebuilt (const MGConstrainedDoFs &mg_
 
 
 
-template<typename VectorType>
+template <typename VectorType>
 MGTransferPrebuilt<VectorType>::MGTransferPrebuilt (const ConstraintMatrix &/*c*/, const MGConstrainedDoFs &mg_c)
 {
   this->mg_constrained_dofs = &mg_c;
@@ -63,12 +56,6 @@ MGTransferPrebuilt<VectorType>::MGTransferPrebuilt (const ConstraintMatrix &/*c*
 
 
 
-template <typename VectorType>
-MGTransferPrebuilt<VectorType>::~MGTransferPrebuilt ()
-{}
-
-
-
 template <typename VectorType>
 void MGTransferPrebuilt<VectorType>::initialize_constraints
 (const MGConstrainedDoFs &mg_c)
@@ -124,6 +111,27 @@ void MGTransferPrebuilt<VectorType>::restrict_and_add (const unsigned int from_l
 }
 
 
+namespace
+{
+  /**
+   * Helper function for build_matrices. Checks for identity constrained dofs
+   * and replace with the indices of the dofs to which they are constrained
+   */
+  void replace (const MGConstrainedDoFs              *mg_constrained_dofs,
+                const unsigned int                   level,
+                std::vector<types::global_dof_index> &dof_indices)
+  {
+    if (mg_constrained_dofs != nullptr &&
+        mg_constrained_dofs->get_level_constraint_matrix(level).n_constraints() > 0)
+      for (auto &ind : dof_indices)
+        if (mg_constrained_dofs->get_level_constraint_matrix(level).is_identity_constrained(ind))
+          {
+            Assert(mg_constrained_dofs->get_level_constraint_matrix(level).get_constraint_entries(ind)->size() == 1,
+                   ExcInternalError());
+            ind = mg_constrained_dofs->get_level_constraint_matrix(level).get_constraint_entries(ind)->front().first;
+          }
+  }
+}
 
 template <typename VectorType>
 template <int dim, int spacedim>
@@ -149,13 +157,15 @@ void MGTransferPrebuilt<VectorType>::build_matrices
   // by itself
   prolongation_matrices.resize (0);
   prolongation_sparsities.resize (0);
+  prolongation_matrices.reserve (n_levels - 1);
+  prolongation_sparsities.reserve (n_levels - 1);
 
   for (unsigned int i=0; i<n_levels-1; ++i)
     {
-      prolongation_sparsities.push_back
-      (std_cxx11::shared_ptr<typename internal::MatrixSelector<VectorType>::Sparsity> (new typename internal::MatrixSelector<VectorType>::Sparsity));
-      prolongation_matrices.push_back
-      (std_cxx11::shared_ptr<typename internal::MatrixSelector<VectorType>::Matrix> (new typename internal::MatrixSelector<VectorType>::Matrix));
+      prolongation_sparsities.emplace_back
+      (new typename internal::MatrixSelector<VectorType>::Sparsity);
+      prolongation_matrices.emplace_back
+      (new typename internal::MatrixSelector<VectorType>::Matrix);
     }
 
   // two fields which will store the
@@ -194,6 +204,8 @@ void MGTransferPrebuilt<VectorType>::build_matrices
           {
             cell->get_mg_dof_indices (dof_indices_parent);
 
+            replace(this->mg_constrained_dofs, level, dof_indices_parent);
+
             Assert(cell->n_children()==GeometryInfo<dim>::max_children_per_cell,
                    ExcNotImplemented());
             for (unsigned int child=0; child<cell->n_children(); ++child)
@@ -207,6 +219,8 @@ void MGTransferPrebuilt<VectorType>::build_matrices
 
                 cell->child(child)->get_mg_dof_indices (dof_indices_child);
 
+                replace(this->mg_constrained_dofs, level+1, dof_indices_child);
+
                 // now tag the entries in the
                 // matrix which will be used
                 // for this pair of parent/child
@@ -240,6 +254,8 @@ void MGTransferPrebuilt<VectorType>::build_matrices
           {
             cell->get_mg_dof_indices (dof_indices_parent);
 
+            replace(this->mg_constrained_dofs, level, dof_indices_parent);
+
             Assert(cell->n_children()==GeometryInfo<dim>::max_children_per_cell,
                    ExcNotImplemented());
             for (unsigned int child=0; child<cell->n_children(); ++child)
@@ -249,7 +265,7 @@ void MGTransferPrebuilt<VectorType>::build_matrices
                   = mg_dof.get_fe().get_prolongation_matrix (child,
                                                              cell->refinement_case());
 
-                if (this->mg_constrained_dofs != 0 &&
+                if (this->mg_constrained_dofs != nullptr &&
                     this->mg_constrained_dofs->have_boundary_indices())
                   for (unsigned int j=0; j<dofs_per_cell; ++j)
                     if (this->mg_constrained_dofs->is_boundary_index(level, dof_indices_parent[j]))
@@ -258,11 +274,13 @@ void MGTransferPrebuilt<VectorType>::build_matrices
 
                 cell->child(child)->get_mg_dof_indices (dof_indices_child);
 
+                replace(this->mg_constrained_dofs, level+1, dof_indices_child);
+
                 // now set the entries in the matrix
                 for (unsigned int i=0; i<dofs_per_cell; ++i)
                   prolongation_matrices[level]->set (dof_indices_child[i],
                                                      dofs_per_cell,
-                                                     &dof_indices_parent[0],
+                                                     dof_indices_parent.data(),
                                                      &prolongation(i,0),
                                                      true);
               }
