@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 2000 - 2016 by the deal.II authors
+// Copyright (C) 2000 - 2017 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -18,10 +18,10 @@
 #include <deal.II/lac/la_vector.h>
 #include <deal.II/lac/la_parallel_vector.h>
 #include <deal.II/lac/la_parallel_block_vector.h>
-#include <deal.II/lac/petsc_vector.h>
-#include <deal.II/lac/petsc_block_vector.h>
+#include <deal.II/lac/petsc_parallel_vector.h>
+#include <deal.II/lac/petsc_parallel_block_vector.h>
 #include <deal.II/lac/trilinos_vector.h>
-#include <deal.II/lac/trilinos_block_vector.h>
+#include <deal.II/lac/trilinos_parallel_block_vector.h>
 #include <deal.II/lac/sparse_matrix.h>
 #include <deal.II/lac/block_sparse_matrix.h>
 #include <deal.II/multigrid/mg_transfer.h>
@@ -61,14 +61,14 @@ MGTransferBlockBase::MGTransferBlockBase (const ConstraintMatrix &/*c*/,
 template <typename number>
 MGTransferBlock<number>::MGTransferBlock ()
   :
-  memory(0, typeid(*this).name())
+  memory(nullptr, typeid(*this).name())
 {}
 
 
 template <typename number>
 MGTransferBlock<number>::~MGTransferBlock ()
 {
-  if (memory != 0) memory = 0;
+  if (memory != nullptr) memory = nullptr;
 }
 
 
@@ -126,7 +126,7 @@ void MGTransferBlock<number>::restrict_and_add (
         {
           if (factors.size() != 0)
             {
-              Assert (memory != 0, ExcNotInitialized());
+              Assert (memory != nullptr, ExcNotInitialized());
               Vector<number> *aux = memory->alloc();
               aux->reinit(dst.block(this->mg_block[b]));
               prolongation_matrices[from_level-1]->block(b,b).Tvmult (
@@ -198,7 +198,7 @@ MGTransferBlockBase::memory_consumption () const
 
 //----------------------------------------------------------------------//
 
-template<typename number>
+template <typename number>
 MGTransferSelect<number>::MGTransferSelect ()
   :
   selected_component (0),
@@ -206,7 +206,7 @@ MGTransferSelect<number>::MGTransferSelect ()
 {}
 
 
-template<typename number>
+template <typename number>
 MGTransferSelect<number>::MGTransferSelect (const ConstraintMatrix &c)
   :
   selected_component (0),
@@ -214,9 +214,6 @@ MGTransferSelect<number>::MGTransferSelect (const ConstraintMatrix &c)
   constraints(&c)
 {}
 
-template <typename number>
-MGTransferSelect<number>::~MGTransferSelect ()
-{}
 
 
 template <typename number>
@@ -279,12 +276,6 @@ MGTransferBlockSelect<number>::MGTransferBlockSelect (const ConstraintMatrix &/*
 
 
 
-template <typename number>
-MGTransferBlockSelect<number>::~MGTransferBlockSelect ()
-{}
-
-
-
 template <typename number>
 void
 MGTransferBlockSelect<number>::prolongate (const unsigned int   to_level,
