@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 1999 - 2016 by the deal.II authors
+// Copyright (C) 1999 - 2018 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -33,7 +33,7 @@ DEAL_II_NAMESPACE_OPEN
 
 namespace internal
 {
-  namespace DataOut
+  namespace DataOutImplementation
   {
     template <int dim, int spacedim>
     ParallelData<dim,spacedim>::
@@ -41,7 +41,7 @@ namespace internal
                   const unsigned int n_subdivisions,
                   const std::vector<unsigned int> &n_postprocessor_outputs,
                   const Mapping<dim,spacedim> &mapping,
-                  const std::vector<std_cxx11::shared_ptr<dealii::hp::FECollection<dim,spacedim> > > &finite_elements,
+                  const std::vector<std::shared_ptr<dealii::hp::FECollection<dim,spacedim> > > &finite_elements,
                   const UpdateFlags update_flags,
                   const std::vector<std::vector<unsigned int> > &cell_to_patch_index_map)
       :
@@ -64,7 +64,7 @@ void
 DataOut<dim,DoFHandlerType>::
 build_one_patch
 (const std::pair<cell_iterator, unsigned int>                                                *cell_and_index,
- internal::DataOut::ParallelData<DoFHandlerType::dimension, DoFHandlerType::space_dimension> &scratch_data,
+ internal::DataOutImplementation::ParallelData<DoFHandlerType::dimension, DoFHandlerType::space_dimension> &scratch_data,
  const unsigned int                                                                           n_subdivisions,
  const CurvedCellRegion                                                                       curved_cell_region)
 {
@@ -145,7 +145,7 @@ build_one_patch
           const DataPostprocessor<DoFHandlerType::space_dimension> *postprocessor
             = this->dof_data[dataset]->postprocessor;
 
-          if (postprocessor != 0)
+          if (postprocessor != nullptr)
             {
               // we have to postprocess the data, so determine, which fields
               // have to be updated
@@ -156,12 +156,15 @@ build_one_patch
                   // gradient etc.
                   if (update_flags & update_values)
                     this->dof_data[dataset]->get_function_values (this_fe_patch_values,
+                                                                  internal::DataOutImplementation::ComponentExtractor::real_part,
                                                                   scratch_data.patch_values_scalar.solution_values);
                   if (update_flags & update_gradients)
                     this->dof_data[dataset]->get_function_gradients (this_fe_patch_values,
+                                                                     internal::DataOutImplementation::ComponentExtractor::real_part,
                                                                      scratch_data.patch_values_scalar.solution_gradients);
                   if (update_flags & update_hessians)
                     this->dof_data[dataset]->get_function_hessians (this_fe_patch_values,
+                                                                    internal::DataOutImplementation::ComponentExtractor::real_part,
                                                                     scratch_data.patch_values_scalar.solution_hessians);
 
                   if (update_flags & update_quadrature_points)
@@ -185,12 +188,15 @@ build_one_patch
                   // derivative...
                   if (update_flags & update_values)
                     this->dof_data[dataset]->get_function_values (this_fe_patch_values,
+                                                                  internal::DataOutImplementation::ComponentExtractor::real_part,
                                                                   scratch_data.patch_values_system.solution_values);
                   if (update_flags & update_gradients)
                     this->dof_data[dataset]->get_function_gradients (this_fe_patch_values,
+                                                                     internal::DataOutImplementation::ComponentExtractor::real_part,
                                                                      scratch_data.patch_values_system.solution_gradients);
                   if (update_flags & update_hessians)
                     this->dof_data[dataset]->get_function_hessians (this_fe_patch_values,
+                                                                    internal::DataOutImplementation::ComponentExtractor::real_part,
                                                                     scratch_data.patch_values_system.solution_hessians);
 
                   if (update_flags & update_quadrature_points)
@@ -215,29 +221,60 @@ build_one_patch
                     = scratch_data.postprocessed_values[dataset][q](component);
             }
           else
-            // now we use the given data vector without modifications. again,
+            // use the given data vector directly, without a postprocessor. again,
             // we treat single component functions separately for efficiency
             // reasons.
             if (n_components == 1)
               {
+                // first output the real part of the solution vector
                 this->dof_data[dataset]->get_function_values (this_fe_patch_values,
+                                                              internal::DataOutImplementation::ComponentExtractor::real_part,
                                                               scratch_data.patch_values_scalar.solution_values);
                 for (unsigned int q=0; q<n_q_points; ++q)
                   patch.data(offset,q) = scratch_data.patch_values_scalar.solution_values[q];
+
+                // and if there is one, also output the imaginary part
+                if (this->dof_data[dataset]->is_complex_valued() == true)
+                  {
+                    this->dof_data[dataset]->get_function_values (this_fe_patch_values,
+                                                                  internal::DataOutImplementation::ComponentExtractor::imaginary_part,
+                                                                  scratch_data.patch_values_scalar.solution_values);
+                    for (unsigned int q=0; q<n_q_points; ++q)
+                      patch.data(offset+1,q) = scratch_data.patch_values_scalar.solution_values[q];
+                  }
               }
             else
               {
                 scratch_data.resize_system_vectors(n_components);
+
+                // same as above: first the real part
+                const unsigned int stride = (this->dof_data[dataset]->is_complex_valued() ? 2 : 1);
                 this->dof_data[dataset]->get_function_values (this_fe_patch_values,
+                                                              internal::DataOutImplementation::ComponentExtractor::real_part,
                                                               scratch_data.patch_values_system.solution_values);
                 for (unsigned int component=0; component<n_components;
                      ++component)
                   for (unsigned int q=0; q<n_q_points; ++q)
-                    patch.data(offset+component,q) =
+                    patch.data(offset+component*stride,q) =
                       scratch_data.patch_values_system.solution_values[q](component);
+
+                // and if there is one, also output the imaginary part
+                if (this->dof_data[dataset]->is_complex_valued() == true)
+                  {
+                    this->dof_data[dataset]->get_function_values (this_fe_patch_values,
+                                                                  internal::DataOutImplementation::ComponentExtractor::imaginary_part,
+                                                                  scratch_data.patch_values_system.solution_values);
+                    for (unsigned int component=0; component<n_components;
+                         ++component)
+                      for (unsigned int q=0; q<n_q_points; ++q)
+                        patch.data(offset+component*stride+1,q) =
+                          scratch_data.patch_values_system.solution_values[q](component);
+                  }
               }
+
           // increment the counter for the actual data record
-          offset+=this->dof_data[dataset]->n_output_variables;
+          offset += this->dof_data[dataset]->n_output_variables *
+                    (this->dof_data[dataset]->is_complex_valued() ? 2 : 1);
         }
 
       // then do the cell data. only compute the number of a cell if needed;
@@ -249,10 +286,26 @@ build_one_patch
 
           for (unsigned int dataset=0; dataset<this->cell_data.size(); ++dataset)
             {
-              const double value
-                = this->cell_data[dataset]->get_cell_data_value (cell_and_index->second);
-              for (unsigned int q=0; q<n_q_points; ++q)
-                patch.data(offset+dataset,q) = value;
+              // as above, first output the real part
+              {
+                const double value
+                  = this->cell_data[dataset]->get_cell_data_value (cell_and_index->second,
+                                                                   internal::DataOutImplementation::ComponentExtractor::real_part);
+                for (unsigned int q=0; q<n_q_points; ++q)
+                  patch.data(offset,q) = value;
+              }
+
+              // and if there is one, also output the imaginary part
+              if (this->cell_data[dataset]->is_complex_valued() == true)
+                {
+                  const double value
+                    = this->cell_data[dataset]->get_cell_data_value (cell_and_index->second,
+                                                                     internal::DataOutImplementation::ComponentExtractor::imaginary_part);
+                  for (unsigned int q=0; q<n_q_points; ++q)
+                    patch.data(offset+1,q) = value;
+                }
+
+              offset += (this->cell_data[dataset]->is_complex_valued() ? 2 : 1);
             }
         }
     }
@@ -331,14 +384,14 @@ void DataOut<dim,DoFHandlerType>::build_patches
   // Check consistency of redundant template parameter
   Assert (dim==DoFHandlerType::dimension, ExcDimensionMismatch(dim, DoFHandlerType::dimension));
 
-  Assert (this->triangulation != 0,
-          Exceptions::DataOut::ExcNoTriangulationSelected());
+  Assert (this->triangulation != nullptr,
+          Exceptions::DataOutImplementation::ExcNoTriangulationSelected());
 
   const unsigned int n_subdivisions = (n_subdivisions_ != 0)
                                       ? n_subdivisions_
                                       : this->default_subdivisions;
   Assert (n_subdivisions >= 1,
-          Exceptions::DataOut::ExcInvalidNumberOfSubdivisions(n_subdivisions));
+          Exceptions::DataOutImplementation::ExcInvalidNumberOfSubdivisions(n_subdivisions));
 
   this->validate_dataset_names();
 
@@ -406,7 +459,7 @@ void DataOut<dim,DoFHandlerType>::build_patches
                 ExcInternalError());
         cell_to_patch_index_map[cell->level()][cell->index()] = all_cells.size();
 
-        all_cells.push_back (std::make_pair(cell, active_index));
+        all_cells.emplace_back (cell, active_index);
       }
   }
 
@@ -414,9 +467,12 @@ void DataOut<dim,DoFHandlerType>::build_patches
   this->patches.resize(all_cells.size());
 
   // now create a default object for the WorkStream object to work with
-  unsigned int n_datasets=this->cell_data.size();
+  unsigned int n_datasets = 0;
+  for (unsigned int i=0; i<this->cell_data.size(); ++i)
+    n_datasets += (this->cell_data[i]->is_complex_valued() ? 2 : 1);
   for (unsigned int i=0; i<this->dof_data.size(); ++i)
-    n_datasets += this->dof_data[i]->n_output_variables;
+    n_datasets += (this->dof_data[i]->n_output_variables
+                   * (this->dof_data[i]->is_complex_valued() ? 2 : 1));
 
   std::vector<unsigned int> n_postprocessor_outputs (this->dof_data.size());
   for (unsigned int dataset=0; dataset<this->dof_data.size(); ++dataset)
@@ -441,11 +497,11 @@ void DataOut<dim,DoFHandlerType>::build_patches
           ExcMessage("The update of normal vectors may not be requested for evaluation of "
                      "data on cells via DataPostprocessor."));
 
-  internal::DataOut::ParallelData<DoFHandlerType::dimension, DoFHandlerType::space_dimension>
+  internal::DataOutImplementation::ParallelData<DoFHandlerType::dimension, DoFHandlerType::space_dimension>
   thread_data (n_datasets, n_subdivisions,
                n_postprocessor_outputs,
                mapping,
-               this->get_finite_elements(),
+               this->get_fes(),
                update_flags,
                cell_to_patch_index_map);
 
@@ -453,17 +509,17 @@ void DataOut<dim,DoFHandlerType>::build_patches
   if (all_cells.size() > 0)
     WorkStream::run (&all_cells[0],
                      &all_cells[0]+all_cells.size(),
-                     std_cxx11::bind(&DataOut<dim,DoFHandlerType>::build_one_patch,
-                                     this,
-                                     std_cxx11::_1,
-                                     std_cxx11::_2,
-                                     /* no std_cxx11::_3, since this function doesn't actually need a
-                                        copy data object -- it just writes everything right into the
-                                        output array */
-                                     n_subdivisions,
-                                     curved_cell_region),
+                     std::bind(&DataOut<dim,DoFHandlerType>::build_one_patch,
+                               this,
+                               std::placeholders::_1,
+                               std::placeholders::_2,
+                               /* no std::placeholders::_3, since this function doesn't actually need a
+                                  copy data object -- it just writes everything right into the
+                                  output array */
+                               n_subdivisions,
+                               curved_cell_region),
                      // no copy-local-to-global function needed here
-                     std_cxx11::function<void (const int &)>(),
+                     std::function<void (const int &)>(),
                      thread_data,
                      /* dummy CopyData object = */ 0,
                      // experimenting shows that we can make things run a bit
