@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 2000 - 2016 by the deal.II authors
+// Copyright (C) 2000 - 2018 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -32,7 +32,7 @@ DEAL_II_NAMESPACE_OPEN
 
 namespace internal
 {
-  namespace DataOutFaces
+  namespace DataOutFacesImplementation
   {
     template <int dim, int spacedim>
     ParallelData<dim,spacedim>::
@@ -40,10 +40,10 @@ namespace internal
                   const unsigned int n_subdivisions,
                   const std::vector<unsigned int> &n_postprocessor_outputs,
                   const Mapping<dim,spacedim> &mapping,
-                  const std::vector<std_cxx11::shared_ptr<dealii::hp::FECollection<dim,spacedim> > > &finite_elements,
+                  const std::vector<std::shared_ptr<dealii::hp::FECollection<dim,spacedim> > > &finite_elements,
                   const UpdateFlags update_flags)
       :
-      internal::DataOut::
+      internal::DataOutImplementation::
       ParallelDataBase<dim,spacedim> (n_datasets,
                                       n_subdivisions,
                                       n_postprocessor_outputs,
@@ -87,7 +87,7 @@ template <int dim, typename DoFHandlerType>
 void
 DataOutFaces<dim,DoFHandlerType>::
 build_one_patch (const FaceDescriptor *cell_and_face,
-                 internal::DataOutFaces::ParallelData<dimension, dimension> &data,
+                 internal::DataOutFacesImplementation::ParallelData<dimension, dimension> &data,
                  DataOutBase::Patch<dimension-1,space_dimension>  &patch)
 {
   Assert (cell_and_face->first->is_locally_owned(),
@@ -143,7 +143,7 @@ build_one_patch (const FaceDescriptor *cell_and_face,
           const unsigned int n_components
             = this_fe_patch_values.get_fe().n_components();
           const DataPostprocessor<dim> *postprocessor=this->dof_data[dataset]->postprocessor;
-          if (postprocessor != 0)
+          if (postprocessor != nullptr)
             {
               // we have to postprocess the data, so determine, which fields
               // have to be updated
@@ -155,12 +155,15 @@ build_one_patch (const FaceDescriptor *cell_and_face,
                   // gradient etc.
                   if (update_flags & update_values)
                     this->dof_data[dataset]->get_function_values (this_fe_patch_values,
+                                                                  internal::DataOutImplementation::ComponentExtractor::real_part,
                                                                   data.patch_values_scalar.solution_values);
                   if (update_flags & update_gradients)
                     this->dof_data[dataset]->get_function_gradients (this_fe_patch_values,
+                                                                     internal::DataOutImplementation::ComponentExtractor::real_part,
                                                                      data.patch_values_scalar.solution_gradients);
                   if (update_flags & update_hessians)
                     this->dof_data[dataset]->get_function_hessians (this_fe_patch_values,
+                                                                    internal::DataOutImplementation::ComponentExtractor::real_part,
                                                                     data.patch_values_scalar.solution_hessians);
 
                   if (update_flags & update_quadrature_points)
@@ -186,12 +189,15 @@ build_one_patch (const FaceDescriptor *cell_and_face,
                   data.resize_system_vectors(n_components);
                   if (update_flags & update_values)
                     this->dof_data[dataset]->get_function_values (this_fe_patch_values,
+                                                                  internal::DataOutImplementation::ComponentExtractor::real_part,
                                                                   data.patch_values_system.solution_values);
                   if (update_flags & update_gradients)
                     this->dof_data[dataset]->get_function_gradients (this_fe_patch_values,
+                                                                     internal::DataOutImplementation::ComponentExtractor::real_part,
                                                                      data.patch_values_system.solution_gradients);
                   if (update_flags & update_hessians)
                     this->dof_data[dataset]->get_function_hessians (this_fe_patch_values,
+                                                                    internal::DataOutImplementation::ComponentExtractor::real_part,
                                                                     data.patch_values_system.solution_hessians);
 
                   if (update_flags & update_quadrature_points)
@@ -224,6 +230,7 @@ build_one_patch (const FaceDescriptor *cell_and_face,
             if (n_components == 1)
               {
                 this->dof_data[dataset]->get_function_values (this_fe_patch_values,
+                                                              internal::DataOutImplementation::ComponentExtractor::real_part,
                                                               data.patch_values_scalar.solution_values);
                 for (unsigned int q=0; q<n_q_points; ++q)
                   patch.data(offset,q) = data.patch_values_scalar.solution_values[q];
@@ -232,6 +239,7 @@ build_one_patch (const FaceDescriptor *cell_and_face,
               {
                 data.resize_system_vectors(n_components);
                 this->dof_data[dataset]->get_function_values (this_fe_patch_values,
+                                                              internal::DataOutImplementation::ComponentExtractor::real_part,
                                                               data.patch_values_system.solution_values);
                 for (unsigned int component=0; component<n_components;
                      ++component)
@@ -258,7 +266,8 @@ build_one_patch (const FaceDescriptor *cell_and_face,
                              typename Triangulation<dimension,space_dimension>::active_cell_iterator(cell_and_face->first));
 
           const double value
-            = this->cell_data[dataset]->get_cell_data_value (cell_number);
+            = this->cell_data[dataset]->get_cell_data_value (cell_number,
+                                                             internal::DataOutImplementation::ComponentExtractor::real_part);
           for (unsigned int q=0; q<n_q_points; ++q)
             patch.data(dataset+offset,q) = value;
         }
@@ -288,10 +297,10 @@ void DataOutFaces<dim,DoFHandlerType>::build_patches (const Mapping<dimension> &
                                       : this->default_subdivisions;
 
   Assert (n_subdivisions >= 1,
-          Exceptions::DataOut::ExcInvalidNumberOfSubdivisions(n_subdivisions));
+          Exceptions::DataOutImplementation::ExcInvalidNumberOfSubdivisions(n_subdivisions));
 
-  Assert (this->triangulation != 0,
-          Exceptions::DataOut::ExcNoTriangulationSelected());
+  Assert (this->triangulation != nullptr,
+          Exceptions::DataOutImplementation::ExcNoTriangulationSelected());
 
   this->validate_dataset_names();
 
@@ -330,12 +339,12 @@ void DataOutFaces<dim,DoFHandlerType>::build_patches (const Mapping<dimension> &
       update_flags |= this->dof_data[i]->postprocessor->get_needed_update_flags();
   update_flags |= update_quadrature_points;
 
-  internal::DataOutFaces::ParallelData<dimension, space_dimension>
+  internal::DataOutFacesImplementation::ParallelData<dimension, space_dimension>
   thread_data (n_datasets,
                n_subdivisions,
                n_postprocessor_outputs,
                mapping,
-               this->get_finite_elements(),
+               this->get_fes(),
                update_flags);
   DataOutBase::Patch<dimension-1,space_dimension> sample_patch;
   sample_patch.n_subdivisions = n_subdivisions;
@@ -345,11 +354,11 @@ void DataOutFaces<dim,DoFHandlerType>::build_patches (const Mapping<dimension> &
   // now build the patches in parallel
   WorkStream::run (&all_faces[0],
                    &all_faces[0]+all_faces.size(),
-                   std_cxx11::bind(&DataOutFaces<dim,DoFHandlerType>::build_one_patch,
-                                   this, std_cxx11::_1, std_cxx11::_2, std_cxx11::_3),
-                   std_cxx11::bind(&internal::DataOutFaces::
-                                   append_patch_to_list<dim,space_dimension>,
-                                   std_cxx11::_1, std_cxx11::ref(this->patches)),
+                   std::bind(&DataOutFaces<dim,DoFHandlerType>::build_one_patch,
+                             this, std::placeholders::_1, std::placeholders::_2, std::placeholders::_3),
+                   std::bind(&internal::DataOutFacesImplementation::
+                             append_patch_to_list<dim,space_dimension>,
+                             std::placeholders::_1, std::ref(this->patches)),
                    thread_data,
                    sample_patch);
 }
