@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 2000 - 2017 by the deal.II authors
+// Copyright (C) 2000 - 2018 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -45,7 +45,7 @@ DEAL_II_NAMESPACE_OPEN
 
 namespace internal
 {
-  namespace DataOutRotation
+  namespace DataOutRotationImplementation
   {
     template <int dim, int spacedim>
     ParallelData<dim,spacedim>::
@@ -54,10 +54,10 @@ namespace internal
                   const unsigned int n_patches_per_circle,
                   const std::vector<unsigned int> &n_postprocessor_outputs,
                   const Mapping<dim,spacedim> &mapping,
-                  const std::vector<std_cxx11::shared_ptr<dealii::hp::FECollection<dim,spacedim> > > &finite_elements,
+                  const std::vector<std::shared_ptr<dealii::hp::FECollection<dim,spacedim> > > &finite_elements,
                   const UpdateFlags update_flags)
       :
-      internal::DataOut::
+      internal::DataOutImplementation::
       ParallelDataBase<dim,spacedim> (n_datasets,
                                       n_subdivisions,
                                       n_postprocessor_outputs,
@@ -94,7 +94,7 @@ template <int dim, typename DoFHandlerType>
 void
 DataOutRotation<dim,DoFHandlerType>::
 build_one_patch (const cell_iterator                                                 *cell,
-                 internal::DataOutRotation::ParallelData<dimension, space_dimension> &data,
+                 internal::DataOutRotationImplementation::ParallelData<dimension, space_dimension> &data,
                  std::vector<DataOutBase::Patch<dimension+1,space_dimension+1> >     &my_patches)
 {
   if (dim == 3)
@@ -156,8 +156,7 @@ build_one_patch (const cell_iterator
             {
               const Point<dimension> v = (*cell)->vertex(vertex);
 
-              // make sure that the radial variable does attain negative
-              // values
+              // make sure that the radial variable is nonnegative
               Assert (v(0) >= 0, ExcRadialVariableHasNegativeValues(v(0)));
 
               // now set the vertices of the patch
@@ -191,7 +190,7 @@ build_one_patch (const cell_iterator
               const unsigned int n_components
                 = fe_patch_values.get_fe().n_components();
               const DataPostprocessor<dim> *postprocessor=this->dof_data[dataset]->postprocessor;
-              if (postprocessor != 0)
+              if (postprocessor != nullptr)
                 {
                   // we have to postprocess the
                   // data, so determine, which
@@ -205,12 +204,15 @@ build_one_patch (const cell_iterator
                       // value, gradient etc.
                       if (update_flags & update_values)
                         this->dof_data[dataset]->get_function_values (fe_patch_values,
+                                                                      internal::DataOutImplementation::ComponentExtractor::real_part,
                                                                       data.patch_values_scalar.solution_values);
                       if (update_flags & update_gradients)
                         this->dof_data[dataset]->get_function_gradients (fe_patch_values,
+                                                                         internal::DataOutImplementation::ComponentExtractor::real_part,
                                                                          data.patch_values_scalar.solution_gradients);
                       if (update_flags & update_hessians)
                         this->dof_data[dataset]->get_function_hessians (fe_patch_values,
+                                                                        internal::DataOutImplementation::ComponentExtractor::real_part,
                                                                         data.patch_values_scalar.solution_hessians);
 
                       if (update_flags & update_quadrature_points)
@@ -234,12 +236,15 @@ build_one_patch (const cell_iterator
                       // its derivative...
                       if (update_flags & update_values)
                         this->dof_data[dataset]->get_function_values (fe_patch_values,
+                                                                      internal::DataOutImplementation::ComponentExtractor::real_part,
                                                                       data.patch_values_system.solution_values);
                       if (update_flags & update_gradients)
                         this->dof_data[dataset]->get_function_gradients (fe_patch_values,
+                                                                         internal::DataOutImplementation::ComponentExtractor::real_part,
                                                                          data.patch_values_system.solution_gradients);
                       if (update_flags & update_hessians)
                         this->dof_data[dataset]->get_function_hessians (fe_patch_values,
+                                                                        internal::DataOutImplementation::ComponentExtractor::real_part,
                                                                         data.patch_values_system.solution_hessians);
 
                       if (update_flags & update_quadrature_points)
@@ -289,6 +294,7 @@ build_one_patch (const cell_iterator
               else if (n_components == 1)
                 {
                   this->dof_data[dataset]->get_function_values (fe_patch_values,
+                                                                internal::DataOutImplementation::ComponentExtractor::real_part,
                                                                 data.patch_values_scalar.solution_values);
 
                   switch (dimension)
@@ -321,6 +327,7 @@ build_one_patch (const cell_iterator
                 {
                   data.resize_system_vectors(n_components);
                   this->dof_data[dataset]->get_function_values (fe_patch_values,
+                                                                internal::DataOutImplementation::ComponentExtractor::real_part,
                                                                 data.patch_values_system.solution_values);
 
                   for (unsigned int component=0; component<n_components;
@@ -367,7 +374,8 @@ build_one_patch (const cell_iterator
                 = std::distance (this->triangulation->begin_active(),
                                  typename Triangulation<dimension,space_dimension>::active_cell_iterator(*cell));
               const double value
-                = this->cell_data[dataset]->get_cell_data_value (cell_number);
+                = this->cell_data[dataset]->get_cell_data_value (cell_number,
+                                                                 internal::DataOutImplementation::ComponentExtractor::real_part);
               switch (dimension)
                 {
                 case 1:
@@ -407,14 +415,14 @@ void DataOutRotation<dim,DoFHandlerType>::build_patches (const unsigned int n_pa
   // Check consistency of redundant
   // template parameter
   Assert (dim==dimension, ExcDimensionMismatch(dim, dimension));
-  Assert (this->triangulation != 0,
-          Exceptions::DataOut::ExcNoTriangulationSelected());
+  Assert (this->triangulation != nullptr,
+          Exceptions::DataOutImplementation::ExcNoTriangulationSelected());
 
   const unsigned int n_subdivisions = (nnnn_subdivisions != 0)
                                       ? nnnn_subdivisions
                                       : this->default_subdivisions;
   Assert (n_subdivisions >= 1,
-          Exceptions::DataOut::ExcInvalidNumberOfSubdivisions(n_subdivisions));
+          Exceptions::DataOutImplementation::ExcInvalidNumberOfSubdivisions(n_subdivisions));
 
   this->validate_dataset_names();
 
@@ -457,12 +465,12 @@ void DataOutRotation<dim,DoFHandlerType>::build_patches (const unsigned int n_pa
     else
       n_postprocessor_outputs[dataset] = 0;
 
-  internal::DataOutRotation::ParallelData<dimension, space_dimension>
+  internal::DataOutRotationImplementation::ParallelData<dimension, space_dimension>
   thread_data (n_datasets,
                n_subdivisions, n_patches_per_circle,
                n_postprocessor_outputs,
                StaticMappingQ1<dimension,space_dimension>::mapping,
-               this->get_finite_elements(),
+               this->get_fes(),
                update_flags);
   std::vector<DataOutBase::Patch<dimension+1,space_dimension+1> >
   new_patches (n_patches_per_circle);
@@ -476,11 +484,11 @@ void DataOutRotation<dim,DoFHandlerType>::build_patches (const unsigned int n_pa
   // now build the patches in parallel
   WorkStream::run (&all_cells[0],
                    &all_cells[0]+all_cells.size(),
-                   std_cxx11::bind(&DataOutRotation<dim,DoFHandlerType>::build_one_patch,
-                                   this, std_cxx11::_1, std_cxx11::_2, std_cxx11::_3),
-                   std_cxx11::bind(&internal::DataOutRotation
-                                   ::append_patch_to_list<dim,space_dimension>,
-                                   std_cxx11::_1, std_cxx11::ref(this->patches)),
+                   std::bind(&DataOutRotation<dim,DoFHandlerType>::build_one_patch,
+                             this, std::placeholders::_1, std::placeholders::_2, std::placeholders::_3),
+                   std::bind(&internal::DataOutRotationImplementation
+                             ::append_patch_to_list<dim,space_dimension>,
+                             std::placeholders::_1, std::ref(this->patches)),
                    thread_data,
                    new_patches);
 }
