@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 1998 - 2016 by the deal.II authors
+// Copyright (C) 1998 - 2018 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -22,10 +22,10 @@
 #include <deal.II/lac/la_parallel_vector.h>
 #include <deal.II/lac/block_vector.h>
 #include <deal.II/lac/la_parallel_block_vector.h>
-#include <deal.II/lac/petsc_vector.h>
-#include <deal.II/lac/petsc_block_vector.h>
+#include <deal.II/lac/petsc_parallel_vector.h>
+#include <deal.II/lac/petsc_parallel_block_vector.h>
 #include <deal.II/lac/trilinos_vector.h>
-#include <deal.II/lac/trilinos_block_vector.h>
+#include <deal.II/lac/trilinos_parallel_block_vector.h>
 #include <deal.II/grid/tria_iterator.h>
 #include <deal.II/base/geometry_info.h>
 #include <deal.II/dofs/dof_handler.h>
@@ -40,12 +40,12 @@
 #include <deal.II/numerics/error_estimator.h>
 #include <deal.II/distributed/tria.h>
 
-#include <deal.II/base/std_cxx11/bind.h>
 
 #include <numeric>
 #include <algorithm>
 #include <cmath>
 #include <vector>
+#include <functional>
 
 DEAL_II_NAMESPACE_OPEN
 
@@ -58,20 +58,21 @@ KellyErrorEstimator<1,spacedim>::
 estimate (const Mapping<1,spacedim>                  &mapping,
           const DoFHandlerType                       &dof_handler,
           const Quadrature<0>                        &quadrature,
-          const typename FunctionMap<spacedim>::type &neumann_bc,
+          const typename FunctionMap<spacedim,typename InputVector::value_type>::type &neumann_bc,
           const InputVector                          &solution,
           Vector<float>                              &error,
           const ComponentMask                        &component_mask,
           const Function<spacedim>                   *coefficients,
           const unsigned int                          n_threads,
           const types::subdomain_id                   subdomain_id,
-          const types::material_id                    material_id)
+          const types::material_id                    material_id,
+          const Strategy                              strategy)
 {
   // just pass on to the other function
   const std::vector<const InputVector *> solutions (1, &solution);
   std::vector<Vector<float>*>              errors (1, &error);
   estimate (mapping, dof_handler, quadrature, neumann_bc, solutions, errors,
-            component_mask, coefficients, n_threads, subdomain_id, material_id);
+            component_mask, coefficients, n_threads, subdomain_id, material_id,strategy);
 }
 
 
@@ -82,17 +83,18 @@ void
 KellyErrorEstimator<1,spacedim>::
 estimate (const DoFHandlerType                       &dof_handler,
           const Quadrature<0>                        &quadrature,
-          const typename FunctionMap<spacedim>::type &neumann_bc,
+          const typename FunctionMap<spacedim,typename InputVector::value_type>::type &neumann_bc,
           const InputVector                          &solution,
           Vector<float>                              &error,
           const ComponentMask                        &component_mask,
           const Function<spacedim>                   *coefficients,
           const unsigned int                          n_threads,
           const types::subdomain_id                   subdomain_id,
-          const types::material_id                    material_id)
+          const types::material_id                    material_id,
+          const Strategy                              strategy)
 {
   estimate(StaticMappingQ1<1,spacedim>::mapping, dof_handler, quadrature, neumann_bc, solution,
-           error, component_mask, coefficients, n_threads, subdomain_id, material_id);
+           error, component_mask, coefficients, n_threads, subdomain_id, material_id,strategy);
 }
 
 
@@ -103,17 +105,18 @@ void
 KellyErrorEstimator<1,spacedim>::
 estimate (const DoFHandlerType                       &dof_handler,
           const Quadrature<0>                        &quadrature,
-          const typename FunctionMap<spacedim>::type &neumann_bc,
+          const typename FunctionMap<spacedim,typename InputVector::value_type>::type &neumann_bc,
           const std::vector<const InputVector *>     &solutions,
           std::vector<Vector<float>*>                &errors,
           const ComponentMask                        &component_mask,
           const Function<spacedim>                   *coefficients,
           const unsigned int                          n_threads,
           const types::subdomain_id                   subdomain_id,
-          const types::material_id                    material_id)
+          const types::material_id                    material_id,
+          const Strategy                              strategy)
 {
   estimate(StaticMappingQ1<1,spacedim>::mapping, dof_handler, quadrature, neumann_bc, solutions,
-           errors, component_mask, coefficients, n_threads, subdomain_id, material_id);
+           errors, component_mask, coefficients, n_threads, subdomain_id, material_id,strategy);
 }
 
 
@@ -125,20 +128,21 @@ KellyErrorEstimator<1,spacedim>::
 estimate (const Mapping<1,spacedim>                  &mapping,
           const DoFHandlerType                       &dof_handler,
           const hp::QCollection<0>                   &quadrature,
-          const typename FunctionMap<spacedim>::type &neumann_bc,
+          const typename FunctionMap<spacedim,typename InputVector::value_type>::type &neumann_bc,
           const InputVector                          &solution,
           Vector<float>                              &error,
           const ComponentMask                        &component_mask,
           const Function<spacedim>                   *coefficients,
           const unsigned int                          n_threads,
           const types::subdomain_id                   subdomain_id,
-          const types::material_id                    material_id)
+          const types::material_id                    material_id,
+          const Strategy                              strategy)
 {
   // just pass on to the other function
   const std::vector<const InputVector *> solutions (1, &solution);
   std::vector<Vector<float>*>              errors (1, &error);
   estimate (mapping, dof_handler, quadrature, neumann_bc, solutions, errors,
-            component_mask, coefficients, n_threads, subdomain_id, material_id);
+            component_mask, coefficients, n_threads, subdomain_id, material_id, strategy);
 }
 
 
@@ -148,17 +152,18 @@ void
 KellyErrorEstimator<1,spacedim>::
 estimate (const DoFHandlerType                       &dof_handler,
           const hp::QCollection<0>                   &quadrature,
-          const typename FunctionMap<spacedim>::type &neumann_bc,
+          const typename FunctionMap<spacedim,typename InputVector::value_type>::type &neumann_bc,
           const InputVector                          &solution,
           Vector<float>                              &error,
           const ComponentMask                        &component_mask,
           const Function<spacedim>                   *coefficients,
           const unsigned int                          n_threads,
           const types::subdomain_id                   subdomain_id,
-          const types::material_id                    material_id)
+          const types::material_id                    material_id,
+          const Strategy                              strategy)
 {
   estimate(StaticMappingQ1<1,spacedim>::mapping, dof_handler, quadrature, neumann_bc, solution,
-           error, component_mask, coefficients, n_threads, subdomain_id, material_id);
+           error, component_mask, coefficients, n_threads, subdomain_id, material_id,strategy);
 }
 
 
@@ -169,17 +174,18 @@ void
 KellyErrorEstimator<1,spacedim>::
 estimate (const DoFHandlerType                       &dof_handler,
           const hp::QCollection<0>                   &quadrature,
-          const typename FunctionMap<spacedim>::type &neumann_bc,
+          const typename FunctionMap<spacedim,typename InputVector::value_type>::type &neumann_bc,
           const std::vector<const InputVector *>     &solutions,
           std::vector<Vector<float>*>                &errors,
           const ComponentMask                        &component_mask,
           const Function<spacedim>                   *coefficients,
           const unsigned int                          n_threads,
           const types::subdomain_id                   subdomain_id,
-          const types::material_id                    material_id)
+          const types::material_id                    material_id,
+          const Strategy                              strategy)
 {
   estimate(StaticMappingQ1<1,spacedim>::mapping, dof_handler, quadrature, neumann_bc, solutions,
-           errors, component_mask, coefficients, n_threads, subdomain_id, material_id);
+           errors, component_mask, coefficients, n_threads, subdomain_id, material_id,strategy);
 }
 
 
@@ -191,14 +197,15 @@ void KellyErrorEstimator<1,spacedim>::
 estimate (const Mapping<1,spacedim>                  & /*mapping*/,
           const DoFHandlerType                       & /*dof_handler*/,
           const hp::QCollection<0> &,
-          const typename FunctionMap<spacedim>::type & /*neumann_bc*/,
+          const typename FunctionMap<spacedim,typename InputVector::value_type>::type & /*neumann_bc*/,
           const std::vector<const InputVector *>     & /*solutions*/,
           std::vector<Vector<float>*>                & /*errors*/,
           const ComponentMask                        & /*component_mask_*/,
           const Function<spacedim>                   * /*coefficient*/,
           const unsigned int,
           const types::subdomain_id                    /*subdomain_id*/,
-          const types::material_id                     /*material_id*/)
+          const types::material_id                     /*material_id*/,
+          const Strategy                              /*strategy*/)
 {
   Assert (false, ExcInternalError());
 }
@@ -211,20 +218,22 @@ void KellyErrorEstimator<1,spacedim>::
 estimate (const Mapping<1,spacedim>                  &mapping,
           const DoFHandlerType                       &dof_handler,
           const Quadrature<0> &,
-          const typename FunctionMap<spacedim>::type &neumann_bc,
+          const typename FunctionMap<spacedim,typename InputVector::value_type>::type &neumann_bc,
           const std::vector<const InputVector *>     &solutions,
           std::vector<Vector<float>*>                &errors,
           const ComponentMask                        &component_mask,
           const Function<spacedim>                   *coefficient,
           const unsigned int,
           const types::subdomain_id                   subdomain_id_,
-          const types::material_id                    material_id)
+          const types::material_id                    material_id,
+          const Strategy                              strategy)
 {
+  AssertThrow(strategy==cell_diameter_over_24, ExcNotImplemented());
   typedef typename InputVector::value_type number;
 #ifdef DEAL_II_WITH_P4EST
   if (dynamic_cast<const parallel::distributed::Triangulation<1,spacedim>*>
       (&dof_handler.get_triangulation())
-      != 0)
+      != nullptr)
     Assert ((subdomain_id_ == numbers::invalid_subdomain_id)
             ||
             (subdomain_id_ ==
@@ -237,7 +246,7 @@ estimate (const Mapping<1,spacedim>                  &mapping,
   const types::subdomain_id subdomain_id
     = ((dynamic_cast<const parallel::distributed::Triangulation<1,spacedim>*>
         (&dof_handler.get_triangulation())
-        != 0)
+        != nullptr)
        ?
        dynamic_cast<const parallel::distributed::Triangulation<1,spacedim>&>
        (dof_handler.get_triangulation()).locally_owned_subdomain()
@@ -248,7 +257,7 @@ estimate (const Mapping<1,spacedim>                  &mapping,
     = subdomain_id_;
 #endif
 
-  const unsigned int n_components       = dof_handler.get_fe().n_components();
+  const unsigned int n_components       = dof_handler.get_fe(0).n_components();
   const unsigned int n_solution_vectors = solutions.size();
 
   // sanity checks
@@ -257,7 +266,7 @@ estimate (const Mapping<1,spacedim>                  &mapping,
                      "indicator for internal boundaries in your boundary "
                      "value map."));
 
-  for (typename FunctionMap<spacedim>::type::const_iterator i=neumann_bc.begin();
+  for (typename FunctionMap<spacedim,typename InputVector::value_type>::type::const_iterator i=neumann_bc.begin();
        i!=neumann_bc.end(); ++i)
     Assert (i->second->n_components == n_components,
             ExcInvalidBoundaryFunction(i->first,
@@ -269,7 +278,7 @@ estimate (const Mapping<1,spacedim>                  &mapping,
   Assert (component_mask.n_selected_components(n_components) > 0,
           ExcInvalidComponentMask());
 
-  Assert ((coefficient == 0) ||
+  Assert ((coefficient == nullptr) ||
           (coefficient->n_components == n_components) ||
           (coefficient->n_components == 1),
           ExcInvalidCoefficient());
@@ -283,12 +292,12 @@ estimate (const Mapping<1,spacedim>                  &mapping,
             ExcDimensionMismatch(solutions[n]->size(),
                                  dof_handler.n_dofs()));
 
-  Assert ((coefficient == 0) ||
+  Assert ((coefficient == nullptr) ||
           (coefficient->n_components == n_components) ||
           (coefficient->n_components == 1),
           ExcInvalidCoefficient());
 
-  for (typename FunctionMap<spacedim>::type::const_iterator i=neumann_bc.begin();
+  for (typename FunctionMap<spacedim,typename InputVector::value_type>::type::const_iterator i=neumann_bc.begin();
        i!=neumann_bc.end(); ++i)
     Assert (i->second->n_components == n_components,
             ExcInvalidBoundaryFunction(i->first,
@@ -308,14 +317,14 @@ estimate (const Mapping<1,spacedim>                  &mapping,
                   std::vector<std::vector<Tensor<1,spacedim,number> > >(2, std::vector<Tensor<1,spacedim,number> >(n_components)));
   std::vector<std::vector<std::vector<Tensor<1,spacedim,number> > > >
   gradients_neighbor (gradients_here);
-  std::vector<Vector<number> >
-  grad_neighbor (n_solution_vectors, Vector<number>(n_components));
+  std::vector<Vector<typename ProductType<number,double>::type> >
+  grad_dot_n_neighbor (n_solution_vectors, Vector<typename ProductType<number,double>::type>(n_components));
 
   // reserve some space for coefficient values at one point.  if there is no
   // coefficient, then we fill it by unity once and for all and don't set it
   // any more
   Vector<double> coefficient_values (n_components);
-  if (coefficient == 0)
+  if (coefficient == nullptr)
     for (unsigned int c=0; c<n_components; ++c)
       coefficient_values(c) = 1;
 
@@ -324,7 +333,7 @@ estimate (const Mapping<1,spacedim>                  &mapping,
   const QGauss<0> face_quadrature(1);
   const hp::QCollection<0> q_face_collection(face_quadrature);
 
-  const hp::FECollection<1,spacedim> fe (dof_handler.get_fe());
+  const hp::FECollection<1,spacedim> &fe = dof_handler.get_fe_collection();
 
   hp::MappingCollection<1,spacedim> mapping_collection;
   mapping_collection.push_back (mapping);
@@ -386,7 +395,7 @@ estimate (const Mapping<1,spacedim>                  &mapping,
                 // extract the gradient in normal direction of all the components.
                 for (unsigned int s=0; s<n_solution_vectors; ++s)
                   for (unsigned int c=0; c<n_components; ++c)
-                    grad_neighbor[s](c)
+                    grad_dot_n_neighbor[s](c)
                       = - (gradients_neighbor[s][n==0 ? 1 : 0][c]*neighbor_normal);
               }
             else if (neumann_bc.find(n) != neumann_bc.end())
@@ -395,29 +404,29 @@ estimate (const Mapping<1,spacedim>                  &mapping,
               {
                 if (n_components==1)
                   {
-                    const double
+                    const typename InputVector::value_type
                     v = neumann_bc.find(n)->second->value(cell->vertex(n));
 
                     for (unsigned int s=0; s<n_solution_vectors; ++s)
-                      grad_neighbor[s](0) = v;
+                      grad_dot_n_neighbor[s](0) = v;
                   }
                 else
                   {
-                    Vector<double> v(n_components);
+                    Vector<typename InputVector::value_type> v(n_components);
                     neumann_bc.find(n)->second->vector_value(cell->vertex(n), v);
 
                     for (unsigned int s=0; s<n_solution_vectors; ++s)
-                      grad_neighbor[s] = v;
+                      grad_dot_n_neighbor[s] = v;
                   }
               }
             else
               // fill with zeroes.
               for (unsigned int s=0; s<n_solution_vectors; ++s)
-                grad_neighbor[s] = 0;
+                grad_dot_n_neighbor[s] = 0;
 
             // if there is a coefficient, then evaluate it at the present
             // position. if there is none, reuse the preset values.
-            if (coefficient != 0)
+            if (coefficient != nullptr)
               {
                 if (coefficient->n_components == 1)
                   {
@@ -436,12 +445,14 @@ estimate (const Mapping<1,spacedim>                  &mapping,
                 if (component_mask[component] == true)
                   {
                     // get gradient here
-                    const number grad_here = gradients_here[s][n][component]
-                                             * normal;
+                    const typename ProductType<number,double>::type
+                    grad_dot_n_here = gradients_here[s][n][component] * normal;
 
-                    const number jump = ((grad_here - grad_neighbor[s](component)) *
-                                         coefficient_values(component));
-                    (*errors[s])(cell->active_cell_index()) += numbers::NumberTraits<number>::abs_square(jump) * cell->diameter();
+                    const typename ProductType<number,double>::type
+                    jump = ((grad_dot_n_here - grad_dot_n_neighbor[s](component)) *
+                            coefficient_values(component));
+                    (*errors[s])(cell->active_cell_index())
+                    += numbers::NumberTraits<typename ProductType<number,double>::type>::abs_square(jump) * cell->diameter();
                   }
           }
 
