@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 2014 - 2017 by the deal.II authors
+// Copyright (C) 2014 - 2018 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -18,7 +18,6 @@
 
 #ifdef DEAL_II_WITH_OPENCASCADE
 
-DEAL_II_DISABLE_EXTRA_DIAGNOSTICS
 
 #include <GCPnts_AbscissaPoint.hxx>
 #include <BRepAdaptor_Curve.hxx>
@@ -37,7 +36,6 @@ DEAL_II_DISABLE_EXTRA_DIAGNOSTICS
 #  include <Handle_Adaptor3d_HCurve.hxx>
 #endif
 
-DEAL_II_ENABLE_EXTRA_DIAGNOSTICS
 
 DEAL_II_NAMESPACE_OPEN
 
@@ -88,10 +86,20 @@ namespace OpenCASCADE
   }
 
 
+
+  template<int dim, int spacedim>
+  std::unique_ptr<Manifold<dim, spacedim> >
+  NormalProjectionBoundary<dim,spacedim>::clone() const
+  {
+    return std::unique_ptr<Manifold<dim,spacedim> >(new NormalProjectionBoundary(sh, tolerance));
+  }
+
+
+
   template <int dim, int spacedim>
   Point<spacedim>  NormalProjectionBoundary<dim,spacedim>::
-  project_to_manifold (const std::vector<Point<spacedim> > &surrounding_points,
-                       const Point<spacedim> &candidate) const
+  project_to_manifold (const ArrayView<const Point<spacedim>> &surrounding_points,
+                       const Point<spacedim>                  &candidate) const
   {
     (void)surrounding_points;
 #ifdef DEBUG
@@ -99,7 +107,7 @@ namespace OpenCASCADE
       Assert(closest_point(sh, surrounding_points[i], tolerance)
              .distance(surrounding_points[i]) <
              std::max(tolerance*surrounding_points[i].norm(), tolerance),
-             ExcPointNotOnManifold(surrounding_points[i]));
+             ExcPointNotOnManifold<spacedim>(surrounding_points[i]));
 #endif
     return closest_point(sh, candidate,tolerance);
   }
@@ -118,10 +126,21 @@ namespace OpenCASCADE
   }
 
 
+
+  template<int dim, int spacedim>
+  std::unique_ptr<Manifold<dim, spacedim> >
+  DirectionalProjectionBoundary<dim,spacedim>::clone() const
+  {
+    return std::unique_ptr<Manifold<dim,spacedim> >
+           (new DirectionalProjectionBoundary(sh, direction, tolerance));
+  }
+
+
+
   template <int dim, int spacedim>
   Point<spacedim>  DirectionalProjectionBoundary<dim,spacedim>::
-  project_to_manifold (const std::vector<Point<spacedim> > &surrounding_points,
-                       const Point<spacedim> &candidate) const
+  project_to_manifold (const ArrayView<const Point<spacedim>> &surrounding_points,
+                       const Point<spacedim>                  &candidate) const
   {
     (void)surrounding_points;
 #ifdef DEBUG
@@ -129,7 +148,7 @@ namespace OpenCASCADE
       Assert(closest_point(sh, surrounding_points[i],tolerance)
              .distance(surrounding_points[i]) <
              std::max(tolerance*surrounding_points[i].norm(), tolerance),
-             ExcPointNotOnManifold(surrounding_points[i]));
+             ExcPointNotOnManifold<spacedim>(surrounding_points[i]));
 #endif
     return line_intersection(sh, candidate, direction, tolerance);
   }
@@ -144,15 +163,23 @@ namespace OpenCASCADE
     tolerance(tolerance)
   {
     Assert(spacedim == 3, ExcNotImplemented());
-    Assert(std_cxx11::get<0>(count_elements(sh)) > 0,
+    Assert(std::get<0>(count_elements(sh)) > 0,
            ExcMessage("NormalToMeshProjectionBoundary needs a shape containing faces to operate."));
   }
 
+  template<int dim, int spacedim>
+  std::unique_ptr<Manifold<dim, spacedim> >
+  NormalToMeshProjectionBoundary<dim,spacedim>::clone() const
+  {
+    return std::unique_ptr<Manifold<dim, spacedim> >
+           (new NormalToMeshProjectionBoundary<dim,spacedim>(sh,tolerance));
+  }
+
 
   template <int dim, int spacedim>
   Point<spacedim>  NormalToMeshProjectionBoundary<dim,spacedim>::
-  project_to_manifold (const std::vector<Point<spacedim> > &surrounding_points,
-                       const Point<spacedim> &candidate) const
+  project_to_manifold (const ArrayView<const Point<spacedim>> &surrounding_points,
+                       const Point<spacedim>                  &candidate) const
   {
     TopoDS_Shape out_shape;
     Tensor<1,3> average_normal;
@@ -162,7 +189,7 @@ namespace OpenCASCADE
         Assert(closest_point(sh, surrounding_points[i], tolerance)
                .distance(surrounding_points[i]) <
                std::max(tolerance*surrounding_points[i].norm(), tolerance),
-               ExcPointNotOnManifold(surrounding_points[i]));
+               ExcPointNotOnManifold<spacedim>(surrounding_points[i]));
       }
 #endif
 
@@ -172,12 +199,12 @@ namespace OpenCASCADE
       {
         for (unsigned int i=0; i<surrounding_points.size(); ++i)
           {
-            std_cxx11::tuple<Point<3>,  Tensor<1,3>, double, double>
+            std::tuple<Point<3>,  Tensor<1,3>, double, double>
             p_and_diff_forms =
               closest_point_and_differential_forms(sh,
                                                    surrounding_points[i],
                                                    tolerance);
-            average_normal += std_cxx11::get<1>(p_and_diff_forms);
+            average_normal += std::get<1>(p_and_diff_forms);
           }
 
         average_normal/=2.0;
@@ -191,6 +218,27 @@ namespace OpenCASCADE
         average_normal /= average_normal.norm();
         break;
       }
+      case 4:
+      {
+        Tensor<1,3> u = surrounding_points[1]-surrounding_points[0];
+        Tensor<1,3> v = surrounding_points[2]-surrounding_points[0];
+        const double n1_coords[3] = {u[1] *v[2]-u[2] *v[1],u[2] *v[0]-u[0] *v[2],u[0] *v[1]-u[1] *v[0]};
+        Tensor<1,3> n1(n1_coords);
+        n1 = n1/n1.norm();
+        u = surrounding_points[2]-surrounding_points[3];
+        v = surrounding_points[1]-surrounding_points[3];
+        const double n2_coords[3] = {u[1] *v[2]-u[2] *v[1],u[2] *v[0]-u[0] *v[2],u[0] *v[1]-u[1] *v[0]};
+        Tensor<1,3> n2(n2_coords);
+        n2 = n2/n2.norm();
+
+        average_normal = (n1+n2)/2.0;
+
+        Assert(average_normal.norm() > tolerance,
+               ExcMessage("Failed to refine cell: the normal estimated via the surrounding points turns out to be a null vector, making the projection direction undetermined."));
+
+        average_normal /= average_normal.norm();
+        break;
+      }
       case 8:
       {
         Tensor<1,3> u = surrounding_points[1]-surrounding_points[0];
@@ -213,10 +261,6 @@ namespace OpenCASCADE
         const double n4_coords[3] = {u[1] *v[2]-u[2] *v[1],u[2] *v[0]-u[0] *v[2],u[0] *v[1]-u[1] *v[0]};
         Tensor<1,3> n4(n4_coords);
         n4 = n4/n4.norm();
-        //for (unsigned int i=0; i<surrounding_points.size(); ++i)
-        //    cout<<surrounding_points[i]<<endl;
-        //cout<<"-"<<endl;
-        //cout<<n1<<endl;cout<<n2<<endl;cout<<n3<<endl;cout<<n4<<endl;
 
         average_normal = (n1+n2+n3+n4)/4.0;
 
@@ -245,14 +289,26 @@ namespace OpenCASCADE
     ChartManifold<dim,spacedim,1>(sh.Closed() ?
                                   Point<1>(shape_length(sh)) :
                                   Point<1>()),
+    sh(sh),
     curve(curve_adaptor(sh)),
     tolerance(tolerance),
     length(shape_length(sh))
   {
-    Assert(spacedim == 3, ExcNotImplemented());
+    Assert(spacedim >= 2, ExcImpossibleInDimSpacedim(dim, spacedim));
   }
 
 
+
+  template<int dim, int spacedim>
+  std::unique_ptr<Manifold<dim, spacedim> >
+  ArclengthProjectionLineManifold<dim, spacedim>::clone() const
+  {
+    return std::unique_ptr<Manifold<dim, spacedim> >
+           (new ArclengthProjectionLineManifold(sh,tolerance));
+  }
+
+
+
   template <int dim, int spacedim>
   Point<1>
   ArclengthProjectionLineManifold<dim,spacedim>::pull_back(const Point<spacedim> &space_point) const
@@ -261,7 +317,7 @@ namespace OpenCASCADE
     ShapeAnalysis_Curve curve_analysis;
     gp_Pnt proj;
     const double dist = curve_analysis.Project(curve->GetCurve(), point(space_point), tolerance, proj, t, true);
-    Assert(dist < tolerance*length, ExcPointNotOnManifold(space_point));
+    Assert(dist < tolerance*length, ExcPointNotOnManifold<spacedim>(space_point));
     (void)dist; // Silence compiler warning in Release mode.
     return Point<1>(GCPnts_AbscissaPoint::Length(curve->GetCurve(),curve->GetCurve().FirstParameter(),t));
   }
@@ -274,7 +330,7 @@ namespace OpenCASCADE
   {
     GCPnts_AbscissaPoint AP(curve->GetCurve(), chart_point[0], curve->GetCurve().FirstParameter());
     gp_Pnt P = curve->GetCurve().Value(AP.Parameter());
-    return point(P);
+    return point<spacedim>(P);
   }
 
   template <int dim, int spacedim>
@@ -286,10 +342,21 @@ namespace OpenCASCADE
     tolerance(tolerance)
   {}
 
-  template<>
-  Point<2>
-  NURBSPatchManifold<2, 3>::
-  pull_back(const Point<3> &space_point) const
+
+
+  template<int dim, int spacedim>
+  std::unique_ptr<Manifold<dim, spacedim> >
+  NURBSPatchManifold<dim,spacedim>::clone() const
+  {
+    return std::unique_ptr<Manifold<dim, spacedim> >
+           (new NURBSPatchManifold<dim,spacedim>(face,tolerance));
+  }
+
+
+
+  template <int dim, int spacedim>  Point<2>
+  NURBSPatchManifold<dim, spacedim>::
+  pull_back(const Point<spacedim> &space_point) const
   {
     Handle(Geom_Surface) SurfToProj = BRep_Tool::Surface(face);
 
@@ -302,20 +369,20 @@ namespace OpenCASCADE
     return Point<2>(u,v);
   }
 
-  template<>
-  Point<3>
-  NURBSPatchManifold<2, 3>::
+  template <int dim, int spacedim>
+  Point<spacedim>
+  NURBSPatchManifold<dim, spacedim>::
   push_forward(const Point<2> &chart_point) const
   {
-    return ::dealii::OpenCASCADE::push_forward(face, chart_point[0], chart_point[1]);
+    return ::dealii::OpenCASCADE::push_forward<spacedim>(face, chart_point[0], chart_point[1]);
   }
 
-  template<>
-  DerivativeForm<1,2,3>
-  NURBSPatchManifold<2, 3>::
+  template <int dim, int spacedim>
+  DerivativeForm<1,2,spacedim>
+  NURBSPatchManifold<dim,spacedim>::
   push_forward_gradient(const Point<2> &chart_point) const
   {
-    DerivativeForm<1,2,3> DX;
+    DerivativeForm<1,2,spacedim> DX;
     Handle(Geom_Surface) surf = BRep_Tool::Surface(face);
 
     gp_Pnt q;
@@ -324,26 +391,32 @@ namespace OpenCASCADE
 
     DX[0][0] = Du.X();
     DX[1][0] = Du.Y();
-    DX[2][0] = Du.Z();
+    if (spacedim>2)
+      DX[2][0] = Du.Z();
+    else
+      Assert(std::abs(Du.Z()) < tolerance,
+             ExcMessage("Expecting derivative along Z to be zero! Bailing out."));
     DX[0][1] = Dv.X();
     DX[1][1] = Dv.Y();
-    DX[2][1] = Dv.Z();
-
+    if (spacedim>2)
+      DX[2][1] = Dv.Z();
+    else
+      Assert(std::abs(Dv.Z()) < tolerance,
+             ExcMessage("Expecting derivative along Z to be zero! Bailing out."));
     return DX;
   }
 
-  template<>
-  std_cxx11::tuple<double, double, double, double>
-  NURBSPatchManifold<2, 3>::
+  template <int dim, int spacedim>
+  std::tuple<double, double, double, double>
+  NURBSPatchManifold<dim,spacedim>::
   get_uv_bounds() const
   {
     Standard_Real umin, umax, vmin, vmax;
     BRepTools::UVBounds(face, umin, umax, vmin, vmax);
-    return std_cxx11::make_tuple(umin, umax, vmin, vmax);
+    return std::make_tuple(umin, umax, vmin, vmax);
   }
 
 // Explicit instantiations
-  template class NURBSPatchManifold<2, 3 >;
 #include "boundary_lib.inst"
 
 } // end namespace OpenCASCADE
