@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 2008 - 2015 by the deal.II authors
+// Copyright (C) 2008 - 2017 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -18,7 +18,6 @@
 // Test DoFTools::count_dofs_per_component
 
 
-#include <deal.II/base/logstream.h>
 #include <deal.II/base/tensor.h>
 #include <deal.II/distributed/tria.h>
 #include <deal.II/grid/tria_accessor.h>
@@ -32,13 +31,11 @@
 #include <deal.II/fe/fe_q.h>
 #include <deal.II/fe/fe_dgq.h>
 
-#include <fstream>
 #include <numeric>
-#include <cstdlib>
 
 using namespace dealii;
 
-template<int dim>
+template <int dim>
 void test()
 {
   parallel::distributed::Triangulation<dim>
